(function (global, factory) {
typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
typeof define === 'function' && define.amd ? define(factory) :
(global = global || self, global.Tangram = factory());
}(this, function () { 'use strict';

// define() gets called for each chunk generated by the first Rollup pass.
// The order the chunks are called in is controlled by the imports in bundle.js:
//
// shared.js: shared dependencies between main and worker threads
// scene_worker.js: worker thread code
// index.js: main thread code

// Once all chunks have been provided, the worker thread code is assembled,
// incorporating the shared chunk code, then turned into a blob URL which
// can be used to instantiate the worker.

var shared, worker, Tangram = {};

function define(_, chunk) {
	if (!shared) {
	    shared = chunk;
	} else if (!worker) {
	    worker = chunk;
	} else {
	    var worker_bundle = 'var shared_chunk = {}; (' + shared + ')(shared_chunk); (' + worker + ')(shared_chunk);'
	    var shared_chunk = {};
	    shared(shared_chunk);
	    Tangram = chunk(shared_chunk);
	    Tangram.workerURL = window.URL.createObjectURL(new Blob([worker_bundle], { type: 'text/javascript' }));
	}
}


define(['exports'], function (exports) { 'use strict';

function commonjsRequire () {
	throw new Error('Dynamic requires are not currently supported by rollup-plugin-commonjs');
}

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var _core = createCommonjsModule(function (module) {
var core = module.exports = { version: '2.6.3' };
if (typeof __e == 'number') __e = core; // eslint-disable-line no-undef
});
var _core_1 = _core.version;

var _global = createCommonjsModule(function (module) {
// https://github.com/zloirock/core-js/issues/86#issuecomment-115759028
var global = module.exports = typeof window != 'undefined' && window.Math == Math
  ? window : typeof self != 'undefined' && self.Math == Math ? self
  // eslint-disable-next-line no-new-func
  : Function('return this')();
if (typeof __g == 'number') __g = global; // eslint-disable-line no-undef
});

var _library = false;

var _shared = createCommonjsModule(function (module) {
var SHARED = '__core-js_shared__';
var store = _global[SHARED] || (_global[SHARED] = {});

(module.exports = function (key, value) {
  return store[key] || (store[key] = value !== undefined ? value : {});
})('versions', []).push({
  version: _core.version,
  mode: 'global',
  copyright: '© 2019 Denis Pushkarev (zloirock.ru)'
});
});

var id = 0;
var px = Math.random();
var _uid = function (key) {
  return 'Symbol('.concat(key === undefined ? '' : key, ')_', (++id + px).toString(36));
};

var _wks = createCommonjsModule(function (module) {
var store = _shared('wks');

var Symbol = _global.Symbol;
var USE_SYMBOL = typeof Symbol == 'function';

var $exports = module.exports = function (name) {
  return store[name] || (store[name] =
    USE_SYMBOL && Symbol[name] || (USE_SYMBOL ? Symbol : _uid)('Symbol.' + name));
};

$exports.store = store;
});

var _isObject = function (it) {
  return typeof it === 'object' ? it !== null : typeof it === 'function';
};

var _anObject = function (it) {
  if (!_isObject(it)) throw TypeError(it + ' is not an object!');
  return it;
};

var _fails = function (exec) {
  try {
    return !!exec();
  } catch (e) {
    return true;
  }
};

// Thank's IE8 for his funny defineProperty
var _descriptors = !_fails(function () {
  return Object.defineProperty({}, 'a', { get: function () { return 7; } }).a != 7;
});

var document$1 = _global.document;
// typeof document.createElement is 'object' in old IE
var is = _isObject(document$1) && _isObject(document$1.createElement);
var _domCreate = function (it) {
  return is ? document$1.createElement(it) : {};
};

var _ie8DomDefine = !_descriptors && !_fails(function () {
  return Object.defineProperty(_domCreate('div'), 'a', { get: function () { return 7; } }).a != 7;
});

// 7.1.1 ToPrimitive(input [, PreferredType])

// instead of the ES6 spec version, we didn't implement @@toPrimitive case
// and the second argument - flag - preferred type is a string
var _toPrimitive = function (it, S) {
  if (!_isObject(it)) return it;
  var fn, val;
  if (S && typeof (fn = it.toString) == 'function' && !_isObject(val = fn.call(it))) return val;
  if (typeof (fn = it.valueOf) == 'function' && !_isObject(val = fn.call(it))) return val;
  if (!S && typeof (fn = it.toString) == 'function' && !_isObject(val = fn.call(it))) return val;
  throw TypeError("Can't convert object to primitive value");
};

var dP = Object.defineProperty;

var f = _descriptors ? Object.defineProperty : function defineProperty(O, P, Attributes) {
  _anObject(O);
  P = _toPrimitive(P, true);
  _anObject(Attributes);
  if (_ie8DomDefine) try {
    return dP(O, P, Attributes);
  } catch (e) { /* empty */ }
  if ('get' in Attributes || 'set' in Attributes) throw TypeError('Accessors not supported!');
  if ('value' in Attributes) O[P] = Attributes.value;
  return O;
};

var _objectDp = {
	f: f
};

var _propertyDesc = function (bitmap, value) {
  return {
    enumerable: !(bitmap & 1),
    configurable: !(bitmap & 2),
    writable: !(bitmap & 4),
    value: value
  };
};

var _hide = _descriptors ? function (object, key, value) {
  return _objectDp.f(object, key, _propertyDesc(1, value));
} : function (object, key, value) {
  object[key] = value;
  return object;
};

// 22.1.3.31 Array.prototype[@@unscopables]
var UNSCOPABLES = _wks('unscopables');
var ArrayProto = Array.prototype;
if (ArrayProto[UNSCOPABLES] == undefined) _hide(ArrayProto, UNSCOPABLES, {});
var _addToUnscopables = function (key) {
  ArrayProto[UNSCOPABLES][key] = true;
};

var _iterStep = function (done, value) {
  return { value: value, done: !!done };
};

var _iterators = {};

var toString = {}.toString;

var _cof = function (it) {
  return toString.call(it).slice(8, -1);
};

// fallback for non-array-like ES3 and non-enumerable old V8 strings

// eslint-disable-next-line no-prototype-builtins
var _iobject = Object('z').propertyIsEnumerable(0) ? Object : function (it) {
  return _cof(it) == 'String' ? it.split('') : Object(it);
};

// 7.2.1 RequireObjectCoercible(argument)
var _defined = function (it) {
  if (it == undefined) throw TypeError("Can't call method on  " + it);
  return it;
};

// to indexed object, toObject with fallback for non-array-like ES3 strings


var _toIobject = function (it) {
  return _iobject(_defined(it));
};

var hasOwnProperty = {}.hasOwnProperty;
var _has = function (it, key) {
  return hasOwnProperty.call(it, key);
};

var _redefine = createCommonjsModule(function (module) {
var SRC = _uid('src');
var TO_STRING = 'toString';
var $toString = Function[TO_STRING];
var TPL = ('' + $toString).split(TO_STRING);

_core.inspectSource = function (it) {
  return $toString.call(it);
};

(module.exports = function (O, key, val, safe) {
  var isFunction = typeof val == 'function';
  if (isFunction) _has(val, 'name') || _hide(val, 'name', key);
  if (O[key] === val) return;
  if (isFunction) _has(val, SRC) || _hide(val, SRC, O[key] ? '' + O[key] : TPL.join(String(key)));
  if (O === _global) {
    O[key] = val;
  } else if (!safe) {
    delete O[key];
    _hide(O, key, val);
  } else if (O[key]) {
    O[key] = val;
  } else {
    _hide(O, key, val);
  }
// add fake Function#toString for correct work wrapped methods / constructors with methods like LoDash isNative
})(Function.prototype, TO_STRING, function toString() {
  return typeof this == 'function' && this[SRC] || $toString.call(this);
});
});

var _aFunction = function (it) {
  if (typeof it != 'function') throw TypeError(it + ' is not a function!');
  return it;
};

// optional / simple context binding

var _ctx = function (fn, that, length) {
  _aFunction(fn);
  if (that === undefined) return fn;
  switch (length) {
    case 1: return function (a) {
      return fn.call(that, a);
    };
    case 2: return function (a, b) {
      return fn.call(that, a, b);
    };
    case 3: return function (a, b, c) {
      return fn.call(that, a, b, c);
    };
  }
  return function (/* ...args */) {
    return fn.apply(that, arguments);
  };
};

var PROTOTYPE = 'prototype';

var $export = function (type, name, source) {
  var IS_FORCED = type & $export.F;
  var IS_GLOBAL = type & $export.G;
  var IS_STATIC = type & $export.S;
  var IS_PROTO = type & $export.P;
  var IS_BIND = type & $export.B;
  var target = IS_GLOBAL ? _global : IS_STATIC ? _global[name] || (_global[name] = {}) : (_global[name] || {})[PROTOTYPE];
  var exports = IS_GLOBAL ? _core : _core[name] || (_core[name] = {});
  var expProto = exports[PROTOTYPE] || (exports[PROTOTYPE] = {});
  var key, own, out, exp;
  if (IS_GLOBAL) source = name;
  for (key in source) {
    // contains in native
    own = !IS_FORCED && target && target[key] !== undefined;
    // export native or passed
    out = (own ? target : source)[key];
    // bind timers to global for call from export context
    exp = IS_BIND && own ? _ctx(out, _global) : IS_PROTO && typeof out == 'function' ? _ctx(Function.call, out) : out;
    // extend global
    if (target) _redefine(target, key, out, type & $export.U);
    // export
    if (exports[key] != out) _hide(exports, key, exp);
    if (IS_PROTO && expProto[key] != out) expProto[key] = out;
  }
};
_global.core = _core;
// type bitmap
$export.F = 1;   // forced
$export.G = 2;   // global
$export.S = 4;   // static
$export.P = 8;   // proto
$export.B = 16;  // bind
$export.W = 32;  // wrap
$export.U = 64;  // safe
$export.R = 128; // real proto method for `library`
var _export = $export;

// 7.1.4 ToInteger
var ceil = Math.ceil;
var floor = Math.floor;
var _toInteger = function (it) {
  return isNaN(it = +it) ? 0 : (it > 0 ? floor : ceil)(it);
};

// 7.1.15 ToLength

var min = Math.min;
var _toLength = function (it) {
  return it > 0 ? min(_toInteger(it), 0x1fffffffffffff) : 0; // pow(2, 53) - 1 == 9007199254740991
};

var max = Math.max;
var min$1 = Math.min;
var _toAbsoluteIndex = function (index, length) {
  index = _toInteger(index);
  return index < 0 ? max(index + length, 0) : min$1(index, length);
};

// false -> Array#indexOf
// true  -> Array#includes



var _arrayIncludes = function (IS_INCLUDES) {
  return function ($this, el, fromIndex) {
    var O = _toIobject($this);
    var length = _toLength(O.length);
    var index = _toAbsoluteIndex(fromIndex, length);
    var value;
    // Array#includes uses SameValueZero equality algorithm
    // eslint-disable-next-line no-self-compare
    if (IS_INCLUDES && el != el) while (length > index) {
      value = O[index++];
      // eslint-disable-next-line no-self-compare
      if (value != value) return true;
    // Array#indexOf ignores holes, Array#includes - not
    } else for (;length > index; index++) if (IS_INCLUDES || index in O) {
      if (O[index] === el) return IS_INCLUDES || index || 0;
    } return !IS_INCLUDES && -1;
  };
};

var shared = _shared('keys');

var _sharedKey = function (key) {
  return shared[key] || (shared[key] = _uid(key));
};

var arrayIndexOf = _arrayIncludes(false);
var IE_PROTO = _sharedKey('IE_PROTO');

var _objectKeysInternal = function (object, names) {
  var O = _toIobject(object);
  var i = 0;
  var result = [];
  var key;
  for (key in O) if (key != IE_PROTO) _has(O, key) && result.push(key);
  // Don't enum bug & hidden keys
  while (names.length > i) if (_has(O, key = names[i++])) {
    ~arrayIndexOf(result, key) || result.push(key);
  }
  return result;
};

// IE 8- don't enum bug keys
var _enumBugKeys = (
  'constructor,hasOwnProperty,isPrototypeOf,propertyIsEnumerable,toLocaleString,toString,valueOf'
).split(',');

// 19.1.2.14 / 15.2.3.14 Object.keys(O)



var _objectKeys = Object.keys || function keys(O) {
  return _objectKeysInternal(O, _enumBugKeys);
};

var _objectDps = _descriptors ? Object.defineProperties : function defineProperties(O, Properties) {
  _anObject(O);
  var keys = _objectKeys(Properties);
  var length = keys.length;
  var i = 0;
  var P;
  while (length > i) _objectDp.f(O, P = keys[i++], Properties[P]);
  return O;
};

var document$2 = _global.document;
var _html = document$2 && document$2.documentElement;

// 19.1.2.2 / 15.2.3.5 Object.create(O [, Properties])



var IE_PROTO$1 = _sharedKey('IE_PROTO');
var Empty = function () { /* empty */ };
var PROTOTYPE$1 = 'prototype';

// Create object with fake `null` prototype: use iframe Object with cleared prototype
var createDict = function () {
  // Thrash, waste and sodomy: IE GC bug
  var iframe = _domCreate('iframe');
  var i = _enumBugKeys.length;
  var lt = '<';
  var gt = '>';
  var iframeDocument;
  iframe.style.display = 'none';
  _html.appendChild(iframe);
  iframe.src = 'javascript:'; // eslint-disable-line no-script-url
  // createDict = iframe.contentWindow.Object;
  // html.removeChild(iframe);
  iframeDocument = iframe.contentWindow.document;
  iframeDocument.open();
  iframeDocument.write(lt + 'script' + gt + 'document.F=Object' + lt + '/script' + gt);
  iframeDocument.close();
  createDict = iframeDocument.F;
  while (i--) delete createDict[PROTOTYPE$1][_enumBugKeys[i]];
  return createDict();
};

var _objectCreate = Object.create || function create(O, Properties) {
  var result;
  if (O !== null) {
    Empty[PROTOTYPE$1] = _anObject(O);
    result = new Empty();
    Empty[PROTOTYPE$1] = null;
    // add "__proto__" for Object.getPrototypeOf polyfill
    result[IE_PROTO$1] = O;
  } else result = createDict();
  return Properties === undefined ? result : _objectDps(result, Properties);
};

var def = _objectDp.f;

var TAG = _wks('toStringTag');

var _setToStringTag = function (it, tag, stat) {
  if (it && !_has(it = stat ? it : it.prototype, TAG)) def(it, TAG, { configurable: true, value: tag });
};

var IteratorPrototype = {};

// 25.1.2.1.1 %IteratorPrototype%[@@iterator]()
_hide(IteratorPrototype, _wks('iterator'), function () { return this; });

var _iterCreate = function (Constructor, NAME, next) {
  Constructor.prototype = _objectCreate(IteratorPrototype, { next: _propertyDesc(1, next) });
  _setToStringTag(Constructor, NAME + ' Iterator');
};

// 7.1.13 ToObject(argument)

var _toObject = function (it) {
  return Object(_defined(it));
};

// 19.1.2.9 / 15.2.3.2 Object.getPrototypeOf(O)


var IE_PROTO$2 = _sharedKey('IE_PROTO');
var ObjectProto = Object.prototype;

var _objectGpo = Object.getPrototypeOf || function (O) {
  O = _toObject(O);
  if (_has(O, IE_PROTO$2)) return O[IE_PROTO$2];
  if (typeof O.constructor == 'function' && O instanceof O.constructor) {
    return O.constructor.prototype;
  } return O instanceof Object ? ObjectProto : null;
};

var ITERATOR = _wks('iterator');
var BUGGY = !([].keys && 'next' in [].keys()); // Safari has buggy iterators w/o `next`
var FF_ITERATOR = '@@iterator';
var KEYS = 'keys';
var VALUES = 'values';

var returnThis = function () { return this; };

var _iterDefine = function (Base, NAME, Constructor, next, DEFAULT, IS_SET, FORCED) {
  _iterCreate(Constructor, NAME, next);
  var getMethod = function (kind) {
    if (!BUGGY && kind in proto) return proto[kind];
    switch (kind) {
      case KEYS: return function keys() { return new Constructor(this, kind); };
      case VALUES: return function values() { return new Constructor(this, kind); };
    } return function entries() { return new Constructor(this, kind); };
  };
  var TAG = NAME + ' Iterator';
  var DEF_VALUES = DEFAULT == VALUES;
  var VALUES_BUG = false;
  var proto = Base.prototype;
  var $native = proto[ITERATOR] || proto[FF_ITERATOR] || DEFAULT && proto[DEFAULT];
  var $default = $native || getMethod(DEFAULT);
  var $entries = DEFAULT ? !DEF_VALUES ? $default : getMethod('entries') : undefined;
  var $anyNative = NAME == 'Array' ? proto.entries || $native : $native;
  var methods, key, IteratorPrototype;
  // Fix native
  if ($anyNative) {
    IteratorPrototype = _objectGpo($anyNative.call(new Base()));
    if (IteratorPrototype !== Object.prototype && IteratorPrototype.next) {
      // Set @@toStringTag to native iterators
      _setToStringTag(IteratorPrototype, TAG, true);
      // fix for some old engines
      if (!_library && typeof IteratorPrototype[ITERATOR] != 'function') _hide(IteratorPrototype, ITERATOR, returnThis);
    }
  }
  // fix Array#{values, @@iterator}.name in V8 / FF
  if (DEF_VALUES && $native && $native.name !== VALUES) {
    VALUES_BUG = true;
    $default = function values() { return $native.call(this); };
  }
  // Define iterator
  if ((!_library || FORCED) && (BUGGY || VALUES_BUG || !proto[ITERATOR])) {
    _hide(proto, ITERATOR, $default);
  }
  // Plug for library
  _iterators[NAME] = $default;
  _iterators[TAG] = returnThis;
  if (DEFAULT) {
    methods = {
      values: DEF_VALUES ? $default : getMethod(VALUES),
      keys: IS_SET ? $default : getMethod(KEYS),
      entries: $entries
    };
    if (FORCED) for (key in methods) {
      if (!(key in proto)) _redefine(proto, key, methods[key]);
    } else _export(_export.P + _export.F * (BUGGY || VALUES_BUG), NAME, methods);
  }
  return methods;
};

// 22.1.3.4 Array.prototype.entries()
// 22.1.3.13 Array.prototype.keys()
// 22.1.3.29 Array.prototype.values()
// 22.1.3.30 Array.prototype[@@iterator]()
var es6_array_iterator = _iterDefine(Array, 'Array', function (iterated, kind) {
  this._t = _toIobject(iterated); // target
  this._i = 0;                   // next index
  this._k = kind;                // kind
// 22.1.5.2.1 %ArrayIteratorPrototype%.next()
}, function () {
  var O = this._t;
  var kind = this._k;
  var index = this._i++;
  if (!O || index >= O.length) {
    this._t = undefined;
    return _iterStep(1);
  }
  if (kind == 'keys') return _iterStep(0, index);
  if (kind == 'values') return _iterStep(0, O[index]);
  return _iterStep(0, [index, O[index]]);
}, 'values');

// argumentsList[@@iterator] is %ArrayProto_values% (9.4.4.6, 9.4.4.7)
_iterators.Arguments = _iterators.Array;

_addToUnscopables('keys');
_addToUnscopables('values');
_addToUnscopables('entries');

var ITERATOR$1 = _wks('iterator');
var TO_STRING_TAG = _wks('toStringTag');
var ArrayValues = _iterators.Array;

var DOMIterables = {
  CSSRuleList: true, // TODO: Not spec compliant, should be false.
  CSSStyleDeclaration: false,
  CSSValueList: false,
  ClientRectList: false,
  DOMRectList: false,
  DOMStringList: false,
  DOMTokenList: true,
  DataTransferItemList: false,
  FileList: false,
  HTMLAllCollection: false,
  HTMLCollection: false,
  HTMLFormElement: false,
  HTMLSelectElement: false,
  MediaList: true, // TODO: Not spec compliant, should be false.
  MimeTypeArray: false,
  NamedNodeMap: false,
  NodeList: true,
  PaintRequestList: false,
  Plugin: false,
  PluginArray: false,
  SVGLengthList: false,
  SVGNumberList: false,
  SVGPathSegList: false,
  SVGPointList: false,
  SVGStringList: false,
  SVGTransformList: false,
  SourceBufferList: false,
  StyleSheetList: true, // TODO: Not spec compliant, should be false.
  TextTrackCueList: false,
  TextTrackList: false,
  TouchList: false
};

for (var collections = _objectKeys(DOMIterables), i = 0; i < collections.length; i++) {
  var NAME = collections[i];
  var explicit = DOMIterables[NAME];
  var Collection = _global[NAME];
  var proto = Collection && Collection.prototype;
  var key;
  if (proto) {
    if (!proto[ITERATOR$1]) _hide(proto, ITERATOR$1, ArrayValues);
    if (!proto[TO_STRING_TAG]) _hide(proto, TO_STRING_TAG, NAME);
    _iterators[NAME] = ArrayValues;
    if (explicit) for (key in es6_array_iterator) if (!proto[key]) _redefine(proto, key, es6_array_iterator[key], true);
  }
}

var f$1 = Object.getOwnPropertySymbols;

var _objectGops = {
	f: f$1
};

var f$2 = {}.propertyIsEnumerable;

var _objectPie = {
	f: f$2
};

// 19.1.2.1 Object.assign(target, source, ...)





var $assign = Object.assign;

// should work with symbols and should have deterministic property order (V8 bug)
var _objectAssign = !$assign || _fails(function () {
  var A = {};
  var B = {};
  // eslint-disable-next-line no-undef
  var S = Symbol();
  var K = 'abcdefghijklmnopqrst';
  A[S] = 7;
  K.split('').forEach(function (k) { B[k] = k; });
  return $assign({}, A)[S] != 7 || Object.keys($assign({}, B)).join('') != K;
}) ? function assign(target, source) { // eslint-disable-line no-unused-vars
  var T = _toObject(target);
  var aLen = arguments.length;
  var index = 1;
  var getSymbols = _objectGops.f;
  var isEnum = _objectPie.f;
  while (aLen > index) {
    var S = _iobject(arguments[index++]);
    var keys = getSymbols ? _objectKeys(S).concat(getSymbols(S)) : _objectKeys(S);
    var length = keys.length;
    var j = 0;
    var key;
    while (length > j) if (isEnum.call(S, key = keys[j++])) T[key] = S[key];
  } return T;
} : $assign;

// 19.1.3.1 Object.assign(target, source)


_export(_export.S + _export.F, 'Object', { assign: _objectAssign });

/*jshint worker: true*/
// Mark thread as main or worker
var Thread = {};

try {
  if (window instanceof Window && window.document instanceof HTMLDocument) {
    // jshint ignore:line
    Thread.is_worker = false;
    Thread.is_main = true;
  }
} catch (e) {
  Thread.is_worker = true;
  Thread.is_main = false; // Patch for 3rd party libs that require these globals to be present. Specifically, FontFaceObserver.
  // Brittle solution but allows that library to load on worker threads.

  self.window = {
    document: {}
  };
  self.document = self.window.document;
}

// true  -> String#at
// false -> String#codePointAt
var _stringAt = function (TO_STRING) {
  return function (that, pos) {
    var s = String(_defined(that));
    var i = _toInteger(pos);
    var l = s.length;
    var a, b;
    if (i < 0 || i >= l) return TO_STRING ? '' : undefined;
    a = s.charCodeAt(i);
    return a < 0xd800 || a > 0xdbff || i + 1 === l || (b = s.charCodeAt(i + 1)) < 0xdc00 || b > 0xdfff
      ? TO_STRING ? s.charAt(i) : a
      : TO_STRING ? s.slice(i, i + 2) : (a - 0xd800 << 10) + (b - 0xdc00) + 0x10000;
  };
};

var at = _stringAt(true);

 // `AdvanceStringIndex` abstract operation
// https://tc39.github.io/ecma262/#sec-advancestringindex
var _advanceStringIndex = function (S, index, unicode) {
  return index + (unicode ? at(S, index).length : 1);
};

// getting tag from 19.1.3.6 Object.prototype.toString()

var TAG$1 = _wks('toStringTag');
// ES3 wrong here
var ARG = _cof(function () { return arguments; }()) == 'Arguments';

// fallback for IE11 Script Access Denied error
var tryGet = function (it, key) {
  try {
    return it[key];
  } catch (e) { /* empty */ }
};

var _classof = function (it) {
  var O, T, B;
  return it === undefined ? 'Undefined' : it === null ? 'Null'
    // @@toStringTag case
    : typeof (T = tryGet(O = Object(it), TAG$1)) == 'string' ? T
    // builtinTag case
    : ARG ? _cof(O)
    // ES3 arguments fallback
    : (B = _cof(O)) == 'Object' && typeof O.callee == 'function' ? 'Arguments' : B;
};

var builtinExec = RegExp.prototype.exec;

 // `RegExpExec` abstract operation
// https://tc39.github.io/ecma262/#sec-regexpexec
var _regexpExecAbstract = function (R, S) {
  var exec = R.exec;
  if (typeof exec === 'function') {
    var result = exec.call(R, S);
    if (typeof result !== 'object') {
      throw new TypeError('RegExp exec method returned something other than an Object or null');
    }
    return result;
  }
  if (_classof(R) !== 'RegExp') {
    throw new TypeError('RegExp#exec called on incompatible receiver');
  }
  return builtinExec.call(R, S);
};

// 21.2.5.3 get RegExp.prototype.flags

var _flags = function () {
  var that = _anObject(this);
  var result = '';
  if (that.global) result += 'g';
  if (that.ignoreCase) result += 'i';
  if (that.multiline) result += 'm';
  if (that.unicode) result += 'u';
  if (that.sticky) result += 'y';
  return result;
};

var nativeExec = RegExp.prototype.exec;
// This always refers to the native implementation, because the
// String#replace polyfill uses ./fix-regexp-well-known-symbol-logic.js,
// which loads this file before patching the method.
var nativeReplace = String.prototype.replace;

var patchedExec = nativeExec;

var LAST_INDEX = 'lastIndex';

var UPDATES_LAST_INDEX_WRONG = (function () {
  var re1 = /a/,
      re2 = /b*/g;
  nativeExec.call(re1, 'a');
  nativeExec.call(re2, 'a');
  return re1[LAST_INDEX] !== 0 || re2[LAST_INDEX] !== 0;
})();

// nonparticipating capturing group, copied from es5-shim's String#split patch.
var NPCG_INCLUDED = /()??/.exec('')[1] !== undefined;

var PATCH = UPDATES_LAST_INDEX_WRONG || NPCG_INCLUDED;

if (PATCH) {
  patchedExec = function exec(str) {
    var re = this;
    var lastIndex, reCopy, match, i;

    if (NPCG_INCLUDED) {
      reCopy = new RegExp('^' + re.source + '$(?!\\s)', _flags.call(re));
    }
    if (UPDATES_LAST_INDEX_WRONG) lastIndex = re[LAST_INDEX];

    match = nativeExec.call(re, str);

    if (UPDATES_LAST_INDEX_WRONG && match) {
      re[LAST_INDEX] = re.global ? match.index + match[0].length : lastIndex;
    }
    if (NPCG_INCLUDED && match && match.length > 1) {
      // Fix browsers whose `exec` methods don't consistently return `undefined`
      // for NPCG, like IE8. NOTE: This doesn' work for /(.?)?/
      // eslint-disable-next-line no-loop-func
      nativeReplace.call(match[0], reCopy, function () {
        for (i = 1; i < arguments.length - 2; i++) {
          if (arguments[i] === undefined) match[i] = undefined;
        }
      });
    }

    return match;
  };
}

var _regexpExec = patchedExec;

_export({
  target: 'RegExp',
  proto: true,
  forced: _regexpExec !== /./.exec
}, {
  exec: _regexpExec
});

var SPECIES = _wks('species');

var REPLACE_SUPPORTS_NAMED_GROUPS = !_fails(function () {
  // #replace needs built-in support for named groups.
  // #match works fine because it just return the exec results, even if it has
  // a "grops" property.
  var re = /./;
  re.exec = function () {
    var result = [];
    result.groups = { a: '7' };
    return result;
  };
  return ''.replace(re, '$<a>') !== '7';
});

var SPLIT_WORKS_WITH_OVERWRITTEN_EXEC = (function () {
  // Chrome 51 has a buggy "split" implementation when RegExp#exec !== nativeExec
  var re = /(?:)/;
  var originalExec = re.exec;
  re.exec = function () { return originalExec.apply(this, arguments); };
  var result = 'ab'.split(re);
  return result.length === 2 && result[0] === 'a' && result[1] === 'b';
})();

var _fixReWks = function (KEY, length, exec) {
  var SYMBOL = _wks(KEY);

  var DELEGATES_TO_SYMBOL = !_fails(function () {
    // String methods call symbol-named RegEp methods
    var O = {};
    O[SYMBOL] = function () { return 7; };
    return ''[KEY](O) != 7;
  });

  var DELEGATES_TO_EXEC = DELEGATES_TO_SYMBOL ? !_fails(function () {
    // Symbol-named RegExp methods call .exec
    var execCalled = false;
    var re = /a/;
    re.exec = function () { execCalled = true; return null; };
    if (KEY === 'split') {
      // RegExp[@@split] doesn't call the regex's exec method, but first creates
      // a new one. We need to return the patched regex when creating the new one.
      re.constructor = {};
      re.constructor[SPECIES] = function () { return re; };
    }
    re[SYMBOL]('');
    return !execCalled;
  }) : undefined;

  if (
    !DELEGATES_TO_SYMBOL ||
    !DELEGATES_TO_EXEC ||
    (KEY === 'replace' && !REPLACE_SUPPORTS_NAMED_GROUPS) ||
    (KEY === 'split' && !SPLIT_WORKS_WITH_OVERWRITTEN_EXEC)
  ) {
    var nativeRegExpMethod = /./[SYMBOL];
    var fns = exec(
      _defined,
      SYMBOL,
      ''[KEY],
      function maybeCallNative(nativeMethod, regexp, str, arg2, forceStringMethod) {
        if (regexp.exec === _regexpExec) {
          if (DELEGATES_TO_SYMBOL && !forceStringMethod) {
            // The native String method already delegates to @@method (this
            // polyfilled function), leasing to infinite recursion.
            // We avoid it by directly calling the native @@method method.
            return { done: true, value: nativeRegExpMethod.call(regexp, str, arg2) };
          }
          return { done: true, value: nativeMethod.call(str, regexp, arg2) };
        }
        return { done: false };
      }
    );
    var strfn = fns[0];
    var rxfn = fns[1];

    _redefine(String.prototype, KEY, strfn);
    _hide(RegExp.prototype, SYMBOL, length == 2
      // 21.2.5.8 RegExp.prototype[@@replace](string, replaceValue)
      // 21.2.5.11 RegExp.prototype[@@split](string, limit)
      ? function (string, arg) { return rxfn.call(string, this, arg); }
      // 21.2.5.6 RegExp.prototype[@@match](string)
      // 21.2.5.9 RegExp.prototype[@@search](string)
      : function (string) { return rxfn.call(string, this); }
    );
  }
};

var max$1 = Math.max;
var min$2 = Math.min;
var floor$1 = Math.floor;
var SUBSTITUTION_SYMBOLS = /\$([$&`']|\d\d?|<[^>]*>)/g;
var SUBSTITUTION_SYMBOLS_NO_NAMED = /\$([$&`']|\d\d?)/g;

var maybeToString = function (it) {
  return it === undefined ? it : String(it);
};

// @@replace logic
_fixReWks('replace', 2, function (defined, REPLACE, $replace, maybeCallNative) {
  return [
    // `String.prototype.replace` method
    // https://tc39.github.io/ecma262/#sec-string.prototype.replace
    function replace(searchValue, replaceValue) {
      var O = defined(this);
      var fn = searchValue == undefined ? undefined : searchValue[REPLACE];
      return fn !== undefined
        ? fn.call(searchValue, O, replaceValue)
        : $replace.call(String(O), searchValue, replaceValue);
    },
    // `RegExp.prototype[@@replace]` method
    // https://tc39.github.io/ecma262/#sec-regexp.prototype-@@replace
    function (regexp, replaceValue) {
      var res = maybeCallNative($replace, regexp, this, replaceValue);
      if (res.done) return res.value;

      var rx = _anObject(regexp);
      var S = String(this);
      var functionalReplace = typeof replaceValue === 'function';
      if (!functionalReplace) replaceValue = String(replaceValue);
      var global = rx.global;
      if (global) {
        var fullUnicode = rx.unicode;
        rx.lastIndex = 0;
      }
      var results = [];
      while (true) {
        var result = _regexpExecAbstract(rx, S);
        if (result === null) break;
        results.push(result);
        if (!global) break;
        var matchStr = String(result[0]);
        if (matchStr === '') rx.lastIndex = _advanceStringIndex(S, _toLength(rx.lastIndex), fullUnicode);
      }
      var accumulatedResult = '';
      var nextSourcePosition = 0;
      for (var i = 0; i < results.length; i++) {
        result = results[i];
        var matched = String(result[0]);
        var position = max$1(min$2(_toInteger(result.index), S.length), 0);
        var captures = [];
        // NOTE: This is equivalent to
        //   captures = result.slice(1).map(maybeToString)
        // but for some reason `nativeSlice.call(result, 1, result.length)` (called in
        // the slice polyfill when slicing native arrays) "doesn't work" in safari 9 and
        // causes a crash (https://pastebin.com/N21QzeQA) when trying to debug it.
        for (var j = 1; j < result.length; j++) captures.push(maybeToString(result[j]));
        var namedCaptures = result.groups;
        if (functionalReplace) {
          var replacerArgs = [matched].concat(captures, position, S);
          if (namedCaptures !== undefined) replacerArgs.push(namedCaptures);
          var replacement = String(replaceValue.apply(undefined, replacerArgs));
        } else {
          replacement = getSubstitution(matched, S, position, captures, namedCaptures, replaceValue);
        }
        if (position >= nextSourcePosition) {
          accumulatedResult += S.slice(nextSourcePosition, position) + replacement;
          nextSourcePosition = position + matched.length;
        }
      }
      return accumulatedResult + S.slice(nextSourcePosition);
    }
  ];

    // https://tc39.github.io/ecma262/#sec-getsubstitution
  function getSubstitution(matched, str, position, captures, namedCaptures, replacement) {
    var tailPos = position + matched.length;
    var m = captures.length;
    var symbols = SUBSTITUTION_SYMBOLS_NO_NAMED;
    if (namedCaptures !== undefined) {
      namedCaptures = _toObject(namedCaptures);
      symbols = SUBSTITUTION_SYMBOLS;
    }
    return $replace.call(replacement, symbols, function (match, ch) {
      var capture;
      switch (ch.charAt(0)) {
        case '$': return '$';
        case '&': return matched;
        case '`': return str.slice(0, position);
        case "'": return str.slice(tailPos);
        case '<':
          capture = namedCaptures[ch.slice(1, -1)];
          break;
        default: // \d\d?
          var n = +ch;
          if (n === 0) return match;
          if (n > m) {
            var f = floor$1(n / 10);
            if (f === 0) return match;
            if (f <= m) return captures[f - 1] === undefined ? ch.charAt(1) : captures[f - 1] + ch.charAt(1);
            return match;
          }
          capture = captures[n - 1];
      }
      return capture === undefined ? '' : capture;
    });
  }
});

var _strictMethod = function (method, arg) {
  return !!method && _fails(function () {
    // eslint-disable-next-line no-useless-call
    arg ? method.call(null, function () { /* empty */ }, 1) : method.call(null);
  });
};

var $sort = [].sort;
var test = [1, 2, 3];

_export(_export.P + _export.F * (_fails(function () {
  // IE8-
  test.sort(undefined);
}) || !_fails(function () {
  // V8 bug
  test.sort(null);
  // Old WebKit
}) || !_strictMethod($sort)), 'Array', {
  // 22.1.3.25 Array.prototype.sort(comparefn)
  sort: function sort(comparefn) {
    return comparefn === undefined
      ? $sort.call(_toObject(this))
      : $sort.call(_toObject(this), _aFunction(comparefn));
  }
});

var _meta = createCommonjsModule(function (module) {
var META = _uid('meta');


var setDesc = _objectDp.f;
var id = 0;
var isExtensible = Object.isExtensible || function () {
  return true;
};
var FREEZE = !_fails(function () {
  return isExtensible(Object.preventExtensions({}));
});
var setMeta = function (it) {
  setDesc(it, META, { value: {
    i: 'O' + ++id, // object ID
    w: {}          // weak collections IDs
  } });
};
var fastKey = function (it, create) {
  // return primitive with prefix
  if (!_isObject(it)) return typeof it == 'symbol' ? it : (typeof it == 'string' ? 'S' : 'P') + it;
  if (!_has(it, META)) {
    // can't set metadata to uncaught frozen object
    if (!isExtensible(it)) return 'F';
    // not necessary to add metadata
    if (!create) return 'E';
    // add missing metadata
    setMeta(it);
  // return object ID
  } return it[META].i;
};
var getWeak = function (it, create) {
  if (!_has(it, META)) {
    // can't set metadata to uncaught frozen object
    if (!isExtensible(it)) return true;
    // not necessary to add metadata
    if (!create) return false;
    // add missing metadata
    setMeta(it);
  // return hash weak collections IDs
  } return it[META].w;
};
// add metadata on freeze-family methods calling
var onFreeze = function (it) {
  if (FREEZE && meta.NEED && isExtensible(it) && !_has(it, META)) setMeta(it);
  return it;
};
var meta = module.exports = {
  KEY: META,
  NEED: false,
  fastKey: fastKey,
  getWeak: getWeak,
  onFreeze: onFreeze
};
});
var _meta_1 = _meta.KEY;
var _meta_2 = _meta.NEED;
var _meta_3 = _meta.fastKey;
var _meta_4 = _meta.getWeak;
var _meta_5 = _meta.onFreeze;

// 7.2.2 IsArray(argument)

var _isArray = Array.isArray || function isArray(arg) {
  return _cof(arg) == 'Array';
};

// 19.1.2.7 / 15.2.3.4 Object.getOwnPropertyNames(O)

var hiddenKeys = _enumBugKeys.concat('length', 'prototype');

var f$3 = Object.getOwnPropertyNames || function getOwnPropertyNames(O) {
  return _objectKeysInternal(O, hiddenKeys);
};

var _objectGopn = {
	f: f$3
};

var gOPD = Object.getOwnPropertyDescriptor;

var f$4 = _descriptors ? gOPD : function getOwnPropertyDescriptor(O, P) {
  O = _toIobject(O);
  P = _toPrimitive(P, true);
  if (_ie8DomDefine) try {
    return gOPD(O, P);
  } catch (e) { /* empty */ }
  if (_has(O, P)) return _propertyDesc(!_objectPie.f.call(O, P), O[P]);
};

var _objectGopd = {
	f: f$4
};

var $at = _stringAt(true);

// 21.1.3.27 String.prototype[@@iterator]()
_iterDefine(String, 'String', function (iterated) {
  this._t = String(iterated); // target
  this._i = 0;                // next index
// 21.1.5.2.1 %StringIteratorPrototype%.next()
}, function () {
  var O = this._t;
  var index = this._i;
  var point;
  if (index >= O.length) return { value: undefined, done: true };
  point = $at(O, index);
  this._i += point.length;
  return { value: point, done: false };
});

var _anInstance = function (it, Constructor, name, forbiddenField) {
  if (!(it instanceof Constructor) || (forbiddenField !== undefined && forbiddenField in it)) {
    throw TypeError(name + ': incorrect invocation!');
  } return it;
};

// call something on iterator step with safe closing on error

var _iterCall = function (iterator, fn, value, entries) {
  try {
    return entries ? fn(_anObject(value)[0], value[1]) : fn(value);
  // 7.4.6 IteratorClose(iterator, completion)
  } catch (e) {
    var ret = iterator['return'];
    if (ret !== undefined) _anObject(ret.call(iterator));
    throw e;
  }
};

// check on default Array iterator

var ITERATOR$2 = _wks('iterator');
var ArrayProto$1 = Array.prototype;

var _isArrayIter = function (it) {
  return it !== undefined && (_iterators.Array === it || ArrayProto$1[ITERATOR$2] === it);
};

var ITERATOR$3 = _wks('iterator');

var core_getIteratorMethod = _core.getIteratorMethod = function (it) {
  if (it != undefined) return it[ITERATOR$3]
    || it['@@iterator']
    || _iterators[_classof(it)];
};

var _forOf = createCommonjsModule(function (module) {
var BREAK = {};
var RETURN = {};
var exports = module.exports = function (iterable, entries, fn, that, ITERATOR) {
  var iterFn = ITERATOR ? function () { return iterable; } : core_getIteratorMethod(iterable);
  var f = _ctx(fn, that, entries ? 2 : 1);
  var index = 0;
  var length, step, iterator, result;
  if (typeof iterFn != 'function') throw TypeError(iterable + ' is not iterable!');
  // fast case for arrays with default iterator
  if (_isArrayIter(iterFn)) for (length = _toLength(iterable.length); length > index; index++) {
    result = entries ? f(_anObject(step = iterable[index])[0], step[1]) : f(iterable[index]);
    if (result === BREAK || result === RETURN) return result;
  } else for (iterator = iterFn.call(iterable); !(step = iterator.next()).done;) {
    result = _iterCall(iterator, f, step.value, entries);
    if (result === BREAK || result === RETURN) return result;
  }
};
exports.BREAK = BREAK;
exports.RETURN = RETURN;
});

// 7.3.20 SpeciesConstructor(O, defaultConstructor)


var SPECIES$1 = _wks('species');
var _speciesConstructor = function (O, D) {
  var C = _anObject(O).constructor;
  var S;
  return C === undefined || (S = _anObject(C)[SPECIES$1]) == undefined ? D : _aFunction(S);
};

// fast apply, http://jsperf.lnkit.com/fast-apply/5
var _invoke = function (fn, args, that) {
  var un = that === undefined;
  switch (args.length) {
    case 0: return un ? fn()
                      : fn.call(that);
    case 1: return un ? fn(args[0])
                      : fn.call(that, args[0]);
    case 2: return un ? fn(args[0], args[1])
                      : fn.call(that, args[0], args[1]);
    case 3: return un ? fn(args[0], args[1], args[2])
                      : fn.call(that, args[0], args[1], args[2]);
    case 4: return un ? fn(args[0], args[1], args[2], args[3])
                      : fn.call(that, args[0], args[1], args[2], args[3]);
  } return fn.apply(that, args);
};

var process = _global.process;
var setTask = _global.setImmediate;
var clearTask = _global.clearImmediate;
var MessageChannel = _global.MessageChannel;
var Dispatch = _global.Dispatch;
var counter = 0;
var queue = {};
var ONREADYSTATECHANGE = 'onreadystatechange';
var defer, channel, port;
var run = function () {
  var id = +this;
  // eslint-disable-next-line no-prototype-builtins
  if (queue.hasOwnProperty(id)) {
    var fn = queue[id];
    delete queue[id];
    fn();
  }
};
var listener = function (event) {
  run.call(event.data);
};
// Node.js 0.9+ & IE10+ has setImmediate, otherwise:
if (!setTask || !clearTask) {
  setTask = function setImmediate(fn) {
    var args = [];
    var i = 1;
    while (arguments.length > i) args.push(arguments[i++]);
    queue[++counter] = function () {
      // eslint-disable-next-line no-new-func
      _invoke(typeof fn == 'function' ? fn : Function(fn), args);
    };
    defer(counter);
    return counter;
  };
  clearTask = function clearImmediate(id) {
    delete queue[id];
  };
  // Node.js 0.8-
  if (_cof(process) == 'process') {
    defer = function (id) {
      process.nextTick(_ctx(run, id, 1));
    };
  // Sphere (JS game engine) Dispatch API
  } else if (Dispatch && Dispatch.now) {
    defer = function (id) {
      Dispatch.now(_ctx(run, id, 1));
    };
  // Browsers with MessageChannel, includes WebWorkers
  } else if (MessageChannel) {
    channel = new MessageChannel();
    port = channel.port2;
    channel.port1.onmessage = listener;
    defer = _ctx(port.postMessage, port, 1);
  // Browsers with postMessage, skip WebWorkers
  // IE8 has postMessage, but it's sync & typeof its postMessage is 'object'
  } else if (_global.addEventListener && typeof postMessage == 'function' && !_global.importScripts) {
    defer = function (id) {
      _global.postMessage(id + '', '*');
    };
    _global.addEventListener('message', listener, false);
  // IE8-
  } else if (ONREADYSTATECHANGE in _domCreate('script')) {
    defer = function (id) {
      _html.appendChild(_domCreate('script'))[ONREADYSTATECHANGE] = function () {
        _html.removeChild(this);
        run.call(id);
      };
    };
  // Rest old browsers
  } else {
    defer = function (id) {
      setTimeout(_ctx(run, id, 1), 0);
    };
  }
}
var _task = {
  set: setTask,
  clear: clearTask
};

var macrotask = _task.set;
var Observer = _global.MutationObserver || _global.WebKitMutationObserver;
var process$1 = _global.process;
var Promise$1 = _global.Promise;
var isNode = _cof(process$1) == 'process';

var _microtask = function () {
  var head, last, notify;

  var flush = function () {
    var parent, fn;
    if (isNode && (parent = process$1.domain)) parent.exit();
    while (head) {
      fn = head.fn;
      head = head.next;
      try {
        fn();
      } catch (e) {
        if (head) notify();
        else last = undefined;
        throw e;
      }
    } last = undefined;
    if (parent) parent.enter();
  };

  // Node.js
  if (isNode) {
    notify = function () {
      process$1.nextTick(flush);
    };
  // browsers with MutationObserver, except iOS Safari - https://github.com/zloirock/core-js/issues/339
  } else if (Observer && !(_global.navigator && _global.navigator.standalone)) {
    var toggle = true;
    var node = document.createTextNode('');
    new Observer(flush).observe(node, { characterData: true }); // eslint-disable-line no-new
    notify = function () {
      node.data = toggle = !toggle;
    };
  // environments with maybe non-completely correct, but existent Promise
  } else if (Promise$1 && Promise$1.resolve) {
    // Promise.resolve without an argument throws an error in LG WebOS 2
    var promise = Promise$1.resolve(undefined);
    notify = function () {
      promise.then(flush);
    };
  // for other environments - macrotask based on:
  // - setImmediate
  // - MessageChannel
  // - window.postMessag
  // - onreadystatechange
  // - setTimeout
  } else {
    notify = function () {
      // strange IE + webpack dev server bug - use .call(global)
      macrotask.call(_global, flush);
    };
  }

  return function (fn) {
    var task = { fn: fn, next: undefined };
    if (last) last.next = task;
    if (!head) {
      head = task;
      notify();
    } last = task;
  };
};

// 25.4.1.5 NewPromiseCapability(C)


function PromiseCapability(C) {
  var resolve, reject;
  this.promise = new C(function ($$resolve, $$reject) {
    if (resolve !== undefined || reject !== undefined) throw TypeError('Bad Promise constructor');
    resolve = $$resolve;
    reject = $$reject;
  });
  this.resolve = _aFunction(resolve);
  this.reject = _aFunction(reject);
}

var f$5 = function (C) {
  return new PromiseCapability(C);
};

var _newPromiseCapability = {
	f: f$5
};

var _perform = function (exec) {
  try {
    return { e: false, v: exec() };
  } catch (e) {
    return { e: true, v: e };
  }
};

var navigator$1 = _global.navigator;

var _userAgent = navigator$1 && navigator$1.userAgent || '';

var _promiseResolve = function (C, x) {
  _anObject(C);
  if (_isObject(x) && x.constructor === C) return x;
  var promiseCapability = _newPromiseCapability.f(C);
  var resolve = promiseCapability.resolve;
  resolve(x);
  return promiseCapability.promise;
};

var _redefineAll = function (target, src, safe) {
  for (var key in src) _redefine(target, key, src[key], safe);
  return target;
};

var SPECIES$2 = _wks('species');

var _setSpecies = function (KEY) {
  var C = _global[KEY];
  if (_descriptors && C && !C[SPECIES$2]) _objectDp.f(C, SPECIES$2, {
    configurable: true,
    get: function () { return this; }
  });
};

var ITERATOR$4 = _wks('iterator');
var SAFE_CLOSING = false;

try {
  var riter = [7][ITERATOR$4]();
  riter['return'] = function () { SAFE_CLOSING = true; };
} catch (e) { /* empty */ }

var _iterDetect = function (exec, skipClosing) {
  if (!skipClosing && !SAFE_CLOSING) return false;
  var safe = false;
  try {
    var arr = [7];
    var iter = arr[ITERATOR$4]();
    iter.next = function () { return { done: safe = true }; };
    arr[ITERATOR$4] = function () { return iter; };
    exec(arr);
  } catch (e) { /* empty */ }
  return safe;
};

var task = _task.set;
var microtask = _microtask();




var PROMISE = 'Promise';
var TypeError$1 = _global.TypeError;
var process$2 = _global.process;
var versions = process$2 && process$2.versions;
var v8 = versions && versions.v8 || '';
var $Promise = _global[PROMISE];
var isNode$1 = _classof(process$2) == 'process';
var empty = function () { /* empty */ };
var Internal, newGenericPromiseCapability, OwnPromiseCapability, Wrapper;
var newPromiseCapability = newGenericPromiseCapability = _newPromiseCapability.f;

var USE_NATIVE = !!function () {
  try {
    // correct subclassing with @@species support
    var promise = $Promise.resolve(1);
    var FakePromise = (promise.constructor = {})[_wks('species')] = function (exec) {
      exec(empty, empty);
    };
    // unhandled rejections tracking support, NodeJS Promise without it fails @@species test
    return (isNode$1 || typeof PromiseRejectionEvent == 'function')
      && promise.then(empty) instanceof FakePromise
      // v8 6.6 (Node 10 and Chrome 66) have a bug with resolving custom thenables
      // https://bugs.chromium.org/p/chromium/issues/detail?id=830565
      // we can't detect it synchronously, so just check versions
      && v8.indexOf('6.6') !== 0
      && _userAgent.indexOf('Chrome/66') === -1;
  } catch (e) { /* empty */ }
}();

// helpers
var isThenable = function (it) {
  var then;
  return _isObject(it) && typeof (then = it.then) == 'function' ? then : false;
};
var notify = function (promise, isReject) {
  if (promise._n) return;
  promise._n = true;
  var chain = promise._c;
  microtask(function () {
    var value = promise._v;
    var ok = promise._s == 1;
    var i = 0;
    var run = function (reaction) {
      var handler = ok ? reaction.ok : reaction.fail;
      var resolve = reaction.resolve;
      var reject = reaction.reject;
      var domain = reaction.domain;
      var result, then, exited;
      try {
        if (handler) {
          if (!ok) {
            if (promise._h == 2) onHandleUnhandled(promise);
            promise._h = 1;
          }
          if (handler === true) result = value;
          else {
            if (domain) domain.enter();
            result = handler(value); // may throw
            if (domain) {
              domain.exit();
              exited = true;
            }
          }
          if (result === reaction.promise) {
            reject(TypeError$1('Promise-chain cycle'));
          } else if (then = isThenable(result)) {
            then.call(result, resolve, reject);
          } else resolve(result);
        } else reject(value);
      } catch (e) {
        if (domain && !exited) domain.exit();
        reject(e);
      }
    };
    while (chain.length > i) run(chain[i++]); // variable length - can't use forEach
    promise._c = [];
    promise._n = false;
    if (isReject && !promise._h) onUnhandled(promise);
  });
};
var onUnhandled = function (promise) {
  task.call(_global, function () {
    var value = promise._v;
    var unhandled = isUnhandled(promise);
    var result, handler, console;
    if (unhandled) {
      result = _perform(function () {
        if (isNode$1) {
          process$2.emit('unhandledRejection', value, promise);
        } else if (handler = _global.onunhandledrejection) {
          handler({ promise: promise, reason: value });
        } else if ((console = _global.console) && console.error) {
          console.error('Unhandled promise rejection', value);
        }
      });
      // Browsers should not trigger `rejectionHandled` event if it was handled here, NodeJS - should
      promise._h = isNode$1 || isUnhandled(promise) ? 2 : 1;
    } promise._a = undefined;
    if (unhandled && result.e) throw result.v;
  });
};
var isUnhandled = function (promise) {
  return promise._h !== 1 && (promise._a || promise._c).length === 0;
};
var onHandleUnhandled = function (promise) {
  task.call(_global, function () {
    var handler;
    if (isNode$1) {
      process$2.emit('rejectionHandled', promise);
    } else if (handler = _global.onrejectionhandled) {
      handler({ promise: promise, reason: promise._v });
    }
  });
};
var $reject = function (value) {
  var promise = this;
  if (promise._d) return;
  promise._d = true;
  promise = promise._w || promise; // unwrap
  promise._v = value;
  promise._s = 2;
  if (!promise._a) promise._a = promise._c.slice();
  notify(promise, true);
};
var $resolve = function (value) {
  var promise = this;
  var then;
  if (promise._d) return;
  promise._d = true;
  promise = promise._w || promise; // unwrap
  try {
    if (promise === value) throw TypeError$1("Promise can't be resolved itself");
    if (then = isThenable(value)) {
      microtask(function () {
        var wrapper = { _w: promise, _d: false }; // wrap
        try {
          then.call(value, _ctx($resolve, wrapper, 1), _ctx($reject, wrapper, 1));
        } catch (e) {
          $reject.call(wrapper, e);
        }
      });
    } else {
      promise._v = value;
      promise._s = 1;
      notify(promise, false);
    }
  } catch (e) {
    $reject.call({ _w: promise, _d: false }, e); // wrap
  }
};

// constructor polyfill
if (!USE_NATIVE) {
  // 25.4.3.1 Promise(executor)
  $Promise = function Promise(executor) {
    _anInstance(this, $Promise, PROMISE, '_h');
    _aFunction(executor);
    Internal.call(this);
    try {
      executor(_ctx($resolve, this, 1), _ctx($reject, this, 1));
    } catch (err) {
      $reject.call(this, err);
    }
  };
  // eslint-disable-next-line no-unused-vars
  Internal = function Promise(executor) {
    this._c = [];             // <- awaiting reactions
    this._a = undefined;      // <- checked in isUnhandled reactions
    this._s = 0;              // <- state
    this._d = false;          // <- done
    this._v = undefined;      // <- value
    this._h = 0;              // <- rejection state, 0 - default, 1 - handled, 2 - unhandled
    this._n = false;          // <- notify
  };
  Internal.prototype = _redefineAll($Promise.prototype, {
    // 25.4.5.3 Promise.prototype.then(onFulfilled, onRejected)
    then: function then(onFulfilled, onRejected) {
      var reaction = newPromiseCapability(_speciesConstructor(this, $Promise));
      reaction.ok = typeof onFulfilled == 'function' ? onFulfilled : true;
      reaction.fail = typeof onRejected == 'function' && onRejected;
      reaction.domain = isNode$1 ? process$2.domain : undefined;
      this._c.push(reaction);
      if (this._a) this._a.push(reaction);
      if (this._s) notify(this, false);
      return reaction.promise;
    },
    // 25.4.5.1 Promise.prototype.catch(onRejected)
    'catch': function (onRejected) {
      return this.then(undefined, onRejected);
    }
  });
  OwnPromiseCapability = function () {
    var promise = new Internal();
    this.promise = promise;
    this.resolve = _ctx($resolve, promise, 1);
    this.reject = _ctx($reject, promise, 1);
  };
  _newPromiseCapability.f = newPromiseCapability = function (C) {
    return C === $Promise || C === Wrapper
      ? new OwnPromiseCapability(C)
      : newGenericPromiseCapability(C);
  };
}

_export(_export.G + _export.W + _export.F * !USE_NATIVE, { Promise: $Promise });
_setToStringTag($Promise, PROMISE);
_setSpecies(PROMISE);
Wrapper = _core[PROMISE];

// statics
_export(_export.S + _export.F * !USE_NATIVE, PROMISE, {
  // 25.4.4.5 Promise.reject(r)
  reject: function reject(r) {
    var capability = newPromiseCapability(this);
    var $$reject = capability.reject;
    $$reject(r);
    return capability.promise;
  }
});
_export(_export.S + _export.F * (_library || !USE_NATIVE), PROMISE, {
  // 25.4.4.6 Promise.resolve(x)
  resolve: function resolve(x) {
    return _promiseResolve(_library && this === Wrapper ? $Promise : this, x);
  }
});
_export(_export.S + _export.F * !(USE_NATIVE && _iterDetect(function (iter) {
  $Promise.all(iter)['catch'](empty);
})), PROMISE, {
  // 25.4.4.1 Promise.all(iterable)
  all: function all(iterable) {
    var C = this;
    var capability = newPromiseCapability(C);
    var resolve = capability.resolve;
    var reject = capability.reject;
    var result = _perform(function () {
      var values = [];
      var index = 0;
      var remaining = 1;
      _forOf(iterable, false, function (promise) {
        var $index = index++;
        var alreadyCalled = false;
        values.push(undefined);
        remaining++;
        C.resolve(promise).then(function (value) {
          if (alreadyCalled) return;
          alreadyCalled = true;
          values[$index] = value;
          --remaining || resolve(values);
        }, reject);
      });
      --remaining || resolve(values);
    });
    if (result.e) reject(result.v);
    return capability.promise;
  },
  // 25.4.4.4 Promise.race(iterable)
  race: function race(iterable) {
    var C = this;
    var capability = newPromiseCapability(C);
    var reject = capability.reject;
    var result = _perform(function () {
      _forOf(iterable, false, function (promise) {
        C.resolve(promise).then(capability.resolve, reject);
      });
    });
    if (result.e) reject(result.v);
    return capability.promise;
  }
});

var dP$1 = _objectDp.f;
var FProto = Function.prototype;
var nameRE = /^\s*function ([^ (]*)/;
var NAME$1 = 'name';

// 19.2.4.2 name
NAME$1 in FProto || _descriptors && dP$1(FProto, NAME$1, {
  configurable: true,
  get: function () {
    try {
      return ('' + this).match(nameRE)[1];
    } catch (e) {
      return '';
    }
  }
});

// most Object methods by ES6 should accept primitives



var _objectSap = function (KEY, exec) {
  var fn = (_core.Object || {})[KEY] || Object[KEY];
  var exp = {};
  exp[KEY] = exec(fn);
  _export(_export.S + _export.F * _fails(function () { fn(1); }), 'Object', exp);
};

// 19.1.2.14 Object.keys(O)



_objectSap('keys', function () {
  return function keys(it) {
    return _objectKeys(_toObject(it));
  };
});

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;
  subClass.__proto__ = superClass;
}

function _getPrototypeOf(o) {
  _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {
    return o.__proto__ || Object.getPrototypeOf(o);
  };
  return _getPrototypeOf(o);
}

function _setPrototypeOf(o, p) {
  _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
    o.__proto__ = p;
    return o;
  };

  return _setPrototypeOf(o, p);
}

function isNativeReflectConstruct() {
  if (typeof Reflect === "undefined" || !Reflect.construct) return false;
  if (Reflect.construct.sham) return false;
  if (typeof Proxy === "function") return true;

  try {
    Date.prototype.toString.call(Reflect.construct(Date, [], function () {}));
    return true;
  } catch (e) {
    return false;
  }
}

function _construct(Parent, args, Class) {
  if (isNativeReflectConstruct()) {
    _construct = Reflect.construct;
  } else {
    _construct = function _construct(Parent, args, Class) {
      var a = [null];
      a.push.apply(a, args);
      var Constructor = Function.bind.apply(Parent, a);
      var instance = new Constructor();
      if (Class) _setPrototypeOf(instance, Class.prototype);
      return instance;
    };
  }

  return _construct.apply(null, arguments);
}

function _isNativeFunction(fn) {
  return Function.toString.call(fn).indexOf("[native code]") !== -1;
}

function _wrapNativeSuper(Class) {
  var _cache = typeof Map === "function" ? new Map() : undefined;

  _wrapNativeSuper = function _wrapNativeSuper(Class) {
    if (Class === null || !_isNativeFunction(Class)) return Class;

    if (typeof Class !== "function") {
      throw new TypeError("Super expression must either be null or a function");
    }

    if (typeof _cache !== "undefined") {
      if (_cache.has(Class)) return _cache.get(Class);

      _cache.set(Class, Wrapper);
    }

    function Wrapper() {
      return _construct(Class, arguments, _getPrototypeOf(this).constructor);
    }

    Wrapper.prototype = Object.create(Class.prototype, {
      constructor: {
        value: Wrapper,
        enumerable: false,
        writable: true,
        configurable: true
      }
    });
    return _setPrototypeOf(Wrapper, Class);
  };

  return _wrapNativeSuper(Class);
}

var version = "0.18.1";

var version$1 = 'v' + version;

// 7.2.8 IsRegExp(argument)


var MATCH = _wks('match');
var _isRegexp = function (it) {
  var isRegExp;
  return _isObject(it) && ((isRegExp = it[MATCH]) !== undefined ? !!isRegExp : _cof(it) == 'RegExp');
};

var $min = Math.min;
var $push = [].push;
var $SPLIT = 'split';
var LENGTH = 'length';
var LAST_INDEX$1 = 'lastIndex';
var MAX_UINT32 = 0xffffffff;

// babel-minify transpiles RegExp('x', 'y') -> /x/y and it causes SyntaxError
var SUPPORTS_Y = !_fails(function () { });

// @@split logic
_fixReWks('split', 2, function (defined, SPLIT, $split, maybeCallNative) {
  var internalSplit;
  if (
    'abbc'[$SPLIT](/(b)*/)[1] == 'c' ||
    'test'[$SPLIT](/(?:)/, -1)[LENGTH] != 4 ||
    'ab'[$SPLIT](/(?:ab)*/)[LENGTH] != 2 ||
    '.'[$SPLIT](/(.?)(.?)/)[LENGTH] != 4 ||
    '.'[$SPLIT](/()()/)[LENGTH] > 1 ||
    ''[$SPLIT](/.?/)[LENGTH]
  ) {
    // based on es5-shim implementation, need to rework it
    internalSplit = function (separator, limit) {
      var string = String(this);
      if (separator === undefined && limit === 0) return [];
      // If `separator` is not a regex, use native split
      if (!_isRegexp(separator)) return $split.call(string, separator, limit);
      var output = [];
      var flags = (separator.ignoreCase ? 'i' : '') +
                  (separator.multiline ? 'm' : '') +
                  (separator.unicode ? 'u' : '') +
                  (separator.sticky ? 'y' : '');
      var lastLastIndex = 0;
      var splitLimit = limit === undefined ? MAX_UINT32 : limit >>> 0;
      // Make `global` and avoid `lastIndex` issues by working with a copy
      var separatorCopy = new RegExp(separator.source, flags + 'g');
      var match, lastIndex, lastLength;
      while (match = _regexpExec.call(separatorCopy, string)) {
        lastIndex = separatorCopy[LAST_INDEX$1];
        if (lastIndex > lastLastIndex) {
          output.push(string.slice(lastLastIndex, match.index));
          if (match[LENGTH] > 1 && match.index < string[LENGTH]) $push.apply(output, match.slice(1));
          lastLength = match[0][LENGTH];
          lastLastIndex = lastIndex;
          if (output[LENGTH] >= splitLimit) break;
        }
        if (separatorCopy[LAST_INDEX$1] === match.index) separatorCopy[LAST_INDEX$1]++; // Avoid an infinite loop
      }
      if (lastLastIndex === string[LENGTH]) {
        if (lastLength || !separatorCopy.test('')) output.push('');
      } else output.push(string.slice(lastLastIndex));
      return output[LENGTH] > splitLimit ? output.slice(0, splitLimit) : output;
    };
  // Chakra, V8
  } else if ('0'[$SPLIT](undefined, 0)[LENGTH]) {
    internalSplit = function (separator, limit) {
      return separator === undefined && limit === 0 ? [] : $split.call(this, separator, limit);
    };
  } else {
    internalSplit = $split;
  }

  return [
    // `String.prototype.split` method
    // https://tc39.github.io/ecma262/#sec-string.prototype.split
    function split(separator, limit) {
      var O = defined(this);
      var splitter = separator == undefined ? undefined : separator[SPLIT];
      return splitter !== undefined
        ? splitter.call(separator, O, limit)
        : internalSplit.call(String(O), separator, limit);
    },
    // `RegExp.prototype[@@split]` method
    // https://tc39.github.io/ecma262/#sec-regexp.prototype-@@split
    //
    // NOTE: This cannot be properly polyfilled in engines that don't support
    // the 'y' flag.
    function (regexp, limit) {
      var res = maybeCallNative(internalSplit, regexp, this, limit, internalSplit !== $split);
      if (res.done) return res.value;

      var rx = _anObject(regexp);
      var S = String(this);
      var C = _speciesConstructor(rx, RegExp);

      var unicodeMatching = rx.unicode;
      var flags = (rx.ignoreCase ? 'i' : '') +
                  (rx.multiline ? 'm' : '') +
                  (rx.unicode ? 'u' : '') +
                  (SUPPORTS_Y ? 'y' : 'g');

      // ^(? + rx + ) is needed, in combination with some S slicing, to
      // simulate the 'y' flag.
      var splitter = new C(SUPPORTS_Y ? rx : '^(?:' + rx.source + ')', flags);
      var lim = limit === undefined ? MAX_UINT32 : limit >>> 0;
      if (lim === 0) return [];
      if (S.length === 0) return _regexpExecAbstract(splitter, S) === null ? [S] : [];
      var p = 0;
      var q = 0;
      var A = [];
      while (q < S.length) {
        splitter.lastIndex = SUPPORTS_Y ? q : 0;
        var z = _regexpExecAbstract(splitter, SUPPORTS_Y ? S : S.slice(q));
        var e;
        if (
          z === null ||
          (e = $min(_toLength(splitter.lastIndex + (SUPPORTS_Y ? 0 : q)), S.length)) === p
        ) {
          q = _advanceStringIndex(S, q, unicodeMatching);
        } else {
          A.push(S.slice(p, q));
          if (A.length === lim) return A;
          for (var i = 1; i <= z.length - 1; i++) {
            A.push(z[i]);
            if (A.length === lim) return A;
          }
          q = p = e;
        }
      }
      A.push(S.slice(p));
      return A;
    }
  ];
});

var WorkerBroker;
var WorkerBroker$1 = WorkerBroker = {}; // Global list of all worker messages
// Uniquely tracks every call made between main thread and a worker

var message_id = 0;
var messages = {}; // Register an object to receive calls from other thread

WorkerBroker.targets = {};

WorkerBroker.addTarget = function (name, target) {
  WorkerBroker.targets[name] = target;
};

WorkerBroker.removeTarget = function (name) {
  if (name) {
    delete WorkerBroker.targets[name];
  }
}; // Given a dot-notation-style method name, e.g. 'Object.object.method',
// find the object to call the method on from the list of registered targets


function findTarget(method) {
  var chain = [];

  if (typeof method === 'string') {
    chain = method.split('.');
    method = chain.pop();
  }

  var target = WorkerBroker.targets;

  for (var m = 0; m < chain.length; m++) {
    if (target[chain[m]]) {
      target = target[chain[m]];
    } else {
      return [];
    }
  }

  return [method, target];
} // Main thread:
// - Send messages to workers, and optionally receive an async response as a promise
// - Receive messages from workers, and optionally send an async response back as a promise


function setupMainThread() {
  // Send a message to a worker, and optionally get an async response
  // Arguments:
  //   - worker: one or more web worker instances to send the message to (single value or array)
  //   - method: the method with this name, specified with dot-notation, will be invoked in the worker
  //   - message: spread of arguments to call the method with
  // Returns:
  //   - a promise that will be fulfilled if the worker method returns a value (could be immediately, or async)
  //
  WorkerBroker.postMessage = function (worker, method) {
    for (var _len = arguments.length, message = new Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
      message[_key - 2] = arguments[_key];
    }

    // If more than one worker specified, post to multiple
    if (Array.isArray(worker)) {
      return Promise.all(worker.map(function (w) {
        var _WorkerBroker;

        return (_WorkerBroker = WorkerBroker).postMessage.apply(_WorkerBroker, [w, method].concat(message));
      }));
    } // Parse options


    var options = {};

    if (typeof method === 'object') {
      options = method;
      method = method.method;
    } // Track state of this message


    var promise = new Promise(function (resolve, reject) {
      messages[message_id] = {
        method: method,
        message: message,
        resolve: resolve,
        reject: reject
      };
    });
    var payload,
        transferables = [];

    if (message && message.length === 1 && message[0] instanceof WorkerBroker.withTransferables) {
      transferables = message[0].transferables;
      message = message[0].value;
    }

    payload = {
      type: 'main_send',
      // mark message as method invocation from main thread
      message_id: message_id,
      // unique id for this message, for life of program
      method: method,
      // will dispatch to a function of this name within the worker
      message: message // message payload

    };

    if (options.stringify) {
      payload = JSON.stringify(payload);
    }

    worker.postMessage(payload, transferables.map(function (t) {
      return t.object;
    }));
    freeTransferables(transferables);

    if (transferables.length > 0) {
      log('trace', "'" + method + "' transferred " + transferables.length + " objects to worker thread");
    }

    message_id++;
    return promise;
  }; // Add a worker to communicate with - each worker must be registered from the main thread


  WorkerBroker.addWorker = function (worker) {
    if (!(worker instanceof Worker)) {
      throw Error('Worker broker could not add non-Worker object', worker);
    }

    worker.addEventListener('message', function WorkerBrokerMainThreadHandler(event) {
      var data = typeof event.data === 'string' ? JSON.parse(event.data) : event.data;
      var id = data.message_id; // Listen for messages coming back from the worker, and fulfill that message's promise

      if (data.type === 'worker_reply') {
        // Pass the result to the promise
        if (messages[id]) {
          if (data.error) {
            messages[id].reject(data.error);
          } else {
            messages[id].resolve(data.message);
          }

          delete messages[id];
        }
      } // Listen for messages initiating a call from the worker, dispatch them,
      // and send any return value back to the worker
      // Unique id for this message & return call to main thread
      else if (data.type === 'worker_send' && id != null) {
          // Call the requested method and save the return value
          var result, error, target, method_name, method;

          try {
            var _findTarget = findTarget(data.method);

            method_name = _findTarget[0];
            target = _findTarget[1];

            if (!target) {
              throw Error("Worker broker could not dispatch message type " + data.method + " on target " + data.target + " because no object with that name is registered on main thread");
            }

            method = typeof target[method_name] === 'function' && target[method_name];

            if (!method) {
              throw Error("Worker broker could not dispatch message type " + data.method + " on target " + data.target + " because object has no method with that name");
            }

            result = method.apply(target, data.message);
          } catch (e) {
            // Thrown errors will be passed back (in string form) to worker
            error = e;
          } // Send return value to worker


          var payload,
              transferables = []; // Async result

          if (result instanceof Promise) {
            result.then(function (value) {
              if (value instanceof WorkerBroker.withTransferables) {
                transferables = value.transferables;
                value = value.value[0];
              }

              payload = {
                type: 'main_reply',
                message_id: id,
                message: value
              };
              worker.postMessage(payload, transferables.map(function (t) {
                return t.object;
              }));
              freeTransferables(transferables);

              if (transferables.length > 0) {
                log('trace', "'" + method_name + "' transferred " + transferables.length + " objects to worker thread");
              }
            }, function (error) {
              worker.postMessage({
                type: 'main_reply',
                message_id: id,
                error: error instanceof Error ? error.message + ": " + error.stack : error
              });
            });
          } // Immediate result
          else {
              if (result instanceof WorkerBroker.withTransferables) {
                transferables = result.transferables;
                result = result.value[0];
              }

              payload = {
                type: 'main_reply',
                message_id: id,
                message: result,
                error: error instanceof Error ? error.message + ": " + error.stack : error
              };
              worker.postMessage(payload, transferables.map(function (t) {
                return t.object;
              }));
              freeTransferables(transferables);

              if (transferables.length > 0) {
                log('trace', "'" + method_name + "' transferred " + transferables.length + " objects to worker thread");
              }
            }
        }
    });
  }; // Expose for debugging


  WorkerBroker.getMessages = function () {
    return messages;
  };

  WorkerBroker.getMessageId = function () {
    return message_id;
  };
} // Worker threads:
// - Receive messages from main thread, and optionally send an async response back as a promise
// - Send messages to main thread, and optionally receive an async response as a promise


function setupWorkerThread() {
  // Send a message to the main thread, and optionally get an async response as a promise
  // Arguments:
  //   - method: the method with this name, specified with dot-notation, will be invoked on the main thread
  //   - message: array of arguments to call the method with
  // Returns:
  //   - a promise that will be fulfilled if the main thread method returns a value (could be immediately, or async)
  //
  WorkerBroker.postMessage = function (method) {
    for (var _len2 = arguments.length, message = new Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
      message[_key2 - 1] = arguments[_key2];
    }

    // Parse options
    var options = {};

    if (typeof method === 'object') {
      options = method;
      method = method.method;
    } // Track state of this message


    var promise = new Promise(function (resolve, reject) {
      messages[message_id] = {
        method: method,
        message: message,
        resolve: resolve,
        reject: reject
      };
    });
    var payload,
        transferables = [];

    if (message && message.length === 1 && message[0] instanceof WorkerBroker.withTransferables) {
      transferables = message[0].transferables;
      message = message[0].value;
    }

    payload = {
      type: 'worker_send',
      // mark message as method invocation from worker
      message_id: message_id,
      // unique id for this message, for life of program
      method: method,
      // will dispatch to a method of this name on the main thread
      message: message // message payload

    };

    if (options.stringify) {
      payload = JSON.stringify(payload);
    }

    self.postMessage(payload, transferables.map(function (t) {
      return t.object;
    }));
    freeTransferables(transferables);

    if (transferables.length > 0) {
      log('trace', "'" + method + "' transferred " + transferables.length + " objects to main thread");
    }

    message_id++;
    return promise;
  };

  self.addEventListener('message', function WorkerBrokerWorkerThreadHandler(event) {
    var data = typeof event.data === 'string' ? JSON.parse(event.data) : event.data;
    var id = data.message_id; // Listen for messages coming back from the main thread, and fulfill that message's promise

    if (data.type === 'main_reply') {
      // Pass the result to the promise
      if (messages[id]) {
        if (data.error) {
          messages[id].reject(data.error);
        } else {
          messages[id].resolve(data.message);
        }

        delete messages[id];
      }
    } // Receive messages from main thread, dispatch them, and send back a reply
    // Unique id for this message & return call to main thread
    else if (data.type === 'main_send' && id != null) {
        // Call the requested worker method and save the return value
        var result, error, target, method_name, method;

        try {
          var _findTarget2 = findTarget(data.method);

          method_name = _findTarget2[0];
          target = _findTarget2[1];

          if (!target) {
            throw Error("Worker broker could not dispatch message type " + data.method + " on target " + data.target + " because no object with that name is registered on main thread");
          }

          method = typeof target[method_name] === 'function' && target[method_name];

          if (!method) {
            throw Error("Worker broker could not dispatch message type " + data.method + " because worker has no method with that name");
          }

          result = method.apply(target, data.message);
        } catch (e) {
          // Thrown errors will be passed back (in string form) to main thread
          error = e;
        } // Send return value to main thread


        var payload,
            transferables = []; // Async result

        if (result instanceof Promise) {
          result.then(function (value) {
            if (value instanceof WorkerBroker.withTransferables) {
              transferables = value.transferables;
              value = value.value[0];
            }

            payload = {
              type: 'worker_reply',
              message_id: id,
              message: value
            };
            self.postMessage(payload, transferables.map(function (t) {
              return t.object;
            }));
            freeTransferables(transferables);

            if (transferables.length > 0) {
              log('trace', "'" + method_name + "' transferred " + transferables.length + " objects to main thread");
            }
          }, function (error) {
            self.postMessage({
              type: 'worker_reply',
              message_id: id,
              error: error instanceof Error ? error.message + ": " + error.stack : error
            });
          });
        } // Immediate result
        else {
            if (result instanceof WorkerBroker.withTransferables) {
              transferables = result.transferables;
              result = result.value[0];
            }

            payload = {
              type: 'worker_reply',
              message_id: id,
              message: result,
              error: error instanceof Error ? error.message + ": " + error.stack : error
            };
            self.postMessage(payload, transferables.map(function (t) {
              return t.object;
            }));
            freeTransferables(transferables);

            if (transferables.length > 0) {
              log('trace', "'" + method_name + "' transferred " + transferables.length + " objects to main thread");
            }
          }
      }
  });
} // Special value wrapper, to indicate that we want to find and include transferable objects in the message


WorkerBroker.withTransferables = function () {
  for (var _len3 = arguments.length, value = new Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
    value[_key3] = arguments[_key3];
  }

  if (!(this instanceof WorkerBroker.withTransferables)) {
    return _construct(WorkerBroker.withTransferables, value);
  }

  this.value = value;
  this.transferables = findTransferables(this.value);
}; // Build a list of transferable objects from a source object
// Returns a list of info about each transferable:
//   - object: the actual transferable object
//   - parent: the parent object that the transferable is a property of (if any)
//   - property: the property name of the transferable on the parent object (if any)
// TODO: add option in case you DON'T want to transfer objects


function findTransferables(source, parent, property, list) {
  if (parent === void 0) {
    parent = null;
  }

  if (property === void 0) {
    property = null;
  }

  if (list === void 0) {
    list = [];
  }

  if (!source) {
    return list;
  }

  if (Array.isArray(source)) {
    // Check each array element
    source.forEach(function (x, i) {
      return findTransferables(x, source, i, list);
    });
  } else if (typeof source === 'object') {
    // Is the object a transferable array buffer?
    if (source instanceof ArrayBuffer) {
      list.push({
        object: source,
        parent: parent,
        property: property
      });
    } // Or looks like a typed array (has an array buffer property)?
    else if (source.buffer instanceof ArrayBuffer) {
        list.push({
          object: source.buffer,
          parent: parent,
          property: property
        });
      } // Otherwise check each property
      else {
          for (var prop in source) {
            findTransferables(source[prop], source, prop, list);
          }
        }
  }

  return list;
} // Remove neutered transferables from parent objects, as they should no longer be accessed after transfer


function freeTransferables(transferables) {
  if (!Array.isArray(transferables)) {
    return;
  }

  transferables.filter(function (t) {
    return t.parent && t.property;
  }).forEach(function (t) {
    return delete t.parent[t.property];
  });
} // Setup this thread as appropriate


if (Thread.is_main) {
  setupMainThread();
}

if (Thread.is_worker) {
  setupWorkerThread();
}

var LEVELS = {
  silent: -1,
  error: 0,
  warn: 1,
  info: 2,
  debug: 3,
  trace: 4
};
var methods = {};
var logged_once = {};

function methodForLevel(level) {
  if (Thread.is_main) {
    methods[level] = methods[level] || (console[level] ? console[level] : console.log).bind(console); // eslint-disable-line no-console

    return methods[level];
  }
} // Logs message, proxying any log requests from worker threads back to the main thread.
// Returns (asynchronously, due to proxying) a boolean indicating if the message was logged.
// Option `once: true` can be used to only log each unique log message once (e.g. for warnings
// that would otherwise be repetitive or possibly logged thousands of times, such as per feature).


function log(opts) {
  var level = typeof opts === 'object' ? opts.level : opts;

  if (LEVELS[level] <= LEVELS[log.level]) {
    for (var _len = arguments.length, msg = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      msg[_key - 1] = arguments[_key];
    }

    if (Thread.is_worker) {
      // Proxy to main thread
      return WorkerBroker$1.postMessage.apply(WorkerBroker$1, [{
        method: '_logProxy',
        stringify: true
      }, opts].concat(msg));
    } else {
      // Only log message once?
      if (typeof opts === 'object' && opts.once === true) {
        if (logged_once[JSON.stringify(msg)]) {
          return Promise.resolve(false);
        }

        logged_once[JSON.stringify(msg)] = true;
      } // Write to console (on main thread)


      var logger = methodForLevel(level);

      if (msg.length > 1) {
        logger.apply(void 0, ["Tangram " + version$1 + " [" + level + "]: " + msg[0]].concat(msg.slice(1)));
      } else {
        logger("Tangram " + version$1 + " [" + level + "]: " + msg[0]);
      }
    }

    return Promise.resolve(true);
  }

  return Promise.resolve(false);
}
log.level = 'info';
log.workers = null;

log.setLevel = function (level) {
  log.level = level;

  if (Thread.is_main && Array.isArray(log.workers)) {
    WorkerBroker$1.postMessage(log.workers, '_logSetLevelProxy', level);
  }
};

if (Thread.is_main) {
  log.setWorkers = function (workers) {
    log.workers = workers;
  };

  log.reset = function () {
    logged_once = {};
  };
}

WorkerBroker$1.addTarget('_logProxy', log); // proxy log messages from worker to main thread

WorkerBroker$1.addTarget('_logSetLevelProxy', log.setLevel); // proxy log level setting from main to worker thread

// 21.2.5.3 get RegExp.prototype.flags()
if (_descriptors && /./g.flags != 'g') _objectDp.f(RegExp.prototype, 'flags', {
  configurable: true,
  get: _flags
});

var TO_STRING = 'toString';
var $toString = /./[TO_STRING];

var define = function (fn) {
  _redefine(RegExp.prototype, TO_STRING, fn, true);
};

// 21.2.5.14 RegExp.prototype.toString()
if (_fails(function () { return $toString.call({ source: 'a', flags: 'b' }) != '/a/b'; })) {
  define(function toString() {
    var R = _anObject(this);
    return '/'.concat(R.source, '/',
      'flags' in R ? R.flags : !_descriptors && R instanceof RegExp ? _flags.call(R) : undefined);
  });
// FF44- RegExp#toString has a wrong name
} else if ($toString.name != TO_STRING) {
  define(function toString() {
    return $toString.call(this);
  });
}

var Utils = {};
WorkerBroker$1.addTarget('Utils', Utils); // Basic Safari detection
// http://stackoverflow.com/questions/7944460/detect-safari-browser

Utils.isSafari = function () {
  return /^((?!chrome|android).)*safari/i.test(navigator.userAgent);
}; // Basic IE11 or Edge detection


Utils.isMicrosoft = function () {
  return /(Trident\/7.0|Edge[ /](\d+[.\d]+))/i.test(navigator.userAgent);
};

Utils._requests = {}; // XHR requests on current thread

Utils._proxy_requests = {}; // XHR requests proxied to main thread
// `request_key` is a user-provided key that can be later used to cancel the request

Utils.io = function (url, timeout, responseType, method, headers, request_key, proxy) {
  if (timeout === void 0) {
    timeout = 60000;
  }

  if (responseType === void 0) {
    responseType = 'text';
  }

  if (method === void 0) {
    method = 'GET';
  }

  if (headers === void 0) {
    headers = {};
  }

  if (request_key === void 0) {
    request_key = null;
  }

  if (proxy === void 0) {
    proxy = false;
  }

  if (Thread.is_worker && Utils.isMicrosoft()) {
    // Some versions of IE11 and Edge will hang web workers when performing XHR requests
    // These requests can be proxied through the main thread
    // https://developer.microsoft.com/en-us/microsoft-edge/platform/issues/9545866/
    log('debug', 'Proxying request for URL to worker', url);

    if (request_key) {
      Utils._proxy_requests[request_key] = true; // mark as proxied
    }

    return WorkerBroker$1.postMessage('Utils.io', url, timeout, responseType, method, headers, request_key, true);
  } else {
    var request = new XMLHttpRequest();
    var promise = new Promise(function (resolve, reject) {
      request.open(method, url, true);
      request.timeout = timeout;
      request.responseType = responseType; // Attach optional request headers

      if (headers && typeof headers === 'object') {
        for (var key in headers) {
          request.setRequestHeader(key, headers[key]);
        }
      }

      request.onload = function () {
        if (request.status === 200) {
          if (['text', 'json'].indexOf(request.responseType) > -1) {
            resolve(request.responseText);
          } else {
            resolve(request.response);
          }
        } else {
          reject(Error('Request error with a status of ' + request.statusText));
        }
      };

      request.onerror = function (evt) {
        reject(Error('There was a network error' + evt.toString()));
      };

      request.ontimeout = function (evt) {
        reject(Error('timeout ' + evt.toString()));
      };

      request.send();
    });
    promise = promise.then(function (response) {
      if (request_key) {
        delete Utils._requests[request_key];
      }

      if (proxy) {
        return WorkerBroker$1.withTransferables(response);
      }

      return response;
    });

    if (request_key) {
      Utils._requests[request_key] = request;
    }

    return promise;
  }
}; // Çancel a pending network request by user-provided request key


Utils.cancelRequest = function (key) {
  // Check for a request that was proxied to the main thread
  if (Thread.is_worker && Utils._proxy_requests[key]) {
    return WorkerBroker$1.postMessage('Utils.cancelRequest', key); // forward to main thread
  }

  var req = Utils._requests[key];

  if (req) {
    log('trace', "Cancelling network request key '" + key + "'");

    Utils._requests[key].abort();

    delete Utils._requests[key];
  } else {
    log('trace', "Could not find network request key '" + key + "'");
  }
}; // Stringify an object into JSON, but convert functions to strings


Utils.serializeWithFunctions = function (obj) {
  if (typeof obj === 'function') {
    return obj.toString();
  }

  var serialized = JSON.stringify(obj, function (k, v) {
    // Convert functions to strings
    if (typeof v === 'function') {
      return v.toString();
    }

    return v;
  });
  return serialized;
}; // Default to allowing high pixel density
// Returns true if display density changed


Utils.use_high_density_display = true;

Utils.updateDevicePixelRatio = function () {
  var prev = Utils.device_pixel_ratio;
  Utils.device_pixel_ratio = Utils.use_high_density_display && window.devicePixelRatio || 1;
  return Utils.device_pixel_ratio !== prev;
};

if (Thread.is_main) {
  Utils.updateDevicePixelRatio();
} // Used for differentiating between power-of-2 and non-power-of-2 textures
// Via: http://stackoverflow.com/questions/19722247/webgl-wait-for-texture-to-load


Utils.isPowerOf2 = function (value) {
  return (value & value - 1) === 0;
}; // Interpolate 'x' along a series of control points
// 'points' is an array of control points in the form [x, y]
//
// Example:
//     Control points:
//         [0, 5]:  when x=0, y=5
//         [4, 10]: when x=4, y=10
//
//     Utils.interpolate(2, [[0, 5], [4, 10]]);
//     -> computes x=2, halfway between x=0 and x=4: (10 - 5) / 2 +5
//     -> returns 7.5
//
// TODO: add other interpolation methods besides linear
//


Utils.interpolate = function (x, points, transform) {
  // If this doesn't resemble a list of control points, just return the original value
  if (!Array.isArray(points) || !Array.isArray(points[0])) {
    return points;
  } else if (points.length < 1) {
    return points;
  }

  var x1, x2, d, y, y1, y2; // Min bounds

  if (x <= points[0][0]) {
    y = points[0][1];

    if (typeof transform === 'function') {
      y = transform(y);
    }
  } // Max bounds
  else if (x >= points[points.length - 1][0]) {
      y = points[points.length - 1][1];

      if (typeof transform === 'function') {
        y = transform(y);
      }
    } // Find which control points x is between
    else {
        for (var i = 0; i < points.length - 1; i++) {
          if (x >= points[i][0] && x < points[i + 1][0]) {
            // Linear interpolation
            x1 = points[i][0];
            x2 = points[i + 1][0]; // Multiple values

            if (Array.isArray(points[i][1])) {
              y = [];

              for (var c = 0; c < points[i][1].length; c++) {
                if (typeof transform === 'function') {
                  y1 = transform(points[i][1][c]);
                  y2 = transform(points[i + 1][1][c]);
                  d = y2 - y1;
                  y[c] = d * (x - x1) / (x2 - x1) + y1;
                } else {
                  d = points[i + 1][1][c] - points[i][1][c];
                  y[c] = d * (x - x1) / (x2 - x1) + points[i][1][c];
                }
              }
            } // Single value
            else {
                if (typeof transform === 'function') {
                  y1 = transform(points[i][1]);
                  y2 = transform(points[i + 1][1]);
                  d = y2 - y1;
                  y = d * (x - x1) / (x2 - x1) + y1;
                } else {
                  d = points[i + 1][1] - points[i][1];
                  y = d * (x - x1) / (x2 - x1) + points[i][1];
                }
              }

            break;
          }
        }
      }

  return y;
};

Utils.toCSSColor = function (color) {
  if (color[3] === 1) {
    // full opacity
    return "rgb(" + color.slice(0, 3).map(function (c) {
      return Math.round(c * 255);
    }).join(', ') + ")";
  } // RGB is between [0, 255] opacity is between [0, 1]


  return "rgba(" + color.map(function (c, i) {
    return i < 3 && Math.round(c * 255) || c;
  }).join(', ') + ")";
};

var debugSettings;
var debugSettings$1 = debugSettings = {
  // draws a blue rectangle border around the collision box of a label
  draw_label_collision_boxes: false,
  // draws a green rectangle border within the texture box of a label
  draw_label_texture_boxes: false,
  // suppreses fade-in of labels
  suppress_label_fade_in: false,
  // suppress animaton of label snap to pixel grid
  suppress_label_snap_animation: false,
  // show hidden labels for debugging
  show_hidden_labels: false,
  // collect feature/geometry stats on styling layers
  layer_stats: false
};
function mergeDebugSettings(settings) {
  Object.assign(debugSettings, settings);
}

// Works with __proto__ only. Old v8 can't work with null proto objects.
/* eslint-disable no-proto */


var check = function (O, proto) {
  _anObject(O);
  if (!_isObject(proto) && proto !== null) throw TypeError(proto + ": can't set as prototype!");
};
var _setProto = {
  set: Object.setPrototypeOf || ('__proto__' in {} ? // eslint-disable-line
    function (test, buggy, set) {
      try {
        set = _ctx(Function.call, _objectGopd.f(Object.prototype, '__proto__').set, 2);
        set(test, []);
        buggy = !(test instanceof Array);
      } catch (e) { buggy = true; }
      return function setPrototypeOf(O, proto) {
        check(O, proto);
        if (buggy) O.__proto__ = proto;
        else set(O, proto);
        return O;
      };
    }({}, false) : undefined),
  check: check
};

var setPrototypeOf = _setProto.set;
var _inheritIfRequired = function (that, target, C) {
  var S = target.constructor;
  var P;
  if (S !== C && typeof S == 'function' && (P = S.prototype) !== C.prototype && _isObject(P) && setPrototypeOf) {
    setPrototypeOf(that, P);
  } return that;
};

var dP$2 = _objectDp.f;
var gOPN = _objectGopn.f;


var $RegExp = _global.RegExp;
var Base = $RegExp;
var proto$1 = $RegExp.prototype;
var re1 = /a/g;
var re2 = /a/g;
// "new" creates a new object, old webkit buggy here
var CORRECT_NEW = new $RegExp(re1) !== re1;

if (_descriptors && (!CORRECT_NEW || _fails(function () {
  re2[_wks('match')] = false;
  // RegExp constructor can alter flags and IsRegExp works correct with @@match
  return $RegExp(re1) != re1 || $RegExp(re2) == re2 || $RegExp(re1, 'i') != '/a/i';
}))) {
  $RegExp = function RegExp(p, f) {
    var tiRE = this instanceof $RegExp;
    var piRE = _isRegexp(p);
    var fiU = f === undefined;
    return !tiRE && piRE && p.constructor === $RegExp && fiU ? p
      : _inheritIfRequired(CORRECT_NEW
        ? new Base(piRE && !fiU ? p.source : p, f)
        : Base((piRE = p instanceof $RegExp) ? p.source : p, piRE && fiU ? _flags.call(p) : f)
      , tiRE ? this : proto$1, $RegExp);
  };
  var proxy = function (key) {
    key in $RegExp || dP$2($RegExp, key, {
      configurable: true,
      get: function () { return Base[key]; },
      set: function (it) { Base[key] = it; }
    });
  };
  for (var keys = gOPN(Base), i$1 = 0; keys.length > i$1;) proxy(keys[i$1++]);
  proto$1.constructor = $RegExp;
  $RegExp.prototype = proto$1;
  _redefine(_global, 'RegExp', $RegExp);
}

_setSpecies('RegExp');

// 7.2.9 SameValue(x, y)
var _sameValue = Object.is || function is(x, y) {
  // eslint-disable-next-line no-self-compare
  return x === y ? x !== 0 || 1 / x === 1 / y : x != x && y != y;
};

// @@search logic
_fixReWks('search', 1, function (defined, SEARCH, $search, maybeCallNative) {
  return [
    // `String.prototype.search` method
    // https://tc39.github.io/ecma262/#sec-string.prototype.search
    function search(regexp) {
      var O = defined(this);
      var fn = regexp == undefined ? undefined : regexp[SEARCH];
      return fn !== undefined ? fn.call(regexp, O) : new RegExp(regexp)[SEARCH](String(O));
    },
    // `RegExp.prototype[@@search]` method
    // https://tc39.github.io/ecma262/#sec-regexp.prototype-@@search
    function (regexp) {
      var res = maybeCallNative($search, regexp, this);
      if (res.done) return res.value;
      var rx = _anObject(regexp);
      var S = String(this);
      var previousLastIndex = rx.lastIndex;
      if (!_sameValue(previousLastIndex, 0)) rx.lastIndex = 0;
      var result = _regexpExecAbstract(rx, S);
      if (!_sameValue(rx.lastIndex, previousLastIndex)) rx.lastIndex = previousLastIndex;
      return result === null ? -1 : result.index;
    }
  ];
});

function addBaseURL(url, base) {
  if (!url || !isRelativeURL(url)) {
    return url;
  }

  var relative_path = url[0] !== '/';
  var base_info;

  if (base) {
    base_info = document.createElement('a'); // use a temporary element to parse URL

    base_info.href = base;
  } else {
    base_info = window.location;
  }

  if (relative_path) {
    var path = pathForURL(base_info.href);
    url = path + url;
  } else {
    var origin = base_info.origin;

    if (!origin) {
      origin = base_info.protocol + '//' + base_info.host; // IE11 doesn't have origin property
    }

    url = origin + url;
  }

  return url;
}
function pathForURL(url) {
  if (typeof url === 'string' && url.search(/^(data|blob):/) === -1) {
    var qs = url.indexOf('?');

    if (qs > -1) {
      url = url.substr(0, qs);
    }

    var hash = url.indexOf('#');

    if (hash > -1) {
      url = url.substr(0, hash);
    }

    return url.substr(0, url.lastIndexOf('/') + 1) || '';
  }

  return '';
}
function extensionForURL(url) {
  url = url.split('/').pop();
  var last_dot = url.lastIndexOf('.');

  if (last_dot > -1) {
    return url.substring(last_dot + 1);
  }
}
function isLocalURL(url) {
  if (typeof url !== 'string') {
    return;
  }

  return url.search(/^(data|blob):/) > -1;
}
function isRelativeURL(url) {
  if (typeof url !== 'string') {
    return;
  }

  return !(url.search(/^(http|https|data|blob):/) > -1 || url.substr(0, 2) === '//');
} // Resolves './' and '../' components from relative path, to get a "flattened" path

function flattenRelativeURL(url) {
  var dirs = (url || '').split('/');

  for (var d = 1; d < dirs.length; d++) {
    if (dirs[d] === '.') {
      dirs.splice(d, 1);
      d--;
    } else if (dirs[d] === '..') {
      d = d + 0;
      dirs.splice(d - 1, 2);
      d--;
    }
  }

  return dirs.join('/');
} // Add a set of query string params to a URL
// params: hash of key/value pairs of query string parameters
// returns array of: [modified URL, array of duplicate param name and values]

function addParamsToURL(url, params) {
  if (!params || Object.keys(params).length === 0) {
    return [url, []];
  }

  var qs_index = url.indexOf('?');
  var hash_index = url.indexOf('#'); // Save and trim hash

  var hash = '';

  if (hash_index > -1) {
    hash = url.slice(hash_index);
    url = url.slice(0, hash_index);
  } // Start query string


  if (qs_index === -1) {
    qs_index = url.length;
    url += '?';
  }

  qs_index++; // advanced past '?'
  // Build query string params

  var url_params = '';
  var dupes = [];

  for (var p in params) {
    if (getURLParameter(p, url) !== '') {
      dupes.push([p, params[p]]);
      continue;
    }

    url_params += p + "=" + params[p] + "&";
  } // Insert new query string params and restore hash


  url = url.slice(0, qs_index) + url_params + url.slice(qs_index) + hash;
  return [url, dupes];
} // Polyfill (for Safari compatibility)

var _createObjectURL;

function createObjectURL(url) {
  if (_createObjectURL === undefined) {
    _createObjectURL = window.URL && window.URL.createObjectURL || window.webkitURL && window.webkitURL.createObjectURL;

    if (typeof _createObjectURL !== 'function') {
      _createObjectURL = null;
      log('warn', 'window.URL.createObjectURL (or vendor prefix) not found, unable to create local blob URLs');
    }
  }

  if (_createObjectURL) {
    return _createObjectURL(url);
  } else {
    return url;
  }
}

function getURLParameter(name, url) {
  name = name.replace(/[[]/, '\\[').replace(/[\]]/, '\\]');
  var regex = new RegExp('[\\?&]' + name + '=([^&#]*)');
  var results = regex.exec(url);
  return results === null ? '' : decodeURIComponent(results[1].replace(/\+/g, ' '));
}

// import log from './log';
var Task = {
  id: 0,
  // unique id per task
  queue: [],
  // current queue of outstanding tasks
  max_time: 20,
  // default time in which all tasks should complete per frame
  start_time: null,
  // start time for tasks in current frame
  state: {},
  // track flags about environment state (ex: whether user is currently moving the view)
  add: function add(task) {
    task.id = Task.id++;
    task.max_time = task.max_time || Task.max_time; // allow task to run for this much time (tasks have a global collective limit per frame, too)

    task.pause_factor = task.pause_factor || 1; // pause tasks by this many frames when they run too long

    var promise = new Promise(function (resolve, reject) {
      task.resolve = resolve;
      task.reject = reject;
    });
    task.promise = promise;
    task.elapsed = 0;
    task.total_elapsed = 0;
    task.stats = {
      calls: 0
    };
    this.queue.push(task); // Run task immediately if under total frame time

    this.start_time = this.start_time || performance.now(); // start frame timer if necessary

    this.elapsed = performance.now() - this.start_time;

    if (this.elapsed < Task.max_time) {
      this.process(task);
    }

    return task.promise;
  },
  remove: function remove(task) {
    var idx = this.queue.indexOf(task);

    if (idx > -1) {
      this.queue.splice(idx, 1);
    }
  },
  process: function process(task) {
    // Skip task while user is moving the view, if the task requests it
    // (for intensive tasks that lock the UI, like canvas rasterization)
    if (this.state.user_moving_view && task.user_moving_view === false) {
      // log('debug', `*** SKIPPING task id ${task.id}, ${task.type} while user is moving view`);
      return;
    } // Skip task if it's currently paused


    if (task.pause) {
      // log('debug', `*** PAUSING task id ${task.id}, ${task.type} (${task.pause})`);
      task.pause--;
      return true;
    }

    task.stats.calls++;
    task.start_time = performance.now(); // start task timer

    return task.run(task);
  },
  processAll: function processAll() {
    this.start_time = this.start_time || performance.now(); // start frame timer if necessary

    for (var i = 0; i < this.queue.length; i++) {
      // Exceeded either total task time, or total frame time
      var task = this.queue[i];

      if (this.process(task) !== true) {
        // If the task didn't complete, pause it for a task-specific number of frames
        // (can be disabled by setting pause_factor to 0)
        if (!task.pause) {
          task.pause = task.elapsed > task.max_time ? task.pause_factor : 0;
        }

        task.total_elapsed += task.elapsed;
      } // Check total frame time


      this.elapsed = performance.now() - this.start_time;

      if (this.elapsed >= Task.max_time) {
        this.start_time = null; // reset frame timer

        break;
      }
    }
  },
  finish: function finish(task, value) {
    task.elapsed = performance.now() - task.start_time;
    task.total_elapsed += task.elapsed; // log('debug', `task type ${task.type}, tile ${task.id}, finish after ${task.stats.calls} calls, ${task.total_elapsed.toFixed(2)} elapsed`);

    this.remove(task);
    task.resolve(value);
    return task.promise;
  },
  cancel: function cancel(task) {
    var val;

    if (task.cancel instanceof Function) {
      val = task.cancel(task); // optional cancel function
    }

    task.resolve(val);
  },
  shouldContinue: function shouldContinue(task) {
    // Suspend task if it runs over its specific per-frame limit, or the global limit
    task.elapsed = performance.now() - task.start_time;
    this.elapsed = performance.now() - this.start_time;
    return task.elapsed < task.max_time && this.elapsed < Task.max_time;
  },
  removeForTile: function removeForTile(tile_id) {
    for (var idx = this.queue.length - 1; idx >= 0; idx--) {
      if (this.queue[idx].tile_id === tile_id) {
        // log('trace', `Task: remove tasks for tile ${tile_id}`);
        this.cancel(this.queue[idx]);
        this.queue.splice(idx, 1);
      }
    }
  },
  setState: function setState(state) {
    this.state = state;
  }
};

function subscribeMixin(target) {
  var listeners = [];
  return Object.assign(target, {
    subscribe: function subscribe(listener) {
      if (listeners.indexOf(listener) === -1) {
        listeners.push(listener);
      }
    },
    unsubscribe: function unsubscribe(listener) {
      var index = listeners.indexOf(listener);

      if (index > -1) {
        listeners.splice(index, 1);
      }
    },
    unsubscribeAll: function unsubscribeAll() {
      listeners = [];
    },
    trigger: function trigger(event) {
      for (var _len = arguments.length, data = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        data[_key - 1] = arguments[_key];
      }

      listeners.forEach(function (listener) {
        if (typeof listener[event] === 'function') {
          try {
            listener[event].apply(listener, data);
          } catch (e) {
            log('warn', "Caught exception in listener for event '" + event + "':", e);
          }
        }
      });
    },
    hasSubscribersFor: function hasSubscribersFor(event) {
      var has = false;
      listeners.forEach(function (listener) {
        if (typeof listener[event] === 'function') {
          has = true;
        }
      });
      return has;
    }
  });
}

function sliceObject(obj, keys) {
  var sliced = {};
  keys.forEach(function (k) {
    return sliced[k] = obj[k];
  });
  return sliced;
}

var _stringRepeat = function repeat(count) {
  var str = String(_defined(this));
  var res = '';
  var n = _toInteger(count);
  if (n < 0 || n == Infinity) throw RangeError("Count can't be negative");
  for (;n > 0; (n >>>= 1) && (str += str)) if (n & 1) res += str;
  return res;
};

_export(_export.P, 'String', {
  // 21.1.3.13 String.prototype.repeat(count)
  repeat: _stringRepeat
});

var TYPED = _uid('typed_array');
var VIEW = _uid('view');
var ABV = !!(_global.ArrayBuffer && _global.DataView);
var CONSTR = ABV;
var i$2 = 0;
var l = 9;
var Typed;

var TypedArrayConstructors = (
  'Int8Array,Uint8Array,Uint8ClampedArray,Int16Array,Uint16Array,Int32Array,Uint32Array,Float32Array,Float64Array'
).split(',');

while (i$2 < l) {
  if (Typed = _global[TypedArrayConstructors[i$2++]]) {
    _hide(Typed.prototype, TYPED, true);
    _hide(Typed.prototype, VIEW, true);
  } else CONSTR = false;
}

var _typed = {
  ABV: ABV,
  CONSTR: CONSTR,
  TYPED: TYPED,
  VIEW: VIEW
};

// https://tc39.github.io/ecma262/#sec-toindex


var _toIndex = function (it) {
  if (it === undefined) return 0;
  var number = _toInteger(it);
  var length = _toLength(number);
  if (number !== length) throw RangeError('Wrong length!');
  return length;
};

var _arrayFill = function fill(value /* , start = 0, end = @length */) {
  var O = _toObject(this);
  var length = _toLength(O.length);
  var aLen = arguments.length;
  var index = _toAbsoluteIndex(aLen > 1 ? arguments[1] : undefined, length);
  var end = aLen > 2 ? arguments[2] : undefined;
  var endPos = end === undefined ? length : _toAbsoluteIndex(end, length);
  while (endPos > index) O[index++] = value;
  return O;
};

var _typedBuffer = createCommonjsModule(function (module, exports) {











var gOPN = _objectGopn.f;
var dP = _objectDp.f;


var ARRAY_BUFFER = 'ArrayBuffer';
var DATA_VIEW = 'DataView';
var PROTOTYPE = 'prototype';
var WRONG_LENGTH = 'Wrong length!';
var WRONG_INDEX = 'Wrong index!';
var $ArrayBuffer = _global[ARRAY_BUFFER];
var $DataView = _global[DATA_VIEW];
var Math = _global.Math;
var RangeError = _global.RangeError;
// eslint-disable-next-line no-shadow-restricted-names
var Infinity = _global.Infinity;
var BaseBuffer = $ArrayBuffer;
var abs = Math.abs;
var pow = Math.pow;
var floor = Math.floor;
var log = Math.log;
var LN2 = Math.LN2;
var BUFFER = 'buffer';
var BYTE_LENGTH = 'byteLength';
var BYTE_OFFSET = 'byteOffset';
var $BUFFER = _descriptors ? '_b' : BUFFER;
var $LENGTH = _descriptors ? '_l' : BYTE_LENGTH;
var $OFFSET = _descriptors ? '_o' : BYTE_OFFSET;

// IEEE754 conversions based on https://github.com/feross/ieee754
function packIEEE754(value, mLen, nBytes) {
  var buffer = new Array(nBytes);
  var eLen = nBytes * 8 - mLen - 1;
  var eMax = (1 << eLen) - 1;
  var eBias = eMax >> 1;
  var rt = mLen === 23 ? pow(2, -24) - pow(2, -77) : 0;
  var i = 0;
  var s = value < 0 || value === 0 && 1 / value < 0 ? 1 : 0;
  var e, m, c;
  value = abs(value);
  // eslint-disable-next-line no-self-compare
  if (value != value || value === Infinity) {
    // eslint-disable-next-line no-self-compare
    m = value != value ? 1 : 0;
    e = eMax;
  } else {
    e = floor(log(value) / LN2);
    if (value * (c = pow(2, -e)) < 1) {
      e--;
      c *= 2;
    }
    if (e + eBias >= 1) {
      value += rt / c;
    } else {
      value += rt * pow(2, 1 - eBias);
    }
    if (value * c >= 2) {
      e++;
      c /= 2;
    }
    if (e + eBias >= eMax) {
      m = 0;
      e = eMax;
    } else if (e + eBias >= 1) {
      m = (value * c - 1) * pow(2, mLen);
      e = e + eBias;
    } else {
      m = value * pow(2, eBias - 1) * pow(2, mLen);
      e = 0;
    }
  }
  for (; mLen >= 8; buffer[i++] = m & 255, m /= 256, mLen -= 8);
  e = e << mLen | m;
  eLen += mLen;
  for (; eLen > 0; buffer[i++] = e & 255, e /= 256, eLen -= 8);
  buffer[--i] |= s * 128;
  return buffer;
}
function unpackIEEE754(buffer, mLen, nBytes) {
  var eLen = nBytes * 8 - mLen - 1;
  var eMax = (1 << eLen) - 1;
  var eBias = eMax >> 1;
  var nBits = eLen - 7;
  var i = nBytes - 1;
  var s = buffer[i--];
  var e = s & 127;
  var m;
  s >>= 7;
  for (; nBits > 0; e = e * 256 + buffer[i], i--, nBits -= 8);
  m = e & (1 << -nBits) - 1;
  e >>= -nBits;
  nBits += mLen;
  for (; nBits > 0; m = m * 256 + buffer[i], i--, nBits -= 8);
  if (e === 0) {
    e = 1 - eBias;
  } else if (e === eMax) {
    return m ? NaN : s ? -Infinity : Infinity;
  } else {
    m = m + pow(2, mLen);
    e = e - eBias;
  } return (s ? -1 : 1) * m * pow(2, e - mLen);
}

function unpackI32(bytes) {
  return bytes[3] << 24 | bytes[2] << 16 | bytes[1] << 8 | bytes[0];
}
function packI8(it) {
  return [it & 0xff];
}
function packI16(it) {
  return [it & 0xff, it >> 8 & 0xff];
}
function packI32(it) {
  return [it & 0xff, it >> 8 & 0xff, it >> 16 & 0xff, it >> 24 & 0xff];
}
function packF64(it) {
  return packIEEE754(it, 52, 8);
}
function packF32(it) {
  return packIEEE754(it, 23, 4);
}

function addGetter(C, key, internal) {
  dP(C[PROTOTYPE], key, { get: function () { return this[internal]; } });
}

function get(view, bytes, index, isLittleEndian) {
  var numIndex = +index;
  var intIndex = _toIndex(numIndex);
  if (intIndex + bytes > view[$LENGTH]) throw RangeError(WRONG_INDEX);
  var store = view[$BUFFER]._b;
  var start = intIndex + view[$OFFSET];
  var pack = store.slice(start, start + bytes);
  return isLittleEndian ? pack : pack.reverse();
}
function set(view, bytes, index, conversion, value, isLittleEndian) {
  var numIndex = +index;
  var intIndex = _toIndex(numIndex);
  if (intIndex + bytes > view[$LENGTH]) throw RangeError(WRONG_INDEX);
  var store = view[$BUFFER]._b;
  var start = intIndex + view[$OFFSET];
  var pack = conversion(+value);
  for (var i = 0; i < bytes; i++) store[start + i] = pack[isLittleEndian ? i : bytes - i - 1];
}

if (!_typed.ABV) {
  $ArrayBuffer = function ArrayBuffer(length) {
    _anInstance(this, $ArrayBuffer, ARRAY_BUFFER);
    var byteLength = _toIndex(length);
    this._b = _arrayFill.call(new Array(byteLength), 0);
    this[$LENGTH] = byteLength;
  };

  $DataView = function DataView(buffer, byteOffset, byteLength) {
    _anInstance(this, $DataView, DATA_VIEW);
    _anInstance(buffer, $ArrayBuffer, DATA_VIEW);
    var bufferLength = buffer[$LENGTH];
    var offset = _toInteger(byteOffset);
    if (offset < 0 || offset > bufferLength) throw RangeError('Wrong offset!');
    byteLength = byteLength === undefined ? bufferLength - offset : _toLength(byteLength);
    if (offset + byteLength > bufferLength) throw RangeError(WRONG_LENGTH);
    this[$BUFFER] = buffer;
    this[$OFFSET] = offset;
    this[$LENGTH] = byteLength;
  };

  if (_descriptors) {
    addGetter($ArrayBuffer, BYTE_LENGTH, '_l');
    addGetter($DataView, BUFFER, '_b');
    addGetter($DataView, BYTE_LENGTH, '_l');
    addGetter($DataView, BYTE_OFFSET, '_o');
  }

  _redefineAll($DataView[PROTOTYPE], {
    getInt8: function getInt8(byteOffset) {
      return get(this, 1, byteOffset)[0] << 24 >> 24;
    },
    getUint8: function getUint8(byteOffset) {
      return get(this, 1, byteOffset)[0];
    },
    getInt16: function getInt16(byteOffset /* , littleEndian */) {
      var bytes = get(this, 2, byteOffset, arguments[1]);
      return (bytes[1] << 8 | bytes[0]) << 16 >> 16;
    },
    getUint16: function getUint16(byteOffset /* , littleEndian */) {
      var bytes = get(this, 2, byteOffset, arguments[1]);
      return bytes[1] << 8 | bytes[0];
    },
    getInt32: function getInt32(byteOffset /* , littleEndian */) {
      return unpackI32(get(this, 4, byteOffset, arguments[1]));
    },
    getUint32: function getUint32(byteOffset /* , littleEndian */) {
      return unpackI32(get(this, 4, byteOffset, arguments[1])) >>> 0;
    },
    getFloat32: function getFloat32(byteOffset /* , littleEndian */) {
      return unpackIEEE754(get(this, 4, byteOffset, arguments[1]), 23, 4);
    },
    getFloat64: function getFloat64(byteOffset /* , littleEndian */) {
      return unpackIEEE754(get(this, 8, byteOffset, arguments[1]), 52, 8);
    },
    setInt8: function setInt8(byteOffset, value) {
      set(this, 1, byteOffset, packI8, value);
    },
    setUint8: function setUint8(byteOffset, value) {
      set(this, 1, byteOffset, packI8, value);
    },
    setInt16: function setInt16(byteOffset, value /* , littleEndian */) {
      set(this, 2, byteOffset, packI16, value, arguments[2]);
    },
    setUint16: function setUint16(byteOffset, value /* , littleEndian */) {
      set(this, 2, byteOffset, packI16, value, arguments[2]);
    },
    setInt32: function setInt32(byteOffset, value /* , littleEndian */) {
      set(this, 4, byteOffset, packI32, value, arguments[2]);
    },
    setUint32: function setUint32(byteOffset, value /* , littleEndian */) {
      set(this, 4, byteOffset, packI32, value, arguments[2]);
    },
    setFloat32: function setFloat32(byteOffset, value /* , littleEndian */) {
      set(this, 4, byteOffset, packF32, value, arguments[2]);
    },
    setFloat64: function setFloat64(byteOffset, value /* , littleEndian */) {
      set(this, 8, byteOffset, packF64, value, arguments[2]);
    }
  });
} else {
  if (!_fails(function () {
    $ArrayBuffer(1);
  }) || !_fails(function () {
    new $ArrayBuffer(-1); // eslint-disable-line no-new
  }) || _fails(function () {
    new $ArrayBuffer(); // eslint-disable-line no-new
    new $ArrayBuffer(1.5); // eslint-disable-line no-new
    new $ArrayBuffer(NaN); // eslint-disable-line no-new
    return $ArrayBuffer.name != ARRAY_BUFFER;
  })) {
    $ArrayBuffer = function ArrayBuffer(length) {
      _anInstance(this, $ArrayBuffer);
      return new BaseBuffer(_toIndex(length));
    };
    var ArrayBufferProto = $ArrayBuffer[PROTOTYPE] = BaseBuffer[PROTOTYPE];
    for (var keys = gOPN(BaseBuffer), j = 0, key; keys.length > j;) {
      if (!((key = keys[j++]) in $ArrayBuffer)) _hide($ArrayBuffer, key, BaseBuffer[key]);
    }
    ArrayBufferProto.constructor = $ArrayBuffer;
  }
  // iOS Safari 7.x bug
  var view = new $DataView(new $ArrayBuffer(2));
  var $setInt8 = $DataView[PROTOTYPE].setInt8;
  view.setInt8(0, 2147483648);
  view.setInt8(1, 2147483649);
  if (view.getInt8(0) || !view.getInt8(1)) _redefineAll($DataView[PROTOTYPE], {
    setInt8: function setInt8(byteOffset, value) {
      $setInt8.call(this, byteOffset, value << 24 >> 24);
    },
    setUint8: function setUint8(byteOffset, value) {
      $setInt8.call(this, byteOffset, value << 24 >> 24);
    }
  }, true);
}
_setToStringTag($ArrayBuffer, ARRAY_BUFFER);
_setToStringTag($DataView, DATA_VIEW);
_hide($DataView[PROTOTYPE], _typed.VIEW, true);
exports[ARRAY_BUFFER] = $ArrayBuffer;
exports[DATA_VIEW] = $DataView;
});

var SPECIES$3 = _wks('species');

var _arraySpeciesConstructor = function (original) {
  var C;
  if (_isArray(original)) {
    C = original.constructor;
    // cross-realm fallback
    if (typeof C == 'function' && (C === Array || _isArray(C.prototype))) C = undefined;
    if (_isObject(C)) {
      C = C[SPECIES$3];
      if (C === null) C = undefined;
    }
  } return C === undefined ? Array : C;
};

// 9.4.2.3 ArraySpeciesCreate(originalArray, length)


var _arraySpeciesCreate = function (original, length) {
  return new (_arraySpeciesConstructor(original))(length);
};

// 0 -> Array#forEach
// 1 -> Array#map
// 2 -> Array#filter
// 3 -> Array#some
// 4 -> Array#every
// 5 -> Array#find
// 6 -> Array#findIndex





var _arrayMethods = function (TYPE, $create) {
  var IS_MAP = TYPE == 1;
  var IS_FILTER = TYPE == 2;
  var IS_SOME = TYPE == 3;
  var IS_EVERY = TYPE == 4;
  var IS_FIND_INDEX = TYPE == 6;
  var NO_HOLES = TYPE == 5 || IS_FIND_INDEX;
  var create = $create || _arraySpeciesCreate;
  return function ($this, callbackfn, that) {
    var O = _toObject($this);
    var self = _iobject(O);
    var f = _ctx(callbackfn, that, 3);
    var length = _toLength(self.length);
    var index = 0;
    var result = IS_MAP ? create($this, length) : IS_FILTER ? create($this, 0) : undefined;
    var val, res;
    for (;length > index; index++) if (NO_HOLES || index in self) {
      val = self[index];
      res = f(val, index, O);
      if (TYPE) {
        if (IS_MAP) result[index] = res;   // map
        else if (res) switch (TYPE) {
          case 3: return true;             // some
          case 5: return val;              // find
          case 6: return index;            // findIndex
          case 2: result.push(val);        // filter
        } else if (IS_EVERY) return false; // every
      }
    }
    return IS_FIND_INDEX ? -1 : IS_SOME || IS_EVERY ? IS_EVERY : result;
  };
};

var _arrayCopyWithin = [].copyWithin || function copyWithin(target /* = 0 */, start /* = 0, end = @length */) {
  var O = _toObject(this);
  var len = _toLength(O.length);
  var to = _toAbsoluteIndex(target, len);
  var from = _toAbsoluteIndex(start, len);
  var end = arguments.length > 2 ? arguments[2] : undefined;
  var count = Math.min((end === undefined ? len : _toAbsoluteIndex(end, len)) - from, len - to);
  var inc = 1;
  if (from < to && to < from + count) {
    inc = -1;
    from += count - 1;
    to += count - 1;
  }
  while (count-- > 0) {
    if (from in O) O[to] = O[from];
    else delete O[to];
    to += inc;
    from += inc;
  } return O;
};

var _typedArray = createCommonjsModule(function (module) {
if (_descriptors) {
  var LIBRARY = _library;
  var global = _global;
  var fails = _fails;
  var $export = _export;
  var $typed = _typed;
  var $buffer = _typedBuffer;
  var ctx = _ctx;
  var anInstance = _anInstance;
  var propertyDesc = _propertyDesc;
  var hide = _hide;
  var redefineAll = _redefineAll;
  var toInteger = _toInteger;
  var toLength = _toLength;
  var toIndex = _toIndex;
  var toAbsoluteIndex = _toAbsoluteIndex;
  var toPrimitive = _toPrimitive;
  var has = _has;
  var classof = _classof;
  var isObject = _isObject;
  var toObject = _toObject;
  var isArrayIter = _isArrayIter;
  var create = _objectCreate;
  var getPrototypeOf = _objectGpo;
  var gOPN = _objectGopn.f;
  var getIterFn = core_getIteratorMethod;
  var uid = _uid;
  var wks = _wks;
  var createArrayMethod = _arrayMethods;
  var createArrayIncludes = _arrayIncludes;
  var speciesConstructor = _speciesConstructor;
  var ArrayIterators = es6_array_iterator;
  var Iterators = _iterators;
  var $iterDetect = _iterDetect;
  var setSpecies = _setSpecies;
  var arrayFill = _arrayFill;
  var arrayCopyWithin = _arrayCopyWithin;
  var $DP = _objectDp;
  var $GOPD = _objectGopd;
  var dP = $DP.f;
  var gOPD = $GOPD.f;
  var RangeError = global.RangeError;
  var TypeError = global.TypeError;
  var Uint8Array = global.Uint8Array;
  var ARRAY_BUFFER = 'ArrayBuffer';
  var SHARED_BUFFER = 'Shared' + ARRAY_BUFFER;
  var BYTES_PER_ELEMENT = 'BYTES_PER_ELEMENT';
  var PROTOTYPE = 'prototype';
  var ArrayProto = Array[PROTOTYPE];
  var $ArrayBuffer = $buffer.ArrayBuffer;
  var $DataView = $buffer.DataView;
  var arrayForEach = createArrayMethod(0);
  var arrayFilter = createArrayMethod(2);
  var arraySome = createArrayMethod(3);
  var arrayEvery = createArrayMethod(4);
  var arrayFind = createArrayMethod(5);
  var arrayFindIndex = createArrayMethod(6);
  var arrayIncludes = createArrayIncludes(true);
  var arrayIndexOf = createArrayIncludes(false);
  var arrayValues = ArrayIterators.values;
  var arrayKeys = ArrayIterators.keys;
  var arrayEntries = ArrayIterators.entries;
  var arrayLastIndexOf = ArrayProto.lastIndexOf;
  var arrayReduce = ArrayProto.reduce;
  var arrayReduceRight = ArrayProto.reduceRight;
  var arrayJoin = ArrayProto.join;
  var arraySort = ArrayProto.sort;
  var arraySlice = ArrayProto.slice;
  var arrayToString = ArrayProto.toString;
  var arrayToLocaleString = ArrayProto.toLocaleString;
  var ITERATOR = wks('iterator');
  var TAG = wks('toStringTag');
  var TYPED_CONSTRUCTOR = uid('typed_constructor');
  var DEF_CONSTRUCTOR = uid('def_constructor');
  var ALL_CONSTRUCTORS = $typed.CONSTR;
  var TYPED_ARRAY = $typed.TYPED;
  var VIEW = $typed.VIEW;
  var WRONG_LENGTH = 'Wrong length!';

  var $map = createArrayMethod(1, function (O, length) {
    return allocate(speciesConstructor(O, O[DEF_CONSTRUCTOR]), length);
  });

  var LITTLE_ENDIAN = fails(function () {
    // eslint-disable-next-line no-undef
    return new Uint8Array(new Uint16Array([1]).buffer)[0] === 1;
  });

  var FORCED_SET = !!Uint8Array && !!Uint8Array[PROTOTYPE].set && fails(function () {
    new Uint8Array(1).set({});
  });

  var toOffset = function (it, BYTES) {
    var offset = toInteger(it);
    if (offset < 0 || offset % BYTES) throw RangeError('Wrong offset!');
    return offset;
  };

  var validate = function (it) {
    if (isObject(it) && TYPED_ARRAY in it) return it;
    throw TypeError(it + ' is not a typed array!');
  };

  var allocate = function (C, length) {
    if (!(isObject(C) && TYPED_CONSTRUCTOR in C)) {
      throw TypeError('It is not a typed array constructor!');
    } return new C(length);
  };

  var speciesFromList = function (O, list) {
    return fromList(speciesConstructor(O, O[DEF_CONSTRUCTOR]), list);
  };

  var fromList = function (C, list) {
    var index = 0;
    var length = list.length;
    var result = allocate(C, length);
    while (length > index) result[index] = list[index++];
    return result;
  };

  var addGetter = function (it, key, internal) {
    dP(it, key, { get: function () { return this._d[internal]; } });
  };

  var $from = function from(source /* , mapfn, thisArg */) {
    var O = toObject(source);
    var aLen = arguments.length;
    var mapfn = aLen > 1 ? arguments[1] : undefined;
    var mapping = mapfn !== undefined;
    var iterFn = getIterFn(O);
    var i, length, values, result, step, iterator;
    if (iterFn != undefined && !isArrayIter(iterFn)) {
      for (iterator = iterFn.call(O), values = [], i = 0; !(step = iterator.next()).done; i++) {
        values.push(step.value);
      } O = values;
    }
    if (mapping && aLen > 2) mapfn = ctx(mapfn, arguments[2], 2);
    for (i = 0, length = toLength(O.length), result = allocate(this, length); length > i; i++) {
      result[i] = mapping ? mapfn(O[i], i) : O[i];
    }
    return result;
  };

  var $of = function of(/* ...items */) {
    var index = 0;
    var length = arguments.length;
    var result = allocate(this, length);
    while (length > index) result[index] = arguments[index++];
    return result;
  };

  // iOS Safari 6.x fails here
  var TO_LOCALE_BUG = !!Uint8Array && fails(function () { arrayToLocaleString.call(new Uint8Array(1)); });

  var $toLocaleString = function toLocaleString() {
    return arrayToLocaleString.apply(TO_LOCALE_BUG ? arraySlice.call(validate(this)) : validate(this), arguments);
  };

  var proto = {
    copyWithin: function copyWithin(target, start /* , end */) {
      return arrayCopyWithin.call(validate(this), target, start, arguments.length > 2 ? arguments[2] : undefined);
    },
    every: function every(callbackfn /* , thisArg */) {
      return arrayEvery(validate(this), callbackfn, arguments.length > 1 ? arguments[1] : undefined);
    },
    fill: function fill(value /* , start, end */) { // eslint-disable-line no-unused-vars
      return arrayFill.apply(validate(this), arguments);
    },
    filter: function filter(callbackfn /* , thisArg */) {
      return speciesFromList(this, arrayFilter(validate(this), callbackfn,
        arguments.length > 1 ? arguments[1] : undefined));
    },
    find: function find(predicate /* , thisArg */) {
      return arrayFind(validate(this), predicate, arguments.length > 1 ? arguments[1] : undefined);
    },
    findIndex: function findIndex(predicate /* , thisArg */) {
      return arrayFindIndex(validate(this), predicate, arguments.length > 1 ? arguments[1] : undefined);
    },
    forEach: function forEach(callbackfn /* , thisArg */) {
      arrayForEach(validate(this), callbackfn, arguments.length > 1 ? arguments[1] : undefined);
    },
    indexOf: function indexOf(searchElement /* , fromIndex */) {
      return arrayIndexOf(validate(this), searchElement, arguments.length > 1 ? arguments[1] : undefined);
    },
    includes: function includes(searchElement /* , fromIndex */) {
      return arrayIncludes(validate(this), searchElement, arguments.length > 1 ? arguments[1] : undefined);
    },
    join: function join(separator) { // eslint-disable-line no-unused-vars
      return arrayJoin.apply(validate(this), arguments);
    },
    lastIndexOf: function lastIndexOf(searchElement /* , fromIndex */) { // eslint-disable-line no-unused-vars
      return arrayLastIndexOf.apply(validate(this), arguments);
    },
    map: function map(mapfn /* , thisArg */) {
      return $map(validate(this), mapfn, arguments.length > 1 ? arguments[1] : undefined);
    },
    reduce: function reduce(callbackfn /* , initialValue */) { // eslint-disable-line no-unused-vars
      return arrayReduce.apply(validate(this), arguments);
    },
    reduceRight: function reduceRight(callbackfn /* , initialValue */) { // eslint-disable-line no-unused-vars
      return arrayReduceRight.apply(validate(this), arguments);
    },
    reverse: function reverse() {
      var that = this;
      var length = validate(that).length;
      var middle = Math.floor(length / 2);
      var index = 0;
      var value;
      while (index < middle) {
        value = that[index];
        that[index++] = that[--length];
        that[length] = value;
      } return that;
    },
    some: function some(callbackfn /* , thisArg */) {
      return arraySome(validate(this), callbackfn, arguments.length > 1 ? arguments[1] : undefined);
    },
    sort: function sort(comparefn) {
      return arraySort.call(validate(this), comparefn);
    },
    subarray: function subarray(begin, end) {
      var O = validate(this);
      var length = O.length;
      var $begin = toAbsoluteIndex(begin, length);
      return new (speciesConstructor(O, O[DEF_CONSTRUCTOR]))(
        O.buffer,
        O.byteOffset + $begin * O.BYTES_PER_ELEMENT,
        toLength((end === undefined ? length : toAbsoluteIndex(end, length)) - $begin)
      );
    }
  };

  var $slice = function slice(start, end) {
    return speciesFromList(this, arraySlice.call(validate(this), start, end));
  };

  var $set = function set(arrayLike /* , offset */) {
    validate(this);
    var offset = toOffset(arguments[1], 1);
    var length = this.length;
    var src = toObject(arrayLike);
    var len = toLength(src.length);
    var index = 0;
    if (len + offset > length) throw RangeError(WRONG_LENGTH);
    while (index < len) this[offset + index] = src[index++];
  };

  var $iterators = {
    entries: function entries() {
      return arrayEntries.call(validate(this));
    },
    keys: function keys() {
      return arrayKeys.call(validate(this));
    },
    values: function values() {
      return arrayValues.call(validate(this));
    }
  };

  var isTAIndex = function (target, key) {
    return isObject(target)
      && target[TYPED_ARRAY]
      && typeof key != 'symbol'
      && key in target
      && String(+key) == String(key);
  };
  var $getDesc = function getOwnPropertyDescriptor(target, key) {
    return isTAIndex(target, key = toPrimitive(key, true))
      ? propertyDesc(2, target[key])
      : gOPD(target, key);
  };
  var $setDesc = function defineProperty(target, key, desc) {
    if (isTAIndex(target, key = toPrimitive(key, true))
      && isObject(desc)
      && has(desc, 'value')
      && !has(desc, 'get')
      && !has(desc, 'set')
      // TODO: add validation descriptor w/o calling accessors
      && !desc.configurable
      && (!has(desc, 'writable') || desc.writable)
      && (!has(desc, 'enumerable') || desc.enumerable)
    ) {
      target[key] = desc.value;
      return target;
    } return dP(target, key, desc);
  };

  if (!ALL_CONSTRUCTORS) {
    $GOPD.f = $getDesc;
    $DP.f = $setDesc;
  }

  $export($export.S + $export.F * !ALL_CONSTRUCTORS, 'Object', {
    getOwnPropertyDescriptor: $getDesc,
    defineProperty: $setDesc
  });

  if (fails(function () { arrayToString.call({}); })) {
    arrayToString = arrayToLocaleString = function toString() {
      return arrayJoin.call(this);
    };
  }

  var $TypedArrayPrototype$ = redefineAll({}, proto);
  redefineAll($TypedArrayPrototype$, $iterators);
  hide($TypedArrayPrototype$, ITERATOR, $iterators.values);
  redefineAll($TypedArrayPrototype$, {
    slice: $slice,
    set: $set,
    constructor: function () { /* noop */ },
    toString: arrayToString,
    toLocaleString: $toLocaleString
  });
  addGetter($TypedArrayPrototype$, 'buffer', 'b');
  addGetter($TypedArrayPrototype$, 'byteOffset', 'o');
  addGetter($TypedArrayPrototype$, 'byteLength', 'l');
  addGetter($TypedArrayPrototype$, 'length', 'e');
  dP($TypedArrayPrototype$, TAG, {
    get: function () { return this[TYPED_ARRAY]; }
  });

  // eslint-disable-next-line max-statements
  module.exports = function (KEY, BYTES, wrapper, CLAMPED) {
    CLAMPED = !!CLAMPED;
    var NAME = KEY + (CLAMPED ? 'Clamped' : '') + 'Array';
    var GETTER = 'get' + KEY;
    var SETTER = 'set' + KEY;
    var TypedArray = global[NAME];
    var Base = TypedArray || {};
    var TAC = TypedArray && getPrototypeOf(TypedArray);
    var FORCED = !TypedArray || !$typed.ABV;
    var O = {};
    var TypedArrayPrototype = TypedArray && TypedArray[PROTOTYPE];
    var getter = function (that, index) {
      var data = that._d;
      return data.v[GETTER](index * BYTES + data.o, LITTLE_ENDIAN);
    };
    var setter = function (that, index, value) {
      var data = that._d;
      if (CLAMPED) value = (value = Math.round(value)) < 0 ? 0 : value > 0xff ? 0xff : value & 0xff;
      data.v[SETTER](index * BYTES + data.o, value, LITTLE_ENDIAN);
    };
    var addElement = function (that, index) {
      dP(that, index, {
        get: function () {
          return getter(this, index);
        },
        set: function (value) {
          return setter(this, index, value);
        },
        enumerable: true
      });
    };
    if (FORCED) {
      TypedArray = wrapper(function (that, data, $offset, $length) {
        anInstance(that, TypedArray, NAME, '_d');
        var index = 0;
        var offset = 0;
        var buffer, byteLength, length, klass;
        if (!isObject(data)) {
          length = toIndex(data);
          byteLength = length * BYTES;
          buffer = new $ArrayBuffer(byteLength);
        } else if (data instanceof $ArrayBuffer || (klass = classof(data)) == ARRAY_BUFFER || klass == SHARED_BUFFER) {
          buffer = data;
          offset = toOffset($offset, BYTES);
          var $len = data.byteLength;
          if ($length === undefined) {
            if ($len % BYTES) throw RangeError(WRONG_LENGTH);
            byteLength = $len - offset;
            if (byteLength < 0) throw RangeError(WRONG_LENGTH);
          } else {
            byteLength = toLength($length) * BYTES;
            if (byteLength + offset > $len) throw RangeError(WRONG_LENGTH);
          }
          length = byteLength / BYTES;
        } else if (TYPED_ARRAY in data) {
          return fromList(TypedArray, data);
        } else {
          return $from.call(TypedArray, data);
        }
        hide(that, '_d', {
          b: buffer,
          o: offset,
          l: byteLength,
          e: length,
          v: new $DataView(buffer)
        });
        while (index < length) addElement(that, index++);
      });
      TypedArrayPrototype = TypedArray[PROTOTYPE] = create($TypedArrayPrototype$);
      hide(TypedArrayPrototype, 'constructor', TypedArray);
    } else if (!fails(function () {
      TypedArray(1);
    }) || !fails(function () {
      new TypedArray(-1); // eslint-disable-line no-new
    }) || !$iterDetect(function (iter) {
      new TypedArray(); // eslint-disable-line no-new
      new TypedArray(null); // eslint-disable-line no-new
      new TypedArray(1.5); // eslint-disable-line no-new
      new TypedArray(iter); // eslint-disable-line no-new
    }, true)) {
      TypedArray = wrapper(function (that, data, $offset, $length) {
        anInstance(that, TypedArray, NAME);
        var klass;
        // `ws` module bug, temporarily remove validation length for Uint8Array
        // https://github.com/websockets/ws/pull/645
        if (!isObject(data)) return new Base(toIndex(data));
        if (data instanceof $ArrayBuffer || (klass = classof(data)) == ARRAY_BUFFER || klass == SHARED_BUFFER) {
          return $length !== undefined
            ? new Base(data, toOffset($offset, BYTES), $length)
            : $offset !== undefined
              ? new Base(data, toOffset($offset, BYTES))
              : new Base(data);
        }
        if (TYPED_ARRAY in data) return fromList(TypedArray, data);
        return $from.call(TypedArray, data);
      });
      arrayForEach(TAC !== Function.prototype ? gOPN(Base).concat(gOPN(TAC)) : gOPN(Base), function (key) {
        if (!(key in TypedArray)) hide(TypedArray, key, Base[key]);
      });
      TypedArray[PROTOTYPE] = TypedArrayPrototype;
      if (!LIBRARY) TypedArrayPrototype.constructor = TypedArray;
    }
    var $nativeIterator = TypedArrayPrototype[ITERATOR];
    var CORRECT_ITER_NAME = !!$nativeIterator
      && ($nativeIterator.name == 'values' || $nativeIterator.name == undefined);
    var $iterator = $iterators.values;
    hide(TypedArray, TYPED_CONSTRUCTOR, true);
    hide(TypedArrayPrototype, TYPED_ARRAY, NAME);
    hide(TypedArrayPrototype, VIEW, true);
    hide(TypedArrayPrototype, DEF_CONSTRUCTOR, TypedArray);

    if (CLAMPED ? new TypedArray(1)[TAG] != NAME : !(TAG in TypedArrayPrototype)) {
      dP(TypedArrayPrototype, TAG, {
        get: function () { return NAME; }
      });
    }

    O[NAME] = TypedArray;

    $export($export.G + $export.W + $export.F * (TypedArray != Base), O);

    $export($export.S, NAME, {
      BYTES_PER_ELEMENT: BYTES
    });

    $export($export.S + $export.F * fails(function () { Base.of.call(TypedArray, 1); }), NAME, {
      from: $from,
      of: $of
    });

    if (!(BYTES_PER_ELEMENT in TypedArrayPrototype)) hide(TypedArrayPrototype, BYTES_PER_ELEMENT, BYTES);

    $export($export.P, NAME, proto);

    setSpecies(NAME);

    $export($export.P + $export.F * FORCED_SET, NAME, { set: $set });

    $export($export.P + $export.F * !CORRECT_ITER_NAME, NAME, $iterators);

    if (!LIBRARY && TypedArrayPrototype.toString != arrayToString) TypedArrayPrototype.toString = arrayToString;

    $export($export.P + $export.F * fails(function () {
      new TypedArray(1).slice();
    }), NAME, { slice: $slice });

    $export($export.P + $export.F * (fails(function () {
      return [1, 2].toLocaleString() != new TypedArray([1, 2]).toLocaleString();
    }) || !fails(function () {
      TypedArrayPrototype.toLocaleString.call([1, 2]);
    })), NAME, { toLocaleString: $toLocaleString });

    Iterators[NAME] = CORRECT_ITER_NAME ? $nativeIterator : $iterator;
    if (!LIBRARY && !CORRECT_ITER_NAME) hide(TypedArrayPrototype, ITERATOR, $iterator);
  };
} else module.exports = function () { /* empty */ };
});

_typedArray('Uint8', 1, function (init) {
  return function Uint8Array(data, byteOffset, length) {
    return init(this, data, byteOffset, length);
  };
});

var Texture =
/*#__PURE__*/
function () {
  function Texture(gl, name, options) {
    if (options === void 0) {
      options = {};
    }

    options = Texture.sliceOptions(options); // exclude any non-texture-specific props

    this.gl = gl;
    this.texture = gl.createTexture();

    if (this.texture) {
      this.valid = true;
    }

    this.bind();
    this.name = name;
    this.retain_count = 0;
    this.config_type = null;
    this.loading = null; // a Promise object to track the loading state of this texture

    this.loaded = false; // successfully loaded as expected

    this.filtering = options.filtering;
    this.density = options.density || 1; // native pixel density of texture

    this.sprites = options.sprites;
    this.texcoords = {}; // sprite UVs ([0, 1] range)

    this.sizes = {}; // sprite sizes (pixel size)

    this.css_sizes = {}; // sprite sizes, adjusted for native texture pixel density

    this.aspects = {}; // sprite aspect ratios
    // Default to a 1-pixel transparent black texture so we can safely render while we wait for an image to load
    // See: http://stackoverflow.com/questions/19722247/webgl-wait-for-texture-to-load

    this.setData(1, 1, new Uint8Array([0, 0, 0, 0]), {
      filtering: 'nearest'
    });
    this.loaded = false; // don't consider loaded when only placeholder data is present
    // Destroy previous texture if present

    if (Texture.textures[this.name]) {
      // Preserve previous retain count
      this.retain_count = Texture.textures[this.name].retain_count;
      Texture.textures[this.name].retain_count = 0; // allow to be freed

      Texture.textures[this.name].destroy();
    } // Cache texture instance and definition


    Texture.textures[this.name] = this;
    Texture.texture_configs[this.name] = JSON.stringify(Object.assign({
      name: name
    }, options));
    this.load(options);
    log('trace', "creating Texture " + this.name);
  } // Destroy a single texture instance


  var _proto = Texture.prototype;

  _proto.destroy = function destroy(_temp) {
    var _ref = _temp === void 0 ? {} : _temp,
        force = _ref.force;

    if (this.retain_count > 0 && !force) {
      log('error', "Texture '" + this.name + "': destroying texture with retain count of '" + this.retain_count + "'");
      return;
    }

    if (!this.valid) {
      return;
    }

    this.gl.deleteTexture(this.texture);
    this.texture = null;

    if (Texture.textures[this.name] === this) {
      delete Texture.textures[this.name];
      delete Texture.texture_configs[this.name];
    }

    this.valid = false;
    log('trace', "destroying Texture " + this.name);
  };

  _proto.retain = function retain() {
    this.retain_count++;
  };

  _proto.release = function release() {
    if (this.retain_count <= 0) {
      log('error', "Texture '" + this.name + "': releasing texture with retain count of '" + this.retain_count + "'");
    }

    this.retain_count--;

    if (this.retain_count <= 0) {
      this.destroy();
    }
  };

  _proto.bind = function bind(unit) {
    if (unit === void 0) {
      unit = 0;
    }

    if (!this.valid) {
      return;
    }

    if (Texture.activeUnit !== unit) {
      this.gl.activeTexture(this.gl.TEXTURE0 + unit);
      Texture.activeUnit = unit;
      Texture.boundTexture = null; // texture must be re-bound when unit changes
    }

    if (Texture.boundTexture !== this.texture) {
      this.gl.bindTexture(this.gl.TEXTURE_2D, this.texture);
      Texture.boundTexture = this.texture;
    }
  };

  _proto.load = function load(options) {
    var _this = this;

    if (!options) {
      return this.loading || Promise.resolve(this);
    }

    this.loading = null;

    if (typeof options.url === 'string') {
      this.config_type = 'url';
      this.setUrl(options.url, options);
    } else if (options.element) {
      this.config_type = 'element';
      this.setElement(options.element, options);
    } else if (options.data && options.width && options.height) {
      this.config_type = 'data';
      this.setData(options.width, options.height, options.data, options);
    }

    this.loading = this.loading && this.loading.then(function () {
      _this.calculateSprites();

      return _this;
    }) || Promise.resolve(this);
    return this.loading;
  } // Sets texture from an url
  ;

  _proto.setUrl = function setUrl(url, options) {
    var _this2 = this;

    if (options === void 0) {
      options = {};
    }

    if (!this.valid) {
      return;
    }

    this.url = url; // save URL reference (will be overwritten when element is loaded below)

    this.loading = new Promise(function (resolve) {
      var image = new Image();

      image.onload = function () {
        try {
          _this2.setElement(image, options);
        } catch (e) {
          _this2.loaded = false;
          log('warn', "Texture '" + _this2.name + "': failed to load url: '" + _this2.url + "'", e, options);
          Texture.trigger('warning', {
            message: "Failed to load texture from " + _this2.url,
            error: e,
            texture: options
          });
        }

        _this2.loaded = true;
        resolve(_this2);
      };

      image.onerror = function (e) {
        // Warn and resolve on error
        _this2.loaded = false;
        log('warn', "Texture '" + _this2.name + "': failed to load url: '" + _this2.url + "'", e, options);
        Texture.trigger('warning', {
          message: "Failed to load texture from " + _this2.url,
          error: e,
          texture: options
        });
        resolve(_this2);
      }; // Safari has a bug loading data-URL images with CORS enabled, so it must be disabled in that case
      // https://bugs.webkit.org/show_bug.cgi?id=123978


      if (!(Utils.isSafari() && _this2.url.slice(0, 5) === 'data:')) {
        image.crossOrigin = 'anonymous';
      }

      image.src = _this2.url;
    });
    return this.loading;
  } // Sets texture to a raw image buffer
  ;

  _proto.setData = function setData(width, height, data, options) {
    if (options === void 0) {
      options = {};
    }

    this.width = width;
    this.height = height; // Convert regular array to typed array

    if (Array.isArray(data)) {
      data = new Uint8Array(data);
    }

    this.update(data, options);
    this.setFiltering(options);
    this.loaded = true;
    this.loading = Promise.resolve(this);
    return this.loading;
  } // Sets the texture to track a element (canvas/image)
  ;

  _proto.setElement = function setElement(element, options) {
    var el = element; // a string element is interpeted as a CSS selector

    if (typeof element === 'string') {
      element = document.querySelector(element);
    }

    if (element instanceof HTMLCanvasElement || element instanceof HTMLImageElement || element instanceof HTMLVideoElement) {
      this.update(element, options);
      this.setFiltering(options);
    } else {
      this.loaded = false;
      var msg = "the 'element' parameter (`element: " + JSON.stringify(el) + "`) must be a CSS ";
      msg += 'selector string, or a <canvas>, <image> or <video> object';
      log('warn', "Texture '" + this.name + "': " + msg, options);
      Texture.trigger('warning', {
        message: "Failed to load texture because " + msg,
        texture: options
      });
    }

    this.loaded = true;
    this.loading = Promise.resolve(this);
    return this.loading;
  } // Uploads current image or buffer to the GPU (can be used to update animated textures on the fly)
  ;

  _proto.update = function update(source, options) {
    if (options === void 0) {
      options = {};
    }

    if (!this.valid) {
      return;
    }

    this.bind(); // Image or Canvas element

    if (source instanceof HTMLCanvasElement || source instanceof HTMLVideoElement || source instanceof HTMLImageElement && source.complete) {
      this.width = source.width;
      this.height = source.height;
      this.gl.pixelStorei(this.gl.UNPACK_FLIP_Y_WEBGL, options.UNPACK_FLIP_Y_WEBGL === false ? false : true);
      this.gl.pixelStorei(this.gl.UNPACK_PREMULTIPLY_ALPHA_WEBGL, options.UNPACK_PREMULTIPLY_ALPHA_WEBGL || false);
      this.gl.texImage2D(this.gl.TEXTURE_2D, 0, this.gl.RGBA, this.gl.RGBA, this.gl.UNSIGNED_BYTE, source);
    } // Raw image buffer
    else {
        // these pixel store params are deprecated for non-DOM element uploads
        // (e.g. when creating texture from raw data)
        // setting them to null avoids a Firefox warning
        this.gl.pixelStorei(this.gl.UNPACK_FLIP_Y_WEBGL, null);
        this.gl.pixelStorei(this.gl.UNPACK_PREMULTIPLY_ALPHA_WEBGL, null);
        this.gl.texImage2D(this.gl.TEXTURE_2D, 0, this.gl.RGBA, this.width, this.height, 0, this.gl.RGBA, this.gl.UNSIGNED_BYTE, source);
      }

    Texture.trigger('update', this);
  } // Determines appropriate filtering mode
  ;

  _proto.setFiltering = function setFiltering(options) {
    if (options === void 0) {
      options = {};
    }

    if (!this.valid) {
      return;
    }

    options.filtering = options.filtering || 'linear';
    var gl = this.gl;
    this.bind(); // For power-of-2 textures, the following presets are available:
    // mipmap: linear blend from nearest mip
    // linear: linear blend from original image (no mips)
    // nearest: nearest pixel from original image (no mips, 'blocky' look)

    if (Utils.isPowerOf2(this.width) && Utils.isPowerOf2(this.height)) {
      this.power_of_2 = true;
      gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_S, options.TEXTURE_WRAP_S || options.repeat && gl.REPEAT || gl.CLAMP_TO_EDGE);
      gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_T, options.TEXTURE_WRAP_T || options.repeat && gl.REPEAT || gl.CLAMP_TO_EDGE);

      if (options.filtering === 'mipmap') {
        this.filtering = 'mipmap';
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.LINEAR_MIPMAP_LINEAR); // TODO: use trilinear filtering by defualt instead?

        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.LINEAR);
        gl.generateMipmap(gl.TEXTURE_2D);
      } else if (options.filtering === 'linear') {
        this.filtering = 'linear';
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.LINEAR);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.LINEAR);
      } else if (options.filtering === 'nearest') {
        this.filtering = 'nearest';
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.NEAREST);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.NEAREST);
      }
    } else {
      // WebGL has strict requirements on non-power-of-2 textures:
      // No mipmaps and must clamp to edge
      this.power_of_2 = false;
      gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_S, gl.CLAMP_TO_EDGE);
      gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_T, gl.CLAMP_TO_EDGE);

      if (options.filtering === 'nearest') {
        this.filtering = 'nearest';
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.NEAREST);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.NEAREST);
      } else {
        // default to linear for non-power-of-2 textures
        this.filtering = 'linear';
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.LINEAR);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.LINEAR);
      }
    }

    Texture.trigger('update', this);
  } // Pre-calc sprite regions for a texture sprite in UV [0, 1] space
  ;

  _proto.calculateSprites = function calculateSprites() {
    if (this.sprites) {
      for (var s in this.sprites) {
        var sprite = this.sprites[s]; // Map [0, 0] to [1, 1] coords to the appropriate sprite sub-area of the texture

        this.texcoords[s] = Texture.getTexcoordsForSprite([sprite[0], sprite[1]], [sprite[2], sprite[3]], [this.width, this.height]); // Pixel size of sprite
        // Divide by native texture density to get correct CSS pixels

        this.sizes[s] = [sprite[2], sprite[3]];
        this.css_sizes[s] = [sprite[2] / this.density, sprite[3] / this.density];
        this.aspects[s] = sprite[2] / sprite[3];
      }
    }
  } // Get the tetxure size in bytes
  ;

  _proto.byteSize = function byteSize() {
    // mipmaps use 33% additional memory
    return Math.round(this.width * this.height * 4 * (this.filtering == 'mipmap' ? 1.33 : 1));
  };

  return Texture;
}(); // Static/class methods and state

Texture.create = function (gl, name, options) {
  return new Texture(gl, name, options);
};

Texture.retain = function (name) {
  if (Texture.textures[name]) {
    Texture.textures[name].retain();
  }
};

Texture.release = function (name) {
  if (Texture.textures[name]) {
    Texture.textures[name].release();
  }
}; // Destroy all texture instances for a given GL context


Texture.destroy = function (gl) {
  var textures = Object.keys(Texture.textures);
  textures.forEach(function (t) {
    var texture = Texture.textures[t];

    if (texture.gl === gl) {
      texture.destroy({
        force: true
      });
    }
  });
}; // Get sprite pixel size and UVs


Texture.getSpriteInfo = function (texname, sprite) {
  var texture = Texture.textures[texname];
  return texture && {
    size: texture.sizes[sprite],
    css_size: texture.css_sizes[sprite],
    aspect: texture.aspects[sprite],
    texcoords: texture.texcoords[sprite]
  };
}; // Re-scale UVs from [0, 1] range to a smaller area within the image


Texture.getTexcoordsForSprite = function (area_origin, area_size, tex_size) {
  var area_origin_y = tex_size[1] - area_origin[1] - area_size[1];
  return [area_origin[0] / tex_size[0], area_origin_y / tex_size[1], (area_size[0] + area_origin[0]) / tex_size[0], (area_size[1] + area_origin_y) / tex_size[1]];
}; // Create a set of textures keyed in an object
// Optionally load each if it has a URL specified


Texture.createFromObject = function (gl, textures) {
  var loading = [];

  if (textures) {
    for (var texname in textures) {
      var config = textures[texname];

      if (config.skip_create) {
        // explicitly skip (re-)creating this texture
        // used for dynamic canvas textures that we *know* haven't changed
        // (internal raster tiles, vs. user-supplied canvas where pixels may have changed)
        continue;
      } // If texture already exists and definition hasn't changed, no need to re-create
      // Note: to avoid flicker when other textures/scene items change


      if (!Texture.changed(texname, config)) {
        continue;
      }

      var texture = Texture.create(gl, texname, config);
      loading.push(texture.loading);
    }
  }

  return Promise.all(loading);
}; // Create a 'default' texture (1x1 pixel) that can be used as a placeholder
// (for example to prevent GL from complaining about unbound textures)


Texture.default = '__default';

Texture.createDefault = function (gl) {
  return Texture.create(gl, Texture.default);
}; // Only include texture-specific properties (avoid faulty equality comparisons between
// textures when caller may include other ancillary props)


Texture.sliceOptions = function (options) {
  return {
    filtering: options.filtering,
    sprites: options.sprites,
    url: options.url,
    element: options.element,
    data: options.data,
    width: options.width,
    height: options.height,
    density: options.density,
    repeat: options.repeat,
    TEXTURE_WRAP_S: options.TEXTURE_WRAP_S,
    TEXTURE_WRAP_T: options.TEXTURE_WRAP_T,
    UNPACK_FLIP_Y_WEBGL: options.UNPACK_FLIP_Y_WEBGL,
    UNPACK_PREMULTIPLY_ALPHA_WEBGL: options.UNPACK_PREMULTIPLY_ALPHA_WEBGL
  };
}; // Indicate if a texture definition would be a change from the current cache


Texture.changed = function (name, config) {
  var texture = Texture.textures[name];

  if (texture) {
    // cached texture
    // canvas/image-based textures are considered dynamic and always refresh
    if (texture.config_type === 'element' || config.element != null) {
      return true;
    } // compare definitions


    config = Texture.sliceOptions(config); // exclude any non-texture-specific props

    if (Texture.texture_configs[name] === JSON.stringify(Object.assign({
      name: name
    }, config))) {
      return false;
    }
  }

  return true;
}; // Get metadata for a texture by name
// Returns via promise, in case texture is still loading
// Can be called on main thread from worker, to sync texture info to worker


Texture.getInfo = function (name) {
  // Get info for all textures by default
  if (!name) {
    name = Object.keys(Texture.textures);
  } // Get multiple textures


  if (Array.isArray(name)) {
    return Promise.all(name.map(function (n) {
      return Texture.getInfo(n);
    }));
  } // Get single texture


  var tex = Texture.textures[name];

  if (tex) {
    // Wait for this texture to finish loading, or return immediately
    var loading = tex.loading || Promise.resolve(tex);
    return loading.then(function () {
      // Return a subset of texture info
      // (compatible w/structured cloning, suitable for passing to a worker)
      return {
        name: tex.name,
        width: tex.width,
        height: tex.height,
        density: tex.density,
        css_size: [tex.width / tex.density, tex.height / tex.density],
        aspect: tex.width / tex.height,
        sprites: tex.sprites,
        texcoords: tex.texcoords,
        sizes: tex.sizes,
        css_sizes: tex.css_sizes,
        aspects: tex.aspects,
        filtering: tex.filtering,
        power_of_2: tex.power_of_2,
        valid: tex.valid
      };
    });
  } else {
    // No texture found
    return Promise.resolve(null);
  }
}; // Sync texture info to worker
// Called from worker, gets info on one or more textures info from main thread via remote call, then stores it
// locally in worker. 'textures' can be an array of texture names to sync, or if null, all textures are synced.


Texture.syncTexturesToWorker = function (names) {
  return WorkerBroker$1.postMessage('Texture.getInfo', names).then(function (textures) {
    if (textures) {
      textures.filter(function (x) {
        return x;
      }) // remove nulls
      .forEach(function (t) {
        return Texture.textures[t.name] = t;
      });
    }

    return Texture.textures;
  });
}; // Report max texture size for a GL context


Texture.getMaxTextureSize = function (gl) {
  return gl.getParameter(gl.MAX_TEXTURE_SIZE);
}; // Global set of textures, by name


Texture.textures = {};
Texture.texture_configs = {};
Texture.boundTexture = null;
Texture.activeUnit = null;
WorkerBroker$1.addTarget('Texture', Texture);
subscribeMixin(Texture);

var isEnum = _objectPie.f;
var _objectToArray = function (isEntries) {
  return function (it) {
    var O = _toIobject(it);
    var keys = _objectKeys(O);
    var length = keys.length;
    var i = 0;
    var result = [];
    var key;
    while (length > i) if (isEnum.call(O, key = keys[i++])) {
      result.push(isEntries ? [key, O[key]] : O[key]);
    } return result;
  };
};

// https://github.com/tc39/proposal-object-values-entries

var $entries = _objectToArray(true);

_export(_export.S, 'Object', {
  entries: function entries(it) {
    return $entries(it);
  }
});

// @@match logic
_fixReWks('match', 1, function (defined, MATCH, $match, maybeCallNative) {
  return [
    // `String.prototype.match` method
    // https://tc39.github.io/ecma262/#sec-string.prototype.match
    function match(regexp) {
      var O = defined(this);
      var fn = regexp == undefined ? undefined : regexp[MATCH];
      return fn !== undefined ? fn.call(regexp, O) : new RegExp(regexp)[MATCH](String(O));
    },
    // `RegExp.prototype[@@match]` method
    // https://tc39.github.io/ecma262/#sec-regexp.prototype-@@match
    function (regexp) {
      var res = maybeCallNative($match, regexp, this);
      if (res.done) return res.value;
      var rx = _anObject(regexp);
      var S = String(this);
      if (!rx.global) return _regexpExecAbstract(rx, S);
      var fullUnicode = rx.unicode;
      rx.lastIndex = 0;
      var A = [];
      var n = 0;
      var result;
      while ((result = _regexpExecAbstract(rx, S)) !== null) {
        var matchStr = String(result[0]);
        A[n] = matchStr;
        if (matchStr === '') rx.lastIndex = _advanceStringIndex(S, _toLength(rx.lastIndex), fullUnicode);
        n++;
      }
      return n === 0 ? null : A;
    }
  ];
});

var GLSL = {};
/**
    Parse uniforms from a JS object, infers types and returns an array of objects with the
    necessary information to set uniform values on a GL program. Each object in the returned
    array has the form:
    { type, method, name, value }

    type: the GL uniform type, such as 'vec3', 'float', etc.
    method: the GL uniform setter method to use, such as '1f', '3fv', etc.
    name: the fully qualified name of the GL uniform location, e.g. 'array[0].field', etc.
    value: the value to be passed to the GL uniform setter for that type, e.g. [1, 2, 3] for a vec3

    Textures have special behavior: uniforms with string values are treated as textures, and
    the string is used as a unique texture 'key' to be interpreted by the caller (which is responsible
    for actually setting the uniforms). For example, this could be used as a key into a dictionary of
    known texture names, or it could simply be used as a URL to dynamically load the texture from.
*/

GLSL.parseUniforms = function (uniforms) {
  var parsed = [];

  for (var name in uniforms) {
    var key = name; // save the original name

    var uniform = uniforms[name];
    var u; // Single float

    if (typeof uniform === 'number') {
      parsed.push({
        type: 'float',
        method: '1f',
        name: name,
        value: uniform,
        key: key,
        uniforms: uniforms
      });
    } // Array: vector, array of floats, array of textures
    else if (Array.isArray(uniform)) {
        // Numeric values
        if (typeof uniform[0] === 'number') {
          // float vectors (vec2, vec3, vec4)
          if (uniform.length >= 2 && uniform.length <= 4) {
            parsed.push({
              type: 'vec' + uniform.length,
              method: uniform.length + 'fv',
              name: name,
              value: uniform,
              key: key,
              uniforms: uniforms
            });
          } // float array
          else if (uniform.length > 4) {
              parsed.push({
                type: 'float[]',
                method: '1fv',
                name: name + '[0]',
                value: uniform,
                key: key,
                uniforms: uniforms
              });
            } // TODO: assume matrix for (typeof == Float32Array && length == 16)?

        } // Array of textures
        else if (typeof uniform[0] === 'string') {
            for (u = 0; u < uniform.length; u++) {
              parsed.push({
                type: 'sampler2D',
                method: '1i',
                name: name + '[' + u + ']',
                value: uniform[u],
                key: u,
                uniforms: uniform
              });
            }
          } // Array of arrays - but only arrays of vectors are allowed in this case
          else if (Array.isArray(uniform[0]) && typeof uniform[0][0] === 'number') {
              // float vectors (vec2, vec3, vec4)
              if (uniform[0].length >= 2 && uniform[0].length <= 4) {
                // Set each vector in the array
                for (u = 0; u < uniform.length; u++) {
                  parsed.push({
                    type: 'vec' + uniform[0].length,
                    method: uniform[0].length + 'fv',
                    name: name + '[' + u + ']',
                    value: uniform[u],
                    key: u,
                    uniforms: uniform
                  });
                }
              }
            } // TODO: else warning

      } // Boolean
      else if (typeof uniform === 'boolean') {
          parsed.push({
            type: 'bool',
            method: '1i',
            name: name,
            value: uniform,
            key: key,
            uniforms: uniforms
          });
        } // Texture
        else if (typeof uniform === 'string') {
            parsed.push({
              type: 'sampler2D',
              method: '1i',
              name: name,
              value: uniform,
              key: key,
              uniforms: uniforms
            });
          }
  }

  return parsed;
};
/**
    Generate a GLSL variable definition from a JS object
*/


GLSL.defineVariable = function (name, value) {
  var type, array; // Single float

  if (typeof value === 'number') {
    type = 'float';
  } // Multiple floats - vector or array
  else if (Array.isArray(value)) {
      // Numeric values
      if (typeof value[0] === 'number') {
        // float vectors (vec2, vec3, vec4)
        if (value.length >= 2 && value.length <= 4) {
          type = 'vec' + value.length;
        } // float array
        else {
            //if (value.length > 4) {
            type = 'float';
            array = value.length;
          } // TODO: assume matrix for (typeof == Float32Array && length == 16)?

      } // Array of textures
      else if (typeof value[0] === 'string') {
          type = 'sampler2D';
          array = value.length;
        } // Array of arrays - but only arrays of vectors are allowed in this case
        else if (Array.isArray(value[0]) && typeof value[0][0] === 'number') {
            // float vectors (vec2, vec3, vec4)
            if (value[0].length >= 2 && value[0].length <= 4) {
              type = 'vec' + value[0].length;
              array = value.length;
            }
          }
    } // Boolean
    else if (typeof value === 'boolean') {
        type = 'bool';
      } // Texture
      else if (typeof value === 'string') {
          type = 'sampler2D';
        } else {
          return; // no valid type found
        } // Construct variable definition


  var variable = '';
  variable += type + " " + name;

  if (array) {
    variable += "[" + array + "]";
  }

  variable += ';\n';
  return variable;
};
/**
    Generate a GLSL uniform definition from a JS object
*/


GLSL.defineUniform = function (name, value) {
  var def = GLSL.defineVariable(name, value);

  if (!def) {
    return;
  }

  return 'uniform ' + def;
};
/**
    Expand a single value or 2-element array into a 3-element array, with the last ( z )
    coordinate defaulting to 1 (with option to specify). Also runs parseFloat to try to maintain
    data integrity. Returns null if input couldn't be parsed.
*/


GLSL.expandVec3 = function (v, z) {
  if (z === void 0) {
    z = 1;
  }

  var x;

  if (Array.isArray(v)) {
    if (v.length === 2) {
      x = [].concat(v, [z]).map(parseFloat);
    } else {
      return v;
    }
  } else {
    x = [v, v, v].map(parseFloat);
  }

  if (x && x.every(function (n) {
    return typeof n === 'number' && !isNaN(n);
  })) {
    return x;
  }
};
/**
    Expand a single value or 3-element array into a 4-element array, with the last (e.g. w or a)
    coordinate defaulting to 1 (with option to specify). Also runs parseFloat to try to maintain
    data integrity. Returns null if input couldn't be parsed.
*/


GLSL.expandVec4 = function (v, w) {
  if (w === void 0) {
    w = 1;
  }

  var x;

  if (Array.isArray(v)) {
    if (v.length === 3) {
      x = [].concat(v, [w]).map(parseFloat);
    } else {
      return v;
    }
  } else {
    x = [v, v, v, w].map(parseFloat);
  }

  if (x && x.every(function (n) {
    return typeof n === 'number' && !isNaN(n);
  })) {
    return x;
  }
};

// WebGL extension wrapper
// Stores extensions by name and GL context
// list of extension arrays, for each entry, 1st element GL context, 2nd map of extensions by name
var extensions = [];
function getExtension(gl, name) {
  var exts = extensions.filter(function (e) {
    return e[0] === gl;
  })[0];
  exts = exts && exts[1];

  if (!exts) {
    extensions.push([gl, {}]);
    exts = extensions[extensions.length - 1][1];
  }

  if (!exts[name]) {
    exts[name] = gl.getExtension(name);
  }

  return exts[name];
}

// http://werxltd.com/wp/2010/05/13/javascript-implementation-of-javas-string-hashcode-method/
function hashString(string) {
  var hash = 0,
      i,
      chr,
      len;

  if (string.length === 0) {
    return hash;
  }

  for (i = 0, len = string.length; i < len; i++) {
    chr = string.charCodeAt(i);
    hash = (hash << 5) - hash + chr;
    hash |= 0; // Convert to 32bit integer
  }

  return hash;
}

var glShaderErrors = parseErrors;

function parseErrors(log) {
  log = String(log);

  var logs = [];
  var result;

  while (result = log.match(/ERROR\:([^\n]+)/)) {
    log = log.slice(result.index + 1);

    var line = result[1].trim();
    var seps = line.split(':');
    var emsg = seps.slice(2).join(':').trim();
    var file = parseInt(seps[0], 10);
    var line = parseInt(seps[1], 10);

    logs.push({
        message: emsg
      , file: file
      , line: line
    });
  }

  return logs
}

var re_pragma = /^\s*#pragma.*$/gm; // for removing unused pragmas after shader block injection

var re_continue_line = /\\\s*\n/mg; // for removing backslash line continuations

var ShaderProgram =
/*#__PURE__*/
function () {
  function ShaderProgram(gl, vertex_source, fragment_source, options) {
    options = options || {};
    this.gl = gl;
    this.program = null;
    this.compiled = false;
    this.compiling = false;
    this.error = null; // key/values inserted as #defines into shaders at compile-time

    this.defines = Object.assign({}, options.defines || {}); // key/values for blocks that can be injected into shaders at compile-time

    this.blocks = Object.assign({}, options.blocks || {});
    this.block_scopes = Object.assign({}, options.block_scopes || {}); // list of extensions to activate

    this.extensions = options.extensions || []; // JS-object uniforms that are expected by this program, their types are inferred and definitions
    // for each will be injected.

    this.dependent_uniforms = options.uniforms;
    this.uniforms = {}; // program locations of uniforms, lazily added as each uniform is set

    this.attribs = {}; // program locations of vertex attributes, lazily added as each attribute is accessed

    this.vertex_source = vertex_source;
    this.fragment_source = fragment_source;
    this.id = ShaderProgram.id++;
    this.name = options.name; // can provide a program name (useful for debugging)
  }

  var _proto = ShaderProgram.prototype;

  _proto.destroy = function destroy() {
    this.gl.useProgram(null);
    this.gl.deleteProgram(this.program);
    this.program = null;
    this.uniforms = {};
    this.attribs = {};
    this.compiled = false;
  } // Use program wrapper with simple state cache
  ;

  _proto.use = function use() {
    if (!this.compiled) {
      return;
    }

    if (ShaderProgram.current !== this) {
      this.gl.useProgram(this.program);
    }

    ShaderProgram.current = this;
  };

  _proto.compile = function compile() {
    var _this = this;

    if (this.compiling) {
      throw new Error("ShaderProgram.compile(): skipping for " + this.id + " (" + this.name + ") because already compiling");
    }

    this.compiling = true;
    this.compiled = false;
    this.error = null; // Copy sources from pre-modified template

    this.computed_vertex_source = this.vertex_source;
    this.computed_fragment_source = this.fragment_source; // Check for extension availability

    var extensions = this.checkExtensions(); // Make list of defines to be injected later

    var defines = this.buildDefineList(); // Inject user-defined blocks (arbitrary code points matching named #pragmas)
    // Replace according to this pattern:
    // #pragma tangram: [key]
    // e.g. #pragma tangram: global
    // Gather all block code snippets

    var blocks = this.buildShaderBlockList();
    var regexp;

    for (var key in blocks) {
      var block = blocks[key];

      if (!block || Array.isArray(block) && block.length === 0) {
        continue;
      } // First find code replace points in shaders


      regexp = new RegExp('^\\s*#pragma\\s+tangram:\\s+' + key + '\\s*$', 'm');
      var inject_vertex = this.computed_vertex_source.match(regexp);
      var inject_fragment = this.computed_fragment_source.match(regexp); // Avoid network request if nothing to replace

      if (inject_vertex == null && inject_fragment == null) {
        continue;
      } // Combine all blocks into one string


      var source = '';
      block.forEach(function (val) {
        // Mark start and end of each block with metadata (which can be extracted from
        // final source for error handling, debugging, etc.)
        var mark = val.scope + ", " + val.key + ", " + val.num;
        source += "\n// tangram-block-start: " + mark + "\n";
        source += val.source;
        source += "\n// tangram-block-end: " + mark + "\n";
      }); // Inject

      if (inject_vertex != null) {
        this.computed_vertex_source = this.computed_vertex_source.replace(regexp, source);
      }

      if (inject_fragment != null) {
        this.computed_fragment_source = this.computed_fragment_source.replace(regexp, source);
      } // Add a #define for this injection point


      defines['TANGRAM_BLOCK_' + key.replace(/[\s-]+/g, '_').toUpperCase()] = true;
    } // Clean-up any #pragmas that weren't replaced (to prevent compiler warnings)


    this.computed_vertex_source = this.computed_vertex_source.replace(re_pragma, '');
    this.computed_fragment_source = this.computed_fragment_source.replace(re_pragma, ''); // Inject uniform definitions

    this.ensureUniforms(this.dependent_uniforms); // Build & inject extensions & defines
    // This is done *after* code injection so that we can add defines for which code points were injected

    var precision = '';
    var high = this.gl.getShaderPrecisionFormat(this.gl.FRAGMENT_SHADER, this.gl.HIGH_FLOAT);

    if (high && high.precision > 0) {
      precision = 'precision highp float;\n';
    } else {
      precision = 'precision mediump float;\n';
    }

    defines['TANGRAM_VERTEX_SHADER'] = true;
    defines['TANGRAM_FRAGMENT_SHADER'] = false;
    this.computed_vertex_source = precision + ShaderProgram.buildDefineString(defines) + this.computed_vertex_source; // Precision qualifier only valid in fragment shader
    // NB: '#extension' statements added to fragment shader only, as IE11 throws error when they appear in
    // vertex shader (even when guarded by #ifdef), and no WebGL extensions require '#extension' in vertex shaders

    defines['TANGRAM_VERTEX_SHADER'] = false;
    defines['TANGRAM_FRAGMENT_SHADER'] = true;
    this.computed_fragment_source = ShaderProgram.buildExtensionString(extensions) + precision + ShaderProgram.buildDefineString(defines) + this.computed_fragment_source; // Replace multi-line backslashes

    this.computed_vertex_source = this.computed_vertex_source.replace(re_continue_line, '');
    this.computed_fragment_source = this.computed_fragment_source.replace(re_continue_line, ''); // Compile & set uniforms to cached values

    try {
      this.program = ShaderProgram.updateProgram(this.gl, this.program, this.computed_vertex_source, this.computed_fragment_source);
      this.compiled = true;
      this.compiling = false;
    } catch (error) {
      this.program = null;
      this.compiled = false;
      this.compiling = false;
      this.error = error; // shader error info

      this.error.vertex_shader_source = this.computed_vertex_source;
      this.error.fragment_shader_source = this.computed_fragment_source;

      if (error.type === 'vertex' || error.type === 'fragment') {
        this.shader_errors = error.errors;
        this.shader_errors.forEach(function (e) {
          e.type = error.type;
          e.block = _this.block(error.type, e.line);
          e.line = _this.block(error.type, e.line);
        });
        this.error.shader_errors = this.shader_errors;
      }

      throw error;
    } // Discard shader sources after successful compilation


    this.computed_vertex_source = null;
    this.computed_fragment_source = null;
    this.use();
    this.refreshUniforms();
    this.refreshAttributes();
  } // Make list of defines (global, then program-specific)
  ;

  _proto.buildDefineList = function buildDefineList() {
    var d,
        defines = {};

    for (d in ShaderProgram.defines) {
      defines[d] = ShaderProgram.defines[d];
    }

    for (d in this.defines) {
      defines[d] = this.defines[d];
    }

    return defines;
  } // Make list of shader blocks (global, then program-specific)
  ;

  _proto.buildShaderBlockList = function buildShaderBlockList() {
    var key,
        blocks = {}; // Global blocks

    for (key in ShaderProgram.blocks) {
      blocks[key] = [];

      if (Array.isArray(ShaderProgram.blocks[key])) {
        var _blocks$key;

        (_blocks$key = blocks[key]).push.apply(_blocks$key, ShaderProgram.blocks[key].map(function (source, num) {
          return {
            key: key,
            source: source,
            num: num,
            scope: 'ShaderProgram'
          };
        }));
      } else {
        blocks[key] = [{
          key: key,
          source: ShaderProgram.blocks[key],
          num: 0,
          scope: 'ShaderProgram'
        }];
      }
    } // Program-specific blocks


    for (key in this.blocks) {
      blocks[key] = blocks[key] || [];

      if (Array.isArray(this.blocks[key])) {
        var scopes = this.block_scopes && this.block_scopes[key] || [];
        var cur_scope = null,
            num = 0;

        for (var b = 0; b < this.blocks[key].length; b++) {
          // Count blocks relative to current scope
          if (scopes[b] !== cur_scope) {
            cur_scope = scopes[b];
            num = 0;
          }

          blocks[key].push({
            key: key,
            source: this.blocks[key][b],
            num: num,
            scope: cur_scope || this.name
          });
          num++;
        }
      } else {
        // TODO: address discrepancy in array vs. single-value blocks
        // styles assume array when tracking block scopes
        blocks[key].push({
          key: key,
          source: this.blocks[key],
          num: 0,
          scope: this.name
        });
      }
    }

    return blocks;
  } // Inject uniform definitions
  ;

  _proto.ensureUniforms = function ensureUniforms(uniforms) {
    if (!uniforms) {
      return;
    } // Get GLSL definitions


    var inject = Object.entries(uniforms).map(function (_ref) {
      var name = _ref[0],
          uniform = _ref[1];
      return GLSL.defineUniform(name, uniform);
    }).filter(function (x) {
      return x;
    }); // Inject uniforms
    // NOTE: these are injected at the very top of the shaders, even before any #defines or #pragmas are added
    // this could cause some issues with certain #pragmas, or other functions that might expect #defines

    this.computed_vertex_source = inject.join('\n') + this.computed_vertex_source;
    this.computed_fragment_source = inject.join('\n') + this.computed_fragment_source;
  } // Set uniforms from a JS object, with inferred types
  ;

  _proto.setUniforms = function setUniforms(uniforms, reset_texture_unit) {
    if (reset_texture_unit === void 0) {
      reset_texture_unit = true;
    }

    if (!this.compiled) {
      return;
    } // TODO: only update uniforms when changed
    // Texture units must be tracked and incremented each time a texture sampler uniform is set.
    // By default, the texture unit is reset to 0 each time setUniforms is called, but they can
    // also be preserved, for example in cases where multiple calls to setUniforms are expected
    // (e.g. program-specific uniforms followed by mesh-specific ones).


    if (reset_texture_unit) {
      this.texture_unit = 0;
    } // Parse uniform types and values from the JS object


    var parsed = GLSL.parseUniforms(uniforms); // Set each uniform

    for (var u = 0; u < parsed.length; u++) {
      var uniform = parsed[u];

      if (uniform.type === 'sampler2D') {
        // For textures, we need to track texture units, so we have a special setter
        this.setTextureUniform(uniform.name, uniform.value);
      } else {
        this.uniform(uniform.method, uniform.name, uniform.value);
      }
    }
  } // Cache some or all uniform values so they can be restored
  ;

  _proto.saveUniforms = function saveUniforms(subset) {
    var uniforms = subset || this.uniforms;

    for (var u in uniforms) {
      var uniform = this.uniforms[u];

      if (uniform) {
        uniform.saved_value = uniform.value;
      }
    }

    this.saved_texture_unit = this.texture_unit || 0;
  } // Restore some or all uniforms to saved values
  ;

  _proto.restoreUniforms = function restoreUniforms(subset) {
    var uniforms = subset || this.uniforms;

    for (var u in uniforms) {
      var uniform = this.uniforms[u];

      if (uniform && uniform.saved_value) {
        uniform.value = uniform.saved_value;
        this.updateUniform(uniform);
      }
    }

    this.texture_unit = this.saved_texture_unit || 0;
  } // Set a texture uniform, finds texture by name or creates a new one
  ;

  _proto.setTextureUniform = function setTextureUniform(uniform_name, texture_name) {
    var texture = Texture.textures[texture_name];

    if (texture == null) {
      log('warn', "Cannot find texture '" + texture_name + "'");
      return;
    }

    texture.bind(this.texture_unit);
    this.uniform('1i', uniform_name, this.texture_unit);
    this.texture_unit++; // TODO: track max texture units and log/throw errors
  } // ex: program.uniform('3fv', 'position', [x, y, z]);
  // TODO: only update uniforms when changed
  ;

  _proto.uniform = function uniform(method, name, value) {
    // 'value' is a method-appropriate arguments list
    if (!this.compiled) {
      return;
    }

    this.uniforms[name] = this.uniforms[name] || {};
    var uniform = this.uniforms[name];
    uniform.name = name;

    if (uniform.location === undefined) {
      uniform.location = this.gl.getUniformLocation(this.program, name);
    }

    uniform.method = method;
    uniform.value = value;
    this.updateUniform(uniform);
  } // Set a single uniform
  ;

  _proto.updateUniform = function updateUniform(uniform) {
    if (!this.compiled) {
      return;
    }

    if (!uniform || uniform.location == null) {
      return;
    }

    this.use();
    this.commitUniform(uniform);
  } // Commits the uniform to the GPU
  ;

  _proto.commitUniform = function commitUniform(uniform) {
    var location = uniform.location;
    var value = uniform.value;

    switch (uniform.method) {
      case '1i':
        this.gl.uniform1i(location, value);
        break;

      case '1f':
        this.gl.uniform1f(location, value);
        break;

      case '2f':
        this.gl.uniform2f(location, value[0], value[1]);
        break;

      case '3f':
        this.gl.uniform3f(location, value[0], value[1], value[2]);
        break;

      case '4f':
        this.gl.uniform4f(location, value[0], value[1], value[2], value[3]);
        break;

      case '1iv':
        this.gl.uniform1iv(location, value);
        break;

      case '3iv':
        this.gl.uniform3iv(location, value);
        break;

      case '1fv':
        this.gl.uniform1fv(location, value);
        break;

      case '2fv':
        this.gl.uniform2fv(location, value);
        break;

      case '3fv':
        this.gl.uniform3fv(location, value);
        break;

      case '4fv':
        this.gl.uniform4fv(location, value);
        break;

      case 'Matrix3fv':
        this.gl.uniformMatrix3fv(location, false, value);
        break;

      case 'Matrix4fv':
        this.gl.uniformMatrix4fv(location, false, value);
        break;
    }
  } // Refresh uniform locations and set to last cached values
  ;

  _proto.refreshUniforms = function refreshUniforms() {
    if (!this.compiled) {
      return;
    }

    for (var u in this.uniforms) {
      var uniform = this.uniforms[u];
      uniform.location = this.gl.getUniformLocation(this.program, u);
      this.updateUniform(uniform);
    }
  };

  _proto.refreshAttributes = function refreshAttributes() {
    // var len = this.gl.getProgramParameter(this.program, this.gl.ACTIVE_ATTRIBUTES);
    // for (var i=0; i < len; i++) {
    //     var a = this.gl.getActiveAttrib(this.program, i);
    // }
    this.attribs = {};
  } // Get the location of a vertex attribute
  ;

  _proto.attribute = function attribute(name) {
    if (!this.compiled) {
      return;
    }

    var attrib = this.attribs[name] = this.attribs[name] || {};

    if (attrib.location != null) {
      return attrib;
    }

    attrib.name = name;
    attrib.location = this.gl.getAttribLocation(this.program, name); // var info = this.gl.getActiveAttrib(this.program, attrib.location);
    // attrib.type = info.type;
    // attrib.size = info.size;

    return attrib;
  } // Get shader source as string
  ;

  _proto.source = function source(type) {
    if (type === 'vertex') {
      return this.computed_vertex_source;
    } else if (type === 'fragment') {
      return this.computed_fragment_source;
    }
  } // Get shader source as array of line strings
  ;

  _proto.lines = function lines(type) {
    var source = this.source(type);

    if (source) {
      return source.split('\n');
    }

    return [];
  } // Get a specific line from shader source
  ;

  _proto.line = function line(type, num) {
    var source = this.lines(type);

    if (source) {
      return source[num];
    }
  } // Get info on which shader block (if any) a particular line number in a shader is in
  // Returns an object with the following info if a block is found: { name, line, source }
  //  scope: where the shader block originated, either a style name, or global such as ShaderProgram
  //  name: shader block name (e.g. 'color', 'position', 'global')
  //  num: the block number *within* local scope (e.g. if a style has multiple 'color' blocks)
  //  line: line number *within* the shader block (not the whole shader program), useful for error highlighting
  //  source: the code for the line
  // NOTE: this does a bruteforce loop over the shader source and looks for shader block start/end markers
  // We could track line ranges for shader blocks as they are inserted, but as this code is only used for
  // error handling on compilation failure, it was simpler to keep it separate than to burden the core
  // compilation path.
  ;

  _proto.block = function block(type, num) {
    var lines = this.lines(type);
    var block;

    for (var i = 0; i < num && i < lines.length; i++) {
      var line = lines[i];
      var match = line.match(/\/\/ tangram-block-start: ([A-Za-z0-9_-]+), ([A-Za-z0-9_-]+), (\d+)/);

      if (match && match.length > 1) {
        // mark current block
        block = {
          scope: match[1],
          name: match[2],
          num: match[3]
        };
      } else {
        match = line.match(/\/\/ tangram-block-end: ([A-Za-z0-9_-]+), ([A-Za-z0-9_-]+), (\d+)/);

        if (match && match.length > 1) {
          block = null; // clear current block
        }
      } // update line # and content


      if (block) {
        // init to -1 so that line 0 is first actual line of block code, after comment marker
        block.line = block.line == null ? -1 : block.line + 1;
        block.source = line;
      }
    }

    return block;
  } // Returns list of available extensions from those requested
  // Sets internal #defines indicating availability of each requested extension
  ;

  _proto.checkExtensions = function checkExtensions() {
    var _this2 = this;

    var exts = [];
    this.extensions.forEach(function (name) {
      var ext = getExtension(_this2.gl, name);
      var def = "TANGRAM_EXTENSION_" + name;
      _this2.defines[def] = ext != null;

      if (ext) {
        exts.push(name);
      } else {
        log('debug', "Could not enable extension '" + name + "'");
      }
    });
    return exts;
  };

  return ShaderProgram;
}(); // Static methods and state
ShaderProgram.id = 0; // assign each program a unique id

ShaderProgram.current = null; // currently bound program
// Global config applied to all programs (duplicate properties for a specific program will take precedence)

ShaderProgram.defines = {};
ShaderProgram.blocks = {}; // Reset program and shader caches

ShaderProgram.reset = function () {
  ShaderProgram.programs_by_source = {}; // GL program objects by exact vertex + fragment shader source

  ShaderProgram.shaders_by_source = {}; // GL shader objects by exact source
};

ShaderProgram.reset(); // Turn an object of key/value pairs into single string of #define statements

ShaderProgram.buildDefineString = function (defines) {
  var define_str = '';

  for (var d in defines) {
    if (defines[d] == null || defines[d] === false) {
      continue;
    } else if (typeof defines[d] === 'boolean' && defines[d] === true) {
      // booleans are simple defines with no value
      define_str += '#define ' + d + '\n';
    } else if (typeof defines[d] === 'number' && Math.floor(defines[d]) === defines[d]) {
      // int to float conversion to satisfy GLSL floats
      define_str += '#define ' + d + ' ' + defines[d].toFixed(1) + '\n';
    } else {
      // any other float or string value
      define_str += '#define ' + d + ' ' + defines[d] + '\n';
    }
  }

  return define_str;
}; // Turn a list of extension names into single string of #extension statements


ShaderProgram.buildExtensionString = function (extensions) {
  extensions = extensions || [];
  var str = '';
  extensions.forEach(function (ext) {
    str += "#ifdef GL_" + ext + "\n#extension GL_" + ext + " : enable\n#endif\n";
  });
  return str;
};

ShaderProgram.addBlock = function (key) {
  var _ShaderProgram$blocks;

  ShaderProgram.blocks[key] = ShaderProgram.blocks[key] || [];

  for (var _len = arguments.length, blocks = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    blocks[_key - 1] = arguments[_key];
  }

  (_ShaderProgram$blocks = ShaderProgram.blocks[key]).push.apply(_ShaderProgram$blocks, blocks);
}; // Remove all global shader blocks for a given key


ShaderProgram.removeBlock = function (key) {
  ShaderProgram.blocks[key] = [];
};

ShaderProgram.replaceBlock = function (key) {
  ShaderProgram.removeBlock(key);

  for (var _len2 = arguments.length, blocks = new Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
    blocks[_key2 - 1] = arguments[_key2];
  }

  ShaderProgram.addBlock.apply(ShaderProgram, [key].concat(blocks));
}; // Compile & link a WebGL program from provided vertex and fragment shader sources
// update a program if one is passed in. Create one if not. Alert and don't update anything if the shaders don't compile.


ShaderProgram.updateProgram = function (gl, program, vertex_shader_source, fragment_shader_source) {
  // Program with this exact vertex and fragment shader sources already cached?
  var key = hashString(gl._tangram_id + '::' + vertex_shader_source + '::' + fragment_shader_source);

  if (ShaderProgram.programs_by_source[key]) {
    log('trace', 'Reusing identical source GL program object');
    return ShaderProgram.programs_by_source[key];
  }

  var vertex_shader = ShaderProgram.createShader(gl, vertex_shader_source, gl.VERTEX_SHADER);
  var fragment_shader = ShaderProgram.createShader(gl, fragment_shader_source, gl.FRAGMENT_SHADER);
  gl.useProgram(null);

  if (program != null) {
    var old_shaders = gl.getAttachedShaders(program);

    for (var i = 0; i < old_shaders.length; i++) {
      gl.detachShader(program, old_shaders[i]);
    }
  } else {
    program = gl.createProgram();
  }

  if (vertex_shader == null || fragment_shader == null) {
    return program;
  }

  gl.attachShader(program, vertex_shader);
  gl.attachShader(program, fragment_shader); // Require position to be at attribute location 0
  // Attribute 0 should never be disabled (per GL best practices). All of our shader programs have an `a_position`
  // attribute, and it's customary for the vertex position to be the first attribute, so we enforce that here.
  // This can avoid unexpected/undefined interaction between static and dynamic attributes in Safari, and
  // possible warnings/errors in other browsers.
  // See https://stackoverflow.com/questions/20305231/webgl-warning-attribute-0-is-disabled-this-has-significant-performance-penalt/20923946

  gl.bindAttribLocation(program, 0, 'a_position');
  gl.linkProgram(program); // TODO: reference count and delete shader objects when no programs reference them

  if (!gl.getProgramParameter(program, gl.LINK_STATUS)) {
    var message = new Error("WebGL program error:\n            VALIDATE_STATUS: " + gl.getProgramParameter(program, gl.VALIDATE_STATUS) + "\n            ERROR: " + gl.getError() + "\n            --- Vertex Shader ---\n            " + vertex_shader_source + "\n            --- Fragment Shader ---\n            " + fragment_shader_source);
    throw Object.assign(new Error(message), {
      type: 'program'
    });
  }

  ShaderProgram.programs_by_source[key] = program; // cache by exact source

  return program;
}; // Compile a vertex or fragment shader from provided source


ShaderProgram.createShader = function (gl, source, stype) {
  // Program with identical vertex and fragment shader sources already cached?
  var key = hashString(gl._tangram_id + '::' + source);

  if (ShaderProgram.shaders_by_source[key]) {
    log('trace', 'Reusing identical source GL shader object');
    return ShaderProgram.shaders_by_source[key];
  }

  var shader = gl.createShader(stype);
  gl.shaderSource(shader, source);
  gl.compileShader(shader);

  if (!gl.getShaderParameter(shader, gl.COMPILE_STATUS)) {
    var type = stype === gl.VERTEX_SHADER ? 'vertex' : 'fragment';
    var message = gl.getShaderInfoLog(shader);
    var errors = glShaderErrors(message);
    throw Object.assign(new Error(message), {
      type: type,
      errors: errors
    });
  }

  ShaderProgram.shaders_by_source[key] = shader; // cache by exact source

  return shader;
};

// Creates a Vertex Array Object if the extension is available, or falls back on standard attribute calls
var VertexArrayObject = {
  disabled: false,
  // set to true to disable VAOs even if extension is available
  bound_vao: [],
  // currently bound VAO, by GL context
  init: function init(gl) {
    var ext;

    if (this.disabled !== true) {
      ext = getExtension(gl, 'OES_vertex_array_object');
    }

    if (ext != null) {
      log('info', 'Vertex Array Object extension available');
    } else if (this.disabled !== true) {
      log('warn', 'Vertex Array Object extension NOT available');
    } else {
      log('warn', 'Vertex Array Object extension force disabled');
    }
  },
  getExtension: function getExtension$$1(gl, ext_name) {
    if (this.disabled !== true) {
      return getExtension(gl, ext_name);
    }
  },
  create: function create(gl, setup, teardown) {
    var vao = {};
    vao.setup = setup;
    vao.teardown = teardown;
    var ext = this.getExtension(gl, 'OES_vertex_array_object');

    if (ext != null) {
      vao._vao = ext.createVertexArrayOES();
      ext.bindVertexArrayOES(vao._vao);
    }

    vao.setup();
    return vao;
  },
  getCurrentBinding: function getCurrentBinding(gl) {
    var bound = this.bound_vao.filter(function (e) {
      return e[0] === gl;
    })[0];
    return bound && bound[1];
  },
  setCurrentBinding: function setCurrentBinding(gl, vao) {
    var bound_vao = this.bound_vao;
    var binding = bound_vao.filter(function (e) {
      return e[0] === gl;
    })[0];

    if (binding == null) {
      bound_vao.push([gl, vao]);
    } else {
      binding[1] = vao;
    }
  },
  bind: function bind(gl, vao) {
    var ext = this.getExtension(gl, 'OES_vertex_array_object');

    if (vao != null) {
      if (ext != null && vao._vao != null) {
        ext.bindVertexArrayOES(vao._vao);
        this.setCurrentBinding(gl, vao);
      } else {
        vao.setup();
      }
    } else {
      var bound_vao = this.getCurrentBinding(gl);

      if (ext != null) {
        ext.bindVertexArrayOES(null);
      } else if (bound_vao != null && typeof bound_vao.teardown === 'function') {
        bound_vao.teardown();
      }

      this.setCurrentBinding(gl, null);
    }
  },
  destroy: function destroy(gl, vao) {
    var ext = this.getExtension(gl, 'OES_vertex_array_object');

    if (ext != null && vao != null && vao._vao != null) {
      ext.deleteVertexArrayOES(vao._vao);
      vao._vao = null;
    } // destroy is a no-op if VAO extension isn't available

  }
};

// https://github.com/tc39/proposal-object-values-entries

var $values = _objectToArray(false);

_export(_export.S, 'Object', {
  values: function values(it) {
    return $values(it);
  }
});

// 22.1.3.9 Array.prototype.findIndex(predicate, thisArg = undefined)

var $find = _arrayMethods(6);
var KEY = 'findIndex';
var forced = true;
// Shouldn't skip holes
if (KEY in []) Array(1)[KEY](function () { forced = false; });
_export(_export.P + _export.F * forced, 'Array', {
  findIndex: function findIndex(callbackfn /* , that = undefined */) {
    return $find(this, callbackfn, arguments.length > 1 ? arguments[1] : undefined);
  }
});
_addToUnscopables(KEY);

// Deep/recursive merge of one or more source objects into a destination object
function mergeObjects(dest) {
  for (var s = 0; s < (arguments.length <= 1 ? 0 : arguments.length - 1); s++) {
    var source = s + 1 < 1 || arguments.length <= s + 1 ? undefined : arguments[s + 1];

    if (!source) {
      continue;
    }

    for (var key in source) {
      var value = source[key]; // Recursively merge the source into the destination if it is a a non-null key/value object
      // (e.g. don't merge arrays, those are treated as scalar values; null values will overwrite/erase
      // the previous destination value)

      if (value !== null && typeof value === 'object' && !Array.isArray(value)) {
        if (dest[key] !== null && typeof dest[key] === 'object' && !Array.isArray(dest[key])) {
          dest[key] = mergeObjects(dest[key], value);
        } else {
          dest[key] = mergeObjects({}, value); // destination not an object, overwrite
        }
      } // Overwrite the previous destination value if the source property is: a scalar (number/string),
      // an array, or a null value
      else if (value !== undefined) {
          dest[key] = value;
        } // Undefined source properties are ignored

    }
  }

  return dest;
}

// Miscellaneous geo functions
var Geo;
var Geo$1 = Geo = {}; // Projection constants

Geo.default_source_max_zoom = 18;
Geo.default_view_max_zoom = 20;
Geo.max_style_zoom = 25; // max zoom at which styles will be evaluated

Geo.tile_size = 256;
Geo.half_circumference_meters = 20037508.342789244;
Geo.circumference_meters = Geo.half_circumference_meters * 2;
Geo.min_zoom_meters_per_pixel = Geo.circumference_meters / Geo.tile_size; // min zoom draws world as 2 tiles wide

var meters_per_pixel = [];

Geo.metersPerPixel = function (z) {
  meters_per_pixel[z] = meters_per_pixel[z] || Geo.min_zoom_meters_per_pixel / Math.pow(2, z);
  return meters_per_pixel[z];
};

var meters_per_tile = [];

Geo.metersPerTile = function (z) {
  meters_per_tile[z] = meters_per_tile[z] || Geo.circumference_meters / Math.pow(2, z);
  return meters_per_tile[z];
}; // Conversion functions based on an defined tile scale


Geo.tile_scale = 4096; // coordinates are locally scaled to the range [0, tile_scale]

Geo.units_per_pixel = Geo.tile_scale / Geo.tile_size;
Geo.height_scale = 16; // provides sub-meter precision for height values (16ths of a meters)

var units_per_meter = [];

Geo.unitsPerMeter = function (z) {
  units_per_meter[z] = units_per_meter[z] || Geo.tile_scale / (Geo.tile_size * Geo.metersPerPixel(z));
  return units_per_meter[z];
}; // Convert tile location to mercator meters - multiply by pixels per tile, then by meters per pixel, adjust for map origin


Geo.metersForTile = function (tile) {
  return {
    x: tile.x * Geo.circumference_meters / Math.pow(2, tile.z) - Geo.half_circumference_meters,
    y: -(tile.y * Geo.circumference_meters / Math.pow(2, tile.z) - Geo.half_circumference_meters)
  };
};
/**
   Given a point in mercator meters and a zoom level, return the tile X/Y/Z that the point lies in
*/


Geo.tileForMeters = function (_ref, zoom) {
  var x = _ref[0],
      y = _ref[1];
  return {
    x: Math.floor((x + Geo.half_circumference_meters) / (Geo.circumference_meters / Math.pow(2, zoom))),
    y: Math.floor((-y + Geo.half_circumference_meters) / (Geo.circumference_meters / Math.pow(2, zoom))),
    z: zoom
  };
}; // Wrap a tile to positive #s for zoom
// Optionally specify the axes to wrap


Geo.wrapTile = function (_ref2, mask) {
  var x = _ref2.x,
      y = _ref2.y,
      z = _ref2.z;

  if (mask === void 0) {
    mask = {
      x: true,
      y: false
    };
  }

  var m = (1 << z) - 1;

  if (mask.x) {
    x = x & m;
  }

  if (mask.y) {
    y = y & m;
  }

  return {
    x: x,
    y: y,
    z: z
  };
};
/**
   Convert mercator meters to lat-lng, in-place
*/


Geo.metersToLatLng = function (c) {
  c[0] /= Geo.half_circumference_meters;
  c[1] /= Geo.half_circumference_meters;
  c[1] = (2 * Math.atan(Math.exp(c[1] * Math.PI)) - Math.PI / 2) / Math.PI;
  c[0] *= 180;
  c[1] *= 180;
  return c;
};
/**
  Convert lat-lng to mercator meters, in-place
*/


Geo.latLngToMeters = function (c) {
  // Latitude
  c[1] = Math.log(Math.tan(c[1] * Math.PI / 360 + Math.PI / 4)) / Math.PI;
  c[1] *= Geo.half_circumference_meters; // Longitude

  c[0] *= Geo.half_circumference_meters / 180;
  return c;
}; // Transform from local tile coordinats to lat lng


Geo.tileSpaceToLatlng = function (geometry, z, min) {
  var units_per_meter = Geo.unitsPerMeter(z);
  Geo.transformGeometry(geometry, function (coord) {
    coord[0] = coord[0] / units_per_meter + min.x;
    coord[1] = coord[1] / units_per_meter + min.y;
    Geo.metersToLatLng(coord);
  });
  return geometry;
}; // Copy GeoJSON geometry


Geo.copyGeometry = function (geometry) {
  if (geometry == null) {
    return; // skip if missing geometry (valid GeoJSON)
  }

  var copy = {
    type: geometry.type
  };

  if (geometry.type === 'Point') {
    copy.coordinates = [geometry.coordinates[0], geometry.coordinates[1]];
  } else if (geometry.type === 'LineString' || geometry.type === 'MultiPoint') {
    copy.coordinates = geometry.coordinates.map(function (c) {
      return [c[0], c[1]];
    });
  } else if (geometry.type === 'Polygon' || geometry.type === 'MultiLineString') {
    copy.coordinates = geometry.coordinates.map(function (ring) {
      return ring.map(function (c) {
        return [c[0], c[1]];
      });
    });
  } else if (geometry.type === 'MultiPolygon') {
    copy.coordinates = geometry.coordinates.map(function (polygon) {
      return polygon.map(function (ring) {
        return ring.map(function (c) {
          return [c[0], c[1]];
        });
      });
    });
  } // TODO: support GeometryCollection


  return copy;
}; // Run an in-place transform function on each cooordinate in a GeoJSON geometry


Geo.transformGeometry = function (geometry, transform) {
  if (geometry == null) {
    return; // skip if missing geometry (valid GeoJSON)
  }

  if (geometry.type === 'Point') {
    transform(geometry.coordinates);
  } else if (geometry.type === 'LineString' || geometry.type === 'MultiPoint') {
    geometry.coordinates.forEach(transform);
  } else if (geometry.type === 'Polygon' || geometry.type === 'MultiLineString') {
    geometry.coordinates.forEach(function (ring) {
      return ring.forEach(transform);
    });
  } else if (geometry.type === 'MultiPolygon') {
    geometry.coordinates.forEach(function (polygon) {
      polygon.forEach(function (ring) {
        return ring.forEach(transform);
      });
    });
  } // TODO: support GeometryCollection

};

Geo.boxIntersect = function (b1, b2) {
  return !(b2.sw.x > b1.ne.x || b2.ne.x < b1.sw.x || b2.sw.y > b1.ne.y || b2.ne.y < b1.sw.y);
}; // Finds the axis-aligned bounding box for a polygon


Geo.findBoundingBox = function (polygon) {
  var min_x = Infinity,
      max_x = -Infinity,
      min_y = Infinity,
      max_y = -Infinity; // Only need to examine outer ring (polygon[0])

  var num_coords = polygon[0].length;

  for (var c = 0; c < num_coords; c++) {
    var coord = polygon[0][c];

    if (coord[0] < min_x) {
      min_x = coord[0];
    }

    if (coord[1] < min_y) {
      min_y = coord[1];
    }

    if (coord[0] > max_x) {
      max_x = coord[0];
    }

    if (coord[1] > max_y) {
      max_y = coord[1];
    }
  }

  return [min_x, min_y, max_x, max_y];
}; // Convert geometry type to one of: 'point', 'line', 'polygon'


Geo.geometryType = function (type) {
  if (type === 'Polygon' || type === 'MultiPolygon') {
    return 'polygon';
  } else if (type === 'LineString' || type === 'MultiLineString') {
    return 'line';
  }

  if (type === 'Point' || type === 'MultiPoint') {
    return 'point';
  }
}; // Geometric / weighted centroid of polygon
// Adapted from https://github.com/Leaflet/Leaflet/blob/c10f405a112142b19785967ce0e142132a6095ad/src/layer/vector/Polygon.js#L57


Geo.centroid = function (polygon, relative) {
  if (relative === void 0) {
    relative = true;
  }

  if (!polygon || polygon.length === 0) {
    return;
  }

  var x = 0,
      y = 0,
      area = 0;
  var ring = polygon[0]; // only use first ring for now

  var len = ring.length; // optionally calculate relative to first coordinate to avoid precision issues w/small polygons

  var origin;

  if (relative) {
    origin = ring[0];
    ring = ring.map(function (v) {
      return [v[0] - origin[0], v[1] - origin[1]];
    });
  }

  for (var i = 0, j = len - 1; i < len; j = i, i++) {
    var p0 = ring[i];
    var p1 = ring[j];
    var f = p0[1] * p1[0] - p1[1] * p0[0];
    x += (p0[0] + p1[0]) * f;
    y += (p0[1] + p1[1]) * f;
    area += f * 3;
  }

  if (!area) {
    return; // skip degenerate polygons
  }

  var c = [x / area, y / area];

  if (relative) {
    c[0] += origin[0];
    c[1] += origin[1];
  }

  return c;
};

Geo.multiCentroid = function (polygons) {
  var n = 0;
  var centroid = null;

  for (var p = 0; p < polygons.length; p++) {
    var c = Geo.centroid(polygons[p]);

    if (c) {
      // skip degenerate polygons
      centroid = centroid || [0, 0];
      centroid[0] += c[0];
      centroid[1] += c[1];
      n++;
    }
  }

  if (n > 0) {
    centroid[0] /= n;
    centroid[1] /= n;
  }

  return centroid; // will return null if all polygons were degenerate
};

Geo.signedPolygonRingAreaSum = function (ring) {
  var area = 0;
  var n = ring.length;

  for (var i = 0; i < n - 1; i++) {
    var p0 = ring[i];
    var p1 = ring[i + 1];
    area += p0[0] * p1[1] - p1[0] * p0[1];
  }

  area += ring[n - 1][0] * ring[0][1] - ring[0][0] * ring[n - 1][1];
  return area;
};

Geo.polygonRingArea = function (ring) {
  return Math.abs(Geo.signedPolygonRingAreaSum(ring)) / 2;
}; // TODO: subtract inner rings


Geo.polygonArea = function (polygon) {
  if (!polygon) {
    return;
  }

  return Geo.polygonRingArea(polygon[0]);
};

Geo.multiPolygonArea = function (polygons) {
  var area = 0;

  for (var p = 0; p < polygons.length; p++) {
    area += Geo.polygonArea(polygons[p]);
  }

  return area;
};

Geo.ringWinding = function (ring) {
  var area = Geo.signedPolygonRingAreaSum(ring);

  if (area > 0) {
    return 'CW';
  } else if (area < 0) {
    return 'CCW';
  } // return undefined on zero area polygon

};

var _stringWs = '\x09\x0A\x0B\x0C\x0D\x20\xA0\u1680\u180E\u2000\u2001\u2002\u2003' +
  '\u2004\u2005\u2006\u2007\u2008\u2009\u200A\u202F\u205F\u3000\u2028\u2029\uFEFF';

var space = '[' + _stringWs + ']';
var non = '\u200b\u0085';
var ltrim = RegExp('^' + space + space + '*');
var rtrim = RegExp(space + space + '*$');

var exporter = function (KEY, exec, ALIAS) {
  var exp = {};
  var FORCE = _fails(function () {
    return !!_stringWs[KEY]() || non[KEY]() != non;
  });
  var fn = exp[KEY] = FORCE ? exec(trim) : _stringWs[KEY];
  if (ALIAS) exp[ALIAS] = fn;
  _export(_export.P + _export.F * FORCE, 'String', exp);
};

// 1 -> String#trimLeft
// 2 -> String#trimRight
// 3 -> String#trim
var trim = exporter.trim = function (string, TYPE) {
  string = String(_defined(string));
  if (TYPE & 1) string = string.replace(ltrim, '');
  if (TYPE & 2) string = string.replace(rtrim, '');
  return string;
};

var _stringTrim = exporter;

var gOPN$1 = _objectGopn.f;
var gOPD$1 = _objectGopd.f;
var dP$3 = _objectDp.f;
var $trim = _stringTrim.trim;
var NUMBER = 'Number';
var $Number = _global[NUMBER];
var Base$1 = $Number;
var proto$2 = $Number.prototype;
// Opera ~12 has broken Object#toString
var BROKEN_COF = _cof(_objectCreate(proto$2)) == NUMBER;
var TRIM = 'trim' in String.prototype;

// 7.1.3 ToNumber(argument)
var toNumber = function (argument) {
  var it = _toPrimitive(argument, false);
  if (typeof it == 'string' && it.length > 2) {
    it = TRIM ? it.trim() : $trim(it, 3);
    var first = it.charCodeAt(0);
    var third, radix, maxCode;
    if (first === 43 || first === 45) {
      third = it.charCodeAt(2);
      if (third === 88 || third === 120) return NaN; // Number('+0x1') should be NaN, old V8 fix
    } else if (first === 48) {
      switch (it.charCodeAt(1)) {
        case 66: case 98: radix = 2; maxCode = 49; break; // fast equal /^0b[01]+$/i
        case 79: case 111: radix = 8; maxCode = 55; break; // fast equal /^0o[0-7]+$/i
        default: return +it;
      }
      for (var digits = it.slice(2), i = 0, l = digits.length, code; i < l; i++) {
        code = digits.charCodeAt(i);
        // parseInt parses a string to a first unavailable symbol
        // but ToNumber should return NaN if a string contains unavailable symbols
        if (code < 48 || code > maxCode) return NaN;
      } return parseInt(digits, radix);
    }
  } return +it;
};

if (!$Number(' 0o1') || !$Number('0b1') || $Number('+0x1')) {
  $Number = function Number(value) {
    var it = arguments.length < 1 ? 0 : value;
    var that = this;
    return that instanceof $Number
      // check on 1..constructor(foo) case
      && (BROKEN_COF ? _fails(function () { proto$2.valueOf.call(that); }) : _cof(that) != NUMBER)
        ? _inheritIfRequired(new Base$1(toNumber(it)), that, $Number) : toNumber(it);
  };
  for (var keys$1 = _descriptors ? gOPN$1(Base$1) : (
    // ES3:
    'MAX_VALUE,MIN_VALUE,NaN,NEGATIVE_INFINITY,POSITIVE_INFINITY,' +
    // ES6 (in case, if modules with ES6 Number statics required before):
    'EPSILON,isFinite,isInteger,isNaN,isSafeInteger,MAX_SAFE_INTEGER,' +
    'MIN_SAFE_INTEGER,parseFloat,parseInt,isInteger'
  ).split(','), j = 0, key$1; keys$1.length > j; j++) {
    if (_has(Base$1, key$1 = keys$1[j]) && !_has($Number, key$1)) {
      dP$3($Number, key$1, gOPD$1(Base$1, key$1));
    }
  }
  $Number.prototype = proto$2;
  proto$2.constructor = $Number;
  _redefine(_global, NUMBER, $Number);
}

// 20.1.2.10 Number.MIN_SAFE_INTEGER


_export(_export.S, 'Number', { MIN_SAFE_INTEGER: -0x1fffffffffffff });

// 19.1.2.5 Object.freeze(O)

var meta = _meta.onFreeze;

_objectSap('freeze', function ($freeze) {
  return function freeze(it) {
    return $freeze && _isObject(it) ? $freeze(meta(it)) : it;
  };
});

var cache = {
  functions: {},
  num_functions: 0,
  num_cached: 0
};
function clearFunctionStringCache() {
  cache.functions = {};
  cache.num_functions = 0;
  cache.num_cached = 0;
} // Recursively parse an object, compiling string properties that look like functions

function compileFunctionStrings(obj, wrap) {
  // Convert string
  if (typeof obj === 'string') {
    obj = compileFunctionString(obj, wrap);
  } // Loop through object properties
  else if (obj != null && typeof obj === 'object') {
      for (var p in obj) {
        obj[p] = compileFunctionStrings(obj[p], wrap);
      }
    }

  return obj;
} // Compile a string that looks like a function

function compileFunctionString(val, wrap) {
  // Parse function signature and body
  var fmatch = typeof val === 'string' && val.match(/^\s*function[^(]*\(([^)]*)\)\s*?\{([\s\S]*)\}$/m);

  if (fmatch && fmatch.length > 2) {
    try {
      // function body
      var body = fmatch[2];
      var source = typeof wrap === 'function' ? wrap(body) : body; // optionally wrap source
      // compile and cache by unique function source

      var key = hashString(source);

      if (cache.functions[key] === undefined) {
        // function arguments extracted from signature
        var args = fmatch[1].length > 0 && fmatch[1].split(',').map(function (x) {
          return x.trim();
        }).filter(function (x) {
          return x;
        });
        args = args.length > 0 ? args : ['context']; // default to single 'context' argument

        cache.functions[key] = new Function(args.toString(), source); // jshint ignore:line

        cache.functions[key].source = body; // save original, un-wrapped function body source

        cache.num_functions++;
      } else {
        cache.num_cached++;
      }

      return cache.functions[key];
    } catch (e) {
      // fall-back to original value if parsing failed
      return val;
    }
  }

  return val;
}

var csscolorparser = createCommonjsModule(function (module, exports) {
// (c) Dean McNamee <dean@gmail.com>, 2012.
//
// https://github.com/deanm/css-color-parser-js
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.

// http://www.w3.org/TR/css3-color/
var kCSSColorTable = {
  "transparent": [0,0,0,0], "aliceblue": [240,248,255,1],
  "antiquewhite": [250,235,215,1], "aqua": [0,255,255,1],
  "aquamarine": [127,255,212,1], "azure": [240,255,255,1],
  "beige": [245,245,220,1], "bisque": [255,228,196,1],
  "black": [0,0,0,1], "blanchedalmond": [255,235,205,1],
  "blue": [0,0,255,1], "blueviolet": [138,43,226,1],
  "brown": [165,42,42,1], "burlywood": [222,184,135,1],
  "cadetblue": [95,158,160,1], "chartreuse": [127,255,0,1],
  "chocolate": [210,105,30,1], "coral": [255,127,80,1],
  "cornflowerblue": [100,149,237,1], "cornsilk": [255,248,220,1],
  "crimson": [220,20,60,1], "cyan": [0,255,255,1],
  "darkblue": [0,0,139,1], "darkcyan": [0,139,139,1],
  "darkgoldenrod": [184,134,11,1], "darkgray": [169,169,169,1],
  "darkgreen": [0,100,0,1], "darkgrey": [169,169,169,1],
  "darkkhaki": [189,183,107,1], "darkmagenta": [139,0,139,1],
  "darkolivegreen": [85,107,47,1], "darkorange": [255,140,0,1],
  "darkorchid": [153,50,204,1], "darkred": [139,0,0,1],
  "darksalmon": [233,150,122,1], "darkseagreen": [143,188,143,1],
  "darkslateblue": [72,61,139,1], "darkslategray": [47,79,79,1],
  "darkslategrey": [47,79,79,1], "darkturquoise": [0,206,209,1],
  "darkviolet": [148,0,211,1], "deeppink": [255,20,147,1],
  "deepskyblue": [0,191,255,1], "dimgray": [105,105,105,1],
  "dimgrey": [105,105,105,1], "dodgerblue": [30,144,255,1],
  "firebrick": [178,34,34,1], "floralwhite": [255,250,240,1],
  "forestgreen": [34,139,34,1], "fuchsia": [255,0,255,1],
  "gainsboro": [220,220,220,1], "ghostwhite": [248,248,255,1],
  "gold": [255,215,0,1], "goldenrod": [218,165,32,1],
  "gray": [128,128,128,1], "green": [0,128,0,1],
  "greenyellow": [173,255,47,1], "grey": [128,128,128,1],
  "honeydew": [240,255,240,1], "hotpink": [255,105,180,1],
  "indianred": [205,92,92,1], "indigo": [75,0,130,1],
  "ivory": [255,255,240,1], "khaki": [240,230,140,1],
  "lavender": [230,230,250,1], "lavenderblush": [255,240,245,1],
  "lawngreen": [124,252,0,1], "lemonchiffon": [255,250,205,1],
  "lightblue": [173,216,230,1], "lightcoral": [240,128,128,1],
  "lightcyan": [224,255,255,1], "lightgoldenrodyellow": [250,250,210,1],
  "lightgray": [211,211,211,1], "lightgreen": [144,238,144,1],
  "lightgrey": [211,211,211,1], "lightpink": [255,182,193,1],
  "lightsalmon": [255,160,122,1], "lightseagreen": [32,178,170,1],
  "lightskyblue": [135,206,250,1], "lightslategray": [119,136,153,1],
  "lightslategrey": [119,136,153,1], "lightsteelblue": [176,196,222,1],
  "lightyellow": [255,255,224,1], "lime": [0,255,0,1],
  "limegreen": [50,205,50,1], "linen": [250,240,230,1],
  "magenta": [255,0,255,1], "maroon": [128,0,0,1],
  "mediumaquamarine": [102,205,170,1], "mediumblue": [0,0,205,1],
  "mediumorchid": [186,85,211,1], "mediumpurple": [147,112,219,1],
  "mediumseagreen": [60,179,113,1], "mediumslateblue": [123,104,238,1],
  "mediumspringgreen": [0,250,154,1], "mediumturquoise": [72,209,204,1],
  "mediumvioletred": [199,21,133,1], "midnightblue": [25,25,112,1],
  "mintcream": [245,255,250,1], "mistyrose": [255,228,225,1],
  "moccasin": [255,228,181,1], "navajowhite": [255,222,173,1],
  "navy": [0,0,128,1], "oldlace": [253,245,230,1],
  "olive": [128,128,0,1], "olivedrab": [107,142,35,1],
  "orange": [255,165,0,1], "orangered": [255,69,0,1],
  "orchid": [218,112,214,1], "palegoldenrod": [238,232,170,1],
  "palegreen": [152,251,152,1], "paleturquoise": [175,238,238,1],
  "palevioletred": [219,112,147,1], "papayawhip": [255,239,213,1],
  "peachpuff": [255,218,185,1], "peru": [205,133,63,1],
  "pink": [255,192,203,1], "plum": [221,160,221,1],
  "powderblue": [176,224,230,1], "purple": [128,0,128,1],
  "rebeccapurple": [102,51,153,1],
  "red": [255,0,0,1], "rosybrown": [188,143,143,1],
  "royalblue": [65,105,225,1], "saddlebrown": [139,69,19,1],
  "salmon": [250,128,114,1], "sandybrown": [244,164,96,1],
  "seagreen": [46,139,87,1], "seashell": [255,245,238,1],
  "sienna": [160,82,45,1], "silver": [192,192,192,1],
  "skyblue": [135,206,235,1], "slateblue": [106,90,205,1],
  "slategray": [112,128,144,1], "slategrey": [112,128,144,1],
  "snow": [255,250,250,1], "springgreen": [0,255,127,1],
  "steelblue": [70,130,180,1], "tan": [210,180,140,1],
  "teal": [0,128,128,1], "thistle": [216,191,216,1],
  "tomato": [255,99,71,1], "turquoise": [64,224,208,1],
  "violet": [238,130,238,1], "wheat": [245,222,179,1],
  "white": [255,255,255,1], "whitesmoke": [245,245,245,1],
  "yellow": [255,255,0,1], "yellowgreen": [154,205,50,1]};

function clamp_css_byte(i) {  // Clamp to integer 0 .. 255.
  i = Math.round(i);  // Seems to be what Chrome does (vs truncation).
  return i < 0 ? 0 : i > 255 ? 255 : i;
}

function clamp_css_float(f) {  // Clamp to float 0.0 .. 1.0.
  return f < 0 ? 0 : f > 1 ? 1 : f;
}

function parse_css_int(str) {  // int or percentage.
  if (str[str.length - 1] === '%')
    return clamp_css_byte(parseFloat(str) / 100 * 255);
  return clamp_css_byte(parseInt(str));
}

function parse_css_float(str) {  // float or percentage.
  if (str[str.length - 1] === '%')
    return clamp_css_float(parseFloat(str) / 100);
  return clamp_css_float(parseFloat(str));
}

function css_hue_to_rgb(m1, m2, h) {
  if (h < 0) h += 1;
  else if (h > 1) h -= 1;

  if (h * 6 < 1) return m1 + (m2 - m1) * h * 6;
  if (h * 2 < 1) return m2;
  if (h * 3 < 2) return m1 + (m2 - m1) * (2/3 - h) * 6;
  return m1;
}

function parseCSSColor(css_str) {
  // Remove all whitespace, not compliant, but should just be more accepting.
  var str = css_str.replace(/ /g, '').toLowerCase();

  // Color keywords (and transparent) lookup.
  if (str in kCSSColorTable) return kCSSColorTable[str].slice();  // dup.

  // #abc and #abc123 syntax.
  if (str[0] === '#') {
    if (str.length === 4) {
      var iv = parseInt(str.substr(1), 16);  // TODO(deanm): Stricter parsing.
      if (!(iv >= 0 && iv <= 0xfff)) return null;  // Covers NaN.
      return [((iv & 0xf00) >> 4) | ((iv & 0xf00) >> 8),
              (iv & 0xf0) | ((iv & 0xf0) >> 4),
              (iv & 0xf) | ((iv & 0xf) << 4),
              1];
    } else if (str.length === 7) {
      var iv = parseInt(str.substr(1), 16);  // TODO(deanm): Stricter parsing.
      if (!(iv >= 0 && iv <= 0xffffff)) return null;  // Covers NaN.
      return [(iv & 0xff0000) >> 16,
              (iv & 0xff00) >> 8,
              iv & 0xff,
              1];
    }

    return null;
  }

  var op = str.indexOf('('), ep = str.indexOf(')');
  if (op !== -1 && ep + 1 === str.length) {
    var fname = str.substr(0, op);
    var params = str.substr(op+1, ep-(op+1)).split(',');
    var alpha = 1;  // To allow case fallthrough.
    switch (fname) {
      case 'rgba':
        if (params.length !== 4) return null;
        alpha = parse_css_float(params.pop());
        // Fall through.
      case 'rgb':
        if (params.length !== 3) return null;
        return [parse_css_int(params[0]),
                parse_css_int(params[1]),
                parse_css_int(params[2]),
                alpha];
      case 'hsla':
        if (params.length !== 4) return null;
        alpha = parse_css_float(params.pop());
        // Fall through.
      case 'hsl':
        if (params.length !== 3) return null;
        var h = (((parseFloat(params[0]) % 360) + 360) % 360) / 360;  // 0 .. 1
        // NOTE(deanm): According to the CSS spec s/l should only be
        // percentages, but we don't bother and let float or percentage.
        var s = parse_css_float(params[1]);
        var l = parse_css_float(params[2]);
        var m2 = l <= 0.5 ? l * (s + 1) : l + s - l * s;
        var m1 = l * 2 - m2;
        return [clamp_css_byte(css_hue_to_rgb(m1, m2, h+1/3) * 255),
                clamp_css_byte(css_hue_to_rgb(m1, m2, h) * 255),
                clamp_css_byte(css_hue_to_rgb(m1, m2, h-1/3) * 255),
                alpha];
      default:
        return null;
    }
  }

  return null;
}

try { exports.parseCSSColor = parseCSSColor; } catch(e) { }
});
var csscolorparser_1 = csscolorparser.parseCSSColor;

var StyleParser = {};

var clampPositive = function clampPositive(v) {
  return Math.max(v, 0);
};

var noNaN = function noNaN(v) {
  return isNaN(v) ? 0 : v;
};

var parseNumber = function parseNumber(v) {
  return Array.isArray(v) ? v.map(parseFloat).map(noNaN) : noNaN(parseFloat(v));
};

var parsePositiveNumber = function parsePositiveNumber(v) {
  return Array.isArray(v) ? v.map(parseNumber).map(clampPositive) : clampPositive(parseNumber(v));
};

Object.assign(StyleParser, {
  clampPositive: clampPositive,
  noNaN: noNaN,
  parseNumber: parseNumber,
  parsePositiveNumber: parsePositiveNumber
}); // Wraps style functions and provides a scope of commonly accessible data:
// - feature: the 'properties' of the feature, e.g. accessed as 'feature.name'
// - global: user-defined properties on the `global` object in the scene file
// - $zoom: the current map zoom level
// - $geometry: the type of geometry, 'point', 'line', or 'polygon'
// - $meters_per_pixel: conversion for meters/pixels at current map zoom

StyleParser.wrapFunction = function (func) {
  var f = "\n        var feature = context.feature.properties;\n        var global = context.global;\n        var $zoom = context.zoom;\n        var $layer = context.layer;\n        var $source = context.source;\n        var $geometry = context.geometry;\n        var $meters_per_pixel = context.meters_per_pixel;\n\n        var val = (function(){ " + func + " }());\n\n        if (typeof val === 'number' && isNaN(val)) {\n            val = null; // convert NaNs to nulls\n        }\n\n        return val;\n    ";
  return f;
}; // Style parsing


StyleParser.zeroPair = Object.freeze([0, 0]); // single allocation for zero values that won't be modified
// Style defaults

StyleParser.defaults = {
  color: [1, 1, 1, 1],
  width: 1,
  size: 1,
  extrude: false,
  height: 20,
  min_height: 0,
  order: 0,
  z: 0,
  outline: {
    color: [0, 0, 0, 0],
    width: 0
  },
  material: {
    ambient: 1,
    diffuse: 1
  }
}; // Style macros

StyleParser.macros = {
  // pseudo-random color by geometry id
  'Style.color.pseudoRandomColor': function StyleColorPseudoRandomColor() {
    return [0.7 * (parseInt(feature.id, 16) / 100 % 1), // eslint-disable-line no-undef
    0.7 * (parseInt(feature.id, 16) / 10000 % 1), // eslint-disable-line no-undef
    0.7 * (parseInt(feature.id, 16) / 1000000 % 1), // eslint-disable-line no-undef
    1];
  },
  // random color
  'Style.color.randomColor': function StyleColorRandomColor() {
    return [0.7 * Math.random(), 0.7 * Math.random(), 0.7 * Math.random(), 1];
  }
}; // A context object that is passed to style parsing functions to provide a scope of commonly used values

StyleParser.getFeatureParseContext = function (feature, tile, global) {
  return {
    feature: feature,
    tile: tile,
    global: global,
    zoom: tile.style_z,
    geometry: Geo$1.geometryType(feature.geometry.type),
    meters_per_pixel: tile.meters_per_pixel,
    meters_per_pixel_sq: tile.meters_per_pixel_sq,
    units_per_meter_overzoom: tile.units_per_meter_overzoom
  };
}; // Build a style param cache object
// `value` is raw value, cache methods will add other properties as needed
// `transform` is optional transform function to run on values (except function values)


var CACHE_TYPE = {
  STATIC: 0,
  DYNAMIC: 1,
  ZOOM: 2
};
StyleParser.CACHE_TYPE = CACHE_TYPE;

StyleParser.createPropertyCache = function (obj, transform) {
  if (transform === void 0) {
    transform = null;
  }

  if (obj == null) {
    return;
  }

  if (obj.value) {
    return {
      value: obj.value,
      zoom: obj.zoom ? {} : null,
      type: obj.type
    }; // clone existing cache object
  }

  var c = {
    value: obj,
    type: CACHE_TYPE.STATIC
  }; // does value contain zoom stops to be interpolated?

  if (Array.isArray(c.value) && Array.isArray(c.value[0])) {
    c.zoom = {}; // will hold values interpolated by zoom

    c.type = CACHE_TYPE.ZOOM;
  } else if (typeof c.value === 'function') {
    c.type = CACHE_TYPE.DYNAMIC;
  } // apply optional transform function - usually a parsing function


  if (typeof transform === 'function') {
    if (c.zoom) {
      // apply to each zoom stop value
      c.value = c.value.map(function (v, i) {
        return [v[0], transform(v[1], i)];
      });
    } else if (typeof c.value !== 'function') {
      // don't transform functions
      c.value = transform(c.value, 0); // single value, 0 = the first and only item in the array
    }
  }

  return c;
}; // Convert old-style color macro into a function
// TODO: deprecate this macro syntax


StyleParser.createColorPropertyCache = function (obj) {
  return StyleParser.createPropertyCache(obj, function (v) {
    if (v === 'Style.color.pseudoRandomColor') {
      return compileFunctionString(StyleParser.wrapFunction(StyleParser.macros['Style.color.pseudoRandomColor']));
    } else if (v === 'Style.color.randomColor') {
      return StyleParser.macros['Style.color.randomColor'];
    }

    return v;
  });
}; // Parse point sizes, which include optional %-based or aspect-ratio-constrained scaling from sprite size
// Returns a cache object if successful, otherwise throws error message


var isPercent = function isPercent(v) {
  return typeof v === 'string' && v[v.length - 1] === '%';
}; // size computed by %


var isRatio = function isRatio(v) {
  return v === 'auto';
}; // size derived from aspect ratio of one dimension


var isComputed = function isComputed(v) {
  return isPercent(v) || isRatio(v);
};

var dualRatioError = '\'size\' can specify either width or height as derived from aspect ratio, but not both';

StyleParser.createPointSizePropertyCache = function (obj, texture) {
  // obj is the value to be parsed eg "64px" "100%" "auto"
  // mimics the structure of the size value (at each zoom stop if applicable),
  // stores flags indicating if each element is a %-based size or not, or derived from aspect
  var has_pct = null;
  var has_ratio = null;

  if (isPercent(obj)) {
    // 1D size
    has_pct = [true];
  } else if (Array.isArray(obj)) {
    // track which fields are % vals
    if (Array.isArray(obj[0])) {
      // zoom stops
      // could be a 1D value (that could be a %), or a 2D value (either width or height or both could be a %)
      if (obj.some(function (v) {
        return Array.isArray(v[1]) ? v[1].some(function (w) {
          return isComputed(w);
        }) : isPercent(v[1]);
      })) {
        has_pct = obj.map(function (v) {
          return Array.isArray(v[1]) ? v[1].map(function (w) {
            return isPercent(w);
          }) : isPercent(v[1]);
        });
        has_ratio = obj.map(function (v) {
          return Array.isArray(v[1]) && v[1].map(function (w) {
            return isRatio(w);
          });
        });

        if (has_ratio.some(function (v) {
          return Array.isArray(v) && v.every(function (c) {
            return c;
          });
        })) {
          throw dualRatioError; // invalid case where both dims are ratios
        }
      }
    } else if (obj.some(isComputed)) {
      // 2D size
      has_pct = [obj.map(isPercent)];
      has_ratio = [obj.map(isRatio)];

      if (has_ratio[0].every(function (c) {
        return c;
      })) {
        throw dualRatioError; // invalid case where both dims are ratios
      }
    }
  } else if (isRatio(obj)) {
    throw 'this value only allowed as half of an array, eg [16px, auto]:'; // TODO: add this error check for zoom stop parsing above
  }

  if (has_pct || has_ratio) {
    // texture is required when % or ratio sizes are used
    if (!texture) {
      throw '% or \'auto\' keywords can only be used to specify point size when a texture is defined';
    } // per-sprite based evaluation


    obj = {
      value: obj
    };
    obj.has_pct = has_pct;
    obj.has_ratio = has_ratio;
    obj.sprites = {}; // cache by sprite
  } else {
    // no % or aspect ratio sizing, one cache for texture or all sprites
    obj = StyleParser.createPropertyCache(obj, parsePositiveNumber);
  }

  return obj;
};

StyleParser.evalCachedPointSizeProperty = function (val, sprite_info, texture_info, context) {
  // no percentage-based calculation, one cache for all sprites
  if (!val.has_pct && !val.has_ratio) {
    return StyleParser.evalCachedProperty(val, context);
  }

  if (sprite_info) {
    // per-sprite based evaluation, cache sizes per sprite
    if (!val.sprites[sprite_info.sprite]) {
      val.sprites[sprite_info.sprite] = createPointSizeCacheEntry(val, sprite_info);
    }

    return StyleParser.evalCachedProperty(val.sprites[sprite_info.sprite], context);
  } else {
    // texture-based evaluation
    // apply percentage or ratio sizing to a texture
    val.texture = val.texture || createPointSizeCacheEntry(val, texture_info);
    return StyleParser.evalCachedProperty(val.texture, context);
  }
};

function createPointSizeCacheEntry(val, image_info) {
  // the cache property transform function needs access to the image in `val`
  // so it's accessed via a closure here
  return StyleParser.createPropertyCache(val.value, function (v, i) {
    if (Array.isArray(v)) {
      // 2D size
      // either width or height or both could be a %
      v = v.map(function (c, j) {
        return val.has_ratio[i][j] ? c : parsePositiveNumber(c);
      }). // convert non-ratio values to px
      map(function (c, j) {
        return val.has_pct[i][j] ? image_info.css_size[j] * c / 100 : c;
      }); // apply % scaling as needed
      // either width or height could be a ratio

      if (val.has_ratio[i][0]) {
        v[0] = v[1] * image_info.aspect;
      } else if (val.has_ratio[i][1]) {
        v[1] = v[0] / image_info.aspect;
      }
    } else {
      // 1D size
      v = parsePositiveNumber(v);

      if (val.has_pct[i]) {
        v = image_info.css_size.map(function (c) {
          return c * v / 100;
        }); // set size as % of image
      } else {
        v = [v, v]; // expand 1D size to 2D
      }
    }

    return v;
  });
} // Interpolation and caching for a generic property (not a color or distance)
// { value: original, static: val, zoom: { 1: val1, 2: val2, ... }, dynamic: function(){...} }


StyleParser.evalCachedProperty = function (val, context) {
  if (val == null) {
    return;
  } else if (val.dynamic) {
    // function, compute each time (no caching)
    return tryEval(val.dynamic, context);
  } else if (val.static) {
    // single static value
    return val.static;
  } else if (val.zoom && val.zoom[context.zoom]) {
    // interpolated, cached
    return val.zoom[context.zoom];
  } else {
    // not yet evaulated for cache
    // Dynamic function-based
    if (typeof val.value === 'function') {
      val.dynamic = val.value;
      return tryEval(val.dynamic, context);
    } // Array of zoom-interpolated stops, e.g. [zoom, value] pairs
    else if (Array.isArray(val.value) && Array.isArray(val.value[0])) {
        // Calculate value for current zoom
        val.zoom = val.zoom || {};
        val.zoom[context.zoom] = Utils.interpolate(context.zoom, val.value);
        return val.zoom[context.zoom];
      } // Single static value
      else {
          val.static = val.value;
          return val.static;
        }
  }
};

StyleParser.convertUnits = function (val, context) {
  // pre-parsed units
  if (val.val != null) {
    if (val.units === 'px') {
      // convert from pixels
      return val.val * Geo$1.metersPerPixel(context.zoom);
    }

    return val.val;
  } // un-parsed unit string
  else if (typeof val === 'string') {
      if (val.trim().slice(-2) === 'px') {
        val = parseNumber(val);
        val *= Geo$1.metersPerPixel(context.zoom); // convert from pixels
      } else {
        val = parseNumber(val);
      }
    } // multiple values or stops
    else if (Array.isArray(val)) {
        // Array of arrays, e.g. zoom-interpolated stops
        if (Array.isArray(val[0])) {
          return val.map(function (v) {
            return [v[0], StyleParser.convertUnits(v[1], context)];
          });
        } // Array of values
        else {
            return val.map(function (v) {
              return StyleParser.convertUnits(v, context);
            });
          }
      }

  return val;
}; // Pre-parse units from string values


StyleParser.parseUnits = function (val) {
  var obj = {
    val: parseNumber(val)
  };

  if (obj.val !== 0 && typeof val === 'string' && val.trim().slice(-2) === 'px') {
    obj.units = 'px';
  }

  return obj;
}; // Takes a distance cache object and returns a distance value for this zoom
// (caching the result for future use)
// { value: original, zoom: { z: meters }, dynamic: function(){...} }


StyleParser.evalCachedDistanceProperty = function (val, context) {
  if (val == null) {
    return;
  } else if (val.dynamic) {
    return tryEval(val.dynamic, context);
  } else if (val.zoom && val.zoom[context.zoom]) {
    return val.zoom[context.zoom];
  } else {
    // Dynamic function-based
    if (typeof val.value === 'function') {
      val.dynamic = val.value;
      return tryEval(val.dynamic, context);
    } // Array of zoom-interpolated stops, e.g. [zoom, value] pairs
    else if (val.zoom) {
        // Calculate value for current zoom
        // Do final unit conversion as late as possible, when interpolation values have been determined
        val.zoom[context.zoom] = Utils.interpolate(context.zoom, val.value, function (v) {
          return StyleParser.convertUnits(v, context);
        });
        return val.zoom[context.zoom];
      } else {
        return StyleParser.convertUnits(val.value, context);
      }
  }
}; // Cache previously parsed color strings


StyleParser.string_colors = {};

StyleParser.colorForString = function (string) {
  // Cached
  if (StyleParser.string_colors[string]) {
    return StyleParser.string_colors[string];
  } // Calculate and cache


  var color = csscolorparser.parseCSSColor(string);

  if (color && color.length === 4) {
    color[0] /= 255;
    color[1] /= 255;
    color[2] /= 255;
  } else {
    color = StyleParser.defaults.color;
  }

  StyleParser.string_colors[string] = color;
  return color;
}; // Takes a color cache object and returns a color value for this zoom
// (caching the result for future use)
// { value: original, static: [r,g,b,a], zoom: { z: [r,g,b,a] }, dynamic: function(){...} }


StyleParser.evalCachedColorProperty = function (val, context) {
  if (context === void 0) {
    context = {};
  }

  if (val.dynamic) {
    var v = tryEval(val.dynamic, context);

    if (typeof v === 'string') {
      v = StyleParser.colorForString(v);
    }

    if (v && v[3] == null) {
      v[3] = 1; // default alpha
    }

    return v;
  } else if (val.static) {
    return val.static;
  } else if (val.zoom && val.zoom[context.zoom]) {
    return val.zoom[context.zoom];
  } else {
    // Dynamic function-based color
    if (typeof val.value === 'function') {
      val.dynamic = val.value;

      var _v = tryEval(val.dynamic, context);

      if (typeof _v === 'string') {
        _v = StyleParser.colorForString(_v);
      }

      if (_v && _v[3] == null) {
        _v[3] = 1; // default alpha
      }

      return _v;
    } // Single string color
    else if (typeof val.value === 'string') {
        val.static = StyleParser.colorForString(val.value);
        return val.static;
      } // Array of zoom-interpolated stops, e.g. [zoom, color] pairs
      else if (val.zoom) {
          // Parse any string colors inside stops, the first time we encounter this property
          if (!val.zoom_preprocessed) {
            for (var i = 0; i < val.value.length; i++) {
              var _v2 = val.value[i];

              if (_v2 && typeof _v2[1] === 'string') {
                _v2[1] = StyleParser.colorForString(_v2[1]);
              }
            }

            val.zoom_preprocessed = true;
          } // Calculate color for current zoom


          val.zoom[context.zoom] = Utils.interpolate(context.zoom, val.value);
          val.zoom[context.zoom][3] = val.zoom[context.zoom][3] || 1; // default alpha

          return val.zoom[context.zoom];
        } // Single array color
        else {
            val.static = val.value.map(function (x) {
              return x;
            }); // copy to avoid modifying

            if (val.static && val.static[3] == null) {
              val.static[3] = 1; // default alpha
            }

            return val.static;
          }
  }
};

StyleParser.parseColor = function (val, context) {
  if (context === void 0) {
    context = {};
  }

  if (typeof val === 'function') {
    val = tryEval(val, context);
  } // Parse CSS-style colors
  // TODO: change all colors to use 0-255 range internally to avoid dividing and then re-multiplying in geom builder


  if (typeof val === 'string') {
    val = StyleParser.colorForString(val);
  } else if (Array.isArray(val) && Array.isArray(val[0])) {
    // Array of zoom-interpolated stops, e.g. [zoom, color] pairs
    for (var i = 0; i < val.length; i++) {
      var v = val[i];

      if (typeof v[1] === 'string') {
        v[1] = StyleParser.colorForString(v[1]);
      }
    }

    if (context.zoom) {
      val = Utils.interpolate(context.zoom, val);
    }
  } // Defaults


  if (Array.isArray(val)) {
    val = val.map(function (x) {
      return x;
    }); // copy to avoid modifying
    // alpha

    if (val[3] == null) {
      val[3] = 1;
    }
  } else {
    val = [0, 0, 0, 1];
  }

  return val;
};

StyleParser.calculateOrder = function (order, context) {
  // Computed order
  if (typeof order === 'function') {
    order = tryEval(order, context);
  } else if (typeof order === 'string') {
    // Order tied to feature property
    if (context.feature.properties[order]) {
      order = context.feature.properties[order];
    } // Explicit order value
    else {
        order = parsePositiveNumber(order);
      }
  }

  return order;
}; // Evaluate a function-based property, or pass-through static value


StyleParser.evalProperty = function (prop, context) {
  if (typeof prop === 'function') {
    return tryEval(prop, context);
  }

  return prop;
}; // eval property function with try/catch


function tryEval(func, context) {
  try {
    return func(context);
  } catch (e) {
    log('warn', "Property function in layer '" + context.layers[context.layers.length - 1] + "' failed with\n", "error " + e.stack + "\n", "function '" + func.source + "'\n", context.feature, context);
  }
}

// 22.1.3.6 Array.prototype.fill(value, start = 0, end = this.length)


_export(_export.P, 'Array', { fill: _arrayFill });

_addToUnscopables('fill');

var FeatureSelection =
/*#__PURE__*/
function () {
  function FeatureSelection(gl, workers, lock_fn) {
    this.gl = gl;
    this.workers = workers; // pool of workers to request feature look-ups from, keyed by id

    this._lock_fn = typeof lock_fn === 'function' && lock_fn; // indicates if safe to read/write selection buffer this frame

    this.init();
  }

  var _proto = FeatureSelection.prototype;

  _proto.init = function init() {
    // Selection state tracking
    this.requests = {}; // pending selection requests

    this.feature = null; // currently selected feature

    this.read_delay = 0; // delay time from selection render to framebuffer sample, to avoid CPU/GPU sync lock

    this.read_delay_timer = null; // current timer (setTimeout) for delayed selection reads

    this.pixels = null; // allocated lazily on request
    // Frame buffer for selection
    // TODO: initiate lazily in case we don't need to do any selection

    this.fbo = this.gl.createFramebuffer();
    this.gl.bindFramebuffer(this.gl.FRAMEBUFFER, this.fbo);
    this.fbo_size = {
      width: 256,
      height: 256
    }; // TODO: make configurable / adaptive based on canvas size
    // Texture for the FBO color attachment

    var fbo_texture = Texture.create(this.gl, '__selection_fbo', {
      filtering: 'nearest'
    });
    fbo_texture.setData(this.fbo_size.width, this.fbo_size.height, null, {
      filtering: 'nearest'
    });
    this.gl.framebufferTexture2D(this.gl.FRAMEBUFFER, this.gl.COLOR_ATTACHMENT0, this.gl.TEXTURE_2D, fbo_texture.texture, 0); // Renderbuffer for the FBO depth attachment

    var fbo_depth_rb = this.gl.createRenderbuffer();
    this.gl.bindRenderbuffer(this.gl.RENDERBUFFER, fbo_depth_rb);
    this.gl.renderbufferStorage(this.gl.RENDERBUFFER, this.gl.DEPTH_COMPONENT16, this.fbo_size.width, this.fbo_size.height);
    this.gl.framebufferRenderbuffer(this.gl.FRAMEBUFFER, this.gl.DEPTH_ATTACHMENT, this.gl.RENDERBUFFER, fbo_depth_rb);
    this.gl.bindFramebuffer(this.gl.FRAMEBUFFER, null);
  };

  _proto.destroy = function destroy() {
    if (this.gl && this.fbo) {
      this.gl.deleteFramebuffer(this.fbo);
      this.fbo = null;
      this.gl.bindFramebuffer(this.gl.FRAMEBUFFER, null);
    } // TODO: free texture?

  } // external lock function determines when it's safe to read/write from selection buffer
  ;

  _proto.bind = function bind() {
    var _this$gl;

    // Switch to FBO
    this.gl.bindFramebuffer(this.gl.FRAMEBUFFER, this.fbo);
    this.gl.viewport(0, 0, this.fbo_size.width, this.fbo_size.height);

    (_this$gl = this.gl).clearColor.apply(_this$gl, FeatureSelection.defaultColor);
  } // Request feature selection
  // Runs asynchronously, schedules selection buffer to be updated
  ;

  _proto.getFeatureAt = function getFeatureAt(point, _ref) {
    var _this = this;

    var radius = _ref.radius;

    // ensure requested point is in canvas bounds
    if (!point || point.x < 0 || point.y < 0 || point.x > 1 || point.y > 1) {
      return Promise.resolve({
        feature: null,
        changed: false
      });
    }

    return new Promise(function (resolve, reject) {
      // Queue requests for feature selection, and they will be picked up by the render loop
      _this.selection_request_id = _this.selection_request_id + 1 || 0;
      _this.requests[_this.selection_request_id] = {
        id: _this.selection_request_id,
        point: point,
        radius: radius,
        resolve: resolve,
        reject: reject
      };
    });
  } // Any pending selection requests
  ;

  _proto.pendingRequests = function pendingRequests() {
    return Object.keys(this.requests).length ? this.requests : null;
  };

  _proto.hasPendingRequests = function hasPendingRequests() {
    return this.pendingRequests() != null;
  };

  _proto.clearPendingRequests = function clearPendingRequests() {
    for (var r in this.requests) {
      var request = this.requests[r]; // This request was already sent to the worker, we're just awaiting its reply

      if (request.sent) {
        continue;
      } // Reject request since it will never be fulfilled
      // TODO: pass a reason for rejection?


      request.reject({
        request: request
      });
      delete this.requests[r];
    }
  } // Read pending results from the selection buffer. Called after rendering to selection buffer.
  ;

  _proto.read = function read() {
    var _this2 = this;

    // Delay reading the pixel result from the selection buffer to avoid CPU/GPU sync lock.
    // Calling readPixels synchronously caused a massive performance hit, presumably since it
    // forced this function to wait for the GPU to finish rendering and retrieve the texture contents.
    if (this.read_delay_timer != null) {
      clearTimeout(this.read_delay_timer);
    }

    this.read_delay_timer = setTimeout(function () {
      if (_this2.locked) {
        return;
      }

      var gl = _this2.gl;
      gl.bindFramebuffer(gl.FRAMEBUFFER, _this2.fbo);

      for (var r in _this2.requests) {
        var request = _this2.requests[r]; // This request was already sent to the worker, we're just awaiting its reply

        if (request.sent) {
          continue;
        } // Check selection map against FBO


        var feature_key = void 0,
            worker_id = 255;
        var point = request.point,
            radius = request.radius;
        var diam_px = void 0;

        if (!radius) {
          radius = {
            x: 0,
            y: 0
          };
          diam_px = {
            x: 1,
            y: 1
          };
        } else {
          // diameter in selection buffer pixels
          var max_radius = Math.min(_this2.fbo_size.width, _this2.fbo_size.height);
          diam_px = {
            x: Math.min(Math.ceil(radius.x * 2 * _this2.fbo_size.width), max_radius),
            y: Math.min(Math.ceil(radius.y * 2 * _this2.fbo_size.height), max_radius)
          };
        } // allocate or resize


        if (_this2.pixels == null || _this2.pixels.byteLength < diam_px.x * diam_px.y * 4) {
          _this2.pixels = new Uint8Array(diam_px.x * diam_px.y * 4);
        } // clear pixels


        if (_this2.pixels.fill instanceof Function) {
          _this2.pixels.fill(0); // native typed array fill

        } else {
          for (var _p = 0; _p < _this2.pixels.length; _p++) {
            _this2.pixels[_p] = 0;
          }
        } // capture pixels


        gl.readPixels(Math.round((point.x - radius.x) * _this2.fbo_size.width), Math.round((1 - point.y - radius.y) * _this2.fbo_size.height), diam_px.x, diam_px.y, gl.RGBA, gl.UNSIGNED_BYTE, _this2.pixels); // first check center pixel (avoid scanning all pixels if cursor is directly on a feature)

        var p = (Math.round(diam_px.y / 2) * diam_px.x + Math.round(diam_px.x / 2)) * 4;
        var v = _this2.pixels[p] + (_this2.pixels[p + 1] << 8) + (_this2.pixels[p + 2] << 16); // feature id in RGB channels

        if (v > 0) {
          feature_key = v + (_this2.pixels[p + 3] << 24) >>> 0; // worker id in alpha channel

          worker_id = _this2.pixels[p + 3];
        } else {
          // scan all pixels for feature closest to cursor
          var min_dist = -1 >>> 0;
          p = 0;

          for (var y = 0; y < diam_px.y; y++) {
            for (var x = 0; x < diam_px.x; x++, p += 4) {
              v = _this2.pixels[p] + (_this2.pixels[p + 1] << 8) + (_this2.pixels[p + 2] << 16); // feature id in RGB channels

              if (v > 0) {
                // non-zero value indicates a feature
                // check to see if closer than last found feature
                var dist = (x - diam_px.x / 2) * (x - diam_px.x / 2) + (y - diam_px.y / 2) * (y - diam_px.y / 2);

                if (dist <= min_dist) {
                  // get worker id from alpha channel
                  feature_key = v + (_this2.pixels[p + 3] << 24) >>> 0;
                  worker_id = _this2.pixels[p + 3];
                  min_dist = dist;
                }
              }
            }
          }
        } // If feature found, ask appropriate web worker to lookup feature


        if (worker_id !== 255) {
          // 255 indicates an empty selection buffer pixel
          if (_this2.workers[worker_id] != null) {
            WorkerBroker$1.postMessage(_this2.workers[worker_id], 'self.getFeatureSelection', {
              id: request.id,
              key: feature_key
            }).then(function (message) {
              _this2.finishRead(message);
            });
          }
        } // No feature found, but still need to resolve promise
        else {
            _this2.finishRead({
              id: request.id
            });
          }

        request.sent = true;
      }

      gl.bindFramebuffer(gl.FRAMEBUFFER, null);
    }, this.read_delay);
  } // Called on main thread when a web worker finds a feature in the selection buffer
  ;

  _proto.finishRead = function finishRead(message) {
    var request = this.requests[message.id];

    if (!request) {
      log('error', 'FeatureSelection.finishRead(): could not find message', message);
      return; // request was cleared before it returned
    }

    var feature = message.feature;
    var changed = false;

    if (feature != null && this.feature == null || feature == null && this.feature != null || feature != null && this.feature != null && JSON.stringify(feature) !== JSON.stringify(this.feature)) {
      changed = true;
    }

    this.feature = feature; // store the most recently selected feature
    // Resolve the request

    request.resolve({
      feature: feature,
      changed: changed,
      request: request
    });
    delete this.requests[message.id]; // done processing this request
  } // Selection map generation
  // Each worker will create its own independent, 'local' selection map
  // Create a unique 32-bit color to identify a feature
  // Workers independently create/modify selection colors in their own threads, but we also
  // need the main thread to know where each feature color originated. To accomplish this,
  // we partition the map by setting the 4th component (alpha channel) to the worker's id.
  ;

  FeatureSelection.makeEntry = function makeEntry(tile) {
    // 32-bit color key
    this.map_entry++;
    var ir = this.map_entry & 255;
    var ig = this.map_entry >> 8 & 255;
    var ib = this.map_entry >> 16 & 255;
    var ia = this.map_prefix;
    var r = ir / 255;
    var g = ig / 255;
    var b = ib / 255;
    var a = ia / 255;
    var key = ir + (ig << 8) + (ib << 16) + (ia << 24) >>> 0; // need unsigned right shift to convert to positive #

    this.map[key] = {
      color: [r, g, b, a]
    };
    this.map_size++; // Initialize tile-specific tracking info

    if (!this.tiles[tile.key]) {
      this.tiles[tile.key] = {
        entries: [],
        // set of feature entries in this thread
        tile: {
          // subset of tile properties to pass back with feature
          key: tile.key,
          coords: tile.coords,
          style_z: tile.style_z,
          source: tile.source,
          generation: tile.generation
        }
      };
    }

    this.tiles[tile.key].entries.push(key);
    return this.map[key];
  };

  FeatureSelection.makeColor = function makeColor(feature, tile, context) {
    var selector = this.makeEntry(tile);
    selector.feature = {
      id: feature.id,
      properties: feature.properties,
      source_name: context.source,
      source_layer: context.layer,
      layers: context.layers,
      tile: this.tiles[tile.key].tile
    };
    return selector.color;
  };

  FeatureSelection.reset = function reset(sources) {
    var _this3 = this;

    // Clear specific sources
    if (Array.isArray(sources)) {
      sources.forEach(function (source) {
        return _this3.clearSource(source);
      });
    } // Clear all sources
    else {
        this.tiles = {};
        this.map = {};
        this.map_size = 0;
        this.map_entry = 0;
      }
  };

  FeatureSelection.clearSource = function clearSource(source) {
    for (var key in this.tiles) {
      if (this.tiles[key].tile.source === source) {
        this.clearTile(key);
      }
    }
  };

  FeatureSelection.clearTile = function clearTile(key) {
    var _this4 = this;

    if (this.tiles[key]) {
      this.tiles[key].entries.forEach(function (k) {
        return delete _this4.map[k];
      });
      this.map_size -= this.tiles[key].entries.length;
      delete this.tiles[key];
    }
  };

  FeatureSelection.getMapSize = function getMapSize() {
    return this.map_size;
  };

  FeatureSelection.setPrefix = function setPrefix(prefix) {
    this.map_prefix = prefix;
  };

  _createClass(FeatureSelection, [{
    key: "locked",
    get: function get() {
      return this._lock_fn && this._lock_fn() || false;
    }
  }]);

  return FeatureSelection;
}(); // Static properties
FeatureSelection.map = {}; // this will be unique per module instance (so unique per worker)

FeatureSelection.tiles = {}; // selection keys, by tile

FeatureSelection.map_size = 0;
FeatureSelection.map_entry = 0;
FeatureSelection.map_prefix = 0; // set by worker to worker id #

FeatureSelection.defaultColor = [0, 0, 0, 1];

_typedArray('Uint16', 2, function (init) {
  return function Uint16Array(data, byteOffset, length) {
    return init(this, data, byteOffset, length);
  };
});

var VBOMesh =
/*#__PURE__*/
function () {
  function VBOMesh(gl, vertex_data, element_data, vertex_layout, options) {
    options = options || {};
    this.gl = gl;
    this.vertex_data = vertex_data; // typed array

    this.element_data = element_data; // typed array

    this.vertex_layout = vertex_layout;
    this.vertex_buffer = this.gl.createBuffer();
    this.buffer_size = this.vertex_data.byteLength;
    this.draw_mode = options.draw_mode || this.gl.TRIANGLES;
    this.data_usage = options.data_usage || this.gl.STATIC_DRAW;
    this.vertices_per_geometry = 3; // TODO: support lines, strip, fan, etc.

    this.uniforms = options.uniforms;
    this.textures = options.textures; // any textures owned by this mesh

    this.retain = options.retain || false; // whether to retain mesh data in CPU after uploading to GPU

    this.created_at = +new Date();
    this.fade_in_time = options.fade_in_time || 0; // optional time to fade in mesh

    this.vertex_count = this.vertex_data.byteLength / this.vertex_layout.stride;
    this.element_count = 0;
    this.vaos = {}; // map of VertexArrayObjects, keyed by program

    this.toggle_element_array = false;

    if (this.element_data) {
      this.toggle_element_array = true;
      this.element_count = this.element_data.length;
      this.geometry_count = this.element_count / this.vertices_per_geometry;
      this.element_type = this.element_data.constructor === Uint16Array ? this.gl.UNSIGNED_SHORT : this.gl.UNSIGNED_INT;
      this.element_buffer = this.gl.createBuffer();
      this.buffer_size += this.element_data.byteLength;
      this.gl.bindBuffer(this.gl.ELEMENT_ARRAY_BUFFER, this.element_buffer);
      this.gl.bufferData(this.gl.ELEMENT_ARRAY_BUFFER, this.element_data, this.data_usage);
    } else {
      this.geometry_count = this.vertex_count / this.vertices_per_geometry;
    }

    this.upload();

    if (!this.retain) {
      delete this.vertex_data;
      delete this.element_data;
    }

    this.valid = true;
  } // Render, by default with currently bound program, or otherwise with optionally provided one
  // Returns true if mesh requests a render on next frame (e.g. for fade animations)


  var _proto = VBOMesh.prototype;

  _proto.render = function render(options) {
    if (options === void 0) {
      options = {};
    }

    if (!this.valid) {
      return false;
    }

    var program = options.program || ShaderProgram.current;
    program.use();

    if (this.uniforms) {
      program.saveUniforms(this.uniforms);
      program.setUniforms(this.uniforms, false); // don't reset texture unit
    }

    var visible_time = (+new Date() - this.created_at) / 1000;
    program.uniform('1f', 'u_visible_time', visible_time);
    this.bind(program);

    if (this.toggle_element_array) {
      this.gl.drawElements(this.draw_mode, this.element_count, this.element_type, 0);
    } else {
      this.gl.drawArrays(this.draw_mode, 0, this.vertex_count);
    }

    VertexArrayObject.bind(this.gl, null);

    if (this.uniforms) {
      program.restoreUniforms(this.uniforms);
    } // Request next render if mesh is fading in


    return visible_time < this.fade_in_time;
  } // Bind buffers and vertex attributes to prepare for rendering
  ;

  _proto.bind = function bind(program) {
    var _this = this;

    // Bind VAO for this progam, or create one
    var vao = this.vaos[program.id];

    if (vao) {
      VertexArrayObject.bind(this.gl, vao);
    } else {
      this.vaos[program.id] = VertexArrayObject.create(this.gl, function () {
        _this.gl.bindBuffer(_this.gl.ARRAY_BUFFER, _this.vertex_buffer);

        if (_this.toggle_element_array) {
          _this.gl.bindBuffer(_this.gl.ELEMENT_ARRAY_BUFFER, _this.element_buffer);
        }

        _this.vertex_layout.enableDynamicAttributes(_this.gl, program);
      });
    }

    this.vertex_layout.enableStaticAttributes(this.gl, program);
  } // Upload buffer data to GPU
  ;

  _proto.upload = function upload() {
    this.gl.bindBuffer(this.gl.ARRAY_BUFFER, this.vertex_buffer);
    this.gl.bufferData(this.gl.ARRAY_BUFFER, this.vertex_data, this.data_usage);
  };

  _proto.destroy = function destroy() {
    if (!this.valid) {
      return false;
    }

    this.valid = false;

    for (var v in this.vaos) {
      VertexArrayObject.destroy(this.gl, this.vaos[v]);
    }

    this.gl.deleteBuffer(this.vertex_buffer);
    this.vertex_buffer = null;

    if (this.element_buffer) {
      this.gl.deleteBuffer(this.element_buffer);
      this.element_buffer = null;
    }

    delete this.vertex_data;
    delete this.element_data;

    if (this.textures) {
      this.textures.forEach(function (t) {
        return Texture.release(t);
      });
    }

    return true;
  };

  return VBOMesh;
}();

var material_source = "/*\n\nDefines globals:\nmaterial\nlight_accumulator_*\n\n*/\n\n\n// MATERIALS\n//\nstruct Material {\n    #ifdef TANGRAM_MATERIAL_EMISSION\n        vec4 emission;\n        #ifdef TANGRAM_MATERIAL_EMISSION_TEXTURE\n            vec3 emissionScale;\n        #endif\n    #endif\n\n    #ifdef TANGRAM_MATERIAL_AMBIENT\n        vec4 ambient;\n        #ifdef TANGRAM_MATERIAL_AMBIENT_TEXTURE\n            vec3 ambientScale;\n        #endif\n    #endif\n\n    #ifdef TANGRAM_MATERIAL_DIFFUSE\n        vec4 diffuse;\n        #ifdef TANGRAM_MATERIAL_DIFFUSE_TEXTURE\n            vec3 diffuseScale;\n        #endif\n    #endif\n\n    #ifdef TANGRAM_MATERIAL_SPECULAR\n        vec4 specular;\n        float shininess;\n        #ifdef TANGRAM_MATERIAL_SPECULAR_TEXTURE\n            vec3 specularScale;\n        #endif\n    #endif\n\n\n    #ifdef TANGRAM_MATERIAL_NORMAL_TEXTURE\n        vec3 normalScale;\n        float normalAmount;\n    #endif\n};\n\n// Note: uniform is copied to a global instance to allow modification\nuniform Material u_material;\nMaterial material;\n\n#ifdef TANGRAM_MATERIAL_EMISSION_TEXTURE\nuniform sampler2D u_material_emission_texture;\n#endif\n\n#ifdef TANGRAM_MATERIAL_AMBIENT_TEXTURE\nuniform sampler2D u_material_ambient_texture;\n#endif\n\n#ifdef TANGRAM_MATERIAL_DIFFUSE_TEXTURE\nuniform sampler2D u_material_diffuse_texture;\n#endif\n\n#ifdef TANGRAM_MATERIAL_SPECULAR_TEXTURE\nuniform sampler2D u_material_specular_texture;\n#endif\n\n#ifdef TANGRAM_MATERIAL_NORMAL_TEXTURE\nuniform sampler2D u_material_normal_texture;\n#endif\n\n// Global light accumulators for each property\nvec4 light_accumulator_ambient = vec4(vec3(0.0), 1.);\nvec4 light_accumulator_diffuse = vec4(vec3(0.0), 1.);\n#ifdef TANGRAM_MATERIAL_SPECULAR\n    vec4 light_accumulator_specular = vec4(vec3(0.0), 1.);\n#endif\n\n\n#ifdef TANGRAM_MATERIAL_TEXTURE_SPHEREMAP\nvec4 getSphereMap (in sampler2D _tex, in vec3 _eyeToPoint, in vec3 _normal, in vec2 _skew) {\n    // Normalized vector from camera to surface\n    vec3 eye = normalize(_eyeToPoint);\n\n    // Adjust for camera skew\n    eye.xy -= _skew;\n    eye = normalize(eye);\n\n    // Reflection of eye off of surface normal\n    vec3 r = reflect(eye, _normal);\n\n    // Map reflected vector onto the surface of a sphere\n    r.z += 1.0;\n    float m = 2. * length(r);\n\n    // Adjust xy to account for spherical shape, and center in middle of texture\n    vec2 uv = r.xy / m + .5;\n\n    // Sample the environment map\n    return texture2D(_tex, uv);\n}\n#endif\n\n\n#ifdef TANGRAM_MATERIAL_TEXTURE_TRIPLANAR\nvec3 getTriPlanarBlend (in vec3 _normal) {\n    vec3 blending = abs(_normal);\n    blending = normalize(max(blending, 0.00001));\n    float b = (blending.x + blending.y + blending.z);\n    return blending / b;\n}\n\nvec4 getTriPlanar (in sampler2D _tex, in vec3 _pos, in vec3 _normal, in vec3 _scale) {\n    vec3 blending = getTriPlanarBlend(_normal);\n    vec4 xaxis = texture2D(_tex, fract(_pos.yz * _scale.x));\n    vec4 yaxis = texture2D(_tex, fract(_pos.xz * _scale.y));\n    vec4 zaxis = texture2D(_tex, fract(_pos.xy * _scale.z));\n    return  xaxis * blending.x + yaxis * blending.y + zaxis * blending.z;\n}\n#endif\n\n\n#ifdef TANGRAM_MATERIAL_TEXTURE_PLANAR\nvec4 getPlanar (in sampler2D _tex, in vec3 _pos, in vec2 _scale) {\n    return texture2D( _tex, fract(_pos.xy * _scale.x) );\n}\n#endif\n\n\n#ifdef TANGRAM_MATERIAL_NORMAL_TEXTURE\nvoid calculateNormal (inout vec3 _normal) {\n    // Get NORMALMAP\n    //------------------------------------------------\n    #ifdef TANGRAM_MATERIAL_NORMAL_TEXTURE_UV\n    _normal += texture2D(u_material_normal_texture, fract(v_texcoord*material.normalScale.xy)).rgb*2.0-1.0;\n    #endif\n\n    #ifdef TANGRAM_MATERIAL_NORMAL_TEXTURE_PLANAR\n    vec3 normalTex = getPlanar(u_material_normal_texture, v_world_position.xyz, material.normalScale.xy).rgb*2.0-1.0;\n    _normal += normalTex;\n    #endif\n\n    #ifdef TANGRAM_MATERIAL_NORMAL_TEXTURE_TRIPLANAR\n    vec3 normalTex = getTriPlanar(u_material_normal_texture, v_world_position.xyz, _normal, material.normalScale).rgb*2.0-1.0;\n    _normal += normalTex;\n    #endif\n\n    _normal = normalize(_normal);\n}\n#endif\n\nvoid calculateMaterial (in vec3 _eyeToPoint, inout vec3 _normal) {\n    // get EMISSION TEXTUREMAP\n    //------------------------------------------------\n    #ifdef TANGRAM_MATERIAL_EMISSION_TEXTURE\n        #ifdef TANGRAM_MATERIAL_EMISSION_TEXTURE_UV\n        material.emission *= texture2D(u_material_emission_texture,v_texcoord);\n        #endif\n\n        #ifdef TANGRAM_MATERIAL_EMISSION_TEXTURE_PLANAR\n        material.emission *= getPlanar(u_material_emission_texture, v_world_position.xyz, material.emissionScale.xy);\n        #endif\n\n        #ifdef TANGRAM_MATERIAL_EMISSION_TEXTURE_TRIPLANAR\n        material.emission *= getTriPlanar(u_material_emission_texture, v_world_position.xyz, _normal, material.emissionScale);\n        #endif\n\n        #ifdef TANGRAM_MATERIAL_EMISSION_TEXTURE_SPHEREMAP\n        material.emission *= getSphereMap(u_material_emission_texture, _eyeToPoint, _normal, u_vanishing_point);\n        #endif\n    #endif\n\n    // get AMBIENT TEXTUREMAP\n    //------------------------------------------------\n    #ifdef TANGRAM_MATERIAL_AMBIENT_TEXTURE\n        #ifdef TANGRAM_MATERIAL_AMBIENT_TEXTURE_UV\n        material.ambient *= texture2D(u_material_ambient_texture,v_texcoord);\n        #endif\n\n        #ifdef TANGRAM_MATERIAL_AMBIENT_TEXTURE_PLANAR\n        material.ambient *= getPlanar(u_material_ambient_texture, v_world_position.xyz, material.ambientScale.xy);\n        #endif\n\n        #ifdef TANGRAM_MATERIAL_AMBIENT_TEXTURE_TRIPLANAR\n        material.ambient *= getTriPlanar(u_material_ambient_texture, v_world_position.xyz, _normal, material.ambientScale);\n        #endif\n\n        #ifdef TANGRAM_MATERIAL_AMBIENT_TEXTURE_SPHEREMAP\n        material.ambient *= getSphereMap(u_material_ambient_texture, _eyeToPoint, _normal, u_vanishing_point);\n        #endif\n    #endif\n\n    // get DIFFUSE TEXTUREMAP\n    //------------------------------------------------\n    #ifdef TANGRAM_MATERIAL_DIFFUSE_TEXTURE\n        #ifdef TANGRAM_MATERIAL_DIFFUSE_TEXTURE_UV\n        material.diffuse *= texture2D(u_material_diffuse_texture,v_texcoord);\n        #endif\n\n        #ifdef TANGRAM_MATERIAL_DIFFUSE_TEXTURE_PLANAR\n        material.diffuse *= getPlanar(u_material_diffuse_texture, v_world_position.xyz, material.diffuseScale.xy);\n        #endif\n\n        #ifdef TANGRAM_MATERIAL_DIFFUSE_TEXTURE_TRIPLANAR\n        material.diffuse *= getTriPlanar(u_material_diffuse_texture, v_world_position.xyz, _normal, material.diffuseScale);\n        #endif\n\n        #ifdef TANGRAM_MATERIAL_DIFFUSE_TEXTURE_SPHEREMAP\n        material.diffuse *= getSphereMap(u_material_diffuse_texture, _eyeToPoint, _normal, u_vanishing_point);\n        #endif\n    #endif\n\n    // get SPECULAR TEXTUREMAP\n    //------------------------------------------------\n    #ifdef TANGRAM_MATERIAL_SPECULAR_TEXTURE\n        #ifdef TANGRAM_MATERIAL_SPECULAR_TEXTURE_UV\n        material.specular *= texture2D(u_material_specular_texture,v_texcoord);\n        #endif\n\n        #ifdef TANGRAM_MATERIAL_SPECULAR_TEXTURE_PLANAR\n        material.specular *= getPlanar(u_material_specular_texture, v_world_position.xyz, material.specularScale.xy);\n        #endif\n\n        #ifdef TANGRAM_MATERIAL_SPECULAR_TEXTURE_TRIPLANAR\n        material.specular *= getTriPlanar(u_material_specular_texture, v_world_position.xyz, _normal, material.specularScale);\n        #endif\n\n        #ifdef TANGRAM_MATERIAL_SPECULAR_TEXTURE_SPHEREMAP\n        material.specular *= getSphereMap(u_material_specular_texture, _eyeToPoint, _normal, u_vanishing_point);\n        #endif\n    #endif\n}\n";

var material_props = ['emission', 'ambient', 'diffuse', 'specular'];

var Material =
/*#__PURE__*/
function () {
  function Material(config) {
    var _this = this;

    config = config || {}; // These properties all have the same defaults, so they can be set in bulk

    material_props.forEach(function (prop) {
      var value = config[prop];

      if (value != null) {
        if (value.texture) {
          _this[prop] = {
            texture: value.texture,
            mapping: value.mapping || 'spheremap',
            scale: GLSL.expandVec3(value.scale != null ? value.scale : 1),
            amount: GLSL.expandVec4(value.amount != null ? value.amount : 1)
          };
        } else if (typeof value === 'number' || Array.isArray(value)) {
          _this[prop] = {
            amount: GLSL.expandVec4(value)
          };
        } else if (typeof value === 'string') {
          _this[prop] = {
            amount: StyleParser.parseColor(value)
          };
        } else {
          _this[prop] = value;
        }
      }
    }); // Extra specular props

    if (this.specular) {
      this.specular.shininess = config.shininess ? parseFloat(config.shininess) : 0.2;
    } // Normal mapping


    if (config.normal != null) {
      this.normal = {
        texture: config.normal.texture,
        mapping: config.normal.mapping || 'triplanar',
        scale: GLSL.expandVec3(config.normal.scale != null ? config.normal.scale : 1),
        amount: config.normal.amount != null ? config.normal.amount : 1
      };
    }
  } // Determine if a material config block has sufficient properties to create a material


  Material.isValid = function isValid(config) {
    if (config == null) {
      return false;
    }

    if (config.emission == null && config.ambient == null && config.diffuse == null && config.specular == null) {
      return false;
    }

    return true;
  };

  var _proto = Material.prototype;

  _proto.inject = function inject(style) {
    var _this2 = this;

    // For each property, sets defines to configure texture mapping, with a pattern like:
    //   TANGRAM_MATERIAL_DIFFUSE, TANGRAM_MATERIAL_DIFFUSE_TEXTURE, TANGRAM_MATERIAL_DIFFUSE_TEXTURE_SPHEREMAP
    // Also sets flags to keep track of each unique mapping type being used, e.g.:
    //   TANGRAM_MATERIAL_TEXTURE_SPHEREMAP
    // Enables texture coordinates if needed and not already on
    material_props.forEach(function (prop) {
      var def = "TANGRAM_MATERIAL_" + prop.toUpperCase();
      var texdef = def + '_TEXTURE';
      style.defines[def] = _this2[prop] != null;

      if (_this2[prop] && _this2[prop].texture) {
        style.defines[texdef] = true;
        style.defines[texdef + '_' + _this2[prop].mapping.toUpperCase()] = true;
        style.defines["TANGRAM_MATERIAL_TEXTURE_" + _this2[prop].mapping.toUpperCase()] = true;
        style.texcoords = style.texcoords || _this2[prop].mapping === 'uv';
      }
    }); // Normal mapping
    // As anove, sets flags to keep track of each unique mapping type being used, e.g.:
    //   TANGRAM_MATERIAL_TEXTURE_SPHEREMAP

    if (this.normal && this.normal.texture) {
      style.defines['TANGRAM_MATERIAL_NORMAL_TEXTURE'] = true;
      style.defines['TANGRAM_MATERIAL_NORMAL_TEXTURE_' + this.normal.mapping.toUpperCase()] = true;
      style.defines["TANGRAM_MATERIAL_TEXTURE_" + this.normal.mapping.toUpperCase()] = true;
      style.texcoords = style.texcoords || this.normal.mapping === 'uv';
    }

    style.replaceShaderBlock(Material.block, material_source, 'Material');
    style.addShaderBlock('setup', '\nmaterial = u_material;\n', 'Material');
  };

  _proto.setupProgram = function setupProgram(_program) {
    var _this3 = this;

    // For each property, sets uniforms in the pattern:
    // u_material.diffuse, u_material.diffuseScale u_material_diffuse_texture
    material_props.forEach(function (prop) {
      if (_this3[prop]) {
        if (_this3[prop].texture) {
          _program.setTextureUniform("u_material_" + prop + "_texture", _this3[prop].texture);

          _program.uniform('3fv', "u_material." + prop + "Scale", _this3[prop].scale);

          _program.uniform('4fv', "u_material." + prop, _this3[prop].amount);
        } else if (_this3[prop].amount) {
          _program.uniform('4fv', "u_material." + prop, _this3[prop].amount);
        }
      }
    }); // Extra specular props

    if (this.specular) {
      _program.uniform('1f', 'u_material.shininess', this.specular.shininess);
    } // Normal mapping


    if (this.normal && this.normal.texture) {
      _program.setTextureUniform('u_material_normal_texture', this.normal.texture);

      _program.uniform('3fv', 'u_material.normalScale', this.normal.scale);

      _program.uniform('1f', 'u_material.normalAmount', this.normal.amount);
    }
  };

  return Material;
}();
Material.block = 'material';

/*** Vector functions - vectors provided as [x, y] or [x, y, z] arrays ***/
var Vector;
var Vector$1 = Vector = {};

Vector.copy = function (v) {
  var V = [];
  var lim = v.length;

  for (var i = 0; i < lim; i++) {
    V[i] = v[i];
  }

  return V;
};

Vector.neg = function (v) {
  var V = [];
  var lim = v.length;

  for (var i = 0; i < lim; i++) {
    V[i] = -v[i];
  }

  return V;
}; // Addition of two vectors


Vector.add = function (v1, v2) {
  var v = [];
  var lim = Math.min(v1.length, v2.length);

  for (var i = 0; i < lim; i++) {
    v[i] = v1[i] + v2[i];
  }

  return v;
}; // Substraction of two vectors


Vector.sub = function (v1, v2) {
  var v = [];
  var lim = Math.min(v1.length, v2.length);

  for (var i = 0; i < lim; i++) {
    v[i] = v1[i] - v2[i];
  }

  return v;
};

Vector.signed_area = function (v1, v2, v3) {
  return (v2[0] - v1[0]) * (v3[1] - v1[1]) - (v3[0] - v1[0]) * (v2[1] - v1[1]);
}; // Multiplication of two vectors, or a vector and a scalar


Vector.mult = function (v1, v2) {
  var v = [],
      len = v1.length,
      i;

  if (typeof v2 === 'number') {
    // Mulitply by scalar
    for (i = 0; i < len; i++) {
      v[i] = v1[i] * v2;
    }
  } else {
    // Multiply two vectors
    len = Math.min(v1.length, v2.length);

    for (i = 0; i < len; i++) {
      v[i] = v1[i] * v2[i];
    }
  }

  return v;
}; // Division of two vectors


Vector.div = function (v1, v2) {
  var v = [],
      i;

  if (typeof v2 === 'number') {
    // Divide by scalar
    for (i = 0; i < v1.length; i++) {
      v[i] = v1[i] / v2;
    }
  } else {
    // Divide to vectors
    var len = Math.min(v1.length, v2.length);

    for (i = 0; i < len; i++) {
      v[i] = v1[i] / v2[i];
    }
  }

  return v;
}; // Get 2D perpendicular


Vector.perp = function (v1, v2) {
  return [v2[1] - v1[1], v1[0] - v2[0]];
}; // Get 2D vector rotated


Vector.rot = function (v, a) {
  var c = Math.cos(a);
  var s = Math.sin(a);
  return [v[0] * c - v[1] * s, v[0] * s + v[1] * c];
}; // Get 2D counter-clockwise angle
// Angles in quadrant I and II are mapped to [0, PI)
// Angles in quadrant III and IV are mapped to [-PI, 0]


Vector.angle = function (_ref) {
  var x = _ref[0],
      y = _ref[1];
  return Math.atan2(y, x);
}; // Get angle between two vectors


Vector.angleBetween = function (A, B) {
  var delta = Vector.dot(Vector.normalize(Vector.copy(A)), Vector.normalize(Vector.copy(B)));

  if (delta > 1) {
    delta = 1;
  } // protect against floating point error


  return Math.acos(delta);
}; // Compare two points


Vector.isEqual = function (v1, v2) {
  var len = v1.length;

  for (var i = 0; i < len; i++) {
    if (v1[i] !== v2[i]) {
      return false;
    }
  }

  return true;
}; // Vector length squared


Vector.lengthSq = function (v) {
  if (v.length === 2) {
    return v[0] * v[0] + v[1] * v[1];
  } else if (v.length >= 3) {
    return v[0] * v[0] + v[1] * v[1] + v[2] * v[2];
  }

  return 0;
}; // Vector length


Vector.length = function (v) {
  return Math.sqrt(Vector.lengthSq(v));
}; // Normalize a vector *in place* (use Vector.copy() if you need a new vector instance)


Vector.normalize = function (v) {
  var d;

  if (v.length === 2) {
    d = v[0] * v[0] + v[1] * v[1];

    if (d === 1) {
      return v;
    }

    d = Math.sqrt(d);

    if (d !== 0) {
      v[0] /= d;
      v[1] /= d;
    } else {
      v[0] = 0, v[1] = 0;
    }
  } else if (v.length >= 3) {
    d = v[0] * v[0] + v[1] * v[1] + v[2] * v[2];

    if (d === 1) {
      return v;
    }

    d = Math.sqrt(d);

    if (d !== 0) {
      v[0] /= d;
      v[1] /= d;
      v[2] /= d;
    } else {
      v[0] = 0, v[1] = 0, v[2] = 0;
    }
  }

  return v;
}; // Cross product of two vectors


Vector.cross = function (v1, v2) {
  if (v1.length === 2) {
    return v1[0] * v2[1] - v1[1] * v2[0];
  } else if (v1.length === 3) {
    return [v1[1] * v2[2] - v1[2] * v2[1], v1[2] * v2[0] - v1[0] * v2[2], v1[0] * v2[1] - v1[1] * v2[0]];
  }
}; // Dot product of two vectors


Vector.dot = function (v1, v2) {
  var n = 0;
  var lim = Math.min(v1.length, v2.length);

  for (var i = 0; i < lim; i++) {
    n += v1[i] * v2[i];
  }

  return n;
};

var ambient_source = "/*\n\nExpected globals:\nlight_accumulator_*\n\n*/\n\nstruct AmbientLight {\n    vec3 ambient;\n};\n\nvoid calculateLight(in AmbientLight _light, in vec3 _eyeToPoint, in vec3 _normal) {\n    light_accumulator_ambient.rgb += _light.ambient;\n}\n";

var directional_source = "/*\n\nExpected globals:\nmaterial\nlight_accumulator_*\n\n*/\n\nstruct DirectionalLight {\n    vec3 ambient;\n    vec3 diffuse;\n    vec3 specular;\n    vec3 direction;\n};\n\nvoid calculateLight(in DirectionalLight _light, in vec3 _eyeToPoint, in vec3 _normal) {\n\n    light_accumulator_ambient.rgb += _light.ambient;\n\n    float nDotVP = clamp(dot(_normal, -_light.direction), 0.0, 1.0);\n\n    #ifdef TANGRAM_MATERIAL_DIFFUSE\n        light_accumulator_diffuse.rgb += _light.diffuse * nDotVP;\n    #endif\n\n    #ifdef TANGRAM_MATERIAL_SPECULAR\n        float pf = 0.0;\n        if (nDotVP > 0.0) {\n            vec3 reflectVector = reflect(_light.direction, _normal);\n            float eyeDotR = max(dot(normalize(_eyeToPoint), reflectVector), 0.0);\n            pf = pow(eyeDotR, material.shininess);\n        }\n        light_accumulator_specular.rgb += _light.specular * pf;\n    #endif\n}\n";

var point_source = "/*\n\nExpected globals:\nmaterial\nlight_accumulator_*\n\n*/\n\nstruct PointLight {\n    vec3 ambient;\n    vec3 diffuse;\n    vec3 specular;\n    vec4 position;\n\n#ifdef TANGRAM_POINTLIGHT_ATTENUATION_EXPONENT\n    float attenuationExponent;\n#endif\n\n#ifdef TANGRAM_POINTLIGHT_ATTENUATION_INNER_RADIUS\n    float innerRadius;\n#endif\n\n#ifdef TANGRAM_POINTLIGHT_ATTENUATION_OUTER_RADIUS\n    float outerRadius;\n#endif\n};\n\nvoid calculateLight(in PointLight _light, in vec3 _eyeToPoint, in vec3 _normal) {\n\n    float dist = length(_light.position.xyz - _eyeToPoint);\n\n    // Compute vector from surface to light position\n    vec3 VP = (_light.position.xyz - _eyeToPoint) / dist;\n\n    // Normalize the vector from surface to light position\n    float nDotVP = clamp(dot(VP, _normal), 0.0, 1.0);\n\n    // Attenuation defaults\n    float attenuation = 1.0;\n    #ifdef TANGRAM_POINTLIGHT_ATTENUATION_EXPONENT\n        float Rin = 1.0;\n        float e = _light.attenuationExponent;\n\n        #ifdef TANGRAM_POINTLIGHT_ATTENUATION_INNER_RADIUS\n            Rin = _light.innerRadius;\n        #endif\n\n        #ifdef TANGRAM_POINTLIGHT_ATTENUATION_OUTER_RADIUS\n            float Rdiff = _light.outerRadius-Rin;\n            float d = clamp(max(0.0,dist-Rin)/Rdiff, 0.0, 1.0);\n            attenuation = 1.0-(pow(d,e));\n        #else\n            // If no outer is provide behaves like:\n            // https://imdoingitwrong.wordpress.com/2011/01/31/light-attenuation/\n            float d = max(0.0,dist-Rin)/Rin+1.0;\n            attenuation = clamp(1.0/(pow(d,e)), 0.0, 1.0);\n        #endif\n    #else\n        float Rin = 0.0;\n\n        #ifdef TANGRAM_POINTLIGHT_ATTENUATION_INNER_RADIUS\n            Rin = _light.innerRadius;\n            #ifdef TANGRAM_POINTLIGHT_ATTENUATION_OUTER_RADIUS\n                float Rdiff = _light.outerRadius-Rin;\n                float d = clamp(max(0.0,dist-Rin)/Rdiff, 0.0, 1.0);\n                attenuation = 1.0-d*d;\n            #else\n                // If no outer is provide behaves like:\n                // https://imdoingitwrong.wordpress.com/2011/01/31/light-attenuation/\n                float d = max(0.0,dist-Rin)/Rin+1.0;\n                attenuation = clamp(1.0/d, 0.0, 1.0);\n            #endif\n        #else\n            #ifdef TANGRAM_POINTLIGHT_ATTENUATION_OUTER_RADIUS\n                float d = clamp(dist/_light.outerRadius, 0.0, 1.0);\n                attenuation = 1.0-d*d;\n            #else\n                attenuation = 1.0;\n            #endif\n        #endif\n    #endif\n\n    // Computer accumulators\n    light_accumulator_ambient.rgb += _light.ambient * attenuation;\n\n    #ifdef TANGRAM_MATERIAL_DIFFUSE\n        light_accumulator_diffuse.rgb += _light.diffuse * nDotVP * attenuation;\n    #endif\n\n    #ifdef TANGRAM_MATERIAL_SPECULAR\n        float pf = 0.0; // power factor for shiny speculars\n        if (nDotVP > 0.0) {\n            vec3 reflectVector = reflect(-VP, _normal);\n            float eyeDotR = max(0.0, dot(-normalize(_eyeToPoint), reflectVector));\n            pf = pow(eyeDotR, material.shininess);\n        }\n\n        light_accumulator_specular.rgb += _light.specular * pf * attenuation;\n    #endif\n}\n";

var spot_source = "/*\n\nExpected globals:\nmaterial\nlight_accumulator_*\n\n*/\n\nstruct SpotLight {\n    vec3 ambient;\n    vec3 diffuse;\n    vec3 specular;\n    vec4 position;\n\n#ifdef TANGRAM_POINTLIGHT_ATTENUATION_EXPONENT\n    float attenuationExponent;\n#endif\n\n#ifdef TANGRAM_POINTLIGHT_ATTENUATION_INNER_RADIUS\n    float innerRadius;\n#endif\n\n#ifdef TANGRAM_POINTLIGHT_ATTENUATION_OUTER_RADIUS\n    float outerRadius;\n#endif\n\n    vec3 direction;\n    float spotCosCutoff;\n    float spotExponent;\n};\n\nvoid calculateLight(in SpotLight _light, in vec3 _eyeToPoint, in vec3 _normal) {\n\n    float dist = length(_light.position.xyz - _eyeToPoint);\n\n    // Compute vector from surface to light position\n    vec3 VP = (_light.position.xyz - _eyeToPoint) / dist;\n\n    // normal . light direction\n    float nDotVP = clamp(dot(_normal, VP), 0.0, 1.0);\n\n    // Attenuation defaults\n    float attenuation = 1.0;\n    #ifdef TANGRAM_POINTLIGHT_ATTENUATION_EXPONENT\n        float Rin = 1.0;\n        float e = _light.attenuationExponent;\n\n        #ifdef TANGRAM_POINTLIGHT_ATTENUATION_INNER_RADIUS\n            Rin = _light.innerRadius;\n        #endif\n\n        #ifdef TANGRAM_POINTLIGHT_ATTENUATION_OUTER_RADIUS\n            float Rdiff = _light.outerRadius-Rin;\n            float d = clamp(max(0.0,dist-Rin)/Rdiff, 0.0, 1.0);\n            attenuation = 1.0-(pow(d,e));\n        #else\n            // If no outer is provide behaves like:\n            // https://imdoingitwrong.wordpress.com/2011/01/31/light-attenuation/\n            float d = max(0.0,dist-Rin)/Rin+1.0;\n            attenuation = clamp(1.0/(pow(d,e)), 0.0, 1.0);\n        #endif\n    #else\n        float Rin = 0.0;\n\n        #ifdef TANGRAM_POINTLIGHT_ATTENUATION_INNER_RADIUS\n            Rin = _light.innerRadius;\n            #ifdef TANGRAM_POINTLIGHT_ATTENUATION_OUTER_RADIUS\n                float Rdiff = _light.outerRadius-Rin;\n                float d = clamp(max(0.0,dist-Rin)/Rdiff, 0.0, 1.0);\n                attenuation = 1.0-d*d;\n            #else\n                // If no outer is provide behaves like:\n                // https://imdoingitwrong.wordpress.com/2011/01/31/light-attenuation/\n                float d = max(0.0,dist-Rin)/Rin+1.0;\n                attenuation = clamp(1.0/d, 0.0, 1.0);\n            #endif\n        #else\n            #ifdef TANGRAM_POINTLIGHT_ATTENUATION_OUTER_RADIUS\n                float d = clamp(dist/_light.outerRadius, 0.0, 1.0);\n                attenuation = 1.0-d*d;\n            #else\n                attenuation = 1.0;\n            #endif\n        #endif\n    #endif\n\n    // spotlight attenuation factor\n    float spotAttenuation = 0.0;\n\n    // See if point on surface is inside cone of illumination\n    float spotDot = clamp(dot(-VP, _light.direction), 0.0, 1.0);\n\n    if (spotDot >= _light.spotCosCutoff) {\n        spotAttenuation = pow(spotDot, _light.spotExponent);\n    }\n\n    light_accumulator_ambient.rgb += _light.ambient * attenuation * spotAttenuation;\n\n    #ifdef TANGRAM_MATERIAL_DIFFUSE\n        light_accumulator_diffuse.rgb += _light.diffuse * nDotVP * attenuation * spotAttenuation;\n    #endif\n\n    #ifdef TANGRAM_MATERIAL_SPECULAR\n        // Power factor for shiny speculars\n        float pf = 0.0;\n        if (nDotVP > 0.0) {\n            vec3 reflectVector = reflect(-VP, _normal);\n            float eyeDotR = max(dot(-normalize(_eyeToPoint), reflectVector), 0.0);\n            pf = pow(eyeDotR, material.shininess);\n        }\n        light_accumulator_specular.rgb += _light.specular * pf * attenuation * spotAttenuation;\n    #endif\n}\n";

var Light =
/*#__PURE__*/
function () {
  function Light(view, config) {
    this.name = config.name;
    this.view = view;

    if (config.ambient == null || typeof config.ambient === 'number') {
      this.ambient = GLSL.expandVec3(config.ambient || 0);
    } else {
      this.ambient = StyleParser.parseColor(config.ambient).slice(0, 3);
    }

    if (config.diffuse == null || typeof config.diffuse === 'number') {
      this.diffuse = GLSL.expandVec3(config.diffuse != null ? config.diffuse : 1);
    } else {
      this.diffuse = StyleParser.parseColor(config.diffuse).slice(0, 3);
    }

    if (config.specular == null || typeof config.specular === 'number') {
      this.specular = GLSL.expandVec3(config.specular || 0);
    } else {
      this.specular = StyleParser.parseColor(config.specular).slice(0, 3);
    }
  } // Create a light by type name, factory-style
  // 'config' must include 'name' and 'type', along with any other type-specific properties


  Light.create = function create(view, config) {
    if (Light.types[config.type]) {
      return new Light.types[config.type](view, config);
    }
  } // Set light for a style: fragment lighting, vertex lighting, or none
  ;

  Light.setMode = function setMode(mode, style) {
    if (mode === true) {
      mode = 'fragment';
    }

    mode = Light.enabled && (mode != null ? mode : 'fragment'); // default to fragment lighting

    style.defines['TANGRAM_LIGHTING_FRAGMENT'] = mode === 'fragment';
    style.defines['TANGRAM_LIGHTING_VERTEX'] = mode === 'vertex';
  } // Inject all provided light definitions, and calculate cumulative light function
  ;

  Light.inject = function inject(lights) {
    // Clear previous injections
    ShaderProgram.removeBlock(Light.block); // If lighting is globally disabled, nothing is injected (mostly for debugging or live editing)

    if (!Light.enabled) {
      return;
    } // Construct code to calculate each light instance


    var calculateLights = '';

    if (lights && Object.keys(lights).length > 0) {
      // Collect uniques types of lights
      var types = {};

      for (var light_name in lights) {
        types[lights[light_name].type] = true;
      } // Inject each type of light


      for (var type in types) {
        Light.types[type].inject();
      } // Inject per-instance blocks and construct the list of functions to calculate each light


      for (var _light_name in lights) {
        // Define instance
        lights[_light_name].inject(); // Add the calculation function to the list


        calculateLights += "calculateLight(" + _light_name + ", _eyeToPoint, _normal);\n";
      }
    } // Glue together the final lighting function that sums all the lights


    var calculateFunction = "\n            vec4 calculateLighting(in vec3 _eyeToPoint, in vec3 _normal, in vec4 _color) {\n\n                // Do initial material calculations over normal, emission, ambient, diffuse and specular values\n                calculateMaterial(_eyeToPoint,_normal);\n\n                // Un roll the loop of individual ligths to calculate\n                " + calculateLights + "\n\n                //  Final light intensity calculation\n                vec4 color = vec4(vec3(0.), _color.a); // start with vertex color alpha\n\n                #ifdef TANGRAM_MATERIAL_EMISSION\n                    color.rgb = material.emission.rgb;\n                    color.a *= material.emission.a;\n                #endif\n\n                #ifdef TANGRAM_MATERIAL_AMBIENT\n                    color.rgb += light_accumulator_ambient.rgb * _color.rgb * material.ambient.rgb;\n                    color.a *= material.ambient.a;\n                #else\n                    #ifdef TANGRAM_MATERIAL_DIFFUSE\n                        color.rgb += light_accumulator_ambient.rgb * _color.rgb * material.diffuse.rgb;\n                    #endif\n                #endif\n\n                #ifdef TANGRAM_MATERIAL_DIFFUSE\n                    color.rgb += light_accumulator_diffuse.rgb * _color.rgb * material.diffuse.rgb;\n                    color.a *= material.diffuse.a;\n                #endif\n\n                #ifdef TANGRAM_MATERIAL_SPECULAR\n                    color.rgb += light_accumulator_specular.rgb * material.specular.rgb;\n                    color.a *= material.specular.a;\n                #endif\n\n                // Clamp final color\n                color = clamp(color, 0.0, 1.0);\n\n                return color;\n            }";
    ShaderProgram.addBlock(Light.block, calculateFunction);
  } // Common instance definition
  ;

  var _proto = Light.prototype;

  _proto.inject = function inject() {
    var instance = "\n            uniform " + this.struct_name + " u_" + this.name + ";\n            " + this.struct_name + " " + this.name + ";\n            ";
    var assign = "\n            " + this.name + " = u_" + this.name + ";\n\n        ";
    ShaderProgram.addBlock(Light.block, instance);
    ShaderProgram.addBlock('setup', assign);
  } // Update method called once per frame
  ;

  _proto.update = function update() {} // Called once per frame per program (e.g. for main render pass, then for each additional
  // pass for feature selection, etc.)
  ;

  _proto.setupProgram = function setupProgram(_program) {
    //  Three common light properties
    _program.uniform('3fv', "u_" + this.name + ".ambient", this.ambient);

    _program.uniform('3fv', "u_" + this.name + ".diffuse", this.diffuse);

    _program.uniform('3fv', "u_" + this.name + ".specular", this.specular);
  };

  return Light;
}();
Light.types = {}; // references to subclasses by short name

Light.block = 'lighting'; // shader block name

Light.enabled = true; // lighting can be globally enabled/disabled
// Light subclasses

var AmbientLight =
/*#__PURE__*/
function (_Light) {
  _inheritsLoose(AmbientLight, _Light);

  function AmbientLight(view, config) {
    var _this;

    _this = _Light.call(this, view, config) || this;
    _this.type = 'ambient';
    _this.struct_name = 'AmbientLight';
    return _this;
  } // Inject struct and calculate function


  AmbientLight.inject = function inject() {
    ShaderProgram.addBlock(Light.block, ambient_source);
  };

  var _proto2 = AmbientLight.prototype;

  _proto2.setupProgram = function setupProgram(_program) {
    _program.uniform('3fv', "u_" + this.name + ".ambient", this.ambient);
  };

  return AmbientLight;
}(Light);

Light.types['ambient'] = AmbientLight;

var DirectionalLight =
/*#__PURE__*/
function (_Light2) {
  _inheritsLoose(DirectionalLight, _Light2);

  function DirectionalLight(view, config) {
    var _this2;

    _this2 = _Light2.call(this, view, config) || this;
    _this2.type = 'directional';
    _this2.struct_name = 'DirectionalLight';

    if (config.direction) {
      _this2._direction = config.direction;
    } else {
      // Default directional light maintains full intensity on ground, with basic extrusion shading
      var theta = 135; // angle of light in xy plane (rotated around z axis)

      var scale = Math.sin(Math.PI * 60 / 180); // scaling factor to keep total directional intensity to 0.5

      _this2._direction = [Math.cos(Math.PI * theta / 180) * scale, Math.sin(Math.PI * theta / 180) * scale, -0.5];

      if (config.ambient == null) {
        _this2.ambient = GLSL.expandVec3(0.5);
      }
    }

    _this2.direction = _this2._direction.map(parseFloat);
    return _this2;
  }

  // Inject struct and calculate function
  DirectionalLight.inject = function inject() {
    ShaderProgram.addBlock(Light.block, directional_source);
  };

  var _proto3 = DirectionalLight.prototype;

  _proto3.setupProgram = function setupProgram(_program) {
    _Light2.prototype.setupProgram.call(this, _program);

    _program.uniform('3fv', "u_" + this.name + ".direction", this.direction);
  };

  _createClass(DirectionalLight, [{
    key: "direction",
    get: function get() {
      return this._direction;
    },
    set: function set(v) {
      this._direction = Vector$1.normalize(Vector$1.copy(v));
    }
  }]);

  return DirectionalLight;
}(Light);

Light.types['directional'] = DirectionalLight;

var PointLight =
/*#__PURE__*/
function (_Light3) {
  _inheritsLoose(PointLight, _Light3);

  function PointLight(view, config) {
    var _this3;

    _this3 = _Light3.call(this, view, config) || this;
    _this3.type = 'point';
    _this3.struct_name = 'PointLight';
    _this3.position = config.position || [0, 0, '100px'];
    _this3.position_eye = []; // position in eyespace

    _this3.origin = config.origin || 'ground';
    _this3.attenuation = !isNaN(parseFloat(config.attenuation)) ? parseFloat(config.attenuation) : 0;

    if (config.radius) {
      if (Array.isArray(config.radius) && config.radius.length === 2) {
        _this3.radius = config.radius;
      } else {
        _this3.radius = [null, config.radius];
      }
    } else {
      _this3.radius = null;
    }

    return _this3;
  } // Inject struct and calculate function


  PointLight.inject = function inject() {
    ShaderProgram.addBlock(Light.block, point_source);
  } // Inject isntance-specific settings
  ;

  var _proto4 = PointLight.prototype;

  _proto4.inject = function inject() {
    _Light3.prototype.inject.call(this);

    ShaderProgram.defines['TANGRAM_POINTLIGHT_ATTENUATION_EXPONENT'] = this.attenuation !== 0;
    ShaderProgram.defines['TANGRAM_POINTLIGHT_ATTENUATION_INNER_RADIUS'] = this.radius != null && this.radius[0] != null;
    ShaderProgram.defines['TANGRAM_POINTLIGHT_ATTENUATION_OUTER_RADIUS'] = this.radius != null;
  };

  _proto4.update = function update() {
    this.updateEyePosition();
  };

  _proto4.updateEyePosition = function updateEyePosition() {
    if (this.origin === 'world') {
      // For world origin, format is: [longitude, latitude, meters (default) or pixels w/px units]
      // Move light's world position into camera space
      var m = Geo$1.latLngToMeters([].concat(this.position));
      this.position_eye[0] = m[0] - this.view.camera.position_meters[0];
      this.position_eye[1] = m[1] - this.view.camera.position_meters[1];
      this.position_eye[2] = StyleParser.convertUnits(this.position[2], {
        zoom: this.view.zoom,
        meters_per_pixel: Geo$1.metersPerPixel(this.view.zoom)
      });
      this.position_eye[2] = this.position_eye[2] - this.view.camera.position_meters[2];
    } else if (this.origin === 'ground' || this.origin === 'camera') {
      // For camera or ground origin, format is: [x, y, z] in meters (default) or pixels w/px units
      // Light is in camera space by default
      this.position_eye = StyleParser.convertUnits(this.position, {
        zoom: this.view.zoom,
        meters_per_pixel: Geo$1.metersPerPixel(this.view.zoom)
      });

      if (this.origin === 'ground') {
        // Leave light's xy in camera space, but z needs to be moved relative to ground plane
        this.position_eye[2] = this.position_eye[2] - this.view.camera.position_meters[2];
      }
    }

    this.position_eye[3] = 1;
  };

  _proto4.setupProgram = function setupProgram(_program) {
    _Light3.prototype.setupProgram.call(this, _program);

    _program.uniform('4fv', "u_" + this.name + ".position", this.position_eye);

    if (ShaderProgram.defines['TANGRAM_POINTLIGHT_ATTENUATION_EXPONENT']) {
      _program.uniform('1f', "u_" + this.name + ".attenuationExponent", this.attenuation);
    }

    if (ShaderProgram.defines['TANGRAM_POINTLIGHT_ATTENUATION_INNER_RADIUS']) {
      _program.uniform('1f', "u_" + this.name + ".innerRadius", StyleParser.convertUnits(this.radius[0], {
        zoom: this.view.zoom,
        meters_per_pixel: Geo$1.metersPerPixel(this.view.zoom)
      }));
    }

    if (ShaderProgram.defines['TANGRAM_POINTLIGHT_ATTENUATION_OUTER_RADIUS']) {
      _program.uniform('1f', "u_" + this.name + ".outerRadius", StyleParser.convertUnits(this.radius[1], {
        zoom: this.view.zoom,
        meters_per_pixel: Geo$1.metersPerPixel(this.view.zoom)
      }));
    }
  };

  return PointLight;
}(Light);

Light.types['point'] = PointLight;

var SpotLight =
/*#__PURE__*/
function (_PointLight) {
  _inheritsLoose(SpotLight, _PointLight);

  function SpotLight(view, config) {
    var _this4;

    _this4 = _PointLight.call(this, view, config) || this;
    _this4.type = 'spotlight';
    _this4.struct_name = 'SpotLight';
    _this4.direction = _this4._direction = (config.direction || [0, 0, -1]).map(parseFloat); // [x, y, z]

    _this4.exponent = config.exponent ? parseFloat(config.exponent) : 0.2;
    _this4.angle = config.angle ? parseFloat(config.angle) : 20;
    return _this4;
  }

  // Inject struct and calculate function
  SpotLight.inject = function inject() {
    ShaderProgram.addBlock(Light.block, spot_source);
  };

  var _proto5 = SpotLight.prototype;

  _proto5.setupProgram = function setupProgram(_program) {
    _PointLight.prototype.setupProgram.call(this, _program);

    _program.uniform('3fv', "u_" + this.name + ".direction", this.direction);

    _program.uniform('1f', "u_" + this.name + ".spotCosCutoff", Math.cos(this.angle * 3.14159 / 180));

    _program.uniform('1f', "u_" + this.name + ".spotExponent", this.exponent);
  };

  _createClass(SpotLight, [{
    key: "direction",
    get: function get() {
      return this._direction;
    },
    set: function set(v) {
      this._direction = Vector$1.normalize(Vector$1.copy(v));
    }
  }]);

  return SpotLight;
}(PointLight);

Light.types['spotlight'] = SpotLight;

// 20.2.2.22 Math.log2(x)


_export(_export.S, 'Math', {
  log2: function log2(x) {
    return Math.log(x) / Math.LN2;
  }
});

var MethodNotImplemented =
/*#__PURE__*/
function (_Error) {
  _inheritsLoose(MethodNotImplemented, _Error);

  function MethodNotImplemented(methodName) {
    var _this;

    _this = _Error.call(this) || this;
    _this.name = 'MethodNotImplemented';
    _this.message = 'Method ' + methodName + ' must be implemented in subclass';
    return _this;
  }

  return MethodNotImplemented;
}(_wrapNativeSuper(Error));

var DataSource =
/*#__PURE__*/
function () {
  function DataSource(config, sources) {
    var _this = this;

    this.validate(config);
    this.config = config; // save original config

    this.sources = sources; // full set of data sources TODO: centralize these like textures?

    this.id = config.id;
    this.name = config.name;
    this.pad_scale = config.pad_scale || 0.00001; // scale tile up by small factor to cover seams

    this.default_winding = null; // winding order will adapt to data source

    this.rasters = []; // attached raster tile sources

    if (Array.isArray(config.rasters)) {
      // copy unique set of raster sources
      config.rasters.forEach(function (r) {
        if (_this.rasters.indexOf(r) === -1) {
          _this.rasters.push(r);
        }
      });
    } // Optional function to preprocess source data


    this.preprocess = config.preprocess;

    if (typeof this.preprocess === 'function') {
      this.preprocess.bind(this);
    } // Optional function to transform source data


    this.transform = config.transform;

    if (typeof this.transform === 'function') {
      this.transform.bind(this);
    } // Optional additional data to pass to the transform function


    this.extra_data = config.extra_data; // Optional additional scripts made available to the transform function
    // NOTE: these are loaded alongside the library when the workers are instantiated

    this.scripts = config.scripts; // Configure zoom ranges at which new data will be loaded

    this.setZooms(config); // set a custom extra overzoom adjustment factor to load consistently lower zoom levels
    // than the current map zoom level – eg a zoom_offset of 1 would load z3 data at z4

    this.zoom_offset = config.zoom_offset != null ? config.zoom_offset : 0;

    if (this.zoom_offset < 0) {
      var msg = "Data source '" + this.name + "' zoom_offset must not be negative \u2013 setting to 0.";
      log({
        level: 'warn',
        once: true
      }, msg);
      this.zoom_offset = 0;
    }

    this.setTileSize(config.tile_size); // no tiles will be requested or displayed outside of these min/max values

    this.min_display_zoom = Math.max(config.min_display_zoom || 0, this.zooms[0]);
    this.max_display_zoom = config.max_display_zoom != null ? config.max_display_zoom : null;
  } // Register a new data source type name, providing a function that returns the class name
  // to instantiate based on the source definition in the scene


  DataSource.register = function register(type_name, type_func) {
    if (!type_name || !type_func) {
      return;
    }

    DataSource.types[type_name] = type_func;
  } // Create a data source, factory-style
  ;

  DataSource.create = function create(source, sources) {
    // Find the class to instantiate based on the source definition
    if (typeof DataSource.types[source.type] === 'function') {
      var source_class = DataSource.types[source.type](source);

      if (source_class) {
        return new source_class(source, sources);
      }
    }
  } // Check if a data source definition changed in a way that could affect which tiles are in view
  ;

  DataSource.tileLayoutChanged = function tileLayoutChanged(source, prev_source) {
    if (!source || !prev_source) {
      return true;
    } // subset of parameters that affect tile layout


    var rebuild_params = ['max_zoom', 'zooms', 'min_display_zoom', 'max_display_zoom', 'bounds', 'tile_size', 'zoom_offset'];
    var cur = sliceObject(source.config, rebuild_params);
    var prev = sliceObject(prev_source.config, rebuild_params);
    return JSON.stringify(cur) !== JSON.stringify(prev);
  } // Mercator projection
  ;

  DataSource.projectData = function projectData(source) {
    var timer = +new Date();

    for (var t in source.layers) {
      var num_features = source.layers[t].features.length;

      for (var f = 0; f < num_features; f++) {
        var feature = source.layers[t].features[f];
        Geo$1.transformGeometry(feature.geometry, this.projectCoord);
      }
    }

    if (source.debug !== undefined) {
      source.debug.projection = +new Date() - timer;
    }
  };

  DataSource.projectCoord = function projectCoord(coord) {
    Geo$1.latLngToMeters(coord);
  }
  /**
   Re-scale geometries within each source to internal tile units
  */
  ;

  DataSource.scaleData = function scaleData(source, _ref) {
    var z = _ref.coords.z,
        min = _ref.min;
    var units_per_meter = Geo$1.unitsPerMeter(z);

    for (var t in source.layers) {
      var num_features = source.layers[t].features.length;

      for (var f = 0; f < num_features; f++) {
        var feature = source.layers[t].features[f];
        Geo$1.transformGeometry(feature.geometry, function (coord) {
          coord[0] = (coord[0] - min.x) * units_per_meter;
          coord[1] = (coord[1] - min.y) * units_per_meter * -1; // flip coords positive
        });
      }
    }
  };

  var _proto = DataSource.prototype;

  _proto.load = function load(dest) {
    var _this2 = this;

    dest.source_data = {};
    dest.source_data.layers = {};
    dest.pad_scale = this.pad_scale;
    dest.rasters = [].concat(this.rasters); // copy list of rasters to load for tile

    return this._load(dest).then(function (dest) {
      // Post-processing
      for (var layer in dest.source_data.layers) {
        var data = dest.source_data.layers[layer];

        if (data && data.features) {
          data.features.forEach(function (feature) {
            Geo$1.transformGeometry(feature.geometry, function (coord) {
              // Flip Y coords
              coord[1] = -coord[1]; // Slightly scale up tile to cover seams

              if (_this2.pad_scale) {
                coord[0] = Math.round(coord[0] * (1 + _this2.pad_scale) - Geo$1.tile_scale * _this2.pad_scale / 2);
                coord[1] = Math.round(coord[1] * (1 + _this2.pad_scale) - Geo$1.tile_scale * _this2.pad_scale / 2);
              }
            }); // Use first encountered polygon winding order as default for data source

            _this2.updateDefaultWinding(feature.geometry);
          });
        }
      }

      dest.default_winding = _this2.default_winding || 'CCW';
      return dest;
    });
  } // Sub-classes must implement
  ;

  _proto._load = function _load()
  /*dest*/
  {
    throw new MethodNotImplemented('_load');
  } // Copy source data from another tile (so we can reuse source data for overzoomed tiles)
  ;

  _proto.copyTileData = function copyTileData(source, dest) {
    log('trace', "Copy tile data from " + source.key + " to " + dest.key);
    dest.source_data = {
      layers: source.source_data.layers
    };
    dest.rasters = [].concat(source.rasters);
    dest.pad_scale = source.pad_scale;
    dest.default_winding = source.default_winding;
    return dest;
  } // Configure zoom ranges at which new data will be loaded
  // e.g. can be used to skip fetching data for some zooms, reusing data from next lowest available zoom instead
  ;

  _proto.setZooms = function setZooms(_ref2) {
    var max_zoom = _ref2.max_zoom,
        zooms = _ref2.zooms;
    // overzoom will apply for zooms higher than this
    this.max_zoom = max_zoom != null ? max_zoom : Geo$1.default_source_max_zoom;

    if (Array.isArray(zooms)) {
      this.zooms = zooms; // TODO: support range parsing, e.g. [0-4, 6-7, 12]?

      this.max_zoom = this.zooms[this.zooms.length - 1]; // overrides `max_zoom` when both are present
    } else {
      this.zooms = [];

      for (var i = 0; i <= this.max_zoom; i++) {
        this.zooms[i] = i;
      }
    }
  } // Set the internal tile size in pixels, e.g. '256px' (default), '512px', etc.
  // Must be a power of 2, and greater than or equal to 256
  ;

  _proto.setTileSize = function setTileSize(tile_size) {
    this.tile_size = tile_size || 256;

    if (typeof this.tile_size !== 'number' || this.tile_size < 256 || !Utils.isPowerOf2(this.tile_size)) {
      log({
        level: 'warn',
        once: true
      }, "Data source '" + this.name + "': 'tile_size' parameter must be a number that is a power of 2 greater than or equal to 256, but was '" + tile_size + "'");
      this.tile_size = 256;
    } // # of zoom levels bigger than 256px tiles - 8 in place of log2(256)
    // Many Tangram functions assume 256px tiles, this factor adjusts for the
    // case of bigger tile sizes - eg 512px tiles are 1 zoom level bigger,
    // 1024px tiles are 2 levels bigger


    this.zoom_bias = Math.log2(this.tile_size) - 8 + this.zoom_offset;
  } // Infer winding for data source from first ring of provided geometry
  ;

  _proto.updateDefaultWinding = function updateDefaultWinding(geom) {
    if (this.default_winding == null) {
      if (geom.type === 'Polygon') {
        this.default_winding = Geo$1.ringWinding(geom.coordinates[0]);
      } else if (geom.type === 'MultiPolygon') {
        this.default_winding = Geo$1.ringWinding(geom.coordinates[0][0]);
      }
    }

    return this.default_winding;
  } // All data sources support a min zoom, tiled sources can subclass for more specific limits (e.g. bounding box)
  ;

  _proto.includesTile = function includesTile(coords, style_z) {
    // Limit by this data source
    if (coords.z < this.min_display_zoom || this.max_display_zoom != null && style_z > this.max_display_zoom) {
      return false;
    } // Limit by any dependent raster sources


    for (var r = 0; r < this.rasters.length; r++) {
      var source_name = this.rasters[r];

      if (this.sources[source_name] && this.sources[source_name] !== this && !this.sources[source_name].includesTile(coords, coords.z)) {
        return false;
      }
    }

    return true;
  };

  _proto.validate = function validate()
  /*source*/
  {};

  return DataSource;
}();
DataSource.types = {}; // set of supported data source classes, referenced by type name

/*** Generic network loading source - abstract class ***/

var network_request_id = 0; // used to namespace URL requests

var NetworkSource =
/*#__PURE__*/
function (_DataSource) {
  _inheritsLoose(NetworkSource, _DataSource);

  function NetworkSource(source, sources) {
    var _this3;

    _this3 = _DataSource.call(this, source, sources) || this;
    _this3.response_type = ''; // use to set explicit XHR type
    // Add extra URL params, and warn on duplicates

    var _URLs$addParamsToURL = addParamsToURL(source.url, source.url_params),
        url = _URLs$addParamsToURL[0],
        dupes = _URLs$addParamsToURL[1];

    _this3.url = url;
    dupes.forEach(function (_ref3) {
      var param = _ref3[0],
          value = _ref3[1];
      log({
        level: 'warn',
        once: true
      }, "Data source '" + _this3.name + "': parameter '" + param + "' already present in URL '" + source.url + "', " + ("skipping value '" + param + "=" + value + "' specified in 'url_params'"));
    }); // Optional HTTP request headers to send

    if (source.request_headers && typeof source.request_headers === 'object') {
      _this3.request_headers = source.request_headers;
    }

    return _this3;
  }

  var _proto2 = NetworkSource.prototype;

  _proto2._load = function _load(dest) {
    var _this4 = this;

    var url = this.formatURL(this.url, dest);
    var source_data = dest.source_data;
    source_data.url = url;
    dest.debug = dest.debug || {};
    dest.debug.network = +new Date();
    return new Promise(function (resolve) {
      var request_id = network_request_id++ + '-' + url;
      var promise = Utils.io(url, 60 * 1000, _this4.response_type, 'GET', _this4.request_headers, request_id);
      source_data.request_id = request_id;
      source_data.error = null;
      promise.then(function (body) {
        dest.debug.response_size = body.length || body.byteLength;
        dest.debug.network = +new Date() - dest.debug.network;
        dest.debug.parsing = +new Date(); // Apply optional data transform on raw network response

        if (typeof _this4.preprocess === 'function') {
          body = _this4.preprocess(body);
        } // Return data immediately, or after user-returned promise resolves


        body = body instanceof Promise ? body : Promise.resolve(body);
        body.then(function (body) {
          _this4.parseSourceData(dest, source_data, body);

          dest.debug.parsing = +new Date() - dest.debug.parsing;
          resolve(dest);
        });
      }).catch(function (error) {
        source_data.error = error.stack;
        resolve(dest); // resolve request but pass along error
      });
    });
  };

  _proto2.validate = function validate(source) {
    if (typeof source.url !== 'string') {
      throw Error('Network data source must provide a string `url` property');
    }
  } // Sub-classes must implement:
  ;

  _proto2.formatURL = function formatURL()
  /*url_template, dest*/
  {
    throw new MethodNotImplemented('formatURL');
  };

  _proto2.parseSourceData = function parseSourceData()
  /*dest, source, reponse*/
  {
    throw new MethodNotImplemented('parseSourceData');
  };

  return NetworkSource;
}(DataSource);
/*** Generic network tile loading - abstract class ***/

var NetworkTileSource =
/*#__PURE__*/
function (_NetworkSource) {
  _inheritsLoose(NetworkTileSource, _NetworkSource);

  function NetworkTileSource(source, sources) {
    var _this5;

    _this5 = _NetworkSource.call(this, source, sources) || this;
    _this5.tiled = true;
    _this5.bounds = _this5.parseBounds(source); // indicates if source should build geometry tiles, enabled for sources referenced in the scene's layers,
    // and left disabled for sources that are never referenced, or only used as raster textures

    _this5.builds_geometry_tiles = false;
    _this5.tms = source.tms === true; // optionally flip tile coords for TMS
    // optional list of subdomains to round-robin through

    if (_this5.url && _this5.url.search('{s}') > -1) {
      if (Array.isArray(source.url_subdomains) && source.url_subdomains.length > 0) {
        _this5.url_subdomains = source.url_subdomains;
        _this5.next_url_subdomain = 0;
      } else {
        log({
          level: 'warn',
          once: true
        }, "Data source '" + _this5.name + "': source URL includes '{s}' subdomain marker ('" + _this5.url + "'), but no subdomains " + 'were specified in \'url_subdomains\' parameter');
      }
    } // optional list of pixel density scale modifiers


    if (_this5.url && _this5.url.search('{r}') > -1) {
      if (Array.isArray(source.url_density_scales) && source.url_density_scales.length > 0) {
        _this5.url_density_scales = source.url_density_scales;
      } else {
        _this5.url_density_scales = [1, 2]; // default to supporting 1x and 2x display densities
      }
    }

    return _this5;
  } // Get bounds from source config parameters


  var _proto3 = NetworkTileSource.prototype;

  _proto3.parseBounds = function parseBounds(source) {
    if (Array.isArray(source.bounds) && source.bounds.length === 4) {
      var _source$bounds = source.bounds,
          w = _source$bounds[0],
          s = _source$bounds[1],
          e = _source$bounds[2],
          n = _source$bounds[3];
      return {
        latlng: [].concat(source.bounds),
        meters: {
          min: Geo$1.latLngToMeters([w, n]),
          max: Geo$1.latLngToMeters([e, s])
        },
        tiles: {
          // max tile bounds per zoom (lazily evaluated)
          min: {},
          max: {}
        }
      };
    }
  } // Returns false if tile is outside data source's bounds, true if within
  ;

  _proto3.checkBounds = function checkBounds(coords, bounds) {
    // Check tile bounds
    if (bounds) {
      // get tile and bounds coords at current zoom, wrapping to keep x coords in positive range
      coords = Geo$1.wrapTile(coords);
      var min = bounds.tiles.min[coords.z];

      if (!min) {
        min = bounds.tiles.min[coords.z] = Geo$1.tileForMeters(bounds.meters.min, coords.z);
      }

      var max = bounds.tiles.max[coords.z];

      if (!max) {
        max = bounds.tiles.max[coords.z] = Geo$1.tileForMeters(bounds.meters.max, coords.z);
      } // check latitude


      if (coords.y < min.y || coords.y > max.y) {
        return false;
      } // longitude bounds are between meridians


      if (min.x <= max.x) {
        if (coords.x < min.x || coords.x > max.x) {
          return false;
        }
      } // longitude bounds cross the antimeridian
      else if (coords.x > max.x && coords.x < min.x) {
          return false;
        }
    }

    return true;
  };

  _proto3.includesTile = function includesTile(coords, style_z) {
    if (!_NetworkSource.prototype.includesTile.call(this, coords, style_z)) {
      return false;
    } // Check tile bounds


    if (!this.checkBounds(coords, this.bounds)) {
      return false;
    }

    return true;
  };

  _proto3.formatURL = function formatURL(url_template, tile) {
    var coords = Geo$1.wrapTile(tile.coords, {
      x: true
    });

    if (this.tms) {
      coords.y = Math.pow(2, coords.z) - 1 - coords.y; // optionally flip tile coords for TMS
    } // tile URL template replacement


    var url = url_template.replace('{x}', coords.x).replace('{y}', coords.y).replace('{z}', coords.z).replace('{r}', this.getDensityModifier()) // modify URL by display density (e.g. @2x)
    .replace('{q}', this.toQuadKey(coords)); // quadkey for tile coordinates

    if (this.url_subdomains != null) {
      url = url.replace('{s}', this.url_subdomains[this.next_url_subdomain]);
      this.next_url_subdomain = (this.next_url_subdomain + 1) % this.url_subdomains.length;
    }

    return url;
  } // Find the right tile URL modifier based on the display density, e.g. add `@2x` for sources supporting 2x tiles.
  // Source `url_density_scales` param can specify an array of densities supported by the source,
  // each entry serves as a threshold based on the current display density.
  ;

  _proto3.getDensityModifier = function getDensityModifier() {
    if (this.url_density_scales) {
      // find the highest matching density
      var dpr = Utils.device_pixel_ratio;
      var scale = this.url_density_scales.filter(function (s) {
        return dpr >= s;
      }).reverse()[0]; // default to first scale if none matched

      scale = scale != null ? scale : this.url_density_scales[0]; // scales higher than 1x use the `@` modifier (e.g. `@2x`)

      if (scale > 1) {
        return "@" + scale + "x";
      }
    }

    return ''; // for 1x (or less) displays, no URL modifier is used (following @2x URL convention)
  };

  _proto3.toQuadKey = function toQuadKey(_ref4) {
    var x = _ref4.x,
        y = _ref4.y,
        z = _ref4.z;
    var quadkey = '';

    for (var i = z; i > 0; i--) {
      var b = 0;
      var mask = 1 << i - 1;
      if ((x & mask) !== 0) b++;
      if ((y & mask) !== 0) b += 2;
      quadkey += b.toString();
    }

    return quadkey;
  } // Checks for the x/y/z tile pattern in URL template
  ;

  NetworkTileSource.urlHasTilePattern = function urlHasTilePattern(url) {
    return url && (url.search('{x}') > -1 && url.search('{y}') > -1 && url.search('{z}') > -1 || url.search('{q}') > -1);
  };

  return NetworkTileSource;
}(NetworkSource);

var TileID = {
  coord: function coord(c) {
    return {
      x: c.x,
      y: c.y,
      z: c.z,
      key: this.coordKey(c)
    };
  },
  coordKey: function coordKey(_ref) {
    var x = _ref.x,
        y = _ref.y,
        z = _ref.z;
    return x + '/' + y + '/' + z;
  },
  key: function key(coords, source, style_z) {
    if (coords.y < 0 || coords.y >= 1 << coords.z || coords.z < 0) {
      return; // cull tiles out of range (x will wrap)
    }

    return [source.name, coords.x, coords.y, coords.z, style_z].join('/');
  },
  normalizedKey: function normalizedKey(coords, source, style_z) {
    return this.key(this.normalizedCoord(coords, source), source, style_z);
  },
  normalizedCoord: function normalizedCoord(coords, source) {
    if (source.zoom_bias) {
      coords = this.coordAtZoom(coords, Math.max(coords.z - source.zoom_bias, source.zooms[0]));
    }

    return this.coordForTileZooms(coords, source.zooms);
  },
  coordAtZoom: function coordAtZoom(_ref2, zoom) {
    var x = _ref2.x,
        y = _ref2.y,
        z = _ref2.z;
    zoom = Math.max(0, zoom); // zoom can't go below zero

    if (z !== zoom) {
      var zscale = Math.pow(2, z - zoom);
      x = Math.floor(x / zscale);
      y = Math.floor(y / zscale);
      z = zoom;
    }

    return this.coord({
      x: x,
      y: y,
      z: z
    });
  },
  coordForTileZooms: function coordForTileZooms(_ref3, zooms) {
    var x = _ref3.x,
        y = _ref3.y,
        z = _ref3.z;
    var nz = this.findZoomInRange(z, zooms);

    if (nz !== z) {
      return this.coordAtZoom({
        x: x,
        y: y,
        z: z
      }, nz);
    }

    return this.coord({
      x: x,
      y: y,
      z: z
    });
  },
  findZoomInRange: function findZoomInRange(z, zooms) {
    return zooms.filter(function (s) {
      return z >= s;
    }).reverse()[0] || zooms[0];
  },
  isDescendant: function isDescendant(parent, descendant) {
    if (descendant.z > parent.z) {
      var _this$coordAtZoom = this.coordAtZoom(descendant, parent.z),
          x = _this$coordAtZoom.x,
          y = _this$coordAtZoom.y;

      return parent.x === x && parent.y === y;
    }

    return false;
  },
  // Return identifying info for tile's parent tile
  parent: function parent(_ref4) {
    var coords = _ref4.coords,
        source = _ref4.source,
        style_z = _ref4.style_z;

    if (style_z > 0) {
      // no more tiles above style zoom 0
      style_z--;
      var sz = Math.max(style_z - source.zoom_bias, source.zooms[0]); // z can't be lower than tile source

      var c = this.coordForTileZooms(this.coordAtZoom(coords, sz), source.zooms);

      if (c.z > style_z) {
        return null;
      }

      return {
        key: this.key(c, source, style_z),
        coords: c,
        style_z: style_z,
        source: source
      };
    }
  },
  // Return identifying info for tile's child tiles
  children: function children(_ref5, CACHE) {
    var coords = _ref5.coords,
        source = _ref5.source,
        style_z = _ref5.style_z;

    if (CACHE === void 0) {
      CACHE = {};
    }

    style_z++;
    var c = this.coordForTileZooms(this.coordAtZoom(coords, style_z - source.zoom_bias), source.zooms);

    if (c.z === coords.z) {
      // same coord zoom for next level down
      return [{
        key: this.key(c, source, style_z),
        coords: c,
        style_z: style_z,
        source: source
      }];
    } else {
      // coord zoom advanced down
      var key = this.key(c, source, style_z);
      CACHE[source.id] = CACHE[source.id] || {};

      if (CACHE[source.id][key] == null) {
        var span = Math.pow(2, c.z - coords.z);
        var x = coords.x * span;
        var y = coords.y * span;
        var children = [];

        for (var nx = x; nx < x + span; nx++) {
          for (var ny = y; ny < y + span; ny++) {
            var nc = this.coord({
              x: nx,
              y: ny,
              z: c.z
            });
            children.push({
              key: this.key(nc, source, style_z),
              coords: nc,
              style_z: style_z,
              source: source
            });
          }
        }

        CACHE[source.id][key] = children;
      }

      return CACHE[source.id][key];
    }
  }
};

var RasterTileSource =
/*#__PURE__*/
function (_NetworkTileSource) {
  _inheritsLoose(RasterTileSource, _NetworkTileSource);

  function RasterTileSource(source, sources) {
    var _this;

    _this = _NetworkTileSource.call(this, source, sources) || this;

    if (_this.rasters.indexOf(_this.name) === -1) {
      _this.rasters.unshift(_this.name); // add this raster as the first

    }

    _this.filtering = source.filtering; // optional texture filtering (nearest, linear, mipmap)
    // save texture objects by tile key, so URL remains stable if tile is built multiple times,
    // e.g. avoid re-loading the same tile texture under a different subdomain when using tile hosts

    _this.textures = {};
    return _this;
  }

  var _proto = RasterTileSource.prototype;

  _proto.load = function load(tile) {
    return new Promise(function ($return, $error) {
      tile.source_data = {};
      tile.source_data.layers = {};
      tile.pad_scale = this.pad_scale;
      tile.rasters = [].concat(this.rasters); // copy list of rasters to load for tile
      // Generate a single quad that fills the entire tile

      var scale = Geo$1.tile_scale;
      tile.source_data.layers = {
        _default: {
          type: 'FeatureCollection',
          features: [{
            geometry: {
              type: 'Polygon',
              coordinates: [[[0, 0], [scale, 0], [scale, -scale], [0, -scale], [0, 0]]]
            },
            properties: {}
          }]
        }
      };
      tile.default_winding = 'CW';
      return $return(tile);
    }.bind(this));
  } // Return texture info for a raster tile
  ;

  _proto.tileTexture = function tileTexture(tile) {
    return new Promise(function ($return, $error) {
      var coords = this.adjustRasterTileZoom(tile);
      var key = coords.key; // texture definitions are cached to avoid loading the same raster tile multiple times,
      // e.g. due to slightly different URLs when subdomain pattern is used (a.tile.com vs. b.tile.com)

      if (!this.textures[key]) {
        var url = this.formatURL(this.url, {
          coords: coords
        });
        this.textures[key] = {
          name: url,
          url: url,
          filtering: this.filtering,
          coords: coords
        };
      }

      return $return(this.textures[key]);
    }.bind(this));
  } // If the raster is attached to another source, we need to compare their levels of zoom detail
  // to see if any adjustments are needed. Both the `tile_size` and `zoom_offset` data source params
  // cause the zoom level to be downsampled relative to the "base" zoom level of the map view.
  // The attaching source has already applied its own zoom downsampling. If this source has a lower
  // level of detail, we apply the remaining differential here.
  ;

  _proto.adjustRasterTileZoom = function adjustRasterTileZoom(tile) {
    var coords = tile.coords;
    var tile_source = this.sources[tile.source];

    if (tile_source !== this) {
      // no-op if the raster source isn't being rendered as an attachment
      var zdiff = this.zoom_bias - tile_source.zoom_bias; // difference in zoom detail between the sources

      if (zdiff > 0) {
        // raster source is less detailed
        // do extra zoom adjustment and apply this raster source's max zoom
        coords = TileID.normalizedCoord(tile.coords, {
          zoom_bias: zdiff,
          zooms: this.zooms
        });
      } else {
        // raster source supports higher detail, but was downsampled to match (the downsampling already
        // happened upstream, when the attaching source calculated its own tile coordinate)
        if (zdiff < 0) {
          log({
            level: 'warn',
            once: true
          }, "Raster source '" + this.name + "' supports higher zoom detail than source '" + tile_source.name + "' " + ("it's attached to. Downsampling this source " + -zdiff + " extra zoom levels to match."));
        } // no extra zoom adjustment needed, but still need to apply this raster source's max zoom


        coords = TileID.coordForTileZooms(coords, this.zooms);
      }
    }

    return coords;
  };

  return RasterTileSource;
}(NetworkTileSource); // Data source for loading standalone, geo-referenced raster images
// The `bounds` parameter is used to position the raster image on the map
// Currently, only axis-aligned, rectangular North-up images are supported
// TODO: add support for arbitrarily rotated images and quadrangle control points

var RasterSource =
/*#__PURE__*/
function (_RasterTileSource) {
  _inheritsLoose(RasterSource, _RasterTileSource);

  function RasterSource(source, sources) {
    var _this2;

    _this2 = _RasterTileSource.call(this, source, sources) || this;
    _this2.load_image = {}; // resolves to image, cached for life of data source
    // alpha factor applied at time raster images are loaded and tiled (*not* at shader render-time)

    _this2.alpha = source.alpha != null ? Math.max(Math.min(source.alpha, 1), 0) : null; // non-full-alpha pixels should be discarded (for rendering rasters w/opaque blend)

    _this2.mask_alpha = true; // don't retain tiles for this source from nearby zooms (to improve memory usage)

    _this2.preserve_tiles_within_zoom = 0; // optionally set a max pixel density used for generated raster tiles (to improve memory usage)

    _this2.max_display_density = source.max_display_density; // Optionally composite multiple images into one raster layer

    if (Array.isArray(source.composite)) {
      // TODO: calculate enclosing bounding box to speed tile intersection checks
      _this2.images = source.composite.map(function (s) {
        return {
          url: s.url,
          bounds: _this2.parseBounds(s),
          alpha: s.alpha != null ? Math.max(Math.min(s.alpha, 1), 0) : null
        };
      });
    } // Single image raster layer
    else {
        _this2.images = [{
          url: _this2.url,
          bounds: _this2.bounds,
          alpha: _this2.alpha
        }];
      }

    return _this2;
  } // Render the sub-rectangle of the source raster image for the given tile, to a texture.
  // Clipping the source image to individual raster tiles naturally partitions images
  // (which may be large or only have a small portion in current view), and maintains
  // consistency with the raster tile pipeline allowing for sampling within the fragment shader,
  // and clipping the raster against vector source data.


  var _proto2 = RasterSource.prototype;

  _proto2.tileTexture = function tileTexture(tile, _ref) {
    return new Promise(function ($return, $error) {
      var _this3, blend, generation, coords, use_alpha, name, dpr, canvas, ctx, images;

      _this3 = this;
      blend = _ref.blend, generation = _ref.generation;
      coords = this.adjustRasterTileZoom(tile);
      use_alpha = blend !== 'opaque';
      name = "raster-" + this.name + "-" + coords.key + "-" + (use_alpha ? 'alpha' : 'opaque') + "-" + generation;

      // unique texture name
      // only render each raster tile once (per scene generation)
      if (Texture.textures[name]) {
        return $return({
          name: name,
          coords: coords,
          // tell style to skip re-creating this texture
          // we have an explicit flag for this because element-based (e.g. canvas) textures
          // are usually considered dynamic and always re-created when a new tile needs them
          // (because the user could have updated the canvas pixel contents outside of Tangram)
          skip_create: true
        });
      } // Display density, with extra 2x for better intra-zoom scaling, because raster tiles
      // can be scaled up to 100% before next zoom level is loaded


      dpr = Utils.device_pixel_ratio;

      if (this.max_display_density) {
        dpr = Math.min(dpr, this.max_display_density); // optionally cap pixel density
      }

      dpr *= 2;
      canvas = document.createElement('canvas');
      ctx = canvas.getContext('2d');
      canvas.width = this.tile_size * dpr; // adjusted for display density

      canvas.height = this.tile_size * dpr; // Applies nearest neighbor filtering to the canvas image rendering when data source requests it
      // NB: does not work on IE11 (image will be blurry when scaled)

      ctx.imageSmoothingEnabled = this.filtering !== 'nearest'; // Draw one or more images

      images = this.images.filter(function (r) {
        return _this3.checkBounds(tile.coords, r.bounds);
      });
      return Promise.resolve(Promise.all(images.map(function (i) {
        // TODO: log warning if alpha specified but will be ignored (in opaque mode)?
        var alpha = use_alpha ? i.alpha != null ? i.alpha : _this3.alpha : 1;
        return _this3.drawImage(i.url, i.bounds, alpha, tile, dpr, ctx);
      }))).then(function ($await_1) {
        try {
          // texture config
          return $return({
            name: name,
            element: canvas,
            filtering: this.filtering,
            coords: coords
          });
        } catch ($boundEx) {
          return $error($boundEx);
        }
      }.bind(this), $error);
    }.bind(this));
  } // Draw a single image to the tile canvas based on on its bounds
  ;

  _proto2.drawImage = function drawImage(url, bounds, alpha, tile, dpr, ctx) {
    return new Promise(function ($return, $error) {
      var key, image, mpp, dx, dy, sx, sy;
      key = hashString(url);
      // use hash of URL for shorter keys
      this.load_image[key] = this.load_image[key] || this.loadImage(url);
      return Promise.resolve(this.load_image[key]).then(function ($await_2) {
        try {
          image = $await_2;
          mpp = Geo$1.metersPerPixel(tile.coords.z) / dpr / (this.tile_size / Geo$1.tile_size);
          dx = (bounds.meters.min[0] - tile.min.x) / mpp;
          dy = -(bounds.meters.min[1] - tile.min.y) / mpp;
          sx = (bounds.meters.max[0] - bounds.meters.min[0]) / mpp;
          sy = -(bounds.meters.max[1] - bounds.meters.min[1]) / mpp;
          // Draw the raster, clipped to the current tile
          // NB: this is drawing the *whole* raster, no matter how large, and relying on the native Canvas clipping.
          // May want to benchmark with a pre-clipped draw area, though the native implementation is likely fast,
          // and has to apply its own clipping check anyway.
          ctx.globalAlpha = alpha != null ? alpha : 1;
          ctx.drawImage(image, dx, dy, sx, sy);
          return $return();
        } catch ($boundEx) {
          return $error($boundEx);
        }
      }.bind(this), $error);
    }.bind(this));
  } // Load source raster image
  ;

  _proto2.loadImage = function loadImage(url) {
    var _this4 = this;

    return new Promise(function (resolve) {
      var image = new Image();

      image.onload = function () {
        return resolve(image);
      };

      image.onerror = function (e) {
        // Warn and resolve on error
        log('warn', "Raster source '" + _this4.name + "': failed to load url: '" + url + "'", e);
        resolve(null);
      }; // Safari has a bug loading data-URL images with CORS enabled, so it must be disabled in that case
      // https://bugs.webkit.org/show_bug.cgi?id=123978


      if (!(Utils.isSafari() && url.slice(0, 5) === 'data:')) {
        image.crossOrigin = 'anonymous';
      }

      image.src = url;
    });
  } // Checks if tile interects any rasters in this source
  ;

  _proto2.includesTile = function includesTile(coords, style_z) {
    var _this5 = this;

    // Checks zoom range and dependent rasters
    if (!DataSource.prototype.includesTile.call(this, coords, style_z)) {
      return false;
    }

    return this.images.some(function (r) {
      return _this5.checkBounds(coords, r.bounds);
    }); // check if any images intersect
  };

  _proto2.validate = function validate(source) {
    var is_composite = Array.isArray(source.composite);
    var url_msg = 'Raster data source must provide a string `url` parameter, or an array of `composite` raster ';
    url_msg += 'image objects that each have a `url` parameter';
    var bounds_msg = 'Raster data source must provide a `bounds` parameter, or an array of `composite` raster ';
    bounds_msg += 'image objects that each have a `bounds` parameter';
    var mutex_msg = 'Raster data source must have *either* a single image specified as `url` and `bounds `';
    mutex_msg += 'parameters, or an array of `composite` raster image objects, each with `url` and `bounds`.';

    if (is_composite) {
      if (source.composite.some(function (s) {
        return typeof s.url !== 'string';
      })) {
        throw Error(url_msg);
      }

      if (source.composite.some(function (s) {
        return !(Array.isArray(s.bounds) && s.bounds.length === 4);
      })) {
        throw Error(bounds_msg);
      }

      if (source.url != null || source.bounds != null) {
        throw Error(mutex_msg);
      }
    } else {
      if (typeof source.url !== 'string') {
        throw Error(url_msg);
      }

      if (!(Array.isArray(source.bounds) && source.bounds.length === 4)) {
        throw Error(bounds_msg);
      }
    }
  };

  return RasterSource;
}(RasterTileSource); // Check for URL tile pattern, if not found, treat as geo-referenced raster layer

DataSource.register('Raster', function (source) {
  return RasterTileSource.urlHasTilePattern(source.url) ? RasterTileSource : RasterSource;
});

var selection_fragment_source = "// Fragment shader for feature selection passes\n// Renders in silhouette according to selection (picking) color, or black if none defined\n\n#ifdef TANGRAM_FEATURE_SELECTION\n    varying vec4 v_selection_color;\n#endif\n\nvoid main (void) {\n    #ifdef TANGRAM_FEATURE_SELECTION\n        gl_FragColor = v_selection_color;\n    #else\n        gl_FragColor = vec4(0., 0., 0., 1.);\n    #endif\n}\n";

var rasters_source = "// Uniforms defining raster textures and macros for accessing them\n\n#ifdef TANGRAM_FRAGMENT_SHADER\nuniform sampler2D u_rasters[TANGRAM_NUM_RASTER_SOURCES];    // raster tile texture samplers\nuniform vec2 u_raster_sizes[TANGRAM_NUM_RASTER_SOURCES];    // raster tile texture sizes (width/height in pixels)\nuniform vec3 u_raster_offsets[TANGRAM_NUM_RASTER_SOURCES];  // raster tile texture UV starting offset for tile\n\n// Raster sources can optionally mask by the alpha channel (render with only full or no alpha, based on a threshold),\n// which is used for handling transparency outside the raster image when rendering with opaque blending\n#if defined(TANGRAM_HAS_MASKED_RASTERS) && !defined(TANGRAM_ALL_MASKED_RASTERS) // only add uniform if we need it\nuniform bool u_raster_mask_alpha;\n#endif\n\n// Note: the raster accessors below are #defines rather than functions to\n// avoid issues with constant integer expressions for array indices\n\n// Adjusts UVs in model space to account for raster tile texture overzooming\n// (applies scale and offset adjustments)\n#define adjustRasterUV(raster_index, uv) \\\n    ((uv) * u_raster_offsets[raster_index].z + u_raster_offsets[raster_index].xy)\n\n// Returns the UVs of the current model position for a raster sampler\n#define currentRasterUV(raster_index) \\\n    (adjustRasterUV(raster_index, v_modelpos_base_zoom.xy))\n\n// Returns pixel location in raster tile texture at current model position\n#define currentRasterPixel(raster_index) \\\n    (currentRasterUV(raster_index) * rasterPixelSize(raster_index))\n\n// Samples a raster tile texture for the current model position\n#define sampleRaster(raster_index) \\\n    (texture2D(u_rasters[raster_index], currentRasterUV(raster_index)))\n\n// Samples a raster tile texture for a given pixel\n#define sampleRasterAtPixel(raster_index, pixel) \\\n    (texture2D(u_rasters[raster_index], (pixel) / rasterPixelSize(raster_index)))\n\n// Returns size of raster sampler in pixels\n#define rasterPixelSize(raster_index) \\\n    (u_raster_sizes[raster_index])\n\n#endif\n";

var Style = {
  init: function init(_temp) {
    var _ref = _temp === void 0 ? {} : _temp,
        generation = _ref.generation,
        styles = _ref.styles,
        _ref$sources = _ref.sources,
        sources = _ref$sources === void 0 ? {} : _ref$sources,
        introspection = _ref.introspection;

    this.setGeneration(generation);
    this.styles = styles; // styles for scene

    this.sources = sources; // data sources for scene

    this.defines = this.hasOwnProperty('defines') && this.defines || {}; // #defines to be injected into the shaders

    this.shaders = this.hasOwnProperty('shaders') && this.shaders || {}; // shader customization (uniforms, defines, blocks, etc.)

    this.introspection = introspection || false;
    this.selection = this.selection || this.introspection || false; // flag indicating if this style supports feature selection

    this.compile_setup = false; // one-time setup steps for program compilation

    this.program = null; // GL program reference (for main render pass)

    this.selection_program = null; // GL program reference for feature selection render pass

    this.feature_style = {}; // style for feature currently being parsed, shared to lessen GC/memory thrash

    this.vertex_template = []; // shared single-vertex template, filled out by each style

    this.tile_data = {};
    this.stencil_proxy_tiles = true; // applied to proxy tiles w/non-opaque blend mode to avoid compounding alpha
    // Default world coords to wrap every 100,000 meters, can turn off by setting this to 'false'

    this.defines.TANGRAM_WORLD_POSITION_WRAP = 100000; // Blending
    // `opaque` styles are drawn first/under other styles, without alpha blending

    this.blend = this.blend || 'opaque'; // default to opaque

    if (this.blend !== 'opaque') {
      // non-opaque styles can customize their blend order, which will determine which features
      // are drawn first/under each other
      if (this.blend_order == null) {
        this.blend_order = this.default_blend_orders[this.blend];
      }
    } else {
      // `opaque` ignores blend order, always render first/under
      this.blend_order = this.default_blend_orders[this.blend];
    }

    this.defines["TANGRAM_BLEND_" + this.blend.toUpperCase()] = true;
    this.removeShaderBlock('setup'); // clear before material injection
    // If the style defines its own material, replace the inherited material instance

    if (!(this.material instanceof Material)) {
      if (!Material.isValid(this.material)) {
        this.material = StyleParser.defaults.material;
      }

      this.material = new Material(this.material);
    }

    this.material.inject(this); // Set lighting mode: fragment, vertex, or none (specified as 'false')

    Light.setMode(this.lighting, this); // Setup raster samplers if needed

    this.setupRasters();
    this.initialized = true;
  },
  destroy: function destroy() {
    if (this.program) {
      this.program.destroy();
      this.program = null;
    }

    if (this.selection_program) {
      this.selection_program.destroy();
      this.selection_program = null;
    }

    WorkerBroker$1.removeTarget(this.main_thread_target);
    this.gl = null;
    this.initialized = false;
  },
  reset: function reset() {},
  baseStyle: function baseStyle() {
    return this.base || this.name;
  },
  setGeneration: function setGeneration(generation) {
    // Scene generation id this style was created for
    this.generation = generation; // Provide a hook for this object to be called from worker threads

    this.main_thread_target = ['Style', this.name, this.generation].join('_');

    if (Thread.is_main) {
      WorkerBroker$1.addTarget(this.main_thread_target, this);
    }
  },

  /*** Style parsing and geometry construction ***/
  // Returns an object to hold feature data (for a tile or other object)
  startData: function startData(tile) {
    this.tile_data[tile.id] = this.tile_data[tile.id] || {
      meshes: {},
      uniforms: {},
      textures: []
    };
  },
  // Finalizes an object holding feature data (for a tile or other object)
  endData: function endData(tile) {
    return new Promise(function ($return, $error) {
      var tile_data = this.tile_data[tile.id];
      this.tile_data[tile.id] = null;

      if (tile_data && Object.keys(tile_data.meshes).length > 0) {
        for (var variant in tile_data.meshes) {
          var mesh = tile_data.meshes[variant]; // Remove empty mesh variants

          if (mesh.vertex_data.vertex_count === 0) {
            delete tile_data.meshes[variant];
            continue;
          } // Only keep final byte buffer


          mesh.vertex_data.end();
          mesh.vertex_elements = mesh.vertex_data.element_buffer;
          mesh.vertex_data = mesh.vertex_data.vertex_buffer; // convert from instance to raw typed array
        } // Load raster tiles passed from data source
        // Blocks mesh completion to avoid flickering


        return $return(this.buildRasterTextures(tile, tile_data));
      } else {
        return $return(null); // don't send tile data back if doesn't have geometry
      }

      return $return();
    }.bind(this));
  },
  // Has mesh data for a given tile?
  hasDataForTile: function hasDataForTile(tile) {
    return this.tile_data[tile.id] != null;
  },
  getTileMesh: function getTileMesh(tile, variant) {
    var meshes = this.tile_data[tile.id].meshes;

    if (meshes[variant.key] == null) {
      var vertex_layout = this.vertexLayoutForMeshVariant(variant);
      meshes[variant.key] = {
        variant: variant,
        vertex_data: vertex_layout.createVertexData()
      };
    }

    return meshes[variant.key];
  },
  vertexLayoutForMeshVariant: function vertexLayoutForMeshVariant()
  /*variant*/
  {// styles must implement
  },
  meshVariantTypeForDraw: function meshVariantTypeForDraw()
  /*draw*/
  {// styles must implement
  },
  addFeature: function addFeature(feature, draw, context) {
    var tile = context.tile;

    if (tile.generation !== this.generation) {
      return;
    }

    if (!this.tile_data[tile.id]) {
      this.startData(tile);
    }

    var style = this.parseFeature(feature, draw, context);

    if (!style) {
      return; // skip feature
    }

    if (this.buildGeometry(feature.geometry, style, context) > 0) {
      feature.generation = this.generation; // track scene generation that feature was rendered for
    }
  },
  buildGeometry: function buildGeometry(geometry, style, context) {
    var _this = this;

    var geom_count;

    if (geometry.type === 'Polygon') {
      geom_count = this.buildPolygons([geometry.coordinates], style, context);
    } else if (geometry.type === 'MultiPolygon') {
      geom_count = this.buildPolygons(geometry.coordinates, style, context);
    } else if (geometry.type === 'LineString') {
      geom_count = this.buildLines([geometry.coordinates], style, context);
    } else if (geometry.type === 'MultiLineString') {
      geom_count = this.buildLines(geometry.coordinates, style, context);
    } else if (geometry.type === 'Point') {
      geom_count = this.buildPoints([geometry.coordinates], style, context);
    } else if (geometry.type === 'MultiPoint') {
      geom_count = this.buildPoints(geometry.coordinates, style, context);
    } // Optionally collect per-layer stats


    if (geom_count > 0 && debugSettings$1.layer_stats) {
      var tile = context.tile;
      tile.debug.layers = tile.debug.layers || {
        list: {},
        tree: {}
      };
      var list = tile.debug.layers.list;
      var tree = tile.debug.layers.tree;
      var ftree = {}; // tree of layers for this feature

      context.layers.forEach(function (layer) {
        var _addLayerDebugEntry, _addLayerDebugEntry2;

        addLayerDebugEntry(list, layer, 1, geom_count, (_addLayerDebugEntry = {}, _addLayerDebugEntry[_this.name] = geom_count, _addLayerDebugEntry), (_addLayerDebugEntry2 = {}, _addLayerDebugEntry2[_this.baseStyle()] = geom_count, _addLayerDebugEntry2));
        var node = tree;
        var fnode = ftree;
        var levels = layer.split(':');

        for (var i = 0; i < levels.length; i++) {
          var level = levels[i];
          node[level] = node[level] || {
            features: 0,
            geoms: 0,
            styles: {},
            base: {}
          };

          if (fnode[level] == null) {
            var _addLayerDebugEntry3, _addLayerDebugEntry4;

            // only count each layer level once per feature
            fnode[level] = {};
            addLayerDebugEntry(node, level, 1, geom_count, (_addLayerDebugEntry3 = {}, _addLayerDebugEntry3[_this.name] = geom_count, _addLayerDebugEntry3), (_addLayerDebugEntry4 = {}, _addLayerDebugEntry4[_this.baseStyle()] = geom_count, _addLayerDebugEntry4));
          }

          if (i < levels.length - 1) {
            node[level].layers = node[level].layers || {};
          }

          node = node[level].layers;
          fnode = fnode[level];
        }
      });
    }

    return geom_count;
  },
  parseFeature: function parseFeature(feature, draw, context) {
    try {
      var style = this.feature_style; // Calculate order

      style.order = this.parseOrder(draw.order, context);

      if (style.order == null && this.blend !== 'overlay') {
        var msg = "Layer '" + draw.layers.join(', ') + "', draw group '" + draw.group + "': ";
        msg += '\'order\' parameter is required unless blend mode is \'overlay\'';

        if (draw.order != null) {
          msg += '; \'order\' was set to a dynamic value (e.g. string tied to feature property, ';
          msg += 'or JS function), but evaluated to null for one or more features';
        }

        log({
          level: 'warn',
          once: true
        }, msg);
        return;
      } // Subclass implementation


      style = this._parseFeature(feature, draw, context);

      if (!style) {
        return; // skip feature
      } // Feature selection (only if feature is marked as interactive, and style supports it)


      if (this.selection) {
        style.interactive = StyleParser.evalProperty(draw.interactive, context);
      } else {
        style.interactive = false;
      }

      if (style.interactive === true) {
        style.selection_color = FeatureSelection.makeColor(feature, context.tile, context);
      } else {
        style.selection_color = FeatureSelection.defaultColor;
      }

      return style;
    } catch (error) {
      log('error', 'Style.parseFeature: style parsing error', feature, style, error.stack);
    }
  },
  _parseFeature: function _parseFeature()
  /*feature, draw, context*/
  {
    return this.feature_style;
  },
  preprocess: function preprocess(draw) {
    // Preprocess first time
    if (!draw.preprocessed) {
      // Apply draw defaults
      if (this.draw) {
        // Merge each property separately to avoid modifying `draw` instance identity
        for (var param in this.draw) {
          var val = this.draw[param];

          if (typeof val === 'object' && !Array.isArray(val)) {
            // nested param (e.g. `outline`)
            draw[param] = mergeObjects({}, val, draw[param]);
          } else if (draw[param] == null) {
            // simple param (single scalar value or array)
            draw[param] = val;
          }
        }
      }

      if (!this.selection) {
        draw.interactive = false; // always disable feature selection for when style doesn't support it
      } else if (this.introspection) {
        draw.interactive = true; // always enable feature selection for introspection
      }

      draw = this._preprocess(draw); // optional subclass implementation

      if (!draw) {
        return;
      }

      draw.preprocessed = true;
    }

    return draw;
  },
  // optionally implemented by subclass
  _preprocess: function _preprocess(draw) {
    return draw;
  },
  // Parse an order value
  parseOrder: function parseOrder(order, context) {
    // Calculate order if it was not cached
    if (typeof order !== 'number') {
      return StyleParser.calculateOrder(order, context);
    }

    return order;
  },
  // Expand final precision for half-layers (for outlines)
  scaleOrder: function scaleOrder(order) {
    return order * 2;
  },
  // Parse a color of choose a default if acceptable, return undefined if color missing
  parseColor: function parseColor(color, context) {
    // Need either a color, or a shader block for 'color' or 'filter'
    if (color) {
      return StyleParser.evalCachedColorProperty(color, context);
    } else if (this.shaders.blocks.color || this.shaders.blocks.filter) {
      return StyleParser.defaults.color;
    }
  },
  // Build functions are no-ops until overriden
  buildPolygons: function buildPolygons() {
    return 0;
  },
  buildLines: function buildLines() {
    return 0;
  },
  buildPoints: function buildPoints() {
    return 0;
  },

  /*** GL state and rendering ***/
  setGL: function setGL(gl) {
    this.gl = gl;
    this.max_texture_size = Texture.getMaxTextureSize(this.gl);
  },
  makeMesh: function makeMesh(vertex_data, vertex_elements, options) {
    if (options === void 0) {
      options = {};
    }

    var vertex_layout = this.vertexLayoutForMeshVariant(options.variant);
    return new VBOMesh(this.gl, vertex_data, vertex_elements, vertex_layout, options);
  },
  render: function render(mesh) {
    return mesh.render();
  },
  // Get a specific program, compiling if necessary
  getProgram: function getProgram(key) {
    if (key === void 0) {
      key = 'program';
    }

    this.compileSetup();
    var program = this[key];

    if (!program || program.error) {
      return;
    }

    if (!program.compiled) {
      log('debug', "Compiling style '" + this.name + "', program key '" + key + "'");

      try {
        program.compile();
      } catch (e) {
        log('error', "Style: error compiling program for style '" + this.name + "' (program key '" + key + "')", this, e.stack, e.type, e.shader_errors);
      }
    }

    return program;
  },
  // One-time setup for compiling style's programs
  compileSetup: function compileSetup() {
    if (this.compile_setup) {
      return;
    }

    if (!this.gl) {
      throw new Error("style.compile(): skipping for " + this.name + " because no GL context");
    } // Build defines & for selection (need to create a new object since the first is stored as a reference by the program)


    var defines = this.buildDefineList();

    if (this.selection) {
      var selection_defines = Object.assign({}, defines);
      selection_defines.TANGRAM_FEATURE_SELECTION = true;
    } // Shader blocks


    var blocks = this.shaders && this.shaders.blocks;
    var block_scopes = this.shaders && this.shaders.block_scopes; // Uniforms

    var uniforms = Object.assign({}, this.shaders && this.shaders.uniforms);

    for (var u in uniforms) {
      // validate uniforms
      if (uniforms[u] == null) {
        log({
          level: 'warn',
          once: true
        }, "Style '" + this.name + "' has invalid uniform '" + u + "': uniform values must be non-null");
      }
    } // Accept a single extension, or an array of extensions


    var extensions = this.shaders && this.shaders.extensions;

    if (typeof extensions === 'string') {
      extensions = [extensions];
    } // Create shaders


    this.program = new ShaderProgram(this.gl, this.vertex_shader_src, this.fragment_shader_src, {
      name: this.name,
      defines: defines,
      uniforms: uniforms,
      blocks: blocks,
      block_scopes: block_scopes,
      extensions: extensions
    });

    if (this.selection) {
      this.selection_program = new ShaderProgram(this.gl, this.vertex_shader_src, selection_fragment_source, {
        name: this.name + ' (selection)',
        defines: selection_defines,
        uniforms: uniforms,
        blocks: blocks,
        block_scopes: block_scopes,
        extensions: extensions
      });
    } else {
      this.selection_program = null;
    }

    this.compile_setup = true;
  },
  // Add a shader block
  addShaderBlock: function addShaderBlock(key, block, scope) {
    if (scope === void 0) {
      scope = null;
    }

    this.shaders.blocks = this.shaders.blocks || {};
    this.shaders.blocks[key] = this.shaders.blocks[key] || [];
    this.shaders.blocks[key].push(block);
    this.shaders.block_scopes = this.shaders.block_scopes || {};
    this.shaders.block_scopes[key] = this.shaders.block_scopes[key] || [];
    this.shaders.block_scopes[key].push(scope);
  },
  // Remove all shader blocks for key
  removeShaderBlock: function removeShaderBlock(key) {
    if (this.shaders.blocks) {
      this.shaders.blocks[key] = null;
    }

    if (this.shaders.block_scopes) {
      this.shaders.block_scopes[key] = null;
    }
  },
  replaceShaderBlock: function replaceShaderBlock(key, block, scope) {
    if (scope === void 0) {
      scope = null;
    }

    this.removeShaderBlock(key);
    this.addShaderBlock(key, block, scope);
  },

  /** TODO: could probably combine and generalize this with similar method in ShaderProgram
   * (list of define objects that inherit from each other)
   */
  buildDefineList: function buildDefineList() {
    // Add any custom defines to built-in style defines
    var defines = {}; // create a new object to avoid mutating a prototype value that may be shared with other styles

    if (this.defines != null) {
      for (var d in this.defines) {
        defines[d] = this.defines[d];
      }
    }

    if (this.shaders != null && this.shaders.defines != null) {
      for (d in this.shaders.defines) {
        defines[d] = this.shaders.defines[d];
      }
    }

    return defines;
  },
  // Determines if 'raster' parameter is set to a valid value
  hasRasters: function hasRasters() {
    return ['color', 'normal', 'custom'].indexOf(this.raster) > -1;
  },
  // Setup raster access in shaders
  setupRasters: function setupRasters() {
    var _this2 = this;

    if (!this.hasRasters()) {
      return;
    } // Enable raster textures and configure how first raster is applied


    if (this.raster === 'color') {
      this.defines.TANGRAM_RASTER_TEXTURE_COLOR = true;
    } else if (this.raster === 'normal') {
      this.defines.TANGRAM_RASTER_TEXTURE_NORMAL = true;
    } // else custom raster (samplers will be made available but not automatically applied)
    // A given style may be built with multiple data sources, each of which may attach
    // a variable number of raster sources (0 to N, where N is the max number of raster sources
    // defined for the scene). This means we don't know *which* or *how many* rasters will be
    // bound now, at initial compile-time; we only know this at geometry build-time. To ensure
    // that we can bind as many raster sources as needed, we declare our uniform arrays to hold
    // the maximum number of possible sources. At render time, only the necessary number of rasters
    // are bound (the remaining slots aren't intended to be accessed).


    var num_raster_sources = Object.keys(this.sources).filter(function (s) {
      return _this2.sources[s] instanceof RasterTileSource;
    }).length;
    this.defines.TANGRAM_NUM_RASTER_SOURCES = "" + num_raster_sources; // force to string to avoid auto-float conversion

    if (num_raster_sources > 0) {
      // Track how many raster sources have alpha masking (used for handling transparency outside raster image)
      var num_masked_rasters = Object.keys(this.sources).filter(function (s) {
        return _this2.sources[s].mask_alpha;
      }).length;
      this.defines.TANGRAM_HAS_MASKED_RASTERS = num_masked_rasters > 0;
      this.defines.TANGRAM_ALL_MASKED_RASTERS = num_masked_rasters === num_raster_sources; // Use model position of tile's coordinate zoom for raster tile texture UVs

      this.defines.TANGRAM_MODEL_POSITION_BASE_ZOOM_VARYING = true; // Uniforms and samplers

      this.replaceShaderBlock('raster', rasters_source, 'Raster');
    }
  },
  // Load raster tile textures and set uniforms
  buildRasterTextures: function buildRasterTextures(tile, tile_data) {
    return new Promise(function ($return, $error) {
      var textures, u_samplers, u_sizes, u_offsets;

      // skip if style doesn't support rasters
      if (!this.hasRasters()) {
        return $return(tile_data);
      } // skip if source didn't attach any rasters to tile


      if (tile.rasters.length === 0) {
        return $return(tile_data);
      } // Load textures on main thread and return when done
      // We want to block the building of a raster tile mesh until its texture is loaded,
      // to avoid flickering while loading (texture will render as black)


      var $Try_3_Post = function () {
        try {
          if (!textures || textures.length < 1) {
            // no textures found (unexpected)
            // TODO: warning
            return $return(tile_data);
          } else if (textures.some(function (t) {
            return !t.loaded;
          })) {
            // some textures failed, throw out style for this tile
            return $return(null);
          } // Enable alpha masking if needed (for transparency outside raster image, on first raster only)


          tile_data.uniforms['u_raster_mask_alpha'] = this.sources[tile.rasters[0]].mask_alpha === true; // Set texture uniforms (returned after loading from main thread)

          u_samplers = tile_data.uniforms['u_rasters'] = [];
          u_sizes = tile_data.uniforms['u_raster_sizes'] = [];
          u_offsets = tile_data.uniforms['u_raster_offsets'] = [];
          textures.forEach(function (t) {
            var i = t.index;
            u_samplers[i] = t.name;
            tile_data.textures.push(t.name);
            u_sizes[i] = [t.width, t.height]; // Tile geometry may be at a higher zoom than the raster tile texture,
            // (e.g. an overzoomed raster tile), in which case we need to adjust the
            // raster texture UVs to offset to the appropriate starting point for
            // this geometry tile.

            if (tile.coords.z > t.coords.z) {
              var dz = tile.coords.z - t.coords.z; // # of levels raster source is overzoomed

              var dz2 = Math.pow(2, dz);
              u_offsets[i] = [(tile.coords.x % dz2 + dz2) % dz2 / dz2, // double-modulo to handle negative (wrapped) tile coords
              (dz2 - 1 - tile.coords.y % dz2) / dz2, // GL texture coords are +Y up
              1 / dz2];
            } else {
              u_offsets[i] = [0, 0, 1];
            }
          });
          return $return(tile_data);
        } catch ($boundEx) {
          return $error($boundEx);
        }
      }.bind(this);

      var $Try_3_Catch = function (e) {
        try {
          // error thrown if style has been removed from main thread
          return $return(tile_data);
        } catch ($boundEx) {
          return $error($boundEx);
        }
      };

      try {
        return Promise.resolve(WorkerBroker$1.postMessage(this.main_thread_target + ".loadTextures", {
          coords: tile.coords,
          source: tile.source,
          rasters: tile.rasters,
          min: tile.min,
          max: tile.max
        })).then(function ($await_4) {
          try {
            textures = $await_4;
            return $Try_3_Post();
          } catch ($boundEx) {
            return $Try_3_Catch($boundEx);
          }
        }, $Try_3_Catch);
      } catch (e) {
        $Try_3_Catch(e);
      }
    }.bind(this));
  },
  // Determine which raster tile textures need to load for this tile, load them and return metadata to worker
  // Called on main thread
  loadTextures: function loadTextures(tile) {
    return new Promise(function ($return, $error) {
      var _this3, configs, index, queue, textures;

      _this3 = this;
      configs = {};
      index = {};
      queue = [];
      // Find raster textures that need to be loaded
      tile.rasters.map(function (r) {
        return _this3.sources[r];
      }).filter(function (x) {
        return x;
      }).forEach(function (source, i) {
        if (source instanceof RasterTileSource) {
          queue.push(source.tileTexture(tile, _this3).then(function (config) {
            configs[config.name] = config;
            index[config.name] = i;
          }));
        }
      });
      return Promise.resolve(Promise.all(queue)).then(function ($await_5) {
        try {
          return Promise.resolve(Texture.createFromObject(this.gl, configs)).then(function ($await_6) {
            try {
              return Promise.resolve(Promise.all(Object.keys(configs).map(function (t) {
                return Texture.textures[t] && Texture.textures[t].load();
              }).filter(function (x) {
                return x;
              }))).then(function ($await_7) {
                try {
                  textures = $await_7;
                  textures.forEach(function (t) {
                    return t.retain();
                  }); // Take a subset of texture metadata, and decorate with raster-specific info
                  // NB: only return name and size of textures loaded, because we can't send actual texture objects to worker

                  return $return(textures.map(function (t) {
                    return {
                      name: t.name,
                      width: t.width,
                      height: t.height,
                      loaded: t.loaded,
                      index: index[t.name],
                      // raster sampler index
                      coords: configs[t.name].coords // tile coords of raster tile

                    };
                  }));
                } catch ($boundEx) {
                  return $error($boundEx);
                }
              }, $error);
            } catch ($boundEx) {
              return $error($boundEx);
            }
          }, $error);
        } catch ($boundEx) {
          return $error($boundEx);
        }
      }.bind(this), $error);
    }.bind(this));
  },
  // Setup any GL state for rendering
  setup: function setup() {
    this.setUniforms();
    this.material.setupProgram(ShaderProgram.current);
  },
  // Set style uniforms on currently bound program
  setUniforms: function setUniforms() {
    var program = ShaderProgram.current;

    if (!program) {
      return;
    }

    program.setUniforms(this.shaders && this.shaders.uniforms, true); // reset texture unit to 0
  },
  // Render state settings by blend mode
  render_states: {
    opaque: {
      depth_test: true,
      depth_write: true
    },
    translucent: {
      depth_test: true,
      depth_write: true
    },
    add: {
      depth_test: true,
      depth_write: false
    },
    multiply: {
      depth_test: true,
      depth_write: false
    },
    inlay: {
      depth_test: true,
      depth_write: false
    },
    overlay: {
      depth_test: false,
      depth_write: false
    }
  },
  // Default sort order for blend modes
  default_blend_orders: {
    opaque: Number.MIN_SAFE_INTEGER,
    add: 1,
    multiply: 2,
    inlay: 3,
    translucent: 4,
    overlay: 5
  },
  getBlendOrderForDraw: function getBlendOrderForDraw(draw) {
    // Allow draw block to override blend_order for non-opaque blend styles
    return this.blend !== 'opaque' && draw.blend_order != null ? draw.blend_order : this.blend_order;
  }
}; // add feature and geometry counts for a single layer

function addLayerDebugEntry(target, layer, faeture_count, geom_count, styles, bases) {
  target[layer] = target[layer] || {
    features: 0,
    geoms: 0,
    styles: {},
    base: {}
  };
  target[layer].features += faeture_count; // feature count

  target[layer].geoms += geom_count; // geometry count
  // geometry count by style

  for (var style in styles) {
    target[layer].styles[style] = target[layer].styles[style] || 0;
    target[layer].styles[style] += styles[style];
  } // geometry count by base style


  for (var _style in bases) {
    target[layer].base[_style] = target[layer].base[_style] || 0;
    target[layer].base[_style] += bases[_style];
  }
}

// WebGL constants - need to import these separately to make them available in the web worker
var gl;
var gl$1 = gl = {};
/* DataType */

gl.BYTE = 0x1400;
gl.UNSIGNED_BYTE = 0x1401;
gl.SHORT = 0x1402;
gl.UNSIGNED_SHORT = 0x1403;
gl.INT = 0x1404;
gl.UNSIGNED_INT = 0x1405;
gl.FLOAT = 0x1406;

_typedArray('Int16', 2, function (init) {
  return function Int16Array(data, byteOffset, length) {
    return init(this, data, byteOffset, length);
  };
});

_typedArray('Uint32', 4, function (init) {
  return function Uint32Array(data, byteOffset, length) {
    return init(this, data, byteOffset, length);
  };
});

_typedArray('Int32', 4, function (init) {
  return function Int32Array(data, byteOffset, length) {
    return init(this, data, byteOffset, length);
  };
});

_typedArray('Int8', 1, function (init) {
  return function Int8Array(data, byteOffset, length) {
    return init(this, data, byteOffset, length);
  };
});

_typedArray('Float32', 4, function (init) {
  return function Float32Array(data, byteOffset, length) {
    return init(this, data, byteOffset, length);
  };
});

var MAX_VALUE = Math.pow(2, 16) - 1;
var has_element_index_uint = false;

var VertexElements =
/*#__PURE__*/
function () {
  function VertexElements() {
    this.array = [];
    this.has_overflown = false;
  }

  var _proto = VertexElements.prototype;

  _proto.push = function push(value) {
    // If values have overflown and no Uint32 option is available, do not push values
    if (this.has_overflown && !has_element_index_uint) {
      return;
    } // Trigger overflow if value is greater than Uint16 max


    if (value > MAX_VALUE) {
      this.has_overflown = true;

      if (!has_element_index_uint) {
        return;
      }
    }

    this.array.push(value);
  };

  _proto.end = function end() {
    if (this.array.length) {
      var buffer = createBuffer(this.array, this.has_overflown);
      this.array = [];
      this.has_overflown = false;
      return buffer;
    } else {
      return false;
    }
  };

  return VertexElements;
}();

VertexElements.setElementIndexUint = function (flag) {
  has_element_index_uint = flag;
};

function createBuffer(array, overflown) {
  var typedArray = overflown && has_element_index_uint ? Uint32Array : Uint16Array;
  return new typedArray(array);
}

var _array_types;

var array_types = (_array_types = {}, _array_types[gl$1.FLOAT] = Float32Array, _array_types[gl$1.BYTE] = Int8Array, _array_types[gl$1.UNSIGNED_BYTE] = Uint8Array, _array_types[gl$1.INT] = Int32Array, _array_types[gl$1.UNSIGNED_INT] = Uint32Array, _array_types[gl$1.SHORT] = Int16Array, _array_types[gl$1.UNSIGNED_SHORT] = Uint16Array, _array_types); // An intermediary object that holds vertex data in typed arrays, according to a given vertex layout
// Used to construct a mesh/VBO for rendering

var VertexData =
/*#__PURE__*/
function () {
  function VertexData(vertex_layout, _temp) {
    var _ref = _temp === void 0 ? {} : _temp,
        _ref$prealloc = _ref.prealloc,
        prealloc = _ref$prealloc === void 0 ? 500 : _ref$prealloc;

    this.vertex_layout = vertex_layout;
    this.vertex_elements = new VertexElements();
    this.stride = this.vertex_layout.stride;

    if (VertexData.array_pool.length > 0) {
      this.vertex_buffer = VertexData.array_pool.pop();
      this.byte_length = this.vertex_buffer.byteLength;
      this.size = Math.floor(this.byte_length / this.stride);
      log('trace', "VertexData: reused buffer of bytes " + this.byte_length + ", " + this.size + " vertices");
    } else {
      this.size = prealloc; // # of vertices to allocate

      this.byte_length = this.stride * this.size;
      this.vertex_buffer = new Uint8Array(this.byte_length);
    }

    this.offset = 0; // byte offset into currently allocated buffer

    this.vertex_count = 0;
    this.realloc_count = 0;
    this.setBufferViews();
    this.setAddVertexFunction();
  } // (Re-)allocate typed views into the main buffer - only create the types we need for this layout


  var _proto = VertexData.prototype;

  _proto.setBufferViews = function setBufferViews() {
    var _this = this;

    this.views = {};
    this.views[gl$1.UNSIGNED_BYTE] = this.vertex_buffer;
    this.vertex_layout.dynamic_attribs.forEach(function (attrib) {
      // Need view for this type?
      if (_this.views[attrib.type] == null) {
        var array_type = array_types[attrib.type];
        _this.views[attrib.type] = new array_type(_this.vertex_buffer.buffer);
      }
    });
  } // Check allocated buffer size, expand/realloc buffer if needed
  ;

  _proto.checkBufferSize = function checkBufferSize() {
    if (this.offset + this.stride > this.byte_length) {
      this.size = Math.floor(this.size * 1.5);
      this.size -= this.size % 4;
      this.byte_length = this.stride * this.size;
      var new_view = new Uint8Array(this.byte_length);
      new_view.set(this.vertex_buffer); // copy existing data to new buffer

      VertexData.array_pool.push(this.vertex_buffer); // save previous buffer for use by next tile

      this.vertex_buffer = new_view;
      this.setBufferViews();
      this.realloc_count++; // log('info', `VertexData: expanded vertex block to ${this.size} vertices`);
    }
  } // Initialize the add vertex function (lazily compiled by vertex layout)
  ;

  _proto.setAddVertexFunction = function setAddVertexFunction() {
    this.vertexLayoutAddVertex = this.vertex_layout.getAddVertexFunction();
  } // Add a vertex, copied from a plain JS array of elements matching the order of the vertex layout
  ;

  _proto.addVertex = function addVertex(vertex) {
    this.checkBufferSize();
    this.vertexLayoutAddVertex(vertex, this.views, this.offset);
    this.offset += this.stride;
    this.vertex_count++;
  } // Finalize vertex buffer for use in constructing a mesh
  ;

  _proto.end = function end() {
    // Clip the buffer to size used for this VBO
    this.vertex_buffer = this.vertex_buffer.subarray(0, this.offset);
    this.element_buffer = this.vertex_elements.end();
    log('trace', "VertexData: " + this.size + " vertices total, realloc count " + this.realloc_count);
    return this;
  };

  return VertexData;
}();
VertexData.array_pool = []; // pool of currently available (previously used) buffers (uint8)

var VertexLayout =
/*#__PURE__*/
function () {
  // Attribs are an array, in layout order, of: name, size, type, normalized
  // ex: { name: 'position', size: 3, type: gl.FLOAT, normalized: false }
  function VertexLayout(attribs) {
    this.attribs = attribs; // array of attributes, specified as standard GL attrib options

    this.dynamic_attribs = this.attribs.filter(function (x) {
      return x.static == null;
    }); // attributes with per-vertex values, used to build VBOs

    this.static_attribs = this.attribs.filter(function (x) {
      return x.static != null;
    }); // attributes with fixed values

    this.components = []; // list of type and offset info about each attribute component

    this.index = {}; // JS buffer index of each attribute component, e.g. this.index.position

    this.offset = {}; // VBO buffer byte offset of each attribute component, e.g. this.offset.color

    this.stride = 0; // byte stride of a single vertex

    var index = 0,
        count = 0;

    for (var a = 0; a < this.attribs.length; a++) {
      var attrib = this.attribs[a]; // Dynamic attribute

      if (attrib.static == null) {
        attrib.offset = this.stride;
        attrib.byte_size = attrib.size;
        var shift = 0;

        switch (attrib.type) {
          case gl$1.FLOAT:
          case gl$1.INT:
          case gl$1.UNSIGNED_INT:
            attrib.byte_size *= 4;
            shift = 2;
            break;

          case gl$1.SHORT:
          case gl$1.UNSIGNED_SHORT:
            attrib.byte_size *= 2;
            shift = 1;
            break;
        } // Force 4-byte alignment on attributes


        if (attrib.byte_size & 3) {
          // pad to multiple of 4 bytes
          attrib.byte_size += 4 - (attrib.byte_size & 3);
        }

        this.stride += attrib.byte_size; // Add info to list of attribute components (e.g. float is 1 component, vec3 is 3 separate components)
        // Used to map plain JS array to typed arrays

        var offset_typed = attrib.offset >> shift;

        for (var s = 0; s < attrib.size; s++) {
          this.components.push({
            type: attrib.type,
            shift: shift,
            offset: offset_typed++,
            index: count++
          });
        } // Provide an index into the vertex data buffer for each attribute by name


        this.index[attrib.name] = index;
        index += attrib.size; // Store byte offset of each attribute by name

        this.offset[attrib.name] = attrib.offset;
      } // Static attribute
      else {
          attrib.static = Array.isArray(attrib.static) ? attrib.static : [attrib.static]; // convert single value to array

          attrib.method = "vertexAttrib" + attrib.static.length + "fv";
        }
    }
  } // Enables dynamic (array-based) attributes for a given GL program
  // Assumes that the desired vertex buffer (VBO) is already bound
  // If the program doesn't include all attributes, it can still use the vertex layout
  // to read those attribs that it does recognize, using the attrib offsets to skip others.


  var _proto = VertexLayout.prototype;

  _proto.enableDynamicAttributes = function enableDynamicAttributes(gl, program) {
    var _this = this;

    // Disable all attributes
    for (var location in VertexLayout.enabled_attribs) {
      gl.disableVertexAttribArray(location);
    }

    VertexLayout.enabled_attribs = {}; // Enable dynamic attributes for this layout

    this.dynamic_attribs.forEach(function (attrib) {
      var location = program.attribute(attrib.name).location;

      if (location !== -1) {
        gl.enableVertexAttribArray(location);
        gl.vertexAttribPointer(location, attrib.size, attrib.type, attrib.normalized, _this.stride, attrib.offset);
        VertexLayout.enabled_attribs[location] = program;
      }
    });
  } // Enable static attributes for this layout. Since these aren't captured as part of Vertex Array Object state,
  // they are enabled separately.
  ;

  _proto.enableStaticAttributes = function enableStaticAttributes(gl, program) {
    this.static_attribs.forEach(function (attrib) {
      var location = program.attribute(attrib.name).location;

      if (location !== -1 && gl[attrib.method] instanceof Function) {
        // N.B.: Safari appears to require an explicit array enable to set vertex attribute as "active"
        // (the static attribute value method does not work without it). So the attribute is temporarily
        // enabled as an array, then disabled.
        gl.enableVertexAttribArray(location);
        gl[attrib.method](location, attrib.static);
        gl.disableVertexAttribArray(location);
      }
    });
  };

  _proto.createVertexData = function createVertexData() {
    return new VertexData(this);
  } // Lazily create the add vertex function
  ;

  _proto.getAddVertexFunction = function getAddVertexFunction() {
    if (this.addVertex == null) {
      this.createAddVertexFunction();
    }

    return this.addVertex;
  } // Dynamically compile a function to add a plain JS vertex array to this layout's typed VBO arrays
  ;

  _proto.createAddVertexFunction = function createAddVertexFunction() {
    var key = hashString(JSON.stringify(this.attribs));

    if (VertexLayout.add_vertex_funcs[key] == null) {
      // `t` = current typed array to write to
      // `o` = current offset into VBO, in current type size (e.g. divide 2 for shorts, divide by 4 for floats, etc.)
      // `v` = plain JS array containing vertex data
      // `vs` = typed arrays (one per GL type needed for this vertex layout)
      // `off` = current offset into VBO, in bytes
      var src = ['var t, o;']; // Sort by array type to reduce redundant array look-up and offset calculation

      var last_type;
      var components = [].concat(this.components);
      components.sort(function (a, b) {
        return a.type !== b.type ? a.type - b.type : a.index - b.index;
      });

      for (var c = 0; c < components.length; c++) {
        var component = components[c];

        if (last_type !== component.type) {
          src.push("t = vs[" + component.type + "];");
          src.push("o = off" + (component.shift ? ' >> ' + component.shift : '') + ";");
          last_type = component.type;
        }

        src.push("t[o + " + component.offset + "] = v[" + component.index + "];");
      }

      src = src.join('\n');
      var func = new Function('v', 'vs', 'off', src); // jshint ignore:line

      VertexLayout.add_vertex_funcs[key] = func;
    }

    this.addVertex = VertexLayout.add_vertex_funcs[key];
  };

  return VertexLayout;
}(); // Track currently enabled attribs, by the program they are bound to
VertexLayout.enabled_attribs = {}; // Functions to add plain JS vertex array to typed VBO arrays

VertexLayout.add_vertex_funcs = {}; // keyed by unique set of attributes

// Geometry building functions
var tile_bounds = [{
  x: 0,
  y: 0
}, {
  x: Geo$1.tile_scale,
  y: -Geo$1.tile_scale // TODO: correct for flipped y-axis?

}];
var default_uvs = [0, 0, 1, 1]; // Tests if a line segment (from point A to B) is outside the tile bounds
// (within a certain tolerance to account for geometry nearly on tile edges)

function outsideTile(_a, _b, tolerance) {
  var tile_min = tile_bounds[0];
  var tile_max = tile_bounds[1]; // TODO: fix flipped Y coords here, confusing with 'max' reference

  if (_a[0] <= tile_min.x + tolerance && _b[0] <= tile_min.x + tolerance || _a[0] >= tile_max.x - tolerance && _b[0] >= tile_max.x - tolerance || _a[1] >= tile_min.y - tolerance && _b[1] >= tile_min.y - tolerance || _a[1] <= tile_max.y + tolerance && _b[1] <= tile_max.y + tolerance) {
    return true;
  }

  return false;
}
function isCoordOutsideTile(coord, tolerance) {
  tolerance = tolerance || 0;
  var tile_min = tile_bounds[0];
  var tile_max = tile_bounds[1];
  return coord[0] <= tile_min.x + tolerance || coord[0] >= tile_max.x - tolerance || coord[1] >= tile_min.y - tolerance || coord[1] <= tile_max.y + tolerance;
}

var earcut_1 = earcut;

function earcut(data, holeIndices, dim) {

    dim = dim || 2;

    var hasHoles = holeIndices && holeIndices.length,
        outerLen = hasHoles ? holeIndices[0] * dim : data.length,
        outerNode = linkedList(data, 0, outerLen, dim, true),
        triangles = [];

    if (!outerNode) return triangles;

    var minX, minY, maxX, maxY, x, y, size;

    if (hasHoles) outerNode = eliminateHoles(data, holeIndices, outerNode, dim);

    // if the shape is not too simple, we'll use z-order curve hash later; calculate polygon bbox
    if (data.length > 80 * dim) {
        minX = maxX = data[0];
        minY = maxY = data[1];

        for (var i = dim; i < outerLen; i += dim) {
            x = data[i];
            y = data[i + 1];
            if (x < minX) minX = x;
            if (y < minY) minY = y;
            if (x > maxX) maxX = x;
            if (y > maxY) maxY = y;
        }

        // minX, minY and size are later used to transform coords into integers for z-order calculation
        size = Math.max(maxX - minX, maxY - minY);
    }

    earcutLinked(outerNode, triangles, dim, minX, minY, size);

    return triangles;
}

// create a circular doubly linked list from polygon points in the specified winding order
function linkedList(data, start, end, dim, clockwise) {
    var i, last;

    if (clockwise === (signedArea(data, start, end, dim) > 0)) {
        for (i = start; i < end; i += dim) last = insertNode(i, data[i], data[i + 1], last);
    } else {
        for (i = end - dim; i >= start; i -= dim) last = insertNode(i, data[i], data[i + 1], last);
    }

    if (last && equals(last, last.next)) {
        removeNode(last);
        last = last.next;
    }

    return last;
}

// eliminate colinear or duplicate points
function filterPoints(start, end) {
    if (!start) return start;
    if (!end) end = start;

    var p = start,
        again;
    do {
        again = false;

        if (!p.steiner && (equals(p, p.next) || area(p.prev, p, p.next) === 0)) {
            removeNode(p);
            p = end = p.prev;
            if (p === p.next) return null;
            again = true;

        } else {
            p = p.next;
        }
    } while (again || p !== end);

    return end;
}

// main ear slicing loop which triangulates a polygon (given as a linked list)
function earcutLinked(ear, triangles, dim, minX, minY, size, pass) {
    if (!ear) return;

    // interlink polygon nodes in z-order
    if (!pass && size) indexCurve(ear, minX, minY, size);

    var stop = ear,
        prev, next;

    // iterate through ears, slicing them one by one
    while (ear.prev !== ear.next) {
        prev = ear.prev;
        next = ear.next;

        if (size ? isEarHashed(ear, minX, minY, size) : isEar(ear)) {
            // cut off the triangle
            triangles.push(prev.i / dim);
            triangles.push(ear.i / dim);
            triangles.push(next.i / dim);

            removeNode(ear);

            // skipping the next vertice leads to less sliver triangles
            ear = next.next;
            stop = next.next;

            continue;
        }

        ear = next;

        // if we looped through the whole remaining polygon and can't find any more ears
        if (ear === stop) {
            // try filtering points and slicing again
            if (!pass) {
                earcutLinked(filterPoints(ear), triangles, dim, minX, minY, size, 1);

            // if this didn't work, try curing all small self-intersections locally
            } else if (pass === 1) {
                ear = cureLocalIntersections(ear, triangles, dim);
                earcutLinked(ear, triangles, dim, minX, minY, size, 2);

            // as a last resort, try splitting the remaining polygon into two
            } else if (pass === 2) {
                splitEarcut(ear, triangles, dim, minX, minY, size);
            }

            break;
        }
    }
}

// check whether a polygon node forms a valid ear with adjacent nodes
function isEar(ear) {
    var a = ear.prev,
        b = ear,
        c = ear.next;

    if (area(a, b, c) >= 0) return false; // reflex, can't be an ear

    // now make sure we don't have other points inside the potential ear
    var p = ear.next.next;

    while (p !== ear.prev) {
        if (pointInTriangle(a.x, a.y, b.x, b.y, c.x, c.y, p.x, p.y) &&
            area(p.prev, p, p.next) >= 0) return false;
        p = p.next;
    }

    return true;
}

function isEarHashed(ear, minX, minY, size) {
    var a = ear.prev,
        b = ear,
        c = ear.next;

    if (area(a, b, c) >= 0) return false; // reflex, can't be an ear

    // triangle bbox; min & max are calculated like this for speed
    var minTX = a.x < b.x ? (a.x < c.x ? a.x : c.x) : (b.x < c.x ? b.x : c.x),
        minTY = a.y < b.y ? (a.y < c.y ? a.y : c.y) : (b.y < c.y ? b.y : c.y),
        maxTX = a.x > b.x ? (a.x > c.x ? a.x : c.x) : (b.x > c.x ? b.x : c.x),
        maxTY = a.y > b.y ? (a.y > c.y ? a.y : c.y) : (b.y > c.y ? b.y : c.y);

    // z-order range for the current triangle bbox;
    var minZ = zOrder(minTX, minTY, minX, minY, size),
        maxZ = zOrder(maxTX, maxTY, minX, minY, size);

    // first look for points inside the triangle in increasing z-order
    var p = ear.nextZ;

    while (p && p.z <= maxZ) {
        if (p !== ear.prev && p !== ear.next &&
            pointInTriangle(a.x, a.y, b.x, b.y, c.x, c.y, p.x, p.y) &&
            area(p.prev, p, p.next) >= 0) return false;
        p = p.nextZ;
    }

    // then look for points in decreasing z-order
    p = ear.prevZ;

    while (p && p.z >= minZ) {
        if (p !== ear.prev && p !== ear.next &&
            pointInTriangle(a.x, a.y, b.x, b.y, c.x, c.y, p.x, p.y) &&
            area(p.prev, p, p.next) >= 0) return false;
        p = p.prevZ;
    }

    return true;
}

// go through all polygon nodes and cure small local self-intersections
function cureLocalIntersections(start, triangles, dim) {
    var p = start;
    do {
        var a = p.prev,
            b = p.next.next;

        if (!equals(a, b) && intersects(a, p, p.next, b) && locallyInside(a, b) && locallyInside(b, a)) {

            triangles.push(a.i / dim);
            triangles.push(p.i / dim);
            triangles.push(b.i / dim);

            // remove two nodes involved
            removeNode(p);
            removeNode(p.next);

            p = start = b;
        }
        p = p.next;
    } while (p !== start);

    return p;
}

// try splitting polygon into two and triangulate them independently
function splitEarcut(start, triangles, dim, minX, minY, size) {
    // look for a valid diagonal that divides the polygon into two
    var a = start;
    do {
        var b = a.next.next;
        while (b !== a.prev) {
            if (a.i !== b.i && isValidDiagonal(a, b)) {
                // split the polygon in two by the diagonal
                var c = splitPolygon(a, b);

                // filter colinear points around the cuts
                a = filterPoints(a, a.next);
                c = filterPoints(c, c.next);

                // run earcut on each half
                earcutLinked(a, triangles, dim, minX, minY, size);
                earcutLinked(c, triangles, dim, minX, minY, size);
                return;
            }
            b = b.next;
        }
        a = a.next;
    } while (a !== start);
}

// link every hole into the outer loop, producing a single-ring polygon without holes
function eliminateHoles(data, holeIndices, outerNode, dim) {
    var queue = [],
        i, len, start, end, list;

    for (i = 0, len = holeIndices.length; i < len; i++) {
        start = holeIndices[i] * dim;
        end = i < len - 1 ? holeIndices[i + 1] * dim : data.length;
        list = linkedList(data, start, end, dim, false);
        if (list === list.next) list.steiner = true;
        queue.push(getLeftmost(list));
    }

    queue.sort(compareX);

    // process holes from left to right
    for (i = 0; i < queue.length; i++) {
        eliminateHole(queue[i], outerNode);
        outerNode = filterPoints(outerNode, outerNode.next);
    }

    return outerNode;
}

function compareX(a, b) {
    return a.x - b.x;
}

// find a bridge between vertices that connects hole with an outer ring and and link it
function eliminateHole(hole, outerNode) {
    outerNode = findHoleBridge(hole, outerNode);
    if (outerNode) {
        var b = splitPolygon(outerNode, hole);
        filterPoints(b, b.next);
    }
}

// David Eberly's algorithm for finding a bridge between hole and outer polygon
function findHoleBridge(hole, outerNode) {
    var p = outerNode,
        hx = hole.x,
        hy = hole.y,
        qx = -Infinity,
        m;

    // find a segment intersected by a ray from the hole's leftmost point to the left;
    // segment's endpoint with lesser x will be potential connection point
    do {
        if (hy <= p.y && hy >= p.next.y) {
            var x = p.x + (hy - p.y) * (p.next.x - p.x) / (p.next.y - p.y);
            if (x <= hx && x > qx) {
                qx = x;
                if (x === hx) {
                    if (hy === p.y) return p;
                    if (hy === p.next.y) return p.next;
                }
                m = p.x < p.next.x ? p : p.next;
            }
        }
        p = p.next;
    } while (p !== outerNode);

    if (!m) return null;

    if (hx === qx) return m.prev; // hole touches outer segment; pick lower endpoint

    // look for points inside the triangle of hole point, segment intersection and endpoint;
    // if there are no points found, we have a valid connection;
    // otherwise choose the point of the minimum angle with the ray as connection point

    var stop = m,
        mx = m.x,
        my = m.y,
        tanMin = Infinity,
        tan;

    p = m.next;

    while (p !== stop) {
        if (hx >= p.x && p.x >= mx &&
                pointInTriangle(hy < my ? hx : qx, hy, mx, my, hy < my ? qx : hx, hy, p.x, p.y)) {

            tan = Math.abs(hy - p.y) / (hx - p.x); // tangential

            if ((tan < tanMin || (tan === tanMin && p.x > m.x)) && locallyInside(p, hole)) {
                m = p;
                tanMin = tan;
            }
        }

        p = p.next;
    }

    return m;
}

// interlink polygon nodes in z-order
function indexCurve(start, minX, minY, size) {
    var p = start;
    do {
        if (p.z === null) p.z = zOrder(p.x, p.y, minX, minY, size);
        p.prevZ = p.prev;
        p.nextZ = p.next;
        p = p.next;
    } while (p !== start);

    p.prevZ.nextZ = null;
    p.prevZ = null;

    sortLinked(p);
}

// Simon Tatham's linked list merge sort algorithm
// http://www.chiark.greenend.org.uk/~sgtatham/algorithms/listsort.html
function sortLinked(list) {
    var i, p, q, e, tail, numMerges, pSize, qSize,
        inSize = 1;

    do {
        p = list;
        list = null;
        tail = null;
        numMerges = 0;

        while (p) {
            numMerges++;
            q = p;
            pSize = 0;
            for (i = 0; i < inSize; i++) {
                pSize++;
                q = q.nextZ;
                if (!q) break;
            }

            qSize = inSize;

            while (pSize > 0 || (qSize > 0 && q)) {

                if (pSize === 0) {
                    e = q;
                    q = q.nextZ;
                    qSize--;
                } else if (qSize === 0 || !q) {
                    e = p;
                    p = p.nextZ;
                    pSize--;
                } else if (p.z <= q.z) {
                    e = p;
                    p = p.nextZ;
                    pSize--;
                } else {
                    e = q;
                    q = q.nextZ;
                    qSize--;
                }

                if (tail) tail.nextZ = e;
                else list = e;

                e.prevZ = tail;
                tail = e;
            }

            p = q;
        }

        tail.nextZ = null;
        inSize *= 2;

    } while (numMerges > 1);

    return list;
}

// z-order of a point given coords and size of the data bounding box
function zOrder(x, y, minX, minY, size) {
    // coords are transformed into non-negative 15-bit integer range
    x = 32767 * (x - minX) / size;
    y = 32767 * (y - minY) / size;

    x = (x | (x << 8)) & 0x00FF00FF;
    x = (x | (x << 4)) & 0x0F0F0F0F;
    x = (x | (x << 2)) & 0x33333333;
    x = (x | (x << 1)) & 0x55555555;

    y = (y | (y << 8)) & 0x00FF00FF;
    y = (y | (y << 4)) & 0x0F0F0F0F;
    y = (y | (y << 2)) & 0x33333333;
    y = (y | (y << 1)) & 0x55555555;

    return x | (y << 1);
}

// find the leftmost node of a polygon ring
function getLeftmost(start) {
    var p = start,
        leftmost = start;
    do {
        if (p.x < leftmost.x) leftmost = p;
        p = p.next;
    } while (p !== start);

    return leftmost;
}

// check if a point lies within a convex triangle
function pointInTriangle(ax, ay, bx, by, cx, cy, px, py) {
    return (cx - px) * (ay - py) - (ax - px) * (cy - py) >= 0 &&
           (ax - px) * (by - py) - (bx - px) * (ay - py) >= 0 &&
           (bx - px) * (cy - py) - (cx - px) * (by - py) >= 0;
}

// check if a diagonal between two polygon nodes is valid (lies in polygon interior)
function isValidDiagonal(a, b) {
    return a.next.i !== b.i && a.prev.i !== b.i && !intersectsPolygon(a, b) &&
           locallyInside(a, b) && locallyInside(b, a) && middleInside(a, b);
}

// signed area of a triangle
function area(p, q, r) {
    return (q.y - p.y) * (r.x - q.x) - (q.x - p.x) * (r.y - q.y);
}

// check if two points are equal
function equals(p1, p2) {
    return p1.x === p2.x && p1.y === p2.y;
}

// check if two segments intersect
function intersects(p1, q1, p2, q2) {
    if ((equals(p1, q1) && equals(p2, q2)) ||
        (equals(p1, q2) && equals(p2, q1))) return true;
    return area(p1, q1, p2) > 0 !== area(p1, q1, q2) > 0 &&
           area(p2, q2, p1) > 0 !== area(p2, q2, q1) > 0;
}

// check if a polygon diagonal intersects any polygon segments
function intersectsPolygon(a, b) {
    var p = a;
    do {
        if (p.i !== a.i && p.next.i !== a.i && p.i !== b.i && p.next.i !== b.i &&
                intersects(p, p.next, a, b)) return true;
        p = p.next;
    } while (p !== a);

    return false;
}

// check if a polygon diagonal is locally inside the polygon
function locallyInside(a, b) {
    return area(a.prev, a, a.next) < 0 ?
        area(a, b, a.next) >= 0 && area(a, a.prev, b) >= 0 :
        area(a, b, a.prev) < 0 || area(a, a.next, b) < 0;
}

// check if the middle point of a polygon diagonal is inside the polygon
function middleInside(a, b) {
    var p = a,
        inside = false,
        px = (a.x + b.x) / 2,
        py = (a.y + b.y) / 2;
    do {
        if (((p.y > py) !== (p.next.y > py)) && (px < (p.next.x - p.x) * (py - p.y) / (p.next.y - p.y) + p.x))
            inside = !inside;
        p = p.next;
    } while (p !== a);

    return inside;
}

// link two polygon vertices with a bridge; if the vertices belong to the same ring, it splits polygon into two;
// if one belongs to the outer ring and another to a hole, it merges it into a single ring
function splitPolygon(a, b) {
    var a2 = new Node(a.i, a.x, a.y),
        b2 = new Node(b.i, b.x, b.y),
        an = a.next,
        bp = b.prev;

    a.next = b;
    b.prev = a;

    a2.next = an;
    an.prev = a2;

    b2.next = a2;
    a2.prev = b2;

    bp.next = b2;
    b2.prev = bp;

    return b2;
}

// create a node and optionally link it with previous one (in a circular doubly linked list)
function insertNode(i, x, y, last) {
    var p = new Node(i, x, y);

    if (!last) {
        p.prev = p;
        p.next = p;

    } else {
        p.next = last.next;
        p.prev = last;
        last.next.prev = p;
        last.next = p;
    }
    return p;
}

function removeNode(p) {
    p.next.prev = p.prev;
    p.prev.next = p.next;

    if (p.prevZ) p.prevZ.nextZ = p.nextZ;
    if (p.nextZ) p.nextZ.prevZ = p.prevZ;
}

function Node(i, x, y) {
    // vertice index in coordinates array
    this.i = i;

    // vertex coordinates
    this.x = x;
    this.y = y;

    // previous and next vertice nodes in a polygon ring
    this.prev = null;
    this.next = null;

    // z-order curve value
    this.z = null;

    // previous and next nodes in z-order
    this.prevZ = null;
    this.nextZ = null;

    // indicates whether this is a steiner point
    this.steiner = false;
}

// return a percentage difference between the polygon area and its triangulation area;
// used to verify correctness of triangulation
earcut.deviation = function (data, holeIndices, dim, triangles) {
    var hasHoles = holeIndices && holeIndices.length;
    var outerLen = hasHoles ? holeIndices[0] * dim : data.length;

    var polygonArea = Math.abs(signedArea(data, 0, outerLen, dim));
    if (hasHoles) {
        for (var i = 0, len = holeIndices.length; i < len; i++) {
            var start = holeIndices[i] * dim;
            var end = i < len - 1 ? holeIndices[i + 1] * dim : data.length;
            polygonArea -= Math.abs(signedArea(data, start, end, dim));
        }
    }

    var trianglesArea = 0;
    for (i = 0; i < triangles.length; i += 3) {
        var a = triangles[i] * dim;
        var b = triangles[i + 1] * dim;
        var c = triangles[i + 2] * dim;
        trianglesArea += Math.abs(
            (data[a] - data[c]) * (data[b + 1] - data[a + 1]) -
            (data[a] - data[b]) * (data[c + 1] - data[a + 1]));
    }

    return polygonArea === 0 && trianglesArea === 0 ? 0 :
        Math.abs((trianglesArea - polygonArea) / polygonArea);
};

function signedArea(data, start, end, dim) {
    var sum = 0;
    for (var i = start, j = end - dim; i < end; i += dim) {
        sum += (data[j] - data[i]) * (data[i + 1] + data[j + 1]);
        j = i;
    }
    return sum;
}

// turn a polygon in a multi-dimensional array form (e.g. as in GeoJSON) into a form Earcut accepts
earcut.flatten = function (data) {
    var dim = data[0][0].length,
        result = {vertices: [], holes: [], dimensions: dim},
        holeIndex = 0;

    for (var i = 0; i < data.length; i++) {
        for (var j = 0; j < data[i].length; j++) {
            for (var d = 0; d < dim; d++) result.vertices.push(data[i][j][d]);
        }
        if (i > 0) {
            holeIndex += data[i - 1].length;
            result.holes.push(holeIndex);
        }
    }
    return result;
};

// Polygon builders
var up_vec3 = [0, 0, 1];
/**
 * To tesselate flat 2D polygons.
 * The x & y coordinates will be set as first two elements of provided vertex_template.
 * @param {Array.<Array.<Array.<Array.<number>>>>} polygons The polygons to tesselate.
 * @param {VertexData} vertex_data The VertexData were to store the results
 * @param {Array.<number>} vertex_template The vertex template to use
 * @param {Object} options The texture coordinate options to apply
 * @return {number} the number of the resulting geometries (triangles)
 */

function buildPolygons(polygons, vertex_data, vertex_template, _ref) {
  var texcoord_index = _ref.texcoord_index,
      texcoord_scale = _ref.texcoord_scale,
      texcoord_normalize = _ref.texcoord_normalize;
  var vertex_elements = vertex_data.vertex_elements,
      num_polygons = polygons.length,
      geom_count = 0,
      min_u,
      min_v,
      max_u,
      max_v,
      min_x,
      min_y,
      max_x,
      max_y,
      span_x,
      span_y,
      scale_u,
      scale_v;

  if (texcoord_index) {
    texcoord_normalize = texcoord_normalize || 1;

    var _ref2 = texcoord_scale || default_uvs;

    min_u = _ref2[0];
    min_v = _ref2[1];
    max_u = _ref2[2];
    max_v = _ref2[3];
  }

  for (var p = 0; p < num_polygons; p++) {
    var polygon = polygons[p],
        element_offset = vertex_data.vertex_count,
        indices = triangulatePolygon(earcut_1.flatten(polygon)),
        num_indices = indices.length; // The vertices and vertex-elements must not be added if earcut returns no indices:

    if (num_indices) {
      // Find polygon extents to calculate UVs, fit them to the axis-aligned bounding box:
      if (texcoord_index) {
        var _Geo$findBoundingBox;

        (_Geo$findBoundingBox = Geo$1.findBoundingBox(polygon), min_x = _Geo$findBoundingBox[0], min_y = _Geo$findBoundingBox[1], max_x = _Geo$findBoundingBox[2], max_y = _Geo$findBoundingBox[3], _Geo$findBoundingBox), span_x = max_x - min_x, span_y = max_y - min_y, scale_u = (max_u - min_u) / span_x, scale_v = (max_v - min_v) / span_y;
      }

      for (var ring_index = 0; ring_index < polygon.length; ring_index++) {
        // Add vertex data:
        var polygon_ring = polygon[ring_index];

        for (var i = 0; i < polygon_ring.length; i++) {
          var vertex = polygon_ring[i];
          vertex_template[0] = vertex[0];
          vertex_template[1] = vertex[1]; // Add UVs:

          if (texcoord_index) {
            vertex_template[texcoord_index + 0] = ((vertex[0] - min_x) * scale_u + min_u) * texcoord_normalize;
            vertex_template[texcoord_index + 1] = ((vertex[1] - min_y) * scale_v + min_v) * texcoord_normalize;
          }

          vertex_data.addVertex(vertex_template);
        }
      } // Add element indices:


      for (var _i = 0; _i < num_indices; _i++) {
        vertex_elements.push(element_offset + indices[_i]);
      }

      geom_count += num_indices / 3;
    }
  }

  return geom_count;
} // Tesselate and extrude a flat 2D polygon into a simple 3D model with fixed height and add to GL vertex buffer

function buildExtrudedPolygons(polygons, z, height, min_height, vertex_data, vertex_template, normal_index, normal_normalize, _ref3) {
  var remove_tile_edges = _ref3.remove_tile_edges,
      tile_edge_tolerance = _ref3.tile_edge_tolerance,
      texcoord_index = _ref3.texcoord_index,
      texcoord_scale = _ref3.texcoord_scale,
      texcoord_normalize = _ref3.texcoord_normalize,
      winding = _ref3.winding;
  // Top
  var min_z = z + (min_height || 0);
  var max_z = z + height;
  vertex_template[2] = max_z;
  var geom_count = buildPolygons(polygons, vertex_data, vertex_template, {
    texcoord_index: texcoord_index,
    texcoord_scale: texcoord_scale,
    texcoord_normalize: texcoord_normalize
  });
  var vertex_elements = vertex_data.vertex_elements;
  var element_offset = vertex_data.vertex_count; // Walls
  // Fit UVs to wall quad

  if (texcoord_index) {
    texcoord_normalize = texcoord_normalize || 1;

    var _ref4 = texcoord_scale || default_uvs,
        min_u = _ref4[0],
        min_v = _ref4[1],
        max_u = _ref4[2],
        max_v = _ref4[3];

    var texcoords = [[min_u, max_v], [min_u, min_v], [max_u, min_v], [max_u, max_v]];
  }

  var num_polygons = polygons.length;

  for (var p = 0; p < num_polygons; p++) {
    var polygon = polygons[p];

    for (var q = 0; q < polygon.length; q++) {
      var contour = polygon[q];

      for (var w = 0; w < contour.length - 1; w++) {
        if (remove_tile_edges && outsideTile(contour[w], contour[w + 1], tile_edge_tolerance)) {
          continue; // don't extrude tile edges
        } // Wall order is dependent on winding order, so that normals face outward


        var w0 = void 0,
            w1 = void 0;

        if (winding === 'CCW') {
          w0 = w;
          w1 = w + 1;
        } else {
          w0 = w + 1;
          w1 = w;
        } // Two triangles for the quad formed by each vertex pair, going from bottom to top height


        var wall_vertices = [[contour[w1][0], contour[w1][1], max_z], [contour[w1][0], contour[w1][1], min_z], [contour[w0][0], contour[w0][1], min_z], [contour[w0][0], contour[w0][1], max_z]]; // Calc the normal of the wall from up vector and one segment of the wall triangles

        var wall_vec = Vector$1.normalize([contour[w1][0] - contour[w0][0], contour[w1][1] - contour[w0][1], 0]);
        var normal = Vector$1.cross(up_vec3, wall_vec); // Update vertex template with current surface normal

        vertex_template[normal_index + 0] = normal[0] * normal_normalize;
        vertex_template[normal_index + 1] = normal[1] * normal_normalize;
        vertex_template[normal_index + 2] = normal[2] * normal_normalize;

        for (var wv = 0; wv < wall_vertices.length; wv++) {
          vertex_template[0] = wall_vertices[wv][0];
          vertex_template[1] = wall_vertices[wv][1];
          vertex_template[2] = wall_vertices[wv][2];

          if (texcoord_index) {
            vertex_template[texcoord_index + 0] = texcoords[wv][0] * texcoord_normalize;
            vertex_template[texcoord_index + 1] = texcoords[wv][1] * texcoord_normalize;
          }

          vertex_data.addVertex(vertex_template);
        }

        vertex_elements.push(element_offset + 0);
        vertex_elements.push(element_offset + 1);
        vertex_elements.push(element_offset + 2);
        vertex_elements.push(element_offset + 2);
        vertex_elements.push(element_offset + 3);
        vertex_elements.push(element_offset + 0);
        element_offset += 4;
        geom_count += 2;
      }
    }
  }

  return geom_count;
} // Triangulation using earcut
// https://github.com/mapbox/earcut

function triangulatePolygon(data) {
  return earcut_1(data.vertices, data.holes, data.dimensions);
}

var polygons_vs = "uniform vec2 u_resolution;\nuniform float u_time;\nuniform vec3 u_map_position;\nuniform vec4 u_tile_origin;\nuniform float u_tile_proxy_order_offset;\nuniform float u_meters_per_pixel;\nuniform float u_device_pixel_ratio;\n\nuniform mat4 u_model;\nuniform mat4 u_modelView;\nuniform mat3 u_normalMatrix;\nuniform mat3 u_inverseNormalMatrix;\n\nattribute vec4 a_position;\nattribute vec4 a_color;\n\n// Optional normal attribute, otherwise default to up\n#ifdef TANGRAM_NORMAL_ATTRIBUTE\n    attribute vec3 a_normal;\n    #define TANGRAM_NORMAL a_normal\n#else\n    #define TANGRAM_NORMAL vec3(0., 0., 1.)\n#endif\n\n// Optional dynamic line extrusion\n#ifdef TANGRAM_EXTRUDE_LINES\n    attribute vec2 a_extrude; // extrusion direction in xy plane\n    attribute vec2 a_offset;  // offset direction in xy plane\n\n    // Polygon and line styles have slightly different VBO layouts, saving memory by optimizing vertex packing.\n    // All lines have a width scaling factor, but only some have a height (position.z) or offset.\n    // The vertex height is stored in different attributes to account for this.\n    attribute vec2 a_z_and_offset_scale; // stores vertex height in x, and offset scaling factor in y\n    #define TANGRAM_POSITION_Z a_z_and_offset_scale.x // vertex height is stored in separate line-specific attrib\n    #define TANGRAM_OFFSET_SCALING a_z_and_offset_scale.y // zoom scaling factor for line offset\n    #define TANGRAM_WIDTH_SCALING a_position.z // zoom scaling factor for line width (stored in position attrib)\n\n    uniform float u_v_scale_adjust; // scales texture UVs for line dash patterns w/fractional pixel width\n#else\n    #define TANGRAM_POSITION_Z a_position.z // vertex height\n#endif\n\nvarying vec4 v_position;\nvarying vec3 v_normal;\nvarying vec4 v_color;\nvarying vec4 v_world_position;\n\n// Optional texture UVs\n#if defined(TANGRAM_TEXTURE_COORDS) || defined(TANGRAM_EXTRUDE_LINES)\n    attribute vec2 a_texcoord;\n    varying vec2 v_texcoord;\n#endif\n\n// Optional model position varying for tile coordinate zoom\n#ifdef TANGRAM_MODEL_POSITION_BASE_ZOOM_VARYING\n    varying vec4 v_modelpos_base_zoom;\n#endif\n\n#if defined(TANGRAM_LIGHTING_VERTEX)\n    varying vec4 v_lighting;\n#endif\n\n#define TANGRAM_UNPACK_SCALING(x) (x / 1024.)\n\n#pragma tangram: camera\n#pragma tangram: material\n#pragma tangram: lighting\n#pragma tangram: raster\n#pragma tangram: global\n\nvoid main() {\n    // Initialize globals\n    #pragma tangram: setup\n\n    // Texture UVs\n    #ifdef TANGRAM_TEXTURE_COORDS\n        v_texcoord = a_texcoord;\n        #ifdef TANGRAM_EXTRUDE_LINES\n            v_texcoord.y *= u_v_scale_adjust;\n        #endif\n    #endif\n\n    // Pass model position to fragment shader\n    #ifdef TANGRAM_MODEL_POSITION_BASE_ZOOM_VARYING\n        v_modelpos_base_zoom = modelPositionBaseZoom();\n    #endif\n\n    // Position\n    vec4 position = vec4(a_position.xy, TANGRAM_POSITION_Z / TANGRAM_HEIGHT_SCALE, 1.); // convert height back to meters\n\n    #ifdef TANGRAM_EXTRUDE_LINES\n        vec2 extrude = a_extrude.xy;\n        vec2 offset = a_offset.xy;\n\n        // Adjust line width based on zoom level, to prevent proxied lines\n        // from being either too small or too big.\n        // \"Flattens\" the zoom between 1-2 to peg it to 1 (keeps lines from\n        // prematurely shrinking), then interpolate and clamp to 4 (keeps lines\n        // from becoming too small when far away).\n        float dz = clamp(u_map_position.z - u_tile_origin.z, 0., 4.);\n        dz += step(1., dz) * (1. - dz) + mix(0., 2., clamp((dz - 2.) / 2., 0., 1.));\n\n        // Interpolate line width between zooms\n        float mdz = (dz - 0.5) * 2.; // zoom from mid-point\n        extrude -= extrude * TANGRAM_UNPACK_SCALING(TANGRAM_WIDTH_SCALING) * mdz;\n\n        // Interpolate line offset between zooms\n        // Scales from the larger value to the smaller one\n        float dwdz = TANGRAM_UNPACK_SCALING(TANGRAM_OFFSET_SCALING);\n        float sdwdz = sign(step(0., dwdz) - 0.5); // sign indicates \"direction\" of scaling\n        offset -= offset * abs(dwdz) * ((1.-step(0., sdwdz)) - (dz * -sdwdz)); // scale \"up\" or \"down\"\n\n        // Scale line width and offset to be consistent in screen space\n        float ssz = exp2(-dz - (u_tile_origin.z - u_tile_origin.w));\n        extrude *= ssz;\n        offset *= ssz;\n\n        // Modify line width before extrusion\n        #ifdef TANGRAM_BLOCK_WIDTH\n            float width = 1.;\n            #pragma tangram: width\n            extrude *= width;\n        #endif\n\n        position.xy += extrude + offset;\n    #endif\n\n    // World coordinates for 3d procedural textures\n    v_world_position = wrapWorldPosition(u_model * position);\n\n    // Adjust for tile and view position\n    position = u_modelView * position;\n\n    // Modify position before camera projection\n    #pragma tangram: position\n\n    // Setup varyings\n    v_position = position;\n    v_normal = normalize(u_normalMatrix * TANGRAM_NORMAL);\n    v_color = a_color;\n\n    #if defined(TANGRAM_LIGHTING_VERTEX)\n        // Vertex lighting\n        vec3 normal = v_normal;\n\n        // Modify normal before lighting\n        #pragma tangram: normal\n\n        // Pass lighting intensity to fragment shader\n        v_lighting = calculateLighting(position.xyz - u_eye, normal, vec4(1.));\n    #endif\n\n    // Camera\n    cameraProjection(position);\n\n    // +1 is to keep all layers including proxies > 0\n    applyLayerOrder(a_position.w + u_tile_proxy_order_offset + 1., position);\n\n    gl_Position = position;\n}\n";

var polygons_fs = "uniform vec2 u_resolution;\nuniform float u_time;\nuniform vec3 u_map_position;\nuniform vec4 u_tile_origin;\nuniform float u_meters_per_pixel;\nuniform float u_device_pixel_ratio;\n\nuniform mat3 u_normalMatrix;\nuniform mat3 u_inverseNormalMatrix;\n\nvarying vec4 v_position;\nvarying vec3 v_normal;\nvarying vec4 v_color;\nvarying vec4 v_world_position;\n\n#ifdef TANGRAM_EXTRUDE_LINES\n    uniform bool u_has_line_texture;\n    uniform sampler2D u_texture;\n    uniform float u_texture_ratio;\n    uniform vec4 u_dash_background_color;\n#endif\n\n#define TANGRAM_NORMAL v_normal\n\n#if defined(TANGRAM_TEXTURE_COORDS) || defined(TANGRAM_EXTRUDE_LINES)\n    varying vec2 v_texcoord;\n#endif\n\n#ifdef TANGRAM_MODEL_POSITION_BASE_ZOOM_VARYING\n    varying vec4 v_modelpos_base_zoom;\n#endif\n\n#if defined(TANGRAM_LIGHTING_VERTEX)\n    varying vec4 v_lighting;\n#endif\n\n#pragma tangram: camera\n#pragma tangram: material\n#pragma tangram: lighting\n#pragma tangram: raster\n#pragma tangram: global\n\nvoid main (void) {\n    // Initialize globals\n    #pragma tangram: setup\n\n    vec4 color = v_color;\n    vec3 normal = TANGRAM_NORMAL;\n\n    // Apply raster to vertex color\n    #ifdef TANGRAM_RASTER_TEXTURE_COLOR\n    { // enclose in scope to avoid leakage of internal variables\n        vec4 raster_color = sampleRaster(0);\n\n        #if defined(TANGRAM_BLEND_OPAQUE) || defined(TANGRAM_BLEND_TRANSLUCENT) || defined(TANGRAM_BLEND_MULTIPLY)\n            // Raster sources can optionally mask by the alpha channel, which will render with only full or no alpha.\n            // This is used for handling transparency outside the raster image in some blend modes,\n            // which either don't support alpha, or would cause transparent pixels to write to the depth buffer,\n            // obscuring geometry underneath.\n            #ifdef TANGRAM_HAS_MASKED_RASTERS   // skip masking logic if no masked raster sources\n            #ifndef TANGRAM_ALL_MASKED_RASTERS  // skip conditional if *only* masked raster sources (always true)\n            if (u_raster_mask_alpha) {\n            #else\n            {\n            #endif\n                #if defined(TANGRAM_BLEND_TRANSLUCENT) || defined(TANGRAM_BLEND_MULTIPLY)\n                if (raster_color.a < TANGRAM_EPSILON) {\n                    discard;\n                }\n                #else // TANGRAM_BLEND_OPAQUE\n                if (raster_color.a < 1. - TANGRAM_EPSILON) {\n                    discard;\n                }\n                // only allow full alpha in opaque blend mode (avoids artifacts blending w/canvas tile background)\n                raster_color.a = 1.;\n                #endif\n            }\n            #endif\n        #endif\n\n        color *= raster_color; // multiplied to tint texture color\n    }\n    #endif\n\n    // Apply line texture\n    #ifdef TANGRAM_EXTRUDE_LINES\n    { // enclose in scope to avoid leakage of internal variables\n        if (u_has_line_texture) {\n            vec2 _line_st = vec2(v_texcoord.x, fract(v_texcoord.y / u_texture_ratio));\n            vec4 _line_color = texture2D(u_texture, _line_st);\n\n            if (_line_color.a < TANGRAM_ALPHA_TEST) {\n                #if defined(TANGRAM_BLEND_OPAQUE)\n                    // use discard when alpha blending is unavailable\n                    if (u_dash_background_color.a < 1. - TANGRAM_EPSILON) {\n                        discard;\n                    }\n                    color = vec4(u_dash_background_color.rgb, 1.); // only allow full alpha in opaque blend mode\n                #else\n                    // use alpha channel when blending is available\n                    color = vec4(u_dash_background_color.rgb, color.a * step(TANGRAM_EPSILON, u_dash_background_color.a));\n                #endif\n            }\n            else {\n                color *= _line_color;\n            }\n        }\n    }\n    #endif\n\n    // First, get normal from raster tile (if applicable)\n    #ifdef TANGRAM_RASTER_TEXTURE_NORMAL\n        normal = normalize(sampleRaster(0).rgb * 2. - 1.);\n    #endif\n\n    // Second, alter normal with normal map texture (if applicable)\n    #if defined(TANGRAM_LIGHTING_FRAGMENT) && defined(TANGRAM_MATERIAL_NORMAL_TEXTURE)\n        calculateNormal(normal);\n    #endif\n\n    // Normal modification applied here for fragment lighting or no lighting,\n    // and in vertex shader for vertex lighting\n    #if !defined(TANGRAM_LIGHTING_VERTEX)\n        #pragma tangram: normal\n    #endif\n\n    // Color modification before lighting is applied\n    #pragma tangram: color\n\n    #if defined(TANGRAM_LIGHTING_FRAGMENT)\n        // Calculate per-fragment lighting\n        color = calculateLighting(v_position.xyz - u_eye, normal, color);\n    #elif defined(TANGRAM_LIGHTING_VERTEX)\n        // Apply lighting intensity interpolated from vertex shader\n        color *= v_lighting;\n    #endif\n\n    // Post-processing effects (modify color after lighting)\n    #pragma tangram: filter\n\n    gl_FragColor = color;\n}\n";

var Polygons = Object.create(Style);
Polygons.variants = {}; // mesh variants by variant key

Polygons.vertex_layouts = {}; // vertex layouts by variant key

Object.assign(Polygons, {
  name: 'polygons',
  built_in: true,
  vertex_shader_src: polygons_vs,
  fragment_shader_src: polygons_fs,
  selection: true,
  // enable feature selection
  init: function init() {
    Style.init.apply(this, arguments); // Tell the shader about optional attributes (shader is shared with lines style, which has different config)

    this.defines.TANGRAM_NORMAL_ATTRIBUTE = true;
    this.defines.TANGRAM_TEXTURE_COORDS = this.texcoords;
  },
  _parseFeature: function _parseFeature(feature, draw, context) {
    var style = this.feature_style;
    style.color = this.parseColor(draw.color, context);

    if (!style.color) {
      return null;
    }

    style.variant = draw.variant; // pre-calculated mesh variant

    style.z = StyleParser.evalCachedDistanceProperty(draw.z, context) || StyleParser.defaults.z;
    style.z *= Geo$1.height_scale; // provide sub-meter precision of height values

    style.extrude = StyleParser.evalProperty(draw.extrude, context);

    if (style.extrude) {
      // use feature's height and min_height properties
      if (style.extrude === true) {
        style.height = feature.properties.height || StyleParser.defaults.height;
        style.min_height = feature.properties.min_height || StyleParser.defaults.min_height;
      } // explicit height, no min_height
      else if (typeof style.extrude === 'number') {
          style.height = style.extrude;
          style.min_height = 0;
        } // explicit height and min_height
        else if (Array.isArray(style.extrude)) {
            style.min_height = style.extrude[0];
            style.height = style.extrude[1];
          }

      style.height *= Geo$1.height_scale; // provide sub-meter precision of height values

      style.min_height *= Geo$1.height_scale;
    }

    style.tile_edges = draw.tile_edges; // usually activated for debugging, or rare visualization needs

    return style;
  },
  _preprocess: function _preprocess(draw) {
    draw.color = StyleParser.createColorPropertyCache(draw.color);
    draw.z = StyleParser.createPropertyCache(draw.z, StyleParser.parseUnits);
    this.computeVariant(draw);
    return draw;
  },
  // Calculate and store mesh variant (unique by draw group but not feature)
  computeVariant: function computeVariant(draw) {
    // Factors that determine a unique mesh rendering variant
    var selection = draw.interactive ? 1 : 0; // whether feature has interactivity

    var normal = draw.extrude != null ? 1 : 0; // whether feature has extrusion (need per-vertex normals)

    var texcoords = this.texcoords ? 1 : 0; // whether feature has texture UVs

    var blend_order = this.getBlendOrderForDraw(draw);
    var key = [selection, normal, texcoords, blend_order].join('/');
    draw.variant = key;

    if (Polygons.variants[key] == null) {
      Polygons.variants[key] = {
        key: key,
        blend_order: blend_order,
        mesh_order: 0,
        selection: selection,
        normal: normal,
        texcoords: texcoords
      };
    }
  },
  // Override
  // Create or return desired vertex layout permutation based on flags
  vertexLayoutForMeshVariant: function vertexLayoutForMeshVariant(variant) {
    if (Polygons.vertex_layouts[variant.key] == null) {
      // Attributes for this mesh variant
      // Optional attributes have placeholder values assigned with `static` parameter
      var attribs = [{
        name: 'a_position',
        size: 4,
        type: gl$1.SHORT,
        normalized: false
      }, {
        name: 'a_normal',
        size: 3,
        type: gl$1.BYTE,
        normalized: true,
        static: variant.normal ? null : [0, 0, 1]
      }, // gets padded to 4-bytes
      {
        name: 'a_color',
        size: 4,
        type: gl$1.UNSIGNED_BYTE,
        normalized: true
      }, {
        name: 'a_selection_color',
        size: 4,
        type: gl$1.UNSIGNED_BYTE,
        normalized: true,
        static: variant.selection ? null : [0, 0, 0, 0]
      }, {
        name: 'a_texcoord',
        size: 2,
        type: gl$1.UNSIGNED_SHORT,
        normalized: true,
        static: variant.texcoords ? null : [0, 0]
      }];
      Polygons.vertex_layouts[variant.key] = new VertexLayout(attribs);
    }

    return Polygons.vertex_layouts[variant.key];
  },
  // Override
  meshVariantTypeForDraw: function meshVariantTypeForDraw(draw) {
    return this.variants[draw.variant]; // return pre-calculated mesh variant
  },

  /**
   * A "template" that sets constant attibutes for each vertex, which is then modified per vertex or per feature.
   * A plain JS array matching the order of the vertex layout.
   */
  makeVertexTemplate: function makeVertexTemplate(style, mesh) {
    var i = 0; // a_position.xyz - vertex position
    // a_position.w - layer order

    this.vertex_template[i++] = 0;
    this.vertex_template[i++] = 0;
    this.vertex_template[i++] = style.z || 0;
    this.vertex_template[i++] = this.scaleOrder(style.order); // a_normal.xyz - surface normal
    // only stored per-vertex for extruded features (hardcoded to 'up' for others)

    if (mesh.variant.normal) {
      this.vertex_template[i++] = 0;
      this.vertex_template[i++] = 0;
      this.vertex_template[i++] = 1 * 127;
    } // a_color.rgba - feature color


    this.vertex_template[i++] = style.color[0] * 255;
    this.vertex_template[i++] = style.color[1] * 255;
    this.vertex_template[i++] = style.color[2] * 255;
    this.vertex_template[i++] = style.color[3] * 255; // a_selection_color.rgba - selection color

    if (mesh.variant.selection) {
      this.vertex_template[i++] = style.selection_color[0] * 255;
      this.vertex_template[i++] = style.selection_color[1] * 255;
      this.vertex_template[i++] = style.selection_color[2] * 255;
      this.vertex_template[i++] = style.selection_color[3] * 255;
    } // a_texcoord.uv - texture coordinates


    if (mesh.variant.texcoords) {
      this.vertex_template[i++] = 0;
      this.vertex_template[i++] = 0;
    }

    return this.vertex_template;
  },
  buildPolygons: function buildPolygons$$1(polygons, style, context) {
    var mesh = this.getTileMesh(context.tile, this.meshVariantTypeForDraw(style));
    var vertex_data = mesh.vertex_data;
    var vertex_layout = vertex_data.vertex_layout;
    var vertex_template = this.makeVertexTemplate(style, mesh);
    var options = {
      texcoord_index: vertex_layout.index.a_texcoord,
      texcoord_normalize: 65535,
      // scale UVs to unsigned shorts
      remove_tile_edges: !style.tile_edges,
      tile_edge_tolerance: Geo$1.tile_scale * context.tile.pad_scale * 4,
      winding: context.winding
    }; // Extruded polygons (e.g. 3D buildings)

    if (style.extrude && style.height) {
      return buildExtrudedPolygons(polygons, style.z, style.height, style.min_height, vertex_data, vertex_template, vertex_layout.index.a_normal, 127, // scale normals to signed bytes
      options);
    } // Regular polygons
    else {
        return buildPolygons(polygons, vertex_data, vertex_template, options);
      }
  }
});

var quot = /"/g;
// B.2.3.2.1 CreateHTML(string, tag, attribute, value)
var createHTML = function (string, tag, attribute, value) {
  var S = String(_defined(string));
  var p1 = '<' + tag;
  if (attribute !== '') p1 += ' ' + attribute + '="' + String(value).replace(quot, '&quot;') + '"';
  return p1 + '>' + S + '</' + tag + '>';
};
var _stringHtml = function (NAME, exec) {
  var O = {};
  O[NAME] = exec(createHTML);
  _export(_export.P + _export.F * _fails(function () {
    var test = ''[NAME]('"');
    return test !== test.toLowerCase() || test.split('"').length > 3;
  }), 'String', O);
};

// B.2.3.13 String.prototype.sub()
_stringHtml('sub', function (createHTML) {
  return function sub() {
    return createHTML(this, 'sub', '', '');
  };
});

var zero_vec2 = [0, 0]; // Build tessellated triangles for a polyline

var CAP_TYPE = {
  butt: 0,
  square: 1,
  round: 2
};
var JOIN_TYPE = {
  miter: 0,
  bevel: 1,
  round: 2
};
var DEFAULT_MITER_LIMIT = 3;
var MIN_FAN_WIDTH = 5; // Width of line in tile units to place 1 triangle per fan

var TEXCOORD_NORMALIZE = 65536; // Scaling factor for UV attribute values
// Scaling factor to add precision to line texture V coordinate packed as normalized short

var V_SCALE_ADJUST = Geo$1.tile_scale;
var zero_v = [0, 0],
    one_v = [1, 0],
    mid_v = [0.5, 0]; // reusable instances, updated with V coordinate

function buildPolylines(lines, style, vertex_data, vertex_template, vindex, closed_polygon, remove_tile_edges, tile_edge_tolerance) {
  var cap_type = style.cap ? CAP_TYPE[style.cap] : CAP_TYPE.butt;
  var join_type = style.join ? JOIN_TYPE[style.join] : JOIN_TYPE.miter; // Configure miter limit

  if (join_type === JOIN_TYPE.miter) {
    var miter_limit = style.miter_limit || DEFAULT_MITER_LIMIT; // default miter limit

    var miter_len_sq = miter_limit * miter_limit;
  } // Texture Variables


  var v_scale;

  if (vindex.a_texcoord) {
    v_scale = 1 / (style.texcoord_width * V_SCALE_ADJUST); // scales line texture as a ratio of the line's width
  } // Values that are constant for each line and are passed to helper functions


  var context = {
    closed_polygon: closed_polygon,
    remove_tile_edges: remove_tile_edges,
    tile_edge_tolerance: tile_edge_tolerance,
    miter_len_sq: miter_len_sq,
    join_type: join_type,
    cap_type: cap_type,
    vertex_data: vertex_data,
    vertex_template: vertex_template,
    half_width: style.width / 2,
    extrude_index: vindex.a_extrude,
    offset_index: vindex.a_offset,
    v_scale: v_scale,
    texcoord_index: vindex.a_texcoord,
    texcoord_width: style.texcoord_width,
    offset: style.offset,
    geom_count: 0
  }; // Process lines

  for (var i = 0; i < lines.length; i++) {
    buildPolyline(lines[i], context);
  } // Process extra lines (which are created above if lines need to be mutated for easier processing)


  if (context.extra_lines) {
    for (var _i = 0; _i < context.extra_lines.length; _i++) {
      buildPolyline(context.extra_lines[_i], context);
    }
  }

  return context.geom_count;
}

function buildPolyline(line, context) {
  // Skip if line is not valid
  if (line.length < 2) {
    return;
  }

  var coordCurr, coordNext, normPrev, normNext;
  var join_type = context.join_type,
      cap_type = context.cap_type,
      closed_polygon = context.closed_polygon,
      remove_tile_edges = context.remove_tile_edges,
      tile_edge_tolerance = context.tile_edge_tolerance,
      v_scale = context.v_scale,
      miter_len_sq = context.miter_len_sq;
  var has_texcoord = context.texcoord_index != null;
  var v = 0; // Texture v-coordinate
  // Loop backwards through line to a tile boundary if found
  // since you need to draw lines that are only partially inside the tile,
  // so we start at the first index where it is safe to loop through to the last index within the tile

  if (closed_polygon && join_type === JOIN_TYPE.miter) {
    var boundaryIndex = getTileBoundaryIndex(line);

    if (boundaryIndex !== 0) {
      // create new line that is a cyclic permutation of the original
      var permutedLine = permuteLine(line, boundaryIndex);
      context.extra_lines = context.extra_lines || [];
      context.extra_lines.push(permutedLine);
      return;
    }
  }

  var index_start = 0;
  var index_end = line.length - 1;
  var ignored_indices_count = 0; // FIRST POINT
  // loop through beginning points if duplicates

  coordCurr = line[index_start];
  coordNext = line[index_start + 1];

  while (Vector$1.isEqual(coordCurr, coordNext)) {
    index_start++;
    coordCurr = coordNext;
    coordNext = line[index_start + 1];
    ignored_indices_count++;

    if (index_start === line.length - 1) {
      return;
    }
  } // loop through ending points to check for duplicates


  while (Vector$1.isEqual(line[index_end], line[index_end - 1])) {
    index_end--;
    ignored_indices_count++;

    if (index_end === 0) {
      return;
    }
  }

  if (line.length < 2 + ignored_indices_count) {
    return;
  }

  normNext = Vector$1.normalize(Vector$1.perp(coordCurr, coordNext)); // Skip tile boundary lines and append a new line if needed

  if (remove_tile_edges && outsideTile(coordCurr, coordNext, tile_edge_tolerance)) {
    var nonBoundarySegment = getNextNonBoundarySegment(line, index_start, tile_edge_tolerance);

    if (nonBoundarySegment) {
      context.extra_lines = context.extra_lines || [];
      context.extra_lines.push(nonBoundarySegment);
    }

    return;
  }

  if (closed_polygon) {
    // Begin the polygon with a join (connecting the first and last segments)
    normPrev = Vector$1.normalize(Vector$1.perp(line[index_end - 1], coordCurr));
    startPolygon(coordCurr, normPrev, normNext, join_type, context);
  } else {
    // If line begins at edge, don't add a cap
    if (!isCoordOutsideTile(coordCurr)) {
      addCap(coordCurr, v, normNext, cap_type, true, context);

      if (has_texcoord && cap_type !== CAP_TYPE.butt) {
        v += 0.5 * v_scale * context.texcoord_width;
      }
    } // Add first pair of points for the line strip


    addVertex(coordCurr, normNext, normNext, 1, v, context, 1);
    addVertex(coordCurr, normNext, normNext, 0, v, context, -1);
  } // INTERMEDIARY POINTS


  if (has_texcoord) {
    v += v_scale * Vector$1.length(Vector$1.sub(coordNext, coordCurr));
  }

  for (var i = index_start + 1; i < index_end; i++) {
    var currIndex = i;
    var nextIndex = i + 1;
    coordCurr = line[currIndex];
    coordNext = line[nextIndex]; // Skip redundant vertices

    if (Vector$1.isEqual(coordCurr, coordNext)) {
      continue;
    } // Remove tile boundaries


    if (remove_tile_edges && outsideTile(coordCurr, coordNext, tile_edge_tolerance)) {
      addVertex(coordCurr, normNext, normNext, 1, v, context, 1);
      addVertex(coordCurr, normNext, normNext, 0, v, context, -1);
      indexPairs(1, context);
      var nonBoundaryLines = getNextNonBoundarySegment(line, currIndex + 1, tile_edge_tolerance);

      if (nonBoundaryLines) {
        context.extra_lines = context.extra_lines || [];
        context.extra_lines.push(nonBoundaryLines);
      }

      return;
    }

    normPrev = normNext;
    normNext = Vector$1.normalize(Vector$1.perp(coordCurr, coordNext)); // Add join

    if (join_type === JOIN_TYPE.miter) {
      addMiter(v, coordCurr, normPrev, normNext, miter_len_sq, false, context);
    } else {
      addJoin(join_type, v, coordCurr, normPrev, normNext, false, context);
    }

    if (has_texcoord) {
      v += v_scale * Vector$1.length(Vector$1.sub(coordNext, coordCurr));
    }
  } // LAST POINT


  coordCurr = coordNext;
  normPrev = normNext;

  if (closed_polygon) {
    // Close the polygon with a miter joint or butt cap if on a tile boundary
    normNext = Vector$1.normalize(Vector$1.perp(coordCurr, line[1]));
    endPolygon(coordCurr, normPrev, normNext, join_type, v, context);
  } else {
    // Finish the line strip
    addVertex(coordCurr, normPrev, normNext, 1, v, context, 1);
    addVertex(coordCurr, normPrev, normNext, 0, v, context, -1);
    indexPairs(1, context); // If line ends at edge, don't add a cap

    if (!isCoordOutsideTile(coordCurr)) {
      addCap(coordCurr, v, normPrev, cap_type, false, context);
    }
  }
}

function getTileBoundaryIndex(line) {
  if (isCoordOutsideTile(line[0])) {
    return 0;
  }

  for (var backIndex = 0; backIndex < line.length; backIndex++) {
    var coordCurr = line[line.length - 1 - backIndex];

    if (isCoordOutsideTile(coordCurr)) {
      return line.length - 1 - backIndex;
    }
  }

  return 0;
} // Iterate through line from startIndex to find a segment not on a tile boundary, if any.


function getNextNonBoundarySegment(line, startIndex, tolerance) {
  var endIndex = startIndex;

  while (line[endIndex + 1] && outsideTile(line[endIndex], line[endIndex + 1], tolerance)) {
    endIndex++;
  } // If there is a line segment remaining that is within the tile, push it to the lines array


  return line.length - endIndex >= 2 ? line.slice(endIndex) : false;
} // Begin a polygon with a join connecting to the last segment (if valid join-type specified)


function startPolygon(coordCurr, normPrev, normNext, join_type, context) {
  // If polygon starts on a tile boundary, don't add a join
  if (join_type === undefined || isCoordOutsideTile(coordCurr)) {
    addVertex(coordCurr, normNext, normNext, 1, 0, context, 1);
    addVertex(coordCurr, normNext, normNext, 0, 0, context, -1);
  } else {
    // If polygon starts within a tile, add a join
    var v = 0;

    if (join_type === JOIN_TYPE.miter) {
      addMiter(v, coordCurr, normPrev, normNext, context.miter_len_sq, true, context);
    } else {
      addJoin(join_type, v, coordCurr, normPrev, normNext, true, context);
    }
  }
} // End a polygon appropriately


function endPolygon(coordCurr, normPrev, normNext, join_type, v, context) {
  // If polygon ends on a tile boundary, don't add a join
  if (isCoordOutsideTile(coordCurr)) {
    addVertex(coordCurr, normPrev, normPrev, 1, v, context, 1);
    addVertex(coordCurr, normPrev, normPrev, 0, v, context, -1);
    indexPairs(1, context);
  } else {
    // If polygon ends within a tile, add Miter or no joint (join added on startPolygon)
    var miterVec = createMiterVec(normPrev, normNext);

    if (join_type === JOIN_TYPE.miter && Vector$1.lengthSq(miterVec) > context.miter_len_sq) {
      join_type = JOIN_TYPE.bevel; // switch to bevel
    }

    if (join_type === JOIN_TYPE.miter) {
      addVertex(coordCurr, miterVec, normPrev, 1, v, context, 1);
      addVertex(coordCurr, miterVec, normPrev, 0, v, context, -1);
      indexPairs(1, context);
    } else {
      addVertex(coordCurr, normPrev, normPrev, 1, v, context, 1);
      addVertex(coordCurr, normPrev, normPrev, 0, v, context, -1);
      indexPairs(1, context);
    }
  }
}

function createMiterVec(normPrev, normNext) {
  var miterVec = Vector$1.normalize(Vector$1.add(normPrev, normNext));
  var scale = 2 / (1 + Math.abs(Vector$1.dot(normPrev, miterVec)));
  return Vector$1.mult(miterVec, scale * scale);
} // Add a miter vector or a join if the miter is too sharp


function addMiter(v, coordCurr, normPrev, normNext, miter_len_sq, isBeginning, context) {
  var miterVec = createMiterVec(normPrev, normNext); //  Miter limit: if miter join is too sharp, convert to bevel instead

  if (Vector$1.lengthSq(miterVec) > miter_len_sq) {
    addJoin(JOIN_TYPE.bevel, v, coordCurr, normPrev, normNext, isBeginning, context);
  } else {
    addVertex(coordCurr, miterVec, miterVec, 1, v, context, 1);
    addVertex(coordCurr, miterVec, miterVec, 0, v, context, -1);

    if (!isBeginning) {
      indexPairs(1, context);
    }
  }
} // Add a bevel or round join


function addJoin(join_type, v, coordCurr, normPrev, normNext, isBeginning, context) {
  var miterVec = createMiterVec(normPrev, normNext);
  var isClockwise = normNext[0] * normPrev[1] - normNext[1] * normPrev[0] > 0;

  if (context.texcoord_index != null) {
    zero_v[1] = v;
    one_v[1] = v;
  }

  if (isClockwise) {
    addVertex(coordCurr, miterVec, miterVec, 1, v, context, 1);
    addVertex(coordCurr, normPrev, miterVec, 0, v, context, -1);

    if (!isBeginning) {
      indexPairs(1, context);
    }

    addFan(coordCurr, // extrusion vector of first vertex
    Vector$1.neg(normPrev), // controls extrude distance of pivot vertex
    miterVec, // extrusion vector of last vertex
    Vector$1.neg(normNext), // line normal (unused here)
    miterVec, // uv coordinates
    zero_v, one_v, zero_v, false, join_type === JOIN_TYPE.bevel, context);
    addVertex(coordCurr, miterVec, miterVec, 1, v, context, 1);
    addVertex(coordCurr, normNext, miterVec, 0, v, context, -1);
  } else {
    addVertex(coordCurr, normPrev, miterVec, 1, v, context, 1);
    addVertex(coordCurr, miterVec, miterVec, 0, v, context, -1);

    if (!isBeginning) {
      indexPairs(1, context);
    }

    addFan(coordCurr, // extrusion vector of first vertex
    normPrev, // extrusion vector of pivot vertex
    Vector$1.neg(miterVec), // extrusion vector of last vertex
    normNext, // line normal for offset
    miterVec, // uv coordinates
    one_v, zero_v, one_v, false, join_type === JOIN_TYPE.bevel, context);
    addVertex(coordCurr, normNext, miterVec, 1, v, context, 1);
    addVertex(coordCurr, miterVec, miterVec, 0, v, context, -1);
  }
} // Add indices to vertex_elements


function indexPairs(num_pairs, context) {
  var vertex_elements = context.vertex_data.vertex_elements;
  var num_vertices = context.vertex_data.vertex_count;
  var offset = num_vertices - 2 * num_pairs - 2;

  for (var i = 0; i < num_pairs; i++) {
    vertex_elements.push(offset + 2 * i + 2);
    vertex_elements.push(offset + 2 * i + 1);
    vertex_elements.push(offset + 2 * i + 0);
    vertex_elements.push(offset + 2 * i + 2);
    vertex_elements.push(offset + 2 * i + 3);
    vertex_elements.push(offset + 2 * i + 1);
    context.geom_count += 2;
  }
}

function addVertex(position, extrude, normal, u, v, context, flip) {
  var vertex_template = context.vertex_template;
  var vertex_data = context.vertex_data; // set vertex position

  vertex_template[0] = position[0];
  vertex_template[1] = position[1]; // set line extrusion vector

  var len = context.half_width * flip;
  vertex_template[context.extrude_index + 0] = extrude[0] * len;
  vertex_template[context.extrude_index + 1] = extrude[1] * len; // set line offset vector

  if (context.offset) {
    vertex_template[context.offset_index + 0] = normal[0] * context.offset;
    vertex_template[context.offset_index + 1] = normal[1] * context.offset;
  } // set UVs


  if (context.texcoord_index != null) {
    vertex_template[context.texcoord_index + 0] = u * TEXCOORD_NORMALIZE;
    vertex_template[context.texcoord_index + 1] = v * TEXCOORD_NORMALIZE;
  }

  vertex_data.addVertex(vertex_template);
} //  Tesselate a fan geometry between points A ----- B
//  using their normals from a center p      \ . . /
//  and interpolating their UVs               \ p /
//                                             \./
//                                              C


var uvCurr = [0, 0];

function addFan(coord, eA, eC, eB, normal, uvA, uvC, uvB, isCap, isBevel, context) {
  // eA = extrusion vector of first outer vertex
  // eC = extrusion vector of inner vertex
  // eA, eC, eB = extrusion vectors
  // normal = line normal for calculating cap offsets
  // coord = center point p - vertex connecting two line segments
  var cross = eA[0] * eB[1] - eA[1] * eB[0];
  var dot = Vector$1.dot(eA, eB);
  var angle = Math.atan2(cross, dot);

  while (angle >= Math.PI) {
    angle -= 2 * Math.PI;
  }

  if (isBevel) {
    numTriangles = 1;
  } else {
    // vary number of triangles in fan with angle (based on MIN_FAN_WIDTH)
    var numTriangles = trianglesPerArc(angle, context.half_width);

    if (numTriangles < 1) {
      return;
    }
  }

  var pivotIndex = context.vertex_data.vertex_count;
  var vertex_elements = context.vertex_data.vertex_elements;

  if (angle < 0) {
    // cw
    addVertex(coord, eC, normal, uvC[0], uvC[1], context, 1);
    addVertex(coord, eA, normal, uvA[0], uvA[1], context, 1);
  } else {
    // ccw
    addVertex(coord, eC, normal, uvC[0], uvC[1], context, 1);
    addVertex(coord, eA, normal, uvA[0], uvA[1], context, 1);
  }

  var blade = eA;
  var has_texcoord = context.texcoord_index != null;

  if (has_texcoord) {
    if (isCap) {
      var affine_uvCurr = Vector$1.sub(uvA, uvC);
    } else {
      uvCurr = Vector$1.copy(uvA);
      var uv_delta = Vector$1.div(Vector$1.sub(uvB, uvA), numTriangles);
    }
  }

  var angle_step = angle / numTriangles;
  var flip = angle < 0 ? -1 : 1; // if angle < 0, is cw - set 'flip' flag
  // add outside vertices in reverse order depending on sign of angle

  var v1, v2;

  if (cross > 0) {
    v1 = 2;
    v2 = 1;
  } else {
    v1 = 1;
    v2 = 2;
  }

  for (var i = 0; i < numTriangles; i++) {
    if (i === 0 && angle < 0) {
      // if ccw, flip the extrusion vector so offsets work properly
      blade = Vector$1.neg(blade);
    }

    blade = Vector$1.rot(blade, angle_step);

    if (has_texcoord) {
      if (isCap) {
        // UV textures go "through" the cap
        affine_uvCurr = Vector$1.rot(affine_uvCurr, angle_step);
        uvCurr[0] = affine_uvCurr[0] + uvC[0];
        uvCurr[1] = affine_uvCurr[1] * context.texcoord_width * context.v_scale + uvC[1]; // scale the v-coordinate
      } else {
        // UV textures go "around" the join
        uvCurr = Vector$1.add(uvCurr, uv_delta);
      }
    }

    addVertex(coord, blade, normal, uvCurr[0], uvCurr[1], context, flip);
    vertex_elements.push(pivotIndex + i + v1);
    vertex_elements.push(pivotIndex);
    vertex_elements.push(pivotIndex + i + v2);
  }
} //  Function to add the vertices needed for line caps,
//  because to re-use the buffers they need to be at the end


function addCap(coord, v, normal, type, isBeginning, context) {
  var neg_normal = Vector$1.neg(normal);
  var has_texcoord = context.texcoord_index != null;

  switch (type) {
    case CAP_TYPE.square:
      var tangent; // first vertex on the lineString

      if (isBeginning) {
        tangent = [normal[1], -normal[0]];
        addVertex(coord, Vector$1.add(normal, tangent), normal, 1, v, context, 1);
        addVertex(coord, Vector$1.add(neg_normal, tangent), normal, 0, v, context, 1);

        if (has_texcoord) {
          // Add length of square cap to texture coordinate
          v += 0.5 * context.texcoord_width * context.v_scale;
        }

        addVertex(coord, normal, normal, 1, v, context, 1);
        addVertex(coord, neg_normal, normal, 0, v, context, 1);
      } // last vertex on the lineString
      else {
          tangent = [-normal[1], normal[0]];
          addVertex(coord, normal, normal, 1, v, context, 1);
          addVertex(coord, neg_normal, normal, 0, v, context, 1);

          if (has_texcoord) {
            // Add length of square cap to texture coordinate
            v += 0.5 * context.texcoord_width * context.v_scale;
          }

          addVertex(coord, Vector$1.add(normal, tangent), normal, 1, v, context, 1);
          addVertex(coord, Vector$1.add(neg_normal, tangent), normal, 0, v, context, 1);
        }

      indexPairs(1, context);
      break;

    case CAP_TYPE.round:
      // default for end cap, beginning cap will overwrite below (this way we're always passing a non-null value,
      // even if texture coords are disabled)
      var uvA = zero_v,
          uvB = one_v,
          uvC = mid_v;
      var nA, nB; // first vertex on the lineString

      if (isBeginning) {
        nA = normal;
        nB = neg_normal;

        if (has_texcoord) {
          v += 0.5 * context.texcoord_width * context.v_scale;
          uvA = one_v, uvB = zero_v, uvC = mid_v; // update cap UV order
        }
      } // last vertex on the lineString - flip the direction of the cap
      else {
          nA = neg_normal;
          nB = normal;
        }

      if (has_texcoord) {
        zero_v[1] = v, one_v[1] = v, mid_v[1] = v; // update cap UV values
      }

      addFan(coord, nA, zero_vec2, nB, // extrusion normal
      normal, // line normal, for offsets
      uvA, uvC, uvB, // texture coords (ignored if disabled)
      true, false, context);
      break;

    case CAP_TYPE.butt:
      return;
  }
} // Calculate number of triangles for a fan given an angle and line width


function trianglesPerArc(angle, width) {
  if (angle < 0) {
    angle = -angle;
  }

  var numTriangles = width > 2 * MIN_FAN_WIDTH ? Math.log2(width / MIN_FAN_WIDTH) : 1;
  return Math.ceil(angle / Math.PI * numTriangles);
} // Cyclically permute closed line starting at an index


function permuteLine(line, startIndex) {
  var newLine = [];

  for (var i = 0; i < line.length; i++) {
    var index = (i + startIndex) % line.length; // skip the first (repeated) index

    if (index !== 0) {
      newLine.push(line[index]);
    }
  }

  newLine.push(newLine[0]);
  return newLine;
}

// Renders an array specifying a line pattern of alternating dashes and spaces,
// similar to an SVG `dasharray` or Canvas setLineDash(), into a byte array of RGBA pixels
// Returns:
// {
//    pixel: rendered image in Uint8Array buffer
//    length: pixel length of rendered dash pattern (sum of all dashes and spaces)
// }
//
// https://www.w3.org/TR/SVG/painting.html#StrokeDasharrayProperty
// https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/stroke-dasharray
// https://developer.mozilla.org/en-US/docs/Web/API/CanvasRenderingContext2D/setLineDash
var default_dash_color = [255, 255, 255, 255];
var default_background_color = [0, 0, 0, 0];
function renderDashArray(pattern, options) {
  if (options === void 0) {
    options = {};
  }

  var dash_pixel = options.dash_color || default_dash_color;
  var background_color = options.background_color || default_background_color;
  var dashes = pattern;
  var scale = options.scale || 1; // If pattern is odd, repeat it to make it even (see SVG spec)

  if (dashes.length % 2 === 1) {
    Array.prototype.push.apply(dashes, dashes);
  }

  var dash = true;
  var pixels = [];

  for (var i = 0; i < dashes.length; i++) {
    var segment = Math.floor(dashes[i] * scale);

    for (var s = 0; s < segment; s++) {
      Array.prototype.push.apply(pixels, dash ? dash_pixel : background_color);
    }

    dash = !dash; // alternate between dashes and spaces
  }

  pixels.reverse(); // flip Y (GL textures are upside down)

  pixels = new Uint8Array(pixels); // convert to typed array

  var length = pixels.length / 4; // one RGBA byte sequences to one pixel

  return {
    pixels: pixels,
    length: length
  };
}

var Lines = Object.create(Style);
Lines.variants = {}; // mesh variants by variant key

Lines.vertex_layouts = {}; // vertex layouts by variant key

var DASH_SCALE = 20; // adjustment factor for UV scale to for line dash patterns w/fractional pixel width

Object.assign(Lines, {
  name: 'lines',
  built_in: true,
  vertex_shader_src: polygons_vs,
  fragment_shader_src: polygons_fs,
  selection: true,
  // enable feature selection
  init: function init() {
    Style.init.apply(this, arguments); // Tell the shader we want a order in vertex attributes, and to extrude lines

    this.defines.TANGRAM_EXTRUDE_LINES = true;
    this.defines.TANGRAM_TEXTURE_COORDS = true; // texcoords attribute is set to static when not needed
    // Additional single-allocated object used for holding outline style as it is processed
    // Separate from this.feature_style so that outline properties do not overwrite calculated
    // inline properties (outline call is made *within* the inline call)

    this.outline_feature_style = {};
    this.inline_feature_style = this.feature_style; // save reference to main computed style object

    this.dash_textures = {}; // cache previously rendered line dash pattern textures
  },
  // Calculate width or offset at zoom given in `context`
  calcDistance: function calcDistance(prop, context) {
    return StyleParser.evalCachedDistanceProperty(prop, context) || 0;
  },
  // Calculate width or offset at next zoom (used for zoom-based interpolation in shader)
  calcDistanceNextZoom: function calcDistanceNextZoom(prop, context) {
    context.zoom++;
    var val = this.calcDistance(prop, context);
    context.zoom--;
    return val;
  },
  // Calculate width at current and next zoom, and scaling factor between
  calcWidth: function calcWidth(draw, style, context) {
    // line width in meters
    var width = this.calcDistance(draw.width, context);

    if (width < 0) {
      return; // skip lines with negative width
    }

    var next_width;

    if (draw.next_width) {
      next_width = this.calcDistanceNextZoom(draw.next_width, context);
    } else {
      next_width = width / 2; // when width is static, width at next zoom is just half as many tile units
    }

    if (width === 0 && next_width === 0 || next_width < 0) {
      return false; // skip lines that don't interpolate to a positive value at next zoom
    } // these values are saved for later calculating the outline width, which needs to add the base line's width


    style.width_unscaled = width;
    style.next_width_unscaled = next_width; // calculate relative change in line width between zooms
    // interpolate from the line width at the zoom mid-point, towards/away from the previous/next integer zoom

    if (draw.next_width) {
      next_width *= 2; // NB: a given width is twice as big in screen space at the next zoom

      var mid_width = (width + next_width) * 0.5;
      style.width = mid_width * context.units_per_meter_overzoom; // width at zoom mid-point

      style.width_scale = 1 - next_width / mid_width;
    } else {
      style.width = width * context.units_per_meter_overzoom;
      style.width_scale = 0;
    } // optional adjustment to texcoord width based on scale


    if (draw.texcoords) {
      // when drawing an outline, use the inline's texture scale
      // (e.g. keeps dashed outline pattern locked to inline pattern)
      if (draw.inline_texcoord_width) {
        style.texcoord_width = draw.inline_texcoord_width;
      } // when drawing an inline, calculate UVs based on line width
      else {
          // UVs can't calc for zero-width, use next zoom width in that case
          style.texcoord_width = (style.width_unscaled || style.next_width_unscaled) * context.units_per_meter_overzoom / context.tile.overzoom2; // shorten calcs
        }
    }

    return true;
  },
  // Calculate offset at current and next zoom, and scaling factor between
  calcOffset: function calcOffset(draw, style, context) {
    // Pre-calculated offset passed
    // This happens when a line passes pre-computed offset values to its outline
    if (draw.offset_precalc) {
      style.offset = draw.offset_precalc;
      style.offset_scale = draw.offset_scale_precalc;
    } // Offset to calculate
    else if (draw.offset) {
        var offset = this.calcDistance(draw.offset, context);

        if (draw.next_offset) {
          var next_offset = this.calcDistanceNextZoom(draw.next_offset, context) * 2;

          if (Math.abs(offset) >= Math.abs(next_offset)) {
            style.offset = offset * context.units_per_meter_overzoom;

            if (offset !== 0) {
              style.offset_scale = 1 - next_offset / offset;
            } else {
              style.offset_scale = 0;
            }
          } else {
            style.offset = next_offset * context.units_per_meter_overzoom;

            if (next_offset !== 0) {
              style.offset_scale = (1 - offset / next_offset) * -1;
            } else {
              style.offset_scale = 0;
            }
          }
        } else {
          style.offset = offset * context.units_per_meter_overzoom;
          style.offset_scale = 0;
        }
      } // No offset
      else {
          style.offset = 0;
          style.offset_scale = 0;
        }
  },
  _parseFeature: function _parseFeature(feature, draw, context) {
    var style = this.feature_style; // calculate line width at current and next zoom

    if (this.calcWidth(draw, style, context) === false) {
      return; // missing or zero width
    } // calculate line offset at current and next zoom


    this.calcOffset(draw, style, context);
    style.color = this.parseColor(draw.color, context);

    if (!style.color) {
      return;
    }

    style.variant = draw.variant; // pre-calculated mesh variant
    // height defaults to feature height, but extrude style can dynamically adjust height by returning a number or array (instead of a boolean)

    style.z = StyleParser.evalCachedDistanceProperty(draw.z, context) || StyleParser.defaults.z;
    style.height = feature.properties.height || StyleParser.defaults.height;
    style.extrude = StyleParser.evalProperty(draw.extrude, context);

    if (style.extrude) {
      if (typeof style.extrude === 'number') {
        style.height = style.extrude;
      } else if (Array.isArray(style.extrude)) {
        style.height = style.extrude[1];
      }
    } // Raise line height if extruded


    if (style.extrude && style.height) {
      style.z += style.height;
    }

    style.z *= Geo$1.height_scale; // provide sub-meter precision of height values

    style.cap = draw.cap;
    style.join = draw.join;
    style.miter_limit = draw.miter_limit;
    style.tile_edges = draw.tile_edges; // usually activated for debugging, or rare visualization needs
    // Construct an outline style
    // Reusable outline style object, marked as already wrapped in cache objects (preprocessed = true)

    style.outline = style.outline || {
      width: {},
      next_width: {},
      preprocessed: true
    };

    if (draw.outline && draw.outline.visible !== false && draw.outline.color && draw.outline.width) {
      // outline width in meters
      // NB: multiply by 2 because outline is applied on both sides of line
      var outline_width = this.calcDistance(draw.outline.width, context) * 2;
      var outline_next_width = this.calcDistanceNextZoom(draw.outline.next_width, context) * 2;

      if (outline_width === 0 && outline_next_width === 0 || outline_width < 0 || outline_next_width < 0) {
        // skip lines that don't interpolate between zero or greater width
        style.outline.width.value = null;
        style.outline.next_width.value = null;
        style.outline.color = null;
        style.outline.inline_texcoord_width = null;
        style.outline.texcoords = false;
      } else {
        // Maintain consistent outline width around the line fill
        style.outline.width.value = outline_width + style.width_unscaled;
        style.outline.next_width.value = outline_next_width + style.next_width_unscaled;
        style.outline.inline_texcoord_width = style.texcoord_width; // Offset is directly copied from fill to outline, no need to re-calculate it

        style.outline.offset_precalc = style.offset;
        style.outline.offset_scale_precalc = style.offset_scale; // Inherited properties

        style.outline.color = draw.outline.color;
        style.outline.interactive = draw.outline.interactive;
        style.outline.cap = draw.outline.cap;
        style.outline.join = draw.outline.join;
        style.outline.miter_limit = draw.outline.miter_limit;
        style.outline.texcoords = draw.outline.texcoords;
        style.outline.style = draw.outline.style;
        style.outline.variant = draw.outline.variant; // Explicitly defined outline order, or inherited from inner line

        if (draw.outline.order) {
          style.outline.order = this.parseOrder(draw.outline.order, context);
        } else {
          style.outline.order = style.order;
        } // Don't let outline be above inner line


        if (style.outline.order > style.order) {
          style.outline.order = style.order;
        } // Outlines are always at half-layer intervals to avoid conflicting with inner lines


        style.outline.order -= 0.5;
      }
    } else {
      style.outline.width.value = null;
      style.outline.next_width.value = null;
      style.outline.color = null;
      style.outline.inline_texcoord_width = null;
    }

    return style;
  },
  _preprocess: function _preprocess(draw) {
    draw.color = StyleParser.createColorPropertyCache(draw.color);
    draw.width = StyleParser.createPropertyCache(draw.width, StyleParser.parseUnits);

    if (draw.width && draw.width.type !== StyleParser.CACHE_TYPE.STATIC) {
      draw.next_width = StyleParser.createPropertyCache(draw.width, StyleParser.parseUnits);
    }

    draw.offset = draw.offset && StyleParser.createPropertyCache(draw.offset, StyleParser.parseUnits);

    if (draw.offset && draw.offset.type !== StyleParser.CACHE_TYPE.STATIC) {
      draw.next_offset = StyleParser.createPropertyCache(draw.offset, StyleParser.parseUnits);
    }

    draw.z = StyleParser.createPropertyCache(draw.z, StyleParser.parseUnits);
    draw.dash = draw.dash !== undefined ? draw.dash : this.dash;
    draw.dash_key = draw.dash && this.dashTextureKey(draw.dash);
    draw.dash_background_color = draw.dash_background_color !== undefined ? draw.dash_background_color : this.dash_background_color;
    draw.dash_background_color = draw.dash_background_color && StyleParser.parseColor(draw.dash_background_color);
    draw.texture_merged = draw.dash_key || (draw.texture !== undefined ? draw.texture : this.texture);
    draw.texcoords = this.texcoords || draw.texture_merged ? 1 : 0;
    this.computeVariant(draw);

    if (draw.outline) {
      draw.outline.is_outline = true; // mark as outline (so mesh variant can be adjusted for render order, etc.)

      draw.outline.style = draw.outline.style || this.name;
      draw.outline.color = StyleParser.createColorPropertyCache(draw.outline.color);
      draw.outline.width = StyleParser.createPropertyCache(draw.outline.width, StyleParser.parseUnits);
      draw.outline.next_width = StyleParser.createPropertyCache(draw.outline.width, StyleParser.parseUnits); // width re-computed for next zoom

      draw.outline.interactive = draw.outline.interactive != null ? draw.outline.interactive : draw.interactive;
      draw.outline.cap = draw.outline.cap || draw.cap;
      draw.outline.join = draw.outline.join || draw.join;
      draw.outline.miter_limit = draw.outline.miter_limit != null ? draw.outline.miter_limit : draw.miter_limit;
      draw.outline.offset = draw.offset; // always apply inline offset to outline
      // outline inhertits dash pattern, but NOT explicit texture

      var outline_style = this.styles[draw.outline.style];

      if (outline_style) {
        draw.outline.dash = draw.outline.dash !== undefined ? draw.outline.dash : outline_style.dash;
        draw.outline.texture = draw.outline.texture !== undefined ? draw.outline.texture : outline_style.texture;

        if (draw.outline.dash != null) {
          // dash was defined by outline draw or style
          draw.outline.dash_key = draw.outline.dash && this.dashTextureKey(draw.outline.dash);
          draw.outline.texture_merged = draw.outline.dash_key;
        } else if (draw.outline.dash === null) {
          // dash explicitly disabled by outline draw or style
          draw.outline.dash_key = null;
          draw.outline.texture_merged = draw.outline.texture;
        } else if (draw.outline.texture != null) {
          // texture was defined by outline draw or style
          draw.outline.dash_key = null; // outline explicitly turning off dash

          draw.outline.texture_merged = draw.outline.texture;
        } else {
          // no dash or texture defined for outline, inherit parent dash
          draw.outline.dash = draw.dash;
          draw.outline.dash_key = draw.outline.dash && this.dashTextureKey(draw.outline.dash);
          draw.outline.texture_merged = draw.outline.dash_key;
        }

        draw.outline.dash_background_color = draw.outline.dash_background_color !== undefined ? draw.outline.dash_background_color : outline_style.dash_background_color;
        draw.outline.dash_background_color = draw.outline.dash_background_color !== undefined ? draw.outline.dash_background_color : draw.dash_background_color;
        draw.outline.dash_background_color = draw.outline.dash_background_color && StyleParser.parseColor(draw.outline.dash_background_color);
        draw.outline.texcoords = outline_style.texcoords || draw.outline.texture_merged ? 1 : 0; // outline inherits draw blend order from parent inline, unless explicitly turned off with null

        if (draw.outline.blend_order === undefined && draw.blend_order != null) {
          draw.outline.blend_order = draw.blend_order;
        }

        this.computeVariant(draw.outline, outline_style);
      } else {
        log({
          level: 'warn',
          once: true
        }, "Layer group '" + draw.layers.join(', ') + "': " + ("line 'outline' specifies non-existent draw style '" + draw.outline.style + "' (or maybe the style is ") + 'defined but is missing a \'base\' or has another error), skipping outlines for features matching this layer group');
        draw.outline = null;
      }
    }

    return draw;
  },
  // Unique string key for a dash pattern (used as texture name)
  dashTextureKey: function dashTextureKey(dash) {
    return '__dash_' + JSON.stringify(dash);
  },
  // Return or render a dash pattern texture
  getDashTexture: function getDashTexture(dash) {
    var dash_key = this.dashTextureKey(dash);

    if (this.dash_textures[dash_key] == null) {
      this.dash_textures[dash_key] = true; // Render line pattern

      var dash_texture = renderDashArray(dash, {
        scale: DASH_SCALE
      });
      Texture.create(this.gl, dash_key, {
        data: dash_texture.pixels,
        height: dash_texture.length,
        width: 1,
        filtering: 'nearest'
      });
    }
  },
  // Override
  endData: function endData(tile) {
    return new Promise(function ($return, $error) {
      var _this, tile_data, pending, _loop;

      _this = this;
      return Promise.resolve(Style.endData.call(this, tile)).then(function ($await_9) {
        try {
          tile_data = $await_9;

          if (tile_data) {
            tile_data.uniforms.u_has_line_texture = false;
            tile_data.uniforms.u_texture = Texture.default;
            tile_data.uniforms.u_v_scale_adjust = Geo$1.tile_scale;
            pending = [];

            _loop = function _loop(m) {
              return new Promise(function ($return, $error) {
                var variant, uniforms, texture;
                variant = tile_data.meshes[m].variant;

                if (variant.texture) {
                  uniforms = tile_data.meshes[m].uniforms = tile_data.meshes[m].uniforms || {};
                  uniforms.u_has_line_texture = true;
                  uniforms.u_texture = variant.texture;
                  uniforms.u_texture_ratio = 1;

                  if (variant.dash) {
                    uniforms.u_v_scale_adjust = Geo$1.tile_scale * DASH_SCALE;
                    uniforms.u_dash_background_color = variant.dash_background_color || [0, 0, 0, 0];
                  }

                  if (variant.dash_key && _this.dash_textures[variant.dash_key] == null) {
                    _this.dash_textures[variant.dash_key] = true;

                    var $Try_1_Post = function () {
                      try {
                        return $If_4.call(this);
                      } catch ($boundEx) {
                        return $error($boundEx);
                      }
                    }.bind(this);

                    var $Try_1_Catch = function (e) {
                      try {
                        log('trace', _this.name + ": line dash texture create failed because style no longer on main thread");
                        return $Try_1_Post();
                      } catch ($boundEx) {
                        return $error($boundEx);
                      }
                    };

                    try {
                      return Promise.resolve(WorkerBroker$1.postMessage(_this.main_thread_target + '.getDashTexture', variant.dash)).then(function ($await_10) {
                        try {
                          return $Try_1_Post();
                        } catch ($boundEx) {
                          return $Try_1_Catch($boundEx);
                        }
                      }, $Try_1_Catch);
                    } catch (e) {
                      $Try_1_Catch(e);
                    }
                  }

                  function $If_4() {
                    if (Texture.textures[variant.texture] == null) {
                      pending.push(Texture.syncTexturesToWorker([variant.texture]).then(function (textures) {
                        var texture = textures[variant.texture];

                        if (texture) {
                          uniforms.u_texture_ratio = texture.height / texture.width;
                        }
                      }));
                    } else {
                      texture = Texture.textures[variant.texture];
                      uniforms.u_texture_ratio = texture.height / texture.width;
                    }

                    return $If_3.call(this);
                  }

                  return $If_4.call(this);
                }

                function $If_3() {
                  return $return();
                }

                return $If_3.call(this);
              });
            };

            var $idx_5,
                $in_6 = [];

            for ($idx_5 in tile_data.meshes) $in_6.push($idx_5);

            var m;
            var $Loop_7_trampoline;

            function $Loop_7() {
              if ($in_6.length) {
                m = $in_6.shift();
                return Promise.resolve(_loop(m)).then(function ($await_11) {
                  try {
                    return $Loop_7;
                  } catch ($boundEx) {
                    return $error($boundEx);
                  }
                }, $error);
              } else return [1];
            }

            return ($Loop_7_trampoline = function (q) {
              while (q) {
                if (q.then) return void q.then($Loop_7_trampoline, $error);

                try {
                  if (q.pop) {
                    if (q.length) return q.pop() ? $Loop_7_exit.call(this) : q;else q = $Loop_7;
                  } else q = q.call(this);
                } catch (_exception) {
                  return $error(_exception);
                }
              }
            }.bind(this))($Loop_7);

            function $Loop_7_exit() {
              return Promise.resolve(Promise.all(pending)).then(function ($await_12) {
                try {
                  return $If_2.call(this);
                } catch ($boundEx) {
                  return $error($boundEx);
                }
              }.bind(this), $error);
            }
          }

          function $If_2() {
            return $return(tile_data);
          }

          return $If_2.call(this);
        } catch ($boundEx) {
          return $error($boundEx);
        }
      }.bind(this), $error);
    }.bind(this));
  },
  // Calculate and store mesh variant (unique by draw group but not feature)
  computeVariant: function computeVariant(draw, style) {
    if (style === void 0) {
      style = this;
    }

    // Factors that determine a unique mesh rendering variant
    var key = draw.offset ? 1 : 0; // whether feature has a line offset

    key += '/' + draw.texcoords; // whether feature has texture UVs

    key += '/' + (draw.interactive ? 1 : 0); // whether feature has interactivity

    key += '/' + (draw.extrude || draw.z ? 1 : 0); // whether feature has a z coordinate

    key += '/' + draw.is_outline; // whether this is an outline of a line feature

    if (draw.dash_key) {
      // whether feature has a line dash pattern
      key += draw.dash_key;

      if (draw.dash_background_color) {
        key += draw.dash_background_color;
      }
    }

    if (draw.texture_merged) {
      // whether feature has a line texture
      key += draw.texture_merged;
    }

    var blend_order = style.getBlendOrderForDraw(draw);
    key += '/' + blend_order; // Create unique key

    key = hashString(key);
    draw.variant = key;

    if (Lines.variants[key] == null) {
      Lines.variants[key] = {
        key: key,
        blend_order: blend_order,
        mesh_order: draw.is_outline ? 0 : 1,
        // outlines should be drawn first, so inline is on top
        selection: draw.interactive ? 1 : 0,
        offset: draw.offset ? 1 : 0,
        z_or_offset: draw.offset || draw.extrude || draw.z ? 1 : 0,
        texcoords: draw.texcoords,
        texture: draw.texture_merged,
        dash: draw.dash,
        dash_key: draw.dash_key,
        dash_background_color: draw.dash_background_color
      };
    }
  },
  // Override
  // Create or return desired vertex layout permutation based on flags
  vertexLayoutForMeshVariant: function vertexLayoutForMeshVariant(variant) {
    if (Lines.vertex_layouts[variant.key] == null) {
      // Attributes for this mesh variant
      // Optional attributes have placeholder values assigned with `static` parameter
      var attribs = [{
        name: 'a_position',
        size: 4,
        type: gl$1.SHORT,
        normalized: false
      }, {
        name: 'a_extrude',
        size: 2,
        type: gl$1.SHORT,
        normalized: false
      }, {
        name: 'a_offset',
        size: 2,
        type: gl$1.SHORT,
        normalized: false,
        static: variant.offset ? null : [0, 0]
      }, {
        name: 'a_z_and_offset_scale',
        size: 2,
        type: gl$1.SHORT,
        normalized: false,
        static: variant.z_or_offset ? null : [0, 0]
      }, {
        name: 'a_texcoord',
        size: 2,
        type: gl$1.UNSIGNED_SHORT,
        normalized: true,
        static: variant.texcoords ? null : [0, 0]
      }, {
        name: 'a_color',
        size: 4,
        type: gl$1.UNSIGNED_BYTE,
        normalized: true
      }, {
        name: 'a_selection_color',
        size: 4,
        type: gl$1.UNSIGNED_BYTE,
        normalized: true,
        static: variant.selection ? null : [0, 0, 0, 0]
      }];
      Lines.vertex_layouts[variant.key] = new VertexLayout(attribs);
    }

    return Lines.vertex_layouts[variant.key];
  },
  // Override
  meshVariantTypeForDraw: function meshVariantTypeForDraw(draw) {
    return Lines.variants[draw.variant]; // return pre-calculated mesh variant
  },

  /**
   * A "template" that sets constant attibutes for each vertex, which is then modified per vertex or per feature.
   * A plain JS array matching the order of the vertex layout.
   */
  makeVertexTemplate: function makeVertexTemplate(style, mesh) {
    var i = 0; // a_position.xy - vertex position
    // a_position.z - line width scaling factor
    // a_position.w - layer order

    this.vertex_template[i++] = 0;
    this.vertex_template[i++] = 0;
    this.vertex_template[i++] = style.width_scale * 1024;
    this.vertex_template[i++] = this.scaleOrder(style.order); // a_extrude.xy - extrusion vector (vertex extrusion away from center of line)

    this.vertex_template[i++] = 0;
    this.vertex_template[i++] = 0; // a_offset.xy - normal vector
    // offset can be static or dynamic depending on style

    if (mesh.variant.offset) {
      this.vertex_template[i++] = 0;
      this.vertex_template[i++] = 0;
    } // a_z_and_offset_scale.xy


    if (mesh.variant.z_or_offset) {
      this.vertex_template[i++] = style.z || 0; // feature z position

      this.vertex_template[i++] = style.offset_scale * 1024; // line offset scaling factor
    } // a_texcoord.uv - texture coordinates


    if (mesh.variant.texcoords) {
      this.vertex_template[i++] = 0;
      this.vertex_template[i++] = 0;
    } // a_color.rgba - feature color


    this.vertex_template[i++] = style.color[0] * 255;
    this.vertex_template[i++] = style.color[1] * 255;
    this.vertex_template[i++] = style.color[2] * 255;
    this.vertex_template[i++] = style.color[3] * 255; // a_selection_color.rgba - selection color

    if (mesh.variant.selection) {
      this.vertex_template[i++] = style.selection_color[0] * 255;
      this.vertex_template[i++] = style.selection_color[1] * 255;
      this.vertex_template[i++] = style.selection_color[2] * 255;
      this.vertex_template[i++] = style.selection_color[3] * 255;
    }

    return this.vertex_template;
  },
  buildLines: function buildLines(lines, style, context, options) {
    // Outline (build first so that blended geometry without a depth test is drawn first/under the inner line)
    this.feature_style = this.outline_feature_style; // swap in outline-specific style holder

    if (style.outline && style.outline.color != null && style.outline.width.value != null) {
      var outline_style = this.styles[style.outline.style];

      if (outline_style) {
        outline_style.addFeature(context.feature, style.outline, context);
      }
    } // Main line


    this.feature_style = this.inline_feature_style; // restore calculated style for inline

    var mesh = this.getTileMesh(context.tile, this.meshVariantTypeForDraw(style));
    var vertex_data = mesh.vertex_data;
    var vertex_layout = vertex_data.vertex_layout;
    var vertex_template = this.makeVertexTemplate(style, mesh);
    return buildPolylines(lines, style, vertex_data, vertex_template, vertex_layout.index, options && options.closed_polygon, // closed_polygon
    !style.tile_edges && options && options.remove_tile_edges, // remove_tile_edges
    Geo$1.tile_scale * context.tile.pad_scale * 2 // tile_edge_tolerance
    );
  },
  buildPolygons: function buildPolygons(polygons, style, context) {
    // Render polygons as individual lines
    var geom_count = 0;

    for (var p = 0; p < polygons.length; p++) {
      geom_count += this.buildLines(polygons[p], style, context, {
        closed_polygon: true,
        remove_tile_edges: true
      });
    }

    return geom_count;
  }
});

_export(_export.P + _export.F * _fails(function () {
  return new Date(NaN).toJSON() !== null
    || Date.prototype.toJSON.call({ toISOString: function () { return 1; } }) !== 1;
}), 'Date', {
  // eslint-disable-next-line no-unused-vars
  toJSON: function toJSON(key) {
    var O = _toObject(this);
    var pv = _toPrimitive(O);
    return typeof pv == 'number' && !isFinite(pv) ? null : O.toISOString();
  }
});

// B.2.3.2 String.prototype.anchor(name)
_stringHtml('anchor', function (createHTML) {
  return function anchor(name) {
    return createHTML(this, 'a', 'name', name);
  };
});

// Point builders

var pre_angles_normalize = 128 / Math.PI;
var angles_normalize = 16384 / Math.PI;
var offsets_normalize = 64;
var texcoord_normalize = 65535;
var size_normalize = 128; // width/height are 8.8 fixed-point, but are halved (so multiply by 128 instead of 256)
// These index values map a 4-element vertex position counter from this pattern (used for size and UVs):
//  [min_x, min_y, max_x, max_y]
// to this pattern:
//  [min_x, min_y],
//  [max_x, min_y],
//  [max_x, max_y],
//  [min_x, max_y]

var ix = [0, 2, 2, 0];
var iy = [1, 1, 3, 3];
var shape = new Array(4); // single, reusable allocation
// Build a billboard sprite quad centered on a point. Sprites are intended to be drawn in screenspace, and have
// properties for width, height, angle, and texture UVs. Curved label segment sprites have additional properties
// for interpolating their position and angle across zooms.

function buildQuadForPoint(point, vertex_data, vertex_template, vindex, size, offset, offsets, pre_angles, angle, angles, texcoords, curve) {
  // Half-sized point dimensions in fixed point
  var w2 = size[0] * size_normalize;
  var h2 = size[1] * size_normalize;
  shape[0] = -w2;
  shape[1] = -h2;
  shape[2] = w2;
  shape[3] = h2;
  var uvs = texcoords || default_uvs;
  var vertex_elements = vertex_data.vertex_elements;
  var element_offset = vertex_data.vertex_count;

  for (var p = 0; p < 4; p++) {
    vertex_template[vindex.a_position + 0] = point[0];
    vertex_template[vindex.a_position + 1] = point[1];
    vertex_template[vindex.a_shape + 0] = shape[ix[p]];
    vertex_template[vindex.a_shape + 1] = shape[iy[p]];
    vertex_template[vindex.a_shape + 2] = angle;
    vertex_template[vindex.a_offset + 0] = offset[0];
    vertex_template[vindex.a_offset + 1] = offset[1]; // Add texcoords

    if (vindex.a_texcoord) {
      vertex_template[vindex.a_texcoord + 0] = uvs[ix[p]] * texcoord_normalize;
      vertex_template[vindex.a_texcoord + 1] = uvs[iy[p]] * texcoord_normalize;
    } // Add curved label segment props


    if (curve) {
      // 1 byte (signed) range: [-127, 128]
      // actual range: [-2pi, 2pi]
      // total: multiply by 128 / (2 PI)
      vertex_template[vindex.a_pre_angles + 0] = pre_angles_normalize * pre_angles[0];
      vertex_template[vindex.a_pre_angles + 1] = pre_angles_normalize * pre_angles[1];
      vertex_template[vindex.a_pre_angles + 2] = pre_angles_normalize * pre_angles[2];
      vertex_template[vindex.a_pre_angles + 3] = pre_angles_normalize * pre_angles[3]; // 2 byte (signed) of resolution [-32767, 32768]
      // actual range: [-2pi, 2pi]
      // total: multiply by 32768 / (2 PI) = 16384 / PI

      vertex_template[vindex.a_angles + 0] = angles_normalize * angles[0];
      vertex_template[vindex.a_angles + 1] = angles_normalize * angles[1];
      vertex_template[vindex.a_angles + 2] = angles_normalize * angles[2];
      vertex_template[vindex.a_angles + 3] = angles_normalize * angles[3]; // offset range can be [0, 65535]
      // actual range: [0, 1024]

      vertex_template[vindex.a_offsets + 0] = offsets_normalize * offsets[0];
      vertex_template[vindex.a_offsets + 1] = offsets_normalize * offsets[1];
      vertex_template[vindex.a_offsets + 2] = offsets_normalize * offsets[2];
      vertex_template[vindex.a_offsets + 3] = offsets_normalize * offsets[3];
    }

    vertex_data.addVertex(vertex_template);
  }

  vertex_elements.push(element_offset + 0);
  vertex_elements.push(element_offset + 1);
  vertex_elements.push(element_offset + 2);
  vertex_elements.push(element_offset + 2);
  vertex_elements.push(element_offset + 3);
  vertex_elements.push(element_offset + 0);
  return 2; // geom count is always two triangles, for one quad
}

// Sets of values to match for directional and corner anchors
var lefts = ['left', 'top-left', 'bottom-left'];
var rights = ['right', 'top-right', 'bottom-right'];
var tops = ['top', 'top-left', 'top-right'];
var bottoms = ['bottom', 'bottom-left', 'bottom-right'];
var PointAnchor = {
  computeOffset: function computeOffset(offset, size, anchor, buffer) {
    if (buffer === void 0) {
      buffer = null;
    }

    if (!anchor || anchor === 'center') {
      return offset;
    }

    var offset2 = [offset[0], offset[1]];
    buffer = buffer || this.default_buffer; // An optional left/right offset

    if (this.isLeftAnchor(anchor)) {
      offset2[0] -= size[0] / 2;

      if (anchor === 'left') {
        offset2[0] -= buffer[0];
      }
    } else if (this.isRightAnchor(anchor)) {
      offset2[0] += size[0] / 2;

      if (anchor === 'right') {
        offset2[0] += buffer[1];
      }
    } // An optional top/bottom offset


    if (this.isTopAnchor(anchor)) {
      offset2[1] -= size[1] / 2;

      if (anchor === 'top') {
        offset2[1] -= buffer[2];
      }
    } else if (this.isBottomAnchor(anchor)) {
      offset2[1] += size[1] / 2;

      if (anchor === 'bottom') {
        offset2[1] += buffer[3];
      }
    }

    return offset2;
  },
  alignForAnchor: function alignForAnchor(anchor) {
    if (anchor && anchor !== 'center') {
      if (this.isLeftAnchor(anchor)) {
        return 'right';
      } else if (this.isRightAnchor(anchor)) {
        return 'left';
      }
    }

    return 'center';
  },
  isLeftAnchor: function isLeftAnchor(anchor) {
    return lefts.indexOf(anchor) > -1;
  },
  isRightAnchor: function isRightAnchor(anchor) {
    return rights.indexOf(anchor) > -1;
  },
  isTopAnchor: function isTopAnchor(anchor) {
    return tops.indexOf(anchor) > -1;
  },
  isBottomAnchor: function isBottomAnchor(anchor) {
    return bottoms.indexOf(anchor) > -1;
  },
  // Buffers: [left, right, top, bottom]
  default_buffer: [2.5, 2.5, 1.5, 0.75],
  zero_buffer: [0, 0, 0, 0]
};

// Do AABB `a` and `b` intersect?
function boxIntersectsBox(a, b) {
  if (a[2] < b[0] || // a is left of b
  a[0] > b[2] || // a is right of b
  a[3] < b[1] || // a is above b
  a[1] > b[3]) {
    // a is below b
    return false;
  }

  return true; // boxes overlap
} // Does AABB `a` intersect any of the AABBs in array `boxes`?
// Invokes `callback` with index of intersecting box
// Stops intersecting if `callback` returns non-null value (continues otherwise)

function boxIntersectsList(a, boxes, callback) {
  for (var i = 0; i < boxes.length; i++) {
    if (boxIntersectsBox(a, boxes[i])) {
      if (callback(i) != null) {
        break;
      }
    }
  }
}

var ZERO_AXES = [[1, 0], [0, 1]];
var proj_a = [],
    proj_b = [];
var d0, d1, d2, d3;

var OBB =
/*#__PURE__*/
function () {
  function OBB(x, y, a, w, h) {
    this.dimension = [w / 2, h / 2]; // store half-dimension as that's what's needed in calculations below

    this.angle = a;
    this.centroid = [x, y];
    this.quad = null;
    this.axis_0 = null;
    this.axis_1 = null;
    this.update();
  }

  var _proto = OBB.prototype;

  _proto.toJSON = function toJSON() {
    return {
      x: this.centroid[0],
      y: this.centroid[1],
      a: this.angle,
      w: this.dimension[0],
      h: this.dimension[1]
    };
  };

  _proto.getExtent = function getExtent() {
    // special handling to skip calculations for 0-angle
    if (this.angle === 0) {
      return [this.quad[0], this.quad[1], // lower-left
      this.quad[4], this.quad[5] // upper-right
      ];
    }

    var aabb = [Math.min(this.quad[0], this.quad[2], this.quad[4], this.quad[6]), // min x
    Math.min(this.quad[1], this.quad[3], this.quad[5], this.quad[7]), // min y
    Math.max(this.quad[0], this.quad[2], this.quad[4], this.quad[6]), // max x
    Math.max(this.quad[1], this.quad[3], this.quad[5], this.quad[7]) // max y
    ];
    return aabb;
  };

  _proto.updateAxes = function updateAxes() {
    // upper-left to upper-right
    this.axis_0 = Vector$1.normalize([this.quad[4] - this.quad[6], this.quad[5] - this.quad[7]]); // lower-right to upper-right

    this.axis_1 = Vector$1.normalize([this.quad[4] - this.quad[2], this.quad[5] - this.quad[3]]);
  };

  _proto.update = function update() {
    var c = this.centroid;
    var w2 = this.dimension[0];
    var h2 = this.dimension[1]; // special handling to skip calculations for 0-angle

    if (this.angle === 0) {
      // quad is a flat array storing 4 [x, y] vectors
      this.quad = [c[0] - w2, c[1] - h2, // lower-left
      c[0] + w2, c[1] - h2, // lower-right
      c[0] + w2, c[1] + h2, // upper-right
      c[0] - w2, c[1] + h2 // upper-left
      ];
      this.axis_0 = ZERO_AXES[0];
      this.axis_1 = ZERO_AXES[1];
    } // calculate axes and enclosing quad
    else {
        var x0 = Math.cos(this.angle) * w2;
        var x1 = Math.sin(this.angle) * w2;
        var y0 = -Math.sin(this.angle) * h2;
        var y1 = Math.cos(this.angle) * h2; // quad is a flat array storing 4 [x, y] vectors

        this.quad = [c[0] - x0 - y0, c[1] - x1 - y1, // lower-left
        c[0] + x0 - y0, c[1] + x1 - y1, // lower-right
        c[0] + x0 + y0, c[1] + x1 + y1, // upper-right
        c[0] - x0 + y0, c[1] - x1 + y1 // upper-left
        ];
        this.updateAxes();
      }
  };

  OBB.projectToAxis = function projectToAxis(obb, axis, proj) {
    // for each axis, project obb quad to it and find min and max values
    var quad = obb.quad;
    d0 = quad[0] * axis[0] + quad[1] * axis[1];
    d1 = quad[2] * axis[0] + quad[3] * axis[1];
    d2 = quad[4] * axis[0] + quad[5] * axis[1];
    d3 = quad[6] * axis[0] + quad[7] * axis[1];
    proj[0] = Math.min(d0, d1, d2, d3);
    proj[1] = Math.max(d0, d1, d2, d3);
    return proj;
  };

  OBB.axisCollide = function axisCollide(obb_a, obb_b, axis_0, axis_1) {
    OBB.projectToAxis(obb_a, axis_0, proj_a);
    OBB.projectToAxis(obb_b, axis_0, proj_b);

    if (proj_b[0] > proj_a[1] || proj_b[1] < proj_a[0]) {
      return false;
    }

    OBB.projectToAxis(obb_a, axis_1, proj_a);
    OBB.projectToAxis(obb_b, axis_1, proj_b);

    if (proj_b[0] > proj_a[1] || proj_b[1] < proj_a[0]) {
      return false;
    }

    return true;
  };

  OBB.intersect = function intersect(obb_a, obb_b) {
    return OBB.axisCollide(obb_a, obb_b, obb_a.axis_0, obb_a.axis_1) && OBB.axisCollide(obb_a, obb_b, obb_b.axis_0, obb_b.axis_1);
  };

  return OBB;
}();

var Label =
/*#__PURE__*/
function () {
  function Label(size, layout) {
    if (layout === void 0) {
      layout = {};
    }

    this.id = Label.nextLabelId();
    this.type = ''; // set by subclass

    this.size = size;
    this.layout = layout;
    this.position = null;
    this.angle = 0;
    this.anchor = Array.isArray(this.layout.anchor) ? this.layout.anchor[0] : this.layout.anchor; // initial anchor

    this.placed = null;
    this.offset = layout.offset;
    this.unit_scale = this.layout.units_per_pixel;
    this.aabb = null;
    this.obb = null;
    this.align = 'center';
    this.throw_away = false; // if label does not fit (exceeds tile boundary, etc) this boolean will be true
  } // Minimal representation of label


  var _proto = Label.prototype;

  _proto.toJSON = function toJSON() {
    return {
      id: this.id,
      type: this.type,
      obb: this.obb.toJSON(),
      position: this.position,
      angle: this.angle,
      size: this.size,
      offset: this.offset,
      breach: this.breach,
      may_repeat_across_tiles: this.may_repeat_across_tiles,
      layout: textLayoutToJSON(this.layout)
    };
  };

  _proto.update = function update() {
    this.align = this.layout.align || PointAnchor.alignForAnchor(this.anchor);
  } // check for overlaps with other labels in the tile
  ;

  _proto.occluded = function occluded(bboxes, exclude) {
    var _this = this;

    if (exclude === void 0) {
      exclude = null;
    }

    var intersect = false;
    var aabbs = bboxes.aabb;
    var obbs = bboxes.obb; // Broad phase

    if (aabbs.length > 0) {
      boxIntersectsList(this.aabb, aabbs, function (j) {
        // log('trace', 'collision: broad phase collide', this.layout.id, this, this.aabb, aabbs[j]);
        // Skip if colliding with excluded label
        if (exclude && aabbs[j] === exclude.aabb) {
          // log('trace', 'collision: skipping due to explicit exclusion', this, exclude);
          return;
        } // Skip narrow phase collision if no rotation


        if (_this.obb.angle === 0 && obbs[j].angle === 0) {
          // log('trace', 'collision: skip narrow phase collide because neither is rotated', this.layout.id, this, this.obb, obbs[j]);
          intersect = true;
          return true;
        } // Narrow phase


        if (OBB.intersect(_this.obb, obbs[j])) {
          // log('trace', 'collision: narrow phase collide', this.layout.id, this, this.obb, obbs[j]);
          intersect = true;
          return true;
        }
      });
    }

    return intersect;
  } // checks whether the label is within the tile boundaries
  ;

  _proto.inTileBounds = function inTileBounds() {
    if (this.aabb[0] >= 0 && this.aabb[1] > -Geo$1.tile_scale && this.aabb[0] < Geo$1.tile_scale && this.aabb[1] <= 0 || this.aabb[2] >= 0 && this.aabb[3] > -Geo$1.tile_scale && this.aabb[2] < Geo$1.tile_scale && this.aabb[3] <= 0) {
      return false;
    }

    return true;
  } // some labels need further repeat culling checks on the main thread
  // checks whether the label is within its repeat distance of the tile boundaries
  ;

  _proto.mayRepeatAcrossTiles = function mayRepeatAcrossTiles() {
    if (this.layout.collide) {
      return true; // additional collision pass will already apply, so skip further distance checks
    }

    var dist = this.layout.repeat_distance;

    if (dist === 0) {
      return false;
    }

    return Math.abs(this.position[0]) < dist || Math.abs(this.position[0] - Geo$1.tile_scale) < dist || Math.abs(this.position[1]) < dist || Math.abs(-(this.position[1] - Geo$1.tile_scale)) < dist;
  } // Whether the label should be discarded
  // Depends on whether label must fit in the tile bounds, and if so, can it be moved to fit there
  ;

  _proto.discard = function discard(bboxes, exclude) {
    if (exclude === void 0) {
      exclude = null;
    }

    if (this.throw_away) {
      return true;
    }

    return this.occluded(bboxes, exclude);
  };

  return Label;
}(); // Generic label placement function, adds a label's bounding boxes to the currently placed set

Label.add = function (label, bboxes) {
  label.placed = true;

  if (label.aabb) {
    bboxes.aabb.push(label.aabb);
    bboxes.obb.push(label.obb);
  }

  if (label.aabbs) {
    for (var i = 0; i < label.aabbs.length; i++) {
      bboxes.aabb.push(label.aabbs[i]);
      bboxes.obb.push(label.obbs[i]);
    }
  }
};

Label.id = 0;
Label.id_prefix = 0; // id prefix scoped to worker thread

Label.id_multiplier = 0; // multiplier to keep label ids distinct across threads

Label.nextLabelId = function () {
  return Label.id_prefix + Label.id++ * Label.id_multiplier;
};

Label.epsilon = 0.9999; // tolerance around collision boxes, prevent perfectly adjacent objects from colliding
// Minimal representation of text layout, sent to main thread for label collisions

function textLayoutToJSON(layout) {
  return {
    priority: layout.priority,
    collide: layout.collide,
    repeat_distance: layout.repeat_distance,
    repeat_group: layout.repeat_group,
    buffer: layout.buffer,
    italic: layout.italic // affects bounding box size

  };
}

var RepeatGroup =
/*#__PURE__*/
function () {
  function RepeatGroup(key, repeat_dist) {
    this.key = key;
    this.repeat_dist = repeat_dist;
    this.repeat_dist_sq = this.repeat_dist * this.repeat_dist;
    this.positions = [];
  } // Check an object to see if it's a repeat in this group


  var _proto = RepeatGroup.prototype;

  _proto.check = function check(obj) {
    // Check distance from new object to objects already in group
    var p1 = obj.position;

    for (var i = 0; i < this.positions.length; i++) {
      var p2 = this.positions[i];
      var dx = p1[0] - p2[0];
      var dy = p1[1] - p2[1];
      var dist_sq = dx * dx + dy * dy; // Found an existing object within allowed distance

      if (dist_sq < this.repeat_dist_sq) {
        return true;
      }
    }
  } // Add object to this group
  ;

  _proto.add = function add(obj) {
    // only store object's position, to save space / prevent unnecessary references
    if (obj && obj.position) {
      this.positions.push(obj.position);
    }
  } // Static methods are used to manage repeat groups, within and across tiles
  // Reset all groups for this tile
  ;

  RepeatGroup.clear = function clear(tile) {
    this.groups[tile] = {};
  } // Check an object to see if it's a repeat within its designated group
  ;

  RepeatGroup.check = function check(obj, layout, tile) {
    if (layout.repeat_distance && layout.repeat_group && this.groups[tile][layout.repeat_group]) {
      return this.groups[tile][layout.repeat_group].check(obj);
    }
  } // Add an object to its designated group
  ;

  RepeatGroup.add = function add(obj, layout, tile) {
    if (layout.repeat_distance && layout.repeat_group) {
      if (this.groups[tile][layout.repeat_group] == null) {
        this.groups[tile][layout.repeat_group] = new RepeatGroup(layout.repeat_group, layout.repeat_distance * layout.repeat_scale);
      }

      this.groups[tile][layout.repeat_group].add(obj);
    }
  };

  return RepeatGroup;
}(); // Current set of repeat groups, grouped and keyed by tile
RepeatGroup.groups = {};

var Collision = {
  tiles: {},
  startTile: function startTile(tile, _temp) {
    var _ref = _temp === void 0 ? {} : _temp,
        _ref$apply_repeat_gro = _ref.apply_repeat_groups,
        apply_repeat_groups = _ref$apply_repeat_gro === void 0 ? true : _ref$apply_repeat_gro,
        _ref$return_hidden = _ref.return_hidden,
        return_hidden = _ref$return_hidden === void 0 ? false : _ref$return_hidden;

    var state = this.tiles[tile] = {
      bboxes: {
        // current set of placed bounding boxes
        aabb: [],
        obb: []
      },
      objects: {},
      // objects to collide, grouped by priority, then by style
      labels: {},
      // objects post-collision, grouped by style, marked as show/hide
      styles: {},
      // styles contributing collision objects
      repeat: apply_repeat_groups,
      return_hidden: return_hidden
    }; // Promise resolved when all registered styles have added objects

    if (state.complete == null) {
      state.complete = new Promise(function (resolve, reject) {
        state.resolve = resolve;
        state.reject = reject;
      });
    }
  },
  resetTile: function resetTile(tile) {
    delete this.tiles[tile];
  },
  abortTile: function abortTile(tile) {
    if (this.tiles[tile] && this.tiles[tile].resolve) {
      this.tiles[tile].resolve([]);
    }

    this.resetTile(tile);
  },
  // Add a style to the pending set, collision will block on all styles submitting to collision set
  addStyle: function addStyle(style, tile) {
    this.tiles[tile].styles[style] = true;
  },
  // Add collision objects for a style
  collide: function collide(objects, style, tile) {
    var state = this.tiles[tile];

    if (!state) {
      log('trace', 'Collision.collide() called with null tile', tile, this.tiles, style, objects);
      return Promise.resolve([]);
    } // Group by priority and style


    var tile_objects = state.objects;

    for (var i = 0; i < objects.length; i++) {
      var obj = objects[i];
      var priority = obj.label.layout.priority;
      tile_objects[priority] = tile_objects[priority] || {};
      tile_objects[priority][style] = tile_objects[priority][style] || [];
      tile_objects[priority][style].push(obj);
    } // Remove from pending style set, if no more styles, do collision & finish tile


    delete state.styles[style];

    if (Object.keys(state.styles).length === 0) {
      this.endTile(tile);
    } // Wait for objects to be added from all styles


    return state.complete.then(function () {
      state.resolve = null;
      return state.labels[style] || [];
    });
  },
  // Test labels for collisions, higher to lower priority
  // When two collide, hide the lower-priority label
  endTile: function endTile(tile) {
    var state = this.tiles[tile];
    var labels = state.labels;

    if (state.repeat) {
      RepeatGroup.clear(tile);
    } // Process labels by priority, then by style


    var priorities = Object.keys(state.objects).sort(function (a, b) {
      return a - b;
    });

    for (var p = 0; p < priorities.length; p++) {
      var style_objects = state.objects[priorities[p]];

      if (!style_objects) {
        // no labels at this priority, skip to next
        continue;
      } // For each style


      for (var style in style_objects) {
        var objects = style_objects[style];
        labels[style] = labels[style] || [];

        for (var i = 0; i < objects.length; i++) {
          var object = objects[i];

          if (this.canBePlaced(object, tile, object.linked, state)) {
            // show object if it isn't dependent on a parent object
            if (!object.linked) {
              object.show = true;
              labels[style].push(object);
              this.place(object, tile, state);
            } // If object is dependent on a parent, only show if both can be placed
            else if (this.canBePlaced(object.linked, tile, object, state)) {
                object.show = true; // If a label is breach, its linked label should be considered breach as well
                // (this keeps linked labels from staying (in)visible in tandem)

                if (object.label.breach || object.linked.label.breach) {
                  object.label.breach = true;
                  object.linked.label.breach = true;
                } // Similarly for labels that need main thread repeat culling, keep linked labels in sync


                if (object.label.may_repeat_across_tiles || object.linked.label.may_repeat_across_tiles) {
                  object.label.may_repeat_across_tiles = true;
                  object.linked.label.may_repeat_across_tiles = true;
                }

                labels[style].push(object);
                this.place(object, tile, state);
                this.place(object.linked, tile, state);
              } else if (state.return_hidden) {
                object.show = false;
                labels[style].push(object);
              }
          } else if (state.return_hidden) {
            object.show = false;
            labels[style].push(object);
          }
        }
      }
    }

    delete this.tiles[tile];
    state.resolve();
  },
  // Run collision and repeat check to see if label can currently be placed
  canBePlaced: function canBePlaced(object, tile, exclude, _temp2) {
    if (exclude === void 0) {
      exclude = null;
    }

    var _ref2 = _temp2 === void 0 ? {} : _temp2,
        _ref2$repeat = _ref2.repeat,
        repeat = _ref2$repeat === void 0 ? true : _ref2$repeat;

    var label = object.label;
    var layout = object.label.layout; // Skip if already processed (e.g. by parent object)

    if (label.placed != null) {
      return label.placed;
    } // Test the label for intersections with other labels in the tile


    var bboxes = this.tiles[tile].bboxes;

    if (!layout.collide || !label.discard(bboxes, exclude && exclude.label)) {
      // repeat culling with nearby labels
      if (repeat && RepeatGroup.check(label, layout, tile)) {
        label.placed = false;
      } else {
        return true;
      }
    } else if (layout.collide) {
      // log('trace', `hide label '${label.text}' due to collision`);
      label.placed = false;
    }

    return label.placed;
  },
  // Place label
  place: function place(_ref3, tile, _ref4) {
    var label = _ref3.label;
    var _ref4$repeat = _ref4.repeat,
        repeat = _ref4$repeat === void 0 ? true : _ref4$repeat;

    // Skip if already processed (e.g. by parent object)
    if (label.placed != null) {
      return;
    } // Register as placed for future collision and repeat culling


    if (repeat) {
      RepeatGroup.add(label, label.layout, tile);
    }

    Label.add(label, this.tiles[tile].bboxes);
  }
};

var LabelPoint =
/*#__PURE__*/
function (_Label) {
  _inheritsLoose(LabelPoint, _Label);

  function LabelPoint(position, size, layout, angle) {
    var _this;

    if (angle === void 0) {
      angle = 0;
    }

    _this = _Label.call(this, size, layout) || this;
    _this.type = 'point';
    _this.position = [position[0], position[1]];
    _this.angle = angle;
    _this.parent = _this.layout.parent;

    _this.update();

    _this.start_anchor_index = 1;
    _this.degenerate = !_this.size[0] && !_this.size[1] && !_this.layout.buffer[0] && !_this.layout.buffer[1];
    _this.throw_away = false;
    return _this;
  }

  var _proto = LabelPoint.prototype;

  _proto.update = function update() {
    _Label.prototype.update.call(this);

    this.computeOffset();
    this.updateBBoxes();
  };

  _proto.computeOffset = function computeOffset() {
    this.offset = [this.layout.offset[0], this.layout.offset[1]]; // Additional anchor/offset for point:

    if (this.parent) {
      var parent = this.parent; // point's own anchor, text anchor applied to point, additional point offset

      this.offset = PointAnchor.computeOffset(this.offset, parent.size, parent.anchor, PointAnchor.zero_buffer);
      this.offset = PointAnchor.computeOffset(this.offset, parent.size, this.anchor, PointAnchor.zero_buffer);

      if (parent.offset !== StyleParser.zeroPair) {
        // point has an offset
        if (this.offset === StyleParser.zeroPair) {
          // no text offset, use point's
          this.offset = parent.offset;
        } else {
          // text has offset, add point's
          this.offset[0] += parent.offset[0];
          this.offset[1] += parent.offset[1];
        }
      }
    }

    this.offset = PointAnchor.computeOffset(this.offset, this.size, this.anchor);
  };

  _proto.updateBBoxes = function updateBBoxes() {
    var width = (this.size[0] + this.layout.buffer[0] * 2) * this.unit_scale * Label.epsilon;
    var height = (this.size[1] + this.layout.buffer[1] * 2) * this.unit_scale * Label.epsilon; // fudge width value as text may overflow bounding box if it has italic, bold, etc style

    if (this.layout.italic) {
      width += 5 * this.unit_scale;
    } // make bounding boxes


    this.obb = new OBB(this.position[0] + this.offset[0] * this.unit_scale, this.position[1] - this.offset[1] * this.unit_scale, -this.angle, // angle is negative because tile system y axis is pointing down
    width, height);
    this.aabb = this.obb.getExtent();

    if (this.inTileBounds) {
      this.breach = !this.inTileBounds();
    }

    if (this.mayRepeatAcrossTiles) {
      this.may_repeat_across_tiles = this.mayRepeatAcrossTiles();
    }
  };

  _proto.discard = function discard(bboxes, exclude) {
    if (exclude === void 0) {
      exclude = null;
    }

    if (this.degenerate) {
      return false;
    }

    if (_Label.prototype.discard.call(this, bboxes, exclude)) {
      // If more than one anchor specified, try them in order
      if (Array.isArray(this.layout.anchor)) {
        // Start on second anchor (first anchor was set on creation)
        for (var i = this.start_anchor_index; i < this.layout.anchor.length; i++) {
          this.anchor = this.layout.anchor[i];
          this.update();

          if (!_Label.prototype.discard.call(this, bboxes, exclude)) {
            return false;
          }
        }
      }

      return true;
    }

    return false;
  };

  return LabelPoint;
}(Label); // Placement strategies
LabelPoint.PLACEMENT = {
  VERTEX: 0,
  // place labels at endpoints of line segments
  MIDPOINT: 1,
  // place labels at midpoints of line segments
  SPACED: 2,
  // place labels equally spaced along line
  CENTROID: 3 // place labels at center of polygons

};

// Logic for placing point labels along a line geometry
var PLACEMENT = LabelPoint.PLACEMENT;
var default_spacing = 80; // spacing of points along line in pixels

function placePointsOnLine(line, size, layout) {
  var labels = [];
  var strategy = layout.placement;
  var min_length = Math.max(size[0], size[1]) * layout.placement_min_length_ratio * layout.units_per_pixel;

  if (strategy === PLACEMENT.SPACED) {
    var result = getPositionsAndAngles(line, min_length, layout); // false will be returned if line have no length

    if (!result) {
      return [];
    }

    var positions = result.positions;
    var angles = result.angles;

    for (var i = 0; i < positions.length; i++) {
      var position = positions[i];
      var angle = angles[i];

      if (layout.tile_edges === true || !isCoordOutsideTile(position)) {
        labels.push(new LabelPoint(position, size, layout, angle));
      }
    }
  } else if (strategy === PLACEMENT.VERTEX) {
    var p, q;

    for (var _i = 0; _i < line.length - 1; _i++) {
      p = line[_i];
      q = line[_i + 1];

      if (layout.tile_edges === true || !isCoordOutsideTile(p)) {
        var _angle2 = getAngle(p, q, layout.angle);

        labels.push(new LabelPoint(p, size, layout, _angle2));
      }
    } // add last endpoint


    var _angle = getAngle(p, q, layout.angle);

    labels.push(new LabelPoint(q, size, layout, _angle));
  } else if (strategy === PLACEMENT.MIDPOINT) {
    for (var _i2 = 0; _i2 < line.length - 1; _i2++) {
      var _p = line[_i2];
      var _q = line[_i2 + 1];
      var _position = [0.5 * (_p[0] + _q[0]), 0.5 * (_p[1] + _q[1])];

      if (layout.tile_edges === true || !isCoordOutsideTile(_position)) {
        if (!min_length || norm(_p, _q) > min_length) {
          var _angle3 = getAngle(_p, _q, layout.angle);

          labels.push(new LabelPoint(_position, size, layout, _angle3));
        }
      }
    }
  }

  return labels;
}

function getPositionsAndAngles(line, min_length, layout) {
  var upp = layout.units_per_pixel;
  var spacing = (layout.placement_spacing || default_spacing) * upp;
  var length = getLineLength(line);

  if (length <= min_length) {
    return false;
  }

  var num_labels = Math.max(Math.floor(length / spacing), 1);
  var remainder = length - (num_labels - 1) * spacing;
  var positions = [];
  var angles = [];
  var distance = 0.5 * remainder;

  for (var i = 0; i < num_labels; i++) {
    var _interpolateLine = interpolateLine(line, distance, min_length, layout),
        position = _interpolateLine.position,
        angle = _interpolateLine.angle;

    if (position != null && angle != null) {
      positions.push(position);
      angles.push(angle);
    }

    distance += spacing;
  }

  return {
    positions: positions,
    angles: angles
  };
}

function getAngle(p, q, angle) {
  if (angle === void 0) {
    angle = 0;
  }

  return angle === 'auto' ? Math.atan2(q[0] - p[0], q[1] - p[1]) : angle;
}

function getLineLength(line) {
  var distance = 0;

  for (var i = 0; i < line.length - 1; i++) {
    distance += norm(line[i], line[i + 1]);
  }

  return distance;
}

function norm(p, q) {
  return Math.sqrt(Math.pow(p[0] - q[0], 2) + Math.pow(p[1] - q[1], 2));
} // TODO: can be optimized.
// you don't have to start from the first index every time for placement


function interpolateLine(line, distance, min_length, layout) {
  var sum = 0;
  var position, angle;

  for (var i = 0; i < line.length - 1; i++) {
    var p = line[i];
    var q = line[i + 1];
    var length = norm(p, q);

    if (length <= min_length) {
      continue;
    }

    sum += length;

    if (sum > distance) {
      position = interpolateSegment(p, q, sum - distance);
      angle = getAngle(p, q, layout.angle);
      break;
    }
  }

  return {
    position: position,
    angle: angle
  };
}

function interpolateSegment(p, q, distance) {
  var length = norm(p, q);
  var ratio = distance / length;
  return [ratio * p[0] + (1 - ratio) * q[0], ratio * p[1] + (1 - ratio) * q[1]];
}

var TextSettings = {
  // A key for grouping all labels of the same text style (e.g. same Canvas state, to minimize state changes)
  key: function key(settings) {
    return [settings.style, settings.weight, settings.family, settings.px_size, settings.fill, settings.stroke, settings.stroke_width, settings.transform, settings.text_wrap, settings.max_lines, settings.supersample, Utils.device_pixel_ratio].join('/');
  },
  defaults: {
    style: 'normal',
    weight: null,
    size: '12px',
    px_size: 12,
    family: 'Helvetica',
    fill: 'white',
    text_wrap: 15,
    max_lines: 5,
    align: 'center',
    stroke: null,
    stroke_width: 0
  },
  compute: function compute(feature, draw, context) {
    var style = {};
    draw.font = draw.font || this.defaults; // LineString labels can articulate while point labels cannot. Needed for future texture coordinate calculations.

    style.can_articulate = draw.can_articulate; // Use fill if specified, or default

    style.fill = draw.font.fill && Utils.toCSSColor(StyleParser.evalCachedColorProperty(draw.font.fill, context)) || this.defaults.fill; // Font properties are modeled after CSS names:
    // - family: Helvetica, Futura, etc.
    // - size: in pt, px, or em
    // - style: normal, italic, oblique
    // - weight: normal, bold, etc.
    // - transform: capitalize, uppercase, lowercase

    style.style = draw.font.style || this.defaults.style;
    style.weight = draw.font.weight || this.defaults.weight;

    if (draw.font.family) {
      style.family = draw.font.family;

      if (style.family !== this.defaults.family) {
        style.family += ', ' + this.defaults.family;
      }
    } else {
      style.family = this.defaults.family;
    }

    style.transform = draw.font.transform; // calculated pixel size

    style.supersample = draw.supersample_text ? 1.5 : 1; // optionally render text at 150% to improve clarity

    style.px_size = StyleParser.evalCachedProperty(draw.font.px_size, context) * style.supersample; // Use stroke if specified

    if (draw.font.stroke && draw.font.stroke.color) {
      style.stroke = Utils.toCSSColor(StyleParser.evalCachedColorProperty(draw.font.stroke.color, context) || this.defaults.stroke);
      style.stroke_width = StyleParser.evalCachedProperty(draw.font.stroke.width, context) || this.defaults.stroke_width;
    }

    style.font_css = this.fontCSS(style); // Word wrap and text alignment
    // Not a font properties, but affect atlas of unique text textures

    var text_wrap = draw.text_wrap; // use explicitly set value

    if (text_wrap == null && Geo$1.geometryType(feature.geometry.type) !== 'line') {
      // point labels (for point and polygon features) have word wrap on w/default max length,
      // line labels default off
      text_wrap = true;
    } // setting to 'true' causes default wrap value to be used


    if (text_wrap === true) {
      text_wrap = this.defaults.text_wrap;
    }

    style.text_wrap = text_wrap; // max_lines setting to truncate very long labels with an ellipsis

    style.max_lines = draw.max_lines || this.defaults.max_lines;
    return style;
  },
  // Build CSS-style font string (to set Canvas draw state)
  fontCSS: function fontCSS(_ref) {
    var style = _ref.style,
        weight = _ref.weight,
        px_size = _ref.px_size,
        family = _ref.family;
    return [style, weight, px_size + 'px', family].filter(function (x) {
      return x;
    }) // remove null props
    .join(' ');
  }
};

var fontfaceobserver_standalone = createCommonjsModule(function (module) {
(function(){function m(a,b){document.addEventListener?a.addEventListener("scroll",b,!1):a.attachEvent("scroll",b);}function n(a){document.body?a():document.addEventListener?document.addEventListener("DOMContentLoaded",function c(){document.removeEventListener("DOMContentLoaded",c);a();}):document.attachEvent("onreadystatechange",function l(){if("interactive"==document.readyState||"complete"==document.readyState)document.detachEvent("onreadystatechange",l),a();});}function t(a){this.a=document.createElement("div");this.a.setAttribute("aria-hidden","true");this.a.appendChild(document.createTextNode(a));this.b=document.createElement("span");this.c=document.createElement("span");this.h=document.createElement("span");this.f=document.createElement("span");this.g=-1;this.b.style.cssText="max-width:none;display:inline-block;position:absolute;height:100%;width:100%;overflow:scroll;font-size:16px;";this.c.style.cssText="max-width:none;display:inline-block;position:absolute;height:100%;width:100%;overflow:scroll;font-size:16px;";
this.f.style.cssText="max-width:none;display:inline-block;position:absolute;height:100%;width:100%;overflow:scroll;font-size:16px;";this.h.style.cssText="display:inline-block;width:200%;height:200%;font-size:16px;max-width:none;";this.b.appendChild(this.h);this.c.appendChild(this.f);this.a.appendChild(this.b);this.a.appendChild(this.c);}
function x(a,b){a.a.style.cssText="max-width:none;min-width:20px;min-height:20px;display:inline-block;overflow:hidden;position:absolute;width:auto;margin:0;padding:0;top:-999px;left:-999px;white-space:nowrap;font:"+b+";";}function y(a){var b=a.a.offsetWidth,c=b+100;a.f.style.width=c+"px";a.c.scrollLeft=c;a.b.scrollLeft=a.b.scrollWidth+100;return a.g!==b?(a.g=b,!0):!1}function z(a,b){function c(){var a=l;y(a)&&a.a.parentNode&&b(a.g);}var l=a;m(a.b,c);m(a.c,c);y(a);}function A(a,b){var c=b||{};this.family=a;this.style=c.style||"normal";this.weight=c.weight||"normal";this.stretch=c.stretch||"normal";}var B=null,C=null,E=null,F=null;function I(){if(null===E){var a=document.createElement("div");try{a.style.font="condensed 100px sans-serif";}catch(b){}E=""!==a.style.font;}return E}function J(a,b){return [a.style,a.weight,I()?a.stretch:"","100px",b].join(" ")}
A.prototype.load=function(a,b){var c=this,l=a||"BESbswy",r=0,D=b||3E3,G=(new Date).getTime();return new Promise(function(a,b){var e;null===F&&(F=!!document.fonts);if(e=F)null===C&&(C=/OS X.*Version\/10\..*Safari/.test(navigator.userAgent)&&/Apple/.test(navigator.vendor)),e=!C;if(e){e=new Promise(function(a,b){function f(){(new Date).getTime()-G>=D?b():document.fonts.load(J(c,'"'+c.family+'"'),l).then(function(c){1<=c.length?a():setTimeout(f,25);},function(){b();});}f();});var K=new Promise(function(a,
c){r=setTimeout(c,D);});Promise.race([K,e]).then(function(){clearTimeout(r);a(c);},function(){b(c);});}else n(function(){function e(){var b;if(b=-1!=g&&-1!=h||-1!=g&&-1!=k||-1!=h&&-1!=k)(b=g!=h&&g!=k&&h!=k)||(null===B&&(b=/AppleWebKit\/([0-9]+)(?:\.([0-9]+))/.exec(window.navigator.userAgent),B=!!b&&(536>parseInt(b[1],10)||536===parseInt(b[1],10)&&11>=parseInt(b[2],10))),b=B&&(g==u&&h==u&&k==u||g==v&&h==v&&k==v||g==w&&h==w&&k==w)),b=!b;b&&(d.parentNode&&d.parentNode.removeChild(d),clearTimeout(r),a(c));}
function H(){if((new Date).getTime()-G>=D)d.parentNode&&d.parentNode.removeChild(d),b(c);else{var a=document.hidden;if(!0===a||void 0===a)g=f.a.offsetWidth,h=p.a.offsetWidth,k=q.a.offsetWidth,e();r=setTimeout(H,50);}}var f=new t(l),p=new t(l),q=new t(l),g=-1,h=-1,k=-1,u=-1,v=-1,w=-1,d=document.createElement("div");d.dir="ltr";x(f,J(c,"sans-serif"));x(p,J(c,"serif"));x(q,J(c,"monospace"));d.appendChild(f.a);d.appendChild(p.a);d.appendChild(q.a);document.body.appendChild(d);u=f.a.offsetWidth;v=p.a.offsetWidth;
w=q.a.offsetWidth;H();z(f,function(a){g=a;e();});x(f,J(c,'"'+c.family+'",sans-serif'));z(p,function(a){h=a;e();});x(p,J(c,'"'+c.family+'",serif'));z(q,function(a){k=a;e();});x(q,J(c,'"'+c.family+'",monospace'));});})};module.exports=A;}());
});

var FontManager = {
  // Font detection
  fonts_loaded: Promise.resolve(),
  // resolves when all requested fonts have been detected
  last_loaded: null,
  // tracks last set of fonts loaded
  // Load set of custom font faces
  // `fonts` is an object where the key is a font family name, and the value is one or more font face
  // definitions. The value can be either a single object, or an array of such objects.
  // If the special string value 'external' is used, it indicates the the font will be loaded via external CSS.
  loadFonts: function loadFonts(fonts) {
    var _this = this;

    var same = JSON.stringify(fonts) === this.last_loaded;

    if (fonts && !same) {
      (function () {
        var queue = [];

        var _loop = function _loop(family) {
          if (Array.isArray(fonts[family])) {
            fonts[family].forEach(function (face) {
              return queue.push(_this.loadFontFace(family, face));
            });
          } else {
            queue.push(_this.loadFontFace(family, fonts[family]));
          }
        };

        for (var family in fonts) {
          _loop(family);
        }

        _this.last_loaded = JSON.stringify(fonts);
        _this.fonts_loaded = Promise.all(queue.filter(function (x) {
          return x;
        }));
      })();
    }

    return this.fonts_loaded;
  },
  // Load a single font face
  // `face` contains the font face definition, with optional parameters for `weight`, `style`, and `url`.
  // If the `url` is defined, the font is injected into the document as a CSS font-face.
  // If the object's value is the special string 'external', or if no `url` is defined, then the font face
  // is assumed is assumed to been loaded via external CSS. In either case, the function returns a promise
  // that resolves when the font face has loaded, or times out.
  loadFontFace: function loadFontFace(family, face) {
    return new Promise(function ($return, $error) {
      var options, observer;

      if (face == null || typeof face !== 'object' && face !== 'external') {
        return $return();
      }

      options = {
        family: family
      };

      if (typeof face === 'object') {
        Object.assign(options, face); // If URL is defined, inject font into document

        if (typeof face.url === 'string') {
          return Promise.resolve(this.injectFontFace(options)).then(function ($await_5) {
            try {
              return $If_3.call(this);
            } catch ($boundEx) {
              return $error($boundEx);
            }
          }.bind(this), $error);
        }

        function $If_3() {
          return $If_2.call(this);
        }

        return $If_3.call(this);
      } // Wait for font to load


      function $If_2() {
        var $Try_1_Post = function () {
          try {
            return $return();
          } catch ($boundEx) {
            return $error($boundEx);
          }
        };

        var $Try_1_Catch = function (e) {
          try {
            // Promise rejects, font is not available
            log('debug', "Font face '" + family + "' is NOT available", options);
            return $Try_1_Post();
          } catch ($boundEx) {
            return $error($boundEx);
          }
        };

        try {
          observer = new fontfaceobserver_standalone(family, options);
          return Promise.resolve(observer.load()).then(function ($await_6) {
            try {
              // Promise resolves, font is available
              log('debug', "Font face '" + family + "' is available", options);
              return $Try_1_Post();
            } catch ($boundEx) {
              return $Try_1_Catch($boundEx);
            }
          }, $Try_1_Catch);
        } catch (e) {
          $Try_1_Catch(e);
        }
      }

      return $If_2.call(this);
    }.bind(this));
  },
  // Loads a font face via either the native FontFace API, or CSS injection
  // TODO: consider support for multiple format URLs per face, unicode ranges
  injectFontFace: function injectFontFace(_ref) {
    return new Promise(function ($return, $error) {
      var family, url, weight, style, data, bytes, str, i, face, css, style_el;
      family = _ref.family, url = _ref.url, weight = _ref.weight, style = _ref.style;

      if (this.supports_native_font_loading === undefined) {
        this.supports_native_font_loading = window.FontFace !== undefined;
      } // Convert blob URLs, depending on whether the native FontFace API will be used or not.
      //
      // When the FontFace API *is* supported, the blob URL is read into a raw data array.
      // NB: it's inefficient to be converting blob URLs into typed arrays here, since they originated
      // as raw data *before* they were converted into blob URLs. However, this process should be fast since
      // these are native browser functions and all data is local (no network request), and it keeps the
      // logic streamlined by allowing us to continue to use a URL-based interface for all scene resources.
      //
      // When the FontFace API is *not* supported, the blob URL data is converted to a base64 data URL.
      // This avoids security restricions in some browsers.
      // Also see https://github.com/bramstein/fontloader/blob/598e9399117bdc946ff786fa2c5007a6bd7d3b9e/src/fontface.js#L145-L153


      data = url;

      if (url.slice(0, 5) === 'blob:') {
        return Promise.resolve(Utils.io(url, 60000, 'arraybuffer')).then(function ($await_7) {
          try {
            data = $await_7;
            bytes = new Uint8Array(data);

            if (this.supports_native_font_loading) {
              data = bytes; // use raw binary data
            } else {
              str = '';

              for (i = 0; i < bytes.length; i++) {
                str += String.fromCharCode(bytes[i]);
              }

              data = 'data:font/opentype;base64,' + btoa(str); // base64 encode as data URL
            }

            return $If_4.call(this);
          } catch ($boundEx) {
            return $error($boundEx);
          }
        }.bind(this), $error);
      }

      function $If_4() {
        if (this.supports_native_font_loading) {
          if (typeof data === 'string') {
            // add as URL
            face = new FontFace(family, "url(" + encodeURI(data) + ")", {
              weight: weight,
              style: style
            });
          } else if (data instanceof Uint8Array) {
            // add as binary data
            face = new FontFace(family, data, {
              weight: weight,
              style: style
            });
          }

          document.fonts.add(face);
          log('trace', 'Adding FontFace to document.fonts:', face);
        } else {
          css = "\n                @font-face {\n                    font-family: '" + family + "';\n                    font-weight: " + (weight || 'normal') + ";\n                    font-style: " + (style || 'normal') + ";\n                    src: url(" + encodeURI(data) + ");\n                }";
          style_el = document.createElement('style');
          style_el.appendChild(document.createTextNode(''));
          document.head.appendChild(style_el);
          style_el.sheet.insertRule(css, 0);
          log('trace', 'Injecting CSS font face:', css);
        }

        return $return();
      }

      return $If_4.call(this);
    }.bind(this));
  }
};

// Text directionality (right-to-left, bi-directional) and segmentation (curved labels, Arabic handling)
// Right-to-left / bi-directional text handling
// Taken from http://stackoverflow.com/questions/12006095/javascript-how-to-check-if-character-is-rtl
var rtl_test = new RegExp("[\u0591-\u07FF\u200F\u202B\u202E\uFB1D-\uFDFD\uFE70-\uFEFC]");
function isTextRTL(s) {
  return rtl_test.test(s);
}
var neutral_chars = "\0-/:-@[-`{-\xBF\xD7\xF7\u02B9-\u02FF\u2000-\u2BFF\u2010-\u2029\u202C\u202F-\u2BFF";
var neutral_test = new RegExp('[' + neutral_chars + ']+');
function isTextNeutral(s) {
  return neutral_test.test(s);
}
var RTL_MARKER = "\u200F"; // explicit right-to-left marker
// Arabic script ranges
// test http://localhost:8000/#16.72917/30.08541/31.28466

var arabic_range = new RegExp('^[' + neutral_chars + "\u0600-\u06FF]+"); // all characters are Arabic or neutral

var arabic_splitters = new RegExp('[' + neutral_chars + "\u0622-\u0625\u0627\u062F-\u0632\u0648\u0671-\u0677\u0688-\u0699\u06C4-\u06CB\u06CF\u06D2\u06D3\u06EE\u06EF]");
var arabic_vowels = new RegExp("^[\u0610-\u061A\u064B-\u065F\u0670\u06D6-\u06DC\u06DF-\u06E4\u06E7\u06E8\u06EA-\u06ED]+"); // Complex script ranges (non-Arabic)

var accents_and_vowels = "[\u0300-\u036F" + // Combining Diacritical Marks
"\u0591-\u05BD\u05BF\u05C1\u05C2\u05C4\u05C5\u05C7" + // Hebrew
"\u07A6-\u07B0" + // Thaana
"\u0900-\u0903\u093A-\u094C\u094E\u094F\u0951-\u0957\u0962\u0963" + // Devanagari
"\u0981-\u0983\u09BC\u09BE-\u09CC\u09D7\u09E2\u09E3" + // Bengali
"\u0A01-\u0A03\u0A3C-\u0A4C\u0A51" + // Gurmukhi
"\u0A81-\u0A83\u0ABC\u0ABE-\u0ACC\u0AE2\u0AE3" + // Gujarati
"\u0B01-\u0B03\u0B3C\u0B3E-\u0B4C\u0B56\u0B57\u0B62\u0B63" + // Oriya
"\u0B82\u0BBE-\u0BCD\u0BD7" + // Tamil
"\u0C00-\u0C03\u0C3E-\u0C4C\u0C55\u0C56\u0C62\u0C63" + // Telugu
"\u0C81-\u0C83\u0CBC\u0CBE-\u0CCC\u0CD5\u0CD6\u0CE2\u0CE3" + // Kannada
"\u0D01-\u0D03\u0D3E-\u0D4C\u0D4E\u0D57\u0D62\u0D63" + // Malayalam
"\u0D82\u0D83\u0DCA-\u0DDF\u0DF2\u0DF3" + // Sinhala
"\u0E31\u0E34-\u0E3A\u0E47-\u0E4E" + // Thai
"\u0EB1\u0EB4-\u0EBC\u0EC8-\u0ECD" + // Lao
"\u0F18\u0F19\u0F35\u0F37\u0F39\u0F3E\u0F3F\u0F71-\u0F83\u0F86\u0F87\u0F8D-\u0FBC\u0FC6" + // Tibetan
"\u102B-\u1038\u103A-\u103E\u1056-\u1059\u105E-\u1060\u1062-\u1064\u1067-\u106D\u1071-\u1074\u1082-\u108D\u108F\u109A-\u109D" + // Burmese
"\u17B4-\u17D1\u17D3" + // Khmer
"\u1A55-\u1A5E\u1A61-\u1A7C" + // Tai Tham
"\u1DC0-\u1DFF" + // Combining Diacritical Marks Supplement
"\u20D0-\u20FF" + // Combining Diacritical Marks for Symbols
']';
var combo_characters = "[\u094D\u09CD\u0A4D\u0ACD\u0B4D\u0C4D\u0CCD\u0D4D\u0F84\u1039\u17D2\u1A60\u1A7F]"; // Find the next grapheme cluster (non-Arabic)

var grapheme_match = new RegExp('^.(?:' + accents_and_vowels + '+)?' + '(' + combo_characters + '\\W(?:' + accents_and_vowels + '+)?)*'); // Scripts that cannot be curved due (due to contextual shaping and/or layout complexity)

var curve_blacklist = {
  Mongolian: "\u1800-\u18AF"
};
var curve_blacklist_range = Object.keys(curve_blacklist).map(function (r) {
  return curve_blacklist[r];
}).join('');
var curve_blacklist_test = new RegExp('[' + curve_blacklist_range + ']');
function isTextCurveBlacklisted(s) {
  return curve_blacklist_test.test(s);
} // Splitting strategy for chopping a label into segments

var default_segment_length = 2; // character length of each segment when dividing up label text

function splitLabelText(text, rtl, cache) {
  // Use single-character segments for RTL, to avoid additional handling for neutral characters
  // (see https://github.com/tangrams/tangram/issues/541)
  var segment_length = rtl ? 1 : default_segment_length; // Only one segment

  if (text.length < segment_length) {
    return [text];
  } // Check segment cache first (skips processing for labels we've seen before)


  var key = text;

  if (cache.segment[key]) {
    cache.stats.segment_hits++;
    return cache.segment[key];
  }

  var segments = []; // Arabic-specific text handling
  // NB: works for strings that are *only* Arabic; mixed-script labels may need more work

  if (arabic_range.exec(text)) {
    segments = text.split(arabic_splitters);
    var offset = -1;

    for (var s = 0; s < segments.length - 1; s++) {
      if (s > 0) {
        var carryover_vowels = arabic_vowels.exec(segments[s]);

        if (carryover_vowels) {
          segments[s] = segments[s].substring(carryover_vowels[0].length);
          segments[s - 1] += carryover_vowels[0];
          offset += carryover_vowels[0].length;
        }
      }

      offset += 1 + segments[s].length;
      segments[s] += text.slice(offset, offset + 1);
    }

    text = ''; // will skip non-Arabic handling below
  } // Non-Arabic text handling


  while (text.length) {
    var segment = '';
    var test_text = text;
    var grapheme_count = 0;

    for (grapheme_count; grapheme_count < segment_length && test_text.length; grapheme_count++) {
      var grapheme_cluster = (grapheme_match.exec(test_text) || test_text)[0];
      segment += grapheme_cluster;
      test_text = test_text.substring(grapheme_cluster.length);
    }

    segments.push(segment);
    text = text.substring(segment.length);
  } // Reverse segments if needed


  if (rtl) {
    segments.reverse();
  } // Cache and return


  cache.stats.segment_misses++;
  cache.segment[key] = segments;
  return segments;
}

// "text wrap" and "max line" values

var MultiLine =
/*#__PURE__*/
function () {
  function MultiLine(context, max_lines, text_wrap) {
    if (max_lines === void 0) {
      max_lines = Infinity;
    }

    if (text_wrap === void 0) {
      text_wrap = Infinity;
    }

    this.width = 0;
    this.height = 0;
    this.lines = [];
    this.max_lines = max_lines;
    this.text_wrap = text_wrap;
    this.context = context;
  }

  var _proto = MultiLine.prototype;

  _proto.createLine = function createLine(line_height) {
    if (this.lines.length < this.max_lines) {
      return new Line(line_height, this.text_wrap);
    } else {
      return false;
    }
  };

  _proto.push = function push(line) {
    if (this.lines.length < this.max_lines) {
      // measure line width
      var line_width = this.context.measureText(line.text).width;
      line.width = line_width;

      if (line_width > this.width) {
        this.width = Math.ceil(line_width);
      } // add to lines and increment height


      this.lines.push(line);
      this.height += line.height;
      return true;
    } else {
      this.addEllipsis();
      return false;
    }
  } // pushes to the lines array and returns a new line if possible (false otherwise)
  ;

  _proto.advance = function advance(line, line_height) {
    var can_push = this.push(line);

    if (can_push) {
      return this.createLine(line_height);
    } else {
      return false;
    }
  };

  _proto.addEllipsis = function addEllipsis() {
    var last_line = this.lines[this.lines.length - 1];
    var ellipsis_width = Math.ceil(this.context.measureText(MultiLine.ellipsis).width);
    last_line.append(MultiLine.ellipsis);
    last_line.width += ellipsis_width;

    if (last_line.width > this.width) {
      this.width = last_line.width;
    }
  };

  _proto.finish = function finish(line) {
    if (line) {
      this.push(line);
    } else {
      this.addEllipsis();
    }
  };

  MultiLine.parse = function parse(str, text_wrap, max_lines, line_height, ctx) {
    // Word wrapping
    // Line breaks can be caused by:
    //  - implicit line break when a maximum character threshold is exceeded per line (text_wrap)
    //  - explicit line break in the label text (\n)
    var words;

    if (typeof text_wrap === 'number') {
      words = str.split(' '); // split words on spaces
    } else {
      words = [str]; // no max line word wrapping (but new lines will still be in effect)
    }

    var multiline = new MultiLine(ctx, max_lines, text_wrap);
    var line = multiline.createLine(line_height); // First iterate on space-break groups (will be one if max line length off), then iterate on line-break groups

    for (var i = 0; i < words.length; i++) {
      var breaks = words[i].split('\n'); // split on line breaks

      var new_line = i === 0 ? true : false;

      for (var n = 0; n < breaks.length; n++) {
        if (!line) {
          break;
        }

        var word = breaks[n]; // force punctuation (neutral chars) at the end of a RTL line, so they stay attached to original word

        if (isTextRTL(word) && isTextNeutral(word[word.length - 1])) {
          word += RTL_MARKER;
        }

        var spaced_word = new_line ? word : ' ' + word; // if adding current word would overflow, add a new line instead
        // first word (i === 0) always appends

        if (text_wrap && i > 0 && line.exceedsTextwrap(spaced_word)) {
          line = multiline.advance(line, line_height);

          if (!line) {
            break;
          }

          line.append(word);
          new_line = true;
        } else {
          line.append(spaced_word);
        } // if line breaks present, add new line (unless on last line)


        if (n < breaks.length - 1) {
          line = multiline.advance(line, line_height);
          new_line = true;
        }
      }

      if (i === words.length - 1) {
        multiline.finish(line);
      }
    }

    return multiline;
  };

  return MultiLine;
}();
MultiLine.ellipsis = '...'; // A Private class used by MultiLine to contain the logic for a single line
// including character count, width, height and text

var Line =
/*#__PURE__*/
function () {
  function Line(height, text_wrap) {
    if (height === void 0) {
      height = 0;
    }

    if (text_wrap === void 0) {
      text_wrap = 0;
    }

    this.chars = 0;
    this.text = '';
    this.height = Math.ceil(height);
    this.text_wrap = text_wrap;
  }

  var _proto2 = Line.prototype;

  _proto2.append = function append(text) {
    this.chars += text.length;
    this.text += text;
  };

  _proto2.exceedsTextwrap = function exceedsTextwrap(text) {
    return text.length + this.chars > this.text_wrap;
  };

  return Line;
}();

var TextCanvas =
/*#__PURE__*/
function () {
  function TextCanvas() {
    this.createCanvas(); // create initial canvas and context

    this.vertical_text_buffer = 8; // vertical pixel padding around text

    this.horizontal_text_buffer = 4; // text styling such as italic emphasis is not measured by the Canvas API, so padding is necessary
  }

  var _proto = TextCanvas.prototype;

  _proto.createCanvas = function createCanvas() {
    this.canvas = document.createElement('canvas');
    this.canvas.style.backgroundColor = 'transparent'; // render text on transparent background

    this.context = this.canvas.getContext('2d');
  };

  _proto.resize = function resize(width, height) {
    this.canvas.width = width;
    this.canvas.height = height;
    this.context.clearRect(0, 0, width, height);
  } // Set font style params for canvas drawing
  ;

  _proto.setFont = function setFont(_ref) {
    var font_css = _ref.font_css,
        fill = _ref.fill,
        stroke = _ref.stroke,
        stroke_width = _ref.stroke_width,
        px_size = _ref.px_size,
        supersample = _ref.supersample;
    this.px_size = px_size;
    var ctx = this.context;
    var dpr = Utils.device_pixel_ratio * supersample;

    if (stroke && stroke_width > 0) {
      ctx.strokeStyle = stroke;
      ctx.lineWidth = stroke_width * dpr;
    }

    ctx.fillStyle = fill;
    ctx.font = font_css;
    ctx.miterLimit = 2;
  };

  _proto.textSizes = function textSizes(tile_id, texts) {
    return new Promise(function ($return, $error) {
      return Promise.resolve(FontManager.loadFonts()).then(function ($await_1) {
        try {
          return $return(Task.add({
            type: 'textSizes',
            run: this.processTextSizesTask.bind(this),
            texts: texts,
            tile_id: tile_id,
            cursor: {
              styles: Object.keys(texts),
              texts: null,
              style_idx: null,
              text_idx: null
            }
          }));
        } catch ($boundEx) {
          return $error($boundEx);
        }
      }.bind(this), $error);
    }.bind(this));
  };

  _proto.processTextSizesTask = function processTextSizesTask(task) {
    var cursor = task.cursor,
        texts = task.texts;
    cursor.style_idx = cursor.style_idx || 0;

    while (cursor.style_idx < cursor.styles.length) {
      var style = cursor.styles[cursor.style_idx];

      if (cursor.text_idx == null) {
        cursor.text_idx = 0;
        cursor.texts = Object.keys(texts[style]);
      }

      var text_infos = texts[style];
      var first = true;

      while (cursor.text_idx < cursor.texts.length) {
        var text = cursor.texts[cursor.text_idx];
        var text_info = text_infos[text];
        var text_settings = text_info.text_settings;

        if (first) {
          this.setFont(text_settings);
          first = false;
        } // add size of full text string


        text_info.size = this.textSize(style, text, text_settings).size; // if text may curve, calculate per-segment as well

        if (text_settings.can_articulate) {
          var rtl = false;
          var bidi = false;

          if (isTextRTL(text)) {
            if (!isTextNeutral(text)) {
              bidi = true;
            } else {
              rtl = true;
            }
          }

          text_info.isRTL = rtl;
          text_info.no_curving = bidi || isTextCurveBlacklisted(text); // used in LabelLine to prevent curved labels

          text_info.vertical_buffer = this.vertical_text_buffer;
          text_info.segment_sizes = [];

          if (!text_info.no_curving) {
            var segments = splitLabelText(text, rtl, TextCanvas.cache);
            text_info.segments = segments;

            for (var i = 0; i < segments.length; i++) {
              text_info.segment_sizes.push(this.textSize(style, segments[i], text_settings).size);
            }
          }
        }

        cursor.text_idx++;

        if (!Task.shouldContinue(task)) {
          return false;
        }
      }

      cursor.text_idx = null;
      cursor.style_idx++;
    }

    Task.finish(task, texts);
    return true;
  } // Computes width and height of text based on current font style
  // Includes word wrapping, returns size info for whole text block and individual lines
  ;

  _proto.textSize = function textSize(style, text, _ref2) {
    var transform = _ref2.transform,
        text_wrap = _ref2.text_wrap,
        max_lines = _ref2.max_lines,
        _ref2$stroke_width = _ref2.stroke_width,
        stroke_width = _ref2$stroke_width === void 0 ? 0 : _ref2$stroke_width,
        supersample = _ref2.supersample;
    // Check cache first
    TextCanvas.cache.text[style] = TextCanvas.cache.text[style] || {};

    if (TextCanvas.cache.text[style][text]) {
      TextCanvas.cache.stats.text_hits++;
      return TextCanvas.cache.text[style][text];
    }

    TextCanvas.cache.stats.text_misses++;
    TextCanvas.cache.text_count++; // Calc and store in cache

    var dpr = Utils.device_pixel_ratio * supersample;
    var str = this.applyTextTransform(text, transform);
    var ctx = this.context;
    var vertical_buffer = this.vertical_text_buffer * dpr;
    var horizontal_buffer = dpr * (stroke_width + this.horizontal_text_buffer);
    var leading = 2 * dpr; // make configurable and/or use Canvas TextMetrics when available

    var line_height = this.px_size + leading; // px_size already in device pixels
    // Parse string into series of lines if it exceeds the text wrapping value or contains line breaks

    var multiline = MultiLine.parse(str, text_wrap, max_lines, line_height, ctx); // Final dimensions of text

    var height = multiline.height;
    var width = multiline.width;
    var lines = multiline.lines;
    var collision_size = [width / dpr, height / dpr];
    var texture_size = [width + 2 * horizontal_buffer, height + 2 * vertical_buffer];
    var logical_size = [texture_size[0] / dpr, texture_size[1] / dpr]; // Returns lines (w/per-line info for drawing) and text's overall bounding box + canvas size

    TextCanvas.cache.text[style][text] = {
      lines: lines,
      size: {
        collision_size: collision_size,
        texture_size: texture_size,
        logical_size: logical_size,
        line_height: line_height
      }
    };
    return TextCanvas.cache.text[style][text];
  } // Draw multiple lines of text
  ;

  _proto.drawTextMultiLine = function drawTextMultiLine(lines, _ref3, size, _ref4, type) {
    var x = _ref3[0],
        y = _ref3[1];
    var stroke = _ref4.stroke,
        _ref4$stroke_width = _ref4.stroke_width,
        stroke_width = _ref4$stroke_width === void 0 ? 0 : _ref4$stroke_width,
        transform = _ref4.transform,
        align = _ref4.align,
        supersample = _ref4.supersample;
    var line_height = size.line_height;
    var height = y;

    for (var line_num = 0; line_num < lines.length; line_num++) {
      var line = lines[line_num];
      this.drawTextLine(line, [x, height], size, {
        stroke: stroke,
        stroke_width: stroke_width,
        transform: transform,
        align: align,
        supersample: supersample
      }, type);
      height += line_height;
    } // Draw bounding boxes for debugging


    if (debugSettings$1.draw_label_collision_boxes) {
      this.context.save();
      var dpr = Utils.device_pixel_ratio * supersample;
      var horizontal_buffer = dpr * (this.horizontal_text_buffer + stroke_width);
      var vertical_buffer = dpr * this.vertical_text_buffer;
      var collision_size = size.collision_size;
      var lineWidth = 2;
      this.context.strokeStyle = 'blue';
      this.context.lineWidth = lineWidth;
      this.context.strokeRect(x + horizontal_buffer, y + vertical_buffer, dpr * collision_size[0], dpr * collision_size[1]);

      if (type === 'curved') {
        this.context.strokeRect(x + size.texture_size[0] + horizontal_buffer, y + vertical_buffer, dpr * collision_size[0], dpr * collision_size[1]);
      }

      this.context.restore();
    }

    if (debugSettings$1.draw_label_texture_boxes) {
      this.context.save();
      var texture_size = size.texture_size;
      var _lineWidth = 2;
      this.context.strokeStyle = 'green';
      this.context.lineWidth = _lineWidth; // stroke is applied internally, so the outer border is the edge of the texture

      this.context.strokeRect(x + _lineWidth, y + _lineWidth, texture_size[0] - 2 * _lineWidth, texture_size[1] - 2 * _lineWidth);

      if (type === 'curved') {
        this.context.strokeRect(x + _lineWidth + size.texture_size[0], y + _lineWidth, texture_size[0] - 2 * _lineWidth, texture_size[1] - 2 * _lineWidth);
      }

      this.context.restore();
    }
  } // Draw single line of text at specified location, adjusting for buffer and baseline
  ;

  _proto.drawTextLine = function drawTextLine(line, _ref5, size, _ref6, type) {
    var x = _ref5[0],
        y = _ref5[1];
    var stroke = _ref6.stroke,
        _ref6$stroke_width = _ref6.stroke_width,
        stroke_width = _ref6$stroke_width === void 0 ? 0 : _ref6$stroke_width,
        transform = _ref6.transform,
        align = _ref6.align,
        supersample = _ref6.supersample;
    var dpr = Utils.device_pixel_ratio * supersample;
    align = align || 'center';
    var vertical_buffer = this.vertical_text_buffer * dpr;
    var texture_size = size.texture_size;
    var line_height = size.line_height;
    var horizontal_buffer = dpr * (stroke_width + this.horizontal_text_buffer);
    var str = this.applyTextTransform(line.text, transform); // Text alignment

    var tx;

    if (align === 'left') {
      tx = x + horizontal_buffer;
    } else if (align === 'center') {
      tx = x + texture_size[0] / 2 - line.width / 2;
    } else if (align === 'right') {
      tx = x + texture_size[0] - line.width - horizontal_buffer;
    } // In the absence of better Canvas TextMetrics (not supported by browsers yet),
    // 0.75 buffer produces a better approximate vertical centering of text


    var ty = y + vertical_buffer * 0.75 + line_height; // Draw stroke and fill separately for curved text. Offset stroke in texture atlas by shift.

    if (stroke && stroke_width > 0) {
      var shift = type === 'curved' ? texture_size[0] : 0;
      this.context.strokeText(str, tx + shift, ty);
    }

    this.context.fillText(str, tx, ty);
  };

  _proto.rasterize = function rasterize(texts, textures, tile_id, texture_prefix, gl) {
    return Task.add({
      type: 'rasterizeLabels',
      run: this.processRasterizeTask.bind(this),
      cancel: this.cancelRasterizeTask.bind(this),
      pause_factor: 2,
      // pause 2 frames when task run past allowed time
      user_moving_view: false,
      // don't run task when user is moving view
      texts: texts,
      textures: textures,
      texture_prefix: texture_prefix,
      gl: gl,
      tile_id: tile_id,
      cursor: {
        styles: Object.keys(texts),
        texts: null,
        style_idx: 0,
        text_idx: null,
        texture_idx: 0,
        texture_resize: true,
        texture_names: []
      }
    });
  };

  _proto.processRasterizeTask = function processRasterizeTask(task) {
    var cursor = task.cursor,
        texts = task.texts,
        textures = task.textures;
    var texture; // Rasterize one texture at a time, so we only have to keep one canvas in memory (they can be large)

    while (cursor.texture_idx < task.textures.length) {
      texture = textures[cursor.texture_idx];

      if (cursor.texture_resize) {
        cursor.texture_resize = false;
        this.resize.apply(this, texture.texture_size);
      }

      while (cursor.style_idx < cursor.styles.length) {
        var style = cursor.styles[cursor.style_idx];

        if (cursor.text_idx == null) {
          cursor.text_idx = 0;
          cursor.texts = Object.keys(texts[style]);
        }

        var text_infos = texts[style];
        var first = true;

        while (cursor.text_idx < cursor.texts.length) {
          var text = cursor.texts[cursor.text_idx];
          var text_info = text_infos[text];
          var text_settings = text_info.text_settings; // set font on first occurence of new font style

          if (first) {
            this.setFont(text_settings);
            first = false;
          }

          if (text_settings.can_articulate) {
            text_info.texcoords = text_info.texcoords || {};

            for (var t = 0; t < text_info.type.length; t++) {
              var type = text_info.type[t];

              if (type === 'straight') {
                // Only render for current texture
                if (text_info.textures[t] !== cursor.texture_idx) {
                  continue;
                }

                var word = text_info.isRTL ? text.split().reverse().join() : text;
                var cache = texture.texcoord_cache[style][word];
                var texcoord = void 0;

                if (cache.texcoord) {
                  texcoord = cache.texcoord;
                } else {
                  var texture_position = cache.texture_position;

                  var _this$textSize = this.textSize(style, word, text_settings),
                      size = _this$textSize.size,
                      lines = _this$textSize.lines;

                  this.drawTextMultiLine(lines, texture_position, size, text_settings, type);
                  texcoord = Texture.getTexcoordsForSprite(texture_position, size.texture_size, texture.texture_size);
                  cache.texcoord = texcoord;
                }

                text_info.texcoords[type] = {
                  texcoord: texcoord,
                  texture_id: cache.texture_id
                };
              } else if (type === 'curved') {
                var words = text_info.segments;
                text_info.texcoords.curved = text_info.texcoords.curved || [];
                text_info.texcoords_stroke = text_info.texcoords_stroke || [];

                for (var w = 0; w < words.length; w++) {
                  // Only render for current texture
                  if (text_info.textures[t][w] !== cursor.texture_idx) {
                    continue;
                  }

                  var _word = words[w];
                  var _cache = texture.texcoord_cache[style][_word];

                  var _texcoord = void 0;

                  var texcoord_stroke = void 0;

                  if (_cache.texcoord) {
                    _texcoord = _cache.texcoord;
                    texcoord_stroke = _cache.texcoord_stroke;
                    text_info.texcoords_stroke.push(texcoord_stroke);
                  } else {
                    var _texture_position = _cache.texture_position;

                    var _this$textSize2 = this.textSize(style, _word, text_settings),
                        _size = _this$textSize2.size,
                        _lines = _this$textSize2.lines;

                    this.drawTextMultiLine(_lines, _texture_position, _size, text_settings, type);
                    _texcoord = Texture.getTexcoordsForSprite(_texture_position, _size.texture_size, texture.texture_size);
                    var texture_position_stroke = [_texture_position[0] + _size.texture_size[0], _texture_position[1]];
                    texcoord_stroke = Texture.getTexcoordsForSprite(texture_position_stroke, _size.texture_size, texture.texture_size);
                    _cache.texcoord = _texcoord;
                    _cache.texcoord_stroke = texcoord_stroke; // NB: texture_id is the same between stroke and fill, so it's not duplicated here

                    text_info.texcoords_stroke.push(texcoord_stroke);
                  }

                  text_info.texcoords.curved.push({
                    texcoord: _texcoord,
                    texture_id: _cache.texture_id
                  });
                }
              }
            }
          } else {
            var _lines2 = this.textSize(style, text, text_settings).lines;

            for (var align in text_info.align) {
              // Only render for current texture
              if (text_info.align[align].texture_id !== cursor.texture_idx) {
                continue;
              }

              this.drawTextMultiLine(_lines2, text_info.align[align].texture_position, text_info.size, {
                stroke: text_settings.stroke,
                stroke_width: text_settings.stroke_width,
                transform: text_settings.transform,
                supersample: text_settings.supersample,
                align: align
              });
              text_info.align[align].texcoords = Texture.getTexcoordsForSprite(text_info.align[align].texture_position, text_info.size.texture_size, texture.texture_size);
            }
          }

          cursor.text_idx++;

          if (!Task.shouldContinue(task)) {
            return false;
          }
        }

        cursor.text_idx = null;
        cursor.style_idx++;
      } // Create GL texture (canvas element will be reused for next texture)


      var tname = task.texture_prefix + cursor.texture_idx;
      Texture.create(task.gl, tname, {
        element: this.canvas,
        filtering: 'linear',
        UNPACK_PREMULTIPLY_ALPHA_WEBGL: true
      });
      Texture.retain(tname);
      cursor.texture_names.push(tname);
      cursor.texture_idx++;
      cursor.texture_resize = true;
      cursor.style_idx = 0;
    }

    Task.finish(task, cursor.texture_names);
    return true;
  } // Free any textures that have been allocated part-way through label rasterization for a tile
  ;

  _proto.cancelRasterizeTask = function cancelRasterizeTask(task) {
    log('trace', "RasterizeTask: release textures [" + task.cursor.texture_names.join(', ') + "]");
    task.cursor.texture_names.forEach(function (t) {
      return Texture.release(t);
    });
  } // Place text labels within an atlas of the given max size
  ;

  _proto.setTextureTextPositions = function setTextureTextPositions(texts, max_texture_size) {
    var texture = {
      cx: 0,
      cy: 0,
      width: 0,
      height: 0,
      column_width: 0,
      texture_id: 0,
      texcoord_cache: {}
    },
        textures = [];

    for (var style in texts) {
      var text_infos = texts[style];

      for (var text in text_infos) {
        var text_info = text_infos[text];
        var texture_position = void 0;

        if (text_info.text_settings.can_articulate) {
          text_info.textures = [];
          texture.texcoord_cache[style] = texture.texcoord_cache[style] || {};

          for (var t = 0; t < text_info.type.length; t++) {
            var type = text_info.type[t];

            if (type === 'straight') {
              var word = text_info.isRTL ? text.split().reverse().join() : text;

              if (!texture.texcoord_cache[style][word]) {
                var size = text_info.size.texture_size;
                texture_position = this.placeText(size[0], size[1], style, texture, textures, max_texture_size);
                texture.texcoord_cache[style][word] = {
                  texture_id: texture.texture_id,
                  texture_position: texture_position
                };
              }

              text_info.textures[t] = texture.texture_id;
            } else if (type === 'curved') {
              text_info.textures[t] = [];

              for (var w = 0; w < text_info.segment_sizes.length; w++) {
                var _word2 = text_info.segments[w];

                if (!texture.texcoord_cache[style][_word2]) {
                  var _size2 = text_info.segment_sizes[w].texture_size;
                  var width = 2 * _size2[0]; // doubled to account for side-by-side rendering of fill and stroke

                  texture_position = this.placeText(width, _size2[1], style, texture, textures, max_texture_size);
                  texture.texcoord_cache[style][_word2] = {
                    texture_id: texture.texture_id,
                    texture_position: texture_position
                  };
                }

                text_info.textures[t].push(texture.texture_id);
              }
            }
          }
        } else {
          // rendered size is same for all alignments
          var _size3 = text_info.size.texture_size; // but each alignment needs to be rendered separately

          for (var align in text_info.align) {
            texture_position = this.placeText(_size3[0], _size3[1], style, texture, textures, max_texture_size);
            text_info.align[align].texture_id = texture.texture_id;
            text_info.align[align].texture_position = texture_position;
          }
        }
      }
    } // save final texture


    if (texture.column_width > 0 && texture.height > 0) {
      textures[texture.texture_id] = {
        texture_size: [texture.width, texture.height],
        texcoord_cache: texture.texcoord_cache
      };
    } // return computed texture sizes and UV cache


    return textures;
  } // Place text sprite in texture atlas, enlarging current texture, or starting new one if max texture size reached
  ;

  _proto.placeText = function placeText(text_width, text_height, style, texture, textures, max_texture_size) {
    var texture_position; // TODO: what if first label is wider than entire max texture?

    if (texture.cy + text_height > max_texture_size) {
      // start new column
      texture.cx += texture.column_width;
      texture.cy = 0;
      texture.column_width = text_width;
    } else {
      // expand current column
      texture.column_width = Math.max(texture.column_width, text_width);
    }

    if (texture.cx + texture.column_width <= max_texture_size) {
      // add label to current texture
      texture_position = [texture.cx, texture.cy];
      texture.cy += text_height; // expand texture if needed

      texture.height = Math.max(texture.height, texture.cy);
      texture.width = Math.max(texture.width, texture.cx + texture.column_width);
    } else {
      // start new texture
      // save size and cache of last texture
      textures[texture.texture_id] = {
        texture_size: [texture.width, texture.height],
        texcoord_cache: texture.texcoord_cache
      };
      texture.texcoord_cache = {}; // reset cache

      texture.texcoord_cache[style] = {};
      texture.texture_id++;
      texture.cx = 0;
      texture.cy = text_height;
      texture.column_width = text_width;
      texture.width = text_width;
      texture.height = text_height;
      texture_position = [0, 0]; // TODO: allocate zero array once
    }

    return texture_position;
  } // Called before rasterization
  ;

  _proto.applyTextTransform = function applyTextTransform(text, transform) {
    if (transform === 'capitalize') {
      return text.replace(/\w\S*/g, function (txt) {
        return txt.charAt(0).toUpperCase() + txt.substr(1);
      });
    } else if (transform === 'uppercase') {
      return text.toUpperCase();
    } else if (transform === 'lowercase') {
      return text.toLowerCase();
    }

    return text;
  } // Convert font CSS-style size ('12px', '14pt', '1.5em', etc.) to pixel size (adjusted for device pixel ratio)
  // Defaults units to pixels if not specified
  ;

  TextCanvas.fontPixelSize = function fontPixelSize(size) {
    if (size == null) {
      return;
    }

    size = typeof size === 'string' ? size : String(size); // need a string for regex

    var _ref7 = size.match(TextCanvas.font_size_re) || [],
        px_size = _ref7[1],
        units = _ref7[2];

    units = units || 'px';

    if (units === 'em') {
      px_size *= 16;
    } else if (units === 'pt') {
      px_size /= 0.75;
    } else if (units === '%') {
      px_size /= 6.25;
    }

    px_size = StyleParser.parsePositiveNumber(px_size);
    px_size *= Utils.device_pixel_ratio;
    return px_size;
  };

  TextCanvas.pruneTextCache = function pruneTextCache() {
    if (TextCanvas.cache.text_count > TextCanvas.cache.text_count_max) {
      TextCanvas.cache.text = {};
      TextCanvas.cache.text_count = 0;
      log('debug', 'TextCanvas: pruning text cache');
    }

    if (Object.keys(TextCanvas.cache.segment).length > TextCanvas.cache.segment_count_max) {
      TextCanvas.cache.segment = {};
      log('debug', 'TextCanvas: pruning segment cache');
    }
  };

  return TextCanvas;
}(); // Extract font size and units
TextCanvas.font_size_re = /((?:[0-9]*\.)?[0-9]+)\s*(px|pt|em|%)?/; // Cache sizes of rendered text

TextCanvas.cache = {
  text: {},
  // size and line parsing, by text style, then text string
  text_count: 0,
  // current size of cache (measured as # of entries)
  text_count_max: 2000,
  // prune cache when it exceeds this size
  segment: {},
  // segmentation of text (by run of characters or grapheme clusters), by text string
  segment_count_max: 2000,
  // prune cache when it exceeds this size
  stats: {
    text_hits: 0,
    text_misses: 0,
    segment_hits: 0,
    segment_misses: 0
  }
};

var text_texture_id = 0;
var TextLabels = {
  resetText: function resetText() {
    if (Thread.is_main) {
      this.canvas = new TextCanvas();
    } else if (Thread.is_worker) {
      this.texts = {}; // unique texts, grouped by tile, by style
    }
  },
  freeText: function freeText(tile) {
    delete this.texts[tile.id];
  },
  parseTextFeature: function parseTextFeature(feature, draw, context, tile) {
    // Compute label text
    var text = this.parseTextSource(feature, draw, context);

    if (text == null || text === '') {
      return; // no text for this feature
    } // Compute text style and layout settings for this feature label


    var text_settings = TextSettings.compute(feature, draw, context);
    var text_settings_key = TextSettings.key(text_settings); // first label in tile, or with this style?

    this.texts[tile.id] = this.texts[tile.id] || {};
    var sizes = this.texts[tile.id][text_settings_key] = this.texts[tile.id][text_settings_key] || {};

    if (text instanceof Object) {
      var results = []; // add both left/right text elements to repeat group to improve repeat culling
      // avoids one component of a boundary label (e.g. Colorado) being culled too aggressively when it also
      // appears in nearby boundary labels (e.g. Colorado/Utah & Colorado/New Mexico repeat as separate groups)

      var repeat_group_prefix = text.left + '-' + text.right; // NB: should be all text keys, not just left/right

      for (var key in text) {
        var current_text = text[key];

        if (!current_text) {
          continue;
        }

        var layout = this.computeTextLayout({}, feature, draw, context, tile, current_text, text_settings, repeat_group_prefix, key);

        if (!sizes[current_text]) {
          // first label with this text/style/tile combination, make a new label entry
          sizes[current_text] = {
            text_settings: text_settings,
            ref: 0 // # of times this text/style combo appears in tile

          };
        }

        results.push({
          draw: draw,
          text: current_text,
          text_settings_key: text_settings_key,
          layout: layout
        });
      }

      return results.length > 0 && results; // return null if no boundary labels found
    } else {
      // unique text strings, grouped by text drawing style
      var _layout = this.computeTextLayout({}, feature, draw, context, tile, text, text_settings);

      if (!sizes[text]) {
        // first label with this text/style/tile combination, make a new label entry
        sizes[text] = {
          text_settings: text_settings,
          ref: 0 // # of times this text/style combo appears in tile

        };
      }

      return {
        draw: draw,
        text: text,
        text_settings_key: text_settings_key,
        layout: _layout
      };
    }
  },
  // Compute the label text, default is value of feature.properties.name
  // - String value indicates a feature property look-up, e.g. `short_name` means use feature.properties.short_name
  // - Function will use the return value as the label text (for custom labels)
  // - Array (of strings and/or functions) defines a list of fallbacks, evaluated according to the above rules,
  //   with the first non-null value used as the label text
  //   e.g. `[name:es, name:en, name]` prefers Spanish names, followed by English, and last the default local name
  parseTextSource: function parseTextSource(feature, draw, context) {
    var text;
    var source = draw.text_source || 'name';

    if (source != null && !Array.isArray(source) && typeof source === 'object') {
      // left/right boundary labels
      text = {};

      for (var key in source) {
        text[key] = this.parseTextSourceValue(source[key], feature, context);
      }
    } else {
      // single label
      text = this.parseTextSourceValue(source, feature, context);
    }

    return text;
  },
  parseTextSourceValue: function parseTextSourceValue(source, feature, context) {
    var text;

    if (Array.isArray(source)) {
      for (var s = 0; s < source.length; s++) {
        if (typeof source[s] === 'string') {
          text = feature.properties[source[s]];
        } else if (typeof source[s] === 'function') {
          text = source[s](context);
        }

        if (text) {
          return text; // stop if we found a text property
        }
      }
    } else if (typeof source === 'string') {
      text = feature.properties[source];
    } else if (source instanceof Function) {
      text = source(context);
    }

    return text;
  },
  prepareTextLabels: function prepareTextLabels(tile, queue) {
    return new Promise(function ($return, $error) {
      var texts;

      if (Object.keys(this.texts[tile.id] || {}).length === 0) {
        return $return([]);
      } // first call to main thread, ask for text pixel sizes

      var $Try_1_Catch = function (e) {
        try {
          // error thrown if style has been removed from main thread
          Collision.abortTile(tile.id);
          return $return([]);
        } catch ($boundEx) {
          return $error($boundEx);
        }
      };

      try {
        return Promise.resolve(WorkerBroker$1.postMessage(this.main_thread_target + '.calcTextSizes', tile.id, this.texts[tile.id])).then(function ($await_3) {
          try {
            texts = $await_3;

            if (tile.canceled) {
              log('trace', "Style " + this.name + ": stop tile build because tile was canceled: " + tile.key + ", post-calcTextSizes()");
              return $return([]);
            }

            this.texts[tile.id] = texts || [];

            if (!texts) {
              Collision.abortTile(tile.id);
              return $return([]);
            }

            return $return(this.buildTextLabels(tile, queue));
          } catch ($boundEx) {
            return $Try_1_Catch($boundEx);
          }
        }.bind(this), $Try_1_Catch);
      } catch (e) {
        $Try_1_Catch(e);
      }
    }.bind(this));
  },
  collideAndRenderTextLabels: function collideAndRenderTextLabels(tile, collision_group, queue) {
    return new Promise(function ($return, $error) {
      var labels, texts, rasterized;
      return Promise.resolve(this.prepareTextLabels(tile, queue)).then(function ($await_4) {
        try {
          labels = $await_4;

          if (labels.length === 0) {
            Collision.collide([], collision_group, tile.id);
            return $return({});
          }

          return Promise.resolve(Collision.collide(labels, collision_group, tile.id)).then(function ($await_5) {
            try {
              labels = $await_5;

              if (tile.canceled) {
                log('trace', "stop tile build because tile was canceled: " + tile.key + ", post-collide()");
                return $return({});
              }

              texts = this.texts[tile.id];

              if (texts == null || labels.length === 0) {
                return $return({});
              }

              this.cullTextStyles(texts, labels); // set alignments

              labels.forEach(function (q) {
                var text_settings_key = q.text_settings_key;
                var text_info = texts[text_settings_key] && texts[text_settings_key][q.text];

                if (!text_info.text_settings.can_articulate) {
                  text_info.align = text_info.align || {};
                  text_info.align[q.label.align] = {};
                } else {
                  // consider making it a set
                  if (!text_info.type) {
                    text_info.type = [];
                  }

                  if (text_info.type.indexOf(q.label.type) === -1) {
                    text_info.type.push(q.label.type);
                  }
                }
              }); // second call to main thread, for rasterizing the set of texts

              var $Try_2_Catch = function (e) {
                try {
                  // error thrown if style has been removed from main thread
                  return $return({});
                } catch ($boundEx) {
                  return $error($boundEx);
                }
              };

              try {
                return Promise.resolve(WorkerBroker$1.postMessage(this.main_thread_target + '.rasterizeTexts', tile.id, tile.key, texts)).then(function ($await_6) {
                  try {
                    rasterized = $await_6;

                    if (tile.canceled) {
                      log('trace', "stop tile build because tile was canceled: " + tile.key + ", post-rasterizeTexts()");
                      return $return({});
                    }

                    return $return(Object.assign({
                      labels: labels
                    }, rasterized));
                  } catch ($boundEx) {
                    return $Try_2_Catch($boundEx);
                  }
                }, $Try_2_Catch);
              } catch (e) {
                $Try_2_Catch(e);
              }
            } catch ($boundEx) {
              return $error($boundEx);
            }
          }.bind(this), $error);
        } catch ($boundEx) {
          return $error($boundEx);
        }
      }.bind(this), $error);
    }.bind(this));
  },
  // Remove unused text/style combinations to avoid unnecessary rasterization
  cullTextStyles: function cullTextStyles(texts, labels) {
    // Count how many times each text/style combination is used
    for (var i = 0; i < labels.length; i++) {
      var label = labels[i];
      texts[label.text_settings_key][label.text].ref++;
    } // Remove text/style combinations that have no visible labels


    for (var style in texts) {
      for (var text in texts[style]) {
        // no labels for this text
        if (texts[style][text].ref < 1) {
          delete texts[style][text];
        }
      }
    }

    for (var _style in texts) {
      // no labels for this style
      if (Object.keys(texts[_style]).length === 0) {
        delete texts[_style];
      }
    }
  },
  // Called on main thread from worker, to compute the size of each text string,
  // were it to be rendered. This info is then used to perform initial label culling, *before*
  // labels are actually rendered.
  calcTextSizes: function calcTextSizes(tile_id, texts) {
    return this.canvas.textSizes(tile_id, texts);
  },
  // Called on main thread from worker, to create atlas of labels for a tile
  rasterizeTexts: function rasterizeTexts(tile_id, tile_key, texts) {
    return new Promise(function ($return, $error) {
      var canvas, max_texture_size, textures, texture_prefix;
      canvas = new TextCanvas();
      max_texture_size = Math.min(this.max_texture_size, 2048);
      textures = canvas.setTextureTextPositions(texts, max_texture_size);
      texture_prefix = ['labels', this.name, tile_key, tile_id, text_texture_id, ''].join('-');
      text_texture_id++;
      return Promise.resolve(canvas.rasterize(texts, textures, tile_id, texture_prefix, this.gl)).then(function ($await_7) {
        try {
          textures = $await_7;

          if (!textures) {
            return $return({});
          }

          return $return({
            texts: texts,
            textures: textures
          });
        } catch ($boundEx) {
          return $error($boundEx);
        }
      }, $error);
    }.bind(this));
  },
  preprocessText: function preprocessText(draw) {
    // Font settings are required
    if (!draw || !draw.font || typeof draw.font !== 'object') {
      return;
    } // Colors


    draw.font.fill = StyleParser.createPropertyCache(draw.font.fill);

    if (draw.font.stroke) {
      draw.font.stroke.color = StyleParser.createPropertyCache(draw.font.stroke.color);
    } // Convert font and text stroke sizes


    draw.font.px_size = StyleParser.createPropertyCache(draw.font.size || TextSettings.defaults.size, TextCanvas.fontPixelSize);

    if (draw.font.stroke && draw.font.stroke.width != null) {
      draw.font.stroke.width = StyleParser.createPropertyCache(draw.font.stroke.width, StyleParser.parsePositiveNumber);
    } // Offset (2d array)


    draw.offset = StyleParser.createPropertyCache(draw.offset, function (v) {
      return Array.isArray(v) && v.map(StyleParser.parseNumber);
    }); // Buffer (1d value or or 2d array) - must be >= 0

    draw.buffer = StyleParser.createPropertyCache(draw.buffer, function (v) {
      return (Array.isArray(v) ? v : [v, v]).map(StyleParser.parsePositiveNumber);
    }); // Repeat rules - for text labels, defaults to tile size

    draw.repeat_distance = StyleParser.createPropertyCache(draw.repeat_distance != null ? draw.repeat_distance : Geo$1.tile_size, StyleParser.parsePositiveNumber);
    return draw;
  },
  // Additional text-specific layout settings
  computeTextLayout: function computeTextLayout(target, feature, draw, context, tile, text, text_settings, repeat_group_prefix, orientation) {
    var layout = target || {}; // common settings w/points

    layout = this.computeLayout(layout, feature, draw, context, tile); // repeat rules include the text

    if (layout.repeat_distance) {
      if (repeat_group_prefix) {
        layout.repeat_group += '/' + repeat_group_prefix;
      }

      layout.repeat_group += '/' + text;
    } // Max number of subdivisions to try


    layout.subdiv = tile.overzoom2;
    layout.align = draw.align; // used to fudge width value as text may overflow bounding box if it has italic, bold, etc style
    // TODO rename to more generic, not italic-specific (bold)

    layout.italic = text_settings.style !== 'normal'; // used to determine orientation of text if the text_source has a `left` or `right` key

    if (orientation === 'right') {
      layout.orientation = 1;
    } else if (orientation === 'left') {
      layout.orientation = -1;
    }

    return layout;
  }
};

_typedArray('Float64', 8, function (init) {
  return function Float64Array(data, byteOffset, length) {
    return init(this, data, byteOffset, length);
  };
});

var normalFromMat4_1 = normalFromMat4;

/**
* Calculates a 3x3 normal matrix (transpose inverse) from the 4x4 matrix
*
* @alias mat3.normalFromMat4
* @param {mat3} out mat3 receiving operation result
* @param {mat4} a Mat4 to derive the normal matrix from
*
* @returns {mat3} out
*/
function normalFromMat4(out, a) {
  var a00 = a[0], a01 = a[1], a02 = a[2], a03 = a[3];
  var a10 = a[4], a11 = a[5], a12 = a[6], a13 = a[7];
  var a20 = a[8], a21 = a[9], a22 = a[10], a23 = a[11];
  var a30 = a[12], a31 = a[13], a32 = a[14], a33 = a[15];

  var b00 = a00 * a11 - a01 * a10;
  var b01 = a00 * a12 - a02 * a10;
  var b02 = a00 * a13 - a03 * a10;
  var b03 = a01 * a12 - a02 * a11;
  var b04 = a01 * a13 - a03 * a11;
  var b05 = a02 * a13 - a03 * a12;
  var b06 = a20 * a31 - a21 * a30;
  var b07 = a20 * a32 - a22 * a30;
  var b08 = a20 * a33 - a23 * a30;
  var b09 = a21 * a32 - a22 * a31;
  var b10 = a21 * a33 - a23 * a31;
  var b11 = a22 * a33 - a23 * a32;

  // Calculate the determinant
  var det = b00 * b11
          - b01 * b10
          + b02 * b09
          + b03 * b08
          - b04 * b07
          + b05 * b06;

  if (!det) return null
  det = 1.0 / det;

  out[0] = (a11 * b11 - a12 * b10 + a13 * b09) * det;
  out[1] = (a12 * b08 - a10 * b11 - a13 * b07) * det;
  out[2] = (a10 * b10 - a11 * b08 + a13 * b06) * det;

  out[3] = (a02 * b10 - a01 * b11 - a03 * b09) * det;
  out[4] = (a00 * b11 - a02 * b08 + a03 * b07) * det;
  out[5] = (a01 * b08 - a00 * b10 - a03 * b06) * det;

  out[6] = (a31 * b05 - a32 * b04 + a33 * b03) * det;
  out[7] = (a32 * b02 - a30 * b05 - a33 * b01) * det;
  out[8] = (a30 * b04 - a31 * b02 + a33 * b00) * det;

  return out
}

var invert_1 = invert;

/**
 * Inverts a mat3
 *
 * @alias mat3.invert
 * @param {mat3} out the receiving matrix
 * @param {mat3} a the source matrix
 * @returns {mat3} out
 */
function invert(out, a) {
  var a00 = a[0], a01 = a[1], a02 = a[2];
  var a10 = a[3], a11 = a[4], a12 = a[5];
  var a20 = a[6], a21 = a[7], a22 = a[8];

  var b01 = a22 * a11 - a12 * a21;
  var b11 = -a22 * a10 + a12 * a20;
  var b21 = a21 * a10 - a11 * a20;

  // Calculate the determinant
  var det = a00 * b01 + a01 * b11 + a02 * b21;

  if (!det) return null
  det = 1.0 / det;

  out[0] = b01 * det;
  out[1] = (-a22 * a01 + a02 * a21) * det;
  out[2] = (a12 * a01 - a02 * a11) * det;
  out[3] = b11 * det;
  out[4] = (a22 * a00 - a02 * a20) * det;
  out[5] = (-a12 * a00 + a02 * a10) * det;
  out[6] = b21 * det;
  out[7] = (-a21 * a00 + a01 * a20) * det;
  out[8] = (a11 * a00 - a01 * a10) * det;

  return out
}

var multiply_1 = multiply;

/**
 * Multiplies two mat4's
 *
 * @param {mat4} out the receiving matrix
 * @param {mat4} a the first operand
 * @param {mat4} b the second operand
 * @returns {mat4} out
 */
function multiply(out, a, b) {
    var a00 = a[0], a01 = a[1], a02 = a[2], a03 = a[3],
        a10 = a[4], a11 = a[5], a12 = a[6], a13 = a[7],
        a20 = a[8], a21 = a[9], a22 = a[10], a23 = a[11],
        a30 = a[12], a31 = a[13], a32 = a[14], a33 = a[15];

    // Cache only the current line of the second matrix
    var b0  = b[0], b1 = b[1], b2 = b[2], b3 = b[3];  
    out[0] = b0*a00 + b1*a10 + b2*a20 + b3*a30;
    out[1] = b0*a01 + b1*a11 + b2*a21 + b3*a31;
    out[2] = b0*a02 + b1*a12 + b2*a22 + b3*a32;
    out[3] = b0*a03 + b1*a13 + b2*a23 + b3*a33;

    b0 = b[4]; b1 = b[5]; b2 = b[6]; b3 = b[7];
    out[4] = b0*a00 + b1*a10 + b2*a20 + b3*a30;
    out[5] = b0*a01 + b1*a11 + b2*a21 + b3*a31;
    out[6] = b0*a02 + b1*a12 + b2*a22 + b3*a32;
    out[7] = b0*a03 + b1*a13 + b2*a23 + b3*a33;

    b0 = b[8]; b1 = b[9]; b2 = b[10]; b3 = b[11];
    out[8] = b0*a00 + b1*a10 + b2*a20 + b3*a30;
    out[9] = b0*a01 + b1*a11 + b2*a21 + b3*a31;
    out[10] = b0*a02 + b1*a12 + b2*a22 + b3*a32;
    out[11] = b0*a03 + b1*a13 + b2*a23 + b3*a33;

    b0 = b[12]; b1 = b[13]; b2 = b[14]; b3 = b[15];
    out[12] = b0*a00 + b1*a10 + b2*a20 + b3*a30;
    out[13] = b0*a01 + b1*a11 + b2*a21 + b3*a31;
    out[14] = b0*a02 + b1*a12 + b2*a22 + b3*a32;
    out[15] = b0*a03 + b1*a13 + b2*a23 + b3*a33;
    return out;
}

var translate_1 = translate;

/**
 * Translate a mat4 by the given vector
 *
 * @param {mat4} out the receiving matrix
 * @param {mat4} a the matrix to translate
 * @param {vec3} v vector to translate by
 * @returns {mat4} out
 */
function translate(out, a, v) {
    var x = v[0], y = v[1], z = v[2],
        a00, a01, a02, a03,
        a10, a11, a12, a13,
        a20, a21, a22, a23;

    if (a === out) {
        out[12] = a[0] * x + a[4] * y + a[8] * z + a[12];
        out[13] = a[1] * x + a[5] * y + a[9] * z + a[13];
        out[14] = a[2] * x + a[6] * y + a[10] * z + a[14];
        out[15] = a[3] * x + a[7] * y + a[11] * z + a[15];
    } else {
        a00 = a[0]; a01 = a[1]; a02 = a[2]; a03 = a[3];
        a10 = a[4]; a11 = a[5]; a12 = a[6]; a13 = a[7];
        a20 = a[8]; a21 = a[9]; a22 = a[10]; a23 = a[11];

        out[0] = a00; out[1] = a01; out[2] = a02; out[3] = a03;
        out[4] = a10; out[5] = a11; out[6] = a12; out[7] = a13;
        out[8] = a20; out[9] = a21; out[10] = a22; out[11] = a23;

        out[12] = a00 * x + a10 * y + a20 * z + a[12];
        out[13] = a01 * x + a11 * y + a21 * z + a[13];
        out[14] = a02 * x + a12 * y + a22 * z + a[14];
        out[15] = a03 * x + a13 * y + a23 * z + a[15];
    }

    return out;
}

var scale_1 = scale;

/**
 * Scales the mat4 by the dimensions in the given vec3
 *
 * @param {mat4} out the receiving matrix
 * @param {mat4} a the matrix to scale
 * @param {vec3} v the vec3 to scale the matrix by
 * @returns {mat4} out
 **/
function scale(out, a, v) {
    var x = v[0], y = v[1], z = v[2];

    out[0] = a[0] * x;
    out[1] = a[1] * x;
    out[2] = a[2] * x;
    out[3] = a[3] * x;
    out[4] = a[4] * y;
    out[5] = a[5] * y;
    out[6] = a[6] * y;
    out[7] = a[7] * y;
    out[8] = a[8] * z;
    out[9] = a[9] * z;
    out[10] = a[10] * z;
    out[11] = a[11] * z;
    out[12] = a[12];
    out[13] = a[13];
    out[14] = a[14];
    out[15] = a[15];
    return out;
}

var perspective_1 = perspective;

/**
 * Generates a perspective projection matrix with the given bounds
 *
 * @param {mat4} out mat4 frustum matrix will be written into
 * @param {number} fovy Vertical field of view in radians
 * @param {number} aspect Aspect ratio. typically viewport width/height
 * @param {number} near Near bound of the frustum
 * @param {number} far Far bound of the frustum
 * @returns {mat4} out
 */
function perspective(out, fovy, aspect, near, far) {
    var f = 1.0 / Math.tan(fovy / 2),
        nf = 1 / (near - far);
    out[0] = f / aspect;
    out[1] = 0;
    out[2] = 0;
    out[3] = 0;
    out[4] = 0;
    out[5] = f;
    out[6] = 0;
    out[7] = 0;
    out[8] = 0;
    out[9] = 0;
    out[10] = (far + near) * nf;
    out[11] = -1;
    out[12] = 0;
    out[13] = 0;
    out[14] = (2 * far * near) * nf;
    out[15] = 0;
    return out;
}

var identity_1 = identity;

/**
 * Set a mat4 to the identity matrix
 *
 * @param {mat4} out the receiving matrix
 * @returns {mat4} out
 */
function identity(out) {
    out[0] = 1;
    out[1] = 0;
    out[2] = 0;
    out[3] = 0;
    out[4] = 0;
    out[5] = 1;
    out[6] = 0;
    out[7] = 0;
    out[8] = 0;
    out[9] = 0;
    out[10] = 1;
    out[11] = 0;
    out[12] = 0;
    out[13] = 0;
    out[14] = 0;
    out[15] = 1;
    return out;
}

var lookAt_1 = lookAt;

/**
 * Generates a look-at matrix with the given eye position, focal point, and up axis
 *
 * @param {mat4} out mat4 frustum matrix will be written into
 * @param {vec3} eye Position of the viewer
 * @param {vec3} center Point the viewer is looking at
 * @param {vec3} up vec3 pointing up
 * @returns {mat4} out
 */
function lookAt(out, eye, center, up) {
    var x0, x1, x2, y0, y1, y2, z0, z1, z2, len,
        eyex = eye[0],
        eyey = eye[1],
        eyez = eye[2],
        upx = up[0],
        upy = up[1],
        upz = up[2],
        centerx = center[0],
        centery = center[1],
        centerz = center[2];

    if (Math.abs(eyex - centerx) < 0.000001 &&
        Math.abs(eyey - centery) < 0.000001 &&
        Math.abs(eyez - centerz) < 0.000001) {
        return identity_1(out);
    }

    z0 = eyex - centerx;
    z1 = eyey - centery;
    z2 = eyez - centerz;

    len = 1 / Math.sqrt(z0 * z0 + z1 * z1 + z2 * z2);
    z0 *= len;
    z1 *= len;
    z2 *= len;

    x0 = upy * z2 - upz * z1;
    x1 = upz * z0 - upx * z2;
    x2 = upx * z1 - upy * z0;
    len = Math.sqrt(x0 * x0 + x1 * x1 + x2 * x2);
    if (!len) {
        x0 = 0;
        x1 = 0;
        x2 = 0;
    } else {
        len = 1 / len;
        x0 *= len;
        x1 *= len;
        x2 *= len;
    }

    y0 = z1 * x2 - z2 * x1;
    y1 = z2 * x0 - z0 * x2;
    y2 = z0 * x1 - z1 * x0;

    len = Math.sqrt(y0 * y0 + y1 * y1 + y2 * y2);
    if (!len) {
        y0 = 0;
        y1 = 0;
        y2 = 0;
    } else {
        len = 1 / len;
        y0 *= len;
        y1 *= len;
        y2 *= len;
    }

    out[0] = x0;
    out[1] = y0;
    out[2] = z0;
    out[3] = 0;
    out[4] = x1;
    out[5] = y1;
    out[6] = z1;
    out[7] = 0;
    out[8] = x2;
    out[9] = y2;
    out[10] = z2;
    out[11] = 0;
    out[12] = -(x0 * eyex + x1 * eyey + x2 * eyez);
    out[13] = -(y0 * eyex + y1 * eyey + y2 * eyez);
    out[14] = -(z0 * eyex + z1 * eyey + z2 * eyez);
    out[15] = 1;

    return out;
}

var copy_1 = copy;

/**
 * Copy the values from one mat4 to another
 *
 * @param {mat4} out the receiving matrix
 * @param {mat4} a the source matrix
 * @returns {mat4} out
 */
function copy(out, a) {
    out[0] = a[0];
    out[1] = a[1];
    out[2] = a[2];
    out[3] = a[3];
    out[4] = a[4];
    out[5] = a[5];
    out[6] = a[6];
    out[7] = a[7];
    out[8] = a[8];
    out[9] = a[9];
    out[10] = a[10];
    out[11] = a[11];
    out[12] = a[12];
    out[13] = a[13];
    out[14] = a[14];
    out[15] = a[15];
    return out;
}

// Partial import of gl-matrix via modularized stack-gl forks
// https://github.com/toji/gl-matrix
// https://github.com/stackgl
// vec3
// Substitute 64-bit version
// We need the extra precision when multiplying matrices w/mercator projected values
var vec3 = {
  fromValues: function fromValues(x, y, z) {
    var out = new Float64Array(3);
    out[0] = x;
    out[1] = y;
    out[2] = z;
    return out;
  }
}; // mat3
var mat3 = {
  normalFromMat4: normalFromMat4_1,
  invert: invert_1
}; // mat4
var mat4 = {
  multiply: multiply_1,
  translate: translate_1,
  scale: scale_1,
  perspective: perspective_1,
  lookAt: lookAt_1,
  identity: identity_1,
  copy: copy_1
};

var Camera =
/*#__PURE__*/
function () {
  function Camera(name, view, options) {
    if (options === void 0) {
      options = {};
    }

    this.view = view;
    this.position = options.position;
    this.zoom = options.zoom;
  } // Create a camera by type name, factory-style


  Camera.create = function create(name, view, config) {
    switch (config.type) {
      case 'isometric':
        return new IsometricCamera(name, view, config);

      case 'flat':
        return new FlatCamera(name, view, config);

      case 'perspective':
      /* falls through */

      default:
        return new PerspectiveCamera(name, view, config);
    }
  } // Update method called once per frame
  ;

  var _proto = Camera.prototype;

  _proto.update = function update() {} // Called once per frame per program (e.g. for main render pass, then for each additional pass for feature selection, etc.)
  ;

  _proto.setupProgram = function setupProgram()
  /*program*/
  {} // Sync camera position/zoom to scene view
  ;

  _proto.updateView = function updateView() {
    if (this.position || this.zoom) {
      var view = {};

      if (this.position) {
        view = {
          lng: this.position[0],
          lat: this.position[1],
          zoom: this.position[2]
        };
      }

      if (this.zoom) {
        view.zoom = this.zoom;
      }

      this.view.setView(view);
    }
  } // Set model-view and normal matrices
  ;

  _proto.setupMatrices = function setupMatrices(matrices, program) {
    // Model view matrix - transform tile space into view space (meters, relative to camera)
    mat4.multiply(matrices.model_view32, this.view_matrix, matrices.model);
    program.uniform('Matrix4fv', 'u_modelView', matrices.model_view32); // Normal matrices - transforms surface normals into view space

    mat3.normalFromMat4(matrices.normal32, matrices.model_view32);
    mat3.invert(matrices.inverse_normal32, matrices.normal32);
    program.uniform('Matrix3fv', 'u_normalMatrix', matrices.normal32);
    program.uniform('Matrix3fv', 'u_inverseNormalMatrix', matrices.inverse_normal32);
  };

  return Camera;
}();

var PerspectiveCamera =
/*#__PURE__*/
function (_Camera) {
  _inheritsLoose(PerspectiveCamera, _Camera);

  function PerspectiveCamera(name, view, options) {
    var _this;

    if (options === void 0) {
      options = {};
    }

    _this = _Camera.call(this, name, view, options) || this;
    _this.type = 'perspective'; // a single scalar, or pairs of stops mapping zoom levels, e.g. [zoom, focal length]

    _this.focal_length = options.focal_length;
    _this.fov = options.fov;

    if (!_this.focal_length && !_this.fov) {
      // Default focal length ranges by zoom
      _this.focal_length = [[16, 2], [17, 2.5], [18, 3], [19, 4], [20, 6]];
    }

    _this.vanishing_point = options.vanishing_point || [0, 0]; // [x, y]

    _this.vanishing_point = _this.vanishing_point.map(parseFloat); // we implicitly only support px units here

    _this.vanishing_point_skew = [];
    _this.position_meters = null;
    _this.view_matrix = new Float64Array(16);
    _this.projection_matrix = new Float32Array(16); // 'camera' is the name of the shader block, e.g. determines where in the shader this code is injected

    ShaderProgram.replaceBlock('camera', "\n            uniform mat4 u_projection;\n            uniform vec3 u_eye;\n            uniform vec2 u_vanishing_point;\n\n            void cameraProjection (inout vec4 position) {\n                position = u_projection * position;\n            }");
    return _this;
  } // Constrains the camera so that the viewable area matches given the viewport height
  // (in world space, e.g. meters), given either a camera focal length or field-of-view
  // (focal length is used if both are passed).


  var _proto2 = PerspectiveCamera.prototype;

  _proto2.constrainCamera = function constrainCamera(_ref) {
    var view_height = _ref.view_height,
        height = _ref.height,
        focal_length = _ref.focal_length,
        fov = _ref.fov;

    // Solve for camera height
    if (!height) {
      // We have focal length, calculate FOV
      if (focal_length) {
        fov = Math.atan(1 / focal_length) * 2;
      } // We have FOV, calculate focal length
      else if (fov) {
          fov = fov * Math.PI / 180; // convert FOV degrees to radians

          focal_length = 1 / Math.tan(fov / 2);
        } // Distance that camera should be from ground such that it fits the field of view expected
      // for a conventional web mercator map at the current zoom level and camera focal length


      height = view_height / 2 * focal_length;
    } // Solve for camera focal length / field-of-view
    else {
        focal_length = 2 * height / view_height;
        fov = Math.atan(1 / focal_length) * 2;
      }

    return {
      view_height: view_height,
      height: height,
      focal_length: focal_length,
      fov: fov
    };
  };

  _proto2.updateMatrices = function updateMatrices() {
    // TODO: only re-calculate these vars when necessary
    // Height of the viewport in meters at current zoom
    var viewport_height = this.view.size.css.height * this.view.meters_per_pixel; // Compute camera properties to fit desired view

    var _this$constrainCamera = this.constrainCamera({
      view_height: viewport_height,
      focal_length: Utils.interpolate(this.view.zoom, this.focal_length),
      fov: Utils.interpolate(this.view.zoom, this.fov)
    }),
        height = _this$constrainCamera.height,
        fov = _this$constrainCamera.fov; // View matrix


    var position = [this.view.center.meters.x, this.view.center.meters.y, height];
    this.position_meters = position; // mat4.lookAt(this.view_matrix,
    //     vec3.fromValues(...position),
    //     vec3.fromValues(position[0], position[1], height - 1),
    //     vec3.fromValues(0, 1, 0));
    // Exclude camera height from view matrix

    mat4.lookAt(this.view_matrix, vec3.fromValues(position[0], position[1], 0), vec3.fromValues(position[0], position[1], -1), vec3.fromValues(0, 1, 0)); // Projection matrix

    mat4.perspective(this.projection_matrix, fov, this.view.aspect, 1, height * 2); // Convert vanishing point from pixels to viewport space

    this.vanishing_point_skew[0] = this.vanishing_point[0] / this.view.size.css.width;
    this.vanishing_point_skew[1] = this.vanishing_point[1] / this.view.size.css.height; // Adjust projection matrix to include vanishing point skew

    this.projection_matrix[8] = -this.vanishing_point_skew[0] * 2; // z column of x row, e.g. amount z skews x

    this.projection_matrix[9] = -this.vanishing_point_skew[1] * 2; // z column of y row, e.g. amount z skews y
    // Translate geometry into the distance so that camera is appropriate height above ground
    // Additionally, adjust xy to compensate for any vanishing point skew, e.g. move geometry so that the displayed g
    // plane of the map matches that expected by a traditional web mercator map at this [lat, lng, zoom].

    mat4.translate(this.projection_matrix, this.projection_matrix, vec3.fromValues(viewport_height / 2 * this.view.aspect * (-this.vanishing_point_skew[0] * 2), viewport_height / 2 * (-this.vanishing_point_skew[1] * 2), 0)); // Include camera height in projection matrix

    mat4.translate(this.projection_matrix, this.projection_matrix, vec3.fromValues(0, 0, -height));
  };

  _proto2.update = function update() {
    _Camera.prototype.update.call(this);

    this.updateMatrices();
  };

  _proto2.setupProgram = function setupProgram(program) {
    program.uniform('Matrix4fv', 'u_projection', this.projection_matrix);
    program.uniform('3f', 'u_eye', [0, 0, this.position_meters[2]]);
    program.uniform('2fv', 'u_vanishing_point', this.vanishing_point_skew);
  };

  return PerspectiveCamera;
}(Camera); // Isometric-style projection
// Note: this is actually an "axonometric" projection, but I'm using the colloquial term isometric because it is more recognizable.
// An isometric projection is a specific subset of axonometric projections.
// 'axis' determines the xy skew applied to a vertex based on its z coordinate, e.g. [0, 1] axis causes buildings to be drawn
// straight upwards on screen at their true height, [0, .5] would draw them up at half-height, [1, 0] would be sideways, etc.


var IsometricCamera =
/*#__PURE__*/
function (_Camera2) {
  _inheritsLoose(IsometricCamera, _Camera2);

  function IsometricCamera(name, view, options) {
    var _this2;

    if (options === void 0) {
      options = {};
    }

    _this2 = _Camera2.call(this, name, view, options) || this;
    _this2.type = 'isometric';
    _this2.axis = options.axis || {
      x: 0,
      y: 1
    };

    if (_this2.axis.length === 2) {
      _this2.axis = {
        x: _this2.axis[0],
        y: _this2.axis[1]
      }; // allow axis to also be passed as 2-elem array
    }

    _this2.position_meters = null;
    _this2.viewport_height = null;
    _this2.view_matrix = new Float64Array(16);
    _this2.projection_matrix = new Float32Array(16); // 'camera' is the name of the shader block, e.g. determines where in the shader this code is injected

    ShaderProgram.replaceBlock('camera', "\n            uniform mat4 u_projection;\n            uniform vec3 u_eye;\n            uniform vec2 u_vanishing_point;\n\n            void cameraProjection (inout vec4 position) {\n                position = u_projection * position;\n                // position.xy += position.z * u_isometric_axis;\n\n                // Reverse z for depth buffer so up is negative,\n                // and scale down values so objects higher than one screen height will not get clipped\n                // pull forward slightly to avoid going past far clipping plane\n                position.z = -position.z / 100. + 1. - 0.001;\n            }");
    return _this2;
  }

  var _proto3 = IsometricCamera.prototype;

  _proto3.update = function update() {
    _Camera2.prototype.update.call(this);

    this.viewport_height = this.view.size.css.height * this.view.meters_per_pixel;
    var position = [this.view.center.meters.x, this.view.center.meters.y, this.viewport_height];
    this.position_meters = position; // View

    mat4.identity(this.view_matrix);
    mat4.translate(this.view_matrix, this.view_matrix, vec3.fromValues(-position[0], -position[1], 0)); // Projection

    mat4.identity(this.projection_matrix); // apply isometric skew

    this.projection_matrix[8] = this.axis.x / this.view.aspect; // z column of x row, e.g. amount z skews x

    this.projection_matrix[9] = this.axis.y; // z column of x row, e.g. amount z skews y
    // convert meters to viewport

    mat4.scale(this.projection_matrix, this.projection_matrix, vec3.fromValues(2 / this.view.size.meters.x, 2 / this.view.size.meters.y, 2 / this.view.size.meters.y));
  };

  _proto3.setupProgram = function setupProgram(program) {
    program.uniform('Matrix4fv', 'u_projection', this.projection_matrix);
    program.uniform('3fv', 'u_eye', [0, 0, this.viewport_height]); // program.uniform('3f', 'u_eye', this.viewport_height * this.axis.x, this.viewport_height * this.axis.y, this.viewport_height);

    program.uniform('2fv', 'u_vanishing_point', [0, 0]);
  };

  return IsometricCamera;
}(Camera); // Flat projection (e.g. just top-down, no perspective) - a degenerate isometric camera


var FlatCamera =
/*#__PURE__*/
function (_IsometricCamera) {
  _inheritsLoose(FlatCamera, _IsometricCamera);

  function FlatCamera(name, view, options) {
    var _this3;

    if (options === void 0) {
      options = {};
    }

    _this3 = _IsometricCamera.call(this, name, view, options) || this;
    _this3.type = 'flat';
    return _this3;
  }

  var _proto4 = FlatCamera.prototype;

  _proto4.update = function update() {
    // Axis is fixed to (0, 0) for flat camera
    this.axis.x = 0;
    this.axis.y = 0;

    _IsometricCamera.prototype.update.call(this);
  };

  return FlatCamera;
}(IsometricCamera);

var VIEW_PAN_SNAP_TIME = 0.5;

var View =
/*#__PURE__*/
function () {
  function View(scene, options) {
    subscribeMixin(this);
    this.scene = scene;
    this.createMatrices();
    this.zoom = null;
    this.center = null;
    this.bounds = null;
    this.meters_per_pixel = null;
    this.panning = false;
    this.panning_stop_at = 0;
    this.pan_snap_timer = 0;
    this.zoom_direction = 0;
    this.user_input_at = 0;
    this.user_input_timeout = 50;
    this.user_input_active = false; // Size of viewport in CSS pixels, device pixels, and mercator meters

    this.size = {
      css: {},
      device: {},
      meters: {}
    };
    this.aspect = null;
    this.buffer = 0;
    this.continuous_zoom = typeof options.continuousZoom === 'boolean' ? options.continuousZoom : true;
    this.wrap = options.wrapView === false ? false : true;
    this.preserve_tiles_within_zoom = 1;
    this.reset();
  } // Reset state before scene config is updated


  var _proto = View.prototype;

  _proto.reset = function reset() {
    this.createCamera();
  } // Create camera
  ;

  _proto.createCamera = function createCamera() {
    var active_camera = this.getActiveCamera();

    if (active_camera) {
      this.camera = Camera.create(active_camera, this, this.scene.config.cameras[active_camera]);
      this.camera.updateView();
    }
  } // Get active camera - for public API
  ;

  _proto.getActiveCamera = function getActiveCamera() {
    if (this.scene.config && this.scene.config.cameras) {
      for (var name in this.scene.config.cameras) {
        if (this.scene.config.cameras[name].active) {
          return name;
        }
      } // If no camera set as active, use first one


      var keys = Object.keys(this.scene.config.cameras);
      return keys.length && keys[0];
    }
  } // Set active camera and recompile - for public API
  ;

  _proto.setActiveCamera = function setActiveCamera(name) {
    var prev = this.getActiveCamera();

    if (prev === name) {
      return name;
    }

    if (this.scene.config.cameras[name]) {
      this.scene.config.cameras[name].active = true; // Clear previously active camera

      if (prev && this.scene.config.cameras[prev]) {
        delete this.scene.config.cameras[prev].active;
      }
    }

    this.scene.updateConfig({
      rebuild: false,
      normalize: false
    });
    return this.getActiveCamera();
  } // Update method called once per frame
  ;

  _proto.update = function update() {
    if (this.camera != null && this.ready()) {
      this.camera.update();
    }

    this.pan_snap_timer = (+new Date() - this.panning_stop_at) / 1000;
    this.user_input_active = +new Date() - this.user_input_at < this.user_input_timeout;
  } // Set logical pixel size of viewport
  ;

  _proto.setViewportSize = function setViewportSize(width, height) {
    this.size.css = {
      width: width,
      height: height
    };
    this.size.device = {
      width: Math.round(this.size.css.width * Utils.device_pixel_ratio),
      height: Math.round(this.size.css.height * Utils.device_pixel_ratio)
    };
    this.aspect = this.size.css.width / this.size.css.height;
    this.updateBounds();
  } // Set the map view, can be passed an object with lat/lng and/or zoom
  ;

  _proto.setView = function setView(_temp) {
    var _ref = _temp === void 0 ? {} : _temp,
        lng = _ref.lng,
        lat = _ref.lat,
        zoom = _ref.zoom;

    var changed = false; // Set center

    if (typeof lng === 'number' && typeof lat === 'number') {
      if (!this.center || lng !== this.center.lng || lat !== this.center.lat) {
        changed = true;
        this.center = {
          lng: lng,
          lat: lat
        };
      }
    } // Set zoom


    if (typeof zoom === 'number' && zoom !== this.zoom) {
      changed = true;
      this.setZoom(zoom);
    }

    if (changed) {
      this.updateBounds();
    }

    return changed;
  };

  _proto.setZoom = function setZoom(zoom) {
    var last_tile_zoom = this.tile_zoom;
    var tile_zoom = this.baseZoom(zoom);

    if (!this.continuous_zoom) {
      zoom = tile_zoom;
    }

    if (tile_zoom !== last_tile_zoom) {
      this.zoom_direction = tile_zoom > last_tile_zoom ? 1 : -1;
    }

    this.zoom = zoom;
    this.tile_zoom = tile_zoom;
    this.updateBounds();
    this.scene.requestRedraw();
  } // Choose the base zoom level to use for a given fractional zoom
  ;

  _proto.baseZoom = function baseZoom(zoom) {
    return Math.floor(zoom);
  };

  _proto.setPanning = function setPanning(panning) {
    this.panning = panning;

    if (!this.panning) {
      this.panning_stop_at = +new Date();
    }
  };

  _proto.markUserInput = function markUserInput() {
    this.user_input_at = +new Date();
  };

  _proto.ready = function ready() {
    // TODO: better concept of "readiness" state?
    if (typeof this.size.css.width !== 'number' || typeof this.size.css.height !== 'number' || this.center == null || typeof this.zoom !== 'number') {
      return false;
    }

    return true;
  } // Calculate viewport bounds based on current center and zoom
  ;

  _proto.updateBounds = function updateBounds() {
    if (!this.ready()) {
      return;
    }

    this.meters_per_pixel = Geo$1.metersPerPixel(this.zoom); // Size of the half-viewport in meters at current zoom

    this.size.meters = {
      x: this.size.css.width * this.meters_per_pixel,
      y: this.size.css.height * this.meters_per_pixel
    }; // Center of viewport in meters, and tile

    var m = Geo$1.latLngToMeters([this.center.lng, this.center.lat]);
    this.center.meters = {
      x: m[0],
      y: m[1]
    };
    this.center.tile = Geo$1.tileForMeters([this.center.meters.x, this.center.meters.y], this.tile_zoom); // Bounds in meters

    this.bounds = {
      sw: {
        x: this.center.meters.x - this.size.meters.x / 2,
        y: this.center.meters.y - this.size.meters.y / 2
      },
      ne: {
        x: this.center.meters.x + this.size.meters.x / 2,
        y: this.center.meters.y + this.size.meters.y / 2
      }
    };
    this.scene.tile_manager.updateTilesForView();
    this.trigger('move');
    this.scene.requestRedraw(); // TODO automate via move event?
  };

  _proto.findVisibleTileCoordinates = function findVisibleTileCoordinates() {
    if (!this.bounds) {
      return [];
    }

    var z = this.tile_zoom;
    var sw = Geo$1.tileForMeters([this.bounds.sw.x, this.bounds.sw.y], z);
    var ne = Geo$1.tileForMeters([this.bounds.ne.x, this.bounds.ne.y], z);
    var range = [sw.x - this.buffer, ne.x + this.buffer, // x
    ne.y - this.buffer, sw.y + this.buffer // y
    ];

    if (this.wrap === false) {
      // prevent tiles from wrapping across antimeridian
      var tmax = (1 << z) - 1; // max xy tile number for this zoom

      range = range.map(function (v) {
        return Math.min(Math.max(0, v), tmax);
      });
    }

    var coords = [];

    for (var x = range[0]; x <= range[1]; x++) {
      for (var y = range[2]; y <= range[3]; y++) {
        coords.push(TileID.coord({
          x: x,
          y: y,
          z: z
        }));
      }
    }

    return coords;
  } // Remove tiles too far outside of view
  ;

  _proto.pruneTilesForView = function pruneTilesForView() {
    var _this = this;

    // TODO: will this function ever be called when view isn't ready?
    if (!this.ready()) {
      return;
    }

    this.scene.tile_manager.removeTiles(function (tile) {
      // Ignore visible tiles
      if (tile.visible || tile.isProxy()) {
        return false;
      } // Remove tiles outside given zoom that are still loading


      if (tile.loading && tile.style_z !== _this.tile_zoom) {
        return true;
      } // Discard if too far from current zoom


      var zdiff = Math.abs(tile.style_z - _this.tile_zoom);
      var preserve_tiles_within_zoom = tile.preserve_tiles_within_zoom != null ? tile.preserve_tiles_within_zoom : _this.preserve_tiles_within_zoom; // optionally tile source specific

      if (zdiff > preserve_tiles_within_zoom) {
        return true;
      } // Discard tiles outside an area surrounding the viewport, handling tiles at different zooms
      // Get min and max tiles for the viewport, at the scale of the tile currently being evaluated


      var view_buffer = _this.meters_per_pixel * Geo$1.tile_size; // buffer area to keep tiles surrounding viewport

      var view_tile_min = TileID.coordAtZoom(Geo$1.tileForMeters([_this.center.meters.x - _this.size.meters.x / 2 - view_buffer, _this.center.meters.y + _this.size.meters.y / 2 + view_buffer], _this.tile_zoom), tile.coords.z);
      var view_tile_max = TileID.coordAtZoom(Geo$1.tileForMeters([_this.center.meters.x + _this.size.meters.x / 2 + view_buffer, _this.center.meters.y - _this.size.meters.y / 2 - view_buffer], _this.tile_zoom), tile.coords.z);

      if (tile.coords.x < view_tile_min.x || tile.coords.x > view_tile_max.x || tile.coords.y < view_tile_min.y || tile.coords.y > view_tile_max.y) {
        log('trace', "View: remove tile " + tile.key + " (as " + tile.coords.key + ") " + ("for being too far out of visible area (" + view_tile_min.key + ", " + view_tile_max.key + ")"));
        return true;
      }

      return false;
    });
  } // Allocate model-view matrices
  // 64-bit versions are for CPU calcuations
  // 32-bit versions are downsampled and sent to GPU
  ;

  _proto.createMatrices = function createMatrices() {
    this.matrices = {};
    this.matrices.model = new Float64Array(16);
    this.matrices.model32 = new Float32Array(16);
    this.matrices.model_view = new Float64Array(16);
    this.matrices.model_view32 = new Float32Array(16);
    this.matrices.normal = new Float64Array(9);
    this.matrices.normal32 = new Float32Array(9);
    this.matrices.inverse_normal32 = new Float32Array(9);
  } // Calculate and set model/view and normal matrices for a tile
  ;

  _proto.setupTile = function setupTile(tile, program) {
    // Tile-specific state
    // TODO: calc these once per tile (currently being needlessly re-calculated per-tile-per-style)
    tile.setupProgram(this.matrices, program); // Model-view and normal matrices

    this.camera.setupMatrices(this.matrices, program);
  } // Set general uniforms that must be updated once per program
  ;

  _proto.setupProgram = function setupProgram(program) {
    program.uniform('2fv', 'u_resolution', [this.size.device.width, this.size.device.height]);
    program.uniform('3fv', 'u_map_position', [this.center.meters.x, this.center.meters.y, this.zoom]);
    program.uniform('1f', 'u_meters_per_pixel', this.meters_per_pixel);
    program.uniform('1f', 'u_device_pixel_ratio', Utils.device_pixel_ratio);
    program.uniform('1f', 'u_view_pan_snap_timer', this.pan_snap_timer);
    program.uniform('1i', 'u_view_panning', this.panning);
    this.camera.setupProgram(program);
  } // View requires some animation, such as after panning stops
  ;

  _proto.isAnimating = function isAnimating() {
    return this.pan_snap_timer <= VIEW_PAN_SNAP_TIME;
  };

  return View;
}();

var points_vs = "uniform vec2 u_resolution;\nuniform float u_time;\nuniform vec3 u_map_position;\nuniform vec4 u_tile_origin;\nuniform float u_tile_proxy_order_offset;\nuniform bool u_tile_fade_in;\nuniform float u_meters_per_pixel;\nuniform float u_device_pixel_ratio;\nuniform float u_visible_time;\nuniform bool u_view_panning;\nuniform float u_view_pan_snap_timer;\n\nuniform mat4 u_model;\nuniform mat4 u_modelView;\nuniform mat3 u_normalMatrix;\nuniform mat3 u_inverseNormalMatrix;\n\nattribute vec4 a_position;\nattribute vec4 a_shape;\nattribute vec4 a_color;\nattribute vec2 a_texcoord;\nattribute vec2 a_offset;\n\nuniform float u_point_type;\n\n#ifdef TANGRAM_CURVED_LABEL\n    attribute vec4 a_offsets;\n    attribute vec4 a_pre_angles;\n    attribute vec4 a_angles;\n#endif\n\nvarying vec4 v_color;\nvarying vec2 v_texcoord;\nvarying vec4 v_world_position;\nvarying float v_alpha_factor;\n\n#ifdef TANGRAM_HAS_SHADER_POINTS\n    attribute float a_outline_edge;\n    attribute vec4 a_outline_color;\n\n    varying float v_outline_edge;\n    varying vec4 v_outline_color;\n    varying float v_aa_offset;\n#endif\n\n#ifdef TANGRAM_SHOW_HIDDEN_LABELS\n    varying float v_label_hidden;\n#endif\n\n#define TANGRAM_PI 3.14159265359\n#define TANGRAM_NORMAL vec3(0., 0., 1.)\n\n#pragma tangram: camera\n#pragma tangram: material\n#pragma tangram: lighting\n#pragma tangram: raster\n#pragma tangram: global\n\nvec2 rotate2D(vec2 _st, float _angle) {\n    return mat2(cos(_angle),-sin(_angle),\n                sin(_angle),cos(_angle)) * _st;\n}\n\n#ifdef TANGRAM_CURVED_LABEL\n    // Assumes stops are [0, 0.33, 0.66, 0.99];\n    float mix4linear(float a, float b, float c, float d, float x) {\n        x = clamp(x, 0., 1.);\n        return mix(mix(a, b, 3. * x),\n                   mix(b,\n                       mix(c, d, 3. * (max(x, .66) - .66)),\n                       3. * (clamp(x, .33, .66) - .33)),\n                   step(0.33, x)\n                );\n    }\n#endif\n\nvoid main() {\n    // Initialize globals\n    #pragma tangram: setup\n\n    // discard hidden labels by collapsing into degenerate triangle\n    #ifndef TANGRAM_SHOW_HIDDEN_LABELS\n        if (a_shape.w == 0.) {\n            gl_Position = vec4(0., 0., 0., 1.);\n            return;\n        }\n    #else\n        // highlight hidden label in fragment shader for debugging\n        if (a_shape.w == 0.) {\n            v_label_hidden = 1.; // label debug testing\n        }\n        else {\n            v_label_hidden = 0.;\n        }\n    #endif\n\n    v_alpha_factor = 1.0;\n    v_color = a_color;\n    v_texcoord = a_texcoord; // UV from vertex attribute\n\n    #ifdef TANGRAM_HAS_SHADER_POINTS\n        v_outline_color = a_outline_color;\n        v_outline_edge = a_outline_edge;\n\n        if (u_point_type == TANGRAM_POINT_TYPE_SHADER) { // shader point\n            // use point dimensions for UVs instead (ignore attribute), add antialiasing info for fragment shader\n            float size = abs(a_shape.x / 128.); // radius in pixels\n            v_texcoord = sign(a_shape.xy) * (size + 1.) / size;\n            size += 2.;\n            v_aa_offset = 2. / size;\n        }\n    #endif\n\n    // Position\n    vec4 position = u_modelView * vec4(a_position.xyz, 1.);\n\n    // Apply positioning and scaling in screen space\n    vec2 shape = a_shape.xy / 256.;                 // values have an 8-bit fraction\n    vec2 offset = vec2(a_offset.x, -a_offset.y);    // flip y to make it point down\n\n    float zoom = clamp(u_map_position.z - u_tile_origin.z, 0., 1.); //fract(u_map_position.z);\n    float theta = a_shape.z / 4096.;\n\n    #ifdef TANGRAM_CURVED_LABEL\n        //TODO: potential bug? null is passed in for non-curved labels, otherwise the first offset will be 0\n        if (a_offsets[0] != 0.){\n            vec4 angles_scaled = (TANGRAM_PI / 16384.) * a_angles;\n            vec4 pre_angles_scaled = (TANGRAM_PI / 128.) * a_pre_angles;\n            vec4 offsets_scaled = (1. / 64.) * a_offsets;\n\n            float pre_angle = mix4linear(pre_angles_scaled[0], pre_angles_scaled[1], pre_angles_scaled[2], pre_angles_scaled[3], zoom);\n            float angle = mix4linear(angles_scaled[0], angles_scaled[1], angles_scaled[2], angles_scaled[3], zoom);\n            float offset_curve = mix4linear(offsets_scaled[0], offsets_scaled[1], offsets_scaled[2], offsets_scaled[3], zoom);\n\n            shape = rotate2D(shape, pre_angle); // rotate in place\n            shape.x += offset_curve;            // offset for curved label segment\n            shape = rotate2D(shape, angle);     // rotate relative to curved label anchor\n            shape += rotate2D(offset, theta);   // offset if specified in the scene file\n        }\n        else {\n            shape = rotate2D(shape + offset, theta);\n        }\n    #else\n        shape = rotate2D(shape + offset, theta);\n    #endif\n\n    // Fade in (if requested) based on time mesh has been visible.\n    // Value passed to fragment shader in the v_alpha_factor varying\n    #ifdef TANGRAM_FADE_IN_RATE\n        if (u_tile_fade_in) {\n            v_alpha_factor *= clamp(u_visible_time * TANGRAM_FADE_IN_RATE, 0., 1.);\n        }\n    #endif\n\n    // World coordinates for 3d procedural textures\n    v_world_position = u_model * position;\n    v_world_position.xy += shape * u_meters_per_pixel;\n    v_world_position = wrapWorldPosition(v_world_position);\n\n    // Modify position before camera projection\n    #pragma tangram: position\n\n    cameraProjection(position);\n\n    #ifdef TANGRAM_LAYER_ORDER\n        // +1 is to keep all layers including proxies > 0\n        applyLayerOrder(a_position.w + u_tile_proxy_order_offset + 1., position);\n    #endif\n\n    // Apply pixel offset in screen-space\n    // Multiply by 2 is because screen is 2 units wide Normalized Device Coords (and u_resolution device pixels wide)\n    // Device pixel ratio adjustment is because shape is in logical pixels\n    position.xy += shape * position.w * 2. * u_device_pixel_ratio / u_resolution;\n    #ifdef TANGRAM_HAS_SHADER_POINTS\n        if (u_point_type == TANGRAM_POINT_TYPE_SHADER) { // shader point\n            // enlarge by 1px to catch missed MSAA fragments\n            position.xy += sign(shape) * position.w * u_device_pixel_ratio / u_resolution;\n        }\n    #endif\n\n    // Snap to pixel grid\n    // Only applied to fully upright sprites/labels (not shader-drawn points), while panning is not active\n    #ifdef TANGRAM_HAS_SHADER_POINTS\n    if (!u_view_panning && (abs(theta) < TANGRAM_EPSILON) && u_point_type != TANGRAM_POINT_TYPE_SHADER) {\n    #else\n    if (!u_view_panning && (abs(theta) < TANGRAM_EPSILON)) {\n    #endif\n        vec2 position_fract = fract((((position.xy / position.w) + 1.) * .5) * u_resolution);\n        vec2 position_snap = position.xy + ((step(0.5, position_fract) - position_fract) * position.w * 2. / u_resolution);\n\n        // Animate the snapping to smooth the transition and make it less noticeable\n        #ifdef TANGRAM_VIEW_PAN_SNAP_RATE\n            position.xy = mix(position.xy, position_snap, clamp(u_view_pan_snap_timer * TANGRAM_VIEW_PAN_SNAP_RATE, 0., 1.));\n        #else\n            position.xy = position_snap;\n        #endif\n    }\n\n    gl_Position = position;\n}\n";

var points_fs = "uniform vec2 u_resolution;\nuniform float u_time;\nuniform vec3 u_map_position;\nuniform vec4 u_tile_origin;\nuniform float u_meters_per_pixel;\nuniform float u_device_pixel_ratio;\nuniform float u_visible_time;\n\nuniform mat3 u_normalMatrix;\nuniform mat3 u_inverseNormalMatrix;\n\nuniform sampler2D u_texture;\nuniform float u_point_type;\nuniform bool u_apply_color_blocks;\n\nvarying vec4 v_color;\nvarying vec2 v_texcoord;\nvarying vec4 v_world_position;\nvarying float v_alpha_factor;\n\n#ifdef TANGRAM_HAS_SHADER_POINTS\n    varying vec4 v_outline_color;\n    varying float v_outline_edge;\n    varying float v_aa_offset;\n#endif\n\n#ifdef TANGRAM_SHOW_HIDDEN_LABELS\n    varying float v_label_hidden;\n#endif\n\n#define TANGRAM_NORMAL vec3(0., 0., 1.)\n\n#pragma tangram: camera\n#pragma tangram: material\n#pragma tangram: lighting\n#pragma tangram: raster\n#pragma tangram: global\n\n#ifdef TANGRAM_HAS_SHADER_POINTS\n    //l is the distance from the center to the fragment, R is the radius of the drawn point\n    float _tangram_antialias(float l, float R){\n        float low  = R - v_aa_offset;\n        float high = R + v_aa_offset;\n        return 1. - smoothstep(low, high, l);\n    }\n#endif\n\nvoid main (void) {\n    // Initialize globals\n    #pragma tangram: setup\n\n    vec4 color = v_color;\n\n    #ifdef TANGRAM_HAS_SHADER_POINTS\n        // Only apply shader blocks to point, not to attached text (N.B.: for compatibility with ES)\n        if (u_point_type == TANGRAM_POINT_TYPE_TEXTURE) { // sprite texture\n            color *= texture2D(u_texture, v_texcoord);\n        }\n        else if (u_point_type == TANGRAM_POINT_TYPE_LABEL) { // label texture\n            color = texture2D(u_texture, v_texcoord);\n            color.rgb /= max(color.a, 0.001); // un-multiply canvas texture\n        }\n        else if (u_point_type == TANGRAM_POINT_TYPE_SHADER) { // shader point\n            float outline_edge = v_outline_edge;\n            vec4 outlineColor  = v_outline_color;\n\n            // Mask of outermost circle, either outline or point boundary\n            float l = length(v_texcoord); // distance to this fragment from the point center\n            float outer_alpha = _tangram_antialias(l, 1.);\n            float fill_alpha = _tangram_antialias(l, 1. - (v_outline_edge * 0.5)) * color.a;\n            float stroke_alpha = (outer_alpha - _tangram_antialias(l, 1. - v_outline_edge)) * outlineColor.a;\n\n            // Apply alpha compositing with stroke 'over' fill.\n            #ifdef TANGRAM_BLEND_ADD\n                color.a = stroke_alpha + fill_alpha;\n                color.rgb = color.rgb * fill_alpha + outlineColor.rgb * stroke_alpha;\n            #else // TANGRAM_BLEND_OVERLAY (and fallback for not implemented blending modes)\n                color.a = stroke_alpha + fill_alpha * (1. - stroke_alpha);\n                color.rgb = mix(color.rgb * fill_alpha, outlineColor.rgb, stroke_alpha) / max(color.a, 0.001); // avoid divide by zero\n            #endif\n        }\n    #else\n        // If shader points not supported, assume label texture\n        color = texture2D(u_texture, v_texcoord);\n        color.rgb /= max(color.a, 0.001); // un-multiply canvas texture\n    #endif\n\n    // Shader blocks for color/filter are only applied for sprites, shader points, and standalone text,\n    // NOT for text attached to a point (N.B.: for compatibility with ES)\n    if (u_apply_color_blocks) {\n        #pragma tangram: color\n        #pragma tangram: filter\n    }\n\n    color.a *= v_alpha_factor;\n\n    // highlight hidden label in fragment shader for debugging\n    #ifdef TANGRAM_SHOW_HIDDEN_LABELS\n        if (v_label_hidden > 0.) {\n            color.a *= 0.5;\n            color.rgb = vec3(1., 0., 0.);\n        }\n    #endif\n\n    // Use alpha test as a lower-quality substitute\n    // For opaque and translucent: avoid transparent pixels writing to depth buffer, obscuring geometry underneath\n    // For multiply: avoid transparent pixels multiplying geometry underneath to zero/full black\n    #if defined(TANGRAM_BLEND_OPAQUE) || defined(TANGRAM_BLEND_TRANSLUCENT) || defined(TANGRAM_BLEND_MULTIPLY)\n        if (color.a < TANGRAM_ALPHA_TEST) {\n            discard;\n        }\n    #endif\n\n    gl_FragColor = color;\n}\n";

var PLACEMENT$1 = LabelPoint.PLACEMENT;
var Points = Object.create(Style);
Points.variants = {}; // mesh variants by variant key

Points.vertex_layouts = {}; // vertex layouts by variant key

var SHADER_POINT_VARIANT = '__shader_point'; // texture types

var TANGRAM_POINT_TYPE_TEXTURE = 1; // style texture/sprites (assigned by user)

var TANGRAM_POINT_TYPE_LABEL = 2; // labels (generated by rendering labels to canvas)

var TANGRAM_POINT_TYPE_SHADER = 3; // point (drawn in shader)
// default point size in pixels

var DEFAULT_POINT_SIZE = 16; // Mixin text label methods

Object.assign(Points, TextLabels);
Object.assign(Points, {
  name: 'points',
  built_in: true,
  vertex_shader_src: points_vs,
  fragment_shader_src: points_fs,
  selection: true,
  // enable feature selection
  collision: true,
  // style includes a collision pass
  blend: 'overlay',
  // overlays drawn on top of all other styles, with blending
  init: function init(options) {
    if (options === void 0) {
      options = {};
    }

    Style.init.call(this, options); // Shader defines

    this.setupDefines(); // Include code for SDF-drawn shader points

    this.defines.TANGRAM_HAS_SHADER_POINTS = true; // texture types

    this.defines.TANGRAM_POINT_TYPE_TEXTURE = TANGRAM_POINT_TYPE_TEXTURE;
    this.defines.TANGRAM_POINT_TYPE_LABEL = TANGRAM_POINT_TYPE_LABEL;
    this.defines.TANGRAM_POINT_TYPE_SHADER = TANGRAM_POINT_TYPE_SHADER;
    this.collision_group_points = this.name + '-points';
    this.collision_group_text = this.name + '-text'; // Stenciling proxy tiles (to avoid compounding alpha artifacts) doesn't work well with
    // points/text labels, which have pure transparent pixels that interfere with the stencil buffer,
    // causing a "cut-out"/"x-ray" effect (preventing pixels that would usually be covered by proxy tiles
    // underneath from being rendered).

    this.stencil_proxy_tiles = false;
    this.reset();
  },
  // Setup defines common to points base and child (text) styles
  setupDefines: function setupDefines() {
    // If we're not rendering as overlay, we need a layer attribute
    if (this.blend !== 'overlay') {
      this.defines.TANGRAM_LAYER_ORDER = true;
    } // Fade in labels


    if (debugSettings$1.suppress_label_fade_in === true) {
      this.fade_in_time = 0;
      this.defines.TANGRAM_FADE_IN_RATE = null;
    } else {
      this.fade_in_time = 0.15; // time in seconds

      this.defines.TANGRAM_FADE_IN_RATE = 1 / this.fade_in_time;
    } // Snap points to pixel grid after panning stop


    if (debugSettings$1.suppress_label_snap_animation !== true) {
      this.defines.TANGRAM_VIEW_PAN_SNAP_RATE = 1 / VIEW_PAN_SNAP_TIME; // inverse time in seconds
    } // Show hidden labels for debugging


    if (debugSettings$1.show_hidden_labels === true) {
      this.defines.TANGRAM_SHOW_HIDDEN_LABELS = true;
    }
  },
  reset: function reset() {
    this.queues = {};
    this.resetText();
    this.texture_missing_sprites = {}; // track which missing sprites we've found (reduce dupe log messages)
  },
  // Override to queue features instead of processing immediately
  addFeature: function addFeature(feature, draw, context) {
    var tile = context.tile;

    if (tile.generation !== this.generation) {
      return;
    } // Point styling


    var style = {};
    style.color = this.parseColor(draw.color, context);
    style.texture = draw.texture; // optional point texture, specified in `draw` or at style level

    style.label_texture = null; // assigned by labelling code if needed

    style.blend_order = draw.blend_order; // copy pre-computed blend order
    // require color or texture

    if (!style.color && !style.texture) {
      return;
    } // optional sprite and texture


    var sprite_info;

    if (this.hasSprites(style)) {
      // populate sprite_info object with used sprites
      sprite_info = this.parseSprite(style, draw, context);

      if (sprite_info) {
        style.texcoords = sprite_info.texcoords;
      } else {
        // sprites are defined in the style's texture, but none are used in the current layer
        log({
          level: 'debug',
          once: true
        }, "Layer group '" + draw.layers.join(', ') + "' " + ("uses a texture '" + style.texture + "', but doesn't specify which sprite to draw. ") + 'Features that match this layer group won\'t be drawn without specifying the sprite with the ' + '\'sprite\' or \'sprite_default\' properties. The merged draw parameters for this layer group are:', draw).then(function (logged) {
          if (logged) {
            log('debug', "Example feature for layer group '" + draw.layers.join(', ') + "'", feature);
          }
        });
        return;
      }
    } else if (draw.sprite) {
      // sprite specified in the draw layer but no sprites defined in the texture
      log({
        level: 'warn',
        once: true
      }, "Layer group '" + draw.layers.join(', ') + "' " + ("specifies sprite '" + draw.sprite + "', but the texture '" + draw.texture + "' doesn't define any sprites. ") + 'Features that match this layer group won\'t be drawn. The merged draw parameters for this layer group are:', draw);
      return;
    }

    this.calcSize(draw, style, sprite_info, context); // incorporate outline into size

    if (draw.outline) {
      style.outline_width = StyleParser.evalCachedProperty(draw.outline.width, context) || StyleParser.defaults.outline.width;
      style.outline_color = this.parseColor(draw.outline.color, context);
    }

    style.outline_edge_pct = 0;

    if (style.outline_width && style.outline_color) {
      var outline_width = style.outline_width;
      style.size[0] += outline_width;
      style.size[1] += outline_width;
      style.outline_edge_pct = outline_width / Math.min(style.size[0], style.size[1]) * 2; // UV distance at which outline starts
    } // size will be scaled to 16-bit signed int, so max allowed width + height of 256 pixels


    style.size[0] = Math.min(style.size[0], 256);
    style.size[1] = Math.min(style.size[1], 256); // Placement strategy

    style.placement = draw.placement;
    style.placement_min_length_ratio = StyleParser.evalCachedProperty(draw.placement_min_length_ratio, context); // Spacing parameter (in pixels) to equally space points along a line

    if (style.placement === PLACEMENT$1.SPACED && draw.placement_spacing) {
      style.placement_spacing = StyleParser.evalCachedProperty(draw.placement_spacing, context);
    } // Angle parameter (can be a number or the string "auto")


    style.angle = StyleParser.evalProperty(draw.angle, context) || 0; // points can be placed off the ground

    style.z = StyleParser.evalCachedDistanceProperty(draw.z, context) || StyleParser.defaults.z;
    style.tile_edges = draw.tile_edges; // usually activated for debugging, or rare visualization needs

    this.computeLayout(style, feature, draw, context, tile); // Text styling

    var tf = draw.text && draw.text.visible !== false && // explicitly handle `visible` property for nested text
    this.parseTextFeature(feature, draw.text, context, tile);

    if (Array.isArray(tf)) {
      tf = null; // NB: boundary labels not supported for point label attachments, should log warning

      log({
        level: 'warn',
        once: true
      }, "Layer group '" + draw.layers.join(', ') + "': " + 'cannot use boundary labels (e.g. \'text_source: { left: ..., right: ... }\') for \'text\' labels attached to \'points\'; ' + ("provided 'text_source' value was " + JSON.stringify(draw.text.text_source)));
    }

    if (tf) {
      tf.layout.parent = style; // parent point will apply additional anchor/offset to text
      // Text labels have a default priority of 0.5 below their parent point (+0.5, priority is lower-is-better)
      // This can be overriden, as long as it is less than or equal to the default

      tf.layout.priority = draw.text.priority ? Math.max(tf.layout.priority, style.priority + 0.5) : style.priority + 0.5; // Text labels attached to points should not be moved into tile
      // (they should stay fixed relative to the point)

      tf.layout.move_into_tile = false;
      Collision.addStyle(this.collision_group_text, tile.id);
    }

    this.queueFeature({
      feature: feature,
      draw: draw,
      context: context,
      style: style,
      text_feature: tf
    }, tile); // queue the feature for later processing
    // Register with collision manager

    Collision.addStyle(this.collision_group_points, tile.id);
  },
  // Calcuate the size for the current point feature
  calcSize: function calcSize(draw, style, sprite_info, context) {
    // point size defined explicitly, or defaults to sprite size, or generic fallback
    style.size = draw.size;

    if (!style.size) {
      // a 'size' property has not been set in the draw layer -
      // use the sprite size if it exists and a generic fallback if it doesn't
      style.size = sprite_info && sprite_info.css_size || [DEFAULT_POINT_SIZE, DEFAULT_POINT_SIZE];
    } else {
      // check for a cached size, passing the texture and any sprite references
      style.size = StyleParser.evalCachedPointSizeProperty(draw.size, sprite_info, Texture.textures[style.texture], context);

      if (style.size == null) {
        // the StyleParser couldn't evaluate a sprite size
        log({
          level: 'warn',
          once: true
        }, "Layer group '" + draw.layers.join(', ') + "': " + ("'size' (" + JSON.stringify(draw.size.value) + ") couldn't be interpreted, features that match ") + 'this layer group won\'t be drawn');
        return;
      } else if (typeof style.size === 'number') {
        style.size = [style.size, style.size]; // convert 1d size to 2d
      }
    }
  },
  hasSprites: function hasSprites(style) {
    return style.texture && Texture.textures[style.texture] && Texture.textures[style.texture].sprites;
  },
  // Generate a sprite_info object
  getSpriteInfo: function getSpriteInfo(style, sprite) {
    var info = Texture.textures[style.texture].sprites[sprite] && Texture.getSpriteInfo(style.texture, sprite);

    if (sprite && !info) {
      // track missing sprites (per texture)
      this.texture_missing_sprites[style.texture] = this.texture_missing_sprites[style.texture] || {};

      if (!this.texture_missing_sprites[style.texture][sprite]) {
        // only log each missing sprite once
        log('debug', "Style: in style '" + this.name + "', could not find sprite '" + sprite + "' for texture '" + style.texture + "'");
        this.texture_missing_sprites[style.texture][sprite] = true;
      }
    } else if (info) {
      info.sprite = sprite;
    }

    return info;
  },
  // Check a sprite name against available sprites and return a sprite_info object
  parseSprite: function parseSprite(style, draw, context) {
    // check for functions
    var sprite = StyleParser.evalProperty(draw.sprite, context);
    var sprite_info = this.getSpriteInfo(style, sprite) || this.getSpriteInfo(style, draw.sprite_default);
    return sprite_info;
  },
  // Queue features for deferred processing (collect all features first so we can do collision on the whole group)
  queueFeature: function queueFeature(q, tile) {
    if (!this.tile_data[tile.id] || !this.queues[tile.id]) {
      this.startData(tile);
    }

    this.queues[tile.id] = this.queues[tile.id] || [];
    this.queues[tile.id].push(q);
  },
  // Override
  endData: function endData(tile) {
    return new Promise(function ($return, $error) {
      var _this, queue, text_objs, point_objs, _ref, _ref$, labels, texts, textures, tile_data, _tile_data$textures;

      _this = this;

      if (tile.canceled) {
        log('trace', "Style " + this.name + ": stop tile build because tile was canceled: " + tile.key);
        return $return(null);
      }

      queue = this.queues[tile.id];
      delete this.queues[tile.id]; // For each point feature, create one or more labels

      text_objs = [];
      point_objs = [];
      queue.forEach(function (q) {
        var style = q.style;
        var feature = q.feature;
        var geometry = feature.geometry;

        var feature_labels = _this.buildLabels(style.size, geometry, style);

        for (var i = 0; i < feature_labels.length; i++) {
          var label = feature_labels[i];
          var point_obj = {
            feature: feature,
            draw: q.draw,
            context: q.context,
            style: style,
            label: label
          };
          point_objs.push(point_obj);

          if (q.text_feature) {
            var text_obj = {
              feature: feature,
              draw: q.text_feature.draw,
              context: q.context,
              text: q.text_feature.text,
              text_settings_key: q.text_feature.text_settings_key,
              layout: q.text_feature.layout,
              point_label: label,
              linked: point_obj // link so text only renders when parent point is placed

            };
            text_objs.push(text_obj); // Unless text feature is optional, create a two-way link so that parent
            // point will only render when text is also placed

            if (!q.draw.text.optional) {
              point_obj.linked = text_obj; // two-way link
            }
          }
        }
      }); // Collide both points and text, then build features

      return Promise.resolve(Promise.all([// Points
      Collision.collide(point_objs, this.collision_group_points, tile.id).then(function (point_objs) {
        point_objs.forEach(function (q) {
          _this.feature_style = q.style;
          _this.feature_style.label = q.label;
          _this.feature_style.linked = q.linked; // TODO: move linked into label to avoid extra prop tracking?

          Style.addFeature.call(_this, q.feature, q.draw, q.context);
        });
      }), // Labels
      this.collideAndRenderTextLabels(tile, this.collision_group_text, text_objs)])).then(function ($await_2) {
        try {
          _ref = $await_2, _ref$ = _ref[1], labels = _ref$.labels, texts = _ref$.texts, textures = _ref$.textures;

          // Process labels
          if (labels && texts) {
            // Build queued features
            labels.forEach(function (q) {
              var text_settings_key = q.text_settings_key;
              var text_info = texts[text_settings_key] && texts[text_settings_key][q.text]; // setup styling object expected by Style class

              var style = _this.feature_style;
              style.label = q.label;
              style.linked = q.linked; // TODO: move linked into label to avoid extra prop tracking?

              style.size = text_info.size.logical_size;
              style.texcoords = text_info.align[q.label.align].texcoords;
              style.label_texture = textures[text_info.align[q.label.align].texture_id];
              style.blend_order = q.draw.blend_order; // copy blend order from parent point

              Style.addFeature.call(_this, q.feature, q.draw, q.context);
            });
          }

          this.freeText(tile); // Finish tile mesh

          return Promise.resolve(Style.endData.call(this, tile)).then(function ($await_3) {
            try {
              tile_data = $await_3;

              // Attach tile-specific label atlas to mesh as a texture uniform
              if (tile_data && textures && textures.length) {
                tile_data.textures = tile_data.textures || [];

                (_tile_data$textures = tile_data.textures).push.apply(_tile_data$textures, textures); // assign texture ownership to tile

              }

              return $return(tile_data);
            } catch ($boundEx) {
              return $error($boundEx);
            }
          }, $error);
        } catch ($boundEx) {
          return $error($boundEx);
        }
      }.bind(this), $error);
    }.bind(this));
  },
  _preprocess: function _preprocess(draw) {
    draw.color = StyleParser.createColorPropertyCache(draw.color);
    draw.texture = draw.texture !== undefined ? draw.texture : this.texture; // optional or default texture

    draw.blend_order = this.getBlendOrderForDraw(draw); // from draw block, or fall back on default style blend order

    if (draw.outline) {
      draw.outline.color = StyleParser.createColorPropertyCache(draw.outline.color);
      draw.outline.width = StyleParser.createPropertyCache(draw.outline.width, StyleParser.parsePositiveNumber);
    }

    draw.z = StyleParser.createPropertyCache(draw.z, StyleParser.parseUnits); // Size (1d value or 2d array)

    try {
      draw.size = StyleParser.createPointSizePropertyCache(draw.size, draw.texture);
    } catch (e) {
      log({
        level: 'warn',
        once: true
      }, "Layer group '" + draw.layers.join(', ') + "': " + (e + " (" + JSON.stringify(draw.size) + "), features that match this layer group won't be drawn."));
      return null;
    } // Offset (2d array)


    draw.offset = StyleParser.createPropertyCache(draw.offset, function (v) {
      return Array.isArray(v) && v.map(StyleParser.parseNumber);
    }); // Buffer (1d value or or 2d array) - must be >= 0

    draw.buffer = StyleParser.createPropertyCache(draw.buffer, function (v) {
      return (Array.isArray(v) ? v : [v, v]).map(StyleParser.parsePositiveNumber);
    }); // Repeat rules - no repeat limitation for points by default

    draw.repeat_distance = StyleParser.createPropertyCache(draw.repeat_distance, StyleParser.parseNumber); // Placement strategies

    draw.placement = PLACEMENT$1[draw.placement && draw.placement.toUpperCase()];

    if (draw.placement == null) {
      draw.placement = PLACEMENT$1.VERTEX;
    }

    draw.placement_spacing = draw.placement_spacing != null ? draw.placement_spacing : 80; // default spacing

    draw.placement_spacing = StyleParser.createPropertyCache(draw.placement_spacing, StyleParser.parsePositiveNumber);
    draw.placement_min_length_ratio = draw.placement_min_length_ratio != null ? draw.placement_min_length_ratio : 1;
    draw.placement_min_length_ratio = StyleParser.createPropertyCache(draw.placement_min_length_ratio, StyleParser.parsePositiveNumber);

    if (typeof draw.angle === 'number') {
      draw.angle = draw.angle * Math.PI / 180; // convert static value to radians
    } else if (typeof draw.angle === 'function') {
      // convert function return value to radians
      var angle_func = draw.angle;

      draw.angle = function (context) {
        return angle_func(context) * Math.PI / 180;
      };
    } else {
      draw.angle = draw.angle || 0; // angle can be a string like "auto" (use angle of geometry)
    } // Optional text styling


    draw.text = this.preprocessText(draw.text); // will return null if valid text styling wasn't provided

    if (draw.text) {
      draw.text.key = draw.key; // inherits parent properties

      draw.text.group = draw.group;
      draw.text.layers = draw.layers;
      draw.text.order = draw.order;
      draw.text.blend_order = draw.blend_order;
      draw.text.repeat_group = draw.text.repeat_group != null ? draw.text.repeat_group : draw.repeat_group;
      draw.text.anchor = draw.text.anchor || this.default_anchor;
      draw.text.optional = typeof draw.text.optional === 'boolean' ? draw.text.optional : false; // default text to required

      draw.text.interactive = draw.text.interactive || draw.interactive; // inherits from point
    }

    return draw;
  },
  // Default to trying all anchor placements
  default_anchor: ['bottom', 'top', 'right', 'left'],
  // Compute label layout-related properties
  computeLayout: function computeLayout(target, feature, draw, context, tile) {
    var layout = target || {};
    layout.id = feature;
    layout.units_per_pixel = tile.units_per_pixel || 1; // collision flag

    layout.collide = draw.collide === false ? false : true; // label anchors (point labels only)
    // label position will be adjusted in the given direction, relative to its original point
    // one of: left, right, top, bottom, top-left, top-right, bottom-left, bottom-right

    layout.anchor = draw.anchor; // label offset and buffer in pixel (applied in screen space)

    layout.offset = StyleParser.evalCachedProperty(draw.offset, context) || StyleParser.zeroPair;
    layout.buffer = StyleParser.evalCachedProperty(draw.buffer, context) || StyleParser.zeroPair; // repeat rules

    layout.repeat_distance = StyleParser.evalCachedProperty(draw.repeat_distance, context);

    if (layout.repeat_distance) {
      layout.repeat_distance *= layout.units_per_pixel;
      layout.repeat_scale = 1; // initial repeat pass in tile with full scale

      if (typeof draw.repeat_group === 'function') {
        layout.repeat_group = draw.repeat_group(context); // dynamic repeat group
      } else {
        // default to top-level layer name
        // (e.g. all labels under `roads` layer, including sub-layers, are in one repeat group)
        layout.repeat_group = draw.repeat_group || context.layer;
      }
    } // label priority (lower is higher)


    var priority = draw.priority;

    if (priority != null) {
      if (typeof priority === 'function') {
        priority = priority(context);
      }
    } else {
      priority = -1 >>> 0; // default to max priority value if none set
    }

    layout.priority = priority;
    return layout;
  },
  // Implements label building for TextLabels mixin
  buildTextLabels: function buildTextLabels(tile, feature_queue) {
    var labels = [];

    for (var f = 0; f < feature_queue.length; f++) {
      var fq = feature_queue[f];
      var text_info = this.texts[tile.id][fq.text_settings_key][fq.text];
      var size = text_info.size.collision_size;
      fq.label = new LabelPoint(fq.point_label.position, size, fq.layout);
      labels.push(fq);
    }

    return labels;
  },
  // Builds one or more point labels for a geometry
  buildLabels: function buildLabels(size, geometry, layout) {
    var labels = [];

    if (geometry.type === 'Point') {
      labels.push(new LabelPoint(geometry.coordinates, size, layout, layout.angle));
    } else if (geometry.type === 'MultiPoint') {
      var points = geometry.coordinates;

      for (var i = 0; i < points.length; ++i) {
        var point = points[i];
        labels.push(new LabelPoint(point, size, layout, layout.angle));
      }
    } else if (geometry.type === 'LineString') {
      var line = geometry.coordinates;
      var point_labels = placePointsOnLine(line, size, layout);

      for (var _i = 0; _i < point_labels.length; ++_i) {
        labels.push(point_labels[_i]);
      }
    } else if (geometry.type === 'MultiLineString') {
      var lines = geometry.coordinates;

      for (var ln = 0; ln < lines.length; ln++) {
        var _line = lines[ln];

        var _point_labels = placePointsOnLine(_line, size, layout);

        for (var _i2 = 0; _i2 < _point_labels.length; ++_i2) {
          labels.push(_point_labels[_i2]);
        }
      }
    } else if (geometry.type === 'Polygon') {
      // Point at polygon centroid (of outer ring)
      if (layout.placement === PLACEMENT$1.CENTROID) {
        var centroid = Geo$1.centroid(geometry.coordinates);

        if (centroid) {
          // skip degenerate polygons
          labels.push(new LabelPoint(centroid, size, layout, layout.angle));
        }
      } // Point at each polygon vertex (all rings)
      else {
          var rings = geometry.coordinates;

          for (var _ln = 0; _ln < rings.length; _ln++) {
            var _point_labels2 = placePointsOnLine(rings[_ln], size, layout);

            for (var _i3 = 0; _i3 < _point_labels2.length; ++_i3) {
              labels.push(_point_labels2[_i3]);
            }
          }
        }
    } else if (geometry.type === 'MultiPolygon') {
      if (layout.placement === PLACEMENT$1.CENTROID) {
        var _centroid = Geo$1.multiCentroid(geometry.coordinates);

        if (_centroid) {
          // skip degenerate polygons
          labels.push(new LabelPoint(_centroid, size, layout, layout.angle));
        }
      } else {
        var polys = geometry.coordinates;

        for (var p = 0; p < polys.length; p++) {
          var _rings = polys[p];

          for (var _ln2 = 0; _ln2 < _rings.length; _ln2++) {
            var _point_labels3 = placePointsOnLine(_rings[_ln2], size, layout);

            for (var _i4 = 0; _i4 < _point_labels3.length; ++_i4) {
              labels.push(_point_labels3[_i4]);
            }
          }
        }
      }
    }

    return labels;
  },

  /**
   * A "template" that sets constant attibutes for each vertex, which is then modified per vertex or per feature.
   * A plain JS array matching the order of the vertex layout.
   */
  makeVertexTemplate: function makeVertexTemplate(style, mesh) {
    var i = 0; // a_position.xyz - vertex position
    // a_position.w - layer order

    this.vertex_template[i++] = 0;
    this.vertex_template[i++] = 0;
    this.vertex_template[i++] = style.z || 0;
    this.vertex_template[i++] = this.scaleOrder(style.order); // a_shape.xy - size of point in pixels (scaling vector)
    // a_shape.z - angle of point
    // a_shape.w - show/hide flag

    this.vertex_template[i++] = 0;
    this.vertex_template[i++] = 0;
    this.vertex_template[i++] = 0;
    this.vertex_template[i++] = style.label.layout.collide ? 0 : 1; // set initial label hide/show state
    // a_texcoord.xy - texture coords

    if (!mesh.variant.shader_point) {
      this.vertex_template[i++] = 0;
      this.vertex_template[i++] = 0;
    } // a_offset.xy - offset of point from center, in pixels


    this.vertex_template[i++] = 0;
    this.vertex_template[i++] = 0; // a_color.rgba - feature color

    var color = style.color || StyleParser.defaults.color;
    this.vertex_template[i++] = color[0] * 255;
    this.vertex_template[i++] = color[1] * 255;
    this.vertex_template[i++] = color[2] * 255;
    this.vertex_template[i++] = color[3] * 255; // a_selection_color.rgba - selection color

    if (mesh.variant.selection) {
      this.vertex_template[i++] = style.selection_color[0] * 255;
      this.vertex_template[i++] = style.selection_color[1] * 255;
      this.vertex_template[i++] = style.selection_color[2] * 255;
      this.vertex_template[i++] = style.selection_color[3] * 255;
    } // point outline


    if (mesh.variant.shader_point) {
      // a_outline_color.rgba - outline color
      var outline_color = style.outline_color || StyleParser.defaults.outline.color;
      this.vertex_template[i++] = outline_color[0] * 255;
      this.vertex_template[i++] = outline_color[1] * 255;
      this.vertex_template[i++] = outline_color[2] * 255;
      this.vertex_template[i++] = outline_color[3] * 255; // a_outline_edge - point outline edge (as % of point size where outline begins)

      this.vertex_template[i++] = style.outline_edge_pct || StyleParser.defaults.outline.width;
    }

    return this.vertex_template;
  },
  buildQuad: function buildQuad(point, size, angle, angles, pre_angles, offset, offsets, texcoords, curve, vertex_data, vertex_template) {
    if (size[0] <= 0 || size[1] <= 0) {
      return 0; // size must be positive
    }

    return buildQuadForPoint(point, vertex_data, vertex_template, vertex_data.vertex_layout.index, size, offset, offsets, pre_angles, angle * 4096, // angle values have a 12-bit fraction
    angles, texcoords, curve);
  },
  // Build quad for point sprite
  build: function build(style, context) {
    var label = style.label;

    if (label.type === 'curved') {
      return this.buildCurvedLabel(label, style, context);
    } else {
      return this.buildStraightLabel(label, style, context);
    }
  },
  buildStraightLabel: function buildStraightLabel(label, style, context) {
    var mesh = this.getTileMesh(context.tile, this.meshVariantTypeForDraw(style));
    var vertex_template = this.makeVertexTemplate(style, mesh);
    var size, texcoords;

    if (label.type !== 'point') {
      size = style.size[label.type];
      texcoords = style.texcoords[label.type].texcoord;
    } else {
      size = style.size;
      texcoords = style.texcoords;
    } // setup style or label texture if applicable


    mesh.uniforms = mesh.uniforms || {};

    if (style.label_texture) {
      mesh.uniforms.u_texture = style.label_texture;
      mesh.uniforms.u_point_type = TANGRAM_POINT_TYPE_LABEL;
      mesh.uniforms.u_apply_color_blocks = false;
    } else if (style.texture) {
      mesh.uniforms.u_texture = style.texture;
      mesh.uniforms.u_point_type = TANGRAM_POINT_TYPE_TEXTURE;
      mesh.uniforms.u_apply_color_blocks = true;
    } else {
      mesh.uniforms.u_texture = Texture.default; // ensure a tetxure is always bound to avoid GL warnings ('no texture bound to unit' in Chrome)

      mesh.uniforms.u_point_type = TANGRAM_POINT_TYPE_SHADER;
      mesh.uniforms.u_apply_color_blocks = true;
    }

    var offset = label.offset; // TODO: instead of passing null, pass arrays with fingerprintable values
    // This value is checked in the shader to determine whether to apply curving logic

    var geom_count = this.buildQuad(label.position, // position
    size, // size in pixels
    label.angle, // angle in radians
    null, // placeholder for multiple angles
    null, // placeholder for multiple pre_angles
    offset, // offset from center in pixels
    null, // placeholder for multiple offsets
    texcoords, // texture UVs
    false, // if curved boolean
    mesh.vertex_data, vertex_template // VBO and data for current vertex
    ); // track label mesh buffer data

    var linked = style.linked && style.linked.label.id;
    this.trackLabel(label, linked, mesh, geom_count, context);
    return geom_count;
  },
  buildCurvedLabel: function buildCurvedLabel(label, style, context) {
    var mesh, vertex_template;
    var geom_count = 0; // two passes for stroke and fill, where stroke needs to be drawn first (painter's algorithm)
    // this ensures strokes don't overlap on other fills
    // pass for stroke

    for (var i = 0; i < label.num_segments; i++) {
      var size = style.size[label.type][i];
      var texcoord_stroke = style.texcoords_stroke[i]; // re-point to correct label texture

      style.label_texture = style.label_textures[i];
      mesh = this.getTileMesh(context.tile, this.meshVariantTypeForDraw(style));
      vertex_template = this.makeVertexTemplate(style, mesh); // add label texture uniform if needed

      mesh.uniforms = mesh.uniforms || {};
      mesh.uniforms.u_texture = style.label_texture;
      mesh.uniforms.u_point_type = TANGRAM_POINT_TYPE_LABEL;
      mesh.uniforms.u_apply_color_blocks = false;
      var offset = label.offset || [0, 0];
      var position = label.position;
      var angles = label.angles[i];
      var offsets = label.offsets[i];
      var pre_angles = label.pre_angles[i];
      var seg_count = this.buildQuad(position, // position
      size, // size in pixels
      label.angle, // angle in degrees
      angles, // angles per segment
      pre_angles, // pre_angle array (rotation applied before offseting)
      offset, // offset from center in pixels
      offsets, // offsets per segment
      texcoord_stroke, // texture UVs for stroked text
      true, // if curved
      mesh.vertex_data, vertex_template // VBO and data for current vertex
      );
      geom_count += seg_count; // track label mesh buffer data

      var linked = style.linked && style.linked.label.id;
      this.trackLabel(label, linked, mesh, seg_count, context);
    } // pass for fill


    for (var _i5 = 0; _i5 < label.num_segments; _i5++) {
      var _size = style.size[label.type][_i5];
      var texcoord = style.texcoords[label.type][_i5].texcoord; // re-point to correct label texture

      style.label_texture = style.label_textures[_i5];
      mesh = this.getTileMesh(context.tile, this.meshVariantTypeForDraw(style));
      vertex_template = this.makeVertexTemplate(style, mesh); // add label texture uniform if needed

      mesh.uniforms = mesh.uniforms || {};
      mesh.uniforms.u_texture = style.label_texture;
      mesh.uniforms.u_point_type = TANGRAM_POINT_TYPE_LABEL;
      mesh.uniforms.u_apply_color_blocks = false;

      var _offset = label.offset || [0, 0];

      var _position = label.position;
      var _angles = label.angles[_i5];
      var _offsets = label.offsets[_i5];
      var _pre_angles = label.pre_angles[_i5];

      var _seg_count = this.buildQuad(_position, // position
      _size, // size in pixels
      label.angle, // angle in degrees
      _angles, // angles per segment
      _pre_angles, // pre_angle array (rotation applied before offseting)
      _offset, // offset from center in pixels
      _offsets, // offsets per segment
      texcoord, // texture UVs for fill text
      true, // if curved
      mesh.vertex_data, vertex_template // VBO and data for current vertex
      );

      geom_count += _seg_count; // track label mesh buffer data

      var _linked = style.linked && style.linked.label.id;

      this.trackLabel(label, _linked, mesh, _seg_count, context);
    }

    return geom_count;
  },
  // track mesh data for label on main thread, for additional cross-tile collision/repeat passes
  trackLabel: function trackLabel(label, linked, mesh, geom_count
  /*, context*/
  ) {
    // track if collision is enabled, or if the label is near enough to the tile edge to
    // necessitate further repeat checking
    if (label.layout.collide || label.may_repeat_across_tiles) {
      mesh.labels = mesh.labels || {};
      mesh.labels[label.id] = mesh.labels[label.id] || {
        container: {
          label: label.toJSON(),
          linked: linked
        },
        ranges: [] // debug: { // uncomment and pass in context for debugging
        //     id: context.feature.properties.id,
        //     name: context.feature.properties.name,
        //     props: JSON.stringify(context.feature.properties),
        //     point_type: mesh.uniforms.u_point_type
        // }

      }; // store byte ranges occupied by label in VBO, so they can be updated on main thread

      var vertex_count = geom_count * 2; // geom count is triangles: 2 triangles = 1 quad = 4 vertices

      var start = mesh.vertex_data.offset - mesh.vertex_data.stride * vertex_count; // start offset of byte range

      mesh.labels[label.id].ranges.push([start, vertex_count]);
    }
  },
  // Override to pass-through to generic point builder
  buildLines: function buildLines(lines, style, context) {
    return this.build(style, context);
  },
  buildPoints: function buildPoints(points, style, context) {
    return this.build(style, context);
  },
  buildPolygons: function buildPolygons(points, style, context) {
    return this.build(style, context);
  },
  // Override
  // Create or return desired vertex layout permutation based on flags
  vertexLayoutForMeshVariant: function vertexLayoutForMeshVariant(variant) {
    // Vertex layout only depends on shader point flag, so using it as layout key to avoid duplicate layouts
    if (Points.vertex_layouts[variant.shader_point] == null) {
      // Attributes for this mesh variant
      // Optional attributes have placeholder values assigned with `static` parameter
      // TODO: could support optional attributes for selection and offset, but may not be worth it
      // since points generally don't consume much memory anyway
      var attribs = [{
        name: 'a_position',
        size: 4,
        type: gl$1.SHORT,
        normalized: false
      }, {
        name: 'a_shape',
        size: 4,
        type: gl$1.SHORT,
        normalized: false
      }, {
        name: 'a_texcoord',
        size: 2,
        type: gl$1.UNSIGNED_SHORT,
        normalized: true,
        static: variant.shader_point ? [0, 0] : null
      }, {
        name: 'a_offset',
        size: 2,
        type: gl$1.SHORT,
        normalized: false
      }, {
        name: 'a_color',
        size: 4,
        type: gl$1.UNSIGNED_BYTE,
        normalized: true
      }, {
        name: 'a_selection_color',
        size: 4,
        type: gl$1.UNSIGNED_BYTE,
        normalized: true,
        static: variant.selection ? null : [0, 0, 0, 0]
      }, {
        name: 'a_outline_color',
        size: 4,
        type: gl$1.UNSIGNED_BYTE,
        normalized: true,
        static: variant.shader_point ? null : [0, 0, 0, 0]
      }, {
        name: 'a_outline_edge',
        size: 1,
        type: gl$1.FLOAT,
        normalized: false,
        static: variant.shader_point ? null : 0
      }];
      Points.vertex_layouts[variant.shader_point] = new VertexLayout(attribs);
    }

    return Points.vertex_layouts[variant.shader_point];
  },
  // Override
  meshVariantTypeForDraw: function meshVariantTypeForDraw(draw) {
    var texture = draw.label_texture || draw.texture || SHADER_POINT_VARIANT; // unique key by texture name

    var key = texture + '/' + draw.blend_order;

    if (Points.variants[key] == null) {
      Points.variants[key] = {
        key: key,
        selection: 1,
        // TODO: make this vary by draw params
        shader_point: texture === SHADER_POINT_VARIANT,
        // is shader point
        blend_order: draw.blend_order,
        mesh_order: draw.label_texture ? 1 : 0 // put text on top of points (e.g. for highway shields, etc.)

      };
    }

    return Points.variants[key]; // return pre-calculated mesh variant
  },
  // Override
  makeMesh: function makeMesh(vertex_data, vertex_elements, options) {
    if (options === void 0) {
      options = {};
    }

    // Add label fade time
    options = Object.assign({}, options, {
      fade_in_time: this.fade_in_time
    });
    return Style.makeMesh.call(this, vertex_data, vertex_elements, options);
  }
});

// 20.2.2.17 Math.hypot([value1[, value2[, … ]]])

var abs = Math.abs;

_export(_export.S, 'Math', {
  hypot: function hypot(value1, value2) { // eslint-disable-line no-unused-vars
    var sum = 0;
    var i = 0;
    var aLen = arguments.length;
    var larg = 0;
    var arg, div;
    while (i < aLen) {
      arg = abs(arguments[i++]);
      if (larg < arg) {
        div = larg / arg;
        sum = sum * div * div + 1;
        larg = arg;
      } else if (arg > 0) {
        div = arg / larg;
        sum += div * div;
      } else sum += arg;
    }
    return larg === Infinity ? Infinity : larg * Math.sqrt(sum);
  }
});

var STOPS = [0, 0.33, 0.66, 0.99]; // zoom levels for curved label snapshot data (offsets and angles)

var LINE_EXCEED_STRAIGHT = 1.5; // minimal ratio for straight labels (label length) / (line length)

var LINE_EXCEED_STRAIGHT_NO_CURVE = 1.8; // minimal ratio for straight labels that have no curved option (like Arabic)

var LINE_EXCEED_STAIGHT_LOOSE = 2.3; // 2nd pass minimal ratio for straight labels

var STRAIGHT_ANGLE_TOLERANCE = 0.1; // multiple "almost straight" segments within this angle tolerance can be considered one straight segment (in radians)

var CURVE_MIN_TOTAL_COST = 1.3; // curved line total curvature tolerance (sum in radians)

var CURVE_MIN_AVG_COST = 0.4; // curved line average curvature tolerance (mean)

var CURVE_MAX_ANGLE = 1; // curved line singular curvature tolerance (value in radians)

var ORIENTED_LABEL_OFFSET_FACTOR = 1.2; // multiply offset by this amount to avoid linked label collision

var VERTICAL_ANGLE_TOLERANCE = 0.01; // nearly vertical lines considered vertical within this angle tolerance

var LabelLine = {
  // Given a label's bounding box size and size of broken up individual segments
  // return a label that fits along the line geometry that is either straight (preferred) or curved (if straight tolerances aren't met)
  create: function create(segment_sizes, total_size, line, layout) {
    // The passes done for fitting a label, and provided tolerances for each pass
    // First straight is chosen with a low tolerance. Then curved. Then straight with a higher tolerance.
    var passes = [{
      type: 'straight',
      tolerance: layout.no_curving ? LINE_EXCEED_STRAIGHT_NO_CURVE : LINE_EXCEED_STRAIGHT
    }, {
      type: 'curved'
    }, {
      type: 'straight',
      tolerance: LINE_EXCEED_STAIGHT_LOOSE
    }]; // loop through passes. first label found wins.

    for (var i = 0; i < passes.length; i++) {
      var check = passes[i];
      var label = void 0;

      if (check.type === 'straight') {
        label = new LabelLineStraight(total_size, line, layout, check.tolerance);
      } else if (check.type === 'curved' && !layout.no_curving && line.length > 2) {
        label = new LabelLineCurved(segment_sizes, line, layout);
      }

      if (label && !label.throw_away) {
        return label;
      }
    }

    return false;
  }
};

var LabelLineBase =
/*#__PURE__*/
function () {
  function LabelLineBase(layout) {
    this.id = Label.nextLabelId();
    this.layout = layout;
    this.position = [];
    this.angle = 0;
    this.offset = layout.offset.slice();
    this.unit_scale = this.layout.units_per_pixel;
    this.obbs = [];
    this.aabbs = [];
    this.type = ''; // "curved" or "straight" to be set by child class

    this.throw_away = false; // boolean that determines if label should be discarded
  } // Minimal representation of label


  var _proto = LabelLineBase.prototype;

  _proto.toJSON = function toJSON() {
    return {
      id: this.id,
      type: this.type,
      position: this.position,
      size: this.size,
      offset: this.offset,
      angle: this.angle,
      breach: this.breach,
      may_repeat_across_tiles: this.may_repeat_across_tiles,
      layout: textLayoutToJSON(this.layout)
    };
  } // Given a line, find the longest series of segments that maintains a constant orientation in the x-direction.
  // This assures us that the line has no orientation flip, so text would not appear upside-down.
  // If the line's orientation is reversed, the flip return value will be true, otherwise false
  ;

  LabelLineBase.splitLineByOrientation = function splitLineByOrientation(line) {
    var current_line = [line[0]];
    var current_length = 0;
    var max_length = 0;
    var orientation = 0;
    var longest_line = current_line;
    var flip = false;

    for (var i = 1; i < line.length; i++) {
      var pt = line[i];
      var prev_pt = line[i - 1];
      var length = Vector$1.length(Vector$1.sub(pt, prev_pt));

      if (pt[0] > prev_pt[0]) {
        // positive orientation
        if (orientation === 1) {
          current_line.push(pt);
          current_length += length;

          if (current_length > max_length) {
            longest_line = current_line;
            max_length = current_length;
            flip = false;
          }
        } else {
          current_line = [prev_pt, pt];
          current_length = length;

          if (current_length > max_length) {
            longest_line = current_line;
            max_length = current_length;
            flip = false;
          }

          orientation = 1;
        }
      } else if (pt[0] < prev_pt[0]) {
        // negative orientation
        if (orientation === -1) {
          current_line.unshift(pt);
          current_length += length;

          if (current_length > max_length) {
            longest_line = current_line;
            max_length = current_length;
            flip = true;
          }
        } else {
          // prepend points (reverse order)
          current_line = [pt, prev_pt];
          current_length = length;

          if (current_length > max_length) {
            longest_line = current_line;
            max_length = current_length;
            flip = true;
          }

          orientation = -1;
        }
      } else {
        // vertical line (doesn't change previous orientation)
        if (orientation === -1) {
          current_line.unshift(pt);
        } else {
          current_line.push(pt);
          orientation = 1;
        }

        current_length += length;

        if (current_length > max_length) {
          longest_line = current_line;
          max_length = current_length;
          flip = orientation === -1;
        }
      }
    }

    return [longest_line, flip];
  } // Checks each segment to see if it should be discarded (via collision). If any segment fails this test, they all fail.
  ;

  _proto.discard = function discard(bboxes, exclude) {
    if (exclude === void 0) {
      exclude = null;
    }

    if (this.throw_away) {
      return true;
    }

    for (var i = 0; i < this.obbs.length; i++) {
      var aabb = this.aabbs[i];
      var obb = this.obbs[i];
      var obj = {
        aabb: aabb,
        obb: obb
      };
      var shouldDiscard = Label.prototype.occluded.call(obj, bboxes, exclude);

      if (shouldDiscard) {
        return true;
      }
    }

    return false;
  } // Checks each segment to see if it is within the tile. If any segment fails this test, they all fail.
  ;

  _proto.inTileBounds = function inTileBounds() {
    for (var i = 0; i < this.aabbs.length; i++) {
      var aabb = this.aabbs[i];
      var obj = {
        aabb: aabb
      };
      var in_bounds = Label.prototype.inTileBounds.call(obj);

      if (!in_bounds) {
        return false;
      }
    }

    return true;
  } // Method to calculate oriented bounding box
  // "angle" is the angle of the text segment, "angle_offset" is the angle applied to the offset.
  // Offset angle is constant for the entire label, while segment angles are not.
  ;

  LabelLineBase.createOBB = function createOBB(position, width, height, angle, angle_offset, offset, upp) {
    var p0 = position[0];
    var p1 = position[1]; // apply offset, x positive, y pointing down

    if (offset && (offset[0] !== 0 || offset[1] !== 0)) {
      offset = Vector$1.rot(offset, angle_offset);
      p0 += offset[0] * upp;
      p1 -= offset[1] * upp;
    } // the angle of the obb is negative since it's the tile system y axis is pointing down


    return new OBB(p0, p1, -angle, width, height);
  };

  return LabelLineBase;
}(); // Class for straight labels.
// Extends base LabelLine class.

var LabelLineStraight =
/*#__PURE__*/
function (_LabelLineBase) {
  _inheritsLoose(LabelLineStraight, _LabelLineBase);

  function LabelLineStraight(size, line, layout, tolerance) {
    var _this;

    _this = _LabelLineBase.call(this, layout) || this;
    _this.type = 'straight';
    _this.size = size;
    _this.throw_away = !_this.fit(size, line, layout, tolerance);
    return _this;
  } // Determine if the label can fit the geometry within provided tolerance
  // A straight label is generally placed at segment midpoints, but can "look ahead" to further segments
  // if they are within an angle bound given by STRAIGHT_ANGLE_TOLERANCE and place at the midpoint between non-consecutive segments


  var _proto2 = LabelLineStraight.prototype;

  _proto2.fit = function fit(size, line, layout, tolerance) {
    var upp = this.unit_scale;
    var flipped; // boolean indicating if orientation of line is changed
    // Make new copy of line, with consistent orientation

    var _LabelLineBase$splitL = LabelLineBase.splitLineByOrientation(line);

    line = _LabelLineBase$splitL[0];
    flipped = _LabelLineBase$splitL[1];

    // matches for "left" or "right" labels where the offset angle is dependent on the geometry
    if (typeof layout.orientation === 'number') {
      this.offset[1] += ORIENTED_LABEL_OFFSET_FACTOR * (size[1] - layout.vertical_buffer); // if line is flipped, or the orientation is "left" (-1), flip the offset's y-axis

      if (flipped) {
        this.offset[1] *= -1;
      }

      if (layout.orientation === -1) {
        this.offset[1] *= -1;
      }
    }

    var line_lengths = getLineLengths(line);
    var label_length = size[0] * upp; // loop through line looking for a placement for the label

    for (var i = 0; i < line.length - 1; i++) {
      var curr = line[i];
      var curve_tolerance = 0;
      var length = 0;
      var ahead_index = i + 1;
      var prev_angle = void 0; // look ahead to further line segments within an angle tolerance

      while (ahead_index < line.length) {
        var ahead_curr = line[ahead_index - 1];
        var ahead_next = line[ahead_index];
        var next_angle = getAngleForSegment(ahead_curr, ahead_next);

        if (ahead_index !== i + 1) {
          curve_tolerance += getAbsAngleDiff(next_angle, prev_angle);
        } // if curve tolerance is exceeded, break out of loop


        if (Math.abs(curve_tolerance) > STRAIGHT_ANGLE_TOLERANCE) {
          break;
        }

        length += line_lengths[ahead_index - 1]; // check if label fits geometry

        if (calcFitness(length, label_length) < tolerance) {
          var curr_midpt = Vector$1.mult(Vector$1.add(curr, ahead_next), 0.5); // TODO: modify angle if line chosen within curve_angle_tolerance
          // Currently line angle is the same as the starting angle, perhaps it should average across segments?

          this.angle = -next_angle; // ensure that all vertical labels point up (not down) by snapping angles close to pi/2 to -pi/2

          if (Math.abs(this.angle - Math.PI / 2) < VERTICAL_ANGLE_TOLERANCE) {
            // flip angle and offset
            this.angle = -Math.PI / 2;

            if (typeof layout.orientation === 'number') {
              this.offset[1] *= -1;
            }
          }

          this.position = curr_midpt;
          this.updateBBoxes(this.position, size, this.angle, this.angle, this.offset);
          return true; // use this placement
        }

        prev_angle = next_angle;
        ahead_index++;
      }
    }

    return false;
  } // Calculate bounding boxes
  ;

  _proto2.updateBBoxes = function updateBBoxes(position, size, angle, angle_offset, offset) {
    var upp = this.unit_scale; // reset bounding boxes

    this.obbs = [];
    this.aabbs = [];
    var width = (size[0] + 2 * this.layout.buffer[0]) * upp * Label.epsilon;
    var height = (size[1] + 2 * this.layout.buffer[1]) * upp * Label.epsilon;
    var obb = LabelLineBase.createOBB(position, width, height, angle, angle_offset, offset, upp);
    var aabb = obb.getExtent();
    this.obbs.push(obb);
    this.aabbs.push(aabb);

    if (this.inTileBounds) {
      this.breach = !this.inTileBounds();
    }

    if (this.mayRepeatAcrossTiles) {
      this.may_repeat_across_tiles = this.mayRepeatAcrossTiles();
    }
  };

  return LabelLineStraight;
}(LabelLineBase); // Class for curved labels
// Extends base LabelLine class to support angles, pre_angles, offsets as arrays for each segment

var LabelLineCurved =
/*#__PURE__*/
function (_LabelLineBase2) {
  _inheritsLoose(LabelLineCurved, _LabelLineBase2);

  function LabelLineCurved(segment_sizes, line, layout) {
    var _this2;

    _this2 = _LabelLineBase2.call(this, layout) || this;
    _this2.type = 'curved'; // extra data for curved labels

    _this2.angles = [];
    _this2.pre_angles = [];
    _this2.offsets = [];
    _this2.num_segments = segment_sizes.length;
    _this2.sizes = segment_sizes;
    _this2.throw_away = !_this2.fit(_this2.sizes, line, layout);
    return _this2;
  } // Minimal representation of label


  var _proto3 = LabelLineCurved.prototype;

  _proto3.toJSON = function toJSON() {
    return {
      id: this.id,
      type: this.type,
      obbs: this.obbs.map(function (o) {
        return o.toJSON();
      }),
      position: this.position,
      breach: this.breach,
      may_repeat_across_tiles: this.may_repeat_across_tiles,
      layout: textLayoutToJSON(this.layout)
    };
  } // Determine if the curved label can fit the geometry.
  // No tolerance is provided because the label must fit entirely within the line geometry.
  ;

  _proto3.fit = function fit(size, line, layout) {
    var upp = this.unit_scale;
    var flipped; // boolean determining if the line orientation is reversed

    var height_px = Math.max.apply(Math, size.map(function (s) {
      return s[1];
    })); // use max segment height

    var height = height_px * upp; // Make new copy of line, with consistent orientation

    var _LabelLineBase$splitL2 = LabelLineBase.splitLineByOrientation(line);

    line = _LabelLineBase$splitL2[0];
    flipped = _LabelLineBase$splitL2[1];

    // matches for "left" or "right" labels where the offset angle is dependent on the geometry
    if (typeof layout.orientation === 'number') {
      this.offset[1] += ORIENTED_LABEL_OFFSET_FACTOR * (height_px - layout.vertical_buffer); // if line is flipped, or the orientation is "left" (-1), flip the offset's y-axis

      if (flipped) {
        this.offset[1] *= -1;
      }

      if (layout.orientation === -1) {
        this.offset[1] *= -1;
      }
    }

    var line_lengths = getLineLengths(line);
    var label_lengths = size.map(function (size) {
      return size[0] * upp;
    });
    var total_line_length = line_lengths.reduce(function (prev, next) {
      return prev + next;
    }, 0);
    var total_label_length = label_lengths.reduce(function (prev, next) {
      return prev + next;
    }, 0); // if label displacement is longer than the line, no fit can be possible

    if (total_label_length > total_line_length) {
      return false;
    } // need two line segments for a curved label
    // NB: single segment lines should still be labeled if possible during straight label placement pass


    var start_index = 0,
        end_index = line.length - 1;

    if (end_index - start_index < 2) {
      return false;
    } // all positional offsets of the label are relative to the anchor


    var anchor_index = LabelLineCurved.curvaturePlacement(line, total_line_length, line_lengths, total_label_length, start_index, end_index);
    var anchor = line[anchor_index]; // if anchor not found, or greater than the end_index, no fit possible

    if (anchor_index === -1 || end_index - anchor_index < 2) {
      return false;
    } // set start position at anchor position


    this.position = anchor; // Loop through labels at each zoom level stop
    // TODO: Can be made faster since we are computing every segment for every zoom stop
    // We can skip a segment's calculation once a segment's angle equals its fully zoomed angle

    for (var i = 0; i < label_lengths.length; i++) {
      this.offsets[i] = [];
      this.angles[i] = [];
      this.pre_angles[i] = []; // loop through stops (z = [0, .33, .66, .99] + base zoom)

      for (var j = 0; j < STOPS.length; j++) {
        var stop = STOPS[j]; // scale the line geometry by the zoom magnification

        var _LabelLineCurved$scal = LabelLineCurved.scaleLine(stop, line),
            new_line = _LabelLineCurved$scal[0],
            _line_lengths = _LabelLineCurved$scal[1];

        anchor = new_line[anchor_index]; // calculate label data relative to anchor position

        var _LabelLineCurved$plac = LabelLineCurved.placeAtIndex(anchor_index, new_line, _line_lengths, label_lengths),
            positions = _LabelLineCurved$plac.positions,
            offsets = _LabelLineCurved$plac.offsets,
            angles = _LabelLineCurved$plac.angles,
            pre_angles = _LabelLineCurved$plac.pre_angles; // translate 2D offsets into "polar coordinates"" (1D distances with angles)


        var offsets1d = offsets.map(function (offset) {
          return Math.sqrt(offset[0] * offset[0] + offset[1] * offset[1]) / upp;
        }); // Calculate everything that is independent of zoom level (angle for offset, bounding boxes, etc)

        if (stop === 0) {
          // use average angle for a global label offset (if offset is specified)
          this.angle = 1 / angles.length * angles.reduce(function (prev, next) {
            return prev + next;
          }); // calculate bounding boxes for collision at zoom level 0

          for (var _i = 0; _i < positions.length; _i++) {
            var position = positions[_i];
            var pre_angle = pre_angles[_i];
            var width = label_lengths[_i];
            var angle_segment = pre_angle + angles[_i];
            var angle_offset = this.angle;
            var obb = LabelLineBase.createOBB(position, width, height, angle_segment, angle_offset, this.offset, upp);
            var aabb = obb.getExtent();
            this.obbs.push(obb);
            this.aabbs.push(aabb);
          }
        } // push offsets/angles/pre_angles for each zoom and for each label segment


        this.offsets[i].push(offsets1d[i]);
        this.angles[i].push(angles[i]);
        this.pre_angles[i].push(pre_angles[i]);
      }
    }

    return true;
  } // Find optimal starting segment for placing a curved label along a line within provided tolerances
  // This is determined by calculating the curvature at each interior vertex of a line
  // then construct a "window" whose breadth is the length of the label. Place this label at each vertex
  // and add the curvatures of each vertex within the window. The vertex mimimizing this value is the "best" placement.
  // Return -1 is no placement found.
  ;

  LabelLineCurved.curvaturePlacement = function curvaturePlacement(line, total_line_length, line_lengths, label_length, start_index, end_index) {
    start_index = start_index || 0;
    end_index = end_index || line.length - 1;
    var curvatures = []; // array of curvature values per line vertex
    // calculate curvature values

    for (var i = start_index + 1; i < end_index; i++) {
      var prev = line[i - 1];
      var curr = line[i];
      var next = line[i + 1];
      var norm_1 = Vector$1.perp(curr, prev);
      var norm_2 = Vector$1.perp(next, curr);
      var curvature = Vector$1.angleBetween(norm_1, norm_2); // If curvature at a vertex is greater than the tolerance, remove it from consideration
      // by giving it an infinite penalty

      if (curvature > CURVE_MAX_ANGLE) {
        curvature = Infinity;
      }

      curvatures.push(curvature);
    }

    curvatures.push(Infinity); // Infinite penalty for going off end of line
    // calculate curvature costs

    var total_costs = [];
    var avg_costs = [];
    var line_index = start_index;
    var position = 0;

    for (var _i2 = 0; _i2 < start_index; _i2++) {
      position += line_lengths[_i2];
    } // move window along line, starting at first vertex


    while (position + label_length < total_line_length) {
      // define window breadth
      var window_start = position;
      var window_end = window_start + label_length;
      var line_position = window_start;
      var ahead_index = line_index;
      var cost = 0; // iterate through points on line intersecting window

      while (ahead_index < end_index && line_position + line_lengths[ahead_index] < window_end) {
        cost += curvatures[ahead_index];

        if (cost === Infinity) {
          break; // no further progress can be made
        }

        line_position += line_lengths[ahead_index];
        ahead_index++;
      } // if optimal cost, break out


      if (cost === 0) {
        return line_index;
      }

      var avg_cost = cost / (ahead_index - line_index);
      total_costs.push(cost);
      avg_costs.push(avg_cost);
      position += line_lengths[line_index];
      line_index++;
    }

    if (total_costs.length === 0) {
      return -1;
    } // calculate min cost and avg cost to determine if label can fit within curvatures tolerances


    var min_total_cost = Math.min.apply(null, total_costs);
    var min_index = total_costs.indexOf(min_total_cost);
    var min_avg_cost = avg_costs[min_index];

    if (min_total_cost < CURVE_MIN_TOTAL_COST && min_avg_cost < CURVE_MIN_AVG_COST) {
      // return index with best placement (least curvature)
      return total_costs.indexOf(min_total_cost);
    } else {
      // if tolerances aren't satisfied, throw away tile
      return -1;
    }
  } // Scale the line by a scale factor (used for computing the angles and offsets at fractional zoom levels)
  // Return the new line positions and their lengths
  ;

  LabelLineCurved.scaleLine = function scaleLine(scale, line) {
    var new_line = [line[0]];
    var line_lengths = [];
    line.forEach(function (pt, i) {
      if (i === line.length - 1) {
        return;
      }

      var v = Vector$1.sub(line[i + 1], line[i]);
      var delta = Vector$1.mult(v, 1 + scale);
      new_line.push(Vector$1.add(new_line[i], delta));
      line_lengths.push(Vector$1.length(delta));
    });
    return [new_line, line_lengths];
  } // Place a label at a given line index
  ;

  LabelLineCurved.placeAtIndex = function placeAtIndex(anchor_index, line, line_lengths, label_lengths) {
    var anchor = line[anchor_index]; // Use flat coordinates. Get nearest line vertex index, and offset from the vertex for all labels.

    var _LabelLineCurved$getI = LabelLineCurved.getIndicesAndOffsets(anchor_index, line_lengths, label_lengths),
        indices = _LabelLineCurved$getI[0],
        relative_offsets = _LabelLineCurved$getI[1]; // get 2D positions based on "flat" indices and offsets


    var positions = LabelLineCurved.getPositionsFromIndicesAndOffsets(line, indices, relative_offsets); // get 2d offsets, angles and pre_angles relative to anchor

    var _LabelLineCurved$getA = LabelLineCurved.getAnglesFromIndicesAndOffsets(anchor, indices, line, positions),
        offsets = _LabelLineCurved$getA[0],
        angles = _LabelLineCurved$getA[1],
        pre_angles = _LabelLineCurved$getA[2];

    return {
      positions: positions,
      offsets: offsets,
      angles: angles,
      pre_angles: pre_angles
    };
  } // Given label lengths to place along a line broken into several lengths, computer what indices and at which offsets
  // the labels will appear on the line. Assume the line is straight, as it is not necessary to consider angles.
  //
  // Label lengths:
  // |-----|----|-----|-----------------|-------------|
  //
  // Line Lengths;
  // |---------|---------|-------------|------------|----------|-------|
  //
  // Result: indices: [0,0,1,1,3,4]
  ;

  LabelLineCurved.getIndicesAndOffsets = function getIndicesAndOffsets(line_index, line_lengths, label_lengths) {
    var num_labels = label_lengths.length;
    var indices = [];
    var offsets = [];
    var label_index = 0;
    var label_offset = 0;
    var line_offset = 0; // iterate along line

    while (label_index < num_labels) {
      var label_length = label_lengths[label_index]; // iterate along labels within the line segment

      while (label_index < num_labels && label_offset + 0.5 * label_length <= line_offset + line_lengths[line_index]) {
        var offset = label_offset - line_offset + 0.5 * label_length;
        offsets.push(offset);
        indices.push(line_index);
        label_offset += label_length;
        label_index++;
        label_length = label_lengths[label_index];
      }

      line_offset += line_lengths[line_index];
      line_index++;
    }

    return [indices, offsets];
  } // Given indices and 1D offsets on a line, compute their 2D positions
  ;

  LabelLineCurved.getPositionsFromIndicesAndOffsets = function getPositionsFromIndicesAndOffsets(line, indices, offsets) {
    var positions = [];

    for (var i = 0; i < indices.length; i++) {
      var index = indices[i];
      var offset = offsets[i];
      var angle = getAngleForSegment(line[index], line[index + 1]);
      var offset2d = Vector$1.rot([offset, 0], angle);
      var position = Vector$1.add(line[index], offset2d);
      positions.push(position);
    }

    return positions;
  } // Given indices and 1D offsets on a line, compute their angles and pre-angles from a reference anchor point
  ;

  LabelLineCurved.getAnglesFromIndicesAndOffsets = function getAnglesFromIndicesAndOffsets(anchor, indices, line, positions) {
    var angles = [];
    var pre_angles = [];
    var offsets = [];

    for (var i = 0; i < positions.length; i++) {
      var position = positions[i];
      var index = indices[i];
      var offset = Vector$1.sub(position, anchor);
      var offset_angle = -Vector$1.angle(offset);
      var angle = getTextAngleForSegment(line[index], line[index + 1]);
      var pre_angle = angle - offset_angle;

      if (i > 0) {
        var prev_angle = angles[i - 1];
        var prev_pre_angle = pre_angles[i - 1];

        if (Math.abs(offset_angle - prev_angle) > Math.PI) {
          offset_angle += offset_angle > prev_angle ? -2 * Math.PI : 2 * Math.PI;
        }

        if (Math.abs(prev_pre_angle - pre_angle) > Math.PI) {
          pre_angle += pre_angle > prev_pre_angle ? -2 * Math.PI : 2 * Math.PI;
        }
      }

      angles.push(offset_angle);
      pre_angles.push(pre_angle);
      offsets.push(offset);
    }

    return [offsets, angles, pre_angles];
  };

  return LabelLineCurved;
}(LabelLineBase); // Fitness function (label length / line length)


function calcFitness(line_length, label_length) {
  return label_length / line_length;
}

function getAngleForSegment(p, q) {
  var pq = Vector$1.sub(q, p);
  return Vector$1.angle(pq);
}

function getTextAngleForSegment(pt1, pt2) {
  return -getAngleForSegment(pt1, pt2);
}

function getLineLengths(line) {
  var lengths = [];

  for (var i = 0; i < line.length - 1; i++) {
    var p = line[i];
    var q = line[i + 1];
    var length = Math.hypot(p[0] - q[0], p[1] - q[1]);
    lengths.push(length);
  }

  return lengths;
}

function getAbsAngleDiff(angle1, angle2) {
  var small, big;

  if (angle1 > angle2) {
    small = angle2;
    big = angle1;
  } else {
    small = angle1;
    big = angle2;
  }

  while (big - small > Math.PI) {
    small += 2 * Math.PI;
  }

  return Math.abs(big - small);
}

var TextStyle = Object.create(Points);
TextStyle.vertex_layouts = {}; // vertex layouts by variant key

Object.assign(TextStyle, {
  name: 'text',
  super: Points,
  built_in: true,
  init: function init(options) {
    if (options === void 0) {
      options = {};
    }

    Style.init.call(this, options); // Shader defines

    this.setupDefines(); // Omit some code for SDF-drawn shader points

    this.defines.TANGRAM_HAS_SHADER_POINTS = false; // Indicate vertex shader should apply zoom-interpolated offsets and angles for curved labels

    this.defines.TANGRAM_CURVED_LABEL = true;
    this.reset();
  },

  /**
   * A "template" that sets constant attibutes for each vertex, which is then modified per vertex or per feature.
   * A plain JS array matching the order of the vertex layout.
   */
  makeVertexTemplate: function makeVertexTemplate(style, mesh) {
    this.super.makeVertexTemplate.apply(this, arguments);
    var vertex_layout = mesh.vertex_data.vertex_layout;
    var i = vertex_layout.index.a_pre_angles; // a_pre_angles.xyzw - rotation of entire curved label
    // a_angles.xyzw - angle of each curved label segment
    // a_offsets.xyzw - offset of each curved label segment

    for (var j = 0; j < 12; j++) {
      this.vertex_template[i++] = 0;
    }

    return this.vertex_template;
  },
  reset: function reset() {
    this.queues = {};
    this.resetText();
  },
  // Override to queue features instead of processing immediately
  addFeature: function addFeature(feature, draw, context) {
    var _this = this;

    var tile = context.tile;

    if (tile.generation !== this.generation) {
      return;
    }

    var type = feature.geometry.type;
    draw.can_articulate = type === 'LineString' || type === 'MultiLineString'; // supersample text rendering for angled labels, to improve clarity

    draw.supersample_text = type === 'LineString' || type === 'MultiLineString';
    var q = this.parseTextFeature(feature, draw, context, tile);

    if (!q) {
      return;
    } // text can be an array if a `left` or `right` orientation key is defined for the text source
    // in which case, push both text sources to the queue


    if (q instanceof Array) {
      q.forEach(function (q) {
        q.feature = feature;
        q.context = context;
        q.layout.vertex = false; // vertex placement option not applicable to standalone labels

        _this.queueFeature(q, tile); // queue the feature for later processing

      });
    } else {
      q.feature = feature;
      q.context = context;
      q.layout.vertex = false; // vertex placement option not applicable to standalone labels

      this.queueFeature(q, tile); // queue the feature for later processing
    } // Register with collision manager


    Collision.addStyle(this.name, tile.id);
  },
  // Override
  endData: function endData(tile) {
    return new Promise(function ($return, $error) {
      var _this2, queue, _ref, labels, texts, textures, tile_data, _tile_data$textures;

      _this2 = this;
      queue = this.queues[tile.id];
      delete this.queues[tile.id];
      return Promise.resolve(this.collideAndRenderTextLabels(tile, this.name, queue)).then(function ($await_1) {
        try {
          _ref = $await_1, labels = _ref.labels, texts = _ref.texts, textures = _ref.textures;

          if (labels && texts) {
            this.texts[tile.id] = texts; // Build queued features

            labels.forEach(function (q) {
              var text_settings_key = q.text_settings_key;
              var text_info = _this2.texts[tile.id][text_settings_key] && _this2.texts[tile.id][text_settings_key][q.text]; // setup styling object expected by Style class

              var style = _this2.feature_style;
              style.label = q.label;

              if (text_info.text_settings.can_articulate) {
                // unpack logical sizes of each segment into an array for the style
                style.size = {};
                style.texcoords = {};

                if (q.label.type === 'straight') {
                  style.size.straight = text_info.size.logical_size;
                  style.texcoords.straight = text_info.texcoords.straight;
                  style.label_texture = textures[text_info.texcoords.straight.texture_id];
                } else {
                  style.size.curved = text_info.segment_sizes.map(function (size) {
                    return size.logical_size;
                  });
                  style.texcoords_stroke = text_info.texcoords_stroke;
                  style.texcoords.curved = text_info.texcoords.curved;
                  style.label_textures = text_info.texcoords.curved.map(function (t) {
                    return textures[t.texture_id];
                  });
                }
              } else {
                style.size = text_info.size.logical_size;
                style.texcoords = text_info.align[q.label.align].texcoords;
                style.label_texture = textures[text_info.align[q.label.align].texture_id];
              }

              style.blend_order = q.draw.blend_order; // copy pre-computed blend order

              Style.addFeature.call(_this2, q.feature, q.draw, q.context);
            });
          }

          this.freeText(tile); // Finish tile mesh

          return Promise.resolve(Style.endData.call(this, tile)).then(function ($await_2) {
            try {
              tile_data = $await_2;

              if (tile_data) {
                // Attach tile-specific label atlas to mesh as a texture uniform
                if (textures && textures.length) {
                  (_tile_data$textures = tile_data.textures).push.apply(_tile_data$textures, textures); // assign texture ownership to tile

                } // Always apply shader blocks to standalone text


                for (var m in tile_data.meshes) {
                  tile_data.meshes[m].uniforms.u_apply_color_blocks = true;
                }
              }

              return $return(tile_data);
            } catch ($boundEx) {
              return $error($boundEx);
            }
          }, $error);
        } catch ($boundEx) {
          return $error($boundEx);
        }
      }.bind(this), $error);
    }.bind(this));
  },
  // Sets up caching for draw properties
  _preprocess: function _preprocess(draw) {
    draw.blend_order = this.getBlendOrderForDraw(draw); // from draw block, or fall back on default style blend order

    return this.preprocessText(draw);
  },
  // Implements label building for TextLabels mixin
  buildTextLabels: function buildTextLabels(tile, feature_queue) {
    var labels = [];

    for (var f = 0; f < feature_queue.length; f++) {
      var fq = feature_queue[f];
      var text_info = this.texts[tile.id][fq.text_settings_key][fq.text];
      var feature_labels = void 0;
      fq.layout.vertical_buffer = text_info.vertical_buffer;

      if (text_info.text_settings.can_articulate) {
        var sizes = text_info.segment_sizes.map(function (size) {
          return size.collision_size;
        });
        fq.layout.no_curving = text_info.no_curving;
        feature_labels = this.buildLabels(sizes, fq.feature.geometry, fq.layout, text_info.size.collision_size);
      } else {
        feature_labels = this.buildLabels(text_info.size.collision_size, fq.feature.geometry, fq.layout);
      }

      for (var i = 0; i < feature_labels.length; i++) {
        var fql = Object.create(fq);
        fql.label = feature_labels[i];
        labels.push(fql);
      }
    }

    return labels;
  },
  // Builds one or more labels for a geometry
  buildLabels: function buildLabels(size, geometry, layout, total_size) {
    var labels = [];

    if (geometry.type === 'LineString') {
      Array.prototype.push.apply(labels, this.buildLineLabels(geometry.coordinates, size, layout, total_size));
    } else if (geometry.type === 'MultiLineString') {
      var lines = geometry.coordinates;

      for (var i = 0; i < lines.length; ++i) {
        Array.prototype.push.apply(labels, this.buildLineLabels(lines[i], size, layout, total_size));
      }
    } else if (geometry.type === 'Point') {
      labels.push(new LabelPoint(geometry.coordinates, size, layout));
    } else if (geometry.type === 'MultiPoint') {
      var points = geometry.coordinates;

      for (var _i = 0; _i < points.length; ++_i) {
        labels.push(new LabelPoint(points[_i], size, layout));
      }
    } else if (geometry.type === 'Polygon') {
      var centroid = Geo$1.centroid(geometry.coordinates);

      if (centroid) {
        // skip degenerate polygons
        labels.push(new LabelPoint(centroid, size, layout));
      }
    } else if (geometry.type === 'MultiPolygon') {
      var _centroid = Geo$1.multiCentroid(geometry.coordinates);

      if (_centroid) {
        // skip degenerate polygons
        labels.push(new LabelPoint(_centroid, size, layout));
      }
    }

    return labels;
  },
  // Build one or more labels for a line geometry
  buildLineLabels: function buildLineLabels(line, size, layout, total_size) {
    var labels = [];
    var subdiv = Math.min(layout.subdiv, line.length - 1);

    if (subdiv > 1) {
      // Create multiple labels for line, with each allotted a range of segments
      // in which it will attempt to place
      var seg_per_div = (line.length - 1) / subdiv;

      for (var i = 0; i < subdiv; i++) {
        var start = Math.floor(i * seg_per_div);
        var end = Math.floor((i + 1) * seg_per_div) + 1;
        var line_segment = line.slice(start, end);
        var label = LabelLine.create(size, total_size, line_segment, layout);

        if (label) {
          labels.push(label);
        }
      }
    } // Consider full line for label placement if no subdivisions requested, or as last resort if not enough
    // labels placed (e.g. fewer than requested subdivisions)
    // TODO: refactor multiple label placements per line / move into label placement class for better effectiveness


    if (labels.length < subdiv) {
      var _label = LabelLine.create(size, total_size, line, layout);

      if (_label) {
        labels.push(_label);
      }
    }

    return labels;
  },
  // Override
  // Create or return vertex layout
  vertexLayoutForMeshVariant: function vertexLayoutForMeshVariant(variant) {
    // Vertex layout only depends on shader point flag, so using it as layout key to avoid duplicate layouts
    if (TextStyle.vertex_layouts[variant.shader_point] == null) {
      // TODO: could make selection, offset, and curved label attribs optional, but may not be worth it
      // since text points generally don't consume much memory anyway
      var attribs = [{
        name: 'a_position',
        size: 4,
        type: gl$1.SHORT,
        normalized: false
      }, {
        name: 'a_shape',
        size: 4,
        type: gl$1.SHORT,
        normalized: false
      }, {
        name: 'a_texcoord',
        size: 2,
        type: gl$1.UNSIGNED_SHORT,
        normalized: true
      }, {
        name: 'a_offset',
        size: 2,
        type: gl$1.SHORT,
        normalized: false
      }, {
        name: 'a_color',
        size: 4,
        type: gl$1.UNSIGNED_BYTE,
        normalized: true
      }, {
        name: 'a_selection_color',
        size: 4,
        type: gl$1.UNSIGNED_BYTE,
        normalized: true,
        static: variant.selection ? null : [0, 0, 0, 0]
      }, {
        name: 'a_pre_angles',
        size: 4,
        type: gl$1.BYTE,
        normalized: false
      }, {
        name: 'a_angles',
        size: 4,
        type: gl$1.SHORT,
        normalized: false
      }, {
        name: 'a_offsets',
        size: 4,
        type: gl$1.UNSIGNED_SHORT,
        normalized: false
      }];
      TextStyle.vertex_layouts[variant.shader_point] = new VertexLayout(attribs);
    }

    return TextStyle.vertex_layouts[variant.shader_point];
  }
});
TextStyle.texture_id = 0; // namespaces per-tile label textures

var RasterStyle = Object.create(Polygons);
Object.assign(RasterStyle, {
  name: 'raster',
  super: Polygons,
  built_in: true,
  init: function init() {
    // Required for raster tiles
    this.raster = this.raster || 'color';
    this.super.init.apply(this, arguments);
    this.selection = false; // raster styles can't support feature selection
  },
  _preprocess: function _preprocess(draw) {
    // Raster tiles default to white vertex color, as this color will tint the underlying texture
    draw.color = draw.color || StyleParser.defaults.color;
    return this.super._preprocess.apply(this, arguments);
  }
});

var style_globals_source = "#ifdef TANGRAM_VERTEX_SHADER\n\n// Apply layer ordering to avoid z-fighting\nvoid applyLayerOrder (float layer, inout vec4 position) {\n    position.z -= layer * TANGRAM_LAYER_DELTA * position.w;\n}\n\n// Vertex position in model space: [0, 1] range over the local tile\n// Note positions can be outside that range due to unclipped geometry, geometry higher than a unit cube, etc.\nvec4 modelPosition() {\n    return\n        vec4(\n            a_position.xyz / TANGRAM_TILE_SCALE         // scale coords to ~0-1 range\n            * exp2(u_tile_origin.z - u_tile_origin.w),  // adjust for tile overzooming\n        1.)\n        + vec4(0., 1., 0., 0.);\n        // NB: additional offset to account for unusual Tangram JS y coords,\n        // should be refactored to remove\n}\n\n// Position in model space as above, but according to tile coordinate (as opposed to style) zoom\n// e.g. unadjusted for tile overzooming\nvec4 modelPositionBaseZoom() {\n    return\n        vec4(\n            a_position.xyz / TANGRAM_TILE_SCALE,    // scale coords to ~0-1 range\n        1.)\n        + vec4(0., 1., 0., 0.); // see note on offset above\n}\n\n#endif\n\n// Vertex position in world coordinates, useful for 3d procedural textures, etc.\nvec4 worldPosition() {\n    return v_world_position;\n}\n\n// Optionally wrap world coordinates (allows more precision at higher zooms)\n// e.g. at wrap 1000, the world space will wrap every 1000 meters\n#ifdef TANGRAM_VERTEX_SHADER\n\nvec4 wrapWorldPosition(vec4 world_position) {\n    #if defined(TANGRAM_WORLD_POSITION_WRAP)\n    vec2 anchor = u_tile_origin.xy - mod(u_tile_origin.xy, TANGRAM_WORLD_POSITION_WRAP);\n        world_position.xy -= anchor;\n    #endif\n    return world_position;\n}\n\n#endif\n\n// Normal in world space\n#if defined(TANGRAM_VERTEX_SHADER)\n\nvec3 worldNormal() {\n    return TANGRAM_NORMAL;\n}\n\n#elif defined(TANGRAM_FRAGMENT_SHADER)\n\nvec3 worldNormal() {\n    return u_inverseNormalMatrix * TANGRAM_NORMAL;\n}\n\n#endif\n";

var selection_globals_source = "// Vertex attribute + varying for feature selection\n#if defined(TANGRAM_FEATURE_SELECTION) && defined(TANGRAM_VERTEX_SHADER)\n    attribute vec4 a_selection_color;\n    varying vec4 v_selection_color;\n#endif\n";

var selection_vertex_source = "// Selection pass-specific rendering\n#if defined(TANGRAM_FEATURE_SELECTION) && defined(TANGRAM_VERTEX_SHADER)\n    if (a_selection_color.rgb == vec3(0.)) {\n        // Discard by forcing invalid triangle if we're in the feature\n        // selection pass but have no selection info\n        // TODO: in some cases we may actually want non-selectable features to occlude selectable ones?\n        gl_Position = vec4(0., 0., 0., 1.);\n        return;\n    }\n    v_selection_color = a_selection_color;\n#endif\n";

var StyleManager =
/*#__PURE__*/
function () {
  function StyleManager() {
    this.styles = {};
    this.base_styles = {};
    this.active_styles = [];
    this.active_blend_orders = []; // Add built-in rendering styles

    this.register(Object.create(Polygons));
    this.register(Object.create(Lines));
    this.register(Object.create(Points));
    this.register(Object.create(TextStyle));
    this.register(Object.create(RasterStyle));
  } // Global configuration for all styles


  var _proto = StyleManager.prototype;

  _proto.init = function init() {
    ShaderProgram.removeBlock('global');
    ShaderProgram.removeBlock('setup'); // Model and world position accessors, layer re-ordering function

    ShaderProgram.addBlock('global', style_globals_source); // Feature selection global

    ShaderProgram.addBlock('global', selection_globals_source); // Feature selection vertex shader support

    ShaderProgram.replaceBlock('setup', selection_vertex_source); // Minimum value for float comparisons

    ShaderProgram.defines.TANGRAM_EPSILON = 0.00001; // Minimum depth buffer value separating each `order` unit
    // Assume min 16-bit depth buffer, in practice uses 14-bits, 1 extra bit to handle virtual half-layers
    // for outlines (inserted in between layers), another extra bit to prevent precision loss

    ShaderProgram.defines.TANGRAM_LAYER_DELTA = 1 / (1 << 14); // Internal tile scale

    ShaderProgram.defines.TANGRAM_TILE_SCALE = "vec3(" + Geo$1.tile_scale + "., " + Geo$1.tile_scale + "., u_meters_per_pixel * " + Geo$1.tile_size + ".)"; // Increases precision for height values

    ShaderProgram.defines.TANGRAM_HEIGHT_SCALE = Geo$1.height_scale; // Alpha discard threshold (substitute for alpha blending)

    ShaderProgram.defines.TANGRAM_ALPHA_TEST = 0.5;
  } // Destroy all styles for a given GL context
  ;

  _proto.destroy = function destroy(gl) {
    var _this = this;

    Object.keys(this.styles).forEach(function (_name) {
      var style = _this.styles[_name];

      if (style.gl === gl) {
        log('trace', "StyleManager.destroy: destroying render style " + style.name);

        if (style.base) {
          _this.remove(style.name);
        }

        style.destroy();
      }
    });
  } // Register a style
  ;

  _proto.register = function register(style) {
    this.styles[style.name] = style;
    this.base_styles[style.name] = style;
  } // Remove a style
  ;

  _proto.remove = function remove(name) {
    delete this.styles[name];
  };

  _proto.getActiveStyles = function getActiveStyles() {
    return this.active_styles;
  } // Get list of active styles based on a set of tiles
  ;

  _proto.updateActiveStyles = function updateActiveStyles(tiles) {
    this.active_styles = Object.keys(tiles.reduce(function (active, tile) {
      Object.keys(tile.meshes).forEach(function (s) {
        return active[s] = true;
      });
      return active;
    }, {}));
    return this.active_styles;
  };

  _proto.getActiveBlendOrders = function getActiveBlendOrders() {
    return this.active_blend_orders;
  };

  _proto.updateActiveBlendOrders = function updateActiveBlendOrders(tiles) {
    var orders = [];
    tiles.forEach(function (tile) {
      Object.entries(tile.meshes).forEach(function (_ref) {
        var style = _ref[0],
            style_meshes = _ref[1];
        // for each tile's set of meshes, keyed by style name
        style_meshes.forEach(function (mesh) {
          // for each style's list of meshes
          // find entry for this mesh's blend order, insert if first entry
          var blend_order = mesh.variant.blend_order;
          var oi = orders.findIndex(function (x) {
            return x.blend_order === blend_order;
          });
          oi = oi > -1 ? oi : orders.push({
            blend_order: blend_order,
            styles: []
          }) - 1; // add style to list for this blend order

          if (orders[oi].styles.indexOf(style) === -1) {
            orders[oi].styles.push(style);
          }
        });
      });
    }); // sort ascending by blend order

    this.active_blend_orders = orders.sort(function (a, b) {
      return a.blend_order - b.blend_order;
    });
  };

  _proto.mix = function mix(style, styles) {
    // Exit early if we have already applied mixins to this style
    if (style.mixed) {
      return style;
    }

    style.mixed = {}; // Mixin sources, in order

    var sources = [];

    if (style.mix) {
      if (Array.isArray(style.mix)) {
        var _sources;

        (_sources = sources).push.apply(_sources, style.mix);
      } else {
        sources.push(style.mix);
      }

      sources = sources.map(function (x) {
        return styles[x];
      }).filter(function (x) {
        return x && x !== style;
      }); // TODO: warning on trying to mix into self
      // Track which styles were mixed into this one

      sources.forEach(function (s) {
        return style.mixed[s.name] = true;
      });
    }

    sources.push(style); // Flags - OR'd, true if any style has it set

    style.animated = sources.some(function (x) {
      return x && x.animated;
    });
    style.texcoords = sources.some(function (x) {
      return x && x.texcoords;
    }); // Overwrites - last definition wins

    style.base = sources.map(function (x) {
      return x.base;
    }).filter(function (x) {
      return x;
    }).pop();
    style.lighting = sources.map(function (x) {
      return x.lighting;
    }).filter(function (x) {
      return x != null;
    }).pop();
    style.texture = sources.map(function (x) {
      return x.texture;
    }).filter(function (x) {
      return x;
    }).pop();
    style.raster = sources.map(function (x) {
      return x.raster;
    }).filter(function (x) {
      return x != null;
    }).pop();
    style.dash = sources.map(function (x) {
      return x.dash;
    }).filter(function (x) {
      return x != null;
    }).pop();
    style.dash_background_color = sources.map(function (x) {
      return x.dash_background_color;
    }).filter(function (x) {
      return x != null;
    }).pop();

    if (sources.some(function (x) {
      return x.hasOwnProperty('blend') && x.blend;
    })) {
      // only mix blend if explicitly set, otherwise let base style choose blending mode
      // hasOwnProperty check gives preference to base style prototype
      style.blend = sources.map(function (x) {
        return x.hasOwnProperty('blend') && x.blend;
      }).filter(function (x) {
        return x;
      }).pop();
    }

    style.blend_order = sources.map(function (x) {
      return x.blend_order;
    }).filter(function (x) {
      return x != null;
    }).pop(); // Merges - property-specific rules for merging values

    style.defines = Object.assign.apply(Object, [{}].concat(sources.map(function (x) {
      return x.defines;
    }).filter(function (x) {
      return x;
    }))); // internal defines (not user-defined)

    style.material = Object.assign.apply(Object, [{}].concat(sources.map(function (x) {
      return x.material;
    }).filter(function (x) {
      return x;
    })));
    var draws = sources.map(function (x) {
      return x.draw;
    }).filter(function (x) {
      return x;
    }); // draw defaults

    if (draws.length > 0) {
      style.draw = mergeObjects.apply(void 0, [{}].concat(draws));
    } // Mix shader properties


    this.mixShaders(style, styles, sources);
    return style;
  } // Mix the propertes in the "shaders" block
  ;

  _proto.mixShaders = function mixShaders(style, styles, sources) {
    var shaders = {}; // newly mixed shaders properties

    var shader_merges = sources.map(function (x) {
      return x.shaders;
    }).filter(function (x) {
      return x;
    }); // just the source styles with shader properties
    // Defines

    shaders.defines = Object.assign.apply(Object, [{}].concat(shader_merges.map(function (x) {
      return x.defines;
    }).filter(function (x) {
      return x;
    }))); // Uniforms

    shaders.uniforms = {}; // uniforms for this style, both explicitly defined, and mixed from other styles

    shaders._uniforms = style.shaders && style.shaders.uniforms || {}; // uniforms explicitly defined by *this* style

    shaders._uniform_scopes = {}; // tracks which style each uniform originated from (this one, or ancestor)
    // Mix in uniforms from ancestors, providing means to access

    sources.filter(function (x) {
      return x.shaders && x.shaders.uniforms;
    }).forEach(function (x) {
      var _loop = function _loop(u) {
        shaders._uniform_scopes[u] = x.name; // Define getter and setter for this uniform
        // Getter returns value for this style if present, otherwise asks appropriate ancestor for it
        // Setter sets the value for this style (whether previously present in this style or not)
        // Mimics JS prototype/hasOwnProperty behavior, but with multiple ancestors (via mixins)

        Object.defineProperty(shaders.uniforms, u, {
          enumerable: true,
          configurable: true,
          get: function get() {
            // Uniform is explicitly defined on this style
            if (shaders._uniforms[u] !== undefined) {
              return shaders._uniforms[u];
            } // Uniform was mixed from another style, forward request there
            // Identity check is needed to prevent infinite recursion if a previously defined uniform
            // is set to undefined
            else if (styles[shaders._uniform_scopes[u]].shaders.uniforms !== shaders.uniforms) {
                return styles[shaders._uniform_scopes[u]].shaders.uniforms[u];
              }

            return undefined;
          },
          set: function set(v) {
            shaders._uniforms[u] = v;
          }
        });
      };

      for (var u in x.shaders.uniforms) {
        _loop(u);
      }
    }); // Extensions: build a list of unique extensions

    shaders.extensions = Object.keys(shader_merges.map(function (x) {
      return x.extensions;
    }).filter(function (x) {
      return x;
    }).reduce(function (prev, cur) {
      // single extension
      if (typeof cur === 'string') {
        prev[cur] = true;
      } // array of extensions
      else {
          cur.forEach(function (x) {
            return prev[x] = true;
          });
        }

      return prev;
    }, {}) || {}); // Shader blocks
    // Mark all shader blocks for the target style as originating with its own name

    if (style.shaders && style.shaders.blocks) {
      style.shaders.block_scopes = style.shaders.block_scopes || {};

      for (var k in style.shaders.blocks) {
        var block = style.shaders.blocks[k];
        style.shaders.block_scopes[k] = style.shaders.block_scopes[k] || [];

        if (Array.isArray(block)) {
          var _style$shaders$block_;

          (_style$shaders$block_ = style.shaders.block_scopes[k]).push.apply(_style$shaders$block_, block.map(function () {
            return style.name;
          }));
        } else {
          style.shaders.block_scopes[k].push(style.name);
        }
      }
    } // Merge shader blocks, keeping track of which style each block originated from


    var mixed = {}; // all scopes mixed so far

    shader_merges.forEach(function (source) {
      if (!source.blocks) {
        return;
      }

      shaders.blocks = shaders.blocks || {};
      shaders.block_scopes = shaders.block_scopes || {};
      var mixed_source = {}; // scopes mixed for this source style

      for (var t in source.blocks) {
        var _block = source.blocks[t];
        var block_scope = source.block_scopes[t];
        shaders.blocks[t] = shaders.blocks[t] || [];
        shaders.block_scopes[t] = shaders.block_scopes[t] || []; // standardize on arrays (block can be single or multi-value)

        _block = Array.isArray(_block) ? _block : [_block];
        block_scope = Array.isArray(block_scope) ? block_scope : [block_scope];

        for (var b = 0; b < _block.length; b++) {
          // Skip blocks we've already mixed in from the same scope
          // Repeating scope indicates a diamond pattern where a style is being mixed multiple times
          if (mixed[block_scope[b]]) {
            continue;
          }

          mixed_source[block_scope[b]] = true;
          shaders.blocks[t].push(_block[b]);
          shaders.block_scopes[t].push(block_scope[b]);
        }
      } // Add styles mixed in from this source - they could be multi-level ancestors,
      // beyond the first-level "parents" defined in this style's `mix` list


      Object.assign(mixed, mixed_source);
    });
    Object.assign(style.mixed, mixed); // add all newly mixed styles

    style.shaders = shaders; // assign back to style

    return style;
  } // Create a new style
  // name: name of new style
  // config: properties of new style
  // styles: working set of styles being built (used for mixing in existing styles)
  ;

  _proto.create = function create(name, config, styles) {
    if (styles === void 0) {
      styles = {};
    }

    var style = mergeObjects({}, config); // deep copy

    style.name = name; // Style mixins

    style = this.mix(style, styles); // Has base style?
    // Only renderable (instantiated) styles should be included for run-time use
    // Others are intermediary/abstract, used during style composition but not execution

    if (style.base && this.base_styles[style.base]) {
      this.styles[name] = style = Object.assign(Object.create(this.base_styles[style.base]), style);
    } else {
      style.base = null; // null out invalid base style
    }

    return style;
  } // Called to create and initialize styles
  ;

  _proto.build = function build(styles) {
    var _this2 = this;

    // Un-register existing styles from cross-thread communication
    if (this.styles) {
      Object.values(this.styles).forEach(function (s) {
        return WorkerBroker$1.removeTarget(s.main_thread_target);
      });
    } // Sort styles by dependency, then build them


    var style_deps = Object.keys(styles).sort(function (a, b) {
      return _this2.inheritanceDepth(a, styles) - _this2.inheritanceDepth(b, styles);
    }); // Only keep built-in base styles

    for (var sname in this.styles) {
      if (!this.base_styles[sname]) {
        delete this.styles[sname];
      } else {
        this.styles[sname].reset();
      }
    } // Working set of styles being built


    var ws = {};
    style_deps.forEach(function (sname) {
      ws[sname] = _this2.create(sname, styles[sname], ws);
    });
    return this.styles;
  } // Initialize all styles
  ;

  _proto.initStyles = function initStyles(scene) {
    if (scene === void 0) {
      scene = {};
    }

    // Initialize all
    for (var sname in this.styles) {
      this.styles[sname].init(scene);
    }
  } // Given a style key in a set of styles to add, count the length of the inheritance chain
  ;

  _proto.inheritanceDepth = function inheritanceDepth(key, styles) {
    var _this3 = this;

    var parents = 0;

    for (;;) {
      var style = styles[key];

      if (!style) {
        // this is a scene def error, trying to extend a style that doesn't exist
        break;
      } // Dependency chain ends when this style isn't mixing in any others


      if (!style.mix) {
        break;
      } // Traverse next parent style


      parents++;

      if (Array.isArray(style.mix)) {
        // If multiple mixins, find the deepest one
        parents += Math.max.apply(Math, style.mix.map(function (s) {
          // Trying to mix into itself!
          if (key === s) {
            return;
          }

          return _this3.inheritanceDepth(s, styles);
        }));
        break;
      } else {
        // Trying to mix into itself!
        if (key === style.mix) {
          break;
        } // If single mixin, continue loop up the tree


        key = style.mix;
      }
    }

    return parents;
  };

  return StyleManager;
}();

// 20.1.2.6 Number.MAX_SAFE_INTEGER


_export(_export.S, 'Number', { MAX_SAFE_INTEGER: 0x1fffffffffffff });

function notNull(x) {
  return x != null;
}

function wrap(x) {
  return '(' + x + ')';
}

function maybeQuote(value) {
  if (typeof value === 'string') {
    return '"' + value + '"';
  }

  return value;
}

function lookUp(key) {
  if (key[0] === '$') {
    return 'context[\'' + key.substring(1) + '\']';
  }

  return 'context.feature.properties[\'' + key + '\']';
}

function nullValue()
/*key, value*/
{
  return ' true ';
}

function propertyEqual(key, value) {
  return wrap(maybeQuote(value) + ' === ' + lookUp(key));
}

function propertyOr(key, values) {
  return wrap(values.map(function (x) {
    return propertyEqual(key, x);
  }).join(' || '));
}

function printNested(values, joiner) {
  return wrap(values.filter(notNull).map(function (x) {
    return wrap(x.join(' && '));
  }).join(' ' + joiner + ' '));
}

function any(_, values, options) {
  return values && values.length > 0 ? printNested(values.map(function (v) {
    return parseFilter(v, options);
  }), '||') : 'true';
}

function all(_, values, options) {
  return values && values.length > 0 ? printNested(values.map(function (v) {
    return parseFilter(v, options);
  }), '&&') : 'true';
}

function not(key, value, options) {
  return '!' + wrap(parseFilter(value, options).join(' && '));
}

function none(key, values, options) {
  return '!' + wrap(any(null, values, options));
}

function propertyMatchesBoolean(key, value) {
  return wrap(lookUp(key) + (value ? ' != ' : ' == ') + 'null');
}

function rangeMatch(key, values, options) {
  var expressions = [];
  var transform = options && typeof options.rangeTransform === 'function' && options.rangeTransform;

  if (values.max) {
    var max = transform ? transform(values.max) : values.max;
    expressions.push('' + lookUp(key) + ' < ' + max);
  }

  if (values.min) {
    var min = transform ? min = transform(values.min) : values.min;
    expressions.push('' + lookUp(key) + ' >= ' + min);
  }

  return wrap(expressions.join(' && '));
}

function parseFilter(filter, options) {
  var filterAST = []; // Function filter

  if (typeof filter === 'function') {
    return [wrap(filter.toString() + '(context)')];
  } // Array filter, implicit 'any'
  else if (Array.isArray(filter)) {
      return [any(null, filter, options)];
    } // Null filter object
    else if (filter == null) {
        return ['true'];
      } // Object filter, e.g. implicit 'all'


  var keys = Object.keys(filter);

  for (var k = 0; k < keys.length; k++) {
    var key = keys[k];
    var value = filter[key],
        type = typeof value;

    if (type === 'string' || type === 'number') {
      filterAST.push(propertyEqual(key, value));
    } else if (type === 'boolean') {
      filterAST.push(propertyMatchesBoolean(key, value));
    } else if (key === 'not') {
      filterAST.push(not(key, value, options));
    } else if (key === 'any') {
      filterAST.push(any(key, value, options));
    } else if (key === 'all') {
      filterAST.push(all(key, value, options));
    } else if (key === 'none') {
      filterAST.push(none(key, value, options));
    } else if (Array.isArray(value)) {
      filterAST.push(propertyOr(key, value));
    } else if (type === 'object' && value != null) {
      if (value.max || value.min) {
        filterAST.push(rangeMatch(key, value, options));
      }
    } else if (value == null) {
      filterAST.push(nullValue(key, value));
    } else {
      throw new Error('Unknown Query syntax: ' + value);
    }
  }

  return keys.length === 0 ? ['true'] : filterAST;
}

function filterToString(filterAST) {
  return wrap(filterAST.join(' && '));
}

function buildFilter(filter, options) {
  if (filter == null) {
    return function () {
      return true;
    };
  } // jshint evil: true


  return new Function('context', 'return ' + filterToString(parseFilter(filter, options)) + ';');
}

var reserved = ['filter', 'draw', 'visible', 'enabled', 'data', 'exclusive', 'priority'];
var layer_cache = {};
function layerCache() {
  return layer_cache;
}

function cacheKey(layers) {
  if (layers.length > 1) {
    var k = layers[0];

    for (var i = 1; i < layers.length; i++) {
      k += '/' + layers[i];
    }

    return k;
  }

  return layers[0];
} // Merge matching layer trees into a final draw group


function mergeTrees(matchingTrees, group) {
  var draws,
      treeDepth = 0; // Find deepest tree

  for (var t = 0; t < matchingTrees.length; t++) {
    if (matchingTrees[t].length > treeDepth) {
      treeDepth = matchingTrees[t].length;
    }
  } // No layers to parse


  if (treeDepth === 0) {
    return null;
  } // Merged draw group object


  var draw = {
    visible: true // visible by default

  }; // Iterate trees in parallel

  var _loop = function _loop(x) {
    // Pull out the requested draw group, for each tree, at this depth (avoiding duplicates at the same level in tree)
    draws = [];
    matchingTrees.forEach(function (tree) {
      if (tree[x] && tree[x][group] && draws.indexOf(tree[x][group]) === -1) {
        draws.push(tree[x][group]);
      }
    });

    if (draws.length === 0) {
      return "continue";
    } // Merge draw objects


    mergeObjects.apply(void 0, [draw].concat(draws));
  };

  for (var x = 0; x < treeDepth; x++) {
    var _ret = _loop(x);

    if (_ret === "continue") continue;
  } // Short-circuit if not visible


  if (draw.visible === false) {
    return null;
  }

  return draw;
}
var blacklist = ['any', 'all', 'not', 'none'];

var Layer =
/*#__PURE__*/
function () {
  function Layer(_ref) {
    var layer = _ref.layer,
        name = _ref.name,
        parent = _ref.parent,
        draw = _ref.draw,
        visible = _ref.visible,
        enabled = _ref.enabled,
        filter = _ref.filter,
        exclusive = _ref.exclusive,
        priority = _ref.priority,
        styles = _ref.styles;
    this.id = Layer.id++;
    this.config_data = layer.data;
    this.parent = parent;
    this.name = name;
    this.full_name = this.parent ? this.parent.full_name + ':' + this.name : this.name;
    this.draw = draw;
    this.filter = filter;
    this.exclusive = exclusive === true;
    this.priority = priority != null ? priority : Number.MAX_SAFE_INTEGER;
    this.styles = styles;
    this.is_built = false;
    enabled = enabled === undefined ? visible : enabled; // `visible` property is backwards compatible for `enabled`

    this.enabled = enabled !== false; // layer is enabled unless explicitly set to disabled
    // Denormalize layer name to draw groups

    if (this.draw) {
      for (var group in this.draw) {
        this.draw[group] = this.draw[group] == null ? {} : this.draw[group];

        if (typeof this.draw[group] !== 'object') {
          // Invalid draw group
          var msg = "Draw group '" + group + "' for layer " + this.full_name + " is invalid, must be an object, ";
          msg += "but was set to `" + group + ": " + this.draw[group] + "` instead";
          log('warn', msg); // TODO: fire external event that clients to subscribe to

          delete this.draw[group];
        }
      }
    }
  }

  var _proto = Layer.prototype;

  _proto.build = function build() {
    log('trace', "Building layer '" + this.full_name + "'");
    this.buildFilter();
    this.buildDraw();
    this.is_built = true;
  };

  _proto.buildDraw = function buildDraw() {
    this.draw = compileFunctionStrings(this.draw, StyleParser.wrapFunction);
    this.calculatedDraw = calculateDraw(this);
  };

  _proto.buildFilter = function buildFilter$$1() {
    this.filter_original = this.filter;
    this.filter = compileFunctionStrings(this.filter, StyleParser.wrapFunction);
    var type = typeof this.filter;

    if (this.filter != null && type !== 'object' && type !== 'function') {
      // Invalid filter
      var msg = "Filter for layer " + this.full_name + " is invalid, filter value must be an object or function, ";
      msg += "but was set to `filter: " + this.filter + "` instead";
      log('warn', msg); // TODO: fire external event that clients to subscribe to

      return;
    }

    try {
      this.buildZooms();
      this.buildPropMatches();

      if (this.filter != null && (typeof this.filter === 'function' || Object.keys(this.filter).length > 0)) {
        this.filter = buildFilter(this.filter, FilterOptions);
      } else {
        this.filter = null;
      }
    } catch (e) {
      // Invalid filter
      var _msg = "Filter for layer " + this.full_name + " is invalid, `filter: " + JSON.stringify(this.filter) + "` ";

      _msg += "failed with error '" + e.message + "', stack trace: " + e.stack;
      log('warn', _msg); // TODO: fire external event that clients to subscribe to
    }
  } // Zooms often cull large swaths of the layer tree, so they get special treatment and are checked first
  ;

  _proto.buildZooms = function buildZooms() {
    var zoom = this.filter && this.filter.$zoom; // has an explicit zoom filter

    var ztype = typeof zoom;

    if (zoom != null) {
      this.zooms = {};

      if (ztype === 'number') {
        this.zooms[zoom] = true;
      } else if (Array.isArray(zoom)) {
        for (var z = 0; z < zoom.length; z++) {
          this.zooms[zoom[z]] = true;
        }
      } else if (ztype === 'object' && (zoom.min != null || zoom.max != null)) {
        var zmin = zoom.min || 0;
        var zmax = zoom.max || Geo$1.max_style_zoom;

        for (var _z = zmin; _z < zmax; _z++) {
          this.zooms[_z] = true;
        }
      }

      delete this.filter.$zoom; // don't process zoom through usual generic filter logic
    }
  };

  _proto.buildPropMatches = function buildPropMatches() {
    var _this = this;

    if (!this.filter || Array.isArray(this.filter) || typeof this.filter === 'function') {
      return;
    }

    Object.keys(this.filter).forEach(function (key) {
      if (blacklist.indexOf(key) === -1) {
        var val = _this.filter[key];
        var type = typeof val;
        var array = Array.isArray(val);

        if (!(array || type === 'string' || type === 'number')) {
          return;
        }

        if (key[0] === '$') {
          // Context property
          _this.context_prop_matches = _this.context_prop_matches || [];

          _this.context_prop_matches.push([key.substring(1), array ? val : [val]]);
        } else {
          // Feature property
          _this.feature_prop_matches = _this.feature_prop_matches || [];

          _this.feature_prop_matches.push([key, array ? val : [val]]);
        }

        delete _this.filter[key];
      }
    });
  };

  _proto.doPropMatches = function doPropMatches(context) {
    if (this.feature_prop_matches) {
      for (var r = 0; r < this.feature_prop_matches.length; r++) {
        var match = this.feature_prop_matches[r];
        var val = context.feature.properties[match[0]];

        if (val == null || match[1].indexOf(val) === -1) {
          return false;
        }
      }
    }

    if (this.context_prop_matches) {
      for (var _r = 0; _r < this.context_prop_matches.length; _r++) {
        var _match = this.context_prop_matches[_r];
        var _val = context[_match[0]];

        if (_val == null || _match[1].indexOf(_val) === -1) {
          return false;
        }
      }
    }

    return true;
  };

  _proto.doesMatch = function doesMatch(context) {
    if (!this.enabled) {
      return false;
    }

    if (!this.is_built) {
      this.build();
    } // zoom pre-filter: skip rest of filter if out of layer zoom range


    if (this.zooms != null && !this.zooms[context.zoom]) {
      return false;
    } // direct feature property matches


    if (!this.doPropMatches(context)) {
      return false;
    } // any remaining filter (more complex matches or dynamic function)


    var match;

    if (this.filter instanceof Function) {
      try {
        match = this.filter(context);
      } catch (error) {
        // Filter function error
        var msg = "Filter for this " + this.full_name + ": `filter: " + this.filter_original + "` ";
        msg += "failed with error '" + error.message + "', stack trace: " + error.stack;
        log('error', msg, context.feature);
      }
    } else {
      match = this.filter == null;
    }

    if (match) {
      if (this.children_to_parse) {
        parseLayerChildren(this, this.children_to_parse, this.styles);
        delete this.children_to_parse;
      }

      return true;
    }

    return false;
  };

  return Layer;
}();

Layer.id = 0;
var LayerLeaf =
/*#__PURE__*/
function (_Layer) {
  _inheritsLoose(LayerLeaf, _Layer);

  function LayerLeaf(config) {
    var _this2;

    _this2 = _Layer.call(this, config) || this;
    _this2.is_leaf = true;
    return _this2;
  }

  return LayerLeaf;
}(Layer);
var LayerTree =
/*#__PURE__*/
function (_Layer2) {
  _inheritsLoose(LayerTree, _Layer2);

  function LayerTree(config) {
    var _this3;

    _this3 = _Layer2.call(this, config) || this;
    _this3.is_tree = true;
    _this3.layers = config.layers || [];
    return _this3;
  }

  var _proto2 = LayerTree.prototype;

  _proto2.addLayer = function addLayer(layer) {
    this.layers.push(layer);
  };

  _proto2.buildDrawGroups = function buildDrawGroups(context) {
    var layers = [],
        layer_ids = [];
    matchFeature(context, [this], layers, layer_ids);

    if (layers.length > 0) {
      var cache_key = cacheKey(layer_ids); // Only evaluate each layer combination once (undefined means not yet evaluated,
      // null means evaluated with no draw object)

      if (layer_cache[cache_key] === undefined) {
        // Find all the unique visible draw blocks for this layer tree
        var draw_groups = layers.map(function (x) {
          return x && x.visible !== false && x.calculatedDraw;
        });
        var draw_keys = {};

        for (var r = 0; r < draw_groups.length; r++) {
          var stack = draw_groups[r];

          if (!stack) {
            continue;
          }

          for (var g = 0; g < stack.length; g++) {
            var group = stack[g];

            for (var key in group) {
              draw_keys[key] = true;
            }
          }
        } // Calculate each draw group


        for (var draw_key in draw_keys) {
          layer_cache[cache_key] = layer_cache[cache_key] || {};
          layer_cache[cache_key][draw_key] = mergeTrees(draw_groups, draw_key); // Only save the ones that weren't null

          if (!layer_cache[cache_key][draw_key]) {
            delete layer_cache[cache_key][draw_key];
          } else {
            layer_cache[cache_key][draw_key].key = cache_key + '/' + draw_key;
            layer_cache[cache_key][draw_key].layers = layers.map(function (x) {
              return x && x.full_name;
            });
            layer_cache[cache_key][draw_key].group = draw_key;
          }
        } // No layers evaluated


        if (layer_cache[cache_key] && Object.keys(layer_cache[cache_key]).length === 0) {
          layer_cache[cache_key] = null;
        }
      }

      return layer_cache[cache_key];
    }
  };

  return LayerTree;
}(Layer);
var FilterOptions = {
  // Handle unit conversions on filter ranges
  rangeTransform: function rangeTransform(val) {
    if (typeof val === 'string' && val.trim().slice(-3) === 'px2') {
      return parseFloat(val) + " * context.meters_per_pixel_sq";
    }

    return val;
  }
};
function isReserved(key) {
  return reserved.indexOf(key) > -1;
}

function isEmpty(obj) {
  return Object.keys(obj).length === 0;
}

function groupProps(obj) {
  var reserved = {},
      children = {};

  for (var key in obj) {
    if (isReserved(key)) {
      reserved[key] = obj[key];
    } else {
      children[key] = obj[key];
    }
  }

  return [reserved, children];
}
function calculateDraw(layer) {
  var draw = [];

  if (layer.parent) {
    var cs = layer.parent.calculatedDraw || [];
    draw.push.apply(draw, cs);
  }

  draw.push(layer.draw);
  return draw;
}
function parseLayerNode(name, layer, parent, styles) {
  layer = layer == null ? {} : layer;
  var properties = {
    name: name,
    layer: layer,
    parent: parent,
    styles: styles
  };

  var _groupProps = groupProps(layer),
      reserved = _groupProps[0],
      children = _groupProps[1];

  var empty = isEmpty(children);
  var Create;

  if (empty && parent != null) {
    Create = LayerLeaf;
  } else {
    Create = LayerTree;
  }

  var r = new Create(Object.assign(properties, reserved)); // only process child layers if this layer is enabled

  if (r.enabled) {
    if (parent) {
      parent.addLayer(r);
    }

    r.children_to_parse = empty ? null : children;
  }

  return r;
}

function parseLayerChildren(parent, children, styles) {
  for (var key in children) {
    var child = children[key];

    if (typeof child === 'object' && !Array.isArray(child)) {
      parseLayerNode(key, child, parent, styles);
    } else {
      // Invalid layer
      var msg = "Layer value must be an object: cannot create layer '" + key + ": " + JSON.stringify(child) + "'";
      msg += ", under parent layer '" + parent.full_name + "'."; // If the parent is a style name, this may be an incorrectly nested layer

      if (styles[parent.name]) {
        msg += " The parent name '" + parent.name + "' is also the name of a style, did you mean to create a 'draw' group";

        if (parent.parent) {
          msg += " under '" + parent.parent.name + "'";
        }

        msg += ' instead?';
      }

      log('warn', msg); // TODO: fire external event that clients to subscribe to
    }
  } // Sort sub-layers so they are applied deterministically when multiple layers modify the same properties
  // Sort order is: exclusive layers first, then by explicit layer priority, then by layer name


  parent.layers.sort(function (a, b) {
    // Exclusive layers come first
    // If an exclusive layer matches, no further sibling layers are matched
    if (a.exclusive < b.exclusive) return 1;else if (a.exclusive > b.exclusive) return -1; // When sub-sorting exclusive layers, sort the higher priority layers first, since only one exlcusive layer
    // can match and the first one that matches should be the highest priority.
    // When sub-sorting non-exclusive layers, sort the lower priority layers first, since multiple layers may
    // match, and when they are merged in order, the later layers will overwrite the earlier ones -- so we want
    // the higher priority ones to match last so that they "win".

    var direction = a.exclusive ? 1 : -1; // Sub-sort by explicit priority

    if (a.priority > b.priority) return direction;else if (a.priority < b.priority) return -direction; // Sub-sort by layer name as last resort

    if (a.full_name < b.full_name) return direction;else if (a.full_name > b.full_name) return -direction;
  });
}

function parseLayers(layers, styles) {
  layer_cache = {}; // clear layer cache

  var layer_trees = {};

  for (var key in layers) {
    var layer = layers[key];

    if (layer) {
      layer_trees[key] = parseLayerNode(key, layer, null, styles);
    }
  }

  return layer_trees;
}
function matchFeature(context, layers, collected_layers, collected_layers_ids) {
  var matched = false;
  var child_matched = false;

  if (layers.length === 0) {
    return;
  }

  for (var r = 0; r < layers.length; r++) {
    var current = layers[r];

    if (current.is_leaf) {
      if (current.doesMatch(context)) {
        matched = true;
        collected_layers.push(current);
        collected_layers_ids.push(current.id);

        if (current.exclusive) {
          break; // only one exclusive layer can match, stop matching further sibling layers
        }
      }
    } else if (current.is_tree) {
      if (current.doesMatch(context)) {
        matched = true;
        child_matched = matchFeature(context, current.layers, collected_layers, collected_layers_ids);

        if (!child_matched) {
          collected_layers.push(current);
          collected_layers_ids.push(current.id);
        }

        if (current.exclusive) {
          break; // only one exclusive layer can match, stop matching further sibling layers
        }
      }
    }
  }

  return matched;
}

var id$2 = 0; // unique tile id

var build_id = 0; // id tracking order in which tiles were build

var Tile =
/*#__PURE__*/
function () {
  /**
      Tile
      @constructor
      Required properties:
      coords: object with {x, y, z} properties identifying tile coordinate location
      worker: web worker to handle tile construction
  */
  function Tile(_ref) {
    var coords = _ref.coords,
        style_z = _ref.style_z,
        source = _ref.source,
        workers = _ref.workers,
        view = _ref.view;
    this.id = id$2++;
    this.view = view;
    this.source = source;
    this.generation = null;
    this.valid = true;
    this.visible = false;
    this.proxy_for = null;
    this.proxied_as = null;
    this.proxy_level = 0;
    this.proxy_order_offset = 0;
    this.fade_in = true;
    this.loading = false;
    this.loaded = false;
    this.built = false;
    this.labeled = false;
    this.error = null;
    this.debug = {};
    this.style_z = style_z; // zoom level to be used for styling

    this.coords = TileID.normalizedCoord(coords, this.source);
    this.key = TileID.key(this.coords, this.source, this.style_z);
    this.overzoom = Math.max(this.style_z - this.coords.z, 0); // number of levels of overzooming

    this.overzoom2 = Math.pow(2, this.overzoom);
    this.min = Geo$1.metersForTile(this.coords);
    this.max = Geo$1.metersForTile({
      x: this.coords.x + 1,
      y: this.coords.y + 1,
      z: this.coords.z
    }), this.span = {
      x: this.max.x - this.min.x,
      y: -(this.max.y - this.min.y)
    };
    this.bounds = {
      sw: {
        x: this.min.x,
        y: this.max.y
      },
      ne: {
        x: this.max.x,
        y: this.min.y
      }
    };
    this.meters_per_pixel = Geo$1.metersPerPixel(this.style_z);
    this.meters_per_pixel_sq = this.meters_per_pixel * this.meters_per_pixel;
    this.units_per_pixel = Geo$1.units_per_pixel / this.overzoom2; // adjusted for overzoom

    this.units_per_meter_overzoom = Geo$1.unitsPerMeter(this.coords.z) * this.overzoom2; // adjusted for overzoom

    this.preserve_tiles_within_zoom = this.source.preserve_tiles_within_zoom; // source-specific tile retention policy

    this.meshes = {}; // renderable VBO meshes keyed by style

    this.new_mesh_styles = []; // meshes that have been built so far in current build generation

    this.pending_label_meshes = null; // meshes that are pending collision (shouldn't be displayed yet)

    this.setWorker(workers);
  } // Free resources owned by tile


  var _proto = Tile.prototype;

  _proto.freeResources = function freeResources() {
    for (var m in this.meshes) {
      this.meshes[m].forEach(function (m) {
        return m.destroy();
      });
    }

    this.meshes = {};

    if (this.pending_label_meshes) {
      for (var _m in this.pending_label_meshes) {
        this.pending_label_meshes[_m].forEach(function (m) {
          return m.destroy();
        });
      }
    }

    this.pending_label_meshes = null;
  };

  _proto.destroy = function destroy() {
    Task.removeForTile(this.id);
    this.workerMessage('self.removeTile', this.key);
    this.freeResources();
    this.worker = null;
    this.valid = false;
  };

  _proto.buildAsMessage = function buildAsMessage() {
    return {
      id: this.id,
      key: this.key,
      source: this.source.name,
      coords: this.coords,
      min: this.min,
      max: this.max,
      units_per_pixel: this.units_per_pixel,
      meters_per_pixel: this.meters_per_pixel,
      meters_per_pixel_sq: this.meters_per_pixel_sq,
      units_per_meter_overzoom: this.units_per_meter_overzoom,
      style_z: this.style_z,
      overzoom: this.overzoom,
      overzoom2: this.overzoom2,
      generation: this.generation,
      debug: this.debug
    };
  } // Find the appropriate worker thread for this tile
  ;

  _proto.setWorker = function setWorker(workers) {
    if (this.source.tiled) {
      // Pin tile to a worker thread based on its coordinates
      this.worker_id = Math.abs(this.coords.x + this.coords.y + this.coords.z) % workers.length;
    } else {
      // Pin all tiles from each non-tiled source to a single worker
      // Prevents data for these sources from being loaded more than once
      this.worker_id = this.source.id % workers.length;
    }

    this.worker = workers[this.worker_id];
  };

  _proto.workerMessage = function workerMessage() {
    for (var _len = arguments.length, message = new Array(_len), _key = 0; _key < _len; _key++) {
      message[_key] = arguments[_key];
    }

    return WorkerBroker$1.postMessage.apply(WorkerBroker$1, [this.worker].concat(message));
  };

  _proto.build = function build(generation, _temp) {
    var _ref2 = _temp === void 0 ? {} : _temp,
        _ref2$fade_in = _ref2.fade_in,
        fade_in = _ref2$fade_in === void 0 ? true : _ref2$fade_in;

    this.generation = generation;
    this.fade_in = fade_in;

    if (!this.loaded) {
      this.loading = true;
      this.built = false;
      this.labeled = false;
    }

    return this.workerMessage('self.buildTile', {
      tile: this.buildAsMessage()
    }).catch(function (e) {
      throw e;
    });
  }
  /**
      Called on worker to cancel loading
      Static method because the worker only has object representations of tile data, there is no
      tile instance created yet.
  */
  ;

  Tile.cancel = function cancel(tile) {
    if (tile) {
      tile.canceled = true;

      if (tile.source_data && tile.source_data.request_id) {
        Utils.cancelRequest(tile.source_data.request_id); // cancel pending tile network request

        tile.source_data.request_id = null;
      }

      Tile.abortBuild(tile);
    }
  } // Process geometry for tile - called by web worker
  // Returns a set of tile keys that should be sent to the main thread (so that we can minimize data exchange between worker and main thread)
  ;

  Tile.buildGeometry = function buildGeometry(tile, _ref3) {
    var scene_id = _ref3.scene_id,
        layers = _ref3.layers,
        styles = _ref3.styles,
        global = _ref3.global;
    var data = tile.source_data;
    tile.debug.building = +new Date();
    tile.debug.feature_count = 0;
    tile.debug.layers = null;
    Collision.startTile(tile.id, {
      apply_repeat_groups: true
    }); // Process each top-level layer

    for (var layer_name in layers) {
      var layer = layers[layer_name]; // Skip layers with no data source defined

      if (!layer || !layer.config_data) {
        log('warn', "Layer " + layer_name + " was defined without a geometry data source and will not be rendered.");
        continue;
      } // Source names don't match


      if (layer.config_data.source !== tile.source) {
        continue;
      } // Get data for one or more layers from source


      var source_layers = Tile.getDataForSource(data, layer.config_data, layer_name); // Build features in layer

      for (var s = 0; s < source_layers.length; s++) {
        var source_layer = source_layers[s];
        var geom = source_layer.geom;

        if (!geom) {
          continue;
        }

        for (var f = 0; f < geom.features.length; f++) {
          var feature = geom.features[f];

          if (feature.geometry == null) {
            continue; // skip features w/o geometry (valid GeoJSON)
          }

          var context = StyleParser.getFeatureParseContext(feature, tile, global);
          context.winding = tile.default_winding;
          context.source = tile.source; // add data source name

          context.layer = source_layer.layer; // add data source layer name
          // Get draw groups for this feature

          var draw_groups = layer.buildDrawGroups(context, true);

          if (!draw_groups) {
            continue;
          } // Build draw groups


          for (var group_name in draw_groups) {
            var group = draw_groups[group_name]; // Add to style

            var style_name = group.style || group_name;
            var style = styles[style_name];

            if (!style) {
              log('warn', "Style '" + style_name + "' not found, skipping layer '" + layer_name + "':", group, feature);
              continue;
            }

            group = style.preprocess(group);

            if (group == null || group.visible === false) {
              continue;
            }

            context.layers = group.layers; // add matching draw layers

            style.addFeature(feature, group, context);
          }

          tile.debug.feature_count++;
        }
      }
    }

    tile.debug.building = +new Date() - tile.debug.building; // Send styles back to main thread as they finish building, in two groups: collision vs. non-collision

    var tile_styles = this.stylesForTile(tile, styles).map(function (s) {
      return styles[s];
    });
    Tile.buildStyleGroups(tile, tile_styles, scene_id, function (style) {
      return style.collision ? 'collision' : 'non-collision';
    }); // Tile.buildStyleGroups(tile, tile_styles, scene_id, style => style.name); // call for each style
    // Tile.buildStyleGroups(tile, tile_styles, scene_id, style => 'styles'); // all styles in single call (previous behavior)
  };

  Tile.stylesForTile = function stylesForTile(tile, styles) {
    var tile_styles = [];

    for (var s in styles) {
      if (styles[s].hasDataForTile(tile)) {
        tile_styles.push(s);
      }
    }

    return tile_styles;
  } // Build styles (grouped by the provided function) and send back to main thread as they finish building
  ;

  Tile.buildStyleGroups = function buildStyleGroups(tile, styles, scene_id, group_by) {
    // Group the styles; each group will be sent to the main thread when the styles in the group finish building.
    var groups = styles.reduce(function (groups, style) {
      var group = group_by(style);
      groups[group] = groups[group] || [];
      groups[group].push(style);
      return groups;
    }, {}); // If nothing to build, return empty tile to main thread

    if (Object.keys(groups).length === 0) {
      WorkerBroker$1.postMessage("TileManager_" + scene_id + ".buildTileStylesCompleted", WorkerBroker$1.withTransferables({
        tile: Tile.slice(tile),
        progress: {
          start: true,
          done: true
        }
      }));
      Collision.resetTile(tile.id); // clear collision if we're done with the tile

      return;
    } // Build each group of styles


    var progress = {};

    for (var group_name in groups) {
      Tile.buildStyleGroup({
        group_name: group_name,
        groups: groups,
        tile: tile,
        progress: progress,
        scene_id: scene_id
      });
    }
  } // Build a single group of styles
  ;

  Tile.buildStyleGroup = function buildStyleGroup(_ref4) {
    return new Promise(function ($return, $error) {
      var group_name, groups, tile, progress, scene_id, group, mesh_data;
      group_name = _ref4.group_name, groups = _ref4.groups, tile = _ref4.tile, progress = _ref4.progress, scene_id = _ref4.scene_id;
      group = groups[group_name];
      mesh_data = {};

      var $Try_1_Post = function () {
        try {
          return $return();
        } catch ($boundEx) {
          return $error($boundEx);
        }
      };

      var $Try_1_Catch = function (e) {
        try {
          log('error', "Error for style group '" + group_name + "' for tile " + tile.key, e && e.stack || e);
          return $Try_1_Post();
        } catch ($boundEx) {
          return $error($boundEx);
        }
      };

      try {
        return Promise.resolve(Promise.all(group.map(function (style) {
          return new Promise(function ($return, $error) {
            var style_data;
            return Promise.resolve(style.endData(tile)).then(function ($await_2) {
              try {
                style_data = $await_2;

                if (style_data) {
                  mesh_data[style.name] = style_data;
                }

                return $return();
              } catch ($boundEx) {
                return $error($boundEx);
              }
            }, $error);
          });
        }))).then(function ($await_3) {
          try {
            // Mark the group as done, and check if all groups have finished
            log('trace', "Finished style group '" + group_name + "' for tile " + tile.key);
            groups[group_name] = null;

            if (Object.keys(groups).every(function (g) {
              return groups[g] == null;
            })) {
              progress.done = true;
            } // Send meshes to main thread


            WorkerBroker$1.postMessage("TileManager_" + scene_id + ".buildTileStylesCompleted", WorkerBroker$1.withTransferables({
              tile: Object.assign({}, Tile.slice(tile), {
                mesh_data: mesh_data
              }),
              progress: progress
            }));

            if (progress.done) {
              Collision.resetTile(tile.id); // clear collision if we're done with the tile
            }

            return $Try_1_Post();
          } catch ($boundEx) {
            return $Try_1_Catch($boundEx);
          }
        }, $Try_1_Catch);
      } catch (e) {
        $Try_1_Catch(e);
      }
    });
  }
  /**
      Retrieves geometry from a tile according to a data source definition
      Returns an array of objects with:
          layer: source layer name
          geom: GeoJSON FeatureCollection
  */
  ;

  Tile.getDataForSource = function getDataForSource(source_data, source_config, scene_layer) {
    var layers = [];

    if (source_config != null && source_data != null && source_data.layers != null) {
      // If no layer specified, and a default source layer exists
      if (!source_config.layer && source_data.layers._default) {
        layers.push({
          geom: source_data.layers._default
        });
      } // If no layer specified, and a layer for the scene layer name exists
      else if (!source_config.layer && scene_layer) {
          layers.push({
            layer: scene_layer,
            geom: source_data.layers[scene_layer]
          });
        } // If a layer is specified by name, use it
        else if (typeof source_config.layer === 'string') {
            layers.push({
              layer: source_config.layer,
              geom: source_data.layers[source_config.layer]
            });
          } // If multiple layers are specified by name, combine them
          else if (Array.isArray(source_config.layer)) {
              source_config.layer.forEach(function (layer) {
                if (source_data.layers[layer] && source_data.layers[layer].features) {
                  layers.push({
                    layer: layer,
                    geom: source_data.layers[layer]
                  });
                }
              });
            }
    }

    return layers;
  }
  /**
     Called on main thread when a web worker completes processing
     for a single tile.
  */
  ;

  _proto.buildMeshes = function buildMeshes(styles, progress) {
    if (this.error) {
      return;
    }

    this.build_id = build_id++; // record order in which tile was built
    // Debug

    if (progress.start) {
      this.debug.geometry_count = 0;
      this.debug.buffer_size = 0;
    } // Create VBOs


    var meshes = {}; // new data to be added to tile

    var mesh_data = this.mesh_data;

    if (mesh_data) {
      for (var s in mesh_data) {
        for (var variant in mesh_data[s].meshes) {
          var mesh_variant = mesh_data[s].meshes[variant];

          if (mesh_variant.vertex_data) {
            if (!styles[s]) {
              log('warn', "Could not create mesh because style '" + s + "' not found, for tile " + this.key + ", aborting tile");
              break;
            } // first add style-level uniforms, then add any mesh-specific ones


            var mesh_options = Object.assign({}, mesh_data[s]);
            mesh_options.uniforms = Object.assign({}, mesh_options.uniforms, mesh_variant.uniforms);
            mesh_options.variant = mesh_variant.variant; // for labels, keep buffer data on CPU so they can be modified later

            if (mesh_variant.labels) {
              mesh_options.retain = true;
            }

            var mesh = styles[s].makeMesh(mesh_variant.vertex_data, mesh_variant.vertex_elements, mesh_options);
            mesh.variant = mesh_options.variant;
            mesh.labels = mesh_variant.labels;
            meshes[s] = meshes[s] || [];
            meshes[s].push(mesh);

            if (mesh.variant.mesh_order == null) {
              mesh.variant.mesh_order = meshes[s].length - 1; // assign default variant render order
            }

            this.debug.buffer_size += mesh.buffer_size;
            this.debug.geometry_count += mesh.geometry_count;
          }
        } // Sort mesh variants by explicit render order (if present)


        if (meshes[s]) {
          meshes[s].sort(function (a, b) {
            // Sort variant order ascending if present, then all null values (where order is unspecified)
            var ao = a.variant.mesh_order,
                bo = b.variant.mesh_order;
            return ao == null ? 1 : bo == null ? -1 : ao < bo ? -1 : 1;
          });
        }
      }
    }

    delete this.mesh_data; // New meshes

    for (var m in meshes) {
      // swap in non-collision meshes right away
      if (!styles[m].collision) {
        if (this.meshes[m]) {
          this.meshes[m].forEach(function (m) {
            return m.destroy();
          }); // free old meshes
        }

        this.meshes[m] = meshes[m]; // set new mesh

        this.new_mesh_styles.push(m);
      } // keep label meshes out of view until collision is complete
      else {
          this.pending_label_meshes = this.pending_label_meshes || {};
          this.pending_label_meshes[m] = meshes[m];
        }
    }

    if (progress.done) {
      // Release un-replaced meshes (existing in previous generation, but weren't built for this one)
      for (var _m2 in this.meshes) {
        if (this.new_mesh_styles.indexOf(_m2) === -1 && (!this.pending_label_meshes || this.pending_label_meshes[_m2] == null)) {
          this.meshes[_m2].forEach(function (m) {
            return m.destroy();
          });

          delete this.meshes[_m2];
        }
      }

      this.new_mesh_styles = [];
      this.debug.geometry_ratio = (this.debug.geometry_count / this.debug.feature_count).toFixed(1);
    }

    this.printDebug(progress);
  } // How many styles are currently pending label collision
  ;

  _proto.pendingLabelStyleCount = function pendingLabelStyleCount() {
    return this.pending_label_meshes ? Object.keys(this.pending_label_meshes).length : 0;
  } // Swap label style meshes after collision is complete
  ;

  _proto.swapPendingLabels = function swapPendingLabels() {
    this.labeled = true; // mark as labeled

    if (this.pending_label_meshes) {
      for (var m in this.pending_label_meshes) {
        if (this.meshes[m]) {
          this.meshes[m].forEach(function (m) {
            return m.destroy();
          }); // free old meshes
        }

        this.meshes[m] = this.pending_label_meshes[m]; // set new mesh
      }

      this.pending_label_meshes = null;
    }
  }
  /**
      Called on main thread when web worker completes processing, but tile has since been discarded
      Frees resources that would have been transferred to the tile object.
      Static method because the tile object no longer exists (the tile data returned by the worker is passed instead).
  */
  ;

  Tile.abortBuild = function abortBuild(tile) {
    Task.removeForTile(tile.id);
    Collision.abortTile(tile.id); // Releases meshes

    if (tile.mesh_data) {
      for (var s in tile.mesh_data) {
        var textures = tile.mesh_data[s].textures;

        if (textures) {
          textures.forEach(function (t) {
            var texture = Texture.textures[t];

            if (texture) {
              log('trace', "releasing texture " + t + " for tile " + tile.key);
              texture.release();
            }
          });
        }
      }
    }
  } // Set as a proxy tile for another tile
  ;

  _proto.setProxyFor = function setProxyFor(tile) {
    if (tile) {
      this.visible = true;
      this.proxy_for = this.proxy_for || [];
      this.proxy_for.push(tile);
      this.proxy_order_offset = 1; // draw proxies a half-layer back (order is scaled 2x to avoid integer truncation)

      tile.proxied_as = tile.style_z > this.style_z ? 'child' : 'parent';
      this.proxy_level = Math.abs(tile.style_z - this.style_z); // # of zoom levels proxy is above/below target tile
    } else {
      this.proxy_for = null;
      this.proxy_order_offset = 0;
      this.proxy_level = 0;
    }
  };

  _proto.isProxy = function isProxy() {
    return this.proxy_for != null;
  } // Proxy tiles only need to render a specific style if any of the tiles they are proxying *for*
  // haven't finished loading that style yet. If all proxied tiles *have* data for that style, then it's
  // safe to hide the proxy tile's version.
  ;

  _proto.shouldProxyForStyle = function shouldProxyForStyle(style) {
    return !this.proxy_for || this.proxy_for.some(function (t) {
      return t.meshes[style] == null;
    });
  } // Update model matrix and tile uniforms
  ;

  _proto.setupProgram = function setupProgram(_ref5, program) {
    var model = _ref5.model,
        model32 = _ref5.model32;
    // Tile origin
    program.uniform('4fv', 'u_tile_origin', [this.min.x, this.min.y, this.style_z, this.coords.z]);
    program.uniform('1f', 'u_tile_proxy_order_offset', this.proxy_order_offset); // Model - transform tile space into world space (meters, absolute mercator position)

    mat4.identity(model);
    mat4.translate(model, model, vec3.fromValues(this.min.x, this.min.y, 0));
    mat4.scale(model, model, vec3.fromValues(this.span.x / Geo$1.tile_scale, this.span.y / Geo$1.tile_scale, 1)); // scale tile local coords to meters

    mat4.copy(model32, model);
    program.uniform('Matrix4fv', 'u_model', model32); // Fade in labels according to proxy status, avoiding "flickering" where
    // labels quickly go from invisible back to visible

    program.uniform('1i', 'u_tile_fade_in', this.fade_in && this.proxied_as !== 'child');
  } // Slice a subset of keys out of a tile
  // Includes a minimum set of pre-defined keys for load state, debug. etc.
  // We use this to send a subset of the tile back to the main thread, to minimize unnecessary data transfer
  // (e.g. very large items like feature geometry are not needed on the main thread)
  ;

  Tile.slice = function slice(tile, keys) {
    var keep = ['id', 'key', 'loading', 'loaded', 'generation', 'error', 'debug'];

    if (Array.isArray(keys)) {
      keep.push.apply(keep, keys);
    } // Build the tile subset


    var tile_subset = {};

    for (var k = 0; k < keep.length; k++) {
      var key = keep[k];
      tile_subset[key] = tile[key];
    }

    return tile_subset;
  };

  _proto.merge = function merge(other) {
    this.loading = other.loading;
    this.loaded = other.loaded;
    this.generation = other.generation;
    this.error = other.error;
    this.mesh_data = other.mesh_data;
    this.debug = mergeObjects(this.debug, other.debug);
    return this;
  };

  _proto.printDebug = function printDebug(progress) {
    var exclude = ['layers'];
    var copy = {};

    for (var key in this.debug) {
      if (exclude.indexOf(key) === -1) {
        copy[key] = this.debug[key];
      }
    }

    log('debug', "Tile " + (progress.done ? '(done)' : '') + ": debug for " + this.key + ": [  " + JSON.stringify(copy) + " ]");
  };

  return Tile;
}();
Tile.coord_children = {}; // only allocate children coordinates once per coordinate
// Sum up layer feature/geometry stats from a set of tiles

function debugSumLayerStats(tiles) {
  var list = {},
      tree = {};
  tiles.filter(function (tile) {
    return tile.debug.layers;
  }).forEach(function (tile) {
    // layer list
    Object.keys(tile.debug.layers.list).forEach(function (layer) {
      var counts = tile.debug.layers.list[layer];
      addLayerDebugEntry(list, layer, counts.features, counts.geoms, counts.styles, counts.base);
    }); // layer tree

    addDebugLayers(tile.debug.layers.tree, tree);
  });
  return {
    list: list,
    tree: tree
  };
} // build debug stats layer tree

function addDebugLayers(node, tree) {
  for (var layer in node) {
    var counts = node[layer];
    addLayerDebugEntry(tree, layer, counts.features, counts.geoms, counts.styles, counts.base);

    if (counts.layers) {
      tree[layer].layers = tree[layer].layers || {};
      addDebugLayers(counts.layers, tree[layer].layers); // process child layers
    }
  }
}

var read = function (buffer, offset, isLE, mLen, nBytes) {
  var e, m;
  var eLen = nBytes * 8 - mLen - 1;
  var eMax = (1 << eLen) - 1;
  var eBias = eMax >> 1;
  var nBits = -7;
  var i = isLE ? (nBytes - 1) : 0;
  var d = isLE ? -1 : 1;
  var s = buffer[offset + i];

  i += d;

  e = s & ((1 << (-nBits)) - 1);
  s >>= (-nBits);
  nBits += eLen;
  for (; nBits > 0; e = e * 256 + buffer[offset + i], i += d, nBits -= 8) {}

  m = e & ((1 << (-nBits)) - 1);
  e >>= (-nBits);
  nBits += mLen;
  for (; nBits > 0; m = m * 256 + buffer[offset + i], i += d, nBits -= 8) {}

  if (e === 0) {
    e = 1 - eBias;
  } else if (e === eMax) {
    return m ? NaN : ((s ? -1 : 1) * Infinity)
  } else {
    m = m + Math.pow(2, mLen);
    e = e - eBias;
  }
  return (s ? -1 : 1) * m * Math.pow(2, e - mLen)
};

var write = function (buffer, value, offset, isLE, mLen, nBytes) {
  var e, m, c;
  var eLen = nBytes * 8 - mLen - 1;
  var eMax = (1 << eLen) - 1;
  var eBias = eMax >> 1;
  var rt = (mLen === 23 ? Math.pow(2, -24) - Math.pow(2, -77) : 0);
  var i = isLE ? 0 : (nBytes - 1);
  var d = isLE ? 1 : -1;
  var s = value < 0 || (value === 0 && 1 / value < 0) ? 1 : 0;

  value = Math.abs(value);

  if (isNaN(value) || value === Infinity) {
    m = isNaN(value) ? 1 : 0;
    e = eMax;
  } else {
    e = Math.floor(Math.log(value) / Math.LN2);
    if (value * (c = Math.pow(2, -e)) < 1) {
      e--;
      c *= 2;
    }
    if (e + eBias >= 1) {
      value += rt / c;
    } else {
      value += rt * Math.pow(2, 1 - eBias);
    }
    if (value * c >= 2) {
      e++;
      c /= 2;
    }

    if (e + eBias >= eMax) {
      m = 0;
      e = eMax;
    } else if (e + eBias >= 1) {
      m = (value * c - 1) * Math.pow(2, mLen);
      e = e + eBias;
    } else {
      m = value * Math.pow(2, eBias - 1) * Math.pow(2, mLen);
      e = 0;
    }
  }

  for (; mLen >= 8; buffer[offset + i] = m & 0xff, i += d, m /= 256, mLen -= 8) {}

  e = (e << mLen) | m;
  eLen += mLen;
  for (; eLen > 0; buffer[offset + i] = e & 0xff, i += d, e /= 256, eLen -= 8) {}

  buffer[offset + i - d] |= s * 128;
};

var ieee754 = {
	read: read,
	write: write
};

var pbf = Pbf;



function Pbf(buf) {
    this.buf = ArrayBuffer.isView && ArrayBuffer.isView(buf) ? buf : new Uint8Array(buf || 0);
    this.pos = 0;
    this.type = 0;
    this.length = this.buf.length;
}

Pbf.Varint  = 0; // varint: int32, int64, uint32, uint64, sint32, sint64, bool, enum
Pbf.Fixed64 = 1; // 64-bit: double, fixed64, sfixed64
Pbf.Bytes   = 2; // length-delimited: string, bytes, embedded messages, packed repeated fields
Pbf.Fixed32 = 5; // 32-bit: float, fixed32, sfixed32

var SHIFT_LEFT_32 = (1 << 16) * (1 << 16),
    SHIFT_RIGHT_32 = 1 / SHIFT_LEFT_32;

Pbf.prototype = {

    destroy: function() {
        this.buf = null;
    },

    // === READING =================================================================

    readFields: function(readField, result, end) {
        end = end || this.length;

        while (this.pos < end) {
            var val = this.readVarint(),
                tag = val >> 3,
                startPos = this.pos;

            this.type = val & 0x7;
            readField(tag, result, this);

            if (this.pos === startPos) this.skip(val);
        }
        return result;
    },

    readMessage: function(readField, result) {
        return this.readFields(readField, result, this.readVarint() + this.pos);
    },

    readFixed32: function() {
        var val = readUInt32(this.buf, this.pos);
        this.pos += 4;
        return val;
    },

    readSFixed32: function() {
        var val = readInt32(this.buf, this.pos);
        this.pos += 4;
        return val;
    },

    // 64-bit int handling is based on github.com/dpw/node-buffer-more-ints (MIT-licensed)

    readFixed64: function() {
        var val = readUInt32(this.buf, this.pos) + readUInt32(this.buf, this.pos + 4) * SHIFT_LEFT_32;
        this.pos += 8;
        return val;
    },

    readSFixed64: function() {
        var val = readUInt32(this.buf, this.pos) + readInt32(this.buf, this.pos + 4) * SHIFT_LEFT_32;
        this.pos += 8;
        return val;
    },

    readFloat: function() {
        var val = ieee754.read(this.buf, this.pos, true, 23, 4);
        this.pos += 4;
        return val;
    },

    readDouble: function() {
        var val = ieee754.read(this.buf, this.pos, true, 52, 8);
        this.pos += 8;
        return val;
    },

    readVarint: function(isSigned) {
        var buf = this.buf,
            val, b;

        b = buf[this.pos++]; val  =  b & 0x7f;        if (b < 0x80) return val;
        b = buf[this.pos++]; val |= (b & 0x7f) << 7;  if (b < 0x80) return val;
        b = buf[this.pos++]; val |= (b & 0x7f) << 14; if (b < 0x80) return val;
        b = buf[this.pos++]; val |= (b & 0x7f) << 21; if (b < 0x80) return val;
        b = buf[this.pos];   val |= (b & 0x0f) << 28;

        return readVarintRemainder(val, isSigned, this);
    },

    readVarint64: function() { // for compatibility with v2.0.1
        return this.readVarint(true);
    },

    readSVarint: function() {
        var num = this.readVarint();
        return num % 2 === 1 ? (num + 1) / -2 : num / 2; // zigzag encoding
    },

    readBoolean: function() {
        return Boolean(this.readVarint());
    },

    readString: function() {
        var end = this.readVarint() + this.pos,
            str = readUtf8(this.buf, this.pos, end);
        this.pos = end;
        return str;
    },

    readBytes: function() {
        var end = this.readVarint() + this.pos,
            buffer = this.buf.subarray(this.pos, end);
        this.pos = end;
        return buffer;
    },

    // verbose for performance reasons; doesn't affect gzipped size

    readPackedVarint: function(arr, isSigned) {
        var end = readPackedEnd(this);
        arr = arr || [];
        while (this.pos < end) arr.push(this.readVarint(isSigned));
        return arr;
    },
    readPackedSVarint: function(arr) {
        var end = readPackedEnd(this);
        arr = arr || [];
        while (this.pos < end) arr.push(this.readSVarint());
        return arr;
    },
    readPackedBoolean: function(arr) {
        var end = readPackedEnd(this);
        arr = arr || [];
        while (this.pos < end) arr.push(this.readBoolean());
        return arr;
    },
    readPackedFloat: function(arr) {
        var end = readPackedEnd(this);
        arr = arr || [];
        while (this.pos < end) arr.push(this.readFloat());
        return arr;
    },
    readPackedDouble: function(arr) {
        var end = readPackedEnd(this);
        arr = arr || [];
        while (this.pos < end) arr.push(this.readDouble());
        return arr;
    },
    readPackedFixed32: function(arr) {
        var end = readPackedEnd(this);
        arr = arr || [];
        while (this.pos < end) arr.push(this.readFixed32());
        return arr;
    },
    readPackedSFixed32: function(arr) {
        var end = readPackedEnd(this);
        arr = arr || [];
        while (this.pos < end) arr.push(this.readSFixed32());
        return arr;
    },
    readPackedFixed64: function(arr) {
        var end = readPackedEnd(this);
        arr = arr || [];
        while (this.pos < end) arr.push(this.readFixed64());
        return arr;
    },
    readPackedSFixed64: function(arr) {
        var end = readPackedEnd(this);
        arr = arr || [];
        while (this.pos < end) arr.push(this.readSFixed64());
        return arr;
    },

    skip: function(val) {
        var type = val & 0x7;
        if (type === Pbf.Varint) while (this.buf[this.pos++] > 0x7f) {}
        else if (type === Pbf.Bytes) this.pos = this.readVarint() + this.pos;
        else if (type === Pbf.Fixed32) this.pos += 4;
        else if (type === Pbf.Fixed64) this.pos += 8;
        else throw new Error('Unimplemented type: ' + type);
    },

    // === WRITING =================================================================

    writeTag: function(tag, type) {
        this.writeVarint((tag << 3) | type);
    },

    realloc: function(min) {
        var length = this.length || 16;

        while (length < this.pos + min) length *= 2;

        if (length !== this.length) {
            var buf = new Uint8Array(length);
            buf.set(this.buf);
            this.buf = buf;
            this.length = length;
        }
    },

    finish: function() {
        this.length = this.pos;
        this.pos = 0;
        return this.buf.subarray(0, this.length);
    },

    writeFixed32: function(val) {
        this.realloc(4);
        writeInt32(this.buf, val, this.pos);
        this.pos += 4;
    },

    writeSFixed32: function(val) {
        this.realloc(4);
        writeInt32(this.buf, val, this.pos);
        this.pos += 4;
    },

    writeFixed64: function(val) {
        this.realloc(8);
        writeInt32(this.buf, val & -1, this.pos);
        writeInt32(this.buf, Math.floor(val * SHIFT_RIGHT_32), this.pos + 4);
        this.pos += 8;
    },

    writeSFixed64: function(val) {
        this.realloc(8);
        writeInt32(this.buf, val & -1, this.pos);
        writeInt32(this.buf, Math.floor(val * SHIFT_RIGHT_32), this.pos + 4);
        this.pos += 8;
    },

    writeVarint: function(val) {
        val = +val || 0;

        if (val > 0xfffffff || val < 0) {
            writeBigVarint(val, this);
            return;
        }

        this.realloc(4);

        this.buf[this.pos++] =           val & 0x7f  | (val > 0x7f ? 0x80 : 0); if (val <= 0x7f) return;
        this.buf[this.pos++] = ((val >>>= 7) & 0x7f) | (val > 0x7f ? 0x80 : 0); if (val <= 0x7f) return;
        this.buf[this.pos++] = ((val >>>= 7) & 0x7f) | (val > 0x7f ? 0x80 : 0); if (val <= 0x7f) return;
        this.buf[this.pos++] =   (val >>> 7) & 0x7f;
    },

    writeSVarint: function(val) {
        this.writeVarint(val < 0 ? -val * 2 - 1 : val * 2);
    },

    writeBoolean: function(val) {
        this.writeVarint(Boolean(val));
    },

    writeString: function(str) {
        str = String(str);
        this.realloc(str.length * 4);

        this.pos++; // reserve 1 byte for short string length

        var startPos = this.pos;
        // write the string directly to the buffer and see how much was written
        this.pos = writeUtf8(this.buf, str, this.pos);
        var len = this.pos - startPos;

        if (len >= 0x80) makeRoomForExtraLength(startPos, len, this);

        // finally, write the message length in the reserved place and restore the position
        this.pos = startPos - 1;
        this.writeVarint(len);
        this.pos += len;
    },

    writeFloat: function(val) {
        this.realloc(4);
        ieee754.write(this.buf, val, this.pos, true, 23, 4);
        this.pos += 4;
    },

    writeDouble: function(val) {
        this.realloc(8);
        ieee754.write(this.buf, val, this.pos, true, 52, 8);
        this.pos += 8;
    },

    writeBytes: function(buffer) {
        var len = buffer.length;
        this.writeVarint(len);
        this.realloc(len);
        for (var i = 0; i < len; i++) this.buf[this.pos++] = buffer[i];
    },

    writeRawMessage: function(fn, obj) {
        this.pos++; // reserve 1 byte for short message length

        // write the message directly to the buffer and see how much was written
        var startPos = this.pos;
        fn(obj, this);
        var len = this.pos - startPos;

        if (len >= 0x80) makeRoomForExtraLength(startPos, len, this);

        // finally, write the message length in the reserved place and restore the position
        this.pos = startPos - 1;
        this.writeVarint(len);
        this.pos += len;
    },

    writeMessage: function(tag, fn, obj) {
        this.writeTag(tag, Pbf.Bytes);
        this.writeRawMessage(fn, obj);
    },

    writePackedVarint:   function(tag, arr) { this.writeMessage(tag, writePackedVarint, arr);   },
    writePackedSVarint:  function(tag, arr) { this.writeMessage(tag, writePackedSVarint, arr);  },
    writePackedBoolean:  function(tag, arr) { this.writeMessage(tag, writePackedBoolean, arr);  },
    writePackedFloat:    function(tag, arr) { this.writeMessage(tag, writePackedFloat, arr);    },
    writePackedDouble:   function(tag, arr) { this.writeMessage(tag, writePackedDouble, arr);   },
    writePackedFixed32:  function(tag, arr) { this.writeMessage(tag, writePackedFixed32, arr);  },
    writePackedSFixed32: function(tag, arr) { this.writeMessage(tag, writePackedSFixed32, arr); },
    writePackedFixed64:  function(tag, arr) { this.writeMessage(tag, writePackedFixed64, arr);  },
    writePackedSFixed64: function(tag, arr) { this.writeMessage(tag, writePackedSFixed64, arr); },

    writeBytesField: function(tag, buffer) {
        this.writeTag(tag, Pbf.Bytes);
        this.writeBytes(buffer);
    },
    writeFixed32Field: function(tag, val) {
        this.writeTag(tag, Pbf.Fixed32);
        this.writeFixed32(val);
    },
    writeSFixed32Field: function(tag, val) {
        this.writeTag(tag, Pbf.Fixed32);
        this.writeSFixed32(val);
    },
    writeFixed64Field: function(tag, val) {
        this.writeTag(tag, Pbf.Fixed64);
        this.writeFixed64(val);
    },
    writeSFixed64Field: function(tag, val) {
        this.writeTag(tag, Pbf.Fixed64);
        this.writeSFixed64(val);
    },
    writeVarintField: function(tag, val) {
        this.writeTag(tag, Pbf.Varint);
        this.writeVarint(val);
    },
    writeSVarintField: function(tag, val) {
        this.writeTag(tag, Pbf.Varint);
        this.writeSVarint(val);
    },
    writeStringField: function(tag, str) {
        this.writeTag(tag, Pbf.Bytes);
        this.writeString(str);
    },
    writeFloatField: function(tag, val) {
        this.writeTag(tag, Pbf.Fixed32);
        this.writeFloat(val);
    },
    writeDoubleField: function(tag, val) {
        this.writeTag(tag, Pbf.Fixed64);
        this.writeDouble(val);
    },
    writeBooleanField: function(tag, val) {
        this.writeVarintField(tag, Boolean(val));
    }
};

function readVarintRemainder(l, s, p) {
    var buf = p.buf,
        h, b;

    b = buf[p.pos++]; h  = (b & 0x70) >> 4;  if (b < 0x80) return toNum(l, h, s);
    b = buf[p.pos++]; h |= (b & 0x7f) << 3;  if (b < 0x80) return toNum(l, h, s);
    b = buf[p.pos++]; h |= (b & 0x7f) << 10; if (b < 0x80) return toNum(l, h, s);
    b = buf[p.pos++]; h |= (b & 0x7f) << 17; if (b < 0x80) return toNum(l, h, s);
    b = buf[p.pos++]; h |= (b & 0x7f) << 24; if (b < 0x80) return toNum(l, h, s);
    b = buf[p.pos++]; h |= (b & 0x01) << 31; if (b < 0x80) return toNum(l, h, s);

    throw new Error('Expected varint not more than 10 bytes');
}

function readPackedEnd(pbf) {
    return pbf.type === Pbf.Bytes ?
        pbf.readVarint() + pbf.pos : pbf.pos + 1;
}

function toNum(low, high, isSigned) {
    if (isSigned) {
        return high * 0x100000000 + (low >>> 0);
    }

    return ((high >>> 0) * 0x100000000) + (low >>> 0);
}

function writeBigVarint(val, pbf) {
    var low, high;

    if (val >= 0) {
        low  = (val % 0x100000000) | 0;
        high = (val / 0x100000000) | 0;
    } else {
        low  = ~(-val % 0x100000000);
        high = ~(-val / 0x100000000);

        if (low ^ 0xffffffff) {
            low = (low + 1) | 0;
        } else {
            low = 0;
            high = (high + 1) | 0;
        }
    }

    if (val >= 0x10000000000000000 || val < -0x10000000000000000) {
        throw new Error('Given varint doesn\'t fit into 10 bytes');
    }

    pbf.realloc(10);

    writeBigVarintLow(low, high, pbf);
    writeBigVarintHigh(high, pbf);
}

function writeBigVarintLow(low, high, pbf) {
    pbf.buf[pbf.pos++] = low & 0x7f | 0x80; low >>>= 7;
    pbf.buf[pbf.pos++] = low & 0x7f | 0x80; low >>>= 7;
    pbf.buf[pbf.pos++] = low & 0x7f | 0x80; low >>>= 7;
    pbf.buf[pbf.pos++] = low & 0x7f | 0x80; low >>>= 7;
    pbf.buf[pbf.pos]   = low & 0x7f;
}

function writeBigVarintHigh(high, pbf) {
    var lsb = (high & 0x07) << 4;

    pbf.buf[pbf.pos++] |= lsb         | ((high >>>= 3) ? 0x80 : 0); if (!high) return;
    pbf.buf[pbf.pos++]  = high & 0x7f | ((high >>>= 7) ? 0x80 : 0); if (!high) return;
    pbf.buf[pbf.pos++]  = high & 0x7f | ((high >>>= 7) ? 0x80 : 0); if (!high) return;
    pbf.buf[pbf.pos++]  = high & 0x7f | ((high >>>= 7) ? 0x80 : 0); if (!high) return;
    pbf.buf[pbf.pos++]  = high & 0x7f | ((high >>>= 7) ? 0x80 : 0); if (!high) return;
    pbf.buf[pbf.pos++]  = high & 0x7f;
}

function makeRoomForExtraLength(startPos, len, pbf) {
    var extraLen =
        len <= 0x3fff ? 1 :
        len <= 0x1fffff ? 2 :
        len <= 0xfffffff ? 3 : Math.ceil(Math.log(len) / (Math.LN2 * 7));

    // if 1 byte isn't enough for encoding message length, shift the data to the right
    pbf.realloc(extraLen);
    for (var i = pbf.pos - 1; i >= startPos; i--) pbf.buf[i + extraLen] = pbf.buf[i];
}

function writePackedVarint(arr, pbf)   { for (var i = 0; i < arr.length; i++) pbf.writeVarint(arr[i]);   }
function writePackedSVarint(arr, pbf)  { for (var i = 0; i < arr.length; i++) pbf.writeSVarint(arr[i]);  }
function writePackedFloat(arr, pbf)    { for (var i = 0; i < arr.length; i++) pbf.writeFloat(arr[i]);    }
function writePackedDouble(arr, pbf)   { for (var i = 0; i < arr.length; i++) pbf.writeDouble(arr[i]);   }
function writePackedBoolean(arr, pbf)  { for (var i = 0; i < arr.length; i++) pbf.writeBoolean(arr[i]);  }
function writePackedFixed32(arr, pbf)  { for (var i = 0; i < arr.length; i++) pbf.writeFixed32(arr[i]);  }
function writePackedSFixed32(arr, pbf) { for (var i = 0; i < arr.length; i++) pbf.writeSFixed32(arr[i]); }
function writePackedFixed64(arr, pbf)  { for (var i = 0; i < arr.length; i++) pbf.writeFixed64(arr[i]);  }
function writePackedSFixed64(arr, pbf) { for (var i = 0; i < arr.length; i++) pbf.writeSFixed64(arr[i]); }

// Buffer code below from https://github.com/feross/buffer, MIT-licensed

function readUInt32(buf, pos) {
    return ((buf[pos]) |
        (buf[pos + 1] << 8) |
        (buf[pos + 2] << 16)) +
        (buf[pos + 3] * 0x1000000);
}

function writeInt32(buf, val, pos) {
    buf[pos] = val;
    buf[pos + 1] = (val >>> 8);
    buf[pos + 2] = (val >>> 16);
    buf[pos + 3] = (val >>> 24);
}

function readInt32(buf, pos) {
    return ((buf[pos]) |
        (buf[pos + 1] << 8) |
        (buf[pos + 2] << 16)) +
        (buf[pos + 3] << 24);
}

function readUtf8(buf, pos, end) {
    var str = '';
    var i = pos;

    while (i < end) {
        var b0 = buf[i];
        var c = null; // codepoint
        var bytesPerSequence =
            b0 > 0xEF ? 4 :
            b0 > 0xDF ? 3 :
            b0 > 0xBF ? 2 : 1;

        if (i + bytesPerSequence > end) break;

        var b1, b2, b3;

        if (bytesPerSequence === 1) {
            if (b0 < 0x80) {
                c = b0;
            }
        } else if (bytesPerSequence === 2) {
            b1 = buf[i + 1];
            if ((b1 & 0xC0) === 0x80) {
                c = (b0 & 0x1F) << 0x6 | (b1 & 0x3F);
                if (c <= 0x7F) {
                    c = null;
                }
            }
        } else if (bytesPerSequence === 3) {
            b1 = buf[i + 1];
            b2 = buf[i + 2];
            if ((b1 & 0xC0) === 0x80 && (b2 & 0xC0) === 0x80) {
                c = (b0 & 0xF) << 0xC | (b1 & 0x3F) << 0x6 | (b2 & 0x3F);
                if (c <= 0x7FF || (c >= 0xD800 && c <= 0xDFFF)) {
                    c = null;
                }
            }
        } else if (bytesPerSequence === 4) {
            b1 = buf[i + 1];
            b2 = buf[i + 2];
            b3 = buf[i + 3];
            if ((b1 & 0xC0) === 0x80 && (b2 & 0xC0) === 0x80 && (b3 & 0xC0) === 0x80) {
                c = (b0 & 0xF) << 0x12 | (b1 & 0x3F) << 0xC | (b2 & 0x3F) << 0x6 | (b3 & 0x3F);
                if (c <= 0xFFFF || c >= 0x110000) {
                    c = null;
                }
            }
        }

        if (c === null) {
            c = 0xFFFD;
            bytesPerSequence = 1;

        } else if (c > 0xFFFF) {
            c -= 0x10000;
            str += String.fromCharCode(c >>> 10 & 0x3FF | 0xD800);
            c = 0xDC00 | c & 0x3FF;
        }

        str += String.fromCharCode(c);
        i += bytesPerSequence;
    }

    return str;
}

function writeUtf8(buf, str, pos) {
    for (var i = 0, c, lead; i < str.length; i++) {
        c = str.charCodeAt(i); // code point

        if (c > 0xD7FF && c < 0xE000) {
            if (lead) {
                if (c < 0xDC00) {
                    buf[pos++] = 0xEF;
                    buf[pos++] = 0xBF;
                    buf[pos++] = 0xBD;
                    lead = c;
                    continue;
                } else {
                    c = lead - 0xD800 << 10 | c - 0xDC00 | 0x10000;
                    lead = null;
                }
            } else {
                if (c > 0xDBFF || (i + 1 === str.length)) {
                    buf[pos++] = 0xEF;
                    buf[pos++] = 0xBF;
                    buf[pos++] = 0xBD;
                } else {
                    lead = c;
                }
                continue;
            }
        } else if (lead) {
            buf[pos++] = 0xEF;
            buf[pos++] = 0xBF;
            buf[pos++] = 0xBD;
            lead = null;
        }

        if (c < 0x80) {
            buf[pos++] = c;
        } else {
            if (c < 0x800) {
                buf[pos++] = c >> 0x6 | 0xC0;
            } else {
                if (c < 0x10000) {
                    buf[pos++] = c >> 0xC | 0xE0;
                } else {
                    buf[pos++] = c >> 0x12 | 0xF0;
                    buf[pos++] = c >> 0xC & 0x3F | 0x80;
                }
                buf[pos++] = c >> 0x6 & 0x3F | 0x80;
            }
            buf[pos++] = c & 0x3F | 0x80;
        }
    }
    return pos;
}

var pointGeometry = Point;

/**
 * A standalone point geometry with useful accessor, comparison, and
 * modification methods.
 *
 * @class Point
 * @param {Number} x the x-coordinate. this could be longitude or screen
 * pixels, or any other sort of unit.
 * @param {Number} y the y-coordinate. this could be latitude or screen
 * pixels, or any other sort of unit.
 * @example
 * var point = new Point(-77, 38);
 */
function Point(x, y) {
    this.x = x;
    this.y = y;
}

Point.prototype = {

    /**
     * Clone this point, returning a new point that can be modified
     * without affecting the old one.
     * @return {Point} the clone
     */
    clone: function() { return new Point(this.x, this.y); },

    /**
     * Add this point's x & y coordinates to another point,
     * yielding a new point.
     * @param {Point} p the other point
     * @return {Point} output point
     */
    add:     function(p) { return this.clone()._add(p); },

    /**
     * Subtract this point's x & y coordinates to from point,
     * yielding a new point.
     * @param {Point} p the other point
     * @return {Point} output point
     */
    sub:     function(p) { return this.clone()._sub(p); },

    /**
     * Multiply this point's x & y coordinates by point,
     * yielding a new point.
     * @param {Point} p the other point
     * @return {Point} output point
     */
    multByPoint:    function(p) { return this.clone()._multByPoint(p); },

    /**
     * Divide this point's x & y coordinates by point,
     * yielding a new point.
     * @param {Point} p the other point
     * @return {Point} output point
     */
    divByPoint:     function(p) { return this.clone()._divByPoint(p); },

    /**
     * Multiply this point's x & y coordinates by a factor,
     * yielding a new point.
     * @param {Point} k factor
     * @return {Point} output point
     */
    mult:    function(k) { return this.clone()._mult(k); },

    /**
     * Divide this point's x & y coordinates by a factor,
     * yielding a new point.
     * @param {Point} k factor
     * @return {Point} output point
     */
    div:     function(k) { return this.clone()._div(k); },

    /**
     * Rotate this point around the 0, 0 origin by an angle a,
     * given in radians
     * @param {Number} a angle to rotate around, in radians
     * @return {Point} output point
     */
    rotate:  function(a) { return this.clone()._rotate(a); },

    /**
     * Rotate this point around p point by an angle a,
     * given in radians
     * @param {Number} a angle to rotate around, in radians
     * @param {Point} p Point to rotate around
     * @return {Point} output point
     */
    rotateAround:  function(a,p) { return this.clone()._rotateAround(a,p); },

    /**
     * Multiply this point by a 4x1 transformation matrix
     * @param {Array<Number>} m transformation matrix
     * @return {Point} output point
     */
    matMult: function(m) { return this.clone()._matMult(m); },

    /**
     * Calculate this point but as a unit vector from 0, 0, meaning
     * that the distance from the resulting point to the 0, 0
     * coordinate will be equal to 1 and the angle from the resulting
     * point to the 0, 0 coordinate will be the same as before.
     * @return {Point} unit vector point
     */
    unit:    function() { return this.clone()._unit(); },

    /**
     * Compute a perpendicular point, where the new y coordinate
     * is the old x coordinate and the new x coordinate is the old y
     * coordinate multiplied by -1
     * @return {Point} perpendicular point
     */
    perp:    function() { return this.clone()._perp(); },

    /**
     * Return a version of this point with the x & y coordinates
     * rounded to integers.
     * @return {Point} rounded point
     */
    round:   function() { return this.clone()._round(); },

    /**
     * Return the magitude of this point: this is the Euclidean
     * distance from the 0, 0 coordinate to this point's x and y
     * coordinates.
     * @return {Number} magnitude
     */
    mag: function() {
        return Math.sqrt(this.x * this.x + this.y * this.y);
    },

    /**
     * Judge whether this point is equal to another point, returning
     * true or false.
     * @param {Point} other the other point
     * @return {boolean} whether the points are equal
     */
    equals: function(other) {
        return this.x === other.x &&
               this.y === other.y;
    },

    /**
     * Calculate the distance from this point to another point
     * @param {Point} p the other point
     * @return {Number} distance
     */
    dist: function(p) {
        return Math.sqrt(this.distSqr(p));
    },

    /**
     * Calculate the distance from this point to another point,
     * without the square root step. Useful if you're comparing
     * relative distances.
     * @param {Point} p the other point
     * @return {Number} distance
     */
    distSqr: function(p) {
        var dx = p.x - this.x,
            dy = p.y - this.y;
        return dx * dx + dy * dy;
    },

    /**
     * Get the angle from the 0, 0 coordinate to this point, in radians
     * coordinates.
     * @return {Number} angle
     */
    angle: function() {
        return Math.atan2(this.y, this.x);
    },

    /**
     * Get the angle from this point to another point, in radians
     * @param {Point} b the other point
     * @return {Number} angle
     */
    angleTo: function(b) {
        return Math.atan2(this.y - b.y, this.x - b.x);
    },

    /**
     * Get the angle between this point and another point, in radians
     * @param {Point} b the other point
     * @return {Number} angle
     */
    angleWith: function(b) {
        return this.angleWithSep(b.x, b.y);
    },

    /*
     * Find the angle of the two vectors, solving the formula for
     * the cross product a x b = |a||b|sin(θ) for θ.
     * @param {Number} x the x-coordinate
     * @param {Number} y the y-coordinate
     * @return {Number} the angle in radians
     */
    angleWithSep: function(x, y) {
        return Math.atan2(
            this.x * y - this.y * x,
            this.x * x + this.y * y);
    },

    _matMult: function(m) {
        var x = m[0] * this.x + m[1] * this.y,
            y = m[2] * this.x + m[3] * this.y;
        this.x = x;
        this.y = y;
        return this;
    },

    _add: function(p) {
        this.x += p.x;
        this.y += p.y;
        return this;
    },

    _sub: function(p) {
        this.x -= p.x;
        this.y -= p.y;
        return this;
    },

    _mult: function(k) {
        this.x *= k;
        this.y *= k;
        return this;
    },

    _div: function(k) {
        this.x /= k;
        this.y /= k;
        return this;
    },

    _multByPoint: function(p) {
        this.x *= p.x;
        this.y *= p.y;
        return this;
    },

    _divByPoint: function(p) {
        this.x /= p.x;
        this.y /= p.y;
        return this;
    },

    _unit: function() {
        this._div(this.mag());
        return this;
    },

    _perp: function() {
        var y = this.y;
        this.y = this.x;
        this.x = -y;
        return this;
    },

    _rotate: function(angle) {
        var cos = Math.cos(angle),
            sin = Math.sin(angle),
            x = cos * this.x - sin * this.y,
            y = sin * this.x + cos * this.y;
        this.x = x;
        this.y = y;
        return this;
    },

    _rotateAround: function(angle, p) {
        var cos = Math.cos(angle),
            sin = Math.sin(angle),
            x = p.x + cos * (this.x - p.x) - sin * (this.y - p.y),
            y = p.y + sin * (this.x - p.x) + cos * (this.y - p.y);
        this.x = x;
        this.y = y;
        return this;
    },

    _round: function() {
        this.x = Math.round(this.x);
        this.y = Math.round(this.y);
        return this;
    }
};

/**
 * Construct a point from an array if necessary, otherwise if the input
 * is already a Point, or an unknown type, return it unchanged
 * @param {Array<Number>|Point|*} a any kind of input value
 * @return {Point} constructed point, or passed-through value.
 * @example
 * // this
 * var point = Point.convert([0, 1]);
 * // is equivalent to
 * var point = new Point(0, 1);
 */
Point.convert = function (a) {
    if (a instanceof Point) {
        return a;
    }
    if (Array.isArray(a)) {
        return new Point(a[0], a[1]);
    }
    return a;
};

var vectortilefeature = VectorTileFeature;

function VectorTileFeature(pbf, end, extent, keys, values) {
    // Public
    this.properties = {};
    this.extent = extent;
    this.type = 0;

    // Private
    this._pbf = pbf;
    this._geometry = -1;
    this._keys = keys;
    this._values = values;

    pbf.readFields(readFeature, this, end);
}

function readFeature(tag, feature, pbf) {
    if (tag == 1) feature.id = pbf.readVarint();
    else if (tag == 2) readTag(pbf, feature);
    else if (tag == 3) feature.type = pbf.readVarint();
    else if (tag == 4) feature._geometry = pbf.pos;
}

function readTag(pbf, feature) {
    var end = pbf.readVarint() + pbf.pos;

    while (pbf.pos < end) {
        var key = feature._keys[pbf.readVarint()],
            value = feature._values[pbf.readVarint()];
        feature.properties[key] = value;
    }
}

VectorTileFeature.types = ['Unknown', 'Point', 'LineString', 'Polygon'];

VectorTileFeature.prototype.loadGeometry = function() {
    var pbf = this._pbf;
    pbf.pos = this._geometry;

    var end = pbf.readVarint() + pbf.pos,
        cmd = 1,
        length = 0,
        x = 0,
        y = 0,
        lines = [],
        line;

    while (pbf.pos < end) {
        if (!length) {
            var cmdLen = pbf.readVarint();
            cmd = cmdLen & 0x7;
            length = cmdLen >> 3;
        }

        length--;

        if (cmd === 1 || cmd === 2) {
            x += pbf.readSVarint();
            y += pbf.readSVarint();

            if (cmd === 1) { // moveTo
                if (line) lines.push(line);
                line = [];
            }

            line.push(new pointGeometry(x, y));

        } else if (cmd === 7) {

            // Workaround for https://github.com/mapbox/mapnik-vector-tile/issues/90
            if (line) {
                line.push(line[0].clone()); // closePolygon
            }

        } else {
            throw new Error('unknown command ' + cmd);
        }
    }

    if (line) lines.push(line);

    return lines;
};

VectorTileFeature.prototype.bbox = function() {
    var pbf = this._pbf;
    pbf.pos = this._geometry;

    var end = pbf.readVarint() + pbf.pos,
        cmd = 1,
        length = 0,
        x = 0,
        y = 0,
        x1 = Infinity,
        x2 = -Infinity,
        y1 = Infinity,
        y2 = -Infinity;

    while (pbf.pos < end) {
        if (!length) {
            var cmdLen = pbf.readVarint();
            cmd = cmdLen & 0x7;
            length = cmdLen >> 3;
        }

        length--;

        if (cmd === 1 || cmd === 2) {
            x += pbf.readSVarint();
            y += pbf.readSVarint();
            if (x < x1) x1 = x;
            if (x > x2) x2 = x;
            if (y < y1) y1 = y;
            if (y > y2) y2 = y;

        } else if (cmd !== 7) {
            throw new Error('unknown command ' + cmd);
        }
    }

    return [x1, y1, x2, y2];
};

VectorTileFeature.prototype.toGeoJSON = function(x, y, z) {
    var size = this.extent * Math.pow(2, z),
        x0 = this.extent * x,
        y0 = this.extent * y,
        coords = this.loadGeometry(),
        type = VectorTileFeature.types[this.type],
        i, j;

    function project(line) {
        for (var j = 0; j < line.length; j++) {
            var p = line[j], y2 = 180 - (p.y + y0) * 360 / size;
            line[j] = [
                (p.x + x0) * 360 / size - 180,
                360 / Math.PI * Math.atan(Math.exp(y2 * Math.PI / 180)) - 90
            ];
        }
    }

    switch (this.type) {
    case 1:
        var points = [];
        for (i = 0; i < coords.length; i++) {
            points[i] = coords[i][0];
        }
        coords = points;
        project(coords);
        break;

    case 2:
        for (i = 0; i < coords.length; i++) {
            project(coords[i]);
        }
        break;

    case 3:
        coords = classifyRings(coords);
        for (i = 0; i < coords.length; i++) {
            for (j = 0; j < coords[i].length; j++) {
                project(coords[i][j]);
            }
        }
        break;
    }

    if (coords.length === 1) {
        coords = coords[0];
    } else {
        type = 'Multi' + type;
    }

    var result = {
        type: "Feature",
        geometry: {
            type: type,
            coordinates: coords
        },
        properties: this.properties
    };

    if ('id' in this) {
        result.id = this.id;
    }

    return result;
};

// classifies an array of rings into polygons with outer rings and holes

function classifyRings(rings) {
    var len = rings.length;

    if (len <= 1) return [rings];

    var polygons = [],
        polygon,
        ccw;

    for (var i = 0; i < len; i++) {
        var area = signedArea$1(rings[i]);
        if (area === 0) continue;

        if (ccw === undefined) ccw = area < 0;

        if (ccw === area < 0) {
            if (polygon) polygons.push(polygon);
            polygon = [rings[i]];

        } else {
            polygon.push(rings[i]);
        }
    }
    if (polygon) polygons.push(polygon);

    return polygons;
}

function signedArea$1(ring) {
    var sum = 0;
    for (var i = 0, len = ring.length, j = len - 1, p1, p2; i < len; j = i++) {
        p1 = ring[i];
        p2 = ring[j];
        sum += (p2.x - p1.x) * (p1.y + p2.y);
    }
    return sum;
}

var vectortilelayer = VectorTileLayer;

function VectorTileLayer(pbf, end) {
    // Public
    this.version = 1;
    this.name = null;
    this.extent = 4096;
    this.length = 0;

    // Private
    this._pbf = pbf;
    this._keys = [];
    this._values = [];
    this._features = [];

    pbf.readFields(readLayer, this, end);

    this.length = this._features.length;
}

function readLayer(tag, layer, pbf) {
    if (tag === 15) layer.version = pbf.readVarint();
    else if (tag === 1) layer.name = pbf.readString();
    else if (tag === 5) layer.extent = pbf.readVarint();
    else if (tag === 2) layer._features.push(pbf.pos);
    else if (tag === 3) layer._keys.push(pbf.readString());
    else if (tag === 4) layer._values.push(readValueMessage(pbf));
}

function readValueMessage(pbf) {
    var value = null,
        end = pbf.readVarint() + pbf.pos;

    while (pbf.pos < end) {
        var tag = pbf.readVarint() >> 3;

        value = tag === 1 ? pbf.readString() :
            tag === 2 ? pbf.readFloat() :
            tag === 3 ? pbf.readDouble() :
            tag === 4 ? pbf.readVarint64() :
            tag === 5 ? pbf.readVarint() :
            tag === 6 ? pbf.readSVarint() :
            tag === 7 ? pbf.readBoolean() : null;
    }

    return value;
}

// return feature `i` from this layer as a `VectorTileFeature`
VectorTileLayer.prototype.feature = function(i) {
    if (i < 0 || i >= this._features.length) throw new Error('feature index out of bounds');

    this._pbf.pos = this._features[i];

    var end = this._pbf.readVarint() + this._pbf.pos;
    return new vectortilefeature(this._pbf, end, this.extent, this._keys, this._values);
};

var vectortile = VectorTile;

function VectorTile(pbf, end) {
    this.layers = pbf.readFields(readTile, {}, end);
}

function readTile(tag, layers, pbf) {
    if (tag === 3) {
        var layer = new vectortilelayer(pbf, pbf.readVarint() + pbf.pos);
        if (layer.length) layers[layer.name] = layer;
    }
}

var VectorTile$1 = vectortile;
var VectorTileFeature$1 = vectortilefeature;

/**
 Mapbox Vector Tile format
 @class MVTSource
*/

var MVTSource =
/*#__PURE__*/
function (_NetworkTileSource) {
  _inheritsLoose(MVTSource, _NetworkTileSource);

  function MVTSource(source, sources) {
    var _this;

    _this = _NetworkTileSource.call(this, source, sources) || this;
    _this.response_type = 'arraybuffer'; // binary data

    return _this;
  }

  var _proto = MVTSource.prototype;

  _proto.parseSourceData = function parseSourceData(tile, source, response) {
    // Convert Mapbox vector tile to GeoJSON
    var data = new Uint8Array(response);
    var buffer = new pbf(data);
    source.data = new VectorTile$1(buffer);
    source.layers = this.toGeoJSON(source.data); // Apply optional data transform

    if (typeof this.transform === 'function') {
      var tile_data = {
        min: Object.assign({}, tile.min),
        max: Object.assign({}, tile.max),
        coords: Object.assign({}, tile.coords)
      };
      source.layers = this.transform(source.layers, this.extra_data, tile_data);
    }

    delete source.data; // comment out to save raw data for debugging
  } // Loop through layers/features using Mapbox lib API, convert to GeoJSON features
  // Returns an object with keys for each layer, e.g. { layer: geojson }
  ;

  _proto.toGeoJSON = function toGeoJSON(tile) {
    var layers = {};

    for (var l in tile.layers) {
      var layer = tile.layers[l];
      var scale = Geo$1.tile_scale / layer.extent;
      var layer_geojson = {
        type: 'FeatureCollection',
        features: []
      };

      for (var f = 0; f < layer.length; f++) {
        var feature = layer.feature(f);
        var feature_geojson = {
          type: 'Feature',
          geometry: {},
          properties: feature.properties
        };
        var geometry = feature_geojson.geometry;
        var coordinates = feature.loadGeometry();

        for (var r = 0; r < coordinates.length; r++) {
          var ring = coordinates[r];

          for (var c = 0; c < ring.length; c++) {
            ring[c] = [ring[c].x * scale, ring[c].y * scale];
          }
        }

        geometry.coordinates = coordinates;

        if (VectorTileFeature$1.types[feature.type] === 'Point') {
          geometry.type = 'Point';
          geometry.coordinates = geometry.coordinates[0][0];
        } else if (VectorTileFeature$1.types[feature.type] === 'LineString') {
          if (coordinates.length === 1) {
            geometry.type = 'LineString';
            geometry.coordinates = geometry.coordinates[0];
          } else {
            geometry.type = 'MultiLineString';
          }
        } else if (VectorTileFeature$1.types[feature.type] === 'Polygon') {
          geometry = decodeMultiPolygon(geometry); // un-flatten rings
        }

        layer_geojson.features.push(feature_geojson);
      }

      layers[l] = layer_geojson;
    }

    return layers;
  };

  return MVTSource;
}(NetworkTileSource); // Decode multipolygons, which are encoded as a single set of rings
// Winding order of first ring is assumed to indicate exterior ring,
// the opposite winding order indicates the start of a new polygon.

function decodeMultiPolygon(geom) {
  var polys = [];
  var poly = [];
  var outer_winding;

  for (var r = 0; r < geom.coordinates.length; r++) {
    var ring = geom.coordinates[r];
    var winding = Geo$1.ringWinding(ring);

    if (winding == null) {
      continue; // skip zero-area rings
    }

    outer_winding = outer_winding || winding; // assume first ring indicates outer ring winding

    if (winding === outer_winding && poly.length > 0) {
      polys.push(poly);
      poly = [];
    }

    poly.push(ring);
  }

  if (poly.length > 0) {
    polys.push(poly);
  } // Single or multi?


  if (polys.length === 1) {
    geom.type = 'Polygon';
    geom.coordinates = polys[0];
  } else if (polys.length > 1) {
    geom.type = 'MultiPolygon';
    geom.coordinates = polys;
  } else {
    geom = null;
  }

  return geom;
}
DataSource.register('MVT', function () {
  return MVTSource;
});

var simplify_1 = simplify;

// calculate simplification data using optimized Douglas-Peucker algorithm

function simplify(points, tolerance) {

    var sqTolerance = tolerance * tolerance,
        len = points.length,
        first = 0,
        last = len - 1,
        stack = [],
        i, maxSqDist, sqDist, index;

    // always retain the endpoints (1 is the max value)
    points[first][2] = 1;
    points[last][2] = 1;

    // avoid recursion by using a stack
    while (last) {

        maxSqDist = 0;

        for (i = first + 1; i < last; i++) {
            sqDist = getSqSegDist(points[i], points[first], points[last]);

            if (sqDist > maxSqDist) {
                index = i;
                maxSqDist = sqDist;
            }
        }

        if (maxSqDist > sqTolerance) {
            points[index][2] = maxSqDist; // save the point importance in squared pixels as a z coordinate
            stack.push(first);
            stack.push(index);
            first = index;

        } else {
            last = stack.pop();
            first = stack.pop();
        }
    }
}

// square distance from a point to a segment
function getSqSegDist(p, a, b) {

    var x = a[0], y = a[1],
        bx = b[0], by = b[1],
        px = p[0], py = p[1],
        dx = bx - x,
        dy = by - y;

    if (dx !== 0 || dy !== 0) {

        var t = ((px - x) * dx + (py - y) * dy) / (dx * dx + dy * dy);

        if (t > 1) {
            x = bx;
            y = by;

        } else if (t > 0) {
            x += dx * t;
            y += dy * t;
        }
    }

    dx = px - x;
    dy = py - y;

    return dx * dx + dy * dy;
}

var feature$1 = createFeature;

function createFeature(tags, type, geom, id) {
    var feature = {
        id: id || null,
        type: type,
        geometry: geom,
        tags: tags || null,
        min: [Infinity, Infinity], // initial bbox values
        max: [-Infinity, -Infinity]
    };
    calcBBox(feature);
    return feature;
}

// calculate the feature bounding box for faster clipping later
function calcBBox(feature) {
    var geometry = feature.geometry,
        min = feature.min,
        max = feature.max;

    if (feature.type === 1) {
        calcRingBBox(min, max, geometry);
    } else {
        for (var i = 0; i < geometry.length; i++) {
            calcRingBBox(min, max, geometry[i]);
        }
    }

    return feature;
}

function calcRingBBox(min, max, points) {
    for (var i = 0, p; i < points.length; i++) {
        p = points[i];
        min[0] = Math.min(p[0], min[0]);
        max[0] = Math.max(p[0], max[0]);
        min[1] = Math.min(p[1], min[1]);
        max[1] = Math.max(p[1], max[1]);
    }
}

var convert_1 = convert;




// converts GeoJSON feature into an intermediate projected JSON vector format with simplification data

function convert(data, tolerance) {
    var features = [];

    if (data.type === 'FeatureCollection') {
        for (var i = 0; i < data.features.length; i++) {
            convertFeature(features, data.features[i], tolerance);
        }
    } else if (data.type === 'Feature') {
        convertFeature(features, data, tolerance);

    } else {
        // single geometry or a geometry collection
        convertFeature(features, {geometry: data}, tolerance);
    }
    return features;
}

function convertFeature(features, feature, tolerance) {
    if (feature.geometry === null) {
        // ignore features with null geometry
        return;
    }

    var geom = feature.geometry,
        type = geom.type,
        coords = geom.coordinates,
        tags = feature.properties,
        id = feature.id,
        i, j, rings, projectedRing;

    if (type === 'Point') {
        features.push(feature$1(tags, 1, [projectPoint(coords)], id));

    } else if (type === 'MultiPoint') {
        features.push(feature$1(tags, 1, project(coords), id));

    } else if (type === 'LineString') {
        features.push(feature$1(tags, 2, [project(coords, tolerance)], id));

    } else if (type === 'MultiLineString' || type === 'Polygon') {
        rings = [];
        for (i = 0; i < coords.length; i++) {
            projectedRing = project(coords[i], tolerance);
            if (type === 'Polygon') projectedRing.outer = (i === 0);
            rings.push(projectedRing);
        }
        features.push(feature$1(tags, type === 'Polygon' ? 3 : 2, rings, id));

    } else if (type === 'MultiPolygon') {
        rings = [];
        for (i = 0; i < coords.length; i++) {
            for (j = 0; j < coords[i].length; j++) {
                projectedRing = project(coords[i][j], tolerance);
                projectedRing.outer = (j === 0);
                rings.push(projectedRing);
            }
        }
        features.push(feature$1(tags, 3, rings, id));

    } else if (type === 'GeometryCollection') {
        for (i = 0; i < geom.geometries.length; i++) {
            convertFeature(features, {
                geometry: geom.geometries[i],
                properties: tags
            }, tolerance);
        }

    } else {
        throw new Error('Input data is not a valid GeoJSON object.');
    }
}

function project(lonlats, tolerance) {
    var projected = [];
    for (var i = 0; i < lonlats.length; i++) {
        projected.push(projectPoint(lonlats[i]));
    }
    if (tolerance) {
        simplify_1(projected, tolerance);
        calcSize(projected);
    }
    return projected;
}

function projectPoint(p) {
    var sin = Math.sin(p[1] * Math.PI / 180),
        x = (p[0] / 360 + 0.5),
        y = (0.5 - 0.25 * Math.log((1 + sin) / (1 - sin)) / Math.PI);

    y = y < 0 ? 0 :
        y > 1 ? 1 : y;

    return [x, y, 0];
}

// calculate area and length of the poly
function calcSize(points) {
    var area = 0,
        dist = 0;

    for (var i = 0, a, b; i < points.length - 1; i++) {
        a = b || points[i];
        b = points[i + 1];

        area += a[0] * b[1] - b[0] * a[1];

        // use Manhattan distance instead of Euclidian one to avoid expensive square root computation
        dist += Math.abs(b[0] - a[0]) + Math.abs(b[1] - a[1]);
    }
    points.area = Math.abs(area / 2);
    points.dist = dist;
}

var tile = transformTile;
var point = transformPoint;

// Transforms the coordinates of each feature in the given tile from
// mercator-projected space into (extent x extent) tile space.
function transformTile(tile, extent) {
    if (tile.transformed) return tile;

    var z2 = tile.z2,
        tx = tile.x,
        ty = tile.y,
        i, j, k;

    for (i = 0; i < tile.features.length; i++) {
        var feature = tile.features[i],
            geom = feature.geometry,
            type = feature.type;

        if (type === 1) {
            for (j = 0; j < geom.length; j++) geom[j] = transformPoint(geom[j], extent, z2, tx, ty);

        } else {
            for (j = 0; j < geom.length; j++) {
                var ring = geom[j];
                for (k = 0; k < ring.length; k++) ring[k] = transformPoint(ring[k], extent, z2, tx, ty);
            }
        }
    }

    tile.transformed = true;

    return tile;
}

function transformPoint(p, extent, z2, tx, ty) {
    var x = Math.round(extent * (p[0] * z2 - tx)),
        y = Math.round(extent * (p[1] * z2 - ty));
    return [x, y];
}

var transform = {
	tile: tile,
	point: point
};

var clip_1 = clip;



/* clip features between two axis-parallel lines:
 *     |        |
 *  ___|___     |     /
 * /   |   \____|____/
 *     |        |
 */

function clip(features, scale, k1, k2, axis, intersect, minAll, maxAll) {

    k1 /= scale;
    k2 /= scale;

    if (minAll >= k1 && maxAll <= k2) return features; // trivial accept
    else if (minAll > k2 || maxAll < k1) return null; // trivial reject

    var clipped = [];

    for (var i = 0; i < features.length; i++) {

        var feature = features[i],
            geometry = feature.geometry,
            type = feature.type,
            min, max;

        min = feature.min[axis];
        max = feature.max[axis];

        if (min >= k1 && max <= k2) { // trivial accept
            clipped.push(feature);
            continue;
        } else if (min > k2 || max < k1) continue; // trivial reject

        var slices = type === 1 ?
                clipPoints(geometry, k1, k2, axis) :
                clipGeometry(geometry, k1, k2, axis, intersect, type === 3);

        if (slices.length) {
            // if a feature got clipped, it will likely get clipped on the next zoom level as well,
            // so there's no need to recalculate bboxes
            clipped.push(feature$1(feature.tags, type, slices, feature.id));
        }
    }

    return clipped.length ? clipped : null;
}

function clipPoints(geometry, k1, k2, axis) {
    var slice = [];

    for (var i = 0; i < geometry.length; i++) {
        var a = geometry[i],
            ak = a[axis];

        if (ak >= k1 && ak <= k2) slice.push(a);
    }
    return slice;
}

function clipGeometry(geometry, k1, k2, axis, intersect, closed) {

    var slices = [];

    for (var i = 0; i < geometry.length; i++) {

        var ak = 0,
            bk = 0,
            b = null,
            points = geometry[i],
            area = points.area,
            dist = points.dist,
            outer = points.outer,
            len = points.length,
            a, j, last;

        var slice = [];

        for (j = 0; j < len - 1; j++) {
            a = b || points[j];
            b = points[j + 1];
            ak = bk || a[axis];
            bk = b[axis];

            if (ak < k1) {

                if ((bk > k2)) { // ---|-----|-->
                    slice.push(intersect(a, b, k1), intersect(a, b, k2));
                    if (!closed) slice = newSlice(slices, slice, area, dist, outer);

                } else if (bk >= k1) slice.push(intersect(a, b, k1)); // ---|-->  |

            } else if (ak > k2) {

                if ((bk < k1)) { // <--|-----|---
                    slice.push(intersect(a, b, k2), intersect(a, b, k1));
                    if (!closed) slice = newSlice(slices, slice, area, dist, outer);

                } else if (bk <= k2) slice.push(intersect(a, b, k2)); // |  <--|---

            } else {

                slice.push(a);

                if (bk < k1) { // <--|---  |
                    slice.push(intersect(a, b, k1));
                    if (!closed) slice = newSlice(slices, slice, area, dist, outer);

                } else if (bk > k2) { // |  ---|-->
                    slice.push(intersect(a, b, k2));
                    if (!closed) slice = newSlice(slices, slice, area, dist, outer);
                }
                // | --> |
            }
        }

        // add the last point
        a = points[len - 1];
        ak = a[axis];
        if (ak >= k1 && ak <= k2) slice.push(a);

        // close the polygon if its endpoints are not the same after clipping

        last = slice[slice.length - 1];
        if (closed && last && (slice[0][0] !== last[0] || slice[0][1] !== last[1])) slice.push(slice[0]);

        // add the final slice
        newSlice(slices, slice, area, dist, outer);
    }

    return slices;
}

function newSlice(slices, slice, area, dist, outer) {
    if (slice.length) {
        // we don't recalculate the area/length of the unclipped geometry because the case where it goes
        // below the visibility threshold as a result of clipping is rare, so we avoid doing unnecessary work
        slice.area = area;
        slice.dist = dist;
        if (outer !== undefined) slice.outer = outer;

        slices.push(slice);
    }
    return [];
}

var wrap_1 = wrap$1;

function wrap$1(features, buffer, intersectX) {
    var merged = features,
        left  = clip_1(features, 1, -1 - buffer, buffer,     0, intersectX, -1, 2), // left world copy
        right = clip_1(features, 1,  1 - buffer, 2 + buffer, 0, intersectX, -1, 2); // right world copy

    if (left || right) {
        merged = clip_1(features, 1, -buffer, 1 + buffer, 0, intersectX, -1, 2) || []; // center world copy

        if (left) merged = shiftFeatureCoords(left, 1).concat(merged); // merge left into center
        if (right) merged = merged.concat(shiftFeatureCoords(right, -1)); // merge right into center
    }

    return merged;
}

function shiftFeatureCoords(features, offset) {
    var newFeatures = [];

    for (var i = 0; i < features.length; i++) {
        var feature = features[i],
            type = feature.type;

        var newGeometry;

        if (type === 1) {
            newGeometry = shiftCoords(feature.geometry, offset);
        } else {
            newGeometry = [];
            for (var j = 0; j < feature.geometry.length; j++) {
                newGeometry.push(shiftCoords(feature.geometry[j], offset));
            }
        }

        newFeatures.push(feature$1(feature.tags, type, newGeometry, feature.id));
    }

    return newFeatures;
}

function shiftCoords(points, offset) {
    var newPoints = [];
    newPoints.area = points.area;
    newPoints.dist = points.dist;

    for (var i = 0; i < points.length; i++) {
        newPoints.push([points[i][0] + offset, points[i][1], points[i][2]]);
    }
    return newPoints;
}

var tile$1 = createTile;

function createTile(features, z2, tx, ty, tolerance, noSimplify) {
    var tile = {
        features: [],
        numPoints: 0,
        numSimplified: 0,
        numFeatures: 0,
        source: null,
        x: tx,
        y: ty,
        z2: z2,
        transformed: false,
        min: [2, 1],
        max: [-1, 0]
    };
    for (var i = 0; i < features.length; i++) {
        tile.numFeatures++;
        addFeature(tile, features[i], tolerance, noSimplify);

        var min = features[i].min,
            max = features[i].max;

        if (min[0] < tile.min[0]) tile.min[0] = min[0];
        if (min[1] < tile.min[1]) tile.min[1] = min[1];
        if (max[0] > tile.max[0]) tile.max[0] = max[0];
        if (max[1] > tile.max[1]) tile.max[1] = max[1];
    }
    return tile;
}

function addFeature(tile, feature, tolerance, noSimplify) {

    var geom = feature.geometry,
        type = feature.type,
        simplified = [],
        sqTolerance = tolerance * tolerance,
        i, j, ring, p;

    if (type === 1) {
        for (i = 0; i < geom.length; i++) {
            simplified.push(geom[i]);
            tile.numPoints++;
            tile.numSimplified++;
        }

    } else {

        // simplify and transform projected coordinates for tile geometry
        for (i = 0; i < geom.length; i++) {
            ring = geom[i];

            // filter out tiny polylines & polygons
            if (!noSimplify && ((type === 2 && ring.dist < tolerance) ||
                                (type === 3 && ring.area < sqTolerance))) {
                tile.numPoints += ring.length;
                continue;
            }

            var simplifiedRing = [];

            for (j = 0; j < ring.length; j++) {
                p = ring[j];
                // keep points with importance > tolerance
                if (noSimplify || p[2] > sqTolerance) {
                    simplifiedRing.push(p);
                    tile.numSimplified++;
                }
                tile.numPoints++;
            }

            if (type === 3) rewind(simplifiedRing, ring.outer);

            simplified.push(simplifiedRing);
        }
    }

    if (simplified.length) {
        var tileFeature = {
            geometry: simplified,
            type: type,
            tags: feature.tags || null
        };
        if (feature.id !== null) {
            tileFeature.id = feature.id;
        }
        tile.features.push(tileFeature);
    }
}

function rewind(ring, clockwise) {
    var area = signedArea$2(ring);
    if (area < 0 === clockwise) ring.reverse();
}

function signedArea$2(ring) {
    var sum = 0;
    for (var i = 0, len = ring.length, j = len - 1, p1, p2; i < len; j = i++) {
        p1 = ring[i];
        p2 = ring[j];
        sum += (p2[0] - p1[0]) * (p1[1] + p2[1]);
    }
    return sum;
}

var src = geojsonvt;

     // final simplified tile generation


function geojsonvt(data, options) {
    return new GeoJSONVT(data, options);
}

function GeoJSONVT(data, options) {
    options = this.options = extend(Object.create(this.options), options);

    var debug = options.debug;

    if (debug) console.time('preprocess data');

    var z2 = 1 << options.maxZoom, // 2^z
        features = convert_1(data, options.tolerance / (z2 * options.extent));

    this.tiles = {};
    this.tileCoords = [];

    if (debug) {
        console.timeEnd('preprocess data');
        console.log('index: maxZoom: %d, maxPoints: %d', options.indexMaxZoom, options.indexMaxPoints);
        console.time('generate tiles');
        this.stats = {};
        this.total = 0;
    }

    features = wrap_1(features, options.buffer / options.extent, intersectX);

    // start slicing from the top tile down
    if (features.length) this.splitTile(features, 0, 0, 0);

    if (debug) {
        if (features.length) console.log('features: %d, points: %d', this.tiles[0].numFeatures, this.tiles[0].numPoints);
        console.timeEnd('generate tiles');
        console.log('tiles generated:', this.total, JSON.stringify(this.stats));
    }
}

GeoJSONVT.prototype.options = {
    maxZoom: 14,            // max zoom to preserve detail on
    indexMaxZoom: 5,        // max zoom in the tile index
    indexMaxPoints: 100000, // max number of points per tile in the tile index
    solidChildren: false,   // whether to tile solid square tiles further
    tolerance: 3,           // simplification tolerance (higher means simpler)
    extent: 4096,           // tile extent
    buffer: 64,             // tile buffer on each side
    debug: 0                // logging level (0, 1 or 2)
};

GeoJSONVT.prototype.splitTile = function (features, z, x, y, cz, cx, cy) {

    var stack = [features, z, x, y],
        options = this.options,
        debug = options.debug,
        solid = null;

    // avoid recursion by using a processing queue
    while (stack.length) {
        y = stack.pop();
        x = stack.pop();
        z = stack.pop();
        features = stack.pop();

        var z2 = 1 << z,
            id = toID(z, x, y),
            tile = this.tiles[id],
            tileTolerance = z === options.maxZoom ? 0 : options.tolerance / (z2 * options.extent);

        if (!tile) {
            if (debug > 1) console.time('creation');

            tile = this.tiles[id] = tile$1(features, z2, x, y, tileTolerance, z === options.maxZoom);
            this.tileCoords.push({z: z, x: x, y: y});

            if (debug) {
                if (debug > 1) {
                    console.log('tile z%d-%d-%d (features: %d, points: %d, simplified: %d)',
                        z, x, y, tile.numFeatures, tile.numPoints, tile.numSimplified);
                    console.timeEnd('creation');
                }
                var key = 'z' + z;
                this.stats[key] = (this.stats[key] || 0) + 1;
                this.total++;
            }
        }

        // save reference to original geometry in tile so that we can drill down later if we stop now
        tile.source = features;

        // if it's the first-pass tiling
        if (!cz) {
            // stop tiling if we reached max zoom, or if the tile is too simple
            if (z === options.indexMaxZoom || tile.numPoints <= options.indexMaxPoints) continue;

        // if a drilldown to a specific tile
        } else {
            // stop tiling if we reached base zoom or our target tile zoom
            if (z === options.maxZoom || z === cz) continue;

            // stop tiling if it's not an ancestor of the target tile
            var m = 1 << (cz - z);
            if (x !== Math.floor(cx / m) || y !== Math.floor(cy / m)) continue;
        }

        // stop tiling if the tile is solid clipped square
        if (!options.solidChildren && isClippedSquare(tile, options.extent, options.buffer)) {
            if (cz) solid = z; // and remember the zoom if we're drilling down
            continue;
        }

        // if we slice further down, no need to keep source geometry
        tile.source = null;

        if (debug > 1) console.time('clipping');

        // values we'll use for clipping
        var k1 = 0.5 * options.buffer / options.extent,
            k2 = 0.5 - k1,
            k3 = 0.5 + k1,
            k4 = 1 + k1,
            tl, bl, tr, br, left, right;

        tl = bl = tr = br = null;

        left  = clip_1(features, z2, x - k1, x + k3, 0, intersectX, tile.min[0], tile.max[0]);
        right = clip_1(features, z2, x + k2, x + k4, 0, intersectX, tile.min[0], tile.max[0]);

        if (left) {
            tl = clip_1(left, z2, y - k1, y + k3, 1, intersectY, tile.min[1], tile.max[1]);
            bl = clip_1(left, z2, y + k2, y + k4, 1, intersectY, tile.min[1], tile.max[1]);
        }

        if (right) {
            tr = clip_1(right, z2, y - k1, y + k3, 1, intersectY, tile.min[1], tile.max[1]);
            br = clip_1(right, z2, y + k2, y + k4, 1, intersectY, tile.min[1], tile.max[1]);
        }

        if (debug > 1) console.timeEnd('clipping');

        if (features.length) {
            stack.push(tl || [], z + 1, x * 2,     y * 2);
            stack.push(bl || [], z + 1, x * 2,     y * 2 + 1);
            stack.push(tr || [], z + 1, x * 2 + 1, y * 2);
            stack.push(br || [], z + 1, x * 2 + 1, y * 2 + 1);
        }
    }

    return solid;
};

GeoJSONVT.prototype.getTile = function (z, x, y) {
    var options = this.options,
        extent = options.extent,
        debug = options.debug;

    var z2 = 1 << z;
    x = ((x % z2) + z2) % z2; // wrap tile x coordinate

    var id = toID(z, x, y);
    if (this.tiles[id]) return transform.tile(this.tiles[id], extent);

    if (debug > 1) console.log('drilling down to z%d-%d-%d', z, x, y);

    var z0 = z,
        x0 = x,
        y0 = y,
        parent;

    while (!parent && z0 > 0) {
        z0--;
        x0 = Math.floor(x0 / 2);
        y0 = Math.floor(y0 / 2);
        parent = this.tiles[toID(z0, x0, y0)];
    }

    if (!parent || !parent.source) return null;

    // if we found a parent tile containing the original geometry, we can drill down from it
    if (debug > 1) console.log('found parent tile z%d-%d-%d', z0, x0, y0);

    // it parent tile is a solid clipped square, return it instead since it's identical
    if (isClippedSquare(parent, extent, options.buffer)) return transform.tile(parent, extent);

    if (debug > 1) console.time('drilling down');
    var solid = this.splitTile(parent.source, z0, x0, y0, z, x, y);
    if (debug > 1) console.timeEnd('drilling down');

    // one of the parent tiles was a solid clipped square
    if (solid !== null) {
        var m = 1 << (z - solid);
        id = toID(solid, Math.floor(x / m), Math.floor(y / m));
    }

    return this.tiles[id] ? transform.tile(this.tiles[id], extent) : null;
};

function toID(z, x, y) {
    return (((1 << z) * y + x) * 32) + z;
}

function intersectX(a, b, x) {
    return [x, (x - a[0]) * (b[1] - a[1]) / (b[0] - a[0]) + a[1], 1];
}
function intersectY(a, b, y) {
    return [(y - a[1]) * (b[0] - a[0]) / (b[1] - a[1]) + a[0], y, 1];
}

function extend(dest, src) {
    for (var i in src) dest[i] = src[i];
    return dest;
}

// checks whether a tile is a whole-area fill after clipping; if it is, there's no sense slicing it further
function isClippedSquare(tile, extent, buffer) {

    var features = tile.source;
    if (features.length !== 1) return false;

    var feature = features[0];
    if (feature.type !== 3 || feature.geometry.length > 1) return false;

    var len = feature.geometry[0].length;
    if (len !== 5) return false;

    for (var i = 0; i < len; i++) {
        var p = transform.point(feature.geometry[0][i], extent, tile.z2, tile.x, tile.y);
        if ((p[0] !== -buffer && p[0] !== extent + buffer) ||
            (p[1] !== -buffer && p[1] !== extent + buffer)) return false;
    }

    return true;
}

/**
 GeoJSON standalone (non-tiled) source
 Uses geojson-vt split into tiles client-side
*/

var GeoJSONSource =
/*#__PURE__*/
function (_NetworkSource) {
  _inheritsLoose(GeoJSONSource, _NetworkSource);

  function GeoJSONSource(source, sources) {
    var _this;

    _this = _NetworkSource.call(this, source, sources) || this;
    _this.load_data = null;
    _this.tile_indexes = {}; // geojson-vt tile indices, by layer name

    _this.setTileSize(512); // auto-tile to 512px tiles for fewer internal tiles


    _this.pad_scale = 0; // we don't want padding on auto-tiled sources

    return _this;
  }

  var _proto = GeoJSONSource.prototype;

  _proto._load = function _load(dest) {
    var _this2 = this;

    if (!this.load_data) {
      this.load_data = _NetworkSource.prototype._load.call(this, {
        source_data: {
          layers: {}
        }
      }).then(function (data) {
        // Warn and continue on data source error
        if (data.source_data.error) {
          log('warn', "data source load error(s) for source '" + _this2.name + "', URL '" + _this2.url + "': " + data.source_data.error);
        }

        var layers = data.source_data.layers;

        for (var layer_name in layers) {
          _this2.tile_indexes[layer_name] = src(layers[layer_name], {
            maxZoom: _this2.max_zoom,
            // max zoom to preserve detail on
            tolerance: 1.5,
            // simplification tolerance (higher means simpler) NB: half the default to accomodate 512px tiles
            extent: Geo$1.tile_scale,
            // tile extent (both width and height)
            buffer: 0.0001 // tile buffer on each side

          });
        }

        _this2.loaded = true;
        return data;
      });
    }

    return this.load_data.then(function () {
      for (var layer_name in _this2.tile_indexes) {
        dest.source_data.layers[layer_name] = _this2.getTileFeatures(dest, layer_name);
      }

      return dest;
    });
  };

  _proto.getTileFeatures = function getTileFeatures(tile, layer_name) {
    var coords = Geo$1.wrapTile(tile.coords, {
      x: true
    }); // request a particular tile

    var t = this.tile_indexes[layer_name].getTile(coords.z, coords.x, coords.y); // Convert from MVT-style JSON struct to GeoJSON

    var collection;

    if (t && t.features) {
      collection = {
        type: 'FeatureCollection',
        features: []
      };

      for (var i = 0; i < t.features.length; i++) {
        var feature = t.features[i]; // GeoJSON feature

        var f = {
          type: 'Feature',
          geometry: {},
          properties: feature.tags
        };

        if (feature.type === 1) {
          f.geometry.coordinates = feature.geometry.map(function (coord) {
            return [coord[0], coord[1]];
          });
          f.geometry.type = 'MultiPoint';
        } else if (feature.type === 2 || feature.type === 3) {
          f.geometry.coordinates = feature.geometry.map(function (ring) {
            return ring.map(function (coord) {
              return [coord[0], coord[1]];
            });
          });

          if (feature.type === 2) {
            f.geometry.type = 'MultiLineString';
          } else {
            f.geometry = decodeMultiPolygon(f.geometry); // un-flatten rings

            if (f.geometry == null) {
              // skip polys that couldn't be decoded (e.g. degenerate)
              continue;
            }
          }
        } else {
          continue;
        }

        collection.features.push(f);
      }
    }

    return collection;
  };

  _proto.formatURL = function formatURL() {
    return this.url;
  };

  _proto.parseSourceData = function parseSourceData(tile, source, response) {
    var data = typeof response === 'string' ? JSON.parse(response) : response;
    var layers = this.getLayers(data);
    source.layers = this.preprocessLayers(layers, tile);
  };

  _proto.preprocessLayers = function preprocessLayers(layers, tile) {
    for (var key in layers) {
      var layer = layers[key];
      layer.features = this.preprocessFeatures(layer.features);
    } // Apply optional data transform


    if (typeof this.transform === 'function') {
      var tile_data = {
        min: Object.assign({}, tile.min),
        max: Object.assign({}, tile.max),
        coords: Object.assign({}, tile.coords)
      };

      if (Object.keys(layers).length === 1 && layers._default) {
        layers._default = this.transform(layers._default, this.extra_data, tile_data); // single-layer
      } else {
        layers = this.transform(layers, this.extra_data, tile_data); // multiple layers
      }
    }

    return layers;
  } // Preprocess features. Currently used to add a new "centroid" feature for polygon labeling
  ;

  _proto.preprocessFeatures = function preprocessFeatures(features) {
    // Remove features without geometry (which is valid GeoJSON)
    features = features.filter(function (f) {
      return f.geometry != null;
    }); // Define centroids for polygons for centroid label placement
    // Avoids redundant label placement for each generated tile at higher zoom levels

    if (this.config.generate_label_centroids) {
      var features_centroid = [];
      var centroid_properties = {
        'label_placement': true
      };
      features.forEach(function (feature) {
        var coordinates, centroid_feature;

        if (feature.geometry.type === 'Polygon') {
          coordinates = feature.geometry.coordinates;
          centroid_feature = getCentroidFeatureForPolygon(coordinates, feature.properties, centroid_properties);
          features_centroid.push(centroid_feature);
        } else if (feature.geometry.type === 'MultiPolygon') {
          // Add centroid feature for largest polygon
          coordinates = feature.geometry.coordinates;
          var max_area = -Infinity;
          var max_area_index = 0;

          for (var index = 0; index < coordinates.length; index++) {
            var area = Geo$1.polygonArea(coordinates[index]);

            if (area > max_area) {
              max_area = area;
              max_area_index = index;
            }
          }

          centroid_feature = getCentroidFeatureForPolygon(coordinates[max_area_index], feature.properties, centroid_properties);
          features_centroid.push(centroid_feature);
        }
      }); // append centroid features to features array

      features_centroid = features_centroid.filter(function (x) {
        return x;
      }); // remove null features

      Array.prototype.push.apply(features, features_centroid);
    }

    return features;
  } // Detect single or multiple layers in returned data
  ;

  _proto.getLayers = function getLayers(data) {
    if (data.type === 'Feature') {
      return {
        _default: {
          type: 'FeatureCollection',
          features: [data]
        }
      };
    } else if (data.type === 'FeatureCollection') {
      return {
        _default: data
      };
    } else {
      return data;
    }
  };

  return GeoJSONSource;
}(NetworkSource);
/**
 GeoJSON vector tiles
 @class GeoJSONTileSource
*/

var GeoJSONTileSource =
/*#__PURE__*/
function (_NetworkTileSource) {
  _inheritsLoose(GeoJSONTileSource, _NetworkTileSource);

  function GeoJSONTileSource(source, sources) {
    return _NetworkTileSource.call(this, source, sources) || this;
  }

  var _proto2 = GeoJSONTileSource.prototype;

  _proto2.parseSourceData = function parseSourceData(tile, source, response) {
    var data = typeof response === 'string' ? JSON.parse(response) : response;
    this.prepareGeoJSON(data, tile, source);
  };

  _proto2.prepareGeoJSON = function prepareGeoJSON(data, tile, source) {
    // Apply optional data transform
    if (typeof this.transform === 'function') {
      var tile_data = {
        min: Object.assign({}, tile.min),
        max: Object.assign({}, tile.max),
        coords: Object.assign({}, tile.coords)
      };
      data = this.transform(data, this.extra_data, tile_data);
    }

    source.layers = GeoJSONSource.prototype.getLayers(data); // A "synthetic" tile that adjusts the tile min anchor to account for tile longitude wrapping

    var anchor = {
      coords: tile.coords,
      min: Geo$1.metersForTile(Geo$1.wrapTile(tile.coords, {
        x: true
      }))
    };
    DataSource.projectData(source); // mercator projection

    DataSource.scaleData(source, anchor); // re-scale from meters to local tile coords
  };

  return GeoJSONTileSource;
}(NetworkTileSource); // Check for URL tile pattern, if not found, treat as standalone GeoJSON/TopoJSON object

DataSource.register('GeoJSON', function (source) {
  return GeoJSONTileSource.urlHasTilePattern(source.url) ? GeoJSONTileSource : GeoJSONSource;
}); // Helper function to create centroid point feature from polygon coordinates and provided feature meta-data

function getCentroidFeatureForPolygon(coordinates, properties, newProperties) {
  var centroid = Geo$1.centroid(coordinates);

  if (!centroid) {
    return;
  } // clone properties and mixix newProperties


  var centroid_properties = {};
  Object.assign(centroid_properties, properties, newProperties);
  return {
    type: 'Feature',
    properties: centroid_properties,
    geometry: {
      type: 'Point',
      coordinates: centroid
    }
  };
}

function reverse(array, n) {
  var t, j = array.length, i = j - n;
  while (i < --j) t = array[i], array[i++] = array[j], array[j] = t;
}

function identity$1(x) {
  return x;
}

function transform$1(topology) {
  if ((transform = topology.transform) == null) return identity$1;
  var transform,
      x0,
      y0,
      kx = transform.scale[0],
      ky = transform.scale[1],
      dx = transform.translate[0],
      dy = transform.translate[1];
  return function(point, i) {
    if (!i) x0 = y0 = 0;
    point[0] = (x0 += point[0]) * kx + dx;
    point[1] = (y0 += point[1]) * ky + dy;
    return point;
  };
}

function feature$2(topology, o) {
  return o.type === "GeometryCollection"
      ? {type: "FeatureCollection", features: o.geometries.map(function(o) { return feature$3(topology, o); })}
      : feature$3(topology, o);
}

function feature$3(topology, o) {
  var id = o.id,
      bbox = o.bbox,
      properties = o.properties == null ? {} : o.properties,
      geometry = object(topology, o);
  return id == null && bbox == null ? {type: "Feature", properties: properties, geometry: geometry}
      : bbox == null ? {type: "Feature", id: id, properties: properties, geometry: geometry}
      : {type: "Feature", id: id, bbox: bbox, properties: properties, geometry: geometry};
}

function object(topology, o) {
  var transformPoint = transform$1(topology),
      arcs = topology.arcs;

  function arc(i, points) {
    if (points.length) points.pop();
    for (var a = arcs[i < 0 ? ~i : i], k = 0, n = a.length; k < n; ++k) {
      points.push(transformPoint(a[k].slice(), k));
    }
    if (i < 0) reverse(points, n);
  }

  function point(p) {
    return transformPoint(p.slice());
  }

  function line(arcs) {
    var points = [];
    for (var i = 0, n = arcs.length; i < n; ++i) arc(arcs[i], points);
    if (points.length < 2) points.push(points[0].slice());
    return points;
  }

  function ring(arcs) {
    var points = line(arcs);
    while (points.length < 4) points.push(points[0].slice());
    return points;
  }

  function polygon(arcs) {
    return arcs.map(ring);
  }

  function geometry(o) {
    var type = o.type, coordinates;
    switch (type) {
      case "GeometryCollection": return {type: type, geometries: o.geometries.map(geometry)};
      case "Point": coordinates = point(o.coordinates); break;
      case "MultiPoint": coordinates = o.coordinates.map(point); break;
      case "LineString": coordinates = line(o.arcs); break;
      case "MultiLineString": coordinates = o.arcs.map(line); break;
      case "Polygon": coordinates = polygon(o.arcs); break;
      case "MultiPolygon": coordinates = o.arcs.map(polygon); break;
      default: return null;
    }
    return {type: type, coordinates: coordinates};
  }

  return geometry(o);
}

// export {default as bbox} from "./src/bbox";
// export {default as mesh, meshArcs} from "./src/mesh";
// export {default as merge, mergeArcs} from "./src/merge";
// export {default as neighbors} from "./src/neighbors";
// export {default as quantize} from "./src/quantize";
// export {default as transform} from "./src/transform";
// export {default as untransform} from "./src/untransform";

/**
 TopoJSON standalone (non-tiled) source
 Uses geojson-vt split into tiles client-side
*/

var TopoJSONSource =
/*#__PURE__*/
function (_GeoJSONSource) {
  _inheritsLoose(TopoJSONSource, _GeoJSONSource);

  function TopoJSONSource() {
    return _GeoJSONSource.apply(this, arguments) || this;
  }

  var _proto = TopoJSONSource.prototype;

  _proto.parseSourceData = function parseSourceData(tile, source, response) {
    var data = typeof response === 'string' ? JSON.parse(response) : response;
    data = this.toGeoJSON(data);
    var layers = this.getLayers(data);

    _GeoJSONSource.prototype.preprocessLayers.call(this, layers, tile);

    source.layers = layers;
  };

  _proto.toGeoJSON = function toGeoJSON(data) {
    // Single layer
    if (data.objects && Object.keys(data.objects).length === 1) {
      var layer = Object.keys(data.objects)[0];
      data = getTopoJSONFeature(data, data.objects[layer]);
    } // Multiple layers
    else {
        var layers = {};

        for (var key in data.objects) {
          layers[key] = getTopoJSONFeature(data, data.objects[key]);
        }

        data = layers;
      }

    return data;
  };

  return TopoJSONSource;
}(GeoJSONSource);

function getTopoJSONFeature(topology, object) {
  var feature = feature$2(topology, object); // Convert single feature to a feature collection

  if (feature.type === 'Feature') {
    feature = {
      type: 'FeatureCollection',
      features: [feature]
    };
  }

  return feature;
}
/**
 TopoJSON vector tiles
 @class TopoJSONTileSource
*/


var TopoJSONTileSource =
/*#__PURE__*/
function (_GeoJSONTileSource) {
  _inheritsLoose(TopoJSONTileSource, _GeoJSONTileSource);

  function TopoJSONTileSource(source, sources) {
    return _GeoJSONTileSource.call(this, source, sources) || this;
  }

  var _proto2 = TopoJSONTileSource.prototype;

  _proto2.parseSourceData = function parseSourceData(tile, source, response) {
    var data = typeof response === 'string' ? JSON.parse(response) : response;
    data = TopoJSONSource.prototype.toGeoJSON(data);
    this.prepareGeoJSON(data, tile, source);
  };

  return TopoJSONTileSource;
}(GeoJSONTileSource); // Check for URL tile pattern, if not found, treat as standalone GeoJSON/TopoJSON object

DataSource.register('TopoJSON', function (source) {
  return TopoJSONTileSource.urlHasTilePattern(source.url) ? TopoJSONTileSource : TopoJSONSource;
});

// add all data source types

exports.require$$0 = _wks;
exports.require$$1 = _core;
exports.global = _global;
exports.require$$1$1 = _objectDp;
exports.getKeys = _objectKeys;
exports.gOPS = _objectGops;
exports.require$$0$1 = _objectPie;
exports.toIObject = _toIobject;
exports.require$$0$2 = _objectGopn;
exports.require$$0$3 = _meta;
exports.require$$0$4 = _shared;
exports.DESCRIPTORS = _descriptors;
exports.fails = _fails;
exports.require$$22 = _objectCreate;
exports.uid = _uid;
exports.redefine = _redefine;
exports.LIBRARY = _library;
exports.$export = _export;
exports.require$$17 = _has;
exports.isObject = _isObject;
exports.isArray = _isArray;
exports.hide = _hide;
exports.setToStringTag = _setToStringTag;
exports.require$$1$2 = _objectGopd;
exports.anObject = _anObject;
exports.require$$16 = _toPrimitive;
exports.require$$9 = _propertyDesc;
exports.createCommonjsModule = createCommonjsModule;
exports.commonjsRequire = commonjsRequire;
exports.Utils = Utils;
exports.isRelativeURL = isRelativeURL;
exports.addBaseURL = addBaseURL;
exports.pathForURL = pathForURL;
exports.extensionForURL = extensionForURL;
exports._inheritsLoose = _inheritsLoose;
exports.flattenRelativeURL = flattenRelativeURL;
exports.createObjectURL = createObjectURL;
exports.isLocalURL = isLocalURL;
exports.subscribeMixin = subscribeMixin;
exports.log = log;
exports.mergeObjects = mergeObjects;
exports.isReserved = isReserved;
exports.GLSL = GLSL;
exports.TileID = TileID;
exports.Collision = Collision;
exports.Geo = Geo$1;
exports.LabelPoint = LabelPoint;
exports.LabelLineStraight = LabelLineStraight;
exports.OBB = OBB;
exports.Label = Label;
exports.WorkerBroker = WorkerBroker$1;
exports.Task = Task;
exports.Tile = Tile;
exports._createClass = _createClass;
exports.require$$0$5 = _typedArray;
exports.StyleParser = StyleParser;
exports.Texture = Texture;
exports.debugSettings = debugSettings$1;
exports.debugSumLayerStats = debugSumLayerStats;
exports.ShaderProgram = ShaderProgram;
exports.VertexArrayObject = VertexArrayObject;
exports.TextCanvas = TextCanvas;
exports.DataSource = DataSource;
exports.Light = Light;
exports.FontManager = FontManager;
exports.FeatureSelection = FeatureSelection;
exports.View = View;
exports.StyleManager = StyleManager;
exports.Style = Style;
exports.sliceObject = sliceObject;
exports.Thread = Thread;
exports.mergeDebugSettings = mergeDebugSettings;
exports.version = version$1;
exports.Vector = Vector$1;
exports.VertexData = VertexData;
exports.Material = Material;
exports.VertexElements = VertexElements;
exports.compileFunctionStrings = compileFunctionStrings;
exports.parseLayers = parseLayers;
exports.buildFilter = buildFilter;
exports.FilterOptions = FilterOptions;
exports.layerCache = layerCache;
exports.cache = cache;
exports.clearFunctionStringCache = clearFunctionStringCache;

});

define(['./shared.js'], function (__chunk_1) { 'use strict';

var SceneWorker = Object.assign(self, {
  FeatureSelection: __chunk_1.FeatureSelection,
  sources: {},
  styles: {},
  layers: {},
  tiles: {},
  // Initialize worker
  init: function init(scene_id, worker_id, num_workers, log_level, device_pixel_ratio, has_element_index_unit, external_scripts) {
    this.scene_id = scene_id;
    this._worker_id = worker_id;
    this.num_workers = num_workers;
    __chunk_1.log.setLevel(log_level);
    __chunk_1.Utils.device_pixel_ratio = device_pixel_ratio;
    __chunk_1.VertexElements.setElementIndexUint(has_element_index_unit);
    __chunk_1.FeatureSelection.setPrefix(this._worker_id);
    this.style_manager = new __chunk_1.StyleManager();
    this.importExternalScripts(external_scripts);
    __chunk_1.Label.id_prefix = worker_id;
    __chunk_1.Label.id_multiplier = num_workers;
    return worker_id;
  },
  // Import custom external scripts
  importExternalScripts: function importExternalScripts(scripts) {
    var _this = this;

    if (scripts.length === 0) {
      return;
    }

    __chunk_1.log('debug', 'loading custom data source scripts in worker:', scripts); // `window` is already shimmed to allow compatibility with some other libraries (e.g. FontFaceObserver)
    // So there's an extra dance here to look for any additional `window` properties added by these script imports,
    // then add them to the worker `self` scope.

    var prev_names = Object.getOwnPropertyNames(window);
    importScripts.apply(void 0, scripts);
    Object.getOwnPropertyNames(window).forEach(function (prop) {
      if (prev_names.indexOf(prop) === -1) {
        _this[prop] = window[prop]; // new property added to window, also add it to self
      }
    });
  },
  // Starts a config refresh
  updateConfig: function updateConfig(_ref, debug) {
    var config = _ref.config,
        generation = _ref.generation,
        introspection = _ref.introspection;
    config = JSON.parse(config);
    __chunk_1.mergeDebugSettings(debug);
    this.generation = generation;
    this.introspection = introspection; // Expand global properties

    this.global = __chunk_1.compileFunctionStrings(config.global); // Create data sources

    this.createDataSources(config); // Expand styles

    config.styles = __chunk_1.compileFunctionStrings(config.styles, __chunk_1.StyleParser.wrapFunction);
    this.styles = this.style_manager.build(config.styles);
    this.style_manager.initStyles({
      generation: this.generation,
      styles: this.styles,
      sources: this.sources,
      introspection: this.introspection
    }); // Parse each top-level layer as a separate tree

    this.layers = __chunk_1.parseLayers(config.layers, this.style_manager.styles); // Sync tetxure info from main thread

    this.syncing_textures = this.syncTextures(config.textures); // Return promise for when config refresh finishes

    this.configuring = this.syncing_textures.then(function () {
      __chunk_1.log('debug', 'updated config');
    });
    return this.configuring;
  },
  // Create data sources and clear tile cache if necessary
  createDataSources: function createDataSources(config) {
    var _this2 = this;

    // Save and compare previous sources
    this.last_config_sources = this.config_sources || {};
    this.config_sources = config.sources;
    var last_sources = this.sources;
    var changed = []; // Parse new sources

    this.sources = {}; // clear previous sources

    for (var name in config.sources) {
      if (JSON.stringify(this.last_config_sources[name]) === JSON.stringify(config.sources[name])) {
        this.sources[name] = last_sources[name];
        continue;
      } // compile any user-defined JS functions


      config.sources[name] = __chunk_1.compileFunctionStrings(config.sources[name]);
      var source = void 0;

      try {
        source = __chunk_1.DataSource.create(Object.assign({}, config.sources[name], {
          name: name
        }), this.sources);
      } catch (e) {
        continue;
      }

      if (!source) {
        continue;
      }

      this.sources[name] = source;
      changed.push(name);
    } // Clear tile cache for data sources that changed


    changed.forEach(function (source) {
      for (var t in _this2.tiles) {
        if (_this2.tiles[t].source === source) {
          delete _this2.tiles[t];
        }
      }
    });
  },
  // Returns a promise that fulfills when config refresh is finished
  awaitConfiguration: function awaitConfiguration() {
    return this.configuring;
  },
  // Build a tile: load from tile source if building for first time, otherwise rebuild with existing data
  buildTile: function buildTile(_ref2) {
    var _this3 = this;

    var tile = _ref2.tile;

    // Tile cached?
    if (this.getTile(tile.key) != null) {
      // Already loading?
      if (this.getTile(tile.key).loading === true) {
        return;
      }
    } // Update tile cache


    tile = this.tiles[tile.key] = Object.assign(this.getTile(tile.key) || {}, tile); // Update config (styles, etc.), then build tile

    return this.awaitConfiguration().then(function () {
      // First time building the tile
      if (tile.loaded !== true) {
        tile.loading = true;
        tile.loaded = false;
        tile.error = null;

        _this3.loadTileSourceData(tile).then(function () {
          if (!_this3.getTile(tile.key)) {
            __chunk_1.log('trace', "stop tile build after data source load because tile was removed: " + tile.key);
            return;
          } // Warn and continue on data source error


          if (tile.source_data.error) {
            __chunk_1.log('warn', "tile load error(s) for " + tile.key + ": " + tile.source_data.error);
          }

          tile.loading = false;
          tile.loaded = true;
          __chunk_1.Tile.buildGeometry(tile, _this3);
        }).catch(function (error) {
          tile.loading = false;
          tile.loaded = false;
          tile.error = error.stack;
          __chunk_1.log('error', "tile load error for " + tile.key + ": " + tile.error); // Send error to main thread

          __chunk_1.WorkerBroker.postMessage("TileManager_" + _this3.scene_id + ".buildTileError", __chunk_1.Tile.slice(tile));
        });
      } // Tile already loaded, just rebuild
      else {
          __chunk_1.log('trace', "used worker cache for tile " + tile.key); // Build geometry

          try {
            __chunk_1.Tile.buildGeometry(tile, _this3);
          } catch (error) {
            // Send error to main thread
            tile.error = error.toString();
            __chunk_1.WorkerBroker.postMessage("TileManager_" + _this3.scene_id + ".buildTileError", __chunk_1.Tile.slice(tile));
          }
        }
    });
  },
  // Load this tile's data source, or copy from an existing tile's data
  loadTileSourceData: function loadTileSourceData(tile) {
    var source = this.sources[tile.source];

    if (source) {
      // Search existing tiles to see if we can reuse existing source data for this coordinate
      for (var t in this.tiles) {
        var ref = this.tiles[t];

        if (ref.source === tile.source && ref.coords.key === tile.coords.key && ref.loaded) {
          return Promise.resolve(source.copyTileData(ref, tile));
        }
      } // Load new tile data (no existing data found)


      return source.load(tile);
    } else {
      tile.source_data = {};
      return Promise.resolve(tile);
    }
  },
  getTile: function getTile(key) {
    return this.tiles[key];
  },
  // Remove tile
  removeTile: function removeTile(key) {
    var tile = this.tiles[key];

    if (tile != null) {
      // Cancel if loading
      if (tile.loading === true) {
        __chunk_1.log('trace', "cancel tile load for " + key);
        tile.loading = false;
        __chunk_1.Tile.cancel(tile);
      } // Remove from cache


      __chunk_1.FeatureSelection.clearTile(key);
      delete this.tiles[key];
      __chunk_1.log('trace', "remove tile from cache for " + key);
    }
  },
  // Query features within visible tiles, with optional filter conditions
  queryFeatures: function queryFeatures(_ref3) {
    var _this4 = this;

    var filter = _ref3.filter,
        visible = _ref3.visible,
        geometry = _ref3.geometry,
        tile_keys = _ref3.tile_keys;
    var features = [];
    var tiles = tile_keys.map(function (t) {
      return _this4.tiles[t];
    }).filter(function (t) {
      return t && t.loaded;
    }); // Compile feature filter

    if (filter != null) {
      filter = ['{', '['].indexOf(filter[0]) > -1 ? JSON.parse(filter) : filter; // de-serialize if looks like an object

      filter = __chunk_1.compileFunctionStrings(filter, __chunk_1.StyleParser.wrapFunction);
    }

    filter = __chunk_1.buildFilter(filter, __chunk_1.FilterOptions);
    tiles.forEach(function (tile) {
      var _loop = function _loop(layer) {
        var data = tile.source_data.layers[layer];
        data.features.forEach(function (feature) {
          // Optionally check if feature is visible (e.g. was rendered for current generation)
          var feature_visible = feature.generation === _this4.generation;

          if (visible === true && !feature_visible || visible === false && feature_visible) {
            return;
          } // Apply feature filter


          var context = __chunk_1.StyleParser.getFeatureParseContext(feature, tile, _this4.global);
          context.source = tile.source; // add data source name

          context.layer = layer; // add data source layer name

          if (!filter(context)) {
            return;
          } // Info to return with each feature


          var subset = {
            type: feature.type,
            properties: Object.assign({}, feature.properties, {
              $source: context.source,
              $layer: context.layer,
              $geometry: context.geometry,
              $visible: feature_visible
            })
          }; // Optionally include geometry in response

          if (geometry === true) {
            // Transform back to lat lng (copy geometry to avoid local modification)
            subset.geometry = __chunk_1.Geo.copyGeometry(feature.geometry);
            __chunk_1.Geo.tileSpaceToLatlng(subset.geometry, tile.coords.z, tile.min);
          }

          features.push(subset);
        });
      };

      for (var layer in tile.source_data.layers) {
        _loop(layer);
      }
    });
    return features;
  },
  // Get a feature from the selection map
  getFeatureSelection: function getFeatureSelection(_temp) {
    var _ref4 = _temp === void 0 ? {} : _temp,
        id = _ref4.id,
        key = _ref4.key;

    var selection = __chunk_1.FeatureSelection.map[key];
    return {
      id: id,
      feature: selection && selection.feature
    };
  },
  // Resets the feature selection state
  resetFeatureSelection: function resetFeatureSelection(sources) {
    if (sources === void 0) {
      sources = null;
    }

    __chunk_1.FeatureSelection.reset(sources);
  },
  // Selection map size for this worker
  getFeatureSelectionMapSize: function getFeatureSelectionMapSize() {
    return __chunk_1.FeatureSelection.getMapSize();
  },
  // Texture info needs to be synced from main thread, e.g. width/height, which we only know after the texture loads
  syncTextures: function syncTextures(tex_config) {
    var textures = [];

    if (tex_config) {
      textures.push.apply(textures, Object.keys(tex_config));
    }

    __chunk_1.log('trace', 'sync textures to worker:', textures);

    if (textures.length > 0) {
      return __chunk_1.Texture.syncTexturesToWorker(textures);
    }

    return Promise.resolve();
  },
  // Sync device pixel ratio from main thread
  updateDevicePixelRatio: function updateDevicePixelRatio(device_pixel_ratio) {
    __chunk_1.Utils.device_pixel_ratio = device_pixel_ratio;
  },
  clearFunctionStringCache: function clearFunctionStringCache() {
    __chunk_1.clearFunctionStringCache();
  },
  // Profiling helpers
  profile: function profile(name) {
    console.profile("worker " + this._worker_id + ": " + name); // eslint-disable-line no-console
  },
  profileEnd: function profileEnd(name) {
    console.profileEnd("worker " + this._worker_id + ": " + name); // eslint-disable-line no-console
  },
  debug: {
    debugSettings: __chunk_1.debugSettings,
    layerCache: __chunk_1.layerCache,
    functionStringCache: __chunk_1.cache
  }
});
__chunk_1.WorkerBroker.addTarget('self', SceneWorker);

});

define(['./shared.js'], function (__chunk_1) { 'use strict';

var f = __chunk_1.require$$0;

var _wksExt = {
	f: f
};

var defineProperty = __chunk_1.require$$1$1.f;
var _wksDefine = function (name) {
  var $Symbol = __chunk_1.require$$1.Symbol || (__chunk_1.require$$1.Symbol = __chunk_1.global.Symbol || {});
  if (name.charAt(0) != '_' && !(name in $Symbol)) defineProperty($Symbol, name, { value: _wksExt.f(name) });
};

_wksDefine('asyncIterator');

// all enumerable object keys, includes symbols



var _enumKeys = function (it) {
  var result = __chunk_1.getKeys(it);
  var getSymbols = __chunk_1.gOPS.f;
  if (getSymbols) {
    var symbols = getSymbols(it);
    var isEnum = __chunk_1.require$$0$1.f;
    var i = 0;
    var key;
    while (symbols.length > i) if (isEnum.call(it, key = symbols[i++])) result.push(key);
  } return result;
};

// fallback for IE11 buggy Object.getOwnPropertyNames with iframe and window

var gOPN = __chunk_1.require$$0$2.f;
var toString = {}.toString;

var windowNames = typeof window == 'object' && window && Object.getOwnPropertyNames
  ? Object.getOwnPropertyNames(window) : [];

var getWindowNames = function (it) {
  try {
    return gOPN(it);
  } catch (e) {
    return windowNames.slice();
  }
};

var f$1 = function getOwnPropertyNames(it) {
  return windowNames && toString.call(it) == '[object Window]' ? getWindowNames(it) : gOPN(__chunk_1.toIObject(it));
};

var _objectGopnExt = {
	f: f$1
};

// ECMAScript 6 symbols shim





var META = __chunk_1.require$$0$3.KEY;



















var gOPD = __chunk_1.require$$1$2.f;
var dP = __chunk_1.require$$1$1.f;
var gOPN$1 = _objectGopnExt.f;
var $Symbol = __chunk_1.global.Symbol;
var $JSON = __chunk_1.global.JSON;
var _stringify = $JSON && $JSON.stringify;
var PROTOTYPE = 'prototype';
var HIDDEN = __chunk_1.require$$0('_hidden');
var TO_PRIMITIVE = __chunk_1.require$$0('toPrimitive');
var isEnum = {}.propertyIsEnumerable;
var SymbolRegistry = __chunk_1.require$$0$4('symbol-registry');
var AllSymbols = __chunk_1.require$$0$4('symbols');
var OPSymbols = __chunk_1.require$$0$4('op-symbols');
var ObjectProto = Object[PROTOTYPE];
var USE_NATIVE = typeof $Symbol == 'function';
var QObject = __chunk_1.global.QObject;
// Don't use setters in Qt Script, https://github.com/zloirock/core-js/issues/173
var setter = !QObject || !QObject[PROTOTYPE] || !QObject[PROTOTYPE].findChild;

// fallback for old Android, https://code.google.com/p/v8/issues/detail?id=687
var setSymbolDesc = __chunk_1.DESCRIPTORS && __chunk_1.fails(function () {
  return __chunk_1.require$$22(dP({}, 'a', {
    get: function () { return dP(this, 'a', { value: 7 }).a; }
  })).a != 7;
}) ? function (it, key, D) {
  var protoDesc = gOPD(ObjectProto, key);
  if (protoDesc) delete ObjectProto[key];
  dP(it, key, D);
  if (protoDesc && it !== ObjectProto) dP(ObjectProto, key, protoDesc);
} : dP;

var wrap = function (tag) {
  var sym = AllSymbols[tag] = __chunk_1.require$$22($Symbol[PROTOTYPE]);
  sym._k = tag;
  return sym;
};

var isSymbol = USE_NATIVE && typeof $Symbol.iterator == 'symbol' ? function (it) {
  return typeof it == 'symbol';
} : function (it) {
  return it instanceof $Symbol;
};

var $defineProperty = function defineProperty(it, key, D) {
  if (it === ObjectProto) $defineProperty(OPSymbols, key, D);
  __chunk_1.anObject(it);
  key = __chunk_1.require$$16(key, true);
  __chunk_1.anObject(D);
  if (__chunk_1.require$$17(AllSymbols, key)) {
    if (!D.enumerable) {
      if (!__chunk_1.require$$17(it, HIDDEN)) dP(it, HIDDEN, __chunk_1.require$$9(1, {}));
      it[HIDDEN][key] = true;
    } else {
      if (__chunk_1.require$$17(it, HIDDEN) && it[HIDDEN][key]) it[HIDDEN][key] = false;
      D = __chunk_1.require$$22(D, { enumerable: __chunk_1.require$$9(0, false) });
    } return setSymbolDesc(it, key, D);
  } return dP(it, key, D);
};
var $defineProperties = function defineProperties(it, P) {
  __chunk_1.anObject(it);
  var keys = _enumKeys(P = __chunk_1.toIObject(P));
  var i = 0;
  var l = keys.length;
  var key;
  while (l > i) $defineProperty(it, key = keys[i++], P[key]);
  return it;
};
var $create = function create(it, P) {
  return P === undefined ? __chunk_1.require$$22(it) : $defineProperties(__chunk_1.require$$22(it), P);
};
var $propertyIsEnumerable = function propertyIsEnumerable(key) {
  var E = isEnum.call(this, key = __chunk_1.require$$16(key, true));
  if (this === ObjectProto && __chunk_1.require$$17(AllSymbols, key) && !__chunk_1.require$$17(OPSymbols, key)) return false;
  return E || !__chunk_1.require$$17(this, key) || !__chunk_1.require$$17(AllSymbols, key) || __chunk_1.require$$17(this, HIDDEN) && this[HIDDEN][key] ? E : true;
};
var $getOwnPropertyDescriptor = function getOwnPropertyDescriptor(it, key) {
  it = __chunk_1.toIObject(it);
  key = __chunk_1.require$$16(key, true);
  if (it === ObjectProto && __chunk_1.require$$17(AllSymbols, key) && !__chunk_1.require$$17(OPSymbols, key)) return;
  var D = gOPD(it, key);
  if (D && __chunk_1.require$$17(AllSymbols, key) && !(__chunk_1.require$$17(it, HIDDEN) && it[HIDDEN][key])) D.enumerable = true;
  return D;
};
var $getOwnPropertyNames = function getOwnPropertyNames(it) {
  var names = gOPN$1(__chunk_1.toIObject(it));
  var result = [];
  var i = 0;
  var key;
  while (names.length > i) {
    if (!__chunk_1.require$$17(AllSymbols, key = names[i++]) && key != HIDDEN && key != META) result.push(key);
  } return result;
};
var $getOwnPropertySymbols = function getOwnPropertySymbols(it) {
  var IS_OP = it === ObjectProto;
  var names = gOPN$1(IS_OP ? OPSymbols : __chunk_1.toIObject(it));
  var result = [];
  var i = 0;
  var key;
  while (names.length > i) {
    if (__chunk_1.require$$17(AllSymbols, key = names[i++]) && (IS_OP ? __chunk_1.require$$17(ObjectProto, key) : true)) result.push(AllSymbols[key]);
  } return result;
};

// 19.4.1.1 Symbol([description])
if (!USE_NATIVE) {
  $Symbol = function Symbol() {
    if (this instanceof $Symbol) throw TypeError('Symbol is not a constructor!');
    var tag = __chunk_1.uid(arguments.length > 0 ? arguments[0] : undefined);
    var $set = function (value) {
      if (this === ObjectProto) $set.call(OPSymbols, value);
      if (__chunk_1.require$$17(this, HIDDEN) && __chunk_1.require$$17(this[HIDDEN], tag)) this[HIDDEN][tag] = false;
      setSymbolDesc(this, tag, __chunk_1.require$$9(1, value));
    };
    if (__chunk_1.DESCRIPTORS && setter) setSymbolDesc(ObjectProto, tag, { configurable: true, set: $set });
    return wrap(tag);
  };
  __chunk_1.redefine($Symbol[PROTOTYPE], 'toString', function toString() {
    return this._k;
  });

  __chunk_1.require$$1$2.f = $getOwnPropertyDescriptor;
  __chunk_1.require$$1$1.f = $defineProperty;
  __chunk_1.require$$0$2.f = _objectGopnExt.f = $getOwnPropertyNames;
  __chunk_1.require$$0$1.f = $propertyIsEnumerable;
  __chunk_1.gOPS.f = $getOwnPropertySymbols;

  if (__chunk_1.DESCRIPTORS && !__chunk_1.LIBRARY) {
    __chunk_1.redefine(ObjectProto, 'propertyIsEnumerable', $propertyIsEnumerable, true);
  }

  _wksExt.f = function (name) {
    return wrap(__chunk_1.require$$0(name));
  };
}

__chunk_1.$export(__chunk_1.$export.G + __chunk_1.$export.W + __chunk_1.$export.F * !USE_NATIVE, { Symbol: $Symbol });

for (var es6Symbols = (
  // 19.4.2.2, 19.4.2.3, 19.4.2.4, 19.4.2.6, 19.4.2.8, 19.4.2.9, 19.4.2.10, 19.4.2.11, 19.4.2.12, 19.4.2.13, 19.4.2.14
  'hasInstance,isConcatSpreadable,iterator,match,replace,search,species,split,toPrimitive,toStringTag,unscopables'
).split(','), j = 0; es6Symbols.length > j;)__chunk_1.require$$0(es6Symbols[j++]);

for (var wellKnownSymbols = __chunk_1.getKeys(__chunk_1.require$$0.store), k = 0; wellKnownSymbols.length > k;) _wksDefine(wellKnownSymbols[k++]);

__chunk_1.$export(__chunk_1.$export.S + __chunk_1.$export.F * !USE_NATIVE, 'Symbol', {
  // 19.4.2.1 Symbol.for(key)
  'for': function (key) {
    return __chunk_1.require$$17(SymbolRegistry, key += '')
      ? SymbolRegistry[key]
      : SymbolRegistry[key] = $Symbol(key);
  },
  // 19.4.2.5 Symbol.keyFor(sym)
  keyFor: function keyFor(sym) {
    if (!isSymbol(sym)) throw TypeError(sym + ' is not a symbol!');
    for (var key in SymbolRegistry) if (SymbolRegistry[key] === sym) return key;
  },
  useSetter: function () { setter = true; },
  useSimple: function () { setter = false; }
});

__chunk_1.$export(__chunk_1.$export.S + __chunk_1.$export.F * !USE_NATIVE, 'Object', {
  // 19.1.2.2 Object.create(O [, Properties])
  create: $create,
  // 19.1.2.4 Object.defineProperty(O, P, Attributes)
  defineProperty: $defineProperty,
  // 19.1.2.3 Object.defineProperties(O, Properties)
  defineProperties: $defineProperties,
  // 19.1.2.6 Object.getOwnPropertyDescriptor(O, P)
  getOwnPropertyDescriptor: $getOwnPropertyDescriptor,
  // 19.1.2.7 Object.getOwnPropertyNames(O)
  getOwnPropertyNames: $getOwnPropertyNames,
  // 19.1.2.8 Object.getOwnPropertySymbols(O)
  getOwnPropertySymbols: $getOwnPropertySymbols
});

// 24.3.2 JSON.stringify(value [, replacer [, space]])
$JSON && __chunk_1.$export(__chunk_1.$export.S + __chunk_1.$export.F * (!USE_NATIVE || __chunk_1.fails(function () {
  var S = $Symbol();
  // MS Edge converts symbol values to JSON as {}
  // WebKit converts symbol values to JSON as null
  // V8 throws on boxed symbols
  return _stringify([S]) != '[null]' || _stringify({ a: S }) != '{}' || _stringify(Object(S)) != '{}';
})), 'JSON', {
  stringify: function stringify(it) {
    var args = [it];
    var i = 1;
    var replacer, $replacer;
    while (arguments.length > i) args.push(arguments[i++]);
    $replacer = replacer = args[1];
    if (!__chunk_1.isObject(replacer) && it === undefined || isSymbol(it)) return; // IE8 returns string on undefined
    if (!__chunk_1.isArray(replacer)) replacer = function (key, value) {
      if (typeof $replacer == 'function') value = $replacer.call(this, key, value);
      if (!isSymbol(value)) return value;
    };
    args[1] = replacer;
    return _stringify.apply($JSON, args);
  }
});

// 19.4.3.4 Symbol.prototype[@@toPrimitive](hint)
$Symbol[PROTOTYPE][TO_PRIMITIVE] || __chunk_1.hide($Symbol[PROTOTYPE], TO_PRIMITIVE, $Symbol[PROTOTYPE].valueOf);
// 19.4.3.5 Symbol.prototype[@@toStringTag]
__chunk_1.setToStringTag($Symbol, 'Symbol');
// 20.2.1.9 Math[@@toStringTag]
__chunk_1.setToStringTag(Math, 'Math', true);
// 24.3.3 JSON[@@toStringTag]
__chunk_1.setToStringTag(__chunk_1.global.JSON, 'JSON', true);

// WebGL context wrapper
var Context;
var Context$1 = Context = {};
var context_id = 0; // Setup a WebGL context
// If no canvas element is provided, one is created and added to the document body

Context.getContext = function getContext(canvas, options) {
  var fullscreen = false;

  if (canvas == null) {
    canvas = document.createElement('canvas');
    canvas.style.position = 'absolute';
    canvas.style.top = 0;
    canvas.style.left = 0;
    canvas.style.zIndex = -1;
    document.body.appendChild(canvas);
    fullscreen = true;
  }

  var gl = canvas.getContext('webgl', options) || canvas.getContext('experimental-webgl', options);

  if (!gl) {
    throw new Error('Couldn\'t create WebGL context.');
  }

  gl._tangram_id = context_id++;

  if (!fullscreen) {
    Context.resize(gl, parseFloat(canvas.style.width), parseFloat(canvas.style.height), options.device_pixel_ratio);
  } else {
    Context.resize(gl, window.innerWidth, window.innerHeight, options.device_pixel_ratio);
    window.addEventListener('resize', function () {
      Context.resize(gl, window.innerWidth, window.innerHeight, options.device_pixel_ratio);
    });
  }

  return gl;
};

Context.resize = function (gl, width, height, device_pixel_ratio) {
  device_pixel_ratio = device_pixel_ratio || window.devicePixelRatio || 1;
  gl.canvas.style.width = width + 'px';
  gl.canvas.style.height = height + 'px';
  gl.canvas.width = Math.round(width * device_pixel_ratio);
  gl.canvas.height = Math.round(height * device_pixel_ratio);
  gl.bindFramebuffer(gl.FRAMEBUFFER, null);
  gl.viewport(0, 0, gl.canvas.width, gl.canvas.height);
};

var global$1 = (typeof global !== "undefined" ? global :
            typeof self !== "undefined" ? self :
            typeof window !== "undefined" ? window : {});

var lookup = [];
var revLookup = [];
var Arr = typeof Uint8Array !== 'undefined' ? Uint8Array : Array;
var inited = false;
function init () {
  inited = true;
  var code = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
  for (var i = 0, len = code.length; i < len; ++i) {
    lookup[i] = code[i];
    revLookup[code.charCodeAt(i)] = i;
  }

  revLookup['-'.charCodeAt(0)] = 62;
  revLookup['_'.charCodeAt(0)] = 63;
}

function toByteArray (b64) {
  if (!inited) {
    init();
  }
  var i, j, l, tmp, placeHolders, arr;
  var len = b64.length;

  if (len % 4 > 0) {
    throw new Error('Invalid string. Length must be a multiple of 4')
  }

  // the number of equal signs (place holders)
  // if there are two placeholders, than the two characters before it
  // represent one byte
  // if there is only one, then the three characters before it represent 2 bytes
  // this is just a cheap hack to not do indexOf twice
  placeHolders = b64[len - 2] === '=' ? 2 : b64[len - 1] === '=' ? 1 : 0;

  // base64 is 4/3 + up to two characters of the original data
  arr = new Arr(len * 3 / 4 - placeHolders);

  // if there are placeholders, only get up to the last complete 4 chars
  l = placeHolders > 0 ? len - 4 : len;

  var L = 0;

  for (i = 0, j = 0; i < l; i += 4, j += 3) {
    tmp = (revLookup[b64.charCodeAt(i)] << 18) | (revLookup[b64.charCodeAt(i + 1)] << 12) | (revLookup[b64.charCodeAt(i + 2)] << 6) | revLookup[b64.charCodeAt(i + 3)];
    arr[L++] = (tmp >> 16) & 0xFF;
    arr[L++] = (tmp >> 8) & 0xFF;
    arr[L++] = tmp & 0xFF;
  }

  if (placeHolders === 2) {
    tmp = (revLookup[b64.charCodeAt(i)] << 2) | (revLookup[b64.charCodeAt(i + 1)] >> 4);
    arr[L++] = tmp & 0xFF;
  } else if (placeHolders === 1) {
    tmp = (revLookup[b64.charCodeAt(i)] << 10) | (revLookup[b64.charCodeAt(i + 1)] << 4) | (revLookup[b64.charCodeAt(i + 2)] >> 2);
    arr[L++] = (tmp >> 8) & 0xFF;
    arr[L++] = tmp & 0xFF;
  }

  return arr
}

function tripletToBase64 (num) {
  return lookup[num >> 18 & 0x3F] + lookup[num >> 12 & 0x3F] + lookup[num >> 6 & 0x3F] + lookup[num & 0x3F]
}

function encodeChunk (uint8, start, end) {
  var tmp;
  var output = [];
  for (var i = start; i < end; i += 3) {
    tmp = (uint8[i] << 16) + (uint8[i + 1] << 8) + (uint8[i + 2]);
    output.push(tripletToBase64(tmp));
  }
  return output.join('')
}

function fromByteArray (uint8) {
  if (!inited) {
    init();
  }
  var tmp;
  var len = uint8.length;
  var extraBytes = len % 3; // if we have 1 byte left, pad 2 bytes
  var output = '';
  var parts = [];
  var maxChunkLength = 16383; // must be multiple of 3

  // go through the array every three bytes, we'll deal with trailing stuff later
  for (var i = 0, len2 = len - extraBytes; i < len2; i += maxChunkLength) {
    parts.push(encodeChunk(uint8, i, (i + maxChunkLength) > len2 ? len2 : (i + maxChunkLength)));
  }

  // pad the end with zeros, but make sure to not forget the extra bytes
  if (extraBytes === 1) {
    tmp = uint8[len - 1];
    output += lookup[tmp >> 2];
    output += lookup[(tmp << 4) & 0x3F];
    output += '==';
  } else if (extraBytes === 2) {
    tmp = (uint8[len - 2] << 8) + (uint8[len - 1]);
    output += lookup[tmp >> 10];
    output += lookup[(tmp >> 4) & 0x3F];
    output += lookup[(tmp << 2) & 0x3F];
    output += '=';
  }

  parts.push(output);

  return parts.join('')
}

function read (buffer, offset, isLE, mLen, nBytes) {
  var e, m;
  var eLen = nBytes * 8 - mLen - 1;
  var eMax = (1 << eLen) - 1;
  var eBias = eMax >> 1;
  var nBits = -7;
  var i = isLE ? (nBytes - 1) : 0;
  var d = isLE ? -1 : 1;
  var s = buffer[offset + i];

  i += d;

  e = s & ((1 << (-nBits)) - 1);
  s >>= (-nBits);
  nBits += eLen;
  for (; nBits > 0; e = e * 256 + buffer[offset + i], i += d, nBits -= 8) {}

  m = e & ((1 << (-nBits)) - 1);
  e >>= (-nBits);
  nBits += mLen;
  for (; nBits > 0; m = m * 256 + buffer[offset + i], i += d, nBits -= 8) {}

  if (e === 0) {
    e = 1 - eBias;
  } else if (e === eMax) {
    return m ? NaN : ((s ? -1 : 1) * Infinity)
  } else {
    m = m + Math.pow(2, mLen);
    e = e - eBias;
  }
  return (s ? -1 : 1) * m * Math.pow(2, e - mLen)
}

function write (buffer, value, offset, isLE, mLen, nBytes) {
  var e, m, c;
  var eLen = nBytes * 8 - mLen - 1;
  var eMax = (1 << eLen) - 1;
  var eBias = eMax >> 1;
  var rt = (mLen === 23 ? Math.pow(2, -24) - Math.pow(2, -77) : 0);
  var i = isLE ? 0 : (nBytes - 1);
  var d = isLE ? 1 : -1;
  var s = value < 0 || (value === 0 && 1 / value < 0) ? 1 : 0;

  value = Math.abs(value);

  if (isNaN(value) || value === Infinity) {
    m = isNaN(value) ? 1 : 0;
    e = eMax;
  } else {
    e = Math.floor(Math.log(value) / Math.LN2);
    if (value * (c = Math.pow(2, -e)) < 1) {
      e--;
      c *= 2;
    }
    if (e + eBias >= 1) {
      value += rt / c;
    } else {
      value += rt * Math.pow(2, 1 - eBias);
    }
    if (value * c >= 2) {
      e++;
      c /= 2;
    }

    if (e + eBias >= eMax) {
      m = 0;
      e = eMax;
    } else if (e + eBias >= 1) {
      m = (value * c - 1) * Math.pow(2, mLen);
      e = e + eBias;
    } else {
      m = value * Math.pow(2, eBias - 1) * Math.pow(2, mLen);
      e = 0;
    }
  }

  for (; mLen >= 8; buffer[offset + i] = m & 0xff, i += d, m /= 256, mLen -= 8) {}

  e = (e << mLen) | m;
  eLen += mLen;
  for (; eLen > 0; buffer[offset + i] = e & 0xff, i += d, e /= 256, eLen -= 8) {}

  buffer[offset + i - d] |= s * 128;
}

var toString$1 = {}.toString;

var isArray = Array.isArray || function (arr) {
  return toString$1.call(arr) == '[object Array]';
};

var INSPECT_MAX_BYTES = 50;

/**
 * If `Buffer.TYPED_ARRAY_SUPPORT`:
 *   === true    Use Uint8Array implementation (fastest)
 *   === false   Use Object implementation (most compatible, even IE6)
 *
 * Browsers that support typed arrays are IE 10+, Firefox 4+, Chrome 7+, Safari 5.1+,
 * Opera 11.6+, iOS 4.2+.
 *
 * Due to various browser bugs, sometimes the Object implementation will be used even
 * when the browser supports typed arrays.
 *
 * Note:
 *
 *   - Firefox 4-29 lacks support for adding new properties to `Uint8Array` instances,
 *     See: https://bugzilla.mozilla.org/show_bug.cgi?id=695438.
 *
 *   - Chrome 9-10 is missing the `TypedArray.prototype.subarray` function.
 *
 *   - IE10 has a broken `TypedArray.prototype.subarray` function which returns arrays of
 *     incorrect length in some situations.

 * We detect these buggy browsers and set `Buffer.TYPED_ARRAY_SUPPORT` to `false` so they
 * get the Object implementation, which is slower but behaves correctly.
 */
Buffer.TYPED_ARRAY_SUPPORT = global$1.TYPED_ARRAY_SUPPORT !== undefined
  ? global$1.TYPED_ARRAY_SUPPORT
  : true;

/*
 * Export kMaxLength after typed array support is determined.
 */
var _kMaxLength = kMaxLength();

function kMaxLength () {
  return Buffer.TYPED_ARRAY_SUPPORT
    ? 0x7fffffff
    : 0x3fffffff
}

function createBuffer (that, length) {
  if (kMaxLength() < length) {
    throw new RangeError('Invalid typed array length')
  }
  if (Buffer.TYPED_ARRAY_SUPPORT) {
    // Return an augmented `Uint8Array` instance, for best performance
    that = new Uint8Array(length);
    that.__proto__ = Buffer.prototype;
  } else {
    // Fallback: Return an object instance of the Buffer class
    if (that === null) {
      that = new Buffer(length);
    }
    that.length = length;
  }

  return that
}

/**
 * The Buffer constructor returns instances of `Uint8Array` that have their
 * prototype changed to `Buffer.prototype`. Furthermore, `Buffer` is a subclass of
 * `Uint8Array`, so the returned instances will have all the node `Buffer` methods
 * and the `Uint8Array` methods. Square bracket notation works as expected -- it
 * returns a single octet.
 *
 * The `Uint8Array` prototype remains unmodified.
 */

function Buffer (arg, encodingOrOffset, length) {
  if (!Buffer.TYPED_ARRAY_SUPPORT && !(this instanceof Buffer)) {
    return new Buffer(arg, encodingOrOffset, length)
  }

  // Common case.
  if (typeof arg === 'number') {
    if (typeof encodingOrOffset === 'string') {
      throw new Error(
        'If encoding is specified then the first argument must be a string'
      )
    }
    return allocUnsafe(this, arg)
  }
  return from(this, arg, encodingOrOffset, length)
}

Buffer.poolSize = 8192; // not used by this implementation

// TODO: Legacy, not needed anymore. Remove in next major version.
Buffer._augment = function (arr) {
  arr.__proto__ = Buffer.prototype;
  return arr
};

function from (that, value, encodingOrOffset, length) {
  if (typeof value === 'number') {
    throw new TypeError('"value" argument must not be a number')
  }

  if (typeof ArrayBuffer !== 'undefined' && value instanceof ArrayBuffer) {
    return fromArrayBuffer(that, value, encodingOrOffset, length)
  }

  if (typeof value === 'string') {
    return fromString(that, value, encodingOrOffset)
  }

  return fromObject(that, value)
}

/**
 * Functionally equivalent to Buffer(arg, encoding) but throws a TypeError
 * if value is a number.
 * Buffer.from(str[, encoding])
 * Buffer.from(array)
 * Buffer.from(buffer)
 * Buffer.from(arrayBuffer[, byteOffset[, length]])
 **/
Buffer.from = function (value, encodingOrOffset, length) {
  return from(null, value, encodingOrOffset, length)
};

if (Buffer.TYPED_ARRAY_SUPPORT) {
  Buffer.prototype.__proto__ = Uint8Array.prototype;
  Buffer.__proto__ = Uint8Array;
}

function assertSize (size) {
  if (typeof size !== 'number') {
    throw new TypeError('"size" argument must be a number')
  } else if (size < 0) {
    throw new RangeError('"size" argument must not be negative')
  }
}

function alloc (that, size, fill, encoding) {
  assertSize(size);
  if (size <= 0) {
    return createBuffer(that, size)
  }
  if (fill !== undefined) {
    // Only pay attention to encoding if it's a string. This
    // prevents accidentally sending in a number that would
    // be interpretted as a start offset.
    return typeof encoding === 'string'
      ? createBuffer(that, size).fill(fill, encoding)
      : createBuffer(that, size).fill(fill)
  }
  return createBuffer(that, size)
}

/**
 * Creates a new filled Buffer instance.
 * alloc(size[, fill[, encoding]])
 **/
Buffer.alloc = function (size, fill, encoding) {
  return alloc(null, size, fill, encoding)
};

function allocUnsafe (that, size) {
  assertSize(size);
  that = createBuffer(that, size < 0 ? 0 : checked(size) | 0);
  if (!Buffer.TYPED_ARRAY_SUPPORT) {
    for (var i = 0; i < size; ++i) {
      that[i] = 0;
    }
  }
  return that
}

/**
 * Equivalent to Buffer(num), by default creates a non-zero-filled Buffer instance.
 * */
Buffer.allocUnsafe = function (size) {
  return allocUnsafe(null, size)
};
/**
 * Equivalent to SlowBuffer(num), by default creates a non-zero-filled Buffer instance.
 */
Buffer.allocUnsafeSlow = function (size) {
  return allocUnsafe(null, size)
};

function fromString (that, string, encoding) {
  if (typeof encoding !== 'string' || encoding === '') {
    encoding = 'utf8';
  }

  if (!Buffer.isEncoding(encoding)) {
    throw new TypeError('"encoding" must be a valid string encoding')
  }

  var length = byteLength(string, encoding) | 0;
  that = createBuffer(that, length);

  var actual = that.write(string, encoding);

  if (actual !== length) {
    // Writing a hex string, for example, that contains invalid characters will
    // cause everything after the first invalid character to be ignored. (e.g.
    // 'abxxcd' will be treated as 'ab')
    that = that.slice(0, actual);
  }

  return that
}

function fromArrayLike (that, array) {
  var length = array.length < 0 ? 0 : checked(array.length) | 0;
  that = createBuffer(that, length);
  for (var i = 0; i < length; i += 1) {
    that[i] = array[i] & 255;
  }
  return that
}

function fromArrayBuffer (that, array, byteOffset, length) {
  array.byteLength; // this throws if `array` is not a valid ArrayBuffer

  if (byteOffset < 0 || array.byteLength < byteOffset) {
    throw new RangeError('\'offset\' is out of bounds')
  }

  if (array.byteLength < byteOffset + (length || 0)) {
    throw new RangeError('\'length\' is out of bounds')
  }

  if (byteOffset === undefined && length === undefined) {
    array = new Uint8Array(array);
  } else if (length === undefined) {
    array = new Uint8Array(array, byteOffset);
  } else {
    array = new Uint8Array(array, byteOffset, length);
  }

  if (Buffer.TYPED_ARRAY_SUPPORT) {
    // Return an augmented `Uint8Array` instance, for best performance
    that = array;
    that.__proto__ = Buffer.prototype;
  } else {
    // Fallback: Return an object instance of the Buffer class
    that = fromArrayLike(that, array);
  }
  return that
}

function fromObject (that, obj) {
  if (internalIsBuffer(obj)) {
    var len = checked(obj.length) | 0;
    that = createBuffer(that, len);

    if (that.length === 0) {
      return that
    }

    obj.copy(that, 0, 0, len);
    return that
  }

  if (obj) {
    if ((typeof ArrayBuffer !== 'undefined' &&
        obj.buffer instanceof ArrayBuffer) || 'length' in obj) {
      if (typeof obj.length !== 'number' || isnan(obj.length)) {
        return createBuffer(that, 0)
      }
      return fromArrayLike(that, obj)
    }

    if (obj.type === 'Buffer' && isArray(obj.data)) {
      return fromArrayLike(that, obj.data)
    }
  }

  throw new TypeError('First argument must be a string, Buffer, ArrayBuffer, Array, or array-like object.')
}

function checked (length) {
  // Note: cannot use `length < kMaxLength()` here because that fails when
  // length is NaN (which is otherwise coerced to zero.)
  if (length >= kMaxLength()) {
    throw new RangeError('Attempt to allocate Buffer larger than maximum ' +
                         'size: 0x' + kMaxLength().toString(16) + ' bytes')
  }
  return length | 0
}

function SlowBuffer (length) {
  if (+length != length) { // eslint-disable-line eqeqeq
    length = 0;
  }
  return Buffer.alloc(+length)
}
Buffer.isBuffer = isBuffer;
function internalIsBuffer (b) {
  return !!(b != null && b._isBuffer)
}

Buffer.compare = function compare (a, b) {
  if (!internalIsBuffer(a) || !internalIsBuffer(b)) {
    throw new TypeError('Arguments must be Buffers')
  }

  if (a === b) return 0

  var x = a.length;
  var y = b.length;

  for (var i = 0, len = Math.min(x, y); i < len; ++i) {
    if (a[i] !== b[i]) {
      x = a[i];
      y = b[i];
      break
    }
  }

  if (x < y) return -1
  if (y < x) return 1
  return 0
};

Buffer.isEncoding = function isEncoding (encoding) {
  switch (String(encoding).toLowerCase()) {
    case 'hex':
    case 'utf8':
    case 'utf-8':
    case 'ascii':
    case 'latin1':
    case 'binary':
    case 'base64':
    case 'ucs2':
    case 'ucs-2':
    case 'utf16le':
    case 'utf-16le':
      return true
    default:
      return false
  }
};

Buffer.concat = function concat (list, length) {
  if (!isArray(list)) {
    throw new TypeError('"list" argument must be an Array of Buffers')
  }

  if (list.length === 0) {
    return Buffer.alloc(0)
  }

  var i;
  if (length === undefined) {
    length = 0;
    for (i = 0; i < list.length; ++i) {
      length += list[i].length;
    }
  }

  var buffer = Buffer.allocUnsafe(length);
  var pos = 0;
  for (i = 0; i < list.length; ++i) {
    var buf = list[i];
    if (!internalIsBuffer(buf)) {
      throw new TypeError('"list" argument must be an Array of Buffers')
    }
    buf.copy(buffer, pos);
    pos += buf.length;
  }
  return buffer
};

function byteLength (string, encoding) {
  if (internalIsBuffer(string)) {
    return string.length
  }
  if (typeof ArrayBuffer !== 'undefined' && typeof ArrayBuffer.isView === 'function' &&
      (ArrayBuffer.isView(string) || string instanceof ArrayBuffer)) {
    return string.byteLength
  }
  if (typeof string !== 'string') {
    string = '' + string;
  }

  var len = string.length;
  if (len === 0) return 0

  // Use a for loop to avoid recursion
  var loweredCase = false;
  for (;;) {
    switch (encoding) {
      case 'ascii':
      case 'latin1':
      case 'binary':
        return len
      case 'utf8':
      case 'utf-8':
      case undefined:
        return utf8ToBytes(string).length
      case 'ucs2':
      case 'ucs-2':
      case 'utf16le':
      case 'utf-16le':
        return len * 2
      case 'hex':
        return len >>> 1
      case 'base64':
        return base64ToBytes(string).length
      default:
        if (loweredCase) return utf8ToBytes(string).length // assume utf8
        encoding = ('' + encoding).toLowerCase();
        loweredCase = true;
    }
  }
}
Buffer.byteLength = byteLength;

function slowToString (encoding, start, end) {
  var loweredCase = false;

  // No need to verify that "this.length <= MAX_UINT32" since it's a read-only
  // property of a typed array.

  // This behaves neither like String nor Uint8Array in that we set start/end
  // to their upper/lower bounds if the value passed is out of range.
  // undefined is handled specially as per ECMA-262 6th Edition,
  // Section 13.3.3.7 Runtime Semantics: KeyedBindingInitialization.
  if (start === undefined || start < 0) {
    start = 0;
  }
  // Return early if start > this.length. Done here to prevent potential uint32
  // coercion fail below.
  if (start > this.length) {
    return ''
  }

  if (end === undefined || end > this.length) {
    end = this.length;
  }

  if (end <= 0) {
    return ''
  }

  // Force coersion to uint32. This will also coerce falsey/NaN values to 0.
  end >>>= 0;
  start >>>= 0;

  if (end <= start) {
    return ''
  }

  if (!encoding) encoding = 'utf8';

  while (true) {
    switch (encoding) {
      case 'hex':
        return hexSlice(this, start, end)

      case 'utf8':
      case 'utf-8':
        return utf8Slice(this, start, end)

      case 'ascii':
        return asciiSlice(this, start, end)

      case 'latin1':
      case 'binary':
        return latin1Slice(this, start, end)

      case 'base64':
        return base64Slice(this, start, end)

      case 'ucs2':
      case 'ucs-2':
      case 'utf16le':
      case 'utf-16le':
        return utf16leSlice(this, start, end)

      default:
        if (loweredCase) throw new TypeError('Unknown encoding: ' + encoding)
        encoding = (encoding + '').toLowerCase();
        loweredCase = true;
    }
  }
}

// The property is used by `Buffer.isBuffer` and `is-buffer` (in Safari 5-7) to detect
// Buffer instances.
Buffer.prototype._isBuffer = true;

function swap (b, n, m) {
  var i = b[n];
  b[n] = b[m];
  b[m] = i;
}

Buffer.prototype.swap16 = function swap16 () {
  var len = this.length;
  if (len % 2 !== 0) {
    throw new RangeError('Buffer size must be a multiple of 16-bits')
  }
  for (var i = 0; i < len; i += 2) {
    swap(this, i, i + 1);
  }
  return this
};

Buffer.prototype.swap32 = function swap32 () {
  var len = this.length;
  if (len % 4 !== 0) {
    throw new RangeError('Buffer size must be a multiple of 32-bits')
  }
  for (var i = 0; i < len; i += 4) {
    swap(this, i, i + 3);
    swap(this, i + 1, i + 2);
  }
  return this
};

Buffer.prototype.swap64 = function swap64 () {
  var len = this.length;
  if (len % 8 !== 0) {
    throw new RangeError('Buffer size must be a multiple of 64-bits')
  }
  for (var i = 0; i < len; i += 8) {
    swap(this, i, i + 7);
    swap(this, i + 1, i + 6);
    swap(this, i + 2, i + 5);
    swap(this, i + 3, i + 4);
  }
  return this
};

Buffer.prototype.toString = function toString () {
  var length = this.length | 0;
  if (length === 0) return ''
  if (arguments.length === 0) return utf8Slice(this, 0, length)
  return slowToString.apply(this, arguments)
};

Buffer.prototype.equals = function equals (b) {
  if (!internalIsBuffer(b)) throw new TypeError('Argument must be a Buffer')
  if (this === b) return true
  return Buffer.compare(this, b) === 0
};

Buffer.prototype.inspect = function inspect () {
  var str = '';
  var max = INSPECT_MAX_BYTES;
  if (this.length > 0) {
    str = this.toString('hex', 0, max).match(/.{2}/g).join(' ');
    if (this.length > max) str += ' ... ';
  }
  return '<Buffer ' + str + '>'
};

Buffer.prototype.compare = function compare (target, start, end, thisStart, thisEnd) {
  if (!internalIsBuffer(target)) {
    throw new TypeError('Argument must be a Buffer')
  }

  if (start === undefined) {
    start = 0;
  }
  if (end === undefined) {
    end = target ? target.length : 0;
  }
  if (thisStart === undefined) {
    thisStart = 0;
  }
  if (thisEnd === undefined) {
    thisEnd = this.length;
  }

  if (start < 0 || end > target.length || thisStart < 0 || thisEnd > this.length) {
    throw new RangeError('out of range index')
  }

  if (thisStart >= thisEnd && start >= end) {
    return 0
  }
  if (thisStart >= thisEnd) {
    return -1
  }
  if (start >= end) {
    return 1
  }

  start >>>= 0;
  end >>>= 0;
  thisStart >>>= 0;
  thisEnd >>>= 0;

  if (this === target) return 0

  var x = thisEnd - thisStart;
  var y = end - start;
  var len = Math.min(x, y);

  var thisCopy = this.slice(thisStart, thisEnd);
  var targetCopy = target.slice(start, end);

  for (var i = 0; i < len; ++i) {
    if (thisCopy[i] !== targetCopy[i]) {
      x = thisCopy[i];
      y = targetCopy[i];
      break
    }
  }

  if (x < y) return -1
  if (y < x) return 1
  return 0
};

// Finds either the first index of `val` in `buffer` at offset >= `byteOffset`,
// OR the last index of `val` in `buffer` at offset <= `byteOffset`.
//
// Arguments:
// - buffer - a Buffer to search
// - val - a string, Buffer, or number
// - byteOffset - an index into `buffer`; will be clamped to an int32
// - encoding - an optional encoding, relevant is val is a string
// - dir - true for indexOf, false for lastIndexOf
function bidirectionalIndexOf (buffer, val, byteOffset, encoding, dir) {
  // Empty buffer means no match
  if (buffer.length === 0) return -1

  // Normalize byteOffset
  if (typeof byteOffset === 'string') {
    encoding = byteOffset;
    byteOffset = 0;
  } else if (byteOffset > 0x7fffffff) {
    byteOffset = 0x7fffffff;
  } else if (byteOffset < -0x80000000) {
    byteOffset = -0x80000000;
  }
  byteOffset = +byteOffset;  // Coerce to Number.
  if (isNaN(byteOffset)) {
    // byteOffset: it it's undefined, null, NaN, "foo", etc, search whole buffer
    byteOffset = dir ? 0 : (buffer.length - 1);
  }

  // Normalize byteOffset: negative offsets start from the end of the buffer
  if (byteOffset < 0) byteOffset = buffer.length + byteOffset;
  if (byteOffset >= buffer.length) {
    if (dir) return -1
    else byteOffset = buffer.length - 1;
  } else if (byteOffset < 0) {
    if (dir) byteOffset = 0;
    else return -1
  }

  // Normalize val
  if (typeof val === 'string') {
    val = Buffer.from(val, encoding);
  }

  // Finally, search either indexOf (if dir is true) or lastIndexOf
  if (internalIsBuffer(val)) {
    // Special case: looking for empty string/buffer always fails
    if (val.length === 0) {
      return -1
    }
    return arrayIndexOf(buffer, val, byteOffset, encoding, dir)
  } else if (typeof val === 'number') {
    val = val & 0xFF; // Search for a byte value [0-255]
    if (Buffer.TYPED_ARRAY_SUPPORT &&
        typeof Uint8Array.prototype.indexOf === 'function') {
      if (dir) {
        return Uint8Array.prototype.indexOf.call(buffer, val, byteOffset)
      } else {
        return Uint8Array.prototype.lastIndexOf.call(buffer, val, byteOffset)
      }
    }
    return arrayIndexOf(buffer, [ val ], byteOffset, encoding, dir)
  }

  throw new TypeError('val must be string, number or Buffer')
}

function arrayIndexOf (arr, val, byteOffset, encoding, dir) {
  var indexSize = 1;
  var arrLength = arr.length;
  var valLength = val.length;

  if (encoding !== undefined) {
    encoding = String(encoding).toLowerCase();
    if (encoding === 'ucs2' || encoding === 'ucs-2' ||
        encoding === 'utf16le' || encoding === 'utf-16le') {
      if (arr.length < 2 || val.length < 2) {
        return -1
      }
      indexSize = 2;
      arrLength /= 2;
      valLength /= 2;
      byteOffset /= 2;
    }
  }

  function read$$1 (buf, i) {
    if (indexSize === 1) {
      return buf[i]
    } else {
      return buf.readUInt16BE(i * indexSize)
    }
  }

  var i;
  if (dir) {
    var foundIndex = -1;
    for (i = byteOffset; i < arrLength; i++) {
      if (read$$1(arr, i) === read$$1(val, foundIndex === -1 ? 0 : i - foundIndex)) {
        if (foundIndex === -1) foundIndex = i;
        if (i - foundIndex + 1 === valLength) return foundIndex * indexSize
      } else {
        if (foundIndex !== -1) i -= i - foundIndex;
        foundIndex = -1;
      }
    }
  } else {
    if (byteOffset + valLength > arrLength) byteOffset = arrLength - valLength;
    for (i = byteOffset; i >= 0; i--) {
      var found = true;
      for (var j = 0; j < valLength; j++) {
        if (read$$1(arr, i + j) !== read$$1(val, j)) {
          found = false;
          break
        }
      }
      if (found) return i
    }
  }

  return -1
}

Buffer.prototype.includes = function includes (val, byteOffset, encoding) {
  return this.indexOf(val, byteOffset, encoding) !== -1
};

Buffer.prototype.indexOf = function indexOf (val, byteOffset, encoding) {
  return bidirectionalIndexOf(this, val, byteOffset, encoding, true)
};

Buffer.prototype.lastIndexOf = function lastIndexOf (val, byteOffset, encoding) {
  return bidirectionalIndexOf(this, val, byteOffset, encoding, false)
};

function hexWrite (buf, string, offset, length) {
  offset = Number(offset) || 0;
  var remaining = buf.length - offset;
  if (!length) {
    length = remaining;
  } else {
    length = Number(length);
    if (length > remaining) {
      length = remaining;
    }
  }

  // must be an even number of digits
  var strLen = string.length;
  if (strLen % 2 !== 0) throw new TypeError('Invalid hex string')

  if (length > strLen / 2) {
    length = strLen / 2;
  }
  for (var i = 0; i < length; ++i) {
    var parsed = parseInt(string.substr(i * 2, 2), 16);
    if (isNaN(parsed)) return i
    buf[offset + i] = parsed;
  }
  return i
}

function utf8Write (buf, string, offset, length) {
  return blitBuffer(utf8ToBytes(string, buf.length - offset), buf, offset, length)
}

function asciiWrite (buf, string, offset, length) {
  return blitBuffer(asciiToBytes(string), buf, offset, length)
}

function latin1Write (buf, string, offset, length) {
  return asciiWrite(buf, string, offset, length)
}

function base64Write (buf, string, offset, length) {
  return blitBuffer(base64ToBytes(string), buf, offset, length)
}

function ucs2Write (buf, string, offset, length) {
  return blitBuffer(utf16leToBytes(string, buf.length - offset), buf, offset, length)
}

Buffer.prototype.write = function write$$1 (string, offset, length, encoding) {
  // Buffer#write(string)
  if (offset === undefined) {
    encoding = 'utf8';
    length = this.length;
    offset = 0;
  // Buffer#write(string, encoding)
  } else if (length === undefined && typeof offset === 'string') {
    encoding = offset;
    length = this.length;
    offset = 0;
  // Buffer#write(string, offset[, length][, encoding])
  } else if (isFinite(offset)) {
    offset = offset | 0;
    if (isFinite(length)) {
      length = length | 0;
      if (encoding === undefined) encoding = 'utf8';
    } else {
      encoding = length;
      length = undefined;
    }
  // legacy write(string, encoding, offset, length) - remove in v0.13
  } else {
    throw new Error(
      'Buffer.write(string, encoding, offset[, length]) is no longer supported'
    )
  }

  var remaining = this.length - offset;
  if (length === undefined || length > remaining) length = remaining;

  if ((string.length > 0 && (length < 0 || offset < 0)) || offset > this.length) {
    throw new RangeError('Attempt to write outside buffer bounds')
  }

  if (!encoding) encoding = 'utf8';

  var loweredCase = false;
  for (;;) {
    switch (encoding) {
      case 'hex':
        return hexWrite(this, string, offset, length)

      case 'utf8':
      case 'utf-8':
        return utf8Write(this, string, offset, length)

      case 'ascii':
        return asciiWrite(this, string, offset, length)

      case 'latin1':
      case 'binary':
        return latin1Write(this, string, offset, length)

      case 'base64':
        // Warning: maxLength not taken into account in base64Write
        return base64Write(this, string, offset, length)

      case 'ucs2':
      case 'ucs-2':
      case 'utf16le':
      case 'utf-16le':
        return ucs2Write(this, string, offset, length)

      default:
        if (loweredCase) throw new TypeError('Unknown encoding: ' + encoding)
        encoding = ('' + encoding).toLowerCase();
        loweredCase = true;
    }
  }
};

Buffer.prototype.toJSON = function toJSON () {
  return {
    type: 'Buffer',
    data: Array.prototype.slice.call(this._arr || this, 0)
  }
};

function base64Slice (buf, start, end) {
  if (start === 0 && end === buf.length) {
    return fromByteArray(buf)
  } else {
    return fromByteArray(buf.slice(start, end))
  }
}

function utf8Slice (buf, start, end) {
  end = Math.min(buf.length, end);
  var res = [];

  var i = start;
  while (i < end) {
    var firstByte = buf[i];
    var codePoint = null;
    var bytesPerSequence = (firstByte > 0xEF) ? 4
      : (firstByte > 0xDF) ? 3
      : (firstByte > 0xBF) ? 2
      : 1;

    if (i + bytesPerSequence <= end) {
      var secondByte, thirdByte, fourthByte, tempCodePoint;

      switch (bytesPerSequence) {
        case 1:
          if (firstByte < 0x80) {
            codePoint = firstByte;
          }
          break
        case 2:
          secondByte = buf[i + 1];
          if ((secondByte & 0xC0) === 0x80) {
            tempCodePoint = (firstByte & 0x1F) << 0x6 | (secondByte & 0x3F);
            if (tempCodePoint > 0x7F) {
              codePoint = tempCodePoint;
            }
          }
          break
        case 3:
          secondByte = buf[i + 1];
          thirdByte = buf[i + 2];
          if ((secondByte & 0xC0) === 0x80 && (thirdByte & 0xC0) === 0x80) {
            tempCodePoint = (firstByte & 0xF) << 0xC | (secondByte & 0x3F) << 0x6 | (thirdByte & 0x3F);
            if (tempCodePoint > 0x7FF && (tempCodePoint < 0xD800 || tempCodePoint > 0xDFFF)) {
              codePoint = tempCodePoint;
            }
          }
          break
        case 4:
          secondByte = buf[i + 1];
          thirdByte = buf[i + 2];
          fourthByte = buf[i + 3];
          if ((secondByte & 0xC0) === 0x80 && (thirdByte & 0xC0) === 0x80 && (fourthByte & 0xC0) === 0x80) {
            tempCodePoint = (firstByte & 0xF) << 0x12 | (secondByte & 0x3F) << 0xC | (thirdByte & 0x3F) << 0x6 | (fourthByte & 0x3F);
            if (tempCodePoint > 0xFFFF && tempCodePoint < 0x110000) {
              codePoint = tempCodePoint;
            }
          }
      }
    }

    if (codePoint === null) {
      // we did not generate a valid codePoint so insert a
      // replacement char (U+FFFD) and advance only 1 byte
      codePoint = 0xFFFD;
      bytesPerSequence = 1;
    } else if (codePoint > 0xFFFF) {
      // encode to utf16 (surrogate pair dance)
      codePoint -= 0x10000;
      res.push(codePoint >>> 10 & 0x3FF | 0xD800);
      codePoint = 0xDC00 | codePoint & 0x3FF;
    }

    res.push(codePoint);
    i += bytesPerSequence;
  }

  return decodeCodePointsArray(res)
}

// Based on http://stackoverflow.com/a/22747272/680742, the browser with
// the lowest limit is Chrome, with 0x10000 args.
// We go 1 magnitude less, for safety
var MAX_ARGUMENTS_LENGTH = 0x1000;

function decodeCodePointsArray (codePoints) {
  var len = codePoints.length;
  if (len <= MAX_ARGUMENTS_LENGTH) {
    return String.fromCharCode.apply(String, codePoints) // avoid extra slice()
  }

  // Decode in chunks to avoid "call stack size exceeded".
  var res = '';
  var i = 0;
  while (i < len) {
    res += String.fromCharCode.apply(
      String,
      codePoints.slice(i, i += MAX_ARGUMENTS_LENGTH)
    );
  }
  return res
}

function asciiSlice (buf, start, end) {
  var ret = '';
  end = Math.min(buf.length, end);

  for (var i = start; i < end; ++i) {
    ret += String.fromCharCode(buf[i] & 0x7F);
  }
  return ret
}

function latin1Slice (buf, start, end) {
  var ret = '';
  end = Math.min(buf.length, end);

  for (var i = start; i < end; ++i) {
    ret += String.fromCharCode(buf[i]);
  }
  return ret
}

function hexSlice (buf, start, end) {
  var len = buf.length;

  if (!start || start < 0) start = 0;
  if (!end || end < 0 || end > len) end = len;

  var out = '';
  for (var i = start; i < end; ++i) {
    out += toHex(buf[i]);
  }
  return out
}

function utf16leSlice (buf, start, end) {
  var bytes = buf.slice(start, end);
  var res = '';
  for (var i = 0; i < bytes.length; i += 2) {
    res += String.fromCharCode(bytes[i] + bytes[i + 1] * 256);
  }
  return res
}

Buffer.prototype.slice = function slice (start, end) {
  var len = this.length;
  start = ~~start;
  end = end === undefined ? len : ~~end;

  if (start < 0) {
    start += len;
    if (start < 0) start = 0;
  } else if (start > len) {
    start = len;
  }

  if (end < 0) {
    end += len;
    if (end < 0) end = 0;
  } else if (end > len) {
    end = len;
  }

  if (end < start) end = start;

  var newBuf;
  if (Buffer.TYPED_ARRAY_SUPPORT) {
    newBuf = this.subarray(start, end);
    newBuf.__proto__ = Buffer.prototype;
  } else {
    var sliceLen = end - start;
    newBuf = new Buffer(sliceLen, undefined);
    for (var i = 0; i < sliceLen; ++i) {
      newBuf[i] = this[i + start];
    }
  }

  return newBuf
};

/*
 * Need to make sure that buffer isn't trying to write out of bounds.
 */
function checkOffset (offset, ext, length) {
  if ((offset % 1) !== 0 || offset < 0) throw new RangeError('offset is not uint')
  if (offset + ext > length) throw new RangeError('Trying to access beyond buffer length')
}

Buffer.prototype.readUIntLE = function readUIntLE (offset, byteLength, noAssert) {
  offset = offset | 0;
  byteLength = byteLength | 0;
  if (!noAssert) checkOffset(offset, byteLength, this.length);

  var val = this[offset];
  var mul = 1;
  var i = 0;
  while (++i < byteLength && (mul *= 0x100)) {
    val += this[offset + i] * mul;
  }

  return val
};

Buffer.prototype.readUIntBE = function readUIntBE (offset, byteLength, noAssert) {
  offset = offset | 0;
  byteLength = byteLength | 0;
  if (!noAssert) {
    checkOffset(offset, byteLength, this.length);
  }

  var val = this[offset + --byteLength];
  var mul = 1;
  while (byteLength > 0 && (mul *= 0x100)) {
    val += this[offset + --byteLength] * mul;
  }

  return val
};

Buffer.prototype.readUInt8 = function readUInt8 (offset, noAssert) {
  if (!noAssert) checkOffset(offset, 1, this.length);
  return this[offset]
};

Buffer.prototype.readUInt16LE = function readUInt16LE (offset, noAssert) {
  if (!noAssert) checkOffset(offset, 2, this.length);
  return this[offset] | (this[offset + 1] << 8)
};

Buffer.prototype.readUInt16BE = function readUInt16BE (offset, noAssert) {
  if (!noAssert) checkOffset(offset, 2, this.length);
  return (this[offset] << 8) | this[offset + 1]
};

Buffer.prototype.readUInt32LE = function readUInt32LE (offset, noAssert) {
  if (!noAssert) checkOffset(offset, 4, this.length);

  return ((this[offset]) |
      (this[offset + 1] << 8) |
      (this[offset + 2] << 16)) +
      (this[offset + 3] * 0x1000000)
};

Buffer.prototype.readUInt32BE = function readUInt32BE (offset, noAssert) {
  if (!noAssert) checkOffset(offset, 4, this.length);

  return (this[offset] * 0x1000000) +
    ((this[offset + 1] << 16) |
    (this[offset + 2] << 8) |
    this[offset + 3])
};

Buffer.prototype.readIntLE = function readIntLE (offset, byteLength, noAssert) {
  offset = offset | 0;
  byteLength = byteLength | 0;
  if (!noAssert) checkOffset(offset, byteLength, this.length);

  var val = this[offset];
  var mul = 1;
  var i = 0;
  while (++i < byteLength && (mul *= 0x100)) {
    val += this[offset + i] * mul;
  }
  mul *= 0x80;

  if (val >= mul) val -= Math.pow(2, 8 * byteLength);

  return val
};

Buffer.prototype.readIntBE = function readIntBE (offset, byteLength, noAssert) {
  offset = offset | 0;
  byteLength = byteLength | 0;
  if (!noAssert) checkOffset(offset, byteLength, this.length);

  var i = byteLength;
  var mul = 1;
  var val = this[offset + --i];
  while (i > 0 && (mul *= 0x100)) {
    val += this[offset + --i] * mul;
  }
  mul *= 0x80;

  if (val >= mul) val -= Math.pow(2, 8 * byteLength);

  return val
};

Buffer.prototype.readInt8 = function readInt8 (offset, noAssert) {
  if (!noAssert) checkOffset(offset, 1, this.length);
  if (!(this[offset] & 0x80)) return (this[offset])
  return ((0xff - this[offset] + 1) * -1)
};

Buffer.prototype.readInt16LE = function readInt16LE (offset, noAssert) {
  if (!noAssert) checkOffset(offset, 2, this.length);
  var val = this[offset] | (this[offset + 1] << 8);
  return (val & 0x8000) ? val | 0xFFFF0000 : val
};

Buffer.prototype.readInt16BE = function readInt16BE (offset, noAssert) {
  if (!noAssert) checkOffset(offset, 2, this.length);
  var val = this[offset + 1] | (this[offset] << 8);
  return (val & 0x8000) ? val | 0xFFFF0000 : val
};

Buffer.prototype.readInt32LE = function readInt32LE (offset, noAssert) {
  if (!noAssert) checkOffset(offset, 4, this.length);

  return (this[offset]) |
    (this[offset + 1] << 8) |
    (this[offset + 2] << 16) |
    (this[offset + 3] << 24)
};

Buffer.prototype.readInt32BE = function readInt32BE (offset, noAssert) {
  if (!noAssert) checkOffset(offset, 4, this.length);

  return (this[offset] << 24) |
    (this[offset + 1] << 16) |
    (this[offset + 2] << 8) |
    (this[offset + 3])
};

Buffer.prototype.readFloatLE = function readFloatLE (offset, noAssert) {
  if (!noAssert) checkOffset(offset, 4, this.length);
  return read(this, offset, true, 23, 4)
};

Buffer.prototype.readFloatBE = function readFloatBE (offset, noAssert) {
  if (!noAssert) checkOffset(offset, 4, this.length);
  return read(this, offset, false, 23, 4)
};

Buffer.prototype.readDoubleLE = function readDoubleLE (offset, noAssert) {
  if (!noAssert) checkOffset(offset, 8, this.length);
  return read(this, offset, true, 52, 8)
};

Buffer.prototype.readDoubleBE = function readDoubleBE (offset, noAssert) {
  if (!noAssert) checkOffset(offset, 8, this.length);
  return read(this, offset, false, 52, 8)
};

function checkInt (buf, value, offset, ext, max, min) {
  if (!internalIsBuffer(buf)) throw new TypeError('"buffer" argument must be a Buffer instance')
  if (value > max || value < min) throw new RangeError('"value" argument is out of bounds')
  if (offset + ext > buf.length) throw new RangeError('Index out of range')
}

Buffer.prototype.writeUIntLE = function writeUIntLE (value, offset, byteLength, noAssert) {
  value = +value;
  offset = offset | 0;
  byteLength = byteLength | 0;
  if (!noAssert) {
    var maxBytes = Math.pow(2, 8 * byteLength) - 1;
    checkInt(this, value, offset, byteLength, maxBytes, 0);
  }

  var mul = 1;
  var i = 0;
  this[offset] = value & 0xFF;
  while (++i < byteLength && (mul *= 0x100)) {
    this[offset + i] = (value / mul) & 0xFF;
  }

  return offset + byteLength
};

Buffer.prototype.writeUIntBE = function writeUIntBE (value, offset, byteLength, noAssert) {
  value = +value;
  offset = offset | 0;
  byteLength = byteLength | 0;
  if (!noAssert) {
    var maxBytes = Math.pow(2, 8 * byteLength) - 1;
    checkInt(this, value, offset, byteLength, maxBytes, 0);
  }

  var i = byteLength - 1;
  var mul = 1;
  this[offset + i] = value & 0xFF;
  while (--i >= 0 && (mul *= 0x100)) {
    this[offset + i] = (value / mul) & 0xFF;
  }

  return offset + byteLength
};

Buffer.prototype.writeUInt8 = function writeUInt8 (value, offset, noAssert) {
  value = +value;
  offset = offset | 0;
  if (!noAssert) checkInt(this, value, offset, 1, 0xff, 0);
  if (!Buffer.TYPED_ARRAY_SUPPORT) value = Math.floor(value);
  this[offset] = (value & 0xff);
  return offset + 1
};

function objectWriteUInt16 (buf, value, offset, littleEndian) {
  if (value < 0) value = 0xffff + value + 1;
  for (var i = 0, j = Math.min(buf.length - offset, 2); i < j; ++i) {
    buf[offset + i] = (value & (0xff << (8 * (littleEndian ? i : 1 - i)))) >>>
      (littleEndian ? i : 1 - i) * 8;
  }
}

Buffer.prototype.writeUInt16LE = function writeUInt16LE (value, offset, noAssert) {
  value = +value;
  offset = offset | 0;
  if (!noAssert) checkInt(this, value, offset, 2, 0xffff, 0);
  if (Buffer.TYPED_ARRAY_SUPPORT) {
    this[offset] = (value & 0xff);
    this[offset + 1] = (value >>> 8);
  } else {
    objectWriteUInt16(this, value, offset, true);
  }
  return offset + 2
};

Buffer.prototype.writeUInt16BE = function writeUInt16BE (value, offset, noAssert) {
  value = +value;
  offset = offset | 0;
  if (!noAssert) checkInt(this, value, offset, 2, 0xffff, 0);
  if (Buffer.TYPED_ARRAY_SUPPORT) {
    this[offset] = (value >>> 8);
    this[offset + 1] = (value & 0xff);
  } else {
    objectWriteUInt16(this, value, offset, false);
  }
  return offset + 2
};

function objectWriteUInt32 (buf, value, offset, littleEndian) {
  if (value < 0) value = 0xffffffff + value + 1;
  for (var i = 0, j = Math.min(buf.length - offset, 4); i < j; ++i) {
    buf[offset + i] = (value >>> (littleEndian ? i : 3 - i) * 8) & 0xff;
  }
}

Buffer.prototype.writeUInt32LE = function writeUInt32LE (value, offset, noAssert) {
  value = +value;
  offset = offset | 0;
  if (!noAssert) checkInt(this, value, offset, 4, 0xffffffff, 0);
  if (Buffer.TYPED_ARRAY_SUPPORT) {
    this[offset + 3] = (value >>> 24);
    this[offset + 2] = (value >>> 16);
    this[offset + 1] = (value >>> 8);
    this[offset] = (value & 0xff);
  } else {
    objectWriteUInt32(this, value, offset, true);
  }
  return offset + 4
};

Buffer.prototype.writeUInt32BE = function writeUInt32BE (value, offset, noAssert) {
  value = +value;
  offset = offset | 0;
  if (!noAssert) checkInt(this, value, offset, 4, 0xffffffff, 0);
  if (Buffer.TYPED_ARRAY_SUPPORT) {
    this[offset] = (value >>> 24);
    this[offset + 1] = (value >>> 16);
    this[offset + 2] = (value >>> 8);
    this[offset + 3] = (value & 0xff);
  } else {
    objectWriteUInt32(this, value, offset, false);
  }
  return offset + 4
};

Buffer.prototype.writeIntLE = function writeIntLE (value, offset, byteLength, noAssert) {
  value = +value;
  offset = offset | 0;
  if (!noAssert) {
    var limit = Math.pow(2, 8 * byteLength - 1);

    checkInt(this, value, offset, byteLength, limit - 1, -limit);
  }

  var i = 0;
  var mul = 1;
  var sub = 0;
  this[offset] = value & 0xFF;
  while (++i < byteLength && (mul *= 0x100)) {
    if (value < 0 && sub === 0 && this[offset + i - 1] !== 0) {
      sub = 1;
    }
    this[offset + i] = ((value / mul) >> 0) - sub & 0xFF;
  }

  return offset + byteLength
};

Buffer.prototype.writeIntBE = function writeIntBE (value, offset, byteLength, noAssert) {
  value = +value;
  offset = offset | 0;
  if (!noAssert) {
    var limit = Math.pow(2, 8 * byteLength - 1);

    checkInt(this, value, offset, byteLength, limit - 1, -limit);
  }

  var i = byteLength - 1;
  var mul = 1;
  var sub = 0;
  this[offset + i] = value & 0xFF;
  while (--i >= 0 && (mul *= 0x100)) {
    if (value < 0 && sub === 0 && this[offset + i + 1] !== 0) {
      sub = 1;
    }
    this[offset + i] = ((value / mul) >> 0) - sub & 0xFF;
  }

  return offset + byteLength
};

Buffer.prototype.writeInt8 = function writeInt8 (value, offset, noAssert) {
  value = +value;
  offset = offset | 0;
  if (!noAssert) checkInt(this, value, offset, 1, 0x7f, -0x80);
  if (!Buffer.TYPED_ARRAY_SUPPORT) value = Math.floor(value);
  if (value < 0) value = 0xff + value + 1;
  this[offset] = (value & 0xff);
  return offset + 1
};

Buffer.prototype.writeInt16LE = function writeInt16LE (value, offset, noAssert) {
  value = +value;
  offset = offset | 0;
  if (!noAssert) checkInt(this, value, offset, 2, 0x7fff, -0x8000);
  if (Buffer.TYPED_ARRAY_SUPPORT) {
    this[offset] = (value & 0xff);
    this[offset + 1] = (value >>> 8);
  } else {
    objectWriteUInt16(this, value, offset, true);
  }
  return offset + 2
};

Buffer.prototype.writeInt16BE = function writeInt16BE (value, offset, noAssert) {
  value = +value;
  offset = offset | 0;
  if (!noAssert) checkInt(this, value, offset, 2, 0x7fff, -0x8000);
  if (Buffer.TYPED_ARRAY_SUPPORT) {
    this[offset] = (value >>> 8);
    this[offset + 1] = (value & 0xff);
  } else {
    objectWriteUInt16(this, value, offset, false);
  }
  return offset + 2
};

Buffer.prototype.writeInt32LE = function writeInt32LE (value, offset, noAssert) {
  value = +value;
  offset = offset | 0;
  if (!noAssert) checkInt(this, value, offset, 4, 0x7fffffff, -0x80000000);
  if (Buffer.TYPED_ARRAY_SUPPORT) {
    this[offset] = (value & 0xff);
    this[offset + 1] = (value >>> 8);
    this[offset + 2] = (value >>> 16);
    this[offset + 3] = (value >>> 24);
  } else {
    objectWriteUInt32(this, value, offset, true);
  }
  return offset + 4
};

Buffer.prototype.writeInt32BE = function writeInt32BE (value, offset, noAssert) {
  value = +value;
  offset = offset | 0;
  if (!noAssert) checkInt(this, value, offset, 4, 0x7fffffff, -0x80000000);
  if (value < 0) value = 0xffffffff + value + 1;
  if (Buffer.TYPED_ARRAY_SUPPORT) {
    this[offset] = (value >>> 24);
    this[offset + 1] = (value >>> 16);
    this[offset + 2] = (value >>> 8);
    this[offset + 3] = (value & 0xff);
  } else {
    objectWriteUInt32(this, value, offset, false);
  }
  return offset + 4
};

function checkIEEE754 (buf, value, offset, ext, max, min) {
  if (offset + ext > buf.length) throw new RangeError('Index out of range')
  if (offset < 0) throw new RangeError('Index out of range')
}

function writeFloat (buf, value, offset, littleEndian, noAssert) {
  if (!noAssert) {
    checkIEEE754(buf, value, offset, 4, 3.4028234663852886e+38, -3.4028234663852886e+38);
  }
  write(buf, value, offset, littleEndian, 23, 4);
  return offset + 4
}

Buffer.prototype.writeFloatLE = function writeFloatLE (value, offset, noAssert) {
  return writeFloat(this, value, offset, true, noAssert)
};

Buffer.prototype.writeFloatBE = function writeFloatBE (value, offset, noAssert) {
  return writeFloat(this, value, offset, false, noAssert)
};

function writeDouble (buf, value, offset, littleEndian, noAssert) {
  if (!noAssert) {
    checkIEEE754(buf, value, offset, 8, 1.7976931348623157E+308, -1.7976931348623157E+308);
  }
  write(buf, value, offset, littleEndian, 52, 8);
  return offset + 8
}

Buffer.prototype.writeDoubleLE = function writeDoubleLE (value, offset, noAssert) {
  return writeDouble(this, value, offset, true, noAssert)
};

Buffer.prototype.writeDoubleBE = function writeDoubleBE (value, offset, noAssert) {
  return writeDouble(this, value, offset, false, noAssert)
};

// copy(targetBuffer, targetStart=0, sourceStart=0, sourceEnd=buffer.length)
Buffer.prototype.copy = function copy (target, targetStart, start, end) {
  if (!start) start = 0;
  if (!end && end !== 0) end = this.length;
  if (targetStart >= target.length) targetStart = target.length;
  if (!targetStart) targetStart = 0;
  if (end > 0 && end < start) end = start;

  // Copy 0 bytes; we're done
  if (end === start) return 0
  if (target.length === 0 || this.length === 0) return 0

  // Fatal error conditions
  if (targetStart < 0) {
    throw new RangeError('targetStart out of bounds')
  }
  if (start < 0 || start >= this.length) throw new RangeError('sourceStart out of bounds')
  if (end < 0) throw new RangeError('sourceEnd out of bounds')

  // Are we oob?
  if (end > this.length) end = this.length;
  if (target.length - targetStart < end - start) {
    end = target.length - targetStart + start;
  }

  var len = end - start;
  var i;

  if (this === target && start < targetStart && targetStart < end) {
    // descending copy from end
    for (i = len - 1; i >= 0; --i) {
      target[i + targetStart] = this[i + start];
    }
  } else if (len < 1000 || !Buffer.TYPED_ARRAY_SUPPORT) {
    // ascending copy from start
    for (i = 0; i < len; ++i) {
      target[i + targetStart] = this[i + start];
    }
  } else {
    Uint8Array.prototype.set.call(
      target,
      this.subarray(start, start + len),
      targetStart
    );
  }

  return len
};

// Usage:
//    buffer.fill(number[, offset[, end]])
//    buffer.fill(buffer[, offset[, end]])
//    buffer.fill(string[, offset[, end]][, encoding])
Buffer.prototype.fill = function fill (val, start, end, encoding) {
  // Handle string cases:
  if (typeof val === 'string') {
    if (typeof start === 'string') {
      encoding = start;
      start = 0;
      end = this.length;
    } else if (typeof end === 'string') {
      encoding = end;
      end = this.length;
    }
    if (val.length === 1) {
      var code = val.charCodeAt(0);
      if (code < 256) {
        val = code;
      }
    }
    if (encoding !== undefined && typeof encoding !== 'string') {
      throw new TypeError('encoding must be a string')
    }
    if (typeof encoding === 'string' && !Buffer.isEncoding(encoding)) {
      throw new TypeError('Unknown encoding: ' + encoding)
    }
  } else if (typeof val === 'number') {
    val = val & 255;
  }

  // Invalid ranges are not set to a default, so can range check early.
  if (start < 0 || this.length < start || this.length < end) {
    throw new RangeError('Out of range index')
  }

  if (end <= start) {
    return this
  }

  start = start >>> 0;
  end = end === undefined ? this.length : end >>> 0;

  if (!val) val = 0;

  var i;
  if (typeof val === 'number') {
    for (i = start; i < end; ++i) {
      this[i] = val;
    }
  } else {
    var bytes = internalIsBuffer(val)
      ? val
      : utf8ToBytes(new Buffer(val, encoding).toString());
    var len = bytes.length;
    for (i = 0; i < end - start; ++i) {
      this[i + start] = bytes[i % len];
    }
  }

  return this
};

// HELPER FUNCTIONS
// ================

var INVALID_BASE64_RE = /[^+\/0-9A-Za-z-_]/g;

function base64clean (str) {
  // Node strips out invalid characters like \n and \t from the string, base64-js does not
  str = stringtrim(str).replace(INVALID_BASE64_RE, '');
  // Node converts strings with length < 2 to ''
  if (str.length < 2) return ''
  // Node allows for non-padded base64 strings (missing trailing ===), base64-js does not
  while (str.length % 4 !== 0) {
    str = str + '=';
  }
  return str
}

function stringtrim (str) {
  if (str.trim) return str.trim()
  return str.replace(/^\s+|\s+$/g, '')
}

function toHex (n) {
  if (n < 16) return '0' + n.toString(16)
  return n.toString(16)
}

function utf8ToBytes (string, units) {
  units = units || Infinity;
  var codePoint;
  var length = string.length;
  var leadSurrogate = null;
  var bytes = [];

  for (var i = 0; i < length; ++i) {
    codePoint = string.charCodeAt(i);

    // is surrogate component
    if (codePoint > 0xD7FF && codePoint < 0xE000) {
      // last char was a lead
      if (!leadSurrogate) {
        // no lead yet
        if (codePoint > 0xDBFF) {
          // unexpected trail
          if ((units -= 3) > -1) bytes.push(0xEF, 0xBF, 0xBD);
          continue
        } else if (i + 1 === length) {
          // unpaired lead
          if ((units -= 3) > -1) bytes.push(0xEF, 0xBF, 0xBD);
          continue
        }

        // valid lead
        leadSurrogate = codePoint;

        continue
      }

      // 2 leads in a row
      if (codePoint < 0xDC00) {
        if ((units -= 3) > -1) bytes.push(0xEF, 0xBF, 0xBD);
        leadSurrogate = codePoint;
        continue
      }

      // valid surrogate pair
      codePoint = (leadSurrogate - 0xD800 << 10 | codePoint - 0xDC00) + 0x10000;
    } else if (leadSurrogate) {
      // valid bmp char, but last char was a lead
      if ((units -= 3) > -1) bytes.push(0xEF, 0xBF, 0xBD);
    }

    leadSurrogate = null;

    // encode utf8
    if (codePoint < 0x80) {
      if ((units -= 1) < 0) break
      bytes.push(codePoint);
    } else if (codePoint < 0x800) {
      if ((units -= 2) < 0) break
      bytes.push(
        codePoint >> 0x6 | 0xC0,
        codePoint & 0x3F | 0x80
      );
    } else if (codePoint < 0x10000) {
      if ((units -= 3) < 0) break
      bytes.push(
        codePoint >> 0xC | 0xE0,
        codePoint >> 0x6 & 0x3F | 0x80,
        codePoint & 0x3F | 0x80
      );
    } else if (codePoint < 0x110000) {
      if ((units -= 4) < 0) break
      bytes.push(
        codePoint >> 0x12 | 0xF0,
        codePoint >> 0xC & 0x3F | 0x80,
        codePoint >> 0x6 & 0x3F | 0x80,
        codePoint & 0x3F | 0x80
      );
    } else {
      throw new Error('Invalid code point')
    }
  }

  return bytes
}

function asciiToBytes (str) {
  var byteArray = [];
  for (var i = 0; i < str.length; ++i) {
    // Node's code seems to be doing this and not & 0x7F..
    byteArray.push(str.charCodeAt(i) & 0xFF);
  }
  return byteArray
}

function utf16leToBytes (str, units) {
  var c, hi, lo;
  var byteArray = [];
  for (var i = 0; i < str.length; ++i) {
    if ((units -= 2) < 0) break

    c = str.charCodeAt(i);
    hi = c >> 8;
    lo = c % 256;
    byteArray.push(lo);
    byteArray.push(hi);
  }

  return byteArray
}


function base64ToBytes (str) {
  return toByteArray(base64clean(str))
}

function blitBuffer (src, dst, offset, length) {
  for (var i = 0; i < length; ++i) {
    if ((i + offset >= dst.length) || (i >= src.length)) break
    dst[i + offset] = src[i];
  }
  return i
}

function isnan (val) {
  return val !== val // eslint-disable-line no-self-compare
}


// the following is from is-buffer, also by Feross Aboukhadijeh and with same lisence
// The _isBuffer check is for Safari 5-7 support, because it's missing
// Object.prototype.constructor. Remove this eventually
function isBuffer(obj) {
  return obj != null && (!!obj._isBuffer || isFastBuffer(obj) || isSlowBuffer(obj))
}

function isFastBuffer (obj) {
  return !!obj.constructor && typeof obj.constructor.isBuffer === 'function' && obj.constructor.isBuffer(obj)
}

// For Node v0.10 support. Remove this eventually.
function isSlowBuffer (obj) {
  return typeof obj.readFloatLE === 'function' && typeof obj.slice === 'function' && isFastBuffer(obj.slice(0, 0))
}

var bufferEs6 = /*#__PURE__*/Object.freeze({
INSPECT_MAX_BYTES: INSPECT_MAX_BYTES,
kMaxLength: _kMaxLength,
Buffer: Buffer,
SlowBuffer: SlowBuffer,
isBuffer: isBuffer
});

var domain;

// This constructor is used to store event handlers. Instantiating this is
// faster than explicitly calling `Object.create(null)` to get a "clean" empty
// object (tested with v8 v4.9).
function EventHandlers() {}
EventHandlers.prototype = Object.create(null);

function EventEmitter() {
  EventEmitter.init.call(this);
}

// nodejs oddity
// require('events') === require('events').EventEmitter
EventEmitter.EventEmitter = EventEmitter;

EventEmitter.usingDomains = false;

EventEmitter.prototype.domain = undefined;
EventEmitter.prototype._events = undefined;
EventEmitter.prototype._maxListeners = undefined;

// By default EventEmitters will print a warning if more than 10 listeners are
// added to it. This is a useful default which helps finding memory leaks.
EventEmitter.defaultMaxListeners = 10;

EventEmitter.init = function() {
  this.domain = null;
  if (EventEmitter.usingDomains) {
    // if there is an active domain, then attach to it.
    if (domain.active && !(this instanceof domain.Domain)) ;
  }

  if (!this._events || this._events === Object.getPrototypeOf(this)._events) {
    this._events = new EventHandlers();
    this._eventsCount = 0;
  }

  this._maxListeners = this._maxListeners || undefined;
};

// Obviously not all Emitters should be limited to 10. This function allows
// that to be increased. Set to zero for unlimited.
EventEmitter.prototype.setMaxListeners = function setMaxListeners(n) {
  if (typeof n !== 'number' || n < 0 || isNaN(n))
    throw new TypeError('"n" argument must be a positive number');
  this._maxListeners = n;
  return this;
};

function $getMaxListeners(that) {
  if (that._maxListeners === undefined)
    return EventEmitter.defaultMaxListeners;
  return that._maxListeners;
}

EventEmitter.prototype.getMaxListeners = function getMaxListeners() {
  return $getMaxListeners(this);
};

// These standalone emit* functions are used to optimize calling of event
// handlers for fast cases because emit() itself often has a variable number of
// arguments and can be deoptimized because of that. These functions always have
// the same number of arguments and thus do not get deoptimized, so the code
// inside them can execute faster.
function emitNone(handler, isFn, self) {
  if (isFn)
    handler.call(self);
  else {
    var len = handler.length;
    var listeners = arrayClone(handler, len);
    for (var i = 0; i < len; ++i)
      listeners[i].call(self);
  }
}
function emitOne(handler, isFn, self, arg1) {
  if (isFn)
    handler.call(self, arg1);
  else {
    var len = handler.length;
    var listeners = arrayClone(handler, len);
    for (var i = 0; i < len; ++i)
      listeners[i].call(self, arg1);
  }
}
function emitTwo(handler, isFn, self, arg1, arg2) {
  if (isFn)
    handler.call(self, arg1, arg2);
  else {
    var len = handler.length;
    var listeners = arrayClone(handler, len);
    for (var i = 0; i < len; ++i)
      listeners[i].call(self, arg1, arg2);
  }
}
function emitThree(handler, isFn, self, arg1, arg2, arg3) {
  if (isFn)
    handler.call(self, arg1, arg2, arg3);
  else {
    var len = handler.length;
    var listeners = arrayClone(handler, len);
    for (var i = 0; i < len; ++i)
      listeners[i].call(self, arg1, arg2, arg3);
  }
}

function emitMany(handler, isFn, self, args) {
  if (isFn)
    handler.apply(self, args);
  else {
    var len = handler.length;
    var listeners = arrayClone(handler, len);
    for (var i = 0; i < len; ++i)
      listeners[i].apply(self, args);
  }
}

EventEmitter.prototype.emit = function emit(type) {
  var er, handler, len, args, i, events, domain;
  var doError = (type === 'error');

  events = this._events;
  if (events)
    doError = (doError && events.error == null);
  else if (!doError)
    return false;

  domain = this.domain;

  // If there is no 'error' event listener then throw.
  if (doError) {
    er = arguments[1];
    if (domain) {
      if (!er)
        er = new Error('Uncaught, unspecified "error" event');
      er.domainEmitter = this;
      er.domain = domain;
      er.domainThrown = false;
      domain.emit('error', er);
    } else if (er instanceof Error) {
      throw er; // Unhandled 'error' event
    } else {
      // At least give some kind of context to the user
      var err = new Error('Uncaught, unspecified "error" event. (' + er + ')');
      err.context = er;
      throw err;
    }
    return false;
  }

  handler = events[type];

  if (!handler)
    return false;

  var isFn = typeof handler === 'function';
  len = arguments.length;
  switch (len) {
    // fast cases
    case 1:
      emitNone(handler, isFn, this);
      break;
    case 2:
      emitOne(handler, isFn, this, arguments[1]);
      break;
    case 3:
      emitTwo(handler, isFn, this, arguments[1], arguments[2]);
      break;
    case 4:
      emitThree(handler, isFn, this, arguments[1], arguments[2], arguments[3]);
      break;
    // slower
    default:
      args = new Array(len - 1);
      for (i = 1; i < len; i++)
        args[i - 1] = arguments[i];
      emitMany(handler, isFn, this, args);
  }

  return true;
};

function _addListener(target, type, listener, prepend) {
  var m;
  var events;
  var existing;

  if (typeof listener !== 'function')
    throw new TypeError('"listener" argument must be a function');

  events = target._events;
  if (!events) {
    events = target._events = new EventHandlers();
    target._eventsCount = 0;
  } else {
    // To avoid recursion in the case that type === "newListener"! Before
    // adding it to the listeners, first emit "newListener".
    if (events.newListener) {
      target.emit('newListener', type,
                  listener.listener ? listener.listener : listener);

      // Re-assign `events` because a newListener handler could have caused the
      // this._events to be assigned to a new object
      events = target._events;
    }
    existing = events[type];
  }

  if (!existing) {
    // Optimize the case of one listener. Don't need the extra array object.
    existing = events[type] = listener;
    ++target._eventsCount;
  } else {
    if (typeof existing === 'function') {
      // Adding the second element, need to change to array.
      existing = events[type] = prepend ? [listener, existing] :
                                          [existing, listener];
    } else {
      // If we've already got an array, just append.
      if (prepend) {
        existing.unshift(listener);
      } else {
        existing.push(listener);
      }
    }

    // Check for listener leak
    if (!existing.warned) {
      m = $getMaxListeners(target);
      if (m && m > 0 && existing.length > m) {
        existing.warned = true;
        var w = new Error('Possible EventEmitter memory leak detected. ' +
                            existing.length + ' ' + type + ' listeners added. ' +
                            'Use emitter.setMaxListeners() to increase limit');
        w.name = 'MaxListenersExceededWarning';
        w.emitter = target;
        w.type = type;
        w.count = existing.length;
        emitWarning(w);
      }
    }
  }

  return target;
}
function emitWarning(e) {
  typeof console.warn === 'function' ? console.warn(e) : console.log(e);
}
EventEmitter.prototype.addListener = function addListener(type, listener) {
  return _addListener(this, type, listener, false);
};

EventEmitter.prototype.on = EventEmitter.prototype.addListener;

EventEmitter.prototype.prependListener =
    function prependListener(type, listener) {
      return _addListener(this, type, listener, true);
    };

function _onceWrap(target, type, listener) {
  var fired = false;
  function g() {
    target.removeListener(type, g);
    if (!fired) {
      fired = true;
      listener.apply(target, arguments);
    }
  }
  g.listener = listener;
  return g;
}

EventEmitter.prototype.once = function once(type, listener) {
  if (typeof listener !== 'function')
    throw new TypeError('"listener" argument must be a function');
  this.on(type, _onceWrap(this, type, listener));
  return this;
};

EventEmitter.prototype.prependOnceListener =
    function prependOnceListener(type, listener) {
      if (typeof listener !== 'function')
        throw new TypeError('"listener" argument must be a function');
      this.prependListener(type, _onceWrap(this, type, listener));
      return this;
    };

// emits a 'removeListener' event iff the listener was removed
EventEmitter.prototype.removeListener =
    function removeListener(type, listener) {
      var list, events, position, i, originalListener;

      if (typeof listener !== 'function')
        throw new TypeError('"listener" argument must be a function');

      events = this._events;
      if (!events)
        return this;

      list = events[type];
      if (!list)
        return this;

      if (list === listener || (list.listener && list.listener === listener)) {
        if (--this._eventsCount === 0)
          this._events = new EventHandlers();
        else {
          delete events[type];
          if (events.removeListener)
            this.emit('removeListener', type, list.listener || listener);
        }
      } else if (typeof list !== 'function') {
        position = -1;

        for (i = list.length; i-- > 0;) {
          if (list[i] === listener ||
              (list[i].listener && list[i].listener === listener)) {
            originalListener = list[i].listener;
            position = i;
            break;
          }
        }

        if (position < 0)
          return this;

        if (list.length === 1) {
          list[0] = undefined;
          if (--this._eventsCount === 0) {
            this._events = new EventHandlers();
            return this;
          } else {
            delete events[type];
          }
        } else {
          spliceOne(list, position);
        }

        if (events.removeListener)
          this.emit('removeListener', type, originalListener || listener);
      }

      return this;
    };

EventEmitter.prototype.removeAllListeners =
    function removeAllListeners(type) {
      var listeners, events;

      events = this._events;
      if (!events)
        return this;

      // not listening for removeListener, no need to emit
      if (!events.removeListener) {
        if (arguments.length === 0) {
          this._events = new EventHandlers();
          this._eventsCount = 0;
        } else if (events[type]) {
          if (--this._eventsCount === 0)
            this._events = new EventHandlers();
          else
            delete events[type];
        }
        return this;
      }

      // emit removeListener for all listeners on all events
      if (arguments.length === 0) {
        var keys = Object.keys(events);
        for (var i = 0, key; i < keys.length; ++i) {
          key = keys[i];
          if (key === 'removeListener') continue;
          this.removeAllListeners(key);
        }
        this.removeAllListeners('removeListener');
        this._events = new EventHandlers();
        this._eventsCount = 0;
        return this;
      }

      listeners = events[type];

      if (typeof listeners === 'function') {
        this.removeListener(type, listeners);
      } else if (listeners) {
        // LIFO order
        do {
          this.removeListener(type, listeners[listeners.length - 1]);
        } while (listeners[0]);
      }

      return this;
    };

EventEmitter.prototype.listeners = function listeners(type) {
  var evlistener;
  var ret;
  var events = this._events;

  if (!events)
    ret = [];
  else {
    evlistener = events[type];
    if (!evlistener)
      ret = [];
    else if (typeof evlistener === 'function')
      ret = [evlistener.listener || evlistener];
    else
      ret = unwrapListeners(evlistener);
  }

  return ret;
};

EventEmitter.listenerCount = function(emitter, type) {
  if (typeof emitter.listenerCount === 'function') {
    return emitter.listenerCount(type);
  } else {
    return listenerCount.call(emitter, type);
  }
};

EventEmitter.prototype.listenerCount = listenerCount;
function listenerCount(type) {
  var events = this._events;

  if (events) {
    var evlistener = events[type];

    if (typeof evlistener === 'function') {
      return 1;
    } else if (evlistener) {
      return evlistener.length;
    }
  }

  return 0;
}

EventEmitter.prototype.eventNames = function eventNames() {
  return this._eventsCount > 0 ? Reflect.ownKeys(this._events) : [];
};

// About 1.5x faster than the two-arg version of Array#splice().
function spliceOne(list, index) {
  for (var i = index, k = i + 1, n = list.length; k < n; i += 1, k += 1)
    list[i] = list[k];
  list.pop();
}

function arrayClone(arr, i) {
  var copy = new Array(i);
  while (i--)
    copy[i] = arr[i];
  return copy;
}

function unwrapListeners(arr) {
  var ret = new Array(arr.length);
  for (var i = 0; i < ret.length; ++i) {
    ret[i] = arr[i].listener || arr[i];
  }
  return ret;
}

// shim for using process in browser
// based off https://github.com/defunctzombie/node-process/blob/master/browser.js

function defaultSetTimout() {
    throw new Error('setTimeout has not been defined');
}
function defaultClearTimeout () {
    throw new Error('clearTimeout has not been defined');
}
var cachedSetTimeout = defaultSetTimout;
var cachedClearTimeout = defaultClearTimeout;
if (typeof global$1.setTimeout === 'function') {
    cachedSetTimeout = setTimeout;
}
if (typeof global$1.clearTimeout === 'function') {
    cachedClearTimeout = clearTimeout;
}

function runTimeout(fun) {
    if (cachedSetTimeout === setTimeout) {
        //normal enviroments in sane situations
        return setTimeout(fun, 0);
    }
    // if setTimeout wasn't available but was latter defined
    if ((cachedSetTimeout === defaultSetTimout || !cachedSetTimeout) && setTimeout) {
        cachedSetTimeout = setTimeout;
        return setTimeout(fun, 0);
    }
    try {
        // when when somebody has screwed with setTimeout but no I.E. maddness
        return cachedSetTimeout(fun, 0);
    } catch(e){
        try {
            // When we are in I.E. but the script has been evaled so I.E. doesn't trust the global object when called normally
            return cachedSetTimeout.call(null, fun, 0);
        } catch(e){
            // same as above but when it's a version of I.E. that must have the global object for 'this', hopfully our context correct otherwise it will throw a global error
            return cachedSetTimeout.call(this, fun, 0);
        }
    }


}
function runClearTimeout(marker) {
    if (cachedClearTimeout === clearTimeout) {
        //normal enviroments in sane situations
        return clearTimeout(marker);
    }
    // if clearTimeout wasn't available but was latter defined
    if ((cachedClearTimeout === defaultClearTimeout || !cachedClearTimeout) && clearTimeout) {
        cachedClearTimeout = clearTimeout;
        return clearTimeout(marker);
    }
    try {
        // when when somebody has screwed with setTimeout but no I.E. maddness
        return cachedClearTimeout(marker);
    } catch (e){
        try {
            // When we are in I.E. but the script has been evaled so I.E. doesn't  trust the global object when called normally
            return cachedClearTimeout.call(null, marker);
        } catch (e){
            // same as above but when it's a version of I.E. that must have the global object for 'this', hopfully our context correct otherwise it will throw a global error.
            // Some versions of I.E. have different rules for clearTimeout vs setTimeout
            return cachedClearTimeout.call(this, marker);
        }
    }



}
var queue = [];
var draining = false;
var currentQueue;
var queueIndex = -1;

function cleanUpNextTick() {
    if (!draining || !currentQueue) {
        return;
    }
    draining = false;
    if (currentQueue.length) {
        queue = currentQueue.concat(queue);
    } else {
        queueIndex = -1;
    }
    if (queue.length) {
        drainQueue();
    }
}

function drainQueue() {
    if (draining) {
        return;
    }
    var timeout = runTimeout(cleanUpNextTick);
    draining = true;

    var len = queue.length;
    while(len) {
        currentQueue = queue;
        queue = [];
        while (++queueIndex < len) {
            if (currentQueue) {
                currentQueue[queueIndex].run();
            }
        }
        queueIndex = -1;
        len = queue.length;
    }
    currentQueue = null;
    draining = false;
    runClearTimeout(timeout);
}
function nextTick(fun) {
    var args = new Array(arguments.length - 1);
    if (arguments.length > 1) {
        for (var i = 1; i < arguments.length; i++) {
            args[i - 1] = arguments[i];
        }
    }
    queue.push(new Item(fun, args));
    if (queue.length === 1 && !draining) {
        runTimeout(drainQueue);
    }
}
// v8 likes predictible objects
function Item(fun, array) {
    this.fun = fun;
    this.array = array;
}
Item.prototype.run = function () {
    this.fun.apply(null, this.array);
};
var title = 'browser';
var platform = 'browser';
var browser = true;
var env = {};
var argv = [];
var version = ''; // empty string to avoid regexp issues
var versions = {};
var release = {};
var config = {};

function noop() {}

var on = noop;
var addListener = noop;
var once = noop;
var off = noop;
var removeListener = noop;
var removeAllListeners = noop;
var emit = noop;

function binding(name) {
    throw new Error('process.binding is not supported');
}

function cwd () { return '/' }
function chdir (dir) {
    throw new Error('process.chdir is not supported');
}function umask() { return 0; }

// from https://github.com/kumavis/browser-process-hrtime/blob/master/index.js
var performance$1 = global$1.performance || {};
var performanceNow =
  performance$1.now        ||
  performance$1.mozNow     ||
  performance$1.msNow      ||
  performance$1.oNow       ||
  performance$1.webkitNow  ||
  function(){ return (new Date()).getTime() };

// generate timestamp or delta
// see http://nodejs.org/api/process.html#process_process_hrtime
function hrtime(previousTimestamp){
  var clocktime = performanceNow.call(performance$1)*1e-3;
  var seconds = Math.floor(clocktime);
  var nanoseconds = Math.floor((clocktime%1)*1e9);
  if (previousTimestamp) {
    seconds = seconds - previousTimestamp[0];
    nanoseconds = nanoseconds - previousTimestamp[1];
    if (nanoseconds<0) {
      seconds--;
      nanoseconds += 1e9;
    }
  }
  return [seconds,nanoseconds]
}

var startTime = new Date();
function uptime() {
  var currentTime = new Date();
  var dif = currentTime - startTime;
  return dif / 1000;
}

var process = {
  nextTick: nextTick,
  title: title,
  browser: browser,
  env: env,
  argv: argv,
  version: version,
  versions: versions,
  on: on,
  addListener: addListener,
  once: once,
  off: off,
  removeListener: removeListener,
  removeAllListeners: removeAllListeners,
  emit: emit,
  binding: binding,
  cwd: cwd,
  chdir: chdir,
  umask: umask,
  hrtime: hrtime,
  platform: platform,
  release: release,
  config: config,
  uptime: uptime
};

var isBufferBrowser = function isBuffer(arg) {
  return arg && typeof arg === 'object'
    && typeof arg.copy === 'function'
    && typeof arg.fill === 'function'
    && typeof arg.readUInt8 === 'function';
};

var inherits_browser = __chunk_1.createCommonjsModule(function (module) {
if (typeof Object.create === 'function') {
  // implementation from standard node.js 'util' module
  module.exports = function inherits(ctor, superCtor) {
    ctor.super_ = superCtor;
    ctor.prototype = Object.create(superCtor.prototype, {
      constructor: {
        value: ctor,
        enumerable: false,
        writable: true,
        configurable: true
      }
    });
  };
} else {
  // old school shim for old browsers
  module.exports = function inherits(ctor, superCtor) {
    ctor.super_ = superCtor;
    var TempCtor = function () {};
    TempCtor.prototype = superCtor.prototype;
    ctor.prototype = new TempCtor();
    ctor.prototype.constructor = ctor;
  };
}
});

var util = __chunk_1.createCommonjsModule(function (module, exports) {
// Copyright Joyent, Inc. and other Node contributors.
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
// USE OR OTHER DEALINGS IN THE SOFTWARE.

var formatRegExp = /%[sdj%]/g;
exports.format = function(f) {
  if (!isString(f)) {
    var objects = [];
    for (var i = 0; i < arguments.length; i++) {
      objects.push(inspect(arguments[i]));
    }
    return objects.join(' ');
  }

  var i = 1;
  var args = arguments;
  var len = args.length;
  var str = String(f).replace(formatRegExp, function(x) {
    if (x === '%%') return '%';
    if (i >= len) return x;
    switch (x) {
      case '%s': return String(args[i++]);
      case '%d': return Number(args[i++]);
      case '%j':
        try {
          return JSON.stringify(args[i++]);
        } catch (_) {
          return '[Circular]';
        }
      default:
        return x;
    }
  });
  for (var x = args[i]; i < len; x = args[++i]) {
    if (isNull(x) || !isObject(x)) {
      str += ' ' + x;
    } else {
      str += ' ' + inspect(x);
    }
  }
  return str;
};


// Mark that a method should not be used.
// Returns a modified function which warns once by default.
// If --no-deprecation is set, then it is a no-op.
exports.deprecate = function(fn, msg) {
  // Allow for deprecating things in the process of starting up.
  if (isUndefined(global$1.process)) {
    return function() {
      return exports.deprecate(fn, msg).apply(this, arguments);
    };
  }

  var warned = false;
  function deprecated() {
    if (!warned) {
      {
        console.error(msg);
      }
      warned = true;
    }
    return fn.apply(this, arguments);
  }

  return deprecated;
};


var debugs = {};
var debugEnviron;
exports.debuglog = function(set) {
  if (isUndefined(debugEnviron))
    debugEnviron = process.env.NODE_DEBUG || '';
  set = set.toUpperCase();
  if (!debugs[set]) {
    if (new RegExp('\\b' + set + '\\b', 'i').test(debugEnviron)) {
      var pid = process.pid;
      debugs[set] = function() {
        var msg = exports.format.apply(exports, arguments);
        console.error('%s %d: %s', set, pid, msg);
      };
    } else {
      debugs[set] = function() {};
    }
  }
  return debugs[set];
};


/**
 * Echos the value of a value. Trys to print the value out
 * in the best way possible given the different types.
 *
 * @param {Object} obj The object to print out.
 * @param {Object} opts Optional options object that alters the output.
 */
/* legacy: obj, showHidden, depth, colors*/
function inspect(obj, opts) {
  // default options
  var ctx = {
    seen: [],
    stylize: stylizeNoColor
  };
  // legacy...
  if (arguments.length >= 3) ctx.depth = arguments[2];
  if (arguments.length >= 4) ctx.colors = arguments[3];
  if (isBoolean(opts)) {
    // legacy...
    ctx.showHidden = opts;
  } else if (opts) {
    // got an "options" object
    exports._extend(ctx, opts);
  }
  // set default options
  if (isUndefined(ctx.showHidden)) ctx.showHidden = false;
  if (isUndefined(ctx.depth)) ctx.depth = 2;
  if (isUndefined(ctx.colors)) ctx.colors = false;
  if (isUndefined(ctx.customInspect)) ctx.customInspect = true;
  if (ctx.colors) ctx.stylize = stylizeWithColor;
  return formatValue(ctx, obj, ctx.depth);
}
exports.inspect = inspect;


// http://en.wikipedia.org/wiki/ANSI_escape_code#graphics
inspect.colors = {
  'bold' : [1, 22],
  'italic' : [3, 23],
  'underline' : [4, 24],
  'inverse' : [7, 27],
  'white' : [37, 39],
  'grey' : [90, 39],
  'black' : [30, 39],
  'blue' : [34, 39],
  'cyan' : [36, 39],
  'green' : [32, 39],
  'magenta' : [35, 39],
  'red' : [31, 39],
  'yellow' : [33, 39]
};

// Don't use 'blue' not visible on cmd.exe
inspect.styles = {
  'special': 'cyan',
  'number': 'yellow',
  'boolean': 'yellow',
  'undefined': 'grey',
  'null': 'bold',
  'string': 'green',
  'date': 'magenta',
  // "name": intentionally not styling
  'regexp': 'red'
};


function stylizeWithColor(str, styleType) {
  var style = inspect.styles[styleType];

  if (style) {
    return '\u001b[' + inspect.colors[style][0] + 'm' + str +
           '\u001b[' + inspect.colors[style][1] + 'm';
  } else {
    return str;
  }
}


function stylizeNoColor(str, styleType) {
  return str;
}


function arrayToHash(array) {
  var hash = {};

  array.forEach(function(val, idx) {
    hash[val] = true;
  });

  return hash;
}


function formatValue(ctx, value, recurseTimes) {
  // Provide a hook for user-specified inspect functions.
  // Check that value is an object with an inspect function on it
  if (ctx.customInspect &&
      value &&
      isFunction(value.inspect) &&
      // Filter out the util module, it's inspect function is special
      value.inspect !== exports.inspect &&
      // Also filter out any prototype objects using the circular check.
      !(value.constructor && value.constructor.prototype === value)) {
    var ret = value.inspect(recurseTimes, ctx);
    if (!isString(ret)) {
      ret = formatValue(ctx, ret, recurseTimes);
    }
    return ret;
  }

  // Primitive types cannot have properties
  var primitive = formatPrimitive(ctx, value);
  if (primitive) {
    return primitive;
  }

  // Look up the keys of the object.
  var keys = Object.keys(value);
  var visibleKeys = arrayToHash(keys);

  if (ctx.showHidden) {
    keys = Object.getOwnPropertyNames(value);
  }

  // IE doesn't make error fields non-enumerable
  // http://msdn.microsoft.com/en-us/library/ie/dww52sbt(v=vs.94).aspx
  if (isError(value)
      && (keys.indexOf('message') >= 0 || keys.indexOf('description') >= 0)) {
    return formatError(value);
  }

  // Some type of object without properties can be shortcutted.
  if (keys.length === 0) {
    if (isFunction(value)) {
      var name = value.name ? ': ' + value.name : '';
      return ctx.stylize('[Function' + name + ']', 'special');
    }
    if (isRegExp(value)) {
      return ctx.stylize(RegExp.prototype.toString.call(value), 'regexp');
    }
    if (isDate(value)) {
      return ctx.stylize(Date.prototype.toString.call(value), 'date');
    }
    if (isError(value)) {
      return formatError(value);
    }
  }

  var base = '', array = false, braces = ['{', '}'];

  // Make Array say that they are Array
  if (isArray(value)) {
    array = true;
    braces = ['[', ']'];
  }

  // Make functions say that they are functions
  if (isFunction(value)) {
    var n = value.name ? ': ' + value.name : '';
    base = ' [Function' + n + ']';
  }

  // Make RegExps say that they are RegExps
  if (isRegExp(value)) {
    base = ' ' + RegExp.prototype.toString.call(value);
  }

  // Make dates with properties first say the date
  if (isDate(value)) {
    base = ' ' + Date.prototype.toUTCString.call(value);
  }

  // Make error with message first say the error
  if (isError(value)) {
    base = ' ' + formatError(value);
  }

  if (keys.length === 0 && (!array || value.length == 0)) {
    return braces[0] + base + braces[1];
  }

  if (recurseTimes < 0) {
    if (isRegExp(value)) {
      return ctx.stylize(RegExp.prototype.toString.call(value), 'regexp');
    } else {
      return ctx.stylize('[Object]', 'special');
    }
  }

  ctx.seen.push(value);

  var output;
  if (array) {
    output = formatArray(ctx, value, recurseTimes, visibleKeys, keys);
  } else {
    output = keys.map(function(key) {
      return formatProperty(ctx, value, recurseTimes, visibleKeys, key, array);
    });
  }

  ctx.seen.pop();

  return reduceToSingleString(output, base, braces);
}


function formatPrimitive(ctx, value) {
  if (isUndefined(value))
    return ctx.stylize('undefined', 'undefined');
  if (isString(value)) {
    var simple = '\'' + JSON.stringify(value).replace(/^"|"$/g, '')
                                             .replace(/'/g, "\\'")
                                             .replace(/\\"/g, '"') + '\'';
    return ctx.stylize(simple, 'string');
  }
  if (isNumber(value))
    return ctx.stylize('' + value, 'number');
  if (isBoolean(value))
    return ctx.stylize('' + value, 'boolean');
  // For some reason typeof null is "object", so special case here.
  if (isNull(value))
    return ctx.stylize('null', 'null');
}


function formatError(value) {
  return '[' + Error.prototype.toString.call(value) + ']';
}


function formatArray(ctx, value, recurseTimes, visibleKeys, keys) {
  var output = [];
  for (var i = 0, l = value.length; i < l; ++i) {
    if (hasOwnProperty(value, String(i))) {
      output.push(formatProperty(ctx, value, recurseTimes, visibleKeys,
          String(i), true));
    } else {
      output.push('');
    }
  }
  keys.forEach(function(key) {
    if (!key.match(/^\d+$/)) {
      output.push(formatProperty(ctx, value, recurseTimes, visibleKeys,
          key, true));
    }
  });
  return output;
}


function formatProperty(ctx, value, recurseTimes, visibleKeys, key, array) {
  var name, str, desc;
  desc = Object.getOwnPropertyDescriptor(value, key) || { value: value[key] };
  if (desc.get) {
    if (desc.set) {
      str = ctx.stylize('[Getter/Setter]', 'special');
    } else {
      str = ctx.stylize('[Getter]', 'special');
    }
  } else {
    if (desc.set) {
      str = ctx.stylize('[Setter]', 'special');
    }
  }
  if (!hasOwnProperty(visibleKeys, key)) {
    name = '[' + key + ']';
  }
  if (!str) {
    if (ctx.seen.indexOf(desc.value) < 0) {
      if (isNull(recurseTimes)) {
        str = formatValue(ctx, desc.value, null);
      } else {
        str = formatValue(ctx, desc.value, recurseTimes - 1);
      }
      if (str.indexOf('\n') > -1) {
        if (array) {
          str = str.split('\n').map(function(line) {
            return '  ' + line;
          }).join('\n').substr(2);
        } else {
          str = '\n' + str.split('\n').map(function(line) {
            return '   ' + line;
          }).join('\n');
        }
      }
    } else {
      str = ctx.stylize('[Circular]', 'special');
    }
  }
  if (isUndefined(name)) {
    if (array && key.match(/^\d+$/)) {
      return str;
    }
    name = JSON.stringify('' + key);
    if (name.match(/^"([a-zA-Z_][a-zA-Z_0-9]*)"$/)) {
      name = name.substr(1, name.length - 2);
      name = ctx.stylize(name, 'name');
    } else {
      name = name.replace(/'/g, "\\'")
                 .replace(/\\"/g, '"')
                 .replace(/(^"|"$)/g, "'");
      name = ctx.stylize(name, 'string');
    }
  }

  return name + ': ' + str;
}


function reduceToSingleString(output, base, braces) {
  var length = output.reduce(function(prev, cur) {
    if (cur.indexOf('\n') >= 0) ;
    return prev + cur.replace(/\u001b\[\d\d?m/g, '').length + 1;
  }, 0);

  if (length > 60) {
    return braces[0] +
           (base === '' ? '' : base + '\n ') +
           ' ' +
           output.join(',\n  ') +
           ' ' +
           braces[1];
  }

  return braces[0] + base + ' ' + output.join(', ') + ' ' + braces[1];
}


// NOTE: These type checking functions intentionally don't use `instanceof`
// because it is fragile and can be easily faked with `Object.create()`.
function isArray(ar) {
  return Array.isArray(ar);
}
exports.isArray = isArray;

function isBoolean(arg) {
  return typeof arg === 'boolean';
}
exports.isBoolean = isBoolean;

function isNull(arg) {
  return arg === null;
}
exports.isNull = isNull;

function isNullOrUndefined(arg) {
  return arg == null;
}
exports.isNullOrUndefined = isNullOrUndefined;

function isNumber(arg) {
  return typeof arg === 'number';
}
exports.isNumber = isNumber;

function isString(arg) {
  return typeof arg === 'string';
}
exports.isString = isString;

function isSymbol(arg) {
  return typeof arg === 'symbol';
}
exports.isSymbol = isSymbol;

function isUndefined(arg) {
  return arg === void 0;
}
exports.isUndefined = isUndefined;

function isRegExp(re) {
  return isObject(re) && objectToString(re) === '[object RegExp]';
}
exports.isRegExp = isRegExp;

function isObject(arg) {
  return typeof arg === 'object' && arg !== null;
}
exports.isObject = isObject;

function isDate(d) {
  return isObject(d) && objectToString(d) === '[object Date]';
}
exports.isDate = isDate;

function isError(e) {
  return isObject(e) &&
      (objectToString(e) === '[object Error]' || e instanceof Error);
}
exports.isError = isError;

function isFunction(arg) {
  return typeof arg === 'function';
}
exports.isFunction = isFunction;

function isPrimitive(arg) {
  return arg === null ||
         typeof arg === 'boolean' ||
         typeof arg === 'number' ||
         typeof arg === 'string' ||
         typeof arg === 'symbol' ||  // ES6 symbol
         typeof arg === 'undefined';
}
exports.isPrimitive = isPrimitive;

exports.isBuffer = isBufferBrowser;

function objectToString(o) {
  return Object.prototype.toString.call(o);
}


function pad(n) {
  return n < 10 ? '0' + n.toString(10) : n.toString(10);
}


var months = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep',
              'Oct', 'Nov', 'Dec'];

// 26 Feb 16:19:34
function timestamp() {
  var d = new Date();
  var time = [pad(d.getHours()),
              pad(d.getMinutes()),
              pad(d.getSeconds())].join(':');
  return [d.getDate(), months[d.getMonth()], time].join(' ');
}


// log is just a thin wrapper to console.log that prepends a timestamp
exports.log = function() {
  console.log('%s - %s', timestamp(), exports.format.apply(exports, arguments));
};


/**
 * Inherit the prototype methods from one constructor into another.
 *
 * The Function.prototype.inherits from lang.js rewritten as a standalone
 * function (not on Function.prototype). NOTE: If this file is to be loaded
 * during bootstrapping this function needs to be rewritten using some native
 * functions as prototype setup using normal JavaScript does not work as
 * expected during bootstrapping (see mirror.js in r114903).
 *
 * @param {function} ctor Constructor function which needs to inherit the
 *     prototype.
 * @param {function} superCtor Constructor function to inherit prototype from.
 */
exports.inherits = inherits_browser;

exports._extend = function(origin, add) {
  // Don't do anything if add isn't an object
  if (!add || !isObject(add)) return origin;

  var keys = Object.keys(add);
  var i = keys.length;
  while (i--) {
    origin[keys[i]] = add[keys[i]];
  }
  return origin;
};

function hasOwnProperty(obj, prop) {
  return Object.prototype.hasOwnProperty.call(obj, prop);
}
});
var util_1 = util.format;
var util_2 = util.deprecate;
var util_3 = util.debuglog;
var util_4 = util.inspect;
var util_5 = util.isArray;
var util_6 = util.isBoolean;
var util_7 = util.isNull;
var util_8 = util.isNullOrUndefined;
var util_9 = util.isNumber;
var util_10 = util.isString;
var util_11 = util.isSymbol;
var util_12 = util.isUndefined;
var util_13 = util.isRegExp;
var util_14 = util.isObject;
var util_15 = util.isDate;
var util_16 = util.isError;
var util_17 = util.isFunction;
var util_18 = util.isPrimitive;
var util_19 = util.isBuffer;
var util_20 = util.log;
var util_21 = util.inherits;
var util_22 = util._extend;

function BufferList() {
  this.head = null;
  this.tail = null;
  this.length = 0;
}

BufferList.prototype.push = function (v) {
  var entry = { data: v, next: null };
  if (this.length > 0) this.tail.next = entry;else this.head = entry;
  this.tail = entry;
  ++this.length;
};

BufferList.prototype.unshift = function (v) {
  var entry = { data: v, next: this.head };
  if (this.length === 0) this.tail = entry;
  this.head = entry;
  ++this.length;
};

BufferList.prototype.shift = function () {
  if (this.length === 0) return;
  var ret = this.head.data;
  if (this.length === 1) this.head = this.tail = null;else this.head = this.head.next;
  --this.length;
  return ret;
};

BufferList.prototype.clear = function () {
  this.head = this.tail = null;
  this.length = 0;
};

BufferList.prototype.join = function (s) {
  if (this.length === 0) return '';
  var p = this.head;
  var ret = '' + p.data;
  while (p = p.next) {
    ret += s + p.data;
  }return ret;
};

BufferList.prototype.concat = function (n) {
  if (this.length === 0) return Buffer.alloc(0);
  if (this.length === 1) return this.head.data;
  var ret = Buffer.allocUnsafe(n >>> 0);
  var p = this.head;
  var i = 0;
  while (p) {
    p.data.copy(ret, i);
    i += p.data.length;
    p = p.next;
  }
  return ret;
};

var safeBuffer = __chunk_1.createCommonjsModule(function (module, exports) {
/* eslint-disable node/no-deprecated-api */

var Buffer = bufferEs6.Buffer;

// alternative to using Object.keys for old browsers
function copyProps (src, dst) {
  for (var key in src) {
    dst[key] = src[key];
  }
}
if (Buffer.from && Buffer.alloc && Buffer.allocUnsafe && Buffer.allocUnsafeSlow) {
  module.exports = bufferEs6;
} else {
  // Copy properties from require('buffer')
  copyProps(bufferEs6, exports);
  exports.Buffer = SafeBuffer;
}

function SafeBuffer (arg, encodingOrOffset, length) {
  return Buffer(arg, encodingOrOffset, length)
}

// Copy static methods from Buffer
copyProps(Buffer, SafeBuffer);

SafeBuffer.from = function (arg, encodingOrOffset, length) {
  if (typeof arg === 'number') {
    throw new TypeError('Argument must not be a number')
  }
  return Buffer(arg, encodingOrOffset, length)
};

SafeBuffer.alloc = function (size, fill, encoding) {
  if (typeof size !== 'number') {
    throw new TypeError('Argument must be a number')
  }
  var buf = Buffer(size);
  if (fill !== undefined) {
    if (typeof encoding === 'string') {
      buf.fill(fill, encoding);
    } else {
      buf.fill(fill);
    }
  } else {
    buf.fill(0);
  }
  return buf
};

SafeBuffer.allocUnsafe = function (size) {
  if (typeof size !== 'number') {
    throw new TypeError('Argument must be a number')
  }
  return Buffer(size)
};

SafeBuffer.allocUnsafeSlow = function (size) {
  if (typeof size !== 'number') {
    throw new TypeError('Argument must be a number')
  }
  return bufferEs6.SlowBuffer(size)
};
});
var safeBuffer_1 = safeBuffer.Buffer;

var Buffer$1 = safeBuffer.Buffer;

var isEncoding = Buffer$1.isEncoding || function (encoding) {
  encoding = '' + encoding;
  switch (encoding && encoding.toLowerCase()) {
    case 'hex':case 'utf8':case 'utf-8':case 'ascii':case 'binary':case 'base64':case 'ucs2':case 'ucs-2':case 'utf16le':case 'utf-16le':case 'raw':
      return true;
    default:
      return false;
  }
};

function _normalizeEncoding(enc) {
  if (!enc) return 'utf8';
  var retried;
  while (true) {
    switch (enc) {
      case 'utf8':
      case 'utf-8':
        return 'utf8';
      case 'ucs2':
      case 'ucs-2':
      case 'utf16le':
      case 'utf-16le':
        return 'utf16le';
      case 'latin1':
      case 'binary':
        return 'latin1';
      case 'base64':
      case 'ascii':
      case 'hex':
        return enc;
      default:
        if (retried) return; // undefined
        enc = ('' + enc).toLowerCase();
        retried = true;
    }
  }
}
// Do not cache `Buffer.isEncoding` when checking encoding names as some
// modules monkey-patch it to support additional encodings
function normalizeEncoding(enc) {
  var nenc = _normalizeEncoding(enc);
  if (typeof nenc !== 'string' && (Buffer$1.isEncoding === isEncoding || !isEncoding(enc))) throw new Error('Unknown encoding: ' + enc);
  return nenc || enc;
}

// StringDecoder provides an interface for efficiently splitting a series of
// buffers into a series of JS strings without breaking apart multi-byte
// characters.
var StringDecoder_1 = StringDecoder;
function StringDecoder(encoding) {
  this.encoding = normalizeEncoding(encoding);
  var nb;
  switch (this.encoding) {
    case 'utf16le':
      this.text = utf16Text;
      this.end = utf16End;
      nb = 4;
      break;
    case 'utf8':
      this.fillLast = utf8FillLast;
      nb = 4;
      break;
    case 'base64':
      this.text = base64Text;
      this.end = base64End;
      nb = 3;
      break;
    default:
      this.write = simpleWrite;
      this.end = simpleEnd;
      return;
  }
  this.lastNeed = 0;
  this.lastTotal = 0;
  this.lastChar = Buffer$1.allocUnsafe(nb);
}

StringDecoder.prototype.write = function (buf) {
  if (buf.length === 0) return '';
  var r;
  var i;
  if (this.lastNeed) {
    r = this.fillLast(buf);
    if (r === undefined) return '';
    i = this.lastNeed;
    this.lastNeed = 0;
  } else {
    i = 0;
  }
  if (i < buf.length) return r ? r + this.text(buf, i) : this.text(buf, i);
  return r || '';
};

StringDecoder.prototype.end = utf8End;

// Returns only complete characters in a Buffer
StringDecoder.prototype.text = utf8Text;

// Attempts to complete a partial non-UTF-8 character using bytes from a Buffer
StringDecoder.prototype.fillLast = function (buf) {
  if (this.lastNeed <= buf.length) {
    buf.copy(this.lastChar, this.lastTotal - this.lastNeed, 0, this.lastNeed);
    return this.lastChar.toString(this.encoding, 0, this.lastTotal);
  }
  buf.copy(this.lastChar, this.lastTotal - this.lastNeed, 0, buf.length);
  this.lastNeed -= buf.length;
};

// Checks the type of a UTF-8 byte, whether it's ASCII, a leading byte, or a
// continuation byte.
function utf8CheckByte(byte) {
  if (byte <= 0x7F) return 0;else if (byte >> 5 === 0x06) return 2;else if (byte >> 4 === 0x0E) return 3;else if (byte >> 3 === 0x1E) return 4;
  return -1;
}

// Checks at most 3 bytes at the end of a Buffer in order to detect an
// incomplete multi-byte UTF-8 character. The total number of bytes (2, 3, or 4)
// needed to complete the UTF-8 character (if applicable) are returned.
function utf8CheckIncomplete(self, buf, i) {
  var j = buf.length - 1;
  if (j < i) return 0;
  var nb = utf8CheckByte(buf[j]);
  if (nb >= 0) {
    if (nb > 0) self.lastNeed = nb - 1;
    return nb;
  }
  if (--j < i) return 0;
  nb = utf8CheckByte(buf[j]);
  if (nb >= 0) {
    if (nb > 0) self.lastNeed = nb - 2;
    return nb;
  }
  if (--j < i) return 0;
  nb = utf8CheckByte(buf[j]);
  if (nb >= 0) {
    if (nb > 0) {
      if (nb === 2) nb = 0;else self.lastNeed = nb - 3;
    }
    return nb;
  }
  return 0;
}

// Validates as many continuation bytes for a multi-byte UTF-8 character as
// needed or are available. If we see a non-continuation byte where we expect
// one, we "replace" the validated continuation bytes we've seen so far with
// UTF-8 replacement characters ('\ufffd'), to match v8's UTF-8 decoding
// behavior. The continuation byte check is included three times in the case
// where all of the continuation bytes for a character exist in the same buffer.
// It is also done this way as a slight performance increase instead of using a
// loop.
function utf8CheckExtraBytes(self, buf, p) {
  if ((buf[0] & 0xC0) !== 0x80) {
    self.lastNeed = 0;
    return '\ufffd'.repeat(p);
  }
  if (self.lastNeed > 1 && buf.length > 1) {
    if ((buf[1] & 0xC0) !== 0x80) {
      self.lastNeed = 1;
      return '\ufffd'.repeat(p + 1);
    }
    if (self.lastNeed > 2 && buf.length > 2) {
      if ((buf[2] & 0xC0) !== 0x80) {
        self.lastNeed = 2;
        return '\ufffd'.repeat(p + 2);
      }
    }
  }
}

// Attempts to complete a multi-byte UTF-8 character using bytes from a Buffer.
function utf8FillLast(buf) {
  var p = this.lastTotal - this.lastNeed;
  var r = utf8CheckExtraBytes(this, buf, p);
  if (r !== undefined) return r;
  if (this.lastNeed <= buf.length) {
    buf.copy(this.lastChar, p, 0, this.lastNeed);
    return this.lastChar.toString(this.encoding, 0, this.lastTotal);
  }
  buf.copy(this.lastChar, p, 0, buf.length);
  this.lastNeed -= buf.length;
}

// Returns all complete UTF-8 characters in a Buffer. If the Buffer ended on a
// partial character, the character's bytes are buffered until the required
// number of bytes are available.
function utf8Text(buf, i) {
  var total = utf8CheckIncomplete(this, buf, i);
  if (!this.lastNeed) return buf.toString('utf8', i);
  this.lastTotal = total;
  var end = buf.length - (total - this.lastNeed);
  buf.copy(this.lastChar, 0, end);
  return buf.toString('utf8', i, end);
}

// For UTF-8, a replacement character for each buffered byte of a (partial)
// character needs to be added to the output.
function utf8End(buf) {
  var r = buf && buf.length ? this.write(buf) : '';
  if (this.lastNeed) return r + '\ufffd'.repeat(this.lastTotal - this.lastNeed);
  return r;
}

// UTF-16LE typically needs two bytes per character, but even if we have an even
// number of bytes available, we need to check if we end on a leading/high
// surrogate. In that case, we need to wait for the next two bytes in order to
// decode the last character properly.
function utf16Text(buf, i) {
  if ((buf.length - i) % 2 === 0) {
    var r = buf.toString('utf16le', i);
    if (r) {
      var c = r.charCodeAt(r.length - 1);
      if (c >= 0xD800 && c <= 0xDBFF) {
        this.lastNeed = 2;
        this.lastTotal = 4;
        this.lastChar[0] = buf[buf.length - 2];
        this.lastChar[1] = buf[buf.length - 1];
        return r.slice(0, -1);
      }
    }
    return r;
  }
  this.lastNeed = 1;
  this.lastTotal = 2;
  this.lastChar[0] = buf[buf.length - 1];
  return buf.toString('utf16le', i, buf.length - 1);
}

// For UTF-16LE we do not explicitly append special replacement characters if we
// end on a partial character, we simply let v8 handle that.
function utf16End(buf) {
  var r = buf && buf.length ? this.write(buf) : '';
  if (this.lastNeed) {
    var end = this.lastTotal - this.lastNeed;
    return r + this.lastChar.toString('utf16le', 0, end);
  }
  return r;
}

function base64Text(buf, i) {
  var n = (buf.length - i) % 3;
  if (n === 0) return buf.toString('base64', i);
  this.lastNeed = 3 - n;
  this.lastTotal = 3;
  if (n === 1) {
    this.lastChar[0] = buf[buf.length - 1];
  } else {
    this.lastChar[0] = buf[buf.length - 2];
    this.lastChar[1] = buf[buf.length - 1];
  }
  return buf.toString('base64', i, buf.length - n);
}

function base64End(buf) {
  var r = buf && buf.length ? this.write(buf) : '';
  if (this.lastNeed) return r + this.lastChar.toString('base64', 0, 3 - this.lastNeed);
  return r;
}

// Pass bytes on through for single-byte encodings (e.g. ascii, latin1, hex)
function simpleWrite(buf) {
  return buf.toString(this.encoding);
}

function simpleEnd(buf) {
  return buf && buf.length ? this.write(buf) : '';
}

Readable.ReadableState = ReadableState;

var debug = util_3('stream');
util_21(Readable, EventEmitter);

function prependListener(emitter, event, fn) {
  // Sadly this is not cacheable as some libraries bundle their own
  // event emitter implementation with them.
  if (typeof emitter.prependListener === 'function') {
    return emitter.prependListener(event, fn);
  } else {
    // This is a hack to make sure that our error handler is attached before any
    // userland ones.  NEVER DO THIS. This is here only because this code needs
    // to continue to work with older versions of Node.js that do not include
    // the prependListener() method. The goal is to eventually remove this hack.
    if (!emitter._events || !emitter._events[event])
      emitter.on(event, fn);
    else if (Array.isArray(emitter._events[event]))
      emitter._events[event].unshift(fn);
    else
      emitter._events[event] = [fn, emitter._events[event]];
  }
}
function listenerCount$1 (emitter, type) {
  return emitter.listeners(type).length;
}
function ReadableState(options, stream) {

  options = options || {};

  // object stream flag. Used to make read(n) ignore n and to
  // make all the buffer merging and length checks go away
  this.objectMode = !!options.objectMode;

  if (stream instanceof Duplex) this.objectMode = this.objectMode || !!options.readableObjectMode;

  // the point at which it stops calling _read() to fill the buffer
  // Note: 0 is a valid value, means "don't call _read preemptively ever"
  var hwm = options.highWaterMark;
  var defaultHwm = this.objectMode ? 16 : 16 * 1024;
  this.highWaterMark = hwm || hwm === 0 ? hwm : defaultHwm;

  // cast to ints.
  this.highWaterMark = ~ ~this.highWaterMark;

  // A linked list is used to store data chunks instead of an array because the
  // linked list can remove elements from the beginning faster than
  // array.shift()
  this.buffer = new BufferList();
  this.length = 0;
  this.pipes = null;
  this.pipesCount = 0;
  this.flowing = null;
  this.ended = false;
  this.endEmitted = false;
  this.reading = false;

  // a flag to be able to tell if the onwrite cb is called immediately,
  // or on a later tick.  We set this to true at first, because any
  // actions that shouldn't happen until "later" should generally also
  // not happen before the first write call.
  this.sync = true;

  // whenever we return null, then we set a flag to say
  // that we're awaiting a 'readable' event emission.
  this.needReadable = false;
  this.emittedReadable = false;
  this.readableListening = false;
  this.resumeScheduled = false;

  // Crypto is kind of old and crusty.  Historically, its default string
  // encoding is 'binary' so we have to make this configurable.
  // Everything else in the universe uses 'utf8', though.
  this.defaultEncoding = options.defaultEncoding || 'utf8';

  // when piping, we only care about 'readable' events that happen
  // after read()ing all the bytes and not getting any pushback.
  this.ranOut = false;

  // the number of writers that are awaiting a drain event in .pipe()s
  this.awaitDrain = 0;

  // if true, a maybeReadMore has been scheduled
  this.readingMore = false;

  this.decoder = null;
  this.encoding = null;
  if (options.encoding) {
    this.decoder = new StringDecoder_1(options.encoding);
    this.encoding = options.encoding;
  }
}
function Readable(options) {

  if (!(this instanceof Readable)) return new Readable(options);

  this._readableState = new ReadableState(options, this);

  // legacy
  this.readable = true;

  if (options && typeof options.read === 'function') this._read = options.read;

  EventEmitter.call(this);
}

// Manually shove something into the read() buffer.
// This returns true if the highWaterMark has not been hit yet,
// similar to how Writable.write() returns true if you should
// write() some more.
Readable.prototype.push = function (chunk, encoding) {
  var state = this._readableState;

  if (!state.objectMode && typeof chunk === 'string') {
    encoding = encoding || state.defaultEncoding;
    if (encoding !== state.encoding) {
      chunk = Buffer.from(chunk, encoding);
      encoding = '';
    }
  }

  return readableAddChunk(this, state, chunk, encoding, false);
};

// Unshift should *always* be something directly out of read()
Readable.prototype.unshift = function (chunk) {
  var state = this._readableState;
  return readableAddChunk(this, state, chunk, '', true);
};

Readable.prototype.isPaused = function () {
  return this._readableState.flowing === false;
};

function readableAddChunk(stream, state, chunk, encoding, addToFront) {
  var er = chunkInvalid(state, chunk);
  if (er) {
    stream.emit('error', er);
  } else if (chunk === null) {
    state.reading = false;
    onEofChunk(stream, state);
  } else if (state.objectMode || chunk && chunk.length > 0) {
    if (state.ended && !addToFront) {
      var e = new Error('stream.push() after EOF');
      stream.emit('error', e);
    } else if (state.endEmitted && addToFront) {
      var _e = new Error('stream.unshift() after end event');
      stream.emit('error', _e);
    } else {
      var skipAdd;
      if (state.decoder && !addToFront && !encoding) {
        chunk = state.decoder.write(chunk);
        skipAdd = !state.objectMode && chunk.length === 0;
      }

      if (!addToFront) state.reading = false;

      // Don't add to the buffer if we've decoded to an empty string chunk and
      // we're not in object mode
      if (!skipAdd) {
        // if we want the data now, just emit it.
        if (state.flowing && state.length === 0 && !state.sync) {
          stream.emit('data', chunk);
          stream.read(0);
        } else {
          // update the buffer info.
          state.length += state.objectMode ? 1 : chunk.length;
          if (addToFront) state.buffer.unshift(chunk);else state.buffer.push(chunk);

          if (state.needReadable) emitReadable(stream);
        }
      }

      maybeReadMore(stream, state);
    }
  } else if (!addToFront) {
    state.reading = false;
  }

  return needMoreData(state);
}

// if it's past the high water mark, we can push in some more.
// Also, if we have no data yet, we can stand some
// more bytes.  This is to work around cases where hwm=0,
// such as the repl.  Also, if the push() triggered a
// readable event, and the user called read(largeNumber) such that
// needReadable was set, then we ought to push more, so that another
// 'readable' event will be triggered.
function needMoreData(state) {
  return !state.ended && (state.needReadable || state.length < state.highWaterMark || state.length === 0);
}

// backwards compatibility.
Readable.prototype.setEncoding = function (enc) {
  this._readableState.decoder = new StringDecoder_1(enc);
  this._readableState.encoding = enc;
  return this;
};

// Don't raise the hwm > 8MB
var MAX_HWM = 0x800000;
function computeNewHighWaterMark(n) {
  if (n >= MAX_HWM) {
    n = MAX_HWM;
  } else {
    // Get the next highest power of 2 to prevent increasing hwm excessively in
    // tiny amounts
    n--;
    n |= n >>> 1;
    n |= n >>> 2;
    n |= n >>> 4;
    n |= n >>> 8;
    n |= n >>> 16;
    n++;
  }
  return n;
}

// This function is designed to be inlinable, so please take care when making
// changes to the function body.
function howMuchToRead(n, state) {
  if (n <= 0 || state.length === 0 && state.ended) return 0;
  if (state.objectMode) return 1;
  if (n !== n) {
    // Only flow one buffer at a time
    if (state.flowing && state.length) return state.buffer.head.data.length;else return state.length;
  }
  // If we're asking for more than the current hwm, then raise the hwm.
  if (n > state.highWaterMark) state.highWaterMark = computeNewHighWaterMark(n);
  if (n <= state.length) return n;
  // Don't have enough
  if (!state.ended) {
    state.needReadable = true;
    return 0;
  }
  return state.length;
}

// you can override either this method, or the async _read(n) below.
Readable.prototype.read = function (n) {
  debug('read', n);
  n = parseInt(n, 10);
  var state = this._readableState;
  var nOrig = n;

  if (n !== 0) state.emittedReadable = false;

  // if we're doing read(0) to trigger a readable event, but we
  // already have a bunch of data in the buffer, then just trigger
  // the 'readable' event and move on.
  if (n === 0 && state.needReadable && (state.length >= state.highWaterMark || state.ended)) {
    debug('read: emitReadable', state.length, state.ended);
    if (state.length === 0 && state.ended) endReadable(this);else emitReadable(this);
    return null;
  }

  n = howMuchToRead(n, state);

  // if we've ended, and we're now clear, then finish it up.
  if (n === 0 && state.ended) {
    if (state.length === 0) endReadable(this);
    return null;
  }

  // All the actual chunk generation logic needs to be
  // *below* the call to _read.  The reason is that in certain
  // synthetic stream cases, such as passthrough streams, _read
  // may be a completely synchronous operation which may change
  // the state of the read buffer, providing enough data when
  // before there was *not* enough.
  //
  // So, the steps are:
  // 1. Figure out what the state of things will be after we do
  // a read from the buffer.
  //
  // 2. If that resulting state will trigger a _read, then call _read.
  // Note that this may be asynchronous, or synchronous.  Yes, it is
  // deeply ugly to write APIs this way, but that still doesn't mean
  // that the Readable class should behave improperly, as streams are
  // designed to be sync/async agnostic.
  // Take note if the _read call is sync or async (ie, if the read call
  // has returned yet), so that we know whether or not it's safe to emit
  // 'readable' etc.
  //
  // 3. Actually pull the requested chunks out of the buffer and return.

  // if we need a readable event, then we need to do some reading.
  var doRead = state.needReadable;
  debug('need readable', doRead);

  // if we currently have less than the highWaterMark, then also read some
  if (state.length === 0 || state.length - n < state.highWaterMark) {
    doRead = true;
    debug('length less than watermark', doRead);
  }

  // however, if we've ended, then there's no point, and if we're already
  // reading, then it's unnecessary.
  if (state.ended || state.reading) {
    doRead = false;
    debug('reading or ended', doRead);
  } else if (doRead) {
    debug('do read');
    state.reading = true;
    state.sync = true;
    // if the length is currently zero, then we *need* a readable event.
    if (state.length === 0) state.needReadable = true;
    // call internal read method
    this._read(state.highWaterMark);
    state.sync = false;
    // If _read pushed data synchronously, then `reading` will be false,
    // and we need to re-evaluate how much data we can return to the user.
    if (!state.reading) n = howMuchToRead(nOrig, state);
  }

  var ret;
  if (n > 0) ret = fromList(n, state);else ret = null;

  if (ret === null) {
    state.needReadable = true;
    n = 0;
  } else {
    state.length -= n;
  }

  if (state.length === 0) {
    // If we have nothing in the buffer, then we want to know
    // as soon as we *do* get something into the buffer.
    if (!state.ended) state.needReadable = true;

    // If we tried to read() past the EOF, then emit end on the next tick.
    if (nOrig !== n && state.ended) endReadable(this);
  }

  if (ret !== null) this.emit('data', ret);

  return ret;
};

function chunkInvalid(state, chunk) {
  var er = null;
  if (!isBuffer(chunk) && typeof chunk !== 'string' && chunk !== null && chunk !== undefined && !state.objectMode) {
    er = new TypeError('Invalid non-string/buffer chunk');
  }
  return er;
}

function onEofChunk(stream, state) {
  if (state.ended) return;
  if (state.decoder) {
    var chunk = state.decoder.end();
    if (chunk && chunk.length) {
      state.buffer.push(chunk);
      state.length += state.objectMode ? 1 : chunk.length;
    }
  }
  state.ended = true;

  // emit 'readable' now to make sure it gets picked up.
  emitReadable(stream);
}

// Don't emit readable right away in sync mode, because this can trigger
// another read() call => stack overflow.  This way, it might trigger
// a nextTick recursion warning, but that's not so bad.
function emitReadable(stream) {
  var state = stream._readableState;
  state.needReadable = false;
  if (!state.emittedReadable) {
    debug('emitReadable', state.flowing);
    state.emittedReadable = true;
    if (state.sync) nextTick(emitReadable_, stream);else emitReadable_(stream);
  }
}

function emitReadable_(stream) {
  debug('emit readable');
  stream.emit('readable');
  flow(stream);
}

// at this point, the user has presumably seen the 'readable' event,
// and called read() to consume some data.  that may have triggered
// in turn another _read(n) call, in which case reading = true if
// it's in progress.
// However, if we're not ended, or reading, and the length < hwm,
// then go ahead and try to read some more preemptively.
function maybeReadMore(stream, state) {
  if (!state.readingMore) {
    state.readingMore = true;
    nextTick(maybeReadMore_, stream, state);
  }
}

function maybeReadMore_(stream, state) {
  var len = state.length;
  while (!state.reading && !state.flowing && !state.ended && state.length < state.highWaterMark) {
    debug('maybeReadMore read 0');
    stream.read(0);
    if (len === state.length)
      // didn't get any data, stop spinning.
      break;else len = state.length;
  }
  state.readingMore = false;
}

// abstract method.  to be overridden in specific implementation classes.
// call cb(er, data) where data is <= n in length.
// for virtual (non-string, non-buffer) streams, "length" is somewhat
// arbitrary, and perhaps not very meaningful.
Readable.prototype._read = function (n) {
  this.emit('error', new Error('not implemented'));
};

Readable.prototype.pipe = function (dest, pipeOpts) {
  var src = this;
  var state = this._readableState;

  switch (state.pipesCount) {
    case 0:
      state.pipes = dest;
      break;
    case 1:
      state.pipes = [state.pipes, dest];
      break;
    default:
      state.pipes.push(dest);
      break;
  }
  state.pipesCount += 1;
  debug('pipe count=%d opts=%j', state.pipesCount, pipeOpts);

  var doEnd = (!pipeOpts || pipeOpts.end !== false);

  var endFn = doEnd ? onend : cleanup;
  if (state.endEmitted) nextTick(endFn);else src.once('end', endFn);

  dest.on('unpipe', onunpipe);
  function onunpipe(readable) {
    debug('onunpipe');
    if (readable === src) {
      cleanup();
    }
  }

  function onend() {
    debug('onend');
    dest.end();
  }

  // when the dest drains, it reduces the awaitDrain counter
  // on the source.  This would be more elegant with a .once()
  // handler in flow(), but adding and removing repeatedly is
  // too slow.
  var ondrain = pipeOnDrain(src);
  dest.on('drain', ondrain);

  var cleanedUp = false;
  function cleanup() {
    debug('cleanup');
    // cleanup event handlers once the pipe is broken
    dest.removeListener('close', onclose);
    dest.removeListener('finish', onfinish);
    dest.removeListener('drain', ondrain);
    dest.removeListener('error', onerror);
    dest.removeListener('unpipe', onunpipe);
    src.removeListener('end', onend);
    src.removeListener('end', cleanup);
    src.removeListener('data', ondata);

    cleanedUp = true;

    // if the reader is waiting for a drain event from this
    // specific writer, then it would cause it to never start
    // flowing again.
    // So, if this is awaiting a drain, then we just call it now.
    // If we don't know, then assume that we are waiting for one.
    if (state.awaitDrain && (!dest._writableState || dest._writableState.needDrain)) ondrain();
  }

  // If the user pushes more data while we're writing to dest then we'll end up
  // in ondata again. However, we only want to increase awaitDrain once because
  // dest will only emit one 'drain' event for the multiple writes.
  // => Introduce a guard on increasing awaitDrain.
  var increasedAwaitDrain = false;
  src.on('data', ondata);
  function ondata(chunk) {
    debug('ondata');
    increasedAwaitDrain = false;
    var ret = dest.write(chunk);
    if (false === ret && !increasedAwaitDrain) {
      // If the user unpiped during `dest.write()`, it is possible
      // to get stuck in a permanently paused state if that write
      // also returned false.
      // => Check whether `dest` is still a piping destination.
      if ((state.pipesCount === 1 && state.pipes === dest || state.pipesCount > 1 && indexOf(state.pipes, dest) !== -1) && !cleanedUp) {
        debug('false write response, pause', src._readableState.awaitDrain);
        src._readableState.awaitDrain++;
        increasedAwaitDrain = true;
      }
      src.pause();
    }
  }

  // if the dest has an error, then stop piping into it.
  // however, don't suppress the throwing behavior for this.
  function onerror(er) {
    debug('onerror', er);
    unpipe();
    dest.removeListener('error', onerror);
    if (listenerCount$1(dest, 'error') === 0) dest.emit('error', er);
  }

  // Make sure our error handler is attached before userland ones.
  prependListener(dest, 'error', onerror);

  // Both close and finish should trigger unpipe, but only once.
  function onclose() {
    dest.removeListener('finish', onfinish);
    unpipe();
  }
  dest.once('close', onclose);
  function onfinish() {
    debug('onfinish');
    dest.removeListener('close', onclose);
    unpipe();
  }
  dest.once('finish', onfinish);

  function unpipe() {
    debug('unpipe');
    src.unpipe(dest);
  }

  // tell the dest that it's being piped to
  dest.emit('pipe', src);

  // start the flow if it hasn't been started already.
  if (!state.flowing) {
    debug('pipe resume');
    src.resume();
  }

  return dest;
};

function pipeOnDrain(src) {
  return function () {
    var state = src._readableState;
    debug('pipeOnDrain', state.awaitDrain);
    if (state.awaitDrain) state.awaitDrain--;
    if (state.awaitDrain === 0 && src.listeners('data').length) {
      state.flowing = true;
      flow(src);
    }
  };
}

Readable.prototype.unpipe = function (dest) {
  var state = this._readableState;

  // if we're not piping anywhere, then do nothing.
  if (state.pipesCount === 0) return this;

  // just one destination.  most common case.
  if (state.pipesCount === 1) {
    // passed in one, but it's not the right one.
    if (dest && dest !== state.pipes) return this;

    if (!dest) dest = state.pipes;

    // got a match.
    state.pipes = null;
    state.pipesCount = 0;
    state.flowing = false;
    if (dest) dest.emit('unpipe', this);
    return this;
  }

  // slow case. multiple pipe destinations.

  if (!dest) {
    // remove all.
    var dests = state.pipes;
    var len = state.pipesCount;
    state.pipes = null;
    state.pipesCount = 0;
    state.flowing = false;

    for (var _i = 0; _i < len; _i++) {
      dests[_i].emit('unpipe', this);
    }return this;
  }

  // try to find the right one.
  var i = indexOf(state.pipes, dest);
  if (i === -1) return this;

  state.pipes.splice(i, 1);
  state.pipesCount -= 1;
  if (state.pipesCount === 1) state.pipes = state.pipes[0];

  dest.emit('unpipe', this);

  return this;
};

// set up data events if they are asked for
// Ensure readable listeners eventually get something
Readable.prototype.on = function (ev, fn) {
  var res = EventEmitter.prototype.on.call(this, ev, fn);

  if (ev === 'data') {
    // Start flowing on next tick if stream isn't explicitly paused
    if (this._readableState.flowing !== false) this.resume();
  } else if (ev === 'readable') {
    var state = this._readableState;
    if (!state.endEmitted && !state.readableListening) {
      state.readableListening = state.needReadable = true;
      state.emittedReadable = false;
      if (!state.reading) {
        nextTick(nReadingNextTick, this);
      } else if (state.length) {
        emitReadable(this, state);
      }
    }
  }

  return res;
};
Readable.prototype.addListener = Readable.prototype.on;

function nReadingNextTick(self) {
  debug('readable nexttick read 0');
  self.read(0);
}

// pause() and resume() are remnants of the legacy readable stream API
// If the user uses them, then switch into old mode.
Readable.prototype.resume = function () {
  var state = this._readableState;
  if (!state.flowing) {
    debug('resume');
    state.flowing = true;
    resume(this, state);
  }
  return this;
};

function resume(stream, state) {
  if (!state.resumeScheduled) {
    state.resumeScheduled = true;
    nextTick(resume_, stream, state);
  }
}

function resume_(stream, state) {
  if (!state.reading) {
    debug('resume read 0');
    stream.read(0);
  }

  state.resumeScheduled = false;
  state.awaitDrain = 0;
  stream.emit('resume');
  flow(stream);
  if (state.flowing && !state.reading) stream.read(0);
}

Readable.prototype.pause = function () {
  debug('call pause flowing=%j', this._readableState.flowing);
  if (false !== this._readableState.flowing) {
    debug('pause');
    this._readableState.flowing = false;
    this.emit('pause');
  }
  return this;
};

function flow(stream) {
  var state = stream._readableState;
  debug('flow', state.flowing);
  while (state.flowing && stream.read() !== null) {}
}

// wrap an old-style stream as the async data source.
// This is *not* part of the readable stream interface.
// It is an ugly unfortunate mess of history.
Readable.prototype.wrap = function (stream) {
  var state = this._readableState;
  var paused = false;

  var self = this;
  stream.on('end', function () {
    debug('wrapped end');
    if (state.decoder && !state.ended) {
      var chunk = state.decoder.end();
      if (chunk && chunk.length) self.push(chunk);
    }

    self.push(null);
  });

  stream.on('data', function (chunk) {
    debug('wrapped data');
    if (state.decoder) chunk = state.decoder.write(chunk);

    // don't skip over falsy values in objectMode
    if (state.objectMode && (chunk === null || chunk === undefined)) return;else if (!state.objectMode && (!chunk || !chunk.length)) return;

    var ret = self.push(chunk);
    if (!ret) {
      paused = true;
      stream.pause();
    }
  });

  // proxy all the other methods.
  // important when wrapping filters and duplexes.
  for (var i in stream) {
    if (this[i] === undefined && typeof stream[i] === 'function') {
      this[i] = function (method) {
        return function () {
          return stream[method].apply(stream, arguments);
        };
      }(i);
    }
  }

  // proxy certain important events.
  var events = ['error', 'close', 'destroy', 'pause', 'resume'];
  forEach(events, function (ev) {
    stream.on(ev, self.emit.bind(self, ev));
  });

  // when we try to consume some more bytes, simply unpause the
  // underlying stream.
  self._read = function (n) {
    debug('wrapped _read', n);
    if (paused) {
      paused = false;
      stream.resume();
    }
  };

  return self;
};

// exposed for testing purposes only.
Readable._fromList = fromList;

// Pluck off n bytes from an array of buffers.
// Length is the combined lengths of all the buffers in the list.
// This function is designed to be inlinable, so please take care when making
// changes to the function body.
function fromList(n, state) {
  // nothing buffered
  if (state.length === 0) return null;

  var ret;
  if (state.objectMode) ret = state.buffer.shift();else if (!n || n >= state.length) {
    // read it all, truncate the list
    if (state.decoder) ret = state.buffer.join('');else if (state.buffer.length === 1) ret = state.buffer.head.data;else ret = state.buffer.concat(state.length);
    state.buffer.clear();
  } else {
    // read part of list
    ret = fromListPartial(n, state.buffer, state.decoder);
  }

  return ret;
}

// Extracts only enough buffered data to satisfy the amount requested.
// This function is designed to be inlinable, so please take care when making
// changes to the function body.
function fromListPartial(n, list, hasStrings) {
  var ret;
  if (n < list.head.data.length) {
    // slice is the same for buffers and strings
    ret = list.head.data.slice(0, n);
    list.head.data = list.head.data.slice(n);
  } else if (n === list.head.data.length) {
    // first chunk is a perfect match
    ret = list.shift();
  } else {
    // result spans more than one buffer
    ret = hasStrings ? copyFromBufferString(n, list) : copyFromBuffer(n, list);
  }
  return ret;
}

// Copies a specified amount of characters from the list of buffered data
// chunks.
// This function is designed to be inlinable, so please take care when making
// changes to the function body.
function copyFromBufferString(n, list) {
  var p = list.head;
  var c = 1;
  var ret = p.data;
  n -= ret.length;
  while (p = p.next) {
    var str = p.data;
    var nb = n > str.length ? str.length : n;
    if (nb === str.length) ret += str;else ret += str.slice(0, n);
    n -= nb;
    if (n === 0) {
      if (nb === str.length) {
        ++c;
        if (p.next) list.head = p.next;else list.head = list.tail = null;
      } else {
        list.head = p;
        p.data = str.slice(nb);
      }
      break;
    }
    ++c;
  }
  list.length -= c;
  return ret;
}

// Copies a specified amount of bytes from the list of buffered data chunks.
// This function is designed to be inlinable, so please take care when making
// changes to the function body.
function copyFromBuffer(n, list) {
  var ret = Buffer.allocUnsafe(n);
  var p = list.head;
  var c = 1;
  p.data.copy(ret);
  n -= p.data.length;
  while (p = p.next) {
    var buf = p.data;
    var nb = n > buf.length ? buf.length : n;
    buf.copy(ret, ret.length - n, 0, nb);
    n -= nb;
    if (n === 0) {
      if (nb === buf.length) {
        ++c;
        if (p.next) list.head = p.next;else list.head = list.tail = null;
      } else {
        list.head = p;
        p.data = buf.slice(nb);
      }
      break;
    }
    ++c;
  }
  list.length -= c;
  return ret;
}

function endReadable(stream) {
  var state = stream._readableState;

  // If we get here before consuming all the bytes, then that is a
  // bug in node.  Should never happen.
  if (state.length > 0) throw new Error('"endReadable()" called on non-empty stream');

  if (!state.endEmitted) {
    state.ended = true;
    nextTick(endReadableNT, state, stream);
  }
}

function endReadableNT(state, stream) {
  // Check that we didn't get one last unshift.
  if (!state.endEmitted && state.length === 0) {
    state.endEmitted = true;
    stream.readable = false;
    stream.emit('end');
  }
}

function forEach(xs, f) {
  for (var i = 0, l = xs.length; i < l; i++) {
    f(xs[i], i);
  }
}

function indexOf(xs, x) {
  for (var i = 0, l = xs.length; i < l; i++) {
    if (xs[i] === x) return i;
  }
  return -1;
}

// A bit simpler than readable streams.
Writable.WritableState = WritableState;
util_21(Writable, EventEmitter);

function nop() {}

function WriteReq(chunk, encoding, cb) {
  this.chunk = chunk;
  this.encoding = encoding;
  this.callback = cb;
  this.next = null;
}

function WritableState(options, stream) {
  Object.defineProperty(this, 'buffer', {
    get: util_2(function () {
      return this.getBuffer();
    }, '_writableState.buffer is deprecated. Use _writableState.getBuffer ' + 'instead.')
  });
  options = options || {};

  // object stream flag to indicate whether or not this stream
  // contains buffers or objects.
  this.objectMode = !!options.objectMode;

  if (stream instanceof Duplex) this.objectMode = this.objectMode || !!options.writableObjectMode;

  // the point at which write() starts returning false
  // Note: 0 is a valid value, means that we always return false if
  // the entire buffer is not flushed immediately on write()
  var hwm = options.highWaterMark;
  var defaultHwm = this.objectMode ? 16 : 16 * 1024;
  this.highWaterMark = hwm || hwm === 0 ? hwm : defaultHwm;

  // cast to ints.
  this.highWaterMark = ~ ~this.highWaterMark;

  this.needDrain = false;
  // at the start of calling end()
  this.ending = false;
  // when end() has been called, and returned
  this.ended = false;
  // when 'finish' is emitted
  this.finished = false;

  // should we decode strings into buffers before passing to _write?
  // this is here so that some node-core streams can optimize string
  // handling at a lower level.
  var noDecode = options.decodeStrings === false;
  this.decodeStrings = !noDecode;

  // Crypto is kind of old and crusty.  Historically, its default string
  // encoding is 'binary' so we have to make this configurable.
  // Everything else in the universe uses 'utf8', though.
  this.defaultEncoding = options.defaultEncoding || 'utf8';

  // not an actual buffer we keep track of, but a measurement
  // of how much we're waiting to get pushed to some underlying
  // socket or file.
  this.length = 0;

  // a flag to see when we're in the middle of a write.
  this.writing = false;

  // when true all writes will be buffered until .uncork() call
  this.corked = 0;

  // a flag to be able to tell if the onwrite cb is called immediately,
  // or on a later tick.  We set this to true at first, because any
  // actions that shouldn't happen until "later" should generally also
  // not happen before the first write call.
  this.sync = true;

  // a flag to know if we're processing previously buffered items, which
  // may call the _write() callback in the same tick, so that we don't
  // end up in an overlapped onwrite situation.
  this.bufferProcessing = false;

  // the callback that's passed to _write(chunk,cb)
  this.onwrite = function (er) {
    onwrite(stream, er);
  };

  // the callback that the user supplies to write(chunk,encoding,cb)
  this.writecb = null;

  // the amount that is being written when _write is called.
  this.writelen = 0;

  this.bufferedRequest = null;
  this.lastBufferedRequest = null;

  // number of pending user-supplied write callbacks
  // this must be 0 before 'finish' can be emitted
  this.pendingcb = 0;

  // emit prefinish if the only thing we're waiting for is _write cbs
  // This is relevant for synchronous Transform streams
  this.prefinished = false;

  // True if the error was already emitted and should not be thrown again
  this.errorEmitted = false;

  // count buffered requests
  this.bufferedRequestCount = 0;

  // allocate the first CorkedRequest, there is always
  // one allocated and free to use, and we maintain at most two
  this.corkedRequestsFree = new CorkedRequest(this);
}

WritableState.prototype.getBuffer = function writableStateGetBuffer() {
  var current = this.bufferedRequest;
  var out = [];
  while (current) {
    out.push(current);
    current = current.next;
  }
  return out;
};
function Writable(options) {

  // Writable ctor is applied to Duplexes, though they're not
  // instanceof Writable, they're instanceof Readable.
  if (!(this instanceof Writable) && !(this instanceof Duplex)) return new Writable(options);

  this._writableState = new WritableState(options, this);

  // legacy.
  this.writable = true;

  if (options) {
    if (typeof options.write === 'function') this._write = options.write;

    if (typeof options.writev === 'function') this._writev = options.writev;
  }

  EventEmitter.call(this);
}

// Otherwise people can pipe Writable streams, which is just wrong.
Writable.prototype.pipe = function () {
  this.emit('error', new Error('Cannot pipe, not readable'));
};

function writeAfterEnd(stream, cb) {
  var er = new Error('write after end');
  // TODO: defer error events consistently everywhere, not just the cb
  stream.emit('error', er);
  nextTick(cb, er);
}

// If we get something that is not a buffer, string, null, or undefined,
// and we're not in objectMode, then that's an error.
// Otherwise stream chunks are all considered to be of length=1, and the
// watermarks determine how many objects to keep in the buffer, rather than
// how many bytes or characters.
function validChunk(stream, state, chunk, cb) {
  var valid = true;
  var er = false;
  // Always throw error if a null is written
  // if we are not in object mode then throw
  // if it is not a buffer, string, or undefined.
  if (chunk === null) {
    er = new TypeError('May not write null values to stream');
  } else if (!Buffer.isBuffer(chunk) && typeof chunk !== 'string' && chunk !== undefined && !state.objectMode) {
    er = new TypeError('Invalid non-string/buffer chunk');
  }
  if (er) {
    stream.emit('error', er);
    nextTick(cb, er);
    valid = false;
  }
  return valid;
}

Writable.prototype.write = function (chunk, encoding, cb) {
  var state = this._writableState;
  var ret = false;

  if (typeof encoding === 'function') {
    cb = encoding;
    encoding = null;
  }

  if (Buffer.isBuffer(chunk)) encoding = 'buffer';else if (!encoding) encoding = state.defaultEncoding;

  if (typeof cb !== 'function') cb = nop;

  if (state.ended) writeAfterEnd(this, cb);else if (validChunk(this, state, chunk, cb)) {
    state.pendingcb++;
    ret = writeOrBuffer(this, state, chunk, encoding, cb);
  }

  return ret;
};

Writable.prototype.cork = function () {
  var state = this._writableState;

  state.corked++;
};

Writable.prototype.uncork = function () {
  var state = this._writableState;

  if (state.corked) {
    state.corked--;

    if (!state.writing && !state.corked && !state.finished && !state.bufferProcessing && state.bufferedRequest) clearBuffer(this, state);
  }
};

Writable.prototype.setDefaultEncoding = function setDefaultEncoding(encoding) {
  // node::ParseEncoding() requires lower case.
  if (typeof encoding === 'string') encoding = encoding.toLowerCase();
  if (!(['hex', 'utf8', 'utf-8', 'ascii', 'binary', 'base64', 'ucs2', 'ucs-2', 'utf16le', 'utf-16le', 'raw'].indexOf((encoding + '').toLowerCase()) > -1)) throw new TypeError('Unknown encoding: ' + encoding);
  this._writableState.defaultEncoding = encoding;
  return this;
};

function decodeChunk(state, chunk, encoding) {
  if (!state.objectMode && state.decodeStrings !== false && typeof chunk === 'string') {
    chunk = Buffer.from(chunk, encoding);
  }
  return chunk;
}

// if we're already writing something, then just put this
// in the queue, and wait our turn.  Otherwise, call _write
// If we return false, then we need a drain event, so set that flag.
function writeOrBuffer(stream, state, chunk, encoding, cb) {
  chunk = decodeChunk(state, chunk, encoding);

  if (Buffer.isBuffer(chunk)) encoding = 'buffer';
  var len = state.objectMode ? 1 : chunk.length;

  state.length += len;

  var ret = state.length < state.highWaterMark;
  // we must ensure that previous needDrain will not be reset to false.
  if (!ret) state.needDrain = true;

  if (state.writing || state.corked) {
    var last = state.lastBufferedRequest;
    state.lastBufferedRequest = new WriteReq(chunk, encoding, cb);
    if (last) {
      last.next = state.lastBufferedRequest;
    } else {
      state.bufferedRequest = state.lastBufferedRequest;
    }
    state.bufferedRequestCount += 1;
  } else {
    doWrite(stream, state, false, len, chunk, encoding, cb);
  }

  return ret;
}

function doWrite(stream, state, writev, len, chunk, encoding, cb) {
  state.writelen = len;
  state.writecb = cb;
  state.writing = true;
  state.sync = true;
  if (writev) stream._writev(chunk, state.onwrite);else stream._write(chunk, encoding, state.onwrite);
  state.sync = false;
}

function onwriteError(stream, state, sync, er, cb) {
  --state.pendingcb;
  if (sync) nextTick(cb, er);else cb(er);

  stream._writableState.errorEmitted = true;
  stream.emit('error', er);
}

function onwriteStateUpdate(state) {
  state.writing = false;
  state.writecb = null;
  state.length -= state.writelen;
  state.writelen = 0;
}

function onwrite(stream, er) {
  var state = stream._writableState;
  var sync = state.sync;
  var cb = state.writecb;

  onwriteStateUpdate(state);

  if (er) onwriteError(stream, state, sync, er, cb);else {
    // Check if we're actually ready to finish, but don't emit yet
    var finished = needFinish(state);

    if (!finished && !state.corked && !state.bufferProcessing && state.bufferedRequest) {
      clearBuffer(stream, state);
    }

    if (sync) {
      /*<replacement>*/
        nextTick(afterWrite, stream, state, finished, cb);
      /*</replacement>*/
    } else {
        afterWrite(stream, state, finished, cb);
      }
  }
}

function afterWrite(stream, state, finished, cb) {
  if (!finished) onwriteDrain(stream, state);
  state.pendingcb--;
  cb();
  finishMaybe(stream, state);
}

// Must force callback to be called on nextTick, so that we don't
// emit 'drain' before the write() consumer gets the 'false' return
// value, and has a chance to attach a 'drain' listener.
function onwriteDrain(stream, state) {
  if (state.length === 0 && state.needDrain) {
    state.needDrain = false;
    stream.emit('drain');
  }
}

// if there's something in the buffer waiting, then process it
function clearBuffer(stream, state) {
  state.bufferProcessing = true;
  var entry = state.bufferedRequest;

  if (stream._writev && entry && entry.next) {
    // Fast case, write everything using _writev()
    var l = state.bufferedRequestCount;
    var buffer = new Array(l);
    var holder = state.corkedRequestsFree;
    holder.entry = entry;

    var count = 0;
    while (entry) {
      buffer[count] = entry;
      entry = entry.next;
      count += 1;
    }

    doWrite(stream, state, true, state.length, buffer, '', holder.finish);

    // doWrite is almost always async, defer these to save a bit of time
    // as the hot path ends with doWrite
    state.pendingcb++;
    state.lastBufferedRequest = null;
    if (holder.next) {
      state.corkedRequestsFree = holder.next;
      holder.next = null;
    } else {
      state.corkedRequestsFree = new CorkedRequest(state);
    }
  } else {
    // Slow case, write chunks one-by-one
    while (entry) {
      var chunk = entry.chunk;
      var encoding = entry.encoding;
      var cb = entry.callback;
      var len = state.objectMode ? 1 : chunk.length;

      doWrite(stream, state, false, len, chunk, encoding, cb);
      entry = entry.next;
      // if we didn't call the onwrite immediately, then
      // it means that we need to wait until it does.
      // also, that means that the chunk and cb are currently
      // being processed, so move the buffer counter past them.
      if (state.writing) {
        break;
      }
    }

    if (entry === null) state.lastBufferedRequest = null;
  }

  state.bufferedRequestCount = 0;
  state.bufferedRequest = entry;
  state.bufferProcessing = false;
}

Writable.prototype._write = function (chunk, encoding, cb) {
  cb(new Error('not implemented'));
};

Writable.prototype._writev = null;

Writable.prototype.end = function (chunk, encoding, cb) {
  var state = this._writableState;

  if (typeof chunk === 'function') {
    cb = chunk;
    chunk = null;
    encoding = null;
  } else if (typeof encoding === 'function') {
    cb = encoding;
    encoding = null;
  }

  if (chunk !== null && chunk !== undefined) this.write(chunk, encoding);

  // .end() fully uncorks
  if (state.corked) {
    state.corked = 1;
    this.uncork();
  }

  // ignore unnecessary end() calls.
  if (!state.ending && !state.finished) endWritable(this, state, cb);
};

function needFinish(state) {
  return state.ending && state.length === 0 && state.bufferedRequest === null && !state.finished && !state.writing;
}

function prefinish(stream, state) {
  if (!state.prefinished) {
    state.prefinished = true;
    stream.emit('prefinish');
  }
}

function finishMaybe(stream, state) {
  var need = needFinish(state);
  if (need) {
    if (state.pendingcb === 0) {
      prefinish(stream, state);
      state.finished = true;
      stream.emit('finish');
    } else {
      prefinish(stream, state);
    }
  }
  return need;
}

function endWritable(stream, state, cb) {
  state.ending = true;
  finishMaybe(stream, state);
  if (cb) {
    if (state.finished) nextTick(cb);else stream.once('finish', cb);
  }
  state.ended = true;
  stream.writable = false;
}

// It seems a linked list but it is not
// there will be only 2 of these for each stream
function CorkedRequest(state) {
  var _this = this;

  this.next = null;
  this.entry = null;

  this.finish = function (err) {
    var entry = _this.entry;
    _this.entry = null;
    while (entry) {
      var cb = entry.callback;
      state.pendingcb--;
      cb(err);
      entry = entry.next;
    }
    if (state.corkedRequestsFree) {
      state.corkedRequestsFree.next = _this;
    } else {
      state.corkedRequestsFree = _this;
    }
  };
}

util_21(Duplex, Readable);

var keys = Object.keys(Writable.prototype);
for (var v = 0; v < keys.length; v++) {
  var method = keys[v];
  if (!Duplex.prototype[method]) Duplex.prototype[method] = Writable.prototype[method];
}
function Duplex(options) {
  if (!(this instanceof Duplex)) return new Duplex(options);

  Readable.call(this, options);
  Writable.call(this, options);

  if (options && options.readable === false) this.readable = false;

  if (options && options.writable === false) this.writable = false;

  this.allowHalfOpen = true;
  if (options && options.allowHalfOpen === false) this.allowHalfOpen = false;

  this.once('end', onend);
}

// the no-half-open enforcer
function onend() {
  // if we allow half-open state, or if the writable side ended,
  // then we're ok.
  if (this.allowHalfOpen || this._writableState.ended) return;

  // no more data can be written.
  // But allow more writes to happen in this tick.
  nextTick(onEndNT, this);
}

function onEndNT(self) {
  self.end();
}

// a transform stream is a readable/writable stream where you do
util_21(Transform, Duplex);

function TransformState(stream) {
  this.afterTransform = function (er, data) {
    return afterTransform(stream, er, data);
  };

  this.needTransform = false;
  this.transforming = false;
  this.writecb = null;
  this.writechunk = null;
  this.writeencoding = null;
}

function afterTransform(stream, er, data) {
  var ts = stream._transformState;
  ts.transforming = false;

  var cb = ts.writecb;

  if (!cb) return stream.emit('error', new Error('no writecb in Transform class'));

  ts.writechunk = null;
  ts.writecb = null;

  if (data !== null && data !== undefined) stream.push(data);

  cb(er);

  var rs = stream._readableState;
  rs.reading = false;
  if (rs.needReadable || rs.length < rs.highWaterMark) {
    stream._read(rs.highWaterMark);
  }
}
function Transform(options) {
  if (!(this instanceof Transform)) return new Transform(options);

  Duplex.call(this, options);

  this._transformState = new TransformState(this);

  // when the writable side finishes, then flush out anything remaining.
  var stream = this;

  // start out asking for a readable event once data is transformed.
  this._readableState.needReadable = true;

  // we have implemented the _read method, and done the other things
  // that Readable wants before the first _read call, so unset the
  // sync guard flag.
  this._readableState.sync = false;

  if (options) {
    if (typeof options.transform === 'function') this._transform = options.transform;

    if (typeof options.flush === 'function') this._flush = options.flush;
  }

  this.once('prefinish', function () {
    if (typeof this._flush === 'function') this._flush(function (er) {
      done(stream, er);
    });else done(stream);
  });
}

Transform.prototype.push = function (chunk, encoding) {
  this._transformState.needTransform = false;
  return Duplex.prototype.push.call(this, chunk, encoding);
};

// This is the part where you do stuff!
// override this function in implementation classes.
// 'chunk' is an input chunk.
//
// Call `push(newChunk)` to pass along transformed output
// to the readable side.  You may call 'push' zero or more times.
//
// Call `cb(err)` when you are done with this chunk.  If you pass
// an error, then that'll put the hurt on the whole operation.  If you
// never call cb(), then you'll never get another chunk.
Transform.prototype._transform = function (chunk, encoding, cb) {
  throw new Error('Not implemented');
};

Transform.prototype._write = function (chunk, encoding, cb) {
  var ts = this._transformState;
  ts.writecb = cb;
  ts.writechunk = chunk;
  ts.writeencoding = encoding;
  if (!ts.transforming) {
    var rs = this._readableState;
    if (ts.needTransform || rs.needReadable || rs.length < rs.highWaterMark) this._read(rs.highWaterMark);
  }
};

// Doesn't matter what the args are here.
// _transform does all the work.
// That we got here means that the readable side wants more data.
Transform.prototype._read = function (n) {
  var ts = this._transformState;

  if (ts.writechunk !== null && ts.writecb && !ts.transforming) {
    ts.transforming = true;
    this._transform(ts.writechunk, ts.writeencoding, ts.afterTransform);
  } else {
    // mark that we need a transform, so that any data that comes in
    // will get processed, now that we've asked for it.
    ts.needTransform = true;
  }
};

function done(stream, er) {
  if (er) return stream.emit('error', er);

  // if there's nothing in the write buffer, then that means
  // that nothing more will ever be provided
  var ws = stream._writableState;
  var ts = stream._transformState;

  if (ws.length) throw new Error('Calling transform done when ws.length != 0');

  if (ts.transforming) throw new Error('Calling transform done when still transforming');

  return stream.push(null);
}

util_21(PassThrough, Transform);
function PassThrough(options) {
  if (!(this instanceof PassThrough)) return new PassThrough(options);

  Transform.call(this, options);
}

PassThrough.prototype._transform = function (chunk, encoding, cb) {
  cb(null, chunk);
};

util_21(Stream, EventEmitter);
Stream.Readable = Readable;
Stream.Writable = Writable;
Stream.Duplex = Duplex;
Stream.Transform = Transform;
Stream.PassThrough = PassThrough;

// Backwards-compat with node 0.4.x
Stream.Stream = Stream;

// old-style streams.  Note that the pipe method (the only relevant
// part of this class) is overridden in the Readable class.

function Stream() {
  EventEmitter.call(this);
}

Stream.prototype.pipe = function(dest, options) {
  var source = this;

  function ondata(chunk) {
    if (dest.writable) {
      if (false === dest.write(chunk) && source.pause) {
        source.pause();
      }
    }
  }

  source.on('data', ondata);

  function ondrain() {
    if (source.readable && source.resume) {
      source.resume();
    }
  }

  dest.on('drain', ondrain);

  // If the 'end' option is not supplied, dest.end() will be called when
  // source gets the 'end' or 'close' events.  Only dest.end() once.
  if (!dest._isStdio && (!options || options.end !== false)) {
    source.on('end', onend);
    source.on('close', onclose);
  }

  var didOnEnd = false;
  function onend() {
    if (didOnEnd) return;
    didOnEnd = true;

    dest.end();
  }


  function onclose() {
    if (didOnEnd) return;
    didOnEnd = true;

    if (typeof dest.destroy === 'function') dest.destroy();
  }

  // don't leave dangling pipes when there are errors.
  function onerror(er) {
    cleanup();
    if (EventEmitter.listenerCount(this, 'error') === 0) {
      throw er; // Unhandled stream error in pipe.
    }
  }

  source.on('error', onerror);
  dest.on('error', onerror);

  // remove all the event listeners that were added.
  function cleanup() {
    source.removeListener('data', ondata);
    dest.removeListener('drain', ondrain);

    source.removeListener('end', onend);
    source.removeListener('close', onclose);

    source.removeListener('error', onerror);
    dest.removeListener('error', onerror);

    source.removeListener('end', cleanup);
    source.removeListener('close', cleanup);

    dest.removeListener('close', cleanup);
  }

  source.on('end', cleanup);
  source.on('close', cleanup);

  dest.on('close', cleanup);

  dest.emit('pipe', source);

  // Allow for unix-like usage: A.pipe(B).pipe(C)
  return dest;
};

/*
 * This file is used by module bundlers (browserify/webpack/etc) when
 * including a stream implementation. We use "readable-stream" to get a
 * consistent behavior between nodejs versions but bundlers often have a shim
 * for "stream". Using this shim greatly improve the compatibility and greatly
 * reduce the final size of the bundle (only one stream implementation, not
 * two).
 */
var readableStreamBrowser = Stream;

var support = __chunk_1.createCommonjsModule(function (module, exports) {

exports.base64 = true;
exports.array = true;
exports.string = true;
exports.arraybuffer = typeof ArrayBuffer !== "undefined" && typeof Uint8Array !== "undefined";
exports.nodebuffer = typeof Buffer !== "undefined";
// contains true if JSZip can read/generate Uint8Array, false otherwise.
exports.uint8array = typeof Uint8Array !== "undefined";

if (typeof ArrayBuffer === "undefined") {
    exports.blob = false;
}
else {
    var buffer = new ArrayBuffer(0);
    try {
        exports.blob = new Blob([buffer], {
            type: "application/zip"
        }).size === 0;
    }
    catch (e) {
        try {
            var Builder = self.BlobBuilder || self.WebKitBlobBuilder || self.MozBlobBuilder || self.MSBlobBuilder;
            var builder = new Builder();
            builder.append(buffer);
            exports.blob = builder.getBlob('application/zip').size === 0;
        }
        catch (e) {
            exports.blob = false;
        }
    }
}

try {
    exports.nodestream = !!readableStreamBrowser.Readable;
} catch(e) {
    exports.nodestream = false;
}
});
var support_1 = support.base64;
var support_2 = support.array;
var support_3 = support.string;
var support_4 = support.arraybuffer;
var support_5 = support.nodebuffer;
var support_6 = support.uint8array;
var support_7 = support.blob;
var support_8 = support.nodestream;

// private property
var _keyStr = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";


// public method for encoding
var encode = function(input) {
    var output = [];
    var chr1, chr2, chr3, enc1, enc2, enc3, enc4;
    var i = 0, len = input.length, remainingBytes = len;

    var isArray = utils.getTypeOf(input) !== "string";
    while (i < input.length) {
        remainingBytes = len - i;

        if (!isArray) {
            chr1 = input.charCodeAt(i++);
            chr2 = i < len ? input.charCodeAt(i++) : 0;
            chr3 = i < len ? input.charCodeAt(i++) : 0;
        } else {
            chr1 = input[i++];
            chr2 = i < len ? input[i++] : 0;
            chr3 = i < len ? input[i++] : 0;
        }

        enc1 = chr1 >> 2;
        enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
        enc3 = remainingBytes > 1 ? (((chr2 & 15) << 2) | (chr3 >> 6)) : 64;
        enc4 = remainingBytes > 2 ? (chr3 & 63) : 64;

        output.push(_keyStr.charAt(enc1) + _keyStr.charAt(enc2) + _keyStr.charAt(enc3) + _keyStr.charAt(enc4));

    }

    return output.join("");
};

// public method for decoding
var decode = function(input) {
    var chr1, chr2, chr3;
    var enc1, enc2, enc3, enc4;
    var i = 0, resultIndex = 0;

    var dataUrlPrefix = "data:";

    if (input.substr(0, dataUrlPrefix.length) === dataUrlPrefix) {
        // This is a common error: people give a data url
        // (data:image/png;base64,iVBOR...) with a {base64: true} and
        // wonders why things don't work.
        // We can detect that the string input looks like a data url but we
        // *can't* be sure it is one: removing everything up to the comma would
        // be too dangerous.
        throw new Error("Invalid base64 input, it looks like a data url.");
    }

    input = input.replace(/[^A-Za-z0-9\+\/\=]/g, "");

    var totalLength = input.length * 3 / 4;
    if(input.charAt(input.length - 1) === _keyStr.charAt(64)) {
        totalLength--;
    }
    if(input.charAt(input.length - 2) === _keyStr.charAt(64)) {
        totalLength--;
    }
    if (totalLength % 1 !== 0) {
        // totalLength is not an integer, the length does not match a valid
        // base64 content. That can happen if:
        // - the input is not a base64 content
        // - the input is *almost* a base64 content, with a extra chars at the
        //   beginning or at the end
        // - the input uses a base64 variant (base64url for example)
        throw new Error("Invalid base64 input, bad content length.");
    }
    var output;
    if (support.uint8array) {
        output = new Uint8Array(totalLength|0);
    } else {
        output = new Array(totalLength|0);
    }

    while (i < input.length) {

        enc1 = _keyStr.indexOf(input.charAt(i++));
        enc2 = _keyStr.indexOf(input.charAt(i++));
        enc3 = _keyStr.indexOf(input.charAt(i++));
        enc4 = _keyStr.indexOf(input.charAt(i++));

        chr1 = (enc1 << 2) | (enc2 >> 4);
        chr2 = ((enc2 & 15) << 4) | (enc3 >> 2);
        chr3 = ((enc3 & 3) << 6) | enc4;

        output[resultIndex++] = chr1;

        if (enc3 !== 64) {
            output[resultIndex++] = chr2;
        }
        if (enc4 !== 64) {
            output[resultIndex++] = chr3;
        }

    }

    return output;
};

var base64 = {
	encode: encode,
	decode: decode
};

var nodejsUtils = {
    /**
     * True if this is running in Nodejs, will be undefined in a browser.
     * In a browser, browserify won't include this file and the whole module
     * will be resolved an empty object.
     */
    isNode : typeof Buffer !== "undefined",
    /**
     * Create a new nodejs Buffer from an existing content.
     * @param {Object} data the data to pass to the constructor.
     * @param {String} encoding the encoding to use.
     * @return {Buffer} a new Buffer.
     */
    newBufferFrom: function(data, encoding) {
        // XXX We can't use `Buffer.from` which comes from `Uint8Array.from`
        // in nodejs v4 (< v.4.5). It's not the expected implementation (and
        // has a different signature).
        // see https://github.com/nodejs/node/issues/8053
        // A condition on nodejs' version won't solve the issue as we don't
        // control the Buffer polyfills that may or may not be used.
        return new Buffer(data, encoding);
    },
    /**
     * Create a new nodejs Buffer with the specified size.
     * @param {Integer} size the size of the buffer.
     * @return {Buffer} a new Buffer.
     */
    allocBuffer: function (size) {
        if (Buffer.alloc) {
            return Buffer.alloc(size);
        } else {
            return new Buffer(size);
        }
    },
    /**
     * Find out if an object is a Buffer.
     * @param {Object} b the object to test.
     * @return {Boolean} true if the object is a Buffer, false otherwise.
     */
    isBuffer : function(b){
        return isBuffer(b);
    },

    isStream : function (obj) {
        return obj &&
            typeof obj.on === "function" &&
            typeof obj.pause === "function" &&
            typeof obj.resume === "function";
    }
};

var _global = __chunk_1.createCommonjsModule(function (module) {
// https://github.com/zloirock/core-js/issues/86#issuecomment-115759028
var global = module.exports = typeof window != 'undefined' && window.Math == Math
  ? window : typeof self != 'undefined' && self.Math == Math ? self : Function('return this')();
if(typeof __g == 'number')__g = global; // eslint-disable-line no-undef
});

var _core = __chunk_1.createCommonjsModule(function (module) {
var core = module.exports = {version: '2.3.0'};
if(typeof __e == 'number')__e = core; // eslint-disable-line no-undef
});
var _core_1 = _core.version;

var _aFunction = function(it){
  if(typeof it != 'function')throw TypeError(it + ' is not a function!');
  return it;
};

// optional / simple context binding

var _ctx = function(fn, that, length){
  _aFunction(fn);
  if(that === undefined)return fn;
  switch(length){
    case 1: return function(a){
      return fn.call(that, a);
    };
    case 2: return function(a, b){
      return fn.call(that, a, b);
    };
    case 3: return function(a, b, c){
      return fn.call(that, a, b, c);
    };
  }
  return function(/* ...args */){
    return fn.apply(that, arguments);
  };
};

var _isObject = function(it){
  return typeof it === 'object' ? it !== null : typeof it === 'function';
};

var _anObject = function(it){
  if(!_isObject(it))throw TypeError(it + ' is not an object!');
  return it;
};

var _fails = function(exec){
  try {
    return !!exec();
  } catch(e){
    return true;
  }
};

// Thank's IE8 for his funny defineProperty
var _descriptors = !_fails(function(){
  return Object.defineProperty({}, 'a', {get: function(){ return 7; }}).a != 7;
});

var document$1 = _global.document
  // in old IE typeof document.createElement is 'object'
  , is = _isObject(document$1) && _isObject(document$1.createElement);
var _domCreate = function(it){
  return is ? document$1.createElement(it) : {};
};

var _ie8DomDefine = !_descriptors && !_fails(function(){
  return Object.defineProperty(_domCreate('div'), 'a', {get: function(){ return 7; }}).a != 7;
});

// 7.1.1 ToPrimitive(input [, PreferredType])

// instead of the ES6 spec version, we didn't implement @@toPrimitive case
// and the second argument - flag - preferred type is a string
var _toPrimitive = function(it, S){
  if(!_isObject(it))return it;
  var fn, val;
  if(S && typeof (fn = it.toString) == 'function' && !_isObject(val = fn.call(it)))return val;
  if(typeof (fn = it.valueOf) == 'function' && !_isObject(val = fn.call(it)))return val;
  if(!S && typeof (fn = it.toString) == 'function' && !_isObject(val = fn.call(it)))return val;
  throw TypeError("Can't convert object to primitive value");
};

var dP$1             = Object.defineProperty;

var f$2 = _descriptors ? Object.defineProperty : function defineProperty(O, P, Attributes){
  _anObject(O);
  P = _toPrimitive(P, true);
  _anObject(Attributes);
  if(_ie8DomDefine)try {
    return dP$1(O, P, Attributes);
  } catch(e){ /* empty */ }
  if('get' in Attributes || 'set' in Attributes)throw TypeError('Accessors not supported!');
  if('value' in Attributes)O[P] = Attributes.value;
  return O;
};

var _objectDp = {
	f: f$2
};

var _propertyDesc = function(bitmap, value){
  return {
    enumerable  : !(bitmap & 1),
    configurable: !(bitmap & 2),
    writable    : !(bitmap & 4),
    value       : value
  };
};

var _hide = _descriptors ? function(object, key, value){
  return _objectDp.f(object, key, _propertyDesc(1, value));
} : function(object, key, value){
  object[key] = value;
  return object;
};

var PROTOTYPE$1 = 'prototype';

var $export = function(type, name, source){
  var IS_FORCED = type & $export.F
    , IS_GLOBAL = type & $export.G
    , IS_STATIC = type & $export.S
    , IS_PROTO  = type & $export.P
    , IS_BIND   = type & $export.B
    , IS_WRAP   = type & $export.W
    , exports   = IS_GLOBAL ? _core : _core[name] || (_core[name] = {})
    , expProto  = exports[PROTOTYPE$1]
    , target    = IS_GLOBAL ? _global : IS_STATIC ? _global[name] : (_global[name] || {})[PROTOTYPE$1]
    , key, own, out;
  if(IS_GLOBAL)source = name;
  for(key in source){
    // contains in native
    own = !IS_FORCED && target && target[key] !== undefined;
    if(own && key in exports)continue;
    // export native or passed
    out = own ? target[key] : source[key];
    // prevent global pollution for namespaces
    exports[key] = IS_GLOBAL && typeof target[key] != 'function' ? source[key]
    // bind timers to global for call from export context
    : IS_BIND && own ? _ctx(out, _global)
    // wrap global constructors for prevent change them in library
    : IS_WRAP && target[key] == out ? (function(C){
      var F = function(a, b, c){
        if(this instanceof C){
          switch(arguments.length){
            case 0: return new C;
            case 1: return new C(a);
            case 2: return new C(a, b);
          } return new C(a, b, c);
        } return C.apply(this, arguments);
      };
      F[PROTOTYPE$1] = C[PROTOTYPE$1];
      return F;
    // make static versions for prototype methods
    })(out) : IS_PROTO && typeof out == 'function' ? _ctx(Function.call, out) : out;
    // export proto methods to core.%CONSTRUCTOR%.methods.%NAME%
    if(IS_PROTO){
      (exports.virtual || (exports.virtual = {}))[key] = out;
      // export proto methods to core.%CONSTRUCTOR%.prototype.%NAME%
      if(type & $export.R && expProto && !expProto[key])_hide(expProto, key, out);
    }
  }
};
// type bitmap
$export.F = 1;   // forced
$export.G = 2;   // global
$export.S = 4;   // static
$export.P = 8;   // proto
$export.B = 16;  // bind
$export.W = 32;  // wrap
$export.U = 64;  // safe
$export.R = 128; // real proto method for `library` 
var _export = $export;

// fast apply, http://jsperf.lnkit.com/fast-apply/5
var _invoke = function(fn, args, that){
  var un = that === undefined;
  switch(args.length){
    case 0: return un ? fn()
                      : fn.call(that);
    case 1: return un ? fn(args[0])
                      : fn.call(that, args[0]);
    case 2: return un ? fn(args[0], args[1])
                      : fn.call(that, args[0], args[1]);
    case 3: return un ? fn(args[0], args[1], args[2])
                      : fn.call(that, args[0], args[1], args[2]);
    case 4: return un ? fn(args[0], args[1], args[2], args[3])
                      : fn.call(that, args[0], args[1], args[2], args[3]);
  } return              fn.apply(that, args);
};

var _html = _global.document && document.documentElement;

var toString$2 = {}.toString;

var _cof = function(it){
  return toString$2.call(it).slice(8, -1);
};

var process$1            = _global.process
  , setTask            = _global.setImmediate
  , clearTask          = _global.clearImmediate
  , MessageChannel     = _global.MessageChannel
  , counter            = 0
  , queue$1              = {}
  , ONREADYSTATECHANGE = 'onreadystatechange'
  , defer, channel, port;
var run = function(){
  var id = +this;
  if(queue$1.hasOwnProperty(id)){
    var fn = queue$1[id];
    delete queue$1[id];
    fn();
  }
};
var listener = function(event){
  run.call(event.data);
};
// Node.js 0.9+ & IE10+ has setImmediate, otherwise:
if(!setTask || !clearTask){
  setTask = function setImmediate(fn){
    var args = [], i = 1;
    while(arguments.length > i)args.push(arguments[i++]);
    queue$1[++counter] = function(){
      _invoke(typeof fn == 'function' ? fn : Function(fn), args);
    };
    defer(counter);
    return counter;
  };
  clearTask = function clearImmediate(id){
    delete queue$1[id];
  };
  // Node.js 0.8-
  if(_cof(process$1) == 'process'){
    defer = function(id){
      process$1.nextTick(_ctx(run, id, 1));
    };
  // Browsers with MessageChannel, includes WebWorkers
  } else if(MessageChannel){
    channel = new MessageChannel;
    port    = channel.port2;
    channel.port1.onmessage = listener;
    defer = _ctx(port.postMessage, port, 1);
  // Browsers with postMessage, skip WebWorkers
  // IE8 has postMessage, but it's sync & typeof its postMessage is 'object'
  } else if(_global.addEventListener && typeof postMessage == 'function' && !_global.importScripts){
    defer = function(id){
      _global.postMessage(id + '', '*');
    };
    _global.addEventListener('message', listener, false);
  // IE8-
  } else if(ONREADYSTATECHANGE in _domCreate('script')){
    defer = function(id){
      _html.appendChild(_domCreate('script'))[ONREADYSTATECHANGE] = function(){
        _html.removeChild(this);
        run.call(id);
      };
    };
  // Rest old browsers
  } else {
    defer = function(id){
      setTimeout(_ctx(run, id, 1), 0);
    };
  }
}
var _task = {
  set:   setTask,
  clear: clearTask
};

_export(_export.G + _export.B, {
  setImmediate:   _task.set,
  clearImmediate: _task.clear
});

var setImmediate = _core.setImmediate;

var Mutation = global$1.MutationObserver || global$1.WebKitMutationObserver;

var scheduleDrain;

{
  if (Mutation) {
    var called = 0;
    var observer = new Mutation(nextTick$1);
    var element = global$1.document.createTextNode('');
    observer.observe(element, {
      characterData: true
    });
    scheduleDrain = function () {
      element.data = (called = ++called % 2);
    };
  } else if (!global$1.setImmediate && typeof global$1.MessageChannel !== 'undefined') {
    var channel$1 = new global$1.MessageChannel();
    channel$1.port1.onmessage = nextTick$1;
    scheduleDrain = function () {
      channel$1.port2.postMessage(0);
    };
  } else if ('document' in global$1 && 'onreadystatechange' in global$1.document.createElement('script')) {
    scheduleDrain = function () {

      // Create a <script> element; its readystatechange event will be fired asynchronously once it is inserted
      // into the document. Do so, thus queuing up the task. Remember to clean up once it's been called.
      var scriptEl = global$1.document.createElement('script');
      scriptEl.onreadystatechange = function () {
        nextTick$1();

        scriptEl.onreadystatechange = null;
        scriptEl.parentNode.removeChild(scriptEl);
        scriptEl = null;
      };
      global$1.document.documentElement.appendChild(scriptEl);
    };
  } else {
    scheduleDrain = function () {
      setTimeout(nextTick$1, 0);
    };
  }
}

var draining$1;
var queue$2 = [];
//named nextTick for less confusing stack traces
function nextTick$1() {
  draining$1 = true;
  var i, oldQueue;
  var len = queue$2.length;
  while (len) {
    oldQueue = queue$2;
    queue$2 = [];
    i = -1;
    while (++i < len) {
      oldQueue[i]();
    }
    len = queue$2.length;
  }
  draining$1 = false;
}

var browser$1 = immediate;
function immediate(task) {
  if (queue$2.push(task) === 1 && !draining$1) {
    scheduleDrain();
  }
}

/* istanbul ignore next */
function INTERNAL() {}

var handlers = {};

var REJECTED = ['REJECTED'];
var FULFILLED = ['FULFILLED'];
var PENDING = ['PENDING'];

var browser$2 = Promise$1;

function Promise$1(resolver) {
  if (typeof resolver !== 'function') {
    throw new TypeError('resolver must be a function');
  }
  this.state = PENDING;
  this.queue = [];
  this.outcome = void 0;
  if (resolver !== INTERNAL) {
    safelyResolveThenable(this, resolver);
  }
}

Promise$1.prototype["catch"] = function (onRejected) {
  return this.then(null, onRejected);
};
Promise$1.prototype.then = function (onFulfilled, onRejected) {
  if (typeof onFulfilled !== 'function' && this.state === FULFILLED ||
    typeof onRejected !== 'function' && this.state === REJECTED) {
    return this;
  }
  var promise = new this.constructor(INTERNAL);
  if (this.state !== PENDING) {
    var resolver = this.state === FULFILLED ? onFulfilled : onRejected;
    unwrap(promise, resolver, this.outcome);
  } else {
    this.queue.push(new QueueItem(promise, onFulfilled, onRejected));
  }

  return promise;
};
function QueueItem(promise, onFulfilled, onRejected) {
  this.promise = promise;
  if (typeof onFulfilled === 'function') {
    this.onFulfilled = onFulfilled;
    this.callFulfilled = this.otherCallFulfilled;
  }
  if (typeof onRejected === 'function') {
    this.onRejected = onRejected;
    this.callRejected = this.otherCallRejected;
  }
}
QueueItem.prototype.callFulfilled = function (value) {
  handlers.resolve(this.promise, value);
};
QueueItem.prototype.otherCallFulfilled = function (value) {
  unwrap(this.promise, this.onFulfilled, value);
};
QueueItem.prototype.callRejected = function (value) {
  handlers.reject(this.promise, value);
};
QueueItem.prototype.otherCallRejected = function (value) {
  unwrap(this.promise, this.onRejected, value);
};

function unwrap(promise, func, value) {
  browser$1(function () {
    var returnValue;
    try {
      returnValue = func(value);
    } catch (e) {
      return handlers.reject(promise, e);
    }
    if (returnValue === promise) {
      handlers.reject(promise, new TypeError('Cannot resolve promise with itself'));
    } else {
      handlers.resolve(promise, returnValue);
    }
  });
}

handlers.resolve = function (self, value) {
  var result = tryCatch(getThen, value);
  if (result.status === 'error') {
    return handlers.reject(self, result.value);
  }
  var thenable = result.value;

  if (thenable) {
    safelyResolveThenable(self, thenable);
  } else {
    self.state = FULFILLED;
    self.outcome = value;
    var i = -1;
    var len = self.queue.length;
    while (++i < len) {
      self.queue[i].callFulfilled(value);
    }
  }
  return self;
};
handlers.reject = function (self, error) {
  self.state = REJECTED;
  self.outcome = error;
  var i = -1;
  var len = self.queue.length;
  while (++i < len) {
    self.queue[i].callRejected(error);
  }
  return self;
};

function getThen(obj) {
  // Make sure we only access the accessor once as required by the spec
  var then = obj && obj.then;
  if (obj && (typeof obj === 'object' || typeof obj === 'function') && typeof then === 'function') {
    return function appyThen() {
      then.apply(obj, arguments);
    };
  }
}

function safelyResolveThenable(self, thenable) {
  // Either fulfill, reject or reject with error
  var called = false;
  function onError(value) {
    if (called) {
      return;
    }
    called = true;
    handlers.reject(self, value);
  }

  function onSuccess(value) {
    if (called) {
      return;
    }
    called = true;
    handlers.resolve(self, value);
  }

  function tryToUnwrap() {
    thenable(onSuccess, onError);
  }

  var result = tryCatch(tryToUnwrap);
  if (result.status === 'error') {
    onError(result.value);
  }
}

function tryCatch(func, value) {
  var out = {};
  try {
    out.value = func(value);
    out.status = 'success';
  } catch (e) {
    out.status = 'error';
    out.value = e;
  }
  return out;
}

Promise$1.resolve = resolve;
function resolve(value) {
  if (value instanceof this) {
    return value;
  }
  return handlers.resolve(new this(INTERNAL), value);
}

Promise$1.reject = reject;
function reject(reason) {
  var promise = new this(INTERNAL);
  return handlers.reject(promise, reason);
}

Promise$1.all = all;
function all(iterable) {
  var self = this;
  if (Object.prototype.toString.call(iterable) !== '[object Array]') {
    return this.reject(new TypeError('must be an array'));
  }

  var len = iterable.length;
  var called = false;
  if (!len) {
    return this.resolve([]);
  }

  var values = new Array(len);
  var resolved = 0;
  var i = -1;
  var promise = new this(INTERNAL);

  while (++i < len) {
    allResolver(iterable[i], i);
  }
  return promise;
  function allResolver(value, i) {
    self.resolve(value).then(resolveFromAll, function (error) {
      if (!called) {
        called = true;
        handlers.reject(promise, error);
      }
    });
    function resolveFromAll(outValue) {
      values[i] = outValue;
      if (++resolved === len && !called) {
        called = true;
        handlers.resolve(promise, values);
      }
    }
  }
}

Promise$1.race = race;
function race(iterable) {
  var self = this;
  if (Object.prototype.toString.call(iterable) !== '[object Array]') {
    return this.reject(new TypeError('must be an array'));
  }

  var len = iterable.length;
  var called = false;
  if (!len) {
    return this.resolve([]);
  }

  var i = -1;
  var promise = new this(INTERNAL);

  while (++i < len) {
    resolver(iterable[i]);
  }
  return promise;
  function resolver(value) {
    self.resolve(value).then(function (response) {
      if (!called) {
        called = true;
        handlers.resolve(promise, response);
      }
    }, function (error) {
      if (!called) {
        called = true;
        handlers.reject(promise, error);
      }
    });
  }
}

// load the global object first:
// - it should be better integrated in the system (unhandledRejection in node)
// - the environment may have a custom Promise implementation (see zone.js)
var ES6Promise = null;
if (typeof Promise !== "undefined") {
    ES6Promise = Promise;
} else {
    ES6Promise = browser$2;
}

/**
 * Let the user use/change some implementations.
 */
var external = {
    Promise: ES6Promise
};

var utils = __chunk_1.createCommonjsModule(function (module, exports) {








/**
 * Convert a string that pass as a "binary string": it should represent a byte
 * array but may have > 255 char codes. Be sure to take only the first byte
 * and returns the byte array.
 * @param {String} str the string to transform.
 * @return {Array|Uint8Array} the string in a binary format.
 */
function string2binary(str) {
    var result = null;
    if (support.uint8array) {
      result = new Uint8Array(str.length);
    } else {
      result = new Array(str.length);
    }
    return stringToArrayLike(str, result);
}

/**
 * Create a new blob with the given content and the given type.
 * @param {String|ArrayBuffer} part the content to put in the blob. DO NOT use
 * an Uint8Array because the stock browser of android 4 won't accept it (it
 * will be silently converted to a string, "[object Uint8Array]").
 *
 * Use only ONE part to build the blob to avoid a memory leak in IE11 / Edge:
 * when a large amount of Array is used to create the Blob, the amount of
 * memory consumed is nearly 100 times the original data amount.
 *
 * @param {String} type the mime type of the blob.
 * @return {Blob} the created blob.
 */
exports.newBlob = function(part, type) {
    exports.checkSupport("blob");

    try {
        // Blob constructor
        return new Blob([part], {
            type: type
        });
    }
    catch (e) {

        try {
            // deprecated, browser only, old way
            var Builder = self.BlobBuilder || self.WebKitBlobBuilder || self.MozBlobBuilder || self.MSBlobBuilder;
            var builder = new Builder();
            builder.append(part);
            return builder.getBlob(type);
        }
        catch (e) {

            // well, fuck ?!
            throw new Error("Bug : can't construct the Blob.");
        }
    }


};
/**
 * The identity function.
 * @param {Object} input the input.
 * @return {Object} the same input.
 */
function identity(input) {
    return input;
}

/**
 * Fill in an array with a string.
 * @param {String} str the string to use.
 * @param {Array|ArrayBuffer|Uint8Array|Buffer} array the array to fill in (will be mutated).
 * @return {Array|ArrayBuffer|Uint8Array|Buffer} the updated array.
 */
function stringToArrayLike(str, array) {
    for (var i = 0; i < str.length; ++i) {
        array[i] = str.charCodeAt(i) & 0xFF;
    }
    return array;
}

/**
 * An helper for the function arrayLikeToString.
 * This contains static informations and functions that
 * can be optimized by the browser JIT compiler.
 */
var arrayToStringHelper = {
    /**
     * Transform an array of int into a string, chunk by chunk.
     * See the performances notes on arrayLikeToString.
     * @param {Array|ArrayBuffer|Uint8Array|Buffer} array the array to transform.
     * @param {String} type the type of the array.
     * @param {Integer} chunk the chunk size.
     * @return {String} the resulting string.
     * @throws Error if the chunk is too big for the stack.
     */
    stringifyByChunk: function(array, type, chunk) {
        var result = [], k = 0, len = array.length;
        // shortcut
        if (len <= chunk) {
            return String.fromCharCode.apply(null, array);
        }
        while (k < len) {
            if (type === "array" || type === "nodebuffer") {
                result.push(String.fromCharCode.apply(null, array.slice(k, Math.min(k + chunk, len))));
            }
            else {
                result.push(String.fromCharCode.apply(null, array.subarray(k, Math.min(k + chunk, len))));
            }
            k += chunk;
        }
        return result.join("");
    },
    /**
     * Call String.fromCharCode on every item in the array.
     * This is the naive implementation, which generate A LOT of intermediate string.
     * This should be used when everything else fail.
     * @param {Array|ArrayBuffer|Uint8Array|Buffer} array the array to transform.
     * @return {String} the result.
     */
    stringifyByChar: function(array){
        var resultStr = "";
        for(var i = 0; i < array.length; i++) {
            resultStr += String.fromCharCode(array[i]);
        }
        return resultStr;
    },
    applyCanBeUsed : {
        /**
         * true if the browser accepts to use String.fromCharCode on Uint8Array
         */
        uint8array : (function () {
            try {
                return support.uint8array && String.fromCharCode.apply(null, new Uint8Array(1)).length === 1;
            } catch (e) {
                return false;
            }
        })(),
        /**
         * true if the browser accepts to use String.fromCharCode on nodejs Buffer.
         */
        nodebuffer : (function () {
            try {
                return support.nodebuffer && String.fromCharCode.apply(null, nodejsUtils.allocBuffer(1)).length === 1;
            } catch (e) {
                return false;
            }
        })()
    }
};

/**
 * Transform an array-like object to a string.
 * @param {Array|ArrayBuffer|Uint8Array|Buffer} array the array to transform.
 * @return {String} the result.
 */
function arrayLikeToString(array) {
    // Performances notes :
    // --------------------
    // String.fromCharCode.apply(null, array) is the fastest, see
    // see http://jsperf.com/converting-a-uint8array-to-a-string/2
    // but the stack is limited (and we can get huge arrays !).
    //
    // result += String.fromCharCode(array[i]); generate too many strings !
    //
    // This code is inspired by http://jsperf.com/arraybuffer-to-string-apply-performance/2
    // TODO : we now have workers that split the work. Do we still need that ?
    var chunk = 65536,
        type = exports.getTypeOf(array),
        canUseApply = true;
    if (type === "uint8array") {
        canUseApply = arrayToStringHelper.applyCanBeUsed.uint8array;
    } else if (type === "nodebuffer") {
        canUseApply = arrayToStringHelper.applyCanBeUsed.nodebuffer;
    }

    if (canUseApply) {
        while (chunk > 1) {
            try {
                return arrayToStringHelper.stringifyByChunk(array, type, chunk);
            } catch (e) {
                chunk = Math.floor(chunk / 2);
            }
        }
    }

    // no apply or chunk error : slow and painful algorithm
    // default browser on android 4.*
    return arrayToStringHelper.stringifyByChar(array);
}

exports.applyFromCharCode = arrayLikeToString;


/**
 * Copy the data from an array-like to an other array-like.
 * @param {Array|ArrayBuffer|Uint8Array|Buffer} arrayFrom the origin array.
 * @param {Array|ArrayBuffer|Uint8Array|Buffer} arrayTo the destination array which will be mutated.
 * @return {Array|ArrayBuffer|Uint8Array|Buffer} the updated destination array.
 */
function arrayLikeToArrayLike(arrayFrom, arrayTo) {
    for (var i = 0; i < arrayFrom.length; i++) {
        arrayTo[i] = arrayFrom[i];
    }
    return arrayTo;
}

// a matrix containing functions to transform everything into everything.
var transform = {};

// string to ?
transform["string"] = {
    "string": identity,
    "array": function(input) {
        return stringToArrayLike(input, new Array(input.length));
    },
    "arraybuffer": function(input) {
        return transform["string"]["uint8array"](input).buffer;
    },
    "uint8array": function(input) {
        return stringToArrayLike(input, new Uint8Array(input.length));
    },
    "nodebuffer": function(input) {
        return stringToArrayLike(input, nodejsUtils.allocBuffer(input.length));
    }
};

// array to ?
transform["array"] = {
    "string": arrayLikeToString,
    "array": identity,
    "arraybuffer": function(input) {
        return (new Uint8Array(input)).buffer;
    },
    "uint8array": function(input) {
        return new Uint8Array(input);
    },
    "nodebuffer": function(input) {
        return nodejsUtils.newBufferFrom(input);
    }
};

// arraybuffer to ?
transform["arraybuffer"] = {
    "string": function(input) {
        return arrayLikeToString(new Uint8Array(input));
    },
    "array": function(input) {
        return arrayLikeToArrayLike(new Uint8Array(input), new Array(input.byteLength));
    },
    "arraybuffer": identity,
    "uint8array": function(input) {
        return new Uint8Array(input);
    },
    "nodebuffer": function(input) {
        return nodejsUtils.newBufferFrom(new Uint8Array(input));
    }
};

// uint8array to ?
transform["uint8array"] = {
    "string": arrayLikeToString,
    "array": function(input) {
        return arrayLikeToArrayLike(input, new Array(input.length));
    },
    "arraybuffer": function(input) {
        return input.buffer;
    },
    "uint8array": identity,
    "nodebuffer": function(input) {
        return nodejsUtils.newBufferFrom(input);
    }
};

// nodebuffer to ?
transform["nodebuffer"] = {
    "string": arrayLikeToString,
    "array": function(input) {
        return arrayLikeToArrayLike(input, new Array(input.length));
    },
    "arraybuffer": function(input) {
        return transform["nodebuffer"]["uint8array"](input).buffer;
    },
    "uint8array": function(input) {
        return arrayLikeToArrayLike(input, new Uint8Array(input.length));
    },
    "nodebuffer": identity
};

/**
 * Transform an input into any type.
 * The supported output type are : string, array, uint8array, arraybuffer, nodebuffer.
 * If no output type is specified, the unmodified input will be returned.
 * @param {String} outputType the output type.
 * @param {String|Array|ArrayBuffer|Uint8Array|Buffer} input the input to convert.
 * @throws {Error} an Error if the browser doesn't support the requested output type.
 */
exports.transformTo = function(outputType, input) {
    if (!input) {
        // undefined, null, etc
        // an empty string won't harm.
        input = "";
    }
    if (!outputType) {
        return input;
    }
    exports.checkSupport(outputType);
    var inputType = exports.getTypeOf(input);
    var result = transform[inputType][outputType](input);
    return result;
};

/**
 * Return the type of the input.
 * The type will be in a format valid for JSZip.utils.transformTo : string, array, uint8array, arraybuffer.
 * @param {Object} input the input to identify.
 * @return {String} the (lowercase) type of the input.
 */
exports.getTypeOf = function(input) {
    if (typeof input === "string") {
        return "string";
    }
    if (Object.prototype.toString.call(input) === "[object Array]") {
        return "array";
    }
    if (support.nodebuffer && nodejsUtils.isBuffer(input)) {
        return "nodebuffer";
    }
    if (support.uint8array && input instanceof Uint8Array) {
        return "uint8array";
    }
    if (support.arraybuffer && input instanceof ArrayBuffer) {
        return "arraybuffer";
    }
};

/**
 * Throw an exception if the type is not supported.
 * @param {String} type the type to check.
 * @throws {Error} an Error if the browser doesn't support the requested type.
 */
exports.checkSupport = function(type) {
    var supported = support[type.toLowerCase()];
    if (!supported) {
        throw new Error(type + " is not supported by this platform");
    }
};

exports.MAX_VALUE_16BITS = 65535;
exports.MAX_VALUE_32BITS = -1; // well, "\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF" is parsed as -1

/**
 * Prettify a string read as binary.
 * @param {string} str the string to prettify.
 * @return {string} a pretty string.
 */
exports.pretty = function(str) {
    var res = '',
        code, i;
    for (i = 0; i < (str || "").length; i++) {
        code = str.charCodeAt(i);
        res += '\\x' + (code < 16 ? "0" : "") + code.toString(16).toUpperCase();
    }
    return res;
};

/**
 * Defer the call of a function.
 * @param {Function} callback the function to call asynchronously.
 * @param {Array} args the arguments to give to the callback.
 */
exports.delay = function(callback, args, self) {
    setImmediate(function () {
        callback.apply(self || null, args || []);
    });
};

/**
 * Extends a prototype with an other, without calling a constructor with
 * side effects. Inspired by nodejs' `utils.inherits`
 * @param {Function} ctor the constructor to augment
 * @param {Function} superCtor the parent constructor to use
 */
exports.inherits = function (ctor, superCtor) {
    var Obj = function() {};
    Obj.prototype = superCtor.prototype;
    ctor.prototype = new Obj();
};

/**
 * Merge the objects passed as parameters into a new one.
 * @private
 * @param {...Object} var_args All objects to merge.
 * @return {Object} a new object with the data of the others.
 */
exports.extend = function() {
    var result = {}, i, attr;
    for (i = 0; i < arguments.length; i++) { // arguments is not enumerable in some browsers
        for (attr in arguments[i]) {
            if (arguments[i].hasOwnProperty(attr) && typeof result[attr] === "undefined") {
                result[attr] = arguments[i][attr];
            }
        }
    }
    return result;
};

/**
 * Transform arbitrary content into a Promise.
 * @param {String} name a name for the content being processed.
 * @param {Object} inputData the content to process.
 * @param {Boolean} isBinary true if the content is not an unicode string
 * @param {Boolean} isOptimizedBinaryString true if the string content only has one byte per character.
 * @param {Boolean} isBase64 true if the string content is encoded with base64.
 * @return {Promise} a promise in a format usable by JSZip.
 */
exports.prepareContent = function(name, inputData, isBinary, isOptimizedBinaryString, isBase64) {

    // if inputData is already a promise, this flatten it.
    var promise = external.Promise.resolve(inputData).then(function(data) {
        
        
        var isBlob = support.blob && (data instanceof Blob || ['[object File]', '[object Blob]'].indexOf(Object.prototype.toString.call(data)) !== -1);

        if (isBlob && typeof FileReader !== "undefined") {
            return new external.Promise(function (resolve, reject) {
                var reader = new FileReader();

                reader.onload = function(e) {
                    resolve(e.target.result);
                };
                reader.onerror = function(e) {
                    reject(e.target.error);
                };
                reader.readAsArrayBuffer(data);
            });
        } else {
            return data;
        }
    });

    return promise.then(function(data) {
        var dataType = exports.getTypeOf(data);

        if (!dataType) {
            return external.Promise.reject(
                new Error("Can't read the data of '" + name + "'. Is it " +
                          "in a supported JavaScript type (String, Blob, ArrayBuffer, etc) ?")
            );
        }
        // special case : it's way easier to work with Uint8Array than with ArrayBuffer
        if (dataType === "arraybuffer") {
            data = exports.transformTo("uint8array", data);
        } else if (dataType === "string") {
            if (isBase64) {
                data = base64.decode(data);
            }
            else if (isBinary) {
                // optimizedBinaryString === true means that the file has already been filtered with a 0xFF mask
                if (isOptimizedBinaryString !== true) {
                    // this is a string, not in a base64 format.
                    // Be sure that this is a correct "binary string"
                    data = string2binary(data);
                }
            }
        }
        return data;
    });
};
});
var utils_1 = utils.newBlob;
var utils_2 = utils.applyFromCharCode;
var utils_3 = utils.transformTo;
var utils_4 = utils.getTypeOf;
var utils_5 = utils.checkSupport;
var utils_6 = utils.MAX_VALUE_16BITS;
var utils_7 = utils.MAX_VALUE_32BITS;
var utils_8 = utils.pretty;
var utils_9 = utils.delay;
var utils_10 = utils.inherits;
var utils_11 = utils.extend;
var utils_12 = utils.prepareContent;

/**
 * A worker that does nothing but passing chunks to the next one. This is like
 * a nodejs stream but with some differences. On the good side :
 * - it works on IE 6-9 without any issue / polyfill
 * - it weights less than the full dependencies bundled with browserify
 * - it forwards errors (no need to declare an error handler EVERYWHERE)
 *
 * A chunk is an object with 2 attributes : `meta` and `data`. The former is an
 * object containing anything (`percent` for example), see each worker for more
 * details. The latter is the real data (String, Uint8Array, etc).
 *
 * @constructor
 * @param {String} name the name of the stream (mainly used for debugging purposes)
 */
function GenericWorker(name) {
    // the name of the worker
    this.name = name || "default";
    // an object containing metadata about the workers chain
    this.streamInfo = {};
    // an error which happened when the worker was paused
    this.generatedError = null;
    // an object containing metadata to be merged by this worker into the general metadata
    this.extraStreamInfo = {};
    // true if the stream is paused (and should not do anything), false otherwise
    this.isPaused = true;
    // true if the stream is finished (and should not do anything), false otherwise
    this.isFinished = false;
    // true if the stream is locked to prevent further structure updates (pipe), false otherwise
    this.isLocked = false;
    // the event listeners
    this._listeners = {
        'data':[],
        'end':[],
        'error':[]
    };
    // the previous worker, if any
    this.previous = null;
}

GenericWorker.prototype = {
    /**
     * Push a chunk to the next workers.
     * @param {Object} chunk the chunk to push
     */
    push : function (chunk) {
        this.emit("data", chunk);
    },
    /**
     * End the stream.
     * @return {Boolean} true if this call ended the worker, false otherwise.
     */
    end : function () {
        if (this.isFinished) {
            return false;
        }

        this.flush();
        try {
            this.emit("end");
            this.cleanUp();
            this.isFinished = true;
        } catch (e) {
            this.emit("error", e);
        }
        return true;
    },
    /**
     * End the stream with an error.
     * @param {Error} e the error which caused the premature end.
     * @return {Boolean} true if this call ended the worker with an error, false otherwise.
     */
    error : function (e) {
        if (this.isFinished) {
            return false;
        }

        if(this.isPaused) {
            this.generatedError = e;
        } else {
            this.isFinished = true;

            this.emit("error", e);

            // in the workers chain exploded in the middle of the chain,
            // the error event will go downward but we also need to notify
            // workers upward that there has been an error.
            if(this.previous) {
                this.previous.error(e);
            }

            this.cleanUp();
        }
        return true;
    },
    /**
     * Add a callback on an event.
     * @param {String} name the name of the event (data, end, error)
     * @param {Function} listener the function to call when the event is triggered
     * @return {GenericWorker} the current object for chainability
     */
    on : function (name, listener) {
        this._listeners[name].push(listener);
        return this;
    },
    /**
     * Clean any references when a worker is ending.
     */
    cleanUp : function () {
        this.streamInfo = this.generatedError = this.extraStreamInfo = null;
        this._listeners = [];
    },
    /**
     * Trigger an event. This will call registered callback with the provided arg.
     * @param {String} name the name of the event (data, end, error)
     * @param {Object} arg the argument to call the callback with.
     */
    emit : function (name, arg) {
        if (this._listeners[name]) {
            for(var i = 0; i < this._listeners[name].length; i++) {
                this._listeners[name][i].call(this, arg);
            }
        }
    },
    /**
     * Chain a worker with an other.
     * @param {Worker} next the worker receiving events from the current one.
     * @return {worker} the next worker for chainability
     */
    pipe : function (next) {
        return next.registerPrevious(this);
    },
    /**
     * Same as `pipe` in the other direction.
     * Using an API with `pipe(next)` is very easy.
     * Implementing the API with the point of view of the next one registering
     * a source is easier, see the ZipFileWorker.
     * @param {Worker} previous the previous worker, sending events to this one
     * @return {Worker} the current worker for chainability
     */
    registerPrevious : function (previous) {
        if (this.isLocked) {
            throw new Error("The stream '" + this + "' has already been used.");
        }

        // sharing the streamInfo...
        this.streamInfo = previous.streamInfo;
        // ... and adding our own bits
        this.mergeStreamInfo();
        this.previous =  previous;
        var self = this;
        previous.on('data', function (chunk) {
            self.processChunk(chunk);
        });
        previous.on('end', function () {
            self.end();
        });
        previous.on('error', function (e) {
            self.error(e);
        });
        return this;
    },
    /**
     * Pause the stream so it doesn't send events anymore.
     * @return {Boolean} true if this call paused the worker, false otherwise.
     */
    pause : function () {
        if(this.isPaused || this.isFinished) {
            return false;
        }
        this.isPaused = true;

        if(this.previous) {
            this.previous.pause();
        }
        return true;
    },
    /**
     * Resume a paused stream.
     * @return {Boolean} true if this call resumed the worker, false otherwise.
     */
    resume : function () {
        if(!this.isPaused || this.isFinished) {
            return false;
        }
        this.isPaused = false;

        // if true, the worker tried to resume but failed
        var withError = false;
        if(this.generatedError) {
            this.error(this.generatedError);
            withError = true;
        }
        if(this.previous) {
            this.previous.resume();
        }

        return !withError;
    },
    /**
     * Flush any remaining bytes as the stream is ending.
     */
    flush : function () {},
    /**
     * Process a chunk. This is usually the method overridden.
     * @param {Object} chunk the chunk to process.
     */
    processChunk : function(chunk) {
        this.push(chunk);
    },
    /**
     * Add a key/value to be added in the workers chain streamInfo once activated.
     * @param {String} key the key to use
     * @param {Object} value the associated value
     * @return {Worker} the current worker for chainability
     */
    withStreamInfo : function (key, value) {
        this.extraStreamInfo[key] = value;
        this.mergeStreamInfo();
        return this;
    },
    /**
     * Merge this worker's streamInfo into the chain's streamInfo.
     */
    mergeStreamInfo : function () {
        for(var key in this.extraStreamInfo) {
            if (!this.extraStreamInfo.hasOwnProperty(key)) {
                continue;
            }
            this.streamInfo[key] = this.extraStreamInfo[key];
        }
    },

    /**
     * Lock the stream to prevent further updates on the workers chain.
     * After calling this method, all calls to pipe will fail.
     */
    lock: function () {
        if (this.isLocked) {
            throw new Error("The stream '" + this + "' has already been used.");
        }
        this.isLocked = true;
        if (this.previous) {
            this.previous.lock();
        }
    },

    /**
     *
     * Pretty print the workers chain.
     */
    toString : function () {
        var me = "Worker " + this.name;
        if (this.previous) {
            return this.previous + " -> " + me;
        } else {
            return me;
        }
    }
};

var GenericWorker_1 = GenericWorker;

var utf8 = __chunk_1.createCommonjsModule(function (module, exports) {






/**
 * The following functions come from pako, from pako/lib/utils/strings
 * released under the MIT license, see pako https://github.com/nodeca/pako/
 */

// Table with utf8 lengths (calculated by first byte of sequence)
// Note, that 5 & 6-byte values and some 4-byte values can not be represented in JS,
// because max possible codepoint is 0x10ffff
var _utf8len = new Array(256);
for (var i=0; i<256; i++) {
  _utf8len[i] = (i >= 252 ? 6 : i >= 248 ? 5 : i >= 240 ? 4 : i >= 224 ? 3 : i >= 192 ? 2 : 1);
}
_utf8len[254]=_utf8len[254]=1; // Invalid sequence start

// convert string to array (typed, when possible)
var string2buf = function (str) {
    var buf, c, c2, m_pos, i, str_len = str.length, buf_len = 0;

    // count binary size
    for (m_pos = 0; m_pos < str_len; m_pos++) {
        c = str.charCodeAt(m_pos);
        if ((c & 0xfc00) === 0xd800 && (m_pos+1 < str_len)) {
            c2 = str.charCodeAt(m_pos+1);
            if ((c2 & 0xfc00) === 0xdc00) {
                c = 0x10000 + ((c - 0xd800) << 10) + (c2 - 0xdc00);
                m_pos++;
            }
        }
        buf_len += c < 0x80 ? 1 : c < 0x800 ? 2 : c < 0x10000 ? 3 : 4;
    }

    // allocate buffer
    if (support.uint8array) {
        buf = new Uint8Array(buf_len);
    } else {
        buf = new Array(buf_len);
    }

    // convert
    for (i=0, m_pos = 0; i < buf_len; m_pos++) {
        c = str.charCodeAt(m_pos);
        if ((c & 0xfc00) === 0xd800 && (m_pos+1 < str_len)) {
            c2 = str.charCodeAt(m_pos+1);
            if ((c2 & 0xfc00) === 0xdc00) {
                c = 0x10000 + ((c - 0xd800) << 10) + (c2 - 0xdc00);
                m_pos++;
            }
        }
        if (c < 0x80) {
            /* one byte */
            buf[i++] = c;
        } else if (c < 0x800) {
            /* two bytes */
            buf[i++] = 0xC0 | (c >>> 6);
            buf[i++] = 0x80 | (c & 0x3f);
        } else if (c < 0x10000) {
            /* three bytes */
            buf[i++] = 0xE0 | (c >>> 12);
            buf[i++] = 0x80 | (c >>> 6 & 0x3f);
            buf[i++] = 0x80 | (c & 0x3f);
        } else {
            /* four bytes */
            buf[i++] = 0xf0 | (c >>> 18);
            buf[i++] = 0x80 | (c >>> 12 & 0x3f);
            buf[i++] = 0x80 | (c >>> 6 & 0x3f);
            buf[i++] = 0x80 | (c & 0x3f);
        }
    }

    return buf;
};

// Calculate max possible position in utf8 buffer,
// that will not break sequence. If that's not possible
// - (very small limits) return max size as is.
//
// buf[] - utf8 bytes array
// max   - length limit (mandatory);
var utf8border = function(buf, max) {
    var pos;

    max = max || buf.length;
    if (max > buf.length) { max = buf.length; }

    // go back from last position, until start of sequence found
    pos = max-1;
    while (pos >= 0 && (buf[pos] & 0xC0) === 0x80) { pos--; }

    // Fuckup - very small and broken sequence,
    // return max, because we should return something anyway.
    if (pos < 0) { return max; }

    // If we came to start of buffer - that means vuffer is too small,
    // return max too.
    if (pos === 0) { return max; }

    return (pos + _utf8len[buf[pos]] > max) ? pos : max;
};

// convert array to string
var buf2string = function (buf) {
    var i, out, c, c_len;
    var len = buf.length;

    // Reserve max possible length (2 words per char)
    // NB: by unknown reasons, Array is significantly faster for
    //     String.fromCharCode.apply than Uint16Array.
    var utf16buf = new Array(len*2);

    for (out=0, i=0; i<len;) {
        c = buf[i++];
        // quick process ascii
        if (c < 0x80) { utf16buf[out++] = c; continue; }

        c_len = _utf8len[c];
        // skip 5 & 6 byte codes
        if (c_len > 4) { utf16buf[out++] = 0xfffd; i += c_len-1; continue; }

        // apply mask on first byte
        c &= c_len === 2 ? 0x1f : c_len === 3 ? 0x0f : 0x07;
        // join the rest
        while (c_len > 1 && i < len) {
            c = (c << 6) | (buf[i++] & 0x3f);
            c_len--;
        }

        // terminated by end of string?
        if (c_len > 1) { utf16buf[out++] = 0xfffd; continue; }

        if (c < 0x10000) {
            utf16buf[out++] = c;
        } else {
            c -= 0x10000;
            utf16buf[out++] = 0xd800 | ((c >> 10) & 0x3ff);
            utf16buf[out++] = 0xdc00 | (c & 0x3ff);
        }
    }

    // shrinkBuf(utf16buf, out)
    if (utf16buf.length !== out) {
        if(utf16buf.subarray) {
            utf16buf = utf16buf.subarray(0, out);
        } else {
            utf16buf.length = out;
        }
    }

    // return String.fromCharCode.apply(null, utf16buf);
    return utils.applyFromCharCode(utf16buf);
};


// That's all for the pako functions.


/**
 * Transform a javascript string into an array (typed if possible) of bytes,
 * UTF-8 encoded.
 * @param {String} str the string to encode
 * @return {Array|Uint8Array|Buffer} the UTF-8 encoded string.
 */
exports.utf8encode = function utf8encode(str) {
    if (support.nodebuffer) {
        return nodejsUtils.newBufferFrom(str, "utf-8");
    }

    return string2buf(str);
};


/**
 * Transform a bytes array (or a representation) representing an UTF-8 encoded
 * string into a javascript string.
 * @param {Array|Uint8Array|Buffer} buf the data de decode
 * @return {String} the decoded string.
 */
exports.utf8decode = function utf8decode(buf) {
    if (support.nodebuffer) {
        return utils.transformTo("nodebuffer", buf).toString("utf-8");
    }

    buf = utils.transformTo(support.uint8array ? "uint8array" : "array", buf);

    return buf2string(buf);
};

/**
 * A worker to decode utf8 encoded binary chunks into string chunks.
 * @constructor
 */
function Utf8DecodeWorker() {
    GenericWorker_1.call(this, "utf-8 decode");
    // the last bytes if a chunk didn't end with a complete codepoint.
    this.leftOver = null;
}
utils.inherits(Utf8DecodeWorker, GenericWorker_1);

/**
 * @see GenericWorker.processChunk
 */
Utf8DecodeWorker.prototype.processChunk = function (chunk) {

    var data = utils.transformTo(support.uint8array ? "uint8array" : "array", chunk.data);

    // 1st step, re-use what's left of the previous chunk
    if (this.leftOver && this.leftOver.length) {
        if(support.uint8array) {
            var previousData = data;
            data = new Uint8Array(previousData.length + this.leftOver.length);
            data.set(this.leftOver, 0);
            data.set(previousData, this.leftOver.length);
        } else {
            data = this.leftOver.concat(data);
        }
        this.leftOver = null;
    }

    var nextBoundary = utf8border(data);
    var usableData = data;
    if (nextBoundary !== data.length) {
        if (support.uint8array) {
            usableData = data.subarray(0, nextBoundary);
            this.leftOver = data.subarray(nextBoundary, data.length);
        } else {
            usableData = data.slice(0, nextBoundary);
            this.leftOver = data.slice(nextBoundary, data.length);
        }
    }

    this.push({
        data : exports.utf8decode(usableData),
        meta : chunk.meta
    });
};

/**
 * @see GenericWorker.flush
 */
Utf8DecodeWorker.prototype.flush = function () {
    if(this.leftOver && this.leftOver.length) {
        this.push({
            data : exports.utf8decode(this.leftOver),
            meta : {}
        });
        this.leftOver = null;
    }
};
exports.Utf8DecodeWorker = Utf8DecodeWorker;

/**
 * A worker to endcode string chunks into utf8 encoded binary chunks.
 * @constructor
 */
function Utf8EncodeWorker() {
    GenericWorker_1.call(this, "utf-8 encode");
}
utils.inherits(Utf8EncodeWorker, GenericWorker_1);

/**
 * @see GenericWorker.processChunk
 */
Utf8EncodeWorker.prototype.processChunk = function (chunk) {
    this.push({
        data : exports.utf8encode(chunk.data),
        meta : chunk.meta
    });
};
exports.Utf8EncodeWorker = Utf8EncodeWorker;
});
var utf8_1 = utf8.utf8encode;
var utf8_2 = utf8.utf8decode;
var utf8_3 = utf8.Utf8DecodeWorker;
var utf8_4 = utf8.Utf8EncodeWorker;

/**
 * A worker which convert chunks to a specified type.
 * @constructor
 * @param {String} destType the destination type.
 */
function ConvertWorker(destType) {
    GenericWorker_1.call(this, "ConvertWorker to " + destType);
    this.destType = destType;
}
utils.inherits(ConvertWorker, GenericWorker_1);

/**
 * @see GenericWorker.processChunk
 */
ConvertWorker.prototype.processChunk = function (chunk) {
    this.push({
        data : utils.transformTo(this.destType, chunk.data),
        meta : chunk.meta
    });
};
var ConvertWorker_1 = ConvertWorker;

var Readable$1 = readableStreamBrowser.Readable;


utils.inherits(NodejsStreamOutputAdapter, Readable$1);

/**
* A nodejs stream using a worker as source.
* @see the SourceWrapper in http://nodejs.org/api/stream.html
* @constructor
* @param {StreamHelper} helper the helper wrapping the worker
* @param {Object} options the nodejs stream options
* @param {Function} updateCb the update callback.
*/
function NodejsStreamOutputAdapter(helper, options, updateCb) {
    Readable$1.call(this, options);
    this._helper = helper;

    var self = this;
    helper.on("data", function (data, meta) {
        if (!self.push(data)) {
            self._helper.pause();
        }
        if(updateCb) {
            updateCb(meta);
        }
    })
    .on("error", function(e) {
        self.emit('error', e);
    })
    .on("end", function () {
        self.push(null);
    });
}


NodejsStreamOutputAdapter.prototype._read = function() {
    this._helper.resume();
};

var NodejsStreamOutputAdapter_1 = NodejsStreamOutputAdapter;

var NodejsStreamOutputAdapter$1 = null;
if (support.nodestream) {
    try {
        NodejsStreamOutputAdapter$1 = NodejsStreamOutputAdapter_1;
    } catch(e) {}
}

/**
 * Apply the final transformation of the data. If the user wants a Blob for
 * example, it's easier to work with an U8intArray and finally do the
 * ArrayBuffer/Blob conversion.
 * @param {String} type the name of the final type
 * @param {String|Uint8Array|Buffer} content the content to transform
 * @param {String} mimeType the mime type of the content, if applicable.
 * @return {String|Uint8Array|ArrayBuffer|Buffer|Blob} the content in the right format.
 */
function transformZipOutput(type, content, mimeType) {
    switch(type) {
        case "blob" :
            return utils.newBlob(utils.transformTo("arraybuffer", content), mimeType);
        case "base64" :
            return base64.encode(content);
        default :
            return utils.transformTo(type, content);
    }
}

/**
 * Concatenate an array of data of the given type.
 * @param {String} type the type of the data in the given array.
 * @param {Array} dataArray the array containing the data chunks to concatenate
 * @return {String|Uint8Array|Buffer} the concatenated data
 * @throws Error if the asked type is unsupported
 */
function concat (type, dataArray) {
    var i, index = 0, res = null, totalLength = 0;
    for(i = 0; i < dataArray.length; i++) {
        totalLength += dataArray[i].length;
    }
    switch(type) {
        case "string":
            return dataArray.join("");
          case "array":
            return Array.prototype.concat.apply([], dataArray);
        case "uint8array":
            res = new Uint8Array(totalLength);
            for(i = 0; i < dataArray.length; i++) {
                res.set(dataArray[i], index);
                index += dataArray[i].length;
            }
            return res;
        case "nodebuffer":
            return Buffer.concat(dataArray);
        default:
            throw new Error("concat : unsupported type '"  + type + "'");
    }
}

/**
 * Listen a StreamHelper, accumulate its content and concatenate it into a
 * complete block.
 * @param {StreamHelper} helper the helper to use.
 * @param {Function} updateCallback a callback called on each update. Called
 * with one arg :
 * - the metadata linked to the update received.
 * @return Promise the promise for the accumulation.
 */
function accumulate(helper, updateCallback) {
    return new external.Promise(function (resolve, reject){
        var dataArray = [];
        var chunkType = helper._internalType,
            resultType = helper._outputType,
            mimeType = helper._mimeType;
        helper
        .on('data', function (data, meta) {
            dataArray.push(data);
            if(updateCallback) {
                updateCallback(meta);
            }
        })
        .on('error', function(err) {
            dataArray = [];
            reject(err);
        })
        .on('end', function (){
            try {
                var result = transformZipOutput(resultType, concat(chunkType, dataArray), mimeType);
                resolve(result);
            } catch (e) {
                reject(e);
            }
            dataArray = [];
        })
        .resume();
    });
}

/**
 * An helper to easily use workers outside of JSZip.
 * @constructor
 * @param {Worker} worker the worker to wrap
 * @param {String} outputType the type of data expected by the use
 * @param {String} mimeType the mime type of the content, if applicable.
 */
function StreamHelper(worker, outputType, mimeType) {
    var internalType = outputType;
    switch(outputType) {
        case "blob":
        case "arraybuffer":
            internalType = "uint8array";
        break;
        case "base64":
            internalType = "string";
        break;
    }

    try {
        // the type used internally
        this._internalType = internalType;
        // the type used to output results
        this._outputType = outputType;
        // the mime type
        this._mimeType = mimeType;
        utils.checkSupport(internalType);
        this._worker = worker.pipe(new ConvertWorker_1(internalType));
        // the last workers can be rewired without issues but we need to
        // prevent any updates on previous workers.
        worker.lock();
    } catch(e) {
        this._worker = new GenericWorker_1("error");
        this._worker.error(e);
    }
}

StreamHelper.prototype = {
    /**
     * Listen a StreamHelper, accumulate its content and concatenate it into a
     * complete block.
     * @param {Function} updateCb the update callback.
     * @return Promise the promise for the accumulation.
     */
    accumulate : function (updateCb) {
        return accumulate(this, updateCb);
    },
    /**
     * Add a listener on an event triggered on a stream.
     * @param {String} evt the name of the event
     * @param {Function} fn the listener
     * @return {StreamHelper} the current helper.
     */
    on : function (evt, fn) {
        var self = this;

        if(evt === "data") {
            this._worker.on(evt, function (chunk) {
                fn.call(self, chunk.data, chunk.meta);
            });
        } else {
            this._worker.on(evt, function () {
                utils.delay(fn, arguments, self);
            });
        }
        return this;
    },
    /**
     * Resume the flow of chunks.
     * @return {StreamHelper} the current helper.
     */
    resume : function () {
        utils.delay(this._worker.resume, [], this._worker);
        return this;
    },
    /**
     * Pause the flow of chunks.
     * @return {StreamHelper} the current helper.
     */
    pause : function () {
        this._worker.pause();
        return this;
    },
    /**
     * Return a nodejs stream for this helper.
     * @param {Function} updateCb the update callback.
     * @return {NodejsStreamOutputAdapter} the nodejs stream.
     */
    toNodejsStream : function (updateCb) {
        utils.checkSupport("nodestream");
        if (this._outputType !== "nodebuffer") {
            // an object stream containing blob/arraybuffer/uint8array/string
            // is strange and I don't know if it would be useful.
            // I you find this comment and have a good usecase, please open a
            // bug report !
            throw new Error(this._outputType + " is not supported by this method");
        }

        return new NodejsStreamOutputAdapter$1(this, {
            objectMode : this._outputType !== "nodebuffer"
        }, updateCb);
    }
};


var StreamHelper_1 = StreamHelper;

var base64$1 = false;
var binary = false;
var dir = false;
var createFolders = true;
var date = null;
var compression = null;
var compressionOptions = null;
var comment = null;
var unixPermissions = null;
var dosPermissions = null;

var defaults = {
	base64: base64$1,
	binary: binary,
	dir: dir,
	createFolders: createFolders,
	date: date,
	compression: compression,
	compressionOptions: compressionOptions,
	comment: comment,
	unixPermissions: unixPermissions,
	dosPermissions: dosPermissions
};

// the size of the generated chunks
// TODO expose this as a public variable
var DEFAULT_BLOCK_SIZE = 16 * 1024;

/**
 * A worker that reads a content and emits chunks.
 * @constructor
 * @param {Promise} dataP the promise of the data to split
 */
function DataWorker(dataP) {
    GenericWorker_1.call(this, "DataWorker");
    var self = this;
    this.dataIsReady = false;
    this.index = 0;
    this.max = 0;
    this.data = null;
    this.type = "";

    this._tickScheduled = false;

    dataP.then(function (data) {
        self.dataIsReady = true;
        self.data = data;
        self.max = data && data.length || 0;
        self.type = utils.getTypeOf(data);
        if(!self.isPaused) {
            self._tickAndRepeat();
        }
    }, function (e) {
        self.error(e);
    });
}

utils.inherits(DataWorker, GenericWorker_1);

/**
 * @see GenericWorker.cleanUp
 */
DataWorker.prototype.cleanUp = function () {
    GenericWorker_1.prototype.cleanUp.call(this);
    this.data = null;
};

/**
 * @see GenericWorker.resume
 */
DataWorker.prototype.resume = function () {
    if(!GenericWorker_1.prototype.resume.call(this)) {
        return false;
    }

    if (!this._tickScheduled && this.dataIsReady) {
        this._tickScheduled = true;
        utils.delay(this._tickAndRepeat, [], this);
    }
    return true;
};

/**
 * Trigger a tick a schedule an other call to this function.
 */
DataWorker.prototype._tickAndRepeat = function() {
    this._tickScheduled = false;
    if(this.isPaused || this.isFinished) {
        return;
    }
    this._tick();
    if(!this.isFinished) {
        utils.delay(this._tickAndRepeat, [], this);
        this._tickScheduled = true;
    }
};

/**
 * Read and push a chunk.
 */
DataWorker.prototype._tick = function() {

    if(this.isPaused || this.isFinished) {
        return false;
    }

    var size = DEFAULT_BLOCK_SIZE;
    var data = null, nextIndex = Math.min(this.max, this.index + size);
    if (this.index >= this.max) {
        // EOF
        return this.end();
    } else {
        switch(this.type) {
            case "string":
                data = this.data.substring(this.index, nextIndex);
            break;
            case "uint8array":
                data = this.data.subarray(this.index, nextIndex);
            break;
            case "array":
            case "nodebuffer":
                data = this.data.slice(this.index, nextIndex);
            break;
        }
        this.index = nextIndex;
        return this.push({
            data : data,
            meta : {
                percent : this.max ? this.index / this.max * 100 : 0
            }
        });
    }
};

var DataWorker_1 = DataWorker;

/**
 * A worker which calculate the total length of the data flowing through.
 * @constructor
 * @param {String} propName the name used to expose the length
 */
function DataLengthProbe(propName) {
    GenericWorker_1.call(this, "DataLengthProbe for " + propName);
    this.propName = propName;
    this.withStreamInfo(propName, 0);
}
utils.inherits(DataLengthProbe, GenericWorker_1);

/**
 * @see GenericWorker.processChunk
 */
DataLengthProbe.prototype.processChunk = function (chunk) {
    if(chunk) {
        var length = this.streamInfo[this.propName] || 0;
        this.streamInfo[this.propName] = length + chunk.data.length;
    }
    GenericWorker_1.prototype.processChunk.call(this, chunk);
};
var DataLengthProbe_1 = DataLengthProbe;

/**
 * The following functions come from pako, from pako/lib/zlib/crc32.js
 * released under the MIT license, see pako https://github.com/nodeca/pako/
 */

// Use ordinary array, since untyped makes no boost here
function makeTable() {
    var c, table = [];

    for(var n =0; n < 256; n++){
        c = n;
        for(var k =0; k < 8; k++){
            c = ((c&1) ? (0xEDB88320 ^ (c >>> 1)) : (c >>> 1));
        }
        table[n] = c;
    }

    return table;
}

// Create table on load. Just 255 signed longs. Not a problem.
var crcTable = makeTable();


function crc32(crc, buf, len, pos) {
    var t = crcTable, end = pos + len;

    crc = crc ^ (-1);

    for (var i = pos; i < end; i++ ) {
        crc = (crc >>> 8) ^ t[(crc ^ buf[i]) & 0xFF];
    }

    return (crc ^ (-1)); // >>> 0;
}

// That's all for the pako functions.

/**
 * Compute the crc32 of a string.
 * This is almost the same as the function crc32, but for strings. Using the
 * same function for the two use cases leads to horrible performances.
 * @param {Number} crc the starting value of the crc.
 * @param {String} str the string to use.
 * @param {Number} len the length of the string.
 * @param {Number} pos the starting position for the crc32 computation.
 * @return {Number} the computed crc32.
 */
function crc32str(crc, str, len, pos) {
    var t = crcTable, end = pos + len;

    crc = crc ^ (-1);

    for (var i = pos; i < end; i++ ) {
        crc = (crc >>> 8) ^ t[(crc ^ str.charCodeAt(i)) & 0xFF];
    }

    return (crc ^ (-1)); // >>> 0;
}

var crc32_1 = function crc32wrapper(input, crc) {
    if (typeof input === "undefined" || !input.length) {
        return 0;
    }

    var isArray = utils.getTypeOf(input) !== "string";

    if(isArray) {
        return crc32(crc|0, input, input.length, 0);
    } else {
        return crc32str(crc|0, input, input.length, 0);
    }
};

/**
 * A worker which calculate the crc32 of the data flowing through.
 * @constructor
 */
function Crc32Probe() {
    GenericWorker_1.call(this, "Crc32Probe");
    this.withStreamInfo("crc32", 0);
}
utils.inherits(Crc32Probe, GenericWorker_1);

/**
 * @see GenericWorker.processChunk
 */
Crc32Probe.prototype.processChunk = function (chunk) {
    this.streamInfo.crc32 = crc32_1(chunk.data, this.streamInfo.crc32 || 0);
    this.push(chunk);
};
var Crc32Probe_1 = Crc32Probe;

/**
 * Represent a compressed object, with everything needed to decompress it.
 * @constructor
 * @param {number} compressedSize the size of the data compressed.
 * @param {number} uncompressedSize the size of the data after decompression.
 * @param {number} crc32 the crc32 of the decompressed file.
 * @param {object} compression the type of compression, see lib/compressions.js.
 * @param {String|ArrayBuffer|Uint8Array|Buffer} data the compressed data.
 */
function CompressedObject(compressedSize, uncompressedSize, crc32, compression, data) {
    this.compressedSize = compressedSize;
    this.uncompressedSize = uncompressedSize;
    this.crc32 = crc32;
    this.compression = compression;
    this.compressedContent = data;
}

CompressedObject.prototype = {
    /**
     * Create a worker to get the uncompressed content.
     * @return {GenericWorker} the worker.
     */
    getContentWorker : function () {
        var worker = new DataWorker_1(external.Promise.resolve(this.compressedContent))
        .pipe(this.compression.uncompressWorker())
        .pipe(new DataLengthProbe_1("data_length"));

        var that = this;
        worker.on("end", function () {
            if(this.streamInfo['data_length'] !== that.uncompressedSize) {
                throw new Error("Bug : uncompressed data size mismatch");
            }
        });
        return worker;
    },
    /**
     * Create a worker to get the compressed content.
     * @return {GenericWorker} the worker.
     */
    getCompressedWorker : function () {
        return new DataWorker_1(external.Promise.resolve(this.compressedContent))
        .withStreamInfo("compressedSize", this.compressedSize)
        .withStreamInfo("uncompressedSize", this.uncompressedSize)
        .withStreamInfo("crc32", this.crc32)
        .withStreamInfo("compression", this.compression)
        ;
    }
};

/**
 * Chain the given worker with other workers to compress the content with the
 * given compresion.
 * @param {GenericWorker} uncompressedWorker the worker to pipe.
 * @param {Object} compression the compression object.
 * @param {Object} compressionOptions the options to use when compressing.
 * @return {GenericWorker} the new worker compressing the content.
 */
CompressedObject.createWorkerFrom = function (uncompressedWorker, compression, compressionOptions) {
    return uncompressedWorker
    .pipe(new Crc32Probe_1())
    .pipe(new DataLengthProbe_1("uncompressedSize"))
    .pipe(compression.compressWorker(compressionOptions))
    .pipe(new DataLengthProbe_1("compressedSize"))
    .withStreamInfo("compression", compression);
};

var compressedObject = CompressedObject;

/**
 * A simple object representing a file in the zip file.
 * @constructor
 * @param {string} name the name of the file
 * @param {String|ArrayBuffer|Uint8Array|Buffer} data the data
 * @param {Object} options the options of the file
 */
var ZipObject = function(name, data, options) {
    this.name = name;
    this.dir = options.dir;
    this.date = options.date;
    this.comment = options.comment;
    this.unixPermissions = options.unixPermissions;
    this.dosPermissions = options.dosPermissions;

    this._data = data;
    this._dataBinary = options.binary;
    // keep only the compression
    this.options = {
        compression : options.compression,
        compressionOptions : options.compressionOptions
    };
};

ZipObject.prototype = {
    /**
     * Create an internal stream for the content of this object.
     * @param {String} type the type of each chunk.
     * @return StreamHelper the stream.
     */
    internalStream: function (type) {
        var result = null, outputType = "string";
        try {
            if (!type) {
                throw new Error("No output type specified.");
            }
            outputType = type.toLowerCase();
            var askUnicodeString = outputType === "string" || outputType === "text";
            if (outputType === "binarystring" || outputType === "text") {
                outputType = "string";
            }
            result = this._decompressWorker();

            var isUnicodeString = !this._dataBinary;

            if (isUnicodeString && !askUnicodeString) {
                result = result.pipe(new utf8.Utf8EncodeWorker());
            }
            if (!isUnicodeString && askUnicodeString) {
                result = result.pipe(new utf8.Utf8DecodeWorker());
            }
        } catch (e) {
            result = new GenericWorker_1("error");
            result.error(e);
        }

        return new StreamHelper_1(result, outputType, "");
    },

    /**
     * Prepare the content in the asked type.
     * @param {String} type the type of the result.
     * @param {Function} onUpdate a function to call on each internal update.
     * @return Promise the promise of the result.
     */
    async: function (type, onUpdate) {
        return this.internalStream(type).accumulate(onUpdate);
    },

    /**
     * Prepare the content as a nodejs stream.
     * @param {String} type the type of each chunk.
     * @param {Function} onUpdate a function to call on each internal update.
     * @return Stream the stream.
     */
    nodeStream: function (type, onUpdate) {
        return this.internalStream(type || "nodebuffer").toNodejsStream(onUpdate);
    },

    /**
     * Return a worker for the compressed content.
     * @private
     * @param {Object} compression the compression object to use.
     * @param {Object} compressionOptions the options to use when compressing.
     * @return Worker the worker.
     */
    _compressWorker: function (compression, compressionOptions) {
        if (
            this._data instanceof compressedObject &&
            this._data.compression.magic === compression.magic
        ) {
            return this._data.getCompressedWorker();
        } else {
            var result = this._decompressWorker();
            if(!this._dataBinary) {
                result = result.pipe(new utf8.Utf8EncodeWorker());
            }
            return compressedObject.createWorkerFrom(result, compression, compressionOptions);
        }
    },
    /**
     * Return a worker for the decompressed content.
     * @private
     * @return Worker the worker.
     */
    _decompressWorker : function () {
        if (this._data instanceof compressedObject) {
            return this._data.getContentWorker();
        } else if (this._data instanceof GenericWorker_1) {
            return this._data;
        } else {
            return new DataWorker_1(this._data);
        }
    }
};

var removedMethods = ["asText", "asBinary", "asNodeBuffer", "asUint8Array", "asArrayBuffer"];
var removedFn = function () {
    throw new Error("This method has been removed in JSZip 3.0, please check the upgrade guide.");
};

for(var i = 0; i < removedMethods.length; i++) {
    ZipObject.prototype[removedMethods[i]] = removedFn;
}
var zipObject = ZipObject;

var common = __chunk_1.createCommonjsModule(function (module, exports) {


var TYPED_OK =  (typeof Uint8Array !== 'undefined') &&
                (typeof Uint16Array !== 'undefined') &&
                (typeof Int32Array !== 'undefined');

function _has(obj, key) {
  return Object.prototype.hasOwnProperty.call(obj, key);
}

exports.assign = function (obj /*from1, from2, from3, ...*/) {
  var sources = Array.prototype.slice.call(arguments, 1);
  while (sources.length) {
    var source = sources.shift();
    if (!source) { continue; }

    if (typeof source !== 'object') {
      throw new TypeError(source + 'must be non-object');
    }

    for (var p in source) {
      if (_has(source, p)) {
        obj[p] = source[p];
      }
    }
  }

  return obj;
};


// reduce buffer size, avoiding mem copy
exports.shrinkBuf = function (buf, size) {
  if (buf.length === size) { return buf; }
  if (buf.subarray) { return buf.subarray(0, size); }
  buf.length = size;
  return buf;
};


var fnTyped = {
  arraySet: function (dest, src, src_offs, len, dest_offs) {
    if (src.subarray && dest.subarray) {
      dest.set(src.subarray(src_offs, src_offs + len), dest_offs);
      return;
    }
    // Fallback to ordinary array
    for (var i = 0; i < len; i++) {
      dest[dest_offs + i] = src[src_offs + i];
    }
  },
  // Join array of chunks to single array.
  flattenChunks: function (chunks) {
    var i, l, len, pos, chunk, result;

    // calculate data length
    len = 0;
    for (i = 0, l = chunks.length; i < l; i++) {
      len += chunks[i].length;
    }

    // join chunks
    result = new Uint8Array(len);
    pos = 0;
    for (i = 0, l = chunks.length; i < l; i++) {
      chunk = chunks[i];
      result.set(chunk, pos);
      pos += chunk.length;
    }

    return result;
  }
};

var fnUntyped = {
  arraySet: function (dest, src, src_offs, len, dest_offs) {
    for (var i = 0; i < len; i++) {
      dest[dest_offs + i] = src[src_offs + i];
    }
  },
  // Join array of chunks to single array.
  flattenChunks: function (chunks) {
    return [].concat.apply([], chunks);
  }
};


// Enable/Disable typed arrays use, for testing
//
exports.setTyped = function (on) {
  if (on) {
    exports.Buf8  = Uint8Array;
    exports.Buf16 = Uint16Array;
    exports.Buf32 = Int32Array;
    exports.assign(exports, fnTyped);
  } else {
    exports.Buf8  = Array;
    exports.Buf16 = Array;
    exports.Buf32 = Array;
    exports.assign(exports, fnUntyped);
  }
};

exports.setTyped(TYPED_OK);
});
var common_1 = common.assign;
var common_2 = common.shrinkBuf;
var common_3 = common.setTyped;
var common_4 = common.Buf8;
var common_5 = common.Buf16;
var common_6 = common.Buf32;

// (C) 1995-2013 Jean-loup Gailly and Mark Adler
// (C) 2014-2017 Vitaly Puzrin and Andrey Tupitsin
//
// This software is provided 'as-is', without any express or implied
// warranty. In no event will the authors be held liable for any damages
// arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must not
//   claim that you wrote the original software. If you use this software
//   in a product, an acknowledgment in the product documentation would be
//   appreciated but is not required.
// 2. Altered source versions must be plainly marked as such, and must not be
//   misrepresented as being the original software.
// 3. This notice may not be removed or altered from any source distribution.



/* Public constants ==========================================================*/
/* ===========================================================================*/


//var Z_FILTERED          = 1;
//var Z_HUFFMAN_ONLY      = 2;
//var Z_RLE               = 3;
var Z_FIXED               = 4;
//var Z_DEFAULT_STRATEGY  = 0;

/* Possible values of the data_type field (though see inflate()) */
var Z_BINARY              = 0;
var Z_TEXT                = 1;
//var Z_ASCII             = 1; // = Z_TEXT
var Z_UNKNOWN             = 2;

/*============================================================================*/


function zero(buf) { var len = buf.length; while (--len >= 0) { buf[len] = 0; } }

// From zutil.h

var STORED_BLOCK = 0;
var STATIC_TREES = 1;
var DYN_TREES    = 2;
/* The three kinds of block type */

var MIN_MATCH    = 3;
var MAX_MATCH    = 258;
/* The minimum and maximum match lengths */

// From deflate.h
/* ===========================================================================
 * Internal compression state.
 */

var LENGTH_CODES  = 29;
/* number of length codes, not counting the special END_BLOCK code */

var LITERALS      = 256;
/* number of literal bytes 0..255 */

var L_CODES       = LITERALS + 1 + LENGTH_CODES;
/* number of Literal or Length codes, including the END_BLOCK code */

var D_CODES       = 30;
/* number of distance codes */

var BL_CODES      = 19;
/* number of codes used to transfer the bit lengths */

var HEAP_SIZE     = 2 * L_CODES + 1;
/* maximum heap size */

var MAX_BITS      = 15;
/* All codes must not exceed MAX_BITS bits */

var Buf_size      = 16;
/* size of bit buffer in bi_buf */


/* ===========================================================================
 * Constants
 */

var MAX_BL_BITS = 7;
/* Bit length codes must not exceed MAX_BL_BITS bits */

var END_BLOCK   = 256;
/* end of block literal code */

var REP_3_6     = 16;
/* repeat previous bit length 3-6 times (2 bits of repeat count) */

var REPZ_3_10   = 17;
/* repeat a zero length 3-10 times  (3 bits of repeat count) */

var REPZ_11_138 = 18;
/* repeat a zero length 11-138 times  (7 bits of repeat count) */

/* eslint-disable comma-spacing,array-bracket-spacing */
var extra_lbits =   /* extra bits for each length code */
  [0,0,0,0,0,0,0,0,1,1,1,1,2,2,2,2,3,3,3,3,4,4,4,4,5,5,5,5,0];

var extra_dbits =   /* extra bits for each distance code */
  [0,0,0,0,1,1,2,2,3,3,4,4,5,5,6,6,7,7,8,8,9,9,10,10,11,11,12,12,13,13];

var extra_blbits =  /* extra bits for each bit length code */
  [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,3,7];

var bl_order =
  [16,17,18,0,8,7,9,6,10,5,11,4,12,3,13,2,14,1,15];
/* eslint-enable comma-spacing,array-bracket-spacing */

/* The lengths of the bit length codes are sent in order of decreasing
 * probability, to avoid transmitting the lengths for unused bit length codes.
 */

/* ===========================================================================
 * Local data. These are initialized only once.
 */

// We pre-fill arrays with 0 to avoid uninitialized gaps

var DIST_CODE_LEN = 512; /* see definition of array dist_code below */

// !!!! Use flat array instead of structure, Freq = i*2, Len = i*2+1
var static_ltree  = new Array((L_CODES + 2) * 2);
zero(static_ltree);
/* The static literal tree. Since the bit lengths are imposed, there is no
 * need for the L_CODES extra codes used during heap construction. However
 * The codes 286 and 287 are needed to build a canonical tree (see _tr_init
 * below).
 */

var static_dtree  = new Array(D_CODES * 2);
zero(static_dtree);
/* The static distance tree. (Actually a trivial tree since all codes use
 * 5 bits.)
 */

var _dist_code    = new Array(DIST_CODE_LEN);
zero(_dist_code);
/* Distance codes. The first 256 values correspond to the distances
 * 3 .. 258, the last 256 values correspond to the top 8 bits of
 * the 15 bit distances.
 */

var _length_code  = new Array(MAX_MATCH - MIN_MATCH + 1);
zero(_length_code);
/* length code for each normalized match length (0 == MIN_MATCH) */

var base_length   = new Array(LENGTH_CODES);
zero(base_length);
/* First normalized length for each code (0 = MIN_MATCH) */

var base_dist     = new Array(D_CODES);
zero(base_dist);
/* First normalized distance for each code (0 = distance of 1) */


function StaticTreeDesc(static_tree, extra_bits, extra_base, elems, max_length) {

  this.static_tree  = static_tree;  /* static tree or NULL */
  this.extra_bits   = extra_bits;   /* extra bits for each code or NULL */
  this.extra_base   = extra_base;   /* base index for extra_bits */
  this.elems        = elems;        /* max number of elements in the tree */
  this.max_length   = max_length;   /* max bit length for the codes */

  // show if `static_tree` has data or dummy - needed for monomorphic objects
  this.has_stree    = static_tree && static_tree.length;
}


var static_l_desc;
var static_d_desc;
var static_bl_desc;


function TreeDesc(dyn_tree, stat_desc) {
  this.dyn_tree = dyn_tree;     /* the dynamic tree */
  this.max_code = 0;            /* largest code with non zero frequency */
  this.stat_desc = stat_desc;   /* the corresponding static tree */
}



function d_code(dist) {
  return dist < 256 ? _dist_code[dist] : _dist_code[256 + (dist >>> 7)];
}


/* ===========================================================================
 * Output a short LSB first on the stream.
 * IN assertion: there is enough room in pendingBuf.
 */
function put_short(s, w) {
//    put_byte(s, (uch)((w) & 0xff));
//    put_byte(s, (uch)((ush)(w) >> 8));
  s.pending_buf[s.pending++] = (w) & 0xff;
  s.pending_buf[s.pending++] = (w >>> 8) & 0xff;
}


/* ===========================================================================
 * Send a value on a given number of bits.
 * IN assertion: length <= 16 and value fits in length bits.
 */
function send_bits(s, value, length) {
  if (s.bi_valid > (Buf_size - length)) {
    s.bi_buf |= (value << s.bi_valid) & 0xffff;
    put_short(s, s.bi_buf);
    s.bi_buf = value >> (Buf_size - s.bi_valid);
    s.bi_valid += length - Buf_size;
  } else {
    s.bi_buf |= (value << s.bi_valid) & 0xffff;
    s.bi_valid += length;
  }
}


function send_code(s, c, tree) {
  send_bits(s, tree[c * 2]/*.Code*/, tree[c * 2 + 1]/*.Len*/);
}


/* ===========================================================================
 * Reverse the first len bits of a code, using straightforward code (a faster
 * method would use a table)
 * IN assertion: 1 <= len <= 15
 */
function bi_reverse(code, len) {
  var res = 0;
  do {
    res |= code & 1;
    code >>>= 1;
    res <<= 1;
  } while (--len > 0);
  return res >>> 1;
}


/* ===========================================================================
 * Flush the bit buffer, keeping at most 7 bits in it.
 */
function bi_flush(s) {
  if (s.bi_valid === 16) {
    put_short(s, s.bi_buf);
    s.bi_buf = 0;
    s.bi_valid = 0;

  } else if (s.bi_valid >= 8) {
    s.pending_buf[s.pending++] = s.bi_buf & 0xff;
    s.bi_buf >>= 8;
    s.bi_valid -= 8;
  }
}


/* ===========================================================================
 * Compute the optimal bit lengths for a tree and update the total bit length
 * for the current block.
 * IN assertion: the fields freq and dad are set, heap[heap_max] and
 *    above are the tree nodes sorted by increasing frequency.
 * OUT assertions: the field len is set to the optimal bit length, the
 *     array bl_count contains the frequencies for each bit length.
 *     The length opt_len is updated; static_len is also updated if stree is
 *     not null.
 */
function gen_bitlen(s, desc)
//    deflate_state *s;
//    tree_desc *desc;    /* the tree descriptor */
{
  var tree            = desc.dyn_tree;
  var max_code        = desc.max_code;
  var stree           = desc.stat_desc.static_tree;
  var has_stree       = desc.stat_desc.has_stree;
  var extra           = desc.stat_desc.extra_bits;
  var base            = desc.stat_desc.extra_base;
  var max_length      = desc.stat_desc.max_length;
  var h;              /* heap index */
  var n, m;           /* iterate over the tree elements */
  var bits;           /* bit length */
  var xbits;          /* extra bits */
  var f;              /* frequency */
  var overflow = 0;   /* number of elements with bit length too large */

  for (bits = 0; bits <= MAX_BITS; bits++) {
    s.bl_count[bits] = 0;
  }

  /* In a first pass, compute the optimal bit lengths (which may
   * overflow in the case of the bit length tree).
   */
  tree[s.heap[s.heap_max] * 2 + 1]/*.Len*/ = 0; /* root of the heap */

  for (h = s.heap_max + 1; h < HEAP_SIZE; h++) {
    n = s.heap[h];
    bits = tree[tree[n * 2 + 1]/*.Dad*/ * 2 + 1]/*.Len*/ + 1;
    if (bits > max_length) {
      bits = max_length;
      overflow++;
    }
    tree[n * 2 + 1]/*.Len*/ = bits;
    /* We overwrite tree[n].Dad which is no longer needed */

    if (n > max_code) { continue; } /* not a leaf node */

    s.bl_count[bits]++;
    xbits = 0;
    if (n >= base) {
      xbits = extra[n - base];
    }
    f = tree[n * 2]/*.Freq*/;
    s.opt_len += f * (bits + xbits);
    if (has_stree) {
      s.static_len += f * (stree[n * 2 + 1]/*.Len*/ + xbits);
    }
  }
  if (overflow === 0) { return; }

  // Trace((stderr,"\nbit length overflow\n"));
  /* This happens for example on obj2 and pic of the Calgary corpus */

  /* Find the first bit length which could increase: */
  do {
    bits = max_length - 1;
    while (s.bl_count[bits] === 0) { bits--; }
    s.bl_count[bits]--;      /* move one leaf down the tree */
    s.bl_count[bits + 1] += 2; /* move one overflow item as its brother */
    s.bl_count[max_length]--;
    /* The brother of the overflow item also moves one step up,
     * but this does not affect bl_count[max_length]
     */
    overflow -= 2;
  } while (overflow > 0);

  /* Now recompute all bit lengths, scanning in increasing frequency.
   * h is still equal to HEAP_SIZE. (It is simpler to reconstruct all
   * lengths instead of fixing only the wrong ones. This idea is taken
   * from 'ar' written by Haruhiko Okumura.)
   */
  for (bits = max_length; bits !== 0; bits--) {
    n = s.bl_count[bits];
    while (n !== 0) {
      m = s.heap[--h];
      if (m > max_code) { continue; }
      if (tree[m * 2 + 1]/*.Len*/ !== bits) {
        // Trace((stderr,"code %d bits %d->%d\n", m, tree[m].Len, bits));
        s.opt_len += (bits - tree[m * 2 + 1]/*.Len*/) * tree[m * 2]/*.Freq*/;
        tree[m * 2 + 1]/*.Len*/ = bits;
      }
      n--;
    }
  }
}


/* ===========================================================================
 * Generate the codes for a given tree and bit counts (which need not be
 * optimal).
 * IN assertion: the array bl_count contains the bit length statistics for
 * the given tree and the field len is set for all tree elements.
 * OUT assertion: the field code is set for all tree elements of non
 *     zero code length.
 */
function gen_codes(tree, max_code, bl_count)
//    ct_data *tree;             /* the tree to decorate */
//    int max_code;              /* largest code with non zero frequency */
//    ushf *bl_count;            /* number of codes at each bit length */
{
  var next_code = new Array(MAX_BITS + 1); /* next code value for each bit length */
  var code = 0;              /* running code value */
  var bits;                  /* bit index */
  var n;                     /* code index */

  /* The distribution counts are first used to generate the code values
   * without bit reversal.
   */
  for (bits = 1; bits <= MAX_BITS; bits++) {
    next_code[bits] = code = (code + bl_count[bits - 1]) << 1;
  }
  /* Check that the bit counts in bl_count are consistent. The last code
   * must be all ones.
   */
  //Assert (code + bl_count[MAX_BITS]-1 == (1<<MAX_BITS)-1,
  //        "inconsistent bit counts");
  //Tracev((stderr,"\ngen_codes: max_code %d ", max_code));

  for (n = 0;  n <= max_code; n++) {
    var len = tree[n * 2 + 1]/*.Len*/;
    if (len === 0) { continue; }
    /* Now reverse the bits */
    tree[n * 2]/*.Code*/ = bi_reverse(next_code[len]++, len);

    //Tracecv(tree != static_ltree, (stderr,"\nn %3d %c l %2d c %4x (%x) ",
    //     n, (isgraph(n) ? n : ' '), len, tree[n].Code, next_code[len]-1));
  }
}


/* ===========================================================================
 * Initialize the various 'constant' tables.
 */
function tr_static_init() {
  var n;        /* iterates over tree elements */
  var bits;     /* bit counter */
  var length;   /* length value */
  var code;     /* code value */
  var dist;     /* distance index */
  var bl_count = new Array(MAX_BITS + 1);
  /* number of codes at each bit length for an optimal tree */

  // do check in _tr_init()
  //if (static_init_done) return;

  /* For some embedded targets, global variables are not initialized: */
/*#ifdef NO_INIT_GLOBAL_POINTERS
  static_l_desc.static_tree = static_ltree;
  static_l_desc.extra_bits = extra_lbits;
  static_d_desc.static_tree = static_dtree;
  static_d_desc.extra_bits = extra_dbits;
  static_bl_desc.extra_bits = extra_blbits;
#endif*/

  /* Initialize the mapping length (0..255) -> length code (0..28) */
  length = 0;
  for (code = 0; code < LENGTH_CODES - 1; code++) {
    base_length[code] = length;
    for (n = 0; n < (1 << extra_lbits[code]); n++) {
      _length_code[length++] = code;
    }
  }
  //Assert (length == 256, "tr_static_init: length != 256");
  /* Note that the length 255 (match length 258) can be represented
   * in two different ways: code 284 + 5 bits or code 285, so we
   * overwrite length_code[255] to use the best encoding:
   */
  _length_code[length - 1] = code;

  /* Initialize the mapping dist (0..32K) -> dist code (0..29) */
  dist = 0;
  for (code = 0; code < 16; code++) {
    base_dist[code] = dist;
    for (n = 0; n < (1 << extra_dbits[code]); n++) {
      _dist_code[dist++] = code;
    }
  }
  //Assert (dist == 256, "tr_static_init: dist != 256");
  dist >>= 7; /* from now on, all distances are divided by 128 */
  for (; code < D_CODES; code++) {
    base_dist[code] = dist << 7;
    for (n = 0; n < (1 << (extra_dbits[code] - 7)); n++) {
      _dist_code[256 + dist++] = code;
    }
  }
  //Assert (dist == 256, "tr_static_init: 256+dist != 512");

  /* Construct the codes of the static literal tree */
  for (bits = 0; bits <= MAX_BITS; bits++) {
    bl_count[bits] = 0;
  }

  n = 0;
  while (n <= 143) {
    static_ltree[n * 2 + 1]/*.Len*/ = 8;
    n++;
    bl_count[8]++;
  }
  while (n <= 255) {
    static_ltree[n * 2 + 1]/*.Len*/ = 9;
    n++;
    bl_count[9]++;
  }
  while (n <= 279) {
    static_ltree[n * 2 + 1]/*.Len*/ = 7;
    n++;
    bl_count[7]++;
  }
  while (n <= 287) {
    static_ltree[n * 2 + 1]/*.Len*/ = 8;
    n++;
    bl_count[8]++;
  }
  /* Codes 286 and 287 do not exist, but we must include them in the
   * tree construction to get a canonical Huffman tree (longest code
   * all ones)
   */
  gen_codes(static_ltree, L_CODES + 1, bl_count);

  /* The static distance tree is trivial: */
  for (n = 0; n < D_CODES; n++) {
    static_dtree[n * 2 + 1]/*.Len*/ = 5;
    static_dtree[n * 2]/*.Code*/ = bi_reverse(n, 5);
  }

  // Now data ready and we can init static trees
  static_l_desc = new StaticTreeDesc(static_ltree, extra_lbits, LITERALS + 1, L_CODES, MAX_BITS);
  static_d_desc = new StaticTreeDesc(static_dtree, extra_dbits, 0,          D_CODES, MAX_BITS);
  static_bl_desc = new StaticTreeDesc(new Array(0), extra_blbits, 0,         BL_CODES, MAX_BL_BITS);

  //static_init_done = true;
}


/* ===========================================================================
 * Initialize a new block.
 */
function init_block(s) {
  var n; /* iterates over tree elements */

  /* Initialize the trees. */
  for (n = 0; n < L_CODES;  n++) { s.dyn_ltree[n * 2]/*.Freq*/ = 0; }
  for (n = 0; n < D_CODES;  n++) { s.dyn_dtree[n * 2]/*.Freq*/ = 0; }
  for (n = 0; n < BL_CODES; n++) { s.bl_tree[n * 2]/*.Freq*/ = 0; }

  s.dyn_ltree[END_BLOCK * 2]/*.Freq*/ = 1;
  s.opt_len = s.static_len = 0;
  s.last_lit = s.matches = 0;
}


/* ===========================================================================
 * Flush the bit buffer and align the output on a byte boundary
 */
function bi_windup(s)
{
  if (s.bi_valid > 8) {
    put_short(s, s.bi_buf);
  } else if (s.bi_valid > 0) {
    //put_byte(s, (Byte)s->bi_buf);
    s.pending_buf[s.pending++] = s.bi_buf;
  }
  s.bi_buf = 0;
  s.bi_valid = 0;
}

/* ===========================================================================
 * Copy a stored block, storing first the length and its
 * one's complement if requested.
 */
function copy_block(s, buf, len, header)
//DeflateState *s;
//charf    *buf;    /* the input data */
//unsigned len;     /* its length */
//int      header;  /* true if block header must be written */
{
  bi_windup(s);        /* align on byte boundary */

  if (header) {
    put_short(s, len);
    put_short(s, ~len);
  }
//  while (len--) {
//    put_byte(s, *buf++);
//  }
  common.arraySet(s.pending_buf, s.window, buf, len, s.pending);
  s.pending += len;
}

/* ===========================================================================
 * Compares to subtrees, using the tree depth as tie breaker when
 * the subtrees have equal frequency. This minimizes the worst case length.
 */
function smaller(tree, n, m, depth) {
  var _n2 = n * 2;
  var _m2 = m * 2;
  return (tree[_n2]/*.Freq*/ < tree[_m2]/*.Freq*/ ||
         (tree[_n2]/*.Freq*/ === tree[_m2]/*.Freq*/ && depth[n] <= depth[m]));
}

/* ===========================================================================
 * Restore the heap property by moving down the tree starting at node k,
 * exchanging a node with the smallest of its two sons if necessary, stopping
 * when the heap property is re-established (each father smaller than its
 * two sons).
 */
function pqdownheap(s, tree, k)
//    deflate_state *s;
//    ct_data *tree;  /* the tree to restore */
//    int k;               /* node to move down */
{
  var v = s.heap[k];
  var j = k << 1;  /* left son of k */
  while (j <= s.heap_len) {
    /* Set j to the smallest of the two sons: */
    if (j < s.heap_len &&
      smaller(tree, s.heap[j + 1], s.heap[j], s.depth)) {
      j++;
    }
    /* Exit if v is smaller than both sons */
    if (smaller(tree, v, s.heap[j], s.depth)) { break; }

    /* Exchange v with the smallest son */
    s.heap[k] = s.heap[j];
    k = j;

    /* And continue down the tree, setting j to the left son of k */
    j <<= 1;
  }
  s.heap[k] = v;
}


// inlined manually
// var SMALLEST = 1;

/* ===========================================================================
 * Send the block data compressed using the given Huffman trees
 */
function compress_block(s, ltree, dtree)
//    deflate_state *s;
//    const ct_data *ltree; /* literal tree */
//    const ct_data *dtree; /* distance tree */
{
  var dist;           /* distance of matched string */
  var lc;             /* match length or unmatched char (if dist == 0) */
  var lx = 0;         /* running index in l_buf */
  var code;           /* the code to send */
  var extra;          /* number of extra bits to send */

  if (s.last_lit !== 0) {
    do {
      dist = (s.pending_buf[s.d_buf + lx * 2] << 8) | (s.pending_buf[s.d_buf + lx * 2 + 1]);
      lc = s.pending_buf[s.l_buf + lx];
      lx++;

      if (dist === 0) {
        send_code(s, lc, ltree); /* send a literal byte */
        //Tracecv(isgraph(lc), (stderr," '%c' ", lc));
      } else {
        /* Here, lc is the match length - MIN_MATCH */
        code = _length_code[lc];
        send_code(s, code + LITERALS + 1, ltree); /* send the length code */
        extra = extra_lbits[code];
        if (extra !== 0) {
          lc -= base_length[code];
          send_bits(s, lc, extra);       /* send the extra length bits */
        }
        dist--; /* dist is now the match distance - 1 */
        code = d_code(dist);
        //Assert (code < D_CODES, "bad d_code");

        send_code(s, code, dtree);       /* send the distance code */
        extra = extra_dbits[code];
        if (extra !== 0) {
          dist -= base_dist[code];
          send_bits(s, dist, extra);   /* send the extra distance bits */
        }
      } /* literal or match pair ? */

      /* Check that the overlay between pending_buf and d_buf+l_buf is ok: */
      //Assert((uInt)(s->pending) < s->lit_bufsize + 2*lx,
      //       "pendingBuf overflow");

    } while (lx < s.last_lit);
  }

  send_code(s, END_BLOCK, ltree);
}


/* ===========================================================================
 * Construct one Huffman tree and assigns the code bit strings and lengths.
 * Update the total bit length for the current block.
 * IN assertion: the field freq is set for all tree elements.
 * OUT assertions: the fields len and code are set to the optimal bit length
 *     and corresponding code. The length opt_len is updated; static_len is
 *     also updated if stree is not null. The field max_code is set.
 */
function build_tree(s, desc)
//    deflate_state *s;
//    tree_desc *desc; /* the tree descriptor */
{
  var tree     = desc.dyn_tree;
  var stree    = desc.stat_desc.static_tree;
  var has_stree = desc.stat_desc.has_stree;
  var elems    = desc.stat_desc.elems;
  var n, m;          /* iterate over heap elements */
  var max_code = -1; /* largest code with non zero frequency */
  var node;          /* new node being created */

  /* Construct the initial heap, with least frequent element in
   * heap[SMALLEST]. The sons of heap[n] are heap[2*n] and heap[2*n+1].
   * heap[0] is not used.
   */
  s.heap_len = 0;
  s.heap_max = HEAP_SIZE;

  for (n = 0; n < elems; n++) {
    if (tree[n * 2]/*.Freq*/ !== 0) {
      s.heap[++s.heap_len] = max_code = n;
      s.depth[n] = 0;

    } else {
      tree[n * 2 + 1]/*.Len*/ = 0;
    }
  }

  /* The pkzip format requires that at least one distance code exists,
   * and that at least one bit should be sent even if there is only one
   * possible code. So to avoid special checks later on we force at least
   * two codes of non zero frequency.
   */
  while (s.heap_len < 2) {
    node = s.heap[++s.heap_len] = (max_code < 2 ? ++max_code : 0);
    tree[node * 2]/*.Freq*/ = 1;
    s.depth[node] = 0;
    s.opt_len--;

    if (has_stree) {
      s.static_len -= stree[node * 2 + 1]/*.Len*/;
    }
    /* node is 0 or 1 so it does not have extra bits */
  }
  desc.max_code = max_code;

  /* The elements heap[heap_len/2+1 .. heap_len] are leaves of the tree,
   * establish sub-heaps of increasing lengths:
   */
  for (n = (s.heap_len >> 1/*int /2*/); n >= 1; n--) { pqdownheap(s, tree, n); }

  /* Construct the Huffman tree by repeatedly combining the least two
   * frequent nodes.
   */
  node = elems;              /* next internal node of the tree */
  do {
    //pqremove(s, tree, n);  /* n = node of least frequency */
    /*** pqremove ***/
    n = s.heap[1/*SMALLEST*/];
    s.heap[1/*SMALLEST*/] = s.heap[s.heap_len--];
    pqdownheap(s, tree, 1/*SMALLEST*/);
    /***/

    m = s.heap[1/*SMALLEST*/]; /* m = node of next least frequency */

    s.heap[--s.heap_max] = n; /* keep the nodes sorted by frequency */
    s.heap[--s.heap_max] = m;

    /* Create a new node father of n and m */
    tree[node * 2]/*.Freq*/ = tree[n * 2]/*.Freq*/ + tree[m * 2]/*.Freq*/;
    s.depth[node] = (s.depth[n] >= s.depth[m] ? s.depth[n] : s.depth[m]) + 1;
    tree[n * 2 + 1]/*.Dad*/ = tree[m * 2 + 1]/*.Dad*/ = node;

    /* and insert the new node in the heap */
    s.heap[1/*SMALLEST*/] = node++;
    pqdownheap(s, tree, 1/*SMALLEST*/);

  } while (s.heap_len >= 2);

  s.heap[--s.heap_max] = s.heap[1/*SMALLEST*/];

  /* At this point, the fields freq and dad are set. We can now
   * generate the bit lengths.
   */
  gen_bitlen(s, desc);

  /* The field len is now set, we can generate the bit codes */
  gen_codes(tree, max_code, s.bl_count);
}


/* ===========================================================================
 * Scan a literal or distance tree to determine the frequencies of the codes
 * in the bit length tree.
 */
function scan_tree(s, tree, max_code)
//    deflate_state *s;
//    ct_data *tree;   /* the tree to be scanned */
//    int max_code;    /* and its largest code of non zero frequency */
{
  var n;                     /* iterates over all tree elements */
  var prevlen = -1;          /* last emitted length */
  var curlen;                /* length of current code */

  var nextlen = tree[0 * 2 + 1]/*.Len*/; /* length of next code */

  var count = 0;             /* repeat count of the current code */
  var max_count = 7;         /* max repeat count */
  var min_count = 4;         /* min repeat count */

  if (nextlen === 0) {
    max_count = 138;
    min_count = 3;
  }
  tree[(max_code + 1) * 2 + 1]/*.Len*/ = 0xffff; /* guard */

  for (n = 0; n <= max_code; n++) {
    curlen = nextlen;
    nextlen = tree[(n + 1) * 2 + 1]/*.Len*/;

    if (++count < max_count && curlen === nextlen) {
      continue;

    } else if (count < min_count) {
      s.bl_tree[curlen * 2]/*.Freq*/ += count;

    } else if (curlen !== 0) {

      if (curlen !== prevlen) { s.bl_tree[curlen * 2]/*.Freq*/++; }
      s.bl_tree[REP_3_6 * 2]/*.Freq*/++;

    } else if (count <= 10) {
      s.bl_tree[REPZ_3_10 * 2]/*.Freq*/++;

    } else {
      s.bl_tree[REPZ_11_138 * 2]/*.Freq*/++;
    }

    count = 0;
    prevlen = curlen;

    if (nextlen === 0) {
      max_count = 138;
      min_count = 3;

    } else if (curlen === nextlen) {
      max_count = 6;
      min_count = 3;

    } else {
      max_count = 7;
      min_count = 4;
    }
  }
}


/* ===========================================================================
 * Send a literal or distance tree in compressed form, using the codes in
 * bl_tree.
 */
function send_tree(s, tree, max_code)
//    deflate_state *s;
//    ct_data *tree; /* the tree to be scanned */
//    int max_code;       /* and its largest code of non zero frequency */
{
  var n;                     /* iterates over all tree elements */
  var prevlen = -1;          /* last emitted length */
  var curlen;                /* length of current code */

  var nextlen = tree[0 * 2 + 1]/*.Len*/; /* length of next code */

  var count = 0;             /* repeat count of the current code */
  var max_count = 7;         /* max repeat count */
  var min_count = 4;         /* min repeat count */

  /* tree[max_code+1].Len = -1; */  /* guard already set */
  if (nextlen === 0) {
    max_count = 138;
    min_count = 3;
  }

  for (n = 0; n <= max_code; n++) {
    curlen = nextlen;
    nextlen = tree[(n + 1) * 2 + 1]/*.Len*/;

    if (++count < max_count && curlen === nextlen) {
      continue;

    } else if (count < min_count) {
      do { send_code(s, curlen, s.bl_tree); } while (--count !== 0);

    } else if (curlen !== 0) {
      if (curlen !== prevlen) {
        send_code(s, curlen, s.bl_tree);
        count--;
      }
      //Assert(count >= 3 && count <= 6, " 3_6?");
      send_code(s, REP_3_6, s.bl_tree);
      send_bits(s, count - 3, 2);

    } else if (count <= 10) {
      send_code(s, REPZ_3_10, s.bl_tree);
      send_bits(s, count - 3, 3);

    } else {
      send_code(s, REPZ_11_138, s.bl_tree);
      send_bits(s, count - 11, 7);
    }

    count = 0;
    prevlen = curlen;
    if (nextlen === 0) {
      max_count = 138;
      min_count = 3;

    } else if (curlen === nextlen) {
      max_count = 6;
      min_count = 3;

    } else {
      max_count = 7;
      min_count = 4;
    }
  }
}


/* ===========================================================================
 * Construct the Huffman tree for the bit lengths and return the index in
 * bl_order of the last bit length code to send.
 */
function build_bl_tree(s) {
  var max_blindex;  /* index of last bit length code of non zero freq */

  /* Determine the bit length frequencies for literal and distance trees */
  scan_tree(s, s.dyn_ltree, s.l_desc.max_code);
  scan_tree(s, s.dyn_dtree, s.d_desc.max_code);

  /* Build the bit length tree: */
  build_tree(s, s.bl_desc);
  /* opt_len now includes the length of the tree representations, except
   * the lengths of the bit lengths codes and the 5+5+4 bits for the counts.
   */

  /* Determine the number of bit length codes to send. The pkzip format
   * requires that at least 4 bit length codes be sent. (appnote.txt says
   * 3 but the actual value used is 4.)
   */
  for (max_blindex = BL_CODES - 1; max_blindex >= 3; max_blindex--) {
    if (s.bl_tree[bl_order[max_blindex] * 2 + 1]/*.Len*/ !== 0) {
      break;
    }
  }
  /* Update opt_len to include the bit length tree and counts */
  s.opt_len += 3 * (max_blindex + 1) + 5 + 5 + 4;
  //Tracev((stderr, "\ndyn trees: dyn %ld, stat %ld",
  //        s->opt_len, s->static_len));

  return max_blindex;
}


/* ===========================================================================
 * Send the header for a block using dynamic Huffman trees: the counts, the
 * lengths of the bit length codes, the literal tree and the distance tree.
 * IN assertion: lcodes >= 257, dcodes >= 1, blcodes >= 4.
 */
function send_all_trees(s, lcodes, dcodes, blcodes)
//    deflate_state *s;
//    int lcodes, dcodes, blcodes; /* number of codes for each tree */
{
  var rank;                    /* index in bl_order */

  //Assert (lcodes >= 257 && dcodes >= 1 && blcodes >= 4, "not enough codes");
  //Assert (lcodes <= L_CODES && dcodes <= D_CODES && blcodes <= BL_CODES,
  //        "too many codes");
  //Tracev((stderr, "\nbl counts: "));
  send_bits(s, lcodes - 257, 5); /* not +255 as stated in appnote.txt */
  send_bits(s, dcodes - 1,   5);
  send_bits(s, blcodes - 4,  4); /* not -3 as stated in appnote.txt */
  for (rank = 0; rank < blcodes; rank++) {
    //Tracev((stderr, "\nbl code %2d ", bl_order[rank]));
    send_bits(s, s.bl_tree[bl_order[rank] * 2 + 1]/*.Len*/, 3);
  }
  //Tracev((stderr, "\nbl tree: sent %ld", s->bits_sent));

  send_tree(s, s.dyn_ltree, lcodes - 1); /* literal tree */
  //Tracev((stderr, "\nlit tree: sent %ld", s->bits_sent));

  send_tree(s, s.dyn_dtree, dcodes - 1); /* distance tree */
  //Tracev((stderr, "\ndist tree: sent %ld", s->bits_sent));
}


/* ===========================================================================
 * Check if the data type is TEXT or BINARY, using the following algorithm:
 * - TEXT if the two conditions below are satisfied:
 *    a) There are no non-portable control characters belonging to the
 *       "black list" (0..6, 14..25, 28..31).
 *    b) There is at least one printable character belonging to the
 *       "white list" (9 {TAB}, 10 {LF}, 13 {CR}, 32..255).
 * - BINARY otherwise.
 * - The following partially-portable control characters form a
 *   "gray list" that is ignored in this detection algorithm:
 *   (7 {BEL}, 8 {BS}, 11 {VT}, 12 {FF}, 26 {SUB}, 27 {ESC}).
 * IN assertion: the fields Freq of dyn_ltree are set.
 */
function detect_data_type(s) {
  /* black_mask is the bit mask of black-listed bytes
   * set bits 0..6, 14..25, and 28..31
   * 0xf3ffc07f = binary 11110011111111111100000001111111
   */
  var black_mask = 0xf3ffc07f;
  var n;

  /* Check for non-textual ("black-listed") bytes. */
  for (n = 0; n <= 31; n++, black_mask >>>= 1) {
    if ((black_mask & 1) && (s.dyn_ltree[n * 2]/*.Freq*/ !== 0)) {
      return Z_BINARY;
    }
  }

  /* Check for textual ("white-listed") bytes. */
  if (s.dyn_ltree[9 * 2]/*.Freq*/ !== 0 || s.dyn_ltree[10 * 2]/*.Freq*/ !== 0 ||
      s.dyn_ltree[13 * 2]/*.Freq*/ !== 0) {
    return Z_TEXT;
  }
  for (n = 32; n < LITERALS; n++) {
    if (s.dyn_ltree[n * 2]/*.Freq*/ !== 0) {
      return Z_TEXT;
    }
  }

  /* There are no "black-listed" or "white-listed" bytes:
   * this stream either is empty or has tolerated ("gray-listed") bytes only.
   */
  return Z_BINARY;
}


var static_init_done = false;

/* ===========================================================================
 * Initialize the tree data structures for a new zlib stream.
 */
function _tr_init(s)
{

  if (!static_init_done) {
    tr_static_init();
    static_init_done = true;
  }

  s.l_desc  = new TreeDesc(s.dyn_ltree, static_l_desc);
  s.d_desc  = new TreeDesc(s.dyn_dtree, static_d_desc);
  s.bl_desc = new TreeDesc(s.bl_tree, static_bl_desc);

  s.bi_buf = 0;
  s.bi_valid = 0;

  /* Initialize the first block of the first file: */
  init_block(s);
}


/* ===========================================================================
 * Send a stored block
 */
function _tr_stored_block(s, buf, stored_len, last)
//DeflateState *s;
//charf *buf;       /* input block */
//ulg stored_len;   /* length of input block */
//int last;         /* one if this is the last block for a file */
{
  send_bits(s, (STORED_BLOCK << 1) + (last ? 1 : 0), 3);    /* send block type */
  copy_block(s, buf, stored_len, true); /* with header */
}


/* ===========================================================================
 * Send one empty static block to give enough lookahead for inflate.
 * This takes 10 bits, of which 7 may remain in the bit buffer.
 */
function _tr_align(s) {
  send_bits(s, STATIC_TREES << 1, 3);
  send_code(s, END_BLOCK, static_ltree);
  bi_flush(s);
}


/* ===========================================================================
 * Determine the best encoding for the current block: dynamic trees, static
 * trees or store, and output the encoded block to the zip file.
 */
function _tr_flush_block(s, buf, stored_len, last)
//DeflateState *s;
//charf *buf;       /* input block, or NULL if too old */
//ulg stored_len;   /* length of input block */
//int last;         /* one if this is the last block for a file */
{
  var opt_lenb, static_lenb;  /* opt_len and static_len in bytes */
  var max_blindex = 0;        /* index of last bit length code of non zero freq */

  /* Build the Huffman trees unless a stored block is forced */
  if (s.level > 0) {

    /* Check if the file is binary or text */
    if (s.strm.data_type === Z_UNKNOWN) {
      s.strm.data_type = detect_data_type(s);
    }

    /* Construct the literal and distance trees */
    build_tree(s, s.l_desc);
    // Tracev((stderr, "\nlit data: dyn %ld, stat %ld", s->opt_len,
    //        s->static_len));

    build_tree(s, s.d_desc);
    // Tracev((stderr, "\ndist data: dyn %ld, stat %ld", s->opt_len,
    //        s->static_len));
    /* At this point, opt_len and static_len are the total bit lengths of
     * the compressed block data, excluding the tree representations.
     */

    /* Build the bit length tree for the above two trees, and get the index
     * in bl_order of the last bit length code to send.
     */
    max_blindex = build_bl_tree(s);

    /* Determine the best encoding. Compute the block lengths in bytes. */
    opt_lenb = (s.opt_len + 3 + 7) >>> 3;
    static_lenb = (s.static_len + 3 + 7) >>> 3;

    // Tracev((stderr, "\nopt %lu(%lu) stat %lu(%lu) stored %lu lit %u ",
    //        opt_lenb, s->opt_len, static_lenb, s->static_len, stored_len,
    //        s->last_lit));

    if (static_lenb <= opt_lenb) { opt_lenb = static_lenb; }

  } else {
    // Assert(buf != (char*)0, "lost buf");
    opt_lenb = static_lenb = stored_len + 5; /* force a stored block */
  }

  if ((stored_len + 4 <= opt_lenb) && (buf !== -1)) {
    /* 4: two words for the lengths */

    /* The test buf != NULL is only necessary if LIT_BUFSIZE > WSIZE.
     * Otherwise we can't have processed more than WSIZE input bytes since
     * the last block flush, because compression would have been
     * successful. If LIT_BUFSIZE <= WSIZE, it is never too late to
     * transform a block into a stored block.
     */
    _tr_stored_block(s, buf, stored_len, last);

  } else if (s.strategy === Z_FIXED || static_lenb === opt_lenb) {

    send_bits(s, (STATIC_TREES << 1) + (last ? 1 : 0), 3);
    compress_block(s, static_ltree, static_dtree);

  } else {
    send_bits(s, (DYN_TREES << 1) + (last ? 1 : 0), 3);
    send_all_trees(s, s.l_desc.max_code + 1, s.d_desc.max_code + 1, max_blindex + 1);
    compress_block(s, s.dyn_ltree, s.dyn_dtree);
  }
  // Assert (s->compressed_len == s->bits_sent, "bad compressed size");
  /* The above check is made mod 2^32, for files larger than 512 MB
   * and uLong implemented on 32 bits.
   */
  init_block(s);

  if (last) {
    bi_windup(s);
  }
  // Tracev((stderr,"\ncomprlen %lu(%lu) ", s->compressed_len>>3,
  //       s->compressed_len-7*last));
}

/* ===========================================================================
 * Save the match info and tally the frequency counts. Return true if
 * the current block must be flushed.
 */
function _tr_tally(s, dist, lc)
//    deflate_state *s;
//    unsigned dist;  /* distance of matched string */
//    unsigned lc;    /* match length-MIN_MATCH or unmatched char (if dist==0) */
{
  //var out_length, in_length, dcode;

  s.pending_buf[s.d_buf + s.last_lit * 2]     = (dist >>> 8) & 0xff;
  s.pending_buf[s.d_buf + s.last_lit * 2 + 1] = dist & 0xff;

  s.pending_buf[s.l_buf + s.last_lit] = lc & 0xff;
  s.last_lit++;

  if (dist === 0) {
    /* lc is the unmatched char */
    s.dyn_ltree[lc * 2]/*.Freq*/++;
  } else {
    s.matches++;
    /* Here, lc is the match length - MIN_MATCH */
    dist--;             /* dist = match distance - 1 */
    //Assert((ush)dist < (ush)MAX_DIST(s) &&
    //       (ush)lc <= (ush)(MAX_MATCH-MIN_MATCH) &&
    //       (ush)d_code(dist) < (ush)D_CODES,  "_tr_tally: bad match");

    s.dyn_ltree[(_length_code[lc] + LITERALS + 1) * 2]/*.Freq*/++;
    s.dyn_dtree[d_code(dist) * 2]/*.Freq*/++;
  }

// (!) This block is disabled in zlib defaults,
// don't enable it for binary compatibility

//#ifdef TRUNCATE_BLOCK
//  /* Try to guess if it is profitable to stop the current block here */
//  if ((s.last_lit & 0x1fff) === 0 && s.level > 2) {
//    /* Compute an upper bound for the compressed length */
//    out_length = s.last_lit*8;
//    in_length = s.strstart - s.block_start;
//
//    for (dcode = 0; dcode < D_CODES; dcode++) {
//      out_length += s.dyn_dtree[dcode*2]/*.Freq*/ * (5 + extra_dbits[dcode]);
//    }
//    out_length >>>= 3;
//    //Tracev((stderr,"\nlast_lit %u, in %ld, out ~%ld(%ld%%) ",
//    //       s->last_lit, in_length, out_length,
//    //       100L - out_length*100L/in_length));
//    if (s.matches < (s.last_lit>>1)/*int /2*/ && out_length < (in_length>>1)/*int /2*/) {
//      return true;
//    }
//  }
//#endif

  return (s.last_lit === s.lit_bufsize - 1);
  /* We avoid equality with lit_bufsize because of wraparound at 64K
   * on 16 bit machines and because stored blocks are restricted to
   * 64K-1 bytes.
   */
}

var _tr_init_1  = _tr_init;
var _tr_stored_block_1 = _tr_stored_block;
var _tr_flush_block_1  = _tr_flush_block;
var _tr_tally_1 = _tr_tally;
var _tr_align_1 = _tr_align;

var trees = {
	_tr_init: _tr_init_1,
	_tr_stored_block: _tr_stored_block_1,
	_tr_flush_block: _tr_flush_block_1,
	_tr_tally: _tr_tally_1,
	_tr_align: _tr_align_1
};

// Note: adler32 takes 12% for level 0 and 2% for level 6.
// It isn't worth it to make additional optimizations as in original.
// Small size is preferable.

// (C) 1995-2013 Jean-loup Gailly and Mark Adler
// (C) 2014-2017 Vitaly Puzrin and Andrey Tupitsin
//
// This software is provided 'as-is', without any express or implied
// warranty. In no event will the authors be held liable for any damages
// arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must not
//   claim that you wrote the original software. If you use this software
//   in a product, an acknowledgment in the product documentation would be
//   appreciated but is not required.
// 2. Altered source versions must be plainly marked as such, and must not be
//   misrepresented as being the original software.
// 3. This notice may not be removed or altered from any source distribution.

function adler32(adler, buf, len, pos) {
  var s1 = (adler & 0xffff) |0,
      s2 = ((adler >>> 16) & 0xffff) |0,
      n = 0;

  while (len !== 0) {
    // Set limit ~ twice less than 5552, to keep
    // s2 in 31-bits, because we force signed ints.
    // in other case %= will fail.
    n = len > 2000 ? 2000 : len;
    len -= n;

    do {
      s1 = (s1 + buf[pos++]) |0;
      s2 = (s2 + s1) |0;
    } while (--n);

    s1 %= 65521;
    s2 %= 65521;
  }

  return (s1 | (s2 << 16)) |0;
}


var adler32_1 = adler32;

// Note: we can't get significant speed boost here.
// So write code to minimize size - no pregenerated tables
// and array tools dependencies.

// (C) 1995-2013 Jean-loup Gailly and Mark Adler
// (C) 2014-2017 Vitaly Puzrin and Andrey Tupitsin
//
// This software is provided 'as-is', without any express or implied
// warranty. In no event will the authors be held liable for any damages
// arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must not
//   claim that you wrote the original software. If you use this software
//   in a product, an acknowledgment in the product documentation would be
//   appreciated but is not required.
// 2. Altered source versions must be plainly marked as such, and must not be
//   misrepresented as being the original software.
// 3. This notice may not be removed or altered from any source distribution.

// Use ordinary array, since untyped makes no boost here
function makeTable$1() {
  var c, table = [];

  for (var n = 0; n < 256; n++) {
    c = n;
    for (var k = 0; k < 8; k++) {
      c = ((c & 1) ? (0xEDB88320 ^ (c >>> 1)) : (c >>> 1));
    }
    table[n] = c;
  }

  return table;
}

// Create table on load. Just 255 signed longs. Not a problem.
var crcTable$1 = makeTable$1();


function crc32$1(crc, buf, len, pos) {
  var t = crcTable$1,
      end = pos + len;

  crc ^= -1;

  for (var i = pos; i < end; i++) {
    crc = (crc >>> 8) ^ t[(crc ^ buf[i]) & 0xFF];
  }

  return (crc ^ (-1)); // >>> 0;
}


var crc32_1$1 = crc32$1;

// (C) 1995-2013 Jean-loup Gailly and Mark Adler
// (C) 2014-2017 Vitaly Puzrin and Andrey Tupitsin
//
// This software is provided 'as-is', without any express or implied
// warranty. In no event will the authors be held liable for any damages
// arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must not
//   claim that you wrote the original software. If you use this software
//   in a product, an acknowledgment in the product documentation would be
//   appreciated but is not required.
// 2. Altered source versions must be plainly marked as such, and must not be
//   misrepresented as being the original software.
// 3. This notice may not be removed or altered from any source distribution.

var messages = {
  2:      'need dictionary',     /* Z_NEED_DICT       2  */
  1:      'stream end',          /* Z_STREAM_END      1  */
  0:      '',                    /* Z_OK              0  */
  '-1':   'file error',          /* Z_ERRNO         (-1) */
  '-2':   'stream error',        /* Z_STREAM_ERROR  (-2) */
  '-3':   'data error',          /* Z_DATA_ERROR    (-3) */
  '-4':   'insufficient memory', /* Z_MEM_ERROR     (-4) */
  '-5':   'buffer error',        /* Z_BUF_ERROR     (-5) */
  '-6':   'incompatible version' /* Z_VERSION_ERROR (-6) */
};

// (C) 1995-2013 Jean-loup Gailly and Mark Adler
// (C) 2014-2017 Vitaly Puzrin and Andrey Tupitsin
//
// This software is provided 'as-is', without any express or implied
// warranty. In no event will the authors be held liable for any damages
// arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must not
//   claim that you wrote the original software. If you use this software
//   in a product, an acknowledgment in the product documentation would be
//   appreciated but is not required.
// 2. Altered source versions must be plainly marked as such, and must not be
//   misrepresented as being the original software.
// 3. This notice may not be removed or altered from any source distribution.







/* Public constants ==========================================================*/
/* ===========================================================================*/


/* Allowed flush values; see deflate() and inflate() below for details */
var Z_NO_FLUSH      = 0;
var Z_PARTIAL_FLUSH = 1;
//var Z_SYNC_FLUSH    = 2;
var Z_FULL_FLUSH    = 3;
var Z_FINISH        = 4;
var Z_BLOCK         = 5;
//var Z_TREES         = 6;


/* Return codes for the compression/decompression functions. Negative values
 * are errors, positive values are used for special but normal events.
 */
var Z_OK            = 0;
var Z_STREAM_END    = 1;
//var Z_NEED_DICT     = 2;
//var Z_ERRNO         = -1;
var Z_STREAM_ERROR  = -2;
var Z_DATA_ERROR    = -3;
//var Z_MEM_ERROR     = -4;
var Z_BUF_ERROR     = -5;
//var Z_VERSION_ERROR = -6;


/* compression levels */
//var Z_NO_COMPRESSION      = 0;
//var Z_BEST_SPEED          = 1;
//var Z_BEST_COMPRESSION    = 9;
var Z_DEFAULT_COMPRESSION = -1;


var Z_FILTERED            = 1;
var Z_HUFFMAN_ONLY        = 2;
var Z_RLE                 = 3;
var Z_FIXED$1               = 4;
var Z_DEFAULT_STRATEGY    = 0;

/* Possible values of the data_type field (though see inflate()) */
//var Z_BINARY              = 0;
//var Z_TEXT                = 1;
//var Z_ASCII               = 1; // = Z_TEXT
var Z_UNKNOWN$1             = 2;


/* The deflate compression method */
var Z_DEFLATED  = 8;

/*============================================================================*/


var MAX_MEM_LEVEL = 9;
/* Maximum value for memLevel in deflateInit2 */
var MAX_WBITS = 15;
/* 32K LZ77 window */
var DEF_MEM_LEVEL = 8;


var LENGTH_CODES$1  = 29;
/* number of length codes, not counting the special END_BLOCK code */
var LITERALS$1      = 256;
/* number of literal bytes 0..255 */
var L_CODES$1       = LITERALS$1 + 1 + LENGTH_CODES$1;
/* number of Literal or Length codes, including the END_BLOCK code */
var D_CODES$1       = 30;
/* number of distance codes */
var BL_CODES$1      = 19;
/* number of codes used to transfer the bit lengths */
var HEAP_SIZE$1     = 2 * L_CODES$1 + 1;
/* maximum heap size */
var MAX_BITS$1  = 15;
/* All codes must not exceed MAX_BITS bits */

var MIN_MATCH$1 = 3;
var MAX_MATCH$1 = 258;
var MIN_LOOKAHEAD = (MAX_MATCH$1 + MIN_MATCH$1 + 1);

var PRESET_DICT = 0x20;

var INIT_STATE = 42;
var EXTRA_STATE = 69;
var NAME_STATE = 73;
var COMMENT_STATE = 91;
var HCRC_STATE = 103;
var BUSY_STATE = 113;
var FINISH_STATE = 666;

var BS_NEED_MORE      = 1; /* block not completed, need more input or more output */
var BS_BLOCK_DONE     = 2; /* block flush performed */
var BS_FINISH_STARTED = 3; /* finish started, need only more output at next deflate */
var BS_FINISH_DONE    = 4; /* finish done, accept no more input or output */

var OS_CODE = 0x03; // Unix :) . Don't detect, use this default.

function err(strm, errorCode) {
  strm.msg = messages[errorCode];
  return errorCode;
}

function rank(f) {
  return ((f) << 1) - ((f) > 4 ? 9 : 0);
}

function zero$1(buf) { var len = buf.length; while (--len >= 0) { buf[len] = 0; } }


/* =========================================================================
 * Flush as much pending output as possible. All deflate() output goes
 * through this function so some applications may wish to modify it
 * to avoid allocating a large strm->output buffer and copying into it.
 * (See also read_buf()).
 */
function flush_pending(strm) {
  var s = strm.state;

  //_tr_flush_bits(s);
  var len = s.pending;
  if (len > strm.avail_out) {
    len = strm.avail_out;
  }
  if (len === 0) { return; }

  common.arraySet(strm.output, s.pending_buf, s.pending_out, len, strm.next_out);
  strm.next_out += len;
  s.pending_out += len;
  strm.total_out += len;
  strm.avail_out -= len;
  s.pending -= len;
  if (s.pending === 0) {
    s.pending_out = 0;
  }
}


function flush_block_only(s, last) {
  trees._tr_flush_block(s, (s.block_start >= 0 ? s.block_start : -1), s.strstart - s.block_start, last);
  s.block_start = s.strstart;
  flush_pending(s.strm);
}


function put_byte(s, b) {
  s.pending_buf[s.pending++] = b;
}


/* =========================================================================
 * Put a short in the pending buffer. The 16-bit value is put in MSB order.
 * IN assertion: the stream state is correct and there is enough room in
 * pending_buf.
 */
function putShortMSB(s, b) {
//  put_byte(s, (Byte)(b >> 8));
//  put_byte(s, (Byte)(b & 0xff));
  s.pending_buf[s.pending++] = (b >>> 8) & 0xff;
  s.pending_buf[s.pending++] = b & 0xff;
}


/* ===========================================================================
 * Read a new buffer from the current input stream, update the adler32
 * and total number of bytes read.  All deflate() input goes through
 * this function so some applications may wish to modify it to avoid
 * allocating a large strm->input buffer and copying from it.
 * (See also flush_pending()).
 */
function read_buf(strm, buf, start, size) {
  var len = strm.avail_in;

  if (len > size) { len = size; }
  if (len === 0) { return 0; }

  strm.avail_in -= len;

  // zmemcpy(buf, strm->next_in, len);
  common.arraySet(buf, strm.input, strm.next_in, len, start);
  if (strm.state.wrap === 1) {
    strm.adler = adler32_1(strm.adler, buf, len, start);
  }

  else if (strm.state.wrap === 2) {
    strm.adler = crc32_1$1(strm.adler, buf, len, start);
  }

  strm.next_in += len;
  strm.total_in += len;

  return len;
}


/* ===========================================================================
 * Set match_start to the longest match starting at the given string and
 * return its length. Matches shorter or equal to prev_length are discarded,
 * in which case the result is equal to prev_length and match_start is
 * garbage.
 * IN assertions: cur_match is the head of the hash chain for the current
 *   string (strstart) and its distance is <= MAX_DIST, and prev_length >= 1
 * OUT assertion: the match length is not greater than s->lookahead.
 */
function longest_match(s, cur_match) {
  var chain_length = s.max_chain_length;      /* max hash chain length */
  var scan = s.strstart; /* current string */
  var match;                       /* matched string */
  var len;                           /* length of current match */
  var best_len = s.prev_length;              /* best match length so far */
  var nice_match = s.nice_match;             /* stop if match long enough */
  var limit = (s.strstart > (s.w_size - MIN_LOOKAHEAD)) ?
      s.strstart - (s.w_size - MIN_LOOKAHEAD) : 0/*NIL*/;

  var _win = s.window; // shortcut

  var wmask = s.w_mask;
  var prev  = s.prev;

  /* Stop when cur_match becomes <= limit. To simplify the code,
   * we prevent matches with the string of window index 0.
   */

  var strend = s.strstart + MAX_MATCH$1;
  var scan_end1  = _win[scan + best_len - 1];
  var scan_end   = _win[scan + best_len];

  /* The code is optimized for HASH_BITS >= 8 and MAX_MATCH-2 multiple of 16.
   * It is easy to get rid of this optimization if necessary.
   */
  // Assert(s->hash_bits >= 8 && MAX_MATCH == 258, "Code too clever");

  /* Do not waste too much time if we already have a good match: */
  if (s.prev_length >= s.good_match) {
    chain_length >>= 2;
  }
  /* Do not look for matches beyond the end of the input. This is necessary
   * to make deflate deterministic.
   */
  if (nice_match > s.lookahead) { nice_match = s.lookahead; }

  // Assert((ulg)s->strstart <= s->window_size-MIN_LOOKAHEAD, "need lookahead");

  do {
    // Assert(cur_match < s->strstart, "no future");
    match = cur_match;

    /* Skip to next match if the match length cannot increase
     * or if the match length is less than 2.  Note that the checks below
     * for insufficient lookahead only occur occasionally for performance
     * reasons.  Therefore uninitialized memory will be accessed, and
     * conditional jumps will be made that depend on those values.
     * However the length of the match is limited to the lookahead, so
     * the output of deflate is not affected by the uninitialized values.
     */

    if (_win[match + best_len]     !== scan_end  ||
        _win[match + best_len - 1] !== scan_end1 ||
        _win[match]                !== _win[scan] ||
        _win[++match]              !== _win[scan + 1]) {
      continue;
    }

    /* The check at best_len-1 can be removed because it will be made
     * again later. (This heuristic is not always a win.)
     * It is not necessary to compare scan[2] and match[2] since they
     * are always equal when the other bytes match, given that
     * the hash keys are equal and that HASH_BITS >= 8.
     */
    scan += 2;
    match++;
    // Assert(*scan == *match, "match[2]?");

    /* We check for insufficient lookahead only every 8th comparison;
     * the 256th check will be made at strstart+258.
     */
    do {
      /*jshint noempty:false*/
    } while (_win[++scan] === _win[++match] && _win[++scan] === _win[++match] &&
             _win[++scan] === _win[++match] && _win[++scan] === _win[++match] &&
             _win[++scan] === _win[++match] && _win[++scan] === _win[++match] &&
             _win[++scan] === _win[++match] && _win[++scan] === _win[++match] &&
             scan < strend);

    // Assert(scan <= s->window+(unsigned)(s->window_size-1), "wild scan");

    len = MAX_MATCH$1 - (strend - scan);
    scan = strend - MAX_MATCH$1;

    if (len > best_len) {
      s.match_start = cur_match;
      best_len = len;
      if (len >= nice_match) {
        break;
      }
      scan_end1  = _win[scan + best_len - 1];
      scan_end   = _win[scan + best_len];
    }
  } while ((cur_match = prev[cur_match & wmask]) > limit && --chain_length !== 0);

  if (best_len <= s.lookahead) {
    return best_len;
  }
  return s.lookahead;
}


/* ===========================================================================
 * Fill the window when the lookahead becomes insufficient.
 * Updates strstart and lookahead.
 *
 * IN assertion: lookahead < MIN_LOOKAHEAD
 * OUT assertions: strstart <= window_size-MIN_LOOKAHEAD
 *    At least one byte has been read, or avail_in == 0; reads are
 *    performed for at least two bytes (required for the zip translate_eol
 *    option -- not supported here).
 */
function fill_window(s) {
  var _w_size = s.w_size;
  var p, n, m, more, str;

  //Assert(s->lookahead < MIN_LOOKAHEAD, "already enough lookahead");

  do {
    more = s.window_size - s.lookahead - s.strstart;

    // JS ints have 32 bit, block below not needed
    /* Deal with !@#$% 64K limit: */
    //if (sizeof(int) <= 2) {
    //    if (more == 0 && s->strstart == 0 && s->lookahead == 0) {
    //        more = wsize;
    //
    //  } else if (more == (unsigned)(-1)) {
    //        /* Very unlikely, but possible on 16 bit machine if
    //         * strstart == 0 && lookahead == 1 (input done a byte at time)
    //         */
    //        more--;
    //    }
    //}


    /* If the window is almost full and there is insufficient lookahead,
     * move the upper half to the lower one to make room in the upper half.
     */
    if (s.strstart >= _w_size + (_w_size - MIN_LOOKAHEAD)) {

      common.arraySet(s.window, s.window, _w_size, _w_size, 0);
      s.match_start -= _w_size;
      s.strstart -= _w_size;
      /* we now have strstart >= MAX_DIST */
      s.block_start -= _w_size;

      /* Slide the hash table (could be avoided with 32 bit values
       at the expense of memory usage). We slide even when level == 0
       to keep the hash table consistent if we switch back to level > 0
       later. (Using level 0 permanently is not an optimal usage of
       zlib, so we don't care about this pathological case.)
       */

      n = s.hash_size;
      p = n;
      do {
        m = s.head[--p];
        s.head[p] = (m >= _w_size ? m - _w_size : 0);
      } while (--n);

      n = _w_size;
      p = n;
      do {
        m = s.prev[--p];
        s.prev[p] = (m >= _w_size ? m - _w_size : 0);
        /* If n is not on any hash chain, prev[n] is garbage but
         * its value will never be used.
         */
      } while (--n);

      more += _w_size;
    }
    if (s.strm.avail_in === 0) {
      break;
    }

    /* If there was no sliding:
     *    strstart <= WSIZE+MAX_DIST-1 && lookahead <= MIN_LOOKAHEAD - 1 &&
     *    more == window_size - lookahead - strstart
     * => more >= window_size - (MIN_LOOKAHEAD-1 + WSIZE + MAX_DIST-1)
     * => more >= window_size - 2*WSIZE + 2
     * In the BIG_MEM or MMAP case (not yet supported),
     *   window_size == input_size + MIN_LOOKAHEAD  &&
     *   strstart + s->lookahead <= input_size => more >= MIN_LOOKAHEAD.
     * Otherwise, window_size == 2*WSIZE so more >= 2.
     * If there was sliding, more >= WSIZE. So in all cases, more >= 2.
     */
    //Assert(more >= 2, "more < 2");
    n = read_buf(s.strm, s.window, s.strstart + s.lookahead, more);
    s.lookahead += n;

    /* Initialize the hash value now that we have some input: */
    if (s.lookahead + s.insert >= MIN_MATCH$1) {
      str = s.strstart - s.insert;
      s.ins_h = s.window[str];

      /* UPDATE_HASH(s, s->ins_h, s->window[str + 1]); */
      s.ins_h = ((s.ins_h << s.hash_shift) ^ s.window[str + 1]) & s.hash_mask;
//#if MIN_MATCH != 3
//        Call update_hash() MIN_MATCH-3 more times
//#endif
      while (s.insert) {
        /* UPDATE_HASH(s, s->ins_h, s->window[str + MIN_MATCH-1]); */
        s.ins_h = ((s.ins_h << s.hash_shift) ^ s.window[str + MIN_MATCH$1 - 1]) & s.hash_mask;

        s.prev[str & s.w_mask] = s.head[s.ins_h];
        s.head[s.ins_h] = str;
        str++;
        s.insert--;
        if (s.lookahead + s.insert < MIN_MATCH$1) {
          break;
        }
      }
    }
    /* If the whole input has less than MIN_MATCH bytes, ins_h is garbage,
     * but this is not important since only literal bytes will be emitted.
     */

  } while (s.lookahead < MIN_LOOKAHEAD && s.strm.avail_in !== 0);

  /* If the WIN_INIT bytes after the end of the current data have never been
   * written, then zero those bytes in order to avoid memory check reports of
   * the use of uninitialized (or uninitialised as Julian writes) bytes by
   * the longest match routines.  Update the high water mark for the next
   * time through here.  WIN_INIT is set to MAX_MATCH since the longest match
   * routines allow scanning to strstart + MAX_MATCH, ignoring lookahead.
   */
//  if (s.high_water < s.window_size) {
//    var curr = s.strstart + s.lookahead;
//    var init = 0;
//
//    if (s.high_water < curr) {
//      /* Previous high water mark below current data -- zero WIN_INIT
//       * bytes or up to end of window, whichever is less.
//       */
//      init = s.window_size - curr;
//      if (init > WIN_INIT)
//        init = WIN_INIT;
//      zmemzero(s->window + curr, (unsigned)init);
//      s->high_water = curr + init;
//    }
//    else if (s->high_water < (ulg)curr + WIN_INIT) {
//      /* High water mark at or above current data, but below current data
//       * plus WIN_INIT -- zero out to current data plus WIN_INIT, or up
//       * to end of window, whichever is less.
//       */
//      init = (ulg)curr + WIN_INIT - s->high_water;
//      if (init > s->window_size - s->high_water)
//        init = s->window_size - s->high_water;
//      zmemzero(s->window + s->high_water, (unsigned)init);
//      s->high_water += init;
//    }
//  }
//
//  Assert((ulg)s->strstart <= s->window_size - MIN_LOOKAHEAD,
//    "not enough room for search");
}

/* ===========================================================================
 * Copy without compression as much as possible from the input stream, return
 * the current block state.
 * This function does not insert new strings in the dictionary since
 * uncompressible data is probably not useful. This function is used
 * only for the level=0 compression option.
 * NOTE: this function should be optimized to avoid extra copying from
 * window to pending_buf.
 */
function deflate_stored(s, flush) {
  /* Stored blocks are limited to 0xffff bytes, pending_buf is limited
   * to pending_buf_size, and each stored block has a 5 byte header:
   */
  var max_block_size = 0xffff;

  if (max_block_size > s.pending_buf_size - 5) {
    max_block_size = s.pending_buf_size - 5;
  }

  /* Copy as much as possible from input to output: */
  for (;;) {
    /* Fill the window as much as possible: */
    if (s.lookahead <= 1) {

      //Assert(s->strstart < s->w_size+MAX_DIST(s) ||
      //  s->block_start >= (long)s->w_size, "slide too late");
//      if (!(s.strstart < s.w_size + (s.w_size - MIN_LOOKAHEAD) ||
//        s.block_start >= s.w_size)) {
//        throw  new Error("slide too late");
//      }

      fill_window(s);
      if (s.lookahead === 0 && flush === Z_NO_FLUSH) {
        return BS_NEED_MORE;
      }

      if (s.lookahead === 0) {
        break;
      }
      /* flush the current block */
    }
    //Assert(s->block_start >= 0L, "block gone");
//    if (s.block_start < 0) throw new Error("block gone");

    s.strstart += s.lookahead;
    s.lookahead = 0;

    /* Emit a stored block if pending_buf will be full: */
    var max_start = s.block_start + max_block_size;

    if (s.strstart === 0 || s.strstart >= max_start) {
      /* strstart == 0 is possible when wraparound on 16-bit machine */
      s.lookahead = s.strstart - max_start;
      s.strstart = max_start;
      /*** FLUSH_BLOCK(s, 0); ***/
      flush_block_only(s, false);
      if (s.strm.avail_out === 0) {
        return BS_NEED_MORE;
      }
      /***/


    }
    /* Flush if we may have to slide, otherwise block_start may become
     * negative and the data will be gone:
     */
    if (s.strstart - s.block_start >= (s.w_size - MIN_LOOKAHEAD)) {
      /*** FLUSH_BLOCK(s, 0); ***/
      flush_block_only(s, false);
      if (s.strm.avail_out === 0) {
        return BS_NEED_MORE;
      }
      /***/
    }
  }

  s.insert = 0;

  if (flush === Z_FINISH) {
    /*** FLUSH_BLOCK(s, 1); ***/
    flush_block_only(s, true);
    if (s.strm.avail_out === 0) {
      return BS_FINISH_STARTED;
    }
    /***/
    return BS_FINISH_DONE;
  }

  if (s.strstart > s.block_start) {
    /*** FLUSH_BLOCK(s, 0); ***/
    flush_block_only(s, false);
    if (s.strm.avail_out === 0) {
      return BS_NEED_MORE;
    }
    /***/
  }

  return BS_NEED_MORE;
}

/* ===========================================================================
 * Compress as much as possible from the input stream, return the current
 * block state.
 * This function does not perform lazy evaluation of matches and inserts
 * new strings in the dictionary only for unmatched strings or for short
 * matches. It is used only for the fast compression options.
 */
function deflate_fast(s, flush) {
  var hash_head;        /* head of the hash chain */
  var bflush;           /* set if current block must be flushed */

  for (;;) {
    /* Make sure that we always have enough lookahead, except
     * at the end of the input file. We need MAX_MATCH bytes
     * for the next match, plus MIN_MATCH bytes to insert the
     * string following the next match.
     */
    if (s.lookahead < MIN_LOOKAHEAD) {
      fill_window(s);
      if (s.lookahead < MIN_LOOKAHEAD && flush === Z_NO_FLUSH) {
        return BS_NEED_MORE;
      }
      if (s.lookahead === 0) {
        break; /* flush the current block */
      }
    }

    /* Insert the string window[strstart .. strstart+2] in the
     * dictionary, and set hash_head to the head of the hash chain:
     */
    hash_head = 0/*NIL*/;
    if (s.lookahead >= MIN_MATCH$1) {
      /*** INSERT_STRING(s, s.strstart, hash_head); ***/
      s.ins_h = ((s.ins_h << s.hash_shift) ^ s.window[s.strstart + MIN_MATCH$1 - 1]) & s.hash_mask;
      hash_head = s.prev[s.strstart & s.w_mask] = s.head[s.ins_h];
      s.head[s.ins_h] = s.strstart;
      /***/
    }

    /* Find the longest match, discarding those <= prev_length.
     * At this point we have always match_length < MIN_MATCH
     */
    if (hash_head !== 0/*NIL*/ && ((s.strstart - hash_head) <= (s.w_size - MIN_LOOKAHEAD))) {
      /* To simplify the code, we prevent matches with the string
       * of window index 0 (in particular we have to avoid a match
       * of the string with itself at the start of the input file).
       */
      s.match_length = longest_match(s, hash_head);
      /* longest_match() sets match_start */
    }
    if (s.match_length >= MIN_MATCH$1) {
      // check_match(s, s.strstart, s.match_start, s.match_length); // for debug only

      /*** _tr_tally_dist(s, s.strstart - s.match_start,
                     s.match_length - MIN_MATCH, bflush); ***/
      bflush = trees._tr_tally(s, s.strstart - s.match_start, s.match_length - MIN_MATCH$1);

      s.lookahead -= s.match_length;

      /* Insert new strings in the hash table only if the match length
       * is not too large. This saves time but degrades compression.
       */
      if (s.match_length <= s.max_lazy_match/*max_insert_length*/ && s.lookahead >= MIN_MATCH$1) {
        s.match_length--; /* string at strstart already in table */
        do {
          s.strstart++;
          /*** INSERT_STRING(s, s.strstart, hash_head); ***/
          s.ins_h = ((s.ins_h << s.hash_shift) ^ s.window[s.strstart + MIN_MATCH$1 - 1]) & s.hash_mask;
          hash_head = s.prev[s.strstart & s.w_mask] = s.head[s.ins_h];
          s.head[s.ins_h] = s.strstart;
          /***/
          /* strstart never exceeds WSIZE-MAX_MATCH, so there are
           * always MIN_MATCH bytes ahead.
           */
        } while (--s.match_length !== 0);
        s.strstart++;
      } else
      {
        s.strstart += s.match_length;
        s.match_length = 0;
        s.ins_h = s.window[s.strstart];
        /* UPDATE_HASH(s, s.ins_h, s.window[s.strstart+1]); */
        s.ins_h = ((s.ins_h << s.hash_shift) ^ s.window[s.strstart + 1]) & s.hash_mask;

//#if MIN_MATCH != 3
//                Call UPDATE_HASH() MIN_MATCH-3 more times
//#endif
        /* If lookahead < MIN_MATCH, ins_h is garbage, but it does not
         * matter since it will be recomputed at next deflate call.
         */
      }
    } else {
      /* No match, output a literal byte */
      //Tracevv((stderr,"%c", s.window[s.strstart]));
      /*** _tr_tally_lit(s, s.window[s.strstart], bflush); ***/
      bflush = trees._tr_tally(s, 0, s.window[s.strstart]);

      s.lookahead--;
      s.strstart++;
    }
    if (bflush) {
      /*** FLUSH_BLOCK(s, 0); ***/
      flush_block_only(s, false);
      if (s.strm.avail_out === 0) {
        return BS_NEED_MORE;
      }
      /***/
    }
  }
  s.insert = ((s.strstart < (MIN_MATCH$1 - 1)) ? s.strstart : MIN_MATCH$1 - 1);
  if (flush === Z_FINISH) {
    /*** FLUSH_BLOCK(s, 1); ***/
    flush_block_only(s, true);
    if (s.strm.avail_out === 0) {
      return BS_FINISH_STARTED;
    }
    /***/
    return BS_FINISH_DONE;
  }
  if (s.last_lit) {
    /*** FLUSH_BLOCK(s, 0); ***/
    flush_block_only(s, false);
    if (s.strm.avail_out === 0) {
      return BS_NEED_MORE;
    }
    /***/
  }
  return BS_BLOCK_DONE;
}

/* ===========================================================================
 * Same as above, but achieves better compression. We use a lazy
 * evaluation for matches: a match is finally adopted only if there is
 * no better match at the next window position.
 */
function deflate_slow(s, flush) {
  var hash_head;          /* head of hash chain */
  var bflush;              /* set if current block must be flushed */

  var max_insert;

  /* Process the input block. */
  for (;;) {
    /* Make sure that we always have enough lookahead, except
     * at the end of the input file. We need MAX_MATCH bytes
     * for the next match, plus MIN_MATCH bytes to insert the
     * string following the next match.
     */
    if (s.lookahead < MIN_LOOKAHEAD) {
      fill_window(s);
      if (s.lookahead < MIN_LOOKAHEAD && flush === Z_NO_FLUSH) {
        return BS_NEED_MORE;
      }
      if (s.lookahead === 0) { break; } /* flush the current block */
    }

    /* Insert the string window[strstart .. strstart+2] in the
     * dictionary, and set hash_head to the head of the hash chain:
     */
    hash_head = 0/*NIL*/;
    if (s.lookahead >= MIN_MATCH$1) {
      /*** INSERT_STRING(s, s.strstart, hash_head); ***/
      s.ins_h = ((s.ins_h << s.hash_shift) ^ s.window[s.strstart + MIN_MATCH$1 - 1]) & s.hash_mask;
      hash_head = s.prev[s.strstart & s.w_mask] = s.head[s.ins_h];
      s.head[s.ins_h] = s.strstart;
      /***/
    }

    /* Find the longest match, discarding those <= prev_length.
     */
    s.prev_length = s.match_length;
    s.prev_match = s.match_start;
    s.match_length = MIN_MATCH$1 - 1;

    if (hash_head !== 0/*NIL*/ && s.prev_length < s.max_lazy_match &&
        s.strstart - hash_head <= (s.w_size - MIN_LOOKAHEAD)/*MAX_DIST(s)*/) {
      /* To simplify the code, we prevent matches with the string
       * of window index 0 (in particular we have to avoid a match
       * of the string with itself at the start of the input file).
       */
      s.match_length = longest_match(s, hash_head);
      /* longest_match() sets match_start */

      if (s.match_length <= 5 &&
         (s.strategy === Z_FILTERED || (s.match_length === MIN_MATCH$1 && s.strstart - s.match_start > 4096/*TOO_FAR*/))) {

        /* If prev_match is also MIN_MATCH, match_start is garbage
         * but we will ignore the current match anyway.
         */
        s.match_length = MIN_MATCH$1 - 1;
      }
    }
    /* If there was a match at the previous step and the current
     * match is not better, output the previous match:
     */
    if (s.prev_length >= MIN_MATCH$1 && s.match_length <= s.prev_length) {
      max_insert = s.strstart + s.lookahead - MIN_MATCH$1;
      /* Do not insert strings in hash table beyond this. */

      //check_match(s, s.strstart-1, s.prev_match, s.prev_length);

      /***_tr_tally_dist(s, s.strstart - 1 - s.prev_match,
                     s.prev_length - MIN_MATCH, bflush);***/
      bflush = trees._tr_tally(s, s.strstart - 1 - s.prev_match, s.prev_length - MIN_MATCH$1);
      /* Insert in hash table all strings up to the end of the match.
       * strstart-1 and strstart are already inserted. If there is not
       * enough lookahead, the last two strings are not inserted in
       * the hash table.
       */
      s.lookahead -= s.prev_length - 1;
      s.prev_length -= 2;
      do {
        if (++s.strstart <= max_insert) {
          /*** INSERT_STRING(s, s.strstart, hash_head); ***/
          s.ins_h = ((s.ins_h << s.hash_shift) ^ s.window[s.strstart + MIN_MATCH$1 - 1]) & s.hash_mask;
          hash_head = s.prev[s.strstart & s.w_mask] = s.head[s.ins_h];
          s.head[s.ins_h] = s.strstart;
          /***/
        }
      } while (--s.prev_length !== 0);
      s.match_available = 0;
      s.match_length = MIN_MATCH$1 - 1;
      s.strstart++;

      if (bflush) {
        /*** FLUSH_BLOCK(s, 0); ***/
        flush_block_only(s, false);
        if (s.strm.avail_out === 0) {
          return BS_NEED_MORE;
        }
        /***/
      }

    } else if (s.match_available) {
      /* If there was no match at the previous position, output a
       * single literal. If there was a match but the current match
       * is longer, truncate the previous match to a single literal.
       */
      //Tracevv((stderr,"%c", s->window[s->strstart-1]));
      /*** _tr_tally_lit(s, s.window[s.strstart-1], bflush); ***/
      bflush = trees._tr_tally(s, 0, s.window[s.strstart - 1]);

      if (bflush) {
        /*** FLUSH_BLOCK_ONLY(s, 0) ***/
        flush_block_only(s, false);
        /***/
      }
      s.strstart++;
      s.lookahead--;
      if (s.strm.avail_out === 0) {
        return BS_NEED_MORE;
      }
    } else {
      /* There is no previous match to compare with, wait for
       * the next step to decide.
       */
      s.match_available = 1;
      s.strstart++;
      s.lookahead--;
    }
  }
  //Assert (flush != Z_NO_FLUSH, "no flush?");
  if (s.match_available) {
    //Tracevv((stderr,"%c", s->window[s->strstart-1]));
    /*** _tr_tally_lit(s, s.window[s.strstart-1], bflush); ***/
    bflush = trees._tr_tally(s, 0, s.window[s.strstart - 1]);

    s.match_available = 0;
  }
  s.insert = s.strstart < MIN_MATCH$1 - 1 ? s.strstart : MIN_MATCH$1 - 1;
  if (flush === Z_FINISH) {
    /*** FLUSH_BLOCK(s, 1); ***/
    flush_block_only(s, true);
    if (s.strm.avail_out === 0) {
      return BS_FINISH_STARTED;
    }
    /***/
    return BS_FINISH_DONE;
  }
  if (s.last_lit) {
    /*** FLUSH_BLOCK(s, 0); ***/
    flush_block_only(s, false);
    if (s.strm.avail_out === 0) {
      return BS_NEED_MORE;
    }
    /***/
  }

  return BS_BLOCK_DONE;
}


/* ===========================================================================
 * For Z_RLE, simply look for runs of bytes, generate matches only of distance
 * one.  Do not maintain a hash table.  (It will be regenerated if this run of
 * deflate switches away from Z_RLE.)
 */
function deflate_rle(s, flush) {
  var bflush;            /* set if current block must be flushed */
  var prev;              /* byte at distance one to match */
  var scan, strend;      /* scan goes up to strend for length of run */

  var _win = s.window;

  for (;;) {
    /* Make sure that we always have enough lookahead, except
     * at the end of the input file. We need MAX_MATCH bytes
     * for the longest run, plus one for the unrolled loop.
     */
    if (s.lookahead <= MAX_MATCH$1) {
      fill_window(s);
      if (s.lookahead <= MAX_MATCH$1 && flush === Z_NO_FLUSH) {
        return BS_NEED_MORE;
      }
      if (s.lookahead === 0) { break; } /* flush the current block */
    }

    /* See how many times the previous byte repeats */
    s.match_length = 0;
    if (s.lookahead >= MIN_MATCH$1 && s.strstart > 0) {
      scan = s.strstart - 1;
      prev = _win[scan];
      if (prev === _win[++scan] && prev === _win[++scan] && prev === _win[++scan]) {
        strend = s.strstart + MAX_MATCH$1;
        do {
          /*jshint noempty:false*/
        } while (prev === _win[++scan] && prev === _win[++scan] &&
                 prev === _win[++scan] && prev === _win[++scan] &&
                 prev === _win[++scan] && prev === _win[++scan] &&
                 prev === _win[++scan] && prev === _win[++scan] &&
                 scan < strend);
        s.match_length = MAX_MATCH$1 - (strend - scan);
        if (s.match_length > s.lookahead) {
          s.match_length = s.lookahead;
        }
      }
      //Assert(scan <= s->window+(uInt)(s->window_size-1), "wild scan");
    }

    /* Emit match if have run of MIN_MATCH or longer, else emit literal */
    if (s.match_length >= MIN_MATCH$1) {
      //check_match(s, s.strstart, s.strstart - 1, s.match_length);

      /*** _tr_tally_dist(s, 1, s.match_length - MIN_MATCH, bflush); ***/
      bflush = trees._tr_tally(s, 1, s.match_length - MIN_MATCH$1);

      s.lookahead -= s.match_length;
      s.strstart += s.match_length;
      s.match_length = 0;
    } else {
      /* No match, output a literal byte */
      //Tracevv((stderr,"%c", s->window[s->strstart]));
      /*** _tr_tally_lit(s, s.window[s.strstart], bflush); ***/
      bflush = trees._tr_tally(s, 0, s.window[s.strstart]);

      s.lookahead--;
      s.strstart++;
    }
    if (bflush) {
      /*** FLUSH_BLOCK(s, 0); ***/
      flush_block_only(s, false);
      if (s.strm.avail_out === 0) {
        return BS_NEED_MORE;
      }
      /***/
    }
  }
  s.insert = 0;
  if (flush === Z_FINISH) {
    /*** FLUSH_BLOCK(s, 1); ***/
    flush_block_only(s, true);
    if (s.strm.avail_out === 0) {
      return BS_FINISH_STARTED;
    }
    /***/
    return BS_FINISH_DONE;
  }
  if (s.last_lit) {
    /*** FLUSH_BLOCK(s, 0); ***/
    flush_block_only(s, false);
    if (s.strm.avail_out === 0) {
      return BS_NEED_MORE;
    }
    /***/
  }
  return BS_BLOCK_DONE;
}

/* ===========================================================================
 * For Z_HUFFMAN_ONLY, do not look for matches.  Do not maintain a hash table.
 * (It will be regenerated if this run of deflate switches away from Huffman.)
 */
function deflate_huff(s, flush) {
  var bflush;             /* set if current block must be flushed */

  for (;;) {
    /* Make sure that we have a literal to write. */
    if (s.lookahead === 0) {
      fill_window(s);
      if (s.lookahead === 0) {
        if (flush === Z_NO_FLUSH) {
          return BS_NEED_MORE;
        }
        break;      /* flush the current block */
      }
    }

    /* Output a literal byte */
    s.match_length = 0;
    //Tracevv((stderr,"%c", s->window[s->strstart]));
    /*** _tr_tally_lit(s, s.window[s.strstart], bflush); ***/
    bflush = trees._tr_tally(s, 0, s.window[s.strstart]);
    s.lookahead--;
    s.strstart++;
    if (bflush) {
      /*** FLUSH_BLOCK(s, 0); ***/
      flush_block_only(s, false);
      if (s.strm.avail_out === 0) {
        return BS_NEED_MORE;
      }
      /***/
    }
  }
  s.insert = 0;
  if (flush === Z_FINISH) {
    /*** FLUSH_BLOCK(s, 1); ***/
    flush_block_only(s, true);
    if (s.strm.avail_out === 0) {
      return BS_FINISH_STARTED;
    }
    /***/
    return BS_FINISH_DONE;
  }
  if (s.last_lit) {
    /*** FLUSH_BLOCK(s, 0); ***/
    flush_block_only(s, false);
    if (s.strm.avail_out === 0) {
      return BS_NEED_MORE;
    }
    /***/
  }
  return BS_BLOCK_DONE;
}

/* Values for max_lazy_match, good_match and max_chain_length, depending on
 * the desired pack level (0..9). The values given below have been tuned to
 * exclude worst case performance for pathological files. Better values may be
 * found for specific files.
 */
function Config(good_length, max_lazy, nice_length, max_chain, func) {
  this.good_length = good_length;
  this.max_lazy = max_lazy;
  this.nice_length = nice_length;
  this.max_chain = max_chain;
  this.func = func;
}

var configuration_table;

configuration_table = [
  /*      good lazy nice chain */
  new Config(0, 0, 0, 0, deflate_stored),          /* 0 store only */
  new Config(4, 4, 8, 4, deflate_fast),            /* 1 max speed, no lazy matches */
  new Config(4, 5, 16, 8, deflate_fast),           /* 2 */
  new Config(4, 6, 32, 32, deflate_fast),          /* 3 */

  new Config(4, 4, 16, 16, deflate_slow),          /* 4 lazy matches */
  new Config(8, 16, 32, 32, deflate_slow),         /* 5 */
  new Config(8, 16, 128, 128, deflate_slow),       /* 6 */
  new Config(8, 32, 128, 256, deflate_slow),       /* 7 */
  new Config(32, 128, 258, 1024, deflate_slow),    /* 8 */
  new Config(32, 258, 258, 4096, deflate_slow)     /* 9 max compression */
];


/* ===========================================================================
 * Initialize the "longest match" routines for a new zlib stream
 */
function lm_init(s) {
  s.window_size = 2 * s.w_size;

  /*** CLEAR_HASH(s); ***/
  zero$1(s.head); // Fill with NIL (= 0);

  /* Set the default configuration parameters:
   */
  s.max_lazy_match = configuration_table[s.level].max_lazy;
  s.good_match = configuration_table[s.level].good_length;
  s.nice_match = configuration_table[s.level].nice_length;
  s.max_chain_length = configuration_table[s.level].max_chain;

  s.strstart = 0;
  s.block_start = 0;
  s.lookahead = 0;
  s.insert = 0;
  s.match_length = s.prev_length = MIN_MATCH$1 - 1;
  s.match_available = 0;
  s.ins_h = 0;
}


function DeflateState() {
  this.strm = null;            /* pointer back to this zlib stream */
  this.status = 0;            /* as the name implies */
  this.pending_buf = null;      /* output still pending */
  this.pending_buf_size = 0;  /* size of pending_buf */
  this.pending_out = 0;       /* next pending byte to output to the stream */
  this.pending = 0;           /* nb of bytes in the pending buffer */
  this.wrap = 0;              /* bit 0 true for zlib, bit 1 true for gzip */
  this.gzhead = null;         /* gzip header information to write */
  this.gzindex = 0;           /* where in extra, name, or comment */
  this.method = Z_DEFLATED; /* can only be DEFLATED */
  this.last_flush = -1;   /* value of flush param for previous deflate call */

  this.w_size = 0;  /* LZ77 window size (32K by default) */
  this.w_bits = 0;  /* log2(w_size)  (8..16) */
  this.w_mask = 0;  /* w_size - 1 */

  this.window = null;
  /* Sliding window. Input bytes are read into the second half of the window,
   * and move to the first half later to keep a dictionary of at least wSize
   * bytes. With this organization, matches are limited to a distance of
   * wSize-MAX_MATCH bytes, but this ensures that IO is always
   * performed with a length multiple of the block size.
   */

  this.window_size = 0;
  /* Actual size of window: 2*wSize, except when the user input buffer
   * is directly used as sliding window.
   */

  this.prev = null;
  /* Link to older string with same hash index. To limit the size of this
   * array to 64K, this link is maintained only for the last 32K strings.
   * An index in this array is thus a window index modulo 32K.
   */

  this.head = null;   /* Heads of the hash chains or NIL. */

  this.ins_h = 0;       /* hash index of string to be inserted */
  this.hash_size = 0;   /* number of elements in hash table */
  this.hash_bits = 0;   /* log2(hash_size) */
  this.hash_mask = 0;   /* hash_size-1 */

  this.hash_shift = 0;
  /* Number of bits by which ins_h must be shifted at each input
   * step. It must be such that after MIN_MATCH steps, the oldest
   * byte no longer takes part in the hash key, that is:
   *   hash_shift * MIN_MATCH >= hash_bits
   */

  this.block_start = 0;
  /* Window position at the beginning of the current output block. Gets
   * negative when the window is moved backwards.
   */

  this.match_length = 0;      /* length of best match */
  this.prev_match = 0;        /* previous match */
  this.match_available = 0;   /* set if previous match exists */
  this.strstart = 0;          /* start of string to insert */
  this.match_start = 0;       /* start of matching string */
  this.lookahead = 0;         /* number of valid bytes ahead in window */

  this.prev_length = 0;
  /* Length of the best match at previous step. Matches not greater than this
   * are discarded. This is used in the lazy match evaluation.
   */

  this.max_chain_length = 0;
  /* To speed up deflation, hash chains are never searched beyond this
   * length.  A higher limit improves compression ratio but degrades the
   * speed.
   */

  this.max_lazy_match = 0;
  /* Attempt to find a better match only when the current match is strictly
   * smaller than this value. This mechanism is used only for compression
   * levels >= 4.
   */
  // That's alias to max_lazy_match, don't use directly
  //this.max_insert_length = 0;
  /* Insert new strings in the hash table only if the match length is not
   * greater than this length. This saves time but degrades compression.
   * max_insert_length is used only for compression levels <= 3.
   */

  this.level = 0;     /* compression level (1..9) */
  this.strategy = 0;  /* favor or force Huffman coding*/

  this.good_match = 0;
  /* Use a faster search when the previous match is longer than this */

  this.nice_match = 0; /* Stop searching when current match exceeds this */

              /* used by trees.c: */

  /* Didn't use ct_data typedef below to suppress compiler warning */

  // struct ct_data_s dyn_ltree[HEAP_SIZE];   /* literal and length tree */
  // struct ct_data_s dyn_dtree[2*D_CODES+1]; /* distance tree */
  // struct ct_data_s bl_tree[2*BL_CODES+1];  /* Huffman tree for bit lengths */

  // Use flat array of DOUBLE size, with interleaved fata,
  // because JS does not support effective
  this.dyn_ltree  = new common.Buf16(HEAP_SIZE$1 * 2);
  this.dyn_dtree  = new common.Buf16((2 * D_CODES$1 + 1) * 2);
  this.bl_tree    = new common.Buf16((2 * BL_CODES$1 + 1) * 2);
  zero$1(this.dyn_ltree);
  zero$1(this.dyn_dtree);
  zero$1(this.bl_tree);

  this.l_desc   = null;         /* desc. for literal tree */
  this.d_desc   = null;         /* desc. for distance tree */
  this.bl_desc  = null;         /* desc. for bit length tree */

  //ush bl_count[MAX_BITS+1];
  this.bl_count = new common.Buf16(MAX_BITS$1 + 1);
  /* number of codes at each bit length for an optimal tree */

  //int heap[2*L_CODES+1];      /* heap used to build the Huffman trees */
  this.heap = new common.Buf16(2 * L_CODES$1 + 1);  /* heap used to build the Huffman trees */
  zero$1(this.heap);

  this.heap_len = 0;               /* number of elements in the heap */
  this.heap_max = 0;               /* element of largest frequency */
  /* The sons of heap[n] are heap[2*n] and heap[2*n+1]. heap[0] is not used.
   * The same heap array is used to build all trees.
   */

  this.depth = new common.Buf16(2 * L_CODES$1 + 1); //uch depth[2*L_CODES+1];
  zero$1(this.depth);
  /* Depth of each subtree used as tie breaker for trees of equal frequency
   */

  this.l_buf = 0;          /* buffer index for literals or lengths */

  this.lit_bufsize = 0;
  /* Size of match buffer for literals/lengths.  There are 4 reasons for
   * limiting lit_bufsize to 64K:
   *   - frequencies can be kept in 16 bit counters
   *   - if compression is not successful for the first block, all input
   *     data is still in the window so we can still emit a stored block even
   *     when input comes from standard input.  (This can also be done for
   *     all blocks if lit_bufsize is not greater than 32K.)
   *   - if compression is not successful for a file smaller than 64K, we can
   *     even emit a stored file instead of a stored block (saving 5 bytes).
   *     This is applicable only for zip (not gzip or zlib).
   *   - creating new Huffman trees less frequently may not provide fast
   *     adaptation to changes in the input data statistics. (Take for
   *     example a binary file with poorly compressible code followed by
   *     a highly compressible string table.) Smaller buffer sizes give
   *     fast adaptation but have of course the overhead of transmitting
   *     trees more frequently.
   *   - I can't count above 4
   */

  this.last_lit = 0;      /* running index in l_buf */

  this.d_buf = 0;
  /* Buffer index for distances. To simplify the code, d_buf and l_buf have
   * the same number of elements. To use different lengths, an extra flag
   * array would be necessary.
   */

  this.opt_len = 0;       /* bit length of current block with optimal trees */
  this.static_len = 0;    /* bit length of current block with static trees */
  this.matches = 0;       /* number of string matches in current block */
  this.insert = 0;        /* bytes at end of window left to insert */


  this.bi_buf = 0;
  /* Output buffer. bits are inserted starting at the bottom (least
   * significant bits).
   */
  this.bi_valid = 0;
  /* Number of valid bits in bi_buf.  All bits above the last valid bit
   * are always zero.
   */

  // Used for window memory init. We safely ignore it for JS. That makes
  // sense only for pointers and memory check tools.
  //this.high_water = 0;
  /* High water mark offset in window for initialized bytes -- bytes above
   * this are set to zero in order to avoid memory check warnings when
   * longest match routines access bytes past the input.  This is then
   * updated to the new high water mark.
   */
}


function deflateResetKeep(strm) {
  var s;

  if (!strm || !strm.state) {
    return err(strm, Z_STREAM_ERROR);
  }

  strm.total_in = strm.total_out = 0;
  strm.data_type = Z_UNKNOWN$1;

  s = strm.state;
  s.pending = 0;
  s.pending_out = 0;

  if (s.wrap < 0) {
    s.wrap = -s.wrap;
    /* was made negative by deflate(..., Z_FINISH); */
  }
  s.status = (s.wrap ? INIT_STATE : BUSY_STATE);
  strm.adler = (s.wrap === 2) ?
    0  // crc32(0, Z_NULL, 0)
  :
    1; // adler32(0, Z_NULL, 0)
  s.last_flush = Z_NO_FLUSH;
  trees._tr_init(s);
  return Z_OK;
}


function deflateReset(strm) {
  var ret = deflateResetKeep(strm);
  if (ret === Z_OK) {
    lm_init(strm.state);
  }
  return ret;
}


function deflateSetHeader(strm, head) {
  if (!strm || !strm.state) { return Z_STREAM_ERROR; }
  if (strm.state.wrap !== 2) { return Z_STREAM_ERROR; }
  strm.state.gzhead = head;
  return Z_OK;
}


function deflateInit2(strm, level, method, windowBits, memLevel, strategy) {
  if (!strm) { // === Z_NULL
    return Z_STREAM_ERROR;
  }
  var wrap = 1;

  if (level === Z_DEFAULT_COMPRESSION) {
    level = 6;
  }

  if (windowBits < 0) { /* suppress zlib wrapper */
    wrap = 0;
    windowBits = -windowBits;
  }

  else if (windowBits > 15) {
    wrap = 2;           /* write gzip wrapper instead */
    windowBits -= 16;
  }


  if (memLevel < 1 || memLevel > MAX_MEM_LEVEL || method !== Z_DEFLATED ||
    windowBits < 8 || windowBits > 15 || level < 0 || level > 9 ||
    strategy < 0 || strategy > Z_FIXED$1) {
    return err(strm, Z_STREAM_ERROR);
  }


  if (windowBits === 8) {
    windowBits = 9;
  }
  /* until 256-byte window bug fixed */

  var s = new DeflateState();

  strm.state = s;
  s.strm = strm;

  s.wrap = wrap;
  s.gzhead = null;
  s.w_bits = windowBits;
  s.w_size = 1 << s.w_bits;
  s.w_mask = s.w_size - 1;

  s.hash_bits = memLevel + 7;
  s.hash_size = 1 << s.hash_bits;
  s.hash_mask = s.hash_size - 1;
  s.hash_shift = ~~((s.hash_bits + MIN_MATCH$1 - 1) / MIN_MATCH$1);

  s.window = new common.Buf8(s.w_size * 2);
  s.head = new common.Buf16(s.hash_size);
  s.prev = new common.Buf16(s.w_size);

  // Don't need mem init magic for JS.
  //s.high_water = 0;  /* nothing written to s->window yet */

  s.lit_bufsize = 1 << (memLevel + 6); /* 16K elements by default */

  s.pending_buf_size = s.lit_bufsize * 4;

  //overlay = (ushf *) ZALLOC(strm, s->lit_bufsize, sizeof(ush)+2);
  //s->pending_buf = (uchf *) overlay;
  s.pending_buf = new common.Buf8(s.pending_buf_size);

  // It is offset from `s.pending_buf` (size is `s.lit_bufsize * 2`)
  //s->d_buf = overlay + s->lit_bufsize/sizeof(ush);
  s.d_buf = 1 * s.lit_bufsize;

  //s->l_buf = s->pending_buf + (1+sizeof(ush))*s->lit_bufsize;
  s.l_buf = (1 + 2) * s.lit_bufsize;

  s.level = level;
  s.strategy = strategy;
  s.method = method;

  return deflateReset(strm);
}

function deflateInit(strm, level) {
  return deflateInit2(strm, level, Z_DEFLATED, MAX_WBITS, DEF_MEM_LEVEL, Z_DEFAULT_STRATEGY);
}


function deflate(strm, flush) {
  var old_flush, s;
  var beg, val; // for gzip header write only

  if (!strm || !strm.state ||
    flush > Z_BLOCK || flush < 0) {
    return strm ? err(strm, Z_STREAM_ERROR) : Z_STREAM_ERROR;
  }

  s = strm.state;

  if (!strm.output ||
      (!strm.input && strm.avail_in !== 0) ||
      (s.status === FINISH_STATE && flush !== Z_FINISH)) {
    return err(strm, (strm.avail_out === 0) ? Z_BUF_ERROR : Z_STREAM_ERROR);
  }

  s.strm = strm; /* just in case */
  old_flush = s.last_flush;
  s.last_flush = flush;

  /* Write the header */
  if (s.status === INIT_STATE) {

    if (s.wrap === 2) { // GZIP header
      strm.adler = 0;  //crc32(0L, Z_NULL, 0);
      put_byte(s, 31);
      put_byte(s, 139);
      put_byte(s, 8);
      if (!s.gzhead) { // s->gzhead == Z_NULL
        put_byte(s, 0);
        put_byte(s, 0);
        put_byte(s, 0);
        put_byte(s, 0);
        put_byte(s, 0);
        put_byte(s, s.level === 9 ? 2 :
                    (s.strategy >= Z_HUFFMAN_ONLY || s.level < 2 ?
                     4 : 0));
        put_byte(s, OS_CODE);
        s.status = BUSY_STATE;
      }
      else {
        put_byte(s, (s.gzhead.text ? 1 : 0) +
                    (s.gzhead.hcrc ? 2 : 0) +
                    (!s.gzhead.extra ? 0 : 4) +
                    (!s.gzhead.name ? 0 : 8) +
                    (!s.gzhead.comment ? 0 : 16)
                );
        put_byte(s, s.gzhead.time & 0xff);
        put_byte(s, (s.gzhead.time >> 8) & 0xff);
        put_byte(s, (s.gzhead.time >> 16) & 0xff);
        put_byte(s, (s.gzhead.time >> 24) & 0xff);
        put_byte(s, s.level === 9 ? 2 :
                    (s.strategy >= Z_HUFFMAN_ONLY || s.level < 2 ?
                     4 : 0));
        put_byte(s, s.gzhead.os & 0xff);
        if (s.gzhead.extra && s.gzhead.extra.length) {
          put_byte(s, s.gzhead.extra.length & 0xff);
          put_byte(s, (s.gzhead.extra.length >> 8) & 0xff);
        }
        if (s.gzhead.hcrc) {
          strm.adler = crc32_1$1(strm.adler, s.pending_buf, s.pending, 0);
        }
        s.gzindex = 0;
        s.status = EXTRA_STATE;
      }
    }
    else // DEFLATE header
    {
      var header = (Z_DEFLATED + ((s.w_bits - 8) << 4)) << 8;
      var level_flags = -1;

      if (s.strategy >= Z_HUFFMAN_ONLY || s.level < 2) {
        level_flags = 0;
      } else if (s.level < 6) {
        level_flags = 1;
      } else if (s.level === 6) {
        level_flags = 2;
      } else {
        level_flags = 3;
      }
      header |= (level_flags << 6);
      if (s.strstart !== 0) { header |= PRESET_DICT; }
      header += 31 - (header % 31);

      s.status = BUSY_STATE;
      putShortMSB(s, header);

      /* Save the adler32 of the preset dictionary: */
      if (s.strstart !== 0) {
        putShortMSB(s, strm.adler >>> 16);
        putShortMSB(s, strm.adler & 0xffff);
      }
      strm.adler = 1; // adler32(0L, Z_NULL, 0);
    }
  }

//#ifdef GZIP
  if (s.status === EXTRA_STATE) {
    if (s.gzhead.extra/* != Z_NULL*/) {
      beg = s.pending;  /* start of bytes to update crc */

      while (s.gzindex < (s.gzhead.extra.length & 0xffff)) {
        if (s.pending === s.pending_buf_size) {
          if (s.gzhead.hcrc && s.pending > beg) {
            strm.adler = crc32_1$1(strm.adler, s.pending_buf, s.pending - beg, beg);
          }
          flush_pending(strm);
          beg = s.pending;
          if (s.pending === s.pending_buf_size) {
            break;
          }
        }
        put_byte(s, s.gzhead.extra[s.gzindex] & 0xff);
        s.gzindex++;
      }
      if (s.gzhead.hcrc && s.pending > beg) {
        strm.adler = crc32_1$1(strm.adler, s.pending_buf, s.pending - beg, beg);
      }
      if (s.gzindex === s.gzhead.extra.length) {
        s.gzindex = 0;
        s.status = NAME_STATE;
      }
    }
    else {
      s.status = NAME_STATE;
    }
  }
  if (s.status === NAME_STATE) {
    if (s.gzhead.name/* != Z_NULL*/) {
      beg = s.pending;  /* start of bytes to update crc */
      //int val;

      do {
        if (s.pending === s.pending_buf_size) {
          if (s.gzhead.hcrc && s.pending > beg) {
            strm.adler = crc32_1$1(strm.adler, s.pending_buf, s.pending - beg, beg);
          }
          flush_pending(strm);
          beg = s.pending;
          if (s.pending === s.pending_buf_size) {
            val = 1;
            break;
          }
        }
        // JS specific: little magic to add zero terminator to end of string
        if (s.gzindex < s.gzhead.name.length) {
          val = s.gzhead.name.charCodeAt(s.gzindex++) & 0xff;
        } else {
          val = 0;
        }
        put_byte(s, val);
      } while (val !== 0);

      if (s.gzhead.hcrc && s.pending > beg) {
        strm.adler = crc32_1$1(strm.adler, s.pending_buf, s.pending - beg, beg);
      }
      if (val === 0) {
        s.gzindex = 0;
        s.status = COMMENT_STATE;
      }
    }
    else {
      s.status = COMMENT_STATE;
    }
  }
  if (s.status === COMMENT_STATE) {
    if (s.gzhead.comment/* != Z_NULL*/) {
      beg = s.pending;  /* start of bytes to update crc */
      //int val;

      do {
        if (s.pending === s.pending_buf_size) {
          if (s.gzhead.hcrc && s.pending > beg) {
            strm.adler = crc32_1$1(strm.adler, s.pending_buf, s.pending - beg, beg);
          }
          flush_pending(strm);
          beg = s.pending;
          if (s.pending === s.pending_buf_size) {
            val = 1;
            break;
          }
        }
        // JS specific: little magic to add zero terminator to end of string
        if (s.gzindex < s.gzhead.comment.length) {
          val = s.gzhead.comment.charCodeAt(s.gzindex++) & 0xff;
        } else {
          val = 0;
        }
        put_byte(s, val);
      } while (val !== 0);

      if (s.gzhead.hcrc && s.pending > beg) {
        strm.adler = crc32_1$1(strm.adler, s.pending_buf, s.pending - beg, beg);
      }
      if (val === 0) {
        s.status = HCRC_STATE;
      }
    }
    else {
      s.status = HCRC_STATE;
    }
  }
  if (s.status === HCRC_STATE) {
    if (s.gzhead.hcrc) {
      if (s.pending + 2 > s.pending_buf_size) {
        flush_pending(strm);
      }
      if (s.pending + 2 <= s.pending_buf_size) {
        put_byte(s, strm.adler & 0xff);
        put_byte(s, (strm.adler >> 8) & 0xff);
        strm.adler = 0; //crc32(0L, Z_NULL, 0);
        s.status = BUSY_STATE;
      }
    }
    else {
      s.status = BUSY_STATE;
    }
  }
//#endif

  /* Flush as much pending output as possible */
  if (s.pending !== 0) {
    flush_pending(strm);
    if (strm.avail_out === 0) {
      /* Since avail_out is 0, deflate will be called again with
       * more output space, but possibly with both pending and
       * avail_in equal to zero. There won't be anything to do,
       * but this is not an error situation so make sure we
       * return OK instead of BUF_ERROR at next call of deflate:
       */
      s.last_flush = -1;
      return Z_OK;
    }

    /* Make sure there is something to do and avoid duplicate consecutive
     * flushes. For repeated and useless calls with Z_FINISH, we keep
     * returning Z_STREAM_END instead of Z_BUF_ERROR.
     */
  } else if (strm.avail_in === 0 && rank(flush) <= rank(old_flush) &&
    flush !== Z_FINISH) {
    return err(strm, Z_BUF_ERROR);
  }

  /* User must not provide more input after the first FINISH: */
  if (s.status === FINISH_STATE && strm.avail_in !== 0) {
    return err(strm, Z_BUF_ERROR);
  }

  /* Start a new block or continue the current one.
   */
  if (strm.avail_in !== 0 || s.lookahead !== 0 ||
    (flush !== Z_NO_FLUSH && s.status !== FINISH_STATE)) {
    var bstate = (s.strategy === Z_HUFFMAN_ONLY) ? deflate_huff(s, flush) :
      (s.strategy === Z_RLE ? deflate_rle(s, flush) :
        configuration_table[s.level].func(s, flush));

    if (bstate === BS_FINISH_STARTED || bstate === BS_FINISH_DONE) {
      s.status = FINISH_STATE;
    }
    if (bstate === BS_NEED_MORE || bstate === BS_FINISH_STARTED) {
      if (strm.avail_out === 0) {
        s.last_flush = -1;
        /* avoid BUF_ERROR next call, see above */
      }
      return Z_OK;
      /* If flush != Z_NO_FLUSH && avail_out == 0, the next call
       * of deflate should use the same flush parameter to make sure
       * that the flush is complete. So we don't have to output an
       * empty block here, this will be done at next call. This also
       * ensures that for a very small output buffer, we emit at most
       * one empty block.
       */
    }
    if (bstate === BS_BLOCK_DONE) {
      if (flush === Z_PARTIAL_FLUSH) {
        trees._tr_align(s);
      }
      else if (flush !== Z_BLOCK) { /* FULL_FLUSH or SYNC_FLUSH */

        trees._tr_stored_block(s, 0, 0, false);
        /* For a full flush, this empty block will be recognized
         * as a special marker by inflate_sync().
         */
        if (flush === Z_FULL_FLUSH) {
          /*** CLEAR_HASH(s); ***/             /* forget history */
          zero$1(s.head); // Fill with NIL (= 0);

          if (s.lookahead === 0) {
            s.strstart = 0;
            s.block_start = 0;
            s.insert = 0;
          }
        }
      }
      flush_pending(strm);
      if (strm.avail_out === 0) {
        s.last_flush = -1; /* avoid BUF_ERROR at next call, see above */
        return Z_OK;
      }
    }
  }
  //Assert(strm->avail_out > 0, "bug2");
  //if (strm.avail_out <= 0) { throw new Error("bug2");}

  if (flush !== Z_FINISH) { return Z_OK; }
  if (s.wrap <= 0) { return Z_STREAM_END; }

  /* Write the trailer */
  if (s.wrap === 2) {
    put_byte(s, strm.adler & 0xff);
    put_byte(s, (strm.adler >> 8) & 0xff);
    put_byte(s, (strm.adler >> 16) & 0xff);
    put_byte(s, (strm.adler >> 24) & 0xff);
    put_byte(s, strm.total_in & 0xff);
    put_byte(s, (strm.total_in >> 8) & 0xff);
    put_byte(s, (strm.total_in >> 16) & 0xff);
    put_byte(s, (strm.total_in >> 24) & 0xff);
  }
  else
  {
    putShortMSB(s, strm.adler >>> 16);
    putShortMSB(s, strm.adler & 0xffff);
  }

  flush_pending(strm);
  /* If avail_out is zero, the application will call deflate again
   * to flush the rest.
   */
  if (s.wrap > 0) { s.wrap = -s.wrap; }
  /* write the trailer only once! */
  return s.pending !== 0 ? Z_OK : Z_STREAM_END;
}

function deflateEnd(strm) {
  var status;

  if (!strm/*== Z_NULL*/ || !strm.state/*== Z_NULL*/) {
    return Z_STREAM_ERROR;
  }

  status = strm.state.status;
  if (status !== INIT_STATE &&
    status !== EXTRA_STATE &&
    status !== NAME_STATE &&
    status !== COMMENT_STATE &&
    status !== HCRC_STATE &&
    status !== BUSY_STATE &&
    status !== FINISH_STATE
  ) {
    return err(strm, Z_STREAM_ERROR);
  }

  strm.state = null;

  return status === BUSY_STATE ? err(strm, Z_DATA_ERROR) : Z_OK;
}


/* =========================================================================
 * Initializes the compression dictionary from the given byte
 * sequence without producing any compressed output.
 */
function deflateSetDictionary(strm, dictionary) {
  var dictLength = dictionary.length;

  var s;
  var str, n;
  var wrap;
  var avail;
  var next;
  var input;
  var tmpDict;

  if (!strm/*== Z_NULL*/ || !strm.state/*== Z_NULL*/) {
    return Z_STREAM_ERROR;
  }

  s = strm.state;
  wrap = s.wrap;

  if (wrap === 2 || (wrap === 1 && s.status !== INIT_STATE) || s.lookahead) {
    return Z_STREAM_ERROR;
  }

  /* when using zlib wrappers, compute Adler-32 for provided dictionary */
  if (wrap === 1) {
    /* adler32(strm->adler, dictionary, dictLength); */
    strm.adler = adler32_1(strm.adler, dictionary, dictLength, 0);
  }

  s.wrap = 0;   /* avoid computing Adler-32 in read_buf */

  /* if dictionary would fill window, just replace the history */
  if (dictLength >= s.w_size) {
    if (wrap === 0) {            /* already empty otherwise */
      /*** CLEAR_HASH(s); ***/
      zero$1(s.head); // Fill with NIL (= 0);
      s.strstart = 0;
      s.block_start = 0;
      s.insert = 0;
    }
    /* use the tail */
    // dictionary = dictionary.slice(dictLength - s.w_size);
    tmpDict = new common.Buf8(s.w_size);
    common.arraySet(tmpDict, dictionary, dictLength - s.w_size, s.w_size, 0);
    dictionary = tmpDict;
    dictLength = s.w_size;
  }
  /* insert dictionary into window and hash */
  avail = strm.avail_in;
  next = strm.next_in;
  input = strm.input;
  strm.avail_in = dictLength;
  strm.next_in = 0;
  strm.input = dictionary;
  fill_window(s);
  while (s.lookahead >= MIN_MATCH$1) {
    str = s.strstart;
    n = s.lookahead - (MIN_MATCH$1 - 1);
    do {
      /* UPDATE_HASH(s, s->ins_h, s->window[str + MIN_MATCH-1]); */
      s.ins_h = ((s.ins_h << s.hash_shift) ^ s.window[str + MIN_MATCH$1 - 1]) & s.hash_mask;

      s.prev[str & s.w_mask] = s.head[s.ins_h];

      s.head[s.ins_h] = str;
      str++;
    } while (--n);
    s.strstart = str;
    s.lookahead = MIN_MATCH$1 - 1;
    fill_window(s);
  }
  s.strstart += s.lookahead;
  s.block_start = s.strstart;
  s.insert = s.lookahead;
  s.lookahead = 0;
  s.match_length = s.prev_length = MIN_MATCH$1 - 1;
  s.match_available = 0;
  strm.next_in = next;
  strm.input = input;
  strm.avail_in = avail;
  s.wrap = wrap;
  return Z_OK;
}


var deflateInit_1 = deflateInit;
var deflateInit2_1 = deflateInit2;
var deflateReset_1 = deflateReset;
var deflateResetKeep_1 = deflateResetKeep;
var deflateSetHeader_1 = deflateSetHeader;
var deflate_2 = deflate;
var deflateEnd_1 = deflateEnd;
var deflateSetDictionary_1 = deflateSetDictionary;
var deflateInfo = 'pako deflate (from Nodeca project)';

/* Not implemented
exports.deflateBound = deflateBound;
exports.deflateCopy = deflateCopy;
exports.deflateParams = deflateParams;
exports.deflatePending = deflatePending;
exports.deflatePrime = deflatePrime;
exports.deflateTune = deflateTune;
*/

var deflate_1 = {
	deflateInit: deflateInit_1,
	deflateInit2: deflateInit2_1,
	deflateReset: deflateReset_1,
	deflateResetKeep: deflateResetKeep_1,
	deflateSetHeader: deflateSetHeader_1,
	deflate: deflate_2,
	deflateEnd: deflateEnd_1,
	deflateSetDictionary: deflateSetDictionary_1,
	deflateInfo: deflateInfo
};

// Quick check if we can use fast array to bin string conversion
//
// - apply(Array) can fail on Android 2.2
// - apply(Uint8Array) can fail on iOS 5.1 Safari
//
var STR_APPLY_OK = true;
var STR_APPLY_UIA_OK = true;

try { String.fromCharCode.apply(null, [ 0 ]); } catch (__) { STR_APPLY_OK = false; }
try { String.fromCharCode.apply(null, new Uint8Array(1)); } catch (__) { STR_APPLY_UIA_OK = false; }


// Table with utf8 lengths (calculated by first byte of sequence)
// Note, that 5 & 6-byte values and some 4-byte values can not be represented in JS,
// because max possible codepoint is 0x10ffff
var _utf8len = new common.Buf8(256);
for (var q = 0; q < 256; q++) {
  _utf8len[q] = (q >= 252 ? 6 : q >= 248 ? 5 : q >= 240 ? 4 : q >= 224 ? 3 : q >= 192 ? 2 : 1);
}
_utf8len[254] = _utf8len[254] = 1; // Invalid sequence start


// convert string to array (typed, when possible)
var string2buf = function (str) {
  var buf, c, c2, m_pos, i, str_len = str.length, buf_len = 0;

  // count binary size
  for (m_pos = 0; m_pos < str_len; m_pos++) {
    c = str.charCodeAt(m_pos);
    if ((c & 0xfc00) === 0xd800 && (m_pos + 1 < str_len)) {
      c2 = str.charCodeAt(m_pos + 1);
      if ((c2 & 0xfc00) === 0xdc00) {
        c = 0x10000 + ((c - 0xd800) << 10) + (c2 - 0xdc00);
        m_pos++;
      }
    }
    buf_len += c < 0x80 ? 1 : c < 0x800 ? 2 : c < 0x10000 ? 3 : 4;
  }

  // allocate buffer
  buf = new common.Buf8(buf_len);

  // convert
  for (i = 0, m_pos = 0; i < buf_len; m_pos++) {
    c = str.charCodeAt(m_pos);
    if ((c & 0xfc00) === 0xd800 && (m_pos + 1 < str_len)) {
      c2 = str.charCodeAt(m_pos + 1);
      if ((c2 & 0xfc00) === 0xdc00) {
        c = 0x10000 + ((c - 0xd800) << 10) + (c2 - 0xdc00);
        m_pos++;
      }
    }
    if (c < 0x80) {
      /* one byte */
      buf[i++] = c;
    } else if (c < 0x800) {
      /* two bytes */
      buf[i++] = 0xC0 | (c >>> 6);
      buf[i++] = 0x80 | (c & 0x3f);
    } else if (c < 0x10000) {
      /* three bytes */
      buf[i++] = 0xE0 | (c >>> 12);
      buf[i++] = 0x80 | (c >>> 6 & 0x3f);
      buf[i++] = 0x80 | (c & 0x3f);
    } else {
      /* four bytes */
      buf[i++] = 0xf0 | (c >>> 18);
      buf[i++] = 0x80 | (c >>> 12 & 0x3f);
      buf[i++] = 0x80 | (c >>> 6 & 0x3f);
      buf[i++] = 0x80 | (c & 0x3f);
    }
  }

  return buf;
};

// Helper (used in 2 places)
function buf2binstring(buf, len) {
  // use fallback for big arrays to avoid stack overflow
  if (len < 65537) {
    if ((buf.subarray && STR_APPLY_UIA_OK) || (!buf.subarray && STR_APPLY_OK)) {
      return String.fromCharCode.apply(null, common.shrinkBuf(buf, len));
    }
  }

  var result = '';
  for (var i = 0; i < len; i++) {
    result += String.fromCharCode(buf[i]);
  }
  return result;
}


// Convert byte array to binary string
var buf2binstring_1 = function (buf) {
  return buf2binstring(buf, buf.length);
};


// Convert binary string (typed, when possible)
var binstring2buf = function (str) {
  var buf = new common.Buf8(str.length);
  for (var i = 0, len = buf.length; i < len; i++) {
    buf[i] = str.charCodeAt(i);
  }
  return buf;
};


// convert array to string
var buf2string = function (buf, max) {
  var i, out, c, c_len;
  var len = max || buf.length;

  // Reserve max possible length (2 words per char)
  // NB: by unknown reasons, Array is significantly faster for
  //     String.fromCharCode.apply than Uint16Array.
  var utf16buf = new Array(len * 2);

  for (out = 0, i = 0; i < len;) {
    c = buf[i++];
    // quick process ascii
    if (c < 0x80) { utf16buf[out++] = c; continue; }

    c_len = _utf8len[c];
    // skip 5 & 6 byte codes
    if (c_len > 4) { utf16buf[out++] = 0xfffd; i += c_len - 1; continue; }

    // apply mask on first byte
    c &= c_len === 2 ? 0x1f : c_len === 3 ? 0x0f : 0x07;
    // join the rest
    while (c_len > 1 && i < len) {
      c = (c << 6) | (buf[i++] & 0x3f);
      c_len--;
    }

    // terminated by end of string?
    if (c_len > 1) { utf16buf[out++] = 0xfffd; continue; }

    if (c < 0x10000) {
      utf16buf[out++] = c;
    } else {
      c -= 0x10000;
      utf16buf[out++] = 0xd800 | ((c >> 10) & 0x3ff);
      utf16buf[out++] = 0xdc00 | (c & 0x3ff);
    }
  }

  return buf2binstring(utf16buf, out);
};


// Calculate max possible position in utf8 buffer,
// that will not break sequence. If that's not possible
// - (very small limits) return max size as is.
//
// buf[] - utf8 bytes array
// max   - length limit (mandatory);
var utf8border = function (buf, max) {
  var pos;

  max = max || buf.length;
  if (max > buf.length) { max = buf.length; }

  // go back from last position, until start of sequence found
  pos = max - 1;
  while (pos >= 0 && (buf[pos] & 0xC0) === 0x80) { pos--; }

  // Very small and broken sequence,
  // return max, because we should return something anyway.
  if (pos < 0) { return max; }

  // If we came to start of buffer - that means buffer is too small,
  // return max too.
  if (pos === 0) { return max; }

  return (pos + _utf8len[buf[pos]] > max) ? pos : max;
};

var strings = {
	string2buf: string2buf,
	buf2binstring: buf2binstring_1,
	binstring2buf: binstring2buf,
	buf2string: buf2string,
	utf8border: utf8border
};

// (C) 1995-2013 Jean-loup Gailly and Mark Adler
// (C) 2014-2017 Vitaly Puzrin and Andrey Tupitsin
//
// This software is provided 'as-is', without any express or implied
// warranty. In no event will the authors be held liable for any damages
// arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must not
//   claim that you wrote the original software. If you use this software
//   in a product, an acknowledgment in the product documentation would be
//   appreciated but is not required.
// 2. Altered source versions must be plainly marked as such, and must not be
//   misrepresented as being the original software.
// 3. This notice may not be removed or altered from any source distribution.

function ZStream() {
  /* next input byte */
  this.input = null; // JS specific, because we have no pointers
  this.next_in = 0;
  /* number of bytes available at input */
  this.avail_in = 0;
  /* total number of input bytes read so far */
  this.total_in = 0;
  /* next output byte should be put there */
  this.output = null; // JS specific, because we have no pointers
  this.next_out = 0;
  /* remaining free space at output */
  this.avail_out = 0;
  /* total number of bytes output so far */
  this.total_out = 0;
  /* last error message, NULL if no error */
  this.msg = ''/*Z_NULL*/;
  /* not visible by applications */
  this.state = null;
  /* best guess about the data type: binary or text */
  this.data_type = 2/*Z_UNKNOWN*/;
  /* adler32 value of the uncompressed data */
  this.adler = 0;
}

var zstream = ZStream;

var toString$3 = Object.prototype.toString;

/* Public constants ==========================================================*/
/* ===========================================================================*/

var Z_NO_FLUSH$1      = 0;
var Z_FINISH$1        = 4;

var Z_OK$1            = 0;
var Z_STREAM_END$1    = 1;
var Z_SYNC_FLUSH    = 2;

var Z_DEFAULT_COMPRESSION$1 = -1;

var Z_DEFAULT_STRATEGY$1    = 0;

var Z_DEFLATED$1  = 8;

/* ===========================================================================*/


/**
 * class Deflate
 *
 * Generic JS-style wrapper for zlib calls. If you don't need
 * streaming behaviour - use more simple functions: [[deflate]],
 * [[deflateRaw]] and [[gzip]].
 **/

/* internal
 * Deflate.chunks -> Array
 *
 * Chunks of output data, if [[Deflate#onData]] not overridden.
 **/

/**
 * Deflate.result -> Uint8Array|Array
 *
 * Compressed result, generated by default [[Deflate#onData]]
 * and [[Deflate#onEnd]] handlers. Filled after you push last chunk
 * (call [[Deflate#push]] with `Z_FINISH` / `true` param)  or if you
 * push a chunk with explicit flush (call [[Deflate#push]] with
 * `Z_SYNC_FLUSH` param).
 **/

/**
 * Deflate.err -> Number
 *
 * Error code after deflate finished. 0 (Z_OK) on success.
 * You will not need it in real life, because deflate errors
 * are possible only on wrong options or bad `onData` / `onEnd`
 * custom handlers.
 **/

/**
 * Deflate.msg -> String
 *
 * Error message, if [[Deflate.err]] != 0
 **/


/**
 * new Deflate(options)
 * - options (Object): zlib deflate options.
 *
 * Creates new deflator instance with specified params. Throws exception
 * on bad params. Supported options:
 *
 * - `level`
 * - `windowBits`
 * - `memLevel`
 * - `strategy`
 * - `dictionary`
 *
 * [http://zlib.net/manual.html#Advanced](http://zlib.net/manual.html#Advanced)
 * for more information on these.
 *
 * Additional options, for internal needs:
 *
 * - `chunkSize` - size of generated data chunks (16K by default)
 * - `raw` (Boolean) - do raw deflate
 * - `gzip` (Boolean) - create gzip wrapper
 * - `to` (String) - if equal to 'string', then result will be "binary string"
 *    (each char code [0..255])
 * - `header` (Object) - custom header for gzip
 *   - `text` (Boolean) - true if compressed data believed to be text
 *   - `time` (Number) - modification time, unix timestamp
 *   - `os` (Number) - operation system code
 *   - `extra` (Array) - array of bytes with extra data (max 65536)
 *   - `name` (String) - file name (binary string)
 *   - `comment` (String) - comment (binary string)
 *   - `hcrc` (Boolean) - true if header crc should be added
 *
 * ##### Example:
 *
 * ```javascript
 * var pako = require('pako')
 *   , chunk1 = Uint8Array([1,2,3,4,5,6,7,8,9])
 *   , chunk2 = Uint8Array([10,11,12,13,14,15,16,17,18,19]);
 *
 * var deflate = new pako.Deflate({ level: 3});
 *
 * deflate.push(chunk1, false);
 * deflate.push(chunk2, true);  // true -> last chunk
 *
 * if (deflate.err) { throw new Error(deflate.err); }
 *
 * console.log(deflate.result);
 * ```
 **/
function Deflate(options) {
  if (!(this instanceof Deflate)) return new Deflate(options);

  this.options = common.assign({
    level: Z_DEFAULT_COMPRESSION$1,
    method: Z_DEFLATED$1,
    chunkSize: 16384,
    windowBits: 15,
    memLevel: 8,
    strategy: Z_DEFAULT_STRATEGY$1,
    to: ''
  }, options || {});

  var opt = this.options;

  if (opt.raw && (opt.windowBits > 0)) {
    opt.windowBits = -opt.windowBits;
  }

  else if (opt.gzip && (opt.windowBits > 0) && (opt.windowBits < 16)) {
    opt.windowBits += 16;
  }

  this.err    = 0;      // error code, if happens (0 = Z_OK)
  this.msg    = '';     // error message
  this.ended  = false;  // used to avoid multiple onEnd() calls
  this.chunks = [];     // chunks of compressed data

  this.strm = new zstream();
  this.strm.avail_out = 0;

  var status = deflate_1.deflateInit2(
    this.strm,
    opt.level,
    opt.method,
    opt.windowBits,
    opt.memLevel,
    opt.strategy
  );

  if (status !== Z_OK$1) {
    throw new Error(messages[status]);
  }

  if (opt.header) {
    deflate_1.deflateSetHeader(this.strm, opt.header);
  }

  if (opt.dictionary) {
    var dict;
    // Convert data if needed
    if (typeof opt.dictionary === 'string') {
      // If we need to compress text, change encoding to utf8.
      dict = strings.string2buf(opt.dictionary);
    } else if (toString$3.call(opt.dictionary) === '[object ArrayBuffer]') {
      dict = new Uint8Array(opt.dictionary);
    } else {
      dict = opt.dictionary;
    }

    status = deflate_1.deflateSetDictionary(this.strm, dict);

    if (status !== Z_OK$1) {
      throw new Error(messages[status]);
    }

    this._dict_set = true;
  }
}

/**
 * Deflate#push(data[, mode]) -> Boolean
 * - data (Uint8Array|Array|ArrayBuffer|String): input data. Strings will be
 *   converted to utf8 byte sequence.
 * - mode (Number|Boolean): 0..6 for corresponding Z_NO_FLUSH..Z_TREE modes.
 *   See constants. Skipped or `false` means Z_NO_FLUSH, `true` means Z_FINISH.
 *
 * Sends input data to deflate pipe, generating [[Deflate#onData]] calls with
 * new compressed chunks. Returns `true` on success. The last data block must have
 * mode Z_FINISH (or `true`). That will flush internal pending buffers and call
 * [[Deflate#onEnd]]. For interim explicit flushes (without ending the stream) you
 * can use mode Z_SYNC_FLUSH, keeping the compression context.
 *
 * On fail call [[Deflate#onEnd]] with error code and return false.
 *
 * We strongly recommend to use `Uint8Array` on input for best speed (output
 * array format is detected automatically). Also, don't skip last param and always
 * use the same type in your code (boolean or number). That will improve JS speed.
 *
 * For regular `Array`-s make sure all elements are [0..255].
 *
 * ##### Example
 *
 * ```javascript
 * push(chunk, false); // push one of data chunks
 * ...
 * push(chunk, true);  // push last chunk
 * ```
 **/
Deflate.prototype.push = function (data, mode) {
  var strm = this.strm;
  var chunkSize = this.options.chunkSize;
  var status, _mode;

  if (this.ended) { return false; }

  _mode = (mode === ~~mode) ? mode : ((mode === true) ? Z_FINISH$1 : Z_NO_FLUSH$1);

  // Convert data if needed
  if (typeof data === 'string') {
    // If we need to compress text, change encoding to utf8.
    strm.input = strings.string2buf(data);
  } else if (toString$3.call(data) === '[object ArrayBuffer]') {
    strm.input = new Uint8Array(data);
  } else {
    strm.input = data;
  }

  strm.next_in = 0;
  strm.avail_in = strm.input.length;

  do {
    if (strm.avail_out === 0) {
      strm.output = new common.Buf8(chunkSize);
      strm.next_out = 0;
      strm.avail_out = chunkSize;
    }
    status = deflate_1.deflate(strm, _mode);    /* no bad return value */

    if (status !== Z_STREAM_END$1 && status !== Z_OK$1) {
      this.onEnd(status);
      this.ended = true;
      return false;
    }
    if (strm.avail_out === 0 || (strm.avail_in === 0 && (_mode === Z_FINISH$1 || _mode === Z_SYNC_FLUSH))) {
      if (this.options.to === 'string') {
        this.onData(strings.buf2binstring(common.shrinkBuf(strm.output, strm.next_out)));
      } else {
        this.onData(common.shrinkBuf(strm.output, strm.next_out));
      }
    }
  } while ((strm.avail_in > 0 || strm.avail_out === 0) && status !== Z_STREAM_END$1);

  // Finalize on the last chunk.
  if (_mode === Z_FINISH$1) {
    status = deflate_1.deflateEnd(this.strm);
    this.onEnd(status);
    this.ended = true;
    return status === Z_OK$1;
  }

  // callback interim results if Z_SYNC_FLUSH.
  if (_mode === Z_SYNC_FLUSH) {
    this.onEnd(Z_OK$1);
    strm.avail_out = 0;
    return true;
  }

  return true;
};


/**
 * Deflate#onData(chunk) -> Void
 * - chunk (Uint8Array|Array|String): output data. Type of array depends
 *   on js engine support. When string output requested, each chunk
 *   will be string.
 *
 * By default, stores data blocks in `chunks[]` property and glue
 * those in `onEnd`. Override this handler, if you need another behaviour.
 **/
Deflate.prototype.onData = function (chunk) {
  this.chunks.push(chunk);
};


/**
 * Deflate#onEnd(status) -> Void
 * - status (Number): deflate status. 0 (Z_OK) on success,
 *   other if not.
 *
 * Called once after you tell deflate that the input stream is
 * complete (Z_FINISH) or should be flushed (Z_SYNC_FLUSH)
 * or if an error happened. By default - join collected chunks,
 * free memory and fill `results` / `err` properties.
 **/
Deflate.prototype.onEnd = function (status) {
  // On success - join
  if (status === Z_OK$1) {
    if (this.options.to === 'string') {
      this.result = this.chunks.join('');
    } else {
      this.result = common.flattenChunks(this.chunks);
    }
  }
  this.chunks = [];
  this.err = status;
  this.msg = this.strm.msg;
};


/**
 * deflate(data[, options]) -> Uint8Array|Array|String
 * - data (Uint8Array|Array|String): input data to compress.
 * - options (Object): zlib deflate options.
 *
 * Compress `data` with deflate algorithm and `options`.
 *
 * Supported options are:
 *
 * - level
 * - windowBits
 * - memLevel
 * - strategy
 * - dictionary
 *
 * [http://zlib.net/manual.html#Advanced](http://zlib.net/manual.html#Advanced)
 * for more information on these.
 *
 * Sugar (options):
 *
 * - `raw` (Boolean) - say that we work with raw stream, if you don't wish to specify
 *   negative windowBits implicitly.
 * - `to` (String) - if equal to 'string', then result will be "binary string"
 *    (each char code [0..255])
 *
 * ##### Example:
 *
 * ```javascript
 * var pako = require('pako')
 *   , data = Uint8Array([1,2,3,4,5,6,7,8,9]);
 *
 * console.log(pako.deflate(data));
 * ```
 **/
function deflate$1(input, options) {
  var deflator = new Deflate(options);

  deflator.push(input, true);

  // That will never happens, if you don't cheat with options :)
  if (deflator.err) { throw deflator.msg || messages[deflator.err]; }

  return deflator.result;
}


/**
 * deflateRaw(data[, options]) -> Uint8Array|Array|String
 * - data (Uint8Array|Array|String): input data to compress.
 * - options (Object): zlib deflate options.
 *
 * The same as [[deflate]], but creates raw data, without wrapper
 * (header and adler32 crc).
 **/
function deflateRaw(input, options) {
  options = options || {};
  options.raw = true;
  return deflate$1(input, options);
}


/**
 * gzip(data[, options]) -> Uint8Array|Array|String
 * - data (Uint8Array|Array|String): input data to compress.
 * - options (Object): zlib deflate options.
 *
 * The same as [[deflate]], but create gzip wrapper instead of
 * deflate one.
 **/
function gzip(input, options) {
  options = options || {};
  options.gzip = true;
  return deflate$1(input, options);
}


var Deflate_1 = Deflate;
var deflate_2$1 = deflate$1;
var deflateRaw_1 = deflateRaw;
var gzip_1 = gzip;

var deflate_1$1 = {
	Deflate: Deflate_1,
	deflate: deflate_2$1,
	deflateRaw: deflateRaw_1,
	gzip: gzip_1
};

// (C) 1995-2013 Jean-loup Gailly and Mark Adler
// (C) 2014-2017 Vitaly Puzrin and Andrey Tupitsin
//
// This software is provided 'as-is', without any express or implied
// warranty. In no event will the authors be held liable for any damages
// arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must not
//   claim that you wrote the original software. If you use this software
//   in a product, an acknowledgment in the product documentation would be
//   appreciated but is not required.
// 2. Altered source versions must be plainly marked as such, and must not be
//   misrepresented as being the original software.
// 3. This notice may not be removed or altered from any source distribution.

// See state defs from inflate.js
var BAD = 30;       /* got a data error -- remain here until reset */
var TYPE = 12;      /* i: waiting for type bits, including last-flag bit */

/*
   Decode literal, length, and distance codes and write out the resulting
   literal and match bytes until either not enough input or output is
   available, an end-of-block is encountered, or a data error is encountered.
   When large enough input and output buffers are supplied to inflate(), for
   example, a 16K input buffer and a 64K output buffer, more than 95% of the
   inflate execution time is spent in this routine.

   Entry assumptions:

        state.mode === LEN
        strm.avail_in >= 6
        strm.avail_out >= 258
        start >= strm.avail_out
        state.bits < 8

   On return, state.mode is one of:

        LEN -- ran out of enough output space or enough available input
        TYPE -- reached end of block code, inflate() to interpret next block
        BAD -- error in block data

   Notes:

    - The maximum input bits used by a length/distance pair is 15 bits for the
      length code, 5 bits for the length extra, 15 bits for the distance code,
      and 13 bits for the distance extra.  This totals 48 bits, or six bytes.
      Therefore if strm.avail_in >= 6, then there is enough input to avoid
      checking for available input while decoding.

    - The maximum bytes that a single length/distance pair can output is 258
      bytes, which is the maximum length that can be coded.  inflate_fast()
      requires strm.avail_out >= 258 for each loop to avoid checking for
      output space.
 */
var inffast = function inflate_fast(strm, start) {
  var state;
  var _in;                    /* local strm.input */
  var last;                   /* have enough input while in < last */
  var _out;                   /* local strm.output */
  var beg;                    /* inflate()'s initial strm.output */
  var end;                    /* while out < end, enough space available */
//#ifdef INFLATE_STRICT
  var dmax;                   /* maximum distance from zlib header */
//#endif
  var wsize;                  /* window size or zero if not using window */
  var whave;                  /* valid bytes in the window */
  var wnext;                  /* window write index */
  // Use `s_window` instead `window`, avoid conflict with instrumentation tools
  var s_window;               /* allocated sliding window, if wsize != 0 */
  var hold;                   /* local strm.hold */
  var bits;                   /* local strm.bits */
  var lcode;                  /* local strm.lencode */
  var dcode;                  /* local strm.distcode */
  var lmask;                  /* mask for first level of length codes */
  var dmask;                  /* mask for first level of distance codes */
  var here;                   /* retrieved table entry */
  var op;                     /* code bits, operation, extra bits, or */
                              /*  window position, window bytes to copy */
  var len;                    /* match length, unused bytes */
  var dist;                   /* match distance */
  var from;                   /* where to copy match from */
  var from_source;


  var input, output; // JS specific, because we have no pointers

  /* copy state to local variables */
  state = strm.state;
  //here = state.here;
  _in = strm.next_in;
  input = strm.input;
  last = _in + (strm.avail_in - 5);
  _out = strm.next_out;
  output = strm.output;
  beg = _out - (start - strm.avail_out);
  end = _out + (strm.avail_out - 257);
//#ifdef INFLATE_STRICT
  dmax = state.dmax;
//#endif
  wsize = state.wsize;
  whave = state.whave;
  wnext = state.wnext;
  s_window = state.window;
  hold = state.hold;
  bits = state.bits;
  lcode = state.lencode;
  dcode = state.distcode;
  lmask = (1 << state.lenbits) - 1;
  dmask = (1 << state.distbits) - 1;


  /* decode literals and length/distances until end-of-block or not enough
     input data or output space */

  top:
  do {
    if (bits < 15) {
      hold += input[_in++] << bits;
      bits += 8;
      hold += input[_in++] << bits;
      bits += 8;
    }

    here = lcode[hold & lmask];

    dolen:
    for (;;) { // Goto emulation
      op = here >>> 24/*here.bits*/;
      hold >>>= op;
      bits -= op;
      op = (here >>> 16) & 0xff/*here.op*/;
      if (op === 0) {                          /* literal */
        //Tracevv((stderr, here.val >= 0x20 && here.val < 0x7f ?
        //        "inflate:         literal '%c'\n" :
        //        "inflate:         literal 0x%02x\n", here.val));
        output[_out++] = here & 0xffff/*here.val*/;
      }
      else if (op & 16) {                     /* length base */
        len = here & 0xffff/*here.val*/;
        op &= 15;                           /* number of extra bits */
        if (op) {
          if (bits < op) {
            hold += input[_in++] << bits;
            bits += 8;
          }
          len += hold & ((1 << op) - 1);
          hold >>>= op;
          bits -= op;
        }
        //Tracevv((stderr, "inflate:         length %u\n", len));
        if (bits < 15) {
          hold += input[_in++] << bits;
          bits += 8;
          hold += input[_in++] << bits;
          bits += 8;
        }
        here = dcode[hold & dmask];

        dodist:
        for (;;) { // goto emulation
          op = here >>> 24/*here.bits*/;
          hold >>>= op;
          bits -= op;
          op = (here >>> 16) & 0xff/*here.op*/;

          if (op & 16) {                      /* distance base */
            dist = here & 0xffff/*here.val*/;
            op &= 15;                       /* number of extra bits */
            if (bits < op) {
              hold += input[_in++] << bits;
              bits += 8;
              if (bits < op) {
                hold += input[_in++] << bits;
                bits += 8;
              }
            }
            dist += hold & ((1 << op) - 1);
//#ifdef INFLATE_STRICT
            if (dist > dmax) {
              strm.msg = 'invalid distance too far back';
              state.mode = BAD;
              break top;
            }
//#endif
            hold >>>= op;
            bits -= op;
            //Tracevv((stderr, "inflate:         distance %u\n", dist));
            op = _out - beg;                /* max distance in output */
            if (dist > op) {                /* see if copy from window */
              op = dist - op;               /* distance back in window */
              if (op > whave) {
                if (state.sane) {
                  strm.msg = 'invalid distance too far back';
                  state.mode = BAD;
                  break top;
                }

// (!) This block is disabled in zlib defaults,
// don't enable it for binary compatibility
//#ifdef INFLATE_ALLOW_INVALID_DISTANCE_TOOFAR_ARRR
//                if (len <= op - whave) {
//                  do {
//                    output[_out++] = 0;
//                  } while (--len);
//                  continue top;
//                }
//                len -= op - whave;
//                do {
//                  output[_out++] = 0;
//                } while (--op > whave);
//                if (op === 0) {
//                  from = _out - dist;
//                  do {
//                    output[_out++] = output[from++];
//                  } while (--len);
//                  continue top;
//                }
//#endif
              }
              from = 0; // window index
              from_source = s_window;
              if (wnext === 0) {           /* very common case */
                from += wsize - op;
                if (op < len) {         /* some from window */
                  len -= op;
                  do {
                    output[_out++] = s_window[from++];
                  } while (--op);
                  from = _out - dist;  /* rest from output */
                  from_source = output;
                }
              }
              else if (wnext < op) {      /* wrap around window */
                from += wsize + wnext - op;
                op -= wnext;
                if (op < len) {         /* some from end of window */
                  len -= op;
                  do {
                    output[_out++] = s_window[from++];
                  } while (--op);
                  from = 0;
                  if (wnext < len) {  /* some from start of window */
                    op = wnext;
                    len -= op;
                    do {
                      output[_out++] = s_window[from++];
                    } while (--op);
                    from = _out - dist;      /* rest from output */
                    from_source = output;
                  }
                }
              }
              else {                      /* contiguous in window */
                from += wnext - op;
                if (op < len) {         /* some from window */
                  len -= op;
                  do {
                    output[_out++] = s_window[from++];
                  } while (--op);
                  from = _out - dist;  /* rest from output */
                  from_source = output;
                }
              }
              while (len > 2) {
                output[_out++] = from_source[from++];
                output[_out++] = from_source[from++];
                output[_out++] = from_source[from++];
                len -= 3;
              }
              if (len) {
                output[_out++] = from_source[from++];
                if (len > 1) {
                  output[_out++] = from_source[from++];
                }
              }
            }
            else {
              from = _out - dist;          /* copy direct from output */
              do {                        /* minimum length is three */
                output[_out++] = output[from++];
                output[_out++] = output[from++];
                output[_out++] = output[from++];
                len -= 3;
              } while (len > 2);
              if (len) {
                output[_out++] = output[from++];
                if (len > 1) {
                  output[_out++] = output[from++];
                }
              }
            }
          }
          else if ((op & 64) === 0) {          /* 2nd level distance code */
            here = dcode[(here & 0xffff)/*here.val*/ + (hold & ((1 << op) - 1))];
            continue dodist;
          }
          else {
            strm.msg = 'invalid distance code';
            state.mode = BAD;
            break top;
          }

          break; // need to emulate goto via "continue"
        }
      }
      else if ((op & 64) === 0) {              /* 2nd level length code */
        here = lcode[(here & 0xffff)/*here.val*/ + (hold & ((1 << op) - 1))];
        continue dolen;
      }
      else if (op & 32) {                     /* end-of-block */
        //Tracevv((stderr, "inflate:         end of block\n"));
        state.mode = TYPE;
        break top;
      }
      else {
        strm.msg = 'invalid literal/length code';
        state.mode = BAD;
        break top;
      }

      break; // need to emulate goto via "continue"
    }
  } while (_in < last && _out < end);

  /* return unused bytes (on entry, bits < 8, so in won't go too far back) */
  len = bits >> 3;
  _in -= len;
  bits -= len << 3;
  hold &= (1 << bits) - 1;

  /* update state and return */
  strm.next_in = _in;
  strm.next_out = _out;
  strm.avail_in = (_in < last ? 5 + (last - _in) : 5 - (_in - last));
  strm.avail_out = (_out < end ? 257 + (end - _out) : 257 - (_out - end));
  state.hold = hold;
  state.bits = bits;
  return;
};

// (C) 1995-2013 Jean-loup Gailly and Mark Adler
// (C) 2014-2017 Vitaly Puzrin and Andrey Tupitsin
//
// This software is provided 'as-is', without any express or implied
// warranty. In no event will the authors be held liable for any damages
// arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must not
//   claim that you wrote the original software. If you use this software
//   in a product, an acknowledgment in the product documentation would be
//   appreciated but is not required.
// 2. Altered source versions must be plainly marked as such, and must not be
//   misrepresented as being the original software.
// 3. This notice may not be removed or altered from any source distribution.



var MAXBITS = 15;
var ENOUGH_LENS = 852;
var ENOUGH_DISTS = 592;
//var ENOUGH = (ENOUGH_LENS+ENOUGH_DISTS);

var CODES = 0;
var LENS = 1;
var DISTS = 2;

var lbase = [ /* Length codes 257..285 base */
  3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 15, 17, 19, 23, 27, 31,
  35, 43, 51, 59, 67, 83, 99, 115, 131, 163, 195, 227, 258, 0, 0
];

var lext = [ /* Length codes 257..285 extra */
  16, 16, 16, 16, 16, 16, 16, 16, 17, 17, 17, 17, 18, 18, 18, 18,
  19, 19, 19, 19, 20, 20, 20, 20, 21, 21, 21, 21, 16, 72, 78
];

var dbase = [ /* Distance codes 0..29 base */
  1, 2, 3, 4, 5, 7, 9, 13, 17, 25, 33, 49, 65, 97, 129, 193,
  257, 385, 513, 769, 1025, 1537, 2049, 3073, 4097, 6145,
  8193, 12289, 16385, 24577, 0, 0
];

var dext = [ /* Distance codes 0..29 extra */
  16, 16, 16, 16, 17, 17, 18, 18, 19, 19, 20, 20, 21, 21, 22, 22,
  23, 23, 24, 24, 25, 25, 26, 26, 27, 27,
  28, 28, 29, 29, 64, 64
];

var inftrees = function inflate_table(type, lens, lens_index, codes, table, table_index, work, opts)
{
  var bits = opts.bits;
      //here = opts.here; /* table entry for duplication */

  var len = 0;               /* a code's length in bits */
  var sym = 0;               /* index of code symbols */
  var min = 0, max = 0;          /* minimum and maximum code lengths */
  var root = 0;              /* number of index bits for root table */
  var curr = 0;              /* number of index bits for current table */
  var drop = 0;              /* code bits to drop for sub-table */
  var left = 0;                   /* number of prefix codes available */
  var used = 0;              /* code entries in table used */
  var huff = 0;              /* Huffman code */
  var incr;              /* for incrementing code, index */
  var fill;              /* index for replicating entries */
  var low;               /* low bits for current root entry */
  var mask;              /* mask for low root bits */
  var next;             /* next available space in table */
  var base = null;     /* base value table to use */
  var base_index = 0;
//  var shoextra;    /* extra bits table to use */
  var end;                    /* use base and extra for symbol > end */
  var count = new common.Buf16(MAXBITS + 1); //[MAXBITS+1];    /* number of codes of each length */
  var offs = new common.Buf16(MAXBITS + 1); //[MAXBITS+1];     /* offsets in table for each length */
  var extra = null;
  var extra_index = 0;

  var here_bits, here_op, here_val;

  /*
   Process a set of code lengths to create a canonical Huffman code.  The
   code lengths are lens[0..codes-1].  Each length corresponds to the
   symbols 0..codes-1.  The Huffman code is generated by first sorting the
   symbols by length from short to long, and retaining the symbol order
   for codes with equal lengths.  Then the code starts with all zero bits
   for the first code of the shortest length, and the codes are integer
   increments for the same length, and zeros are appended as the length
   increases.  For the deflate format, these bits are stored backwards
   from their more natural integer increment ordering, and so when the
   decoding tables are built in the large loop below, the integer codes
   are incremented backwards.

   This routine assumes, but does not check, that all of the entries in
   lens[] are in the range 0..MAXBITS.  The caller must assure this.
   1..MAXBITS is interpreted as that code length.  zero means that that
   symbol does not occur in this code.

   The codes are sorted by computing a count of codes for each length,
   creating from that a table of starting indices for each length in the
   sorted table, and then entering the symbols in order in the sorted
   table.  The sorted table is work[], with that space being provided by
   the caller.

   The length counts are used for other purposes as well, i.e. finding
   the minimum and maximum length codes, determining if there are any
   codes at all, checking for a valid set of lengths, and looking ahead
   at length counts to determine sub-table sizes when building the
   decoding tables.
   */

  /* accumulate lengths for codes (assumes lens[] all in 0..MAXBITS) */
  for (len = 0; len <= MAXBITS; len++) {
    count[len] = 0;
  }
  for (sym = 0; sym < codes; sym++) {
    count[lens[lens_index + sym]]++;
  }

  /* bound code lengths, force root to be within code lengths */
  root = bits;
  for (max = MAXBITS; max >= 1; max--) {
    if (count[max] !== 0) { break; }
  }
  if (root > max) {
    root = max;
  }
  if (max === 0) {                     /* no symbols to code at all */
    //table.op[opts.table_index] = 64;  //here.op = (var char)64;    /* invalid code marker */
    //table.bits[opts.table_index] = 1;   //here.bits = (var char)1;
    //table.val[opts.table_index++] = 0;   //here.val = (var short)0;
    table[table_index++] = (1 << 24) | (64 << 16) | 0;


    //table.op[opts.table_index] = 64;
    //table.bits[opts.table_index] = 1;
    //table.val[opts.table_index++] = 0;
    table[table_index++] = (1 << 24) | (64 << 16) | 0;

    opts.bits = 1;
    return 0;     /* no symbols, but wait for decoding to report error */
  }
  for (min = 1; min < max; min++) {
    if (count[min] !== 0) { break; }
  }
  if (root < min) {
    root = min;
  }

  /* check for an over-subscribed or incomplete set of lengths */
  left = 1;
  for (len = 1; len <= MAXBITS; len++) {
    left <<= 1;
    left -= count[len];
    if (left < 0) {
      return -1;
    }        /* over-subscribed */
  }
  if (left > 0 && (type === CODES || max !== 1)) {
    return -1;                      /* incomplete set */
  }

  /* generate offsets into symbol table for each length for sorting */
  offs[1] = 0;
  for (len = 1; len < MAXBITS; len++) {
    offs[len + 1] = offs[len] + count[len];
  }

  /* sort symbols by length, by symbol order within each length */
  for (sym = 0; sym < codes; sym++) {
    if (lens[lens_index + sym] !== 0) {
      work[offs[lens[lens_index + sym]]++] = sym;
    }
  }

  /*
   Create and fill in decoding tables.  In this loop, the table being
   filled is at next and has curr index bits.  The code being used is huff
   with length len.  That code is converted to an index by dropping drop
   bits off of the bottom.  For codes where len is less than drop + curr,
   those top drop + curr - len bits are incremented through all values to
   fill the table with replicated entries.

   root is the number of index bits for the root table.  When len exceeds
   root, sub-tables are created pointed to by the root entry with an index
   of the low root bits of huff.  This is saved in low to check for when a
   new sub-table should be started.  drop is zero when the root table is
   being filled, and drop is root when sub-tables are being filled.

   When a new sub-table is needed, it is necessary to look ahead in the
   code lengths to determine what size sub-table is needed.  The length
   counts are used for this, and so count[] is decremented as codes are
   entered in the tables.

   used keeps track of how many table entries have been allocated from the
   provided *table space.  It is checked for LENS and DIST tables against
   the constants ENOUGH_LENS and ENOUGH_DISTS to guard against changes in
   the initial root table size constants.  See the comments in inftrees.h
   for more information.

   sym increments through all symbols, and the loop terminates when
   all codes of length max, i.e. all codes, have been processed.  This
   routine permits incomplete codes, so another loop after this one fills
   in the rest of the decoding tables with invalid code markers.
   */

  /* set up for code type */
  // poor man optimization - use if-else instead of switch,
  // to avoid deopts in old v8
  if (type === CODES) {
    base = extra = work;    /* dummy value--not used */
    end = 19;

  } else if (type === LENS) {
    base = lbase;
    base_index -= 257;
    extra = lext;
    extra_index -= 257;
    end = 256;

  } else {                    /* DISTS */
    base = dbase;
    extra = dext;
    end = -1;
  }

  /* initialize opts for loop */
  huff = 0;                   /* starting code */
  sym = 0;                    /* starting code symbol */
  len = min;                  /* starting code length */
  next = table_index;              /* current table to fill in */
  curr = root;                /* current table index bits */
  drop = 0;                   /* current bits to drop from code for index */
  low = -1;                   /* trigger new sub-table when len > root */
  used = 1 << root;          /* use root table entries */
  mask = used - 1;            /* mask for comparing low */

  /* check available table space */
  if ((type === LENS && used > ENOUGH_LENS) ||
    (type === DISTS && used > ENOUGH_DISTS)) {
    return 1;
  }

  /* process all codes and make table entries */
  for (;;) {
    /* create table entry */
    here_bits = len - drop;
    if (work[sym] < end) {
      here_op = 0;
      here_val = work[sym];
    }
    else if (work[sym] > end) {
      here_op = extra[extra_index + work[sym]];
      here_val = base[base_index + work[sym]];
    }
    else {
      here_op = 32 + 64;         /* end of block */
      here_val = 0;
    }

    /* replicate for those indices with low len bits equal to huff */
    incr = 1 << (len - drop);
    fill = 1 << curr;
    min = fill;                 /* save offset to next table */
    do {
      fill -= incr;
      table[next + (huff >> drop) + fill] = (here_bits << 24) | (here_op << 16) | here_val |0;
    } while (fill !== 0);

    /* backwards increment the len-bit code huff */
    incr = 1 << (len - 1);
    while (huff & incr) {
      incr >>= 1;
    }
    if (incr !== 0) {
      huff &= incr - 1;
      huff += incr;
    } else {
      huff = 0;
    }

    /* go to next symbol, update count, len */
    sym++;
    if (--count[len] === 0) {
      if (len === max) { break; }
      len = lens[lens_index + work[sym]];
    }

    /* create new sub-table if needed */
    if (len > root && (huff & mask) !== low) {
      /* if first time, transition to sub-tables */
      if (drop === 0) {
        drop = root;
      }

      /* increment past last table */
      next += min;            /* here min is 1 << curr */

      /* determine length of next table */
      curr = len - drop;
      left = 1 << curr;
      while (curr + drop < max) {
        left -= count[curr + drop];
        if (left <= 0) { break; }
        curr++;
        left <<= 1;
      }

      /* check for enough space */
      used += 1 << curr;
      if ((type === LENS && used > ENOUGH_LENS) ||
        (type === DISTS && used > ENOUGH_DISTS)) {
        return 1;
      }

      /* point entry in root table to sub-table */
      low = huff & mask;
      /*table.op[low] = curr;
      table.bits[low] = root;
      table.val[low] = next - opts.table_index;*/
      table[low] = (root << 24) | (curr << 16) | (next - table_index) |0;
    }
  }

  /* fill in remaining table entry if code is incomplete (guaranteed to have
   at most one remaining entry, since if the code is incomplete, the
   maximum code length that was allowed to get this far is one bit) */
  if (huff !== 0) {
    //table.op[next + huff] = 64;            /* invalid code marker */
    //table.bits[next + huff] = len - drop;
    //table.val[next + huff] = 0;
    table[next + huff] = ((len - drop) << 24) | (64 << 16) |0;
  }

  /* set return parameters */
  //opts.table_index += used;
  opts.bits = root;
  return 0;
};

// (C) 1995-2013 Jean-loup Gailly and Mark Adler
// (C) 2014-2017 Vitaly Puzrin and Andrey Tupitsin
//
// This software is provided 'as-is', without any express or implied
// warranty. In no event will the authors be held liable for any damages
// arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must not
//   claim that you wrote the original software. If you use this software
//   in a product, an acknowledgment in the product documentation would be
//   appreciated but is not required.
// 2. Altered source versions must be plainly marked as such, and must not be
//   misrepresented as being the original software.
// 3. This notice may not be removed or altered from any source distribution.







var CODES$1 = 0;
var LENS$1 = 1;
var DISTS$1 = 2;

/* Public constants ==========================================================*/
/* ===========================================================================*/


/* Allowed flush values; see deflate() and inflate() below for details */
//var Z_NO_FLUSH      = 0;
//var Z_PARTIAL_FLUSH = 1;
//var Z_SYNC_FLUSH    = 2;
//var Z_FULL_FLUSH    = 3;
var Z_FINISH$2        = 4;
var Z_BLOCK$1         = 5;
var Z_TREES         = 6;


/* Return codes for the compression/decompression functions. Negative values
 * are errors, positive values are used for special but normal events.
 */
var Z_OK$2            = 0;
var Z_STREAM_END$2    = 1;
var Z_NEED_DICT     = 2;
//var Z_ERRNO         = -1;
var Z_STREAM_ERROR$1  = -2;
var Z_DATA_ERROR$1    = -3;
var Z_MEM_ERROR     = -4;
var Z_BUF_ERROR$1     = -5;
//var Z_VERSION_ERROR = -6;

/* The deflate compression method */
var Z_DEFLATED$2  = 8;


/* STATES ====================================================================*/
/* ===========================================================================*/


var    HEAD = 1;       /* i: waiting for magic header */
var    FLAGS = 2;      /* i: waiting for method and flags (gzip) */
var    TIME = 3;       /* i: waiting for modification time (gzip) */
var    OS = 4;         /* i: waiting for extra flags and operating system (gzip) */
var    EXLEN = 5;      /* i: waiting for extra length (gzip) */
var    EXTRA = 6;      /* i: waiting for extra bytes (gzip) */
var    NAME = 7;       /* i: waiting for end of file name (gzip) */
var    COMMENT = 8;    /* i: waiting for end of comment (gzip) */
var    HCRC = 9;       /* i: waiting for header crc (gzip) */
var    DICTID = 10;    /* i: waiting for dictionary check value */
var    DICT = 11;      /* waiting for inflateSetDictionary() call */
var        TYPE$1 = 12;      /* i: waiting for type bits, including last-flag bit */
var        TYPEDO = 13;    /* i: same, but skip check to exit inflate on new block */
var        STORED = 14;    /* i: waiting for stored size (length and complement) */
var        COPY_ = 15;     /* i/o: same as COPY below, but only first time in */
var        COPY = 16;      /* i/o: waiting for input or output to copy stored block */
var        TABLE = 17;     /* i: waiting for dynamic block table lengths */
var        LENLENS = 18;   /* i: waiting for code length code lengths */
var        CODELENS = 19;  /* i: waiting for length/lit and distance code lengths */
var            LEN_ = 20;      /* i: same as LEN below, but only first time in */
var            LEN = 21;       /* i: waiting for length/lit/eob code */
var            LENEXT = 22;    /* i: waiting for length extra bits */
var            DIST = 23;      /* i: waiting for distance code */
var            DISTEXT = 24;   /* i: waiting for distance extra bits */
var            MATCH = 25;     /* o: waiting for output space to copy string */
var            LIT = 26;       /* o: waiting for output space to write literal */
var    CHECK = 27;     /* i: waiting for 32-bit check value */
var    LENGTH = 28;    /* i: waiting for 32-bit length (gzip) */
var    DONE = 29;      /* finished check, done -- remain here until reset */
var    BAD$1 = 30;       /* got a data error -- remain here until reset */
var    MEM = 31;       /* got an inflate() memory error -- remain here until reset */
var    SYNC = 32;      /* looking for synchronization bytes to restart inflate() */

/* ===========================================================================*/



var ENOUGH_LENS$1 = 852;
var ENOUGH_DISTS$1 = 592;
//var ENOUGH =  (ENOUGH_LENS+ENOUGH_DISTS);

var MAX_WBITS$1 = 15;
/* 32K LZ77 window */
var DEF_WBITS = MAX_WBITS$1;


function zswap32(q) {
  return  (((q >>> 24) & 0xff) +
          ((q >>> 8) & 0xff00) +
          ((q & 0xff00) << 8) +
          ((q & 0xff) << 24));
}


function InflateState() {
  this.mode = 0;             /* current inflate mode */
  this.last = false;          /* true if processing last block */
  this.wrap = 0;              /* bit 0 true for zlib, bit 1 true for gzip */
  this.havedict = false;      /* true if dictionary provided */
  this.flags = 0;             /* gzip header method and flags (0 if zlib) */
  this.dmax = 0;              /* zlib header max distance (INFLATE_STRICT) */
  this.check = 0;             /* protected copy of check value */
  this.total = 0;             /* protected copy of output count */
  // TODO: may be {}
  this.head = null;           /* where to save gzip header information */

  /* sliding window */
  this.wbits = 0;             /* log base 2 of requested window size */
  this.wsize = 0;             /* window size or zero if not using window */
  this.whave = 0;             /* valid bytes in the window */
  this.wnext = 0;             /* window write index */
  this.window = null;         /* allocated sliding window, if needed */

  /* bit accumulator */
  this.hold = 0;              /* input bit accumulator */
  this.bits = 0;              /* number of bits in "in" */

  /* for string and stored block copying */
  this.length = 0;            /* literal or length of data to copy */
  this.offset = 0;            /* distance back to copy string from */

  /* for table and code decoding */
  this.extra = 0;             /* extra bits needed */

  /* fixed and dynamic code tables */
  this.lencode = null;          /* starting table for length/literal codes */
  this.distcode = null;         /* starting table for distance codes */
  this.lenbits = 0;           /* index bits for lencode */
  this.distbits = 0;          /* index bits for distcode */

  /* dynamic table building */
  this.ncode = 0;             /* number of code length code lengths */
  this.nlen = 0;              /* number of length code lengths */
  this.ndist = 0;             /* number of distance code lengths */
  this.have = 0;              /* number of code lengths in lens[] */
  this.next = null;              /* next available space in codes[] */

  this.lens = new common.Buf16(320); /* temporary storage for code lengths */
  this.work = new common.Buf16(288); /* work area for code table building */

  /*
   because we don't have pointers in js, we use lencode and distcode directly
   as buffers so we don't need codes
  */
  //this.codes = new utils.Buf32(ENOUGH);       /* space for code tables */
  this.lendyn = null;              /* dynamic table for length/literal codes (JS specific) */
  this.distdyn = null;             /* dynamic table for distance codes (JS specific) */
  this.sane = 0;                   /* if false, allow invalid distance too far */
  this.back = 0;                   /* bits back of last unprocessed length/lit */
  this.was = 0;                    /* initial length of match */
}

function inflateResetKeep(strm) {
  var state;

  if (!strm || !strm.state) { return Z_STREAM_ERROR$1; }
  state = strm.state;
  strm.total_in = strm.total_out = state.total = 0;
  strm.msg = ''; /*Z_NULL*/
  if (state.wrap) {       /* to support ill-conceived Java test suite */
    strm.adler = state.wrap & 1;
  }
  state.mode = HEAD;
  state.last = 0;
  state.havedict = 0;
  state.dmax = 32768;
  state.head = null/*Z_NULL*/;
  state.hold = 0;
  state.bits = 0;
  //state.lencode = state.distcode = state.next = state.codes;
  state.lencode = state.lendyn = new common.Buf32(ENOUGH_LENS$1);
  state.distcode = state.distdyn = new common.Buf32(ENOUGH_DISTS$1);

  state.sane = 1;
  state.back = -1;
  //Tracev((stderr, "inflate: reset\n"));
  return Z_OK$2;
}

function inflateReset(strm) {
  var state;

  if (!strm || !strm.state) { return Z_STREAM_ERROR$1; }
  state = strm.state;
  state.wsize = 0;
  state.whave = 0;
  state.wnext = 0;
  return inflateResetKeep(strm);

}

function inflateReset2(strm, windowBits) {
  var wrap;
  var state;

  /* get the state */
  if (!strm || !strm.state) { return Z_STREAM_ERROR$1; }
  state = strm.state;

  /* extract wrap request from windowBits parameter */
  if (windowBits < 0) {
    wrap = 0;
    windowBits = -windowBits;
  }
  else {
    wrap = (windowBits >> 4) + 1;
    if (windowBits < 48) {
      windowBits &= 15;
    }
  }

  /* set number of window bits, free window if different */
  if (windowBits && (windowBits < 8 || windowBits > 15)) {
    return Z_STREAM_ERROR$1;
  }
  if (state.window !== null && state.wbits !== windowBits) {
    state.window = null;
  }

  /* update state and reset the rest of it */
  state.wrap = wrap;
  state.wbits = windowBits;
  return inflateReset(strm);
}

function inflateInit2(strm, windowBits) {
  var ret;
  var state;

  if (!strm) { return Z_STREAM_ERROR$1; }
  //strm.msg = Z_NULL;                 /* in case we return an error */

  state = new InflateState();

  //if (state === Z_NULL) return Z_MEM_ERROR;
  //Tracev((stderr, "inflate: allocated\n"));
  strm.state = state;
  state.window = null/*Z_NULL*/;
  ret = inflateReset2(strm, windowBits);
  if (ret !== Z_OK$2) {
    strm.state = null/*Z_NULL*/;
  }
  return ret;
}

function inflateInit(strm) {
  return inflateInit2(strm, DEF_WBITS);
}


/*
 Return state with length and distance decoding tables and index sizes set to
 fixed code decoding.  Normally this returns fixed tables from inffixed.h.
 If BUILDFIXED is defined, then instead this routine builds the tables the
 first time it's called, and returns those tables the first time and
 thereafter.  This reduces the size of the code by about 2K bytes, in
 exchange for a little execution time.  However, BUILDFIXED should not be
 used for threaded applications, since the rewriting of the tables and virgin
 may not be thread-safe.
 */
var virgin = true;

var lenfix, distfix; // We have no pointers in JS, so keep tables separate

function fixedtables(state) {
  /* build fixed huffman tables if first call (may not be thread safe) */
  if (virgin) {
    var sym;

    lenfix = new common.Buf32(512);
    distfix = new common.Buf32(32);

    /* literal/length table */
    sym = 0;
    while (sym < 144) { state.lens[sym++] = 8; }
    while (sym < 256) { state.lens[sym++] = 9; }
    while (sym < 280) { state.lens[sym++] = 7; }
    while (sym < 288) { state.lens[sym++] = 8; }

    inftrees(LENS$1,  state.lens, 0, 288, lenfix,   0, state.work, { bits: 9 });

    /* distance table */
    sym = 0;
    while (sym < 32) { state.lens[sym++] = 5; }

    inftrees(DISTS$1, state.lens, 0, 32,   distfix, 0, state.work, { bits: 5 });

    /* do this just once */
    virgin = false;
  }

  state.lencode = lenfix;
  state.lenbits = 9;
  state.distcode = distfix;
  state.distbits = 5;
}


/*
 Update the window with the last wsize (normally 32K) bytes written before
 returning.  If window does not exist yet, create it.  This is only called
 when a window is already in use, or when output has been written during this
 inflate call, but the end of the deflate stream has not been reached yet.
 It is also called to create a window for dictionary data when a dictionary
 is loaded.

 Providing output buffers larger than 32K to inflate() should provide a speed
 advantage, since only the last 32K of output is copied to the sliding window
 upon return from inflate(), and since all distances after the first 32K of
 output will fall in the output data, making match copies simpler and faster.
 The advantage may be dependent on the size of the processor's data caches.
 */
function updatewindow(strm, src, end, copy) {
  var dist;
  var state = strm.state;

  /* if it hasn't been done already, allocate space for the window */
  if (state.window === null) {
    state.wsize = 1 << state.wbits;
    state.wnext = 0;
    state.whave = 0;

    state.window = new common.Buf8(state.wsize);
  }

  /* copy state->wsize or less output bytes into the circular window */
  if (copy >= state.wsize) {
    common.arraySet(state.window, src, end - state.wsize, state.wsize, 0);
    state.wnext = 0;
    state.whave = state.wsize;
  }
  else {
    dist = state.wsize - state.wnext;
    if (dist > copy) {
      dist = copy;
    }
    //zmemcpy(state->window + state->wnext, end - copy, dist);
    common.arraySet(state.window, src, end - copy, dist, state.wnext);
    copy -= dist;
    if (copy) {
      //zmemcpy(state->window, end - copy, copy);
      common.arraySet(state.window, src, end - copy, copy, 0);
      state.wnext = copy;
      state.whave = state.wsize;
    }
    else {
      state.wnext += dist;
      if (state.wnext === state.wsize) { state.wnext = 0; }
      if (state.whave < state.wsize) { state.whave += dist; }
    }
  }
  return 0;
}

function inflate(strm, flush) {
  var state;
  var input, output;          // input/output buffers
  var next;                   /* next input INDEX */
  var put;                    /* next output INDEX */
  var have, left;             /* available input and output */
  var hold;                   /* bit buffer */
  var bits;                   /* bits in bit buffer */
  var _in, _out;              /* save starting available input and output */
  var copy;                   /* number of stored or match bytes to copy */
  var from;                   /* where to copy match bytes from */
  var from_source;
  var here = 0;               /* current decoding table entry */
  var here_bits, here_op, here_val; // paked "here" denormalized (JS specific)
  //var last;                   /* parent table entry */
  var last_bits, last_op, last_val; // paked "last" denormalized (JS specific)
  var len;                    /* length to copy for repeats, bits to drop */
  var ret;                    /* return code */
  var hbuf = new common.Buf8(4);    /* buffer for gzip header crc calculation */
  var opts;

  var n; // temporary var for NEED_BITS

  var order = /* permutation of code lengths */
    [ 16, 17, 18, 0, 8, 7, 9, 6, 10, 5, 11, 4, 12, 3, 13, 2, 14, 1, 15 ];


  if (!strm || !strm.state || !strm.output ||
      (!strm.input && strm.avail_in !== 0)) {
    return Z_STREAM_ERROR$1;
  }

  state = strm.state;
  if (state.mode === TYPE$1) { state.mode = TYPEDO; }    /* skip check */


  //--- LOAD() ---
  put = strm.next_out;
  output = strm.output;
  left = strm.avail_out;
  next = strm.next_in;
  input = strm.input;
  have = strm.avail_in;
  hold = state.hold;
  bits = state.bits;
  //---

  _in = have;
  _out = left;
  ret = Z_OK$2;

  inf_leave: // goto emulation
  for (;;) {
    switch (state.mode) {
      case HEAD:
        if (state.wrap === 0) {
          state.mode = TYPEDO;
          break;
        }
        //=== NEEDBITS(16);
        while (bits < 16) {
          if (have === 0) { break inf_leave; }
          have--;
          hold += input[next++] << bits;
          bits += 8;
        }
        //===//
        if ((state.wrap & 2) && hold === 0x8b1f) {  /* gzip header */
          state.check = 0/*crc32(0L, Z_NULL, 0)*/;
          //=== CRC2(state.check, hold);
          hbuf[0] = hold & 0xff;
          hbuf[1] = (hold >>> 8) & 0xff;
          state.check = crc32_1$1(state.check, hbuf, 2, 0);
          //===//

          //=== INITBITS();
          hold = 0;
          bits = 0;
          //===//
          state.mode = FLAGS;
          break;
        }
        state.flags = 0;           /* expect zlib header */
        if (state.head) {
          state.head.done = false;
        }
        if (!(state.wrap & 1) ||   /* check if zlib header allowed */
          (((hold & 0xff)/*BITS(8)*/ << 8) + (hold >> 8)) % 31) {
          strm.msg = 'incorrect header check';
          state.mode = BAD$1;
          break;
        }
        if ((hold & 0x0f)/*BITS(4)*/ !== Z_DEFLATED$2) {
          strm.msg = 'unknown compression method';
          state.mode = BAD$1;
          break;
        }
        //--- DROPBITS(4) ---//
        hold >>>= 4;
        bits -= 4;
        //---//
        len = (hold & 0x0f)/*BITS(4)*/ + 8;
        if (state.wbits === 0) {
          state.wbits = len;
        }
        else if (len > state.wbits) {
          strm.msg = 'invalid window size';
          state.mode = BAD$1;
          break;
        }
        state.dmax = 1 << len;
        //Tracev((stderr, "inflate:   zlib header ok\n"));
        strm.adler = state.check = 1/*adler32(0L, Z_NULL, 0)*/;
        state.mode = hold & 0x200 ? DICTID : TYPE$1;
        //=== INITBITS();
        hold = 0;
        bits = 0;
        //===//
        break;
      case FLAGS:
        //=== NEEDBITS(16); */
        while (bits < 16) {
          if (have === 0) { break inf_leave; }
          have--;
          hold += input[next++] << bits;
          bits += 8;
        }
        //===//
        state.flags = hold;
        if ((state.flags & 0xff) !== Z_DEFLATED$2) {
          strm.msg = 'unknown compression method';
          state.mode = BAD$1;
          break;
        }
        if (state.flags & 0xe000) {
          strm.msg = 'unknown header flags set';
          state.mode = BAD$1;
          break;
        }
        if (state.head) {
          state.head.text = ((hold >> 8) & 1);
        }
        if (state.flags & 0x0200) {
          //=== CRC2(state.check, hold);
          hbuf[0] = hold & 0xff;
          hbuf[1] = (hold >>> 8) & 0xff;
          state.check = crc32_1$1(state.check, hbuf, 2, 0);
          //===//
        }
        //=== INITBITS();
        hold = 0;
        bits = 0;
        //===//
        state.mode = TIME;
        /* falls through */
      case TIME:
        //=== NEEDBITS(32); */
        while (bits < 32) {
          if (have === 0) { break inf_leave; }
          have--;
          hold += input[next++] << bits;
          bits += 8;
        }
        //===//
        if (state.head) {
          state.head.time = hold;
        }
        if (state.flags & 0x0200) {
          //=== CRC4(state.check, hold)
          hbuf[0] = hold & 0xff;
          hbuf[1] = (hold >>> 8) & 0xff;
          hbuf[2] = (hold >>> 16) & 0xff;
          hbuf[3] = (hold >>> 24) & 0xff;
          state.check = crc32_1$1(state.check, hbuf, 4, 0);
          //===
        }
        //=== INITBITS();
        hold = 0;
        bits = 0;
        //===//
        state.mode = OS;
        /* falls through */
      case OS:
        //=== NEEDBITS(16); */
        while (bits < 16) {
          if (have === 0) { break inf_leave; }
          have--;
          hold += input[next++] << bits;
          bits += 8;
        }
        //===//
        if (state.head) {
          state.head.xflags = (hold & 0xff);
          state.head.os = (hold >> 8);
        }
        if (state.flags & 0x0200) {
          //=== CRC2(state.check, hold);
          hbuf[0] = hold & 0xff;
          hbuf[1] = (hold >>> 8) & 0xff;
          state.check = crc32_1$1(state.check, hbuf, 2, 0);
          //===//
        }
        //=== INITBITS();
        hold = 0;
        bits = 0;
        //===//
        state.mode = EXLEN;
        /* falls through */
      case EXLEN:
        if (state.flags & 0x0400) {
          //=== NEEDBITS(16); */
          while (bits < 16) {
            if (have === 0) { break inf_leave; }
            have--;
            hold += input[next++] << bits;
            bits += 8;
          }
          //===//
          state.length = hold;
          if (state.head) {
            state.head.extra_len = hold;
          }
          if (state.flags & 0x0200) {
            //=== CRC2(state.check, hold);
            hbuf[0] = hold & 0xff;
            hbuf[1] = (hold >>> 8) & 0xff;
            state.check = crc32_1$1(state.check, hbuf, 2, 0);
            //===//
          }
          //=== INITBITS();
          hold = 0;
          bits = 0;
          //===//
        }
        else if (state.head) {
          state.head.extra = null/*Z_NULL*/;
        }
        state.mode = EXTRA;
        /* falls through */
      case EXTRA:
        if (state.flags & 0x0400) {
          copy = state.length;
          if (copy > have) { copy = have; }
          if (copy) {
            if (state.head) {
              len = state.head.extra_len - state.length;
              if (!state.head.extra) {
                // Use untyped array for more convenient processing later
                state.head.extra = new Array(state.head.extra_len);
              }
              common.arraySet(
                state.head.extra,
                input,
                next,
                // extra field is limited to 65536 bytes
                // - no need for additional size check
                copy,
                /*len + copy > state.head.extra_max - len ? state.head.extra_max : copy,*/
                len
              );
              //zmemcpy(state.head.extra + len, next,
              //        len + copy > state.head.extra_max ?
              //        state.head.extra_max - len : copy);
            }
            if (state.flags & 0x0200) {
              state.check = crc32_1$1(state.check, input, copy, next);
            }
            have -= copy;
            next += copy;
            state.length -= copy;
          }
          if (state.length) { break inf_leave; }
        }
        state.length = 0;
        state.mode = NAME;
        /* falls through */
      case NAME:
        if (state.flags & 0x0800) {
          if (have === 0) { break inf_leave; }
          copy = 0;
          do {
            // TODO: 2 or 1 bytes?
            len = input[next + copy++];
            /* use constant limit because in js we should not preallocate memory */
            if (state.head && len &&
                (state.length < 65536 /*state.head.name_max*/)) {
              state.head.name += String.fromCharCode(len);
            }
          } while (len && copy < have);

          if (state.flags & 0x0200) {
            state.check = crc32_1$1(state.check, input, copy, next);
          }
          have -= copy;
          next += copy;
          if (len) { break inf_leave; }
        }
        else if (state.head) {
          state.head.name = null;
        }
        state.length = 0;
        state.mode = COMMENT;
        /* falls through */
      case COMMENT:
        if (state.flags & 0x1000) {
          if (have === 0) { break inf_leave; }
          copy = 0;
          do {
            len = input[next + copy++];
            /* use constant limit because in js we should not preallocate memory */
            if (state.head && len &&
                (state.length < 65536 /*state.head.comm_max*/)) {
              state.head.comment += String.fromCharCode(len);
            }
          } while (len && copy < have);
          if (state.flags & 0x0200) {
            state.check = crc32_1$1(state.check, input, copy, next);
          }
          have -= copy;
          next += copy;
          if (len) { break inf_leave; }
        }
        else if (state.head) {
          state.head.comment = null;
        }
        state.mode = HCRC;
        /* falls through */
      case HCRC:
        if (state.flags & 0x0200) {
          //=== NEEDBITS(16); */
          while (bits < 16) {
            if (have === 0) { break inf_leave; }
            have--;
            hold += input[next++] << bits;
            bits += 8;
          }
          //===//
          if (hold !== (state.check & 0xffff)) {
            strm.msg = 'header crc mismatch';
            state.mode = BAD$1;
            break;
          }
          //=== INITBITS();
          hold = 0;
          bits = 0;
          //===//
        }
        if (state.head) {
          state.head.hcrc = ((state.flags >> 9) & 1);
          state.head.done = true;
        }
        strm.adler = state.check = 0;
        state.mode = TYPE$1;
        break;
      case DICTID:
        //=== NEEDBITS(32); */
        while (bits < 32) {
          if (have === 0) { break inf_leave; }
          have--;
          hold += input[next++] << bits;
          bits += 8;
        }
        //===//
        strm.adler = state.check = zswap32(hold);
        //=== INITBITS();
        hold = 0;
        bits = 0;
        //===//
        state.mode = DICT;
        /* falls through */
      case DICT:
        if (state.havedict === 0) {
          //--- RESTORE() ---
          strm.next_out = put;
          strm.avail_out = left;
          strm.next_in = next;
          strm.avail_in = have;
          state.hold = hold;
          state.bits = bits;
          //---
          return Z_NEED_DICT;
        }
        strm.adler = state.check = 1/*adler32(0L, Z_NULL, 0)*/;
        state.mode = TYPE$1;
        /* falls through */
      case TYPE$1:
        if (flush === Z_BLOCK$1 || flush === Z_TREES) { break inf_leave; }
        /* falls through */
      case TYPEDO:
        if (state.last) {
          //--- BYTEBITS() ---//
          hold >>>= bits & 7;
          bits -= bits & 7;
          //---//
          state.mode = CHECK;
          break;
        }
        //=== NEEDBITS(3); */
        while (bits < 3) {
          if (have === 0) { break inf_leave; }
          have--;
          hold += input[next++] << bits;
          bits += 8;
        }
        //===//
        state.last = (hold & 0x01)/*BITS(1)*/;
        //--- DROPBITS(1) ---//
        hold >>>= 1;
        bits -= 1;
        //---//

        switch ((hold & 0x03)/*BITS(2)*/) {
          case 0:                             /* stored block */
            //Tracev((stderr, "inflate:     stored block%s\n",
            //        state.last ? " (last)" : ""));
            state.mode = STORED;
            break;
          case 1:                             /* fixed block */
            fixedtables(state);
            //Tracev((stderr, "inflate:     fixed codes block%s\n",
            //        state.last ? " (last)" : ""));
            state.mode = LEN_;             /* decode codes */
            if (flush === Z_TREES) {
              //--- DROPBITS(2) ---//
              hold >>>= 2;
              bits -= 2;
              //---//
              break inf_leave;
            }
            break;
          case 2:                             /* dynamic block */
            //Tracev((stderr, "inflate:     dynamic codes block%s\n",
            //        state.last ? " (last)" : ""));
            state.mode = TABLE;
            break;
          case 3:
            strm.msg = 'invalid block type';
            state.mode = BAD$1;
        }
        //--- DROPBITS(2) ---//
        hold >>>= 2;
        bits -= 2;
        //---//
        break;
      case STORED:
        //--- BYTEBITS() ---// /* go to byte boundary */
        hold >>>= bits & 7;
        bits -= bits & 7;
        //---//
        //=== NEEDBITS(32); */
        while (bits < 32) {
          if (have === 0) { break inf_leave; }
          have--;
          hold += input[next++] << bits;
          bits += 8;
        }
        //===//
        if ((hold & 0xffff) !== ((hold >>> 16) ^ 0xffff)) {
          strm.msg = 'invalid stored block lengths';
          state.mode = BAD$1;
          break;
        }
        state.length = hold & 0xffff;
        //Tracev((stderr, "inflate:       stored length %u\n",
        //        state.length));
        //=== INITBITS();
        hold = 0;
        bits = 0;
        //===//
        state.mode = COPY_;
        if (flush === Z_TREES) { break inf_leave; }
        /* falls through */
      case COPY_:
        state.mode = COPY;
        /* falls through */
      case COPY:
        copy = state.length;
        if (copy) {
          if (copy > have) { copy = have; }
          if (copy > left) { copy = left; }
          if (copy === 0) { break inf_leave; }
          //--- zmemcpy(put, next, copy); ---
          common.arraySet(output, input, next, copy, put);
          //---//
          have -= copy;
          next += copy;
          left -= copy;
          put += copy;
          state.length -= copy;
          break;
        }
        //Tracev((stderr, "inflate:       stored end\n"));
        state.mode = TYPE$1;
        break;
      case TABLE:
        //=== NEEDBITS(14); */
        while (bits < 14) {
          if (have === 0) { break inf_leave; }
          have--;
          hold += input[next++] << bits;
          bits += 8;
        }
        //===//
        state.nlen = (hold & 0x1f)/*BITS(5)*/ + 257;
        //--- DROPBITS(5) ---//
        hold >>>= 5;
        bits -= 5;
        //---//
        state.ndist = (hold & 0x1f)/*BITS(5)*/ + 1;
        //--- DROPBITS(5) ---//
        hold >>>= 5;
        bits -= 5;
        //---//
        state.ncode = (hold & 0x0f)/*BITS(4)*/ + 4;
        //--- DROPBITS(4) ---//
        hold >>>= 4;
        bits -= 4;
        //---//
//#ifndef PKZIP_BUG_WORKAROUND
        if (state.nlen > 286 || state.ndist > 30) {
          strm.msg = 'too many length or distance symbols';
          state.mode = BAD$1;
          break;
        }
//#endif
        //Tracev((stderr, "inflate:       table sizes ok\n"));
        state.have = 0;
        state.mode = LENLENS;
        /* falls through */
      case LENLENS:
        while (state.have < state.ncode) {
          //=== NEEDBITS(3);
          while (bits < 3) {
            if (have === 0) { break inf_leave; }
            have--;
            hold += input[next++] << bits;
            bits += 8;
          }
          //===//
          state.lens[order[state.have++]] = (hold & 0x07);//BITS(3);
          //--- DROPBITS(3) ---//
          hold >>>= 3;
          bits -= 3;
          //---//
        }
        while (state.have < 19) {
          state.lens[order[state.have++]] = 0;
        }
        // We have separate tables & no pointers. 2 commented lines below not needed.
        //state.next = state.codes;
        //state.lencode = state.next;
        // Switch to use dynamic table
        state.lencode = state.lendyn;
        state.lenbits = 7;

        opts = { bits: state.lenbits };
        ret = inftrees(CODES$1, state.lens, 0, 19, state.lencode, 0, state.work, opts);
        state.lenbits = opts.bits;

        if (ret) {
          strm.msg = 'invalid code lengths set';
          state.mode = BAD$1;
          break;
        }
        //Tracev((stderr, "inflate:       code lengths ok\n"));
        state.have = 0;
        state.mode = CODELENS;
        /* falls through */
      case CODELENS:
        while (state.have < state.nlen + state.ndist) {
          for (;;) {
            here = state.lencode[hold & ((1 << state.lenbits) - 1)];/*BITS(state.lenbits)*/
            here_bits = here >>> 24;
            here_op = (here >>> 16) & 0xff;
            here_val = here & 0xffff;

            if ((here_bits) <= bits) { break; }
            //--- PULLBYTE() ---//
            if (have === 0) { break inf_leave; }
            have--;
            hold += input[next++] << bits;
            bits += 8;
            //---//
          }
          if (here_val < 16) {
            //--- DROPBITS(here.bits) ---//
            hold >>>= here_bits;
            bits -= here_bits;
            //---//
            state.lens[state.have++] = here_val;
          }
          else {
            if (here_val === 16) {
              //=== NEEDBITS(here.bits + 2);
              n = here_bits + 2;
              while (bits < n) {
                if (have === 0) { break inf_leave; }
                have--;
                hold += input[next++] << bits;
                bits += 8;
              }
              //===//
              //--- DROPBITS(here.bits) ---//
              hold >>>= here_bits;
              bits -= here_bits;
              //---//
              if (state.have === 0) {
                strm.msg = 'invalid bit length repeat';
                state.mode = BAD$1;
                break;
              }
              len = state.lens[state.have - 1];
              copy = 3 + (hold & 0x03);//BITS(2);
              //--- DROPBITS(2) ---//
              hold >>>= 2;
              bits -= 2;
              //---//
            }
            else if (here_val === 17) {
              //=== NEEDBITS(here.bits + 3);
              n = here_bits + 3;
              while (bits < n) {
                if (have === 0) { break inf_leave; }
                have--;
                hold += input[next++] << bits;
                bits += 8;
              }
              //===//
              //--- DROPBITS(here.bits) ---//
              hold >>>= here_bits;
              bits -= here_bits;
              //---//
              len = 0;
              copy = 3 + (hold & 0x07);//BITS(3);
              //--- DROPBITS(3) ---//
              hold >>>= 3;
              bits -= 3;
              //---//
            }
            else {
              //=== NEEDBITS(here.bits + 7);
              n = here_bits + 7;
              while (bits < n) {
                if (have === 0) { break inf_leave; }
                have--;
                hold += input[next++] << bits;
                bits += 8;
              }
              //===//
              //--- DROPBITS(here.bits) ---//
              hold >>>= here_bits;
              bits -= here_bits;
              //---//
              len = 0;
              copy = 11 + (hold & 0x7f);//BITS(7);
              //--- DROPBITS(7) ---//
              hold >>>= 7;
              bits -= 7;
              //---//
            }
            if (state.have + copy > state.nlen + state.ndist) {
              strm.msg = 'invalid bit length repeat';
              state.mode = BAD$1;
              break;
            }
            while (copy--) {
              state.lens[state.have++] = len;
            }
          }
        }

        /* handle error breaks in while */
        if (state.mode === BAD$1) { break; }

        /* check for end-of-block code (better have one) */
        if (state.lens[256] === 0) {
          strm.msg = 'invalid code -- missing end-of-block';
          state.mode = BAD$1;
          break;
        }

        /* build code tables -- note: do not change the lenbits or distbits
           values here (9 and 6) without reading the comments in inftrees.h
           concerning the ENOUGH constants, which depend on those values */
        state.lenbits = 9;

        opts = { bits: state.lenbits };
        ret = inftrees(LENS$1, state.lens, 0, state.nlen, state.lencode, 0, state.work, opts);
        // We have separate tables & no pointers. 2 commented lines below not needed.
        // state.next_index = opts.table_index;
        state.lenbits = opts.bits;
        // state.lencode = state.next;

        if (ret) {
          strm.msg = 'invalid literal/lengths set';
          state.mode = BAD$1;
          break;
        }

        state.distbits = 6;
        //state.distcode.copy(state.codes);
        // Switch to use dynamic table
        state.distcode = state.distdyn;
        opts = { bits: state.distbits };
        ret = inftrees(DISTS$1, state.lens, state.nlen, state.ndist, state.distcode, 0, state.work, opts);
        // We have separate tables & no pointers. 2 commented lines below not needed.
        // state.next_index = opts.table_index;
        state.distbits = opts.bits;
        // state.distcode = state.next;

        if (ret) {
          strm.msg = 'invalid distances set';
          state.mode = BAD$1;
          break;
        }
        //Tracev((stderr, 'inflate:       codes ok\n'));
        state.mode = LEN_;
        if (flush === Z_TREES) { break inf_leave; }
        /* falls through */
      case LEN_:
        state.mode = LEN;
        /* falls through */
      case LEN:
        if (have >= 6 && left >= 258) {
          //--- RESTORE() ---
          strm.next_out = put;
          strm.avail_out = left;
          strm.next_in = next;
          strm.avail_in = have;
          state.hold = hold;
          state.bits = bits;
          //---
          inffast(strm, _out);
          //--- LOAD() ---
          put = strm.next_out;
          output = strm.output;
          left = strm.avail_out;
          next = strm.next_in;
          input = strm.input;
          have = strm.avail_in;
          hold = state.hold;
          bits = state.bits;
          //---

          if (state.mode === TYPE$1) {
            state.back = -1;
          }
          break;
        }
        state.back = 0;
        for (;;) {
          here = state.lencode[hold & ((1 << state.lenbits) - 1)];  /*BITS(state.lenbits)*/
          here_bits = here >>> 24;
          here_op = (here >>> 16) & 0xff;
          here_val = here & 0xffff;

          if (here_bits <= bits) { break; }
          //--- PULLBYTE() ---//
          if (have === 0) { break inf_leave; }
          have--;
          hold += input[next++] << bits;
          bits += 8;
          //---//
        }
        if (here_op && (here_op & 0xf0) === 0) {
          last_bits = here_bits;
          last_op = here_op;
          last_val = here_val;
          for (;;) {
            here = state.lencode[last_val +
                    ((hold & ((1 << (last_bits + last_op)) - 1))/*BITS(last.bits + last.op)*/ >> last_bits)];
            here_bits = here >>> 24;
            here_op = (here >>> 16) & 0xff;
            here_val = here & 0xffff;

            if ((last_bits + here_bits) <= bits) { break; }
            //--- PULLBYTE() ---//
            if (have === 0) { break inf_leave; }
            have--;
            hold += input[next++] << bits;
            bits += 8;
            //---//
          }
          //--- DROPBITS(last.bits) ---//
          hold >>>= last_bits;
          bits -= last_bits;
          //---//
          state.back += last_bits;
        }
        //--- DROPBITS(here.bits) ---//
        hold >>>= here_bits;
        bits -= here_bits;
        //---//
        state.back += here_bits;
        state.length = here_val;
        if (here_op === 0) {
          //Tracevv((stderr, here.val >= 0x20 && here.val < 0x7f ?
          //        "inflate:         literal '%c'\n" :
          //        "inflate:         literal 0x%02x\n", here.val));
          state.mode = LIT;
          break;
        }
        if (here_op & 32) {
          //Tracevv((stderr, "inflate:         end of block\n"));
          state.back = -1;
          state.mode = TYPE$1;
          break;
        }
        if (here_op & 64) {
          strm.msg = 'invalid literal/length code';
          state.mode = BAD$1;
          break;
        }
        state.extra = here_op & 15;
        state.mode = LENEXT;
        /* falls through */
      case LENEXT:
        if (state.extra) {
          //=== NEEDBITS(state.extra);
          n = state.extra;
          while (bits < n) {
            if (have === 0) { break inf_leave; }
            have--;
            hold += input[next++] << bits;
            bits += 8;
          }
          //===//
          state.length += hold & ((1 << state.extra) - 1)/*BITS(state.extra)*/;
          //--- DROPBITS(state.extra) ---//
          hold >>>= state.extra;
          bits -= state.extra;
          //---//
          state.back += state.extra;
        }
        //Tracevv((stderr, "inflate:         length %u\n", state.length));
        state.was = state.length;
        state.mode = DIST;
        /* falls through */
      case DIST:
        for (;;) {
          here = state.distcode[hold & ((1 << state.distbits) - 1)];/*BITS(state.distbits)*/
          here_bits = here >>> 24;
          here_op = (here >>> 16) & 0xff;
          here_val = here & 0xffff;

          if ((here_bits) <= bits) { break; }
          //--- PULLBYTE() ---//
          if (have === 0) { break inf_leave; }
          have--;
          hold += input[next++] << bits;
          bits += 8;
          //---//
        }
        if ((here_op & 0xf0) === 0) {
          last_bits = here_bits;
          last_op = here_op;
          last_val = here_val;
          for (;;) {
            here = state.distcode[last_val +
                    ((hold & ((1 << (last_bits + last_op)) - 1))/*BITS(last.bits + last.op)*/ >> last_bits)];
            here_bits = here >>> 24;
            here_op = (here >>> 16) & 0xff;
            here_val = here & 0xffff;

            if ((last_bits + here_bits) <= bits) { break; }
            //--- PULLBYTE() ---//
            if (have === 0) { break inf_leave; }
            have--;
            hold += input[next++] << bits;
            bits += 8;
            //---//
          }
          //--- DROPBITS(last.bits) ---//
          hold >>>= last_bits;
          bits -= last_bits;
          //---//
          state.back += last_bits;
        }
        //--- DROPBITS(here.bits) ---//
        hold >>>= here_bits;
        bits -= here_bits;
        //---//
        state.back += here_bits;
        if (here_op & 64) {
          strm.msg = 'invalid distance code';
          state.mode = BAD$1;
          break;
        }
        state.offset = here_val;
        state.extra = (here_op) & 15;
        state.mode = DISTEXT;
        /* falls through */
      case DISTEXT:
        if (state.extra) {
          //=== NEEDBITS(state.extra);
          n = state.extra;
          while (bits < n) {
            if (have === 0) { break inf_leave; }
            have--;
            hold += input[next++] << bits;
            bits += 8;
          }
          //===//
          state.offset += hold & ((1 << state.extra) - 1)/*BITS(state.extra)*/;
          //--- DROPBITS(state.extra) ---//
          hold >>>= state.extra;
          bits -= state.extra;
          //---//
          state.back += state.extra;
        }
//#ifdef INFLATE_STRICT
        if (state.offset > state.dmax) {
          strm.msg = 'invalid distance too far back';
          state.mode = BAD$1;
          break;
        }
//#endif
        //Tracevv((stderr, "inflate:         distance %u\n", state.offset));
        state.mode = MATCH;
        /* falls through */
      case MATCH:
        if (left === 0) { break inf_leave; }
        copy = _out - left;
        if (state.offset > copy) {         /* copy from window */
          copy = state.offset - copy;
          if (copy > state.whave) {
            if (state.sane) {
              strm.msg = 'invalid distance too far back';
              state.mode = BAD$1;
              break;
            }
// (!) This block is disabled in zlib defaults,
// don't enable it for binary compatibility
//#ifdef INFLATE_ALLOW_INVALID_DISTANCE_TOOFAR_ARRR
//          Trace((stderr, "inflate.c too far\n"));
//          copy -= state.whave;
//          if (copy > state.length) { copy = state.length; }
//          if (copy > left) { copy = left; }
//          left -= copy;
//          state.length -= copy;
//          do {
//            output[put++] = 0;
//          } while (--copy);
//          if (state.length === 0) { state.mode = LEN; }
//          break;
//#endif
          }
          if (copy > state.wnext) {
            copy -= state.wnext;
            from = state.wsize - copy;
          }
          else {
            from = state.wnext - copy;
          }
          if (copy > state.length) { copy = state.length; }
          from_source = state.window;
        }
        else {                              /* copy from output */
          from_source = output;
          from = put - state.offset;
          copy = state.length;
        }
        if (copy > left) { copy = left; }
        left -= copy;
        state.length -= copy;
        do {
          output[put++] = from_source[from++];
        } while (--copy);
        if (state.length === 0) { state.mode = LEN; }
        break;
      case LIT:
        if (left === 0) { break inf_leave; }
        output[put++] = state.length;
        left--;
        state.mode = LEN;
        break;
      case CHECK:
        if (state.wrap) {
          //=== NEEDBITS(32);
          while (bits < 32) {
            if (have === 0) { break inf_leave; }
            have--;
            // Use '|' instead of '+' to make sure that result is signed
            hold |= input[next++] << bits;
            bits += 8;
          }
          //===//
          _out -= left;
          strm.total_out += _out;
          state.total += _out;
          if (_out) {
            strm.adler = state.check =
                /*UPDATE(state.check, put - _out, _out);*/
                (state.flags ? crc32_1$1(state.check, output, _out, put - _out) : adler32_1(state.check, output, _out, put - _out));

          }
          _out = left;
          // NB: crc32 stored as signed 32-bit int, zswap32 returns signed too
          if ((state.flags ? hold : zswap32(hold)) !== state.check) {
            strm.msg = 'incorrect data check';
            state.mode = BAD$1;
            break;
          }
          //=== INITBITS();
          hold = 0;
          bits = 0;
          //===//
          //Tracev((stderr, "inflate:   check matches trailer\n"));
        }
        state.mode = LENGTH;
        /* falls through */
      case LENGTH:
        if (state.wrap && state.flags) {
          //=== NEEDBITS(32);
          while (bits < 32) {
            if (have === 0) { break inf_leave; }
            have--;
            hold += input[next++] << bits;
            bits += 8;
          }
          //===//
          if (hold !== (state.total & 0xffffffff)) {
            strm.msg = 'incorrect length check';
            state.mode = BAD$1;
            break;
          }
          //=== INITBITS();
          hold = 0;
          bits = 0;
          //===//
          //Tracev((stderr, "inflate:   length matches trailer\n"));
        }
        state.mode = DONE;
        /* falls through */
      case DONE:
        ret = Z_STREAM_END$2;
        break inf_leave;
      case BAD$1:
        ret = Z_DATA_ERROR$1;
        break inf_leave;
      case MEM:
        return Z_MEM_ERROR;
      case SYNC:
        /* falls through */
      default:
        return Z_STREAM_ERROR$1;
    }
  }

  // inf_leave <- here is real place for "goto inf_leave", emulated via "break inf_leave"

  /*
     Return from inflate(), updating the total counts and the check value.
     If there was no progress during the inflate() call, return a buffer
     error.  Call updatewindow() to create and/or update the window state.
     Note: a memory error from inflate() is non-recoverable.
   */

  //--- RESTORE() ---
  strm.next_out = put;
  strm.avail_out = left;
  strm.next_in = next;
  strm.avail_in = have;
  state.hold = hold;
  state.bits = bits;
  //---

  if (state.wsize || (_out !== strm.avail_out && state.mode < BAD$1 &&
                      (state.mode < CHECK || flush !== Z_FINISH$2))) {
    if (updatewindow(strm, strm.output, strm.next_out, _out - strm.avail_out)) ;
  }
  _in -= strm.avail_in;
  _out -= strm.avail_out;
  strm.total_in += _in;
  strm.total_out += _out;
  state.total += _out;
  if (state.wrap && _out) {
    strm.adler = state.check = /*UPDATE(state.check, strm.next_out - _out, _out);*/
      (state.flags ? crc32_1$1(state.check, output, _out, strm.next_out - _out) : adler32_1(state.check, output, _out, strm.next_out - _out));
  }
  strm.data_type = state.bits + (state.last ? 64 : 0) +
                    (state.mode === TYPE$1 ? 128 : 0) +
                    (state.mode === LEN_ || state.mode === COPY_ ? 256 : 0);
  if (((_in === 0 && _out === 0) || flush === Z_FINISH$2) && ret === Z_OK$2) {
    ret = Z_BUF_ERROR$1;
  }
  return ret;
}

function inflateEnd(strm) {

  if (!strm || !strm.state /*|| strm->zfree == (free_func)0*/) {
    return Z_STREAM_ERROR$1;
  }

  var state = strm.state;
  if (state.window) {
    state.window = null;
  }
  strm.state = null;
  return Z_OK$2;
}

function inflateGetHeader(strm, head) {
  var state;

  /* check state */
  if (!strm || !strm.state) { return Z_STREAM_ERROR$1; }
  state = strm.state;
  if ((state.wrap & 2) === 0) { return Z_STREAM_ERROR$1; }

  /* save header structure */
  state.head = head;
  head.done = false;
  return Z_OK$2;
}

function inflateSetDictionary(strm, dictionary) {
  var dictLength = dictionary.length;

  var state;
  var dictid;
  var ret;

  /* check state */
  if (!strm /* == Z_NULL */ || !strm.state /* == Z_NULL */) { return Z_STREAM_ERROR$1; }
  state = strm.state;

  if (state.wrap !== 0 && state.mode !== DICT) {
    return Z_STREAM_ERROR$1;
  }

  /* check for correct dictionary identifier */
  if (state.mode === DICT) {
    dictid = 1; /* adler32(0, null, 0)*/
    /* dictid = adler32(dictid, dictionary, dictLength); */
    dictid = adler32_1(dictid, dictionary, dictLength, 0);
    if (dictid !== state.check) {
      return Z_DATA_ERROR$1;
    }
  }
  /* copy dictionary to window using updatewindow(), which will amend the
   existing dictionary if appropriate */
  ret = updatewindow(strm, dictionary, dictLength, dictLength);
  if (ret) {
    state.mode = MEM;
    return Z_MEM_ERROR;
  }
  state.havedict = 1;
  // Tracev((stderr, "inflate:   dictionary set\n"));
  return Z_OK$2;
}

var inflateReset_1 = inflateReset;
var inflateReset2_1 = inflateReset2;
var inflateResetKeep_1 = inflateResetKeep;
var inflateInit_1 = inflateInit;
var inflateInit2_1 = inflateInit2;
var inflate_2 = inflate;
var inflateEnd_1 = inflateEnd;
var inflateGetHeader_1 = inflateGetHeader;
var inflateSetDictionary_1 = inflateSetDictionary;
var inflateInfo = 'pako inflate (from Nodeca project)';

/* Not implemented
exports.inflateCopy = inflateCopy;
exports.inflateGetDictionary = inflateGetDictionary;
exports.inflateMark = inflateMark;
exports.inflatePrime = inflatePrime;
exports.inflateSync = inflateSync;
exports.inflateSyncPoint = inflateSyncPoint;
exports.inflateUndermine = inflateUndermine;
*/

var inflate_1 = {
	inflateReset: inflateReset_1,
	inflateReset2: inflateReset2_1,
	inflateResetKeep: inflateResetKeep_1,
	inflateInit: inflateInit_1,
	inflateInit2: inflateInit2_1,
	inflate: inflate_2,
	inflateEnd: inflateEnd_1,
	inflateGetHeader: inflateGetHeader_1,
	inflateSetDictionary: inflateSetDictionary_1,
	inflateInfo: inflateInfo
};

// (C) 1995-2013 Jean-loup Gailly and Mark Adler
// (C) 2014-2017 Vitaly Puzrin and Andrey Tupitsin
//
// This software is provided 'as-is', without any express or implied
// warranty. In no event will the authors be held liable for any damages
// arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must not
//   claim that you wrote the original software. If you use this software
//   in a product, an acknowledgment in the product documentation would be
//   appreciated but is not required.
// 2. Altered source versions must be plainly marked as such, and must not be
//   misrepresented as being the original software.
// 3. This notice may not be removed or altered from any source distribution.

var constants = {

  /* Allowed flush values; see deflate() and inflate() below for details */
  Z_NO_FLUSH:         0,
  Z_PARTIAL_FLUSH:    1,
  Z_SYNC_FLUSH:       2,
  Z_FULL_FLUSH:       3,
  Z_FINISH:           4,
  Z_BLOCK:            5,
  Z_TREES:            6,

  /* Return codes for the compression/decompression functions. Negative values
  * are errors, positive values are used for special but normal events.
  */
  Z_OK:               0,
  Z_STREAM_END:       1,
  Z_NEED_DICT:        2,
  Z_ERRNO:           -1,
  Z_STREAM_ERROR:    -2,
  Z_DATA_ERROR:      -3,
  //Z_MEM_ERROR:     -4,
  Z_BUF_ERROR:       -5,
  //Z_VERSION_ERROR: -6,

  /* compression levels */
  Z_NO_COMPRESSION:         0,
  Z_BEST_SPEED:             1,
  Z_BEST_COMPRESSION:       9,
  Z_DEFAULT_COMPRESSION:   -1,


  Z_FILTERED:               1,
  Z_HUFFMAN_ONLY:           2,
  Z_RLE:                    3,
  Z_FIXED:                  4,
  Z_DEFAULT_STRATEGY:       0,

  /* Possible values of the data_type field (though see inflate()) */
  Z_BINARY:                 0,
  Z_TEXT:                   1,
  //Z_ASCII:                1, // = Z_TEXT (deprecated)
  Z_UNKNOWN:                2,

  /* The deflate compression method */
  Z_DEFLATED:               8
  //Z_NULL:                 null // Use -1 or null inline, depending on var type
};

// (C) 1995-2013 Jean-loup Gailly and Mark Adler
// (C) 2014-2017 Vitaly Puzrin and Andrey Tupitsin
//
// This software is provided 'as-is', without any express or implied
// warranty. In no event will the authors be held liable for any damages
// arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must not
//   claim that you wrote the original software. If you use this software
//   in a product, an acknowledgment in the product documentation would be
//   appreciated but is not required.
// 2. Altered source versions must be plainly marked as such, and must not be
//   misrepresented as being the original software.
// 3. This notice may not be removed or altered from any source distribution.

function GZheader() {
  /* true if compressed data believed to be text */
  this.text       = 0;
  /* modification time */
  this.time       = 0;
  /* extra flags (not used when writing a gzip file) */
  this.xflags     = 0;
  /* operating system */
  this.os         = 0;
  /* pointer to extra field or Z_NULL if none */
  this.extra      = null;
  /* extra field length (valid if extra != Z_NULL) */
  this.extra_len  = 0; // Actually, we don't need it in JS,
                       // but leave for few code modifications

  //
  // Setup limits is not necessary because in js we should not preallocate memory
  // for inflate use constant limit in 65536 bytes
  //

  /* space at extra (only when reading header) */
  // this.extra_max  = 0;
  /* pointer to zero-terminated file name or Z_NULL */
  this.name       = '';
  /* space at name (only when reading header) */
  // this.name_max   = 0;
  /* pointer to zero-terminated comment or Z_NULL */
  this.comment    = '';
  /* space at comment (only when reading header) */
  // this.comm_max   = 0;
  /* true if there was or will be a header crc */
  this.hcrc       = 0;
  /* true when done reading gzip header (not used when writing a gzip file) */
  this.done       = false;
}

var gzheader = GZheader;

var toString$4 = Object.prototype.toString;

/**
 * class Inflate
 *
 * Generic JS-style wrapper for zlib calls. If you don't need
 * streaming behaviour - use more simple functions: [[inflate]]
 * and [[inflateRaw]].
 **/

/* internal
 * inflate.chunks -> Array
 *
 * Chunks of output data, if [[Inflate#onData]] not overridden.
 **/

/**
 * Inflate.result -> Uint8Array|Array|String
 *
 * Uncompressed result, generated by default [[Inflate#onData]]
 * and [[Inflate#onEnd]] handlers. Filled after you push last chunk
 * (call [[Inflate#push]] with `Z_FINISH` / `true` param) or if you
 * push a chunk with explicit flush (call [[Inflate#push]] with
 * `Z_SYNC_FLUSH` param).
 **/

/**
 * Inflate.err -> Number
 *
 * Error code after inflate finished. 0 (Z_OK) on success.
 * Should be checked if broken data possible.
 **/

/**
 * Inflate.msg -> String
 *
 * Error message, if [[Inflate.err]] != 0
 **/


/**
 * new Inflate(options)
 * - options (Object): zlib inflate options.
 *
 * Creates new inflator instance with specified params. Throws exception
 * on bad params. Supported options:
 *
 * - `windowBits`
 * - `dictionary`
 *
 * [http://zlib.net/manual.html#Advanced](http://zlib.net/manual.html#Advanced)
 * for more information on these.
 *
 * Additional options, for internal needs:
 *
 * - `chunkSize` - size of generated data chunks (16K by default)
 * - `raw` (Boolean) - do raw inflate
 * - `to` (String) - if equal to 'string', then result will be converted
 *   from utf8 to utf16 (javascript) string. When string output requested,
 *   chunk length can differ from `chunkSize`, depending on content.
 *
 * By default, when no options set, autodetect deflate/gzip data format via
 * wrapper header.
 *
 * ##### Example:
 *
 * ```javascript
 * var pako = require('pako')
 *   , chunk1 = Uint8Array([1,2,3,4,5,6,7,8,9])
 *   , chunk2 = Uint8Array([10,11,12,13,14,15,16,17,18,19]);
 *
 * var inflate = new pako.Inflate({ level: 3});
 *
 * inflate.push(chunk1, false);
 * inflate.push(chunk2, true);  // true -> last chunk
 *
 * if (inflate.err) { throw new Error(inflate.err); }
 *
 * console.log(inflate.result);
 * ```
 **/
function Inflate(options) {
  if (!(this instanceof Inflate)) return new Inflate(options);

  this.options = common.assign({
    chunkSize: 16384,
    windowBits: 0,
    to: ''
  }, options || {});

  var opt = this.options;

  // Force window size for `raw` data, if not set directly,
  // because we have no header for autodetect.
  if (opt.raw && (opt.windowBits >= 0) && (opt.windowBits < 16)) {
    opt.windowBits = -opt.windowBits;
    if (opt.windowBits === 0) { opt.windowBits = -15; }
  }

  // If `windowBits` not defined (and mode not raw) - set autodetect flag for gzip/deflate
  if ((opt.windowBits >= 0) && (opt.windowBits < 16) &&
      !(options && options.windowBits)) {
    opt.windowBits += 32;
  }

  // Gzip header has no info about windows size, we can do autodetect only
  // for deflate. So, if window size not set, force it to max when gzip possible
  if ((opt.windowBits > 15) && (opt.windowBits < 48)) {
    // bit 3 (16) -> gzipped data
    // bit 4 (32) -> autodetect gzip/deflate
    if ((opt.windowBits & 15) === 0) {
      opt.windowBits |= 15;
    }
  }

  this.err    = 0;      // error code, if happens (0 = Z_OK)
  this.msg    = '';     // error message
  this.ended  = false;  // used to avoid multiple onEnd() calls
  this.chunks = [];     // chunks of compressed data

  this.strm   = new zstream();
  this.strm.avail_out = 0;

  var status  = inflate_1.inflateInit2(
    this.strm,
    opt.windowBits
  );

  if (status !== constants.Z_OK) {
    throw new Error(messages[status]);
  }

  this.header = new gzheader();

  inflate_1.inflateGetHeader(this.strm, this.header);
}

/**
 * Inflate#push(data[, mode]) -> Boolean
 * - data (Uint8Array|Array|ArrayBuffer|String): input data
 * - mode (Number|Boolean): 0..6 for corresponding Z_NO_FLUSH..Z_TREE modes.
 *   See constants. Skipped or `false` means Z_NO_FLUSH, `true` means Z_FINISH.
 *
 * Sends input data to inflate pipe, generating [[Inflate#onData]] calls with
 * new output chunks. Returns `true` on success. The last data block must have
 * mode Z_FINISH (or `true`). That will flush internal pending buffers and call
 * [[Inflate#onEnd]]. For interim explicit flushes (without ending the stream) you
 * can use mode Z_SYNC_FLUSH, keeping the decompression context.
 *
 * On fail call [[Inflate#onEnd]] with error code and return false.
 *
 * We strongly recommend to use `Uint8Array` on input for best speed (output
 * format is detected automatically). Also, don't skip last param and always
 * use the same type in your code (boolean or number). That will improve JS speed.
 *
 * For regular `Array`-s make sure all elements are [0..255].
 *
 * ##### Example
 *
 * ```javascript
 * push(chunk, false); // push one of data chunks
 * ...
 * push(chunk, true);  // push last chunk
 * ```
 **/
Inflate.prototype.push = function (data, mode) {
  var strm = this.strm;
  var chunkSize = this.options.chunkSize;
  var dictionary = this.options.dictionary;
  var status, _mode;
  var next_out_utf8, tail, utf8str;
  var dict;

  // Flag to properly process Z_BUF_ERROR on testing inflate call
  // when we check that all output data was flushed.
  var allowBufError = false;

  if (this.ended) { return false; }
  _mode = (mode === ~~mode) ? mode : ((mode === true) ? constants.Z_FINISH : constants.Z_NO_FLUSH);

  // Convert data if needed
  if (typeof data === 'string') {
    // Only binary strings can be decompressed on practice
    strm.input = strings.binstring2buf(data);
  } else if (toString$4.call(data) === '[object ArrayBuffer]') {
    strm.input = new Uint8Array(data);
  } else {
    strm.input = data;
  }

  strm.next_in = 0;
  strm.avail_in = strm.input.length;

  do {
    if (strm.avail_out === 0) {
      strm.output = new common.Buf8(chunkSize);
      strm.next_out = 0;
      strm.avail_out = chunkSize;
    }

    status = inflate_1.inflate(strm, constants.Z_NO_FLUSH);    /* no bad return value */

    if (status === constants.Z_NEED_DICT && dictionary) {
      // Convert data if needed
      if (typeof dictionary === 'string') {
        dict = strings.string2buf(dictionary);
      } else if (toString$4.call(dictionary) === '[object ArrayBuffer]') {
        dict = new Uint8Array(dictionary);
      } else {
        dict = dictionary;
      }

      status = inflate_1.inflateSetDictionary(this.strm, dict);

    }

    if (status === constants.Z_BUF_ERROR && allowBufError === true) {
      status = constants.Z_OK;
      allowBufError = false;
    }

    if (status !== constants.Z_STREAM_END && status !== constants.Z_OK) {
      this.onEnd(status);
      this.ended = true;
      return false;
    }

    if (strm.next_out) {
      if (strm.avail_out === 0 || status === constants.Z_STREAM_END || (strm.avail_in === 0 && (_mode === constants.Z_FINISH || _mode === constants.Z_SYNC_FLUSH))) {

        if (this.options.to === 'string') {

          next_out_utf8 = strings.utf8border(strm.output, strm.next_out);

          tail = strm.next_out - next_out_utf8;
          utf8str = strings.buf2string(strm.output, next_out_utf8);

          // move tail
          strm.next_out = tail;
          strm.avail_out = chunkSize - tail;
          if (tail) { common.arraySet(strm.output, strm.output, next_out_utf8, tail, 0); }

          this.onData(utf8str);

        } else {
          this.onData(common.shrinkBuf(strm.output, strm.next_out));
        }
      }
    }

    // When no more input data, we should check that internal inflate buffers
    // are flushed. The only way to do it when avail_out = 0 - run one more
    // inflate pass. But if output data not exists, inflate return Z_BUF_ERROR.
    // Here we set flag to process this error properly.
    //
    // NOTE. Deflate does not return error in this case and does not needs such
    // logic.
    if (strm.avail_in === 0 && strm.avail_out === 0) {
      allowBufError = true;
    }

  } while ((strm.avail_in > 0 || strm.avail_out === 0) && status !== constants.Z_STREAM_END);

  if (status === constants.Z_STREAM_END) {
    _mode = constants.Z_FINISH;
  }

  // Finalize on the last chunk.
  if (_mode === constants.Z_FINISH) {
    status = inflate_1.inflateEnd(this.strm);
    this.onEnd(status);
    this.ended = true;
    return status === constants.Z_OK;
  }

  // callback interim results if Z_SYNC_FLUSH.
  if (_mode === constants.Z_SYNC_FLUSH) {
    this.onEnd(constants.Z_OK);
    strm.avail_out = 0;
    return true;
  }

  return true;
};


/**
 * Inflate#onData(chunk) -> Void
 * - chunk (Uint8Array|Array|String): output data. Type of array depends
 *   on js engine support. When string output requested, each chunk
 *   will be string.
 *
 * By default, stores data blocks in `chunks[]` property and glue
 * those in `onEnd`. Override this handler, if you need another behaviour.
 **/
Inflate.prototype.onData = function (chunk) {
  this.chunks.push(chunk);
};


/**
 * Inflate#onEnd(status) -> Void
 * - status (Number): inflate status. 0 (Z_OK) on success,
 *   other if not.
 *
 * Called either after you tell inflate that the input stream is
 * complete (Z_FINISH) or should be flushed (Z_SYNC_FLUSH)
 * or if an error happened. By default - join collected chunks,
 * free memory and fill `results` / `err` properties.
 **/
Inflate.prototype.onEnd = function (status) {
  // On success - join
  if (status === constants.Z_OK) {
    if (this.options.to === 'string') {
      // Glue & convert here, until we teach pako to send
      // utf8 aligned strings to onData
      this.result = this.chunks.join('');
    } else {
      this.result = common.flattenChunks(this.chunks);
    }
  }
  this.chunks = [];
  this.err = status;
  this.msg = this.strm.msg;
};


/**
 * inflate(data[, options]) -> Uint8Array|Array|String
 * - data (Uint8Array|Array|String): input data to decompress.
 * - options (Object): zlib inflate options.
 *
 * Decompress `data` with inflate/ungzip and `options`. Autodetect
 * format via wrapper header by default. That's why we don't provide
 * separate `ungzip` method.
 *
 * Supported options are:
 *
 * - windowBits
 *
 * [http://zlib.net/manual.html#Advanced](http://zlib.net/manual.html#Advanced)
 * for more information.
 *
 * Sugar (options):
 *
 * - `raw` (Boolean) - say that we work with raw stream, if you don't wish to specify
 *   negative windowBits implicitly.
 * - `to` (String) - if equal to 'string', then result will be converted
 *   from utf8 to utf16 (javascript) string. When string output requested,
 *   chunk length can differ from `chunkSize`, depending on content.
 *
 *
 * ##### Example:
 *
 * ```javascript
 * var pako = require('pako')
 *   , input = pako.deflate([1,2,3,4,5,6,7,8,9])
 *   , output;
 *
 * try {
 *   output = pako.inflate(input);
 * } catch (err)
 *   console.log(err);
 * }
 * ```
 **/
function inflate$1(input, options) {
  var inflator = new Inflate(options);

  inflator.push(input, true);

  // That will never happens, if you don't cheat with options :)
  if (inflator.err) { throw inflator.msg || messages[inflator.err]; }

  return inflator.result;
}


/**
 * inflateRaw(data[, options]) -> Uint8Array|Array|String
 * - data (Uint8Array|Array|String): input data to decompress.
 * - options (Object): zlib inflate options.
 *
 * The same as [[inflate]], but creates raw data, without wrapper
 * (header and adler32 crc).
 **/
function inflateRaw(input, options) {
  options = options || {};
  options.raw = true;
  return inflate$1(input, options);
}


/**
 * ungzip(data[, options]) -> Uint8Array|Array|String
 * - data (Uint8Array|Array|String): input data to decompress.
 * - options (Object): zlib inflate options.
 *
 * Just shortcut to [[inflate]], because it autodetects format
 * by header.content. Done for convenience.
 **/


var Inflate_1 = Inflate;
var inflate_2$1 = inflate$1;
var inflateRaw_1 = inflateRaw;
var ungzip  = inflate$1;

var inflate_1$1 = {
	Inflate: Inflate_1,
	inflate: inflate_2$1,
	inflateRaw: inflateRaw_1,
	ungzip: ungzip
};

var assign    = common.assign;





var pako = {};

assign(pako, deflate_1$1, inflate_1$1, constants);

var pako_1 = pako;

var USE_TYPEDARRAY = (typeof Uint8Array !== 'undefined') && (typeof Uint16Array !== 'undefined') && (typeof Uint32Array !== 'undefined');





var ARRAY_TYPE = USE_TYPEDARRAY ? "uint8array" : "array";

var magic = "\x08\x00";

/**
 * Create a worker that uses pako to inflate/deflate.
 * @constructor
 * @param {String} action the name of the pako function to call : either "Deflate" or "Inflate".
 * @param {Object} options the options to use when (de)compressing.
 */
function FlateWorker(action, options) {
    GenericWorker_1.call(this, "FlateWorker/" + action);

    this._pako = null;
    this._pakoAction = action;
    this._pakoOptions = options;
    // the `meta` object from the last chunk received
    // this allow this worker to pass around metadata
    this.meta = {};
}

utils.inherits(FlateWorker, GenericWorker_1);

/**
 * @see GenericWorker.processChunk
 */
FlateWorker.prototype.processChunk = function (chunk) {
    this.meta = chunk.meta;
    if (this._pako === null) {
        this._createPako();
    }
    this._pako.push(utils.transformTo(ARRAY_TYPE, chunk.data), false);
};

/**
 * @see GenericWorker.flush
 */
FlateWorker.prototype.flush = function () {
    GenericWorker_1.prototype.flush.call(this);
    if (this._pako === null) {
        this._createPako();
    }
    this._pako.push([], true);
};
/**
 * @see GenericWorker.cleanUp
 */
FlateWorker.prototype.cleanUp = function () {
    GenericWorker_1.prototype.cleanUp.call(this);
    this._pako = null;
};

/**
 * Create the _pako object.
 * TODO: lazy-loading this object isn't the best solution but it's the
 * quickest. The best solution is to lazy-load the worker list. See also the
 * issue #446.
 */
FlateWorker.prototype._createPako = function () {
    this._pako = new pako_1[this._pakoAction]({
        raw: true,
        level: this._pakoOptions.level || -1 // default compression
    });
    var self = this;
    this._pako.onData = function(data) {
        self.push({
            data : data,
            meta : self.meta
        });
    };
};

var compressWorker = function (compressionOptions) {
    return new FlateWorker("Deflate", compressionOptions);
};
var uncompressWorker = function () {
    return new FlateWorker("Inflate", {});
};

var flate = {
	magic: magic,
	compressWorker: compressWorker,
	uncompressWorker: uncompressWorker
};

var STORE = {
    magic: "\x00\x00",
    compressWorker : function (compressionOptions) {
        return new GenericWorker_1("STORE compression");
    },
    uncompressWorker : function () {
        return new GenericWorker_1("STORE decompression");
    }
};
var DEFLATE = flate;

var compressions = {
	STORE: STORE,
	DEFLATE: DEFLATE
};

var LOCAL_FILE_HEADER = "PK\x03\x04";
var CENTRAL_FILE_HEADER = "PK\x01\x02";
var CENTRAL_DIRECTORY_END = "PK\x05\x06";
var ZIP64_CENTRAL_DIRECTORY_LOCATOR = "PK\x06\x07";
var ZIP64_CENTRAL_DIRECTORY_END = "PK\x06\x06";
var DATA_DESCRIPTOR = "PK\x07\x08";

var signature = {
	LOCAL_FILE_HEADER: LOCAL_FILE_HEADER,
	CENTRAL_FILE_HEADER: CENTRAL_FILE_HEADER,
	CENTRAL_DIRECTORY_END: CENTRAL_DIRECTORY_END,
	ZIP64_CENTRAL_DIRECTORY_LOCATOR: ZIP64_CENTRAL_DIRECTORY_LOCATOR,
	ZIP64_CENTRAL_DIRECTORY_END: ZIP64_CENTRAL_DIRECTORY_END,
	DATA_DESCRIPTOR: DATA_DESCRIPTOR
};

/**
 * Transform an integer into a string in hexadecimal.
 * @private
 * @param {number} dec the number to convert.
 * @param {number} bytes the number of bytes to generate.
 * @returns {string} the result.
 */
var decToHex = function(dec, bytes) {
    var hex = "", i;
    for (i = 0; i < bytes; i++) {
        hex += String.fromCharCode(dec & 0xff);
        dec = dec >>> 8;
    }
    return hex;
};

/**
 * Generate the UNIX part of the external file attributes.
 * @param {Object} unixPermissions the unix permissions or null.
 * @param {Boolean} isDir true if the entry is a directory, false otherwise.
 * @return {Number} a 32 bit integer.
 *
 * adapted from http://unix.stackexchange.com/questions/14705/the-zip-formats-external-file-attribute :
 *
 * TTTTsstrwxrwxrwx0000000000ADVSHR
 * ^^^^____________________________ file type, see zipinfo.c (UNX_*)
 *     ^^^_________________________ setuid, setgid, sticky
 *        ^^^^^^^^^________________ permissions
 *                 ^^^^^^^^^^______ not used ?
 *                           ^^^^^^ DOS attribute bits : Archive, Directory, Volume label, System file, Hidden, Read only
 */
var generateUnixExternalFileAttr = function (unixPermissions, isDir) {

    var result = unixPermissions;
    if (!unixPermissions) {
        // I can't use octal values in strict mode, hence the hexa.
        //  040775 => 0x41fd
        // 0100664 => 0x81b4
        result = isDir ? 0x41fd : 0x81b4;
    }
    return (result & 0xFFFF) << 16;
};

/**
 * Generate the DOS part of the external file attributes.
 * @param {Object} dosPermissions the dos permissions or null.
 * @param {Boolean} isDir true if the entry is a directory, false otherwise.
 * @return {Number} a 32 bit integer.
 *
 * Bit 0     Read-Only
 * Bit 1     Hidden
 * Bit 2     System
 * Bit 3     Volume Label
 * Bit 4     Directory
 * Bit 5     Archive
 */
var generateDosExternalFileAttr = function (dosPermissions, isDir) {

    // the dir flag is already set for compatibility
    return (dosPermissions || 0)  & 0x3F;
};

/**
 * Generate the various parts used in the construction of the final zip file.
 * @param {Object} streamInfo the hash with informations about the compressed file.
 * @param {Boolean} streamedContent is the content streamed ?
 * @param {Boolean} streamingEnded is the stream finished ?
 * @param {number} offset the current offset from the start of the zip file.
 * @param {String} platform let's pretend we are this platform (change platform dependents fields)
 * @param {Function} encodeFileName the function to encode the file name / comment.
 * @return {Object} the zip parts.
 */
var generateZipParts = function(streamInfo, streamedContent, streamingEnded, offset, platform, encodeFileName) {
    var file = streamInfo['file'],
    compression = streamInfo['compression'],
    useCustomEncoding = encodeFileName !== utf8.utf8encode,
    encodedFileName = utils.transformTo("string", encodeFileName(file.name)),
    utfEncodedFileName = utils.transformTo("string", utf8.utf8encode(file.name)),
    comment = file.comment,
    encodedComment = utils.transformTo("string", encodeFileName(comment)),
    utfEncodedComment = utils.transformTo("string", utf8.utf8encode(comment)),
    useUTF8ForFileName = utfEncodedFileName.length !== file.name.length,
    useUTF8ForComment = utfEncodedComment.length !== comment.length,
    dosTime,
    dosDate,
    extraFields = "",
    unicodePathExtraField = "",
    unicodeCommentExtraField = "",
    dir = file.dir,
    date = file.date;


    var dataInfo = {
        crc32 : 0,
        compressedSize : 0,
        uncompressedSize : 0
    };

    // if the content is streamed, the sizes/crc32 are only available AFTER
    // the end of the stream.
    if (!streamedContent || streamingEnded) {
        dataInfo.crc32 = streamInfo['crc32'];
        dataInfo.compressedSize = streamInfo['compressedSize'];
        dataInfo.uncompressedSize = streamInfo['uncompressedSize'];
    }

    var bitflag = 0;
    if (streamedContent) {
        // Bit 3: the sizes/crc32 are set to zero in the local header.
        // The correct values are put in the data descriptor immediately
        // following the compressed data.
        bitflag |= 0x0008;
    }
    if (!useCustomEncoding && (useUTF8ForFileName || useUTF8ForComment)) {
        // Bit 11: Language encoding flag (EFS).
        bitflag |= 0x0800;
    }


    var extFileAttr = 0;
    var versionMadeBy = 0;
    if (dir) {
        // dos or unix, we set the dos dir flag
        extFileAttr |= 0x00010;
    }
    if(platform === "UNIX") {
        versionMadeBy = 0x031E; // UNIX, version 3.0
        extFileAttr |= generateUnixExternalFileAttr(file.unixPermissions, dir);
    } else { // DOS or other, fallback to DOS
        versionMadeBy = 0x0014; // DOS, version 2.0
        extFileAttr |= generateDosExternalFileAttr(file.dosPermissions, dir);
    }

    // date
    // @see http://www.delorie.com/djgpp/doc/rbinter/it/52/13.html
    // @see http://www.delorie.com/djgpp/doc/rbinter/it/65/16.html
    // @see http://www.delorie.com/djgpp/doc/rbinter/it/66/16.html

    dosTime = date.getUTCHours();
    dosTime = dosTime << 6;
    dosTime = dosTime | date.getUTCMinutes();
    dosTime = dosTime << 5;
    dosTime = dosTime | date.getUTCSeconds() / 2;

    dosDate = date.getUTCFullYear() - 1980;
    dosDate = dosDate << 4;
    dosDate = dosDate | (date.getUTCMonth() + 1);
    dosDate = dosDate << 5;
    dosDate = dosDate | date.getUTCDate();

    if (useUTF8ForFileName) {
        // set the unicode path extra field. unzip needs at least one extra
        // field to correctly handle unicode path, so using the path is as good
        // as any other information. This could improve the situation with
        // other archive managers too.
        // This field is usually used without the utf8 flag, with a non
        // unicode path in the header (winrar, winzip). This helps (a bit)
        // with the messy Windows' default compressed folders feature but
        // breaks on p7zip which doesn't seek the unicode path extra field.
        // So for now, UTF-8 everywhere !
        unicodePathExtraField =
            // Version
            decToHex(1, 1) +
            // NameCRC32
            decToHex(crc32_1(encodedFileName), 4) +
            // UnicodeName
            utfEncodedFileName;

        extraFields +=
            // Info-ZIP Unicode Path Extra Field
            "\x75\x70" +
            // size
            decToHex(unicodePathExtraField.length, 2) +
            // content
            unicodePathExtraField;
    }

    if(useUTF8ForComment) {

        unicodeCommentExtraField =
            // Version
            decToHex(1, 1) +
            // CommentCRC32
            decToHex(crc32_1(encodedComment), 4) +
            // UnicodeName
            utfEncodedComment;

        extraFields +=
            // Info-ZIP Unicode Path Extra Field
            "\x75\x63" +
            // size
            decToHex(unicodeCommentExtraField.length, 2) +
            // content
            unicodeCommentExtraField;
    }

    var header = "";

    // version needed to extract
    header += "\x0A\x00";
    // general purpose bit flag
    header += decToHex(bitflag, 2);
    // compression method
    header += compression.magic;
    // last mod file time
    header += decToHex(dosTime, 2);
    // last mod file date
    header += decToHex(dosDate, 2);
    // crc-32
    header += decToHex(dataInfo.crc32, 4);
    // compressed size
    header += decToHex(dataInfo.compressedSize, 4);
    // uncompressed size
    header += decToHex(dataInfo.uncompressedSize, 4);
    // file name length
    header += decToHex(encodedFileName.length, 2);
    // extra field length
    header += decToHex(extraFields.length, 2);


    var fileRecord = signature.LOCAL_FILE_HEADER + header + encodedFileName + extraFields;

    var dirRecord = signature.CENTRAL_FILE_HEADER +
        // version made by (00: DOS)
        decToHex(versionMadeBy, 2) +
        // file header (common to file and central directory)
        header +
        // file comment length
        decToHex(encodedComment.length, 2) +
        // disk number start
        "\x00\x00" +
        // internal file attributes TODO
        "\x00\x00" +
        // external file attributes
        decToHex(extFileAttr, 4) +
        // relative offset of local header
        decToHex(offset, 4) +
        // file name
        encodedFileName +
        // extra field
        extraFields +
        // file comment
        encodedComment;

    return {
        fileRecord: fileRecord,
        dirRecord: dirRecord
    };
};

/**
 * Generate the EOCD record.
 * @param {Number} entriesCount the number of entries in the zip file.
 * @param {Number} centralDirLength the length (in bytes) of the central dir.
 * @param {Number} localDirLength the length (in bytes) of the local dir.
 * @param {String} comment the zip file comment as a binary string.
 * @param {Function} encodeFileName the function to encode the comment.
 * @return {String} the EOCD record.
 */
var generateCentralDirectoryEnd = function (entriesCount, centralDirLength, localDirLength, comment, encodeFileName) {
    var dirEnd = "";
    var encodedComment = utils.transformTo("string", encodeFileName(comment));

    // end of central dir signature
    dirEnd = signature.CENTRAL_DIRECTORY_END +
        // number of this disk
        "\x00\x00" +
        // number of the disk with the start of the central directory
        "\x00\x00" +
        // total number of entries in the central directory on this disk
        decToHex(entriesCount, 2) +
        // total number of entries in the central directory
        decToHex(entriesCount, 2) +
        // size of the central directory   4 bytes
        decToHex(centralDirLength, 4) +
        // offset of start of central directory with respect to the starting disk number
        decToHex(localDirLength, 4) +
        // .ZIP file comment length
        decToHex(encodedComment.length, 2) +
        // .ZIP file comment
        encodedComment;

    return dirEnd;
};

/**
 * Generate data descriptors for a file entry.
 * @param {Object} streamInfo the hash generated by a worker, containing informations
 * on the file entry.
 * @return {String} the data descriptors.
 */
var generateDataDescriptors = function (streamInfo) {
    var descriptor = "";
    descriptor = signature.DATA_DESCRIPTOR +
        // crc-32                          4 bytes
        decToHex(streamInfo['crc32'], 4) +
        // compressed size                 4 bytes
        decToHex(streamInfo['compressedSize'], 4) +
        // uncompressed size               4 bytes
        decToHex(streamInfo['uncompressedSize'], 4);

    return descriptor;
};


/**
 * A worker to concatenate other workers to create a zip file.
 * @param {Boolean} streamFiles `true` to stream the content of the files,
 * `false` to accumulate it.
 * @param {String} comment the comment to use.
 * @param {String} platform the platform to use, "UNIX" or "DOS".
 * @param {Function} encodeFileName the function to encode file names and comments.
 */
function ZipFileWorker(streamFiles, comment, platform, encodeFileName) {
    GenericWorker_1.call(this, "ZipFileWorker");
    // The number of bytes written so far. This doesn't count accumulated chunks.
    this.bytesWritten = 0;
    // The comment of the zip file
    this.zipComment = comment;
    // The platform "generating" the zip file.
    this.zipPlatform = platform;
    // the function to encode file names and comments.
    this.encodeFileName = encodeFileName;
    // Should we stream the content of the files ?
    this.streamFiles = streamFiles;
    // If `streamFiles` is false, we will need to accumulate the content of the
    // files to calculate sizes / crc32 (and write them *before* the content).
    // This boolean indicates if we are accumulating chunks (it will change a lot
    // during the lifetime of this worker).
    this.accumulate = false;
    // The buffer receiving chunks when accumulating content.
    this.contentBuffer = [];
    // The list of generated directory records.
    this.dirRecords = [];
    // The offset (in bytes) from the beginning of the zip file for the current source.
    this.currentSourceOffset = 0;
    // The total number of entries in this zip file.
    this.entriesCount = 0;
    // the name of the file currently being added, null when handling the end of the zip file.
    // Used for the emited metadata.
    this.currentFile = null;



    this._sources = [];
}
utils.inherits(ZipFileWorker, GenericWorker_1);

/**
 * @see GenericWorker.push
 */
ZipFileWorker.prototype.push = function (chunk) {

    var currentFilePercent = chunk.meta.percent || 0;
    var entriesCount = this.entriesCount;
    var remainingFiles = this._sources.length;

    if(this.accumulate) {
        this.contentBuffer.push(chunk);
    } else {
        this.bytesWritten += chunk.data.length;

        GenericWorker_1.prototype.push.call(this, {
            data : chunk.data,
            meta : {
                currentFile : this.currentFile,
                percent : entriesCount ? (currentFilePercent + 100 * (entriesCount - remainingFiles - 1)) / entriesCount : 100
            }
        });
    }
};

/**
 * The worker started a new source (an other worker).
 * @param {Object} streamInfo the streamInfo object from the new source.
 */
ZipFileWorker.prototype.openedSource = function (streamInfo) {
    this.currentSourceOffset = this.bytesWritten;
    this.currentFile = streamInfo['file'].name;

    var streamedContent = this.streamFiles && !streamInfo['file'].dir;

    // don't stream folders (because they don't have any content)
    if(streamedContent) {
        var record = generateZipParts(streamInfo, streamedContent, false, this.currentSourceOffset, this.zipPlatform, this.encodeFileName);
        this.push({
            data : record.fileRecord,
            meta : {percent:0}
        });
    } else {
        // we need to wait for the whole file before pushing anything
        this.accumulate = true;
    }
};

/**
 * The worker finished a source (an other worker).
 * @param {Object} streamInfo the streamInfo object from the finished source.
 */
ZipFileWorker.prototype.closedSource = function (streamInfo) {
    this.accumulate = false;
    var streamedContent = this.streamFiles && !streamInfo['file'].dir;
    var record = generateZipParts(streamInfo, streamedContent, true, this.currentSourceOffset, this.zipPlatform, this.encodeFileName);

    this.dirRecords.push(record.dirRecord);
    if(streamedContent) {
        // after the streamed file, we put data descriptors
        this.push({
            data : generateDataDescriptors(streamInfo),
            meta : {percent:100}
        });
    } else {
        // the content wasn't streamed, we need to push everything now
        // first the file record, then the content
        this.push({
            data : record.fileRecord,
            meta : {percent:0}
        });
        while(this.contentBuffer.length) {
            this.push(this.contentBuffer.shift());
        }
    }
    this.currentFile = null;
};

/**
 * @see GenericWorker.flush
 */
ZipFileWorker.prototype.flush = function () {

    var localDirLength = this.bytesWritten;
    for(var i = 0; i < this.dirRecords.length; i++) {
        this.push({
            data : this.dirRecords[i],
            meta : {percent:100}
        });
    }
    var centralDirLength = this.bytesWritten - localDirLength;

    var dirEnd = generateCentralDirectoryEnd(this.dirRecords.length, centralDirLength, localDirLength, this.zipComment, this.encodeFileName);

    this.push({
        data : dirEnd,
        meta : {percent:100}
    });
};

/**
 * Prepare the next source to be read.
 */
ZipFileWorker.prototype.prepareNextSource = function () {
    this.previous = this._sources.shift();
    this.openedSource(this.previous.streamInfo);
    if (this.isPaused) {
        this.previous.pause();
    } else {
        this.previous.resume();
    }
};

/**
 * @see GenericWorker.registerPrevious
 */
ZipFileWorker.prototype.registerPrevious = function (previous) {
    this._sources.push(previous);
    var self = this;

    previous.on('data', function (chunk) {
        self.processChunk(chunk);
    });
    previous.on('end', function () {
        self.closedSource(self.previous.streamInfo);
        if(self._sources.length) {
            self.prepareNextSource();
        } else {
            self.end();
        }
    });
    previous.on('error', function (e) {
        self.error(e);
    });
    return this;
};

/**
 * @see GenericWorker.resume
 */
ZipFileWorker.prototype.resume = function () {
    if(!GenericWorker_1.prototype.resume.call(this)) {
        return false;
    }

    if (!this.previous && this._sources.length) {
        this.prepareNextSource();
        return true;
    }
    if (!this.previous && !this._sources.length && !this.generatedError) {
        this.end();
        return true;
    }
};

/**
 * @see GenericWorker.error
 */
ZipFileWorker.prototype.error = function (e) {
    var sources = this._sources;
    if(!GenericWorker_1.prototype.error.call(this, e)) {
        return false;
    }
    for(var i = 0; i < sources.length; i++) {
        try {
            sources[i].error(e);
        } catch(e) {
            // the `error` exploded, nothing to do
        }
    }
    return true;
};

/**
 * @see GenericWorker.lock
 */
ZipFileWorker.prototype.lock = function () {
    GenericWorker_1.prototype.lock.call(this);
    var sources = this._sources;
    for(var i = 0; i < sources.length; i++) {
        sources[i].lock();
    }
};

var ZipFileWorker_1 = ZipFileWorker;

/**
 * Find the compression to use.
 * @param {String} fileCompression the compression defined at the file level, if any.
 * @param {String} zipCompression the compression defined at the load() level.
 * @return {Object} the compression object to use.
 */
var getCompression = function (fileCompression, zipCompression) {

    var compressionName = fileCompression || zipCompression;
    var compression = compressions[compressionName];
    if (!compression) {
        throw new Error(compressionName + " is not a valid compression method !");
    }
    return compression;
};

/**
 * Create a worker to generate a zip file.
 * @param {JSZip} zip the JSZip instance at the right root level.
 * @param {Object} options to generate the zip file.
 * @param {String} comment the comment to use.
 */
var generateWorker = function (zip, options, comment) {

    var zipFileWorker = new ZipFileWorker_1(options.streamFiles, comment, options.platform, options.encodeFileName);
    var entriesCount = 0;
    try {

        zip.forEach(function (relativePath, file) {
            entriesCount++;
            var compression = getCompression(file.options.compression, options.compression);
            var compressionOptions = file.options.compressionOptions || options.compressionOptions || {};
            var dir = file.dir, date = file.date;

            file._compressWorker(compression, compressionOptions)
            .withStreamInfo("file", {
                name : relativePath,
                dir : dir,
                date : date,
                comment : file.comment || "",
                unixPermissions : file.unixPermissions,
                dosPermissions : file.dosPermissions
            })
            .pipe(zipFileWorker);
        });
        zipFileWorker.entriesCount = entriesCount;
    } catch (e) {
        zipFileWorker.error(e);
    }

    return zipFileWorker;
};

var generate = {
	generateWorker: generateWorker
};

/**
 * A worker that use a nodejs stream as source.
 * @constructor
 * @param {String} filename the name of the file entry for this stream.
 * @param {Readable} stream the nodejs stream.
 */
function NodejsStreamInputAdapter(filename, stream) {
    GenericWorker_1.call(this, "Nodejs stream input adapter for " + filename);
    this._upstreamEnded = false;
    this._bindStream(stream);
}

utils.inherits(NodejsStreamInputAdapter, GenericWorker_1);

/**
 * Prepare the stream and bind the callbacks on it.
 * Do this ASAP on node 0.10 ! A lazy binding doesn't always work.
 * @param {Stream} stream the nodejs stream to use.
 */
NodejsStreamInputAdapter.prototype._bindStream = function (stream) {
    var self = this;
    this._stream = stream;
    stream.pause();
    stream
    .on("data", function (chunk) {
        self.push({
            data: chunk,
            meta : {
                percent : 0
            }
        });
    })
    .on("error", function (e) {
        if(self.isPaused) {
            this.generatedError = e;
        } else {
            self.error(e);
        }
    })
    .on("end", function () {
        if(self.isPaused) {
            self._upstreamEnded = true;
        } else {
            self.end();
        }
    });
};
NodejsStreamInputAdapter.prototype.pause = function () {
    if(!GenericWorker_1.prototype.pause.call(this)) {
        return false;
    }
    this._stream.pause();
    return true;
};
NodejsStreamInputAdapter.prototype.resume = function () {
    if(!GenericWorker_1.prototype.resume.call(this)) {
        return false;
    }

    if(this._upstreamEnded) {
        this.end();
    } else {
        this._stream.resume();
    }

    return true;
};

var NodejsStreamInputAdapter_1 = NodejsStreamInputAdapter;

/**
 * Add a file in the current folder.
 * @private
 * @param {string} name the name of the file
 * @param {String|ArrayBuffer|Uint8Array|Buffer} data the data of the file
 * @param {Object} originalOptions the options of the file
 * @return {Object} the new file.
 */
var fileAdd = function(name, data, originalOptions) {
    // be sure sub folders exist
    var dataType = utils.getTypeOf(data),
        parent;


    /*
     * Correct options.
     */

    var o = utils.extend(originalOptions || {}, defaults);
    o.date = o.date || new Date();
    if (o.compression !== null) {
        o.compression = o.compression.toUpperCase();
    }

    if (typeof o.unixPermissions === "string") {
        o.unixPermissions = parseInt(o.unixPermissions, 8);
    }

    // UNX_IFDIR  0040000 see zipinfo.c
    if (o.unixPermissions && (o.unixPermissions & 0x4000)) {
        o.dir = true;
    }
    // Bit 4    Directory
    if (o.dosPermissions && (o.dosPermissions & 0x0010)) {
        o.dir = true;
    }

    if (o.dir) {
        name = forceTrailingSlash(name);
    }
    if (o.createFolders && (parent = parentFolder(name))) {
        folderAdd.call(this, parent, true);
    }

    var isUnicodeString = dataType === "string" && o.binary === false && o.base64 === false;
    if (!originalOptions || typeof originalOptions.binary === "undefined") {
        o.binary = !isUnicodeString;
    }


    var isCompressedEmpty = (data instanceof compressedObject) && data.uncompressedSize === 0;

    if (isCompressedEmpty || o.dir || !data || data.length === 0) {
        o.base64 = false;
        o.binary = true;
        data = "";
        o.compression = "STORE";
        dataType = "string";
    }

    /*
     * Convert content to fit.
     */

    var zipObjectContent = null;
    if (data instanceof compressedObject || data instanceof GenericWorker_1) {
        zipObjectContent = data;
    } else if (nodejsUtils.isNode && nodejsUtils.isStream(data)) {
        zipObjectContent = new NodejsStreamInputAdapter_1(name, data);
    } else {
        zipObjectContent = utils.prepareContent(name, data, o.binary, o.optimizedBinaryString, o.base64);
    }

    var object = new zipObject(name, zipObjectContent, o);
    this.files[name] = object;
    /*
    TODO: we can't throw an exception because we have async promises
    (we can have a promise of a Date() for example) but returning a
    promise is useless because file(name, data) returns the JSZip
    object for chaining. Should we break that to allow the user
    to catch the error ?

    return external.Promise.resolve(zipObjectContent)
    .then(function () {
        return object;
    });
    */
};

/**
 * Find the parent folder of the path.
 * @private
 * @param {string} path the path to use
 * @return {string} the parent folder, or ""
 */
var parentFolder = function (path) {
    if (path.slice(-1) === '/') {
        path = path.substring(0, path.length - 1);
    }
    var lastSlash = path.lastIndexOf('/');
    return (lastSlash > 0) ? path.substring(0, lastSlash) : "";
};

/**
 * Returns the path with a slash at the end.
 * @private
 * @param {String} path the path to check.
 * @return {String} the path with a trailing slash.
 */
var forceTrailingSlash = function(path) {
    // Check the name ends with a /
    if (path.slice(-1) !== "/") {
        path += "/"; // IE doesn't like substr(-1)
    }
    return path;
};

/**
 * Add a (sub) folder in the current folder.
 * @private
 * @param {string} name the folder's name
 * @param {boolean=} [createFolders] If true, automatically create sub
 *  folders. Defaults to false.
 * @return {Object} the new folder.
 */
var folderAdd = function(name, createFolders) {
    createFolders = (typeof createFolders !== 'undefined') ? createFolders : defaults.createFolders;

    name = forceTrailingSlash(name);

    // Does this folder already exist?
    if (!this.files[name]) {
        fileAdd.call(this, name, null, {
            dir: true,
            createFolders: createFolders
        });
    }
    return this.files[name];
};

/**
* Cross-window, cross-Node-context regular expression detection
* @param  {Object}  object Anything
* @return {Boolean}        true if the object is a regular expression,
* false otherwise
*/
function isRegExp(object) {
    return Object.prototype.toString.call(object) === "[object RegExp]";
}

// return the actual prototype of JSZip
var out = {
    /**
     * @see loadAsync
     */
    load: function() {
        throw new Error("This method has been removed in JSZip 3.0, please check the upgrade guide.");
    },


    /**
     * Call a callback function for each entry at this folder level.
     * @param {Function} cb the callback function:
     * function (relativePath, file) {...}
     * It takes 2 arguments : the relative path and the file.
     */
    forEach: function(cb) {
        var filename, relativePath, file;
        for (filename in this.files) {
            if (!this.files.hasOwnProperty(filename)) {
                continue;
            }
            file = this.files[filename];
            relativePath = filename.slice(this.root.length, filename.length);
            if (relativePath && filename.slice(0, this.root.length) === this.root) { // the file is in the current root
                cb(relativePath, file); // TODO reverse the parameters ? need to be clean AND consistent with the filter search fn...
            }
        }
    },

    /**
     * Filter nested files/folders with the specified function.
     * @param {Function} search the predicate to use :
     * function (relativePath, file) {...}
     * It takes 2 arguments : the relative path and the file.
     * @return {Array} An array of matching elements.
     */
    filter: function(search) {
        var result = [];
        this.forEach(function (relativePath, entry) {
            if (search(relativePath, entry)) { // the file matches the function
                result.push(entry);
            }

        });
        return result;
    },

    /**
     * Add a file to the zip file, or search a file.
     * @param   {string|RegExp} name The name of the file to add (if data is defined),
     * the name of the file to find (if no data) or a regex to match files.
     * @param   {String|ArrayBuffer|Uint8Array|Buffer} data  The file data, either raw or base64 encoded
     * @param   {Object} o     File options
     * @return  {JSZip|Object|Array} this JSZip object (when adding a file),
     * a file (when searching by string) or an array of files (when searching by regex).
     */
    file: function(name, data, o) {
        if (arguments.length === 1) {
            if (isRegExp(name)) {
                var regexp = name;
                return this.filter(function(relativePath, file) {
                    return !file.dir && regexp.test(relativePath);
                });
            }
            else { // text
                var obj = this.files[this.root + name];
                if (obj && !obj.dir) {
                    return obj;
                } else {
                    return null;
                }
            }
        }
        else { // more than one argument : we have data !
            name = this.root + name;
            fileAdd.call(this, name, data, o);
        }
        return this;
    },

    /**
     * Add a directory to the zip file, or search.
     * @param   {String|RegExp} arg The name of the directory to add, or a regex to search folders.
     * @return  {JSZip} an object with the new directory as the root, or an array containing matching folders.
     */
    folder: function(arg) {
        if (!arg) {
            return this;
        }

        if (isRegExp(arg)) {
            return this.filter(function(relativePath, file) {
                return file.dir && arg.test(relativePath);
            });
        }

        // else, name is a new folder
        var name = this.root + arg;
        var newFolder = folderAdd.call(this, name);

        // Allow chaining by returning a new object with this folder as the root
        var ret = this.clone();
        ret.root = newFolder.name;
        return ret;
    },

    /**
     * Delete a file, or a directory and all sub-files, from the zip
     * @param {string} name the name of the file to delete
     * @return {JSZip} this JSZip object
     */
    remove: function(name) {
        name = this.root + name;
        var file = this.files[name];
        if (!file) {
            // Look for any folders
            if (name.slice(-1) !== "/") {
                name += "/";
            }
            file = this.files[name];
        }

        if (file && !file.dir) {
            // file
            delete this.files[name];
        } else {
            // maybe a folder, delete recursively
            var kids = this.filter(function(relativePath, file) {
                return file.name.slice(0, name.length) === name;
            });
            for (var i = 0; i < kids.length; i++) {
                delete this.files[kids[i].name];
            }
        }

        return this;
    },

    /**
     * Generate the complete zip file
     * @param {Object} options the options to generate the zip file :
     * - compression, "STORE" by default.
     * - type, "base64" by default. Values are : string, base64, uint8array, arraybuffer, blob.
     * @return {String|Uint8Array|ArrayBuffer|Buffer|Blob} the zip file
     */
    generate: function(options) {
        throw new Error("This method has been removed in JSZip 3.0, please check the upgrade guide.");
    },

    /**
     * Generate the complete zip file as an internal stream.
     * @param {Object} options the options to generate the zip file :
     * - compression, "STORE" by default.
     * - type, "base64" by default. Values are : string, base64, uint8array, arraybuffer, blob.
     * @return {StreamHelper} the streamed zip file.
     */
    generateInternalStream: function(options) {
      var worker, opts = {};
      try {
          opts = utils.extend(options || {}, {
              streamFiles: false,
              compression: "STORE",
              compressionOptions : null,
              type: "",
              platform: "DOS",
              comment: null,
              mimeType: 'application/zip',
              encodeFileName: utf8.utf8encode
          });

          opts.type = opts.type.toLowerCase();
          opts.compression = opts.compression.toUpperCase();

          // "binarystring" is prefered but the internals use "string".
          if(opts.type === "binarystring") {
            opts.type = "string";
          }

          if (!opts.type) {
            throw new Error("No output type specified.");
          }

          utils.checkSupport(opts.type);

          // accept nodejs `process.platform`
          if(
              opts.platform === 'darwin' ||
              opts.platform === 'freebsd' ||
              opts.platform === 'linux' ||
              opts.platform === 'sunos'
          ) {
              opts.platform = "UNIX";
          }
          if (opts.platform === 'win32') {
              opts.platform = "DOS";
          }

          var comment = opts.comment || this.comment || "";
          worker = generate.generateWorker(this, opts, comment);
      } catch (e) {
        worker = new GenericWorker_1("error");
        worker.error(e);
      }
      return new StreamHelper_1(worker, opts.type || "string", opts.mimeType);
    },
    /**
     * Generate the complete zip file asynchronously.
     * @see generateInternalStream
     */
    generateAsync: function(options, onUpdate) {
        return this.generateInternalStream(options).accumulate(onUpdate);
    },
    /**
     * Generate the complete zip file asynchronously.
     * @see generateInternalStream
     */
    generateNodeStream: function(options, onUpdate) {
        options = options || {};
        if (!options.type) {
            options.type = "nodebuffer";
        }
        return this.generateInternalStream(options).toNodejsStream(onUpdate);
    }
};
var object = out;

function DataReader(data) {
    this.data = data; // type : see implementation
    this.length = data.length;
    this.index = 0;
    this.zero = 0;
}
DataReader.prototype = {
    /**
     * Check that the offset will not go too far.
     * @param {string} offset the additional offset to check.
     * @throws {Error} an Error if the offset is out of bounds.
     */
    checkOffset: function(offset) {
        this.checkIndex(this.index + offset);
    },
    /**
     * Check that the specified index will not be too far.
     * @param {string} newIndex the index to check.
     * @throws {Error} an Error if the index is out of bounds.
     */
    checkIndex: function(newIndex) {
        if (this.length < this.zero + newIndex || newIndex < 0) {
            throw new Error("End of data reached (data length = " + this.length + ", asked index = " + (newIndex) + "). Corrupted zip ?");
        }
    },
    /**
     * Change the index.
     * @param {number} newIndex The new index.
     * @throws {Error} if the new index is out of the data.
     */
    setIndex: function(newIndex) {
        this.checkIndex(newIndex);
        this.index = newIndex;
    },
    /**
     * Skip the next n bytes.
     * @param {number} n the number of bytes to skip.
     * @throws {Error} if the new index is out of the data.
     */
    skip: function(n) {
        this.setIndex(this.index + n);
    },
    /**
     * Get the byte at the specified index.
     * @param {number} i the index to use.
     * @return {number} a byte.
     */
    byteAt: function(i) {
        // see implementations
    },
    /**
     * Get the next number with a given byte size.
     * @param {number} size the number of bytes to read.
     * @return {number} the corresponding number.
     */
    readInt: function(size) {
        var result = 0,
            i;
        this.checkOffset(size);
        for (i = this.index + size - 1; i >= this.index; i--) {
            result = (result << 8) + this.byteAt(i);
        }
        this.index += size;
        return result;
    },
    /**
     * Get the next string with a given byte size.
     * @param {number} size the number of bytes to read.
     * @return {string} the corresponding string.
     */
    readString: function(size) {
        return utils.transformTo("string", this.readData(size));
    },
    /**
     * Get raw data without conversion, <size> bytes.
     * @param {number} size the number of bytes to read.
     * @return {Object} the raw data, implementation specific.
     */
    readData: function(size) {
        // see implementations
    },
    /**
     * Find the last occurence of a zip signature (4 bytes).
     * @param {string} sig the signature to find.
     * @return {number} the index of the last occurence, -1 if not found.
     */
    lastIndexOfSignature: function(sig) {
        // see implementations
    },
    /**
     * Read the signature (4 bytes) at the current position and compare it with sig.
     * @param {string} sig the expected signature
     * @return {boolean} true if the signature matches, false otherwise.
     */
    readAndCheckSignature: function(sig) {
        // see implementations
    },
    /**
     * Get the next date.
     * @return {Date} the date.
     */
    readDate: function() {
        var dostime = this.readInt(4);
        return new Date(Date.UTC(
        ((dostime >> 25) & 0x7f) + 1980, // year
        ((dostime >> 21) & 0x0f) - 1, // month
        (dostime >> 16) & 0x1f, // day
        (dostime >> 11) & 0x1f, // hour
        (dostime >> 5) & 0x3f, // minute
        (dostime & 0x1f) << 1)); // second
    }
};
var DataReader_1 = DataReader;

function ArrayReader(data) {
    DataReader_1.call(this, data);
	for(var i = 0; i < this.data.length; i++) {
		data[i] = data[i] & 0xFF;
	}
}
utils.inherits(ArrayReader, DataReader_1);
/**
 * @see DataReader.byteAt
 */
ArrayReader.prototype.byteAt = function(i) {
    return this.data[this.zero + i];
};
/**
 * @see DataReader.lastIndexOfSignature
 */
ArrayReader.prototype.lastIndexOfSignature = function(sig) {
    var sig0 = sig.charCodeAt(0),
        sig1 = sig.charCodeAt(1),
        sig2 = sig.charCodeAt(2),
        sig3 = sig.charCodeAt(3);
    for (var i = this.length - 4; i >= 0; --i) {
        if (this.data[i] === sig0 && this.data[i + 1] === sig1 && this.data[i + 2] === sig2 && this.data[i + 3] === sig3) {
            return i - this.zero;
        }
    }

    return -1;
};
/**
 * @see DataReader.readAndCheckSignature
 */
ArrayReader.prototype.readAndCheckSignature = function (sig) {
    var sig0 = sig.charCodeAt(0),
        sig1 = sig.charCodeAt(1),
        sig2 = sig.charCodeAt(2),
        sig3 = sig.charCodeAt(3),
        data = this.readData(4);
    return sig0 === data[0] && sig1 === data[1] && sig2 === data[2] && sig3 === data[3];
};
/**
 * @see DataReader.readData
 */
ArrayReader.prototype.readData = function(size) {
    this.checkOffset(size);
    if(size === 0) {
        return [];
    }
    var result = this.data.slice(this.zero + this.index, this.zero + this.index + size);
    this.index += size;
    return result;
};
var ArrayReader_1 = ArrayReader;

function StringReader(data) {
    DataReader_1.call(this, data);
}
utils.inherits(StringReader, DataReader_1);
/**
 * @see DataReader.byteAt
 */
StringReader.prototype.byteAt = function(i) {
    return this.data.charCodeAt(this.zero + i);
};
/**
 * @see DataReader.lastIndexOfSignature
 */
StringReader.prototype.lastIndexOfSignature = function(sig) {
    return this.data.lastIndexOf(sig) - this.zero;
};
/**
 * @see DataReader.readAndCheckSignature
 */
StringReader.prototype.readAndCheckSignature = function (sig) {
    var data = this.readData(4);
    return sig === data;
};
/**
 * @see DataReader.readData
 */
StringReader.prototype.readData = function(size) {
    this.checkOffset(size);
    // this will work because the constructor applied the "& 0xff" mask.
    var result = this.data.slice(this.zero + this.index, this.zero + this.index + size);
    this.index += size;
    return result;
};
var StringReader_1 = StringReader;

function Uint8ArrayReader(data) {
    ArrayReader_1.call(this, data);
}
utils.inherits(Uint8ArrayReader, ArrayReader_1);
/**
 * @see DataReader.readData
 */
Uint8ArrayReader.prototype.readData = function(size) {
    this.checkOffset(size);
    if(size === 0) {
        // in IE10, when using subarray(idx, idx), we get the array [0x00] instead of [].
        return new Uint8Array(0);
    }
    var result = this.data.subarray(this.zero + this.index, this.zero + this.index + size);
    this.index += size;
    return result;
};
var Uint8ArrayReader_1 = Uint8ArrayReader;

function NodeBufferReader(data) {
    Uint8ArrayReader_1.call(this, data);
}
utils.inherits(NodeBufferReader, Uint8ArrayReader_1);

/**
 * @see DataReader.readData
 */
NodeBufferReader.prototype.readData = function(size) {
    this.checkOffset(size);
    var result = this.data.slice(this.zero + this.index, this.zero + this.index + size);
    this.index += size;
    return result;
};
var NodeBufferReader_1 = NodeBufferReader;

/**
 * Create a reader adapted to the data.
 * @param {String|ArrayBuffer|Uint8Array|Buffer} data the data to read.
 * @return {DataReader} the data reader.
 */
var readerFor = function (data) {
    var type = utils.getTypeOf(data);
    utils.checkSupport(type);
    if (type === "string" && !support.uint8array) {
        return new StringReader_1(data);
    }
    if (type === "nodebuffer") {
        return new NodeBufferReader_1(data);
    }
    if (support.uint8array) {
        return new Uint8ArrayReader_1(utils.transformTo("uint8array", data));
    }
    return new ArrayReader_1(utils.transformTo("array", data));
};

var MADE_BY_DOS = 0x00;
var MADE_BY_UNIX = 0x03;

/**
 * Find a compression registered in JSZip.
 * @param {string} compressionMethod the method magic to find.
 * @return {Object|null} the JSZip compression object, null if none found.
 */
var findCompression = function(compressionMethod) {
    for (var method in compressions) {
        if (!compressions.hasOwnProperty(method)) {
            continue;
        }
        if (compressions[method].magic === compressionMethod) {
            return compressions[method];
        }
    }
    return null;
};

// class ZipEntry {{{
/**
 * An entry in the zip file.
 * @constructor
 * @param {Object} options Options of the current file.
 * @param {Object} loadOptions Options for loading the stream.
 */
function ZipEntry(options, loadOptions) {
    this.options = options;
    this.loadOptions = loadOptions;
}
ZipEntry.prototype = {
    /**
     * say if the file is encrypted.
     * @return {boolean} true if the file is encrypted, false otherwise.
     */
    isEncrypted: function() {
        // bit 1 is set
        return (this.bitFlag & 0x0001) === 0x0001;
    },
    /**
     * say if the file has utf-8 filename/comment.
     * @return {boolean} true if the filename/comment is in utf-8, false otherwise.
     */
    useUTF8: function() {
        // bit 11 is set
        return (this.bitFlag & 0x0800) === 0x0800;
    },
    /**
     * Read the local part of a zip file and add the info in this object.
     * @param {DataReader} reader the reader to use.
     */
    readLocalPart: function(reader) {
        var compression, localExtraFieldsLength;

        // we already know everything from the central dir !
        // If the central dir data are false, we are doomed.
        // On the bright side, the local part is scary  : zip64, data descriptors, both, etc.
        // The less data we get here, the more reliable this should be.
        // Let's skip the whole header and dash to the data !
        reader.skip(22);
        // in some zip created on windows, the filename stored in the central dir contains \ instead of /.
        // Strangely, the filename here is OK.
        // I would love to treat these zip files as corrupted (see http://www.info-zip.org/FAQ.html#backslashes
        // or APPNOTE#4.4.17.1, "All slashes MUST be forward slashes '/'") but there are a lot of bad zip generators...
        // Search "unzip mismatching "local" filename continuing with "central" filename version" on
        // the internet.
        //
        // I think I see the logic here : the central directory is used to display
        // content and the local directory is used to extract the files. Mixing / and \
        // may be used to display \ to windows users and use / when extracting the files.
        // Unfortunately, this lead also to some issues : http://seclists.org/fulldisclosure/2009/Sep/394
        this.fileNameLength = reader.readInt(2);
        localExtraFieldsLength = reader.readInt(2); // can't be sure this will be the same as the central dir
        // the fileName is stored as binary data, the handleUTF8 method will take care of the encoding.
        this.fileName = reader.readData(this.fileNameLength);
        reader.skip(localExtraFieldsLength);

        if (this.compressedSize === -1 || this.uncompressedSize === -1) {
            throw new Error("Bug or corrupted zip : didn't get enough informations from the central directory " + "(compressedSize === -1 || uncompressedSize === -1)");
        }

        compression = findCompression(this.compressionMethod);
        if (compression === null) { // no compression found
            throw new Error("Corrupted zip : compression " + utils.pretty(this.compressionMethod) + " unknown (inner file : " + utils.transformTo("string", this.fileName) + ")");
        }
        this.decompressed = new compressedObject(this.compressedSize, this.uncompressedSize, this.crc32, compression, reader.readData(this.compressedSize));
    },

    /**
     * Read the central part of a zip file and add the info in this object.
     * @param {DataReader} reader the reader to use.
     */
    readCentralPart: function(reader) {
        this.versionMadeBy = reader.readInt(2);
        reader.skip(2);
        // this.versionNeeded = reader.readInt(2);
        this.bitFlag = reader.readInt(2);
        this.compressionMethod = reader.readString(2);
        this.date = reader.readDate();
        this.crc32 = reader.readInt(4);
        this.compressedSize = reader.readInt(4);
        this.uncompressedSize = reader.readInt(4);
        var fileNameLength = reader.readInt(2);
        this.extraFieldsLength = reader.readInt(2);
        this.fileCommentLength = reader.readInt(2);
        this.diskNumberStart = reader.readInt(2);
        this.internalFileAttributes = reader.readInt(2);
        this.externalFileAttributes = reader.readInt(4);
        this.localHeaderOffset = reader.readInt(4);

        if (this.isEncrypted()) {
            throw new Error("Encrypted zip are not supported");
        }

        // will be read in the local part, see the comments there
        reader.skip(fileNameLength);
        this.readExtraFields(reader);
        this.parseZIP64ExtraField(reader);
        this.fileComment = reader.readData(this.fileCommentLength);
    },

    /**
     * Parse the external file attributes and get the unix/dos permissions.
     */
    processAttributes: function () {
        this.unixPermissions = null;
        this.dosPermissions = null;
        var madeBy = this.versionMadeBy >> 8;

        // Check if we have the DOS directory flag set.
        // We look for it in the DOS and UNIX permissions
        // but some unknown platform could set it as a compatibility flag.
        this.dir = this.externalFileAttributes & 0x0010 ? true : false;

        if(madeBy === MADE_BY_DOS) {
            // first 6 bits (0 to 5)
            this.dosPermissions = this.externalFileAttributes & 0x3F;
        }

        if(madeBy === MADE_BY_UNIX) {
            this.unixPermissions = (this.externalFileAttributes >> 16) & 0xFFFF;
            // the octal permissions are in (this.unixPermissions & 0x01FF).toString(8);
        }

        // fail safe : if the name ends with a / it probably means a folder
        if (!this.dir && this.fileNameStr.slice(-1) === '/') {
            this.dir = true;
        }
    },

    /**
     * Parse the ZIP64 extra field and merge the info in the current ZipEntry.
     * @param {DataReader} reader the reader to use.
     */
    parseZIP64ExtraField: function(reader) {

        if (!this.extraFields[0x0001]) {
            return;
        }

        // should be something, preparing the extra reader
        var extraReader = readerFor(this.extraFields[0x0001].value);

        // I really hope that these 64bits integer can fit in 32 bits integer, because js
        // won't let us have more.
        if (this.uncompressedSize === utils.MAX_VALUE_32BITS) {
            this.uncompressedSize = extraReader.readInt(8);
        }
        if (this.compressedSize === utils.MAX_VALUE_32BITS) {
            this.compressedSize = extraReader.readInt(8);
        }
        if (this.localHeaderOffset === utils.MAX_VALUE_32BITS) {
            this.localHeaderOffset = extraReader.readInt(8);
        }
        if (this.diskNumberStart === utils.MAX_VALUE_32BITS) {
            this.diskNumberStart = extraReader.readInt(4);
        }
    },
    /**
     * Read the central part of a zip file and add the info in this object.
     * @param {DataReader} reader the reader to use.
     */
    readExtraFields: function(reader) {
        var end = reader.index + this.extraFieldsLength,
            extraFieldId,
            extraFieldLength,
            extraFieldValue;

        if (!this.extraFields) {
            this.extraFields = {};
        }

        while (reader.index < end) {
            extraFieldId = reader.readInt(2);
            extraFieldLength = reader.readInt(2);
            extraFieldValue = reader.readData(extraFieldLength);

            this.extraFields[extraFieldId] = {
                id: extraFieldId,
                length: extraFieldLength,
                value: extraFieldValue
            };
        }
    },
    /**
     * Apply an UTF8 transformation if needed.
     */
    handleUTF8: function() {
        var decodeParamType = support.uint8array ? "uint8array" : "array";
        if (this.useUTF8()) {
            this.fileNameStr = utf8.utf8decode(this.fileName);
            this.fileCommentStr = utf8.utf8decode(this.fileComment);
        } else {
            var upath = this.findExtraFieldUnicodePath();
            if (upath !== null) {
                this.fileNameStr = upath;
            } else {
                // ASCII text or unsupported code page
                var fileNameByteArray =  utils.transformTo(decodeParamType, this.fileName);
                this.fileNameStr = this.loadOptions.decodeFileName(fileNameByteArray);
            }

            var ucomment = this.findExtraFieldUnicodeComment();
            if (ucomment !== null) {
                this.fileCommentStr = ucomment;
            } else {
                // ASCII text or unsupported code page
                var commentByteArray =  utils.transformTo(decodeParamType, this.fileComment);
                this.fileCommentStr = this.loadOptions.decodeFileName(commentByteArray);
            }
        }
    },

    /**
     * Find the unicode path declared in the extra field, if any.
     * @return {String} the unicode path, null otherwise.
     */
    findExtraFieldUnicodePath: function() {
        var upathField = this.extraFields[0x7075];
        if (upathField) {
            var extraReader = readerFor(upathField.value);

            // wrong version
            if (extraReader.readInt(1) !== 1) {
                return null;
            }

            // the crc of the filename changed, this field is out of date.
            if (crc32_1(this.fileName) !== extraReader.readInt(4)) {
                return null;
            }

            return utf8.utf8decode(extraReader.readData(upathField.length - 5));
        }
        return null;
    },

    /**
     * Find the unicode comment declared in the extra field, if any.
     * @return {String} the unicode comment, null otherwise.
     */
    findExtraFieldUnicodeComment: function() {
        var ucommentField = this.extraFields[0x6375];
        if (ucommentField) {
            var extraReader = readerFor(ucommentField.value);

            // wrong version
            if (extraReader.readInt(1) !== 1) {
                return null;
            }

            // the crc of the comment changed, this field is out of date.
            if (crc32_1(this.fileComment) !== extraReader.readInt(4)) {
                return null;
            }

            return utf8.utf8decode(extraReader.readData(ucommentField.length - 5));
        }
        return null;
    }
};
var zipEntry = ZipEntry;

//  class ZipEntries {{{
/**
 * All the entries in the zip file.
 * @constructor
 * @param {Object} loadOptions Options for loading the stream.
 */
function ZipEntries(loadOptions) {
    this.files = [];
    this.loadOptions = loadOptions;
}
ZipEntries.prototype = {
    /**
     * Check that the reader is on the specified signature.
     * @param {string} expectedSignature the expected signature.
     * @throws {Error} if it is an other signature.
     */
    checkSignature: function(expectedSignature) {
        if (!this.reader.readAndCheckSignature(expectedSignature)) {
            this.reader.index -= 4;
            var signature$$1 = this.reader.readString(4);
            throw new Error("Corrupted zip or bug: unexpected signature " + "(" + utils.pretty(signature$$1) + ", expected " + utils.pretty(expectedSignature) + ")");
        }
    },
    /**
     * Check if the given signature is at the given index.
     * @param {number} askedIndex the index to check.
     * @param {string} expectedSignature the signature to expect.
     * @return {boolean} true if the signature is here, false otherwise.
     */
    isSignature: function(askedIndex, expectedSignature) {
        var currentIndex = this.reader.index;
        this.reader.setIndex(askedIndex);
        var signature$$1 = this.reader.readString(4);
        var result = signature$$1 === expectedSignature;
        this.reader.setIndex(currentIndex);
        return result;
    },
    /**
     * Read the end of the central directory.
     */
    readBlockEndOfCentral: function() {
        this.diskNumber = this.reader.readInt(2);
        this.diskWithCentralDirStart = this.reader.readInt(2);
        this.centralDirRecordsOnThisDisk = this.reader.readInt(2);
        this.centralDirRecords = this.reader.readInt(2);
        this.centralDirSize = this.reader.readInt(4);
        this.centralDirOffset = this.reader.readInt(4);

        this.zipCommentLength = this.reader.readInt(2);
        // warning : the encoding depends of the system locale
        // On a linux machine with LANG=en_US.utf8, this field is utf8 encoded.
        // On a windows machine, this field is encoded with the localized windows code page.
        var zipComment = this.reader.readData(this.zipCommentLength);
        var decodeParamType = support.uint8array ? "uint8array" : "array";
        // To get consistent behavior with the generation part, we will assume that
        // this is utf8 encoded unless specified otherwise.
        var decodeContent = utils.transformTo(decodeParamType, zipComment);
        this.zipComment = this.loadOptions.decodeFileName(decodeContent);
    },
    /**
     * Read the end of the Zip 64 central directory.
     * Not merged with the method readEndOfCentral :
     * The end of central can coexist with its Zip64 brother,
     * I don't want to read the wrong number of bytes !
     */
    readBlockZip64EndOfCentral: function() {
        this.zip64EndOfCentralSize = this.reader.readInt(8);
        this.reader.skip(4);
        // this.versionMadeBy = this.reader.readString(2);
        // this.versionNeeded = this.reader.readInt(2);
        this.diskNumber = this.reader.readInt(4);
        this.diskWithCentralDirStart = this.reader.readInt(4);
        this.centralDirRecordsOnThisDisk = this.reader.readInt(8);
        this.centralDirRecords = this.reader.readInt(8);
        this.centralDirSize = this.reader.readInt(8);
        this.centralDirOffset = this.reader.readInt(8);

        this.zip64ExtensibleData = {};
        var extraDataSize = this.zip64EndOfCentralSize - 44,
            index = 0,
            extraFieldId,
            extraFieldLength,
            extraFieldValue;
        while (index < extraDataSize) {
            extraFieldId = this.reader.readInt(2);
            extraFieldLength = this.reader.readInt(4);
            extraFieldValue = this.reader.readData(extraFieldLength);
            this.zip64ExtensibleData[extraFieldId] = {
                id: extraFieldId,
                length: extraFieldLength,
                value: extraFieldValue
            };
        }
    },
    /**
     * Read the end of the Zip 64 central directory locator.
     */
    readBlockZip64EndOfCentralLocator: function() {
        this.diskWithZip64CentralDirStart = this.reader.readInt(4);
        this.relativeOffsetEndOfZip64CentralDir = this.reader.readInt(8);
        this.disksCount = this.reader.readInt(4);
        if (this.disksCount > 1) {
            throw new Error("Multi-volumes zip are not supported");
        }
    },
    /**
     * Read the local files, based on the offset read in the central part.
     */
    readLocalFiles: function() {
        var i, file;
        for (i = 0; i < this.files.length; i++) {
            file = this.files[i];
            this.reader.setIndex(file.localHeaderOffset);
            this.checkSignature(signature.LOCAL_FILE_HEADER);
            file.readLocalPart(this.reader);
            file.handleUTF8();
            file.processAttributes();
        }
    },
    /**
     * Read the central directory.
     */
    readCentralDir: function() {
        var file;

        this.reader.setIndex(this.centralDirOffset);
        while (this.reader.readAndCheckSignature(signature.CENTRAL_FILE_HEADER)) {
            file = new zipEntry({
                zip64: this.zip64
            }, this.loadOptions);
            file.readCentralPart(this.reader);
            this.files.push(file);
        }

        if (this.centralDirRecords !== this.files.length) {
            if (this.centralDirRecords !== 0 && this.files.length === 0) {
                // We expected some records but couldn't find ANY.
                // This is really suspicious, as if something went wrong.
                throw new Error("Corrupted zip or bug: expected " + this.centralDirRecords + " records in central dir, got " + this.files.length);
            }
        }
    },
    /**
     * Read the end of central directory.
     */
    readEndOfCentral: function() {
        var offset = this.reader.lastIndexOfSignature(signature.CENTRAL_DIRECTORY_END);
        if (offset < 0) {
            // Check if the content is a truncated zip or complete garbage.
            // A "LOCAL_FILE_HEADER" is not required at the beginning (auto
            // extractible zip for example) but it can give a good hint.
            // If an ajax request was used without responseType, we will also
            // get unreadable data.
            var isGarbage = !this.isSignature(0, signature.LOCAL_FILE_HEADER);

            if (isGarbage) {
                throw new Error("Can't find end of central directory : is this a zip file ? " +
                                "If it is, see https://stuk.github.io/jszip/documentation/howto/read_zip.html");
            } else {
                throw new Error("Corrupted zip: can't find end of central directory");
            }

        }
        this.reader.setIndex(offset);
        var endOfCentralDirOffset = offset;
        this.checkSignature(signature.CENTRAL_DIRECTORY_END);
        this.readBlockEndOfCentral();


        /* extract from the zip spec :
            4)  If one of the fields in the end of central directory
                record is too small to hold required data, the field
                should be set to -1 (0xFFFF or 0xFFFFFFFF) and the
                ZIP64 format record should be created.
            5)  The end of central directory record and the
                Zip64 end of central directory locator record must
                reside on the same disk when splitting or spanning
                an archive.
         */
        if (this.diskNumber === utils.MAX_VALUE_16BITS || this.diskWithCentralDirStart === utils.MAX_VALUE_16BITS || this.centralDirRecordsOnThisDisk === utils.MAX_VALUE_16BITS || this.centralDirRecords === utils.MAX_VALUE_16BITS || this.centralDirSize === utils.MAX_VALUE_32BITS || this.centralDirOffset === utils.MAX_VALUE_32BITS) {
            this.zip64 = true;

            /*
            Warning : the zip64 extension is supported, but ONLY if the 64bits integer read from
            the zip file can fit into a 32bits integer. This cannot be solved : JavaScript represents
            all numbers as 64-bit double precision IEEE 754 floating point numbers.
            So, we have 53bits for integers and bitwise operations treat everything as 32bits.
            see https://developer.mozilla.org/en-US/docs/JavaScript/Reference/Operators/Bitwise_Operators
            and http://www.ecma-international.org/publications/files/ECMA-ST/ECMA-262.pdf section 8.5
            */

            // should look for a zip64 EOCD locator
            offset = this.reader.lastIndexOfSignature(signature.ZIP64_CENTRAL_DIRECTORY_LOCATOR);
            if (offset < 0) {
                throw new Error("Corrupted zip: can't find the ZIP64 end of central directory locator");
            }
            this.reader.setIndex(offset);
            this.checkSignature(signature.ZIP64_CENTRAL_DIRECTORY_LOCATOR);
            this.readBlockZip64EndOfCentralLocator();

            // now the zip64 EOCD record
            if (!this.isSignature(this.relativeOffsetEndOfZip64CentralDir, signature.ZIP64_CENTRAL_DIRECTORY_END)) {
                // console.warn("ZIP64 end of central directory not where expected.");
                this.relativeOffsetEndOfZip64CentralDir = this.reader.lastIndexOfSignature(signature.ZIP64_CENTRAL_DIRECTORY_END);
                if (this.relativeOffsetEndOfZip64CentralDir < 0) {
                    throw new Error("Corrupted zip: can't find the ZIP64 end of central directory");
                }
            }
            this.reader.setIndex(this.relativeOffsetEndOfZip64CentralDir);
            this.checkSignature(signature.ZIP64_CENTRAL_DIRECTORY_END);
            this.readBlockZip64EndOfCentral();
        }

        var expectedEndOfCentralDirOffset = this.centralDirOffset + this.centralDirSize;
        if (this.zip64) {
            expectedEndOfCentralDirOffset += 20; // end of central dir 64 locator
            expectedEndOfCentralDirOffset += 12 /* should not include the leading 12 bytes */ + this.zip64EndOfCentralSize;
        }

        var extraBytes = endOfCentralDirOffset - expectedEndOfCentralDirOffset;

        if (extraBytes > 0) {
            // console.warn(extraBytes, "extra bytes at beginning or within zipfile");
            if (this.isSignature(endOfCentralDirOffset, signature.CENTRAL_FILE_HEADER)) ; else {
                // the offset is wrong, update the "zero" of the reader
                // this happens if data has been prepended (crx files for example)
                this.reader.zero = extraBytes;
            }
        } else if (extraBytes < 0) {
            throw new Error("Corrupted zip: missing " + Math.abs(extraBytes) + " bytes.");
        }
    },
    prepareReader: function(data) {
        this.reader = readerFor(data);
    },
    /**
     * Read a zip file and create ZipEntries.
     * @param {String|ArrayBuffer|Uint8Array|Buffer} data the binary string representing a zip file.
     */
    load: function(data) {
        this.prepareReader(data);
        this.readEndOfCentral();
        this.readCentralDir();
        this.readLocalFiles();
    }
};
// }}} end of ZipEntries
var zipEntries = ZipEntries;

/**
 * Check the CRC32 of an entry.
 * @param {ZipEntry} zipEntry the zip entry to check.
 * @return {Promise} the result.
 */
function checkEntryCRC32(zipEntry) {
    return new external.Promise(function (resolve, reject) {
        var worker = zipEntry.decompressed.getContentWorker().pipe(new Crc32Probe_1());
        worker.on("error", function (e) {
            reject(e);
        })
        .on("end", function () {
            if (worker.streamInfo.crc32 !== zipEntry.decompressed.crc32) {
                reject(new Error("Corrupted zip : CRC32 mismatch"));
            } else {
                resolve();
            }
        })
        .resume();
    });
}

var load = function(data, options) {
    var zip = this;
    options = utils.extend(options || {}, {
        base64: false,
        checkCRC32: false,
        optimizedBinaryString: false,
        createFolders: false,
        decodeFileName: utf8.utf8decode
    });

    if (nodejsUtils.isNode && nodejsUtils.isStream(data)) {
        return external.Promise.reject(new Error("JSZip can't accept a stream when loading a zip file."));
    }

    return utils.prepareContent("the loaded zip file", data, true, options.optimizedBinaryString, options.base64)
    .then(function(data) {
        var zipEntries$$1 = new zipEntries(options);
        zipEntries$$1.load(data);
        return zipEntries$$1;
    }).then(function checkCRC32(zipEntries$$1) {
        var promises = [external.Promise.resolve(zipEntries$$1)];
        var files = zipEntries$$1.files;
        if (options.checkCRC32) {
            for (var i = 0; i < files.length; i++) {
                promises.push(checkEntryCRC32(files[i]));
            }
        }
        return external.Promise.all(promises);
    }).then(function addFiles(results) {
        var zipEntries$$1 = results.shift();
        var files = zipEntries$$1.files;
        for (var i = 0; i < files.length; i++) {
            var input = files[i];
            zip.file(input.fileNameStr, input.decompressed, {
                binary: true,
                optimizedBinaryString: true,
                date: input.date,
                dir: input.dir,
                comment : input.fileCommentStr.length ? input.fileCommentStr : null,
                unixPermissions : input.unixPermissions,
                dosPermissions : input.dosPermissions,
                createFolders: options.createFolders
            });
        }
        if (zipEntries$$1.zipComment.length) {
            zip.comment = zipEntries$$1.zipComment;
        }

        return zip;
    });
};

/**
 * Representation a of zip file in js
 * @constructor
 */
function JSZip() {
    // if this constructor is used without `new`, it adds `new` before itself:
    if(!(this instanceof JSZip)) {
        return new JSZip();
    }

    if(arguments.length) {
        throw new Error("The constructor with parameters has been removed in JSZip 3.0, please check the upgrade guide.");
    }

    // object containing the files :
    // {
    //   "folder/" : {...},
    //   "folder/data.txt" : {...}
    // }
    this.files = {};

    this.comment = null;

    // Where we are in the hierarchy
    this.root = "";
    this.clone = function() {
        var newObj = new JSZip();
        for (var i in this) {
            if (typeof this[i] !== "function") {
                newObj[i] = this[i];
            }
        }
        return newObj;
    };
}
JSZip.prototype = object;
JSZip.prototype.loadAsync = load;
JSZip.support = support;
JSZip.defaults = defaults;

// TODO find a better way to handle this version,
// a require('package.json').version doesn't work with webpack, see #327
JSZip.version = "3.1.5";

JSZip.loadAsync = function (content, options) {
    return new JSZip().loadAsync(content, options);
};

JSZip.external = external;
var lib = JSZip;

function isNothing(subject) {
  return (typeof subject === 'undefined') || (subject === null);
}


function isObject(subject) {
  return (typeof subject === 'object') && (subject !== null);
}


function toArray(sequence) {
  if (Array.isArray(sequence)) return sequence;
  else if (isNothing(sequence)) return [];

  return [ sequence ];
}


function extend(target, source) {
  var index, length, key, sourceKeys;

  if (source) {
    sourceKeys = Object.keys(source);

    for (index = 0, length = sourceKeys.length; index < length; index += 1) {
      key = sourceKeys[index];
      target[key] = source[key];
    }
  }

  return target;
}


function repeat(string, count) {
  var result = '', cycle;

  for (cycle = 0; cycle < count; cycle += 1) {
    result += string;
  }

  return result;
}


function isNegativeZero(number) {
  return (number === 0) && (Number.NEGATIVE_INFINITY === 1 / number);
}


var isNothing_1      = isNothing;
var isObject_1       = isObject;
var toArray_1        = toArray;
var repeat_1         = repeat;
var isNegativeZero_1 = isNegativeZero;
var extend_1         = extend;

var common$1 = {
	isNothing: isNothing_1,
	isObject: isObject_1,
	toArray: toArray_1,
	repeat: repeat_1,
	isNegativeZero: isNegativeZero_1,
	extend: extend_1
};

// YAML error class. http://stackoverflow.com/questions/8458984

function YAMLException(reason, mark) {
  // Super constructor
  Error.call(this);

  // Include stack trace in error object
  if (Error.captureStackTrace) {
    // Chrome and NodeJS
    Error.captureStackTrace(this, this.constructor);
  } else {
    // FF, IE 10+ and Safari 6+. Fallback for others
    this.stack = (new Error()).stack || '';
  }

  this.name = 'YAMLException';
  this.reason = reason;
  this.mark = mark;
  this.message = (this.reason || '(unknown reason)') + (this.mark ? ' ' + this.mark.toString() : '');
}


// Inherit from Error
YAMLException.prototype = Object.create(Error.prototype);
YAMLException.prototype.constructor = YAMLException;


YAMLException.prototype.toString = function toString(compact) {
  var result = this.name + ': ';

  result += this.reason || '(unknown reason)';

  if (!compact && this.mark) {
    result += ' ' + this.mark.toString();
  }

  return result;
};


var exception = YAMLException;

function Mark(name, buffer, position, line, column) {
  this.name     = name;
  this.buffer   = buffer;
  this.position = position;
  this.line     = line;
  this.column   = column;
}


Mark.prototype.getSnippet = function getSnippet(indent, maxLength) {
  var head, start, tail, end, snippet;

  if (!this.buffer) return null;

  indent = indent || 4;
  maxLength = maxLength || 75;

  head = '';
  start = this.position;

  while (start > 0 && '\x00\r\n\x85\u2028\u2029'.indexOf(this.buffer.charAt(start - 1)) === -1) {
    start -= 1;
    if (this.position - start > (maxLength / 2 - 1)) {
      head = ' ... ';
      start += 5;
      break;
    }
  }

  tail = '';
  end = this.position;

  while (end < this.buffer.length && '\x00\r\n\x85\u2028\u2029'.indexOf(this.buffer.charAt(end)) === -1) {
    end += 1;
    if (end - this.position > (maxLength / 2 - 1)) {
      tail = ' ... ';
      end -= 5;
      break;
    }
  }

  snippet = this.buffer.slice(start, end);

  return common$1.repeat(' ', indent) + head + snippet + tail + '\n' +
         common$1.repeat(' ', indent + this.position - start + head.length) + '^';
};


Mark.prototype.toString = function toString(compact) {
  var snippet, where = '';

  if (this.name) {
    where += 'in "' + this.name + '" ';
  }

  where += 'at line ' + (this.line + 1) + ', column ' + (this.column + 1);

  if (!compact) {
    snippet = this.getSnippet();

    if (snippet) {
      where += ':\n' + snippet;
    }
  }

  return where;
};


var mark = Mark;

var TYPE_CONSTRUCTOR_OPTIONS = [
  'kind',
  'resolve',
  'construct',
  'instanceOf',
  'predicate',
  'represent',
  'defaultStyle',
  'styleAliases'
];

var YAML_NODE_KINDS = [
  'scalar',
  'sequence',
  'mapping'
];

function compileStyleAliases(map) {
  var result = {};

  if (map !== null) {
    Object.keys(map).forEach(function (style) {
      map[style].forEach(function (alias) {
        result[String(alias)] = style;
      });
    });
  }

  return result;
}

function Type(tag, options) {
  options = options || {};

  Object.keys(options).forEach(function (name) {
    if (TYPE_CONSTRUCTOR_OPTIONS.indexOf(name) === -1) {
      throw new exception('Unknown option "' + name + '" is met in definition of "' + tag + '" YAML type.');
    }
  });

  // TODO: Add tag format check.
  this.tag          = tag;
  this.kind         = options['kind']         || null;
  this.resolve      = options['resolve']      || function () { return true; };
  this.construct    = options['construct']    || function (data) { return data; };
  this.instanceOf   = options['instanceOf']   || null;
  this.predicate    = options['predicate']    || null;
  this.represent    = options['represent']    || null;
  this.defaultStyle = options['defaultStyle'] || null;
  this.styleAliases = compileStyleAliases(options['styleAliases'] || null);

  if (YAML_NODE_KINDS.indexOf(this.kind) === -1) {
    throw new exception('Unknown kind "' + this.kind + '" is specified for "' + tag + '" YAML type.');
  }
}

var type = Type;

/*eslint-disable max-len*/






function compileList(schema, name, result) {
  var exclude = [];

  schema.include.forEach(function (includedSchema) {
    result = compileList(includedSchema, name, result);
  });

  schema[name].forEach(function (currentType) {
    result.forEach(function (previousType, previousIndex) {
      if (previousType.tag === currentType.tag) {
        exclude.push(previousIndex);
      }
    });

    result.push(currentType);
  });

  return result.filter(function (type$$1, index) {
    return exclude.indexOf(index) === -1;
  });
}


function compileMap(/* lists... */) {
  var result = {}, index, length;

  function collectType(type$$1) {
    result[type$$1.tag] = type$$1;
  }

  for (index = 0, length = arguments.length; index < length; index += 1) {
    arguments[index].forEach(collectType);
  }

  return result;
}


function Schema(definition) {
  this.include  = definition.include  || [];
  this.implicit = definition.implicit || [];
  this.explicit = definition.explicit || [];

  this.implicit.forEach(function (type$$1) {
    if (type$$1.loadKind && type$$1.loadKind !== 'scalar') {
      throw new exception('There is a non-scalar type in the implicit list of a schema. Implicit resolving of such types is not supported.');
    }
  });

  this.compiledImplicit = compileList(this, 'implicit', []);
  this.compiledExplicit = compileList(this, 'explicit', []);
  this.compiledTypeMap  = compileMap(this.compiledImplicit, this.compiledExplicit);
}


Schema.DEFAULT = null;


Schema.create = function createSchema() {
  var schemas, types;

  switch (arguments.length) {
    case 1:
      schemas = Schema.DEFAULT;
      types = arguments[0];
      break;

    case 2:
      schemas = arguments[0];
      types = arguments[1];
      break;

    default:
      throw new exception('Wrong number of arguments for Schema.create function');
  }

  schemas = common$1.toArray(schemas);
  types = common$1.toArray(types);

  if (!schemas.every(function (schema) { return schema instanceof Schema; })) {
    throw new exception('Specified list of super schemas (or a single Schema object) contains a non-Schema object.');
  }

  if (!types.every(function (type$$1) { return type$$1 instanceof type; })) {
    throw new exception('Specified list of YAML types (or a single Type object) contains a non-Type object.');
  }

  return new Schema({
    include: schemas,
    explicit: types
  });
};


var schema = Schema;

var str = new type('tag:yaml.org,2002:str', {
  kind: 'scalar',
  construct: function (data) { return data !== null ? data : ''; }
});

var seq = new type('tag:yaml.org,2002:seq', {
  kind: 'sequence',
  construct: function (data) { return data !== null ? data : []; }
});

var map = new type('tag:yaml.org,2002:map', {
  kind: 'mapping',
  construct: function (data) { return data !== null ? data : {}; }
});

var failsafe = new schema({
  explicit: [
    str,
    seq,
    map
  ]
});

function resolveYamlNull(data) {
  if (data === null) return true;

  var max = data.length;

  return (max === 1 && data === '~') ||
         (max === 4 && (data === 'null' || data === 'Null' || data === 'NULL'));
}

function constructYamlNull() {
  return null;
}

function isNull(object) {
  return object === null;
}

var _null = new type('tag:yaml.org,2002:null', {
  kind: 'scalar',
  resolve: resolveYamlNull,
  construct: constructYamlNull,
  predicate: isNull,
  represent: {
    canonical: function () { return '~';    },
    lowercase: function () { return 'null'; },
    uppercase: function () { return 'NULL'; },
    camelcase: function () { return 'Null'; }
  },
  defaultStyle: 'lowercase'
});

function resolveYamlBoolean(data) {
  if (data === null) return false;

  var max = data.length;

  return (max === 4 && (data === 'true' || data === 'True' || data === 'TRUE')) ||
         (max === 5 && (data === 'false' || data === 'False' || data === 'FALSE'));
}

function constructYamlBoolean(data) {
  return data === 'true' ||
         data === 'True' ||
         data === 'TRUE';
}

function isBoolean(object) {
  return Object.prototype.toString.call(object) === '[object Boolean]';
}

var bool = new type('tag:yaml.org,2002:bool', {
  kind: 'scalar',
  resolve: resolveYamlBoolean,
  construct: constructYamlBoolean,
  predicate: isBoolean,
  represent: {
    lowercase: function (object) { return object ? 'true' : 'false'; },
    uppercase: function (object) { return object ? 'TRUE' : 'FALSE'; },
    camelcase: function (object) { return object ? 'True' : 'False'; }
  },
  defaultStyle: 'lowercase'
});

function isHexCode(c) {
  return ((0x30/* 0 */ <= c) && (c <= 0x39/* 9 */)) ||
         ((0x41/* A */ <= c) && (c <= 0x46/* F */)) ||
         ((0x61/* a */ <= c) && (c <= 0x66/* f */));
}

function isOctCode(c) {
  return ((0x30/* 0 */ <= c) && (c <= 0x37/* 7 */));
}

function isDecCode(c) {
  return ((0x30/* 0 */ <= c) && (c <= 0x39/* 9 */));
}

function resolveYamlInteger(data) {
  if (data === null) return false;

  var max = data.length,
      index = 0,
      hasDigits = false,
      ch;

  if (!max) return false;

  ch = data[index];

  // sign
  if (ch === '-' || ch === '+') {
    ch = data[++index];
  }

  if (ch === '0') {
    // 0
    if (index + 1 === max) return true;
    ch = data[++index];

    // base 2, base 8, base 16

    if (ch === 'b') {
      // base 2
      index++;

      for (; index < max; index++) {
        ch = data[index];
        if (ch === '_') continue;
        if (ch !== '0' && ch !== '1') return false;
        hasDigits = true;
      }
      return hasDigits;
    }


    if (ch === 'x') {
      // base 16
      index++;

      for (; index < max; index++) {
        ch = data[index];
        if (ch === '_') continue;
        if (!isHexCode(data.charCodeAt(index))) return false;
        hasDigits = true;
      }
      return hasDigits;
    }

    // base 8
    for (; index < max; index++) {
      ch = data[index];
      if (ch === '_') continue;
      if (!isOctCode(data.charCodeAt(index))) return false;
      hasDigits = true;
    }
    return hasDigits;
  }

  // base 10 (except 0) or base 60

  for (; index < max; index++) {
    ch = data[index];
    if (ch === '_') continue;
    if (ch === ':') break;
    if (!isDecCode(data.charCodeAt(index))) {
      return false;
    }
    hasDigits = true;
  }

  if (!hasDigits) return false;

  // if !base60 - done;
  if (ch !== ':') return true;

  // base60 almost not used, no needs to optimize
  return /^(:[0-5]?[0-9])+$/.test(data.slice(index));
}

function constructYamlInteger(data) {
  var value = data, sign = 1, ch, base, digits = [];

  if (value.indexOf('_') !== -1) {
    value = value.replace(/_/g, '');
  }

  ch = value[0];

  if (ch === '-' || ch === '+') {
    if (ch === '-') sign = -1;
    value = value.slice(1);
    ch = value[0];
  }

  if (value === '0') return 0;

  if (ch === '0') {
    if (value[1] === 'b') return sign * parseInt(value.slice(2), 2);
    if (value[1] === 'x') return sign * parseInt(value, 16);
    return sign * parseInt(value, 8);
  }

  if (value.indexOf(':') !== -1) {
    value.split(':').forEach(function (v) {
      digits.unshift(parseInt(v, 10));
    });

    value = 0;
    base = 1;

    digits.forEach(function (d) {
      value += (d * base);
      base *= 60;
    });

    return sign * value;

  }

  return sign * parseInt(value, 10);
}

function isInteger(object) {
  return (Object.prototype.toString.call(object)) === '[object Number]' &&
         (object % 1 === 0 && !common$1.isNegativeZero(object));
}

var int_1 = new type('tag:yaml.org,2002:int', {
  kind: 'scalar',
  resolve: resolveYamlInteger,
  construct: constructYamlInteger,
  predicate: isInteger,
  represent: {
    binary:      function (object) { return '0b' + object.toString(2); },
    octal:       function (object) { return '0'  + object.toString(8); },
    decimal:     function (object) { return        object.toString(10); },
    hexadecimal: function (object) { return '0x' + object.toString(16).toUpperCase(); }
  },
  defaultStyle: 'decimal',
  styleAliases: {
    binary:      [ 2,  'bin' ],
    octal:       [ 8,  'oct' ],
    decimal:     [ 10, 'dec' ],
    hexadecimal: [ 16, 'hex' ]
  }
});

var YAML_FLOAT_PATTERN = new RegExp(
  '^(?:[-+]?(?:[0-9][0-9_]*)\\.[0-9_]*(?:[eE][-+][0-9]+)?' +
  '|\\.[0-9_]+(?:[eE][-+][0-9]+)?' +
  '|[-+]?[0-9][0-9_]*(?::[0-5]?[0-9])+\\.[0-9_]*' +
  '|[-+]?\\.(?:inf|Inf|INF)' +
  '|\\.(?:nan|NaN|NAN))$');

function resolveYamlFloat(data) {
  if (data === null) return false;

  if (!YAML_FLOAT_PATTERN.test(data)) return false;

  return true;
}

function constructYamlFloat(data) {
  var value, sign, base, digits;

  value  = data.replace(/_/g, '').toLowerCase();
  sign   = value[0] === '-' ? -1 : 1;
  digits = [];

  if ('+-'.indexOf(value[0]) >= 0) {
    value = value.slice(1);
  }

  if (value === '.inf') {
    return (sign === 1) ? Number.POSITIVE_INFINITY : Number.NEGATIVE_INFINITY;

  } else if (value === '.nan') {
    return NaN;

  } else if (value.indexOf(':') >= 0) {
    value.split(':').forEach(function (v) {
      digits.unshift(parseFloat(v, 10));
    });

    value = 0.0;
    base = 1;

    digits.forEach(function (d) {
      value += d * base;
      base *= 60;
    });

    return sign * value;

  }
  return sign * parseFloat(value, 10);
}


var SCIENTIFIC_WITHOUT_DOT = /^[-+]?[0-9]+e/;

function representYamlFloat(object, style) {
  var res;

  if (isNaN(object)) {
    switch (style) {
      case 'lowercase': return '.nan';
      case 'uppercase': return '.NAN';
      case 'camelcase': return '.NaN';
    }
  } else if (Number.POSITIVE_INFINITY === object) {
    switch (style) {
      case 'lowercase': return '.inf';
      case 'uppercase': return '.INF';
      case 'camelcase': return '.Inf';
    }
  } else if (Number.NEGATIVE_INFINITY === object) {
    switch (style) {
      case 'lowercase': return '-.inf';
      case 'uppercase': return '-.INF';
      case 'camelcase': return '-.Inf';
    }
  } else if (common$1.isNegativeZero(object)) {
    return '-0.0';
  }

  res = object.toString(10);

  // JS stringifier can build scientific format without dots: 5e-100,
  // while YAML requres dot: 5.e-100. Fix it with simple hack

  return SCIENTIFIC_WITHOUT_DOT.test(res) ? res.replace('e', '.e') : res;
}

function isFloat(object) {
  return (Object.prototype.toString.call(object) === '[object Number]') &&
         (object % 1 !== 0 || common$1.isNegativeZero(object));
}

var float_1 = new type('tag:yaml.org,2002:float', {
  kind: 'scalar',
  resolve: resolveYamlFloat,
  construct: constructYamlFloat,
  predicate: isFloat,
  represent: representYamlFloat,
  defaultStyle: 'lowercase'
});

var json = new schema({
  include: [
    failsafe
  ],
  implicit: [
    _null,
    bool,
    int_1,
    float_1
  ]
});

var core = new schema({
  include: [
    json
  ]
});

var YAML_TIMESTAMP_REGEXP = new RegExp(
  '^([0-9][0-9][0-9][0-9])'          + // [1] year
  '-([0-9][0-9]?)'                   + // [2] month
  '-([0-9][0-9]?)'                   + // [3] day
  '(?:(?:[Tt]|[ \\t]+)'              + // ...
  '([0-9][0-9]?)'                    + // [4] hour
  ':([0-9][0-9])'                    + // [5] minute
  ':([0-9][0-9])'                    + // [6] second
  '(?:\\.([0-9]*))?'                 + // [7] fraction
  '(?:[ \\t]*(Z|([-+])([0-9][0-9]?)' + // [8] tz [9] tz_sign [10] tz_hour
  '(?::([0-9][0-9]))?))?)?$');         // [11] tz_minute

function resolveYamlTimestamp(data) {
  if (data === null) return false;
  if (YAML_TIMESTAMP_REGEXP.exec(data) === null) return false;
  return true;
}

function constructYamlTimestamp(data) {
  var match, year, month, day, hour, minute, second, fraction = 0,
      delta = null, tz_hour, tz_minute, date;

  match = YAML_TIMESTAMP_REGEXP.exec(data);

  if (match === null) throw new Error('Date resolve error');

  // match: [1] year [2] month [3] day

  year = +(match[1]);
  month = +(match[2]) - 1; // JS month starts with 0
  day = +(match[3]);

  if (!match[4]) { // no hour
    return new Date(Date.UTC(year, month, day));
  }

  // match: [4] hour [5] minute [6] second [7] fraction

  hour = +(match[4]);
  minute = +(match[5]);
  second = +(match[6]);

  if (match[7]) {
    fraction = match[7].slice(0, 3);
    while (fraction.length < 3) { // milli-seconds
      fraction += '0';
    }
    fraction = +fraction;
  }

  // match: [8] tz [9] tz_sign [10] tz_hour [11] tz_minute

  if (match[9]) {
    tz_hour = +(match[10]);
    tz_minute = +(match[11] || 0);
    delta = (tz_hour * 60 + tz_minute) * 60000; // delta in mili-seconds
    if (match[9] === '-') delta = -delta;
  }

  date = new Date(Date.UTC(year, month, day, hour, minute, second, fraction));

  if (delta) date.setTime(date.getTime() - delta);

  return date;
}

function representYamlTimestamp(object /*, style*/) {
  return object.toISOString();
}

var timestamp = new type('tag:yaml.org,2002:timestamp', {
  kind: 'scalar',
  resolve: resolveYamlTimestamp,
  construct: constructYamlTimestamp,
  instanceOf: Date,
  represent: representYamlTimestamp
});

function resolveYamlMerge(data) {
  return data === '<<' || data === null;
}

var merge = new type('tag:yaml.org,2002:merge', {
  kind: 'scalar',
  resolve: resolveYamlMerge
});

var require$$0 = {};

/*eslint-disable no-bitwise*/

// A trick for browserified version.
// Since we make browserifier to ignore `buffer` module, NodeBuffer will be undefined
var NodeBuffer = require$$0.Buffer;



// [ 64, 65, 66 ] -> [ padding, CR, LF ]
var BASE64_MAP = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=\n\r';


function resolveYamlBinary(data) {
  if (data === null) return false;

  var code, idx, bitlen = 0, max = data.length, map = BASE64_MAP;

  // Convert one by one.
  for (idx = 0; idx < max; idx++) {
    code = map.indexOf(data.charAt(idx));

    // Skip CR/LF
    if (code > 64) continue;

    // Fail on illegal characters
    if (code < 0) return false;

    bitlen += 6;
  }

  // If there are any bits left, source was corrupted
  return (bitlen % 8) === 0;
}

function constructYamlBinary(data) {
  var idx, tailbits,
      input = data.replace(/[\r\n=]/g, ''), // remove CR/LF & padding to simplify scan
      max = input.length,
      map = BASE64_MAP,
      bits = 0,
      result = [];

  // Collect by 6*4 bits (3 bytes)

  for (idx = 0; idx < max; idx++) {
    if ((idx % 4 === 0) && idx) {
      result.push((bits >> 16) & 0xFF);
      result.push((bits >> 8) & 0xFF);
      result.push(bits & 0xFF);
    }

    bits = (bits << 6) | map.indexOf(input.charAt(idx));
  }

  // Dump tail

  tailbits = (max % 4) * 6;

  if (tailbits === 0) {
    result.push((bits >> 16) & 0xFF);
    result.push((bits >> 8) & 0xFF);
    result.push(bits & 0xFF);
  } else if (tailbits === 18) {
    result.push((bits >> 10) & 0xFF);
    result.push((bits >> 2) & 0xFF);
  } else if (tailbits === 12) {
    result.push((bits >> 4) & 0xFF);
  }

  return result;
}

function representYamlBinary(object /*, style*/) {
  var result = '', bits = 0, idx, tail,
      max = object.length,
      map = BASE64_MAP;

  // Convert every three bytes to 4 ASCII characters.

  for (idx = 0; idx < max; idx++) {
    if ((idx % 3 === 0) && idx) {
      result += map[(bits >> 18) & 0x3F];
      result += map[(bits >> 12) & 0x3F];
      result += map[(bits >> 6) & 0x3F];
      result += map[bits & 0x3F];
    }

    bits = (bits << 8) + object[idx];
  }

  // Dump tail

  tail = max % 3;

  if (tail === 0) {
    result += map[(bits >> 18) & 0x3F];
    result += map[(bits >> 12) & 0x3F];
    result += map[(bits >> 6) & 0x3F];
    result += map[bits & 0x3F];
  } else if (tail === 2) {
    result += map[(bits >> 10) & 0x3F];
    result += map[(bits >> 4) & 0x3F];
    result += map[(bits << 2) & 0x3F];
    result += map[64];
  } else if (tail === 1) {
    result += map[(bits >> 2) & 0x3F];
    result += map[(bits << 4) & 0x3F];
    result += map[64];
    result += map[64];
  }

  return result;
}

function isBinary(object) {
  return NodeBuffer && NodeBuffer.isBuffer(object);
}

var binary$1 = new type('tag:yaml.org,2002:binary', {
  kind: 'scalar',
  resolve: resolveYamlBinary,
  construct: constructYamlBinary,
  predicate: isBinary,
  represent: representYamlBinary
});

var _hasOwnProperty = Object.prototype.hasOwnProperty;
var _toString       = Object.prototype.toString;

function resolveYamlOmap(data) {
  if (data === null) return true;

  var objectKeys = [], index, length, pair, pairKey, pairHasKey,
      object = data;

  for (index = 0, length = object.length; index < length; index += 1) {
    pair = object[index];
    pairHasKey = false;

    if (_toString.call(pair) !== '[object Object]') return false;

    for (pairKey in pair) {
      if (_hasOwnProperty.call(pair, pairKey)) {
        if (!pairHasKey) pairHasKey = true;
        else return false;
      }
    }

    if (!pairHasKey) return false;

    if (objectKeys.indexOf(pairKey) === -1) objectKeys.push(pairKey);
    else return false;
  }

  return true;
}

function constructYamlOmap(data) {
  return data !== null ? data : [];
}

var omap = new type('tag:yaml.org,2002:omap', {
  kind: 'sequence',
  resolve: resolveYamlOmap,
  construct: constructYamlOmap
});

var _toString$1 = Object.prototype.toString;

function resolveYamlPairs(data) {
  if (data === null) return true;

  var index, length, pair, keys, result,
      object = data;

  result = new Array(object.length);

  for (index = 0, length = object.length; index < length; index += 1) {
    pair = object[index];

    if (_toString$1.call(pair) !== '[object Object]') return false;

    keys = Object.keys(pair);

    if (keys.length !== 1) return false;

    result[index] = [ keys[0], pair[keys[0]] ];
  }

  return true;
}

function constructYamlPairs(data) {
  if (data === null) return [];

  var index, length, pair, keys, result,
      object = data;

  result = new Array(object.length);

  for (index = 0, length = object.length; index < length; index += 1) {
    pair = object[index];

    keys = Object.keys(pair);

    result[index] = [ keys[0], pair[keys[0]] ];
  }

  return result;
}

var pairs = new type('tag:yaml.org,2002:pairs', {
  kind: 'sequence',
  resolve: resolveYamlPairs,
  construct: constructYamlPairs
});

var _hasOwnProperty$1 = Object.prototype.hasOwnProperty;

function resolveYamlSet(data) {
  if (data === null) return true;

  var key, object = data;

  for (key in object) {
    if (_hasOwnProperty$1.call(object, key)) {
      if (object[key] !== null) return false;
    }
  }

  return true;
}

function constructYamlSet(data) {
  return data !== null ? data : {};
}

var set = new type('tag:yaml.org,2002:set', {
  kind: 'mapping',
  resolve: resolveYamlSet,
  construct: constructYamlSet
});

var default_safe = new schema({
  include: [
    core
  ],
  implicit: [
    timestamp,
    merge
  ],
  explicit: [
    binary$1,
    omap,
    pairs,
    set
  ]
});

function resolveJavascriptUndefined() {
  return true;
}

function constructJavascriptUndefined() {
  /*eslint-disable no-undefined*/
  return undefined;
}

function representJavascriptUndefined() {
  return '';
}

function isUndefined(object) {
  return typeof object === 'undefined';
}

var _undefined = new type('tag:yaml.org,2002:js/undefined', {
  kind: 'scalar',
  resolve: resolveJavascriptUndefined,
  construct: constructJavascriptUndefined,
  predicate: isUndefined,
  represent: representJavascriptUndefined
});

function resolveJavascriptRegExp(data) {
  if (data === null) return false;
  if (data.length === 0) return false;

  var regexp = data,
      tail   = /\/([gim]*)$/.exec(data),
      modifiers = '';

  // if regexp starts with '/' it can have modifiers and must be properly closed
  // `/foo/gim` - modifiers tail can be maximum 3 chars
  if (regexp[0] === '/') {
    if (tail) modifiers = tail[1];

    if (modifiers.length > 3) return false;
    // if expression starts with /, is should be properly terminated
    if (regexp[regexp.length - modifiers.length - 1] !== '/') return false;
  }

  return true;
}

function constructJavascriptRegExp(data) {
  var regexp = data,
      tail   = /\/([gim]*)$/.exec(data),
      modifiers = '';

  // `/foo/gim` - tail can be maximum 4 chars
  if (regexp[0] === '/') {
    if (tail) modifiers = tail[1];
    regexp = regexp.slice(1, regexp.length - modifiers.length - 1);
  }

  return new RegExp(regexp, modifiers);
}

function representJavascriptRegExp(object /*, style*/) {
  var result = '/' + object.source + '/';

  if (object.global) result += 'g';
  if (object.multiline) result += 'm';
  if (object.ignoreCase) result += 'i';

  return result;
}

function isRegExp$1(object) {
  return Object.prototype.toString.call(object) === '[object RegExp]';
}

var regexp = new type('tag:yaml.org,2002:js/regexp', {
  kind: 'scalar',
  resolve: resolveJavascriptRegExp,
  construct: constructJavascriptRegExp,
  predicate: isRegExp$1,
  represent: representJavascriptRegExp
});

var esprima;

// Browserified version does not have esprima
//
// 1. For node.js just require module as deps
// 2. For browser try to require mudule via external AMD system.
//    If not found - try to fallback to window.esprima. If not
//    found too - then fail to parse.
//
try {
  // workaround to exclude package from browserify list.
  var _require = __chunk_1.commonjsRequire;
  esprima = _require('esprima');
} catch (_) {
  /*global window */
  if (typeof window !== 'undefined') esprima = window.esprima;
}



function resolveJavascriptFunction(data) {
  if (data === null) return false;

  try {
    var source = '(' + data + ')',
        ast    = esprima.parse(source, { range: true });

    if (ast.type                    !== 'Program'             ||
        ast.body.length             !== 1                     ||
        ast.body[0].type            !== 'ExpressionStatement' ||
        ast.body[0].expression.type !== 'FunctionExpression') {
      return false;
    }

    return true;
  } catch (err) {
    return false;
  }
}

function constructJavascriptFunction(data) {
  /*jslint evil:true*/

  var source = '(' + data + ')',
      ast    = esprima.parse(source, { range: true }),
      params = [],
      body;

  if (ast.type                    !== 'Program'             ||
      ast.body.length             !== 1                     ||
      ast.body[0].type            !== 'ExpressionStatement' ||
      ast.body[0].expression.type !== 'FunctionExpression') {
    throw new Error('Failed to resolve function');
  }

  ast.body[0].expression.params.forEach(function (param) {
    params.push(param.name);
  });

  body = ast.body[0].expression.body.range;

  // Esprima's ranges include the first '{' and the last '}' characters on
  // function expressions. So cut them out.
  /*eslint-disable no-new-func*/
  return new Function(params, source.slice(body[0] + 1, body[1] - 1));
}

function representJavascriptFunction(object /*, style*/) {
  return object.toString();
}

function isFunction(object) {
  return Object.prototype.toString.call(object) === '[object Function]';
}

var _function = new type('tag:yaml.org,2002:js/function', {
  kind: 'scalar',
  resolve: resolveJavascriptFunction,
  construct: constructJavascriptFunction,
  predicate: isFunction,
  represent: representJavascriptFunction
});

var default_full = schema.DEFAULT = new schema({
  include: [
    default_safe
  ],
  explicit: [
    _undefined,
    regexp,
    _function
  ]
});

/*eslint-disable max-len,no-use-before-define*/








var _hasOwnProperty$2 = Object.prototype.hasOwnProperty;


var CONTEXT_FLOW_IN   = 1;
var CONTEXT_FLOW_OUT  = 2;
var CONTEXT_BLOCK_IN  = 3;
var CONTEXT_BLOCK_OUT = 4;


var CHOMPING_CLIP  = 1;
var CHOMPING_STRIP = 2;
var CHOMPING_KEEP  = 3;


var PATTERN_NON_PRINTABLE         = /[\x00-\x08\x0B\x0C\x0E-\x1F\x7F-\x84\x86-\x9F\uFFFE\uFFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF]/;
var PATTERN_NON_ASCII_LINE_BREAKS = /[\x85\u2028\u2029]/;
var PATTERN_FLOW_INDICATORS       = /[,\[\]\{\}]/;
var PATTERN_TAG_HANDLE            = /^(?:!|!!|![a-z\-]+!)$/i;
var PATTERN_TAG_URI               = /^(?:!|[^,\[\]\{\}])(?:%[0-9a-f]{2}|[0-9a-z\-#;\/\?:@&=\+\$,_\.!~\*'\(\)\[\]])*$/i;


function is_EOL(c) {
  return (c === 0x0A/* LF */) || (c === 0x0D/* CR */);
}

function is_WHITE_SPACE(c) {
  return (c === 0x09/* Tab */) || (c === 0x20/* Space */);
}

function is_WS_OR_EOL(c) {
  return (c === 0x09/* Tab */) ||
         (c === 0x20/* Space */) ||
         (c === 0x0A/* LF */) ||
         (c === 0x0D/* CR */);
}

function is_FLOW_INDICATOR(c) {
  return c === 0x2C/* , */ ||
         c === 0x5B/* [ */ ||
         c === 0x5D/* ] */ ||
         c === 0x7B/* { */ ||
         c === 0x7D/* } */;
}

function fromHexCode(c) {
  var lc;

  if ((0x30/* 0 */ <= c) && (c <= 0x39/* 9 */)) {
    return c - 0x30;
  }

  /*eslint-disable no-bitwise*/
  lc = c | 0x20;

  if ((0x61/* a */ <= lc) && (lc <= 0x66/* f */)) {
    return lc - 0x61 + 10;
  }

  return -1;
}

function escapedHexLen(c) {
  if (c === 0x78/* x */) { return 2; }
  if (c === 0x75/* u */) { return 4; }
  if (c === 0x55/* U */) { return 8; }
  return 0;
}

function fromDecimalCode(c) {
  if ((0x30/* 0 */ <= c) && (c <= 0x39/* 9 */)) {
    return c - 0x30;
  }

  return -1;
}

function simpleEscapeSequence(c) {
  return (c === 0x30/* 0 */) ? '\x00' :
        (c === 0x61/* a */) ? '\x07' :
        (c === 0x62/* b */) ? '\x08' :
        (c === 0x74/* t */) ? '\x09' :
        (c === 0x09/* Tab */) ? '\x09' :
        (c === 0x6E/* n */) ? '\x0A' :
        (c === 0x76/* v */) ? '\x0B' :
        (c === 0x66/* f */) ? '\x0C' :
        (c === 0x72/* r */) ? '\x0D' :
        (c === 0x65/* e */) ? '\x1B' :
        (c === 0x20/* Space */) ? ' ' :
        (c === 0x22/* " */) ? '\x22' :
        (c === 0x2F/* / */) ? '/' :
        (c === 0x5C/* \ */) ? '\x5C' :
        (c === 0x4E/* N */) ? '\x85' :
        (c === 0x5F/* _ */) ? '\xA0' :
        (c === 0x4C/* L */) ? '\u2028' :
        (c === 0x50/* P */) ? '\u2029' : '';
}

function charFromCodepoint(c) {
  if (c <= 0xFFFF) {
    return String.fromCharCode(c);
  }
  // Encode UTF-16 surrogate pair
  // https://en.wikipedia.org/wiki/UTF-16#Code_points_U.2B010000_to_U.2B10FFFF
  return String.fromCharCode(((c - 0x010000) >> 10) + 0xD800,
                             ((c - 0x010000) & 0x03FF) + 0xDC00);
}

var simpleEscapeCheck = new Array(256); // integer, for fast access
var simpleEscapeMap = new Array(256);
for (var i$1 = 0; i$1 < 256; i$1++) {
  simpleEscapeCheck[i$1] = simpleEscapeSequence(i$1) ? 1 : 0;
  simpleEscapeMap[i$1] = simpleEscapeSequence(i$1);
}


function State(input, options) {
  this.input = input;

  this.filename  = options['filename']  || null;
  this.schema    = options['schema']    || default_full;
  this.onWarning = options['onWarning'] || null;
  this.legacy    = options['legacy']    || false;
  this.json      = options['json']      || false;
  this.listener  = options['listener']  || null;

  this.implicitTypes = this.schema.compiledImplicit;
  this.typeMap       = this.schema.compiledTypeMap;

  this.length     = input.length;
  this.position   = 0;
  this.line       = 0;
  this.lineStart  = 0;
  this.lineIndent = 0;

  this.documents = [];

  /*
  this.version;
  this.checkLineBreaks;
  this.tagMap;
  this.anchorMap;
  this.tag;
  this.anchor;
  this.kind;
  this.result;*/

}


function generateError(state, message) {
  return new exception(
    message,
    new mark(state.filename, state.input, state.position, state.line, (state.position - state.lineStart)));
}

function throwError(state, message) {
  throw generateError(state, message);
}

function throwWarning(state, message) {
  if (state.onWarning) {
    state.onWarning.call(null, generateError(state, message));
  }
}


var directiveHandlers = {

  YAML: function handleYamlDirective(state, name, args) {

    var match, major, minor;

    if (state.version !== null) {
      throwError(state, 'duplication of %YAML directive');
    }

    if (args.length !== 1) {
      throwError(state, 'YAML directive accepts exactly one argument');
    }

    match = /^([0-9]+)\.([0-9]+)$/.exec(args[0]);

    if (match === null) {
      throwError(state, 'ill-formed argument of the YAML directive');
    }

    major = parseInt(match[1], 10);
    minor = parseInt(match[2], 10);

    if (major !== 1) {
      throwError(state, 'unacceptable YAML version of the document');
    }

    state.version = args[0];
    state.checkLineBreaks = (minor < 2);

    if (minor !== 1 && minor !== 2) {
      throwWarning(state, 'unsupported YAML version of the document');
    }
  },

  TAG: function handleTagDirective(state, name, args) {

    var handle, prefix;

    if (args.length !== 2) {
      throwError(state, 'TAG directive accepts exactly two arguments');
    }

    handle = args[0];
    prefix = args[1];

    if (!PATTERN_TAG_HANDLE.test(handle)) {
      throwError(state, 'ill-formed tag handle (first argument) of the TAG directive');
    }

    if (_hasOwnProperty$2.call(state.tagMap, handle)) {
      throwError(state, 'there is a previously declared suffix for "' + handle + '" tag handle');
    }

    if (!PATTERN_TAG_URI.test(prefix)) {
      throwError(state, 'ill-formed tag prefix (second argument) of the TAG directive');
    }

    state.tagMap[handle] = prefix;
  }
};


function captureSegment(state, start, end, checkJson) {
  var _position, _length, _character, _result;

  if (start < end) {
    _result = state.input.slice(start, end);

    if (checkJson) {
      for (_position = 0, _length = _result.length;
           _position < _length;
           _position += 1) {
        _character = _result.charCodeAt(_position);
        if (!(_character === 0x09 ||
              (0x20 <= _character && _character <= 0x10FFFF))) {
          throwError(state, 'expected valid JSON character');
        }
      }
    } else if (PATTERN_NON_PRINTABLE.test(_result)) {
      throwError(state, 'the stream contains non-printable characters');
    }

    state.result += _result;
  }
}

function mergeMappings(state, destination, source, overridableKeys) {
  var sourceKeys, key, index, quantity;

  if (!common$1.isObject(source)) {
    throwError(state, 'cannot merge mappings; the provided source object is unacceptable');
  }

  sourceKeys = Object.keys(source);

  for (index = 0, quantity = sourceKeys.length; index < quantity; index += 1) {
    key = sourceKeys[index];

    if (!_hasOwnProperty$2.call(destination, key)) {
      destination[key] = source[key];
      overridableKeys[key] = true;
    }
  }
}

function storeMappingPair(state, _result, overridableKeys, keyTag, keyNode, valueNode) {
  var index, quantity;

  keyNode = String(keyNode);

  if (_result === null) {
    _result = {};
  }

  if (keyTag === 'tag:yaml.org,2002:merge') {
    if (Array.isArray(valueNode)) {
      for (index = 0, quantity = valueNode.length; index < quantity; index += 1) {
        mergeMappings(state, _result, valueNode[index], overridableKeys);
      }
    } else {
      mergeMappings(state, _result, valueNode, overridableKeys);
    }
  } else {
    if (!state.json &&
        !_hasOwnProperty$2.call(overridableKeys, keyNode) &&
        _hasOwnProperty$2.call(_result, keyNode)) {
      throwError(state, 'duplicated mapping key');
    }
    _result[keyNode] = valueNode;
    delete overridableKeys[keyNode];
  }

  return _result;
}

function readLineBreak(state) {
  var ch;

  ch = state.input.charCodeAt(state.position);

  if (ch === 0x0A/* LF */) {
    state.position++;
  } else if (ch === 0x0D/* CR */) {
    state.position++;
    if (state.input.charCodeAt(state.position) === 0x0A/* LF */) {
      state.position++;
    }
  } else {
    throwError(state, 'a line break is expected');
  }

  state.line += 1;
  state.lineStart = state.position;
}

function skipSeparationSpace(state, allowComments, checkIndent) {
  var lineBreaks = 0,
      ch = state.input.charCodeAt(state.position);

  while (ch !== 0) {
    while (is_WHITE_SPACE(ch)) {
      ch = state.input.charCodeAt(++state.position);
    }

    if (allowComments && ch === 0x23/* # */) {
      do {
        ch = state.input.charCodeAt(++state.position);
      } while (ch !== 0x0A/* LF */ && ch !== 0x0D/* CR */ && ch !== 0);
    }

    if (is_EOL(ch)) {
      readLineBreak(state);

      ch = state.input.charCodeAt(state.position);
      lineBreaks++;
      state.lineIndent = 0;

      while (ch === 0x20/* Space */) {
        state.lineIndent++;
        ch = state.input.charCodeAt(++state.position);
      }
    } else {
      break;
    }
  }

  if (checkIndent !== -1 && lineBreaks !== 0 && state.lineIndent < checkIndent) {
    throwWarning(state, 'deficient indentation');
  }

  return lineBreaks;
}

function testDocumentSeparator(state) {
  var _position = state.position,
      ch;

  ch = state.input.charCodeAt(_position);

  // Condition state.position === state.lineStart is tested
  // in parent on each call, for efficiency. No needs to test here again.
  if ((ch === 0x2D/* - */ || ch === 0x2E/* . */) &&
      ch === state.input.charCodeAt(_position + 1) &&
      ch === state.input.charCodeAt(_position + 2)) {

    _position += 3;

    ch = state.input.charCodeAt(_position);

    if (ch === 0 || is_WS_OR_EOL(ch)) {
      return true;
    }
  }

  return false;
}

function writeFoldedLines(state, count) {
  if (count === 1) {
    state.result += ' ';
  } else if (count > 1) {
    state.result += common$1.repeat('\n', count - 1);
  }
}


function readPlainScalar(state, nodeIndent, withinFlowCollection) {
  var preceding,
      following,
      captureStart,
      captureEnd,
      hasPendingContent,
      _line,
      _lineStart,
      _lineIndent,
      _kind = state.kind,
      _result = state.result,
      ch;

  ch = state.input.charCodeAt(state.position);

  if (is_WS_OR_EOL(ch)      ||
      is_FLOW_INDICATOR(ch) ||
      ch === 0x23/* # */    ||
      ch === 0x26/* & */    ||
      ch === 0x2A/* * */    ||
      ch === 0x21/* ! */    ||
      ch === 0x7C/* | */    ||
      ch === 0x3E/* > */    ||
      ch === 0x27/* ' */    ||
      ch === 0x22/* " */    ||
      ch === 0x25/* % */    ||
      ch === 0x40/* @ */    ||
      ch === 0x60/* ` */) {
    return false;
  }

  if (ch === 0x3F/* ? */ || ch === 0x2D/* - */) {
    following = state.input.charCodeAt(state.position + 1);

    if (is_WS_OR_EOL(following) ||
        withinFlowCollection && is_FLOW_INDICATOR(following)) {
      return false;
    }
  }

  state.kind = 'scalar';
  state.result = '';
  captureStart = captureEnd = state.position;
  hasPendingContent = false;

  while (ch !== 0) {
    if (ch === 0x3A/* : */) {
      following = state.input.charCodeAt(state.position + 1);

      if (is_WS_OR_EOL(following) ||
          withinFlowCollection && is_FLOW_INDICATOR(following)) {
        break;
      }

    } else if (ch === 0x23/* # */) {
      preceding = state.input.charCodeAt(state.position - 1);

      if (is_WS_OR_EOL(preceding)) {
        break;
      }

    } else if ((state.position === state.lineStart && testDocumentSeparator(state)) ||
               withinFlowCollection && is_FLOW_INDICATOR(ch)) {
      break;

    } else if (is_EOL(ch)) {
      _line = state.line;
      _lineStart = state.lineStart;
      _lineIndent = state.lineIndent;
      skipSeparationSpace(state, false, -1);

      if (state.lineIndent >= nodeIndent) {
        hasPendingContent = true;
        ch = state.input.charCodeAt(state.position);
        continue;
      } else {
        state.position = captureEnd;
        state.line = _line;
        state.lineStart = _lineStart;
        state.lineIndent = _lineIndent;
        break;
      }
    }

    if (hasPendingContent) {
      captureSegment(state, captureStart, captureEnd, false);
      writeFoldedLines(state, state.line - _line);
      captureStart = captureEnd = state.position;
      hasPendingContent = false;
    }

    if (!is_WHITE_SPACE(ch)) {
      captureEnd = state.position + 1;
    }

    ch = state.input.charCodeAt(++state.position);
  }

  captureSegment(state, captureStart, captureEnd, false);

  if (state.result) {
    return true;
  }

  state.kind = _kind;
  state.result = _result;
  return false;
}

function readSingleQuotedScalar(state, nodeIndent) {
  var ch,
      captureStart, captureEnd;

  ch = state.input.charCodeAt(state.position);

  if (ch !== 0x27/* ' */) {
    return false;
  }

  state.kind = 'scalar';
  state.result = '';
  state.position++;
  captureStart = captureEnd = state.position;

  while ((ch = state.input.charCodeAt(state.position)) !== 0) {
    if (ch === 0x27/* ' */) {
      captureSegment(state, captureStart, state.position, true);
      ch = state.input.charCodeAt(++state.position);

      if (ch === 0x27/* ' */) {
        captureStart = captureEnd = state.position;
        state.position++;
      } else {
        return true;
      }

    } else if (is_EOL(ch)) {
      captureSegment(state, captureStart, captureEnd, true);
      writeFoldedLines(state, skipSeparationSpace(state, false, nodeIndent));
      captureStart = captureEnd = state.position;

    } else if (state.position === state.lineStart && testDocumentSeparator(state)) {
      throwError(state, 'unexpected end of the document within a single quoted scalar');

    } else {
      state.position++;
      captureEnd = state.position;
    }
  }

  throwError(state, 'unexpected end of the stream within a single quoted scalar');
}

function readDoubleQuotedScalar(state, nodeIndent) {
  var captureStart,
      captureEnd,
      hexLength,
      hexResult,
      tmp,
      ch;

  ch = state.input.charCodeAt(state.position);

  if (ch !== 0x22/* " */) {
    return false;
  }

  state.kind = 'scalar';
  state.result = '';
  state.position++;
  captureStart = captureEnd = state.position;

  while ((ch = state.input.charCodeAt(state.position)) !== 0) {
    if (ch === 0x22/* " */) {
      captureSegment(state, captureStart, state.position, true);
      state.position++;
      return true;

    } else if (ch === 0x5C/* \ */) {
      captureSegment(state, captureStart, state.position, true);
      ch = state.input.charCodeAt(++state.position);

      if (is_EOL(ch)) {
        skipSeparationSpace(state, false, nodeIndent);

        // TODO: rework to inline fn with no type cast?
      } else if (ch < 256 && simpleEscapeCheck[ch]) {
        state.result += simpleEscapeMap[ch];
        state.position++;

      } else if ((tmp = escapedHexLen(ch)) > 0) {
        hexLength = tmp;
        hexResult = 0;

        for (; hexLength > 0; hexLength--) {
          ch = state.input.charCodeAt(++state.position);

          if ((tmp = fromHexCode(ch)) >= 0) {
            hexResult = (hexResult << 4) + tmp;

          } else {
            throwError(state, 'expected hexadecimal character');
          }
        }

        state.result += charFromCodepoint(hexResult);

        state.position++;

      } else {
        throwError(state, 'unknown escape sequence');
      }

      captureStart = captureEnd = state.position;

    } else if (is_EOL(ch)) {
      captureSegment(state, captureStart, captureEnd, true);
      writeFoldedLines(state, skipSeparationSpace(state, false, nodeIndent));
      captureStart = captureEnd = state.position;

    } else if (state.position === state.lineStart && testDocumentSeparator(state)) {
      throwError(state, 'unexpected end of the document within a double quoted scalar');

    } else {
      state.position++;
      captureEnd = state.position;
    }
  }

  throwError(state, 'unexpected end of the stream within a double quoted scalar');
}

function readFlowCollection(state, nodeIndent) {
  var readNext = true,
      _line,
      _tag     = state.tag,
      _result,
      _anchor  = state.anchor,
      following,
      terminator,
      isPair,
      isExplicitPair,
      isMapping,
      overridableKeys = {},
      keyNode,
      keyTag,
      valueNode,
      ch;

  ch = state.input.charCodeAt(state.position);

  if (ch === 0x5B/* [ */) {
    terminator = 0x5D;/* ] */
    isMapping = false;
    _result = [];
  } else if (ch === 0x7B/* { */) {
    terminator = 0x7D;/* } */
    isMapping = true;
    _result = {};
  } else {
    return false;
  }

  if (state.anchor !== null) {
    state.anchorMap[state.anchor] = _result;
  }

  ch = state.input.charCodeAt(++state.position);

  while (ch !== 0) {
    skipSeparationSpace(state, true, nodeIndent);

    ch = state.input.charCodeAt(state.position);

    if (ch === terminator) {
      state.position++;
      state.tag = _tag;
      state.anchor = _anchor;
      state.kind = isMapping ? 'mapping' : 'sequence';
      state.result = _result;
      return true;
    } else if (!readNext) {
      throwError(state, 'missed comma between flow collection entries');
    }

    keyTag = keyNode = valueNode = null;
    isPair = isExplicitPair = false;

    if (ch === 0x3F/* ? */) {
      following = state.input.charCodeAt(state.position + 1);

      if (is_WS_OR_EOL(following)) {
        isPair = isExplicitPair = true;
        state.position++;
        skipSeparationSpace(state, true, nodeIndent);
      }
    }

    _line = state.line;
    composeNode(state, nodeIndent, CONTEXT_FLOW_IN, false, true);
    keyTag = state.tag;
    keyNode = state.result;
    skipSeparationSpace(state, true, nodeIndent);

    ch = state.input.charCodeAt(state.position);

    if ((isExplicitPair || state.line === _line) && ch === 0x3A/* : */) {
      isPair = true;
      ch = state.input.charCodeAt(++state.position);
      skipSeparationSpace(state, true, nodeIndent);
      composeNode(state, nodeIndent, CONTEXT_FLOW_IN, false, true);
      valueNode = state.result;
    }

    if (isMapping) {
      storeMappingPair(state, _result, overridableKeys, keyTag, keyNode, valueNode);
    } else if (isPair) {
      _result.push(storeMappingPair(state, null, overridableKeys, keyTag, keyNode, valueNode));
    } else {
      _result.push(keyNode);
    }

    skipSeparationSpace(state, true, nodeIndent);

    ch = state.input.charCodeAt(state.position);

    if (ch === 0x2C/* , */) {
      readNext = true;
      ch = state.input.charCodeAt(++state.position);
    } else {
      readNext = false;
    }
  }

  throwError(state, 'unexpected end of the stream within a flow collection');
}

function readBlockScalar(state, nodeIndent) {
  var captureStart,
      folding,
      chomping       = CHOMPING_CLIP,
      detectedIndent = false,
      textIndent     = nodeIndent,
      emptyLines     = 0,
      atMoreIndented = false,
      tmp,
      ch;

  ch = state.input.charCodeAt(state.position);

  if (ch === 0x7C/* | */) {
    folding = false;
  } else if (ch === 0x3E/* > */) {
    folding = true;
  } else {
    return false;
  }

  state.kind = 'scalar';
  state.result = '';

  while (ch !== 0) {
    ch = state.input.charCodeAt(++state.position);

    if (ch === 0x2B/* + */ || ch === 0x2D/* - */) {
      if (CHOMPING_CLIP === chomping) {
        chomping = (ch === 0x2B/* + */) ? CHOMPING_KEEP : CHOMPING_STRIP;
      } else {
        throwError(state, 'repeat of a chomping mode identifier');
      }

    } else if ((tmp = fromDecimalCode(ch)) >= 0) {
      if (tmp === 0) {
        throwError(state, 'bad explicit indentation width of a block scalar; it cannot be less than one');
      } else if (!detectedIndent) {
        textIndent = nodeIndent + tmp - 1;
        detectedIndent = true;
      } else {
        throwError(state, 'repeat of an indentation width identifier');
      }

    } else {
      break;
    }
  }

  if (is_WHITE_SPACE(ch)) {
    do { ch = state.input.charCodeAt(++state.position); }
    while (is_WHITE_SPACE(ch));

    if (ch === 0x23/* # */) {
      do { ch = state.input.charCodeAt(++state.position); }
      while (!is_EOL(ch) && (ch !== 0));
    }
  }

  while (ch !== 0) {
    readLineBreak(state);
    state.lineIndent = 0;

    ch = state.input.charCodeAt(state.position);

    while ((!detectedIndent || state.lineIndent < textIndent) &&
           (ch === 0x20/* Space */)) {
      state.lineIndent++;
      ch = state.input.charCodeAt(++state.position);
    }

    if (!detectedIndent && state.lineIndent > textIndent) {
      textIndent = state.lineIndent;
    }

    if (is_EOL(ch)) {
      emptyLines++;
      continue;
    }

    // End of the scalar.
    if (state.lineIndent < textIndent) {

      // Perform the chomping.
      if (chomping === CHOMPING_KEEP) {
        state.result += common$1.repeat('\n', emptyLines);
      } else if (chomping === CHOMPING_CLIP) {
        if (detectedIndent) { // i.e. only if the scalar is not empty.
          state.result += '\n';
        }
      }

      // Break this `while` cycle and go to the funciton's epilogue.
      break;
    }

    // Folded style: use fancy rules to handle line breaks.
    if (folding) {

      // Lines starting with white space characters (more-indented lines) are not folded.
      if (is_WHITE_SPACE(ch)) {
        atMoreIndented = true;
        state.result += common$1.repeat('\n', emptyLines + 1);

      // End of more-indented block.
      } else if (atMoreIndented) {
        atMoreIndented = false;
        state.result += common$1.repeat('\n', emptyLines + 1);

      // Just one line break - perceive as the same line.
      } else if (emptyLines === 0) {
        if (detectedIndent) { // i.e. only if we have already read some scalar content.
          state.result += ' ';
        }

      // Several line breaks - perceive as different lines.
      } else {
        state.result += common$1.repeat('\n', emptyLines);
      }

    // Literal style: just add exact number of line breaks between content lines.
    } else if (detectedIndent) {
      // If current line isn't the first one - count line break from the last content line.
      state.result += common$1.repeat('\n', emptyLines + 1);
    } else {
      // In case of the first content line - count only empty lines.
      state.result += common$1.repeat('\n', emptyLines);
    }

    detectedIndent = true;
    emptyLines = 0;
    captureStart = state.position;

    while (!is_EOL(ch) && (ch !== 0)) {
      ch = state.input.charCodeAt(++state.position);
    }

    captureSegment(state, captureStart, state.position, false);
  }

  return true;
}

function readBlockSequence(state, nodeIndent) {
  var _line,
      _tag      = state.tag,
      _anchor   = state.anchor,
      _result   = [],
      following,
      detected  = false,
      ch;

  if (state.anchor !== null) {
    state.anchorMap[state.anchor] = _result;
  }

  ch = state.input.charCodeAt(state.position);

  while (ch !== 0) {

    if (ch !== 0x2D/* - */) {
      break;
    }

    following = state.input.charCodeAt(state.position + 1);

    if (!is_WS_OR_EOL(following)) {
      break;
    }

    detected = true;
    state.position++;

    if (skipSeparationSpace(state, true, -1)) {
      if (state.lineIndent <= nodeIndent) {
        _result.push(null);
        ch = state.input.charCodeAt(state.position);
        continue;
      }
    }

    _line = state.line;
    composeNode(state, nodeIndent, CONTEXT_BLOCK_IN, false, true);
    _result.push(state.result);
    skipSeparationSpace(state, true, -1);

    ch = state.input.charCodeAt(state.position);

    if ((state.line === _line || state.lineIndent > nodeIndent) && (ch !== 0)) {
      throwError(state, 'bad indentation of a sequence entry');
    } else if (state.lineIndent < nodeIndent) {
      break;
    }
  }

  if (detected) {
    state.tag = _tag;
    state.anchor = _anchor;
    state.kind = 'sequence';
    state.result = _result;
    return true;
  }
  return false;
}

function readBlockMapping(state, nodeIndent, flowIndent) {
  var following,
      allowCompact,
      _line,
      _tag          = state.tag,
      _anchor       = state.anchor,
      _result       = {},
      overridableKeys = {},
      keyTag        = null,
      keyNode       = null,
      valueNode     = null,
      atExplicitKey = false,
      detected      = false,
      ch;

  if (state.anchor !== null) {
    state.anchorMap[state.anchor] = _result;
  }

  ch = state.input.charCodeAt(state.position);

  while (ch !== 0) {
    following = state.input.charCodeAt(state.position + 1);
    _line = state.line; // Save the current line.

    //
    // Explicit notation case. There are two separate blocks:
    // first for the key (denoted by "?") and second for the value (denoted by ":")
    //
    if ((ch === 0x3F/* ? */ || ch === 0x3A/* : */) && is_WS_OR_EOL(following)) {

      if (ch === 0x3F/* ? */) {
        if (atExplicitKey) {
          storeMappingPair(state, _result, overridableKeys, keyTag, keyNode, null);
          keyTag = keyNode = valueNode = null;
        }

        detected = true;
        atExplicitKey = true;
        allowCompact = true;

      } else if (atExplicitKey) {
        // i.e. 0x3A/* : */ === character after the explicit key.
        atExplicitKey = false;
        allowCompact = true;

      } else {
        throwError(state, 'incomplete explicit mapping pair; a key node is missed');
      }

      state.position += 1;
      ch = following;

    //
    // Implicit notation case. Flow-style node as the key first, then ":", and the value.
    //
    } else if (composeNode(state, flowIndent, CONTEXT_FLOW_OUT, false, true)) {

      if (state.line === _line) {
        ch = state.input.charCodeAt(state.position);

        while (is_WHITE_SPACE(ch)) {
          ch = state.input.charCodeAt(++state.position);
        }

        if (ch === 0x3A/* : */) {
          ch = state.input.charCodeAt(++state.position);

          if (!is_WS_OR_EOL(ch)) {
            throwError(state, 'a whitespace character is expected after the key-value separator within a block mapping');
          }

          if (atExplicitKey) {
            storeMappingPair(state, _result, overridableKeys, keyTag, keyNode, null);
            keyTag = keyNode = valueNode = null;
          }

          detected = true;
          atExplicitKey = false;
          allowCompact = false;
          keyTag = state.tag;
          keyNode = state.result;

        } else if (detected) {
          throwError(state, 'can not read an implicit mapping pair; a colon is missed');

        } else {
          state.tag = _tag;
          state.anchor = _anchor;
          return true; // Keep the result of `composeNode`.
        }

      } else if (detected) {
        throwError(state, 'can not read a block mapping entry; a multiline key may not be an implicit key');

      } else {
        state.tag = _tag;
        state.anchor = _anchor;
        return true; // Keep the result of `composeNode`.
      }

    } else {
      break; // Reading is done. Go to the epilogue.
    }

    //
    // Common reading code for both explicit and implicit notations.
    //
    if (state.line === _line || state.lineIndent > nodeIndent) {
      if (composeNode(state, nodeIndent, CONTEXT_BLOCK_OUT, true, allowCompact)) {
        if (atExplicitKey) {
          keyNode = state.result;
        } else {
          valueNode = state.result;
        }
      }

      if (!atExplicitKey) {
        storeMappingPair(state, _result, overridableKeys, keyTag, keyNode, valueNode);
        keyTag = keyNode = valueNode = null;
      }

      skipSeparationSpace(state, true, -1);
      ch = state.input.charCodeAt(state.position);
    }

    if (state.lineIndent > nodeIndent && (ch !== 0)) {
      throwError(state, 'bad indentation of a mapping entry');
    } else if (state.lineIndent < nodeIndent) {
      break;
    }
  }

  //
  // Epilogue.
  //

  // Special case: last mapping's node contains only the key in explicit notation.
  if (atExplicitKey) {
    storeMappingPair(state, _result, overridableKeys, keyTag, keyNode, null);
  }

  // Expose the resulting mapping.
  if (detected) {
    state.tag = _tag;
    state.anchor = _anchor;
    state.kind = 'mapping';
    state.result = _result;
  }

  return detected;
}

function readTagProperty(state) {
  var _position,
      isVerbatim = false,
      isNamed    = false,
      tagHandle,
      tagName,
      ch;

  ch = state.input.charCodeAt(state.position);

  if (ch !== 0x21/* ! */) return false;

  if (state.tag !== null) {
    throwError(state, 'duplication of a tag property');
  }

  ch = state.input.charCodeAt(++state.position);

  if (ch === 0x3C/* < */) {
    isVerbatim = true;
    ch = state.input.charCodeAt(++state.position);

  } else if (ch === 0x21/* ! */) {
    isNamed = true;
    tagHandle = '!!';
    ch = state.input.charCodeAt(++state.position);

  } else {
    tagHandle = '!';
  }

  _position = state.position;

  if (isVerbatim) {
    do { ch = state.input.charCodeAt(++state.position); }
    while (ch !== 0 && ch !== 0x3E/* > */);

    if (state.position < state.length) {
      tagName = state.input.slice(_position, state.position);
      ch = state.input.charCodeAt(++state.position);
    } else {
      throwError(state, 'unexpected end of the stream within a verbatim tag');
    }
  } else {
    while (ch !== 0 && !is_WS_OR_EOL(ch)) {

      if (ch === 0x21/* ! */) {
        if (!isNamed) {
          tagHandle = state.input.slice(_position - 1, state.position + 1);

          if (!PATTERN_TAG_HANDLE.test(tagHandle)) {
            throwError(state, 'named tag handle cannot contain such characters');
          }

          isNamed = true;
          _position = state.position + 1;
        } else {
          throwError(state, 'tag suffix cannot contain exclamation marks');
        }
      }

      ch = state.input.charCodeAt(++state.position);
    }

    tagName = state.input.slice(_position, state.position);

    if (PATTERN_FLOW_INDICATORS.test(tagName)) {
      throwError(state, 'tag suffix cannot contain flow indicator characters');
    }
  }

  if (tagName && !PATTERN_TAG_URI.test(tagName)) {
    throwError(state, 'tag name cannot contain such characters: ' + tagName);
  }

  if (isVerbatim) {
    state.tag = tagName;

  } else if (_hasOwnProperty$2.call(state.tagMap, tagHandle)) {
    state.tag = state.tagMap[tagHandle] + tagName;

  } else if (tagHandle === '!') {
    state.tag = '!' + tagName;

  } else if (tagHandle === '!!') {
    state.tag = 'tag:yaml.org,2002:' + tagName;

  } else {
    throwError(state, 'undeclared tag handle "' + tagHandle + '"');
  }

  return true;
}

function readAnchorProperty(state) {
  var _position,
      ch;

  ch = state.input.charCodeAt(state.position);

  if (ch !== 0x26/* & */) return false;

  if (state.anchor !== null) {
    throwError(state, 'duplication of an anchor property');
  }

  ch = state.input.charCodeAt(++state.position);
  _position = state.position;

  while (ch !== 0 && !is_WS_OR_EOL(ch) && !is_FLOW_INDICATOR(ch)) {
    ch = state.input.charCodeAt(++state.position);
  }

  if (state.position === _position) {
    throwError(state, 'name of an anchor node must contain at least one character');
  }

  state.anchor = state.input.slice(_position, state.position);
  return true;
}

function readAlias(state) {
  var _position, alias,
      ch;

  ch = state.input.charCodeAt(state.position);

  if (ch !== 0x2A/* * */) return false;

  ch = state.input.charCodeAt(++state.position);
  _position = state.position;

  while (ch !== 0 && !is_WS_OR_EOL(ch) && !is_FLOW_INDICATOR(ch)) {
    ch = state.input.charCodeAt(++state.position);
  }

  if (state.position === _position) {
    throwError(state, 'name of an alias node must contain at least one character');
  }

  alias = state.input.slice(_position, state.position);

  if (!state.anchorMap.hasOwnProperty(alias)) {
    throwError(state, 'unidentified alias "' + alias + '"');
  }

  state.result = state.anchorMap[alias];
  skipSeparationSpace(state, true, -1);
  return true;
}

function composeNode(state, parentIndent, nodeContext, allowToSeek, allowCompact) {
  var allowBlockStyles,
      allowBlockScalars,
      allowBlockCollections,
      indentStatus = 1, // 1: this>parent, 0: this=parent, -1: this<parent
      atNewLine  = false,
      hasContent = false,
      typeIndex,
      typeQuantity,
      type,
      flowIndent,
      blockIndent;

  if (state.listener !== null) {
    state.listener('open', state);
  }

  state.tag    = null;
  state.anchor = null;
  state.kind   = null;
  state.result = null;

  allowBlockStyles = allowBlockScalars = allowBlockCollections =
    CONTEXT_BLOCK_OUT === nodeContext ||
    CONTEXT_BLOCK_IN  === nodeContext;

  if (allowToSeek) {
    if (skipSeparationSpace(state, true, -1)) {
      atNewLine = true;

      if (state.lineIndent > parentIndent) {
        indentStatus = 1;
      } else if (state.lineIndent === parentIndent) {
        indentStatus = 0;
      } else if (state.lineIndent < parentIndent) {
        indentStatus = -1;
      }
    }
  }

  if (indentStatus === 1) {
    while (readTagProperty(state) || readAnchorProperty(state)) {
      if (skipSeparationSpace(state, true, -1)) {
        atNewLine = true;
        allowBlockCollections = allowBlockStyles;

        if (state.lineIndent > parentIndent) {
          indentStatus = 1;
        } else if (state.lineIndent === parentIndent) {
          indentStatus = 0;
        } else if (state.lineIndent < parentIndent) {
          indentStatus = -1;
        }
      } else {
        allowBlockCollections = false;
      }
    }
  }

  if (allowBlockCollections) {
    allowBlockCollections = atNewLine || allowCompact;
  }

  if (indentStatus === 1 || CONTEXT_BLOCK_OUT === nodeContext) {
    if (CONTEXT_FLOW_IN === nodeContext || CONTEXT_FLOW_OUT === nodeContext) {
      flowIndent = parentIndent;
    } else {
      flowIndent = parentIndent + 1;
    }

    blockIndent = state.position - state.lineStart;

    if (indentStatus === 1) {
      if (allowBlockCollections &&
          (readBlockSequence(state, blockIndent) ||
           readBlockMapping(state, blockIndent, flowIndent)) ||
          readFlowCollection(state, flowIndent)) {
        hasContent = true;
      } else {
        if ((allowBlockScalars && readBlockScalar(state, flowIndent)) ||
            readSingleQuotedScalar(state, flowIndent) ||
            readDoubleQuotedScalar(state, flowIndent)) {
          hasContent = true;

        } else if (readAlias(state)) {
          hasContent = true;

          if (state.tag !== null || state.anchor !== null) {
            throwError(state, 'alias node should not have any properties');
          }

        } else if (readPlainScalar(state, flowIndent, CONTEXT_FLOW_IN === nodeContext)) {
          hasContent = true;

          if (state.tag === null) {
            state.tag = '?';
          }
        }

        if (state.anchor !== null) {
          state.anchorMap[state.anchor] = state.result;
        }
      }
    } else if (indentStatus === 0) {
      // Special case: block sequences are allowed to have same indentation level as the parent.
      // http://www.yaml.org/spec/1.2/spec.html#id2799784
      hasContent = allowBlockCollections && readBlockSequence(state, blockIndent);
    }
  }

  if (state.tag !== null && state.tag !== '!') {
    if (state.tag === '?') {
      for (typeIndex = 0, typeQuantity = state.implicitTypes.length;
           typeIndex < typeQuantity;
           typeIndex += 1) {
        type = state.implicitTypes[typeIndex];

        // Implicit resolving is not allowed for non-scalar types, and '?'
        // non-specific tag is only assigned to plain scalars. So, it isn't
        // needed to check for 'kind' conformity.

        if (type.resolve(state.result)) { // `state.result` updated in resolver if matched
          state.result = type.construct(state.result);
          state.tag = type.tag;
          if (state.anchor !== null) {
            state.anchorMap[state.anchor] = state.result;
          }
          break;
        }
      }
    } else if (_hasOwnProperty$2.call(state.typeMap, state.tag)) {
      type = state.typeMap[state.tag];

      if (state.result !== null && type.kind !== state.kind) {
        throwError(state, 'unacceptable node kind for !<' + state.tag + '> tag; it should be "' + type.kind + '", not "' + state.kind + '"');
      }

      if (!type.resolve(state.result)) { // `state.result` updated in resolver if matched
        throwError(state, 'cannot resolve a node with !<' + state.tag + '> explicit tag');
      } else {
        state.result = type.construct(state.result);
        if (state.anchor !== null) {
          state.anchorMap[state.anchor] = state.result;
        }
      }
    } else {
      throwError(state, 'unknown tag !<' + state.tag + '>');
    }
  }

  if (state.listener !== null) {
    state.listener('close', state);
  }
  return state.tag !== null ||  state.anchor !== null || hasContent;
}

function readDocument(state) {
  var documentStart = state.position,
      _position,
      directiveName,
      directiveArgs,
      hasDirectives = false,
      ch;

  state.version = null;
  state.checkLineBreaks = state.legacy;
  state.tagMap = {};
  state.anchorMap = {};

  while ((ch = state.input.charCodeAt(state.position)) !== 0) {
    skipSeparationSpace(state, true, -1);

    ch = state.input.charCodeAt(state.position);

    if (state.lineIndent > 0 || ch !== 0x25/* % */) {
      break;
    }

    hasDirectives = true;
    ch = state.input.charCodeAt(++state.position);
    _position = state.position;

    while (ch !== 0 && !is_WS_OR_EOL(ch)) {
      ch = state.input.charCodeAt(++state.position);
    }

    directiveName = state.input.slice(_position, state.position);
    directiveArgs = [];

    if (directiveName.length < 1) {
      throwError(state, 'directive name must not be less than one character in length');
    }

    while (ch !== 0) {
      while (is_WHITE_SPACE(ch)) {
        ch = state.input.charCodeAt(++state.position);
      }

      if (ch === 0x23/* # */) {
        do { ch = state.input.charCodeAt(++state.position); }
        while (ch !== 0 && !is_EOL(ch));
        break;
      }

      if (is_EOL(ch)) break;

      _position = state.position;

      while (ch !== 0 && !is_WS_OR_EOL(ch)) {
        ch = state.input.charCodeAt(++state.position);
      }

      directiveArgs.push(state.input.slice(_position, state.position));
    }

    if (ch !== 0) readLineBreak(state);

    if (_hasOwnProperty$2.call(directiveHandlers, directiveName)) {
      directiveHandlers[directiveName](state, directiveName, directiveArgs);
    } else {
      throwWarning(state, 'unknown document directive "' + directiveName + '"');
    }
  }

  skipSeparationSpace(state, true, -1);

  if (state.lineIndent === 0 &&
      state.input.charCodeAt(state.position)     === 0x2D/* - */ &&
      state.input.charCodeAt(state.position + 1) === 0x2D/* - */ &&
      state.input.charCodeAt(state.position + 2) === 0x2D/* - */) {
    state.position += 3;
    skipSeparationSpace(state, true, -1);

  } else if (hasDirectives) {
    throwError(state, 'directives end mark is expected');
  }

  composeNode(state, state.lineIndent - 1, CONTEXT_BLOCK_OUT, false, true);
  skipSeparationSpace(state, true, -1);

  if (state.checkLineBreaks &&
      PATTERN_NON_ASCII_LINE_BREAKS.test(state.input.slice(documentStart, state.position))) {
    throwWarning(state, 'non-ASCII line breaks are interpreted as content');
  }

  state.documents.push(state.result);

  if (state.position === state.lineStart && testDocumentSeparator(state)) {

    if (state.input.charCodeAt(state.position) === 0x2E/* . */) {
      state.position += 3;
      skipSeparationSpace(state, true, -1);
    }
    return;
  }

  if (state.position < (state.length - 1)) {
    throwError(state, 'end of the stream or a document separator is expected');
  } else {
    return;
  }
}


function loadDocuments(input, options) {
  input = String(input);
  options = options || {};

  if (input.length !== 0) {

    // Add tailing `\n` if not exists
    if (input.charCodeAt(input.length - 1) !== 0x0A/* LF */ &&
        input.charCodeAt(input.length - 1) !== 0x0D/* CR */) {
      input += '\n';
    }

    // Strip BOM
    if (input.charCodeAt(0) === 0xFEFF) {
      input = input.slice(1);
    }
  }

  var state = new State(input, options);

  // Use 0 as string terminator. That significantly simplifies bounds check.
  state.input += '\0';

  while (state.input.charCodeAt(state.position) === 0x20/* Space */) {
    state.lineIndent += 1;
    state.position += 1;
  }

  while (state.position < (state.length - 1)) {
    readDocument(state);
  }

  return state.documents;
}


function loadAll(input, iterator, options) {
  var documents = loadDocuments(input, options), index, length;

  for (index = 0, length = documents.length; index < length; index += 1) {
    iterator(documents[index]);
  }
}


function load$1(input, options) {
  var documents = loadDocuments(input, options);

  if (documents.length === 0) {
    /*eslint-disable no-undefined*/
    return undefined;
  } else if (documents.length === 1) {
    return documents[0];
  }
  throw new exception('expected a single document in the stream, but found more');
}


function safeLoadAll(input, output, options) {
  loadAll(input, output, common$1.extend({ schema: default_safe }, options));
}


function safeLoad(input, options) {
  return load$1(input, common$1.extend({ schema: default_safe }, options));
}


var loadAll_1     = loadAll;
var load_1        = load$1;
var safeLoadAll_1 = safeLoadAll;
var safeLoad_1    = safeLoad;

var loader = {
	loadAll: loadAll_1,
	load: load_1,
	safeLoadAll: safeLoadAll_1,
	safeLoad: safeLoad_1
};

// var dumper = require('./js-yaml/dumper');


function deprecated(name) {
  return function () {
    throw new Error('Function ' + name + ' is deprecated and cannot be used.');
  };
}


var Type$1                = type;
var Schema$1              = schema;
var FAILSAFE_SCHEMA     = failsafe;
var JSON_SCHEMA         = json;
var CORE_SCHEMA         = core;
var DEFAULT_SAFE_SCHEMA = default_safe;
var DEFAULT_FULL_SCHEMA = default_full;
var load$2                = loader.load;
var loadAll$1             = loader.loadAll;
var safeLoad$1            = loader.safeLoad;
var safeLoadAll$1         = loader.safeLoadAll;
// module.exports.dump                = dumper.dump;
// module.exports.safeDump            = dumper.safeDump;
var YAMLException$1       = exception;

// Deprecated schema names from JS-YAML 2.0.x
var MINIMAL_SCHEMA = failsafe;
var SAFE_SCHEMA    = default_safe;
var DEFAULT_SCHEMA = default_full;

// Deprecated functions from JS-YAML 1.x.x
var scan           = deprecated('scan');
var parse          = deprecated('parse');
var compose        = deprecated('compose');
var addConstructor = deprecated('addConstructor');

var jsYaml = {
	Type: Type$1,
	Schema: Schema$1,
	FAILSAFE_SCHEMA: FAILSAFE_SCHEMA,
	JSON_SCHEMA: JSON_SCHEMA,
	CORE_SCHEMA: CORE_SCHEMA,
	DEFAULT_SAFE_SCHEMA: DEFAULT_SAFE_SCHEMA,
	DEFAULT_FULL_SCHEMA: DEFAULT_FULL_SCHEMA,
	load: load$2,
	loadAll: loadAll$1,
	safeLoad: safeLoad$1,
	safeLoadAll: safeLoadAll$1,
	YAMLException: YAMLException$1,
	MINIMAL_SCHEMA: MINIMAL_SCHEMA,
	SAFE_SCHEMA: SAFE_SCHEMA,
	DEFAULT_SCHEMA: DEFAULT_SCHEMA,
	scan: scan,
	parse: parse,
	compose: compose,
	addConstructor: addConstructor
};

var jsYaml$1 = jsYaml;

var SceneBundle =
/*#__PURE__*/
function () {
  function SceneBundle(url, path, parent) {
    if (parent === void 0) {
      parent = null;
    }

    this.url = url; // If a base path was provided, use it for resolving local bundle resources only if
    // the base path is absolute, or this bundle's path is relative

    if (path && (!__chunk_1.isRelativeURL(path) || __chunk_1.isRelativeURL(this.url))) {
      this.path = path;
    } else {
      this.path = __chunk_1.pathForURL(this.url);
    }

    this.path_for_parent = path || this.path; // for resolving paths relative to a parent bundle

    this.parent = parent; // An ancestor bundle may be a container (e.g. zip file) that needs to resolve relative paths
    // for any scenes it contains, e.g. `root.zip` has a `root.yaml` that includes a `folder/child.yaml`:
    // resources within `child.yaml` must be resolved through the bundle for `root.zip`

    this.container = null;

    if (this.parent) {
      if (this.parent.container) {
        this.container = this.parent.container;
      } else if (this.parent.isContainer()) {
        this.container = this.parent;
      }
    }
  }

  var _proto = SceneBundle.prototype;

  _proto.load = function load() {
    return loadResource(this.url);
  } // Info for retrieving a specific resource from this bundle
  // url: fully qualified URL to retrieve the content of the resource (e.g. zips will transform this to blob URL)
  // path: original path of the resource within the bundle (for resolving paths up the bundle tree)
  // type: file extension (used for determining bundle type, `yaml` or `zip`)
  ;

  _proto.resourceFor = function resourceFor(url) {
    return {
      url: this.urlFor(url),
      path: this.pathFor(url),
      type: this.typeFor(url)
    };
  };

  _proto.urlFor = function urlFor(url) {
    if (isGlobal(url)) {
      return url;
    }

    if (__chunk_1.isRelativeURL(url) && this.container) {
      return this.parent.urlFor(this.path_for_parent + url);
    }

    return __chunk_1.addBaseURL(url, this.path);
  };

  _proto.pathFor = function pathFor(url) {
    return __chunk_1.pathForURL(url);
  };

  _proto.typeFor = function typeFor(url) {
    return __chunk_1.extensionForURL(url);
  };

  _proto.isContainer = function isContainer() {
    return false;
  };

  return SceneBundle;
}();
var ZipSceneBundle =
/*#__PURE__*/
function (_SceneBundle) {
  __chunk_1._inheritsLoose(ZipSceneBundle, _SceneBundle);

  function ZipSceneBundle(url, path, parent) {
    var _this;

    _this = _SceneBundle.call(this, url, path, parent) || this;
    _this.zip = null;
    _this.files = {};
    _this.root = null;
    _this.path = '';
    return _this;
  }

  var _proto2 = ZipSceneBundle.prototype;

  _proto2.isContainer = function isContainer() {
    return true;
  };

  _proto2.load = function load() {
    return new Promise(function ($return, $error) {
      var data;
      this.zip = new lib();

      if (typeof this.url === 'string') {
        return Promise.resolve(__chunk_1.Utils.io(this.url, 60000, 'arraybuffer')).then(function ($await_4) {
          try {
            data = $await_4;
            return Promise.resolve(this.zip.loadAsync(data)).then(function ($await_5) {
              try {
                return Promise.resolve(this.parseZipFiles()).then(function ($await_6) {
                  try {
                    return $return(this.loadRoot());
                  } catch ($boundEx) {
                    return $error($boundEx);
                  }
                }.bind(this), $error);
              } catch ($boundEx) {
                return $error($boundEx);
              }
            }.bind(this), $error);
          } catch ($boundEx) {
            return $error($boundEx);
          }
        }.bind(this), $error);
      } else {
        return $return(this);
      }

      return $return();
    }.bind(this));
  };

  _proto2.urlFor = function urlFor(url) {
    if (isGlobal(url)) {
      return url;
    }

    if (__chunk_1.isRelativeURL(url)) {
      return this.urlForZipFile(__chunk_1.flattenRelativeURL(url));
    }

    return _SceneBundle.prototype.urlFor.call(this, url);
  };

  _proto2.typeFor = function typeFor(url) {
    if (__chunk_1.isRelativeURL(url)) {
      return this.typeForZipFile(url);
    }

    return _SceneBundle.prototype.typeFor.call(this, url);
  };

  _proto2.loadRoot = function loadRoot() {
    this.findRoot();
    return loadResource(this.urlForZipFile(this.root));
  };

  _proto2.findRoot = function findRoot() {
    var _this2 = this;

    // There must be a single YAML file at the top level of the zip
    var yamls = Object.keys(this.files).filter(function (path) {
      return _this2.files[path].depth === 0;
    }).filter(function (path) {
      return __chunk_1.extensionForURL(path) === 'yaml';
    });

    if (yamls.length === 1) {
      this.root = yamls[0];
    } // No root found


    if (!this.root) {
      var msg = "Could not find root scene for bundle '" + this.url + "': ";
      msg += 'The zip archive\'s root level must contain a single scene file with the \'.yaml\' extension. ';

      if (yamls.length > 0) {
        msg += "Found multiple YAML files at the root level: " + yamls.map(function (r) {
          return '\'' + r + '\'';
        }).join(', ') + ".";
      } else {
        msg += 'Found NO YAML files at the root level.';
      }

      throw Error(msg);
    }
  };

  _proto2.parseZipFiles = function parseZipFiles() {
    return new Promise(function ($return, $error) {
      var paths, queue, data, i, path, depth;
      paths = [];
      queue = [];
      this.zip.forEach(function (path, file) {
        if (!file.dir) {
          paths.push(path);
          queue.push(file.async('arraybuffer'));
        }
      });
      return Promise.resolve(Promise.all(queue)).then(function ($await_7) {
        try {
          data = $await_7;

          for (i = 0; i < data.length; i++) {
            path = paths[i];
            depth = path.split('/').length - 1;
            this.files[path] = {
              data: data[i],
              type: __chunk_1.extensionForURL(path),
              depth: depth
            };
          }

          return $return();
        } catch ($boundEx) {
          return $error($boundEx);
        }
      }.bind(this), $error);
    }.bind(this));
  };

  _proto2.urlForZipFile = function urlForZipFile(file) {
    if (this.files[file]) {
      if (!this.files[file].url) {
        this.files[file].url = __chunk_1.createObjectURL(new Blob([this.files[file].data]));
      }

      return this.files[file].url;
    }
  };

  _proto2.typeForZipFile = function typeForZipFile(file) {
    return this.files[file] && this.files[file].type;
  };

  return ZipSceneBundle;
}(SceneBundle);
function createSceneBundle(url, path, parent, type) {
  if (type === void 0) {
    type = null;
  }

  if (type != null && type === 'zip' || typeof url === 'string' && !__chunk_1.isLocalURL(url) && __chunk_1.extensionForURL(url) === 'zip') {
    return new ZipSceneBundle(url, path, parent);
  }

  return new SceneBundle(url, path, parent);
} // References a global property?

function isGlobal(val) {
  if (val && val.slice(0, 7) === 'global.') {
    return true;
  }

  return false;
}

function parseResource(body) {
  var data;

  try {
    // jsyaml 'json' option allows duplicate keys
    // Keeping this for backwards compatibility, but should consider migrating to requiring
    // unique keys, as this is YAML spec. But Tangram ES currently accepts dupe keys as well,
    // so should consider how best to unify.
    data = jsYaml$1.safeLoad(body, {
      json: true
    });
  } catch (e) {
    throw e;
  }

  return data;
}

function loadResource(source) {
  return new Promise(function (resolve, reject) {
    if (typeof source === 'string') {
      __chunk_1.Utils.io(source).then(function (body) {
        try {
          var data = parseResource(body);
          resolve(data);
        } catch (e) {
          reject(e);
        }
      }, reject);
    } else {
      // shallow copy to avoid modifying provided object, allowing a single config object to be loaded multiple times
      // TODO: address possible modifications to nested properties (mostly harmless / due to data normalization)
      source = Object.assign({}, source);
      resolve(source);
    }
  });
}

var SceneLoader;
var SceneLoader$1 = SceneLoader = {
  // Load scenes definitions from URL & proprocess
  loadScene: function loadScene(url, _temp) {
    return new Promise(function ($return, $error) {
      var _this, _ref, path, type, errors, scene, _this$finalize, config, bundle;

      _this = this;
      _ref = _temp === void 0 ? {} : _temp, path = _ref.path, type = _ref.type;
      errors = [];
      return Promise.resolve(this.loadSceneRecursive({
        url: url,
        path: path,
        type: type
      }, null, errors)).then(function ($await_2) {
        try {
          scene = $await_2;
          _this$finalize = this.finalize(scene), config = _this$finalize.config, bundle = _this$finalize.bundle;

          if (!config) {
            // root scene failed to load, reject with first error
            return $error(errors[0]);
          } else if (errors.length > 0) {
            // scene loaded, but some imports had errors
            errors.forEach(function (error) {
              var message = "Failed to import scene: " + error.url;
              __chunk_1.log('error', message, error);

              _this.trigger('error', {
                type: 'scene_import',
                message: message,
                error: error,
                url: error.url
              });
            });
          }

          return $return({
            config: config,
            bundle: bundle
          });
        } catch ($boundEx) {
          return $error($boundEx);
        }
      }.bind(this), $error);
    }.bind(this));
  },
  // Loads scene files from URL, recursively loading 'import' scenes
  // Optional *initial* path only (won't be passed to recursive 'import' calls)
  // Useful for loading resources in base scene file from a separate location
  // (e.g. in Tangram Play, when modified local scene should still refer to original resource URLs)
  loadSceneRecursive: function loadSceneRecursive(_ref2, parent, errors) {
    return new Promise(function ($return, $error) {
      var _this2, url, path, type, bundle, config, imports, queue, configs;

      _this2 = this;
      url = _ref2.url, path = _ref2.path, type = _ref2.type;

      if (errors === void 0) {
        errors = [];
      }

      if (!url) {
        return $return({});
      }

      bundle = createSceneBundle(url, path, parent, type);

      var $Try_1_Catch = function (error) {
        try {
          // Collect scene load errors as we go
          error.url = url;
          errors.push(error);
          return $return({});
        } catch ($boundEx) {
          return $error($boundEx);
        }
      };

      try {
        return Promise.resolve(bundle.load()).then(function ($await_3) {
          try {
            config = $await_3;

            // debugger
            if (config.import == null) {
              this.normalize(config, bundle);
              return $return({
                config: config,
                bundle: bundle
              });
            } // accept single entry or array


            if (!Array.isArray(config.import)) {
              config.import = [config.import]; // convert to array
            } // Collect URLs of scenes to import


            imports = [];
            config.import.forEach(function (url) {
              // Convert scene objects to URLs
              if (typeof url === 'object') {
                url = __chunk_1.createObjectURL(new Blob([JSON.stringify(url)]));
              }

              imports.push(bundle.resourceFor(url));
            });
            delete config.import; // don't want to merge this property
            // load and normalize imports

            queue = imports.map(function (resource) {
              return _this2.loadSceneRecursive(resource, bundle, errors);
            });
            return Promise.resolve(Promise.all(queue)).then(function ($await_4) {
              try {
                configs = $await_4.map(function (r) {
                  return _this2.normalize(r.config, r.bundle);
                }).map(function (r) {
                  return r.config;
                });
                config = __chunk_1.mergeObjects.apply(void 0, configs.concat([config]));
                this.normalize(config, bundle); // last normalize parent, after merge

                return $return({
                  config: config,
                  bundle: bundle
                });
              } catch ($boundEx) {
                return $Try_1_Catch($boundEx);
              }
            }.bind(this), $Try_1_Catch);
          } catch ($boundEx) {
            return $Try_1_Catch($boundEx);
          }
        }.bind(this), $Try_1_Catch);
      } catch (error) {
        $Try_1_Catch(error);
      }
    }.bind(this));
  },
  // Normalize properties that should be adjust within each local scene file (usually by path)
  normalize: function normalize(config, bundle) {
    this.normalizeDataSources(config, bundle);
    this.normalizeFonts(config, bundle);
    this.normalizeTextures(config, bundle);
    this.hoistTextures(config, bundle);
    return {
      config: config,
      bundle: bundle
    };
  },
  // Expand paths for data source
  normalizeDataSources: function normalizeDataSources(config, bundle) {
    config.sources = config.sources || {};

    for (var sn in config.sources) {
      this.normalizeDataSource(config.sources[sn], bundle);
    }

    return config;
  },
  normalizeDataSource: function normalizeDataSource(source, bundle) {
    source.url = bundle.urlFor(source.url); // composite untiled raster sources

    if (Array.isArray(source.composite)) {
      source.composite.forEach(function (c) {
        return c.url = bundle.urlFor(c.url);
      });
    } // custom scripts


    if (source.scripts) {
      // convert legacy array-style scripts to object format (script URL is used as both key and value)
      if (Array.isArray(source.scripts)) {
        source.scripts = source.scripts.reduce(function (val, cur) {
          val[cur] = cur;
          return val;
        }, {});
      } // resolve URLs for external scripts


      for (var s in source.scripts) {
        source.scripts[s] = bundle.urlFor(source.scripts[s]);
      }
    }

    return source;
  },
  // Expand paths for fonts
  normalizeFonts: function normalizeFonts(config, bundle) {
    config.fonts = config.fonts || {};

    for (var family in config.fonts) {
      if (Array.isArray(config.fonts[family])) {
        config.fonts[family].forEach(function (face) {
          face.url = face.url && bundle.urlFor(face.url);
        });
      } else {
        var face = config.fonts[family];
        face.url = face.url && bundle.urlFor(face.url);
      }
    }

    return config;
  },
  // Expand paths and centralize texture definitions for a scene object
  normalizeTextures: function normalizeTextures(config, bundle) {
    config.textures = config.textures || {}; // Add current scene's base path to globally defined textures
    // Only adds path for textures with relative URLs, so textures in imported scenes get the base
    // path of their immediate scene file

    if (config.textures) {
      for (var tn in config.textures) {
        var texture = config.textures[tn];

        if (texture.url) {
          texture.url = bundle.urlFor(texture.url);
        }
      }
    }
  },
  // Move inline (URL string) textures to the scene's top-level set of textures (config.textures).
  // There are 4 such cases of textures:
  // - in a style's `texture` property
  // - in a style's `material` properties
  // - in a style's custom uniforms (`shaders.uniforms`)
  // - in a draw groups `texture` property
  hoistTextures: function hoistTextures(config, bundle) {
    var _this3 = this;

    // Resolve URLs for inline textures
    if (config.styles) {
      var _loop = function _loop(sn) {
        var style = config.styles[sn]; // Style `texture`

        var tex = style.texture;

        if (typeof tex === 'string' && !config.textures[tex]) {
          style.texture = _this3.hoistTexture(tex, config, bundle);
        } // Material


        if (style.material) {
          ['emission', 'ambient', 'diffuse', 'specular', 'normal'].forEach(function (prop) {
            // Material property has a texture
            var tex = style.material[prop] != null && style.material[prop].texture;

            if (typeof tex === 'string' && !config.textures[tex]) {
              style.material[prop].texture = _this3.hoistTexture(tex, config, bundle);
            }
          });
        }
      };

      for (var sn in config.styles) {
        _loop(sn);
      }
    } // Special handling for shader uniforms, exclude globals because they are ambiguous:
    // could later be resolved to a string value indicating a texture, but could also be a vector or other type


    this.hoistStyleShaderUniformTextures(config, bundle, {
      include_globals: false
    }); // Resolve and hoist inline textures in draw blocks

    if (config.layers) {
      var stack = [config.layers];

      while (stack.length > 0) {
        var layer = stack.pop(); // only recurse into objects

        if (typeof layer !== 'object' || Array.isArray(layer)) {
          continue;
        }

        for (var prop in layer) {
          if (prop === 'draw') {
            // process draw groups for current layer
            var draws = layer[prop];

            for (var group in draws) {
              if (draws[group].texture) {
                var tex = draws[group].texture;

                if (typeof tex === 'string' && !config.textures[tex]) {
                  draws[group].texture = this.hoistTexture(tex, config, bundle);
                }
              } // special handling for outlines :(


              if (draws[group].outline && draws[group].outline.texture) {
                var _tex = draws[group].outline.texture;

                if (typeof _tex === 'string' && !config.textures[_tex]) {
                  draws[group].outline.texture = this.hoistTexture(_tex, config, bundle);
                }
              }
            }
          } else if (__chunk_1.isReserved(prop)) {
            continue; // skip reserved keyword
          } else {
            stack.push(layer[prop]); // traverse sublayer
          }
        }
      }
    }
  },
  hoistStyleShaderUniformTextures: function hoistStyleShaderUniformTextures(config, bundle, _ref3) {
    var _this4 = this;

    var include_globals = _ref3.include_globals;

    // Resolve URLs for inline textures
    if (config.styles) {
      for (var sn in config.styles) {
        var style = config.styles[sn]; // Shader uniforms

        if (style.shaders && style.shaders.uniforms) {
          __chunk_1.GLSL.parseUniforms(style.shaders.uniforms).forEach(function (_ref4) {
            var type = _ref4.type,
                value = _ref4.value,
                key = _ref4.key,
                uniforms = _ref4.uniforms;

            // Texture by URL (string-named texture not referencing existing texture definition)
            if (type === 'sampler2D' && typeof value === 'string' && !config.textures[value] && (include_globals || !isGlobal(value))) {
              uniforms[key] = _this4.hoistTexture(value, config, bundle);
            }
          });
        }
      }
    }
  },
  // Convert an inline URL texture to a global one, and return the texture's (possibly modified) name
  hoistTexture: function hoistTexture(tex, config, bundle) {
    var global = isGlobal(tex);
    var url = global ? tex : bundle.urlFor(tex);
    var name = global ? "texture-" + url : url;
    config.textures[name] = {
      url: url
    };
    return name;
  },
  // Substitutes global scene properties (those defined in the `config.global` object) for any style values
  // of the form `global.`, for example `color: global.park_color` would be replaced with the value (if any)
  // defined for the `park_color` property in `config.global.park_color`.
  applyGlobalProperties: function applyGlobalProperties(config) {
    if (!config.global || Object.keys(config.global).length === 0) {
      return config; // no global properties to transform
    }

    var globals = flattenProperties(config.global); // flatten nested globals for simpler string look-ups
    // Find and apply new global properties (and re-apply old ones)

    function applyGlobals(obj, target, key) {
      var prop; // Check for previously applied global substitution

      if (target != null && typeof target === 'object' && target._global_prop && target._global_prop[key]) {
        prop = target._global_prop[key];
      } // Check string for new global substitution
      else if (typeof obj === 'string' && obj.slice(0, 7) === 'global.') {
          prop = obj;
        } // Found global property to substitute


      if (prop) {
        // Mark property as global substitution
        if (target._global_prop == null) {
          Object.defineProperty(target, '_global_prop', {
            value: {}
          });
        }

        target._global_prop[key] = prop; // Get current global value

        var val = globals[prop];
        var stack;

        while (typeof val === 'string' && val.slice(0, 7) === 'global.') {
          // handle globals that refer to other globals, detecting any cyclical references
          stack = stack || [prop];

          if (stack.indexOf(val) > -1) {
            __chunk_1.log({
              level: 'warn',
              once: true
            }, 'Global properties: cyclical reference detected', stack);
            val = null;
            break;
          }

          stack.push(val);
          val = globals[val];
        } // Create getter/setter


        Object.defineProperty(target, key, {
          enumerable: true,
          get: function get() {
            return val; // return substituted value
          },
          set: function set(v) {
            // clear the global substitution and remove the getter/setter
            delete target._global_prop[key];
            delete target[key];
            target[key] = v; // save the new value
          }
        });
      } // Loop through object keys or array indices
      else if (Array.isArray(obj)) {
          for (var p = 0; p < obj.length; p++) {
            applyGlobals(obj[p], obj, p);
          }
        } else if (typeof obj === 'object') {
          for (var _p in obj) {
            applyGlobals(obj[_p], obj, _p);
          }
        }

      return obj;
    }

    return applyGlobals(config);
  },
  // Normalize some scene-wide settings that apply to the final, merged scene
  finalize: function finalize(_ref5) {
    var config = _ref5.config,
        bundle = _ref5.bundle;

    if (!config) {
      return {};
    } // Ensure top-level properties


    config.global = config.global || {};
    config.scene = config.scene || {};
    config.cameras = config.cameras || {};
    config.lights = config.lights || {};
    config.styles = config.styles || {};
    config.layers = config.layers || {}; // If only one camera specified, set it as default

    if (config.camera) {
      config.cameras.default = config.camera;
    } // If no cameras specified, create one


    if (Object.keys(config.cameras).length === 0) {
      config.cameras.default = {};
    } // If no lights specified, create default


    if (Object.keys(config.lights).length === 0 || Object.keys(config.lights).every(function (i) {
      return config.lights[i].visible === false;
    })) {
      config.lights.default_light = {
        type: 'directional'
      };
    }

    return {
      config: config,
      bundle: bundle
    };
  }
}; // Flatten nested properties for simpler string look-ups

function flattenProperties(obj, prefix, globals) {
  if (prefix === void 0) {
    prefix = null;
  }

  if (globals === void 0) {
    globals = {};
  }

  prefix = prefix ? prefix + '.' : 'global.';

  for (var p in obj) {
    var key = prefix + p;
    var val = obj[p];
    globals[key] = val;

    if (typeof val === 'object' && !Array.isArray(val)) {
      flattenProperties(val, key, globals);
    }
  }

  return globals;
}

__chunk_1.subscribeMixin(SceneLoader);

var TilePyramid =
/*#__PURE__*/
function () {
  function TilePyramid() {
    this.tiles = {};
    this.max_proxy_descendant_depth = 6; // # of levels to search up/down for proxy tiles

    this.max_proxy_ancestor_depth = 7;
    this.children_cache = {}; // cache for children of coordinates
  }

  var _proto = TilePyramid.prototype;

  _proto.addTile = function addTile(tile) {
    // Add target tile
    this.tiles[tile.key] = this.tiles[tile.key] || {
      descendants: 0
    };
    this.tiles[tile.key].tile = tile; // Add to parents

    while (tile.style_z >= 0) {
      tile = __chunk_1.TileID.parent(tile);

      if (!tile) {
        return;
      }

      if (!this.tiles[tile.key]) {
        this.tiles[tile.key] = {
          descendants: 0
        };
      }

      this.tiles[tile.key].descendants++;
    }
  };

  _proto.removeTile = function removeTile(tile) {
    // Remove target tile
    if (this.tiles[tile.key]) {
      delete this.tiles[tile.key].tile;

      if (this.tiles[tile.key].descendants === 0) {
        delete this.tiles[tile.key]; // remove whole tile in tree
      }
    } // Decrement reference count up the tile pyramid


    while (tile.style_z >= 0) {
      tile = __chunk_1.TileID.parent(tile);

      if (!tile) {
        return;
      }

      if (this.tiles[tile.key] && this.tiles[tile.key].descendants > 0) {
        this.tiles[tile.key].descendants--;

        if (this.tiles[tile.key].descendants === 0 && !this.tiles[tile.key].tile) {
          delete this.tiles[tile.key]; // remove whole tile in tree
        }
      }
    }
  } // Find the parent tile for a given tile and style zoom level
  ;

  _proto.getAncestor = function getAncestor(tile) {
    var level = 0;

    while (level < this.max_proxy_ancestor_depth) {
      tile = __chunk_1.TileID.parent(tile);

      if (!tile) {
        return;
      }

      if (this.tiles[tile.key] && this.tiles[tile.key].tile && this.tiles[tile.key].tile.loaded) {
        return this.tiles[tile.key].tile;
      }

      level++;
    }
  } // Find the descendant tiles for a given tile and style zoom level
  ;

  _proto.getDescendants = function getDescendants(tile, level) {
    var _this = this;

    if (level === void 0) {
      level = 0;
    }

    var descendants = [];

    if (level < this.max_proxy_descendant_depth) {
      var tiles = __chunk_1.TileID.children(tile, this.children_cache);

      if (!tiles) {
        return;
      }

      tiles.forEach(function (t) {
        if (_this.tiles[t.key]) {
          if (_this.tiles[t.key].tile && _this.tiles[t.key].tile.loaded) {
            descendants.push(_this.tiles[t.key].tile);
          } else if (_this.tiles[t.key].descendants > 0) {
            // didn't find any children, try next level
            descendants.push.apply(descendants, _this.getDescendants(t, level + 1));
          }
        }
      });
    }

    return descendants;
  };

  return TilePyramid;
}();

var visible = {}; // currently visible labels

var prev_visible = {}; // previously visible labels (in last collision run)

function mainThreadLabelCollisionPass(tiles, view_zoom, hide_breach) {
  return new Promise(function ($return, $error) {
    var containers, labels, meshes;

    if (hide_breach === void 0) {
      hide_breach = false;
    }

    // Swap/reset visible label set
    prev_visible = visible; // save last visible label set

    visible = {}; // initialize new visible label set
    // Build label containers from tiles

    containers = buildLabels(tiles, view_zoom);
    // Collide all labels in a single group
    // TODO: maybe rename tile and style to group/subgroup?
    __chunk_1.Collision.startTile('main', {
      apply_repeat_groups: true,
      return_hidden: true
    });
    __chunk_1.Collision.addStyle('main', 'main');
    return Promise.resolve(__chunk_1.Collision.collide(containers, 'main', 'main')).then(function ($await_1) {
      try {
        labels = $await_1;
        meshes = [];
        labels.forEach(function (container) {
          // Hide breach labels (those that cross tile boundaries) while tiles are loading, unless they
          // were previously visible (otherwise fully loaded/collided breach labels will flicker in and out
          // when new tiles load, even if they aren't adjacent)
          var show = 0;

          if (container.show === true && (!hide_breach || !container.label.breach || prev_visible[container.label.id])) {
            show = 1;
          }

          if (show) {
            visible[container.label.id] = true; // track visible labels
          }

          var changed = true; // check if label visibility changed on this collision pass

          container.ranges.forEach(function (r) {
            if (!changed) {
              return; // skip rest of label if state hasn't changed
            }

            var mesh = container.mesh;

            if (!mesh.valid) {
              return;
            }

            var off = mesh.vertex_layout.offset.a_shape; // byte offset (within each vertex) of attribute

            var stride = mesh.vertex_layout.stride; // byte stride per vertex

            for (var i = 0; i < r[1]; i++) {
              // NB: +6 is because attribute is a short int (2 bytes each), and we're skipping to 3rd element, 6=3*2
              if (mesh.vertex_data[r[0] + i * stride + off + 6] === show) {
                changed = false;
                return; // label hasn't changed states, skip further updates
              }

              mesh.vertex_data[r[0] + i * stride + off + 6] = show;
            }

            if (meshes.indexOf(mesh) === -1) {
              meshes.push(mesh);
            }
          });
        }); // Upload updated meshes and make them visible

        meshes.forEach(function (mesh) {
          return mesh.upload();
        });
        tiles.forEach(function (t) {
          return t.swapPendingLabels();
        });
        return $return({
          labels: labels,
          containers: containers
        }); // currently returned for debugging
      } catch ($boundEx) {
        return $error($boundEx);
      }
    }, $error);
  });
}

function buildLabels(tiles, view_zoom) {
  var labels = {};
  var containers = {}; // Collect labels from each tile and turn into new label instances

  tiles.forEach(function (tile) {
    var units_per_meter = __chunk_1.Geo.unitsPerMeter(tile.coords.z); // scale from tile units to mercator meters

    var zoom_scale = Math.pow(2, view_zoom - tile.style_z); // adjust label size by view zoom

    var size_scale = units_per_meter * zoom_scale; // scale from tile units to zoom-adjusted meters

    var meters_per_pixel = __chunk_1.Geo.metersPerPixel(view_zoom); // First pass: create label instances and centralize collision containers
    // Combine existing (previously collided) and pending (waiting to be collided for first time) meshes

    var tile_meshes = Object.assign({}, tile.meshes, tile.pending_label_meshes);

    for (var style in tile_meshes) {
      var meshes = tile_meshes[style];
      meshes.forEach(function (mesh) {
        if (mesh.labels) {
          for (var label_id in mesh.labels) {
            // For proxy tiles, only allow visible labels to be *hidden* by further collisions,
            // don't allow new ones to appear. Promotes label stability and prevents thrash
            // from different labels (often not thematically relevant given the different zoom level of
            // the proxy tile content, e.g. random POIs popping in/out when zooming out to city-wide view).
            if (tile.isProxy() && !prev_visible[label_id]) {
              continue;
            }

            var params = mesh.labels[label_id].container.label;
            var linked = mesh.labels[label_id].container.linked;
            var ranges = mesh.labels[label_id].ranges;
            var debug = Object.assign({}, mesh.labels[label_id].debug, {
              tile: tile,
              params: params,
              label_id: label_id
            });
            var label = labels[label_id] = {};
            label.discard = discard.bind(label);
            label.build_id = tile.build_id; // original order in which tiles were built

            Object.assign(label, params);
            label.layout = Object.assign({}, params.layout); // TODO: ideally remove need to copy props here

            label.layout.repeat_scale = 0.75; // looser second pass on repeat groups, to weed out repeats near tile edges

            label.layout.repeat_distance = label.layout.repeat_distance || 0;
            label.layout.repeat_distance /= size_scale; // TODO: where should this be scaled?

            label.position = [// don't overwrite referenced values
            label.position[0] / units_per_meter + tile.min.x, label.position[1] / units_per_meter + tile.min.y];
            label.unit_scale = meters_per_pixel;

            if (label.type === 'point') {
              // TODO: move to integer constants to avoid excess string copies
              __chunk_1.LabelPoint.prototype.updateBBoxes.call(label);
            } else if (label.type === 'straight') {
              __chunk_1.LabelLineStraight.prototype.updateBBoxes.call(label, label.position, label.size, label.angle, label.angle, label.offset);
            } else if (params.obbs) {
              // NB: this is a very rough approximation of curved label collision at intermediate zooms,
              // because the position/scale of each collision box isn't correctly updated; however,
              // it's good enough to provide some additional label coverage, with less overhead
              var obbs = params.obbs.map(function (o) {
                var x = o.x,
                    y = o.y,
                    a = o.a,
                    w = o.w,
                    h = o.h;
                x = x / units_per_meter + tile.min.x;
                y = y / units_per_meter + tile.min.y;
                w /= size_scale;
                h /= size_scale;
                return new __chunk_1.OBB(x, y, a, w, h);
              });
              label.obbs = obbs;
              label.aabbs = obbs.map(function (o) {
                return o.getExtent();
              });
            }

            containers[label_id] = {
              label: label,
              linked: linked,
              ranges: ranges,
              mesh: mesh,
              debug: debug
            };
          }
        }
      });
    }
  }); // Resolve links between label containers

  for (var c in containers) {
    var container = containers[c];

    if (container.linked) {
      container.linked = containers[container.linked];
    } // NB: if linked label not found, it was discarded in initial tile collision pass

  } // Convert container map to array


  containers = Object.keys(containers).map(function (k) {
    return containers[k];
  });
  return containers;
} // Generic discard function for labels, does simple occlusion with one or more bounding boxes
// (no additional logic to try alternate anchors or other layout options, etc.)


function discard(bboxes, exclude) {
  if (exclude === void 0) {
    exclude = null;
  }

  if (this.obb) {
    // single collision box
    return __chunk_1.Label.prototype.occluded.call(this, bboxes, exclude);
  } else if (this.obbs) {
    // mutliple collision boxes
    for (var i = 0; i < this.obbs.length; i++) {
      var aabb = this.aabbs[i];
      var obb = this.obbs[i];
      var obj = {
        aabb: aabb,
        obb: obb
      };
      var should_discard = __chunk_1.Label.prototype.occluded.call(obj, bboxes, exclude);

      if (should_discard) {
        return true;
      }
    }
  }

  return false;
}

var TileManager =
/*#__PURE__*/
function () {
  function TileManager(_ref) {
    var scene = _ref.scene;
    this.scene = scene;
    this.tiles = {};
    this.pyramid = new TilePyramid();
    this.visible_coords = {};
    this.queued_coords = [];
    this.building_tiles = null;
    this.renderable_tiles = [];
    this.collision = {
      tile_keys: null,
      mesh_set: null,
      zoom: null,
      zoom_steps: 3 // divisions per zoom at which labels are re-collided (e.g. 0, 0.33, 0.66)

    }; // Provide a hook for this object to be called from worker threads

    this.main_thread_target = ['TileManager', this.scene.id].join('_');
    __chunk_1.WorkerBroker.addTarget(this.main_thread_target, this);
  }

  var _proto = TileManager.prototype;

  _proto.destroy = function destroy() {
    this.forEachTile(function (tile) {
      return tile.destroy();
    });
    this.tiles = {};
    this.pyramid = null;
    this.visible_coords = {};
    this.queued_coords = [];
    this.scene = null;
    __chunk_1.WorkerBroker.removeTarget(this.main_thread_target);
  };

  _proto.keepTile = function keepTile(tile) {
    this.tiles[tile.key] = tile;
    this.pyramid.addTile(tile);
  };

  _proto.hasTile = function hasTile(key) {
    return this.tiles[key] !== undefined;
  };

  _proto.forgetTile = function forgetTile(key) {
    if (this.hasTile(key)) {
      var tile = this.tiles[key];
      this.pyramid.removeTile(tile);
    }

    delete this.tiles[key];
    this.tileBuildStop(key);
  } // Remove a single tile
  ;

  _proto.removeTile = function removeTile(key) {
    __chunk_1.log('trace', "tile unload for " + key);
    var tile = this.tiles[key];

    if (tile != null) {
      tile.destroy();
    }

    this.forgetTile(tile.key);
    this.scene.requestRedraw();
  } // Run a function on each tile
  ;

  _proto.forEachTile = function forEachTile(func) {
    for (var t in this.tiles) {
      func(this.tiles[t]);
    }
  } // Remove tiles that pass a filter condition
  ;

  _proto.removeTiles = function removeTiles(filter) {
    var remove_tiles = [];

    for (var t in this.tiles) {
      var tile = this.tiles[t];

      if (filter(tile)) {
        remove_tiles.push(t);
      }
    }

    for (var r = 0; r < remove_tiles.length; r++) {
      var key = remove_tiles[r];
      this.removeTile(key);
    }
  };

  _proto.updateTilesForView = function updateTilesForView() {
    // Find visible tiles and load new ones
    this.visible_coords = {};
    var tile_coords = this.view.findVisibleTileCoordinates();

    for (var c = 0; c < tile_coords.length; c++) {
      var coords = tile_coords[c];
      this.queueCoordinate(coords);
      this.visible_coords[coords.key] = coords;
    }

    this.updateTileStates();
  };

  _proto.updateTileStates = function updateTileStates() {
    var _this = this;

    this.forEachTile(function (tile) {
      _this.updateVisibility(tile);
    });
    this.loadQueuedCoordinates();
    this.updateProxyTiles();
    this.view.pruneTilesForView();
    this.updateRenderableTiles();
    this.style_manager.updateActiveStyles(this.renderable_tiles);
    this.style_manager.updateActiveBlendOrders(this.renderable_tiles);
    return this.updateLabels();
  };

  _proto.updateLabels = function updateLabels() {
    var _this2 = this;

    if (this.scene.building && !this.scene.building.initial) {
      // log('debug', `Skip label layout due to on-going scene rebuild`);
      return Promise.resolve({});
    } // get current visible tiles and sort by key for consistency collision order


    var tiles = this.renderable_tiles.filter(function (t) {
      return t.valid;
    }).filter(function (t) {
      return t.built;
    });

    if (tiles.length === 0) {
      return Promise.resolve({});
    } // Evaluate labels in order of tile build, to prevent previously visible labels
    // from disappearing, e.g. due to a newly loaded repeat label nearby


    tiles.sort(function (a, b) {
      return a.build_id < b.build_id ? -1 : a.build_id > b.build_id ? 1 : 0;
    }); // check if tile set has changed (in ways that affect collision)
    // if not, bail so that the existing collision task can carry on
    // if so, carry on and start a new collision task

    if ( // 1st: check if same zoom level (rounded to a configurable precision)
    this.collision.zoom === roundPrecision(this.view.zoom, this.collision.zoom_steps) && // 2nd: check if same set of tiles
    this.collision.tile_keys === JSON.stringify(tiles.map(function (t) {
      return t.key;
    })) && // 3rd: check if same set of meshes
    this.collision.mesh_set === meshSetString(tiles)) {
      // log('debug', `Skip label layout due to same tile/meshes (zoom ${this.view.zoom.toFixed(2)}, tiles ${this.collision.tile_keys})`);
      return Promise.resolve({});
    } // update collision if not already updating


    if (!this.collision.task) {
      this.collision.zoom = roundPrecision(this.view.zoom, this.collision.zoom_steps);
      this.collision.tile_keys = JSON.stringify(tiles.map(function (t) {
        return t.key;
      }));
      this.collision.mesh_set = meshSetString(tiles); // log('debug', `Update label collisions (zoom ${this.collision.zoom}, ${this.collision.tile_keys})`);
      // make a new collision task

      this.collision.task = {
        type: 'tileManagerUpdateLabels',
        run: function run(task) {
          return new Promise(function ($return, $error) {
            var results;
            return Promise.resolve(mainThreadLabelCollisionPass(tiles, _this2.collision.zoom, _this2.isLoadingVisibleTiles())).then(function ($await_1) {
              try {
                results = $await_1;

                _this2.scene.requestRedraw(); // Clear state to allow another collision pass to start


                _this2.collision.task = null;
                __chunk_1.Task.finish(task, results); // Check if tiles changed during previous collision pass - will start new pass if so

                _this2.updateTileStates();

                return $return();
              } catch ($boundEx) {
                return $error($boundEx);
              }
            }, $error);
          });
        }
      };
      __chunk_1.Task.add(this.collision.task);
    } // else {
    //     log('debug', `Skip label layout due to on-going layout (zoom ${this.view.zoom.toFixed(2)}, tiles ${this.collision.tile_keys})`);
    // }


    return this.collision.task.promise;
  };

  _proto.updateProxyTiles = function updateProxyTiles() {
    var _this3 = this;

    if (this.view.zoom_direction === 0) {
      return;
    } // Clear previous proxies


    this.forEachTile(function (tile) {
      return tile.setProxyFor(null);
    });
    var proxy = false;
    this.forEachTile(function (tile) {
      if (_this3.view.zoom_direction === 1) {
        if (tile.visible && !tile.labeled) {
          var parent = _this3.pyramid.getAncestor(tile);

          if (parent) {
            parent.setProxyFor(tile);
            proxy = true;
          }
        }
      } else if (_this3.view.zoom_direction === -1) {
        if (tile.visible && !tile.labeled) {
          var descendants = _this3.pyramid.getDescendants(tile);

          for (var i = 0; i < descendants.length; i++) {
            descendants[i].setProxyFor(tile);
            proxy = true;
          }
        }
      }
    });

    if (!proxy) {
      this.view.zoom_direction = 0;
    }
  };

  _proto.updateVisibility = function updateVisibility(tile) {
    tile.visible = false;

    if (tile.style_z === this.view.tile_zoom) {
      if (this.visible_coords[tile.coords.key]) {
        tile.visible = true;
      } else {
        // brute force
        for (var key in this.visible_coords) {
          if (__chunk_1.TileID.isDescendant(tile.coords, this.visible_coords[key])) {
            tile.visible = true;
            break;
          }
        }
      }
    }
  } // Remove tiles that aren't visible, and flag remaining visible ones to be updated (for loading, proxy, etc.)
  ;

  _proto.pruneToVisibleTiles = function pruneToVisibleTiles() {
    this.removeTiles(function (tile) {
      return !tile.visible;
    });
  };

  _proto.getRenderableTiles = function getRenderableTiles() {
    return this.renderable_tiles;
  };

  _proto.updateRenderableTiles = function updateRenderableTiles() {
    this.renderable_tiles = [];

    for (var t in this.tiles) {
      var tile = this.tiles[t];

      if (tile.visible && tile.loaded) {
        this.renderable_tiles.push(tile);
      }
    }

    return this.renderable_tiles;
  };

  _proto.isLoadingVisibleTiles = function isLoadingVisibleTiles() {
    var _this4 = this;

    return Object.keys(this.tiles).some(function (k) {
      return _this4.tiles[k].visible && !_this4.tiles[k].built;
    });
  };

  _proto.allVisibleTilesLabeled = function allVisibleTilesLabeled() {
    return this.renderable_tiles.every(function (t) {
      return t.labeled;
    });
  } // Queue a tile for load
  ;

  _proto.queueCoordinate = function queueCoordinate(coords) {
    this.queued_coords[this.queued_coords.length] = coords;
  } // Load all queued tiles
  ;

  _proto.loadQueuedCoordinates = function loadQueuedCoordinates() {
    var _this5 = this;

    if (this.queued_coords.length === 0) {
      return;
    } // Sort queued tiles from center tile


    this.queued_coords.sort(function (a, b) {
      var center = _this5.view.center.meters;
      var half_span = __chunk_1.Geo.metersPerTile(a.z) / 2;
      var ac = __chunk_1.Geo.metersForTile(a);
      ac.x += half_span;
      ac.y -= half_span;
      var bc = __chunk_1.Geo.metersForTile(b);
      bc.x += half_span;
      bc.y -= half_span;
      var ad = Math.abs(center.x - ac.x) + Math.abs(center.y - ac.y);
      var bd = Math.abs(center.x - bc.x) + Math.abs(center.y - bc.y);
      a.center_dist = ad;
      b.center_dist = bd;
      return bd > ad ? -1 : bd === ad ? 0 : 1;
    });
    this.queued_coords.forEach(function (coords) {
      return _this5.loadCoordinate(coords);
    });
    this.queued_coords = [];
  } // Load all tiles to cover a given logical tile coordinate
  ;

  _proto.loadCoordinate = function loadCoordinate(coords) {
    // Skip if not at current scene zoom
    if (coords.z !== this.view.center.tile.z) {
      return;
    } // Determine necessary tiles for each source


    for (var s in this.scene.sources) {
      var source = this.scene.sources[s]; // Check if data source should build this tile

      if (!source.builds_geometry_tiles || !source.includesTile(coords, this.view.tile_zoom)) {
        continue;
      }

      var key = __chunk_1.TileID.normalizedKey(coords, source, this.view.tile_zoom);

      if (key && !this.hasTile(key)) {
        __chunk_1.log('trace', "load tile " + key + ", distance from view center: " + coords.center_dist);
        var tile = new __chunk_1.Tile({
          source: source,
          coords: coords,
          workers: this.scene.workers,
          style_z: this.view.baseZoom(coords.z),
          view: this.view
        });
        this.keepTile(tile);
        this.buildTile(tile);
      }
    }
  } // Start tile build process
  ;

  _proto.buildTile = function buildTile(tile, options) {
    this.tileBuildStart(tile.key);
    this.updateVisibility(tile);
    tile.build(this.scene.generation, options);
  } // Called on main thread when a web worker completes processing for a single tile (initial load, or rebuild)
  ;

  _proto.buildTileStylesCompleted = function buildTileStylesCompleted(_ref2) {
    var tile = _ref2.tile,
        progress = _ref2.progress;

    // Removed this tile during load?
    if (this.tiles[tile.key] == null) {
      __chunk_1.log('trace', "discarded tile " + tile.key + " in TileManager.buildTileStylesCompleted because previously removed");
      __chunk_1.Tile.abortBuild(tile);
      this.updateTileStates();
    } // Built with an outdated scene configuration?
    else if (tile.generation !== this.scene.generation) {
        __chunk_1.log('trace', "discarded tile " + tile.key + " in TileManager.buildTileStylesCompleted because built with " + ("scene config gen " + tile.generation + ", current " + this.scene.generation));
        __chunk_1.Tile.abortBuild(tile);
        this.updateTileStates();
      } else {
        // Update tile with properties from worker
        if (this.tiles[tile.key]) {
          // Ignore if from a previously discarded tile
          if (tile.id < this.tiles[tile.key].id) {
            __chunk_1.log('trace', "discarded tile " + tile.key + " for id " + tile.id + " in TileManager.buildTileStylesCompleted because built for discarded tile id");
            __chunk_1.Tile.abortBuild(tile);
            return;
          }

          tile = this.tiles[tile.key].merge(tile);
        }

        if (progress.done) {
          tile.built = true;
        }

        tile.buildMeshes(this.scene.styles, progress);
        this.updateTileStates();
        this.scene.requestRedraw();
      }

    if (progress.done) {
      this.tileBuildStop(tile.key);
    }
  } // Called on main thread when web worker encounters an error building a tile
  ;

  _proto.buildTileError = function buildTileError(tile) {
    __chunk_1.log('error', "Error building tile " + tile.key + ":", tile.error);
    this.forgetTile(tile.key);
    __chunk_1.Tile.abortBuild(tile);
  } // Track tile build state
  ;

  _proto.tileBuildStart = function tileBuildStart(key) {
    this.building_tiles = this.building_tiles || {};
    this.building_tiles[key] = true;
    __chunk_1.log('trace', "tileBuildStart for " + key + ": " + Object.keys(this.building_tiles).length);
  };

  _proto.tileBuildStop = function tileBuildStop(key) {
    // Done building?
    if (this.building_tiles) {
      __chunk_1.log('trace', "tileBuildStop for " + key + ": " + Object.keys(this.building_tiles).length);
      delete this.building_tiles[key];
      this.checkBuildQueue();
    }
  } // Check status of tile building queue and notify scene when we're done
  ;

  _proto.checkBuildQueue = function checkBuildQueue() {
    if (!this.building_tiles || Object.keys(this.building_tiles).length === 0) {
      this.building_tiles = null;
      this.scene.tileManagerBuildDone();
    }
  } // Get a debug property across tiles
  ;

  _proto.getDebugProp = function getDebugProp(prop, filter) {
    var vals = [];

    for (var t in this.tiles) {
      if (this.tiles[t].debug[prop] != null && (typeof filter !== 'function' || filter(this.tiles[t]) === true)) {
        vals.push(this.tiles[t].debug[prop]);
      }
    }

    return vals;
  } // Sum of a debug property across tiles
  ;

  _proto.getDebugSum = function getDebugSum(prop, filter) {
    var sum = 0;

    for (var t in this.tiles) {
      if (this.tiles[t].debug[prop] != null && (typeof filter !== 'function' || filter(this.tiles[t]) === true)) {
        sum += this.tiles[t].debug[prop];
      }
    }

    return sum;
  } // Average of a debug property across tiles
  ;

  _proto.getDebugAverage = function getDebugAverage(prop, filter) {
    return this.getDebugSum(prop, filter) / Object.keys(this.tiles).length;
  };

  __chunk_1._createClass(TileManager, [{
    key: "view",
    get: function get() {
      return this.scene.view;
    }
  }, {
    key: "style_manager",
    get: function get() {
      return this.scene.style_manager;
    }
  }]);

  return TileManager;
}(); // Round a number to given number of decimal divisions

function roundPrecision(x, d, places) {
  if (places === void 0) {
    places = 2;
  }

  return (Math.floor(x * d) / d).toFixed(places);
} // Create a string representing the current set of meshes for a given set of tiles,
// based on their created timestamp. Used to determine when tiles should be re-collided.


function meshSetString(tiles) {
  return JSON.stringify(Object.entries(tiles).map(function (_ref3) {
    var t = _ref3[1];
    return Object.entries(t.meshes).map(function (_ref4) {
      var s = _ref4[1];
      return s.map(function (m) {
        return m.created_at;
      });
    });
  }));
}

var RenderState =
/*#__PURE__*/
function () {
  function RenderState(value, setup) {
    setup(value);
    this.value = value;
    this.setup = setup;
  }

  var _proto = RenderState.prototype;

  _proto.set = function set(value) {
    // if the states are different, call the GL context for a state change
    if (JSON.stringify(this.value) !== JSON.stringify(value)) {
      this.setup(value);
      this.value = value;
    }
  };

  return RenderState;
}();

var RenderStateManager = function RenderStateManager(gl) {
  this.defaults = {}; // Culling

  this.defaults.culling = true;
  this.defaults.culling_face = gl.BACK; // Blending

  this.defaults.blending = false;
  this.defaults.blending_src = gl.ONE_MINUS_SRC_ALPHA;
  this.defaults.blending_dst = gl.ONE_MINUS_SRC_ALPHA;
  this.defaults.blending_src_alpha = gl.ONE;
  this.defaults.blending_dst_alpha = gl.ONE_MINUS_SRC_ALPHA; // Depth test/write

  this.defaults.depth_write = true;
  this.defaults.depth_test = true;
  gl.depthFunc(gl.LESS); // depth function only needs to be set once
  // Culling

  this.culling = new RenderState({
    cull: this.defaults.culling,
    face: this.defaults.culling_face
  }, function (value) {
    if (value.cull) {
      gl.enable(gl.CULL_FACE);
      gl.cullFace(value.face);
    } else {
      gl.disable(gl.CULL_FACE);
    }
  }); // Blending mode

  this.blending = new RenderState({
    blend: this.defaults.blending,
    src: this.defaults.blending_src,
    dst: this.defaults.blending_dst,
    src_alpha: this.defaults.blending_src_alpha,
    dst_alpha: this.defaults.blending_dst_alpha
  }, function (value) {
    if (value.blend) {
      gl.enable(gl.BLEND);

      if (value.src_alpha && value.dst_alpha) {
        gl.blendFuncSeparate(value.src, value.dst, value.src_alpha, value.dst_alpha);
      } else {
        gl.blendFunc(value.src, value.dst);
      }
    } else {
      gl.disable(gl.BLEND);
    }
  }); // Depth write

  this.depth_write = new RenderState({
    depth_write: this.defaults.depth_write
  }, function (value) {
    gl.depthMask(value.depth_write);
  }); // Depth test

  this.depth_test = new RenderState({
    depth_test: this.defaults.depth_test
  }, function (value) {
    if (value.depth_test) {
      gl.enable(gl.DEPTH_TEST);
    } else {
      gl.disable(gl.DEPTH_TEST);
    }
  });
};

__chunk_1.require$$0$5('Uint8', 1, function (init) {
  return function Uint8ClampedArray(data, byteOffset, length) {
    return init(this, data, byteOffset, length);
  };
}, true);

var MediaCapture =
/*#__PURE__*/
function () {
  function MediaCapture() {
    this.canvas = null;
    this.gl = null;
    this.screenshot_canvas = null;
    this.screenshot_context = null;
    this.queue_screenshot = null;
    this.video_capture = null;
  }

  var _proto = MediaCapture.prototype;

  _proto.setCanvas = function setCanvas(canvas, gl) {
    this.canvas = canvas;
    this.gl = gl;
  } // Take a screenshot, returns a promise that resolves with the screenshot data when available
  // `background`: optional background color to blend screenshot with
  ;

  _proto.screenshot = function screenshot(_temp) {
    var _this = this;

    var _ref = _temp === void 0 ? {} : _temp,
        background = _ref.background;

    if (this.queue_screenshot != null) {
      return this.queue_screenshot.promise; // only capture one screenshot at a time
    } // Will resolve once rendering is complete and render buffer is captured


    this.queue_screenshot = {
      background: background
    };
    this.queue_screenshot.promise = new Promise(function (resolve, reject) {
      _this.queue_screenshot.resolve = resolve;
      _this.queue_screenshot.reject = reject;
    });
    return this.queue_screenshot.promise;
  } // Called after rendering, captures render buffer and resolves promise with the image data
  ;

  _proto.completeScreenshot = function completeScreenshot() {
    if (this.queue_screenshot != null) {
      // Firefox appears to have an issue where its alpha conversion overflows some channels when
      // the WebGL canvas content is captured. To get around this, we read pixels from the GL buffer
      // directly, then flip and unmulitply the alpha on each pixel to get the desired RGB values.
      // See https://github.com/tangrams/tangram/issues/551
      // Get raw pixels from GL
      var w = this.canvas.width;
      var h = this.canvas.height;
      var pixels = new Uint8Array(w * h * 4);
      this.gl.readPixels(0, 0, w, h, this.gl.RGBA, this.gl.UNSIGNED_BYTE, pixels); // Optional background to blend with (only RGB, alpha is ignored)

      var background = this.queue_screenshot.background;

      if (background && background !== 'transparent') {
        background = __chunk_1.StyleParser.parseColor(background).slice(0, 3).map(function (c) {
          return c * 255;
        });
      } else {
        background = null; // skip blend if transparent
      } // Flip Y (GL buffer is upside down)


      var flip = new Uint8ClampedArray(w * h * 4); // canvas requires 'clamped' array type

      for (var y = 0; y < h; y++) {
        for (var x = 0; x < w; x++) {
          var s = ((h - y - 1) * w + x) * 4; // source offset

          var d = (y * w + x) * 4; // destination offset

          var a = pixels[s + 3]; // unmultiply alpha

          flip[d + 0] = pixels[s + 0] * 255 / a;
          flip[d + 1] = pixels[s + 1] * 255 / a;
          flip[d + 2] = pixels[s + 2] * 255 / a;
          flip[d + 3] = a;

          if (background) {
            a /= 255;
            flip[d + 0] = flip[d + 0] * a + background[0] * (1 - a);
            flip[d + 1] = flip[d + 1] * a + background[1] * (1 - a);
            flip[d + 2] = flip[d + 2] * a + background[2] * (1 - a);
            flip[d + 3] = 255;
          }
        }
      } // Draw flipped pixels to a canvas


      this.screenshot_canvas = this.screenshot_canvas || document.createElement('canvas');
      var canvas = this.screenshot_canvas;
      canvas.width = w;
      canvas.height = h;
      this.screenshot_context = this.screenshot_context || canvas.getContext('2d');
      var ctx = this.screenshot_context;
      var image = new ImageData(flip, w, h);
      ctx.putImageData(image, 0, 0); // Get data URL from canvas and convert to blob
      // Strip host/mimetype/etc., convert base64 to binary without UTF-8 mangling
      // Adapted from: https://gist.github.com/unconed/4370822

      var url = canvas.toDataURL('image/png');
      var data = atob(url.slice(22));
      var buffer = new Uint8Array(data.length);

      for (var i = 0; i < data.length; ++i) {
        buffer[i] = data.charCodeAt(i);
      }

      var blob = new Blob([buffer], {
        type: 'image/png'
      }); // Resolve with screenshot data

      this.queue_screenshot.resolve({
        url: url,
        blob: blob,
        type: 'png'
      });
      this.queue_screenshot = null;
    }
  } // Starts capturing a video stream from the canvas
  ;

  _proto.startVideoCapture = function startVideoCapture() {
    var _this2 = this;

    if (typeof window.MediaRecorder !== 'function' || !this.canvas || typeof this.canvas.captureStream !== 'function') {
      __chunk_1.log('warn', 'Video capture (Canvas.captureStream and/or MediaRecorder APIs) not supported by browser');
      return false;
    } else if (this.video_capture) {
      __chunk_1.log('warn', 'Video capture already in progress, call Scene.stopVideoCapture() first');
      return false;
    } // Start a new capture


    try {
      var cap = this.video_capture = {};
      cap.chunks = [];
      cap.stream = this.canvas.captureStream();
      cap.options = {
        mimeType: 'video/webm'
      }; // TODO: support other format options

      cap.media_recorder = new MediaRecorder(cap.stream, cap.options);

      cap.media_recorder.ondataavailable = function (event) {
        if (event.data.size > 0) {
          cap.chunks.push(event.data);
        } // Stopped recording? Create the final capture file blob


        if (cap.resolve) {
          var blob = new Blob(cap.chunks, {
            type: cap.options.mimeType
          });
          var url = __chunk_1.createObjectURL(blob); // Explicitly remove all stream tracks, and set objects to null

          if (cap.stream) {
            var tracks = cap.stream.getTracks() || [];
            tracks.forEach(function (track) {
              track.stop();
              cap.stream.removeTrack(track);
            });
          }

          cap.stream = null;
          cap.media_recorder = null;
          _this2.video_capture = null;
          cap.resolve({
            url: url,
            blob: blob,
            type: 'webm'
          });
        }
      };

      cap.media_recorder.start();
    } catch (e) {
      this.video_capture = null;
      __chunk_1.log('error', 'Scene video capture failed', e);
      return false;
    }

    return true;
  } // Stops capturing a video stream from the canvas, returns a promise that resolves with the video when available
  ;

  _proto.stopVideoCapture = function stopVideoCapture() {
    var _this3 = this;

    if (!this.video_capture) {
      __chunk_1.log('warn', 'No scene video capture in progress, call Scene.startVideoCapture() first');
      return Promise.resolve({});
    } // Promise that will resolve when final stream is available


    this.video_capture.promise = new Promise(function (resolve, reject) {
      _this3.video_capture.resolve = resolve;
      _this3.video_capture.reject = reject;
    }); // Stop recording

    this.video_capture.media_recorder.stop();
    return this.video_capture.promise;
  };

  return MediaCapture;
}();

function setupSceneDebug(scene) {
  scene.debug = {
    // Profile helpers, issues a profile on main thread & all workers
    profile: function profile(name) {
      console.profile("main thread: " + name); // eslint-disable-line no-console

      __chunk_1.WorkerBroker.postMessage(scene.workers, 'self.profile', name);
    },
    profileEnd: function profileEnd(name) {
      console.profileEnd("main thread: " + name); // eslint-disable-line no-console

      __chunk_1.WorkerBroker.postMessage(scene.workers, 'self.profileEnd', name);
    },
    // Rebuild geometry a given # of times and print average, min, max timings
    timeRebuild: function timeRebuild(num, options) {
      if (num === void 0) {
        num = 1;
      }

      if (options === void 0) {
        options = {};
      }

      var times = [];

      var cycle = function cycle() {
        var start = +new Date();
        scene.rebuild(options).then(function () {
          times.push(+new Date() - start);

          if (times.length < num) {
            cycle();
          } else {
            var avg = ~~(times.reduce(function (a, b) {
              return a + b;
            }) / times.length);
            __chunk_1.log('info', "Profiled rebuild " + num + " times: " + avg + " avg (" + Math.min.apply(Math, times) + " min, " + Math.max.apply(Math, times) + " max)");
          }
        });
      };

      cycle();
    },
    // Return geometry counts of visible tiles, grouped by style name
    geometryCountByStyle: function geometryCountByStyle() {
      var counts = {};
      scene.tile_manager.getRenderableTiles().forEach(function (tile) {
        var _loop = function _loop(style) {
          counts[style] = counts[style] || 0;
          tile.meshes[style].forEach(function (mesh) {
            counts[style] += mesh.geometry_count;
          });
        };

        for (var style in tile.meshes) {
          _loop(style);
        }
      });
      return counts;
    },
    // Return geometry counts of visible tiles, grouped by base style name
    geometryCountByBaseStyle: function geometryCountByBaseStyle() {
      var style_counts = scene.debug.geometryCountByStyle();
      var counts = {};

      for (var style in style_counts) {
        var base = scene.styles[style].baseStyle();
        counts[base] = counts[base] || 0;
        counts[base] += style_counts[style];
      }

      return counts;
    },
    // Return sum of all geometry counts for visible tiles
    geometryCountTotal: function geometryCountTotal() {
      var styles = scene.debug.geometryCountByStyle();
      return Object.keys(styles).reduce(function (p, c) {
        return styles[c] + p;
      }, 0);
    },
    // Return geometry GL buffer sizes for visible tiles, grouped by style name
    geometrySizeByStyle: function geometrySizeByStyle() {
      var sizes = {};
      scene.tile_manager.getRenderableTiles().forEach(function (tile) {
        var _loop2 = function _loop2(style) {
          sizes[style] = sizes[style] || 0;
          tile.meshes[style].forEach(function (mesh) {
            sizes[style] += mesh.buffer_size;
          });
        };

        for (var style in tile.meshes) {
          _loop2(style);
        }
      });
      return sizes;
    },
    // Return geometry GL buffer sizes for visible tiles, grouped by base style name
    geometrySizeByBaseStyle: function geometrySizeByBaseStyle() {
      var style_sizes = scene.debug.geometrySizeByStyle();
      var sizes = {};

      for (var style in style_sizes) {
        var base = scene.styles[style].baseStyle();
        sizes[base] = sizes[base] || 0;
        sizes[base] += style_sizes[style];
      }

      return sizes;
    },
    // Return sum of all geometry GL buffer sizes for visible tiles
    geometrySizeTotal: function geometrySizeTotal() {
      var styles = scene.debug.geometrySizeByStyle();
      return Object.keys(styles).reduce(function (p, c) {
        return styles[c] + p;
      }, 0);
    },
    // Return sum of all texture memory usage
    textureSizeTotal: function textureSizeTotal() {
      return Object.values(__chunk_1.Texture.textures).map(function (t) {
        return t.byteSize();
      }).reduce(function (p, c) {
        return p + c;
      });
    },
    layerStats: function layerStats() {
      if (__chunk_1.debugSettings.layer_stats) {
        return __chunk_1.debugSumLayerStats(scene.tile_manager.getRenderableTiles());
      } else {
        __chunk_1.log('warn', 'Enable the \'layer_stats\' debug setting to collect layer stats');
        return {};
      }
    },
    renderableTilesCount: function renderableTilesCount() {
      return scene.tile_manager.getRenderableTiles().length;
    }
  };
}

var Scene =
/*#__PURE__*/
function () {
  function Scene(config_source, options) {
    options = options || {};
    __chunk_1.subscribeMixin(this);
    this.id = Scene.id++;
    this.initialized = false;
    this.initializing = null; // will be a promise that resolves when scene is loaded

    this.sources = {};
    this.view = new __chunk_1.View(this, options);
    this.tile_manager = new TileManager({
      scene: this
    });
    this.num_workers = options.numWorkers || 2;

    if (options.disableVertexArrayObjects === true) {
      __chunk_1.VertexArrayObject.disabled = true;
    }

    __chunk_1.Utils.use_high_density_display = options.highDensityDisplay !== undefined ? options.highDensityDisplay : true;
    __chunk_1.Utils.updateDevicePixelRatio();
    this.config = null;
    this.config_source = config_source;
    this.config_bundle = null;
    this.last_valid_config_source = null;
    this.styles = null;
    this.style_manager = new __chunk_1.StyleManager();
    this.building = null; // tracks current scene building state (tiles being built, etc.)

    this.dirty = true; // request a redraw

    if (options.preUpdate) {
      // optional pre-render loop hook
      this.subscribe({
        'pre_update': options.preUpdate
      });
    }

    if (options.postUpdate) {
      // optional post-render loop hook
      this.subscribe({
        'post_update': options.postUpdate
      });
    }

    this.render_loop = !options.disableRenderLoop; // disable render loop - app will have to manually call Scene.render() per frame

    this.render_loop_active = false;
    this.render_loop_stop = false;
    this.render_count = 0;
    this.last_render_count = 0;
    this.render_count_changed = false;
    this.frame = 0;
    this.last_main_render = -1; // frame counter for last main render pass

    this.last_selection_render = -1; // frame counter for last selection render pass

    this.media_capture = new MediaCapture();
    this.selection = null;
    this.selection_feature_count = 0;
    this.fetching_selection_map = null;
    this.prev_textures = null; // textures from previously loaded scene (used for cleanup)

    this.introspection = options.introspection === true ? true : false;
    this.times = {}; // internal time logs (mostly for dev/profiling)

    this.resetTime();
    this.container = options.container;
    this.canvas = null;
    this.contextOptions = options.webGLContextOptions;
    this.lights = null;
    this.background = null;
    this.createListeners();
    this.updating = 0;
    this.generation = Scene.generation; // an id that is incremented each time the scene config is invalidated

    this.last_complete_generation = Scene.generation; // last generation id with a complete view

    setupSceneDebug(this);
    this.log_level = options.logLevel || 'warn';
    __chunk_1.log.setLevel(this.log_level);
    __chunk_1.log.reset();
  }

  Scene.create = function create(config, options) {
    if (options === void 0) {
      options = {};
    }

    return new Scene(config, options);
  } // Load scene (or reload existing scene if no new source specified)
  // Options:
  //   `base_path`: base URL against which scene resources should be resolved (useful for Play) (default nulll)
  //   `blocking`: should rendering block on scene load completion (default true)
  ;

  var _proto = Scene.prototype;

  _proto.load = function load(config_source, options) {
    var _this = this;

    if (config_source === void 0) {
      config_source = null;
    }

    if (options === void 0) {
      options = {};
    }

    if (this.initializing) {
      return this.initializing;
    }

    __chunk_1.log.reset();
    this.updating++;
    this.initialized = false;
    this.view_complete = false; // track if a view complete event has been triggered yet

    this.times.frame = null; // clear first frame time

    this.times.build = null; // clear first scene build time
    // Backwards compatibilty for passing `base_path` string as second argument
    // (since transitioned to using options argument to accept more parameters)

    options = typeof options === 'string' ? {
      base_path: options
    } : options; // Should rendering block on load (not desirable for initial load, often desired for live style-switching)

    options.blocking = options.blocking !== undefined ? options.blocking : true;

    if (this.render_loop !== false) {
      this.setupRenderLoop();
    } // Load scene definition (sources, styles, etc.), then create styles & workers


    this.createCanvas();
    this.prev_textures = this.config && Object.keys(this.config.textures); // save textures from last scene

    this.initializing = this.loadScene(config_source, options).then(function () {
      return _this.createWorkers();
    }).then(function () {
      // Clean up resources from prior scene
      _this.destroyFeatureSelection();

      __chunk_1.WorkerBroker.postMessage(_this.workers, 'self.clearFunctionStringCache'); // Scene loaded from a JS object, or modified by a `load` event, may contain compiled JS functions
      // which need to be serialized, while one loaded only from a URL does not.

      var serialize_funcs = typeof _this.config_source === 'object' || _this.hasSubscribersFor('load');

      var updating = _this.updateConfig({
        serialize_funcs: serialize_funcs,
        normalize: false,
        loading: true,
        fade_in: true
      });

      if (options.blocking === true) {
        return updating;
      }
    }).then(function () {
      _this.freePreviousTextures();

      _this.updating--;
      _this.initializing = null;
      _this.initialized = true;
      _this.last_valid_config_source = _this.config_source;
      _this.last_valid_options = {
        base_path: options.base_path,
        file_type: options.file_type
      };

      _this.requestRedraw();
    }).catch(function (error) {
      _this.initializing = null;
      _this.updating = 0; // Report and revert to last valid config if available

      var type, message;

      if (error.name === 'YAMLException') {
        type = 'yaml';
        message = 'Error parsing scene YAML';
      } else {
        // TODO: more error types
        message = 'Error initializing scene';
      }

      _this.trigger('error', {
        type: type,
        message: message,
        error: error,
        url: _this.config_source
      });

      message = "Scene.load() failed to load " + JSON.stringify(_this.config_source) + ": " + error.message;

      if (_this.last_valid_config_source) {
        __chunk_1.log('warn', message, error);
        __chunk_1.log('info', 'Scene.load() reverting to last valid configuration');
        return _this.load(_this.last_valid_config_source, _this.last_valid_base_path);
      }

      __chunk_1.log('error', message, error);
      throw error;
    });
    return this.initializing;
  };

  _proto.destroy = function destroy() {
    this.initialized = false;
    this.render_loop_stop = true; // schedule render loop to stop

    this.destroyListeners();
    this.destroyFeatureSelection();

    if (this.canvas && this.canvas.parentNode) {
      this.canvas.parentNode.removeChild(this.canvas);
      this.canvas = null;
    }

    this.container = null;

    if (this.gl) {
      __chunk_1.Texture.destroy(this.gl);
      this.style_manager.destroy(this.gl);
      this.styles = {};
      __chunk_1.ShaderProgram.reset(); // Force context loss

      var ext = this.gl.getExtension('WEBGL_lose_context');

      if (ext) {
        ext.loseContext();
      }

      this.gl = null;
    }

    this.sources = {};
    this.destroyWorkers();
    this.tile_manager.destroy();
    this.tile_manager = null;
    __chunk_1.log.reset();
  };

  _proto.createCanvas = function createCanvas() {
    if (this.canvas) {
      return;
    }

    this.container = this.container || document.body;
    this.canvas = document.createElement('canvas');
    this.canvas.style.position = 'absolute';
    this.canvas.style.top = 0;
    this.canvas.style.left = 0; // Force tangram canvas underneath all leaflet layers, and set background to transparent

    this.container.style.backgroundColor = 'transparent';
    this.container.appendChild(this.canvas);

    try {
      this.gl = Context$1.getContext(this.canvas, Object.assign({
        alpha: true,
        premultipliedAlpha: true,
        stencil: true,
        device_pixel_ratio: __chunk_1.Utils.device_pixel_ratio
      }, this.contextOptions));
    } catch (e) {
      throw new Error('Couldn\'t create WebGL context. ' + 'Your browser may not support WebGL, or it\'s turned off? ' + 'Visit http://webglreport.com/ for more info.');
    }

    this.resizeMap(this.container.clientWidth, this.container.clientHeight);
    __chunk_1.VertexArrayObject.init(this.gl);
    this.render_states = new RenderStateManager(this.gl);
    this.media_capture.setCanvas(this.canvas, this.gl);
  } // Update list of any custom scripts (either at scene-level or data-source-level)
  ;

  _proto.updateExternalScripts = function updateExternalScripts() {
    var prev_scripts = [].concat(this.external_scripts || []); // save list of previously loaded scripts

    var scripts = []; // scene-level scripts

    if (this.config.scene.scripts) {
      for (var f in this.config.scene.scripts) {
        if (scripts.indexOf(this.config.scene.scripts[f]) === -1) {
          scripts.push(this.config.scene.scripts[f]);
        }
      }
    } // data-source-level scripts


    for (var s in this.config.sources) {
      var source = this.config.sources[s];

      if (source.scripts) {
        for (var _f in source.scripts) {
          if (scripts.indexOf(source.scripts[_f]) === -1) {
            scripts.push(source.scripts[_f]);
          }
        }
      }
    }

    this.external_scripts = scripts; // Scripts changed?

    return !(this.external_scripts.length === prev_scripts.length && this.external_scripts.every(function (v, i) {
      return v === prev_scripts[i];
    }));
  } // Web workers handle heavy duty tile construction: networking, geometry processing, etc.
  ;

  _proto.createWorkers = function createWorkers() {
    // Reset old workers (if any) if we need to re-instantiate with new external scripts
    if (this.updateExternalScripts()) {
      this.destroyWorkers();
    }

    if (!this.workers) {
      return this.makeWorkers();
    }

    return Promise.resolve();
  } // Instantiate workers from URL, init event handlers
  ;

  _proto.makeWorkers = function makeWorkers() {
    var _this2 = this;

    // Let VertexElements know if 32 bit indices for element arrays are available
    var has_element_index_uint = this.gl.getExtension('OES_element_index_uint') ? true : false;
    var queue = [];
    this.workers = [];

    var _loop = function _loop(id) {
      var worker = new Worker(Tangram.workerURL); // eslint-disable-line no-undef

      _this2.workers[id] = worker;
      __chunk_1.WorkerBroker.addWorker(worker);
      __chunk_1.log('debug', "Scene.makeWorkers: initializing worker " + id);
      var _id = id;
      queue.push(__chunk_1.WorkerBroker.postMessage(worker, 'self.init', _this2.id, id, _this2.num_workers, _this2.log_level, __chunk_1.Utils.device_pixel_ratio, has_element_index_uint, _this2.external_scripts).then(function (id) {
        __chunk_1.log('debug', "Scene.makeWorkers: initialized worker " + id);
        return id;
      }, function (error) {
        __chunk_1.log('error', "Scene.makeWorkers: failed to initialize worker " + _id + ":", error);
        return Promise.reject(error);
      }));
    };

    for (var id = 0; id < this.num_workers; id++) {
      _loop(id);
    }

    this.next_worker = 0;
    return Promise.all(queue).then(function () {
      __chunk_1.log.setWorkers(_this2.workers);
    });
  };

  _proto.destroyWorkers = function destroyWorkers() {
    this.selection = null; // selection needs to be re-initialized when workers are

    if (Array.isArray(this.workers)) {
      __chunk_1.log.setWorkers(null);
      this.workers.forEach(function (worker) {
        worker.terminate();
      });
      this.workers = null;
    }
  } // Scene is ready for rendering
  ;

  _proto.ready = function ready() {
    if (!this.view.ready() || Object.keys(this.sources).length === 0) {
      return false;
    }

    return true;
  } // Resize the map when device pixel ratio changes, e.g. when switching between displays
  ;

  _proto.updateDevicePixelRatio = function updateDevicePixelRatio() {
    var _this3 = this;

    if (__chunk_1.Utils.updateDevicePixelRatio()) {
      __chunk_1.WorkerBroker.postMessage(this.workers, 'self.updateDevicePixelRatio', __chunk_1.Utils.device_pixel_ratio).then(function () {
        return _this3.rebuild();
      }).then(function () {
        return _this3.resizeMap(_this3.view.size.css.width, _this3.view.size.css.height);
      });
    }
  };

  _proto.resizeMap = function resizeMap(width, height) {
    if (width === 0 && height === 0) {
      return;
    }

    this.dirty = true;
    this.view.setViewportSize(width, height);

    if (this.gl) {
      Context$1.resize(this.gl, width, height, __chunk_1.Utils.device_pixel_ratio);
    }
  } // Request scene be redrawn at next animation loop
  ;

  _proto.requestRedraw = function requestRedraw() {
    this.dirty = true;
  } // Redraw scene immediately - don't wait for animation loop
  // Use sparingly, but for cases where you need the closest possible sync with other UI elements,
  // such as other, non-WebGL map layers (e.g. Leaflet raster layers, markers, etc.)
  ;

  _proto.immediateRedraw = function immediateRedraw() {
    this.dirty = true;
    this.update();
  };

  _proto.renderLoop = function renderLoop() {
    this.render_loop_active = true; // only let the render loop instantiate once
    // Update and render the scene

    this.update(); // Pending background tasks

    __chunk_1.Task.setState({
      user_moving_view: this.view.user_input_active
    });
    __chunk_1.Task.processAll(); // Request the next frame if not scheduled to stop

    if (!this.render_loop_stop) {
      window.requestAnimationFrame(this.renderLoop.bind(this));
    } else {
      this.render_loop_stop = false;
      this.render_loop_active = false;
    }
  } // Setup the render loop
  ;

  _proto.setupRenderLoop = function setupRenderLoop() {
    var _this4 = this;

    if (!this.render_loop_active) {
      setTimeout(function () {
        _this4.renderLoop();
      }, 0); // delay start by one tick
    }
  };

  _proto.update = function update() {
    // Determine which passes (if any) to render
    var main = this.dirty;
    var selection = this.selection ? this.selection.hasPendingRequests() : false;
    var will_render = !(main === false && selection === false || this.initialized === false || this.updating > 0 || this.ready() === false); // Pre-render loop hook

    this.trigger('pre_update', will_render); // Update view (needs to update user input timer even if no render will occur)

    this.view.update(); // Bail if no need to render

    if (!will_render) {
      return false;
    }

    this.dirty = false; // subclasses can set this back to true when animation is needed
    // Render the scene

    this.updateDevicePixelRatio();
    this.render({
      main: main,
      selection: selection
    });
    this.updateViewComplete(); // fires event when rendered tile set or style changes

    this.media_capture.completeScreenshot(); // completes screenshot capture if requested
    // Post-render loop hook

    this.trigger('post_update', will_render); // Redraw every frame if animating

    if (this.animated === true || this.view.isAnimating()) {
      this.dirty = true;
    }

    this.frame++;
    __chunk_1.log('trace', 'Scene.render()');
    return true;
  } // Accepts flags indicating which render passes should be made
  ;

  _proto.render = function render(_ref) {
    var _this5 = this;

    var main = _ref.main,
        selection = _ref.selection;
    var gl = this.gl;
    this.updateBackground();
    Object.keys(this.lights).forEach(function (i) {
      return _this5.lights[i].update();
    }); // Render main pass

    this.render_count_changed = false;

    if (main) {
      this.render_count = this.renderPass();
      this.last_main_render = this.frame; // Update feature selection map if necessary

      if (this.render_count !== this.last_render_count) {
        this.render_count_changed = true;
        this.logFirstFrame();
        this.getFeatureSelectionMapSize().then(function (size) {
          _this5.selection_feature_count = size;
          __chunk_1.log('info', "Scene: rendered " + _this5.render_count + " primitives (" + size + " features in selection map)");
        });
      }

      this.last_render_count = this.render_count;
    } // Render selection pass (if needed)


    if (selection) {
      if (this.view.panning || this.view.user_input_active) {
        this.selection.clearPendingRequests();
        return;
      } // Only re-render if selection buffer is out of date (relative to main render buffer)
      // and not locked (e.g. no tiles are actively building)


      if (!this.selection.locked && this.last_selection_render < this.last_main_render) {
        this.selection.bind(); // switch to FBO

        this.renderPass('selection_program', // render w/alternate program
        {
          allow_blend: false
        }); // Reset to screen buffer

        gl.bindFramebuffer(gl.FRAMEBUFFER, null);
        gl.viewport(0, 0, this.canvas.width, this.canvas.height);
        gl.clearColor.apply(gl, this.background.computed_color); // restore scene background color

        this.last_selection_render = this.frame;
      }

      this.selection.read(); // process any pending results from selection buffer
    }

    return true;
  } // Render all active styles, grouped by blend/depth type (opaque, overlay, etc.) and by program (style)
  // Called both for main render pass, and for secondary passes like selection buffer
  ;

  _proto.renderPass = function renderPass(program_key, _temp) {
    var _this6 = this;

    if (program_key === void 0) {
      program_key = 'program';
    }

    var _ref2 = _temp === void 0 ? {} : _temp,
        allow_blend = _ref2.allow_blend;

    // optionally force alpha off (e.g. for selection pass)
    allow_blend = allow_blend == null ? true : allow_blend;
    this.clearFrame();
    var count = 0; // how many primitives were rendered

    var last_blend; // blend mode active in last render pass
    // Get sorted list of current blend orders, with accompanying list of styles to render for each

    var blend_orders = this.style_manager.getActiveBlendOrders();

    for (var _iterator = blend_orders, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
      var _ref3;

      if (_isArray) {
        if (_i >= _iterator.length) break;
        _ref3 = _iterator[_i++];
      } else {
        _i = _iterator.next();
        if (_i.done) break;
        _ref3 = _i.value;
      }

      var _ref4 = _ref3,
          blend_order = _ref4.blend_order,
          styles = _ref4.styles;

      var _loop2 = function _loop2(s) {
        var style = _this6.styles[styles[s]];

        if (style == null) {
          return "continue";
        } // Only update render state when blend mode changes


        if (style.blend !== last_blend) {
          var state = Object.assign({}, __chunk_1.Style.render_states[style.blend], // render state for blend mode
          {
            blend: allow_blend && style.blend // enable/disable blending (e.g. no blend for selection)

          });

          _this6.setRenderState(state);
        }

        var blend = allow_blend && style.blend;

        if (blend === 'translucent') {
          // Depth pre-pass for translucency
          _this6.gl.colorMask(false, false, false, false);

          _this6.renderStyle(style.name, program_key, blend_order);

          _this6.gl.colorMask(true, true, true, true);

          _this6.gl.depthFunc(_this6.gl.EQUAL); // Stencil buffer mask prevents overlap/flicker from compounding alpha of overlapping polys


          _this6.gl.enable(_this6.gl.STENCIL_TEST);

          _this6.gl.clearStencil(0);

          _this6.gl.clear(_this6.gl.STENCIL_BUFFER_BIT);

          _this6.gl.stencilFunc(_this6.gl.EQUAL, _this6.gl.ZERO, 0xFF);

          _this6.gl.stencilOp(_this6.gl.KEEP, _this6.gl.KEEP, _this6.gl.INCR); // Main render pass


          count += _this6.renderStyle(style.name, program_key, blend_order); // Disable translucency-specific settings

          _this6.gl.disable(_this6.gl.STENCIL_TEST);

          _this6.gl.depthFunc(_this6.gl.LESS);
        } else if (blend !== 'opaque' && style.stencil_proxy_tiles === true) {
          // Mask proxy tiles to with stencil buffer to avoid overlap/flicker from compounding alpha
          // Find unique levels of proxy tiles to render for this style
          var proxy_levels = _this6.tile_manager.getRenderableTiles().filter(function (t) {
            return t.meshes[style.name];
          }) // must have meshes for this style
          .map(function (t) {
            return t.proxy_level;
          }) // get the proxy depth
          .reduce(function (levels, level) {
            // count unique proxy depths
            levels.indexOf(level) > -1 || levels.push(level);
            return levels;
          }, []).sort(); // sort by lower depth first


          if (proxy_levels.length > 1) {
            // When there are multiple "levels" of tiles to render (e.g. non-proxy and one or more proxy
            // tile levels, or multiple proxy tile levels but no non-proxy tiles, etc.):
            // Render each proxy tile level to stencil buffer, masking each level such that it will not
            // render over any pixel rendered by a previous proxy tile level.
            _this6.gl.enable(_this6.gl.STENCIL_TEST);

            _this6.gl.clearStencil(0);

            _this6.gl.clear(_this6.gl.STENCIL_BUFFER_BIT);

            _this6.gl.stencilOp(_this6.gl.KEEP, _this6.gl.KEEP, _this6.gl.REPLACE);

            for (var i = 0; i < proxy_levels.length; i++) {
              // stencil test passes either for zero (not-yet-rendered),
              // or for other pixels at this proxy level (but not previous proxy levels)
              _this6.gl.stencilFunc(_this6.gl.GEQUAL, proxy_levels.length - i, 0xFF);

              count += _this6.renderStyle(style.name, program_key, blend_order, proxy_levels[i]);
            }

            _this6.gl.disable(_this6.gl.STENCIL_TEST);
          } else {
            // No special render handling needed when there are no proxy tiles,
            // or if there is ONLY a single proxy tile level (e.g. with no non-proxy tiles)
            count += _this6.renderStyle(style.name, program_key, blend_order);
          }
        } else {
          // Regular render pass (no special blend handling, or selection buffer pass)
          count += _this6.renderStyle(style.name, program_key, blend_order);
        }

        last_blend = style.blend;
      };

      // Render each style
      for (var s = 0; s < styles.length; s++) {
        var _ret = _loop2(s);

        if (_ret === "continue") continue;
      }
    }

    return count;
  };

  _proto.renderStyle = function renderStyle(style_name, program_key, blend_order, proxy_level) {
    var _this7 = this;

    if (proxy_level === void 0) {
      proxy_level = null;
    }

    var style = this.styles[style_name];
    var first_for_style = true; // TODO: allow this state to be passed in (for multilpe blend orders, stencil tests, etc)

    var render_count = 0;
    var program; // Render tile GL geometries

    var renderable_tiles = this.tile_manager.getRenderableTiles(); // For each tile, only include meshes for the blend order currently being rendered
    // Builds an array tiles and their associated meshes, each as a [tile, meshes] 2-element array

    var tile_meshes = renderable_tiles.filter(function (t) {
      return typeof proxy_level !== 'number' || t.proxy_level === proxy_level;
    }) // optional filter by proxy level
    .map(function (t) {
      if (t.meshes[style_name]) {
        return [t, t.meshes[style_name].filter(function (m) {
          return m.variant.blend_order === blend_order;
        })];
      }
    }).filter(function (x) {
      return x;
    }); // skip tiles with no meshes for this blend order
    // Mesh variants must be rendered in requested order across tiles, to prevent labels that cross
    // tile boundaries from rendering over adjacent tile features meant to be underneath

    var max_mesh_order = Math.max.apply(Math, tile_meshes.map(function (_ref5) {
      var meshes = _ref5[1];
      return Math.max.apply(Math, meshes.map(function (m) {
        return m.variant.mesh_order;
      }));
    })); // One pass per mesh variant order (loop goes to max value +1 because 0 is a valid order value)

    var _loop3 = function _loop3(mo) {
      var _loop5 = function _loop5() {
        if (_isArray2) {
          if (_i2 >= _iterator2.length) return "break";
          _ref6 = _iterator2[_i2++];
        } else {
          _i2 = _iterator2.next();
          if (_i2.done) return "break";
          _ref6 = _i2.value;
        }

        var _ref7 = _ref6,
            tile = _ref7[0],
            meshes = _ref7[1];
        var first_for_tile = true; // Skip proxy tiles if new tiles have finished loading this style

        if (!tile.shouldProxyForStyle(style_name)) {
          // log('trace', `Scene.renderStyle(): Skip proxy tile for style '${style_name}' `, tile, tile.proxy_for);
          return "continue";
        } // Filter meshes further by current variant order


        var order_meshes = meshes.filter(function (m) {
          return m.variant.mesh_order === mo;
        });

        if (order_meshes.length === 0) {
          return "continue";
        } // Style-specific state
        // Only setup style if rendering for first time this frame
        // (lazy init, not all styles will be used in all screen views; some styles might be defined but never used)


        if (first_for_style === true) {
          first_for_style = false;
          program = _this7.setupStyle(style, program_key);

          if (!program) {
            // no program found, e.g. happens when rendering selection pass, but style doesn't support selection
            return {
              v: {
                v: 0
              }
            };
          }
        } // Render each mesh (for current variant order)


        order_meshes.forEach(function (mesh) {
          // Tile-specific state
          if (first_for_tile === true) {
            first_for_tile = false;

            _this7.view.setupTile(tile, program);
          } // Render this mesh variant


          if (style.render(mesh)) {
            _this7.requestRedraw();
          }

          render_count += mesh.geometry_count;
        });
      };

      // Loop over tiles, with meshes pre-filtered by current blend order
      _loop4: for (var _iterator2 = tile_meshes, _isArray2 = Array.isArray(_iterator2), _i2 = 0, _iterator2 = _isArray2 ? _iterator2 : _iterator2[Symbol.iterator]();;) {
        var _ref6;

        var _ret3 = _loop5();

        switch (_ret3) {
          case "break":
            break _loop4;

          case "continue":
            continue;

          default:
            if (typeof _ret3 === "object") return _ret3.v;
        }
      }
    };

    for (var mo = 0; mo < max_mesh_order + 1; mo++) {
      var _ret2 = _loop3(mo);

      if (typeof _ret2 === "object") return _ret2.v;
    }

    return render_count;
  };

  _proto.setupStyle = function setupStyle(style, program_key) {
    // Get shader program from style, lazily compiling if necessary
    var program;

    try {
      program = style.getProgram(program_key);

      if (!program) {
        return;
      }
    } catch (error) {
      this.trigger('warning', {
        type: 'styles',
        message: "Error compiling style " + style.name,
        style: style,
        shader_errors: style.program && style.program.shader_errors
      });
      return;
    }

    program.use();
    style.setup();
    program.uniform('1f', 'u_time', this.animated ? (+new Date() - this.start_time) / 1000 : 0);
    this.view.setupProgram(program);

    for (var i in this.lights) {
      this.lights[i].setupProgram(program);
    }

    return program;
  };

  _proto.clearFrame = function clearFrame() {
    if (!this.initialized) {
      return;
    }

    this.render_states.depth_write.set({
      depth_write: true
    });
    this.gl.clear(this.gl.COLOR_BUFFER_BIT | this.gl.DEPTH_BUFFER_BIT | this.gl.STENCIL_BUFFER_BIT);
  };

  _proto.setRenderState = function setRenderState(_temp2) {
    var _ref8 = _temp2 === void 0 ? {} : _temp2,
        depth_test = _ref8.depth_test,
        depth_write = _ref8.depth_write,
        cull_face = _ref8.cull_face,
        blend = _ref8.blend;

    if (!this.initialized) {
      return;
    } // Defaults
    // TODO: when we abstract out support for multiple render passes, these can be per-pass config options


    var render_states = this.render_states;
    depth_test = depth_test === false ? false : render_states.defaults.depth_test; // default true

    depth_write = depth_write === false ? false : render_states.defaults.depth_write; // default true

    cull_face = cull_face === false ? false : render_states.defaults.culling; // default true

    blend = blend != null ? blend : render_states.defaults.blending; // default false
    // Reset frame state

    var gl = this.gl;
    render_states.depth_test.set({
      depth_test: depth_test
    });
    render_states.depth_write.set({
      depth_write: depth_write
    });
    render_states.culling.set({
      cull: cull_face,
      face: render_states.defaults.culling_face
    }); // Blending of alpha channel is modified to account for WebGL alpha behavior, see:
    // http://webglfundamentals.org/webgl/lessons/webgl-and-alpha.html
    // http://stackoverflow.com/a/11533416

    if (blend) {
      // Opaque: all source, no destination
      if (blend === 'opaque') {
        render_states.blending.set({
          blend: false
        });
      } // Traditional alpha blending
      else if (blend === 'overlay' || blend === 'inlay' || blend === 'translucent') {
          render_states.blending.set({
            blend: true,
            src: gl.SRC_ALPHA,
            dst: gl.ONE_MINUS_SRC_ALPHA,
            src_alpha: gl.ONE,
            dst_alpha: gl.ONE_MINUS_SRC_ALPHA
          });
        } // Additive blending
        else if (blend === 'add') {
            render_states.blending.set({
              blend: true,
              src: gl.ONE,
              dst: gl.ONE
            });
          } // Multiplicative blending
          else if (blend === 'multiply') {
              render_states.blending.set({
                blend: true,
                src: gl.ZERO,
                dst: gl.SRC_COLOR
              });
            }
    } else {
      render_states.blending.set({
        blend: false
      });
    }
  } // Request feature selection at given pixel. Runs async and returns results via a promise.
  ;

  _proto.getFeatureAt = function getFeatureAt(pixel, _temp3) {
    var _ref9 = _temp3 === void 0 ? {} : _temp3,
        radius = _ref9.radius;

    if (!this.initialized) {
      __chunk_1.log('debug', 'Scene.getFeatureAt() called before scene was initialized');
      return Promise.resolve();
    } // skip selection if no interactive features


    if (this.selection_feature_count === 0) {
      return Promise.resolve();
    } // only instantiate feature selection on-demand


    if (!this.selection) {
      this.resetFeatureSelection();
    } // Scale point and radius to [0..1] range


    var point = {
      x: pixel.x / this.view.size.css.width,
      y: pixel.y / this.view.size.css.height
    };

    if (radius > 0) {
      radius = {
        x: radius / this.view.size.css.width,
        y: radius / this.view.size.css.height
      };
    } else {
      radius = null;
    }

    return this.selection.getFeatureAt(point, {
      radius: radius
    }).then(function (selection) {
      return Object.assign(selection, {
        pixel: pixel
      });
    }).catch(function (error) {
      return Promise.resolve({
        error: error
      });
    });
  } // Query features within visible tiles, with optional filter conditions
  ;

  _proto.queryFeatures = function queryFeatures(_temp4) {
    return new Promise(function ($return, $error) {
      var _ref10, filter, _ref10$unique, unique, _ref10$group_by, group_by, _ref10$visible, visible, _ref10$geometry, geometry, uniqueify, group, tile_keys, results, features, keys, groups;

      _ref10 = _temp4 === void 0 ? {} : _temp4, filter = _ref10.filter, _ref10$unique = _ref10.unique, unique = _ref10$unique === void 0 ? true : _ref10$unique, _ref10$group_by = _ref10.group_by, group_by = _ref10$group_by === void 0 ? null : _ref10$group_by, _ref10$visible = _ref10.visible, visible = _ref10$visible === void 0 ? null : _ref10$visible, _ref10$geometry = _ref10.geometry, geometry = _ref10$geometry === void 0 ? false : _ref10$geometry;

      if (!this.initialized) {
        return $return([]);
      }

      filter = __chunk_1.Utils.serializeWithFunctions(filter); // Optional uniqueify criteria
      // Valid values: true, false/null, single property name, or array of property names

      unique = typeof unique === 'string' ? [unique] : unique;

      uniqueify = unique && function (obj) {
        var props = Array.isArray(unique) ? __chunk_1.sliceObject(obj.properties, unique) : obj.properties;

        if (geometry) {
          // when `geometry` flag is set, we need to uniqueify based on *both* feature properties and geometry
          return JSON.stringify({
            geometry: obj.geometry,
            properties: props
          });
        }

        return JSON.stringify(props);
      };

      // Optional grouping criteria
      // Valid values: false/null, single property name, or array of property names
      group_by = (typeof group_by === 'string' || Array.isArray(group_by)) && group_by;

      group = group_by && function (obj) {
        return Array.isArray(group_by) ? JSON.stringify(__chunk_1.sliceObject(obj, group_by)) : obj[group_by];
      };

      tile_keys = this.tile_manager.getRenderableTiles().map(function (t) {
        return t.key;
      });
      return Promise.resolve(__chunk_1.WorkerBroker.postMessage(this.workers, 'self.queryFeatures', {
        filter: filter,
        visible: visible,
        geometry: geometry,
        tile_keys: tile_keys
      })).then(function ($await_4) {
        try {
          results = $await_4;
          features = [];
          keys = {};
          groups = {};
          results.forEach(function (r) {
            return r.forEach(function (feature) {
              if (uniqueify) {
                var str = uniqueify(feature);

                if (keys[str]) {
                  return;
                }

                keys[str] = true;
              }

              if (group) {
                var _str = group(feature.properties);

                groups[_str] = groups[_str] || [];

                groups[_str].push(feature);
              } else {
                features.push(feature);
              }
            });
          });
          return $return(group ? groups : features); // returned grouped results, or all results
        } catch ($boundEx) {
          return $error($boundEx);
        }
      }, $error);
    }.bind(this));
  } // Rebuild all tiles, without re-parsing the config or re-compiling styles
  // sync: boolean of whether to sync the config object to the worker
  // sources: optional array of data sources to selectively rebuild (by default all our rebuilt)
  ;

  _proto.rebuild = function rebuild(_temp5) {
    var _this8 = this;

    var _ref11 = _temp5 === void 0 ? {} : _temp5,
        _ref11$initial = _ref11.initial,
        initial = _ref11$initial === void 0 ? false : _ref11$initial,
        _ref11$new_generation = _ref11.new_generation,
        new_generation = _ref11$new_generation === void 0 ? true : _ref11$new_generation,
        _ref11$sources = _ref11.sources,
        sources = _ref11$sources === void 0 ? null : _ref11$sources,
        serialize_funcs = _ref11.serialize_funcs,
        _ref11$profile = _ref11.profile,
        profile = _ref11$profile === void 0 ? false : _ref11$profile,
        _ref11$fade_in = _ref11.fade_in,
        fade_in = _ref11$fade_in === void 0 ? false : _ref11$fade_in;

    return new Promise(function (resolve, reject) {
      // Skip rebuild if already in progress
      if (_this8.building) {
        // Queue up to one rebuild call at a time, only save last request
        if (_this8.building.queued && _this8.building.queued.reject) {
          // notify previous request that it did not complete
          __chunk_1.log('debug', 'Scene.rebuild: request superceded by a newer call');

          _this8.building.queued.resolve(false); // false flag indicates rebuild request was superceded

        } // Save queued request


        var options = {
          initial: initial,
          new_generation: new_generation,
          sources: sources,
          serialize_funcs: serialize_funcs,
          profile: profile,
          fade_in: fade_in
        };
        _this8.building.queued = {
          resolve: resolve,
          reject: reject,
          options: options
        };
        __chunk_1.log('trace', 'Scene.rebuild(): queuing request');
        return;
      } // Track tile build state


      _this8.building = {
        resolve: resolve,
        reject: reject,
        initial: initial
      }; // Profiling

      if (profile) {
        _this8.debug.profile('Scene.rebuild');
      } // Increment generation to ensure style/tile building stay in sync
      // (skipped if calling function already incremented)


      if (new_generation) {
        _this8.generation = ++Scene.generation;

        for (var style in _this8.styles) {
          _this8.styles[style].setGeneration(_this8.generation);
        }
      } // Update config (in case JS objects were manipulated directly)


      _this8.syncConfigToWorker({
        serialize_funcs: serialize_funcs
      });

      _this8.resetWorkerFeatureSelection(sources);

      _this8.resetTime(); // Rebuild visible tiles


      _this8.tile_manager.pruneToVisibleTiles();

      _this8.tile_manager.forEachTile(function (tile) {
        if (!sources || sources.indexOf(tile.source.name) > -1) {
          _this8.tile_manager.buildTile(tile, {
            fade_in: fade_in
          });
        }
      });

      _this8.tile_manager.updateTilesForView(); // picks up additional tiles for any new/changed data sources


      _this8.tile_manager.checkBuildQueue(); // resolve immediately if no tiles to build

    }).then(function () {
      // Profiling
      if (profile) {
        _this8.debug.profileEnd('Scene.rebuild');
      }
    });
  } // Tile manager finished building tiles
  // TODO move to tile manager
  ;

  _proto.tileManagerBuildDone = function tileManagerBuildDone() {
    __chunk_1.TextCanvas.pruneTextCache();

    if (this.building) {
      __chunk_1.log('info', 'Scene: build geometry finished');

      if (this.building.resolve) {
        this.logFirstBuild();
        this.building.resolve(true);
      } // Another rebuild queued?


      var queued = this.building.queued;
      this.building = null;

      if (queued) {
        __chunk_1.log('debug', 'Scene: starting queued rebuild() request');
        this.rebuild(queued.options).then(queued.resolve, queued.reject);
      } else {
        this.tile_manager.updateLabels(); // refresh label if nothing to rebuild
      }
    }
  }
  /**
     Load (or reload) the scene config
     @return {Promise}
  */
  ;

  _proto.loadScene = function loadScene(config_source, _temp6) {
    var _this9 = this;

    if (config_source === void 0) {
      config_source = null;
    }

    var _ref12 = _temp6 === void 0 ? {} : _temp6,
        base_path = _ref12.base_path,
        file_type = _ref12.file_type;

    this.config_source = config_source || this.config_source;

    if (typeof this.config_source === 'string') {
      this.base_path = __chunk_1.pathForURL(base_path || this.config_source);
    } else {
      this.base_path = __chunk_1.pathForURL(base_path);
    } // backwards compatibility for accessing base path under previous name
    // TODO: schedule for deprecation


    this.config_path = this.base_path;
    return SceneLoader$1.loadScene(this.config_source, {
      path: this.base_path,
      type: file_type
    }).then(function (_ref13) {
      var config = _ref13.config,
          bundle = _ref13.bundle;
      _this9.config = config;
      _this9.config_bundle = bundle;
      return _this9.config;
    });
  } // Add source to a scene, arguments `name` and `config` need to be provided:
  //  - If the name doesn't match a sources it will create it
  //  - the `config` obj follow the YAML scene spec, ex: ```{type: 'TopoJSON', url: "//tile.mapzen.com/mapzen/vector/v1/all/{z}/{x}/{y}.topojson"]}```
  //    that looks like:
  //
  //      scene.setDataSource("osm", {type: 'TopoJSON', url: "//tile.mapzen.com/mapzen/vector/v1/all/{z}/{x}/{y}.topojson" });
  //
  //  - also can be pass a ```data``` obj: ```{type: 'GeoJSON', data: JSObj ]}```
  //
  //      var geojson_data = {};
  //      ...
  //      scene.setDataSource("dynamic_data", {type: 'GeoJSON', data: geojson_data });
  //
  ;

  _proto.setDataSource = function setDataSource(name, config) {
    if (!name || !config || !config.type || !config.url && !config.data) {
      __chunk_1.log('error', 'No name provided or not a valid config:', name, config);
      return;
    }

    var load = this.config.sources[name] == null;
    var source = this.config.sources[name] = Object.assign({}, config); // Convert raw data into blob URL

    if (source.data && typeof source.data === 'object') {
      source.url = __chunk_1.createObjectURL(new Blob([JSON.stringify(source.data)]));
      delete source.data;
    }

    if (load) {
      return this.updateConfig({
        rebuild: {
          sources: [name]
        }
      });
    } else {
      return this.rebuild({
        sources: [name]
      });
    }
  } // (Re-)create all data sources. Re-layout view and rebuild tiles when either:
  // 1) all tiles if `rebuild_all` parameter is specified (used when loading a new scene)
  // 2) the data source has changed in a way that affects tile layout (e.g. tile size, max_zoom, etc.)
  ;

  _proto.createDataSources = function createDataSources(rebuild_all) {
    var _this10 = this;

    if (rebuild_all === void 0) {
      rebuild_all = false;
    }

    var reset = []; // sources to reset

    var prev_source_names = Object.keys(this.sources);
    var source_id = 0;

    for (var name in this.config.sources) {
      var source = this.config.sources[name];
      var prev_source = this.sources[name];

      try {
        var config = Object.assign({}, source, {
          name: name,
          id: source_id++
        });
        this.sources[name] = __chunk_1.DataSource.create(config, this.sources);

        if (!this.sources[name]) {
          throw {};
        }
      } catch (e) {
        delete this.sources[name];
        var message = "Could not create data source: " + e.message;
        __chunk_1.log('warn', "Scene: " + message, source);
        this.trigger('warning', {
          type: 'sources',
          source: source,
          message: message
        });
      } // Data source changed in a way that affects tile layout?
      // If so, we'll re-calculate the tiles in view for this source and rebuild them


      if (rebuild_all || __chunk_1.DataSource.tileLayoutChanged(this.sources[name], prev_source)) {
        reset.push(name);
      }
    } // Sources that were removed


    prev_source_names.forEach(function (s) {
      if (!_this10.config.sources[s]) {
        delete _this10.sources[s]; // TODO: remove from workers too?

        reset.push(s);
      }
    }); // Remove tiles from sources that have changed

    if (reset.length > 0) {
      this.tile_manager.removeTiles(function (tile) {
        return reset.indexOf(tile.source.name) > -1;
      });
    } // Mark sources that will generate geometry tiles (any that are referenced in scene layers)


    for (var ln in this.config.layers) {
      var layer = this.config.layers[ln];

      if (layer.enabled !== false && layer.data && this.sources[layer.data.source]) {
        this.sources[layer.data.source].builds_geometry_tiles = true;
      }
    }
  } // Load all textures in the scene definition
  ;

  _proto.loadTextures = function loadTextures() {
    var _this11 = this;

    return __chunk_1.Texture.createFromObject(this.gl, this.config.textures).then(function () {
      return __chunk_1.Texture.createDefault(_this11.gl);
    }); // create a 'default' texture for placeholders
  } // Free textures from previously loaded scene
  ;

  _proto.freePreviousTextures = function freePreviousTextures() {
    var _this12 = this;

    if (!this.prev_textures) {
      return;
    }

    this.prev_textures.forEach(function (t) {
      // free textures that aren't in the new scene, but are still in the global texture set
      if (!_this12.config.textures[t] && __chunk_1.Texture.textures[t]) {
        __chunk_1.Texture.textures[t].destroy();
      }
    });
    this.prev_textures = null;
  } // Called (currently manually) after styles are updated in stylesheet
  ;

  _proto.updateStyles = function updateStyles() {
    if (!this.initialized && !this.initializing) {
      throw new Error('Scene.updateStyles() called before scene was initialized');
    } // (Re)build styles from config


    this.styles = this.style_manager.build(this.config.styles);
    this.style_manager.initStyles(this); // Optionally set GL context (used when initializing or re-initializing GL resources)

    for (var style in this.styles) {
      this.styles[style].setGL(this.gl);
    }

    this.dirty = true;
  } // Is scene currently animating?
  ;

  // Get active camera - for public API
  _proto.getActiveCamera = function getActiveCamera() {
    return this.view.getActiveCamera();
  } // Set active camera - for public API
  ;

  _proto.setActiveCamera = function setActiveCamera(name) {
    return this.view.setActiveCamera(name);
  } // Create lighting
  ;

  _proto.createLights = function createLights() {
    this.lights = {};

    for (var i in this.config.lights) {
      if (!this.config.lights[i] || typeof this.config.lights[i] !== 'object') {
        continue;
      }

      var light = this.config.lights[i];
      light.name = i.replace('-', '_'); // light names are injected in shaders, can't have hyphens

      light.visible = light.visible === false ? false : true;

      if (light.visible) {
        this.lights[light.name] = __chunk_1.Light.create(this.view, light);
      }
    }

    __chunk_1.Light.inject(this.lights);
  } // Set background color from scene config
  ;

  _proto.setBackground = function setBackground() {
    var bg = this.config.scene.background;
    this.background = {};

    if (bg && bg.color) {
      this.background.color = __chunk_1.StyleParser.createColorPropertyCache(bg.color);
    }

    if (!this.background.color) {
      this.background.color = __chunk_1.StyleParser.createColorPropertyCache([0, 0, 0, 0]); // default background TODO: vary w/scene alpha
    }
  } // Update background color each frame as needed (e.g. may be zoom-interpolated)
  ;

  _proto.updateBackground = function updateBackground() {
    var last_color = this.background.computed_color;
    var color = this.background.computed_color = __chunk_1.StyleParser.evalCachedColorProperty(this.background.color, {
      zoom: this.view.tile_zoom
    }); // update GL/canvas if color has changed

    if (!last_color || color.some(function (v, i) {
      return last_color[i] !== v;
    })) {
      var _this$gl;

      // if background is fully opaque, set canvas background to match
      if (color[3] === 1) {
        this.canvas.style.backgroundColor = "rgba(" + color.map(function (c) {
          return Math.floor(c * 255);
        }).join(', ') + ")";
      } else {
        this.canvas.style.backgroundColor = 'transparent';
      }

      (_this$gl = this.gl).clearColor.apply(_this$gl, color);
    }
  } // Turn introspection mode on/off
  ;

  _proto.setIntrospection = function setIntrospection(val) {
    var _this13 = this;

    if (val !== this.introspection) {
      this.introspection = val != null ? val : false;
      this.updating++;
      return this.updateConfig({
        normalize: false
      }).then(function () {
        return _this13.updating--;
      });
    }

    return Promise.resolve();
  } // Update scene config, and optionally rebuild geometry
  // rebuild can be boolean, or an object containing rebuild options to passthrough
  ;

  _proto.updateConfig = function updateConfig(_temp7) {
    var _this14 = this;

    var _ref14 = _temp7 === void 0 ? {} : _temp7,
        _ref14$loading = _ref14.loading,
        loading = _ref14$loading === void 0 ? false : _ref14$loading,
        _ref14$rebuild = _ref14.rebuild,
        rebuild = _ref14$rebuild === void 0 ? true : _ref14$rebuild,
        serialize_funcs = _ref14.serialize_funcs,
        _ref14$normalize = _ref14.normalize,
        normalize = _ref14$normalize === void 0 ? true : _ref14$normalize,
        _ref14$fade_in = _ref14.fade_in,
        fade_in = _ref14$fade_in === void 0 ? false : _ref14$fade_in;

    this.generation = ++Scene.generation;
    this.updating++;
    this.config = SceneLoader$1.applyGlobalProperties(this.config);

    if (normalize) {
      // normalize whole scene
      SceneLoader$1.normalize(this.config, this.config_bundle);
    } else {
      // special handling for shader uniforms that are globals
      SceneLoader$1.hoistStyleShaderUniformTextures(this.config, this.config_bundle, {
        include_globals: true
      }); // just normalize top-level textures - necessary for adding base path to globals

      SceneLoader$1.normalizeTextures(this.config, this.config_bundle);
    }

    this.trigger(loading ? 'load' : 'update', {
      config: this.config
    });
    this.style_manager.init();
    this.view.reset();
    this.createLights();
    this.createDataSources(loading);
    this.loadTextures();
    this.setBackground();
    __chunk_1.FontManager.loadFonts(this.config.fonts); // TODO: detect changes to styles? already (currently) need to recompile anyway when camera or lights change

    this.updateStyles(); // Optionally rebuild geometry

    var done = rebuild ? this.rebuild(Object.assign({
      initial: loading,
      new_generation: false,
      serialize_funcs: serialize_funcs,
      fade_in: fade_in
    }, typeof rebuild === 'object' && rebuild)) : this.syncConfigToWorker({
      serialize_funcs: serialize_funcs
    }); // rebuild() also syncs config
    // Finish by updating bounds and re-rendering

    this.updating--;
    this.view.updateBounds();
    this.requestRedraw();
    return done.then(function () {
      _this14.last_render_count = 0; // force re-evaluation of selection map

      _this14.requestRedraw();
    });
  } // Serialize config and send to worker
  ;

  _proto.syncConfigToWorker = function syncConfigToWorker(_temp8) {
    var _ref15 = _temp8 === void 0 ? {} : _temp8,
        _ref15$serialize_func = _ref15.serialize_funcs,
        serialize_funcs = _ref15$serialize_func === void 0 ? true : _ref15$serialize_func;

    // Tell workers we're about to rebuild (so they can update styles, etc.)
    var config_serialized = serialize_funcs ? __chunk_1.Utils.serializeWithFunctions(this.config) : JSON.stringify(this.config);
    return __chunk_1.WorkerBroker.postMessage(this.workers, 'self.updateConfig', {
      config: config_serialized,
      generation: this.generation,
      introspection: this.introspection
    }, __chunk_1.debugSettings);
  } // Listen to related objects
  ;

  _proto.createListeners = function createListeners() {
    var _this15 = this;

    this.listeners = {};
    this.listeners.view = {
      move: function move() {
        return _this15.trigger('move');
      }
    };
    this.view.subscribe(this.listeners.view);
    this.listeners.texture = {
      update: function update() {
        return _this15.dirty = true;
      },
      warning: function warning(data) {
        return _this15.trigger('warning', Object.assign({
          type: 'textures'
        }, data));
      }
    };
    __chunk_1.Texture.subscribe(this.listeners.texture);
    this.listeners.scene_loader = {
      error: function error(data) {
        return _this15.trigger('error', Object.assign({
          type: 'scene'
        }, data));
      }
    };
    SceneLoader$1.subscribe(this.listeners.scene_loader);
  };

  _proto.destroyListeners = function destroyListeners() {
    this.unsubscribeAll();
    this.view.unsubscribe(this.listeners.view);
    __chunk_1.Texture.unsubscribe(this.listeners.texture);
    SceneLoader$1.unsubscribe(this.listeners.scene_loader);
    this.listeners = null;
  };

  _proto.destroyFeatureSelection = function destroyFeatureSelection() {
    if (this.selection) {
      this.selection.destroy();
      this.selection = null;
    }
  };

  _proto.resetFeatureSelection = function resetFeatureSelection() {
    var _this16 = this;

    this.selection = new __chunk_1.FeatureSelection(this.gl, this.workers, function () {
      return _this16.building;
    });
    this.last_render_count = 0; // force re-evaluation of selection map
  };

  _proto.resetWorkerFeatureSelection = function resetWorkerFeatureSelection(sources) {
    if (sources === void 0) {
      sources = null;
    }

    if (this.workers) {
      __chunk_1.WorkerBroker.postMessage(this.workers, 'self.resetFeatureSelection', sources);
    }
  } // Gets the current feature selection map size across all workers. Returns a promise.
  ;

  _proto.getFeatureSelectionMapSize = function getFeatureSelectionMapSize() {
    var _this17 = this;

    // Only allow one fetch process to run at a time
    if (this.fetching_selection_map == null) {
      this.fetching_selection_map = __chunk_1.WorkerBroker.postMessage(this.workers, 'self.getFeatureSelectionMapSize').then(function (sizes) {
        _this17.fetching_selection_map = null;
        return sizes.reduce(function (a, b) {
          return a + b;
        });
      });
    }

    return this.fetching_selection_map;
  } // Reset internal clock, mostly useful for consistent experience when changing styles/debugging
  ;

  _proto.resetTime = function resetTime() {
    this.start_time = +new Date();
  } // Fires event when rendered tile set or style changes
  ;

  _proto.updateViewComplete = function updateViewComplete() {
    if ((this.render_count_changed || this.generation !== this.last_complete_generation) && !this.building && !this.tile_manager.isLoadingVisibleTiles() && this.tile_manager.allVisibleTilesLabeled()) {
      this.tile_manager.updateLabels();
      this.last_complete_generation = this.generation;
      this.trigger('view_complete', {
        first: this.view_complete !== true
      });
      this.view_complete = true;
    }
  };

  _proto.resetViewComplete = function resetViewComplete() {
    this.last_complete_generation = null;
  } // Take a screenshot
  // Asynchronous because we have to wait for next render to capture buffer
  // Returns a promise
  ;

  _proto.screenshot = function screenshot(_temp9) {
    var _ref16 = _temp9 === void 0 ? {} : _temp9,
        _ref16$background = _ref16.background,
        background = _ref16$background === void 0 ? 'white' : _ref16$background;

    this.requestRedraw();
    return this.media_capture.screenshot({
      background: background
    });
  };

  _proto.startVideoCapture = function startVideoCapture() {
    this.requestRedraw();
    return this.media_capture.startVideoCapture();
  };

  _proto.stopVideoCapture = function stopVideoCapture() {
    return this.media_capture.stopVideoCapture();
  } // Log first frame rendered (with any geometry)
  ;

  _proto.logFirstFrame = function logFirstFrame() {
    if (this.last_render_count === 0 && !this.times.first_frame) {
      this.times.first_frame = +new Date() - this.start_time;
      __chunk_1.log('debug', "Scene: initial frame time: " + this.times.first_frame);
    }
  } // Log completion of first scene build
  ;

  _proto.logFirstBuild = function logFirstBuild() {
    if (this.times.first_build == null) {
      this.times.first_build = +new Date() - this.start_time;
      __chunk_1.log('debug', "Scene: initial build time: " + this.times.first_build);
    }
  };

  __chunk_1._createClass(Scene, [{
    key: "animated",
    get: function get() {
      var _this18 = this;

      // Disable animation is scene flag requests it, otherwise enable animation if any animated styles are in view
      return this.config.scene.animated === false ? false : this.style_manager.getActiveStyles().some(function (s) {
        return _this18.styles[s].animated;
      });
    }
  }]);

  return Scene;
}();
Scene.id = 0; // unique id for a scene instance

Scene.generation = 0; // id that is incremented each time a scene config is re-parsed

// Debounce a function
// https://davidwalsh.name/javascript-debounce-function
function debounce(func, wait) {
  var timeout;
  return function () {
    var context = this,
        args = arguments;

    var later = function later() {
      timeout = null;
      func.apply(context, args);
    };

    clearTimeout(timeout);
    timeout = setTimeout(later, wait);
  };
}

var LeafletLayer;
function leafletLayer(options) {
  return extendLeaflet(options);
} // save references to overloaded Leaflet methods

var originalHandlers = {
  map: {},
  scrollWheelZoom: {},
  doubleClickZoom: {}
};

function extendLeaflet(options) {
  // If LeafletLayer is already defined when this is called just return that immediately
  // e.g. if you call leafletLayer multiple times (which is valid)
  if (typeof LeafletLayer !== 'undefined') {
    return new LeafletLayer(options);
  } // Leaflet layer functionality is only defined in main thread


  if (__chunk_1.Thread.is_main) {
    var L = options.leaflet || window.L; // Determine if we are extending the leaflet 0.7.x TileLayer class, or the newer
    // leaflet 1.x GridLayer class.

    var layerBaseClass = L.GridLayer ? L.GridLayer : L.TileLayer;
    var leafletVersion = layerBaseClass === L.GridLayer ? '1.x' : '0.7.x';
    var layerClassConfig = {};
    var setZoomAroundNoMoveEnd; // alternate zoom functions defined below
    // If extending leaflet 0.7.x TileLayer, additional modifications are needed

    if (layerBaseClass === L.TileLayer) {
      layerClassConfig._addTile = function () {};

      layerClassConfig._removeTile = function () {};

      layerClassConfig._reset = function () {
        layerBaseClass.prototype._reset.apply(this, arguments); // re-add the canvas since base class `viewreset` event can remove it


        if (this.scene && this.scene.container && this.scene.canvas) {
          this.scene.container.appendChild(this.scene.canvas);
        }
      };
    } // Define custom layer methods


    Object.assign(layerClassConfig, {
      initialize: function initialize(options) {
        // Defaults
        options.showDebug = !options.showDebug ? false : true;
        L.setOptions(this, options);
        this.updateTangramDebugSettings();
        this.createScene();
        this.hooks = {};
        this._updating_tangram = false;
      },
      createScene: function createScene() {
        this.scene = Scene.create(this.options.scene, {
          numWorkers: this.options.numWorkers,
          preUpdate: this.options.preUpdate,
          postUpdate: this.options.postUpdate,
          continuousZoom: LeafletLayer.leafletVersion === '1.x',
          wrapView: this.options.noWrap === true ? false : true,
          highDensityDisplay: this.options.highDensityDisplay,
          logLevel: this.options.logLevel,
          introspection: this.options.introspection,
          // turn scene introspection on/off
          webGLContextOptions: this.options.webGLContextOptions,
          // override/supplement WebGL context options
          disableRenderLoop: this.options.disableRenderLoop // app must call scene.update() per frame

        });
      },
      // Finish initializing scene and setup events when layer is added to map
      onAdd: function onAdd(map) {
        var _this = this;

        if (!this.scene) {
          this.createScene();
        }

        layerBaseClass.prototype.onAdd.apply(this, arguments);

        this.hooks.resize = function () {
          _this._updating_tangram = true;

          _this.updateSize();

          _this._updating_tangram = false;
        };

        map.on('resize', this.hooks.resize);

        this.hooks.move = function () {
          if (_this._updating_tangram) {
            return;
          }

          _this._updating_tangram = true;

          _this.scene.view.setPanning(true);

          var view = map.getCenter();
          view.zoom = Math.max(Math.min(map.getZoom(), map.getMaxZoom() || __chunk_1.Geo.default_view_max_zoom), map.getMinZoom());

          _this.scene.view.setView(view);

          if (_this._mapLayerCount > 1) {
            // if there are other map pane layers active, redraw immediately to stay in better visual sync
            // otherwise, wait until next regular animation loop iteration
            _this.scene.immediateRedraw();
          }

          _this._updating_tangram = false;
        };

        map.on('move', this.hooks.move);

        this.hooks.moveend = function () {
          _this.scene.view.setPanning(false);

          _this.scene.requestRedraw();
        };

        map.on('moveend', this.hooks.moveend);

        this.hooks.drag = function () {
          _this.scene.view.markUserInput();
        };

        map.on('drag', this.hooks.drag); // keep Tangram layer in sync with view via mutation observer

        this._map_pane_observer = new MutationObserver(function (mutations) {
          mutations.forEach(function () {
            return _this.reverseTransform();
          });
        });

        this._map_pane_observer.observe(map.getPanes().mapPane, {
          attributes: true
        }); // Modify default Leaflet behaviors


        this.modifyScrollWheelBehavior(map);
        this.modifyZoomBehavior(map);
        this.trackMapLayerCounts(map); // Setup feature selection

        this.setupSelectionEventHandlers(map);
        this.setSelectionEvents(this.options.events, {
          radius: this.options.selectionRadius
        }); // Add GL canvas to layer container

        this.scene.container = this.getContainer();
        this.updateSize(); // Initial view

        this.updateView();
        this.resizeOnFirstVisible(); // Subscribe to tangram events

        this.scene.subscribe({
          move: this.onTangramViewUpdate.bind(this)
        }); // Use leaflet's existing event system as the callback mechanism

        this.scene.load(this.options.scene, {
          base_path: this.options.sceneBasePath,
          file_type: this.options.sceneFileType,
          blocking: false
        }).then(function () {
          _this._updating_tangram = true;

          _this.updateSize();

          _this.updateView();

          _this.reverseTransform();

          _this._updating_tangram = false;

          _this.fire('init');
        }).catch(function (error) {
          _this.fire('error', error);
        });
      },
      onRemove: function onRemove(map) {
        layerBaseClass.prototype.onRemove.apply(this, arguments);
        map.off('layeradd layerremove overlayadd overlayremove', this._updateMapLayerCount);
        map.off('resize', this.hooks.resize);
        map.off('move', this.hooks.move);
        map.off('moveend', this.hooks.moveend);
        map.off('drag', this.hooks.drag);
        map.off('click', this.hooks.click);
        map.off('mousemove', this.hooks.mousemove);
        map.off('mouseout', this.hooks.mouseout);
        document.removeEventListener('visibilitychange', this.hooks.visibilitychange);
        this.hooks = {};

        this._map_pane_observer.disconnect();

        if (this.scene) {
          this.scene.destroy();
          this.scene = null;
        }
      },
      createTile: function createTile(coords) {
        var key = coords.x + '/' + coords.y + '/' + coords.z;
        var div = document.createElement('div');
        div.setAttribute('data-tile-key', key);
        div.style.width = '256px';
        div.style.height = '256px';

        if (this.options.showDebug) {
          var debug_overlay = document.createElement('div');
          debug_overlay.textContent = key;
          debug_overlay.style.position = 'absolute';
          debug_overlay.style.left = 0;
          debug_overlay.style.top = 0;
          debug_overlay.style.color = 'white';
          debug_overlay.style.fontSize = '16px';
          debug_overlay.style.textOutline = '1px #000000';
          debug_overlay.style.padding = '8px';
          div.appendChild(debug_overlay);
          div.style.borderStyle = 'solid';
          div.style.borderColor = 'white';
          div.style.borderWidth = '1px';
        }

        return div;
      },
      // Modify leaflet's default scroll wheel behavior to render frames more frequently
      // (should generally lead to smoother scroll with Tangram frame re-render)
      modifyScrollWheelBehavior: function modifyScrollWheelBehavior(map) {
        if (this.scene.view.continuous_zoom && map.scrollWheelZoom && this.options.modifyScrollWheel !== false) {
          map.options.zoomSnap = 0;
          var enabled = map.scrollWheelZoom.enabled();
          map.scrollWheelZoom.disable(); // Chrome and Safari have smoother scroll-zoom without actively throttling the mouse wheel,
          // while FF and Edge/IE do better with throttling.
          // TODO: may be related to syncing differences with requestAnimationFrame loop, investigate further

          if (L.Browser.chrome || L.Browser.safari) {
            map.scrollWheelZoom._onWheelScroll = function (e) {
              var delta = L.DomEvent.getWheelDelta(e);
              this._delta += delta;
              this._lastMousePos = this._map.mouseEventToContainerPoint(e);

              this._performZoom();

              L.DomEvent.stop(e);
            };
          } else {
            map.options.wheelDebounceTime = 20; // better default for FF and Edge/IE
          }

          var debounceMoveEnd = debounce(function (map) {
            map._moveEnd(true);

            map.fire('viewreset'); // keep other leaflet layers in sync
          }, map.options.wheelDebounceTime * 2); // save reference to overloaded method

          if (!originalHandlers.scrollWheelZoom._performZoom) {
            originalHandlers.scrollWheelZoom._performZoom = map.scrollWheelZoom._performZoom;
          }

          var layer = this;

          map.scrollWheelZoom._performZoom = function () {
            if (this._map !== layer._map) {
              // only call overloaded method on a tangram layer
              originalHandlers.scrollWheelZoom._performZoom.call(this);

              return;
            }

            var map = this._map,
                zoom = map.getZoom();

            map._stop(); // stop panning and fly animations if any


            var delta = this._delta / (this._map.options.wheelPxPerZoomLevel * 4);
            this._delta = 0;

            if (zoom + delta >= this._map.getMaxZoom()) {
              delta = this._map.getMaxZoom() - zoom; // don't go past max zoom
            } else if (zoom + delta <= this._map.getMinZoom()) {
              delta = this._map.getMinZoom() - zoom; // don't go past min zoom
            }

            if (!delta) {
              return;
            }

            if (map.options.scrollWheelZoom === 'center') {
              setZoomAroundNoMoveEnd(layer, map.getCenter(), zoom + delta);
            } else {
              setZoomAroundNoMoveEnd(layer, this._lastMousePos, zoom + delta);
            }

            debounceMoveEnd(map);
          };

          if (enabled) {
            map.scrollWheelZoom.enable();
          }
        }
      },
      // Modify leaflet's default double-click and zoom in/out behavior, to better keep Tangram layer in sync with marker/SVG layers
      modifyZoomBehavior: function modifyZoomBehavior(map) {
        if (this.scene.view.continuous_zoom && this.options.modifyZoomBehavior !== false) {
          var layer = this; // Simplified version of Leaflet's flyTo, for short animations zooming around a point

          var flyAround = function flyAround(layer, targetCenter, targetZoom) {
            map._stop();

            var startZoom = map._zoom;
            targetCenter = L.latLng(targetCenter);
            targetZoom = targetZoom === undefined ? startZoom : targetZoom;
            targetZoom = Math.min(targetZoom, map.getMaxZoom()); // don't go past max zoom

            var start = Date.now(),
                duration = 75;

            function frame() {
              var t = (Date.now() - start) / duration;

              if (t <= 1) {
                // reuse internal flyTo frame to ensure these animations are canceled like others
                map._flyToFrame = L.Util.requestAnimFrame(frame, map);
                setZoomAroundNoMoveEnd(layer, targetCenter, startZoom + (targetZoom - startZoom) * t);
              } else {
                setZoomAroundNoMoveEnd(layer, targetCenter, targetZoom)._moveEnd(true);
              }
            }

            map._moveStart(true);

            frame.call(map);
            return map;
          }; // Modify the double-click zoom handler to do a short zoom animation
          // See original: https://github.com/Leaflet/Leaflet/blob/cf518ff1a5e0e54a2f63faa144aeaa50888e0bc6/src/map/handler/Map.DoubleClickZoom.js#L29


          if (map.doubleClickZoom) {
            var enabled = map.doubleClickZoom.enabled();
            map.doubleClickZoom.disable(); // save reference to overloaded method

            if (!originalHandlers.doubleClickZoom._onDoubleClick) {
              originalHandlers.doubleClickZoom._onDoubleClick = map.doubleClickZoom._onDoubleClick;
            }

            map.doubleClickZoom._onDoubleClick = function (e) {
              if (this._map !== layer._map) {
                // only call overloaded method on a tangram layer
                originalHandlers.doubleClickZoom._onDoubleClick.call(this, e);

                return;
              }

              var map = this._map,
                  oldZoom = map.getZoom(),
                  delta = map.options.zoomDelta,
                  zoom = e.originalEvent.shiftKey ? oldZoom - delta : oldZoom + delta;

              if (map.options.doubleClickZoom === 'center') {
                flyAround(layer, map.getCenter(), zoom);
              } else {
                flyAround(layer, map.containerPointToLatLng(e.containerPoint), zoom);
              }
            };

            if (enabled) {
              map.doubleClickZoom.enable();
            }
          } // Modify the zoom in/out behavior
          // NOTE: this will NOT fire the 'zoomanim' event, so this modification should be disabled for apps that depend on it
          // See original: https://github.com/Leaflet/Leaflet/blob/cf518ff1a5e0e54a2f63faa144aeaa50888e0bc6/src/map/Map.js#L1610


          if (map._zoomAnimated) {
            // save reference to overloaded method
            if (!originalHandlers.map._animateZoom) {
              originalHandlers.map._animateZoom = map._animateZoom;
            }

            map._animateZoom = function (center, zoom, startAnim, noUpdate) {
              if (this !== layer._map) {
                // only call overloaded method on a tangram layer
                originalHandlers.map._animateZoom.call(this, center, zoom, startAnim, noUpdate);

                return;
              }

              if (startAnim) {
                this._animatingZoom = true; // remember what center/zoom to set after animation

                this._animateToCenter = center;
                this._animateToZoom = zoom; // replace leaflet CSS animation with Tangram animation to keep markers/SVG in sync
                // (this is a workaround from not being able to easily track/sync to on-going CSS animations in JS)

                flyAround(layer, center, zoom);
              } // Work around webkit not firing 'transitionend', see https://github.com/Leaflet/Leaflet/issues/3689, 2693


              setTimeout(L.Util.bind(this._onZoomTransitionEnd, this), 250);
            };
          }
        }
      },
      updateView: function updateView() {
        var view = this._map.getCenter();

        view.zoom = Math.max(Math.min(this._map.getZoom(), this._map.getMaxZoom() || __chunk_1.Geo.default_view_max_zoom), this._map.getMinZoom());
        this.scene.view.setView(view);
      },
      updateSize: function updateSize() {
        var size = this._map.getSize();

        this.scene.resizeMap(size.x, size.y);
      },
      resizeOnFirstVisible: function resizeOnFirstVisible() {
        var _this2 = this;

        var first_visibility = true;

        this.hooks.visibilitychange = function () {
          if (first_visibility) {
            first_visibility = false;

            _this2.updateSize();
          }
        };

        document.addEventListener('visibilitychange', this.hooks.visibilitychange);
      },
      onTangramViewUpdate: function onTangramViewUpdate() {
        if (!this._map || this._updating_tangram) {
          return;
        } // View changed?


        var map_center = this._map.getCenter();

        var view_center = this.scene.view.center;

        if (map_center.lng === view_center.lng && map_center.lat === view_center.lat && this._map.getZoom() === this.scene.view.zoom) {
          return;
        }

        this._updating_tangram = true;

        this._map.setView([this.scene.view.center.lat, this.scene.view.center.lng], this.scene.view.zoom, {
          animate: false
        });

        this._updating_tangram = false;
      },
      render: function render() {
        if (!this.scene) {
          return;
        }

        this.scene.update();
      },
      // Reverse the CSS positioning Leaflet applies to the layer, since Tangram's WebGL canvas
      // is expected to be 'absolutely' positioned.
      reverseTransform: function reverseTransform() {
        if (!this._map || !this.scene || !this.scene.container) {
          return;
        }

        var top_left = this._map.containerPointToLayerPoint([0, 0]);

        L.DomUtil.setPosition(this.scene.container, top_left);
      },
      // Tie Leaflet event handlers to Tangram feature selection
      setupSelectionEventHandlers: function setupSelectionEventHandlers(map) {
        var _this3 = this;

        this._selection_events = {};
        this._selection_radius = null; // optional radius

        this.hooks.click = function (event) {
          if (typeof _this3._selection_events.click === 'function') {
            _this3.scene.getFeatureAt(event.containerPoint, {
              radius: _this3._selection_radius
            }).then(function (selection) {
              var results = Object.assign({}, selection, {
                leaflet_event: event
              });

              _this3._selection_events.click(results);
            });
          }
        };

        map.on('click', this.hooks.click);

        this.hooks.mousemove = function (event) {
          if (typeof _this3._selection_events.hover === 'function') {
            _this3.scene.getFeatureAt(event.containerPoint, {
              radius: _this3._selection_radius
            }).then(function (selection) {
              var results = Object.assign({}, selection, {
                leaflet_event: event
              });

              _this3._selection_events.hover(results);
            });
          }
        };

        map.on('mousemove', this.hooks.mousemove);

        this.hooks.mouseout = function (event) {
          // When mouse leaves map, send an additional selection event to indicate no feature is selected
          if (typeof _this3._selection_events.hover === 'function') {
            _this3._selection_events.hover({
              changed: true,
              leaflet_event: event
            });
          }
        };

        map.on('mouseout', this.hooks.mouseout);
      },
      // Set user-defined handlers for feature selection events
      // Currently only one handler can be defined for each event type
      // Event types are: `click`, `hover` (leaflet `mousemove`)
      setSelectionEvents: function setSelectionEvents(events, _temp) {
        var _ref = _temp === void 0 ? {} : _temp,
            radius = _ref.radius;

        this._selection_events = Object.assign(this._selection_events, events);
        this._selection_radius = radius !== undefined ? radius : this._selection_radius;
      },
      // Track the # of layers in the map pane
      // Used to optimize Tangram redraw sensitivity (redraw more frequently when needing to sync w/other layers)
      trackMapLayerCounts: function trackMapLayerCounts(map) {
        var _this4 = this;

        this._updateMapLayerCount = function () {
          var nodes = map.getPanes().mapPane.childNodes;
          _this4._mapLayerCount = 0;

          for (var i = 0; i < nodes.length; i++) {
            _this4._mapLayerCount += nodes[i].childNodes.length;
          }
        };

        map.on('layeradd layerremove overlayadd overlayremove', this._updateMapLayerCount);

        this._updateMapLayerCount();
      },
      updateTangramDebugSettings: function updateTangramDebugSettings() {
        __chunk_1.mergeDebugSettings(this.options.debug || {});
      }
    }); // Modified version of Leaflet's setZoomAround that doesn't trigger a moveEnd event

    setZoomAroundNoMoveEnd = function setZoomAroundNoMoveEnd(layer, latlng, zoom) {
      var map = layer._map,
          scene = layer.scene,
          scale = map.getZoomScale(zoom),
          viewHalf = map.getSize().divideBy(2),
          containerPoint = latlng instanceof L.Point ? latlng : map.latLngToContainerPoint(latlng),
          centerOffset = containerPoint.subtract(viewHalf).multiplyBy(1 - 1 / scale),
          newCenter = map.containerPointToLatLng(viewHalf.add(centerOffset));

      if (scene) {
        scene.view.markUserInput();
      }

      return map._move(newCenter, zoom, {
        flyTo: true
      });
    }; // Create the layer class


    LeafletLayer = layerBaseClass.extend(layerClassConfig); // Polyfill some 1.0 methods

    if (typeof LeafletLayer.remove !== 'function') {
      LeafletLayer.prototype.remove = function () {
        if (this._map) {
          this._map.removeLayer(this);
        }

        this.fire('remove');
      };
    }

    LeafletLayer.layerBaseClass = layerBaseClass;
    LeafletLayer.leafletVersion = leafletVersion;
    return new LeafletLayer(options);
  }
}

/*jshint worker: true*/

var debug$1 = {
  log: __chunk_1.log,
  yaml: jsYaml$1,
  Utils: __chunk_1.Utils,
  Geo: __chunk_1.Geo,
  Vector: __chunk_1.Vector,
  DataSource: __chunk_1.DataSource,
  GLSL: __chunk_1.GLSL,
  ShaderProgram: __chunk_1.ShaderProgram,
  VertexData: __chunk_1.VertexData,
  Texture: __chunk_1.Texture,
  Material: __chunk_1.Material,
  Light: __chunk_1.Light,
  Scene: Scene,
  WorkerBroker: __chunk_1.WorkerBroker,
  Task: __chunk_1.Task,
  StyleManager: __chunk_1.StyleManager,
  StyleParser: __chunk_1.StyleParser,
  TileID: __chunk_1.TileID,
  Collision: __chunk_1.Collision,
  FeatureSelection: __chunk_1.FeatureSelection,
  TextCanvas: __chunk_1.TextCanvas,
  debugSettings: __chunk_1.debugSettings
};
var index = {
  leafletLayer: leafletLayer,
  debug: debug$1,
  version: __chunk_1.version
};

return index;

});

// Loads each of the chunks produced by the first Rollup pass.

// This allows the rollup ESM build to work within a <script type="module"> tag
// Script modules can't expose exports
try {
	Tangram.debug.ESM = false; // mark build as ES module
	Tangram.debug.SHA = 'b08f8c830cad59c24ca6f79f40b0d1db76b4136e';
	if (false === true && typeof window === 'object') {
	    window.Tangram = Tangram;
	}
} catch(e) {}

return Tangram;

}));
//# sourceMappingURL=tangram.debug.js.map
