// define() gets called for each chunk generated by the first Rollup pass.
// The order the chunks are called in is controlled by the imports in bundle.js:
//
// shared.js: shared dependencies between main and worker threads
// scene_worker.js: worker thread code
// index.js: main thread code

// Once all chunks have been provided, the worker thread code is assembled,
// incorporating the shared chunk code, then turned into a blob URL which
// can be used to instantiate the worker.

var shared, worker, Tangram = {};

function define(_, chunk) {
	if (!shared) {
	    shared = chunk;
	} else if (!worker) {
	    worker = chunk;
	} else {
	    var worker_bundle = 'var shared_chunk = {}; (' + shared + ')(shared_chunk); (' + worker + ')(shared_chunk);'
	    var shared_chunk = {};
	    shared(shared_chunk);
	    Tangram = chunk(shared_chunk);
	    Tangram.workerURL = window.URL.createObjectURL(new Blob([worker_bundle], { type: 'text/javascript' }));
	}
}


define(["exports"],function(e){"use strict";const t={};try{window instanceof Window&&window.document instanceof HTMLDocument&&(t.is_worker=!1,t.is_main=!0);}catch(e){t.is_worker=!0,t.is_main=!1,self.window={document:{}},self.document=self.window.document;}function i(){return (i=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var i=arguments[t];for(var r in i)Object.prototype.hasOwnProperty.call(i,r)&&(e[r]=i[r]);}return e}).apply(this,arguments)}var r,s="v0.18.1",n=r={},o=0,a={};function l(e){var t=[];"string"==typeof e&&(t=e.split("."),e=t.pop());var i=r.targets;for(let e=0;e<t.length;e++){if(!i[t[e]])return [];i=i[t[e]];}return [e,i]}function u(e){Array.isArray(e)&&e.filter(e=>e.parent&&e.property).forEach(e=>delete e.parent[e.property]);}r.targets={},r.addTarget=function(e,t){r.targets[e]=t;},r.removeTarget=function(e){e&&delete r.targets[e];},r.withTransferables=function(){for(var e=arguments.length,t=new Array(e),i=0;i<e;i++)t[i]=arguments[i];if(!(this instanceof r.withTransferables))return new r.withTransferables(...t);this.value=t,this.transferables=function e(t,i,r,s){void 0===i&&(i=null);void 0===r&&(r=null);void 0===s&&(s=[]);if(!t)return s;if(Array.isArray(t))t.forEach((i,r)=>e(i,t,r,s));else if("object"==typeof t)if(t instanceof ArrayBuffer)s.push({object:t,parent:i,property:r});else if(t.buffer instanceof ArrayBuffer)s.push({object:t.buffer,parent:i,property:r});else for(let i in t)e(t[i],t,i,s);return s}(this.value);},t.is_main&&(r.postMessage=function(e,t){for(var i=arguments.length,s=new Array(i>2?i-2:0),n=2;n<i;n++)s[n-2]=arguments[n];if(Array.isArray(e))return Promise.all(e.map(e=>r.postMessage(e,t,...s)));let l={};"object"==typeof t&&(l=t,t=t.method);var h=new Promise((e,i)=>{a[o]={method:t,message:s,resolve:e,reject:i};});let c,d=[];return s&&1===s.length&&s[0]instanceof r.withTransferables&&(d=s[0].transferables,s=s[0].value),c={type:"main_send",message_id:o,method:t,message:s},l.stringify&&(c=JSON.stringify(c)),e.postMessage(c,d.map(e=>e.object)),u(d),d.length>0&&f("trace",`'${t}' transferred ${d.length} objects to worker thread`),o++,h},r.addWorker=function(e){if(!(e instanceof Worker))throw Error("Worker broker could not add non-Worker object",e);e.addEventListener("message",function(t){let i="string"==typeof t.data?JSON.parse(t.data):t.data,s=i.message_id;if("worker_reply"===i.type)a[s]&&(i.error?a[s].reject(i.error):a[s].resolve(i.message),delete a[s]);else if("worker_send"===i.type&&null!=s){let t,o,a,h,c;try{var n=l(i.method);if(h=n[0],!(a=n[1]))throw Error(`Worker broker could not dispatch message type ${i.method} on target ${i.target} because no object with that name is registered on main thread`);if(!(c="function"==typeof a[h]&&a[h]))throw Error(`Worker broker could not dispatch message type ${i.method} on target ${i.target} because object has no method with that name`);t=c.apply(a,i.message);}catch(e){o=e;}let d,_=[];t instanceof Promise?t.then(t=>{t instanceof r.withTransferables&&(_=t.transferables,t=t.value[0]),d={type:"main_reply",message_id:s,message:t},e.postMessage(d,_.map(e=>e.object)),u(_),_.length>0&&f("trace",`'${h}' transferred ${_.length} objects to worker thread`);},t=>{e.postMessage({type:"main_reply",message_id:s,error:t instanceof Error?`${t.message}: ${t.stack}`:t});}):(t instanceof r.withTransferables&&(_=t.transferables,t=t.value[0]),d={type:"main_reply",message_id:s,message:t,error:o instanceof Error?`${o.message}: ${o.stack}`:o},e.postMessage(d,_.map(e=>e.object)),u(_),_.length>0&&f("trace",`'${h}' transferred ${_.length} objects to worker thread`));}});},r.getMessages=function(){return a},r.getMessageId=function(){return o}),t.is_worker&&(r.postMessage=function(e){for(var t=arguments.length,i=new Array(t>1?t-1:0),s=1;s<t;s++)i[s-1]=arguments[s];let n={};"object"==typeof e&&(n=e,e=e.method);var l=new Promise((t,r)=>{a[o]={method:e,message:i,resolve:t,reject:r};});let h,c=[];return i&&1===i.length&&i[0]instanceof r.withTransferables&&(c=i[0].transferables,i=i[0].value),h={type:"worker_send",message_id:o,method:e,message:i},n.stringify&&(h=JSON.stringify(h)),self.postMessage(h,c.map(e=>e.object)),u(c),c.length>0&&f("trace",`'${e}' transferred ${c.length} objects to main thread`),o++,l},self.addEventListener("message",function(e){let t="string"==typeof e.data?JSON.parse(e.data):e.data,i=t.message_id;if("main_reply"===t.type)a[i]&&(t.error?a[i].reject(t.error):a[i].resolve(t.message),delete a[i]);else if("main_send"===t.type&&null!=i){let e,n,o,a,h;try{var s=l(t.method);if(a=s[0],!(o=s[1]))throw Error(`Worker broker could not dispatch message type ${t.method} on target ${t.target} because no object with that name is registered on main thread`);if(!(h="function"==typeof o[a]&&o[a]))throw Error(`Worker broker could not dispatch message type ${t.method} because worker has no method with that name`);e=h.apply(o,t.message);}catch(e){n=e;}let c,d=[];e instanceof Promise?e.then(e=>{e instanceof r.withTransferables&&(d=e.transferables,e=e.value[0]),c={type:"worker_reply",message_id:i,message:e},self.postMessage(c,d.map(e=>e.object)),u(d),d.length>0&&f("trace",`'${a}' transferred ${d.length} objects to main thread`);},e=>{self.postMessage({type:"worker_reply",message_id:i,error:e instanceof Error?`${e.message}: ${e.stack}`:e});}):(e instanceof r.withTransferables&&(d=e.transferables,e=e.value[0]),c={type:"worker_reply",message_id:i,message:e,error:n instanceof Error?`${n.message}: ${n.stack}`:n},self.postMessage(c,d.map(e=>e.object)),u(d),d.length>0&&f("trace",`'${a}' transferred ${d.length} objects to main thread`));}}));const h={silent:-1,error:0,warn:1,info:2,debug:3,trace:4},c={};let d={};function f(e){let i="object"==typeof e?e.level:e;if(h[i]<=h[f.level]){for(var r=arguments.length,o=new Array(r>1?r-1:0),a=1;a<r;a++)o[a-1]=arguments[a];if(t.is_worker)return n.postMessage({method:"_logProxy",stringify:!0},e,...o);{if("object"==typeof e&&!0===e.once){if(d[JSON.stringify(o)])return Promise.resolve(!1);d[JSON.stringify(o)]=!0;}let r=function(e){if(t.is_main)return c[e]=c[e]||(console[e]?console[e]:console.log).bind(console),c[e]}(i);o.length>1?r(`Tangram ${s} [${i}]: ${o[0]}`,...o.slice(1)):r(`Tangram ${s} [${i}]: ${o[0]}`);}return Promise.resolve(!0)}return Promise.resolve(!1)}f.level="info",f.workers=null,f.setLevel=function(e){f.level=e,t.is_main&&Array.isArray(f.workers)&&n.postMessage(f.workers,"_logSetLevelProxy",e);},t.is_main&&(f.setWorkers=function(e){f.workers=e;},f.reset=function(){d={};}),n.addTarget("_logProxy",f),n.addTarget("_logSetLevelProxy",f.setLevel);const _={};let p;n.addTarget("Utils",_),_.isSafari=function(){return /^((?!chrome|android).)*safari/i.test(navigator.userAgent)},_.isMicrosoft=function(){return /(Trident\/7.0|Edge[ \/](\d+[.\d]+))/i.test(navigator.userAgent)},_._requests={},_._proxy_requests={},_.io=function(e,i,r,s,o,a,l){if(void 0===i&&(i=6e4),void 0===r&&(r="text"),void 0===s&&(s="GET"),void 0===o&&(o={}),void 0===a&&(a=null),void 0===l&&(l=!1),t.is_worker&&_.isMicrosoft())return f("debug","Proxying request for URL to worker",e),a&&(_._proxy_requests[a]=!0),n.postMessage("Utils.io",e,i,r,s,o,a,!0);var u=new XMLHttpRequest,h=new Promise((t,n)=>{if(u.open(s,e,!0),u.timeout=i,u.responseType=r,o&&"object"==typeof o)for(let e in o)u.setRequestHeader(e,o[e]);u.onload=(()=>{200===u.status?["text","json"].indexOf(u.responseType)>-1?t(u.responseText):t(u.response):n(Error("Request error with a status of "+u.statusText));}),u.onerror=(e=>{n(Error("There was a network error"+e.toString()));}),u.ontimeout=(e=>{n(Error("timeout "+e.toString()));}),u.send();});return h=h.then(e=>(a&&delete _._requests[a],l?n.withTransferables(e):e)),a&&(_._requests[a]=u),h},_.cancelRequest=function(e){if(t.is_worker&&_._proxy_requests[e])return n.postMessage("Utils.cancelRequest",e);_._requests[e]?(f("trace",`Cancelling network request key '${e}'`),_._requests[e].abort(),delete _._requests[e]):f("trace",`Could not find network request key '${e}'`);},_.serializeWithFunctions=function(e){if("function"==typeof e)return e.toString();return JSON.stringify(e,function(e,t){return "function"==typeof t?t.toString():t})},_.use_high_density_display=!0,_.updateDevicePixelRatio=function(){let e=_.device_pixel_ratio;return _.device_pixel_ratio=_.use_high_density_display&&window.devicePixelRatio||1,_.device_pixel_ratio!==e},t.is_main&&_.updateDevicePixelRatio(),_.isPowerOf2=function(e){return 0==(e&e-1)},_.interpolate=function(e,t,i){if(!Array.isArray(t)||!Array.isArray(t[0]))return t;if(t.length<1)return t;var r,s,n,o,a;if(e<=t[0][0])o=t[0][1],"function"==typeof i&&(o=i(o));else if(e>=t[t.length-1][0])o=t[t.length-1][1],"function"==typeof i&&(o=i(o));else for(var l=0;l<t.length-1;l++)if(e>=t[l][0]&&e<t[l+1][0]){if(r=t[l][0],s=t[l+1][0],Array.isArray(t[l][1])){o=[];for(var u=0;u<t[l][1].length;u++)"function"==typeof i?(a=i(t[l][1][u]),n=i(t[l+1][1][u])-a,o[u]=n*(e-r)/(s-r)+a):(n=t[l+1][1][u]-t[l][1][u],o[u]=n*(e-r)/(s-r)+t[l][1][u]);}else"function"==typeof i?(a=i(t[l][1]),o=(n=i(t[l+1][1])-a)*(e-r)/(s-r)+a):o=(n=t[l+1][1]-t[l][1])*(e-r)/(s-r)+t[l][1];break}return o},_.toCSSColor=function(e){return 1===e[3]?`rgb(${e.slice(0,3).map(e=>Math.round(255*e)).join(", ")})`:`rgba(${e.map((e,t)=>t<3&&Math.round(255*e)||e).join(", ")})`};var m=p={draw_label_collision_boxes:!1,draw_label_texture_boxes:!1,suppress_label_fade_in:!1,suppress_label_snap_animation:!1,show_hidden_labels:!1,layer_stats:!1};function g(e){if("string"==typeof e&&-1===e.search(/^(data|blob):/)){let t=e.indexOf("?");t>-1&&(e=e.substr(0,t));let i=e.indexOf("#");return i>-1&&(e=e.substr(0,i)),e.substr(0,e.lastIndexOf("/")+1)||""}return ""}function x(e){if("string"==typeof e)return !(e.search(/^(http|https|data|blob):/)>-1||"//"===e.substr(0,2))}let y;function v(e,t){e=e.replace(/[[]/,"\\[").replace(/[\]]/,"\\]");var i=new RegExp("[\\?&]"+e+"=([^&#]*)").exec(t);return null===i?"":decodeURIComponent(i[1].replace(/\+/g," "))}const b={id:0,queue:[],max_time:20,start_time:null,state:{},add(e){e.id=b.id++,e.max_time=e.max_time||b.max_time,e.pause_factor=e.pause_factor||1;let t=new Promise((t,i)=>{e.resolve=t,e.reject=i;});return e.promise=t,e.elapsed=0,e.total_elapsed=0,e.stats={calls:0},this.queue.push(e),this.start_time=this.start_time||performance.now(),this.elapsed=performance.now()-this.start_time,this.elapsed<b.max_time&&this.process(e),e.promise},remove(e){let t=this.queue.indexOf(e);t>-1&&this.queue.splice(t,1);},process(e){if(!this.state.user_moving_view||!1!==e.user_moving_view)return e.pause?(e.pause--,!0):(e.stats.calls++,e.start_time=performance.now(),e.run(e))},processAll(){this.start_time=this.start_time||performance.now();for(let e=0;e<this.queue.length;e++){let t=this.queue[e];if(!0!==this.process(t)&&(t.pause||(t.pause=t.elapsed>t.max_time?t.pause_factor:0),t.total_elapsed+=t.elapsed),this.elapsed=performance.now()-this.start_time,this.elapsed>=b.max_time){this.start_time=null;break}}},finish(e,t){return e.elapsed=performance.now()-e.start_time,e.total_elapsed+=e.elapsed,this.remove(e),e.resolve(t),e.promise},cancel(e){let t;e.cancel instanceof Function&&(t=e.cancel(e)),e.resolve(t);},shouldContinue(e){return e.elapsed=performance.now()-e.start_time,this.elapsed=performance.now()-this.start_time,e.elapsed<e.max_time&&this.elapsed<b.max_time},removeForTile(e){for(let t=this.queue.length-1;t>=0;t--)this.queue[t].tile_id===e&&(this.cancel(this.queue[t]),this.queue.splice(t,1));},setState(e){this.state=e;}};function A(e){let t=[];return Object.assign(e,{subscribe(e){-1===t.indexOf(e)&&t.push(e);},unsubscribe(e){let i=t.indexOf(e);i>-1&&t.splice(i,1);},unsubscribeAll(){t=[];},trigger(e){for(var i=arguments.length,r=new Array(i>1?i-1:0),s=1;s<i;s++)r[s-1]=arguments[s];t.forEach(t=>{if("function"==typeof t[e])try{t[e](...r);}catch(t){f("warn",`Caught exception in listener for event '${e}':`,t);}});},hasSubscribersFor(e){let i=!1;return t.forEach(t=>{"function"==typeof t[e]&&(i=!0);}),i}})}function T(e,t){let i={};return t.forEach(t=>i[t]=e[t]),i}class w{constructor(e,t,i){void 0===i&&(i={}),i=w.sliceOptions(i),this.gl=e,this.texture=e.createTexture(),this.texture&&(this.valid=!0),this.bind(),this.name=t,this.retain_count=0,this.config_type=null,this.loading=null,this.loaded=!1,this.filtering=i.filtering,this.density=i.density||1,this.sprites=i.sprites,this.texcoords={},this.sizes={},this.css_sizes={},this.aspects={},this.setData(1,1,new Uint8Array([0,0,0,0]),{filtering:"nearest"}),this.loaded=!1,w.textures[this.name]&&(this.retain_count=w.textures[this.name].retain_count,w.textures[this.name].retain_count=0,w.textures[this.name].destroy()),w.textures[this.name]=this,w.texture_configs[this.name]=JSON.stringify(Object.assign({name:t},i)),this.load(i),f("trace",`creating Texture ${this.name}`);}destroy(e){let t=(void 0===e?{}:e).force;this.retain_count>0&&!t?f("error",`Texture '${this.name}': destroying texture with retain count of '${this.retain_count}'`):this.valid&&(this.gl.deleteTexture(this.texture),this.texture=null,w.textures[this.name]===this&&(delete w.textures[this.name],delete w.texture_configs[this.name]),this.valid=!1,f("trace",`destroying Texture ${this.name}`));}retain(){this.retain_count++;}release(){this.retain_count<=0&&f("error",`Texture '${this.name}': releasing texture with retain count of '${this.retain_count}'`),this.retain_count--,this.retain_count<=0&&this.destroy();}bind(e){void 0===e&&(e=0),this.valid&&(w.activeUnit!==e&&(this.gl.activeTexture(this.gl.TEXTURE0+e),w.activeUnit=e,w.boundTexture=null),w.boundTexture!==this.texture&&(this.gl.bindTexture(this.gl.TEXTURE_2D,this.texture),w.boundTexture=this.texture));}load(e){return e?(this.loading=null,"string"==typeof e.url?(this.config_type="url",this.setUrl(e.url,e)):e.element?(this.config_type="element",this.setElement(e.element,e)):e.data&&e.width&&e.height&&(this.config_type="data",this.setData(e.width,e.height,e.data,e)),this.loading=this.loading&&this.loading.then(()=>(this.calculateSprites(),this))||Promise.resolve(this),this.loading):this.loading||Promise.resolve(this)}setUrl(e,t){if(void 0===t&&(t={}),this.valid)return this.url=e,this.loading=new Promise(e=>{let i=new Image;i.onload=(()=>{try{this.setElement(i,t);}catch(e){this.loaded=!1,f("warn",`Texture '${this.name}': failed to load url: '${this.url}'`,e,t),w.trigger("warning",{message:`Failed to load texture from ${this.url}`,error:e,texture:t});}this.loaded=!0,e(this);}),i.onerror=(i=>{this.loaded=!1,f("warn",`Texture '${this.name}': failed to load url: '${this.url}'`,i,t),w.trigger("warning",{message:`Failed to load texture from ${this.url}`,error:i,texture:t}),e(this);}),_.isSafari()&&"data:"===this.url.slice(0,5)||(i.crossOrigin="anonymous"),i.src=this.url;}),this.loading}setData(e,t,i,r){return void 0===r&&(r={}),this.width=e,this.height=t,Array.isArray(i)&&(i=new Uint8Array(i)),this.update(i,r),this.setFiltering(r),this.loaded=!0,this.loading=Promise.resolve(this),this.loading}setElement(e,t){let i=e;if("string"==typeof e&&(e=document.querySelector(e)),e instanceof HTMLCanvasElement||e instanceof HTMLImageElement||e instanceof HTMLVideoElement)this.update(e,t),this.setFiltering(t);else{this.loaded=!1;let e=`the 'element' parameter (\`element: ${JSON.stringify(i)}\`) must be a CSS `;e+="selector string, or a <canvas>, <image> or <video> object",f("warn",`Texture '${this.name}': ${e}`,t),w.trigger("warning",{message:`Failed to load texture because ${e}`,texture:t});}return this.loaded=!0,this.loading=Promise.resolve(this),this.loading}update(e,t){void 0===t&&(t={}),this.valid&&(this.bind(),e instanceof HTMLCanvasElement||e instanceof HTMLVideoElement||e instanceof HTMLImageElement&&e.complete?(this.width=e.width,this.height=e.height,this.gl.pixelStorei(this.gl.UNPACK_FLIP_Y_WEBGL,!1!==t.UNPACK_FLIP_Y_WEBGL),this.gl.pixelStorei(this.gl.UNPACK_PREMULTIPLY_ALPHA_WEBGL,t.UNPACK_PREMULTIPLY_ALPHA_WEBGL||!1),this.gl.texImage2D(this.gl.TEXTURE_2D,0,this.gl.RGBA,this.gl.RGBA,this.gl.UNSIGNED_BYTE,e)):(this.gl.pixelStorei(this.gl.UNPACK_FLIP_Y_WEBGL,null),this.gl.pixelStorei(this.gl.UNPACK_PREMULTIPLY_ALPHA_WEBGL,null),this.gl.texImage2D(this.gl.TEXTURE_2D,0,this.gl.RGBA,this.width,this.height,0,this.gl.RGBA,this.gl.UNSIGNED_BYTE,e)),w.trigger("update",this));}setFiltering(e){if(void 0===e&&(e={}),this.valid){e.filtering=e.filtering||"linear";var t=this.gl;this.bind(),_.isPowerOf2(this.width)&&_.isPowerOf2(this.height)?(this.power_of_2=!0,t.texParameteri(t.TEXTURE_2D,t.TEXTURE_WRAP_S,e.TEXTURE_WRAP_S||e.repeat&&t.REPEAT||t.CLAMP_TO_EDGE),t.texParameteri(t.TEXTURE_2D,t.TEXTURE_WRAP_T,e.TEXTURE_WRAP_T||e.repeat&&t.REPEAT||t.CLAMP_TO_EDGE),"mipmap"===e.filtering?(this.filtering="mipmap",t.texParameteri(t.TEXTURE_2D,t.TEXTURE_MIN_FILTER,t.LINEAR_MIPMAP_LINEAR),t.texParameteri(t.TEXTURE_2D,t.TEXTURE_MAG_FILTER,t.LINEAR),t.generateMipmap(t.TEXTURE_2D)):"linear"===e.filtering?(this.filtering="linear",t.texParameteri(t.TEXTURE_2D,t.TEXTURE_MIN_FILTER,t.LINEAR),t.texParameteri(t.TEXTURE_2D,t.TEXTURE_MAG_FILTER,t.LINEAR)):"nearest"===e.filtering&&(this.filtering="nearest",t.texParameteri(t.TEXTURE_2D,t.TEXTURE_MIN_FILTER,t.NEAREST),t.texParameteri(t.TEXTURE_2D,t.TEXTURE_MAG_FILTER,t.NEAREST))):(this.power_of_2=!1,t.texParameteri(t.TEXTURE_2D,t.TEXTURE_WRAP_S,t.CLAMP_TO_EDGE),t.texParameteri(t.TEXTURE_2D,t.TEXTURE_WRAP_T,t.CLAMP_TO_EDGE),"nearest"===e.filtering?(this.filtering="nearest",t.texParameteri(t.TEXTURE_2D,t.TEXTURE_MIN_FILTER,t.NEAREST),t.texParameteri(t.TEXTURE_2D,t.TEXTURE_MAG_FILTER,t.NEAREST)):(this.filtering="linear",t.texParameteri(t.TEXTURE_2D,t.TEXTURE_MIN_FILTER,t.LINEAR),t.texParameteri(t.TEXTURE_2D,t.TEXTURE_MAG_FILTER,t.LINEAR))),w.trigger("update",this);}}calculateSprites(){if(this.sprites)for(let e in this.sprites){let t=this.sprites[e];this.texcoords[e]=w.getTexcoordsForSprite([t[0],t[1]],[t[2],t[3]],[this.width,this.height]),this.sizes[e]=[t[2],t[3]],this.css_sizes[e]=[t[2]/this.density,t[3]/this.density],this.aspects[e]=t[2]/t[3];}}byteSize(){return Math.round(this.width*this.height*4*("mipmap"==this.filtering?1.33:1))}}w.create=function(e,t,i){return new w(e,t,i)},w.retain=function(e){w.textures[e]&&w.textures[e].retain();},w.release=function(e){w.textures[e]&&w.textures[e].release();},w.destroy=function(e){Object.keys(w.textures).forEach(t=>{var i=w.textures[t];i.gl===e&&i.destroy({force:!0});});},w.getSpriteInfo=function(e,t){let i=w.textures[e];return i&&{size:i.sizes[t],css_size:i.css_sizes[t],aspect:i.aspects[t],texcoords:i.texcoords[t]}},w.getTexcoordsForSprite=function(e,t,i){var r=i[1]-e[1]-t[1];return [e[0]/i[0],r/i[1],(t[0]+e[0])/i[0],(t[1]+r)/i[1]]},w.createFromObject=function(e,t){let i=[];if(t)for(let r in t){let s=t[r];if(s.skip_create)continue;if(!w.changed(r,s))continue;let n=w.create(e,r,s);i.push(n.loading);}return Promise.all(i)},w.default="__default",w.createDefault=function(e){return w.create(e,w.default)},w.sliceOptions=function(e){return {filtering:e.filtering,sprites:e.sprites,url:e.url,element:e.element,data:e.data,width:e.width,height:e.height,density:e.density,repeat:e.repeat,TEXTURE_WRAP_S:e.TEXTURE_WRAP_S,TEXTURE_WRAP_T:e.TEXTURE_WRAP_T,UNPACK_FLIP_Y_WEBGL:e.UNPACK_FLIP_Y_WEBGL,UNPACK_PREMULTIPLY_ALPHA_WEBGL:e.UNPACK_PREMULTIPLY_ALPHA_WEBGL}},w.changed=function(e,t){let i=w.textures[e];if(i){if("element"===i.config_type||null!=t.element)return !0;if(t=w.sliceOptions(t),w.texture_configs[e]===JSON.stringify(Object.assign({name:e},t)))return !1}return !0},w.getInfo=function(e){if(e||(e=Object.keys(w.textures)),Array.isArray(e))return Promise.all(e.map(e=>w.getInfo(e)));var t=w.textures[e];return t?(t.loading||Promise.resolve(t)).then(()=>({name:t.name,width:t.width,height:t.height,density:t.density,css_size:[t.width/t.density,t.height/t.density],aspect:t.width/t.height,sprites:t.sprites,texcoords:t.texcoords,sizes:t.sizes,css_sizes:t.css_sizes,aspects:t.aspects,filtering:t.filtering,power_of_2:t.power_of_2,valid:t.valid})):Promise.resolve(null)},w.syncTexturesToWorker=function(e){return n.postMessage("Texture.getInfo",e).then(e=>(e&&e.filter(e=>e).forEach(e=>w.textures[e.name]=e),w.textures))},w.getMaxTextureSize=function(e){return e.getParameter(e.MAX_TEXTURE_SIZE)},w.textures={},w.texture_configs={},w.boundTexture=null,w.activeUnit=null,n.addTarget("Texture",w),A(w);var E={parseUniforms:function(e){var t=[];for(var i in e){var r,s=i,n=e[i];if("number"==typeof n)t.push({type:"float",method:"1f",name:i,value:n,key:s,uniforms:e});else if(Array.isArray(n)){if("number"==typeof n[0])n.length>=2&&n.length<=4?t.push({type:"vec"+n.length,method:n.length+"fv",name:i,value:n,key:s,uniforms:e}):n.length>4&&t.push({type:"float[]",method:"1fv",name:i+"[0]",value:n,key:s,uniforms:e});else if("string"==typeof n[0])for(r=0;r<n.length;r++)t.push({type:"sampler2D",method:"1i",name:i+"["+r+"]",value:n[r],key:r,uniforms:n});else if(Array.isArray(n[0])&&"number"==typeof n[0][0]&&n[0].length>=2&&n[0].length<=4)for(r=0;r<n.length;r++)t.push({type:"vec"+n[0].length,method:n[0].length+"fv",name:i+"["+r+"]",value:n[r],key:r,uniforms:n});}else"boolean"==typeof n?t.push({type:"bool",method:"1i",name:i,value:n,key:s,uniforms:e}):"string"==typeof n&&t.push({type:"sampler2D",method:"1i",name:i,value:n,key:s,uniforms:e});}return t},defineVariable:function(e,t){var i,r;if("number"==typeof t)i="float";else if(Array.isArray(t))"number"==typeof t[0]?t.length>=2&&t.length<=4?i="vec"+t.length:(i="float",r=t.length):"string"==typeof t[0]?(i="sampler2D",r=t.length):Array.isArray(t[0])&&"number"==typeof t[0][0]&&t[0].length>=2&&t[0].length<=4&&(i="vec"+t[0].length,r=t.length);else if("boolean"==typeof t)i="bool";else{if("string"!=typeof t)return;i="sampler2D";}var s="";return s+=`${i} ${e}`,r&&(s+=`[${r}]`),s+=";\n"},defineUniform:function(e,t){var i=E.defineVariable(e,t);if(i)return "uniform "+i},expandVec3:function(e,t){let i;if(void 0===t&&(t=1),Array.isArray(e)){if(2!==e.length)return e;i=[...e,t].map(parseFloat);}else i=[e,e,e].map(parseFloat);if(i&&i.every(e=>"number"==typeof e&&!isNaN(e)))return i},expandVec4:function(e,t){let i;if(void 0===t&&(t=1),Array.isArray(e)){if(3!==e.length)return e;i=[...e,t].map(parseFloat);}else i=[e,e,e,t].map(parseFloat);if(i&&i.every(e=>"number"==typeof e&&!isNaN(e)))return i}};let R=[];function M(e,t){let i=R.filter(t=>t[0]===e)[0];return (i=i&&i[1])||(R.push([e,{}]),i=R[R.length-1][1]),i[t]||(i[t]=e.getExtension(t)),i[t]}function z(e){var t,i,r=0;if(0===e.length)return r;for(t=0,i=e.length;t<i;t++)r=(r<<5)-r+e.charCodeAt(t),r|=0;return r}var N=function(e){e=String(e);var t,i=[];for(;t=e.match(/ERROR\:([^\n]+)/);){e=e.slice(t.index+1);var r=t[1].trim(),s=r.split(":"),n=s.slice(2).join(":").trim(),o=parseInt(s[0],10),r=parseInt(s[1],10);i.push({message:n,file:o,line:r});}return i};const S=/^\s*#pragma.*$/gm,P=/\\\s*\n/gm;class k{constructor(e,t,i,r){r=r||{},this.gl=e,this.program=null,this.compiled=!1,this.compiling=!1,this.error=null,this.defines=Object.assign({},r.defines||{}),this.blocks=Object.assign({},r.blocks||{}),this.block_scopes=Object.assign({},r.block_scopes||{}),this.extensions=r.extensions||[],this.dependent_uniforms=r.uniforms,this.uniforms={},this.attribs={},this.vertex_source=t,this.fragment_source=i,this.id=k.id++,this.name=r.name;}destroy(){this.gl.useProgram(null),this.gl.deleteProgram(this.program),this.program=null,this.uniforms={},this.attribs={},this.compiled=!1;}use(){this.compiled&&(k.current!==this&&this.gl.useProgram(this.program),k.current=this);}compile(){if(this.compiling)throw new Error(`ShaderProgram.compile(): skipping for ${this.id} (${this.name}) because already compiling`);this.compiling=!0,this.compiled=!1,this.error=null,this.computed_vertex_source=this.vertex_source,this.computed_fragment_source=this.fragment_source;let e=this.checkExtensions();var t,i=this.buildDefineList(),r=this.buildShaderBlockList();for(var s in r){var n=r[s];if(n&&(!Array.isArray(n)||0!==n.length)){t=new RegExp("^\\s*#pragma\\s+tangram:\\s+"+s+"\\s*$","m");var o=this.computed_vertex_source.match(t),a=this.computed_fragment_source.match(t);if(null!=o||null!=a){var l="";n.forEach(e=>{let t=`${e.scope}, ${e.key}, ${e.num}`;l+=`\n// tangram-block-start: ${t}\n`,l+=e.source,l+=`\n// tangram-block-end: ${t}\n`;}),null!=o&&(this.computed_vertex_source=this.computed_vertex_source.replace(t,l)),null!=a&&(this.computed_fragment_source=this.computed_fragment_source.replace(t,l)),i["TANGRAM_BLOCK_"+s.replace(/[\s-]+/g,"_").toUpperCase()]=!0;}}}this.computed_vertex_source=this.computed_vertex_source.replace(S,""),this.computed_fragment_source=this.computed_fragment_source.replace(S,""),this.ensureUniforms(this.dependent_uniforms);let u="",h=this.gl.getShaderPrecisionFormat(this.gl.FRAGMENT_SHADER,this.gl.HIGH_FLOAT);u=h&&h.precision>0?"precision highp float;\n":"precision mediump float;\n",i.TANGRAM_VERTEX_SHADER=!0,i.TANGRAM_FRAGMENT_SHADER=!1,this.computed_vertex_source=u+k.buildDefineString(i)+this.computed_vertex_source,i.TANGRAM_VERTEX_SHADER=!1,i.TANGRAM_FRAGMENT_SHADER=!0,this.computed_fragment_source=k.buildExtensionString(e)+u+k.buildDefineString(i)+this.computed_fragment_source,this.computed_vertex_source=this.computed_vertex_source.replace(P,""),this.computed_fragment_source=this.computed_fragment_source.replace(P,"");try{this.program=k.updateProgram(this.gl,this.program,this.computed_vertex_source,this.computed_fragment_source),this.compiled=!0,this.compiling=!1;}catch(e){throw this.program=null,this.compiled=!1,this.compiling=!1,this.error=e,this.error.vertex_shader_source=this.computed_vertex_source,this.error.fragment_shader_source=this.computed_fragment_source,"vertex"!==e.type&&"fragment"!==e.type||(this.shader_errors=e.errors,this.shader_errors.forEach(t=>{t.type=e.type,t.block=this.block(e.type,t.line),t.line=this.block(e.type,t.line);}),this.error.shader_errors=this.shader_errors),e}this.computed_vertex_source=null,this.computed_fragment_source=null,this.use(),this.refreshUniforms(),this.refreshAttributes();}buildDefineList(){var e,t={};for(e in k.defines)t[e]=k.defines[e];for(e in this.defines)t[e]=this.defines[e];return t}buildShaderBlockList(){let e,t={};for(e in k.blocks)t[e]=[],Array.isArray(k.blocks[e])?t[e].push(...k.blocks[e].map((t,i)=>({key:e,source:t,num:i,scope:"ShaderProgram"}))):t[e]=[{key:e,source:k.blocks[e],num:0,scope:"ShaderProgram"}];for(e in this.blocks)if(t[e]=t[e]||[],Array.isArray(this.blocks[e])){let i=this.block_scopes&&this.block_scopes[e]||[],r=null,s=0;for(let n=0;n<this.blocks[e].length;n++)i[n]!==r&&(r=i[n],s=0),t[e].push({key:e,source:this.blocks[e][n],num:s,scope:r||this.name}),s++;}else t[e].push({key:e,source:this.blocks[e],num:0,scope:this.name});return t}ensureUniforms(e){if(!e)return;const t=Object.entries(e).map(e=>{let t=e[0],i=e[1];return E.defineUniform(t,i)}).filter(e=>e);this.computed_vertex_source=t.join("\n")+this.computed_vertex_source,this.computed_fragment_source=t.join("\n")+this.computed_fragment_source;}setUniforms(e,t){if(void 0===t&&(t=!0),!this.compiled)return;t&&(this.texture_unit=0);const i=E.parseUniforms(e);for(let e=0;e<i.length;e++){const t=i[e];"sampler2D"===t.type?this.setTextureUniform(t.name,t.value):this.uniform(t.method,t.name,t.value);}}saveUniforms(e){let t=e||this.uniforms;for(let e in t){let t=this.uniforms[e];t&&(t.saved_value=t.value);}this.saved_texture_unit=this.texture_unit||0;}restoreUniforms(e){let t=e||this.uniforms;for(let e in t){let t=this.uniforms[e];t&&t.saved_value&&(t.value=t.saved_value,this.updateUniform(t));}this.texture_unit=this.saved_texture_unit||0;}setTextureUniform(e,t){var i=w.textures[t];null!=i?(i.bind(this.texture_unit),this.uniform("1i",e,this.texture_unit),this.texture_unit++):f("warn",`Cannot find texture '${t}'`);}uniform(e,t,i){if(!this.compiled)return;this.uniforms[t]=this.uniforms[t]||{};let r=this.uniforms[t];r.name=t,void 0===r.location&&(r.location=this.gl.getUniformLocation(this.program,t)),r.method=e,r.value=i,this.updateUniform(r);}updateUniform(e){this.compiled&&e&&null!=e.location&&(this.use(),this.commitUniform(e));}commitUniform(e){let t=e.location,i=e.value;switch(e.method){case"1i":this.gl.uniform1i(t,i);break;case"1f":this.gl.uniform1f(t,i);break;case"2f":this.gl.uniform2f(t,i[0],i[1]);break;case"3f":this.gl.uniform3f(t,i[0],i[1],i[2]);break;case"4f":this.gl.uniform4f(t,i[0],i[1],i[2],i[3]);break;case"1iv":this.gl.uniform1iv(t,i);break;case"3iv":this.gl.uniform3iv(t,i);break;case"1fv":this.gl.uniform1fv(t,i);break;case"2fv":this.gl.uniform2fv(t,i);break;case"3fv":this.gl.uniform3fv(t,i);break;case"4fv":this.gl.uniform4fv(t,i);break;case"Matrix3fv":this.gl.uniformMatrix3fv(t,!1,i);break;case"Matrix4fv":this.gl.uniformMatrix4fv(t,!1,i);}}refreshUniforms(){if(this.compiled)for(var e in this.uniforms){let t=this.uniforms[e];t.location=this.gl.getUniformLocation(this.program,e),this.updateUniform(t);}}refreshAttributes(){this.attribs={};}attribute(e){if(this.compiled){var t=this.attribs[e]=this.attribs[e]||{};return null!=t.location?t:(t.name=e,t.location=this.gl.getAttribLocation(this.program,e),t)}}source(e){return "vertex"===e?this.computed_vertex_source:"fragment"===e?this.computed_fragment_source:void 0}lines(e){let t=this.source(e);return t?t.split("\n"):[]}line(e,t){let i=this.lines(e);if(i)return i[t]}block(e,t){let i,r=this.lines(e);for(let e=0;e<t&&e<r.length;e++){let t=r[e],s=t.match(/\/\/ tangram-block-start: ([A-Za-z0-9_-]+), ([A-Za-z0-9_-]+), (\d+)/);s&&s.length>1?i={scope:s[1],name:s[2],num:s[3]}:(s=t.match(/\/\/ tangram-block-end: ([A-Za-z0-9_-]+), ([A-Za-z0-9_-]+), (\d+)/))&&s.length>1&&(i=null),i&&(i.line=null==i.line?-1:i.line+1,i.source=t);}return i}checkExtensions(){let e=[];return this.extensions.forEach(t=>{let i=M(this.gl,t),r=`TANGRAM_EXTENSION_${t}`;this.defines[r]=null!=i,i?e.push(t):f("debug",`Could not enable extension '${t}'`);}),e}}k.id=0,k.current=null,k.defines={},k.blocks={},k.reset=function(){k.programs_by_source={},k.shaders_by_source={};},k.reset(),k.buildDefineString=function(e){var t="";for(var i in e)null!=e[i]&&!1!==e[i]&&("boolean"==typeof e[i]&&!0===e[i]?t+="#define "+i+"\n":"number"==typeof e[i]&&Math.floor(e[i])===e[i]?t+="#define "+i+" "+e[i].toFixed(1)+"\n":t+="#define "+i+" "+e[i]+"\n");return t},k.buildExtensionString=function(e){let t="";return (e=e||[]).forEach(e=>{t+=`#ifdef GL_${e}\n#extension GL_${e} : enable\n#endif\n`;}),t},k.addBlock=function(e){k.blocks[e]=k.blocks[e]||[];for(var t=arguments.length,i=new Array(t>1?t-1:0),r=1;r<t;r++)i[r-1]=arguments[r];k.blocks[e].push(...i);},k.removeBlock=function(e){k.blocks[e]=[];},k.replaceBlock=function(e){k.removeBlock(e);for(var t=arguments.length,i=new Array(t>1?t-1:0),r=1;r<t;r++)i[r-1]=arguments[r];k.addBlock(e,...i);},k.updateProgram=function(e,t,i,r){let s=z(e._tangram_id+"::"+i+"::"+r);if(k.programs_by_source[s])return f("trace","Reusing identical source GL program object"),k.programs_by_source[s];var n=k.createShader(e,i,e.VERTEX_SHADER),o=k.createShader(e,r,e.FRAGMENT_SHADER);if(e.useProgram(null),null!=t)for(var a=e.getAttachedShaders(t),l=0;l<a.length;l++)e.detachShader(t,a[l]);else t=e.createProgram();if(null==n||null==o)return t;if(e.attachShader(t,n),e.attachShader(t,o),e.bindAttribLocation(t,0,"a_position"),e.linkProgram(t),!e.getProgramParameter(t,e.LINK_STATUS)){let s=new Error(`WebGL program error:\n            VALIDATE_STATUS: ${e.getProgramParameter(t,e.VALIDATE_STATUS)}\n            ERROR: ${e.getError()}\n            --- Vertex Shader ---\n            ${i}\n            --- Fragment Shader ---\n            ${r}`);throw Object.assign(new Error(s),{type:"program"})}return k.programs_by_source[s]=t,t},k.createShader=function(e,t,i){let r=z(e._tangram_id+"::"+t);if(k.shaders_by_source[r])return f("trace","Reusing identical source GL shader object"),k.shaders_by_source[r];let s=e.createShader(i);if(e.shaderSource(s,t),e.compileShader(s),!e.getShaderParameter(s,e.COMPILE_STATUS)){let t=i===e.VERTEX_SHADER?"vertex":"fragment",r=e.getShaderInfoLog(s),n=N(r);throw Object.assign(new Error(r),{type:t,errors:n})}return k.shaders_by_source[r]=s,s};var L,I={disabled:!1,bound_vao:[],init(e){let t;!0!==this.disabled&&(t=M(e,"OES_vertex_array_object")),null!=t?f("info","Vertex Array Object extension available"):!0!==this.disabled?f("warn","Vertex Array Object extension NOT available"):f("warn","Vertex Array Object extension force disabled");},getExtension(e,t){if(!0!==this.disabled)return M(e,t)},create(e,t,i){let r={};r.setup=t,r.teardown=i;let s=this.getExtension(e,"OES_vertex_array_object");return null!=s&&(r._vao=s.createVertexArrayOES(),s.bindVertexArrayOES(r._vao)),r.setup(),r},getCurrentBinding(e){let t=this.bound_vao.filter(t=>t[0]===e)[0];return t&&t[1]},setCurrentBinding(e,t){let i=this.bound_vao,r=i.filter(t=>t[0]===e)[0];null==r?i.push([e,t]):r[1]=t;},bind(e,t){let i=this.getExtension(e,"OES_vertex_array_object");if(null!=t)null!=i&&null!=t._vao?(i.bindVertexArrayOES(t._vao),this.setCurrentBinding(e,t)):t.setup();else{let t=this.getCurrentBinding(e);null!=i?i.bindVertexArrayOES(null):null!=t&&"function"==typeof t.teardown&&t.teardown(),this.setCurrentBinding(e,null);}},destroy(e,t){let i=this.getExtension(e,"OES_vertex_array_object");null!=i&&null!=t&&null!=t._vao&&(i.deleteVertexArrayOES(t._vao),t._vao=null);}};function O(e){for(let t=0;t<(arguments.length<=1?0:arguments.length-1);t++){let i=t+1<1||arguments.length<=t+1?void 0:arguments[t+1];if(i)for(let t in i){let r=i[t];null===r||"object"!=typeof r||Array.isArray(r)?void 0!==r&&(e[t]=r):null===e[t]||"object"!=typeof e[t]||Array.isArray(e[t])?e[t]=O({},r):e[t]=O(e[t],r);}}return e}var F=L={};L.default_source_max_zoom=18,L.default_view_max_zoom=20,L.max_style_zoom=25,L.tile_size=256,L.half_circumference_meters=20037508.342789244,L.circumference_meters=2*L.half_circumference_meters,L.min_zoom_meters_per_pixel=L.circumference_meters/L.tile_size;let U=[];L.metersPerPixel=function(e){return U[e]=U[e]||L.min_zoom_meters_per_pixel/Math.pow(2,e),U[e]};let D=[];L.metersPerTile=function(e){return D[e]=D[e]||L.circumference_meters/Math.pow(2,e),D[e]},L.tile_scale=4096,L.units_per_pixel=L.tile_scale/L.tile_size,L.height_scale=16;let G=[];L.unitsPerMeter=function(e){return G[e]=G[e]||L.tile_scale/(L.tile_size*L.metersPerPixel(e)),G[e]},L.metersForTile=function(e){return {x:e.x*L.circumference_meters/Math.pow(2,e.z)-L.half_circumference_meters,y:-(e.y*L.circumference_meters/Math.pow(2,e.z)-L.half_circumference_meters)}},L.tileForMeters=function(e,t){let i=e[0],r=e[1];return {x:Math.floor((i+L.half_circumference_meters)/(L.circumference_meters/Math.pow(2,t))),y:Math.floor((-r+L.half_circumference_meters)/(L.circumference_meters/Math.pow(2,t))),z:t}},L.wrapTile=function(e,t){let i=e.x,r=e.y,s=e.z;void 0===t&&(t={x:!0,y:!1});var n=(1<<s)-1;return t.x&&(i&=n),t.y&&(r&=n),{x:i,y:r,z:s}},L.metersToLatLng=function(e){return e[0]/=L.half_circumference_meters,e[1]/=L.half_circumference_meters,e[1]=(2*Math.atan(Math.exp(e[1]*Math.PI))-Math.PI/2)/Math.PI,e[0]*=180,e[1]*=180,e},L.latLngToMeters=function(e){return e[1]=Math.log(Math.tan(e[1]*Math.PI/360+Math.PI/4))/Math.PI,e[1]*=L.half_circumference_meters,e[0]*=L.half_circumference_meters/180,e},L.tileSpaceToLatlng=function(e,t,i){const r=L.unitsPerMeter(t);return L.transformGeometry(e,e=>{e[0]=e[0]/r+i.x,e[1]=e[1]/r+i.y,L.metersToLatLng(e);}),e},L.copyGeometry=function(e){if(null==e)return;let t={type:e.type};return "Point"===e.type?t.coordinates=[e.coordinates[0],e.coordinates[1]]:"LineString"===e.type||"MultiPoint"===e.type?t.coordinates=e.coordinates.map(e=>[e[0],e[1]]):"Polygon"===e.type||"MultiLineString"===e.type?t.coordinates=e.coordinates.map(e=>e.map(e=>[e[0],e[1]])):"MultiPolygon"===e.type&&(t.coordinates=e.coordinates.map(e=>e.map(e=>e.map(e=>[e[0],e[1]])))),t},L.transformGeometry=function(e,t){null!=e&&("Point"===e.type?t(e.coordinates):"LineString"===e.type||"MultiPoint"===e.type?e.coordinates.forEach(t):"Polygon"===e.type||"MultiLineString"===e.type?e.coordinates.forEach(e=>e.forEach(t)):"MultiPolygon"===e.type&&e.coordinates.forEach(e=>{e.forEach(e=>e.forEach(t));}));},L.boxIntersect=function(e,t){return !(t.sw.x>e.ne.x||t.ne.x<e.sw.x||t.sw.y>e.ne.y||t.ne.y<e.sw.y)},L.findBoundingBox=function(e){for(var t=1/0,i=-1/0,r=1/0,s=-1/0,n=e[0].length,o=0;o<n;o++){var a=e[0][o];a[0]<t&&(t=a[0]),a[1]<r&&(r=a[1]),a[0]>i&&(i=a[0]),a[1]>s&&(s=a[1]);}return [t,r,i,s]},L.geometryType=function(e){return "Polygon"===e||"MultiPolygon"===e?"polygon":"LineString"===e||"MultiLineString"===e?"line":"Point"===e||"MultiPoint"===e?"point":void 0},L.centroid=function(e,t){if(void 0===t&&(t=!0),!e||0===e.length)return;let i,r=0,s=0,n=0,o=e[0],a=o.length;t&&(i=o[0],o=o.map(e=>[e[0]-i[0],e[1]-i[1]]));for(let e=0,t=a-1;e<a;t=e,e++){let i=o[e],a=o[t],l=i[1]*a[0]-a[1]*i[0];r+=(i[0]+a[0])*l,s+=(i[1]+a[1])*l,n+=3*l;}if(!n)return;let l=[r/n,s/n];return t&&(l[0]+=i[0],l[1]+=i[1]),l},L.multiCentroid=function(e){let t=0,i=null;for(let r=0;r<e.length;r++){let s=L.centroid(e[r]);s&&((i=i||[0,0])[0]+=s[0],i[1]+=s[1],t++);}return t>0&&(i[0]/=t,i[1]/=t),i},L.signedPolygonRingAreaSum=function(e){let t=0,i=e.length;for(let r=0;r<i-1;r++){let i=e[r],s=e[r+1];t+=i[0]*s[1]-s[0]*i[1];}return t+=e[i-1][0]*e[0][1]-e[0][0]*e[i-1][1]},L.polygonRingArea=function(e){return Math.abs(L.signedPolygonRingAreaSum(e))/2},L.polygonArea=function(e){if(e)return L.polygonRingArea(e[0])},L.multiPolygonArea=function(e){let t=0;for(let i=0;i<e.length;i++)t+=L.polygonArea(e[i]);return t},L.ringWinding=function(e){let t=L.signedPolygonRingAreaSum(e);return t>0?"CW":t<0?"CCW":void 0};const C={functions:{},num_functions:0,num_cached:0};function j(e,t){if("string"==typeof e)e=V(e,t);else if(null!=e&&"object"==typeof e)for(let i in e)e[i]=j(e[i],t);return e}function V(e,t){let i="string"==typeof e&&e.match(/^\s*function[^(]*\(([^)]*)\)\s*?\{([\s\S]*)\}$/m);if(i&&i.length>2)try{const r=i[2],s="function"==typeof t?t(r):r,n=z(s);if(void 0===C.functions[n]){let e=i[1].length>0&&i[1].split(",").map(e=>e.trim()).filter(e=>e);e=e.length>0?e:["context"],C.functions[n]=new Function(e.toString(),s),C.functions[n].source=r,C.num_functions++;}else C.num_cached++;return C.functions[n]}catch(t){return e}return e}function B(e,t){return e(t={exports:{}},t.exports),t.exports}var $=B(function(e,t){var i={transparent:[0,0,0,0],aliceblue:[240,248,255,1],antiquewhite:[250,235,215,1],aqua:[0,255,255,1],aquamarine:[127,255,212,1],azure:[240,255,255,1],beige:[245,245,220,1],bisque:[255,228,196,1],black:[0,0,0,1],blanchedalmond:[255,235,205,1],blue:[0,0,255,1],blueviolet:[138,43,226,1],brown:[165,42,42,1],burlywood:[222,184,135,1],cadetblue:[95,158,160,1],chartreuse:[127,255,0,1],chocolate:[210,105,30,1],coral:[255,127,80,1],cornflowerblue:[100,149,237,1],cornsilk:[255,248,220,1],crimson:[220,20,60,1],cyan:[0,255,255,1],darkblue:[0,0,139,1],darkcyan:[0,139,139,1],darkgoldenrod:[184,134,11,1],darkgray:[169,169,169,1],darkgreen:[0,100,0,1],darkgrey:[169,169,169,1],darkkhaki:[189,183,107,1],darkmagenta:[139,0,139,1],darkolivegreen:[85,107,47,1],darkorange:[255,140,0,1],darkorchid:[153,50,204,1],darkred:[139,0,0,1],darksalmon:[233,150,122,1],darkseagreen:[143,188,143,1],darkslateblue:[72,61,139,1],darkslategray:[47,79,79,1],darkslategrey:[47,79,79,1],darkturquoise:[0,206,209,1],darkviolet:[148,0,211,1],deeppink:[255,20,147,1],deepskyblue:[0,191,255,1],dimgray:[105,105,105,1],dimgrey:[105,105,105,1],dodgerblue:[30,144,255,1],firebrick:[178,34,34,1],floralwhite:[255,250,240,1],forestgreen:[34,139,34,1],fuchsia:[255,0,255,1],gainsboro:[220,220,220,1],ghostwhite:[248,248,255,1],gold:[255,215,0,1],goldenrod:[218,165,32,1],gray:[128,128,128,1],green:[0,128,0,1],greenyellow:[173,255,47,1],grey:[128,128,128,1],honeydew:[240,255,240,1],hotpink:[255,105,180,1],indianred:[205,92,92,1],indigo:[75,0,130,1],ivory:[255,255,240,1],khaki:[240,230,140,1],lavender:[230,230,250,1],lavenderblush:[255,240,245,1],lawngreen:[124,252,0,1],lemonchiffon:[255,250,205,1],lightblue:[173,216,230,1],lightcoral:[240,128,128,1],lightcyan:[224,255,255,1],lightgoldenrodyellow:[250,250,210,1],lightgray:[211,211,211,1],lightgreen:[144,238,144,1],lightgrey:[211,211,211,1],lightpink:[255,182,193,1],lightsalmon:[255,160,122,1],lightseagreen:[32,178,170,1],lightskyblue:[135,206,250,1],lightslategray:[119,136,153,1],lightslategrey:[119,136,153,1],lightsteelblue:[176,196,222,1],lightyellow:[255,255,224,1],lime:[0,255,0,1],limegreen:[50,205,50,1],linen:[250,240,230,1],magenta:[255,0,255,1],maroon:[128,0,0,1],mediumaquamarine:[102,205,170,1],mediumblue:[0,0,205,1],mediumorchid:[186,85,211,1],mediumpurple:[147,112,219,1],mediumseagreen:[60,179,113,1],mediumslateblue:[123,104,238,1],mediumspringgreen:[0,250,154,1],mediumturquoise:[72,209,204,1],mediumvioletred:[199,21,133,1],midnightblue:[25,25,112,1],mintcream:[245,255,250,1],mistyrose:[255,228,225,1],moccasin:[255,228,181,1],navajowhite:[255,222,173,1],navy:[0,0,128,1],oldlace:[253,245,230,1],olive:[128,128,0,1],olivedrab:[107,142,35,1],orange:[255,165,0,1],orangered:[255,69,0,1],orchid:[218,112,214,1],palegoldenrod:[238,232,170,1],palegreen:[152,251,152,1],paleturquoise:[175,238,238,1],palevioletred:[219,112,147,1],papayawhip:[255,239,213,1],peachpuff:[255,218,185,1],peru:[205,133,63,1],pink:[255,192,203,1],plum:[221,160,221,1],powderblue:[176,224,230,1],purple:[128,0,128,1],rebeccapurple:[102,51,153,1],red:[255,0,0,1],rosybrown:[188,143,143,1],royalblue:[65,105,225,1],saddlebrown:[139,69,19,1],salmon:[250,128,114,1],sandybrown:[244,164,96,1],seagreen:[46,139,87,1],seashell:[255,245,238,1],sienna:[160,82,45,1],silver:[192,192,192,1],skyblue:[135,206,235,1],slateblue:[106,90,205,1],slategray:[112,128,144,1],slategrey:[112,128,144,1],snow:[255,250,250,1],springgreen:[0,255,127,1],steelblue:[70,130,180,1],tan:[210,180,140,1],teal:[0,128,128,1],thistle:[216,191,216,1],tomato:[255,99,71,1],turquoise:[64,224,208,1],violet:[238,130,238,1],wheat:[245,222,179,1],white:[255,255,255,1],whitesmoke:[245,245,245,1],yellow:[255,255,0,1],yellowgreen:[154,205,50,1]};function r(e){return (e=Math.round(e))<0?0:e>255?255:e}function s(e){return e<0?0:e>1?1:e}function n(e){return "%"===e[e.length-1]?r(parseFloat(e)/100*255):r(parseInt(e))}function o(e){return "%"===e[e.length-1]?s(parseFloat(e)/100):s(parseFloat(e))}function a(e,t,i){return i<0?i+=1:i>1&&(i-=1),6*i<1?e+(t-e)*i*6:2*i<1?t:3*i<2?e+(t-e)*(2/3-i)*6:e}try{t.parseCSSColor=function(e){var t,s=e.replace(/ /g,"").toLowerCase();if(s in i)return i[s].slice();if("#"===s[0])return 4===s.length?(t=parseInt(s.substr(1),16))>=0&&t<=4095?[(3840&t)>>4|(3840&t)>>8,240&t|(240&t)>>4,15&t|(15&t)<<4,1]:null:7===s.length&&(t=parseInt(s.substr(1),16))>=0&&t<=16777215?[(16711680&t)>>16,(65280&t)>>8,255&t,1]:null;var l=s.indexOf("("),u=s.indexOf(")");if(-1!==l&&u+1===s.length){var h=s.substr(0,l),c=s.substr(l+1,u-(l+1)).split(","),d=1;switch(h){case"rgba":if(4!==c.length)return null;d=o(c.pop());case"rgb":return 3!==c.length?null:[n(c[0]),n(c[1]),n(c[2]),d];case"hsla":if(4!==c.length)return null;d=o(c.pop());case"hsl":if(3!==c.length)return null;var f=(parseFloat(c[0])%360+360)%360/360,_=o(c[1]),p=o(c[2]),m=p<=.5?p*(_+1):p+_-p*_,g=2*p-m;return [r(255*a(g,m,f+1/3)),r(255*a(g,m,f)),r(255*a(g,m,f-1/3)),d];default:return null}}return null};}catch(e){}});$.parseCSSColor;const q={},H=e=>Math.max(e,0),X=e=>isNaN(e)?0:e,W=e=>Array.isArray(e)?e.map(parseFloat).map(X):X(parseFloat(e)),Z=e=>Array.isArray(e)?e.map(W).map(H):H(W(e));Object.assign(q,{clampPositive:H,noNaN:X,parseNumber:W,parsePositiveNumber:Z}),q.wrapFunction=function(e){return `\n        var feature = context.feature.properties;\n        var global = context.global;\n        var $zoom = context.zoom;\n        var $layer = context.layer;\n        var $source = context.source;\n        var $geometry = context.geometry;\n        var $meters_per_pixel = context.meters_per_pixel;\n\n        var val = (function(){ ${e} }());\n\n        if (typeof val === 'number' && isNaN(val)) {\n            val = null; // convert NaNs to nulls\n        }\n\n        return val;\n    `},q.zeroPair=Object.freeze([0,0]),q.defaults={color:[1,1,1,1],width:1,size:1,extrude:!1,height:20,min_height:0,order:0,z:0,outline:{color:[0,0,0,0],width:0},material:{ambient:1,diffuse:1}},q.macros={"Style.color.pseudoRandomColor":function(){return [parseInt(feature.id,16)/100%1*.7,parseInt(feature.id,16)/1e4%1*.7,parseInt(feature.id,16)/1e6%1*.7,1]},"Style.color.randomColor":function(){return [.7*Math.random(),.7*Math.random(),.7*Math.random(),1]}},q.getFeatureParseContext=function(e,t,i){return {feature:e,tile:t,global:i,zoom:t.style_z,geometry:F.geometryType(e.geometry.type),meters_per_pixel:t.meters_per_pixel,meters_per_pixel_sq:t.meters_per_pixel_sq,units_per_meter_overzoom:t.units_per_meter_overzoom}};const Y={STATIC:0,DYNAMIC:1,ZOOM:2};q.CACHE_TYPE=Y,q.createPropertyCache=function(e,t){if(void 0===t&&(t=null),null==e)return;if(e.value)return {value:e.value,zoom:e.zoom?{}:null,type:e.type};let i={value:e,type:Y.STATIC};return Array.isArray(i.value)&&Array.isArray(i.value[0])?(i.zoom={},i.type=Y.ZOOM):"function"==typeof i.value&&(i.type=Y.DYNAMIC),"function"==typeof t&&(i.zoom?i.value=i.value.map((e,i)=>[e[0],t(e[1],i)]):"function"!=typeof i.value&&(i.value=t(i.value,0))),i},q.createColorPropertyCache=function(e){return q.createPropertyCache(e,e=>"Style.color.pseudoRandomColor"===e?V(q.wrapFunction(q.macros["Style.color.pseudoRandomColor"])):"Style.color.randomColor"===e?q.macros["Style.color.randomColor"]:e)};const J=e=>"string"==typeof e&&"%"===e[e.length-1],K=e=>"auto"===e,Q=e=>J(e)||K(e),ee="'size' can specify either width or height as derived from aspect ratio, but not both";function te(e,t){return q.createPropertyCache(e.value,(i,r)=>(Array.isArray(i)?(i=i.map((t,i)=>e.has_ratio[r][i]?t:Z(t)).map((i,s)=>e.has_pct[r][s]?t.css_size[s]*i/100:i),e.has_ratio[r][0]?i[0]=i[1]*t.aspect:e.has_ratio[r][1]&&(i[1]=i[0]/t.aspect)):(i=Z(i),i=e.has_pct[r]?t.css_size.map(e=>e*i/100):[i,i]),i))}function ie(e,t){try{return e(t)}catch(i){f("warn",`Property function in layer '${t.layers[t.layers.length-1]}' failed with\n`,`error ${i.stack}\n`,`function '${e.source}'\n`,t.feature,t);}}q.createPointSizePropertyCache=function(e,t){let i=null,r=null;if(J(e))i=[!0];else if(Array.isArray(e)){if(Array.isArray(e[0])){if(e.some(e=>Array.isArray(e[1])?e[1].some(e=>Q(e)):J(e[1]))&&(i=e.map(e=>Array.isArray(e[1])?e[1].map(e=>J(e)):J(e[1])),(r=e.map(e=>Array.isArray(e[1])&&e[1].map(e=>K(e)))).some(e=>Array.isArray(e)&&e.every(e=>e))))throw ee}else if(e.some(Q)&&(i=[e.map(J)],(r=[e.map(K)])[0].every(e=>e)))throw ee}else if(K(e))throw"this value only allowed as half of an array, eg [16px, auto]:";if(i||r){if(!t)throw"% or 'auto' keywords can only be used to specify point size when a texture is defined";(e={value:e}).has_pct=i,e.has_ratio=r,e.sprites={};}else e=q.createPropertyCache(e,Z);return e},q.evalCachedPointSizeProperty=function(e,t,i,r){return e.has_pct||e.has_ratio?t?(e.sprites[t.sprite]||(e.sprites[t.sprite]=te(e,t)),q.evalCachedProperty(e.sprites[t.sprite],r)):(e.texture=e.texture||te(e,i),q.evalCachedProperty(e.texture,r)):q.evalCachedProperty(e,r)},q.evalCachedProperty=function(e,t){return null==e?void 0:e.dynamic?ie(e.dynamic,t):e.static?e.static:e.zoom&&e.zoom[t.zoom]?e.zoom[t.zoom]:"function"==typeof e.value?(e.dynamic=e.value,ie(e.dynamic,t)):Array.isArray(e.value)&&Array.isArray(e.value[0])?(e.zoom=e.zoom||{},e.zoom[t.zoom]=_.interpolate(t.zoom,e.value),e.zoom[t.zoom]):(e.static=e.value,e.static)},q.convertUnits=function(e,t){if(null!=e.val)return "px"===e.units?e.val*F.metersPerPixel(t.zoom):e.val;if("string"==typeof e)"px"===e.trim().slice(-2)?(e=W(e),e*=F.metersPerPixel(t.zoom)):e=W(e);else if(Array.isArray(e))return Array.isArray(e[0])?e.map(e=>[e[0],q.convertUnits(e[1],t)]):e.map(e=>q.convertUnits(e,t));return e},q.parseUnits=function(e){var t={val:W(e)};return 0!==t.val&&"string"==typeof e&&"px"===e.trim().slice(-2)&&(t.units="px"),t},q.evalCachedDistanceProperty=function(e,t){return null==e?void 0:e.dynamic?ie(e.dynamic,t):e.zoom&&e.zoom[t.zoom]?e.zoom[t.zoom]:"function"==typeof e.value?(e.dynamic=e.value,ie(e.dynamic,t)):e.zoom?(e.zoom[t.zoom]=_.interpolate(t.zoom,e.value,e=>q.convertUnits(e,t)),e.zoom[t.zoom]):q.convertUnits(e.value,t)},q.string_colors={},q.colorForString=function(e){if(q.string_colors[e])return q.string_colors[e];let t=$.parseCSSColor(e);return t&&4===t.length?(t[0]/=255,t[1]/=255,t[2]/=255):t=q.defaults.color,q.string_colors[e]=t,t},q.evalCachedColorProperty=function(e,t){if(void 0===t&&(t={}),e.dynamic){let i=ie(e.dynamic,t);return "string"==typeof i&&(i=q.colorForString(i)),i&&null==i[3]&&(i[3]=1),i}if(e.static)return e.static;if(e.zoom&&e.zoom[t.zoom])return e.zoom[t.zoom];if("function"==typeof e.value){e.dynamic=e.value;let i=ie(e.dynamic,t);return "string"==typeof i&&(i=q.colorForString(i)),i&&null==i[3]&&(i[3]=1),i}if("string"==typeof e.value)return e.static=q.colorForString(e.value),e.static;if(e.zoom){if(!e.zoom_preprocessed){for(let t=0;t<e.value.length;t++){let i=e.value[t];i&&"string"==typeof i[1]&&(i[1]=q.colorForString(i[1]));}e.zoom_preprocessed=!0;}return e.zoom[t.zoom]=_.interpolate(t.zoom,e.value),e.zoom[t.zoom][3]=e.zoom[t.zoom][3]||1,e.zoom[t.zoom]}return e.static=e.value.map(e=>e),e.static&&null==e.static[3]&&(e.static[3]=1),e.static},q.parseColor=function(e,t){if(void 0===t&&(t={}),"function"==typeof e&&(e=ie(e,t)),"string"==typeof e)e=q.colorForString(e);else if(Array.isArray(e)&&Array.isArray(e[0])){for(let t=0;t<e.length;t++){let i=e[t];"string"==typeof i[1]&&(i[1]=q.colorForString(i[1]));}t.zoom&&(e=_.interpolate(t.zoom,e));}return Array.isArray(e)?null==(e=e.map(e=>e))[3]&&(e[3]=1):e=[0,0,0,1],e},q.calculateOrder=function(e,t){return "function"==typeof e?e=ie(e,t):"string"==typeof e&&(e=t.feature.properties[e]?t.feature.properties[e]:Z(e)),e},q.evalProperty=function(e,t){return "function"==typeof e?ie(e,t):e};class re{constructor(e,t,i){this.gl=e,this.workers=t,this._lock_fn="function"==typeof i&&i,this.init();}init(){this.requests={},this.feature=null,this.read_delay=0,this.read_delay_timer=null,this.pixels=null,this.fbo=this.gl.createFramebuffer(),this.gl.bindFramebuffer(this.gl.FRAMEBUFFER,this.fbo),this.fbo_size={width:256,height:256};var e=w.create(this.gl,"__selection_fbo",{filtering:"nearest"});e.setData(this.fbo_size.width,this.fbo_size.height,null,{filtering:"nearest"}),this.gl.framebufferTexture2D(this.gl.FRAMEBUFFER,this.gl.COLOR_ATTACHMENT0,this.gl.TEXTURE_2D,e.texture,0);var t=this.gl.createRenderbuffer();this.gl.bindRenderbuffer(this.gl.RENDERBUFFER,t),this.gl.renderbufferStorage(this.gl.RENDERBUFFER,this.gl.DEPTH_COMPONENT16,this.fbo_size.width,this.fbo_size.height),this.gl.framebufferRenderbuffer(this.gl.FRAMEBUFFER,this.gl.DEPTH_ATTACHMENT,this.gl.RENDERBUFFER,t),this.gl.bindFramebuffer(this.gl.FRAMEBUFFER,null);}destroy(){this.gl&&this.fbo&&(this.gl.deleteFramebuffer(this.fbo),this.fbo=null,this.gl.bindFramebuffer(this.gl.FRAMEBUFFER,null));}get locked(){return this._lock_fn&&this._lock_fn()||!1}bind(){this.gl.bindFramebuffer(this.gl.FRAMEBUFFER,this.fbo),this.gl.viewport(0,0,this.fbo_size.width,this.fbo_size.height),this.gl.clearColor(...re.defaultColor);}getFeatureAt(e,t){let i=t.radius;return !e||e.x<0||e.y<0||e.x>1||e.y>1?Promise.resolve({feature:null,changed:!1}):new Promise((t,r)=>{this.selection_request_id=this.selection_request_id+1||0,this.requests[this.selection_request_id]={id:this.selection_request_id,point:e,radius:i,resolve:t,reject:r};})}pendingRequests(){return Object.keys(this.requests).length?this.requests:null}hasPendingRequests(){return null!=this.pendingRequests()}clearPendingRequests(){for(var e in this.requests){var t=this.requests[e];t.sent||(t.reject({request:t}),delete this.requests[e]);}}read(){null!=this.read_delay_timer&&clearTimeout(this.read_delay_timer),this.read_delay_timer=setTimeout(()=>{if(!this.locked){var e=this.gl;for(var t in e.bindFramebuffer(e.FRAMEBUFFER,this.fbo),this.requests){var i=this.requests[t];if(i.sent)continue;let r,s,o=255,a=i.point,l=i.radius;if(l){let e=Math.min(this.fbo_size.width,this.fbo_size.height);s={x:Math.min(Math.ceil(2*l.x*this.fbo_size.width),e),y:Math.min(Math.ceil(2*l.y*this.fbo_size.height),e)};}else l={x:0,y:0},s={x:1,y:1};if((null==this.pixels||this.pixels.byteLength<s.x*s.y*4)&&(this.pixels=new Uint8Array(s.x*s.y*4)),this.pixels.fill instanceof Function)this.pixels.fill(0);else for(let e=0;e<this.pixels.length;e++)this.pixels[e]=0;e.readPixels(Math.round((a.x-l.x)*this.fbo_size.width),Math.round((1-a.y-l.y)*this.fbo_size.height),s.x,s.y,e.RGBA,e.UNSIGNED_BYTE,this.pixels);let u=4*(Math.round(s.y/2)*s.x+Math.round(s.x/2)),h=this.pixels[u]+(this.pixels[u+1]<<8)+(this.pixels[u+2]<<16);if(h>0)r=h+(this.pixels[u+3]<<24)>>>0,o=this.pixels[u+3];else{let e=-1>>>0;u=0;for(let t=0;t<s.y;t++)for(let i=0;i<s.x;i++,u+=4)if((h=this.pixels[u]+(this.pixels[u+1]<<8)+(this.pixels[u+2]<<16))>0){let n=(i-s.x/2)*(i-s.x/2)+(t-s.y/2)*(t-s.y/2);n<=e&&(r=h+(this.pixels[u+3]<<24)>>>0,o=this.pixels[u+3],e=n);}}255!==o?null!=this.workers[o]&&n.postMessage(this.workers[o],"self.getFeatureSelection",{id:i.id,key:r}).then(e=>{this.finishRead(e);}):this.finishRead({id:i.id}),i.sent=!0;}e.bindFramebuffer(e.FRAMEBUFFER,null);}},this.read_delay);}finishRead(e){var t=this.requests[e.id];if(t){var i=e.feature,r=!1;(null!=i&&null==this.feature||null==i&&null!=this.feature||null!=i&&null!=this.feature&&JSON.stringify(i)!==JSON.stringify(this.feature))&&(r=!0),this.feature=i,t.resolve({feature:i,changed:r,request:t}),delete this.requests[e.id];}else f("error","FeatureSelection.finishRead(): could not find message",e);}static makeEntry(e){this.map_entry++;var t=255&this.map_entry,i=this.map_entry>>8&255,r=this.map_entry>>16&255,s=this.map_prefix,n=t/255,o=i/255,a=r/255,l=s/255,u=t+(i<<8)+(r<<16)+(s<<24)>>>0;return this.map[u]={color:[n,o,a,l]},this.map_size++,this.tiles[e.key]||(this.tiles[e.key]={entries:[],tile:{key:e.key,coords:e.coords,style_z:e.style_z,source:e.source,generation:e.generation}}),this.tiles[e.key].entries.push(u),this.map[u]}static makeColor(e,t,i){var r=this.makeEntry(t);return r.feature={id:e.id,properties:e.properties,source_name:i.source,source_layer:i.layer,layers:i.layers,tile:this.tiles[t.key].tile},r.color}static reset(e){Array.isArray(e)?e.forEach(e=>this.clearSource(e)):(this.tiles={},this.map={},this.map_size=0,this.map_entry=0);}static clearSource(e){for(let t in this.tiles)this.tiles[t].tile.source===e&&this.clearTile(t);}static clearTile(e){this.tiles[e]&&(this.tiles[e].entries.forEach(e=>delete this.map[e]),this.map_size-=this.tiles[e].entries.length,delete this.tiles[e]);}static getMapSize(){return this.map_size}static setPrefix(e){this.map_prefix=e;}}re.map={},re.tiles={},re.map_size=0,re.map_entry=0,re.map_prefix=0,re.defaultColor=[0,0,0,1];class se{constructor(e,t,i,r,s){s=s||{},this.gl=e,this.vertex_data=t,this.element_data=i,this.vertex_layout=r,this.vertex_buffer=this.gl.createBuffer(),this.buffer_size=this.vertex_data.byteLength,this.draw_mode=s.draw_mode||this.gl.TRIANGLES,this.data_usage=s.data_usage||this.gl.STATIC_DRAW,this.vertices_per_geometry=3,this.uniforms=s.uniforms,this.textures=s.textures,this.retain=s.retain||!1,this.created_at=+new Date,this.fade_in_time=s.fade_in_time||0,this.vertex_count=this.vertex_data.byteLength/this.vertex_layout.stride,this.element_count=0,this.vaos={},this.toggle_element_array=!1,this.element_data?(this.toggle_element_array=!0,this.element_count=this.element_data.length,this.geometry_count=this.element_count/this.vertices_per_geometry,this.element_type=this.element_data.constructor===Uint16Array?this.gl.UNSIGNED_SHORT:this.gl.UNSIGNED_INT,this.element_buffer=this.gl.createBuffer(),this.buffer_size+=this.element_data.byteLength,this.gl.bindBuffer(this.gl.ELEMENT_ARRAY_BUFFER,this.element_buffer),this.gl.bufferData(this.gl.ELEMENT_ARRAY_BUFFER,this.element_data,this.data_usage)):this.geometry_count=this.vertex_count/this.vertices_per_geometry,this.upload(),this.retain||(delete this.vertex_data,delete this.element_data),this.valid=!0;}render(e){if(void 0===e&&(e={}),!this.valid)return !1;var t=e.program||k.current;t.use(),this.uniforms&&(t.saveUniforms(this.uniforms),t.setUniforms(this.uniforms,!1));let i=(+new Date-this.created_at)/1e3;return t.uniform("1f","u_visible_time",i),this.bind(t),this.toggle_element_array?this.gl.drawElements(this.draw_mode,this.element_count,this.element_type,0):this.gl.drawArrays(this.draw_mode,0,this.vertex_count),I.bind(this.gl,null),this.uniforms&&t.restoreUniforms(this.uniforms),i<this.fade_in_time}bind(e){let t=this.vaos[e.id];t?I.bind(this.gl,t):this.vaos[e.id]=I.create(this.gl,()=>{this.gl.bindBuffer(this.gl.ARRAY_BUFFER,this.vertex_buffer),this.toggle_element_array&&this.gl.bindBuffer(this.gl.ELEMENT_ARRAY_BUFFER,this.element_buffer),this.vertex_layout.enableDynamicAttributes(this.gl,e);}),this.vertex_layout.enableStaticAttributes(this.gl,e);}upload(){this.gl.bindBuffer(this.gl.ARRAY_BUFFER,this.vertex_buffer),this.gl.bufferData(this.gl.ARRAY_BUFFER,this.vertex_data,this.data_usage);}destroy(){if(!this.valid)return !1;this.valid=!1;for(let e in this.vaos)I.destroy(this.gl,this.vaos[e]);return this.gl.deleteBuffer(this.vertex_buffer),this.vertex_buffer=null,this.element_buffer&&(this.gl.deleteBuffer(this.element_buffer),this.element_buffer=null),delete this.vertex_data,delete this.element_data,this.textures&&this.textures.forEach(e=>w.release(e)),!0}}var ne="/*\n\nDefines globals:\nmaterial\nlight_accumulator_*\n\n*/\n\n\n// MATERIALS\n//\nstruct Material {\n    #ifdef TANGRAM_MATERIAL_EMISSION\n        vec4 emission;\n        #ifdef TANGRAM_MATERIAL_EMISSION_TEXTURE\n            vec3 emissionScale;\n        #endif\n    #endif\n\n    #ifdef TANGRAM_MATERIAL_AMBIENT\n        vec4 ambient;\n        #ifdef TANGRAM_MATERIAL_AMBIENT_TEXTURE\n            vec3 ambientScale;\n        #endif\n    #endif\n\n    #ifdef TANGRAM_MATERIAL_DIFFUSE\n        vec4 diffuse;\n        #ifdef TANGRAM_MATERIAL_DIFFUSE_TEXTURE\n            vec3 diffuseScale;\n        #endif\n    #endif\n\n    #ifdef TANGRAM_MATERIAL_SPECULAR\n        vec4 specular;\n        float shininess;\n        #ifdef TANGRAM_MATERIAL_SPECULAR_TEXTURE\n            vec3 specularScale;\n        #endif\n    #endif\n\n\n    #ifdef TANGRAM_MATERIAL_NORMAL_TEXTURE\n        vec3 normalScale;\n        float normalAmount;\n    #endif\n};\n\n// Note: uniform is copied to a global instance to allow modification\nuniform Material u_material;\nMaterial material;\n\n#ifdef TANGRAM_MATERIAL_EMISSION_TEXTURE\nuniform sampler2D u_material_emission_texture;\n#endif\n\n#ifdef TANGRAM_MATERIAL_AMBIENT_TEXTURE\nuniform sampler2D u_material_ambient_texture;\n#endif\n\n#ifdef TANGRAM_MATERIAL_DIFFUSE_TEXTURE\nuniform sampler2D u_material_diffuse_texture;\n#endif\n\n#ifdef TANGRAM_MATERIAL_SPECULAR_TEXTURE\nuniform sampler2D u_material_specular_texture;\n#endif\n\n#ifdef TANGRAM_MATERIAL_NORMAL_TEXTURE\nuniform sampler2D u_material_normal_texture;\n#endif\n\n// Global light accumulators for each property\nvec4 light_accumulator_ambient = vec4(vec3(0.0), 1.);\nvec4 light_accumulator_diffuse = vec4(vec3(0.0), 1.);\n#ifdef TANGRAM_MATERIAL_SPECULAR\n    vec4 light_accumulator_specular = vec4(vec3(0.0), 1.);\n#endif\n\n\n#ifdef TANGRAM_MATERIAL_TEXTURE_SPHEREMAP\nvec4 getSphereMap (in sampler2D _tex, in vec3 _eyeToPoint, in vec3 _normal, in vec2 _skew) {\n    // Normalized vector from camera to surface\n    vec3 eye = normalize(_eyeToPoint);\n\n    // Adjust for camera skew\n    eye.xy -= _skew;\n    eye = normalize(eye);\n\n    // Reflection of eye off of surface normal\n    vec3 r = reflect(eye, _normal);\n\n    // Map reflected vector onto the surface of a sphere\n    r.z += 1.0;\n    float m = 2. * length(r);\n\n    // Adjust xy to account for spherical shape, and center in middle of texture\n    vec2 uv = r.xy / m + .5;\n\n    // Sample the environment map\n    return texture2D(_tex, uv);\n}\n#endif\n\n\n#ifdef TANGRAM_MATERIAL_TEXTURE_TRIPLANAR\nvec3 getTriPlanarBlend (in vec3 _normal) {\n    vec3 blending = abs(_normal);\n    blending = normalize(max(blending, 0.00001));\n    float b = (blending.x + blending.y + blending.z);\n    return blending / b;\n}\n\nvec4 getTriPlanar (in sampler2D _tex, in vec3 _pos, in vec3 _normal, in vec3 _scale) {\n    vec3 blending = getTriPlanarBlend(_normal);\n    vec4 xaxis = texture2D(_tex, fract(_pos.yz * _scale.x));\n    vec4 yaxis = texture2D(_tex, fract(_pos.xz * _scale.y));\n    vec4 zaxis = texture2D(_tex, fract(_pos.xy * _scale.z));\n    return  xaxis * blending.x + yaxis * blending.y + zaxis * blending.z;\n}\n#endif\n\n\n#ifdef TANGRAM_MATERIAL_TEXTURE_PLANAR\nvec4 getPlanar (in sampler2D _tex, in vec3 _pos, in vec2 _scale) {\n    return texture2D( _tex, fract(_pos.xy * _scale.x) );\n}\n#endif\n\n\n#ifdef TANGRAM_MATERIAL_NORMAL_TEXTURE\nvoid calculateNormal (inout vec3 _normal) {\n    // Get NORMALMAP\n    //------------------------------------------------\n    #ifdef TANGRAM_MATERIAL_NORMAL_TEXTURE_UV\n    _normal += texture2D(u_material_normal_texture, fract(v_texcoord*material.normalScale.xy)).rgb*2.0-1.0;\n    #endif\n\n    #ifdef TANGRAM_MATERIAL_NORMAL_TEXTURE_PLANAR\n    vec3 normalTex = getPlanar(u_material_normal_texture, v_world_position.xyz, material.normalScale.xy).rgb*2.0-1.0;\n    _normal += normalTex;\n    #endif\n\n    #ifdef TANGRAM_MATERIAL_NORMAL_TEXTURE_TRIPLANAR\n    vec3 normalTex = getTriPlanar(u_material_normal_texture, v_world_position.xyz, _normal, material.normalScale).rgb*2.0-1.0;\n    _normal += normalTex;\n    #endif\n\n    _normal = normalize(_normal);\n}\n#endif\n\nvoid calculateMaterial (in vec3 _eyeToPoint, inout vec3 _normal) {\n    // get EMISSION TEXTUREMAP\n    //------------------------------------------------\n    #ifdef TANGRAM_MATERIAL_EMISSION_TEXTURE\n        #ifdef TANGRAM_MATERIAL_EMISSION_TEXTURE_UV\n        material.emission *= texture2D(u_material_emission_texture,v_texcoord);\n        #endif\n\n        #ifdef TANGRAM_MATERIAL_EMISSION_TEXTURE_PLANAR\n        material.emission *= getPlanar(u_material_emission_texture, v_world_position.xyz, material.emissionScale.xy);\n        #endif\n\n        #ifdef TANGRAM_MATERIAL_EMISSION_TEXTURE_TRIPLANAR\n        material.emission *= getTriPlanar(u_material_emission_texture, v_world_position.xyz, _normal, material.emissionScale);\n        #endif\n\n        #ifdef TANGRAM_MATERIAL_EMISSION_TEXTURE_SPHEREMAP\n        material.emission *= getSphereMap(u_material_emission_texture, _eyeToPoint, _normal, u_vanishing_point);\n        #endif\n    #endif\n\n    // get AMBIENT TEXTUREMAP\n    //------------------------------------------------\n    #ifdef TANGRAM_MATERIAL_AMBIENT_TEXTURE\n        #ifdef TANGRAM_MATERIAL_AMBIENT_TEXTURE_UV\n        material.ambient *= texture2D(u_material_ambient_texture,v_texcoord);\n        #endif\n\n        #ifdef TANGRAM_MATERIAL_AMBIENT_TEXTURE_PLANAR\n        material.ambient *= getPlanar(u_material_ambient_texture, v_world_position.xyz, material.ambientScale.xy);\n        #endif\n\n        #ifdef TANGRAM_MATERIAL_AMBIENT_TEXTURE_TRIPLANAR\n        material.ambient *= getTriPlanar(u_material_ambient_texture, v_world_position.xyz, _normal, material.ambientScale);\n        #endif\n\n        #ifdef TANGRAM_MATERIAL_AMBIENT_TEXTURE_SPHEREMAP\n        material.ambient *= getSphereMap(u_material_ambient_texture, _eyeToPoint, _normal, u_vanishing_point);\n        #endif\n    #endif\n\n    // get DIFFUSE TEXTUREMAP\n    //------------------------------------------------\n    #ifdef TANGRAM_MATERIAL_DIFFUSE_TEXTURE\n        #ifdef TANGRAM_MATERIAL_DIFFUSE_TEXTURE_UV\n        material.diffuse *= texture2D(u_material_diffuse_texture,v_texcoord);\n        #endif\n\n        #ifdef TANGRAM_MATERIAL_DIFFUSE_TEXTURE_PLANAR\n        material.diffuse *= getPlanar(u_material_diffuse_texture, v_world_position.xyz, material.diffuseScale.xy);\n        #endif\n\n        #ifdef TANGRAM_MATERIAL_DIFFUSE_TEXTURE_TRIPLANAR\n        material.diffuse *= getTriPlanar(u_material_diffuse_texture, v_world_position.xyz, _normal, material.diffuseScale);\n        #endif\n\n        #ifdef TANGRAM_MATERIAL_DIFFUSE_TEXTURE_SPHEREMAP\n        material.diffuse *= getSphereMap(u_material_diffuse_texture, _eyeToPoint, _normal, u_vanishing_point);\n        #endif\n    #endif\n\n    // get SPECULAR TEXTUREMAP\n    //------------------------------------------------\n    #ifdef TANGRAM_MATERIAL_SPECULAR_TEXTURE\n        #ifdef TANGRAM_MATERIAL_SPECULAR_TEXTURE_UV\n        material.specular *= texture2D(u_material_specular_texture,v_texcoord);\n        #endif\n\n        #ifdef TANGRAM_MATERIAL_SPECULAR_TEXTURE_PLANAR\n        material.specular *= getPlanar(u_material_specular_texture, v_world_position.xyz, material.specularScale.xy);\n        #endif\n\n        #ifdef TANGRAM_MATERIAL_SPECULAR_TEXTURE_TRIPLANAR\n        material.specular *= getTriPlanar(u_material_specular_texture, v_world_position.xyz, _normal, material.specularScale);\n        #endif\n\n        #ifdef TANGRAM_MATERIAL_SPECULAR_TEXTURE_SPHEREMAP\n        material.specular *= getSphereMap(u_material_specular_texture, _eyeToPoint, _normal, u_vanishing_point);\n        #endif\n    #endif\n}\n";const oe=["emission","ambient","diffuse","specular"];class ae{constructor(e){e=e||{},oe.forEach(t=>{const i=e[t];null!=i&&(i.texture?this[t]={texture:i.texture,mapping:i.mapping||"spheremap",scale:E.expandVec3(null!=i.scale?i.scale:1),amount:E.expandVec4(null!=i.amount?i.amount:1)}:"number"==typeof i||Array.isArray(i)?this[t]={amount:E.expandVec4(i)}:this[t]="string"==typeof i?{amount:q.parseColor(i)}:i);}),this.specular&&(this.specular.shininess=e.shininess?parseFloat(e.shininess):.2),null!=e.normal&&(this.normal={texture:e.normal.texture,mapping:e.normal.mapping||"triplanar",scale:E.expandVec3(null!=e.normal.scale?e.normal.scale:1),amount:null!=e.normal.amount?e.normal.amount:1});}static isValid(e){return null!=e&&(null!=e.emission||null!=e.ambient||null!=e.diffuse||null!=e.specular)}inject(e){oe.forEach(t=>{let i=`TANGRAM_MATERIAL_${t.toUpperCase()}`,r=i+"_TEXTURE";e.defines[i]=null!=this[t],this[t]&&this[t].texture&&(e.defines[r]=!0,e.defines[r+"_"+this[t].mapping.toUpperCase()]=!0,e.defines[`TANGRAM_MATERIAL_TEXTURE_${this[t].mapping.toUpperCase()}`]=!0,e.texcoords=e.texcoords||"uv"===this[t].mapping);}),this.normal&&this.normal.texture&&(e.defines.TANGRAM_MATERIAL_NORMAL_TEXTURE=!0,e.defines["TANGRAM_MATERIAL_NORMAL_TEXTURE_"+this.normal.mapping.toUpperCase()]=!0,e.defines[`TANGRAM_MATERIAL_TEXTURE_${this.normal.mapping.toUpperCase()}`]=!0,e.texcoords=e.texcoords||"uv"===this.normal.mapping),e.replaceShaderBlock(ae.block,ne,"Material"),e.addShaderBlock("setup","\nmaterial = u_material;\n","Material");}setupProgram(e){oe.forEach(t=>{this[t]&&(this[t].texture?(e.setTextureUniform(`u_material_${t}_texture`,this[t].texture),e.uniform("3fv",`u_material.${t}Scale`,this[t].scale),e.uniform("4fv",`u_material.${t}`,this[t].amount)):this[t].amount&&e.uniform("4fv",`u_material.${t}`,this[t].amount));}),this.specular&&e.uniform("1f","u_material.shininess",this.specular.shininess),this.normal&&this.normal.texture&&(e.setTextureUniform("u_material_normal_texture",this.normal.texture),e.uniform("3fv","u_material.normalScale",this.normal.scale),e.uniform("1f","u_material.normalAmount",this.normal.amount));}}var le;ae.block="material";var ue=le={};le.copy=function(e){for(var t=[],i=e.length,r=0;r<i;r++)t[r]=e[r];return t},le.neg=function(e){for(var t=[],i=e.length,r=0;r<i;r++)t[r]=-e[r];return t},le.add=function(e,t){for(var i=[],r=Math.min(e.length,t.length),s=0;s<r;s++)i[s]=e[s]+t[s];return i},le.sub=function(e,t){for(var i=[],r=Math.min(e.length,t.length),s=0;s<r;s++)i[s]=e[s]-t[s];return i},le.signed_area=function(e,t,i){return (t[0]-e[0])*(i[1]-e[1])-(i[0]-e[0])*(t[1]-e[1])},le.mult=function(e,t){var i,r=[],s=e.length;if("number"==typeof t)for(i=0;i<s;i++)r[i]=e[i]*t;else for(s=Math.min(e.length,t.length),i=0;i<s;i++)r[i]=e[i]*t[i];return r},le.div=function(e,t){var i,r=[];if("number"==typeof t)for(i=0;i<e.length;i++)r[i]=e[i]/t;else{var s=Math.min(e.length,t.length);for(i=0;i<s;i++)r[i]=e[i]/t[i];}return r},le.perp=function(e,t){return [t[1]-e[1],e[0]-t[0]]},le.rot=function(e,t){var i=Math.cos(t),r=Math.sin(t);return [e[0]*i-e[1]*r,e[0]*r+e[1]*i]},le.angle=function(e){let t=e[0],i=e[1];return Math.atan2(i,t)},le.angleBetween=function(e,t){var i=le.dot(le.normalize(le.copy(e)),le.normalize(le.copy(t)));return i>1&&(i=1),Math.acos(i)},le.isEqual=function(e,t){for(var i=e.length,r=0;r<i;r++)if(e[r]!==t[r])return !1;return !0},le.lengthSq=function(e){return 2===e.length?e[0]*e[0]+e[1]*e[1]:e.length>=3?e[0]*e[0]+e[1]*e[1]+e[2]*e[2]:0},le.length=function(e){return Math.sqrt(le.lengthSq(e))},le.normalize=function(e){var t;if(2===e.length){if(1===(t=e[0]*e[0]+e[1]*e[1]))return e;0!==(t=Math.sqrt(t))?(e[0]/=t,e[1]/=t):(e[0]=0,e[1]=0);}else if(e.length>=3){if(1===(t=e[0]*e[0]+e[1]*e[1]+e[2]*e[2]))return e;0!==(t=Math.sqrt(t))?(e[0]/=t,e[1]/=t,e[2]/=t):(e[0]=0,e[1]=0,e[2]=0);}return e},le.cross=function(e,t){return 2===e.length?e[0]*t[1]-e[1]*t[0]:3===e.length?[e[1]*t[2]-e[2]*t[1],e[2]*t[0]-e[0]*t[2],e[0]*t[1]-e[1]*t[0]]:void 0},le.dot=function(e,t){for(var i=0,r=Math.min(e.length,t.length),s=0;s<r;s++)i+=e[s]*t[s];return i};var he="/*\n\nExpected globals:\nlight_accumulator_*\n\n*/\n\nstruct AmbientLight {\n    vec3 ambient;\n};\n\nvoid calculateLight(in AmbientLight _light, in vec3 _eyeToPoint, in vec3 _normal) {\n    light_accumulator_ambient.rgb += _light.ambient;\n}\n",ce="/*\n\nExpected globals:\nmaterial\nlight_accumulator_*\n\n*/\n\nstruct DirectionalLight {\n    vec3 ambient;\n    vec3 diffuse;\n    vec3 specular;\n    vec3 direction;\n};\n\nvoid calculateLight(in DirectionalLight _light, in vec3 _eyeToPoint, in vec3 _normal) {\n\n    light_accumulator_ambient.rgb += _light.ambient;\n\n    float nDotVP = clamp(dot(_normal, -_light.direction), 0.0, 1.0);\n\n    #ifdef TANGRAM_MATERIAL_DIFFUSE\n        light_accumulator_diffuse.rgb += _light.diffuse * nDotVP;\n    #endif\n\n    #ifdef TANGRAM_MATERIAL_SPECULAR\n        float pf = 0.0;\n        if (nDotVP > 0.0) {\n            vec3 reflectVector = reflect(_light.direction, _normal);\n            float eyeDotR = max(dot(normalize(_eyeToPoint), reflectVector), 0.0);\n            pf = pow(eyeDotR, material.shininess);\n        }\n        light_accumulator_specular.rgb += _light.specular * pf;\n    #endif\n}\n",de="/*\n\nExpected globals:\nmaterial\nlight_accumulator_*\n\n*/\n\nstruct PointLight {\n    vec3 ambient;\n    vec3 diffuse;\n    vec3 specular;\n    vec4 position;\n\n#ifdef TANGRAM_POINTLIGHT_ATTENUATION_EXPONENT\n    float attenuationExponent;\n#endif\n\n#ifdef TANGRAM_POINTLIGHT_ATTENUATION_INNER_RADIUS\n    float innerRadius;\n#endif\n\n#ifdef TANGRAM_POINTLIGHT_ATTENUATION_OUTER_RADIUS\n    float outerRadius;\n#endif\n};\n\nvoid calculateLight(in PointLight _light, in vec3 _eyeToPoint, in vec3 _normal) {\n\n    float dist = length(_light.position.xyz - _eyeToPoint);\n\n    // Compute vector from surface to light position\n    vec3 VP = (_light.position.xyz - _eyeToPoint) / dist;\n\n    // Normalize the vector from surface to light position\n    float nDotVP = clamp(dot(VP, _normal), 0.0, 1.0);\n\n    // Attenuation defaults\n    float attenuation = 1.0;\n    #ifdef TANGRAM_POINTLIGHT_ATTENUATION_EXPONENT\n        float Rin = 1.0;\n        float e = _light.attenuationExponent;\n\n        #ifdef TANGRAM_POINTLIGHT_ATTENUATION_INNER_RADIUS\n            Rin = _light.innerRadius;\n        #endif\n\n        #ifdef TANGRAM_POINTLIGHT_ATTENUATION_OUTER_RADIUS\n            float Rdiff = _light.outerRadius-Rin;\n            float d = clamp(max(0.0,dist-Rin)/Rdiff, 0.0, 1.0);\n            attenuation = 1.0-(pow(d,e));\n        #else\n            // If no outer is provide behaves like:\n            // https://imdoingitwrong.wordpress.com/2011/01/31/light-attenuation/\n            float d = max(0.0,dist-Rin)/Rin+1.0;\n            attenuation = clamp(1.0/(pow(d,e)), 0.0, 1.0);\n        #endif\n    #else\n        float Rin = 0.0;\n\n        #ifdef TANGRAM_POINTLIGHT_ATTENUATION_INNER_RADIUS\n            Rin = _light.innerRadius;\n            #ifdef TANGRAM_POINTLIGHT_ATTENUATION_OUTER_RADIUS\n                float Rdiff = _light.outerRadius-Rin;\n                float d = clamp(max(0.0,dist-Rin)/Rdiff, 0.0, 1.0);\n                attenuation = 1.0-d*d;\n            #else\n                // If no outer is provide behaves like:\n                // https://imdoingitwrong.wordpress.com/2011/01/31/light-attenuation/\n                float d = max(0.0,dist-Rin)/Rin+1.0;\n                attenuation = clamp(1.0/d, 0.0, 1.0);\n            #endif\n        #else\n            #ifdef TANGRAM_POINTLIGHT_ATTENUATION_OUTER_RADIUS\n                float d = clamp(dist/_light.outerRadius, 0.0, 1.0);\n                attenuation = 1.0-d*d;\n            #else\n                attenuation = 1.0;\n            #endif\n        #endif\n    #endif\n\n    // Computer accumulators\n    light_accumulator_ambient.rgb += _light.ambient * attenuation;\n\n    #ifdef TANGRAM_MATERIAL_DIFFUSE\n        light_accumulator_diffuse.rgb += _light.diffuse * nDotVP * attenuation;\n    #endif\n\n    #ifdef TANGRAM_MATERIAL_SPECULAR\n        float pf = 0.0; // power factor for shiny speculars\n        if (nDotVP > 0.0) {\n            vec3 reflectVector = reflect(-VP, _normal);\n            float eyeDotR = max(0.0, dot(-normalize(_eyeToPoint), reflectVector));\n            pf = pow(eyeDotR, material.shininess);\n        }\n\n        light_accumulator_specular.rgb += _light.specular * pf * attenuation;\n    #endif\n}\n",fe="/*\n\nExpected globals:\nmaterial\nlight_accumulator_*\n\n*/\n\nstruct SpotLight {\n    vec3 ambient;\n    vec3 diffuse;\n    vec3 specular;\n    vec4 position;\n\n#ifdef TANGRAM_POINTLIGHT_ATTENUATION_EXPONENT\n    float attenuationExponent;\n#endif\n\n#ifdef TANGRAM_POINTLIGHT_ATTENUATION_INNER_RADIUS\n    float innerRadius;\n#endif\n\n#ifdef TANGRAM_POINTLIGHT_ATTENUATION_OUTER_RADIUS\n    float outerRadius;\n#endif\n\n    vec3 direction;\n    float spotCosCutoff;\n    float spotExponent;\n};\n\nvoid calculateLight(in SpotLight _light, in vec3 _eyeToPoint, in vec3 _normal) {\n\n    float dist = length(_light.position.xyz - _eyeToPoint);\n\n    // Compute vector from surface to light position\n    vec3 VP = (_light.position.xyz - _eyeToPoint) / dist;\n\n    // normal . light direction\n    float nDotVP = clamp(dot(_normal, VP), 0.0, 1.0);\n\n    // Attenuation defaults\n    float attenuation = 1.0;\n    #ifdef TANGRAM_POINTLIGHT_ATTENUATION_EXPONENT\n        float Rin = 1.0;\n        float e = _light.attenuationExponent;\n\n        #ifdef TANGRAM_POINTLIGHT_ATTENUATION_INNER_RADIUS\n            Rin = _light.innerRadius;\n        #endif\n\n        #ifdef TANGRAM_POINTLIGHT_ATTENUATION_OUTER_RADIUS\n            float Rdiff = _light.outerRadius-Rin;\n            float d = clamp(max(0.0,dist-Rin)/Rdiff, 0.0, 1.0);\n            attenuation = 1.0-(pow(d,e));\n        #else\n            // If no outer is provide behaves like:\n            // https://imdoingitwrong.wordpress.com/2011/01/31/light-attenuation/\n            float d = max(0.0,dist-Rin)/Rin+1.0;\n            attenuation = clamp(1.0/(pow(d,e)), 0.0, 1.0);\n        #endif\n    #else\n        float Rin = 0.0;\n\n        #ifdef TANGRAM_POINTLIGHT_ATTENUATION_INNER_RADIUS\n            Rin = _light.innerRadius;\n            #ifdef TANGRAM_POINTLIGHT_ATTENUATION_OUTER_RADIUS\n                float Rdiff = _light.outerRadius-Rin;\n                float d = clamp(max(0.0,dist-Rin)/Rdiff, 0.0, 1.0);\n                attenuation = 1.0-d*d;\n            #else\n                // If no outer is provide behaves like:\n                // https://imdoingitwrong.wordpress.com/2011/01/31/light-attenuation/\n                float d = max(0.0,dist-Rin)/Rin+1.0;\n                attenuation = clamp(1.0/d, 0.0, 1.0);\n            #endif\n        #else\n            #ifdef TANGRAM_POINTLIGHT_ATTENUATION_OUTER_RADIUS\n                float d = clamp(dist/_light.outerRadius, 0.0, 1.0);\n                attenuation = 1.0-d*d;\n            #else\n                attenuation = 1.0;\n            #endif\n        #endif\n    #endif\n\n    // spotlight attenuation factor\n    float spotAttenuation = 0.0;\n\n    // See if point on surface is inside cone of illumination\n    float spotDot = clamp(dot(-VP, _light.direction), 0.0, 1.0);\n\n    if (spotDot >= _light.spotCosCutoff) {\n        spotAttenuation = pow(spotDot, _light.spotExponent);\n    }\n\n    light_accumulator_ambient.rgb += _light.ambient * attenuation * spotAttenuation;\n\n    #ifdef TANGRAM_MATERIAL_DIFFUSE\n        light_accumulator_diffuse.rgb += _light.diffuse * nDotVP * attenuation * spotAttenuation;\n    #endif\n\n    #ifdef TANGRAM_MATERIAL_SPECULAR\n        // Power factor for shiny speculars\n        float pf = 0.0;\n        if (nDotVP > 0.0) {\n            vec3 reflectVector = reflect(-VP, _normal);\n            float eyeDotR = max(dot(-normalize(_eyeToPoint), reflectVector), 0.0);\n            pf = pow(eyeDotR, material.shininess);\n        }\n        light_accumulator_specular.rgb += _light.specular * pf * attenuation * spotAttenuation;\n    #endif\n}\n";class _e{constructor(e,t){this.name=t.name,this.view=e,null==t.ambient||"number"==typeof t.ambient?this.ambient=E.expandVec3(t.ambient||0):this.ambient=q.parseColor(t.ambient).slice(0,3),null==t.diffuse||"number"==typeof t.diffuse?this.diffuse=E.expandVec3(null!=t.diffuse?t.diffuse:1):this.diffuse=q.parseColor(t.diffuse).slice(0,3),null==t.specular||"number"==typeof t.specular?this.specular=E.expandVec3(t.specular||0):this.specular=q.parseColor(t.specular).slice(0,3);}static create(e,t){if(_e.types[t.type])return new _e.types[t.type](e,t)}static setMode(e,t){!0===e&&(e="fragment"),e=_e.enabled&&(null!=e?e:"fragment"),t.defines.TANGRAM_LIGHTING_FRAGMENT="fragment"===e,t.defines.TANGRAM_LIGHTING_VERTEX="vertex"===e;}static inject(e){if(k.removeBlock(_e.block),!_e.enabled)return;let t="";if(e&&Object.keys(e).length>0){let i={};for(let t in e)i[e[t].type]=!0;for(let e in i)_e.types[e].inject();for(let i in e)e[i].inject(),t+=`calculateLight(${i}, _eyeToPoint, _normal);\n`;}let i=`\n            vec4 calculateLighting(in vec3 _eyeToPoint, in vec3 _normal, in vec4 _color) {\n\n                // Do initial material calculations over normal, emission, ambient, diffuse and specular values\n                calculateMaterial(_eyeToPoint,_normal);\n\n                // Un roll the loop of individual ligths to calculate\n                ${t}\n\n                //  Final light intensity calculation\n                vec4 color = vec4(vec3(0.), _color.a); // start with vertex color alpha\n\n                #ifdef TANGRAM_MATERIAL_EMISSION\n                    color.rgb = material.emission.rgb;\n                    color.a *= material.emission.a;\n                #endif\n\n                #ifdef TANGRAM_MATERIAL_AMBIENT\n                    color.rgb += light_accumulator_ambient.rgb * _color.rgb * material.ambient.rgb;\n                    color.a *= material.ambient.a;\n                #else\n                    #ifdef TANGRAM_MATERIAL_DIFFUSE\n                        color.rgb += light_accumulator_ambient.rgb * _color.rgb * material.diffuse.rgb;\n                    #endif\n                #endif\n\n                #ifdef TANGRAM_MATERIAL_DIFFUSE\n                    color.rgb += light_accumulator_diffuse.rgb * _color.rgb * material.diffuse.rgb;\n                    color.a *= material.diffuse.a;\n                #endif\n\n                #ifdef TANGRAM_MATERIAL_SPECULAR\n                    color.rgb += light_accumulator_specular.rgb * material.specular.rgb;\n                    color.a *= material.specular.a;\n                #endif\n\n                // Clamp final color\n                color = clamp(color, 0.0, 1.0);\n\n                return color;\n            }`;k.addBlock(_e.block,i);}inject(){let e=`\n            uniform ${this.struct_name} u_${this.name};\n            ${this.struct_name} ${this.name};\n            `,t=`\n            ${this.name} = u_${this.name};\n\n        `;k.addBlock(_e.block,e),k.addBlock("setup",t);}update(){}setupProgram(e){e.uniform("3fv",`u_${this.name}.ambient`,this.ambient),e.uniform("3fv",`u_${this.name}.diffuse`,this.diffuse),e.uniform("3fv",`u_${this.name}.specular`,this.specular);}}_e.types={},_e.block="lighting",_e.enabled=!0;_e.types.ambient=class extends _e{constructor(e,t){super(e,t),this.type="ambient",this.struct_name="AmbientLight";}static inject(){k.addBlock(_e.block,he);}setupProgram(e){e.uniform("3fv",`u_${this.name}.ambient`,this.ambient);}};_e.types.directional=class extends _e{constructor(e,t){if(super(e,t),this.type="directional",this.struct_name="DirectionalLight",t.direction)this._direction=t.direction;else{let e=135,i=Math.sin(60*Math.PI/180);this._direction=[Math.cos(Math.PI*e/180)*i,Math.sin(Math.PI*e/180)*i,-.5],null==t.ambient&&(this.ambient=E.expandVec3(.5));}this.direction=this._direction.map(parseFloat);}get direction(){return this._direction}set direction(e){this._direction=ue.normalize(ue.copy(e));}static inject(){k.addBlock(_e.block,ce);}setupProgram(e){super.setupProgram(e),e.uniform("3fv",`u_${this.name}.direction`,this.direction);}};class pe extends _e{constructor(e,t){super(e,t),this.type="point",this.struct_name="PointLight",this.position=t.position||[0,0,"100px"],this.position_eye=[],this.origin=t.origin||"ground",this.attenuation=isNaN(parseFloat(t.attenuation))?0:parseFloat(t.attenuation),t.radius?Array.isArray(t.radius)&&2===t.radius.length?this.radius=t.radius:this.radius=[null,t.radius]:this.radius=null;}static inject(){k.addBlock(_e.block,de);}inject(){super.inject(),k.defines.TANGRAM_POINTLIGHT_ATTENUATION_EXPONENT=0!==this.attenuation,k.defines.TANGRAM_POINTLIGHT_ATTENUATION_INNER_RADIUS=null!=this.radius&&null!=this.radius[0],k.defines.TANGRAM_POINTLIGHT_ATTENUATION_OUTER_RADIUS=null!=this.radius;}update(){this.updateEyePosition();}updateEyePosition(){if("world"===this.origin){const e=F.latLngToMeters([...this.position]);this.position_eye[0]=e[0]-this.view.camera.position_meters[0],this.position_eye[1]=e[1]-this.view.camera.position_meters[1],this.position_eye[2]=q.convertUnits(this.position[2],{zoom:this.view.zoom,meters_per_pixel:F.metersPerPixel(this.view.zoom)}),this.position_eye[2]=this.position_eye[2]-this.view.camera.position_meters[2];}else"ground"!==this.origin&&"camera"!==this.origin||(this.position_eye=q.convertUnits(this.position,{zoom:this.view.zoom,meters_per_pixel:F.metersPerPixel(this.view.zoom)}),"ground"===this.origin&&(this.position_eye[2]=this.position_eye[2]-this.view.camera.position_meters[2]));this.position_eye[3]=1;}setupProgram(e){super.setupProgram(e),e.uniform("4fv",`u_${this.name}.position`,this.position_eye),k.defines.TANGRAM_POINTLIGHT_ATTENUATION_EXPONENT&&e.uniform("1f",`u_${this.name}.attenuationExponent`,this.attenuation),k.defines.TANGRAM_POINTLIGHT_ATTENUATION_INNER_RADIUS&&e.uniform("1f",`u_${this.name}.innerRadius`,q.convertUnits(this.radius[0],{zoom:this.view.zoom,meters_per_pixel:F.metersPerPixel(this.view.zoom)})),k.defines.TANGRAM_POINTLIGHT_ATTENUATION_OUTER_RADIUS&&e.uniform("1f",`u_${this.name}.outerRadius`,q.convertUnits(this.radius[1],{zoom:this.view.zoom,meters_per_pixel:F.metersPerPixel(this.view.zoom)}));}}_e.types.point=pe;_e.types.spotlight=class extends pe{constructor(e,t){super(e,t),this.type="spotlight",this.struct_name="SpotLight",this.direction=this._direction=(t.direction||[0,0,-1]).map(parseFloat),this.exponent=t.exponent?parseFloat(t.exponent):.2,this.angle=t.angle?parseFloat(t.angle):20;}get direction(){return this._direction}set direction(e){this._direction=ue.normalize(ue.copy(e));}static inject(){k.addBlock(_e.block,fe);}setupProgram(e){super.setupProgram(e),e.uniform("3fv",`u_${this.name}.direction`,this.direction),e.uniform("1f",`u_${this.name}.spotCosCutoff`,Math.cos(3.14159*this.angle/180)),e.uniform("1f",`u_${this.name}.spotExponent`,this.exponent);}};class me extends Error{constructor(e){super(),this.name="MethodNotImplemented",this.message="Method "+e+" must be implemented in subclass";}}class ge{constructor(e,t){if(this.validate(e),this.config=e,this.sources=t,this.id=e.id,this.name=e.name,this.pad_scale=e.pad_scale||1e-5,this.default_winding=null,this.rasters=[],Array.isArray(e.rasters)&&e.rasters.forEach(e=>{-1===this.rasters.indexOf(e)&&this.rasters.push(e);}),this.preprocess=e.preprocess,"function"==typeof this.preprocess&&this.preprocess.bind(this),this.transform=e.transform,"function"==typeof this.transform&&this.transform.bind(this),this.extra_data=e.extra_data,this.scripts=e.scripts,this.setZooms(e),this.zoom_offset=null!=e.zoom_offset?e.zoom_offset:0,this.zoom_offset<0){f({level:"warn",once:!0},`Data source '${this.name}' zoom_offset must not be negative – setting to 0.`),this.zoom_offset=0;}this.setTileSize(e.tile_size),this.min_display_zoom=Math.max(e.min_display_zoom||0,this.zooms[0]),this.max_display_zoom=null!=e.max_display_zoom?e.max_display_zoom:null;}static register(e,t){e&&t&&(ge.types[e]=t);}static create(e,t){if("function"==typeof ge.types[e.type]){const i=ge.types[e.type](e);if(i)return new i(e,t)}}static tileLayoutChanged(e,t){if(!e||!t)return !0;const i=["max_zoom","zooms","min_display_zoom","max_display_zoom","bounds","tile_size","zoom_offset"],r=T(e.config,i),s=T(t.config,i);return JSON.stringify(r)!==JSON.stringify(s)}static projectData(e){var t=+new Date;for(var i in e.layers)for(var r=e.layers[i].features.length,s=0;s<r;s++){var n=e.layers[i].features[s];F.transformGeometry(n.geometry,this.projectCoord);}void 0!==e.debug&&(e.debug.projection=+new Date-t);}static projectCoord(e){F.latLngToMeters(e);}static scaleData(e,t){let i=t.coords.z,r=t.min,s=F.unitsPerMeter(i);for(var n in e.layers)for(var o=e.layers[n].features.length,a=0;a<o;a++){var l=e.layers[n].features[a];F.transformGeometry(l.geometry,e=>{e[0]=(e[0]-r.x)*s,e[1]=(e[1]-r.y)*s*-1;});}}load(e){return e.source_data={},e.source_data.layers={},e.pad_scale=this.pad_scale,e.rasters=[...this.rasters],this._load(e).then(e=>{for(let t in e.source_data.layers){let i=e.source_data.layers[t];i&&i.features&&i.features.forEach(e=>{F.transformGeometry(e.geometry,e=>{e[1]=-e[1],this.pad_scale&&(e[0]=Math.round(e[0]*(1+this.pad_scale)-F.tile_scale*this.pad_scale/2),e[1]=Math.round(e[1]*(1+this.pad_scale)-F.tile_scale*this.pad_scale/2));}),this.updateDefaultWinding(e.geometry);});}return e.default_winding=this.default_winding||"CCW",e})}_load(){throw new me("_load")}copyTileData(e,t){return f("trace",`Copy tile data from ${e.key} to ${t.key}`),t.source_data={layers:e.source_data.layers},t.rasters=[...e.rasters],t.pad_scale=e.pad_scale,t.default_winding=e.default_winding,t}setZooms(e){let t=e.max_zoom,i=e.zooms;if(this.max_zoom=null!=t?t:F.default_source_max_zoom,Array.isArray(i))this.zooms=i,this.max_zoom=this.zooms[this.zooms.length-1];else{this.zooms=[];for(let e=0;e<=this.max_zoom;e++)this.zooms[e]=e;}}setTileSize(e){this.tile_size=e||256,("number"!=typeof this.tile_size||this.tile_size<256||!_.isPowerOf2(this.tile_size))&&(f({level:"warn",once:!0},`Data source '${this.name}': 'tile_size' parameter must be a number that is a power of 2 greater than or equal to 256, but was '${e}'`),this.tile_size=256),this.zoom_bias=Math.log2(this.tile_size)-8+this.zoom_offset;}updateDefaultWinding(e){return null==this.default_winding&&("Polygon"===e.type?this.default_winding=F.ringWinding(e.coordinates[0]):"MultiPolygon"===e.type&&(this.default_winding=F.ringWinding(e.coordinates[0][0]))),this.default_winding}includesTile(e,t){if(e.z<this.min_display_zoom||null!=this.max_display_zoom&&t>this.max_display_zoom)return !1;for(let t=0;t<this.rasters.length;t++){const i=this.rasters[t];if(this.sources[i]&&this.sources[i]!==this&&!this.sources[i].includesTile(e,e.z))return !1}return !0}validate(){}}ge.types={};let xe=0;class ye extends ge{constructor(e,t){super(e,t),this.response_type="";let i=function(e,t){if(!t||0===Object.keys(t).length)return [e,[]];var i=e.indexOf("?"),r=e.indexOf("#"),s="";r>-1&&(s=e.slice(r),e=e.slice(0,r)),-1===i&&(i=e.length,e+="?"),i++;var n="",o=[];for(var a in t)""===v(a,e)?n+=`${a}=${t[a]}&`:o.push([a,t[a]]);return [e=e.slice(0,i)+n+e.slice(i)+s,o]}(e.url,e.url_params),r=i[0],s=i[1];this.url=r,s.forEach(t=>{let i=t[0],r=t[1];f({level:"warn",once:!0},`Data source '${this.name}': parameter '${i}' already present in URL '${e.url}', `+`skipping value '${i}=${r}' specified in 'url_params'`);}),e.request_headers&&"object"==typeof e.request_headers&&(this.request_headers=e.request_headers);}_load(e){let t=this.formatURL(this.url,e),i=e.source_data;return i.url=t,e.debug=e.debug||{},e.debug.network=+new Date,new Promise(r=>{let s=xe+++"-"+t,n=_.io(t,6e4,this.response_type,"GET",this.request_headers,s);i.request_id=s,i.error=null,n.then(t=>{e.debug.response_size=t.length||t.byteLength,e.debug.network=+new Date-e.debug.network,e.debug.parsing=+new Date,"function"==typeof this.preprocess&&(t=this.preprocess(t)),(t=t instanceof Promise?t:Promise.resolve(t)).then(t=>{this.parseSourceData(e,i,t),e.debug.parsing=+new Date-e.debug.parsing,r(e);});}).catch(t=>{i.error=t.stack,r(e);});})}validate(e){if("string"!=typeof e.url)throw Error("Network data source must provide a string `url` property")}formatURL(){throw new me("formatURL")}parseSourceData(){throw new me("parseSourceData")}}class ve extends ye{constructor(e,t){super(e,t),this.tiled=!0,this.bounds=this.parseBounds(e),this.builds_geometry_tiles=!1,this.tms=!0===e.tms,this.url&&this.url.search("{s}")>-1&&(Array.isArray(e.url_subdomains)&&e.url_subdomains.length>0?(this.url_subdomains=e.url_subdomains,this.next_url_subdomain=0):f({level:"warn",once:!0},`Data source '${this.name}': source URL includes '{s}' subdomain marker ('${this.url}'), but no subdomains `+"were specified in 'url_subdomains' parameter")),this.url&&this.url.search("{r}")>-1&&(Array.isArray(e.url_density_scales)&&e.url_density_scales.length>0?this.url_density_scales=e.url_density_scales:this.url_density_scales=[1,2]);}parseBounds(e){if(Array.isArray(e.bounds)&&4===e.bounds.length){const t=e.bounds,i=t[0],r=t[1],s=t[2],n=t[3];return {latlng:[...e.bounds],meters:{min:F.latLngToMeters([i,n]),max:F.latLngToMeters([s,r])},tiles:{min:{},max:{}}}}}checkBounds(e,t){if(t){e=F.wrapTile(e);let i=t.tiles.min[e.z];i||(i=t.tiles.min[e.z]=F.tileForMeters(t.meters.min,e.z));let r=t.tiles.max[e.z];if(r||(r=t.tiles.max[e.z]=F.tileForMeters(t.meters.max,e.z)),e.y<i.y||e.y>r.y)return !1;if(i.x<=r.x){if(e.x<i.x||e.x>r.x)return !1}else if(e.x>r.x&&e.x<i.x)return !1}return !0}includesTile(e,t){return !!super.includesTile(e,t)&&!!this.checkBounds(e,this.bounds)}formatURL(e,t){let i=F.wrapTile(t.coords,{x:!0});this.tms&&(i.y=Math.pow(2,i.z)-1-i.y);let r=e.replace("{x}",i.x).replace("{y}",i.y).replace("{z}",i.z).replace("{r}",this.getDensityModifier()).replace("{q}",this.toQuadKey(i));return null!=this.url_subdomains&&(r=r.replace("{s}",this.url_subdomains[this.next_url_subdomain]),this.next_url_subdomain=(this.next_url_subdomain+1)%this.url_subdomains.length),r}getDensityModifier(){if(this.url_density_scales){const e=_.device_pixel_ratio;let t=this.url_density_scales.filter(t=>e>=t).reverse()[0];if((t=null!=t?t:this.url_density_scales[0])>1)return `@${t}x`}return ""}toQuadKey(e){let t=e.x,i=e.y,r="";for(let s=e.z;s>0;s--){let e=0,n=1<<s-1;0!=(t&n)&&e++,0!=(i&n)&&(e+=2),r+=e.toString();}return r}static urlHasTilePattern(e){return e&&(e.search("{x}")>-1&&e.search("{y}")>-1&&e.search("{z}")>-1||e.search("{q}")>-1)}}const be={coord(e){return {x:e.x,y:e.y,z:e.z,key:this.coordKey(e)}},coordKey:e=>e.x+"/"+e.y+"/"+e.z,key(e,t,i){if(!(e.y<0||e.y>=1<<e.z||e.z<0))return [t.name,e.x,e.y,e.z,i].join("/")},normalizedKey(e,t,i){return this.key(this.normalizedCoord(e,t),t,i)},normalizedCoord(e,t){return t.zoom_bias&&(e=this.coordAtZoom(e,Math.max(e.z-t.zoom_bias,t.zooms[0]))),this.coordForTileZooms(e,t.zooms)},coordAtZoom(e,t){let i=e.x,r=e.y,s=e.z;if(s!==(t=Math.max(0,t))){let e=Math.pow(2,s-t);i=Math.floor(i/e),r=Math.floor(r/e),s=t;}return this.coord({x:i,y:r,z:s})},coordForTileZooms(e,t){let i=e.x,r=e.y,s=e.z;const n=this.findZoomInRange(s,t);return n!==s?this.coordAtZoom({x:i,y:r,z:s},n):this.coord({x:i,y:r,z:s})},findZoomInRange:(e,t)=>t.filter(t=>e>=t).reverse()[0]||t[0],isDescendant(e,t){if(t.z>e.z){let i=this.coordAtZoom(t,e.z),r=i.x,s=i.y;return e.x===r&&e.y===s}return !1},parent(e){let t=e.coords,i=e.source,r=e.style_z;if(r>0){r--;const e=Math.max(r-i.zoom_bias,i.zooms[0]),s=this.coordForTileZooms(this.coordAtZoom(t,e),i.zooms);return s.z>r?null:{key:this.key(s,i,r),coords:s,style_z:r,source:i}}},children(e,t){let i=e.coords,r=e.source,s=e.style_z;void 0===t&&(t={}),s++;const n=this.coordForTileZooms(this.coordAtZoom(i,s-r.zoom_bias),r.zooms);if(n.z===i.z)return [{key:this.key(n,r,s),coords:n,style_z:s,source:r}];{const e=this.key(n,r,s);if(t[r.id]=t[r.id]||{},null==t[r.id][e]){const o=Math.pow(2,n.z-i.z),a=i.x*o,l=i.y*o;let u=[];for(let e=a;e<a+o;e++)for(let t=l;t<l+o;t++){let i=this.coord({x:e,y:t,z:n.z});u.push({key:this.key(i,r,s),coords:i,style_z:s,source:r});}t[r.id][e]=u;}return t[r.id][e]}}};class Ae extends ve{constructor(e,t){super(e,t),-1===this.rasters.indexOf(this.name)&&this.rasters.unshift(this.name),this.filtering=e.filtering,this.textures={};}async load(e){e.source_data={},e.source_data.layers={},e.pad_scale=this.pad_scale,e.rasters=[...this.rasters];let t=F.tile_scale;return e.source_data.layers={_default:{type:"FeatureCollection",features:[{geometry:{type:"Polygon",coordinates:[[[0,0],[t,0],[t,-t],[0,-t],[0,0]]]},properties:{}}]}},e.default_winding="CW",e}async tileTexture(e){let t=this.adjustRasterTileZoom(e),i=t.key;if(!this.textures[i]){let e=this.formatURL(this.url,{coords:t});this.textures[i]={name:e,url:e,filtering:this.filtering,coords:t};}return this.textures[i]}adjustRasterTileZoom(e){let t=e.coords;const i=this.sources[e.source];if(i!==this){let r=this.zoom_bias-i.zoom_bias;r>0?t=be.normalizedCoord(e.coords,{zoom_bias:r,zooms:this.zooms}):(r<0&&f({level:"warn",once:!0},`Raster source '${this.name}' supports higher zoom detail than source '${i.name}' `+`it's attached to. Downsampling this source ${-r} extra zoom levels to match.`),t=be.coordForTileZooms(t,this.zooms));}return t}}class Te extends Ae{constructor(e,t){super(e,t),this.load_image={},this.alpha=null!=e.alpha?Math.max(Math.min(e.alpha,1),0):null,this.mask_alpha=!0,this.preserve_tiles_within_zoom=0,this.max_display_density=e.max_display_density,Array.isArray(e.composite)?this.images=e.composite.map(e=>({url:e.url,bounds:this.parseBounds(e),alpha:null!=e.alpha?Math.max(Math.min(e.alpha,1),0):null})):this.images=[{url:this.url,bounds:this.bounds,alpha:this.alpha}];}async tileTexture(e,t){let i=t.blend,r=t.generation,s=this.adjustRasterTileZoom(e);const n="opaque"!==i,o=`raster-${this.name}-${s.key}-${n?"alpha":"opaque"}-${r}`;if(w.textures[o])return {name:o,coords:s,skip_create:!0};let a=_.device_pixel_ratio;this.max_display_density&&(a=Math.min(a,this.max_display_density)),a*=2;const l=document.createElement("canvas"),u=l.getContext("2d");l.width=this.tile_size*a,l.height=this.tile_size*a,u.imageSmoothingEnabled="nearest"!==this.filtering;const h=this.images.filter(t=>this.checkBounds(e.coords,t.bounds));return await Promise.all(h.map(t=>{const i=n?null!=t.alpha?t.alpha:this.alpha:1;return this.drawImage(t.url,t.bounds,i,e,a,u)})),{name:o,element:l,filtering:this.filtering,coords:s}}async drawImage(e,t,i,r,s,n){const o=z(e);this.load_image[o]=this.load_image[o]||this.loadImage(e);const a=await this.load_image[o],l=F.metersPerPixel(r.coords.z)/s/(this.tile_size/F.tile_size),u=(t.meters.min[0]-r.min.x)/l,h=-(t.meters.min[1]-r.min.y)/l,c=(t.meters.max[0]-t.meters.min[0])/l,d=-(t.meters.max[1]-t.meters.min[1])/l;n.globalAlpha=null!=i?i:1,n.drawImage(a,u,h,c,d);}loadImage(e){return new Promise(t=>{let i=new Image;i.onload=(()=>t(i)),i.onerror=(i=>{f("warn",`Raster source '${this.name}': failed to load url: '${e}'`,i),t(null);}),_.isSafari()&&"data:"===e.slice(0,5)||(i.crossOrigin="anonymous"),i.src=e;})}includesTile(e,t){return !!ge.prototype.includesTile.call(this,e,t)&&this.images.some(t=>this.checkBounds(e,t.bounds))}validate(e){let t="Raster data source must provide a string `url` parameter, or an array of `composite` raster ";t+="image objects that each have a `url` parameter";let i="Raster data source must provide a `bounds` parameter, or an array of `composite` raster ";i+="image objects that each have a `bounds` parameter";let r="Raster data source must have *either* a single image specified as `url` and `bounds `";if(r+="parameters, or an array of `composite` raster image objects, each with `url` and `bounds`.",Array.isArray(e.composite)){if(e.composite.some(e=>"string"!=typeof e.url))throw Error(t);if(e.composite.some(e=>!(Array.isArray(e.bounds)&&4===e.bounds.length)))throw Error(i);if(null!=e.url||null!=e.bounds)throw Error("Raster data source must have *either* a single image specified as `url` and `bounds `parameters, or an array of `composite` raster image objects, each with `url` and `bounds`.")}else{if("string"!=typeof e.url)throw Error(t);if(!Array.isArray(e.bounds)||4!==e.bounds.length)throw Error(i)}}}ge.register("Raster",e=>Ae.urlHasTilePattern(e.url)?Ae:Te);var we,Ee={init(e){let t=void 0===e?{}:e,i=t.generation,r=t.styles,s=t.sources,n=void 0===s?{}:s,o=t.introspection;this.setGeneration(i),this.styles=r,this.sources=n,this.defines=this.hasOwnProperty("defines")&&this.defines||{},this.shaders=this.hasOwnProperty("shaders")&&this.shaders||{},this.introspection=o||!1,this.selection=this.selection||this.introspection||!1,this.compile_setup=!1,this.program=null,this.selection_program=null,this.feature_style={},this.vertex_template=[],this.tile_data={},this.stencil_proxy_tiles=!0,this.defines.TANGRAM_WORLD_POSITION_WRAP=1e5,this.blend=this.blend||"opaque","opaque"!==this.blend?null==this.blend_order&&(this.blend_order=this.default_blend_orders[this.blend]):this.blend_order=this.default_blend_orders[this.blend],this.defines[`TANGRAM_BLEND_${this.blend.toUpperCase()}`]=!0,this.removeShaderBlock("setup"),this.material instanceof ae||(ae.isValid(this.material)||(this.material=q.defaults.material),this.material=new ae(this.material)),this.material.inject(this),_e.setMode(this.lighting,this),this.setupRasters(),this.initialized=!0;},destroy(){this.program&&(this.program.destroy(),this.program=null),this.selection_program&&(this.selection_program.destroy(),this.selection_program=null),n.removeTarget(this.main_thread_target),this.gl=null,this.initialized=!1;},reset(){},baseStyle(){return this.base||this.name},setGeneration(e){this.generation=e,this.main_thread_target=["Style",this.name,this.generation].join("_"),t.is_main&&n.addTarget(this.main_thread_target,this);},startData(e){this.tile_data[e.id]=this.tile_data[e.id]||{meshes:{},uniforms:{},textures:[]};},async endData(e){var t=this.tile_data[e.id];if(this.tile_data[e.id]=null,t&&Object.keys(t.meshes).length>0){for(let e in t.meshes){let i=t.meshes[e];0!==i.vertex_data.vertex_count?(i.vertex_data.end(),i.vertex_elements=i.vertex_data.element_buffer,i.vertex_data=i.vertex_data.vertex_buffer):delete t.meshes[e];}return this.buildRasterTextures(e,t)}return null},hasDataForTile(e){return null!=this.tile_data[e.id]},getTileMesh(e,t){const i=this.tile_data[e.id].meshes;if(null==i[t.key]){const e=this.vertexLayoutForMeshVariant(t);i[t.key]={variant:t,vertex_data:e.createVertexData()};}return i[t.key]},vertexLayoutForMeshVariant(){},meshVariantTypeForDraw(){},addFeature(e,t,i){let r=i.tile;if(r.generation!==this.generation)return;this.tile_data[r.id]||this.startData(r);let s=this.parseFeature(e,t,i);s&&this.buildGeometry(e.geometry,s,i)>0&&(e.generation=this.generation);},buildGeometry(e,t,i){let r;if("Polygon"===e.type?r=this.buildPolygons([e.coordinates],t,i):"MultiPolygon"===e.type?r=this.buildPolygons(e.coordinates,t,i):"LineString"===e.type?r=this.buildLines([e.coordinates],t,i):"MultiLineString"===e.type?r=this.buildLines(e.coordinates,t,i):"Point"===e.type?r=this.buildPoints([e.coordinates],t,i):"MultiPoint"===e.type&&(r=this.buildPoints(e.coordinates,t,i)),r>0&&m.layer_stats){let e=i.tile;e.debug.layers=e.debug.layers||{list:{},tree:{}};let t=e.debug.layers.list,s=e.debug.layers.tree,n={};i.layers.forEach(e=>{Re(t,e,1,r,{[this.name]:r},{[this.baseStyle()]:r});let i=s,o=n,a=e.split(":");for(let e=0;e<a.length;e++){let t=a[e];i[t]=i[t]||{features:0,geoms:0,styles:{},base:{}},null==o[t]&&(o[t]={},Re(i,t,1,r,{[this.name]:r},{[this.baseStyle()]:r})),e<a.length-1&&(i[t].layers=i[t].layers||{}),i=i[t].layers,o=o[t];}});}return r},parseFeature(e,t,i){try{var r=this.feature_style;if(r.order=this.parseOrder(t.order,i),null==r.order&&"overlay"!==this.blend){let e=`Layer '${t.layers.join(", ")}', draw group '${t.group}': `;return e+="'order' parameter is required unless blend mode is 'overlay'",null!=t.order&&(e+="; 'order' was set to a dynamic value (e.g. string tied to feature property, ",e+="or JS function), but evaluated to null for one or more features"),void f({level:"warn",once:!0},e)}if(!(r=this._parseFeature(e,t,i)))return;return this.selection?r.interactive=q.evalProperty(t.interactive,i):r.interactive=!1,!0===r.interactive?r.selection_color=re.makeColor(e,i.tile,i):r.selection_color=re.defaultColor,r}catch(t){f("error","Style.parseFeature: style parsing error",e,r,t.stack);}},_parseFeature(){return this.feature_style},preprocess(e){if(!e.preprocessed){if(this.draw)for(let t in this.draw){let i=this.draw[t];"object"!=typeof i||Array.isArray(i)?null==e[t]&&(e[t]=i):e[t]=O({},i,e[t]);}if(this.selection?this.introspection&&(e.interactive=!0):e.interactive=!1,!(e=this._preprocess(e)))return;e.preprocessed=!0;}return e},_preprocess:e=>e,parseOrder:(e,t)=>"number"!=typeof e?q.calculateOrder(e,t):e,scaleOrder:e=>2*e,parseColor(e,t){return e?q.evalCachedColorProperty(e,t):this.shaders.blocks.color||this.shaders.blocks.filter?q.defaults.color:void 0},buildPolygons:()=>0,buildLines:()=>0,buildPoints:()=>0,setGL(e){this.gl=e,this.max_texture_size=w.getMaxTextureSize(this.gl);},makeMesh(e,t,i){void 0===i&&(i={});let r=this.vertexLayoutForMeshVariant(i.variant);return new se(this.gl,e,t,r,i)},render:e=>e.render(),getProgram(e){void 0===e&&(e="program"),this.compileSetup();const t=this[e];if(t&&!t.error){if(!t.compiled){f("debug",`Compiling style '${this.name}', program key '${e}'`);try{t.compile();}catch(t){f("error",`Style: error compiling program for style '${this.name}' (program key '${e}')`,this,t.stack,t.type,t.shader_errors);}}return t}},compileSetup(){if(!this.compile_setup){if(!this.gl)throw new Error(`style.compile(): skipping for ${this.name} because no GL context`);var e=this.buildDefineList();if(this.selection){var t=Object.assign({},e);t.TANGRAM_FEATURE_SELECTION=!0;}var i=this.shaders&&this.shaders.blocks,r=this.shaders&&this.shaders.block_scopes,s=Object.assign({},this.shaders&&this.shaders.uniforms);for(let e in s)null==s[e]&&f({level:"warn",once:!0},`Style '${this.name}' has invalid uniform '${e}': uniform values must be non-null`);var n=this.shaders&&this.shaders.extensions;"string"==typeof n&&(n=[n]),this.program=new k(this.gl,this.vertex_shader_src,this.fragment_shader_src,{name:this.name,defines:e,uniforms:s,blocks:i,block_scopes:r,extensions:n}),this.selection?this.selection_program=new k(this.gl,this.vertex_shader_src,"// Fragment shader for feature selection passes\n// Renders in silhouette according to selection (picking) color, or black if none defined\n\n#ifdef TANGRAM_FEATURE_SELECTION\n    varying vec4 v_selection_color;\n#endif\n\nvoid main (void) {\n    #ifdef TANGRAM_FEATURE_SELECTION\n        gl_FragColor = v_selection_color;\n    #else\n        gl_FragColor = vec4(0., 0., 0., 1.);\n    #endif\n}\n",{name:this.name+" (selection)",defines:t,uniforms:s,blocks:i,block_scopes:r,extensions:n}):this.selection_program=null,this.compile_setup=!0;}},addShaderBlock(e,t,i){void 0===i&&(i=null),this.shaders.blocks=this.shaders.blocks||{},this.shaders.blocks[e]=this.shaders.blocks[e]||[],this.shaders.blocks[e].push(t),this.shaders.block_scopes=this.shaders.block_scopes||{},this.shaders.block_scopes[e]=this.shaders.block_scopes[e]||[],this.shaders.block_scopes[e].push(i);},removeShaderBlock(e){this.shaders.blocks&&(this.shaders.blocks[e]=null),this.shaders.block_scopes&&(this.shaders.block_scopes[e]=null);},replaceShaderBlock(e,t,i){void 0===i&&(i=null),this.removeShaderBlock(e),this.addShaderBlock(e,t,i);},buildDefineList(){var e={};if(null!=this.defines)for(var t in this.defines)e[t]=this.defines[t];if(null!=this.shaders&&null!=this.shaders.defines)for(t in this.shaders.defines)e[t]=this.shaders.defines[t];return e},hasRasters(){return ["color","normal","custom"].indexOf(this.raster)>-1},setupRasters(){if(!this.hasRasters())return;"color"===this.raster?this.defines.TANGRAM_RASTER_TEXTURE_COLOR=!0:"normal"===this.raster&&(this.defines.TANGRAM_RASTER_TEXTURE_NORMAL=!0);let e=Object.keys(this.sources).filter(e=>this.sources[e]instanceof Ae).length;if(this.defines.TANGRAM_NUM_RASTER_SOURCES=`${e}`,e>0){const t=Object.keys(this.sources).filter(e=>this.sources[e].mask_alpha).length;this.defines.TANGRAM_HAS_MASKED_RASTERS=t>0,this.defines.TANGRAM_ALL_MASKED_RASTERS=t===e,this.defines.TANGRAM_MODEL_POSITION_BASE_ZOOM_VARYING=!0,this.replaceShaderBlock("raster","// Uniforms defining raster textures and macros for accessing them\n\n#ifdef TANGRAM_FRAGMENT_SHADER\nuniform sampler2D u_rasters[TANGRAM_NUM_RASTER_SOURCES];    // raster tile texture samplers\nuniform vec2 u_raster_sizes[TANGRAM_NUM_RASTER_SOURCES];    // raster tile texture sizes (width/height in pixels)\nuniform vec3 u_raster_offsets[TANGRAM_NUM_RASTER_SOURCES];  // raster tile texture UV starting offset for tile\n\n// Raster sources can optionally mask by the alpha channel (render with only full or no alpha, based on a threshold),\n// which is used for handling transparency outside the raster image when rendering with opaque blending\n#if defined(TANGRAM_HAS_MASKED_RASTERS) && !defined(TANGRAM_ALL_MASKED_RASTERS) // only add uniform if we need it\nuniform bool u_raster_mask_alpha;\n#endif\n\n// Note: the raster accessors below are #defines rather than functions to\n// avoid issues with constant integer expressions for array indices\n\n// Adjusts UVs in model space to account for raster tile texture overzooming\n// (applies scale and offset adjustments)\n#define adjustRasterUV(raster_index, uv) \\\n    ((uv) * u_raster_offsets[raster_index].z + u_raster_offsets[raster_index].xy)\n\n// Returns the UVs of the current model position for a raster sampler\n#define currentRasterUV(raster_index) \\\n    (adjustRasterUV(raster_index, v_modelpos_base_zoom.xy))\n\n// Returns pixel location in raster tile texture at current model position\n#define currentRasterPixel(raster_index) \\\n    (currentRasterUV(raster_index) * rasterPixelSize(raster_index))\n\n// Samples a raster tile texture for the current model position\n#define sampleRaster(raster_index) \\\n    (texture2D(u_rasters[raster_index], currentRasterUV(raster_index)))\n\n// Samples a raster tile texture for a given pixel\n#define sampleRasterAtPixel(raster_index, pixel) \\\n    (texture2D(u_rasters[raster_index], (pixel) / rasterPixelSize(raster_index)))\n\n// Returns size of raster sampler in pixels\n#define rasterPixelSize(raster_index) \\\n    (u_raster_sizes[raster_index])\n\n#endif\n","Raster");}},async buildRasterTextures(e,t){if(!this.hasRasters())return t;if(0===e.rasters.length)return t;let i;try{i=await n.postMessage(`${this.main_thread_target}.loadTextures`,{coords:e.coords,source:e.source,rasters:e.rasters,min:e.min,max:e.max});}catch(e){return t}if(!i||i.length<1)return t;if(i.some(e=>!e.loaded))return null;t.uniforms.u_raster_mask_alpha=!0===this.sources[e.rasters[0]].mask_alpha;const r=t.uniforms.u_rasters=[],s=t.uniforms.u_raster_sizes=[],o=t.uniforms.u_raster_offsets=[];return i.forEach(i=>{const n=i.index;if(r[n]=i.name,t.textures.push(i.name),s[n]=[i.width,i.height],e.coords.z>i.coords.z){let t=e.coords.z-i.coords.z,r=Math.pow(2,t);o[n]=[(e.coords.x%r+r)%r/r,(r-1-e.coords.y%r)/r,1/r];}else o[n]=[0,0,1];}),t},async loadTextures(e){let t={},i={},r=[];e.rasters.map(e=>this.sources[e]).filter(e=>e).forEach((s,n)=>{s instanceof Ae&&r.push(s.tileTexture(e,this).then(e=>{t[e.name]=e,i[e.name]=n;}));}),await Promise.all(r),await w.createFromObject(this.gl,t);let s=await Promise.all(Object.keys(t).map(e=>w.textures[e]&&w.textures[e].load()).filter(e=>e));return s.forEach(e=>e.retain()),s.map(e=>({name:e.name,width:e.width,height:e.height,loaded:e.loaded,index:i[e.name],coords:t[e.name].coords}))},setup(){this.setUniforms(),this.material.setupProgram(k.current);},setUniforms(){var e=k.current;e&&e.setUniforms(this.shaders&&this.shaders.uniforms,!0);},render_states:{opaque:{depth_test:!0,depth_write:!0},translucent:{depth_test:!0,depth_write:!0},add:{depth_test:!0,depth_write:!1},multiply:{depth_test:!0,depth_write:!1},inlay:{depth_test:!0,depth_write:!1},overlay:{depth_test:!1,depth_write:!1}},default_blend_orders:{opaque:Number.MIN_SAFE_INTEGER,add:1,multiply:2,inlay:3,translucent:4,overlay:5},getBlendOrderForDraw(e){return "opaque"!==this.blend&&null!=e.blend_order?e.blend_order:this.blend_order}};function Re(e,t,i,r,s,n){e[t]=e[t]||{features:0,geoms:0,styles:{},base:{}},e[t].features+=i,e[t].geoms+=r;for(let i in s)e[t].styles[i]=e[t].styles[i]||0,e[t].styles[i]+=s[i];for(let i in n)e[t].base[i]=e[t].base[i]||0,e[t].base[i]+=n[i];}var Me=we={};we.BYTE=5120,we.UNSIGNED_BYTE=5121,we.SHORT=5122,we.UNSIGNED_SHORT=5123,we.INT=5124,we.UNSIGNED_INT=5125,we.FLOAT=5126;let ze=Math.pow(2,16)-1,Ne=!1;class Se{constructor(){this.array=[],this.has_overflown=!1;}push(e){this.has_overflown&&!Ne||e>ze&&(this.has_overflown=!0,!Ne)||this.array.push(e);}end(){if(this.array.length){let t=(e=this.array,new(this.has_overflown&&Ne?Uint32Array:Uint16Array)(e));return this.array=[],this.has_overflown=!1,t}return !1;var e;}}Se.setElementIndexUint=function(e){Ne=e;};let Pe={[Me.FLOAT]:Float32Array,[Me.BYTE]:Int8Array,[Me.UNSIGNED_BYTE]:Uint8Array,[Me.INT]:Int32Array,[Me.UNSIGNED_INT]:Uint32Array,[Me.SHORT]:Int16Array,[Me.UNSIGNED_SHORT]:Uint16Array};class ke{constructor(e,t){let i=(void 0===t?{}:t).prealloc,r=void 0===i?500:i;this.vertex_layout=e,this.vertex_elements=new Se,this.stride=this.vertex_layout.stride,ke.array_pool.length>0?(this.vertex_buffer=ke.array_pool.pop(),this.byte_length=this.vertex_buffer.byteLength,this.size=Math.floor(this.byte_length/this.stride),f("trace",`VertexData: reused buffer of bytes ${this.byte_length}, ${this.size} vertices`)):(this.size=r,this.byte_length=this.stride*this.size,this.vertex_buffer=new Uint8Array(this.byte_length)),this.offset=0,this.vertex_count=0,this.realloc_count=0,this.setBufferViews(),this.setAddVertexFunction();}setBufferViews(){this.views={},this.views[Me.UNSIGNED_BYTE]=this.vertex_buffer,this.vertex_layout.dynamic_attribs.forEach(e=>{if(null==this.views[e.type]){var t=Pe[e.type];this.views[e.type]=new t(this.vertex_buffer.buffer);}});}checkBufferSize(){if(this.offset+this.stride>this.byte_length){this.size=Math.floor(1.5*this.size),this.size-=this.size%4,this.byte_length=this.stride*this.size;var e=new Uint8Array(this.byte_length);e.set(this.vertex_buffer),ke.array_pool.push(this.vertex_buffer),this.vertex_buffer=e,this.setBufferViews(),this.realloc_count++;}}setAddVertexFunction(){this.vertexLayoutAddVertex=this.vertex_layout.getAddVertexFunction();}addVertex(e){this.checkBufferSize(),this.vertexLayoutAddVertex(e,this.views,this.offset),this.offset+=this.stride,this.vertex_count++;}end(){return this.vertex_buffer=this.vertex_buffer.subarray(0,this.offset),this.element_buffer=this.vertex_elements.end(),f("trace",`VertexData: ${this.size} vertices total, realloc count ${this.realloc_count}`),this}}ke.array_pool=[];class Le{constructor(e){this.attribs=e,this.dynamic_attribs=this.attribs.filter(e=>null==e.static),this.static_attribs=this.attribs.filter(e=>null!=e.static),this.components=[],this.index={},this.offset={},this.stride=0;let t=0,i=0;for(let e=0;e<this.attribs.length;e++){let r=this.attribs[e];if(null==r.static){r.offset=this.stride,r.byte_size=r.size;let e=0;switch(r.type){case Me.FLOAT:case Me.INT:case Me.UNSIGNED_INT:r.byte_size*=4,e=2;break;case Me.SHORT:case Me.UNSIGNED_SHORT:r.byte_size*=2,e=1;}3&r.byte_size&&(r.byte_size+=4-(3&r.byte_size)),this.stride+=r.byte_size;let s=r.offset>>e;for(let t=0;t<r.size;t++)this.components.push({type:r.type,shift:e,offset:s++,index:i++});this.index[r.name]=t,t+=r.size,this.offset[r.name]=r.offset;}else r.static=Array.isArray(r.static)?r.static:[r.static],r.method=`vertexAttrib${r.static.length}fv`;}}enableDynamicAttributes(e,t){for(const t in Le.enabled_attribs)e.disableVertexAttribArray(t);Le.enabled_attribs={},this.dynamic_attribs.forEach(i=>{const r=t.attribute(i.name).location;-1!==r&&(e.enableVertexAttribArray(r),e.vertexAttribPointer(r,i.size,i.type,i.normalized,this.stride,i.offset),Le.enabled_attribs[r]=t);});}enableStaticAttributes(e,t){this.static_attribs.forEach(i=>{const r=t.attribute(i.name).location;-1!==r&&e[i.method]instanceof Function&&(e.enableVertexAttribArray(r),e[i.method](r,i.static),e.disableVertexAttribArray(r));});}createVertexData(){return new ke(this)}getAddVertexFunction(){return null==this.addVertex&&this.createAddVertexFunction(),this.addVertex}createAddVertexFunction(){let e=z(JSON.stringify(this.attribs));if(null==Le.add_vertex_funcs[e]){let t,i=["var t, o;"],r=[...this.components];r.sort((e,t)=>e.type!==t.type?e.type-t.type:e.index-t.index);for(let e=0;e<r.length;e++){let s=r[e];t!==s.type&&(i.push(`t = vs[${s.type}];`),i.push(`o = off${s.shift?" >> "+s.shift:""};`),t=s.type),i.push(`t[o + ${s.offset}] = v[${s.index}];`);}i=i.join("\n");let s=new Function("v","vs","off",i);Le.add_vertex_funcs[e]=s;}this.addVertex=Le.add_vertex_funcs[e];}}Le.enabled_attribs={},Le.add_vertex_funcs={};const Ie=[{x:0,y:0},{x:F.tile_scale,y:-F.tile_scale}],Oe=[0,0,1,1];function Fe(e,t,i){let r=Ie[0],s=Ie[1];return e[0]<=r.x+i&&t[0]<=r.x+i||e[0]>=s.x-i&&t[0]>=s.x-i||e[1]>=r.y-i&&t[1]>=r.y-i||e[1]<=s.y+i&&t[1]<=s.y+i}function Ue(e,t){t=t||0;let i=Ie[0],r=Ie[1];return e[0]<=i.x+t||e[0]>=r.x-t||e[1]>=i.y-t||e[1]<=r.y+t}var De=Ge;function Ge(e,t,i){i=i||2;var r,s,n,o,a,l,u,h=t&&t.length,c=h?t[0]*i:e.length,d=Ce(e,0,c,i,!0),f=[];if(!d)return f;if(h&&(d=function(e,t,i,r){var s,n,o,a,l,u=[];for(s=0,n=t.length;s<n;s++)o=t[s]*r,a=s<n-1?t[s+1]*r:e.length,(l=Ce(e,o,a,r,!1))===l.next&&(l.steiner=!0),u.push(Ye(l));for(u.sort(Xe),s=0;s<u.length;s++)We(u[s],i),i=je(i,i.next);return i}(e,t,d,i)),e.length>80*i){r=n=e[0],s=o=e[1];for(var _=i;_<c;_+=i)(a=e[_])<r&&(r=a),(l=e[_+1])<s&&(s=l),a>n&&(n=a),l>o&&(o=l);u=Math.max(n-r,o-s);}return Ve(d,f,i,r,s,u),f}function Ce(e,t,i,r,s){var n,o;if(s===at(e,t,i,r)>0)for(n=t;n<i;n+=r)o=st(n,e[n],e[n+1],o);else for(n=i-r;n>=t;n-=r)o=st(n,e[n],e[n+1],o);return o&&et(o,o.next)&&(nt(o),o=o.next),o}function je(e,t){if(!e)return e;t||(t=e);var i,r=e;do{if(i=!1,r.steiner||!et(r,r.next)&&0!==Qe(r.prev,r,r.next))r=r.next;else{if(nt(r),(r=t=r.prev)===r.next)return null;i=!0;}}while(i||r!==t);return t}function Ve(e,t,i,r,s,n,o){if(e){!o&&n&&function(e,t,i,r){var s=e;do{null===s.z&&(s.z=Ze(s.x,s.y,t,i,r)),s.prevZ=s.prev,s.nextZ=s.next,s=s.next;}while(s!==e);s.prevZ.nextZ=null,s.prevZ=null,function(e){var t,i,r,s,n,o,a,l,u=1;do{for(i=e,e=null,n=null,o=0;i;){for(o++,r=i,a=0,t=0;t<u&&(a++,r=r.nextZ);t++);for(l=u;a>0||l>0&&r;)0===a?(s=r,r=r.nextZ,l--):0!==l&&r?i.z<=r.z?(s=i,i=i.nextZ,a--):(s=r,r=r.nextZ,l--):(s=i,i=i.nextZ,a--),n?n.nextZ=s:e=s,s.prevZ=n,n=s;i=r;}n.nextZ=null,u*=2;}while(o>1)}(s);}(e,r,s,n);for(var a,l,u=e;e.prev!==e.next;)if(a=e.prev,l=e.next,n?$e(e,r,s,n):Be(e))t.push(a.i/i),t.push(e.i/i),t.push(l.i/i),nt(e),e=l.next,u=l.next;else if((e=l)===u){o?1===o?Ve(e=qe(e,t,i),t,i,r,s,n,2):2===o&&He(e,t,i,r,s,n):Ve(je(e),t,i,r,s,n,1);break}}}function Be(e){var t=e.prev,i=e,r=e.next;if(Qe(t,i,r)>=0)return !1;for(var s=e.next.next;s!==e.prev;){if(Je(t.x,t.y,i.x,i.y,r.x,r.y,s.x,s.y)&&Qe(s.prev,s,s.next)>=0)return !1;s=s.next;}return !0}function $e(e,t,i,r){var s=e.prev,n=e,o=e.next;if(Qe(s,n,o)>=0)return !1;for(var a=s.x<n.x?s.x<o.x?s.x:o.x:n.x<o.x?n.x:o.x,l=s.y<n.y?s.y<o.y?s.y:o.y:n.y<o.y?n.y:o.y,u=s.x>n.x?s.x>o.x?s.x:o.x:n.x>o.x?n.x:o.x,h=s.y>n.y?s.y>o.y?s.y:o.y:n.y>o.y?n.y:o.y,c=Ze(a,l,t,i,r),d=Ze(u,h,t,i,r),f=e.nextZ;f&&f.z<=d;){if(f!==e.prev&&f!==e.next&&Je(s.x,s.y,n.x,n.y,o.x,o.y,f.x,f.y)&&Qe(f.prev,f,f.next)>=0)return !1;f=f.nextZ;}for(f=e.prevZ;f&&f.z>=c;){if(f!==e.prev&&f!==e.next&&Je(s.x,s.y,n.x,n.y,o.x,o.y,f.x,f.y)&&Qe(f.prev,f,f.next)>=0)return !1;f=f.prevZ;}return !0}function qe(e,t,i){var r=e;do{var s=r.prev,n=r.next.next;!et(s,n)&&tt(s,r,r.next,n)&&it(s,n)&&it(n,s)&&(t.push(s.i/i),t.push(r.i/i),t.push(n.i/i),nt(r),nt(r.next),r=e=n),r=r.next;}while(r!==e);return r}function He(e,t,i,r,s,n){var o=e;do{for(var a=o.next.next;a!==o.prev;){if(o.i!==a.i&&Ke(o,a)){var l=rt(o,a);return o=je(o,o.next),l=je(l,l.next),Ve(o,t,i,r,s,n),void Ve(l,t,i,r,s,n)}a=a.next;}o=o.next;}while(o!==e)}function Xe(e,t){return e.x-t.x}function We(e,t){if(t=function(e,t){var i,r=t,s=e.x,n=e.y,o=-1/0;do{if(n<=r.y&&n>=r.next.y){var a=r.x+(n-r.y)*(r.next.x-r.x)/(r.next.y-r.y);if(a<=s&&a>o){if(o=a,a===s){if(n===r.y)return r;if(n===r.next.y)return r.next}i=r.x<r.next.x?r:r.next;}}r=r.next;}while(r!==t);if(!i)return null;if(s===o)return i.prev;var l,u=i,h=i.x,c=i.y,d=1/0;r=i.next;for(;r!==u;)s>=r.x&&r.x>=h&&Je(n<c?s:o,n,h,c,n<c?o:s,n,r.x,r.y)&&((l=Math.abs(n-r.y)/(s-r.x))<d||l===d&&r.x>i.x)&&it(r,e)&&(i=r,d=l),r=r.next;return i}(e,t)){var i=rt(t,e);je(i,i.next);}}function Ze(e,t,i,r,s){return (e=1431655765&((e=858993459&((e=252645135&((e=16711935&((e=32767*(e-i)/s)|e<<8))|e<<4))|e<<2))|e<<1))|(t=1431655765&((t=858993459&((t=252645135&((t=16711935&((t=32767*(t-r)/s)|t<<8))|t<<4))|t<<2))|t<<1))<<1}function Ye(e){var t=e,i=e;do{t.x<i.x&&(i=t),t=t.next;}while(t!==e);return i}function Je(e,t,i,r,s,n,o,a){return (s-o)*(t-a)-(e-o)*(n-a)>=0&&(e-o)*(r-a)-(i-o)*(t-a)>=0&&(i-o)*(n-a)-(s-o)*(r-a)>=0}function Ke(e,t){return e.next.i!==t.i&&e.prev.i!==t.i&&!function(e,t){var i=e;do{if(i.i!==e.i&&i.next.i!==e.i&&i.i!==t.i&&i.next.i!==t.i&&tt(i,i.next,e,t))return !0;i=i.next;}while(i!==e);return !1}(e,t)&&it(e,t)&&it(t,e)&&function(e,t){var i=e,r=!1,s=(e.x+t.x)/2,n=(e.y+t.y)/2;do{i.y>n!=i.next.y>n&&s<(i.next.x-i.x)*(n-i.y)/(i.next.y-i.y)+i.x&&(r=!r),i=i.next;}while(i!==e);return r}(e,t)}function Qe(e,t,i){return (t.y-e.y)*(i.x-t.x)-(t.x-e.x)*(i.y-t.y)}function et(e,t){return e.x===t.x&&e.y===t.y}function tt(e,t,i,r){return !!(et(e,t)&&et(i,r)||et(e,r)&&et(i,t))||Qe(e,t,i)>0!=Qe(e,t,r)>0&&Qe(i,r,e)>0!=Qe(i,r,t)>0}function it(e,t){return Qe(e.prev,e,e.next)<0?Qe(e,t,e.next)>=0&&Qe(e,e.prev,t)>=0:Qe(e,t,e.prev)<0||Qe(e,e.next,t)<0}function rt(e,t){var i=new ot(e.i,e.x,e.y),r=new ot(t.i,t.x,t.y),s=e.next,n=t.prev;return e.next=t,t.prev=e,i.next=s,s.prev=i,r.next=i,i.prev=r,n.next=r,r.prev=n,r}function st(e,t,i,r){var s=new ot(e,t,i);return r?(s.next=r.next,s.prev=r,r.next.prev=s,r.next=s):(s.prev=s,s.next=s),s}function nt(e){e.next.prev=e.prev,e.prev.next=e.next,e.prevZ&&(e.prevZ.nextZ=e.nextZ),e.nextZ&&(e.nextZ.prevZ=e.prevZ);}function ot(e,t,i){this.i=e,this.x=t,this.y=i,this.prev=null,this.next=null,this.z=null,this.prevZ=null,this.nextZ=null,this.steiner=!1;}function at(e,t,i,r){for(var s=0,n=t,o=i-r;n<i;n+=r)s+=(e[o]-e[n])*(e[n+1]+e[o+1]),o=n;return s}Ge.deviation=function(e,t,i,r){var s=t&&t.length,n=s?t[0]*i:e.length,o=Math.abs(at(e,0,n,i));if(s)for(var a=0,l=t.length;a<l;a++){var u=t[a]*i,h=a<l-1?t[a+1]*i:e.length;o-=Math.abs(at(e,u,h,i));}var c=0;for(a=0;a<r.length;a+=3){var d=r[a]*i,f=r[a+1]*i,_=r[a+2]*i;c+=Math.abs((e[d]-e[_])*(e[f+1]-e[d+1])-(e[d]-e[f])*(e[_+1]-e[d+1]));}return 0===o&&0===c?0:Math.abs((c-o)/o)},Ge.flatten=function(e){for(var t=e[0][0].length,i={vertices:[],holes:[],dimensions:t},r=0,s=0;s<e.length;s++){for(var n=0;n<e[s].length;n++)for(var o=0;o<t;o++)i.vertices.push(e[s][n][o]);s>0&&(r+=e[s-1].length,i.holes.push(r));}return i};const lt=[0,0,1];function ut(e,t,i,r){let s,n,o,a,l,u,h,c,d,f,_,p,m=r.texcoord_index,g=r.texcoord_scale,x=r.texcoord_normalize,y=t.vertex_elements,v=e.length,b=0;if(m){x=x||1;var A=g||Oe;s=A[0],n=A[1],o=A[2],a=A[3];}for(let r=0;r<v;r++){let g=e[r],v=t.vertex_count,A=(w=De.flatten(g),De(w.vertices,w.holes,w.dimensions)),E=A.length;if(E){var T;if(m)l=(T=F.findBoundingBox(g))[0],u=T[1],_=(o-s)/(d=(h=T[2])-l),p=(a-n)/(f=(c=T[3])-u);for(let e=0;e<g.length;e++){let r=g[e];for(let e=0;e<r.length;e++){let o=r[e];i[0]=o[0],i[1]=o[1],m&&(i[m+0]=((o[0]-l)*_+s)*x,i[m+1]=((o[1]-u)*p+n)*x),t.addVertex(i);}}for(let e=0;e<E;e++)y.push(v+A[e]);b+=E/3;}}var w;return b}var ht='uniform vec2 u_resolution;\nuniform float u_time;\nuniform vec3 u_map_position;\nuniform vec4 u_tile_origin;\nuniform float u_tile_proxy_order_offset;\nuniform float u_meters_per_pixel;\nuniform float u_device_pixel_ratio;\n\nuniform mat4 u_model;\nuniform mat4 u_modelView;\nuniform mat3 u_normalMatrix;\nuniform mat3 u_inverseNormalMatrix;\n\nattribute vec4 a_position;\nattribute vec4 a_color;\n\n// Optional normal attribute, otherwise default to up\n#ifdef TANGRAM_NORMAL_ATTRIBUTE\n    attribute vec3 a_normal;\n    #define TANGRAM_NORMAL a_normal\n#else\n    #define TANGRAM_NORMAL vec3(0., 0., 1.)\n#endif\n\n// Optional dynamic line extrusion\n#ifdef TANGRAM_EXTRUDE_LINES\n    attribute vec2 a_extrude; // extrusion direction in xy plane\n    attribute vec2 a_offset;  // offset direction in xy plane\n\n    // Polygon and line styles have slightly different VBO layouts, saving memory by optimizing vertex packing.\n    // All lines have a width scaling factor, but only some have a height (position.z) or offset.\n    // The vertex height is stored in different attributes to account for this.\n    attribute vec2 a_z_and_offset_scale; // stores vertex height in x, and offset scaling factor in y\n    #define TANGRAM_POSITION_Z a_z_and_offset_scale.x // vertex height is stored in separate line-specific attrib\n    #define TANGRAM_OFFSET_SCALING a_z_and_offset_scale.y // zoom scaling factor for line offset\n    #define TANGRAM_WIDTH_SCALING a_position.z // zoom scaling factor for line width (stored in position attrib)\n\n    uniform float u_v_scale_adjust; // scales texture UVs for line dash patterns w/fractional pixel width\n#else\n    #define TANGRAM_POSITION_Z a_position.z // vertex height\n#endif\n\nvarying vec4 v_position;\nvarying vec3 v_normal;\nvarying vec4 v_color;\nvarying vec4 v_world_position;\n\n// Optional texture UVs\n#if defined(TANGRAM_TEXTURE_COORDS) || defined(TANGRAM_EXTRUDE_LINES)\n    attribute vec2 a_texcoord;\n    varying vec2 v_texcoord;\n#endif\n\n// Optional model position varying for tile coordinate zoom\n#ifdef TANGRAM_MODEL_POSITION_BASE_ZOOM_VARYING\n    varying vec4 v_modelpos_base_zoom;\n#endif\n\n#if defined(TANGRAM_LIGHTING_VERTEX)\n    varying vec4 v_lighting;\n#endif\n\n#define TANGRAM_UNPACK_SCALING(x) (x / 1024.)\n\n#pragma tangram: camera\n#pragma tangram: material\n#pragma tangram: lighting\n#pragma tangram: raster\n#pragma tangram: global\n\nvoid main() {\n    // Initialize globals\n    #pragma tangram: setup\n\n    // Texture UVs\n    #ifdef TANGRAM_TEXTURE_COORDS\n        v_texcoord = a_texcoord;\n        #ifdef TANGRAM_EXTRUDE_LINES\n            v_texcoord.y *= u_v_scale_adjust;\n        #endif\n    #endif\n\n    // Pass model position to fragment shader\n    #ifdef TANGRAM_MODEL_POSITION_BASE_ZOOM_VARYING\n        v_modelpos_base_zoom = modelPositionBaseZoom();\n    #endif\n\n    // Position\n    vec4 position = vec4(a_position.xy, TANGRAM_POSITION_Z / TANGRAM_HEIGHT_SCALE, 1.); // convert height back to meters\n\n    #ifdef TANGRAM_EXTRUDE_LINES\n        vec2 extrude = a_extrude.xy;\n        vec2 offset = a_offset.xy;\n\n        // Adjust line width based on zoom level, to prevent proxied lines\n        // from being either too small or too big.\n        // "Flattens" the zoom between 1-2 to peg it to 1 (keeps lines from\n        // prematurely shrinking), then interpolate and clamp to 4 (keeps lines\n        // from becoming too small when far away).\n        float dz = clamp(u_map_position.z - u_tile_origin.z, 0., 4.);\n        dz += step(1., dz) * (1. - dz) + mix(0., 2., clamp((dz - 2.) / 2., 0., 1.));\n\n        // Interpolate line width between zooms\n        float mdz = (dz - 0.5) * 2.; // zoom from mid-point\n        extrude -= extrude * TANGRAM_UNPACK_SCALING(TANGRAM_WIDTH_SCALING) * mdz;\n\n        // Interpolate line offset between zooms\n        // Scales from the larger value to the smaller one\n        float dwdz = TANGRAM_UNPACK_SCALING(TANGRAM_OFFSET_SCALING);\n        float sdwdz = sign(step(0., dwdz) - 0.5); // sign indicates "direction" of scaling\n        offset -= offset * abs(dwdz) * ((1.-step(0., sdwdz)) - (dz * -sdwdz)); // scale "up" or "down"\n\n        // Scale line width and offset to be consistent in screen space\n        float ssz = exp2(-dz - (u_tile_origin.z - u_tile_origin.w));\n        extrude *= ssz;\n        offset *= ssz;\n\n        // Modify line width before extrusion\n        #ifdef TANGRAM_BLOCK_WIDTH\n            float width = 1.;\n            #pragma tangram: width\n            extrude *= width;\n        #endif\n\n        position.xy += extrude + offset;\n    #endif\n\n    // World coordinates for 3d procedural textures\n    v_world_position = wrapWorldPosition(u_model * position);\n\n    // Adjust for tile and view position\n    position = u_modelView * position;\n\n    // Modify position before camera projection\n    #pragma tangram: position\n\n    // Setup varyings\n    v_position = position;\n    v_normal = normalize(u_normalMatrix * TANGRAM_NORMAL);\n    v_color = a_color;\n\n    #if defined(TANGRAM_LIGHTING_VERTEX)\n        // Vertex lighting\n        vec3 normal = v_normal;\n\n        // Modify normal before lighting\n        #pragma tangram: normal\n\n        // Pass lighting intensity to fragment shader\n        v_lighting = calculateLighting(position.xyz - u_eye, normal, vec4(1.));\n    #endif\n\n    // Camera\n    cameraProjection(position);\n\n    // +1 is to keep all layers including proxies > 0\n    applyLayerOrder(a_position.w + u_tile_proxy_order_offset + 1., position);\n\n    gl_Position = position;\n}\n',ct="uniform vec2 u_resolution;\nuniform float u_time;\nuniform vec3 u_map_position;\nuniform vec4 u_tile_origin;\nuniform float u_meters_per_pixel;\nuniform float u_device_pixel_ratio;\n\nuniform mat3 u_normalMatrix;\nuniform mat3 u_inverseNormalMatrix;\n\nvarying vec4 v_position;\nvarying vec3 v_normal;\nvarying vec4 v_color;\nvarying vec4 v_world_position;\n\n#ifdef TANGRAM_EXTRUDE_LINES\n    uniform bool u_has_line_texture;\n    uniform sampler2D u_texture;\n    uniform float u_texture_ratio;\n    uniform vec4 u_dash_background_color;\n#endif\n\n#define TANGRAM_NORMAL v_normal\n\n#if defined(TANGRAM_TEXTURE_COORDS) || defined(TANGRAM_EXTRUDE_LINES)\n    varying vec2 v_texcoord;\n#endif\n\n#ifdef TANGRAM_MODEL_POSITION_BASE_ZOOM_VARYING\n    varying vec4 v_modelpos_base_zoom;\n#endif\n\n#if defined(TANGRAM_LIGHTING_VERTEX)\n    varying vec4 v_lighting;\n#endif\n\n#pragma tangram: camera\n#pragma tangram: material\n#pragma tangram: lighting\n#pragma tangram: raster\n#pragma tangram: global\n\nvoid main (void) {\n    // Initialize globals\n    #pragma tangram: setup\n\n    vec4 color = v_color;\n    vec3 normal = TANGRAM_NORMAL;\n\n    // Apply raster to vertex color\n    #ifdef TANGRAM_RASTER_TEXTURE_COLOR\n    { // enclose in scope to avoid leakage of internal variables\n        vec4 raster_color = sampleRaster(0);\n\n        #if defined(TANGRAM_BLEND_OPAQUE) || defined(TANGRAM_BLEND_TRANSLUCENT) || defined(TANGRAM_BLEND_MULTIPLY)\n            // Raster sources can optionally mask by the alpha channel, which will render with only full or no alpha.\n            // This is used for handling transparency outside the raster image in some blend modes,\n            // which either don't support alpha, or would cause transparent pixels to write to the depth buffer,\n            // obscuring geometry underneath.\n            #ifdef TANGRAM_HAS_MASKED_RASTERS   // skip masking logic if no masked raster sources\n            #ifndef TANGRAM_ALL_MASKED_RASTERS  // skip conditional if *only* masked raster sources (always true)\n            if (u_raster_mask_alpha) {\n            #else\n            {\n            #endif\n                #if defined(TANGRAM_BLEND_TRANSLUCENT) || defined(TANGRAM_BLEND_MULTIPLY)\n                if (raster_color.a < TANGRAM_EPSILON) {\n                    discard;\n                }\n                #else // TANGRAM_BLEND_OPAQUE\n                if (raster_color.a < 1. - TANGRAM_EPSILON) {\n                    discard;\n                }\n                // only allow full alpha in opaque blend mode (avoids artifacts blending w/canvas tile background)\n                raster_color.a = 1.;\n                #endif\n            }\n            #endif\n        #endif\n\n        color *= raster_color; // multiplied to tint texture color\n    }\n    #endif\n\n    // Apply line texture\n    #ifdef TANGRAM_EXTRUDE_LINES\n    { // enclose in scope to avoid leakage of internal variables\n        if (u_has_line_texture) {\n            vec2 _line_st = vec2(v_texcoord.x, fract(v_texcoord.y / u_texture_ratio));\n            vec4 _line_color = texture2D(u_texture, _line_st);\n\n            if (_line_color.a < TANGRAM_ALPHA_TEST) {\n                #if defined(TANGRAM_BLEND_OPAQUE)\n                    // use discard when alpha blending is unavailable\n                    if (u_dash_background_color.a < 1. - TANGRAM_EPSILON) {\n                        discard;\n                    }\n                    color = vec4(u_dash_background_color.rgb, 1.); // only allow full alpha in opaque blend mode\n                #else\n                    // use alpha channel when blending is available\n                    color = vec4(u_dash_background_color.rgb, color.a * step(TANGRAM_EPSILON, u_dash_background_color.a));\n                #endif\n            }\n            else {\n                color *= _line_color;\n            }\n        }\n    }\n    #endif\n\n    // First, get normal from raster tile (if applicable)\n    #ifdef TANGRAM_RASTER_TEXTURE_NORMAL\n        normal = normalize(sampleRaster(0).rgb * 2. - 1.);\n    #endif\n\n    // Second, alter normal with normal map texture (if applicable)\n    #if defined(TANGRAM_LIGHTING_FRAGMENT) && defined(TANGRAM_MATERIAL_NORMAL_TEXTURE)\n        calculateNormal(normal);\n    #endif\n\n    // Normal modification applied here for fragment lighting or no lighting,\n    // and in vertex shader for vertex lighting\n    #if !defined(TANGRAM_LIGHTING_VERTEX)\n        #pragma tangram: normal\n    #endif\n\n    // Color modification before lighting is applied\n    #pragma tangram: color\n\n    #if defined(TANGRAM_LIGHTING_FRAGMENT)\n        // Calculate per-fragment lighting\n        color = calculateLighting(v_position.xyz - u_eye, normal, color);\n    #elif defined(TANGRAM_LIGHTING_VERTEX)\n        // Apply lighting intensity interpolated from vertex shader\n        color *= v_lighting;\n    #endif\n\n    // Post-processing effects (modify color after lighting)\n    #pragma tangram: filter\n\n    gl_FragColor = color;\n}\n";const dt=Object.create(Ee);dt.variants={},dt.vertex_layouts={},Object.assign(dt,{name:"polygons",built_in:!0,vertex_shader_src:ht,fragment_shader_src:ct,selection:!0,init(){Ee.init.apply(this,arguments),this.defines.TANGRAM_NORMAL_ATTRIBUTE=!0,this.defines.TANGRAM_TEXTURE_COORDS=this.texcoords;},_parseFeature(e,t,i){var r=this.feature_style;return r.color=this.parseColor(t.color,i),r.color?(r.variant=t.variant,r.z=q.evalCachedDistanceProperty(t.z,i)||q.defaults.z,r.z*=F.height_scale,r.extrude=q.evalProperty(t.extrude,i),r.extrude&&(!0===r.extrude?(r.height=e.properties.height||q.defaults.height,r.min_height=e.properties.min_height||q.defaults.min_height):"number"==typeof r.extrude?(r.height=r.extrude,r.min_height=0):Array.isArray(r.extrude)&&(r.min_height=r.extrude[0],r.height=r.extrude[1]),r.height*=F.height_scale,r.min_height*=F.height_scale),r.tile_edges=t.tile_edges,r):null},_preprocess(e){return e.color=q.createColorPropertyCache(e.color),e.z=q.createPropertyCache(e.z,q.parseUnits),this.computeVariant(e),e},computeVariant(e){const t=e.interactive?1:0,i=null!=e.extrude?1:0,r=this.texcoords?1:0,s=this.getBlendOrderForDraw(e),n=[t,i,r,s].join("/");e.variant=n,null==dt.variants[n]&&(dt.variants[n]={key:n,blend_order:s,mesh_order:0,selection:t,normal:i,texcoords:r});},vertexLayoutForMeshVariant(e){if(null==dt.vertex_layouts[e.key]){const t=[{name:"a_position",size:4,type:Me.SHORT,normalized:!1},{name:"a_normal",size:3,type:Me.BYTE,normalized:!0,static:e.normal?null:[0,0,1]},{name:"a_color",size:4,type:Me.UNSIGNED_BYTE,normalized:!0},{name:"a_selection_color",size:4,type:Me.UNSIGNED_BYTE,normalized:!0,static:e.selection?null:[0,0,0,0]},{name:"a_texcoord",size:2,type:Me.UNSIGNED_SHORT,normalized:!0,static:e.texcoords?null:[0,0]}];dt.vertex_layouts[e.key]=new Le(t);}return dt.vertex_layouts[e.key]},meshVariantTypeForDraw(e){return this.variants[e.variant]},makeVertexTemplate(e,t){let i=0;return this.vertex_template[i++]=0,this.vertex_template[i++]=0,this.vertex_template[i++]=e.z||0,this.vertex_template[i++]=this.scaleOrder(e.order),t.variant.normal&&(this.vertex_template[i++]=0,this.vertex_template[i++]=0,this.vertex_template[i++]=127),this.vertex_template[i++]=255*e.color[0],this.vertex_template[i++]=255*e.color[1],this.vertex_template[i++]=255*e.color[2],this.vertex_template[i++]=255*e.color[3],t.variant.selection&&(this.vertex_template[i++]=255*e.selection_color[0],this.vertex_template[i++]=255*e.selection_color[1],this.vertex_template[i++]=255*e.selection_color[2],this.vertex_template[i++]=255*e.selection_color[3]),t.variant.texcoords&&(this.vertex_template[i++]=0,this.vertex_template[i++]=0),this.vertex_template},buildPolygons(e,t,i){let r=this.getTileMesh(i.tile,this.meshVariantTypeForDraw(t)),s=r.vertex_data,n=s.vertex_layout,o=this.makeVertexTemplate(t,r),a={texcoord_index:n.index.a_texcoord,texcoord_normalize:65535,remove_tile_edges:!t.tile_edges,tile_edge_tolerance:F.tile_scale*i.tile.pad_scale*4,winding:i.winding};return t.extrude&&t.height?function(e,t,i,r,s,n,o,a,l){let u=l.remove_tile_edges,h=l.tile_edge_tolerance,c=l.texcoord_index,d=l.texcoord_scale,f=l.texcoord_normalize,_=l.winding;var p=t+(r||0),m=t+i;n[2]=m;var g=ut(e,s,n,{texcoord_index:c,texcoord_scale:d,texcoord_normalize:f}),x=s.vertex_elements,y=s.vertex_count;if(c){f=f||1;var v=d||Oe,b=v[0],A=v[1],T=v[2],w=v[3],E=[[b,w],[b,A],[T,A],[T,w]];}for(var R=e.length,M=0;M<R;M++)for(var z=e[M],N=0;N<z.length;N++)for(var S=z[N],P=0;P<S.length-1;P++){if(u&&Fe(S[P],S[P+1],h))continue;let e,t;"CCW"===_?(e=P,t=P+1):(e=P+1,t=P);var k=[[S[t][0],S[t][1],m],[S[t][0],S[t][1],p],[S[e][0],S[e][1],p],[S[e][0],S[e][1],m]];let i=ue.normalize([S[t][0]-S[e][0],S[t][1]-S[e][1],0]),r=ue.cross(lt,i);n[o+0]=r[0]*a,n[o+1]=r[1]*a,n[o+2]=r[2]*a;for(var L=0;L<k.length;L++)n[0]=k[L][0],n[1]=k[L][1],n[2]=k[L][2],c&&(n[c+0]=E[L][0]*f,n[c+1]=E[L][1]*f),s.addVertex(n);x.push(y+0),x.push(y+1),x.push(y+2),x.push(y+2),x.push(y+3),x.push(y+0),y+=4,g+=2;}return g}(e,t.z,t.height,t.min_height,s,o,n.index.a_normal,127,a):ut(e,s,o,a)}});const ft=[0,0],_t={butt:0,square:1,round:2},pt={miter:0,bevel:1,round:2},mt=3,gt=5,xt=65536,yt=F.tile_scale,vt=[0,0],bt=[1,0],At=[.5,0];function Tt(e,t){if(!(e.length<2)){var i,r,s,n,o=t.join_type,a=t.cap_type,l=t.closed_polygon,u=t.remove_tile_edges,h=t.tile_edge_tolerance,c=t.v_scale,d=t.miter_len_sq,f=null!=t.texcoord_index,_=0;if(l&&o===pt.miter){var p=function(e){if(Ue(e[0]))return 0;for(var t=0;t<e.length;t++){var i=e[e.length-1-t];if(Ue(i))return e.length-1-t}return 0}(e);if(0!==p){var m=function(e,t){var i=[];for(let s=0;s<e.length;s++){var r=(s+t)%e.length;0!==r&&i.push(e[r]);}return i.push(i[0]),i}(e,p);return t.extra_lines=t.extra_lines||[],void t.extra_lines.push(m)}}var g=0,x=e.length-1,y=0;for(i=e[g],r=e[g+1];ue.isEqual(i,r);)if(i=r,r=e[++g+1],y++,g===e.length-1)return;for(;ue.isEqual(e[x],e[x-1]);)if(y++,0===--x)return;if(!(e.length<2+y))if(n=ue.normalize(ue.perp(i,r)),u&&Fe(i,r,h)){var v=wt(e,g,h);v&&(t.extra_lines=t.extra_lines||[],t.extra_lines.push(v));}else{l?function(e,t,i,r,s){if(void 0===r||Ue(e))Nt(e,i,i,1,0,s,1),Nt(e,i,i,0,0,s,-1);else{r===pt.miter?Rt(0,e,t,i,s.miter_len_sq,!0,s):Mt(r,0,e,t,i,!0,s);}}(i,s=ue.normalize(ue.perp(e[x-1],i)),n,o,t):(Ue(i)||(kt(i,_,n,a,!0,t),f&&a!==_t.butt&&(_+=.5*c*t.texcoord_width)),Nt(i,n,n,1,_,t,1),Nt(i,n,n,0,_,t,-1)),f&&(_+=c*ue.length(ue.sub(r,i)));for(var b=g+1;b<x;b++){var A=b,T=b+1;if(i=e[A],r=e[T],!ue.isEqual(i,r)){if(u&&Fe(i,r,h)){Nt(i,n,n,1,_,t,1),Nt(i,n,n,0,_,t,-1),zt(1,t);var w=wt(e,A+1,h);return void(w&&(t.extra_lines=t.extra_lines||[],t.extra_lines.push(w)))}s=n,n=ue.normalize(ue.perp(i,r)),o===pt.miter?Rt(_,i,s,n,d,!1,t):Mt(o,_,i,s,n,!1,t),f&&(_+=c*ue.length(ue.sub(r,i)));}}i=r,s=n,l?function(e,t,i,r,s,n){if(Ue(e))Nt(e,t,t,1,s,n,1),Nt(e,t,t,0,s,n,-1),zt(1,n);else{var o=Et(t,i);r===pt.miter&&ue.lengthSq(o)>n.miter_len_sq&&(r=pt.bevel),r===pt.miter?(Nt(e,o,t,1,s,n,1),Nt(e,o,t,0,s,n,-1),zt(1,n)):(Nt(e,t,t,1,s,n,1),Nt(e,t,t,0,s,n,-1),zt(1,n));}}(i,s,n=ue.normalize(ue.perp(i,e[1])),o,_,t):(Nt(i,s,n,1,_,t,1),Nt(i,s,n,0,_,t,-1),zt(1,t),Ue(i)||kt(i,_,s,a,!1,t));}}}function wt(e,t,i){for(var r=t;e[r+1]&&Fe(e[r],e[r+1],i);)r++;return e.length-r>=2&&e.slice(r)}function Et(e,t){var i=ue.normalize(ue.add(e,t)),r=2/(1+Math.abs(ue.dot(e,i)));return ue.mult(i,r*r)}function Rt(e,t,i,r,s,n,o){var a=Et(i,r);ue.lengthSq(a)>s?Mt(pt.bevel,e,t,i,r,n,o):(Nt(t,a,a,1,e,o,1),Nt(t,a,a,0,e,o,-1),n||zt(1,o));}function Mt(e,t,i,r,s,n,o){var a=Et(r,s),l=s[0]*r[1]-s[1]*r[0]>0;null!=o.texcoord_index&&(vt[1]=t,bt[1]=t),l?(Nt(i,a,a,1,t,o,1),Nt(i,r,a,0,t,o,-1),n||zt(1,o),Pt(i,ue.neg(r),a,ue.neg(s),a,vt,bt,vt,!1,e===pt.bevel,o),Nt(i,a,a,1,t,o,1),Nt(i,s,a,0,t,o,-1)):(Nt(i,r,a,1,t,o,1),Nt(i,a,a,0,t,o,-1),n||zt(1,o),Pt(i,r,ue.neg(a),s,a,bt,vt,bt,!1,e===pt.bevel,o),Nt(i,s,a,1,t,o,1),Nt(i,a,a,0,t,o,-1));}function zt(e,t){for(var i=t.vertex_data.vertex_elements,r=t.vertex_data.vertex_count-2*e-2,s=0;s<e;s++)i.push(r+2*s+2),i.push(r+2*s+1),i.push(r+2*s+0),i.push(r+2*s+2),i.push(r+2*s+3),i.push(r+2*s+1),t.geom_count+=2;}function Nt(e,t,i,r,s,n,o){var a=n.vertex_template,l=n.vertex_data;a[0]=e[0],a[1]=e[1];let u=n.half_width*o;a[n.extrude_index+0]=t[0]*u,a[n.extrude_index+1]=t[1]*u,n.offset&&(a[n.offset_index+0]=i[0]*n.offset,a[n.offset_index+1]=i[1]*n.offset),null!=n.texcoord_index&&(a[n.texcoord_index+0]=r*xt,a[n.texcoord_index+1]=s*xt),l.addVertex(a);}var St=[0,0];function Pt(e,t,i,r,s,n,o,a,l,u,h){for(var c=t[0]*r[1]-t[1]*r[0],d=ue.dot(t,r),f=Math.atan2(c,d);f>=Math.PI;)f-=2*Math.PI;if(u)_=1;else{var _=function(e,t){e<0&&(e=-e);var i=t>2*gt?Math.log2(t/gt):1;return Math.ceil(e/Math.PI*i)}(f,h.half_width);if(_<1)return}var p=h.vertex_data.vertex_count,m=h.vertex_data.vertex_elements;Nt(e,i,s,o[0],o[1],h,1),Nt(e,t,s,n[0],n[1],h,1);var g=t,x=null!=h.texcoord_index;if(x)if(l)var y=ue.sub(n,o);else{St=ue.copy(n);var v=ue.div(ue.sub(a,n),_);}var b=f/_;let A,T,w=f<0?-1:1;c>0?(A=2,T=1):(A=1,T=2);for(var E=0;E<_;E++)0===E&&f<0&&(g=ue.neg(g)),g=ue.rot(g,b),x&&(l?(y=ue.rot(y,b),St[0]=y[0]+o[0],St[1]=y[1]*h.texcoord_width*h.v_scale+o[1]):St=ue.add(St,v)),Nt(e,g,s,St[0],St[1],h,w),m.push(p+E+A),m.push(p),m.push(p+E+T);}function kt(e,t,i,r,s,n){var o=ue.neg(i),a=null!=n.texcoord_index;switch(r){case _t.square:var l;s?(l=[i[1],-i[0]],Nt(e,ue.add(i,l),i,1,t,n,1),Nt(e,ue.add(o,l),i,0,t,n,1),a&&(t+=.5*n.texcoord_width*n.v_scale),Nt(e,i,i,1,t,n,1),Nt(e,o,i,0,t,n,1)):(l=[-i[1],i[0]],Nt(e,i,i,1,t,n,1),Nt(e,o,i,0,t,n,1),a&&(t+=.5*n.texcoord_width*n.v_scale),Nt(e,ue.add(i,l),i,1,t,n,1),Nt(e,ue.add(o,l),i,0,t,n,1)),zt(1,n);break;case _t.round:var u,h,c=vt,d=bt,f=At;s?(u=i,h=o,a&&(t+=.5*n.texcoord_width*n.v_scale,c=bt,d=vt,f=At)):(u=o,h=i),a&&(vt[1]=t,bt[1]=t,At[1]=t),Pt(e,u,ft,h,i,c,f,d,!0,!1,n);break;case _t.butt:return}}const Lt=[255,255,255,255],It=[0,0,0,0];const Ot=Object.create(Ee);Ot.variants={},Ot.vertex_layouts={};Object.assign(Ot,{name:"lines",built_in:!0,vertex_shader_src:ht,fragment_shader_src:ct,selection:!0,init(){Ee.init.apply(this,arguments),this.defines.TANGRAM_EXTRUDE_LINES=!0,this.defines.TANGRAM_TEXTURE_COORDS=!0,this.outline_feature_style={},this.inline_feature_style=this.feature_style,this.dash_textures={};},calcDistance:(e,t)=>q.evalCachedDistanceProperty(e,t)||0,calcDistanceNextZoom(e,t){t.zoom++;let i=this.calcDistance(e,t);return t.zoom--,i},calcWidth(e,t,i){let r,s=this.calcDistance(e.width,i);if(!(s<0)){if(r=e.next_width?this.calcDistanceNextZoom(e.next_width,i):s/2,0===s&&0===r||r<0)return !1;if(t.width_unscaled=s,t.next_width_unscaled=r,e.next_width){let e=.5*(s+(r*=2));t.width=e*i.units_per_meter_overzoom,t.width_scale=1-r/e;}else t.width=s*i.units_per_meter_overzoom,t.width_scale=0;return e.texcoords&&(e.inline_texcoord_width?t.texcoord_width=e.inline_texcoord_width:t.texcoord_width=(t.width_unscaled||t.next_width_unscaled)*i.units_per_meter_overzoom/i.tile.overzoom2),!0}},calcOffset(e,t,i){if(e.offset_precalc)t.offset=e.offset_precalc,t.offset_scale=e.offset_scale_precalc;else if(e.offset){let r=this.calcDistance(e.offset,i);if(e.next_offset){let s=2*this.calcDistanceNextZoom(e.next_offset,i);Math.abs(r)>=Math.abs(s)?(t.offset=r*i.units_per_meter_overzoom,t.offset_scale=0!==r?1-s/r:0):(t.offset=s*i.units_per_meter_overzoom,t.offset_scale=0!==s?-1*(1-r/s):0);}else t.offset=r*i.units_per_meter_overzoom,t.offset_scale=0;}else t.offset=0,t.offset_scale=0;},_parseFeature(e,t,i){var r=this.feature_style;if(!1!==this.calcWidth(t,r,i)&&(this.calcOffset(t,r,i),r.color=this.parseColor(t.color,i),r.color)){if(r.variant=t.variant,r.z=q.evalCachedDistanceProperty(t.z,i)||q.defaults.z,r.height=e.properties.height||q.defaults.height,r.extrude=q.evalProperty(t.extrude,i),r.extrude&&("number"==typeof r.extrude?r.height=r.extrude:Array.isArray(r.extrude)&&(r.height=r.extrude[1])),r.extrude&&r.height&&(r.z+=r.height),r.z*=F.height_scale,r.cap=t.cap,r.join=t.join,r.miter_limit=t.miter_limit,r.tile_edges=t.tile_edges,r.outline=r.outline||{width:{},next_width:{},preprocessed:!0},t.outline&&!1!==t.outline.visible&&t.outline.color&&t.outline.width){let e=2*this.calcDistance(t.outline.width,i),s=2*this.calcDistanceNextZoom(t.outline.next_width,i);0===e&&0===s||e<0||s<0?(r.outline.width.value=null,r.outline.next_width.value=null,r.outline.color=null,r.outline.inline_texcoord_width=null,r.outline.texcoords=!1):(r.outline.width.value=e+r.width_unscaled,r.outline.next_width.value=s+r.next_width_unscaled,r.outline.inline_texcoord_width=r.texcoord_width,r.outline.offset_precalc=r.offset,r.outline.offset_scale_precalc=r.offset_scale,r.outline.color=t.outline.color,r.outline.interactive=t.outline.interactive,r.outline.cap=t.outline.cap,r.outline.join=t.outline.join,r.outline.miter_limit=t.outline.miter_limit,r.outline.texcoords=t.outline.texcoords,r.outline.style=t.outline.style,r.outline.variant=t.outline.variant,t.outline.order?r.outline.order=this.parseOrder(t.outline.order,i):r.outline.order=r.order,r.outline.order>r.order&&(r.outline.order=r.order),r.outline.order-=.5);}else r.outline.width.value=null,r.outline.next_width.value=null,r.outline.color=null,r.outline.inline_texcoord_width=null;return r}},_preprocess(e){if(e.color=q.createColorPropertyCache(e.color),e.width=q.createPropertyCache(e.width,q.parseUnits),e.width&&e.width.type!==q.CACHE_TYPE.STATIC&&(e.next_width=q.createPropertyCache(e.width,q.parseUnits)),e.offset=e.offset&&q.createPropertyCache(e.offset,q.parseUnits),e.offset&&e.offset.type!==q.CACHE_TYPE.STATIC&&(e.next_offset=q.createPropertyCache(e.offset,q.parseUnits)),e.z=q.createPropertyCache(e.z,q.parseUnits),e.dash=void 0!==e.dash?e.dash:this.dash,e.dash_key=e.dash&&this.dashTextureKey(e.dash),e.dash_background_color=void 0!==e.dash_background_color?e.dash_background_color:this.dash_background_color,e.dash_background_color=e.dash_background_color&&q.parseColor(e.dash_background_color),e.texture_merged=e.dash_key||(void 0!==e.texture?e.texture:this.texture),e.texcoords=this.texcoords||e.texture_merged?1:0,this.computeVariant(e),e.outline){e.outline.is_outline=!0,e.outline.style=e.outline.style||this.name,e.outline.color=q.createColorPropertyCache(e.outline.color),e.outline.width=q.createPropertyCache(e.outline.width,q.parseUnits),e.outline.next_width=q.createPropertyCache(e.outline.width,q.parseUnits),e.outline.interactive=null!=e.outline.interactive?e.outline.interactive:e.interactive,e.outline.cap=e.outline.cap||e.cap,e.outline.join=e.outline.join||e.join,e.outline.miter_limit=null!=e.outline.miter_limit?e.outline.miter_limit:e.miter_limit,e.outline.offset=e.offset;let t=this.styles[e.outline.style];t?(e.outline.dash=void 0!==e.outline.dash?e.outline.dash:t.dash,e.outline.texture=void 0!==e.outline.texture?e.outline.texture:t.texture,null!=e.outline.dash?(e.outline.dash_key=e.outline.dash&&this.dashTextureKey(e.outline.dash),e.outline.texture_merged=e.outline.dash_key):null===e.outline.dash?(e.outline.dash_key=null,e.outline.texture_merged=e.outline.texture):null!=e.outline.texture?(e.outline.dash_key=null,e.outline.texture_merged=e.outline.texture):(e.outline.dash=e.dash,e.outline.dash_key=e.outline.dash&&this.dashTextureKey(e.outline.dash),e.outline.texture_merged=e.outline.dash_key),e.outline.dash_background_color=void 0!==e.outline.dash_background_color?e.outline.dash_background_color:t.dash_background_color,e.outline.dash_background_color=void 0!==e.outline.dash_background_color?e.outline.dash_background_color:e.dash_background_color,e.outline.dash_background_color=e.outline.dash_background_color&&q.parseColor(e.outline.dash_background_color),e.outline.texcoords=t.texcoords||e.outline.texture_merged?1:0,void 0===e.outline.blend_order&&null!=e.blend_order&&(e.outline.blend_order=e.blend_order),this.computeVariant(e.outline,t)):(f({level:"warn",once:!0},`Layer group '${e.layers.join(", ")}': `+`line 'outline' specifies non-existent draw style '${e.outline.style}' (or maybe the style is `+"defined but is missing a 'base' or has another error), skipping outlines for features matching this layer group"),e.outline=null);}return e},dashTextureKey:e=>"__dash_"+JSON.stringify(e),getDashTexture(e){let t=this.dashTextureKey(e);if(null==this.dash_textures[t]){this.dash_textures[t]=!0;const i=function(e,t){void 0===t&&(t={});const i=t.dash_color||Lt,r=t.background_color||It,s=e,n=t.scale||1;s.length%2==1&&Array.prototype.push.apply(s,s);let o=!0,a=[];for(let e=0;e<s.length;e++){let t=Math.floor(s[e]*n);for(let e=0;e<t;e++)Array.prototype.push.apply(a,o?i:r);o=!o;}return a.reverse(),{pixels:a=new Uint8Array(a),length:a.length/4}}(e,{scale:20});w.create(this.gl,t,{data:i.pixels,height:i.length,width:1,filtering:"nearest"});}},async endData(e){const t=await Ee.endData.call(this,e);if(t){t.uniforms.u_has_line_texture=!1,t.uniforms.u_texture=w.default,t.uniforms.u_v_scale_adjust=F.tile_scale;let e=[];for(let i in t.meshes){let r=t.meshes[i].variant;if(r.texture){let s=t.meshes[i].uniforms=t.meshes[i].uniforms||{};if(s.u_has_line_texture=!0,s.u_texture=r.texture,s.u_texture_ratio=1,r.dash&&(s.u_v_scale_adjust=20*F.tile_scale,s.u_dash_background_color=r.dash_background_color||[0,0,0,0]),r.dash_key&&null==this.dash_textures[r.dash_key]){this.dash_textures[r.dash_key]=!0;try{await n.postMessage(this.main_thread_target+".getDashTexture",r.dash);}catch(e){f("trace",`${this.name}: line dash texture create failed because style no longer on main thread`);}}if(null==w.textures[r.texture])e.push(w.syncTexturesToWorker([r.texture]).then(e=>{let t=e[r.texture];t&&(s.u_texture_ratio=t.height/t.width);}));else{let e=w.textures[r.texture];s.u_texture_ratio=e.height/e.width;}}}await Promise.all(e);}return t},computeVariant(e,t){void 0===t&&(t=this);let i=e.offset?1:0;i+="/"+e.texcoords,i+="/"+(e.interactive?1:0),i+="/"+(e.extrude||e.z?1:0),i+="/"+e.is_outline,e.dash_key&&(i+=e.dash_key,e.dash_background_color&&(i+=e.dash_background_color)),e.texture_merged&&(i+=e.texture_merged);const r=t.getBlendOrderForDraw(e);i=z(i+="/"+r),e.variant=i,null==Ot.variants[i]&&(Ot.variants[i]={key:i,blend_order:r,mesh_order:e.is_outline?0:1,selection:e.interactive?1:0,offset:e.offset?1:0,z_or_offset:e.offset||e.extrude||e.z?1:0,texcoords:e.texcoords,texture:e.texture_merged,dash:e.dash,dash_key:e.dash_key,dash_background_color:e.dash_background_color});},vertexLayoutForMeshVariant(e){if(null==Ot.vertex_layouts[e.key]){const t=[{name:"a_position",size:4,type:Me.SHORT,normalized:!1},{name:"a_extrude",size:2,type:Me.SHORT,normalized:!1},{name:"a_offset",size:2,type:Me.SHORT,normalized:!1,static:e.offset?null:[0,0]},{name:"a_z_and_offset_scale",size:2,type:Me.SHORT,normalized:!1,static:e.z_or_offset?null:[0,0]},{name:"a_texcoord",size:2,type:Me.UNSIGNED_SHORT,normalized:!0,static:e.texcoords?null:[0,0]},{name:"a_color",size:4,type:Me.UNSIGNED_BYTE,normalized:!0},{name:"a_selection_color",size:4,type:Me.UNSIGNED_BYTE,normalized:!0,static:e.selection?null:[0,0,0,0]}];Ot.vertex_layouts[e.key]=new Le(t);}return Ot.vertex_layouts[e.key]},meshVariantTypeForDraw:e=>Ot.variants[e.variant],makeVertexTemplate(e,t){let i=0;return this.vertex_template[i++]=0,this.vertex_template[i++]=0,this.vertex_template[i++]=1024*e.width_scale,this.vertex_template[i++]=this.scaleOrder(e.order),this.vertex_template[i++]=0,this.vertex_template[i++]=0,t.variant.offset&&(this.vertex_template[i++]=0,this.vertex_template[i++]=0),t.variant.z_or_offset&&(this.vertex_template[i++]=e.z||0,this.vertex_template[i++]=1024*e.offset_scale),t.variant.texcoords&&(this.vertex_template[i++]=0,this.vertex_template[i++]=0),this.vertex_template[i++]=255*e.color[0],this.vertex_template[i++]=255*e.color[1],this.vertex_template[i++]=255*e.color[2],this.vertex_template[i++]=255*e.color[3],t.variant.selection&&(this.vertex_template[i++]=255*e.selection_color[0],this.vertex_template[i++]=255*e.selection_color[1],this.vertex_template[i++]=255*e.selection_color[2],this.vertex_template[i++]=255*e.selection_color[3]),this.vertex_template},buildLines(e,t,i,r){if(this.feature_style=this.outline_feature_style,t.outline&&null!=t.outline.color&&null!=t.outline.width.value){var s=this.styles[t.outline.style];s&&s.addFeature(i.feature,t.outline,i);}this.feature_style=this.inline_feature_style;let n=this.getTileMesh(i.tile,this.meshVariantTypeForDraw(t)),o=n.vertex_data,a=o.vertex_layout;return function(e,t,i,r,s,n,o,a){var l,u=t.cap?_t[t.cap]:_t.butt,h=t.join?pt[t.join]:pt.miter;if(h===pt.miter){const e=t.miter_limit||mt;var c=e*e;}s.a_texcoord&&(l=1/(t.texcoord_width*yt));var d={closed_polygon:n,remove_tile_edges:o,tile_edge_tolerance:a,miter_len_sq:c,join_type:h,cap_type:u,vertex_data:i,vertex_template:r,half_width:t.width/2,extrude_index:s.a_extrude,offset_index:s.a_offset,v_scale:l,texcoord_index:s.a_texcoord,texcoord_width:t.texcoord_width,offset:t.offset,geom_count:0};for(let t=0;t<e.length;t++)Tt(e[t],d);if(d.extra_lines)for(let e=0;e<d.extra_lines.length;e++)Tt(d.extra_lines[e],d);return d.geom_count}(e,t,o,this.makeVertexTemplate(t,n),a.index,r&&r.closed_polygon,!t.tile_edges&&r&&r.remove_tile_edges,F.tile_scale*i.tile.pad_scale*2)},buildPolygons(e,t,i){let r=0;for(let s=0;s<e.length;s++)r+=this.buildLines(e[s],t,i,{closed_polygon:!0,remove_tile_edges:!0});return r}});const Ft=128/Math.PI,Ut=16384/Math.PI,Dt=64,Gt=65535,Ct=128,jt=[0,2,2,0],Vt=[1,1,3,3],Bt=new Array(4);const $t=["left","top-left","bottom-left"],qt=["right","top-right","bottom-right"],Ht=["top","top-left","top-right"],Xt=["bottom","bottom-left","bottom-right"],Wt={computeOffset(e,t,i,r){if(void 0===r&&(r=null),!i||"center"===i)return e;let s=[e[0],e[1]];return r=r||this.default_buffer,this.isLeftAnchor(i)?(s[0]-=t[0]/2,"left"===i&&(s[0]-=r[0])):this.isRightAnchor(i)&&(s[0]+=t[0]/2,"right"===i&&(s[0]+=r[1])),this.isTopAnchor(i)?(s[1]-=t[1]/2,"top"===i&&(s[1]-=r[2])):this.isBottomAnchor(i)&&(s[1]+=t[1]/2,"bottom"===i&&(s[1]+=r[3])),s},alignForAnchor(e){if(e&&"center"!==e){if(this.isLeftAnchor(e))return "right";if(this.isRightAnchor(e))return "left"}return "center"},isLeftAnchor:e=>$t.indexOf(e)>-1,isRightAnchor:e=>qt.indexOf(e)>-1,isTopAnchor:e=>Ht.indexOf(e)>-1,isBottomAnchor:e=>Xt.indexOf(e)>-1,default_buffer:[2.5,2.5,1.5,.75],zero_buffer:[0,0,0,0]};function Zt(e,t){return !(e[2]<t[0]||e[0]>t[2]||e[3]<t[1]||e[1]>t[3])}const Yt=[[1,0],[0,1]],Jt=[],Kt=[];let Qt,ei,ti,ii;class ri{constructor(e,t,i,r,s){this.dimension=[r/2,s/2],this.angle=i,this.centroid=[e,t],this.quad=null,this.axis_0=null,this.axis_1=null,this.update();}toJSON(){return {x:this.centroid[0],y:this.centroid[1],a:this.angle,w:this.dimension[0],h:this.dimension[1]}}getExtent(){if(0===this.angle)return [this.quad[0],this.quad[1],this.quad[4],this.quad[5]];return [Math.min(this.quad[0],this.quad[2],this.quad[4],this.quad[6]),Math.min(this.quad[1],this.quad[3],this.quad[5],this.quad[7]),Math.max(this.quad[0],this.quad[2],this.quad[4],this.quad[6]),Math.max(this.quad[1],this.quad[3],this.quad[5],this.quad[7])]}updateAxes(){this.axis_0=ue.normalize([this.quad[4]-this.quad[6],this.quad[5]-this.quad[7]]),this.axis_1=ue.normalize([this.quad[4]-this.quad[2],this.quad[5]-this.quad[3]]);}update(){const e=this.centroid,t=this.dimension[0],i=this.dimension[1];if(0===this.angle)this.quad=[e[0]-t,e[1]-i,e[0]+t,e[1]-i,e[0]+t,e[1]+i,e[0]-t,e[1]+i],this.axis_0=Yt[0],this.axis_1=Yt[1];else{let r=Math.cos(this.angle)*t,s=Math.sin(this.angle)*t,n=-Math.sin(this.angle)*i,o=Math.cos(this.angle)*i;this.quad=[e[0]-r-n,e[1]-s-o,e[0]+r-n,e[1]+s-o,e[0]+r+n,e[1]+s+o,e[0]-r+n,e[1]-s+o],this.updateAxes();}}static projectToAxis(e,t,i){let r=e.quad;return Qt=r[0]*t[0]+r[1]*t[1],ei=r[2]*t[0]+r[3]*t[1],ti=r[4]*t[0]+r[5]*t[1],ii=r[6]*t[0]+r[7]*t[1],i[0]=Math.min(Qt,ei,ti,ii),i[1]=Math.max(Qt,ei,ti,ii),i}static axisCollide(e,t,i,r){return ri.projectToAxis(e,i,Jt),ri.projectToAxis(t,i,Kt),!(Kt[0]>Jt[1]||Kt[1]<Jt[0])&&(ri.projectToAxis(e,r,Jt),ri.projectToAxis(t,r,Kt),!(Kt[0]>Jt[1]||Kt[1]<Jt[0]))}static intersect(e,t){return ri.axisCollide(e,t,e.axis_0,e.axis_1)&&ri.axisCollide(e,t,t.axis_0,t.axis_1)}}class si{constructor(e,t){void 0===t&&(t={}),this.id=si.nextLabelId(),this.type="",this.size=e,this.layout=t,this.position=null,this.angle=0,this.anchor=Array.isArray(this.layout.anchor)?this.layout.anchor[0]:this.layout.anchor,this.placed=null,this.offset=t.offset,this.unit_scale=this.layout.units_per_pixel,this.aabb=null,this.obb=null,this.align="center",this.throw_away=!1;}toJSON(){return {id:this.id,type:this.type,obb:this.obb.toJSON(),position:this.position,angle:this.angle,size:this.size,offset:this.offset,breach:this.breach,may_repeat_across_tiles:this.may_repeat_across_tiles,layout:ni(this.layout)}}update(){this.align=this.layout.align||Wt.alignForAnchor(this.anchor);}occluded(e,t){void 0===t&&(t=null);let i=!1,r=e.aabb,s=e.obb;return r.length>0&&function(e,t,i){for(let r=0;r<t.length&&(!Zt(e,t[r])||null==i(r));r++);}(this.aabb,r,e=>{if(!t||r[e]!==t.aabb)return 0===this.obb.angle&&0===s[e].angle?(i=!0,!0):ri.intersect(this.obb,s[e])?(i=!0,!0):void 0}),i}inTileBounds(){return !(this.aabb[0]>=0&&this.aabb[1]>-F.tile_scale&&this.aabb[0]<F.tile_scale&&this.aabb[1]<=0||this.aabb[2]>=0&&this.aabb[3]>-F.tile_scale&&this.aabb[2]<F.tile_scale&&this.aabb[3]<=0)}mayRepeatAcrossTiles(){if(this.layout.collide)return !0;const e=this.layout.repeat_distance;return 0!==e&&(Math.abs(this.position[0])<e||Math.abs(this.position[0]-F.tile_scale)<e||Math.abs(this.position[1])<e||Math.abs(-(this.position[1]-F.tile_scale))<e)}discard(e,t){return void 0===t&&(t=null),!!this.throw_away||this.occluded(e,t)}}function ni(e){return {priority:e.priority,collide:e.collide,repeat_distance:e.repeat_distance,repeat_group:e.repeat_group,buffer:e.buffer,italic:e.italic}}si.add=function(e,t){if(e.placed=!0,e.aabb&&(t.aabb.push(e.aabb),t.obb.push(e.obb)),e.aabbs)for(let i=0;i<e.aabbs.length;i++)t.aabb.push(e.aabbs[i]),t.obb.push(e.obbs[i]);},si.id=0,si.id_prefix=0,si.id_multiplier=0,si.nextLabelId=function(){return si.id_prefix+si.id++*si.id_multiplier},si.epsilon=.9999;class oi{constructor(e,t){this.key=e,this.repeat_dist=t,this.repeat_dist_sq=this.repeat_dist*this.repeat_dist,this.positions=[];}check(e){let t=e.position;for(let e=0;e<this.positions.length;e++){let i=this.positions[e],r=t[0]-i[0],s=t[1]-i[1];if(r*r+s*s<this.repeat_dist_sq)return !0}}add(e){e&&e.position&&this.positions.push(e.position);}static clear(e){this.groups[e]={};}static check(e,t,i){if(t.repeat_distance&&t.repeat_group&&this.groups[i][t.repeat_group])return this.groups[i][t.repeat_group].check(e)}static add(e,t,i){t.repeat_distance&&t.repeat_group&&(null==this.groups[i][t.repeat_group]&&(this.groups[i][t.repeat_group]=new oi(t.repeat_group,t.repeat_distance*t.repeat_scale)),this.groups[i][t.repeat_group].add(e));}}oi.groups={};const ai={tiles:{},startTile(e,t){let i=void 0===t?{}:t,r=i.apply_repeat_groups,s=void 0===r||r,n=i.return_hidden,o=void 0!==n&&n,a=this.tiles[e]={bboxes:{aabb:[],obb:[]},objects:{},labels:{},styles:{},repeat:s,return_hidden:o};null==a.complete&&(a.complete=new Promise((e,t)=>{a.resolve=e,a.reject=t;}));},resetTile(e){delete this.tiles[e];},abortTile(e){this.tiles[e]&&this.tiles[e].resolve&&this.tiles[e].resolve([]),this.resetTile(e);},addStyle(e,t){this.tiles[t].styles[e]=!0;},collide(e,t,i){let r=this.tiles[i];if(!r)return f("trace","Collision.collide() called with null tile",i,this.tiles,t,e),Promise.resolve([]);let s=r.objects;for(let i=0;i<e.length;i++){let r=e[i],n=r.label.layout.priority;s[n]=s[n]||{},s[n][t]=s[n][t]||[],s[n][t].push(r);}return delete r.styles[t],0===Object.keys(r.styles).length&&this.endTile(i),r.complete.then(()=>(r.resolve=null,r.labels[t]||[]))},endTile(e){let t=this.tiles[e],i=t.labels;t.repeat&&oi.clear(e);let r=Object.keys(t.objects).sort((e,t)=>e-t);for(let s=0;s<r.length;s++){let n=t.objects[r[s]];if(n)for(let r in n){let s=n[r];i[r]=i[r]||[];for(let n=0;n<s.length;n++){let o=s[n];this.canBePlaced(o,e,o.linked,t)?o.linked?this.canBePlaced(o.linked,e,o,t)?(o.show=!0,(o.label.breach||o.linked.label.breach)&&(o.label.breach=!0,o.linked.label.breach=!0),(o.label.may_repeat_across_tiles||o.linked.label.may_repeat_across_tiles)&&(o.label.may_repeat_across_tiles=!0,o.linked.label.may_repeat_across_tiles=!0),i[r].push(o),this.place(o,e,t),this.place(o.linked,e,t)):t.return_hidden&&(o.show=!1,i[r].push(o)):(o.show=!0,i[r].push(o),this.place(o,e,t)):t.return_hidden&&(o.show=!1,i[r].push(o));}}}delete this.tiles[e],t.resolve();},canBePlaced(e,t,i,r){void 0===i&&(i=null);let s=(void 0===r?{}:r).repeat,n=void 0===s||s,o=e.label,a=e.label.layout;if(null!=o.placed)return o.placed;let l=this.tiles[t].bboxes;if(a.collide&&o.discard(l,i&&i.label))a.collide&&(o.placed=!1);else{if(!n||!oi.check(o,a,t))return !0;o.placed=!1;}return o.placed},place(e,t,i){let r=e.label,s=i.repeat,n=void 0===s||s;null==r.placed&&(n&&oi.add(r,r.layout,t),si.add(r,this.tiles[t].bboxes));}};class li extends si{constructor(e,t,i,r){void 0===r&&(r=0),super(t,i),this.type="point",this.position=[e[0],e[1]],this.angle=r,this.parent=this.layout.parent,this.update(),this.start_anchor_index=1,this.degenerate=!(this.size[0]||this.size[1]||this.layout.buffer[0]||this.layout.buffer[1]),this.throw_away=!1;}update(){super.update(),this.computeOffset(),this.updateBBoxes();}computeOffset(){if(this.offset=[this.layout.offset[0],this.layout.offset[1]],this.parent){let e=this.parent;this.offset=Wt.computeOffset(this.offset,e.size,e.anchor,Wt.zero_buffer),this.offset=Wt.computeOffset(this.offset,e.size,this.anchor,Wt.zero_buffer),e.offset!==q.zeroPair&&(this.offset===q.zeroPair?this.offset=e.offset:(this.offset[0]+=e.offset[0],this.offset[1]+=e.offset[1]));}this.offset=Wt.computeOffset(this.offset,this.size,this.anchor);}updateBBoxes(){let e=(this.size[0]+2*this.layout.buffer[0])*this.unit_scale*si.epsilon,t=(this.size[1]+2*this.layout.buffer[1])*this.unit_scale*si.epsilon;this.layout.italic&&(e+=5*this.unit_scale),this.obb=new ri(this.position[0]+this.offset[0]*this.unit_scale,this.position[1]-this.offset[1]*this.unit_scale,-this.angle,e,t),this.aabb=this.obb.getExtent(),this.inTileBounds&&(this.breach=!this.inTileBounds()),this.mayRepeatAcrossTiles&&(this.may_repeat_across_tiles=this.mayRepeatAcrossTiles());}discard(e,t){if(void 0===t&&(t=null),this.degenerate)return !1;if(super.discard(e,t)){if(Array.isArray(this.layout.anchor))for(let i=this.start_anchor_index;i<this.layout.anchor.length;i++)if(this.anchor=this.layout.anchor[i],this.update(),!super.discard(e,t))return !1;return !0}return !1}}li.PLACEMENT={VERTEX:0,MIDPOINT:1,SPACED:2,CENTROID:3};const ui=li.PLACEMENT,hi=80;function ci(e,t,i){const r=[],s=i.placement,n=Math.max(t[0],t[1])*i.placement_min_length_ratio*i.units_per_pixel;if(s===ui.SPACED){let s=function(e,t,i){let r=i.units_per_pixel,s=(i.placement_spacing||hi)*r,n=function(e){let t=0;for(let i=0;i<e.length-1;i++)t+=fi(e[i],e[i+1]);return t}(e);if(n<=t)return !1;let o=Math.max(Math.floor(n/s),1),a=[],l=[],u=.5*(n-(o-1)*s);for(let r=0;r<o;r++){let r=_i(e,u,t,i),n=r.position,o=r.angle;null!=n&&null!=o&&(a.push(n),l.push(o)),u+=s;}return {positions:a,angles:l}}(e,n,i);if(!s)return [];let o=s.positions,a=s.angles;for(let e=0;e<o.length;e++){let s=o[e],n=a[e];!0!==i.tile_edges&&Ue(s)||r.push(new li(s,t,i,n));}}else if(s===ui.VERTEX){let s,n;for(let o=0;o<e.length-1;o++)if(s=e[o],n=e[o+1],!0===i.tile_edges||!Ue(s)){const e=di(s,n,i.angle);r.push(new li(s,t,i,e));}const o=di(s,n,i.angle);r.push(new li(n,t,i,o));}else if(s===ui.MIDPOINT)for(let s=0;s<e.length-1;s++){let o=e[s],a=e[s+1],l=[.5*(o[0]+a[0]),.5*(o[1]+a[1])];if((!0===i.tile_edges||!Ue(l))&&(!n||fi(o,a)>n)){const e=di(o,a,i.angle);r.push(new li(l,t,i,e));}}return r}function di(e,t,i){return void 0===i&&(i=0),"auto"===i?Math.atan2(t[0]-e[0],t[1]-e[1]):i}function fi(e,t){return Math.sqrt(Math.pow(e[0]-t[0],2)+Math.pow(e[1]-t[1],2))}function _i(e,t,i,r){let s,n,o=0;for(let a=0;a<e.length-1;a++){let l=e[a],u=e[a+1];const h=fi(l,u);if(!(h<=i)&&(o+=h)>t){s=pi(l,u,o-t),n=di(l,u,r.angle);break}}return {position:s,angle:n}}function pi(e,t,i){let r=i/fi(e,t);return [r*e[0]+(1-r)*t[0],r*e[1]+(1-r)*t[1]]}const mi={key:e=>[e.style,e.weight,e.family,e.px_size,e.fill,e.stroke,e.stroke_width,e.transform,e.text_wrap,e.max_lines,e.supersample,_.device_pixel_ratio].join("/"),defaults:{style:"normal",weight:null,size:"12px",px_size:12,family:"Helvetica",fill:"white",text_wrap:15,max_lines:5,align:"center",stroke:null,stroke_width:0},compute(e,t,i){let r={};t.font=t.font||this.defaults,r.can_articulate=t.can_articulate,r.fill=t.font.fill&&_.toCSSColor(q.evalCachedColorProperty(t.font.fill,i))||this.defaults.fill,r.style=t.font.style||this.defaults.style,r.weight=t.font.weight||this.defaults.weight,t.font.family?(r.family=t.font.family,r.family!==this.defaults.family&&(r.family+=", "+this.defaults.family)):r.family=this.defaults.family,r.transform=t.font.transform,r.supersample=t.supersample_text?1.5:1,r.px_size=q.evalCachedProperty(t.font.px_size,i)*r.supersample,t.font.stroke&&t.font.stroke.color&&(r.stroke=_.toCSSColor(q.evalCachedColorProperty(t.font.stroke.color,i)||this.defaults.stroke),r.stroke_width=q.evalCachedProperty(t.font.stroke.width,i)||this.defaults.stroke_width),r.font_css=this.fontCSS(r);let s=t.text_wrap;return null==s&&"line"!==F.geometryType(e.geometry.type)&&(s=!0),!0===s&&(s=this.defaults.text_wrap),r.text_wrap=s,r.max_lines=t.max_lines||this.defaults.max_lines,r},fontCSS:e=>[e.style,e.weight,e.px_size+"px",e.family].filter(e=>e).join(" ")};var gi=B(function(e){!function(){function t(e,t){document.addEventListener?e.addEventListener("scroll",t,!1):e.attachEvent("scroll",t);}function i(e){this.a=document.createElement("div"),this.a.setAttribute("aria-hidden","true"),this.a.appendChild(document.createTextNode(e)),this.b=document.createElement("span"),this.c=document.createElement("span"),this.h=document.createElement("span"),this.f=document.createElement("span"),this.g=-1,this.b.style.cssText="max-width:none;display:inline-block;position:absolute;height:100%;width:100%;overflow:scroll;font-size:16px;",this.c.style.cssText="max-width:none;display:inline-block;position:absolute;height:100%;width:100%;overflow:scroll;font-size:16px;",this.f.style.cssText="max-width:none;display:inline-block;position:absolute;height:100%;width:100%;overflow:scroll;font-size:16px;",this.h.style.cssText="display:inline-block;width:200%;height:200%;font-size:16px;max-width:none;",this.b.appendChild(this.h),this.c.appendChild(this.f),this.a.appendChild(this.b),this.a.appendChild(this.c);}function r(e,t){e.a.style.cssText="max-width:none;min-width:20px;min-height:20px;display:inline-block;overflow:hidden;position:absolute;width:auto;margin:0;padding:0;top:-999px;left:-999px;white-space:nowrap;font:"+t+";";}function s(e){var t=e.a.offsetWidth,i=t+100;return e.f.style.width=i+"px",e.c.scrollLeft=i,e.b.scrollLeft=e.b.scrollWidth+100,e.g!==t&&(e.g=t,!0)}function n(e,i){function r(){var e=n;s(e)&&e.a.parentNode&&i(e.g);}var n=e;t(e.b,r),t(e.c,r),s(e);}function o(e,t){var i=t||{};this.family=e,this.style=i.style||"normal",this.weight=i.weight||"normal",this.stretch=i.stretch||"normal";}var a=null,l=null,u=null,h=null;function c(){if(null===u){var e=document.createElement("div");try{e.style.font="condensed 100px sans-serif";}catch(e){}u=""!==e.style.font;}return u}function d(e,t){return [e.style,e.weight,c()?e.stretch:"","100px",t].join(" ")}o.prototype.load=function(e,t){var s=this,o=e||"BESbswy",u=0,c=t||3e3,f=(new Date).getTime();return new Promise(function(e,t){var _;if(null===h&&(h=!!document.fonts),(_=h)&&(null===l&&(l=/OS X.*Version\/10\..*Safari/.test(navigator.userAgent)&&/Apple/.test(navigator.vendor)),_=!l),_){_=new Promise(function(e,t){!function i(){(new Date).getTime()-f>=c?t():document.fonts.load(d(s,'"'+s.family+'"'),o).then(function(t){1<=t.length?e():setTimeout(i,25);},function(){t();});}();});var p=new Promise(function(e,t){u=setTimeout(t,c);});Promise.race([p,_]).then(function(){clearTimeout(u),e(s);},function(){t(s);});}else!function(e){document.body?e():document.addEventListener?document.addEventListener("DOMContentLoaded",function t(){document.removeEventListener("DOMContentLoaded",t),e();}):document.attachEvent("onreadystatechange",function t(){"interactive"!=document.readyState&&"complete"!=document.readyState||(document.detachEvent("onreadystatechange",t),e());});}(function(){function l(){var t;(t=-1!=m&&-1!=g||-1!=m&&-1!=x||-1!=g&&-1!=x)&&((t=m!=g&&m!=x&&g!=x)||(null===a&&(t=/AppleWebKit\/([0-9]+)(?:\.([0-9]+))/.exec(window.navigator.userAgent),a=!!t&&(536>parseInt(t[1],10)||536===parseInt(t[1],10)&&11>=parseInt(t[2],10))),t=a&&(m==y&&g==y&&x==y||m==v&&g==v&&x==v||m==b&&g==b&&x==b)),t=!t),t&&(A.parentNode&&A.parentNode.removeChild(A),clearTimeout(u),e(s));}var h=new i(o),_=new i(o),p=new i(o),m=-1,g=-1,x=-1,y=-1,v=-1,b=-1,A=document.createElement("div");A.dir="ltr",r(h,d(s,"sans-serif")),r(_,d(s,"serif")),r(p,d(s,"monospace")),A.appendChild(h.a),A.appendChild(_.a),A.appendChild(p.a),document.body.appendChild(A),y=h.a.offsetWidth,v=_.a.offsetWidth,b=p.a.offsetWidth,function e(){if((new Date).getTime()-f>=c)A.parentNode&&A.parentNode.removeChild(A),t(s);else{var i=document.hidden;!0!==i&&void 0!==i||(m=h.a.offsetWidth,g=_.a.offsetWidth,x=p.a.offsetWidth,l()),u=setTimeout(e,50);}}(),n(h,function(e){m=e,l();}),r(h,d(s,'"'+s.family+'",sans-serif')),n(_,function(e){g=e,l();}),r(_,d(s,'"'+s.family+'",serif')),n(p,function(e){x=e,l();}),r(p,d(s,'"'+s.family+'",monospace'));});})},e.exports=o;}();});const xi={fonts_loaded:Promise.resolve(),last_loaded:null,loadFonts(e){const t=JSON.stringify(e)===this.last_loaded;if(e&&!t){const t=[];for(const i in e)Array.isArray(e[i])?e[i].forEach(e=>t.push(this.loadFontFace(i,e))):t.push(this.loadFontFace(i,e[i]));this.last_loaded=JSON.stringify(e),this.fonts_loaded=Promise.all(t.filter(e=>e));}return this.fonts_loaded},async loadFontFace(e,t){if(null==t||"object"!=typeof t&&"external"!==t)return;const i={family:e};"object"==typeof t&&(Object.assign(i,t),"string"==typeof t.url&&await this.injectFontFace(i));try{const t=new gi(e,i);await t.load(),f("debug",`Font face '${e}' is available`,i);}catch(t){f("debug",`Font face '${e}' is NOT available`,i);}},async injectFontFace(e){let t=e.family,i=e.url,r=e.weight,s=e.style;void 0===this.supports_native_font_loading&&(this.supports_native_font_loading=void 0!==window.FontFace);let n=i;if("blob:"===i.slice(0,5)){n=await _.io(i,6e4,"arraybuffer");let e=new Uint8Array(n);if(this.supports_native_font_loading)n=e;else{let t="";for(let i=0;i<e.length;i++)t+=String.fromCharCode(e[i]);n="data:font/opentype;base64,"+btoa(t);}}if(this.supports_native_font_loading){let e;"string"==typeof n?e=new FontFace(t,`url(${encodeURI(n)})`,{weight:r,style:s}):n instanceof Uint8Array&&(e=new FontFace(t,n,{weight:r,style:s})),document.fonts.add(e),f("trace","Adding FontFace to document.fonts:",e);}else{let e=`\n                @font-face {\n                    font-family: '${t}';\n                    font-weight: ${r||"normal"};\n                    font-style: ${s||"normal"};\n                    src: url(${encodeURI(n)});\n                }`,i=document.createElement("style");i.appendChild(document.createTextNode("")),document.head.appendChild(i),i.sheet.insertRule(e,0),f("trace","Injecting CSS font face:",e);}}},yi=new RegExp("[֑-߿‏‫‮יִ-﷽ﹰ-ﻼ]");function vi(e){return yi.test(e)}const bi="\0-/:-@[-`{-¿×÷ʹ-˿ -⯿‐-\u2029‬ -⯿",Ai=new RegExp("["+bi+"]+");function Ti(e){return Ai.test(e)}const wi="‏",Ei=new RegExp("^["+bi+"؀-ۿ]+"),Ri=new RegExp("["+bi+"آ-إاد-زوٱ-ٷڈ-ڙۄ-ۋۏےۓۮۯ]"),Mi=new RegExp("^[ؐ-ًؚ-ٰٟۖ-ۜ۟-۪ۤۧۨ-ۭ]+"),zi="[̀-֑ͯ-ׇֽֿׁׂׅׄަ-ްऀ-ःऺ-ौॎॏ॑-ॗॢॣঁ-ঃ়া-ৌৗৢৣਁ-ਃ਼-ੌੑઁ-ઃ઼ા-ૌૢૣଁ-ଃ଼ା-ୌୖୗୢୣஂா-்ௗఀ-ఃా-ౌౕౖౢౣಁ-ಃ಼ಾ-ೌೕೖೢೣഁ-ഃാ-ൌൎൗൢൣංඃ්-ෟෲෳัิ-ฺ็-๎ັິ-ຼ່-ໍ༹༘༙༵༷༾༿ཱ-ྃ྆྇ྍ-ྼ࿆ါ-း်-ှၖ-ၙၞ-ၠၢ-ၤၧ-ၭၱ-ၴႂ-ႍႏႚ-ႝ឴-៑៓ᩕ-ᩞᩡ-᩼᷀-᷿⃐-⃿]",Ni=new RegExp("^.(?:"+zi+"+)?([्্੍્୍్್്྄္្᩠᩿]\\W(?:"+zi+"+)?)*"),Si={Mongolian:"᠀-᢯"},Pi=Object.keys(Si).map(e=>Si[e]).join(""),ki=new RegExp("["+Pi+"]");const Li=2;function Ii(e,t,i){const r=t?1:Li;if(e.length<r)return [e];let s=e;if(i.segment[s])return i.stats.segment_hits++,i.segment[s];let n=[];if(Ei.exec(e)){n=e.split(Ri);let t=-1;for(var o=0;o<n.length-1;o++){if(o>0){let e=Mi.exec(n[o]);e&&(n[o]=n[o].substring(e[0].length),n[o-1]+=e[0],t+=e[0].length);}t+=1+n[o].length,n[o]+=e.slice(t,t+1);}e="";}for(;e.length;){let t="",i=e,s=0;for(;s<r&&i.length;s++){let e=(Ni.exec(i)||i)[0];t+=e,i=i.substring(e.length);}n.push(t),e=e.substring(t.length);}return t&&n.reverse(),i.stats.segment_misses++,i.segment[s]=n,n}class Oi{constructor(e,t,i){void 0===t&&(t=1/0),void 0===i&&(i=1/0),this.width=0,this.height=0,this.lines=[],this.max_lines=t,this.text_wrap=i,this.context=e;}createLine(e){return this.lines.length<this.max_lines&&new Fi(e,this.text_wrap)}push(e){if(this.lines.length<this.max_lines){let t=this.context.measureText(e.text).width;return e.width=t,t>this.width&&(this.width=Math.ceil(t)),this.lines.push(e),this.height+=e.height,!0}return this.addEllipsis(),!1}advance(e,t){return !!this.push(e)&&this.createLine(t)}addEllipsis(){let e=this.lines[this.lines.length-1],t=Math.ceil(this.context.measureText(Oi.ellipsis).width);e.append(Oi.ellipsis),e.width+=t,e.width>this.width&&(this.width=e.width);}finish(e){e?this.push(e):this.addEllipsis();}static parse(e,t,i,r,s){let n;n="number"==typeof t?e.split(" "):[e];let o=new Oi(s,i,t),a=o.createLine(r);for(let e=0;e<n.length;e++){let i=n[e].split("\n"),s=0===e;for(let n=0;n<i.length&&a;n++){let l=i[n];vi(l)&&Ti(l[l.length-1])&&(l+=wi);let u=s?l:" "+l;if(t&&e>0&&a.exceedsTextwrap(u)){if(!(a=o.advance(a,r)))break;a.append(l),s=!0;}else a.append(u);n<i.length-1&&(a=o.advance(a,r),s=!0);}e===n.length-1&&o.finish(a);}return o}}Oi.ellipsis="...";class Fi{constructor(e,t){void 0===e&&(e=0),void 0===t&&(t=0),this.chars=0,this.text="",this.height=Math.ceil(e),this.text_wrap=t;}append(e){this.chars+=e.length,this.text+=e;}exceedsTextwrap(e){return e.length+this.chars>this.text_wrap}}class Ui{constructor(){this.createCanvas(),this.vertical_text_buffer=8,this.horizontal_text_buffer=4;}createCanvas(){this.canvas=document.createElement("canvas"),this.canvas.style.backgroundColor="transparent",this.context=this.canvas.getContext("2d");}resize(e,t){this.canvas.width=e,this.canvas.height=t,this.context.clearRect(0,0,e,t);}setFont(e){let t=e.font_css,i=e.fill,r=e.stroke,s=e.stroke_width,n=e.px_size,o=e.supersample;this.px_size=n;let a=this.context,l=_.device_pixel_ratio*o;r&&s>0&&(a.strokeStyle=r,a.lineWidth=s*l),a.fillStyle=i,a.font=t,a.miterLimit=2;}async textSizes(e,t){return await xi.loadFonts(),b.add({type:"textSizes",run:this.processTextSizesTask.bind(this),texts:t,tile_id:e,cursor:{styles:Object.keys(t),texts:null,style_idx:null,text_idx:null}})}processTextSizesTask(e){let t=e.cursor,i=e.texts;for(t.style_idx=t.style_idx||0;t.style_idx<t.styles.length;){let s=t.styles[t.style_idx];null==t.text_idx&&(t.text_idx=0,t.texts=Object.keys(i[s]));let n=i[s],o=!0;for(;t.text_idx<t.texts.length;){let i=t.texts[t.text_idx],a=n[i],l=a.text_settings;if(o&&(this.setFont(l),o=!1),a.size=this.textSize(s,i,l).size,l.can_articulate){let e=!1,t=!1;if(vi(i)&&(Ti(i)?e=!0:t=!0),a.isRTL=e,a.no_curving=t||(r=i,ki.test(r)),a.vertical_buffer=this.vertical_text_buffer,a.segment_sizes=[],!a.no_curving){let t=Ii(i,e,Ui.cache);a.segments=t;for(let e=0;e<t.length;e++)a.segment_sizes.push(this.textSize(s,t[e],l).size);}}if(t.text_idx++,!b.shouldContinue(e))return !1}t.text_idx=null,t.style_idx++;}var r;return b.finish(e,i),!0}textSize(e,t,i){let r=i.transform,s=i.text_wrap,n=i.max_lines,o=i.stroke_width,a=void 0===o?0:o,l=i.supersample;if(Ui.cache.text[e]=Ui.cache.text[e]||{},Ui.cache.text[e][t])return Ui.cache.stats.text_hits++,Ui.cache.text[e][t];Ui.cache.stats.text_misses++,Ui.cache.text_count++;let u=_.device_pixel_ratio*l,h=this.applyTextTransform(t,r),c=this.context,d=this.vertical_text_buffer*u,f=u*(a+this.horizontal_text_buffer),p=2*u,m=this.px_size+p,g=Oi.parse(h,s,n,m,c),x=g.height,y=g.width,v=g.lines,b=[y/u,x/u],A=[y+2*f,x+2*d],T=[A[0]/u,A[1]/u];return Ui.cache.text[e][t]={lines:v,size:{collision_size:b,texture_size:A,logical_size:T,line_height:m}},Ui.cache.text[e][t]}drawTextMultiLine(e,t,i,r,s){let n=t[0],o=t[1],a=r.stroke,l=r.stroke_width,u=void 0===l?0:l,h=r.transform,c=r.align,d=r.supersample,f=i.line_height,p=o;for(let t=0;t<e.length;t++){let r=e[t];this.drawTextLine(r,[n,p],i,{stroke:a,stroke_width:u,transform:h,align:c,supersample:d},s),p+=f;}if(m.draw_label_collision_boxes){this.context.save();let e=_.device_pixel_ratio*d,t=e*(this.horizontal_text_buffer+u),r=e*this.vertical_text_buffer,a=i.collision_size,l=2;this.context.strokeStyle="blue",this.context.lineWidth=l,this.context.strokeRect(n+t,o+r,e*a[0],e*a[1]),"curved"===s&&this.context.strokeRect(n+i.texture_size[0]+t,o+r,e*a[0],e*a[1]),this.context.restore();}if(m.draw_label_texture_boxes){this.context.save();let e=i.texture_size,t=2;this.context.strokeStyle="green",this.context.lineWidth=t,this.context.strokeRect(n+t,o+t,e[0]-2*t,e[1]-2*t),"curved"===s&&this.context.strokeRect(n+t+i.texture_size[0],o+t,e[0]-2*t,e[1]-2*t),this.context.restore();}}drawTextLine(e,t,i,r,s){let n=t[0],o=t[1],a=r.stroke,l=r.stroke_width,u=void 0===l?0:l,h=r.transform,c=r.align,d=r.supersample,f=_.device_pixel_ratio*d;c=c||"center";let p,m=this.vertical_text_buffer*f,g=i.texture_size,x=i.line_height,y=f*(u+this.horizontal_text_buffer),v=this.applyTextTransform(e.text,h);"left"===c?p=n+y:"center"===c?p=n+g[0]/2-e.width/2:"right"===c&&(p=n+g[0]-e.width-y);let b=o+.75*m+x;if(a&&u>0){let e="curved"===s?g[0]:0;this.context.strokeText(v,p+e,b);}this.context.fillText(v,p,b);}rasterize(e,t,i,r,s){return b.add({type:"rasterizeLabels",run:this.processRasterizeTask.bind(this),cancel:this.cancelRasterizeTask.bind(this),pause_factor:2,user_moving_view:!1,texts:e,textures:t,texture_prefix:r,gl:s,tile_id:i,cursor:{styles:Object.keys(e),texts:null,style_idx:0,text_idx:null,texture_idx:0,texture_resize:!0,texture_names:[]}})}processRasterizeTask(e){let t,i=e.cursor,r=e.texts,s=e.textures;for(;i.texture_idx<e.textures.length;){for(t=s[i.texture_idx],i.texture_resize&&(i.texture_resize=!1,this.resize(...t.texture_size));i.style_idx<i.styles.length;){let s=i.styles[i.style_idx];null==i.text_idx&&(i.text_idx=0,i.texts=Object.keys(r[s]));let n=r[s],o=!0;for(;i.text_idx<i.texts.length;){let r=i.texts[i.text_idx],a=n[r],l=a.text_settings;if(o&&(this.setFont(l),o=!1),l.can_articulate){a.texcoords=a.texcoords||{};for(let e=0;e<a.type.length;e++){let n=a.type[e];if("straight"===n){if(a.textures[e]!==i.texture_idx)continue;let o,u=a.isRTL?r.split().reverse().join():r,h=t.texcoord_cache[s][u];if(h.texcoord)o=h.texcoord;else{let e=h.texture_position,i=this.textSize(s,u,l),r=i.size,a=i.lines;this.drawTextMultiLine(a,e,r,l,n),o=w.getTexcoordsForSprite(e,r.texture_size,t.texture_size),h.texcoord=o;}a.texcoords[n]={texcoord:o,texture_id:h.texture_id};}else if("curved"===n){let r=a.segments;a.texcoords.curved=a.texcoords.curved||[],a.texcoords_stroke=a.texcoords_stroke||[];for(let o=0;o<r.length;o++){if(a.textures[e][o]!==i.texture_idx)continue;let u,h,c=r[o],d=t.texcoord_cache[s][c];if(d.texcoord)u=d.texcoord,h=d.texcoord_stroke,a.texcoords_stroke.push(h);else{let e=d.texture_position,i=this.textSize(s,c,l),r=i.size,o=i.lines;this.drawTextMultiLine(o,e,r,l,n),u=w.getTexcoordsForSprite(e,r.texture_size,t.texture_size);let f=[e[0]+r.texture_size[0],e[1]];h=w.getTexcoordsForSprite(f,r.texture_size,t.texture_size),d.texcoord=u,d.texcoord_stroke=h,a.texcoords_stroke.push(h);}a.texcoords.curved.push({texcoord:u,texture_id:d.texture_id});}}}}else{let e=this.textSize(s,r,l).lines;for(let r in a.align)a.align[r].texture_id===i.texture_idx&&(this.drawTextMultiLine(e,a.align[r].texture_position,a.size,{stroke:l.stroke,stroke_width:l.stroke_width,transform:l.transform,supersample:l.supersample,align:r}),a.align[r].texcoords=w.getTexcoordsForSprite(a.align[r].texture_position,a.size.texture_size,t.texture_size));}if(i.text_idx++,!b.shouldContinue(e))return !1}i.text_idx=null,i.style_idx++;}let n=e.texture_prefix+i.texture_idx;w.create(e.gl,n,{element:this.canvas,filtering:"linear",UNPACK_PREMULTIPLY_ALPHA_WEBGL:!0}),w.retain(n),i.texture_names.push(n),i.texture_idx++,i.texture_resize=!0,i.style_idx=0;}return b.finish(e,i.texture_names),!0}cancelRasterizeTask(e){f("trace",`RasterizeTask: release textures [${e.cursor.texture_names.join(", ")}]`),e.cursor.texture_names.forEach(e=>w.release(e));}setTextureTextPositions(e,t){let i={cx:0,cy:0,width:0,height:0,column_width:0,texture_id:0,texcoord_cache:{}},r=[];for(let s in e){let n=e[s];for(let e in n){let o,a=n[e];if(a.text_settings.can_articulate){a.textures=[],i.texcoord_cache[s]=i.texcoord_cache[s]||{};for(let n=0;n<a.type.length;n++){let l=a.type[n];if("straight"===l){let l=a.isRTL?e.split().reverse().join():e;if(!i.texcoord_cache[s][l]){let e=a.size.texture_size;o=this.placeText(e[0],e[1],s,i,r,t),i.texcoord_cache[s][l]={texture_id:i.texture_id,texture_position:o};}a.textures[n]=i.texture_id;}else if("curved"===l){a.textures[n]=[];for(let e=0;e<a.segment_sizes.length;e++){let l=a.segments[e];if(!i.texcoord_cache[s][l]){let n=a.segment_sizes[e].texture_size,u=2*n[0];o=this.placeText(u,n[1],s,i,r,t),i.texcoord_cache[s][l]={texture_id:i.texture_id,texture_position:o};}a.textures[n].push(i.texture_id);}}}}else{let e=a.size.texture_size;for(let n in a.align)o=this.placeText(e[0],e[1],s,i,r,t),a.align[n].texture_id=i.texture_id,a.align[n].texture_position=o;}}}return i.column_width>0&&i.height>0&&(r[i.texture_id]={texture_size:[i.width,i.height],texcoord_cache:i.texcoord_cache}),r}placeText(e,t,i,r,s,n){let o;return r.cy+t>n?(r.cx+=r.column_width,r.cy=0,r.column_width=e):r.column_width=Math.max(r.column_width,e),r.cx+r.column_width<=n?(o=[r.cx,r.cy],r.cy+=t,r.height=Math.max(r.height,r.cy),r.width=Math.max(r.width,r.cx+r.column_width)):(s[r.texture_id]={texture_size:[r.width,r.height],texcoord_cache:r.texcoord_cache},r.texcoord_cache={},r.texcoord_cache[i]={},r.texture_id++,r.cx=0,r.cy=t,r.column_width=e,r.width=e,r.height=t,o=[0,0]),o}applyTextTransform(e,t){return "capitalize"===t?e.replace(/\w\S*/g,function(e){return e.charAt(0).toUpperCase()+e.substr(1)}):"uppercase"===t?e.toUpperCase():"lowercase"===t?e.toLowerCase():e}static fontPixelSize(e){if(null==e)return;let t=(e="string"==typeof e?e:String(e)).match(Ui.font_size_re)||[],i=t[1],r=t[2];return "em"===(r=r||"px")?i*=16:"pt"===r?i/=.75:"%"===r&&(i/=6.25),i=q.parsePositiveNumber(i),i*=_.device_pixel_ratio}static pruneTextCache(){Ui.cache.text_count>Ui.cache.text_count_max&&(Ui.cache.text={},Ui.cache.text_count=0,f("debug","TextCanvas: pruning text cache")),Object.keys(Ui.cache.segment).length>Ui.cache.segment_count_max&&(Ui.cache.segment={},f("debug","TextCanvas: pruning segment cache"));}}Ui.font_size_re=/((?:[0-9]*\.)?[0-9]+)\s*(px|pt|em|%)?/,Ui.cache={text:{},text_count:0,text_count_max:2e3,segment:{},segment_count_max:2e3,stats:{text_hits:0,text_misses:0,segment_hits:0,segment_misses:0}};let Di=0;const Gi={resetText(){t.is_main?this.canvas=new Ui:t.is_worker&&(this.texts={});},freeText(e){delete this.texts[e.id];},parseTextFeature(e,t,i,r){let s=this.parseTextSource(e,t,i);if(null==s||""===s)return;let n=mi.compute(e,t,i),o=mi.key(n);this.texts[r.id]=this.texts[r.id]||{};let a=this.texts[r.id][o]=this.texts[r.id][o]||{};if(s instanceof Object){let l=[],u=s.left+"-"+s.right;for(let h in s){let c=s[h];if(!c)continue;let d=this.computeTextLayout({},e,t,i,r,c,n,u,h);a[c]||(a[c]={text_settings:n,ref:0}),l.push({draw:t,text:c,text_settings_key:o,layout:d});}return l.length>0&&l}{let l=this.computeTextLayout({},e,t,i,r,s,n);return a[s]||(a[s]={text_settings:n,ref:0}),{draw:t,text:s,text_settings_key:o,layout:l}}},parseTextSource(e,t,i){let r,s=t.text_source||"name";if(null==s||Array.isArray(s)||"object"!=typeof s)r=this.parseTextSourceValue(s,e,i);else{r={};for(let t in s)r[t]=this.parseTextSourceValue(s[t],e,i);}return r},parseTextSourceValue(e,t,i){let r;if(Array.isArray(e)){for(let s=0;s<e.length;s++)if("string"==typeof e[s]?r=t.properties[e[s]]:"function"==typeof e[s]&&(r=e[s](i)),r)return r}else"string"==typeof e?r=t.properties[e]:e instanceof Function&&(r=e(i));return r},async prepareTextLabels(e,t){if(0===Object.keys(this.texts[e.id]||{}).length)return [];try{const i=await n.postMessage(this.main_thread_target+".calcTextSizes",e.id,this.texts[e.id]);return e.canceled?(f("trace",`Style ${this.name}: stop tile build because tile was canceled: ${e.key}, post-calcTextSizes()`),[]):(this.texts[e.id]=i||[],i?this.buildTextLabels(e,t):(ai.abortTile(e.id),[]))}catch(t){return ai.abortTile(e.id),[]}},async collideAndRenderTextLabels(e,t,r){let s=await this.prepareTextLabels(e,r);if(0===s.length)return ai.collide([],t,e.id),{};if(s=await ai.collide(s,t,e.id),e.canceled)return f("trace",`stop tile build because tile was canceled: ${e.key}, post-collide()`),{};let o=this.texts[e.id];if(null==o||0===s.length)return {};this.cullTextStyles(o,s),s.forEach(e=>{let t=e.text_settings_key,i=o[t]&&o[t][e.text];i.text_settings.can_articulate?(i.type||(i.type=[]),-1===i.type.indexOf(e.label.type)&&i.type.push(e.label.type)):(i.align=i.align||{},i.align[e.label.align]={});});try{const t=await n.postMessage(this.main_thread_target+".rasterizeTexts",e.id,e.key,o);return e.canceled?(f("trace",`stop tile build because tile was canceled: ${e.key}, post-rasterizeTexts()`),{}):i({labels:s},t)}catch(e){return {}}},cullTextStyles(e,t){for(let i=0;i<t.length;i++){let r=t[i];e[r.text_settings_key][r.text].ref++;}for(let t in e)for(let i in e[t])e[t][i].ref<1&&delete e[t][i];for(let t in e)0===Object.keys(e[t]).length&&delete e[t];},calcTextSizes(e,t){return this.canvas.textSizes(e,t)},async rasterizeTexts(e,t,i){let r=new Ui,s=Math.min(this.max_texture_size,2048),n=r.setTextureTextPositions(i,s),o=["labels",this.name,t,e,Di,""].join("-");return Di++,(n=await r.rasterize(i,n,e,o,this.gl))?{texts:i,textures:n}:{}},preprocessText(e){if(e&&e.font&&"object"==typeof e.font)return e.font.fill=q.createPropertyCache(e.font.fill),e.font.stroke&&(e.font.stroke.color=q.createPropertyCache(e.font.stroke.color)),e.font.px_size=q.createPropertyCache(e.font.size||mi.defaults.size,Ui.fontPixelSize),e.font.stroke&&null!=e.font.stroke.width&&(e.font.stroke.width=q.createPropertyCache(e.font.stroke.width,q.parsePositiveNumber)),e.offset=q.createPropertyCache(e.offset,e=>Array.isArray(e)&&e.map(q.parseNumber)),e.buffer=q.createPropertyCache(e.buffer,e=>(Array.isArray(e)?e:[e,e]).map(q.parsePositiveNumber)),e.repeat_distance=q.createPropertyCache(null!=e.repeat_distance?e.repeat_distance:F.tile_size,q.parsePositiveNumber),e},computeTextLayout(e,t,i,r,s,n,o,a,l){let u=e||{};return (u=this.computeLayout(u,t,i,r,s)).repeat_distance&&(a&&(u.repeat_group+="/"+a),u.repeat_group+="/"+n),u.subdiv=s.overzoom2,u.align=i.align,u.italic="normal"!==o.style,"right"===l?u.orientation=1:"left"===l&&(u.orientation=-1),u}};var Ci=function(e){return e[0]=1,e[1]=0,e[2]=0,e[3]=0,e[4]=0,e[5]=1,e[6]=0,e[7]=0,e[8]=0,e[9]=0,e[10]=1,e[11]=0,e[12]=0,e[13]=0,e[14]=0,e[15]=1,e};const ji={fromValues(e,t,i){var r=new Float64Array(3);return r[0]=e,r[1]=t,r[2]=i,r}},Vi={normalFromMat4:function(e,t){var i=t[0],r=t[1],s=t[2],n=t[3],o=t[4],a=t[5],l=t[6],u=t[7],h=t[8],c=t[9],d=t[10],f=t[11],_=t[12],p=t[13],m=t[14],g=t[15],x=i*a-r*o,y=i*l-s*o,v=i*u-n*o,b=r*l-s*a,A=r*u-n*a,T=s*u-n*l,w=h*p-c*_,E=h*m-d*_,R=h*g-f*_,M=c*m-d*p,z=c*g-f*p,N=d*g-f*m,S=x*N-y*z+v*M+b*R-A*E+T*w;return S?(S=1/S,e[0]=(a*N-l*z+u*M)*S,e[1]=(l*R-o*N-u*E)*S,e[2]=(o*z-a*R+u*w)*S,e[3]=(s*z-r*N-n*M)*S,e[4]=(i*N-s*R+n*E)*S,e[5]=(r*R-i*z-n*w)*S,e[6]=(p*T-m*A+g*b)*S,e[7]=(m*v-_*T-g*y)*S,e[8]=(_*A-p*v+g*x)*S,e):null},invert:function(e,t){var i=t[0],r=t[1],s=t[2],n=t[3],o=t[4],a=t[5],l=t[6],u=t[7],h=t[8],c=h*o-a*u,d=-h*n+a*l,f=u*n-o*l,_=i*c+r*d+s*f;return _?(_=1/_,e[0]=c*_,e[1]=(-h*r+s*u)*_,e[2]=(a*r-s*o)*_,e[3]=d*_,e[4]=(h*i-s*l)*_,e[5]=(-a*i+s*n)*_,e[6]=f*_,e[7]=(-u*i+r*l)*_,e[8]=(o*i-r*n)*_,e):null}},Bi={multiply:function(e,t,i){var r=t[0],s=t[1],n=t[2],o=t[3],a=t[4],l=t[5],u=t[6],h=t[7],c=t[8],d=t[9],f=t[10],_=t[11],p=t[12],m=t[13],g=t[14],x=t[15],y=i[0],v=i[1],b=i[2],A=i[3];return e[0]=y*r+v*a+b*c+A*p,e[1]=y*s+v*l+b*d+A*m,e[2]=y*n+v*u+b*f+A*g,e[3]=y*o+v*h+b*_+A*x,y=i[4],v=i[5],b=i[6],A=i[7],e[4]=y*r+v*a+b*c+A*p,e[5]=y*s+v*l+b*d+A*m,e[6]=y*n+v*u+b*f+A*g,e[7]=y*o+v*h+b*_+A*x,y=i[8],v=i[9],b=i[10],A=i[11],e[8]=y*r+v*a+b*c+A*p,e[9]=y*s+v*l+b*d+A*m,e[10]=y*n+v*u+b*f+A*g,e[11]=y*o+v*h+b*_+A*x,y=i[12],v=i[13],b=i[14],A=i[15],e[12]=y*r+v*a+b*c+A*p,e[13]=y*s+v*l+b*d+A*m,e[14]=y*n+v*u+b*f+A*g,e[15]=y*o+v*h+b*_+A*x,e},translate:function(e,t,i){var r,s,n,o,a,l,u,h,c,d,f,_,p=i[0],m=i[1],g=i[2];t===e?(e[12]=t[0]*p+t[4]*m+t[8]*g+t[12],e[13]=t[1]*p+t[5]*m+t[9]*g+t[13],e[14]=t[2]*p+t[6]*m+t[10]*g+t[14],e[15]=t[3]*p+t[7]*m+t[11]*g+t[15]):(r=t[0],s=t[1],n=t[2],o=t[3],a=t[4],l=t[5],u=t[6],h=t[7],c=t[8],d=t[9],f=t[10],_=t[11],e[0]=r,e[1]=s,e[2]=n,e[3]=o,e[4]=a,e[5]=l,e[6]=u,e[7]=h,e[8]=c,e[9]=d,e[10]=f,e[11]=_,e[12]=r*p+a*m+c*g+t[12],e[13]=s*p+l*m+d*g+t[13],e[14]=n*p+u*m+f*g+t[14],e[15]=o*p+h*m+_*g+t[15]);return e},scale:function(e,t,i){var r=i[0],s=i[1],n=i[2];return e[0]=t[0]*r,e[1]=t[1]*r,e[2]=t[2]*r,e[3]=t[3]*r,e[4]=t[4]*s,e[5]=t[5]*s,e[6]=t[6]*s,e[7]=t[7]*s,e[8]=t[8]*n,e[9]=t[9]*n,e[10]=t[10]*n,e[11]=t[11]*n,e[12]=t[12],e[13]=t[13],e[14]=t[14],e[15]=t[15],e},perspective:function(e,t,i,r,s){var n=1/Math.tan(t/2),o=1/(r-s);return e[0]=n/i,e[1]=0,e[2]=0,e[3]=0,e[4]=0,e[5]=n,e[6]=0,e[7]=0,e[8]=0,e[9]=0,e[10]=(s+r)*o,e[11]=-1,e[12]=0,e[13]=0,e[14]=2*s*r*o,e[15]=0,e},lookAt:function(e,t,i,r){var s,n,o,a,l,u,h,c,d,f,_=t[0],p=t[1],m=t[2],g=r[0],x=r[1],y=r[2],v=i[0],b=i[1],A=i[2];if(Math.abs(_-v)<1e-6&&Math.abs(p-b)<1e-6&&Math.abs(m-A)<1e-6)return Ci(e);h=_-v,c=p-b,d=m-A,f=1/Math.sqrt(h*h+c*c+d*d),s=x*(d*=f)-y*(c*=f),n=y*(h*=f)-g*d,o=g*c-x*h,(f=Math.sqrt(s*s+n*n+o*o))?(s*=f=1/f,n*=f,o*=f):(s=0,n=0,o=0);a=c*o-d*n,l=d*s-h*o,u=h*n-c*s,(f=Math.sqrt(a*a+l*l+u*u))?(a*=f=1/f,l*=f,u*=f):(a=0,l=0,u=0);return e[0]=s,e[1]=a,e[2]=h,e[3]=0,e[4]=n,e[5]=l,e[6]=c,e[7]=0,e[8]=o,e[9]=u,e[10]=d,e[11]=0,e[12]=-(s*_+n*p+o*m),e[13]=-(a*_+l*p+u*m),e[14]=-(h*_+c*p+d*m),e[15]=1,e},identity:Ci,copy:function(e,t){return e[0]=t[0],e[1]=t[1],e[2]=t[2],e[3]=t[3],e[4]=t[4],e[5]=t[5],e[6]=t[6],e[7]=t[7],e[8]=t[8],e[9]=t[9],e[10]=t[10],e[11]=t[11],e[12]=t[12],e[13]=t[13],e[14]=t[14],e[15]=t[15],e}};class $i{constructor(e,t,i){void 0===i&&(i={}),this.view=t,this.position=i.position,this.zoom=i.zoom;}static create(e,t,i){switch(i.type){case"isometric":return new Hi(e,t,i);case"flat":return new Xi(e,t,i);case"perspective":default:return new qi(e,t,i)}}update(){}setupProgram(){}updateView(){if(this.position||this.zoom){var e={};this.position&&(e={lng:this.position[0],lat:this.position[1],zoom:this.position[2]}),this.zoom&&(e.zoom=this.zoom),this.view.setView(e);}}setupMatrices(e,t){Bi.multiply(e.model_view32,this.view_matrix,e.model),t.uniform("Matrix4fv","u_modelView",e.model_view32),Vi.normalFromMat4(e.normal32,e.model_view32),Vi.invert(e.inverse_normal32,e.normal32),t.uniform("Matrix3fv","u_normalMatrix",e.normal32),t.uniform("Matrix3fv","u_inverseNormalMatrix",e.inverse_normal32);}}class qi extends $i{constructor(e,t,i){void 0===i&&(i={}),super(e,t,i),this.type="perspective",this.focal_length=i.focal_length,this.fov=i.fov,this.focal_length||this.fov||(this.focal_length=[[16,2],[17,2.5],[18,3],[19,4],[20,6]]),this.vanishing_point=i.vanishing_point||[0,0],this.vanishing_point=this.vanishing_point.map(parseFloat),this.vanishing_point_skew=[],this.position_meters=null,this.view_matrix=new Float64Array(16),this.projection_matrix=new Float32Array(16),k.replaceBlock("camera","\n            uniform mat4 u_projection;\n            uniform vec3 u_eye;\n            uniform vec2 u_vanishing_point;\n\n            void cameraProjection (inout vec4 position) {\n                position = u_projection * position;\n            }");}constrainCamera(e){let t=e.view_height,i=e.height,r=e.focal_length,s=e.fov;return i?(r=2*i/t,s=2*Math.atan(1/r)):(r?s=2*Math.atan(1/r):s&&(s=s*Math.PI/180,r=1/Math.tan(s/2)),i=t/2*r),{view_height:t,height:i,focal_length:r,fov:s}}updateMatrices(){var e=this.view.size.css.height*this.view.meters_per_pixel,t=this.constrainCamera({view_height:e,focal_length:_.interpolate(this.view.zoom,this.focal_length),fov:_.interpolate(this.view.zoom,this.fov)}),i=t.height,r=t.fov,s=[this.view.center.meters.x,this.view.center.meters.y,i];this.position_meters=s,Bi.lookAt(this.view_matrix,ji.fromValues(s[0],s[1],0),ji.fromValues(s[0],s[1],-1),ji.fromValues(0,1,0)),Bi.perspective(this.projection_matrix,r,this.view.aspect,1,2*i),this.vanishing_point_skew[0]=this.vanishing_point[0]/this.view.size.css.width,this.vanishing_point_skew[1]=this.vanishing_point[1]/this.view.size.css.height,this.projection_matrix[8]=2*-this.vanishing_point_skew[0],this.projection_matrix[9]=2*-this.vanishing_point_skew[1],Bi.translate(this.projection_matrix,this.projection_matrix,ji.fromValues(e/2*this.view.aspect*(2*-this.vanishing_point_skew[0]),e/2*(2*-this.vanishing_point_skew[1]),0)),Bi.translate(this.projection_matrix,this.projection_matrix,ji.fromValues(0,0,-i));}update(){super.update(),this.updateMatrices();}setupProgram(e){e.uniform("Matrix4fv","u_projection",this.projection_matrix),e.uniform("3f","u_eye",[0,0,this.position_meters[2]]),e.uniform("2fv","u_vanishing_point",this.vanishing_point_skew);}}class Hi extends $i{constructor(e,t,i){void 0===i&&(i={}),super(e,t,i),this.type="isometric",this.axis=i.axis||{x:0,y:1},2===this.axis.length&&(this.axis={x:this.axis[0],y:this.axis[1]}),this.position_meters=null,this.viewport_height=null,this.view_matrix=new Float64Array(16),this.projection_matrix=new Float32Array(16),k.replaceBlock("camera","\n            uniform mat4 u_projection;\n            uniform vec3 u_eye;\n            uniform vec2 u_vanishing_point;\n\n            void cameraProjection (inout vec4 position) {\n                position = u_projection * position;\n                // position.xy += position.z * u_isometric_axis;\n\n                // Reverse z for depth buffer so up is negative,\n                // and scale down values so objects higher than one screen height will not get clipped\n                // pull forward slightly to avoid going past far clipping plane\n                position.z = -position.z / 100. + 1. - 0.001;\n            }");}update(){super.update(),this.viewport_height=this.view.size.css.height*this.view.meters_per_pixel;var e=[this.view.center.meters.x,this.view.center.meters.y,this.viewport_height];this.position_meters=e,Bi.identity(this.view_matrix),Bi.translate(this.view_matrix,this.view_matrix,ji.fromValues(-e[0],-e[1],0)),Bi.identity(this.projection_matrix),this.projection_matrix[8]=this.axis.x/this.view.aspect,this.projection_matrix[9]=this.axis.y,Bi.scale(this.projection_matrix,this.projection_matrix,ji.fromValues(2/this.view.size.meters.x,2/this.view.size.meters.y,2/this.view.size.meters.y));}setupProgram(e){e.uniform("Matrix4fv","u_projection",this.projection_matrix),e.uniform("3fv","u_eye",[0,0,this.viewport_height]),e.uniform("2fv","u_vanishing_point",[0,0]);}}class Xi extends Hi{constructor(e,t,i){void 0===i&&(i={}),super(e,t,i),this.type="flat";}update(){this.axis.x=0,this.axis.y=0,super.update();}}const Wi=.5;const Zi=li.PLACEMENT,Yi=Object.create(Ee);Yi.variants={},Yi.vertex_layouts={};Object.assign(Yi,Gi),Object.assign(Yi,{name:"points",built_in:!0,vertex_shader_src:"uniform vec2 u_resolution;\nuniform float u_time;\nuniform vec3 u_map_position;\nuniform vec4 u_tile_origin;\nuniform float u_tile_proxy_order_offset;\nuniform bool u_tile_fade_in;\nuniform float u_meters_per_pixel;\nuniform float u_device_pixel_ratio;\nuniform float u_visible_time;\nuniform bool u_view_panning;\nuniform float u_view_pan_snap_timer;\n\nuniform mat4 u_model;\nuniform mat4 u_modelView;\nuniform mat3 u_normalMatrix;\nuniform mat3 u_inverseNormalMatrix;\n\nattribute vec4 a_position;\nattribute vec4 a_shape;\nattribute vec4 a_color;\nattribute vec2 a_texcoord;\nattribute vec2 a_offset;\n\nuniform float u_point_type;\n\n#ifdef TANGRAM_CURVED_LABEL\n    attribute vec4 a_offsets;\n    attribute vec4 a_pre_angles;\n    attribute vec4 a_angles;\n#endif\n\nvarying vec4 v_color;\nvarying vec2 v_texcoord;\nvarying vec4 v_world_position;\nvarying float v_alpha_factor;\n\n#ifdef TANGRAM_HAS_SHADER_POINTS\n    attribute float a_outline_edge;\n    attribute vec4 a_outline_color;\n\n    varying float v_outline_edge;\n    varying vec4 v_outline_color;\n    varying float v_aa_offset;\n#endif\n\n#ifdef TANGRAM_SHOW_HIDDEN_LABELS\n    varying float v_label_hidden;\n#endif\n\n#define TANGRAM_PI 3.14159265359\n#define TANGRAM_NORMAL vec3(0., 0., 1.)\n\n#pragma tangram: camera\n#pragma tangram: material\n#pragma tangram: lighting\n#pragma tangram: raster\n#pragma tangram: global\n\nvec2 rotate2D(vec2 _st, float _angle) {\n    return mat2(cos(_angle),-sin(_angle),\n                sin(_angle),cos(_angle)) * _st;\n}\n\n#ifdef TANGRAM_CURVED_LABEL\n    // Assumes stops are [0, 0.33, 0.66, 0.99];\n    float mix4linear(float a, float b, float c, float d, float x) {\n        x = clamp(x, 0., 1.);\n        return mix(mix(a, b, 3. * x),\n                   mix(b,\n                       mix(c, d, 3. * (max(x, .66) - .66)),\n                       3. * (clamp(x, .33, .66) - .33)),\n                   step(0.33, x)\n                );\n    }\n#endif\n\nvoid main() {\n    // Initialize globals\n    #pragma tangram: setup\n\n    // discard hidden labels by collapsing into degenerate triangle\n    #ifndef TANGRAM_SHOW_HIDDEN_LABELS\n        if (a_shape.w == 0.) {\n            gl_Position = vec4(0., 0., 0., 1.);\n            return;\n        }\n    #else\n        // highlight hidden label in fragment shader for debugging\n        if (a_shape.w == 0.) {\n            v_label_hidden = 1.; // label debug testing\n        }\n        else {\n            v_label_hidden = 0.;\n        }\n    #endif\n\n    v_alpha_factor = 1.0;\n    v_color = a_color;\n    v_texcoord = a_texcoord; // UV from vertex attribute\n\n    #ifdef TANGRAM_HAS_SHADER_POINTS\n        v_outline_color = a_outline_color;\n        v_outline_edge = a_outline_edge;\n\n        if (u_point_type == TANGRAM_POINT_TYPE_SHADER) { // shader point\n            // use point dimensions for UVs instead (ignore attribute), add antialiasing info for fragment shader\n            float size = abs(a_shape.x / 128.); // radius in pixels\n            v_texcoord = sign(a_shape.xy) * (size + 1.) / size;\n            size += 2.;\n            v_aa_offset = 2. / size;\n        }\n    #endif\n\n    // Position\n    vec4 position = u_modelView * vec4(a_position.xyz, 1.);\n\n    // Apply positioning and scaling in screen space\n    vec2 shape = a_shape.xy / 256.;                 // values have an 8-bit fraction\n    vec2 offset = vec2(a_offset.x, -a_offset.y);    // flip y to make it point down\n\n    float zoom = clamp(u_map_position.z - u_tile_origin.z, 0., 1.); //fract(u_map_position.z);\n    float theta = a_shape.z / 4096.;\n\n    #ifdef TANGRAM_CURVED_LABEL\n        //TODO: potential bug? null is passed in for non-curved labels, otherwise the first offset will be 0\n        if (a_offsets[0] != 0.){\n            vec4 angles_scaled = (TANGRAM_PI / 16384.) * a_angles;\n            vec4 pre_angles_scaled = (TANGRAM_PI / 128.) * a_pre_angles;\n            vec4 offsets_scaled = (1. / 64.) * a_offsets;\n\n            float pre_angle = mix4linear(pre_angles_scaled[0], pre_angles_scaled[1], pre_angles_scaled[2], pre_angles_scaled[3], zoom);\n            float angle = mix4linear(angles_scaled[0], angles_scaled[1], angles_scaled[2], angles_scaled[3], zoom);\n            float offset_curve = mix4linear(offsets_scaled[0], offsets_scaled[1], offsets_scaled[2], offsets_scaled[3], zoom);\n\n            shape = rotate2D(shape, pre_angle); // rotate in place\n            shape.x += offset_curve;            // offset for curved label segment\n            shape = rotate2D(shape, angle);     // rotate relative to curved label anchor\n            shape += rotate2D(offset, theta);   // offset if specified in the scene file\n        }\n        else {\n            shape = rotate2D(shape + offset, theta);\n        }\n    #else\n        shape = rotate2D(shape + offset, theta);\n    #endif\n\n    // Fade in (if requested) based on time mesh has been visible.\n    // Value passed to fragment shader in the v_alpha_factor varying\n    #ifdef TANGRAM_FADE_IN_RATE\n        if (u_tile_fade_in) {\n            v_alpha_factor *= clamp(u_visible_time * TANGRAM_FADE_IN_RATE, 0., 1.);\n        }\n    #endif\n\n    // World coordinates for 3d procedural textures\n    v_world_position = u_model * position;\n    v_world_position.xy += shape * u_meters_per_pixel;\n    v_world_position = wrapWorldPosition(v_world_position);\n\n    // Modify position before camera projection\n    #pragma tangram: position\n\n    cameraProjection(position);\n\n    #ifdef TANGRAM_LAYER_ORDER\n        // +1 is to keep all layers including proxies > 0\n        applyLayerOrder(a_position.w + u_tile_proxy_order_offset + 1., position);\n    #endif\n\n    // Apply pixel offset in screen-space\n    // Multiply by 2 is because screen is 2 units wide Normalized Device Coords (and u_resolution device pixels wide)\n    // Device pixel ratio adjustment is because shape is in logical pixels\n    position.xy += shape * position.w * 2. * u_device_pixel_ratio / u_resolution;\n    #ifdef TANGRAM_HAS_SHADER_POINTS\n        if (u_point_type == TANGRAM_POINT_TYPE_SHADER) { // shader point\n            // enlarge by 1px to catch missed MSAA fragments\n            position.xy += sign(shape) * position.w * u_device_pixel_ratio / u_resolution;\n        }\n    #endif\n\n    // Snap to pixel grid\n    // Only applied to fully upright sprites/labels (not shader-drawn points), while panning is not active\n    #ifdef TANGRAM_HAS_SHADER_POINTS\n    if (!u_view_panning && (abs(theta) < TANGRAM_EPSILON) && u_point_type != TANGRAM_POINT_TYPE_SHADER) {\n    #else\n    if (!u_view_panning && (abs(theta) < TANGRAM_EPSILON)) {\n    #endif\n        vec2 position_fract = fract((((position.xy / position.w) + 1.) * .5) * u_resolution);\n        vec2 position_snap = position.xy + ((step(0.5, position_fract) - position_fract) * position.w * 2. / u_resolution);\n\n        // Animate the snapping to smooth the transition and make it less noticeable\n        #ifdef TANGRAM_VIEW_PAN_SNAP_RATE\n            position.xy = mix(position.xy, position_snap, clamp(u_view_pan_snap_timer * TANGRAM_VIEW_PAN_SNAP_RATE, 0., 1.));\n        #else\n            position.xy = position_snap;\n        #endif\n    }\n\n    gl_Position = position;\n}\n",fragment_shader_src:"uniform vec2 u_resolution;\nuniform float u_time;\nuniform vec3 u_map_position;\nuniform vec4 u_tile_origin;\nuniform float u_meters_per_pixel;\nuniform float u_device_pixel_ratio;\nuniform float u_visible_time;\n\nuniform mat3 u_normalMatrix;\nuniform mat3 u_inverseNormalMatrix;\n\nuniform sampler2D u_texture;\nuniform float u_point_type;\nuniform bool u_apply_color_blocks;\n\nvarying vec4 v_color;\nvarying vec2 v_texcoord;\nvarying vec4 v_world_position;\nvarying float v_alpha_factor;\n\n#ifdef TANGRAM_HAS_SHADER_POINTS\n    varying vec4 v_outline_color;\n    varying float v_outline_edge;\n    varying float v_aa_offset;\n#endif\n\n#ifdef TANGRAM_SHOW_HIDDEN_LABELS\n    varying float v_label_hidden;\n#endif\n\n#define TANGRAM_NORMAL vec3(0., 0., 1.)\n\n#pragma tangram: camera\n#pragma tangram: material\n#pragma tangram: lighting\n#pragma tangram: raster\n#pragma tangram: global\n\n#ifdef TANGRAM_HAS_SHADER_POINTS\n    //l is the distance from the center to the fragment, R is the radius of the drawn point\n    float _tangram_antialias(float l, float R){\n        float low  = R - v_aa_offset;\n        float high = R + v_aa_offset;\n        return 1. - smoothstep(low, high, l);\n    }\n#endif\n\nvoid main (void) {\n    // Initialize globals\n    #pragma tangram: setup\n\n    vec4 color = v_color;\n\n    #ifdef TANGRAM_HAS_SHADER_POINTS\n        // Only apply shader blocks to point, not to attached text (N.B.: for compatibility with ES)\n        if (u_point_type == TANGRAM_POINT_TYPE_TEXTURE) { // sprite texture\n            color *= texture2D(u_texture, v_texcoord);\n        }\n        else if (u_point_type == TANGRAM_POINT_TYPE_LABEL) { // label texture\n            color = texture2D(u_texture, v_texcoord);\n            color.rgb /= max(color.a, 0.001); // un-multiply canvas texture\n        }\n        else if (u_point_type == TANGRAM_POINT_TYPE_SHADER) { // shader point\n            float outline_edge = v_outline_edge;\n            vec4 outlineColor  = v_outline_color;\n\n            // Mask of outermost circle, either outline or point boundary\n            float l = length(v_texcoord); // distance to this fragment from the point center\n            float outer_alpha = _tangram_antialias(l, 1.);\n            float fill_alpha = _tangram_antialias(l, 1. - (v_outline_edge * 0.5)) * color.a;\n            float stroke_alpha = (outer_alpha - _tangram_antialias(l, 1. - v_outline_edge)) * outlineColor.a;\n\n            // Apply alpha compositing with stroke 'over' fill.\n            #ifdef TANGRAM_BLEND_ADD\n                color.a = stroke_alpha + fill_alpha;\n                color.rgb = color.rgb * fill_alpha + outlineColor.rgb * stroke_alpha;\n            #else // TANGRAM_BLEND_OVERLAY (and fallback for not implemented blending modes)\n                color.a = stroke_alpha + fill_alpha * (1. - stroke_alpha);\n                color.rgb = mix(color.rgb * fill_alpha, outlineColor.rgb, stroke_alpha) / max(color.a, 0.001); // avoid divide by zero\n            #endif\n        }\n    #else\n        // If shader points not supported, assume label texture\n        color = texture2D(u_texture, v_texcoord);\n        color.rgb /= max(color.a, 0.001); // un-multiply canvas texture\n    #endif\n\n    // Shader blocks for color/filter are only applied for sprites, shader points, and standalone text,\n    // NOT for text attached to a point (N.B.: for compatibility with ES)\n    if (u_apply_color_blocks) {\n        #pragma tangram: color\n        #pragma tangram: filter\n    }\n\n    color.a *= v_alpha_factor;\n\n    // highlight hidden label in fragment shader for debugging\n    #ifdef TANGRAM_SHOW_HIDDEN_LABELS\n        if (v_label_hidden > 0.) {\n            color.a *= 0.5;\n            color.rgb = vec3(1., 0., 0.);\n        }\n    #endif\n\n    // Use alpha test as a lower-quality substitute\n    // For opaque and translucent: avoid transparent pixels writing to depth buffer, obscuring geometry underneath\n    // For multiply: avoid transparent pixels multiplying geometry underneath to zero/full black\n    #if defined(TANGRAM_BLEND_OPAQUE) || defined(TANGRAM_BLEND_TRANSLUCENT) || defined(TANGRAM_BLEND_MULTIPLY)\n        if (color.a < TANGRAM_ALPHA_TEST) {\n            discard;\n        }\n    #endif\n\n    gl_FragColor = color;\n}\n",selection:!0,collision:!0,blend:"overlay",init(e){void 0===e&&(e={}),Ee.init.call(this,e),this.setupDefines(),this.defines.TANGRAM_HAS_SHADER_POINTS=!0,this.defines.TANGRAM_POINT_TYPE_TEXTURE=1,this.defines.TANGRAM_POINT_TYPE_LABEL=2,this.defines.TANGRAM_POINT_TYPE_SHADER=3,this.collision_group_points=this.name+"-points",this.collision_group_text=this.name+"-text",this.stencil_proxy_tiles=!1,this.reset();},setupDefines(){"overlay"!==this.blend&&(this.defines.TANGRAM_LAYER_ORDER=!0),!0===m.suppress_label_fade_in?(this.fade_in_time=0,this.defines.TANGRAM_FADE_IN_RATE=null):(this.fade_in_time=.15,this.defines.TANGRAM_FADE_IN_RATE=1/this.fade_in_time),!0!==m.suppress_label_snap_animation&&(this.defines.TANGRAM_VIEW_PAN_SNAP_RATE=1/Wi),!0===m.show_hidden_labels&&(this.defines.TANGRAM_SHOW_HIDDEN_LABELS=!0);},reset(){this.queues={},this.resetText(),this.texture_missing_sprites={};},addFeature(e,t,i){let r=i.tile;if(r.generation!==this.generation)return;let s,n={};if(n.color=this.parseColor(t.color,i),n.texture=t.texture,n.label_texture=null,n.blend_order=t.blend_order,!n.color&&!n.texture)return;if(this.hasSprites(n)){if(!(s=this.parseSprite(n,t,i)))return void f({level:"debug",once:!0},`Layer group '${t.layers.join(", ")}' `+`uses a texture '${n.texture}', but doesn't specify which sprite to draw. `+"Features that match this layer group won't be drawn without specifying the sprite with the 'sprite' or 'sprite_default' properties. The merged draw parameters for this layer group are:",t).then(i=>{i&&f("debug",`Example feature for layer group '${t.layers.join(", ")}'`,e);});n.texcoords=s.texcoords;}else if(t.sprite)return void f({level:"warn",once:!0},`Layer group '${t.layers.join(", ")}' `+`specifies sprite '${t.sprite}', but the texture '${t.texture}' doesn't define any sprites. `+"Features that match this layer group won't be drawn. The merged draw parameters for this layer group are:",t);if(this.calcSize(t,n,s,i),t.outline&&(n.outline_width=q.evalCachedProperty(t.outline.width,i)||q.defaults.outline.width,n.outline_color=this.parseColor(t.outline.color,i)),n.outline_edge_pct=0,n.outline_width&&n.outline_color){let e=n.outline_width;n.size[0]+=e,n.size[1]+=e,n.outline_edge_pct=e/Math.min(n.size[0],n.size[1])*2;}n.size[0]=Math.min(n.size[0],256),n.size[1]=Math.min(n.size[1],256),n.placement=t.placement,n.placement_min_length_ratio=q.evalCachedProperty(t.placement_min_length_ratio,i),n.placement===Zi.SPACED&&t.placement_spacing&&(n.placement_spacing=q.evalCachedProperty(t.placement_spacing,i)),n.angle=q.evalProperty(t.angle,i)||0,n.z=q.evalCachedDistanceProperty(t.z,i)||q.defaults.z,n.tile_edges=t.tile_edges,this.computeLayout(n,e,t,i,r);let o=t.text&&!1!==t.text.visible&&this.parseTextFeature(e,t.text,i,r);Array.isArray(o)&&(o=null,f({level:"warn",once:!0},`Layer group '${t.layers.join(", ")}': `+"cannot use boundary labels (e.g. 'text_source: { left: ..., right: ... }') for 'text' labels attached to 'points'; "+`provided 'text_source' value was ${JSON.stringify(t.text.text_source)}`)),o&&(o.layout.parent=n,o.layout.priority=t.text.priority?Math.max(o.layout.priority,n.priority+.5):n.priority+.5,o.layout.move_into_tile=!1,ai.addStyle(this.collision_group_text,r.id)),this.queueFeature({feature:e,draw:t,context:i,style:n,text_feature:o},r),ai.addStyle(this.collision_group_points,r.id);},calcSize(e,t,i,r){if(t.size=e.size,t.size){if(t.size=q.evalCachedPointSizeProperty(e.size,i,w.textures[t.texture],r),null==t.size)return void f({level:"warn",once:!0},`Layer group '${e.layers.join(", ")}': `+`'size' (${JSON.stringify(e.size.value)}) couldn't be interpreted, features that match `+"this layer group won't be drawn");"number"==typeof t.size&&(t.size=[t.size,t.size]);}else t.size=i&&i.css_size||[16,16];},hasSprites:e=>e.texture&&w.textures[e.texture]&&w.textures[e.texture].sprites,getSpriteInfo(e,t){let i=w.textures[e.texture].sprites[t]&&w.getSpriteInfo(e.texture,t);return t&&!i?(this.texture_missing_sprites[e.texture]=this.texture_missing_sprites[e.texture]||{},this.texture_missing_sprites[e.texture][t]||(f("debug",`Style: in style '${this.name}', could not find sprite '${t}' for texture '${e.texture}'`),this.texture_missing_sprites[e.texture][t]=!0)):i&&(i.sprite=t),i},parseSprite(e,t,i){let r=q.evalProperty(t.sprite,i);return this.getSpriteInfo(e,r)||this.getSpriteInfo(e,t.sprite_default)},queueFeature(e,t){this.tile_data[t.id]&&this.queues[t.id]||this.startData(t),this.queues[t.id]=this.queues[t.id]||[],this.queues[t.id].push(e);},async endData(e){if(e.canceled)return f("trace",`Style ${this.name}: stop tile build because tile was canceled: ${e.key}`),null;let t=this.queues[e.id];delete this.queues[e.id];let i=[],r=[];t.forEach(e=>{let t=e.style,s=e.feature,n=s.geometry,o=this.buildLabels(t.size,n,t);for(let n=0;n<o.length;n++){let a=o[n],l={feature:s,draw:e.draw,context:e.context,style:t,label:a};if(r.push(l),e.text_feature){let t={feature:s,draw:e.text_feature.draw,context:e.context,text:e.text_feature.text,text_settings_key:e.text_feature.text_settings_key,layout:e.text_feature.layout,point_label:a,linked:l};i.push(t),e.draw.text.optional||(l.linked=t);}}});const s=(await Promise.all([ai.collide(r,this.collision_group_points,e.id).then(e=>{e.forEach(e=>{this.feature_style=e.style,this.feature_style.label=e.label,this.feature_style.linked=e.linked,Ee.addFeature.call(this,e.feature,e.draw,e.context);});}),this.collideAndRenderTextLabels(e,this.collision_group_text,i)]))[1],n=s.labels,o=s.texts,a=s.textures;n&&o&&n.forEach(e=>{let t=e.text_settings_key,i=o[t]&&o[t][e.text],r=this.feature_style;r.label=e.label,r.linked=e.linked,r.size=i.size.logical_size,r.texcoords=i.align[e.label.align].texcoords,r.label_texture=a[i.align[e.label.align].texture_id],r.blend_order=e.draw.blend_order,Ee.addFeature.call(this,e.feature,e.draw,e.context);}),this.freeText(e);const l=await Ee.endData.call(this,e);return l&&a&&a.length&&(l.textures=l.textures||[],l.textures.push(...a)),l},_preprocess(e){e.color=q.createColorPropertyCache(e.color),e.texture=void 0!==e.texture?e.texture:this.texture,e.blend_order=this.getBlendOrderForDraw(e),e.outline&&(e.outline.color=q.createColorPropertyCache(e.outline.color),e.outline.width=q.createPropertyCache(e.outline.width,q.parsePositiveNumber)),e.z=q.createPropertyCache(e.z,q.parseUnits);try{e.size=q.createPointSizePropertyCache(e.size,e.texture);}catch(t){return f({level:"warn",once:!0},`Layer group '${e.layers.join(", ")}': `+`${t} (${JSON.stringify(e.size)}), features that match this layer group won't be drawn.`),null}if(e.offset=q.createPropertyCache(e.offset,e=>Array.isArray(e)&&e.map(q.parseNumber)),e.buffer=q.createPropertyCache(e.buffer,e=>(Array.isArray(e)?e:[e,e]).map(q.parsePositiveNumber)),e.repeat_distance=q.createPropertyCache(e.repeat_distance,q.parseNumber),e.placement=Zi[e.placement&&e.placement.toUpperCase()],null==e.placement&&(e.placement=Zi.VERTEX),e.placement_spacing=null!=e.placement_spacing?e.placement_spacing:80,e.placement_spacing=q.createPropertyCache(e.placement_spacing,q.parsePositiveNumber),e.placement_min_length_ratio=null!=e.placement_min_length_ratio?e.placement_min_length_ratio:1,e.placement_min_length_ratio=q.createPropertyCache(e.placement_min_length_ratio,q.parsePositiveNumber),"number"==typeof e.angle)e.angle=e.angle*Math.PI/180;else if("function"==typeof e.angle){const t=e.angle;e.angle=(e=>t(e)*Math.PI/180);}else e.angle=e.angle||0;return e.text=this.preprocessText(e.text),e.text&&(e.text.key=e.key,e.text.group=e.group,e.text.layers=e.layers,e.text.order=e.order,e.text.blend_order=e.blend_order,e.text.repeat_group=null!=e.text.repeat_group?e.text.repeat_group:e.repeat_group,e.text.anchor=e.text.anchor||this.default_anchor,e.text.optional="boolean"==typeof e.text.optional&&e.text.optional,e.text.interactive=e.text.interactive||e.interactive),e},default_anchor:["bottom","top","right","left"],computeLayout(e,t,i,r,s){let n=e||{};n.id=t,n.units_per_pixel=s.units_per_pixel||1,n.collide=!1!==i.collide,n.anchor=i.anchor,n.offset=q.evalCachedProperty(i.offset,r)||q.zeroPair,n.buffer=q.evalCachedProperty(i.buffer,r)||q.zeroPair,n.repeat_distance=q.evalCachedProperty(i.repeat_distance,r),n.repeat_distance&&(n.repeat_distance*=n.units_per_pixel,n.repeat_scale=1,"function"==typeof i.repeat_group?n.repeat_group=i.repeat_group(r):n.repeat_group=i.repeat_group||r.layer);let o=i.priority;return null!=o?"function"==typeof o&&(o=o(r)):o=-1>>>0,n.priority=o,n},buildTextLabels(e,t){let i=[];for(let r=0;r<t.length;r++){let s=t[r],n=this.texts[e.id][s.text_settings_key][s.text].size.collision_size;s.label=new li(s.point_label.position,n,s.layout),i.push(s);}return i},buildLabels(e,t,i){let r=[];if("Point"===t.type)r.push(new li(t.coordinates,e,i,i.angle));else if("MultiPoint"===t.type){let s=t.coordinates;for(let t=0;t<s.length;++t){let n=s[t];r.push(new li(n,e,i,i.angle));}}else if("LineString"===t.type){let s=ci(t.coordinates,e,i);for(let e=0;e<s.length;++e)r.push(s[e]);}else if("MultiLineString"===t.type){let s=t.coordinates;for(let t=0;t<s.length;t++){let n=ci(s[t],e,i);for(let e=0;e<n.length;++e)r.push(n[e]);}}else if("Polygon"===t.type)if(i.placement===Zi.CENTROID){let s=F.centroid(t.coordinates);s&&r.push(new li(s,e,i,i.angle));}else{let s=t.coordinates;for(let t=0;t<s.length;t++){let n=ci(s[t],e,i);for(let e=0;e<n.length;++e)r.push(n[e]);}}else if("MultiPolygon"===t.type)if(i.placement===Zi.CENTROID){let s=F.multiCentroid(t.coordinates);s&&r.push(new li(s,e,i,i.angle));}else{let s=t.coordinates;for(let t=0;t<s.length;t++){let n=s[t];for(let t=0;t<n.length;t++){let s=ci(n[t],e,i);for(let e=0;e<s.length;++e)r.push(s[e]);}}}return r},makeVertexTemplate(e,t){let i=0;this.vertex_template[i++]=0,this.vertex_template[i++]=0,this.vertex_template[i++]=e.z||0,this.vertex_template[i++]=this.scaleOrder(e.order),this.vertex_template[i++]=0,this.vertex_template[i++]=0,this.vertex_template[i++]=0,this.vertex_template[i++]=e.label.layout.collide?0:1,t.variant.shader_point||(this.vertex_template[i++]=0,this.vertex_template[i++]=0),this.vertex_template[i++]=0,this.vertex_template[i++]=0;const r=e.color||q.defaults.color;if(this.vertex_template[i++]=255*r[0],this.vertex_template[i++]=255*r[1],this.vertex_template[i++]=255*r[2],this.vertex_template[i++]=255*r[3],t.variant.selection&&(this.vertex_template[i++]=255*e.selection_color[0],this.vertex_template[i++]=255*e.selection_color[1],this.vertex_template[i++]=255*e.selection_color[2],this.vertex_template[i++]=255*e.selection_color[3]),t.variant.shader_point){const t=e.outline_color||q.defaults.outline.color;this.vertex_template[i++]=255*t[0],this.vertex_template[i++]=255*t[1],this.vertex_template[i++]=255*t[2],this.vertex_template[i++]=255*t[3],this.vertex_template[i++]=e.outline_edge_pct||q.defaults.outline.width;}return this.vertex_template},buildQuad:(e,t,i,r,s,n,o,a,l,u,h)=>t[0]<=0||t[1]<=0?0:function(e,t,i,r,s,n,o,a,l,u,h,c){const d=s[0]*Ct,f=s[1]*Ct;Bt[0]=-d,Bt[1]=-f,Bt[2]=d,Bt[3]=f;const _=h||Oe,p=t.vertex_elements;let m=t.vertex_count;for(let s=0;s<4;s++)i[r.a_position+0]=e[0],i[r.a_position+1]=e[1],i[r.a_shape+0]=Bt[jt[s]],i[r.a_shape+1]=Bt[Vt[s]],i[r.a_shape+2]=l,i[r.a_offset+0]=n[0],i[r.a_offset+1]=n[1],r.a_texcoord&&(i[r.a_texcoord+0]=_[jt[s]]*Gt,i[r.a_texcoord+1]=_[Vt[s]]*Gt),c&&(i[r.a_pre_angles+0]=Ft*a[0],i[r.a_pre_angles+1]=Ft*a[1],i[r.a_pre_angles+2]=Ft*a[2],i[r.a_pre_angles+3]=Ft*a[3],i[r.a_angles+0]=Ut*u[0],i[r.a_angles+1]=Ut*u[1],i[r.a_angles+2]=Ut*u[2],i[r.a_angles+3]=Ut*u[3],i[r.a_offsets+0]=Dt*o[0],i[r.a_offsets+1]=Dt*o[1],i[r.a_offsets+2]=Dt*o[2],i[r.a_offsets+3]=Dt*o[3]),t.addVertex(i);return p.push(m+0),p.push(m+1),p.push(m+2),p.push(m+2),p.push(m+3),p.push(m+0),2}(e,u,h,u.vertex_layout.index,t,n,o,s,4096*i,r,a,l),build(e,t){let i=e.label;return "curved"===i.type?this.buildCurvedLabel(i,e,t):this.buildStraightLabel(i,e,t)},buildStraightLabel(e,t,i){let r,s,n=this.getTileMesh(i.tile,this.meshVariantTypeForDraw(t)),o=this.makeVertexTemplate(t,n);"point"!==e.type?(r=t.size[e.type],s=t.texcoords[e.type].texcoord):(r=t.size,s=t.texcoords),n.uniforms=n.uniforms||{},t.label_texture?(n.uniforms.u_texture=t.label_texture,n.uniforms.u_point_type=2,n.uniforms.u_apply_color_blocks=!1):t.texture?(n.uniforms.u_texture=t.texture,n.uniforms.u_point_type=1,n.uniforms.u_apply_color_blocks=!0):(n.uniforms.u_texture=w.default,n.uniforms.u_point_type=3,n.uniforms.u_apply_color_blocks=!0);let a=e.offset,l=this.buildQuad(e.position,r,e.angle,null,null,a,null,s,!1,n.vertex_data,o);const u=t.linked&&t.linked.label.id;return this.trackLabel(e,u,n,l,i),l},buildCurvedLabel(e,t,i){let r,s,n=0;for(let o=0;o<e.num_segments;o++){let a=t.size[e.type][o],l=t.texcoords_stroke[o];t.label_texture=t.label_textures[o],r=this.getTileMesh(i.tile,this.meshVariantTypeForDraw(t)),s=this.makeVertexTemplate(t,r),r.uniforms=r.uniforms||{},r.uniforms.u_texture=t.label_texture,r.uniforms.u_point_type=2,r.uniforms.u_apply_color_blocks=!1;let u=e.offset||[0,0],h=e.position,c=e.angles[o],d=e.offsets[o],f=e.pre_angles[o],_=this.buildQuad(h,a,e.angle,c,f,u,d,l,!0,r.vertex_data,s);n+=_;const p=t.linked&&t.linked.label.id;this.trackLabel(e,p,r,_,i);}for(let o=0;o<e.num_segments;o++){let a=t.size[e.type][o],l=t.texcoords[e.type][o].texcoord;t.label_texture=t.label_textures[o],r=this.getTileMesh(i.tile,this.meshVariantTypeForDraw(t)),s=this.makeVertexTemplate(t,r),r.uniforms=r.uniforms||{},r.uniforms.u_texture=t.label_texture,r.uniforms.u_point_type=2,r.uniforms.u_apply_color_blocks=!1;let u=e.offset||[0,0],h=e.position,c=e.angles[o],d=e.offsets[o],f=e.pre_angles[o],_=this.buildQuad(h,a,e.angle,c,f,u,d,l,!0,r.vertex_data,s);n+=_;const p=t.linked&&t.linked.label.id;this.trackLabel(e,p,r,_,i);}return n},trackLabel(e,t,i,r){if(e.layout.collide||e.may_repeat_across_tiles){i.labels=i.labels||{},i.labels[e.id]=i.labels[e.id]||{container:{label:e.toJSON(),linked:t},ranges:[]};const s=2*r,n=i.vertex_data.offset-i.vertex_data.stride*s;i.labels[e.id].ranges.push([n,s]);}},buildLines(e,t,i){return this.build(t,i)},buildPoints(e,t,i){return this.build(t,i)},buildPolygons(e,t,i){return this.build(t,i)},vertexLayoutForMeshVariant(e){if(null==Yi.vertex_layouts[e.shader_point]){const t=[{name:"a_position",size:4,type:Me.SHORT,normalized:!1},{name:"a_shape",size:4,type:Me.SHORT,normalized:!1},{name:"a_texcoord",size:2,type:Me.UNSIGNED_SHORT,normalized:!0,static:e.shader_point?[0,0]:null},{name:"a_offset",size:2,type:Me.SHORT,normalized:!1},{name:"a_color",size:4,type:Me.UNSIGNED_BYTE,normalized:!0},{name:"a_selection_color",size:4,type:Me.UNSIGNED_BYTE,normalized:!0,static:e.selection?null:[0,0,0,0]},{name:"a_outline_color",size:4,type:Me.UNSIGNED_BYTE,normalized:!0,static:e.shader_point?null:[0,0,0,0]},{name:"a_outline_edge",size:1,type:Me.FLOAT,normalized:!1,static:e.shader_point?null:0}];Yi.vertex_layouts[e.shader_point]=new Le(t);}return Yi.vertex_layouts[e.shader_point]},meshVariantTypeForDraw(e){const t=e.label_texture||e.texture||"__shader_point",i=t+"/"+e.blend_order;return null==Yi.variants[i]&&(Yi.variants[i]={key:i,selection:1,shader_point:"__shader_point"===t,blend_order:e.blend_order,mesh_order:e.label_texture?1:0}),Yi.variants[i]},makeMesh(e,t,i){return void 0===i&&(i={}),i=Object.assign({},i,{fade_in_time:this.fade_in_time}),Ee.makeMesh.call(this,e,t,i)}});const Ji=[0,.33,.66,.99],Ki=.1,Qi=1.3,er=.4,tr=1,ir=1.2,rr=.01;let sr=function(e,t,i,r){const s=[{type:"straight",tolerance:r.no_curving?1.8:1.5},{type:"curved"},{type:"straight",tolerance:2.3}];for(let n=0;n<s.length;n++){let o,a=s[n];if("straight"===a.type?o=new or(t,i,r,a.tolerance):"curved"===a.type&&!r.no_curving&&i.length>2&&(o=new ar(e,i,r)),o&&!o.throw_away)return o}return !1};class nr{constructor(e){this.id=si.nextLabelId(),this.layout=e,this.position=[],this.angle=0,this.offset=e.offset.slice(),this.unit_scale=this.layout.units_per_pixel,this.obbs=[],this.aabbs=[],this.type="",this.throw_away=!1;}toJSON(){return {id:this.id,type:this.type,position:this.position,size:this.size,offset:this.offset,angle:this.angle,breach:this.breach,may_repeat_across_tiles:this.may_repeat_across_tiles,layout:ni(this.layout)}}static splitLineByOrientation(e){let t=[e[0]],i=0,r=0,s=0,n=t,o=!1;for(let a=1;a<e.length;a++){let l=e[a],u=e[a-1],h=ue.length(ue.sub(l,u));l[0]>u[0]?1===s?(t.push(l),(i+=h)>r&&(n=t,r=i,o=!1)):(t=[u,l],(i=h)>r&&(n=t,r=i,o=!1),s=1):l[0]<u[0]?-1===s?(t.unshift(l),(i+=h)>r&&(n=t,r=i,o=!0)):(t=[l,u],(i=h)>r&&(n=t,r=i,o=!0),s=-1):(-1===s?t.unshift(l):(t.push(l),s=1),(i+=h)>r&&(n=t,r=i,o=-1===s));}return [n,o]}discard(e,t){if(void 0===t&&(t=null),this.throw_away)return !0;for(let i=0;i<this.obbs.length;i++){let r={aabb:this.aabbs[i],obb:this.obbs[i]};if(si.prototype.occluded.call(r,e,t))return !0}return !1}inTileBounds(){for(let e=0;e<this.aabbs.length;e++){let t={aabb:this.aabbs[e]};if(!si.prototype.inTileBounds.call(t))return !1}return !0}static createOBB(e,t,i,r,s,n,o){let a=e[0],l=e[1];return !n||0===n[0]&&0===n[1]||(a+=(n=ue.rot(n,s))[0]*o,l-=n[1]*o),new ri(a,l,-r,t,i)}}class or extends nr{constructor(e,t,i,r){super(i),this.type="straight",this.size=e,this.throw_away=!this.fit(e,t,i,r);}fit(e,t,i,r){let s,n=this.unit_scale;var o=nr.splitLineByOrientation(t);t=o[0],s=o[1],"number"==typeof i.orientation&&(this.offset[1]+=ir*(e[1]-i.vertical_buffer),s&&(this.offset[1]*=-1),-1===i.orientation&&(this.offset[1]*=-1));let a=hr(t),l=e[0]*n;for(let s=0;s<t.length-1;s++){let n,o=t[s],u=0,h=0,c=s+1;for(;c<t.length;){let d=t[c-1],f=t[c],_=ur(d,f);if(c!==s+1&&(u+=cr(_,n)),Math.abs(u)>Ki)break;if(lr(h+=a[c-1],l)<r){let t=ue.mult(ue.add(o,f),.5);return this.angle=-_,Math.abs(this.angle-Math.PI/2)<rr&&(this.angle=-Math.PI/2,"number"==typeof i.orientation&&(this.offset[1]*=-1)),this.position=t,this.updateBBoxes(this.position,e,this.angle,this.angle,this.offset),!0}n=_,c++;}}return !1}updateBBoxes(e,t,i,r,s){let n=this.unit_scale;this.obbs=[],this.aabbs=[];let o=(t[0]+2*this.layout.buffer[0])*n*si.epsilon,a=(t[1]+2*this.layout.buffer[1])*n*si.epsilon,l=nr.createOBB(e,o,a,i,r,s,n),u=l.getExtent();this.obbs.push(l),this.aabbs.push(u),this.inTileBounds&&(this.breach=!this.inTileBounds()),this.mayRepeatAcrossTiles&&(this.may_repeat_across_tiles=this.mayRepeatAcrossTiles());}}class ar extends nr{constructor(e,t,i){super(i),this.type="curved",this.angles=[],this.pre_angles=[],this.offsets=[],this.num_segments=e.length,this.sizes=e,this.throw_away=!this.fit(this.sizes,t,i);}toJSON(){return {id:this.id,type:this.type,obbs:this.obbs.map(e=>e.toJSON()),position:this.position,breach:this.breach,may_repeat_across_tiles:this.may_repeat_across_tiles,layout:ni(this.layout)}}fit(e,t,i){let r,s=this.unit_scale,n=Math.max(...e.map(e=>e[1])),o=n*s;var a=nr.splitLineByOrientation(t);t=a[0],r=a[1],"number"==typeof i.orientation&&(this.offset[1]+=ir*(n-i.vertical_buffer),r&&(this.offset[1]*=-1),-1===i.orientation&&(this.offset[1]*=-1));let l=hr(t),u=e.map(e=>e[0]*s),h=l.reduce((e,t)=>e+t,0),c=u.reduce((e,t)=>e+t,0);if(c>h)return !1;let d=t.length-1;if(d-0<2)return !1;let f=ar.curvaturePlacement(t,h,l,c,0,d),_=t[f];if(-1===f||d-f<2)return !1;this.position=_;for(var p=0;p<u.length;p++){this.offsets[p]=[],this.angles[p]=[],this.pre_angles[p]=[];for(var m=0;m<Ji.length;m++){let e=Ji[m],i=ar.scaleLine(e,t),r=i[0],n=i[1];_=r[f];let a=ar.placeAtIndex(f,r,n,u),l=a.positions,h=a.offsets,c=a.angles,d=a.pre_angles,g=h.map(e=>Math.sqrt(e[0]*e[0]+e[1]*e[1])/s);if(0===e){this.angle=1/c.length*c.reduce((e,t)=>e+t);for(let e=0;e<l.length;e++){let t=l[e],i=d[e],r=u[e],n=i+c[e],a=this.angle,h=nr.createOBB(t,r,o,n,a,this.offset,s),f=h.getExtent();this.obbs.push(h),this.aabbs.push(f);}}this.offsets[p].push(g[p]),this.angles[p].push(c[p]),this.pre_angles[p].push(d[p]);}}return !0}static curvaturePlacement(e,t,i,r,s,n){s=s||0,n=n||e.length-1;var o=[];for(let t=s+1;t<n;t++){var a=e[t-1],l=e[t],u=e[t+1],h=ue.perp(l,a),c=ue.perp(u,l),d=ue.angleBetween(h,c);d>tr&&(d=1/0),o.push(d);}o.push(1/0);var f=[],_=[],p=s,m=0;for(let e=0;e<s;e++)m+=i[e];for(;m+r<t;){for(var g=m+r,x=m,y=p,v=0;y<n&&x+i[y]<g&&(v+=o[y])!==1/0;)x+=i[y],y++;if(0===v)return p;var b=v/(y-p);f.push(v),_.push(b),m+=i[p],p++;}if(0===f.length)return -1;var A=Math.min.apply(null,f),T=_[f.indexOf(A)];return A<Qi&&T<er?f.indexOf(A):-1}static scaleLine(e,t){var i=[t[0]],r=[];return t.forEach((s,n)=>{if(n!==t.length-1){var o=ue.sub(t[n+1],t[n]),a=ue.mult(o,1+e);i.push(ue.add(i[n],a)),r.push(ue.length(a));}}),[i,r]}static placeAtIndex(e,t,i,r){let s=t[e],n=ar.getIndicesAndOffsets(e,i,r),o=n[0],a=n[1],l=ar.getPositionsFromIndicesAndOffsets(t,o,a),u=ar.getAnglesFromIndicesAndOffsets(s,o,t,l);return {positions:l,offsets:u[0],angles:u[1],pre_angles:u[2]}}static getIndicesAndOffsets(e,t,i){let r=i.length,s=[],n=[],o=0,a=0,l=0;for(;o<r;){let u=i[o];for(;o<r&&a+.5*u<=l+t[e];){let t=a-l+.5*u;n.push(t),s.push(e),a+=u,u=i[++o];}l+=t[e],e++;}return [s,n]}static getPositionsFromIndicesAndOffsets(e,t,i){let r=[];for(let s=0;s<t.length;s++){let n=t[s],o=i[s],a=ur(e[n],e[n+1]),l=ue.rot([o,0],a),u=ue.add(e[n],l);r.push(u);}return r}static getAnglesFromIndicesAndOffsets(e,t,i,r){let s=[],n=[],o=[];for(let u=0;u<r.length;u++){let h=r[u],c=t[u],d=ue.sub(h,e),f=-ue.angle(d),_=(a=i[c],l=i[c+1],-ur(a,l))-f;if(u>0){let e=s[u-1],t=n[u-1];Math.abs(f-e)>Math.PI&&(f+=f>e?-2*Math.PI:2*Math.PI),Math.abs(t-_)>Math.PI&&(_+=_>t?-2*Math.PI:2*Math.PI);}s.push(f),n.push(_),o.push(d);}var a,l;return [o,s,n]}}function lr(e,t){return t/e}function ur(e,t){let i=ue.sub(t,e);return ue.angle(i)}function hr(e){let t=[];for(let i=0;i<e.length-1;i++){let r=e[i],s=e[i+1],n=Math.hypot(r[0]-s[0],r[1]-s[1]);t.push(n);}return t}function cr(e,t){let i,r;for(e>t?(i=t,r=e):(i=e,r=t);r-i>Math.PI;)i+=2*Math.PI;return Math.abs(r-i)}let dr=Object.create(Yi);dr.vertex_layouts={},Object.assign(dr,{name:"text",super:Yi,built_in:!0,init(e){void 0===e&&(e={}),Ee.init.call(this,e),this.setupDefines(),this.defines.TANGRAM_HAS_SHADER_POINTS=!1,this.defines.TANGRAM_CURVED_LABEL=!0,this.reset();},makeVertexTemplate(e,t){this.super.makeVertexTemplate.apply(this,arguments);let i=t.vertex_data.vertex_layout.index.a_pre_angles;for(let e=0;e<12;e++)this.vertex_template[i++]=0;return this.vertex_template},reset(){this.queues={},this.resetText();},addFeature(e,t,i){let r=i.tile;if(r.generation!==this.generation)return;let s=e.geometry.type;t.can_articulate="LineString"===s||"MultiLineString"===s,t.supersample_text="LineString"===s||"MultiLineString"===s;let n=this.parseTextFeature(e,t,i,r);n&&(n instanceof Array?n.forEach(t=>{t.feature=e,t.context=i,t.layout.vertex=!1,this.queueFeature(t,r);}):(n.feature=e,n.context=i,n.layout.vertex=!1,this.queueFeature(n,r)),ai.addStyle(this.name,r.id));},async endData(e){let t=this.queues[e.id];delete this.queues[e.id];const i=await this.collideAndRenderTextLabels(e,this.name,t),r=i.labels,s=i.texts,n=i.textures;r&&s&&(this.texts[e.id]=s,r.forEach(t=>{let i=t.text_settings_key,r=this.texts[e.id][i]&&this.texts[e.id][i][t.text],s=this.feature_style;s.label=t.label,r.text_settings.can_articulate?(s.size={},s.texcoords={},"straight"===t.label.type?(s.size.straight=r.size.logical_size,s.texcoords.straight=r.texcoords.straight,s.label_texture=n[r.texcoords.straight.texture_id]):(s.size.curved=r.segment_sizes.map(function(e){return e.logical_size}),s.texcoords_stroke=r.texcoords_stroke,s.texcoords.curved=r.texcoords.curved,s.label_textures=r.texcoords.curved.map(e=>n[e.texture_id]))):(s.size=r.size.logical_size,s.texcoords=r.align[t.label.align].texcoords,s.label_texture=n[r.align[t.label.align].texture_id]),s.blend_order=t.draw.blend_order,Ee.addFeature.call(this,t.feature,t.draw,t.context);})),this.freeText(e);const o=await Ee.endData.call(this,e);if(o){n&&n.length&&o.textures.push(...n);for(let e in o.meshes)o.meshes[e].uniforms.u_apply_color_blocks=!0;}return o},_preprocess(e){return e.blend_order=this.getBlendOrderForDraw(e),this.preprocessText(e)},buildTextLabels(e,t){let i=[];for(let s=0;s<t.length;s++){let n,o=t[s],a=this.texts[e.id][o.text_settings_key][o.text];if(o.layout.vertical_buffer=a.vertical_buffer,a.text_settings.can_articulate){var r=a.segment_sizes.map(e=>e.collision_size);o.layout.no_curving=a.no_curving,n=this.buildLabels(r,o.feature.geometry,o.layout,a.size.collision_size);}else n=this.buildLabels(a.size.collision_size,o.feature.geometry,o.layout);for(let e=0;e<n.length;e++){let t=Object.create(o);t.label=n[e],i.push(t);}}return i},buildLabels(e,t,i,r){let s=[];if("LineString"===t.type)Array.prototype.push.apply(s,this.buildLineLabels(t.coordinates,e,i,r));else if("MultiLineString"===t.type){let n=t.coordinates;for(let t=0;t<n.length;++t)Array.prototype.push.apply(s,this.buildLineLabels(n[t],e,i,r));}else if("Point"===t.type)s.push(new li(t.coordinates,e,i));else if("MultiPoint"===t.type){let r=t.coordinates;for(let t=0;t<r.length;++t)s.push(new li(r[t],e,i));}else if("Polygon"===t.type){let r=F.centroid(t.coordinates);r&&s.push(new li(r,e,i));}else if("MultiPolygon"===t.type){let r=F.multiCentroid(t.coordinates);r&&s.push(new li(r,e,i));}return s},buildLineLabels(e,t,i,r){let s=[],n=Math.min(i.subdiv,e.length-1);if(n>1){let o=(e.length-1)/n;for(let a=0;a<n;a++){let n=Math.floor(a*o),l=Math.floor((a+1)*o)+1,u=e.slice(n,l),h=sr(t,r,u,i);h&&s.push(h);}}if(s.length<n){let n=sr(t,r,e,i);n&&s.push(n);}return s},vertexLayoutForMeshVariant(e){if(null==dr.vertex_layouts[e.shader_point]){const t=[{name:"a_position",size:4,type:Me.SHORT,normalized:!1},{name:"a_shape",size:4,type:Me.SHORT,normalized:!1},{name:"a_texcoord",size:2,type:Me.UNSIGNED_SHORT,normalized:!0},{name:"a_offset",size:2,type:Me.SHORT,normalized:!1},{name:"a_color",size:4,type:Me.UNSIGNED_BYTE,normalized:!0},{name:"a_selection_color",size:4,type:Me.UNSIGNED_BYTE,normalized:!0,static:e.selection?null:[0,0,0,0]},{name:"a_pre_angles",size:4,type:Me.BYTE,normalized:!1},{name:"a_angles",size:4,type:Me.SHORT,normalized:!1},{name:"a_offsets",size:4,type:Me.UNSIGNED_SHORT,normalized:!1}];dr.vertex_layouts[e.shader_point]=new Le(t);}return dr.vertex_layouts[e.shader_point]}}),dr.texture_id=0;let fr=Object.create(dt);Object.assign(fr,{name:"raster",super:dt,built_in:!0,init(){this.raster=this.raster||"color",this.super.init.apply(this,arguments),this.selection=!1;},_preprocess(e){return e.color=e.color||q.defaults.color,this.super._preprocess.apply(this,arguments)}});var _r="#ifdef TANGRAM_VERTEX_SHADER\n\n// Apply layer ordering to avoid z-fighting\nvoid applyLayerOrder (float layer, inout vec4 position) {\n    position.z -= layer * TANGRAM_LAYER_DELTA * position.w;\n}\n\n// Vertex position in model space: [0, 1] range over the local tile\n// Note positions can be outside that range due to unclipped geometry, geometry higher than a unit cube, etc.\nvec4 modelPosition() {\n    return\n        vec4(\n            a_position.xyz / TANGRAM_TILE_SCALE         // scale coords to ~0-1 range\n            * exp2(u_tile_origin.z - u_tile_origin.w),  // adjust for tile overzooming\n        1.)\n        + vec4(0., 1., 0., 0.);\n        // NB: additional offset to account for unusual Tangram JS y coords,\n        // should be refactored to remove\n}\n\n// Position in model space as above, but according to tile coordinate (as opposed to style) zoom\n// e.g. unadjusted for tile overzooming\nvec4 modelPositionBaseZoom() {\n    return\n        vec4(\n            a_position.xyz / TANGRAM_TILE_SCALE,    // scale coords to ~0-1 range\n        1.)\n        + vec4(0., 1., 0., 0.); // see note on offset above\n}\n\n#endif\n\n// Vertex position in world coordinates, useful for 3d procedural textures, etc.\nvec4 worldPosition() {\n    return v_world_position;\n}\n\n// Optionally wrap world coordinates (allows more precision at higher zooms)\n// e.g. at wrap 1000, the world space will wrap every 1000 meters\n#ifdef TANGRAM_VERTEX_SHADER\n\nvec4 wrapWorldPosition(vec4 world_position) {\n    #if defined(TANGRAM_WORLD_POSITION_WRAP)\n    vec2 anchor = u_tile_origin.xy - mod(u_tile_origin.xy, TANGRAM_WORLD_POSITION_WRAP);\n        world_position.xy -= anchor;\n    #endif\n    return world_position;\n}\n\n#endif\n\n// Normal in world space\n#if defined(TANGRAM_VERTEX_SHADER)\n\nvec3 worldNormal() {\n    return TANGRAM_NORMAL;\n}\n\n#elif defined(TANGRAM_FRAGMENT_SHADER)\n\nvec3 worldNormal() {\n    return u_inverseNormalMatrix * TANGRAM_NORMAL;\n}\n\n#endif\n",pr="// Vertex attribute + varying for feature selection\n#if defined(TANGRAM_FEATURE_SELECTION) && defined(TANGRAM_VERTEX_SHADER)\n    attribute vec4 a_selection_color;\n    varying vec4 v_selection_color;\n#endif\n",mr="// Selection pass-specific rendering\n#if defined(TANGRAM_FEATURE_SELECTION) && defined(TANGRAM_VERTEX_SHADER)\n    if (a_selection_color.rgb == vec3(0.)) {\n        // Discard by forcing invalid triangle if we're in the feature\n        // selection pass but have no selection info\n        // TODO: in some cases we may actually want non-selectable features to occlude selectable ones?\n        gl_Position = vec4(0., 0., 0., 1.);\n        return;\n    }\n    v_selection_color = a_selection_color;\n#endif\n";function gr(e){return null!=e}function xr(e){return "("+e+")"}function yr(e){return "$"===e[0]?"context['"+e.substring(1)+"']":"context.feature.properties['"+e+"']"}function vr(e,t){return xr(function(e){return "string"==typeof e?'"'+e+'"':e}(t)+" === "+yr(e))}function br(e,t){return xr(t.map(function(t){return vr(e,t)}).join(" || "))}function Ar(e,t){return xr(e.filter(gr).map(function(e){return xr(e.join(" && "))}).join(" "+t+" "))}function Tr(e,t,i){return t&&t.length>0?Ar(t.map(function(e){return Nr(e,i)}),"||"):"true"}function wr(e,t,i){return t&&t.length>0?Ar(t.map(function(e){return Nr(e,i)}),"&&"):"true"}function Er(e,t,i){return "!"+xr(Nr(t,i).join(" && "))}function Rr(e,t,i){return "!"+xr(Tr(0,t,i))}function Mr(e,t){return xr(yr(e)+(t?" != ":" == ")+"null")}function zr(e,t,i){var r=[],s=i&&"function"==typeof i.rangeTransform&&i.rangeTransform;if(t.max){var n=s?s(t.max):t.max;r.push(yr(e)+" < "+n);}if(t.min){var o=s?o=s(t.min):t.min;r.push(yr(e)+" >= "+o);}return xr(r.join(" && "))}function Nr(e,t){var i=[];if("function"==typeof e)return [xr(e.toString()+"(context)")];if(Array.isArray(e))return [Tr(0,e,t)];if(null==e)return ["true"];for(var r=Object.keys(e),s=0;s<r.length;s++){var n=r[s],o=e[n],a=typeof o;if("string"===a||"number"===a)i.push(vr(n,o));else if("boolean"===a)i.push(Mr(n,o));else if("not"===n)i.push(Er(0,o,t));else if("any"===n)i.push(Tr(0,o,t));else if("all"===n)i.push(wr(0,o,t));else if("none"===n)i.push(Rr(0,o,t));else if(Array.isArray(o))i.push(br(n,o));else if("object"===a&&null!=o)(o.max||o.min)&&i.push(zr(n,o,t));else{if(null!=o)throw new Error("Unknown Query syntax: "+o);i.push(" true ");}}return 0===r.length?["true"]:i}function Sr(e,t){return null==e?function(){return !0}:new Function("context","return "+xr(Nr(e,t).join(" && "))+";")}const Pr=["filter","draw","visible","enabled","data","exclusive","priority"];let kr={};function Lr(e,t){let i,r=0;for(let t=0;t<e.length;t++)e[t].length>r&&(r=e[t].length);if(0===r)return null;let s={visible:!0};for(let n=0;n<r;n++)i=[],e.forEach(e=>{e[n]&&e[n][t]&&-1===i.indexOf(e[n][t])&&i.push(e[n][t]);}),0!==i.length&&O(s,...i);return !1===s.visible?null:s}const Ir=["any","all","not","none"];class Or{constructor(e){let t=e.layer,i=e.name,r=e.parent,s=e.draw,n=e.visible,o=e.enabled,a=e.filter,l=e.exclusive,u=e.priority,h=e.styles;if(this.id=Or.id++,this.config_data=t.data,this.parent=r,this.name=i,this.full_name=this.parent?this.parent.full_name+":"+this.name:this.name,this.draw=s,this.filter=a,this.exclusive=!0===l,this.priority=null!=u?u:Number.MAX_SAFE_INTEGER,this.styles=h,this.is_built=!1,o=void 0===o?n:o,this.enabled=!1!==o,this.draw)for(let e in this.draw)if(this.draw[e]=null==this.draw[e]?{}:this.draw[e],"object"!=typeof this.draw[e]){let t=`Draw group '${e}' for layer ${this.full_name} is invalid, must be an object, `;f("warn",t+=`but was set to \`${e}: ${this.draw[e]}\` instead`),delete this.draw[e];}}build(){f("trace",`Building layer '${this.full_name}'`),this.buildFilter(),this.buildDraw(),this.is_built=!0;}buildDraw(){this.draw=j(this.draw,q.wrapFunction),this.calculatedDraw=function(e){let t=[];if(e.parent){let i=e.parent.calculatedDraw||[];t.push(...i);}return t.push(e.draw),t}(this);}buildFilter(){this.filter_original=this.filter,this.filter=j(this.filter,q.wrapFunction);let e=typeof this.filter;if(null==this.filter||"object"===e||"function"===e)try{this.buildZooms(),this.buildPropMatches(),null!=this.filter&&("function"==typeof this.filter||Object.keys(this.filter).length>0)?this.filter=Sr(this.filter,Dr):this.filter=null;}catch(e){let t=`Filter for layer ${this.full_name} is invalid, \`filter: ${JSON.stringify(this.filter)}\` `;f("warn",t+=`failed with error '${e.message}', stack trace: ${e.stack}`);}else{let e=`Filter for layer ${this.full_name} is invalid, filter value must be an object or function, `;f("warn",e+=`but was set to \`filter: ${this.filter}\` instead`);}}buildZooms(){let e=this.filter&&this.filter.$zoom,t=typeof e;if(null!=e){if(this.zooms={},"number"===t)this.zooms[e]=!0;else if(Array.isArray(e))for(let t=0;t<e.length;t++)this.zooms[e[t]]=!0;else if("object"===t&&(null!=e.min||null!=e.max)){let t=e.min||0,i=e.max||F.max_style_zoom;for(let e=t;e<i;e++)this.zooms[e]=!0;}delete this.filter.$zoom;}}buildPropMatches(){this.filter&&!Array.isArray(this.filter)&&"function"!=typeof this.filter&&Object.keys(this.filter).forEach(e=>{if(-1===Ir.indexOf(e)){let t=this.filter[e],i=typeof t,r=Array.isArray(t);if(!r&&"string"!==i&&"number"!==i)return;"$"===e[0]?(this.context_prop_matches=this.context_prop_matches||[],this.context_prop_matches.push([e.substring(1),r?t:[t]])):(this.feature_prop_matches=this.feature_prop_matches||[],this.feature_prop_matches.push([e,r?t:[t]])),delete this.filter[e];}});}doPropMatches(e){if(this.feature_prop_matches)for(let t=0;t<this.feature_prop_matches.length;t++){let i=this.feature_prop_matches[t],r=e.feature.properties[i[0]];if(null==r||-1===i[1].indexOf(r))return !1}if(this.context_prop_matches)for(let t=0;t<this.context_prop_matches.length;t++){let i=this.context_prop_matches[t],r=e[i[0]];if(null==r||-1===i[1].indexOf(r))return !1}return !0}doesMatch(e){if(!this.enabled)return !1;if(this.is_built||this.build(),null!=this.zooms&&!this.zooms[e.zoom])return !1;if(!this.doPropMatches(e))return !1;let t;if(this.filter instanceof Function)try{t=this.filter(e);}catch(t){let i=`Filter for this ${this.full_name}: \`filter: ${this.filter_original}\` `;f("error",i+=`failed with error '${t.message}', stack trace: ${t.stack}`,e.feature);}else t=null==this.filter;return !!t&&(this.children_to_parse&&(!function(e,t,i){for(let r in t){let s=t[r];if("object"!=typeof s||Array.isArray(s)){let t=`Layer value must be an object: cannot create layer '${r}: ${JSON.stringify(s)}'`;t+=`, under parent layer '${e.full_name}'.`,i[e.name]&&(t+=` The parent name '${e.name}' is also the name of a style, did you mean to create a 'draw' group`,e.parent&&(t+=` under '${e.parent.name}'`),t+=" instead?"),f("warn",t);}else Cr(r,s,e,i);}e.layers.sort((e,t)=>{if(e.exclusive<t.exclusive)return 1;if(e.exclusive>t.exclusive)return -1;const i=e.exclusive?1:-1;return e.priority>t.priority?i:e.priority<t.priority?-i:e.full_name<t.full_name?i:e.full_name>t.full_name?-i:void 0});}(this,this.children_to_parse,this.styles),delete this.children_to_parse),!0)}}Or.id=0;class Fr extends Or{constructor(e){super(e),this.is_leaf=!0;}}class Ur extends Or{constructor(e){super(e),this.is_tree=!0,this.layers=e.layers||[];}addLayer(e){this.layers.push(e);}buildDrawGroups(e){let t=[],i=[];if(function e(t,i,r,s){let n=!1;let o=!1;if(0===i.length)return;for(let a=0;a<i.length;a++){let l=i[a];if(l.is_leaf){if(l.doesMatch(t)&&(n=!0,r.push(l),s.push(l.id),l.exclusive))break}else if(l.is_tree&&l.doesMatch(t)&&(n=!0,(o=e(t,l.layers,r,s))||(r.push(l),s.push(l.id)),l.exclusive))break}return n}(e,[this],t,i),t.length>0){let e=function(e){if(e.length>1){for(var t=e[0],i=1;i<e.length;i++)t+="/"+e[i];return t}return e[0]}(i);if(void 0===kr[e]){let i=t.map(e=>e&&!1!==e.visible&&e.calculatedDraw),r={};for(let e=0;e<i.length;e++){let t=i[e];if(t)for(let e=0;e<t.length;e++){let i=t[e];for(let e in i)r[e]=!0;}}for(let s in r)kr[e]=kr[e]||{},kr[e][s]=Lr(i,s),kr[e][s]?(kr[e][s].key=e+"/"+s,kr[e][s].layers=t.map(e=>e&&e.full_name),kr[e][s].group=s):delete kr[e][s];kr[e]&&0===Object.keys(kr[e]).length&&(kr[e]=null);}return kr[e]}}}const Dr={rangeTransform:e=>"string"==typeof e&&"px2"===e.trim().slice(-3)?`${parseFloat(e)} * context.meters_per_pixel_sq`:e};function Gr(e){return Pr.indexOf(e)>-1}function Cr(e,t,i,r){let s={name:e,layer:t=null==t?{}:t,parent:i,styles:r},n=function(e){let t={},i={};for(let r in e)Gr(r)?t[r]=e[r]:i[r]=e[r];return [t,i]}(t),o=n[0],a=n[1],l=(u=a,0===Object.keys(u).length);var u;let h,c=new(h=l&&null!=i?Fr:Ur)(Object.assign(s,o));return c.enabled&&(i&&i.addLayer(c),c.children_to_parse=l?null:a),c}let jr=0,Vr=0;class Br{constructor(e){let t=e.coords,i=e.style_z,r=e.source,s=e.workers,n=e.view;this.id=jr++,this.view=n,this.source=r,this.generation=null,this.valid=!0,this.visible=!1,this.proxy_for=null,this.proxied_as=null,this.proxy_level=0,this.proxy_order_offset=0,this.fade_in=!0,this.loading=!1,this.loaded=!1,this.built=!1,this.labeled=!1,this.error=null,this.debug={},this.style_z=i,this.coords=be.normalizedCoord(t,this.source),this.key=be.key(this.coords,this.source,this.style_z),this.overzoom=Math.max(this.style_z-this.coords.z,0),this.overzoom2=Math.pow(2,this.overzoom),this.min=F.metersForTile(this.coords),this.max=F.metersForTile({x:this.coords.x+1,y:this.coords.y+1,z:this.coords.z}),this.span={x:this.max.x-this.min.x,y:-(this.max.y-this.min.y)},this.bounds={sw:{x:this.min.x,y:this.max.y},ne:{x:this.max.x,y:this.min.y}},this.meters_per_pixel=F.metersPerPixel(this.style_z),this.meters_per_pixel_sq=this.meters_per_pixel*this.meters_per_pixel,this.units_per_pixel=F.units_per_pixel/this.overzoom2,this.units_per_meter_overzoom=F.unitsPerMeter(this.coords.z)*this.overzoom2,this.preserve_tiles_within_zoom=this.source.preserve_tiles_within_zoom,this.meshes={},this.new_mesh_styles=[],this.pending_label_meshes=null,this.setWorker(s);}freeResources(){for(let e in this.meshes)this.meshes[e].forEach(e=>e.destroy());if(this.meshes={},this.pending_label_meshes)for(let e in this.pending_label_meshes)this.pending_label_meshes[e].forEach(e=>e.destroy());this.pending_label_meshes=null;}destroy(){b.removeForTile(this.id),this.workerMessage("self.removeTile",this.key),this.freeResources(),this.worker=null,this.valid=!1;}buildAsMessage(){return {id:this.id,key:this.key,source:this.source.name,coords:this.coords,min:this.min,max:this.max,units_per_pixel:this.units_per_pixel,meters_per_pixel:this.meters_per_pixel,meters_per_pixel_sq:this.meters_per_pixel_sq,units_per_meter_overzoom:this.units_per_meter_overzoom,style_z:this.style_z,overzoom:this.overzoom,overzoom2:this.overzoom2,generation:this.generation,debug:this.debug}}setWorker(e){this.source.tiled?this.worker_id=Math.abs(this.coords.x+this.coords.y+this.coords.z)%e.length:this.worker_id=this.source.id%e.length,this.worker=e[this.worker_id];}workerMessage(){for(var e=arguments.length,t=new Array(e),i=0;i<e;i++)t[i]=arguments[i];return n.postMessage(this.worker,...t)}build(e,t){let i=(void 0===t?{}:t).fade_in,r=void 0===i||i;return this.generation=e,this.fade_in=r,this.loaded||(this.loading=!0,this.built=!1,this.labeled=!1),this.workerMessage("self.buildTile",{tile:this.buildAsMessage()}).catch(e=>{throw e})}static cancel(e){e&&(e.canceled=!0,e.source_data&&e.source_data.request_id&&(_.cancelRequest(e.source_data.request_id),e.source_data.request_id=null),Br.abortBuild(e));}static buildGeometry(e,t){let i=t.scene_id,r=t.layers,s=t.styles,n=t.global,o=e.source_data;e.debug.building=+new Date,e.debug.feature_count=0,e.debug.layers=null,ai.startTile(e.id,{apply_repeat_groups:!0});for(let t in r){let i=r[t];if(!i||!i.config_data){f("warn",`Layer ${t} was defined without a geometry data source and will not be rendered.`);continue}if(i.config_data.source!==e.source)continue;let a=Br.getDataForSource(o,i.config_data,t);for(let r=0;r<a.length;r++){let o=a[r],l=o.geom;if(l)for(let r=0;r<l.features.length;r++){let a=l.features[r];if(null==a.geometry)continue;let u=q.getFeatureParseContext(a,e,n);u.winding=e.default_winding,u.source=e.source,u.layer=o.layer;let h=i.buildDrawGroups(u,!0);if(h){for(let e in h){let i=h[e],r=i.style||e,n=s[r];n?null!=(i=n.preprocess(i))&&!1!==i.visible&&(u.layers=i.layers,n.addFeature(a,i,u)):f("warn",`Style '${r}' not found, skipping layer '${t}':`,i,a);}e.debug.feature_count++;}}}}e.debug.building=+new Date-e.debug.building;let a=this.stylesForTile(e,s).map(e=>s[e]);Br.buildStyleGroups(e,a,i,e=>e.collision?"collision":"non-collision");}static stylesForTile(e,t){let i=[];for(let r in t)t[r].hasDataForTile(e)&&i.push(r);return i}static buildStyleGroups(e,t,i,r){const s=t.reduce((e,t)=>{const i=r(t);return e[i]=e[i]||[],e[i].push(t),e},{});if(0===Object.keys(s).length)return n.postMessage(`TileManager_${i}.buildTileStylesCompleted`,n.withTransferables({tile:Br.slice(e),progress:{start:!0,done:!0}})),void ai.resetTile(e.id);const o={};for(const t in s)Br.buildStyleGroup({group_name:t,groups:s,tile:e,progress:o,scene_id:i});}static async buildStyleGroup(e){let t=e.group_name,r=e.groups,s=e.tile,o=e.progress,a=e.scene_id;const l=r[t],u={};try{await Promise.all(l.map(async e=>{const t=await e.endData(s);t&&(u[e.name]=t);})),f("trace",`Finished style group '${t}' for tile ${s.key}`),r[t]=null,Object.keys(r).every(e=>null==r[e])&&(o.done=!0),n.postMessage(`TileManager_${a}.buildTileStylesCompleted`,n.withTransferables({tile:i({},Br.slice(s),{mesh_data:u}),progress:o})),o.done&&ai.resetTile(s.id);}catch(e){f("error",`Error for style group '${t}' for tile ${s.key}`,e&&e.stack||e);}}static getDataForSource(e,t,i){var r=[];return null!=t&&null!=e&&null!=e.layers&&(!t.layer&&e.layers._default?r.push({geom:e.layers._default}):!t.layer&&i?r.push({layer:i,geom:e.layers[i]}):"string"==typeof t.layer?r.push({layer:t.layer,geom:e.layers[t.layer]}):Array.isArray(t.layer)&&t.layer.forEach(t=>{e.layers[t]&&e.layers[t].features&&r.push({layer:t,geom:e.layers[t]});})),r}buildMeshes(e,t){if(this.error)return;this.build_id=Vr++,t.start&&(this.debug.geometry_count=0,this.debug.buffer_size=0);let i={},r=this.mesh_data;if(r)for(let t in r){for(let s in r[t].meshes){let n=r[t].meshes[s];if(n.vertex_data){if(!e[t]){f("warn",`Could not create mesh because style '${t}' not found, for tile ${this.key}, aborting tile`);break}let s=Object.assign({},r[t]);s.uniforms=Object.assign({},s.uniforms,n.uniforms),s.variant=n.variant,n.labels&&(s.retain=!0);let o=e[t].makeMesh(n.vertex_data,n.vertex_elements,s);o.variant=s.variant,o.labels=n.labels,i[t]=i[t]||[],i[t].push(o),null==o.variant.mesh_order&&(o.variant.mesh_order=i[t].length-1),this.debug.buffer_size+=o.buffer_size,this.debug.geometry_count+=o.geometry_count;}}i[t]&&i[t].sort((e,t)=>{let i=e.variant.mesh_order,r=t.variant.mesh_order;return null==i?1:null==r?-1:i<r?-1:1});}delete this.mesh_data;for(let t in i)e[t].collision?(this.pending_label_meshes=this.pending_label_meshes||{},this.pending_label_meshes[t]=i[t]):(this.meshes[t]&&this.meshes[t].forEach(e=>e.destroy()),this.meshes[t]=i[t],this.new_mesh_styles.push(t));if(t.done){for(let e in this.meshes)-1!==this.new_mesh_styles.indexOf(e)||this.pending_label_meshes&&null!=this.pending_label_meshes[e]||(this.meshes[e].forEach(e=>e.destroy()),delete this.meshes[e]);this.new_mesh_styles=[],this.debug.geometry_ratio=(this.debug.geometry_count/this.debug.feature_count).toFixed(1);}this.printDebug(t);}pendingLabelStyleCount(){return this.pending_label_meshes?Object.keys(this.pending_label_meshes).length:0}swapPendingLabels(){if(this.labeled=!0,this.pending_label_meshes){for(let e in this.pending_label_meshes)this.meshes[e]&&this.meshes[e].forEach(e=>e.destroy()),this.meshes[e]=this.pending_label_meshes[e];this.pending_label_meshes=null;}}static abortBuild(e){if(b.removeForTile(e.id),ai.abortTile(e.id),e.mesh_data)for(let t in e.mesh_data){let i=e.mesh_data[t].textures;i&&i.forEach(t=>{let i=w.textures[t];i&&(f("trace",`releasing texture ${t} for tile ${e.key}`),i.release());});}}setProxyFor(e){e?(this.visible=!0,this.proxy_for=this.proxy_for||[],this.proxy_for.push(e),this.proxy_order_offset=1,e.proxied_as=e.style_z>this.style_z?"child":"parent",this.proxy_level=Math.abs(e.style_z-this.style_z)):(this.proxy_for=null,this.proxy_order_offset=0,this.proxy_level=0);}isProxy(){return null!=this.proxy_for}shouldProxyForStyle(e){return !this.proxy_for||this.proxy_for.some(t=>null==t.meshes[e])}setupProgram(e,t){let i=e.model,r=e.model32;t.uniform("4fv","u_tile_origin",[this.min.x,this.min.y,this.style_z,this.coords.z]),t.uniform("1f","u_tile_proxy_order_offset",this.proxy_order_offset),Bi.identity(i),Bi.translate(i,i,ji.fromValues(this.min.x,this.min.y,0)),Bi.scale(i,i,ji.fromValues(this.span.x/F.tile_scale,this.span.y/F.tile_scale,1)),Bi.copy(r,i),t.uniform("Matrix4fv","u_model",r),t.uniform("1i","u_tile_fade_in",this.fade_in&&"child"!==this.proxied_as);}static slice(e,t){let i=["id","key","loading","loaded","generation","error","debug"];Array.isArray(t)&&i.push(...t);var r={};for(let t=0;t<i.length;t++){const s=i[t];r[s]=e[s];}return r}merge(e){return this.loading=e.loading,this.loaded=e.loaded,this.generation=e.generation,this.error=e.error,this.mesh_data=e.mesh_data,this.debug=O(this.debug,e.debug),this}printDebug(e){const t=["layers"];let i={};for(let e in this.debug)-1===t.indexOf(e)&&(i[e]=this.debug[e]);f("debug",`Tile ${e.done?"(done)":""}: debug for ${this.key}: [  ${JSON.stringify(i)} ]`);}}Br.coord_children={};var $r=function(e,t,i,r,s){var n,o,a=8*s-r-1,l=(1<<a)-1,u=l>>1,h=-7,c=i?s-1:0,d=i?-1:1,f=e[t+c];for(c+=d,n=f&(1<<-h)-1,f>>=-h,h+=a;h>0;n=256*n+e[t+c],c+=d,h-=8);for(o=n&(1<<-h)-1,n>>=-h,h+=r;h>0;o=256*o+e[t+c],c+=d,h-=8);if(0===n)n=1-u;else{if(n===l)return o?NaN:1/0*(f?-1:1);o+=Math.pow(2,r),n-=u;}return (f?-1:1)*o*Math.pow(2,n-r)},qr=function(e,t,i,r,s,n){var o,a,l,u=8*n-s-1,h=(1<<u)-1,c=h>>1,d=23===s?Math.pow(2,-24)-Math.pow(2,-77):0,f=r?0:n-1,_=r?1:-1,p=t<0||0===t&&1/t<0?1:0;for(t=Math.abs(t),isNaN(t)||t===1/0?(a=isNaN(t)?1:0,o=h):(o=Math.floor(Math.log(t)/Math.LN2),t*(l=Math.pow(2,-o))<1&&(o--,l*=2),(t+=o+c>=1?d/l:d*Math.pow(2,1-c))*l>=2&&(o++,l/=2),o+c>=h?(a=0,o=h):o+c>=1?(a=(t*l-1)*Math.pow(2,s),o+=c):(a=t*Math.pow(2,c-1)*Math.pow(2,s),o=0));s>=8;e[i+f]=255&a,f+=_,a/=256,s-=8);for(o=o<<s|a,u+=s;u>0;e[i+f]=255&o,f+=_,o/=256,u-=8);e[i+f-_]|=128*p;},Hr=Xr;function Xr(e){this.buf=ArrayBuffer.isView&&ArrayBuffer.isView(e)?e:new Uint8Array(e||0),this.pos=0,this.type=0,this.length=this.buf.length;}Xr.Varint=0,Xr.Fixed64=1,Xr.Bytes=2,Xr.Fixed32=5;function Wr(e){return e.type===Xr.Bytes?e.readVarint()+e.pos:e.pos+1}function Zr(e,t,i){return i?4294967296*t+(e>>>0):4294967296*(t>>>0)+(e>>>0)}function Yr(e,t,i){var r=t<=16383?1:t<=2097151?2:t<=268435455?3:Math.ceil(Math.log(t)/(7*Math.LN2));i.realloc(r);for(var s=i.pos-1;s>=e;s--)i.buf[s+r]=i.buf[s];}function Jr(e,t){for(var i=0;i<e.length;i++)t.writeVarint(e[i]);}function Kr(e,t){for(var i=0;i<e.length;i++)t.writeSVarint(e[i]);}function Qr(e,t){for(var i=0;i<e.length;i++)t.writeFloat(e[i]);}function es(e,t){for(var i=0;i<e.length;i++)t.writeDouble(e[i]);}function ts(e,t){for(var i=0;i<e.length;i++)t.writeBoolean(e[i]);}function is(e,t){for(var i=0;i<e.length;i++)t.writeFixed32(e[i]);}function rs(e,t){for(var i=0;i<e.length;i++)t.writeSFixed32(e[i]);}function ss(e,t){for(var i=0;i<e.length;i++)t.writeFixed64(e[i]);}function ns(e,t){for(var i=0;i<e.length;i++)t.writeSFixed64(e[i]);}function os(e,t){return (e[t]|e[t+1]<<8|e[t+2]<<16)+16777216*e[t+3]}function as(e,t,i){e[i]=t,e[i+1]=t>>>8,e[i+2]=t>>>16,e[i+3]=t>>>24;}function ls(e,t){return (e[t]|e[t+1]<<8|e[t+2]<<16)+(e[t+3]<<24)}Xr.prototype={destroy:function(){this.buf=null;},readFields:function(e,t,i){for(i=i||this.length;this.pos<i;){var r=this.readVarint(),s=r>>3,n=this.pos;this.type=7&r,e(s,t,this),this.pos===n&&this.skip(r);}return t},readMessage:function(e,t){return this.readFields(e,t,this.readVarint()+this.pos)},readFixed32:function(){var e=os(this.buf,this.pos);return this.pos+=4,e},readSFixed32:function(){var e=ls(this.buf,this.pos);return this.pos+=4,e},readFixed64:function(){var e=os(this.buf,this.pos)+4294967296*os(this.buf,this.pos+4);return this.pos+=8,e},readSFixed64:function(){var e=os(this.buf,this.pos)+4294967296*ls(this.buf,this.pos+4);return this.pos+=8,e},readFloat:function(){var e=$r(this.buf,this.pos,!0,23,4);return this.pos+=4,e},readDouble:function(){var e=$r(this.buf,this.pos,!0,52,8);return this.pos+=8,e},readVarint:function(e){var t,i,r=this.buf;return t=127&(i=r[this.pos++]),i<128?t:(t|=(127&(i=r[this.pos++]))<<7,i<128?t:(t|=(127&(i=r[this.pos++]))<<14,i<128?t:(t|=(127&(i=r[this.pos++]))<<21,i<128?t:function(e,t,i){var r,s,n=i.buf;if(s=n[i.pos++],r=(112&s)>>4,s<128)return Zr(e,r,t);if(s=n[i.pos++],r|=(127&s)<<3,s<128)return Zr(e,r,t);if(s=n[i.pos++],r|=(127&s)<<10,s<128)return Zr(e,r,t);if(s=n[i.pos++],r|=(127&s)<<17,s<128)return Zr(e,r,t);if(s=n[i.pos++],r|=(127&s)<<24,s<128)return Zr(e,r,t);if(s=n[i.pos++],r|=(1&s)<<31,s<128)return Zr(e,r,t);throw new Error("Expected varint not more than 10 bytes")}(t|=(15&(i=r[this.pos]))<<28,e,this))))},readVarint64:function(){return this.readVarint(!0)},readSVarint:function(){var e=this.readVarint();return e%2==1?(e+1)/-2:e/2},readBoolean:function(){return Boolean(this.readVarint())},readString:function(){var e=this.readVarint()+this.pos,t=function(e,t,i){var r="",s=t;for(;s<i;){var n,o,a,l=e[s],u=null,h=l>239?4:l>223?3:l>191?2:1;if(s+h>i)break;1===h?l<128&&(u=l):2===h?128==(192&(n=e[s+1]))&&(u=(31&l)<<6|63&n)<=127&&(u=null):3===h?(n=e[s+1],o=e[s+2],128==(192&n)&&128==(192&o)&&((u=(15&l)<<12|(63&n)<<6|63&o)<=2047||u>=55296&&u<=57343)&&(u=null)):4===h&&(n=e[s+1],o=e[s+2],a=e[s+3],128==(192&n)&&128==(192&o)&&128==(192&a)&&((u=(15&l)<<18|(63&n)<<12|(63&o)<<6|63&a)<=65535||u>=1114112)&&(u=null)),null===u?(u=65533,h=1):u>65535&&(u-=65536,r+=String.fromCharCode(u>>>10&1023|55296),u=56320|1023&u),r+=String.fromCharCode(u),s+=h;}return r}(this.buf,this.pos,e);return this.pos=e,t},readBytes:function(){var e=this.readVarint()+this.pos,t=this.buf.subarray(this.pos,e);return this.pos=e,t},readPackedVarint:function(e,t){var i=Wr(this);for(e=e||[];this.pos<i;)e.push(this.readVarint(t));return e},readPackedSVarint:function(e){var t=Wr(this);for(e=e||[];this.pos<t;)e.push(this.readSVarint());return e},readPackedBoolean:function(e){var t=Wr(this);for(e=e||[];this.pos<t;)e.push(this.readBoolean());return e},readPackedFloat:function(e){var t=Wr(this);for(e=e||[];this.pos<t;)e.push(this.readFloat());return e},readPackedDouble:function(e){var t=Wr(this);for(e=e||[];this.pos<t;)e.push(this.readDouble());return e},readPackedFixed32:function(e){var t=Wr(this);for(e=e||[];this.pos<t;)e.push(this.readFixed32());return e},readPackedSFixed32:function(e){var t=Wr(this);for(e=e||[];this.pos<t;)e.push(this.readSFixed32());return e},readPackedFixed64:function(e){var t=Wr(this);for(e=e||[];this.pos<t;)e.push(this.readFixed64());return e},readPackedSFixed64:function(e){var t=Wr(this);for(e=e||[];this.pos<t;)e.push(this.readSFixed64());return e},skip:function(e){var t=7&e;if(t===Xr.Varint)for(;this.buf[this.pos++]>127;);else if(t===Xr.Bytes)this.pos=this.readVarint()+this.pos;else if(t===Xr.Fixed32)this.pos+=4;else{if(t!==Xr.Fixed64)throw new Error("Unimplemented type: "+t);this.pos+=8;}},writeTag:function(e,t){this.writeVarint(e<<3|t);},realloc:function(e){for(var t=this.length||16;t<this.pos+e;)t*=2;if(t!==this.length){var i=new Uint8Array(t);i.set(this.buf),this.buf=i,this.length=t;}},finish:function(){return this.length=this.pos,this.pos=0,this.buf.subarray(0,this.length)},writeFixed32:function(e){this.realloc(4),as(this.buf,e,this.pos),this.pos+=4;},writeSFixed32:function(e){this.realloc(4),as(this.buf,e,this.pos),this.pos+=4;},writeFixed64:function(e){this.realloc(8),as(this.buf,-1&e,this.pos),as(this.buf,Math.floor(e*(1/4294967296)),this.pos+4),this.pos+=8;},writeSFixed64:function(e){this.realloc(8),as(this.buf,-1&e,this.pos),as(this.buf,Math.floor(e*(1/4294967296)),this.pos+4),this.pos+=8;},writeVarint:function(e){(e=+e||0)>268435455||e<0?function(e,t){var i,r;e>=0?(i=e%4294967296|0,r=e/4294967296|0):(r=~(-e/4294967296),4294967295^(i=~(-e%4294967296))?i=i+1|0:(i=0,r=r+1|0));if(e>=0x10000000000000000||e<-0x10000000000000000)throw new Error("Given varint doesn't fit into 10 bytes");t.realloc(10),function(e,t,i){i.buf[i.pos++]=127&e|128,e>>>=7,i.buf[i.pos++]=127&e|128,e>>>=7,i.buf[i.pos++]=127&e|128,e>>>=7,i.buf[i.pos++]=127&e|128,e>>>=7,i.buf[i.pos]=127&e;}(i,0,t),function(e,t){var i=(7&e)<<4;if(t.buf[t.pos++]|=i|((e>>>=3)?128:0),!e)return;if(t.buf[t.pos++]=127&e|((e>>>=7)?128:0),!e)return;if(t.buf[t.pos++]=127&e|((e>>>=7)?128:0),!e)return;if(t.buf[t.pos++]=127&e|((e>>>=7)?128:0),!e)return;if(t.buf[t.pos++]=127&e|((e>>>=7)?128:0),!e)return;t.buf[t.pos++]=127&e;}(r,t);}(e,this):(this.realloc(4),this.buf[this.pos++]=127&e|(e>127?128:0),e<=127||(this.buf[this.pos++]=127&(e>>>=7)|(e>127?128:0),e<=127||(this.buf[this.pos++]=127&(e>>>=7)|(e>127?128:0),e<=127||(this.buf[this.pos++]=e>>>7&127))));},writeSVarint:function(e){this.writeVarint(e<0?2*-e-1:2*e);},writeBoolean:function(e){this.writeVarint(Boolean(e));},writeString:function(e){e=String(e),this.realloc(4*e.length),this.pos++;var t=this.pos;this.pos=function(e,t,i){for(var r,s,n=0;n<t.length;n++){if((r=t.charCodeAt(n))>55295&&r<57344){if(!s){r>56319||n+1===t.length?(e[i++]=239,e[i++]=191,e[i++]=189):s=r;continue}if(r<56320){e[i++]=239,e[i++]=191,e[i++]=189,s=r;continue}r=s-55296<<10|r-56320|65536,s=null;}else s&&(e[i++]=239,e[i++]=191,e[i++]=189,s=null);r<128?e[i++]=r:(r<2048?e[i++]=r>>6|192:(r<65536?e[i++]=r>>12|224:(e[i++]=r>>18|240,e[i++]=r>>12&63|128),e[i++]=r>>6&63|128),e[i++]=63&r|128);}return i}(this.buf,e,this.pos);var i=this.pos-t;i>=128&&Yr(t,i,this),this.pos=t-1,this.writeVarint(i),this.pos+=i;},writeFloat:function(e){this.realloc(4),qr(this.buf,e,this.pos,!0,23,4),this.pos+=4;},writeDouble:function(e){this.realloc(8),qr(this.buf,e,this.pos,!0,52,8),this.pos+=8;},writeBytes:function(e){var t=e.length;this.writeVarint(t),this.realloc(t);for(var i=0;i<t;i++)this.buf[this.pos++]=e[i];},writeRawMessage:function(e,t){this.pos++;var i=this.pos;e(t,this);var r=this.pos-i;r>=128&&Yr(i,r,this),this.pos=i-1,this.writeVarint(r),this.pos+=r;},writeMessage:function(e,t,i){this.writeTag(e,Xr.Bytes),this.writeRawMessage(t,i);},writePackedVarint:function(e,t){this.writeMessage(e,Jr,t);},writePackedSVarint:function(e,t){this.writeMessage(e,Kr,t);},writePackedBoolean:function(e,t){this.writeMessage(e,ts,t);},writePackedFloat:function(e,t){this.writeMessage(e,Qr,t);},writePackedDouble:function(e,t){this.writeMessage(e,es,t);},writePackedFixed32:function(e,t){this.writeMessage(e,is,t);},writePackedSFixed32:function(e,t){this.writeMessage(e,rs,t);},writePackedFixed64:function(e,t){this.writeMessage(e,ss,t);},writePackedSFixed64:function(e,t){this.writeMessage(e,ns,t);},writeBytesField:function(e,t){this.writeTag(e,Xr.Bytes),this.writeBytes(t);},writeFixed32Field:function(e,t){this.writeTag(e,Xr.Fixed32),this.writeFixed32(t);},writeSFixed32Field:function(e,t){this.writeTag(e,Xr.Fixed32),this.writeSFixed32(t);},writeFixed64Field:function(e,t){this.writeTag(e,Xr.Fixed64),this.writeFixed64(t);},writeSFixed64Field:function(e,t){this.writeTag(e,Xr.Fixed64),this.writeSFixed64(t);},writeVarintField:function(e,t){this.writeTag(e,Xr.Varint),this.writeVarint(t);},writeSVarintField:function(e,t){this.writeTag(e,Xr.Varint),this.writeSVarint(t);},writeStringField:function(e,t){this.writeTag(e,Xr.Bytes),this.writeString(t);},writeFloatField:function(e,t){this.writeTag(e,Xr.Fixed32),this.writeFloat(t);},writeDoubleField:function(e,t){this.writeTag(e,Xr.Fixed64),this.writeDouble(t);},writeBooleanField:function(e,t){this.writeVarintField(e,Boolean(t));}};var us=hs;function hs(e,t){this.x=e,this.y=t;}hs.prototype={clone:function(){return new hs(this.x,this.y)},add:function(e){return this.clone()._add(e)},sub:function(e){return this.clone()._sub(e)},multByPoint:function(e){return this.clone()._multByPoint(e)},divByPoint:function(e){return this.clone()._divByPoint(e)},mult:function(e){return this.clone()._mult(e)},div:function(e){return this.clone()._div(e)},rotate:function(e){return this.clone()._rotate(e)},rotateAround:function(e,t){return this.clone()._rotateAround(e,t)},matMult:function(e){return this.clone()._matMult(e)},unit:function(){return this.clone()._unit()},perp:function(){return this.clone()._perp()},round:function(){return this.clone()._round()},mag:function(){return Math.sqrt(this.x*this.x+this.y*this.y)},equals:function(e){return this.x===e.x&&this.y===e.y},dist:function(e){return Math.sqrt(this.distSqr(e))},distSqr:function(e){var t=e.x-this.x,i=e.y-this.y;return t*t+i*i},angle:function(){return Math.atan2(this.y,this.x)},angleTo:function(e){return Math.atan2(this.y-e.y,this.x-e.x)},angleWith:function(e){return this.angleWithSep(e.x,e.y)},angleWithSep:function(e,t){return Math.atan2(this.x*t-this.y*e,this.x*e+this.y*t)},_matMult:function(e){var t=e[0]*this.x+e[1]*this.y,i=e[2]*this.x+e[3]*this.y;return this.x=t,this.y=i,this},_add:function(e){return this.x+=e.x,this.y+=e.y,this},_sub:function(e){return this.x-=e.x,this.y-=e.y,this},_mult:function(e){return this.x*=e,this.y*=e,this},_div:function(e){return this.x/=e,this.y/=e,this},_multByPoint:function(e){return this.x*=e.x,this.y*=e.y,this},_divByPoint:function(e){return this.x/=e.x,this.y/=e.y,this},_unit:function(){return this._div(this.mag()),this},_perp:function(){var e=this.y;return this.y=this.x,this.x=-e,this},_rotate:function(e){var t=Math.cos(e),i=Math.sin(e),r=t*this.x-i*this.y,s=i*this.x+t*this.y;return this.x=r,this.y=s,this},_rotateAround:function(e,t){var i=Math.cos(e),r=Math.sin(e),s=t.x+i*(this.x-t.x)-r*(this.y-t.y),n=t.y+r*(this.x-t.x)+i*(this.y-t.y);return this.x=s,this.y=n,this},_round:function(){return this.x=Math.round(this.x),this.y=Math.round(this.y),this}},hs.convert=function(e){return e instanceof hs?e:Array.isArray(e)?new hs(e[0],e[1]):e};var cs=ds;function ds(e,t,i,r,s){this.properties={},this.extent=i,this.type=0,this._pbf=e,this._geometry=-1,this._keys=r,this._values=s,e.readFields(fs,this,t);}function fs(e,t,i){1==e?t.id=i.readVarint():2==e?function(e,t){var i=e.readVarint()+e.pos;for(;e.pos<i;){var r=t._keys[e.readVarint()],s=t._values[e.readVarint()];t.properties[r]=s;}}(i,t):3==e?t.type=i.readVarint():4==e&&(t._geometry=i.pos);}function _s(e){for(var t,i,r=0,s=0,n=e.length,o=n-1;s<n;o=s++)t=e[s],r+=((i=e[o]).x-t.x)*(t.y+i.y);return r}ds.types=["Unknown","Point","LineString","Polygon"],ds.prototype.loadGeometry=function(){var e=this._pbf;e.pos=this._geometry;for(var t,i=e.readVarint()+e.pos,r=1,s=0,n=0,o=0,a=[];e.pos<i;){if(!s){var l=e.readVarint();r=7&l,s=l>>3;}if(s--,1===r||2===r)n+=e.readSVarint(),o+=e.readSVarint(),1===r&&(t&&a.push(t),t=[]),t.push(new us(n,o));else{if(7!==r)throw new Error("unknown command "+r);t&&t.push(t[0].clone());}}return t&&a.push(t),a},ds.prototype.bbox=function(){var e=this._pbf;e.pos=this._geometry;for(var t=e.readVarint()+e.pos,i=1,r=0,s=0,n=0,o=1/0,a=-1/0,l=1/0,u=-1/0;e.pos<t;){if(!r){var h=e.readVarint();i=7&h,r=h>>3;}if(r--,1===i||2===i)(s+=e.readSVarint())<o&&(o=s),s>a&&(a=s),(n+=e.readSVarint())<l&&(l=n),n>u&&(u=n);else if(7!==i)throw new Error("unknown command "+i)}return [o,l,a,u]},ds.prototype.toGeoJSON=function(e,t,i){var r,s,n=this.extent*Math.pow(2,i),o=this.extent*e,a=this.extent*t,l=this.loadGeometry(),u=ds.types[this.type];function h(e){for(var t=0;t<e.length;t++){var i=e[t],r=180-360*(i.y+a)/n;e[t]=[360*(i.x+o)/n-180,360/Math.PI*Math.atan(Math.exp(r*Math.PI/180))-90];}}switch(this.type){case 1:var c=[];for(r=0;r<l.length;r++)c[r]=l[r][0];h(l=c);break;case 2:for(r=0;r<l.length;r++)h(l[r]);break;case 3:for(l=function(e){var t=e.length;if(t<=1)return [e];for(var i,r,s=[],n=0;n<t;n++){var o=_s(e[n]);0!==o&&(void 0===r&&(r=o<0),r===o<0?(i&&s.push(i),i=[e[n]]):i.push(e[n]));}i&&s.push(i);return s}(l),r=0;r<l.length;r++)for(s=0;s<l[r].length;s++)h(l[r][s]);}1===l.length?l=l[0]:u="Multi"+u;var d={type:"Feature",geometry:{type:u,coordinates:l},properties:this.properties};return "id"in this&&(d.id=this.id),d};var ps=ms;function ms(e,t){this.version=1,this.name=null,this.extent=4096,this.length=0,this._pbf=e,this._keys=[],this._values=[],this._features=[],e.readFields(gs,this,t),this.length=this._features.length;}function gs(e,t,i){15===e?t.version=i.readVarint():1===e?t.name=i.readString():5===e?t.extent=i.readVarint():2===e?t._features.push(i.pos):3===e?t._keys.push(i.readString()):4===e&&t._values.push(function(e){var t=null,i=e.readVarint()+e.pos;for(;e.pos<i;){var r=e.readVarint()>>3;t=1===r?e.readString():2===r?e.readFloat():3===r?e.readDouble():4===r?e.readVarint64():5===r?e.readVarint():6===r?e.readSVarint():7===r?e.readBoolean():null;}return t}(i));}function xs(e,t,i){if(3===e){var r=new ps(i,i.readVarint()+i.pos);r.length&&(t[r.name]=r);}}ms.prototype.feature=function(e){if(e<0||e>=this._features.length)throw new Error("feature index out of bounds");this._pbf.pos=this._features[e];var t=this._pbf.readVarint()+this._pbf.pos;return new cs(this._pbf,t,this.extent,this._keys,this._values)};var ys=function(e,t){this.layers=e.readFields(xs,{},t);},vs=cs;class bs extends ve{constructor(e,t){super(e,t),this.response_type="arraybuffer";}parseSourceData(e,t,i){var r=new Uint8Array(i),s=new Hr(r);if(t.data=new ys(s),t.layers=this.toGeoJSON(t.data),"function"==typeof this.transform){const i={min:Object.assign({},e.min),max:Object.assign({},e.max),coords:Object.assign({},e.coords)};t.layers=this.transform(t.layers,this.extra_data,i);}delete t.data;}toGeoJSON(e){var t={};for(var i in e.layers){for(var r=e.layers[i],s=F.tile_scale/r.extent,n={type:"FeatureCollection",features:[]},o=0;o<r.length;o++){for(var a=r.feature(o),l={type:"Feature",geometry:{},properties:a.properties},u=l.geometry,h=a.loadGeometry(),c=0;c<h.length;c++)for(var d=h[c],f=0;f<d.length;f++)d[f]=[d[f].x*s,d[f].y*s];u.coordinates=h,"Point"===vs.types[a.type]?(u.type="Point",u.coordinates=u.coordinates[0][0]):"LineString"===vs.types[a.type]?1===h.length?(u.type="LineString",u.coordinates=u.coordinates[0]):u.type="MultiLineString":"Polygon"===vs.types[a.type]&&(u=As(u)),n.features.push(l);}t[i]=n;}return t}}function As(e){let t,i=[],r=[];for(let s=0;s<e.coordinates.length;s++){let n=e.coordinates[s],o=F.ringWinding(n);null!=o&&(o===(t=t||o)&&r.length>0&&(i.push(r),r=[]),r.push(n));}return r.length>0&&i.push(r),1===i.length?(e.type="Polygon",e.coordinates=i[0]):i.length>1?(e.type="MultiPolygon",e.coordinates=i):e=null,e}ge.register("MVT",()=>bs);var Ts=function(e,t){var i,r,s,n,o=t*t,a=e.length,l=0,u=a-1,h=[];e[l][2]=1,e[u][2]=1;for(;u;){for(r=0,i=l+1;i<u;i++)(s=ws(e[i],e[l],e[u]))>r&&(n=i,r=s);r>o?(e[n][2]=r,h.push(l),h.push(n),l=n):(u=h.pop(),l=h.pop());}};function ws(e,t,i){var r=t[0],s=t[1],n=i[0],o=i[1],a=e[0],l=e[1],u=n-r,h=o-s;if(0!==u||0!==h){var c=((a-r)*u+(l-s)*h)/(u*u+h*h);c>1?(r=n,s=o):c>0&&(r+=u*c,s+=h*c);}return (u=a-r)*u+(h=l-s)*h}var Es=function(e,t,i,r){var s={id:r||null,type:t,geometry:i,tags:e||null,min:[1/0,1/0],max:[-1/0,-1/0]};return function(e){var t=e.geometry,i=e.min,r=e.max;if(1===e.type)Rs(i,r,t);else for(var s=0;s<t.length;s++)Rs(i,r,t[s]);}(s),s};function Rs(e,t,i){for(var r,s=0;s<i.length;s++)r=i[s],e[0]=Math.min(r[0],e[0]),t[0]=Math.max(r[0],t[0]),e[1]=Math.min(r[1],e[1]),t[1]=Math.max(r[1],t[1]);}var Ms=function(e,t){var i=[];if("FeatureCollection"===e.type)for(var r=0;r<e.features.length;r++)zs(i,e.features[r],t);else"Feature"===e.type?zs(i,e,t):zs(i,{geometry:e},t);return i};function zs(e,t,i){if(null!==t.geometry){var r,s,n,o,a=t.geometry,l=a.type,u=a.coordinates,h=t.properties,c=t.id;if("Point"===l)e.push(Es(h,1,[Ss(u)],c));else if("MultiPoint"===l)e.push(Es(h,1,Ns(u),c));else if("LineString"===l)e.push(Es(h,2,[Ns(u,i)],c));else if("MultiLineString"===l||"Polygon"===l){for(n=[],r=0;r<u.length;r++)o=Ns(u[r],i),"Polygon"===l&&(o.outer=0===r),n.push(o);e.push(Es(h,"Polygon"===l?3:2,n,c));}else if("MultiPolygon"===l){for(n=[],r=0;r<u.length;r++)for(s=0;s<u[r].length;s++)(o=Ns(u[r][s],i)).outer=0===s,n.push(o);e.push(Es(h,3,n,c));}else{if("GeometryCollection"!==l)throw new Error("Input data is not a valid GeoJSON object.");for(r=0;r<a.geometries.length;r++)zs(e,{geometry:a.geometries[r],properties:h},i);}}}function Ns(e,t){for(var i=[],r=0;r<e.length;r++)i.push(Ss(e[r]));return t&&(Ts(i,t),function(e){for(var t,i,r=0,s=0,n=0;n<e.length-1;n++)t=i||e[n],i=e[n+1],r+=t[0]*i[1]-i[0]*t[1],s+=Math.abs(i[0]-t[0])+Math.abs(i[1]-t[1]);e.area=Math.abs(r/2),e.dist=s;}(i)),i}function Ss(e){var t=Math.sin(e[1]*Math.PI/180),i=e[0]/360+.5,r=.5-.25*Math.log((1+t)/(1-t))/Math.PI;return [i,r=r<0?0:r>1?1:r,0]}function Ps(e,t,i,r,s){return [Math.round(t*(e[0]*i-r)),Math.round(t*(e[1]*i-s))]}var ks={tile:function(e,t){if(e.transformed)return e;var i,r,s,n=e.z2,o=e.x,a=e.y;for(i=0;i<e.features.length;i++){var l=e.features[i],u=l.geometry,h=l.type;if(1===h)for(r=0;r<u.length;r++)u[r]=Ps(u[r],t,n,o,a);else for(r=0;r<u.length;r++){var c=u[r];for(s=0;s<c.length;s++)c[s]=Ps(c[s],t,n,o,a);}}return e.transformed=!0,e},point:Ps},Ls=function(e,t,i,r,s,n,o,a){if(r/=t,o>=(i/=t)&&a<=r)return e;if(o>r||a<i)return null;for(var l=[],u=0;u<e.length;u++){var h,c,d=e[u],f=d.geometry,_=d.type;if(h=d.min[s],c=d.max[s],h>=i&&c<=r)l.push(d);else if(!(h>r||c<i)){var p=1===_?Is(f,i,r,s):Os(f,i,r,s,n,3===_);p.length&&l.push(Es(d.tags,_,p,d.id));}}return l.length?l:null};function Is(e,t,i,r){for(var s=[],n=0;n<e.length;n++){var o=e[n],a=o[r];a>=t&&a<=i&&s.push(o);}return s}function Os(e,t,i,r,s,n){for(var o=[],a=0;a<e.length;a++){var l,u,h,c=0,d=0,f=null,_=e[a],p=_.area,m=_.dist,g=_.outer,x=_.length,y=[];for(u=0;u<x-1;u++)l=f||_[u],f=_[u+1],c=d||l[r],d=f[r],c<t?d>i?(y.push(s(l,f,t),s(l,f,i)),n||(y=Fs(o,y,p,m,g))):d>=t&&y.push(s(l,f,t)):c>i?d<t?(y.push(s(l,f,i),s(l,f,t)),n||(y=Fs(o,y,p,m,g))):d<=i&&y.push(s(l,f,i)):(y.push(l),d<t?(y.push(s(l,f,t)),n||(y=Fs(o,y,p,m,g))):d>i&&(y.push(s(l,f,i)),n||(y=Fs(o,y,p,m,g))));(c=(l=_[x-1])[r])>=t&&c<=i&&y.push(l),h=y[y.length-1],n&&h&&(y[0][0]!==h[0]||y[0][1]!==h[1])&&y.push(y[0]),Fs(o,y,p,m,g);}return o}function Fs(e,t,i,r,s){return t.length&&(t.area=i,t.dist=r,void 0!==s&&(t.outer=s),e.push(t)),[]}var Us=function(e,t,i){var r=e,s=Ls(e,1,-1-t,t,0,i,-1,2),n=Ls(e,1,1-t,2+t,0,i,-1,2);(s||n)&&(r=Ls(e,1,-t,1+t,0,i,-1,2)||[],s&&(r=Ds(s,1).concat(r)),n&&(r=r.concat(Ds(n,-1))));return r};function Ds(e,t){for(var i=[],r=0;r<e.length;r++){var s,n=e[r],o=n.type;if(1===o)s=Gs(n.geometry,t);else{s=[];for(var a=0;a<n.geometry.length;a++)s.push(Gs(n.geometry[a],t));}i.push(Es(n.tags,o,s,n.id));}return i}function Gs(e,t){var i=[];i.area=e.area,i.dist=e.dist;for(var r=0;r<e.length;r++)i.push([e[r][0]+t,e[r][1],e[r][2]]);return i}var Cs=function(e,t,i,r,s,n){for(var o={features:[],numPoints:0,numSimplified:0,numFeatures:0,source:null,x:i,y:r,z2:t,transformed:!1,min:[2,1],max:[-1,0]},a=0;a<e.length;a++){o.numFeatures++,js(o,e[a],s,n);var l=e[a].min,u=e[a].max;l[0]<o.min[0]&&(o.min[0]=l[0]),l[1]<o.min[1]&&(o.min[1]=l[1]),u[0]>o.max[0]&&(o.max[0]=u[0]),u[1]>o.max[1]&&(o.max[1]=u[1]);}return o};function js(e,t,i,r){var s,n,o,a,l=t.geometry,u=t.type,h=[],c=i*i;if(1===u)for(s=0;s<l.length;s++)h.push(l[s]),e.numPoints++,e.numSimplified++;else for(s=0;s<l.length;s++)if(o=l[s],r||!(2===u&&o.dist<i||3===u&&o.area<c)){var d=[];for(n=0;n<o.length;n++)a=o[n],(r||a[2]>c)&&(d.push(a),e.numSimplified++),e.numPoints++;3===u&&Vs(d,o.outer),h.push(d);}else e.numPoints+=o.length;if(h.length){var f={geometry:h,type:u,tags:t.tags||null};null!==t.id&&(f.id=t.id),e.features.push(f);}}function Vs(e,t){(function(e){for(var t,i,r=0,s=0,n=e.length,o=n-1;s<n;o=s++)t=e[s],i=e[o],r+=(i[0]-t[0])*(t[1]+i[1]);return r})(e)<0===t&&e.reverse();}var Bs=function(e,t){return new $s(e,t)};function $s(e,t){var i=(t=this.options=function(e,t){for(var i in t)e[i]=t[i];return e}(Object.create(this.options),t)).debug;i&&console.time("preprocess data");var r=1<<t.maxZoom,s=Ms(e,t.tolerance/(r*t.extent));this.tiles={},this.tileCoords=[],i&&(console.timeEnd("preprocess data"),console.log("index: maxZoom: %d, maxPoints: %d",t.indexMaxZoom,t.indexMaxPoints),console.time("generate tiles"),this.stats={},this.total=0),(s=Us(s,t.buffer/t.extent,Hs)).length&&this.splitTile(s,0,0,0),i&&(s.length&&console.log("features: %d, points: %d",this.tiles[0].numFeatures,this.tiles[0].numPoints),console.timeEnd("generate tiles"),console.log("tiles generated:",this.total,JSON.stringify(this.stats)));}function qs(e,t,i){return 32*((1<<e)*i+t)+e}function Hs(e,t,i){return [i,(i-e[0])*(t[1]-e[1])/(t[0]-e[0])+e[1],1]}function Xs(e,t,i){return [(i-e[1])*(t[0]-e[0])/(t[1]-e[1])+e[0],i,1]}function Ws(e,t,i){var r=e.source;if(1!==r.length)return !1;var s=r[0];if(3!==s.type||s.geometry.length>1)return !1;var n=s.geometry[0].length;if(5!==n)return !1;for(var o=0;o<n;o++){var a=ks.point(s.geometry[0][o],t,e.z2,e.x,e.y);if(a[0]!==-i&&a[0]!==t+i||a[1]!==-i&&a[1]!==t+i)return !1}return !0}$s.prototype.options={maxZoom:14,indexMaxZoom:5,indexMaxPoints:1e5,solidChildren:!1,tolerance:3,extent:4096,buffer:64,debug:0},$s.prototype.splitTile=function(e,t,i,r,s,n,o){for(var a=[e,t,i,r],l=this.options,u=l.debug,h=null;a.length;){r=a.pop(),i=a.pop(),t=a.pop(),e=a.pop();var c=1<<t,d=qs(t,i,r),f=this.tiles[d],_=t===l.maxZoom?0:l.tolerance/(c*l.extent);if(!f&&(u>1&&console.time("creation"),f=this.tiles[d]=Cs(e,c,i,r,_,t===l.maxZoom),this.tileCoords.push({z:t,x:i,y:r}),u)){u>1&&(console.log("tile z%d-%d-%d (features: %d, points: %d, simplified: %d)",t,i,r,f.numFeatures,f.numPoints,f.numSimplified),console.timeEnd("creation"));var p="z"+t;this.stats[p]=(this.stats[p]||0)+1,this.total++;}if(f.source=e,s){if(t===l.maxZoom||t===s)continue;var m=1<<s-t;if(i!==Math.floor(n/m)||r!==Math.floor(o/m))continue}else if(t===l.indexMaxZoom||f.numPoints<=l.indexMaxPoints)continue;if(l.solidChildren||!Ws(f,l.extent,l.buffer)){f.source=null,u>1&&console.time("clipping");var g,x,y,v,b,A,T=.5*l.buffer/l.extent,w=.5-T,E=.5+T,R=1+T;g=x=y=v=null,b=Ls(e,c,i-T,i+E,0,Hs,f.min[0],f.max[0]),A=Ls(e,c,i+w,i+R,0,Hs,f.min[0],f.max[0]),b&&(g=Ls(b,c,r-T,r+E,1,Xs,f.min[1],f.max[1]),x=Ls(b,c,r+w,r+R,1,Xs,f.min[1],f.max[1])),A&&(y=Ls(A,c,r-T,r+E,1,Xs,f.min[1],f.max[1]),v=Ls(A,c,r+w,r+R,1,Xs,f.min[1],f.max[1])),u>1&&console.timeEnd("clipping"),e.length&&(a.push(g||[],t+1,2*i,2*r),a.push(x||[],t+1,2*i,2*r+1),a.push(y||[],t+1,2*i+1,2*r),a.push(v||[],t+1,2*i+1,2*r+1));}else s&&(h=t);}return h},$s.prototype.getTile=function(e,t,i){var r=this.options,s=r.extent,n=r.debug,o=1<<e,a=qs(e,t=(t%o+o)%o,i);if(this.tiles[a])return ks.tile(this.tiles[a],s);n>1&&console.log("drilling down to z%d-%d-%d",e,t,i);for(var l,u=e,h=t,c=i;!l&&u>0;)u--,h=Math.floor(h/2),c=Math.floor(c/2),l=this.tiles[qs(u,h,c)];if(!l||!l.source)return null;if(n>1&&console.log("found parent tile z%d-%d-%d",u,h,c),Ws(l,s,r.buffer))return ks.tile(l,s);n>1&&console.time("drilling down");var d=this.splitTile(l.source,u,h,c,e,t,i);if(n>1&&console.timeEnd("drilling down"),null!==d){var f=1<<e-d;a=qs(d,Math.floor(t/f),Math.floor(i/f));}return this.tiles[a]?ks.tile(this.tiles[a],s):null};class Zs extends ye{constructor(e,t){super(e,t),this.load_data=null,this.tile_indexes={},this.setTileSize(512),this.pad_scale=0;}_load(e){return this.load_data||(this.load_data=super._load({source_data:{layers:{}}}).then(e=>{e.source_data.error&&f("warn",`data source load error(s) for source '${this.name}', URL '${this.url}': ${e.source_data.error}`);let t=e.source_data.layers;for(let e in t)this.tile_indexes[e]=Bs(t[e],{maxZoom:this.max_zoom,tolerance:1.5,extent:F.tile_scale,buffer:1e-4});return this.loaded=!0,e})),this.load_data.then(()=>{for(let t in this.tile_indexes)e.source_data.layers[t]=this.getTileFeatures(e,t);return e})}getTileFeatures(e,t){let i,r=F.wrapTile(e.coords,{x:!0}),s=this.tile_indexes[t].getTile(r.z,r.x,r.y);if(s&&s.features){i={type:"FeatureCollection",features:[]};for(let e=0;e<s.features.length;e++){const t=s.features[e];let r={type:"Feature",geometry:{},properties:t.tags};if(1===t.type)r.geometry.coordinates=t.geometry.map(e=>[e[0],e[1]]),r.geometry.type="MultiPoint";else{if(2!==t.type&&3!==t.type)continue;if(r.geometry.coordinates=t.geometry.map(e=>e.map(e=>[e[0],e[1]])),2===t.type)r.geometry.type="MultiLineString";else if(r.geometry=As(r.geometry),null==r.geometry)continue}i.features.push(r);}}return i}formatURL(){return this.url}parseSourceData(e,t,i){let r="string"==typeof i?JSON.parse(i):i,s=this.getLayers(r);t.layers=this.preprocessLayers(s,e);}preprocessLayers(e,t){for(let t in e){let i=e[t];i.features=this.preprocessFeatures(i.features);}if("function"==typeof this.transform){const i={min:Object.assign({},t.min),max:Object.assign({},t.max),coords:Object.assign({},t.coords)};1===Object.keys(e).length&&e._default?e._default=this.transform(e._default,this.extra_data,i):e=this.transform(e,this.extra_data,i);}return e}preprocessFeatures(e){if(e=e.filter(e=>null!=e.geometry),this.config.generate_label_centroids){let t=[],i={label_placement:!0};e.forEach(e=>{let r,s;if("Polygon"===e.geometry.type)s=Js(r=e.geometry.coordinates,e.properties,i),t.push(s);else if("MultiPolygon"===e.geometry.type){r=e.geometry.coordinates;let n=-1/0,o=0;for(let e=0;e<r.length;e++){let t=F.polygonArea(r[e]);t>n&&(n=t,o=e);}s=Js(r[o],e.properties,i),t.push(s);}}),t=t.filter(e=>e),Array.prototype.push.apply(e,t);}return e}getLayers(e){return "Feature"===e.type?{_default:{type:"FeatureCollection",features:[e]}}:"FeatureCollection"===e.type?{_default:e}:e}}class Ys extends ve{constructor(e,t){super(e,t);}parseSourceData(e,t,i){let r="string"==typeof i?JSON.parse(i):i;this.prepareGeoJSON(r,e,t);}prepareGeoJSON(e,t,i){if("function"==typeof this.transform){const i={min:Object.assign({},t.min),max:Object.assign({},t.max),coords:Object.assign({},t.coords)};e=this.transform(e,this.extra_data,i);}i.layers=Zs.prototype.getLayers(e);let r={coords:t.coords,min:F.metersForTile(F.wrapTile(t.coords,{x:!0}))};ge.projectData(i),ge.scaleData(i,r);}}function Js(e,t,i){let r=F.centroid(e);if(!r)return;let s={};return Object.assign(s,t,i),{type:"Feature",properties:s,geometry:{type:"Point",coordinates:r}}}function Ks(e){return e}function Qs(e,t){var i=t.id,r=t.bbox,s=null==t.properties?{}:t.properties,n=function(e,t){var i=function(e){if(null==(t=e.transform))return Ks;var t,i,r,s=t.scale[0],n=t.scale[1],o=t.translate[0],a=t.translate[1];return function(e,t){return t||(i=r=0),e[0]=(i+=e[0])*s+o,e[1]=(r+=e[1])*n+a,e}}(e),r=e.arcs;function s(e,t){t.length&&t.pop();for(var s=r[e<0?~e:e],n=0,o=s.length;n<o;++n)t.push(i(s[n].slice(),n));e<0&&function(e,t){for(var i,r=e.length,s=r-t;s<--r;)i=e[s],e[s++]=e[r],e[r]=i;}(t,o);}function n(e){return i(e.slice())}function o(e){for(var t=[],i=0,r=e.length;i<r;++i)s(e[i],t);return t.length<2&&t.push(t[0].slice()),t}function a(e){for(var t=o(e);t.length<4;)t.push(t[0].slice());return t}function l(e){return e.map(a)}return function e(t){var i,r=t.type;switch(r){case"GeometryCollection":return {type:r,geometries:t.geometries.map(e)};case"Point":i=n(t.coordinates);break;case"MultiPoint":i=t.coordinates.map(n);break;case"LineString":i=o(t.arcs);break;case"MultiLineString":i=t.arcs.map(o);break;case"Polygon":i=l(t.arcs);break;case"MultiPolygon":i=t.arcs.map(l);break;default:return null}return {type:r,coordinates:i}}(t)}(e,t);return null==i&&null==r?{type:"Feature",properties:s,geometry:n}:null==r?{type:"Feature",id:i,properties:s,geometry:n}:{type:"Feature",id:i,bbox:r,properties:s,geometry:n}}ge.register("GeoJSON",e=>Ys.urlHasTilePattern(e.url)?Ys:Zs);class en extends Zs{parseSourceData(e,t,i){let r="string"==typeof i?JSON.parse(i):i;r=this.toGeoJSON(r);let s=this.getLayers(r);super.preprocessLayers(s,e),t.layers=s;}toGeoJSON(e){if(e.objects&&1===Object.keys(e.objects).length){let t=Object.keys(e.objects)[0];e=tn(e,e.objects[t]);}else{let t={};for(let i in e.objects)t[i]=tn(e,e.objects[i]);e=t;}return e}}function tn(e,t){let i=function(e,t){return "GeometryCollection"===t.type?{type:"FeatureCollection",features:t.geometries.map(function(t){return Qs(e,t)})}:Qs(e,t)}(e,t);return "Feature"===i.type&&(i={type:"FeatureCollection",features:[i]}),i}class rn extends Ys{constructor(e,t){super(e,t);}parseSourceData(e,t,i){let r="string"==typeof i?JSON.parse(i):i;r=en.prototype.toGeoJSON(r),this.prepareGeoJSON(r,e,t);}}ge.register("TopoJSON",e=>rn.urlHasTilePattern(e.url)?rn:en),e.createCommonjsModule=B,e.commonjsRequire=function(){throw new Error("Dynamic requires are not currently supported by rollup-plugin-commonjs")},e.isLocalURL=function(e){if("string"==typeof e)return e.search(/^(data|blob):/)>-1},e.extensionForURL=function(e){let t=(e=e.split("/").pop()).lastIndexOf(".");if(t>-1)return e.substring(t+1)},e.isRelativeURL=x,e.pathForURL=g,e.Utils=_,e.addBaseURL=function(e,t){if(!e||!x(e))return e;var i,r="/"!==e[0];if(t?(i=document.createElement("a")).href=t:i=window.location,r)e=g(i.href)+e;else{let t=i.origin;t||(t=i.protocol+"//"+i.host),e=t+e;}return e},e.flattenRelativeURL=function(e){let t=(e||"").split("/");for(let e=1;e<t.length;e++)"."===t[e]?(t.splice(e,1),e--):".."===t[e]&&(e+=0,t.splice(e-1,2),e--);return t.join("/")},e.createObjectURL=function(e){return void 0===y&&"function"!=typeof(y=window.URL&&window.URL.createObjectURL||window.webkitURL&&window.webkitURL.createObjectURL)&&(y=null,f("warn","window.URL.createObjectURL (or vendor prefix) not found, unable to create local blob URLs")),y?y(e):e},e.subscribeMixin=A,e.log=f,e.mergeObjects=O,e.isReserved=Gr,e.GLSL=E,e.TileID=be,e.Collision=ai,e.Geo=F,e.LabelPoint=li,e.LabelLineStraight=or,e.OBB=ri,e.Label=si,e.WorkerBroker=n,e.Task=b,e.Tile=Br,e.StyleParser=q,e.Texture=w,e.debugSettings=m,e.debugSumLayerStats=function(e){let t={},i={};return e.filter(e=>e.debug.layers).forEach(e=>{Object.keys(e.debug.layers.list).forEach(i=>{let r=e.debug.layers.list[i];Re(t,i,r.features,r.geoms,r.styles,r.base);}),function e(t,i){for(let r in t){let s=t[r];Re(i,r,s.features,s.geoms,s.styles,s.base),s.layers&&(i[r].layers=i[r].layers||{},e(s.layers,i[r].layers));}}(e.debug.layers.tree,i);}),{list:t,tree:i}},e.View=class{constructor(e,t){A(this),this.scene=e,this.createMatrices(),this.zoom=null,this.center=null,this.bounds=null,this.meters_per_pixel=null,this.panning=!1,this.panning_stop_at=0,this.pan_snap_timer=0,this.zoom_direction=0,this.user_input_at=0,this.user_input_timeout=50,this.user_input_active=!1,this.size={css:{},device:{},meters:{}},this.aspect=null,this.buffer=0,this.continuous_zoom="boolean"!=typeof t.continuousZoom||t.continuousZoom,this.wrap=!1!==t.wrapView,this.preserve_tiles_within_zoom=1,this.reset();}reset(){this.createCamera();}createCamera(){let e=this.getActiveCamera();e&&(this.camera=$i.create(e,this,this.scene.config.cameras[e]),this.camera.updateView());}getActiveCamera(){if(this.scene.config&&this.scene.config.cameras){for(let e in this.scene.config.cameras)if(this.scene.config.cameras[e].active)return e;let e=Object.keys(this.scene.config.cameras);return e.length&&e[0]}}setActiveCamera(e){let t=this.getActiveCamera();return t===e?e:(this.scene.config.cameras[e]&&(this.scene.config.cameras[e].active=!0,t&&this.scene.config.cameras[t]&&delete this.scene.config.cameras[t].active),this.scene.updateConfig({rebuild:!1,normalize:!1}),this.getActiveCamera())}update(){null!=this.camera&&this.ready()&&this.camera.update(),this.pan_snap_timer=(+new Date-this.panning_stop_at)/1e3,this.user_input_active=+new Date-this.user_input_at<this.user_input_timeout;}setViewportSize(e,t){this.size.css={width:e,height:t},this.size.device={width:Math.round(this.size.css.width*_.device_pixel_ratio),height:Math.round(this.size.css.height*_.device_pixel_ratio)},this.aspect=this.size.css.width/this.size.css.height,this.updateBounds();}setView(e){let t=void 0===e?{}:e,i=t.lng,r=t.lat,s=t.zoom;var n=!1;return "number"==typeof i&&"number"==typeof r&&(this.center&&i===this.center.lng&&r===this.center.lat||(n=!0,this.center={lng:i,lat:r})),"number"==typeof s&&s!==this.zoom&&(n=!0,this.setZoom(s)),n&&this.updateBounds(),n}setZoom(e){let t=this.tile_zoom,i=this.baseZoom(e);this.continuous_zoom||(e=i),i!==t&&(this.zoom_direction=i>t?1:-1),this.zoom=e,this.tile_zoom=i,this.updateBounds(),this.scene.requestRedraw();}baseZoom(e){return Math.floor(e)}setPanning(e){this.panning=e,this.panning||(this.panning_stop_at=+new Date);}markUserInput(){this.user_input_at=+new Date;}ready(){return "number"==typeof this.size.css.width&&"number"==typeof this.size.css.height&&null!=this.center&&"number"==typeof this.zoom}updateBounds(){if(!this.ready())return;this.meters_per_pixel=F.metersPerPixel(this.zoom),this.size.meters={x:this.size.css.width*this.meters_per_pixel,y:this.size.css.height*this.meters_per_pixel};const e=F.latLngToMeters([this.center.lng,this.center.lat]);this.center.meters={x:e[0],y:e[1]},this.center.tile=F.tileForMeters([this.center.meters.x,this.center.meters.y],this.tile_zoom),this.bounds={sw:{x:this.center.meters.x-this.size.meters.x/2,y:this.center.meters.y-this.size.meters.y/2},ne:{x:this.center.meters.x+this.size.meters.x/2,y:this.center.meters.y+this.size.meters.y/2}},this.scene.tile_manager.updateTilesForView(),this.trigger("move"),this.scene.requestRedraw();}findVisibleTileCoordinates(){if(!this.bounds)return [];let e=this.tile_zoom,t=F.tileForMeters([this.bounds.sw.x,this.bounds.sw.y],e),i=F.tileForMeters([this.bounds.ne.x,this.bounds.ne.y],e),r=[t.x-this.buffer,i.x+this.buffer,i.y-this.buffer,t.y+this.buffer];if(!1===this.wrap){let t=(1<<e)-1;r=r.map(e=>Math.min(Math.max(0,e),t));}let s=[];for(let t=r[0];t<=r[1];t++)for(let i=r[2];i<=r[3];i++)s.push(be.coord({x:t,y:i,z:e}));return s}pruneTilesForView(){this.ready()&&this.scene.tile_manager.removeTiles(e=>{if(e.visible||e.isProxy())return !1;if(e.loading&&e.style_z!==this.tile_zoom)return !0;if(Math.abs(e.style_z-this.tile_zoom)>(null!=e.preserve_tiles_within_zoom?e.preserve_tiles_within_zoom:this.preserve_tiles_within_zoom))return !0;const t=this.meters_per_pixel*F.tile_size,i=be.coordAtZoom(F.tileForMeters([this.center.meters.x-this.size.meters.x/2-t,this.center.meters.y+this.size.meters.y/2+t],this.tile_zoom),e.coords.z),r=be.coordAtZoom(F.tileForMeters([this.center.meters.x+this.size.meters.x/2+t,this.center.meters.y-this.size.meters.y/2-t],this.tile_zoom),e.coords.z);return (e.coords.x<i.x||e.coords.x>r.x||e.coords.y<i.y||e.coords.y>r.y)&&(f("trace",`View: remove tile ${e.key} (as ${e.coords.key}) `+`for being too far out of visible area (${i.key}, ${r.key})`),!0)});}createMatrices(){this.matrices={},this.matrices.model=new Float64Array(16),this.matrices.model32=new Float32Array(16),this.matrices.model_view=new Float64Array(16),this.matrices.model_view32=new Float32Array(16),this.matrices.normal=new Float64Array(9),this.matrices.normal32=new Float32Array(9),this.matrices.inverse_normal32=new Float32Array(9);}setupTile(e,t){e.setupProgram(this.matrices,t),this.camera.setupMatrices(this.matrices,t);}setupProgram(e){e.uniform("2fv","u_resolution",[this.size.device.width,this.size.device.height]),e.uniform("3fv","u_map_position",[this.center.meters.x,this.center.meters.y,this.zoom]),e.uniform("1f","u_meters_per_pixel",this.meters_per_pixel),e.uniform("1f","u_device_pixel_ratio",_.device_pixel_ratio),e.uniform("1f","u_view_pan_snap_timer",this.pan_snap_timer),e.uniform("1i","u_view_panning",this.panning),this.camera.setupProgram(e);}isAnimating(){return this.pan_snap_timer<=Wi}},e.StyleManager=class{constructor(){this.styles={},this.base_styles={},this.active_styles=[],this.active_blend_orders=[],this.register(Object.create(dt)),this.register(Object.create(Ot)),this.register(Object.create(Yi)),this.register(Object.create(dr)),this.register(Object.create(fr));}init(){k.removeBlock("global"),k.removeBlock("setup"),k.addBlock("global",_r),k.addBlock("global",pr),k.replaceBlock("setup",mr),k.defines.TANGRAM_EPSILON=1e-5,k.defines.TANGRAM_LAYER_DELTA=1/16384,k.defines.TANGRAM_TILE_SCALE=`vec3(${F.tile_scale}., ${F.tile_scale}., u_meters_per_pixel * ${F.tile_size}.)`,k.defines.TANGRAM_HEIGHT_SCALE=F.height_scale,k.defines.TANGRAM_ALPHA_TEST=.5;}destroy(e){Object.keys(this.styles).forEach(t=>{let i=this.styles[t];i.gl===e&&(f("trace",`StyleManager.destroy: destroying render style ${i.name}`),i.base&&this.remove(i.name),i.destroy());});}register(e){this.styles[e.name]=e,this.base_styles[e.name]=e;}remove(e){delete this.styles[e];}getActiveStyles(){return this.active_styles}updateActiveStyles(e){return this.active_styles=Object.keys(e.reduce((e,t)=>(Object.keys(t.meshes).forEach(t=>e[t]=!0),e),{})),this.active_styles}getActiveBlendOrders(){return this.active_blend_orders}updateActiveBlendOrders(e){const t=[];e.forEach(e=>{Object.entries(e.meshes).forEach(e=>{let i=e[0];e[1].forEach(e=>{const r=e.variant.blend_order;let s=t.findIndex(e=>e.blend_order===r);s=s>-1?s:t.push({blend_order:r,styles:[]})-1,-1===t[s].styles.indexOf(i)&&t[s].styles.push(i);});});}),this.active_blend_orders=t.sort((e,t)=>e.blend_order-t.blend_order);}mix(e,t){if(e.mixed)return e;e.mixed={};let i=[];e.mix&&(Array.isArray(e.mix)?i.push(...e.mix):i.push(e.mix),(i=i.map(e=>t[e]).filter(t=>t&&t!==e)).forEach(t=>e.mixed[t.name]=!0)),i.push(e),e.animated=i.some(e=>e&&e.animated),e.texcoords=i.some(e=>e&&e.texcoords),e.base=i.map(e=>e.base).filter(e=>e).pop(),e.lighting=i.map(e=>e.lighting).filter(e=>null!=e).pop(),e.texture=i.map(e=>e.texture).filter(e=>e).pop(),e.raster=i.map(e=>e.raster).filter(e=>null!=e).pop(),e.dash=i.map(e=>e.dash).filter(e=>null!=e).pop(),e.dash_background_color=i.map(e=>e.dash_background_color).filter(e=>null!=e).pop(),i.some(e=>e.hasOwnProperty("blend")&&e.blend)&&(e.blend=i.map(e=>e.hasOwnProperty("blend")&&e.blend).filter(e=>e).pop()),e.blend_order=i.map(e=>e.blend_order).filter(e=>null!=e).pop(),e.defines=Object.assign({},...i.map(e=>e.defines).filter(e=>e)),e.material=Object.assign({},...i.map(e=>e.material).filter(e=>e));let r=i.map(e=>e.draw).filter(e=>e);return r.length>0&&(e.draw=O({},...r)),this.mixShaders(e,t,i),e}mixShaders(e,t,i){let r={},s=i.map(e=>e.shaders).filter(e=>e);if(r.defines=Object.assign({},...s.map(e=>e.defines).filter(e=>e)),r.uniforms={},r._uniforms=e.shaders&&e.shaders.uniforms||{},r._uniform_scopes={},i.filter(e=>e.shaders&&e.shaders.uniforms).forEach(e=>{for(let i in e.shaders.uniforms)r._uniform_scopes[i]=e.name,Object.defineProperty(r.uniforms,i,{enumerable:!0,configurable:!0,get:function(){return void 0!==r._uniforms[i]?r._uniforms[i]:t[r._uniform_scopes[i]].shaders.uniforms!==r.uniforms?t[r._uniform_scopes[i]].shaders.uniforms[i]:void 0},set:function(e){r._uniforms[i]=e;}});}),r.extensions=Object.keys(s.map(e=>e.extensions).filter(e=>e).reduce((e,t)=>("string"==typeof t?e[t]=!0:t.forEach(t=>e[t]=!0),e),{})||{}),e.shaders&&e.shaders.blocks){e.shaders.block_scopes=e.shaders.block_scopes||{};for(let t in e.shaders.blocks){let i=e.shaders.blocks[t];e.shaders.block_scopes[t]=e.shaders.block_scopes[t]||[],Array.isArray(i)?e.shaders.block_scopes[t].push(...i.map(()=>e.name)):e.shaders.block_scopes[t].push(e.name);}}let n={};return s.forEach(e=>{if(!e.blocks)return;r.blocks=r.blocks||{},r.block_scopes=r.block_scopes||{};let t={};for(let i in e.blocks){let s=e.blocks[i],o=e.block_scopes[i];r.blocks[i]=r.blocks[i]||[],r.block_scopes[i]=r.block_scopes[i]||[],s=Array.isArray(s)?s:[s],o=Array.isArray(o)?o:[o];for(let e=0;e<s.length;e++)n[o[e]]||(t[o[e]]=!0,r.blocks[i].push(s[e]),r.block_scopes[i].push(o[e]));}Object.assign(n,t);}),Object.assign(e.mixed,n),e.shaders=r,e}create(e,t,i){void 0===i&&(i={});let r=O({},t);return r.name=e,(r=this.mix(r,i)).base&&this.base_styles[r.base]?this.styles[e]=r=Object.assign(Object.create(this.base_styles[r.base]),r):r.base=null,r}build(e){this.styles&&Object.values(this.styles).forEach(e=>n.removeTarget(e.main_thread_target));let t=Object.keys(e).sort((t,i)=>this.inheritanceDepth(t,e)-this.inheritanceDepth(i,e));for(let e in this.styles)this.base_styles[e]?this.styles[e].reset():delete this.styles[e];let i={};return t.forEach(t=>{i[t]=this.create(t,e[t],i);}),this.styles}initStyles(e){void 0===e&&(e={});for(let t in this.styles)this.styles[t].init(e);}inheritanceDepth(e,t){let i=0;for(;;){let r=t[e];if(!r)break;if(!r.mix)break;if(i++,Array.isArray(r.mix)){i+=Math.max(...r.mix.map(i=>{if(e!==i)return this.inheritanceDepth(i,t)}));break}if(e===r.mix)break;e=r.mix;}return i}},e.ShaderProgram=k,e.VertexArrayObject=I,e.Style=Ee,e.TextCanvas=Ui,e._extends=i,e.DataSource=ge,e.Light=_e,e.FontManager=xi,e.FeatureSelection=re,e.sliceObject=T,e.Thread=t,e.mergeDebugSettings=function(e){Object.assign(p,e);},e.version=s,e.Vector=ue,e.VertexData=ke,e.Material=ae,e.VertexElements=Se,e.compileFunctionStrings=j,e.parseLayers=function(e,t){kr={};let i={};for(let r in e){let s=e[r];s&&(i[r]=Cr(r,s,null,t));}return i},e.buildFilter=Sr,e.FilterOptions=Dr,e.layerCache=function(){return kr},e.cache=C,e.clearFunctionStringCache=function(){C.functions={},C.num_functions=0,C.num_cached=0;};});

define(["./shared.js"],function(e){"use strict";const t=Object.assign(self,{FeatureSelection:e.FeatureSelection,sources:{},styles:{},layers:{},tiles:{},init(t,r,i,s,o,l,n){return this.scene_id=t,this._worker_id=r,this.num_workers=i,e.log.setLevel(s),e.Utils.device_pixel_ratio=o,e.VertexElements.setElementIndexUint(l),e.FeatureSelection.setPrefix(this._worker_id),this.style_manager=new e.StyleManager,this.importExternalScripts(n),e.Label.id_prefix=r,e.Label.id_multiplier=i,r},importExternalScripts(t){if(0===t.length)return;e.log("debug","loading custom data source scripts in worker:",t);let r=Object.getOwnPropertyNames(window);importScripts(...t),Object.getOwnPropertyNames(window).forEach(e=>{-1===r.indexOf(e)&&(this[e]=window[e]);});},updateConfig(t,r){let i=t.config,s=t.generation,o=t.introspection;return i=JSON.parse(i),e.mergeDebugSettings(r),this.generation=s,this.introspection=o,this.global=e.compileFunctionStrings(i.global),this.createDataSources(i),i.styles=e.compileFunctionStrings(i.styles,e.StyleParser.wrapFunction),this.styles=this.style_manager.build(i.styles),this.style_manager.initStyles({generation:this.generation,styles:this.styles,sources:this.sources,introspection:this.introspection}),this.layers=e.parseLayers(i.layers,this.style_manager.styles),this.syncing_textures=this.syncTextures(i.textures),this.configuring=this.syncing_textures.then(()=>{e.log("debug","updated config");}),this.configuring},createDataSources(t){this.last_config_sources=this.config_sources||{},this.config_sources=t.sources;let r=this.sources,i=[];this.sources={};for(let s in t.sources){if(JSON.stringify(this.last_config_sources[s])===JSON.stringify(t.sources[s])){this.sources[s]=r[s];continue}let o;t.sources[s]=e.compileFunctionStrings(t.sources[s]);try{o=e.DataSource.create(Object.assign({},t.sources[s],{name:s}),this.sources);}catch(e){continue}o&&(this.sources[s]=o,i.push(s));}i.forEach(e=>{for(let t in this.tiles)this.tiles[t].source===e&&delete this.tiles[t];});},awaitConfiguration(){return this.configuring},buildTile(t){let r=t.tile;if(null==this.getTile(r.key)||!0!==this.getTile(r.key).loading)return r=this.tiles[r.key]=Object.assign(this.getTile(r.key)||{},r),this.awaitConfiguration().then(()=>{if(!0!==r.loaded)r.loading=!0,r.loaded=!1,r.error=null,this.loadTileSourceData(r).then(()=>{this.getTile(r.key)?(r.source_data.error&&e.log("warn",`tile load error(s) for ${r.key}: ${r.source_data.error}`),r.loading=!1,r.loaded=!0,e.Tile.buildGeometry(r,this)):e.log("trace",`stop tile build after data source load because tile was removed: ${r.key}`);}).catch(t=>{r.loading=!1,r.loaded=!1,r.error=t.stack,e.log("error",`tile load error for ${r.key}: ${r.error}`),e.WorkerBroker.postMessage(`TileManager_${this.scene_id}.buildTileError`,e.Tile.slice(r));});else{e.log("trace",`used worker cache for tile ${r.key}`);try{e.Tile.buildGeometry(r,this);}catch(t){r.error=t.toString(),e.WorkerBroker.postMessage(`TileManager_${this.scene_id}.buildTileError`,e.Tile.slice(r));}}})},loadTileSourceData(e){const t=this.sources[e.source];if(t){for(const r in this.tiles){const i=this.tiles[r];if(i.source===e.source&&i.coords.key===e.coords.key&&i.loaded)return Promise.resolve(t.copyTileData(i,e))}return t.load(e)}return e.source_data={},Promise.resolve(e)},getTile(e){return this.tiles[e]},removeTile(t){var r=this.tiles[t];null!=r&&(!0===r.loading&&(e.log("trace",`cancel tile load for ${t}`),r.loading=!1,e.Tile.cancel(r)),e.FeatureSelection.clearTile(t),delete this.tiles[t],e.log("trace",`remove tile from cache for ${t}`));},queryFeatures(t){let r=t.filter,i=t.visible,s=t.geometry,o=t.tile_keys,l=[],n=o.map(e=>this.tiles[e]).filter(e=>e&&e.loaded);return null!=r&&(r=["{","["].indexOf(r[0])>-1?JSON.parse(r):r,r=e.compileFunctionStrings(r,e.StyleParser.wrapFunction)),r=e.buildFilter(r,e.FilterOptions),n.forEach(t=>{for(let o in t.source_data.layers){t.source_data.layers[o].features.forEach(n=>{const a=n.generation===this.generation;if(!0===i&&!a||!1===i&&a)return;let c=e.StyleParser.getFeatureParseContext(n,t,this.global);if(c.source=t.source,c.layer=o,!r(c))return;let u={type:n.type,properties:Object.assign({},n.properties,{$source:c.source,$layer:c.layer,$geometry:c.geometry,$visible:a})};!0===s&&(u.geometry=e.Geo.copyGeometry(n.geometry),e.Geo.tileSpaceToLatlng(u.geometry,t.coords.z,t.min)),l.push(u);});}}),l},getFeatureSelection(t){let r=void 0===t?{}:t,i=r.id,s=r.key;var o=e.FeatureSelection.map[s];return {id:i,feature:o&&o.feature}},resetFeatureSelection(t){void 0===t&&(t=null),e.FeatureSelection.reset(t);},getFeatureSelectionMapSize:()=>e.FeatureSelection.getMapSize(),syncTextures(t){let r=[];return t&&r.push(...Object.keys(t)),e.log("trace","sync textures to worker:",r),r.length>0?e.Texture.syncTexturesToWorker(r):Promise.resolve()},updateDevicePixelRatio(t){e.Utils.device_pixel_ratio=t;},clearFunctionStringCache(){e.clearFunctionStringCache();},profile(e){console.profile(`worker ${this._worker_id}: ${e}`);},profileEnd(e){console.profileEnd(`worker ${this._worker_id}: ${e}`);},debug:{debugSettings:e.debugSettings,layerCache:e.layerCache,functionStringCache:e.cache}});e.WorkerBroker.addTarget("self",t);});

define(["./shared.js"],function(e){"use strict";var t,i=t={};let r=0;t.getContext=function(e,i){var n=!1;null==e&&((e=document.createElement("canvas")).style.position="absolute",e.style.top=0,e.style.left=0,e.style.zIndex=-1,document.body.appendChild(e),n=!0);var s=e.getContext("webgl",i)||e.getContext("experimental-webgl",i);if(!s)throw new Error("Couldn't create WebGL context.");return s._tangram_id=r++,n?(t.resize(s,window.innerWidth,window.innerHeight,i.device_pixel_ratio),window.addEventListener("resize",function(){t.resize(s,window.innerWidth,window.innerHeight,i.device_pixel_ratio);})):t.resize(s,parseFloat(e.style.width),parseFloat(e.style.height),i.device_pixel_ratio),s},t.resize=function(e,t,i,r){r=r||window.devicePixelRatio||1,e.canvas.style.width=t+"px",e.canvas.style.height=i+"px",e.canvas.width=Math.round(t*r),e.canvas.height=Math.round(i*r),e.bindFramebuffer(e.FRAMEBUFFER,null),e.viewport(0,0,e.canvas.width,e.canvas.height);};var n="undefined"!=typeof global?global:"undefined"!=typeof self?self:"undefined"!=typeof window?window:{},s=[],o=[],a="undefined"!=typeof Uint8Array?Uint8Array:Array,l=!1;function h(){l=!0;for(var e="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/",t=0,i=e.length;t<i;++t)s[t]=e[t],o[e.charCodeAt(t)]=t;o["-".charCodeAt(0)]=62,o["_".charCodeAt(0)]=63;}function u(e,t,i){for(var r,n,o=[],a=t;a<i;a+=3)r=(e[a]<<16)+(e[a+1]<<8)+e[a+2],o.push(s[(n=r)>>18&63]+s[n>>12&63]+s[n>>6&63]+s[63&n]);return o.join("")}function c(e){var t;l||h();for(var i=e.length,r=i%3,n="",o=[],a=0,c=i-r;a<c;a+=16383)o.push(u(e,a,a+16383>c?c:a+16383));return 1===r?(t=e[i-1],n+=s[t>>2],n+=s[t<<4&63],n+="=="):2===r&&(t=(e[i-2]<<8)+e[i-1],n+=s[t>>10],n+=s[t>>4&63],n+=s[t<<2&63],n+="="),o.push(n),o.join("")}function f(e,t,i,r,n){var s,o,a=8*n-r-1,l=(1<<a)-1,h=l>>1,u=-7,c=i?n-1:0,f=i?-1:1,d=e[t+c];for(c+=f,s=d&(1<<-u)-1,d>>=-u,u+=a;u>0;s=256*s+e[t+c],c+=f,u-=8);for(o=s&(1<<-u)-1,s>>=-u,u+=r;u>0;o=256*o+e[t+c],c+=f,u-=8);if(0===s)s=1-h;else{if(s===l)return o?NaN:1/0*(d?-1:1);o+=Math.pow(2,r),s-=h;}return (d?-1:1)*o*Math.pow(2,s-r)}function d(e,t,i,r,n,s){var o,a,l,h=8*s-n-1,u=(1<<h)-1,c=u>>1,f=23===n?Math.pow(2,-24)-Math.pow(2,-77):0,d=r?0:s-1,p=r?1:-1,g=t<0||0===t&&1/t<0?1:0;for(t=Math.abs(t),isNaN(t)||t===1/0?(a=isNaN(t)?1:0,o=u):(o=Math.floor(Math.log(t)/Math.LN2),t*(l=Math.pow(2,-o))<1&&(o--,l*=2),(t+=o+c>=1?f/l:f*Math.pow(2,1-c))*l>=2&&(o++,l/=2),o+c>=u?(a=0,o=u):o+c>=1?(a=(t*l-1)*Math.pow(2,n),o+=c):(a=t*Math.pow(2,c-1)*Math.pow(2,n),o=0));n>=8;e[i+d]=255&a,d+=p,a/=256,n-=8);for(o=o<<n|a,h+=n;h>0;e[i+d]=255&o,d+=p,o/=256,h-=8);e[i+d-p]|=128*g;}var p={}.toString,g=Array.isArray||function(e){return "[object Array]"==p.call(e)};v.TYPED_ARRAY_SUPPORT=void 0===n.TYPED_ARRAY_SUPPORT||n.TYPED_ARRAY_SUPPORT;var m=_();function _(){return v.TYPED_ARRAY_SUPPORT?2147483647:1073741823}function y(e,t){if(_()<t)throw new RangeError("Invalid typed array length");return v.TYPED_ARRAY_SUPPORT?(e=new Uint8Array(t)).__proto__=v.prototype:(null===e&&(e=new v(t)),e.length=t),e}function v(e,t,i){if(!(v.TYPED_ARRAY_SUPPORT||this instanceof v))return new v(e,t,i);if("number"==typeof e){if("string"==typeof t)throw new Error("If encoding is specified then the first argument must be a string");return k(this,e)}return b(this,e,t,i)}function b(e,t,i,r){if("number"==typeof t)throw new TypeError('"value" argument must not be a number');return "undefined"!=typeof ArrayBuffer&&t instanceof ArrayBuffer?function(e,t,i,r){if(t.byteLength,i<0||t.byteLength<i)throw new RangeError("'offset' is out of bounds");if(t.byteLength<i+(r||0))throw new RangeError("'length' is out of bounds");t=void 0===i&&void 0===r?new Uint8Array(t):void 0===r?new Uint8Array(t,i):new Uint8Array(t,i,r);v.TYPED_ARRAY_SUPPORT?(e=t).__proto__=v.prototype:e=S(e,t);return e}(e,t,i,r):"string"==typeof t?function(e,t,i){"string"==typeof i&&""!==i||(i="utf8");if(!v.isEncoding(i))throw new TypeError('"encoding" must be a valid string encoding');var r=0|E(t,i),n=(e=y(e,r)).write(t,i);n!==r&&(e=e.slice(0,n));return e}(e,t,i):function(e,t){if(C(t)){var i=0|x(t.length);return 0===(e=y(e,i)).length?e:(t.copy(e,0,0,i),e)}if(t){if("undefined"!=typeof ArrayBuffer&&t.buffer instanceof ArrayBuffer||"length"in t)return "number"!=typeof t.length||(r=t.length)!=r?y(e,0):S(e,t);if("Buffer"===t.type&&g(t.data))return S(e,t.data)}var r;throw new TypeError("First argument must be a string, Buffer, ArrayBuffer, Array, or array-like object.")}(e,t)}function w(e){if("number"!=typeof e)throw new TypeError('"size" argument must be a number');if(e<0)throw new RangeError('"size" argument must not be negative')}function k(e,t){if(w(t),e=y(e,t<0?0:0|x(t)),!v.TYPED_ARRAY_SUPPORT)for(var i=0;i<t;++i)e[i]=0;return e}function S(e,t){var i=t.length<0?0:0|x(t.length);e=y(e,i);for(var r=0;r<i;r+=1)e[r]=255&t[r];return e}function x(e){if(e>=_())throw new RangeError("Attempt to allocate Buffer larger than maximum size: 0x"+_().toString(16)+" bytes");return 0|e}function C(e){return !(null==e||!e._isBuffer)}function E(e,t){if(C(e))return e.length;if("undefined"!=typeof ArrayBuffer&&"function"==typeof ArrayBuffer.isView&&(ArrayBuffer.isView(e)||e instanceof ArrayBuffer))return e.byteLength;"string"!=typeof e&&(e=""+e);var i=e.length;if(0===i)return 0;for(var r=!1;;)switch(t){case"ascii":case"latin1":case"binary":return i;case"utf8":case"utf-8":case void 0:return X(e).length;case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return 2*i;case"hex":return i>>>1;case"base64":return Q(e).length;default:if(r)return X(e).length;t=(""+t).toLowerCase(),r=!0;}}function A(e,t,i){var r=e[t];e[t]=e[i],e[i]=r;}function T(e,t,i,r,n){if(0===e.length)return -1;if("string"==typeof i?(r=i,i=0):i>2147483647?i=2147483647:i<-2147483648&&(i=-2147483648),i=+i,isNaN(i)&&(i=n?0:e.length-1),i<0&&(i=e.length+i),i>=e.length){if(n)return -1;i=e.length-1;}else if(i<0){if(!n)return -1;i=0;}if("string"==typeof t&&(t=v.from(t,r)),C(t))return 0===t.length?-1:z(e,t,i,r,n);if("number"==typeof t)return t&=255,v.TYPED_ARRAY_SUPPORT&&"function"==typeof Uint8Array.prototype.indexOf?n?Uint8Array.prototype.indexOf.call(e,t,i):Uint8Array.prototype.lastIndexOf.call(e,t,i):z(e,[t],i,r,n);throw new TypeError("val must be string, number or Buffer")}function z(e,t,i,r,n){var s,o=1,a=e.length,l=t.length;if(void 0!==r&&("ucs2"===(r=String(r).toLowerCase())||"ucs-2"===r||"utf16le"===r||"utf-16le"===r)){if(e.length<2||t.length<2)return -1;o=2,a/=2,l/=2,i/=2;}function h(e,t){return 1===o?e[t]:e.readUInt16BE(t*o)}if(n){var u=-1;for(s=i;s<a;s++)if(h(e,s)===h(t,-1===u?0:s-u)){if(-1===u&&(u=s),s-u+1===l)return u*o}else-1!==u&&(s-=s-u),u=-1;}else for(i+l>a&&(i=a-l),s=i;s>=0;s--){for(var c=!0,f=0;f<l;f++)if(h(e,s+f)!==h(t,f)){c=!1;break}if(c)return s}return -1}function O(e,t,i,r){i=Number(i)||0;var n=e.length-i;r?(r=Number(r))>n&&(r=n):r=n;var s=t.length;if(s%2!=0)throw new TypeError("Invalid hex string");r>s/2&&(r=s/2);for(var o=0;o<r;++o){var a=parseInt(t.substr(2*o,2),16);if(isNaN(a))return o;e[i+o]=a;}return o}function R(e,t,i,r){return ee(X(t,e.length-i),e,i,r)}function L(e,t,i,r){return ee(function(e){for(var t=[],i=0;i<e.length;++i)t.push(255&e.charCodeAt(i));return t}(t),e,i,r)}function B(e,t,i,r){return L(e,t,i,r)}function I(e,t,i,r){return ee(Q(t),e,i,r)}function F(e,t,i,r){return ee(function(e,t){for(var i,r,n,s=[],o=0;o<e.length&&!((t-=2)<0);++o)i=e.charCodeAt(o),r=i>>8,n=i%256,s.push(n),s.push(r);return s}(t,e.length-i),e,i,r)}function P(e,t,i){return 0===t&&i===e.length?c(e):c(e.slice(t,i))}function M(e,t,i){i=Math.min(e.length,i);for(var r=[],n=t;n<i;){var s,o,a,l,h=e[n],u=null,c=h>239?4:h>223?3:h>191?2:1;if(n+c<=i)switch(c){case 1:h<128&&(u=h);break;case 2:128==(192&(s=e[n+1]))&&(l=(31&h)<<6|63&s)>127&&(u=l);break;case 3:s=e[n+1],o=e[n+2],128==(192&s)&&128==(192&o)&&(l=(15&h)<<12|(63&s)<<6|63&o)>2047&&(l<55296||l>57343)&&(u=l);break;case 4:s=e[n+1],o=e[n+2],a=e[n+3],128==(192&s)&&128==(192&o)&&128==(192&a)&&(l=(15&h)<<18|(63&s)<<12|(63&o)<<6|63&a)>65535&&l<1114112&&(u=l);}null===u?(u=65533,c=1):u>65535&&(u-=65536,r.push(u>>>10&1023|55296),u=56320|1023&u),r.push(u),n+=c;}return function(e){var t=e.length;if(t<=U)return String.fromCharCode.apply(String,e);var i="",r=0;for(;r<t;)i+=String.fromCharCode.apply(String,e.slice(r,r+=U));return i}(r)}v.poolSize=8192,v._augment=function(e){return e.__proto__=v.prototype,e},v.from=function(e,t,i){return b(null,e,t,i)},v.TYPED_ARRAY_SUPPORT&&(v.prototype.__proto__=Uint8Array.prototype,v.__proto__=Uint8Array),v.alloc=function(e,t,i){return function(e,t,i,r){return w(t),t<=0?y(e,t):void 0!==i?"string"==typeof r?y(e,t).fill(i,r):y(e,t).fill(i):y(e,t)}(null,e,t,i)},v.allocUnsafe=function(e){return k(null,e)},v.allocUnsafeSlow=function(e){return k(null,e)},v.isBuffer=te,v.compare=function(e,t){if(!C(e)||!C(t))throw new TypeError("Arguments must be Buffers");if(e===t)return 0;for(var i=e.length,r=t.length,n=0,s=Math.min(i,r);n<s;++n)if(e[n]!==t[n]){i=e[n],r=t[n];break}return i<r?-1:r<i?1:0},v.isEncoding=function(e){switch(String(e).toLowerCase()){case"hex":case"utf8":case"utf-8":case"ascii":case"latin1":case"binary":case"base64":case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return !0;default:return !1}},v.concat=function(e,t){if(!g(e))throw new TypeError('"list" argument must be an Array of Buffers');if(0===e.length)return v.alloc(0);var i;if(void 0===t)for(t=0,i=0;i<e.length;++i)t+=e[i].length;var r=v.allocUnsafe(t),n=0;for(i=0;i<e.length;++i){var s=e[i];if(!C(s))throw new TypeError('"list" argument must be an Array of Buffers');s.copy(r,n),n+=s.length;}return r},v.byteLength=E,v.prototype._isBuffer=!0,v.prototype.swap16=function(){var e=this.length;if(e%2!=0)throw new RangeError("Buffer size must be a multiple of 16-bits");for(var t=0;t<e;t+=2)A(this,t,t+1);return this},v.prototype.swap32=function(){var e=this.length;if(e%4!=0)throw new RangeError("Buffer size must be a multiple of 32-bits");for(var t=0;t<e;t+=4)A(this,t,t+3),A(this,t+1,t+2);return this},v.prototype.swap64=function(){var e=this.length;if(e%8!=0)throw new RangeError("Buffer size must be a multiple of 64-bits");for(var t=0;t<e;t+=8)A(this,t,t+7),A(this,t+1,t+6),A(this,t+2,t+5),A(this,t+3,t+4);return this},v.prototype.toString=function(){var e=0|this.length;return 0===e?"":0===arguments.length?M(this,0,e):function(e,t,i){var r=!1;if((void 0===t||t<0)&&(t=0),t>this.length)return "";if((void 0===i||i>this.length)&&(i=this.length),i<=0)return "";if((i>>>=0)<=(t>>>=0))return "";for(e||(e="utf8");;)switch(e){case"hex":return N(this,t,i);case"utf8":case"utf-8":return M(this,t,i);case"ascii":return D(this,t,i);case"latin1":case"binary":return j(this,t,i);case"base64":return P(this,t,i);case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return Z(this,t,i);default:if(r)throw new TypeError("Unknown encoding: "+e);e=(e+"").toLowerCase(),r=!0;}}.apply(this,arguments)},v.prototype.equals=function(e){if(!C(e))throw new TypeError("Argument must be a Buffer");return this===e||0===v.compare(this,e)},v.prototype.inspect=function(){var e="";return this.length>0&&(e=this.toString("hex",0,50).match(/.{2}/g).join(" "),this.length>50&&(e+=" ... ")),"<Buffer "+e+">"},v.prototype.compare=function(e,t,i,r,n){if(!C(e))throw new TypeError("Argument must be a Buffer");if(void 0===t&&(t=0),void 0===i&&(i=e?e.length:0),void 0===r&&(r=0),void 0===n&&(n=this.length),t<0||i>e.length||r<0||n>this.length)throw new RangeError("out of range index");if(r>=n&&t>=i)return 0;if(r>=n)return -1;if(t>=i)return 1;if(this===e)return 0;for(var s=(n>>>=0)-(r>>>=0),o=(i>>>=0)-(t>>>=0),a=Math.min(s,o),l=this.slice(r,n),h=e.slice(t,i),u=0;u<a;++u)if(l[u]!==h[u]){s=l[u],o=h[u];break}return s<o?-1:o<s?1:0},v.prototype.includes=function(e,t,i){return -1!==this.indexOf(e,t,i)},v.prototype.indexOf=function(e,t,i){return T(this,e,t,i,!0)},v.prototype.lastIndexOf=function(e,t,i){return T(this,e,t,i,!1)},v.prototype.write=function(e,t,i,r){if(void 0===t)r="utf8",i=this.length,t=0;else if(void 0===i&&"string"==typeof t)r=t,i=this.length,t=0;else{if(!isFinite(t))throw new Error("Buffer.write(string, encoding, offset[, length]) is no longer supported");t|=0,isFinite(i)?(i|=0,void 0===r&&(r="utf8")):(r=i,i=void 0);}var n=this.length-t;if((void 0===i||i>n)&&(i=n),e.length>0&&(i<0||t<0)||t>this.length)throw new RangeError("Attempt to write outside buffer bounds");r||(r="utf8");for(var s=!1;;)switch(r){case"hex":return O(this,e,t,i);case"utf8":case"utf-8":return R(this,e,t,i);case"ascii":return L(this,e,t,i);case"latin1":case"binary":return B(this,e,t,i);case"base64":return I(this,e,t,i);case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return F(this,e,t,i);default:if(s)throw new TypeError("Unknown encoding: "+r);r=(""+r).toLowerCase(),s=!0;}},v.prototype.toJSON=function(){return {type:"Buffer",data:Array.prototype.slice.call(this._arr||this,0)}};var U=4096;function D(e,t,i){var r="";i=Math.min(e.length,i);for(var n=t;n<i;++n)r+=String.fromCharCode(127&e[n]);return r}function j(e,t,i){var r="";i=Math.min(e.length,i);for(var n=t;n<i;++n)r+=String.fromCharCode(e[n]);return r}function N(e,t,i){var r=e.length;(!t||t<0)&&(t=0),(!i||i<0||i>r)&&(i=r);for(var n="",s=t;s<i;++s)n+=J(e[s]);return n}function Z(e,t,i){for(var r=e.slice(t,i),n="",s=0;s<r.length;s+=2)n+=String.fromCharCode(r[s]+256*r[s+1]);return n}function W(e,t,i){if(e%1!=0||e<0)throw new RangeError("offset is not uint");if(e+t>i)throw new RangeError("Trying to access beyond buffer length")}function q(e,t,i,r,n,s){if(!C(e))throw new TypeError('"buffer" argument must be a Buffer instance');if(t>n||t<s)throw new RangeError('"value" argument is out of bounds');if(i+r>e.length)throw new RangeError("Index out of range")}function Y(e,t,i,r){t<0&&(t=65535+t+1);for(var n=0,s=Math.min(e.length-i,2);n<s;++n)e[i+n]=(t&255<<8*(r?n:1-n))>>>8*(r?n:1-n);}function V(e,t,i,r){t<0&&(t=4294967295+t+1);for(var n=0,s=Math.min(e.length-i,4);n<s;++n)e[i+n]=t>>>8*(r?n:3-n)&255;}function H(e,t,i,r,n,s){if(i+r>e.length)throw new RangeError("Index out of range");if(i<0)throw new RangeError("Index out of range")}function $(e,t,i,r,n){return n||H(e,0,i,4),d(e,t,i,r,23,4),i+4}function G(e,t,i,r,n){return n||H(e,0,i,8),d(e,t,i,r,52,8),i+8}v.prototype.slice=function(e,t){var i,r=this.length;if((e=~~e)<0?(e+=r)<0&&(e=0):e>r&&(e=r),(t=void 0===t?r:~~t)<0?(t+=r)<0&&(t=0):t>r&&(t=r),t<e&&(t=e),v.TYPED_ARRAY_SUPPORT)(i=this.subarray(e,t)).__proto__=v.prototype;else{var n=t-e;i=new v(n,void 0);for(var s=0;s<n;++s)i[s]=this[s+e];}return i},v.prototype.readUIntLE=function(e,t,i){e|=0,t|=0,i||W(e,t,this.length);for(var r=this[e],n=1,s=0;++s<t&&(n*=256);)r+=this[e+s]*n;return r},v.prototype.readUIntBE=function(e,t,i){e|=0,t|=0,i||W(e,t,this.length);for(var r=this[e+--t],n=1;t>0&&(n*=256);)r+=this[e+--t]*n;return r},v.prototype.readUInt8=function(e,t){return t||W(e,1,this.length),this[e]},v.prototype.readUInt16LE=function(e,t){return t||W(e,2,this.length),this[e]|this[e+1]<<8},v.prototype.readUInt16BE=function(e,t){return t||W(e,2,this.length),this[e]<<8|this[e+1]},v.prototype.readUInt32LE=function(e,t){return t||W(e,4,this.length),(this[e]|this[e+1]<<8|this[e+2]<<16)+16777216*this[e+3]},v.prototype.readUInt32BE=function(e,t){return t||W(e,4,this.length),16777216*this[e]+(this[e+1]<<16|this[e+2]<<8|this[e+3])},v.prototype.readIntLE=function(e,t,i){e|=0,t|=0,i||W(e,t,this.length);for(var r=this[e],n=1,s=0;++s<t&&(n*=256);)r+=this[e+s]*n;return r>=(n*=128)&&(r-=Math.pow(2,8*t)),r},v.prototype.readIntBE=function(e,t,i){e|=0,t|=0,i||W(e,t,this.length);for(var r=t,n=1,s=this[e+--r];r>0&&(n*=256);)s+=this[e+--r]*n;return s>=(n*=128)&&(s-=Math.pow(2,8*t)),s},v.prototype.readInt8=function(e,t){return t||W(e,1,this.length),128&this[e]?-1*(255-this[e]+1):this[e]},v.prototype.readInt16LE=function(e,t){t||W(e,2,this.length);var i=this[e]|this[e+1]<<8;return 32768&i?4294901760|i:i},v.prototype.readInt16BE=function(e,t){t||W(e,2,this.length);var i=this[e+1]|this[e]<<8;return 32768&i?4294901760|i:i},v.prototype.readInt32LE=function(e,t){return t||W(e,4,this.length),this[e]|this[e+1]<<8|this[e+2]<<16|this[e+3]<<24},v.prototype.readInt32BE=function(e,t){return t||W(e,4,this.length),this[e]<<24|this[e+1]<<16|this[e+2]<<8|this[e+3]},v.prototype.readFloatLE=function(e,t){return t||W(e,4,this.length),f(this,e,!0,23,4)},v.prototype.readFloatBE=function(e,t){return t||W(e,4,this.length),f(this,e,!1,23,4)},v.prototype.readDoubleLE=function(e,t){return t||W(e,8,this.length),f(this,e,!0,52,8)},v.prototype.readDoubleBE=function(e,t){return t||W(e,8,this.length),f(this,e,!1,52,8)},v.prototype.writeUIntLE=function(e,t,i,r){(e=+e,t|=0,i|=0,r)||q(this,e,t,i,Math.pow(2,8*i)-1,0);var n=1,s=0;for(this[t]=255&e;++s<i&&(n*=256);)this[t+s]=e/n&255;return t+i},v.prototype.writeUIntBE=function(e,t,i,r){(e=+e,t|=0,i|=0,r)||q(this,e,t,i,Math.pow(2,8*i)-1,0);var n=i-1,s=1;for(this[t+n]=255&e;--n>=0&&(s*=256);)this[t+n]=e/s&255;return t+i},v.prototype.writeUInt8=function(e,t,i){return e=+e,t|=0,i||q(this,e,t,1,255,0),v.TYPED_ARRAY_SUPPORT||(e=Math.floor(e)),this[t]=255&e,t+1},v.prototype.writeUInt16LE=function(e,t,i){return e=+e,t|=0,i||q(this,e,t,2,65535,0),v.TYPED_ARRAY_SUPPORT?(this[t]=255&e,this[t+1]=e>>>8):Y(this,e,t,!0),t+2},v.prototype.writeUInt16BE=function(e,t,i){return e=+e,t|=0,i||q(this,e,t,2,65535,0),v.TYPED_ARRAY_SUPPORT?(this[t]=e>>>8,this[t+1]=255&e):Y(this,e,t,!1),t+2},v.prototype.writeUInt32LE=function(e,t,i){return e=+e,t|=0,i||q(this,e,t,4,4294967295,0),v.TYPED_ARRAY_SUPPORT?(this[t+3]=e>>>24,this[t+2]=e>>>16,this[t+1]=e>>>8,this[t]=255&e):V(this,e,t,!0),t+4},v.prototype.writeUInt32BE=function(e,t,i){return e=+e,t|=0,i||q(this,e,t,4,4294967295,0),v.TYPED_ARRAY_SUPPORT?(this[t]=e>>>24,this[t+1]=e>>>16,this[t+2]=e>>>8,this[t+3]=255&e):V(this,e,t,!1),t+4},v.prototype.writeIntLE=function(e,t,i,r){if(e=+e,t|=0,!r){var n=Math.pow(2,8*i-1);q(this,e,t,i,n-1,-n);}var s=0,o=1,a=0;for(this[t]=255&e;++s<i&&(o*=256);)e<0&&0===a&&0!==this[t+s-1]&&(a=1),this[t+s]=(e/o>>0)-a&255;return t+i},v.prototype.writeIntBE=function(e,t,i,r){if(e=+e,t|=0,!r){var n=Math.pow(2,8*i-1);q(this,e,t,i,n-1,-n);}var s=i-1,o=1,a=0;for(this[t+s]=255&e;--s>=0&&(o*=256);)e<0&&0===a&&0!==this[t+s+1]&&(a=1),this[t+s]=(e/o>>0)-a&255;return t+i},v.prototype.writeInt8=function(e,t,i){return e=+e,t|=0,i||q(this,e,t,1,127,-128),v.TYPED_ARRAY_SUPPORT||(e=Math.floor(e)),e<0&&(e=255+e+1),this[t]=255&e,t+1},v.prototype.writeInt16LE=function(e,t,i){return e=+e,t|=0,i||q(this,e,t,2,32767,-32768),v.TYPED_ARRAY_SUPPORT?(this[t]=255&e,this[t+1]=e>>>8):Y(this,e,t,!0),t+2},v.prototype.writeInt16BE=function(e,t,i){return e=+e,t|=0,i||q(this,e,t,2,32767,-32768),v.TYPED_ARRAY_SUPPORT?(this[t]=e>>>8,this[t+1]=255&e):Y(this,e,t,!1),t+2},v.prototype.writeInt32LE=function(e,t,i){return e=+e,t|=0,i||q(this,e,t,4,2147483647,-2147483648),v.TYPED_ARRAY_SUPPORT?(this[t]=255&e,this[t+1]=e>>>8,this[t+2]=e>>>16,this[t+3]=e>>>24):V(this,e,t,!0),t+4},v.prototype.writeInt32BE=function(e,t,i){return e=+e,t|=0,i||q(this,e,t,4,2147483647,-2147483648),e<0&&(e=4294967295+e+1),v.TYPED_ARRAY_SUPPORT?(this[t]=e>>>24,this[t+1]=e>>>16,this[t+2]=e>>>8,this[t+3]=255&e):V(this,e,t,!1),t+4},v.prototype.writeFloatLE=function(e,t,i){return $(this,e,t,!0,i)},v.prototype.writeFloatBE=function(e,t,i){return $(this,e,t,!1,i)},v.prototype.writeDoubleLE=function(e,t,i){return G(this,e,t,!0,i)},v.prototype.writeDoubleBE=function(e,t,i){return G(this,e,t,!1,i)},v.prototype.copy=function(e,t,i,r){if(i||(i=0),r||0===r||(r=this.length),t>=e.length&&(t=e.length),t||(t=0),r>0&&r<i&&(r=i),r===i)return 0;if(0===e.length||0===this.length)return 0;if(t<0)throw new RangeError("targetStart out of bounds");if(i<0||i>=this.length)throw new RangeError("sourceStart out of bounds");if(r<0)throw new RangeError("sourceEnd out of bounds");r>this.length&&(r=this.length),e.length-t<r-i&&(r=e.length-t+i);var n,s=r-i;if(this===e&&i<t&&t<r)for(n=s-1;n>=0;--n)e[n+t]=this[n+i];else if(s<1e3||!v.TYPED_ARRAY_SUPPORT)for(n=0;n<s;++n)e[n+t]=this[n+i];else Uint8Array.prototype.set.call(e,this.subarray(i,i+s),t);return s},v.prototype.fill=function(e,t,i,r){if("string"==typeof e){if("string"==typeof t?(r=t,t=0,i=this.length):"string"==typeof i&&(r=i,i=this.length),1===e.length){var n=e.charCodeAt(0);n<256&&(e=n);}if(void 0!==r&&"string"!=typeof r)throw new TypeError("encoding must be a string");if("string"==typeof r&&!v.isEncoding(r))throw new TypeError("Unknown encoding: "+r)}else"number"==typeof e&&(e&=255);if(t<0||this.length<t||this.length<i)throw new RangeError("Out of range index");if(i<=t)return this;var s;if(t>>>=0,i=void 0===i?this.length:i>>>0,e||(e=0),"number"==typeof e)for(s=t;s<i;++s)this[s]=e;else{var o=C(e)?e:X(new v(e,r).toString()),a=o.length;for(s=0;s<i-t;++s)this[s+t]=o[s%a];}return this};var K=/[^+\/0-9A-Za-z-_]/g;function J(e){return e<16?"0"+e.toString(16):e.toString(16)}function X(e,t){var i;t=t||1/0;for(var r=e.length,n=null,s=[],o=0;o<r;++o){if((i=e.charCodeAt(o))>55295&&i<57344){if(!n){if(i>56319){(t-=3)>-1&&s.push(239,191,189);continue}if(o+1===r){(t-=3)>-1&&s.push(239,191,189);continue}n=i;continue}if(i<56320){(t-=3)>-1&&s.push(239,191,189),n=i;continue}i=65536+(n-55296<<10|i-56320);}else n&&(t-=3)>-1&&s.push(239,191,189);if(n=null,i<128){if((t-=1)<0)break;s.push(i);}else if(i<2048){if((t-=2)<0)break;s.push(i>>6|192,63&i|128);}else if(i<65536){if((t-=3)<0)break;s.push(i>>12|224,i>>6&63|128,63&i|128);}else{if(!(i<1114112))throw new Error("Invalid code point");if((t-=4)<0)break;s.push(i>>18|240,i>>12&63|128,i>>6&63|128,63&i|128);}}return s}function Q(e){return function(e){var t,i,r,n,s,u;l||h();var c=e.length;if(c%4>0)throw new Error("Invalid string. Length must be a multiple of 4");s="="===e[c-2]?2:"="===e[c-1]?1:0,u=new a(3*c/4-s),r=s>0?c-4:c;var f=0;for(t=0,i=0;t<r;t+=4,i+=3)n=o[e.charCodeAt(t)]<<18|o[e.charCodeAt(t+1)]<<12|o[e.charCodeAt(t+2)]<<6|o[e.charCodeAt(t+3)],u[f++]=n>>16&255,u[f++]=n>>8&255,u[f++]=255&n;return 2===s?(n=o[e.charCodeAt(t)]<<2|o[e.charCodeAt(t+1)]>>4,u[f++]=255&n):1===s&&(n=o[e.charCodeAt(t)]<<10|o[e.charCodeAt(t+1)]<<4|o[e.charCodeAt(t+2)]>>2,u[f++]=n>>8&255,u[f++]=255&n),u}(function(e){if((e=function(e){return e.trim?e.trim():e.replace(/^\s+|\s+$/g,"")}(e).replace(K,"")).length<2)return "";for(;e.length%4!=0;)e+="=";return e}(e))}function ee(e,t,i,r){for(var n=0;n<r&&!(n+i>=t.length||n>=e.length);++n)t[n+i]=e[n];return n}function te(e){return null!=e&&(!!e._isBuffer||ie(e)||function(e){return "function"==typeof e.readFloatLE&&"function"==typeof e.slice&&ie(e.slice(0,0))}(e))}function ie(e){return !!e.constructor&&"function"==typeof e.constructor.isBuffer&&e.constructor.isBuffer(e)}var re=Object.freeze({INSPECT_MAX_BYTES:50,kMaxLength:m,Buffer:v,SlowBuffer:function(e){return +e!=e&&(e=0),v.alloc(+e)},isBuffer:te});function ne(){}function se(){se.init.call(this);}function oe(e){return void 0===e._maxListeners?se.defaultMaxListeners:e._maxListeners}function ae(e,t,i,r){var n,s,o,a;if("function"!=typeof i)throw new TypeError('"listener" argument must be a function');if((s=e._events)?(s.newListener&&(e.emit("newListener",t,i.listener?i.listener:i),s=e._events),o=s[t]):(s=e._events=new ne,e._eventsCount=0),o){if("function"==typeof o?o=s[t]=r?[i,o]:[o,i]:r?o.unshift(i):o.push(i),!o.warned&&(n=oe(e))&&n>0&&o.length>n){o.warned=!0;var l=new Error("Possible EventEmitter memory leak detected. "+o.length+" "+t+" listeners added. Use emitter.setMaxListeners() to increase limit");l.name="MaxListenersExceededWarning",l.emitter=e,l.type=t,l.count=o.length,a=l,"function"==typeof console.warn?console.warn(a):console.log(a);}}else o=s[t]=i,++e._eventsCount;return e}function le(e,t,i){var r=!1;function n(){e.removeListener(t,n),r||(r=!0,i.apply(e,arguments));}return n.listener=i,n}function he(e){var t=this._events;if(t){var i=t[e];if("function"==typeof i)return 1;if(i)return i.length}return 0}function ue(e,t){for(var i=new Array(t);t--;)i[t]=e[t];return i}function ce(){throw new Error("setTimeout has not been defined")}function fe(){throw new Error("clearTimeout has not been defined")}ne.prototype=Object.create(null),se.EventEmitter=se,se.usingDomains=!1,se.prototype.domain=void 0,se.prototype._events=void 0,se.prototype._maxListeners=void 0,se.defaultMaxListeners=10,se.init=function(){this.domain=null,se.usingDomains&&(void 0).active&&(void 0).Domain,this._events&&this._events!==Object.getPrototypeOf(this)._events||(this._events=new ne,this._eventsCount=0),this._maxListeners=this._maxListeners||void 0;},se.prototype.setMaxListeners=function(e){if("number"!=typeof e||e<0||isNaN(e))throw new TypeError('"n" argument must be a positive number');return this._maxListeners=e,this},se.prototype.getMaxListeners=function(){return oe(this)},se.prototype.emit=function(e){var t,i,r,n,s,o,a,l="error"===e;if(o=this._events)l=l&&null==o.error;else if(!l)return !1;if(a=this.domain,l){if(t=arguments[1],!a){if(t instanceof Error)throw t;var h=new Error('Uncaught, unspecified "error" event. ('+t+")");throw h.context=t,h}return t||(t=new Error('Uncaught, unspecified "error" event')),t.domainEmitter=this,t.domain=a,t.domainThrown=!1,a.emit("error",t),!1}if(!(i=o[e]))return !1;var u="function"==typeof i;switch(r=arguments.length){case 1:!function(e,t,i){if(t)e.call(i);else for(var r=e.length,n=ue(e,r),s=0;s<r;++s)n[s].call(i);}(i,u,this);break;case 2:!function(e,t,i,r){if(t)e.call(i,r);else for(var n=e.length,s=ue(e,n),o=0;o<n;++o)s[o].call(i,r);}(i,u,this,arguments[1]);break;case 3:!function(e,t,i,r,n){if(t)e.call(i,r,n);else for(var s=e.length,o=ue(e,s),a=0;a<s;++a)o[a].call(i,r,n);}(i,u,this,arguments[1],arguments[2]);break;case 4:!function(e,t,i,r,n,s){if(t)e.call(i,r,n,s);else for(var o=e.length,a=ue(e,o),l=0;l<o;++l)a[l].call(i,r,n,s);}(i,u,this,arguments[1],arguments[2],arguments[3]);break;default:for(n=new Array(r-1),s=1;s<r;s++)n[s-1]=arguments[s];!function(e,t,i,r){if(t)e.apply(i,r);else for(var n=e.length,s=ue(e,n),o=0;o<n;++o)s[o].apply(i,r);}(i,u,this,n);}return !0},se.prototype.addListener=function(e,t){return ae(this,e,t,!1)},se.prototype.on=se.prototype.addListener,se.prototype.prependListener=function(e,t){return ae(this,e,t,!0)},se.prototype.once=function(e,t){if("function"!=typeof t)throw new TypeError('"listener" argument must be a function');return this.on(e,le(this,e,t)),this},se.prototype.prependOnceListener=function(e,t){if("function"!=typeof t)throw new TypeError('"listener" argument must be a function');return this.prependListener(e,le(this,e,t)),this},se.prototype.removeListener=function(e,t){var i,r,n,s,o;if("function"!=typeof t)throw new TypeError('"listener" argument must be a function');if(!(r=this._events))return this;if(!(i=r[e]))return this;if(i===t||i.listener&&i.listener===t)0==--this._eventsCount?this._events=new ne:(delete r[e],r.removeListener&&this.emit("removeListener",e,i.listener||t));else if("function"!=typeof i){for(n=-1,s=i.length;s-- >0;)if(i[s]===t||i[s].listener&&i[s].listener===t){o=i[s].listener,n=s;break}if(n<0)return this;if(1===i.length){if(i[0]=void 0,0==--this._eventsCount)return this._events=new ne,this;delete r[e];}else!function(e,t){for(var i=t,r=i+1,n=e.length;r<n;i+=1,r+=1)e[i]=e[r];e.pop();}(i,n);r.removeListener&&this.emit("removeListener",e,o||t);}return this},se.prototype.removeAllListeners=function(e){var t,i;if(!(i=this._events))return this;if(!i.removeListener)return 0===arguments.length?(this._events=new ne,this._eventsCount=0):i[e]&&(0==--this._eventsCount?this._events=new ne:delete i[e]),this;if(0===arguments.length){for(var r,n=Object.keys(i),s=0;s<n.length;++s)"removeListener"!==(r=n[s])&&this.removeAllListeners(r);return this.removeAllListeners("removeListener"),this._events=new ne,this._eventsCount=0,this}if("function"==typeof(t=i[e]))this.removeListener(e,t);else if(t)do{this.removeListener(e,t[t.length-1]);}while(t[0]);return this},se.prototype.listeners=function(e){var t,i=this._events;return i&&(t=i[e])?"function"==typeof t?[t.listener||t]:function(e){for(var t=new Array(e.length),i=0;i<t.length;++i)t[i]=e[i].listener||e[i];return t}(t):[]},se.listenerCount=function(e,t){return "function"==typeof e.listenerCount?e.listenerCount(t):he.call(e,t)},se.prototype.listenerCount=he,se.prototype.eventNames=function(){return this._eventsCount>0?Reflect.ownKeys(this._events):[]};var de=ce,pe=fe;function ge(e){if(de===setTimeout)return setTimeout(e,0);if((de===ce||!de)&&setTimeout)return de=setTimeout,setTimeout(e,0);try{return de(e,0)}catch(t){try{return de.call(null,e,0)}catch(t){return de.call(this,e,0)}}}"function"==typeof n.setTimeout&&(de=setTimeout),"function"==typeof n.clearTimeout&&(pe=clearTimeout);var me,_e=[],ye=!1,ve=-1;function be(){ye&&me&&(ye=!1,me.length?_e=me.concat(_e):ve=-1,_e.length&&we());}function we(){if(!ye){var e=ge(be);ye=!0;for(var t=_e.length;t;){for(me=_e,_e=[];++ve<t;)me&&me[ve].run();ve=-1,t=_e.length;}me=null,ye=!1,function(e){if(pe===clearTimeout)return clearTimeout(e);if((pe===fe||!pe)&&clearTimeout)return pe=clearTimeout,clearTimeout(e);try{pe(e);}catch(t){try{return pe.call(null,e)}catch(t){return pe.call(this,e)}}}(e);}}function ke(e){var t=new Array(arguments.length-1);if(arguments.length>1)for(var i=1;i<arguments.length;i++)t[i-1]=arguments[i];_e.push(new Se(e,t)),1!==_e.length||ye||ge(we);}function Se(e,t){this.fun=e,this.array=t;}Se.prototype.run=function(){this.fun.apply(null,this.array);};function xe(){}var Ce=xe,Ee=xe,Ae=xe,Te=xe,ze=xe,Oe=xe,Re=xe;var Le=n.performance||{},Be=Le.now||Le.mozNow||Le.msNow||Le.oNow||Le.webkitNow||function(){return (new Date).getTime()};var Ie=new Date;var Fe={nextTick:ke,title:"browser",browser:!0,env:{},argv:[],version:"",versions:{},on:Ce,addListener:Ee,once:Ae,off:Te,removeListener:ze,removeAllListeners:Oe,emit:Re,binding:function(e){throw new Error("process.binding is not supported")},cwd:function(){return "/"},chdir:function(e){throw new Error("process.chdir is not supported")},umask:function(){return 0},hrtime:function(e){var t=.001*Be.call(Le),i=Math.floor(t),r=Math.floor(t%1*1e9);return e&&(i-=e[0],(r-=e[1])<0&&(i--,r+=1e9)),[i,r]},platform:"browser",release:{},config:{},uptime:function(){return (new Date-Ie)/1e3}},Pe=function(e){return e&&"object"==typeof e&&"function"==typeof e.copy&&"function"==typeof e.fill&&"function"==typeof e.readUInt8},Me=e.createCommonjsModule(function(e){"function"==typeof Object.create?e.exports=function(e,t){e.super_=t,e.prototype=Object.create(t.prototype,{constructor:{value:e,enumerable:!1,writable:!0,configurable:!0}});}:e.exports=function(e,t){e.super_=t;var i=function(){};i.prototype=t.prototype,e.prototype=new i,e.prototype.constructor=e;};}),Ue=e.createCommonjsModule(function(e,t){var i=/%[sdj%]/g;t.format=function(e){if(!m(e)){for(var t=[],r=0;r<arguments.length;r++)t.push(o(arguments[r]));return t.join(" ")}r=1;for(var n=arguments,s=n.length,a=String(e).replace(i,function(e){if("%%"===e)return "%";if(r>=s)return e;switch(e){case"%s":return String(n[r++]);case"%d":return Number(n[r++]);case"%j":try{return JSON.stringify(n[r++])}catch(e){return "[Circular]"}default:return e}}),l=n[r];r<s;l=n[++r])p(l)||!v(l)?a+=" "+l:a+=" "+o(l);return a},t.deprecate=function(e,i){if(_(n.process))return function(){return t.deprecate(e,i).apply(this,arguments)};var r=!1;return function(){return r||(console.error(i),r=!0),e.apply(this,arguments)}};var r,s={};function o(e,i){var r={seen:[],stylize:l};return arguments.length>=3&&(r.depth=arguments[2]),arguments.length>=4&&(r.colors=arguments[3]),d(i)?r.showHidden=i:i&&t._extend(r,i),_(r.showHidden)&&(r.showHidden=!1),_(r.depth)&&(r.depth=2),_(r.colors)&&(r.colors=!1),_(r.customInspect)&&(r.customInspect=!0),r.colors&&(r.stylize=a),h(r,e,r.depth)}function a(e,t){var i=o.styles[t];return i?"["+o.colors[i][0]+"m"+e+"["+o.colors[i][1]+"m":e}function l(e,t){return e}function h(e,i,r){if(e.customInspect&&i&&k(i.inspect)&&i.inspect!==t.inspect&&(!i.constructor||i.constructor.prototype!==i)){var n=i.inspect(r,e);return m(n)||(n=h(e,n,r)),n}var s=function(e,t){if(_(t))return e.stylize("undefined","undefined");if(m(t)){var i="'"+JSON.stringify(t).replace(/^"|"$/g,"").replace(/'/g,"\\'").replace(/\\"/g,'"')+"'";return e.stylize(i,"string")}if(g(t))return e.stylize(""+t,"number");if(d(t))return e.stylize(""+t,"boolean");if(p(t))return e.stylize("null","null")}(e,i);if(s)return s;var o=Object.keys(i),a=function(e){var t={};return e.forEach(function(e,i){t[e]=!0;}),t}(o);if(e.showHidden&&(o=Object.getOwnPropertyNames(i)),w(i)&&(o.indexOf("message")>=0||o.indexOf("description")>=0))return u(i);if(0===o.length){if(k(i)){var l=i.name?": "+i.name:"";return e.stylize("[Function"+l+"]","special")}if(y(i))return e.stylize(RegExp.prototype.toString.call(i),"regexp");if(b(i))return e.stylize(Date.prototype.toString.call(i),"date");if(w(i))return u(i)}var v,S="",x=!1,C=["{","}"];(f(i)&&(x=!0,C=["[","]"]),k(i))&&(S=" [Function"+(i.name?": "+i.name:"")+"]");return y(i)&&(S=" "+RegExp.prototype.toString.call(i)),b(i)&&(S=" "+Date.prototype.toUTCString.call(i)),w(i)&&(S=" "+u(i)),0!==o.length||x&&0!=i.length?r<0?y(i)?e.stylize(RegExp.prototype.toString.call(i),"regexp"):e.stylize("[Object]","special"):(e.seen.push(i),v=x?function(e,t,i,r,n){for(var s=[],o=0,a=t.length;o<a;++o)E(t,String(o))?s.push(c(e,t,i,r,String(o),!0)):s.push("");return n.forEach(function(n){n.match(/^\d+$/)||s.push(c(e,t,i,r,n,!0));}),s}(e,i,r,a,o):o.map(function(t){return c(e,i,r,a,t,x)}),e.seen.pop(),function(e,t,i){if(e.reduce(function(e,t){return t.indexOf("\n"),e+t.replace(/\u001b\[\d\d?m/g,"").length+1},0)>60)return i[0]+(""===t?"":t+"\n ")+" "+e.join(",\n  ")+" "+i[1];return i[0]+t+" "+e.join(", ")+" "+i[1]}(v,S,C)):C[0]+S+C[1]}function u(e){return "["+Error.prototype.toString.call(e)+"]"}function c(e,t,i,r,n,s){var o,a,l;if((l=Object.getOwnPropertyDescriptor(t,n)||{value:t[n]}).get?a=l.set?e.stylize("[Getter/Setter]","special"):e.stylize("[Getter]","special"):l.set&&(a=e.stylize("[Setter]","special")),E(r,n)||(o="["+n+"]"),a||(e.seen.indexOf(l.value)<0?(a=p(i)?h(e,l.value,null):h(e,l.value,i-1)).indexOf("\n")>-1&&(a=s?a.split("\n").map(function(e){return "  "+e}).join("\n").substr(2):"\n"+a.split("\n").map(function(e){return "   "+e}).join("\n")):a=e.stylize("[Circular]","special")),_(o)){if(s&&n.match(/^\d+$/))return a;(o=JSON.stringify(""+n)).match(/^"([a-zA-Z_][a-zA-Z_0-9]*)"$/)?(o=o.substr(1,o.length-2),o=e.stylize(o,"name")):(o=o.replace(/'/g,"\\'").replace(/\\"/g,'"').replace(/(^"|"$)/g,"'"),o=e.stylize(o,"string"));}return o+": "+a}function f(e){return Array.isArray(e)}function d(e){return "boolean"==typeof e}function p(e){return null===e}function g(e){return "number"==typeof e}function m(e){return "string"==typeof e}function _(e){return void 0===e}function y(e){return v(e)&&"[object RegExp]"===S(e)}function v(e){return "object"==typeof e&&null!==e}function b(e){return v(e)&&"[object Date]"===S(e)}function w(e){return v(e)&&("[object Error]"===S(e)||e instanceof Error)}function k(e){return "function"==typeof e}function S(e){return Object.prototype.toString.call(e)}function x(e){return e<10?"0"+e.toString(10):e.toString(10)}t.debuglog=function(e){if(_(r)&&(r=Fe.env.NODE_DEBUG||""),e=e.toUpperCase(),!s[e])if(new RegExp("\\b"+e+"\\b","i").test(r)){var i=Fe.pid;s[e]=function(){var r=t.format.apply(t,arguments);console.error("%s %d: %s",e,i,r);};}else s[e]=function(){};return s[e]},t.inspect=o,o.colors={bold:[1,22],italic:[3,23],underline:[4,24],inverse:[7,27],white:[37,39],grey:[90,39],black:[30,39],blue:[34,39],cyan:[36,39],green:[32,39],magenta:[35,39],red:[31,39],yellow:[33,39]},o.styles={special:"cyan",number:"yellow",boolean:"yellow",undefined:"grey",null:"bold",string:"green",date:"magenta",regexp:"red"},t.isArray=f,t.isBoolean=d,t.isNull=p,t.isNullOrUndefined=function(e){return null==e},t.isNumber=g,t.isString=m,t.isSymbol=function(e){return "symbol"==typeof e},t.isUndefined=_,t.isRegExp=y,t.isObject=v,t.isDate=b,t.isError=w,t.isFunction=k,t.isPrimitive=function(e){return null===e||"boolean"==typeof e||"number"==typeof e||"string"==typeof e||"symbol"==typeof e||void 0===e},t.isBuffer=Pe;var C=["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"];function E(e,t){return Object.prototype.hasOwnProperty.call(e,t)}t.log=function(){var e,i;console.log("%s - %s",(e=new Date,i=[x(e.getHours()),x(e.getMinutes()),x(e.getSeconds())].join(":"),[e.getDate(),C[e.getMonth()],i].join(" ")),t.format.apply(t,arguments));},t.inherits=Me,t._extend=function(e,t){if(!t||!v(t))return e;for(var i=Object.keys(t),r=i.length;r--;)e[i[r]]=t[i[r]];return e};}),De=(Ue.format,Ue.deprecate),je=Ue.debuglog,Ne=(Ue.inspect,Ue.isArray,Ue.isBoolean,Ue.isNull,Ue.isNullOrUndefined,Ue.isNumber,Ue.isString,Ue.isSymbol,Ue.isUndefined,Ue.isRegExp,Ue.isObject,Ue.isDate,Ue.isError,Ue.isFunction,Ue.isPrimitive,Ue.isBuffer,Ue.log,Ue.inherits);Ue._extend;function Ze(){this.head=null,this.tail=null,this.length=0;}Ze.prototype.push=function(e){var t={data:e,next:null};this.length>0?this.tail.next=t:this.head=t,this.tail=t,++this.length;},Ze.prototype.unshift=function(e){var t={data:e,next:this.head};0===this.length&&(this.tail=t),this.head=t,++this.length;},Ze.prototype.shift=function(){if(0!==this.length){var e=this.head.data;return 1===this.length?this.head=this.tail=null:this.head=this.head.next,--this.length,e}},Ze.prototype.clear=function(){this.head=this.tail=null,this.length=0;},Ze.prototype.join=function(e){if(0===this.length)return "";for(var t=this.head,i=""+t.data;t=t.next;)i+=e+t.data;return i},Ze.prototype.concat=function(e){if(0===this.length)return v.alloc(0);if(1===this.length)return this.head.data;for(var t=v.allocUnsafe(e>>>0),i=this.head,r=0;i;)i.data.copy(t,r),r+=i.data.length,i=i.next;return t};var We=e.createCommonjsModule(function(e,t){var i=re.Buffer;function r(e,t){for(var i in e)t[i]=e[i];}function n(e,t,r){return i(e,t,r)}i.from&&i.alloc&&i.allocUnsafe&&i.allocUnsafeSlow?e.exports=re:(r(re,t),t.Buffer=n),r(i,n),n.from=function(e,t,r){if("number"==typeof e)throw new TypeError("Argument must not be a number");return i(e,t,r)},n.alloc=function(e,t,r){if("number"!=typeof e)throw new TypeError("Argument must be a number");var n=i(e);return void 0!==t?"string"==typeof r?n.fill(t,r):n.fill(t):n.fill(0),n},n.allocUnsafe=function(e){if("number"!=typeof e)throw new TypeError("Argument must be a number");return i(e)},n.allocUnsafeSlow=function(e){if("number"!=typeof e)throw new TypeError("Argument must be a number");return re.SlowBuffer(e)};}),qe=(We.Buffer,We.Buffer),Ye=qe.isEncoding||function(e){switch((e=""+e)&&e.toLowerCase()){case"hex":case"utf8":case"utf-8":case"ascii":case"binary":case"base64":case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":case"raw":return !0;default:return !1}};var Ve=He;function He(e){var t;switch(this.encoding=function(e){var t=function(e){if(!e)return "utf8";for(var t;;)switch(e){case"utf8":case"utf-8":return "utf8";case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return "utf16le";case"latin1":case"binary":return "latin1";case"base64":case"ascii":case"hex":return e;default:if(t)return;e=(""+e).toLowerCase(),t=!0;}}(e);if("string"!=typeof t&&(qe.isEncoding===Ye||!Ye(e)))throw new Error("Unknown encoding: "+e);return t||e}(e),this.encoding){case"utf16le":this.text=Ke,this.end=Je,t=4;break;case"utf8":this.fillLast=Ge,t=4;break;case"base64":this.text=Xe,this.end=Qe,t=3;break;default:return this.write=et,void(this.end=tt)}this.lastNeed=0,this.lastTotal=0,this.lastChar=qe.allocUnsafe(t);}function $e(e){return e<=127?0:e>>5==6?2:e>>4==14?3:e>>3==30?4:-1}function Ge(e){var t=this.lastTotal-this.lastNeed,i=function(e,t,i){if(128!=(192&t[0]))return e.lastNeed=0,"�".repeat(i);if(e.lastNeed>1&&t.length>1){if(128!=(192&t[1]))return e.lastNeed=1,"�".repeat(i+1);if(e.lastNeed>2&&t.length>2&&128!=(192&t[2]))return e.lastNeed=2,"�".repeat(i+2)}}(this,e,t);return void 0!==i?i:this.lastNeed<=e.length?(e.copy(this.lastChar,t,0,this.lastNeed),this.lastChar.toString(this.encoding,0,this.lastTotal)):(e.copy(this.lastChar,t,0,e.length),void(this.lastNeed-=e.length))}function Ke(e,t){if((e.length-t)%2==0){var i=e.toString("utf16le",t);if(i){var r=i.charCodeAt(i.length-1);if(r>=55296&&r<=56319)return this.lastNeed=2,this.lastTotal=4,this.lastChar[0]=e[e.length-2],this.lastChar[1]=e[e.length-1],i.slice(0,-1)}return i}return this.lastNeed=1,this.lastTotal=2,this.lastChar[0]=e[e.length-1],e.toString("utf16le",t,e.length-1)}function Je(e){var t=e&&e.length?this.write(e):"";if(this.lastNeed){var i=this.lastTotal-this.lastNeed;return t+this.lastChar.toString("utf16le",0,i)}return t}function Xe(e,t){var i=(e.length-t)%3;return 0===i?e.toString("base64",t):(this.lastNeed=3-i,this.lastTotal=3,1===i?this.lastChar[0]=e[e.length-1]:(this.lastChar[0]=e[e.length-2],this.lastChar[1]=e[e.length-1]),e.toString("base64",t,e.length-i))}function Qe(e){var t=e&&e.length?this.write(e):"";return this.lastNeed?t+this.lastChar.toString("base64",0,3-this.lastNeed):t}function et(e){return e.toString(this.encoding)}function tt(e){return e&&e.length?this.write(e):""}He.prototype.write=function(e){if(0===e.length)return "";var t,i;if(this.lastNeed){if(void 0===(t=this.fillLast(e)))return "";i=this.lastNeed,this.lastNeed=0;}else i=0;return i<e.length?t?t+this.text(e,i):this.text(e,i):t||""},He.prototype.end=function(e){var t=e&&e.length?this.write(e):"";return this.lastNeed?t+"�".repeat(this.lastTotal-this.lastNeed):t},He.prototype.text=function(e,t){var i=function(e,t,i){var r=t.length-1;if(r<i)return 0;var n=$e(t[r]);if(n>=0)return n>0&&(e.lastNeed=n-1),n;if(--r<i)return 0;if((n=$e(t[r]))>=0)return n>0&&(e.lastNeed=n-2),n;if(--r<i)return 0;if((n=$e(t[r]))>=0)return n>0&&(2===n?n=0:e.lastNeed=n-3),n;return 0}(this,e,t);if(!this.lastNeed)return e.toString("utf8",t);this.lastTotal=i;var r=e.length-(i-this.lastNeed);return e.copy(this.lastChar,0,r),e.toString("utf8",t,r)},He.prototype.fillLast=function(e){if(this.lastNeed<=e.length)return e.copy(this.lastChar,this.lastTotal-this.lastNeed,0,this.lastNeed),this.lastChar.toString(this.encoding,0,this.lastTotal);e.copy(this.lastChar,this.lastTotal-this.lastNeed,0,e.length),this.lastNeed-=e.length;},nt.ReadableState=rt;var it=je("stream");function rt(e,t){e=e||{},this.objectMode=!!e.objectMode,t instanceof Lt&&(this.objectMode=this.objectMode||!!e.readableObjectMode);var i=e.highWaterMark,r=this.objectMode?16:16384;this.highWaterMark=i||0===i?i:r,this.highWaterMark=~~this.highWaterMark,this.buffer=new Ze,this.length=0,this.pipes=null,this.pipesCount=0,this.flowing=null,this.ended=!1,this.endEmitted=!1,this.reading=!1,this.sync=!0,this.needReadable=!1,this.emittedReadable=!1,this.readableListening=!1,this.resumeScheduled=!1,this.defaultEncoding=e.defaultEncoding||"utf8",this.ranOut=!1,this.awaitDrain=0,this.readingMore=!1,this.decoder=null,this.encoding=null,e.encoding&&(this.decoder=new Ve(e.encoding),this.encoding=e.encoding);}function nt(e){if(!(this instanceof nt))return new nt(e);this._readableState=new rt(e,this),this.readable=!0,e&&"function"==typeof e.read&&(this._read=e.read),se.call(this);}function st(e,t,i,r,n){var s=function(e,t){var i=null;te(t)||"string"==typeof t||null==t||e.objectMode||(i=new TypeError("Invalid non-string/buffer chunk"));return i}(t,i);if(s)e.emit("error",s);else if(null===i)t.reading=!1,function(e,t){if(t.ended)return;if(t.decoder){var i=t.decoder.end();i&&i.length&&(t.buffer.push(i),t.length+=t.objectMode?1:i.length);}t.ended=!0,lt(e);}(e,t);else if(t.objectMode||i&&i.length>0)if(t.ended&&!n){var o=new Error("stream.push() after EOF");e.emit("error",o);}else if(t.endEmitted&&n){var a=new Error("stream.unshift() after end event");e.emit("error",a);}else{var l;!t.decoder||n||r||(i=t.decoder.write(i),l=!t.objectMode&&0===i.length),n||(t.reading=!1),l||(t.flowing&&0===t.length&&!t.sync?(e.emit("data",i),e.read(0)):(t.length+=t.objectMode?1:i.length,n?t.buffer.unshift(i):t.buffer.push(i),t.needReadable&&lt(e))),function(e,t){t.readingMore||(t.readingMore=!0,ke(ut,e,t));}(e,t);}else n||(t.reading=!1);return function(e){return !e.ended&&(e.needReadable||e.length<e.highWaterMark||0===e.length)}(t)}Ne(nt,se),nt.prototype.push=function(e,t){var i=this._readableState;return i.objectMode||"string"!=typeof e||(t=t||i.defaultEncoding)!==i.encoding&&(e=v.from(e,t),t=""),st(this,i,e,t,!1)},nt.prototype.unshift=function(e){return st(this,this._readableState,e,"",!0)},nt.prototype.isPaused=function(){return !1===this._readableState.flowing},nt.prototype.setEncoding=function(e){return this._readableState.decoder=new Ve(e),this._readableState.encoding=e,this};var ot=8388608;function at(e,t){return e<=0||0===t.length&&t.ended?0:t.objectMode?1:e!=e?t.flowing&&t.length?t.buffer.head.data.length:t.length:(e>t.highWaterMark&&(t.highWaterMark=function(e){return e>=ot?e=ot:(e--,e|=e>>>1,e|=e>>>2,e|=e>>>4,e|=e>>>8,e|=e>>>16,e++),e}(e)),e<=t.length?e:t.ended?t.length:(t.needReadable=!0,0))}function lt(e){var t=e._readableState;t.needReadable=!1,t.emittedReadable||(it("emitReadable",t.flowing),t.emittedReadable=!0,t.sync?ke(ht,e):ht(e));}function ht(e){it("emit readable"),e.emit("readable"),dt(e);}function ut(e,t){for(var i=t.length;!t.reading&&!t.flowing&&!t.ended&&t.length<t.highWaterMark&&(it("maybeReadMore read 0"),e.read(0),i!==t.length);)i=t.length;t.readingMore=!1;}function ct(e){it("readable nexttick read 0"),e.read(0);}function ft(e,t){t.reading||(it("resume read 0"),e.read(0)),t.resumeScheduled=!1,t.awaitDrain=0,e.emit("resume"),dt(e),t.flowing&&!t.reading&&e.read(0);}function dt(e){var t=e._readableState;for(it("flow",t.flowing);t.flowing&&null!==e.read(););}function pt(e,t){return 0===t.length?null:(t.objectMode?i=t.buffer.shift():!e||e>=t.length?(i=t.decoder?t.buffer.join(""):1===t.buffer.length?t.buffer.head.data:t.buffer.concat(t.length),t.buffer.clear()):i=function(e,t,i){var r;e<t.head.data.length?(r=t.head.data.slice(0,e),t.head.data=t.head.data.slice(e)):r=e===t.head.data.length?t.shift():i?function(e,t){var i=t.head,r=1,n=i.data;e-=n.length;for(;i=i.next;){var s=i.data,o=e>s.length?s.length:e;if(o===s.length?n+=s:n+=s.slice(0,e),0===(e-=o)){o===s.length?(++r,i.next?t.head=i.next:t.head=t.tail=null):(t.head=i,i.data=s.slice(o));break}++r;}return t.length-=r,n}(e,t):function(e,t){var i=v.allocUnsafe(e),r=t.head,n=1;r.data.copy(i),e-=r.data.length;for(;r=r.next;){var s=r.data,o=e>s.length?s.length:e;if(s.copy(i,i.length-e,0,o),0===(e-=o)){o===s.length?(++n,r.next?t.head=r.next:t.head=t.tail=null):(t.head=r,r.data=s.slice(o));break}++n;}return t.length-=n,i}(e,t);return r}(e,t.buffer,t.decoder),i);var i;}function gt(e){var t=e._readableState;if(t.length>0)throw new Error('"endReadable()" called on non-empty stream');t.endEmitted||(t.ended=!0,ke(mt,t,e));}function mt(e,t){e.endEmitted||0!==e.length||(e.endEmitted=!0,t.readable=!1,t.emit("end"));}function _t(e,t){for(var i=0,r=e.length;i<r;i++)if(e[i]===t)return i;return -1}function yt(){}function vt(e,t,i){this.chunk=e,this.encoding=t,this.callback=i,this.next=null;}function bt(e,t){Object.defineProperty(this,"buffer",{get:De(function(){return this.getBuffer()},"_writableState.buffer is deprecated. Use _writableState.getBuffer instead.")}),e=e||{},this.objectMode=!!e.objectMode,t instanceof Lt&&(this.objectMode=this.objectMode||!!e.writableObjectMode);var i=e.highWaterMark,r=this.objectMode?16:16384;this.highWaterMark=i||0===i?i:r,this.highWaterMark=~~this.highWaterMark,this.needDrain=!1,this.ending=!1,this.ended=!1,this.finished=!1;var n=!1===e.decodeStrings;this.decodeStrings=!n,this.defaultEncoding=e.defaultEncoding||"utf8",this.length=0,this.writing=!1,this.corked=0,this.sync=!0,this.bufferProcessing=!1,this.onwrite=function(e){!function(e,t){var i=e._writableState,r=i.sync,n=i.writecb;if(function(e){e.writing=!1,e.writecb=null,e.length-=e.writelen,e.writelen=0;}(i),t)!function(e,t,i,r,n){--t.pendingcb,i?ke(n,r):n(r);e._writableState.errorEmitted=!0,e.emit("error",r);}(e,i,r,t,n);else{var s=Ct(i);s||i.corked||i.bufferProcessing||!i.bufferedRequest||xt(e,i),r?ke(St,e,i,s,n):St(e,i,s,n);}}(t,e);},this.writecb=null,this.writelen=0,this.bufferedRequest=null,this.lastBufferedRequest=null,this.pendingcb=0,this.prefinished=!1,this.errorEmitted=!1,this.bufferedRequestCount=0,this.corkedRequestsFree=new Tt(this);}function wt(e){if(!(this instanceof wt||this instanceof Lt))return new wt(e);this._writableState=new bt(e,this),this.writable=!0,e&&("function"==typeof e.write&&(this._write=e.write),"function"==typeof e.writev&&(this._writev=e.writev)),se.call(this);}function kt(e,t,i,r,n,s,o){t.writelen=r,t.writecb=o,t.writing=!0,t.sync=!0,i?e._writev(n,t.onwrite):e._write(n,s,t.onwrite),t.sync=!1;}function St(e,t,i,r){i||function(e,t){0===t.length&&t.needDrain&&(t.needDrain=!1,e.emit("drain"));}(e,t),t.pendingcb--,r(),At(e,t);}function xt(e,t){t.bufferProcessing=!0;var i=t.bufferedRequest;if(e._writev&&i&&i.next){var r=t.bufferedRequestCount,n=new Array(r),s=t.corkedRequestsFree;s.entry=i;for(var o=0;i;)n[o]=i,i=i.next,o+=1;kt(e,t,!0,t.length,n,"",s.finish),t.pendingcb++,t.lastBufferedRequest=null,s.next?(t.corkedRequestsFree=s.next,s.next=null):t.corkedRequestsFree=new Tt(t);}else{for(;i;){var a=i.chunk,l=i.encoding,h=i.callback;if(kt(e,t,!1,t.objectMode?1:a.length,a,l,h),i=i.next,t.writing)break}null===i&&(t.lastBufferedRequest=null);}t.bufferedRequestCount=0,t.bufferedRequest=i,t.bufferProcessing=!1;}function Ct(e){return e.ending&&0===e.length&&null===e.bufferedRequest&&!e.finished&&!e.writing}function Et(e,t){t.prefinished||(t.prefinished=!0,e.emit("prefinish"));}function At(e,t){var i=Ct(t);return i&&(0===t.pendingcb?(Et(e,t),t.finished=!0,e.emit("finish")):Et(e,t)),i}function Tt(e){var t=this;this.next=null,this.entry=null,this.finish=function(i){var r=t.entry;for(t.entry=null;r;){var n=r.callback;e.pendingcb--,n(i),r=r.next;}e.corkedRequestsFree?e.corkedRequestsFree.next=t:e.corkedRequestsFree=t;};}nt.prototype.read=function(e){it("read",e),e=parseInt(e,10);var t=this._readableState,i=e;if(0!==e&&(t.emittedReadable=!1),0===e&&t.needReadable&&(t.length>=t.highWaterMark||t.ended))return it("read: emitReadable",t.length,t.ended),0===t.length&&t.ended?gt(this):lt(this),null;if(0===(e=at(e,t))&&t.ended)return 0===t.length&&gt(this),null;var r,n=t.needReadable;return it("need readable",n),(0===t.length||t.length-e<t.highWaterMark)&&it("length less than watermark",n=!0),t.ended||t.reading?it("reading or ended",n=!1):n&&(it("do read"),t.reading=!0,t.sync=!0,0===t.length&&(t.needReadable=!0),this._read(t.highWaterMark),t.sync=!1,t.reading||(e=at(i,t))),null===(r=e>0?pt(e,t):null)?(t.needReadable=!0,e=0):t.length-=e,0===t.length&&(t.ended||(t.needReadable=!0),i!==e&&t.ended&&gt(this)),null!==r&&this.emit("data",r),r},nt.prototype._read=function(e){this.emit("error",new Error("not implemented"));},nt.prototype.pipe=function(e,t){var i=this,r=this._readableState;switch(r.pipesCount){case 0:r.pipes=e;break;case 1:r.pipes=[r.pipes,e];break;default:r.pipes.push(e);}r.pipesCount+=1,it("pipe count=%d opts=%j",r.pipesCount,t);var n=!t||!1!==t.end?o:h;function s(e){it("onunpipe"),e===i&&h();}function o(){it("onend"),e.end();}r.endEmitted?ke(n):i.once("end",n),e.on("unpipe",s);var a=function(e){return function(){var t=e._readableState;it("pipeOnDrain",t.awaitDrain),t.awaitDrain&&t.awaitDrain--,0===t.awaitDrain&&e.listeners("data").length&&(t.flowing=!0,dt(e));}}(i);e.on("drain",a);var l=!1;function h(){it("cleanup"),e.removeListener("close",d),e.removeListener("finish",p),e.removeListener("drain",a),e.removeListener("error",f),e.removeListener("unpipe",s),i.removeListener("end",o),i.removeListener("end",h),i.removeListener("data",c),l=!0,!r.awaitDrain||e._writableState&&!e._writableState.needDrain||a();}var u=!1;function c(t){it("ondata"),u=!1,!1!==e.write(t)||u||((1===r.pipesCount&&r.pipes===e||r.pipesCount>1&&-1!==_t(r.pipes,e))&&!l&&(it("false write response, pause",i._readableState.awaitDrain),i._readableState.awaitDrain++,u=!0),i.pause());}function f(t){var i;it("onerror",t),g(),e.removeListener("error",f),0===(i="error",e.listeners(i).length)&&e.emit("error",t);}function d(){e.removeListener("finish",p),g();}function p(){it("onfinish"),e.removeListener("close",d),g();}function g(){it("unpipe"),i.unpipe(e);}return i.on("data",c),function(e,t,i){if("function"==typeof e.prependListener)return e.prependListener(t,i);e._events&&e._events[t]?Array.isArray(e._events[t])?e._events[t].unshift(i):e._events[t]=[i,e._events[t]]:e.on(t,i);}(e,"error",f),e.once("close",d),e.once("finish",p),e.emit("pipe",i),r.flowing||(it("pipe resume"),i.resume()),e},nt.prototype.unpipe=function(e){var t=this._readableState;if(0===t.pipesCount)return this;if(1===t.pipesCount)return e&&e!==t.pipes?this:(e||(e=t.pipes),t.pipes=null,t.pipesCount=0,t.flowing=!1,e&&e.emit("unpipe",this),this);if(!e){var i=t.pipes,r=t.pipesCount;t.pipes=null,t.pipesCount=0,t.flowing=!1;for(var n=0;n<r;n++)i[n].emit("unpipe",this);return this}var s=_t(t.pipes,e);return -1===s?this:(t.pipes.splice(s,1),t.pipesCount-=1,1===t.pipesCount&&(t.pipes=t.pipes[0]),e.emit("unpipe",this),this)},nt.prototype.on=function(e,t){var i=se.prototype.on.call(this,e,t);if("data"===e)!1!==this._readableState.flowing&&this.resume();else if("readable"===e){var r=this._readableState;r.endEmitted||r.readableListening||(r.readableListening=r.needReadable=!0,r.emittedReadable=!1,r.reading?r.length&&lt(this):ke(ct,this));}return i},nt.prototype.addListener=nt.prototype.on,nt.prototype.resume=function(){var e=this._readableState;return e.flowing||(it("resume"),e.flowing=!0,function(e,t){t.resumeScheduled||(t.resumeScheduled=!0,ke(ft,e,t));}(this,e)),this},nt.prototype.pause=function(){return it("call pause flowing=%j",this._readableState.flowing),!1!==this._readableState.flowing&&(it("pause"),this._readableState.flowing=!1,this.emit("pause")),this},nt.prototype.wrap=function(e){var t=this._readableState,i=!1,r=this;for(var n in e.on("end",function(){if(it("wrapped end"),t.decoder&&!t.ended){var e=t.decoder.end();e&&e.length&&r.push(e);}r.push(null);}),e.on("data",function(n){(it("wrapped data"),t.decoder&&(n=t.decoder.write(n)),t.objectMode&&null==n)||(t.objectMode||n&&n.length)&&(r.push(n)||(i=!0,e.pause()));}),e)void 0===this[n]&&"function"==typeof e[n]&&(this[n]=function(t){return function(){return e[t].apply(e,arguments)}}(n));return function(e,t){for(var i=0,r=e.length;i<r;i++)t(e[i],i);}(["error","close","destroy","pause","resume"],function(t){e.on(t,r.emit.bind(r,t));}),r._read=function(t){it("wrapped _read",t),i&&(i=!1,e.resume());},r},nt._fromList=pt,wt.WritableState=bt,Ne(wt,se),bt.prototype.getBuffer=function(){for(var e=this.bufferedRequest,t=[];e;)t.push(e),e=e.next;return t},wt.prototype.pipe=function(){this.emit("error",new Error("Cannot pipe, not readable"));},wt.prototype.write=function(e,t,i){var r=this._writableState,n=!1;return "function"==typeof t&&(i=t,t=null),v.isBuffer(e)?t="buffer":t||(t=r.defaultEncoding),"function"!=typeof i&&(i=yt),r.ended?function(e,t){var i=new Error("write after end");e.emit("error",i),ke(t,i);}(this,i):function(e,t,i,r){var n=!0,s=!1;return null===i?s=new TypeError("May not write null values to stream"):v.isBuffer(i)||"string"==typeof i||void 0===i||t.objectMode||(s=new TypeError("Invalid non-string/buffer chunk")),s&&(e.emit("error",s),ke(r,s),n=!1),n}(this,r,e,i)&&(r.pendingcb++,n=function(e,t,i,r,n){i=function(e,t,i){return e.objectMode||!1===e.decodeStrings||"string"!=typeof t||(t=v.from(t,i)),t}(t,i,r),v.isBuffer(i)&&(r="buffer");var s=t.objectMode?1:i.length;t.length+=s;var o=t.length<t.highWaterMark;o||(t.needDrain=!0);if(t.writing||t.corked){var a=t.lastBufferedRequest;t.lastBufferedRequest=new vt(i,r,n),a?a.next=t.lastBufferedRequest:t.bufferedRequest=t.lastBufferedRequest,t.bufferedRequestCount+=1;}else kt(e,t,!1,s,i,r,n);return o}(this,r,e,t,i)),n},wt.prototype.cork=function(){this._writableState.corked++;},wt.prototype.uncork=function(){var e=this._writableState;e.corked&&(e.corked--,e.writing||e.corked||e.finished||e.bufferProcessing||!e.bufferedRequest||xt(this,e));},wt.prototype.setDefaultEncoding=function(e){if("string"==typeof e&&(e=e.toLowerCase()),!(["hex","utf8","utf-8","ascii","binary","base64","ucs2","ucs-2","utf16le","utf-16le","raw"].indexOf((e+"").toLowerCase())>-1))throw new TypeError("Unknown encoding: "+e);return this._writableState.defaultEncoding=e,this},wt.prototype._write=function(e,t,i){i(new Error("not implemented"));},wt.prototype._writev=null,wt.prototype.end=function(e,t,i){var r=this._writableState;"function"==typeof e?(i=e,e=null,t=null):"function"==typeof t&&(i=t,t=null),null!=e&&this.write(e,t),r.corked&&(r.corked=1,this.uncork()),r.ending||r.finished||function(e,t,i){t.ending=!0,At(e,t),i&&(t.finished?ke(i):e.once("finish",i));t.ended=!0,e.writable=!1;}(this,r,i);},Ne(Lt,nt);for(var zt=Object.keys(wt.prototype),Ot=0;Ot<zt.length;Ot++){var Rt=zt[Ot];Lt.prototype[Rt]||(Lt.prototype[Rt]=wt.prototype[Rt]);}function Lt(e){if(!(this instanceof Lt))return new Lt(e);nt.call(this,e),wt.call(this,e),e&&!1===e.readable&&(this.readable=!1),e&&!1===e.writable&&(this.writable=!1),this.allowHalfOpen=!0,e&&!1===e.allowHalfOpen&&(this.allowHalfOpen=!1),this.once("end",Bt);}function Bt(){this.allowHalfOpen||this._writableState.ended||ke(It,this);}function It(e){e.end();}function Ft(e){this.afterTransform=function(t,i){return function(e,t,i){var r=e._transformState;r.transforming=!1;var n=r.writecb;if(!n)return e.emit("error",new Error("no writecb in Transform class"));r.writechunk=null,r.writecb=null,null!=i&&e.push(i);n(t);var s=e._readableState;s.reading=!1,(s.needReadable||s.length<s.highWaterMark)&&e._read(s.highWaterMark);}(e,t,i)},this.needTransform=!1,this.transforming=!1,this.writecb=null,this.writechunk=null,this.writeencoding=null;}function Pt(e){if(!(this instanceof Pt))return new Pt(e);Lt.call(this,e),this._transformState=new Ft(this);var t=this;this._readableState.needReadable=!0,this._readableState.sync=!1,e&&("function"==typeof e.transform&&(this._transform=e.transform),"function"==typeof e.flush&&(this._flush=e.flush)),this.once("prefinish",function(){"function"==typeof this._flush?this._flush(function(e){Mt(t,e);}):Mt(t);});}function Mt(e,t){if(t)return e.emit("error",t);var i=e._writableState,r=e._transformState;if(i.length)throw new Error("Calling transform done when ws.length != 0");if(r.transforming)throw new Error("Calling transform done when still transforming");return e.push(null)}function Ut(e){if(!(this instanceof Ut))return new Ut(e);Pt.call(this,e);}function Dt(){se.call(this);}Ne(Pt,Lt),Pt.prototype.push=function(e,t){return this._transformState.needTransform=!1,Lt.prototype.push.call(this,e,t)},Pt.prototype._transform=function(e,t,i){throw new Error("Not implemented")},Pt.prototype._write=function(e,t,i){var r=this._transformState;if(r.writecb=i,r.writechunk=e,r.writeencoding=t,!r.transforming){var n=this._readableState;(r.needTransform||n.needReadable||n.length<n.highWaterMark)&&this._read(n.highWaterMark);}},Pt.prototype._read=function(e){var t=this._transformState;null!==t.writechunk&&t.writecb&&!t.transforming?(t.transforming=!0,this._transform(t.writechunk,t.writeencoding,t.afterTransform)):t.needTransform=!0;},Ne(Ut,Pt),Ut.prototype._transform=function(e,t,i){i(null,e);},Ne(Dt,se),Dt.Readable=nt,Dt.Writable=wt,Dt.Duplex=Lt,Dt.Transform=Pt,Dt.PassThrough=Ut,Dt.Stream=Dt,Dt.prototype.pipe=function(e,t){var i=this;function r(t){e.writable&&!1===e.write(t)&&i.pause&&i.pause();}function n(){i.readable&&i.resume&&i.resume();}i.on("data",r),e.on("drain",n),e._isStdio||t&&!1===t.end||(i.on("end",o),i.on("close",a));var s=!1;function o(){s||(s=!0,e.end());}function a(){s||(s=!0,"function"==typeof e.destroy&&e.destroy());}function l(e){if(h(),0===se.listenerCount(this,"error"))throw e}function h(){i.removeListener("data",r),e.removeListener("drain",n),i.removeListener("end",o),i.removeListener("close",a),i.removeListener("error",l),e.removeListener("error",l),i.removeListener("end",h),i.removeListener("close",h),e.removeListener("close",h);}return i.on("error",l),e.on("error",l),i.on("end",h),i.on("close",h),e.on("close",h),e.emit("pipe",i),e};var jt=Dt,Nt=e.createCommonjsModule(function(e,t){if(t.base64=!0,t.array=!0,t.string=!0,t.arraybuffer="undefined"!=typeof ArrayBuffer&&"undefined"!=typeof Uint8Array,t.nodebuffer=!0,t.uint8array="undefined"!=typeof Uint8Array,"undefined"==typeof ArrayBuffer)t.blob=!1;else{var i=new ArrayBuffer(0);try{t.blob=0===new Blob([i],{type:"application/zip"}).size;}catch(e){try{var r=new(self.BlobBuilder||self.WebKitBlobBuilder||self.MozBlobBuilder||self.MSBlobBuilder);r.append(i),t.blob=0===r.getBlob("application/zip").size;}catch(e){t.blob=!1;}}}try{t.nodestream=!!jt.Readable;}catch(e){t.nodestream=!1;}}),Zt=(Nt.base64,Nt.array,Nt.string,Nt.arraybuffer,Nt.nodebuffer,Nt.uint8array,Nt.blob,Nt.nodestream,"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/="),Wt={encode:function(e){for(var t,i,r,n,s,o,a,l=[],h=0,u=e.length,c=u,f="string"!==Ki.getTypeOf(e);h<e.length;)c=u-h,f?(t=e[h++],i=h<u?e[h++]:0,r=h<u?e[h++]:0):(t=e.charCodeAt(h++),i=h<u?e.charCodeAt(h++):0,r=h<u?e.charCodeAt(h++):0),n=t>>2,s=(3&t)<<4|i>>4,o=c>1?(15&i)<<2|r>>6:64,a=c>2?63&r:64,l.push(Zt.charAt(n)+Zt.charAt(s)+Zt.charAt(o)+Zt.charAt(a));return l.join("")},decode:function(e){var t,i,r,n,s,o,a=0,l=0;if("data:"===e.substr(0,"data:".length))throw new Error("Invalid base64 input, it looks like a data url.");var h,u=3*(e=e.replace(/[^A-Za-z0-9\+\/\=]/g,"")).length/4;if(e.charAt(e.length-1)===Zt.charAt(64)&&u--,e.charAt(e.length-2)===Zt.charAt(64)&&u--,u%1!=0)throw new Error("Invalid base64 input, bad content length.");for(h=Nt.uint8array?new Uint8Array(0|u):new Array(0|u);a<e.length;)t=Zt.indexOf(e.charAt(a++))<<2|(n=Zt.indexOf(e.charAt(a++)))>>4,i=(15&n)<<4|(s=Zt.indexOf(e.charAt(a++)))>>2,r=(3&s)<<6|(o=Zt.indexOf(e.charAt(a++))),h[l++]=t,64!==s&&(h[l++]=i),64!==o&&(h[l++]=r);return h}},qt=!0,Yt=function(e,t){return new v(e,t)},Vt=function(e){return v.alloc?v.alloc(e):new v(e)},Ht=function(e){return te(e)},$t=function(e){return e&&"function"==typeof e.on&&"function"==typeof e.pause&&"function"==typeof e.resume},Gt=e.createCommonjsModule(function(e){var t=e.exports="undefined"!=typeof window&&window.Math==Math?window:"undefined"!=typeof self&&self.Math==Math?self:Function("return this")();"number"==typeof __g&&(__g=t);}),Kt=e.createCommonjsModule(function(e){var t=e.exports={version:"2.3.0"};"number"==typeof __e&&(__e=t);}),Jt=(Kt.version,function(e,t,i){if(function(e){if("function"!=typeof e)throw TypeError(e+" is not a function!")}(e),void 0===t)return e;switch(i){case 1:return function(i){return e.call(t,i)};case 2:return function(i,r){return e.call(t,i,r)};case 3:return function(i,r,n){return e.call(t,i,r,n)}}return function(){return e.apply(t,arguments)}}),Xt=function(e){return "object"==typeof e?null!==e:"function"==typeof e},Qt=function(e){if(!Xt(e))throw TypeError(e+" is not an object!");return e},ei=function(e){try{return !!e()}catch(e){return !0}},ti=!ei(function(){return 7!=Object.defineProperty({},"a",{get:function(){return 7}}).a}),ii=Gt.document,ri=Xt(ii)&&Xt(ii.createElement),ni=function(e){return ri?ii.createElement(e):{}},si=!ti&&!ei(function(){return 7!=Object.defineProperty(ni("div"),"a",{get:function(){return 7}}).a}),oi=Object.defineProperty,ai={f:ti?Object.defineProperty:function(e,t,i){if(Qt(e),t=function(e,t){if(!Xt(e))return e;var i,r;if(t&&"function"==typeof(i=e.toString)&&!Xt(r=i.call(e)))return r;if("function"==typeof(i=e.valueOf)&&!Xt(r=i.call(e)))return r;if(!t&&"function"==typeof(i=e.toString)&&!Xt(r=i.call(e)))return r;throw TypeError("Can't convert object to primitive value")}(t,!0),Qt(i),si)try{return oi(e,t,i)}catch(e){}if("get"in i||"set"in i)throw TypeError("Accessors not supported!");return "value"in i&&(e[t]=i.value),e}},li=ti?function(e,t,i){return ai.f(e,t,function(e,t){return {enumerable:!(1&e),configurable:!(2&e),writable:!(4&e),value:t}}(1,i))}:function(e,t,i){return e[t]=i,e},hi=function(e,t,i){var r,n,s,o=e&hi.F,a=e&hi.G,l=e&hi.S,h=e&hi.P,u=e&hi.B,c=e&hi.W,f=a?Kt:Kt[t]||(Kt[t]={}),d=f.prototype,p=a?Gt:l?Gt[t]:(Gt[t]||{}).prototype;for(r in a&&(i=t),i)(n=!o&&p&&void 0!==p[r])&&r in f||(s=n?p[r]:i[r],f[r]=a&&"function"!=typeof p[r]?i[r]:u&&n?Jt(s,Gt):c&&p[r]==s?function(e){var t=function(t,i,r){if(this instanceof e){switch(arguments.length){case 0:return new e;case 1:return new e(t);case 2:return new e(t,i)}return new e(t,i,r)}return e.apply(this,arguments)};return t.prototype=e.prototype,t}(s):h&&"function"==typeof s?Jt(Function.call,s):s,h&&((f.virtual||(f.virtual={}))[r]=s,e&hi.R&&d&&!d[r]&&li(d,r,s)));};hi.F=1,hi.G=2,hi.S=4,hi.P=8,hi.B=16,hi.W=32,hi.U=64,hi.R=128;var ui,ci,fi,di,pi=hi,gi=Gt.document&&document.documentElement,mi={}.toString,_i=Gt.process,yi=Gt.setImmediate,vi=Gt.clearImmediate,bi=Gt.MessageChannel,wi=0,ki={},Si=function(){var e=+this;if(ki.hasOwnProperty(e)){var t=ki[e];delete ki[e],t();}},xi=function(e){Si.call(e.data);};yi&&vi||(yi=function(e){for(var t=[],i=1;arguments.length>i;)t.push(arguments[i++]);return ki[++wi]=function(){!function(e,t,i){var r=void 0===i;switch(t.length){case 0:return r?e():e.call(i);case 1:return r?e(t[0]):e.call(i,t[0]);case 2:return r?e(t[0],t[1]):e.call(i,t[0],t[1]);case 3:return r?e(t[0],t[1],t[2]):e.call(i,t[0],t[1],t[2]);case 4:return r?e(t[0],t[1],t[2],t[3]):e.call(i,t[0],t[1],t[2],t[3])}e.apply(i,t);}("function"==typeof e?e:Function(e),t);},ui(wi),wi},vi=function(e){delete ki[e];},"process"==(di=_i,mi.call(di).slice(8,-1))?ui=function(e){_i.nextTick(Jt(Si,e,1));}:bi?(fi=(ci=new bi).port2,ci.port1.onmessage=xi,ui=Jt(fi.postMessage,fi,1)):Gt.addEventListener&&"function"==typeof postMessage&&!Gt.importScripts?(ui=function(e){Gt.postMessage(e+"","*");},Gt.addEventListener("message",xi,!1)):ui="onreadystatechange"in ni("script")?function(e){gi.appendChild(ni("script")).onreadystatechange=function(){gi.removeChild(this),Si.call(e);};}:function(e){setTimeout(Jt(Si,e,1),0);});var Ci={set:yi,clear:vi};pi(pi.G+pi.B,{setImmediate:Ci.set,clearImmediate:Ci.clear});var Ei,Ai,Ti=Kt.setImmediate,zi=n.MutationObserver||n.WebKitMutationObserver;if(zi){var Oi=0,Ri=new zi(Fi),Li=n.document.createTextNode("");Ri.observe(Li,{characterData:!0}),Ei=function(){Li.data=Oi=++Oi%2;};}else if(n.setImmediate||void 0===n.MessageChannel)Ei="document"in n&&"onreadystatechange"in n.document.createElement("script")?function(){var e=n.document.createElement("script");e.onreadystatechange=function(){Fi(),e.onreadystatechange=null,e.parentNode.removeChild(e),e=null;},n.document.documentElement.appendChild(e);}:function(){setTimeout(Fi,0);};else{var Bi=new n.MessageChannel;Bi.port1.onmessage=Fi,Ei=function(){Bi.port2.postMessage(0);};}var Ii=[];function Fi(){var e,t;Ai=!0;for(var i=Ii.length;i;){for(t=Ii,Ii=[],e=-1;++e<i;)t[e]();i=Ii.length;}Ai=!1;}var Pi=function(e){1!==Ii.push(e)||Ai||Ei();};function Mi(){}var Ui={},Di=["REJECTED"],ji=["FULFILLED"],Ni=["PENDING"],Zi=Wi;function Wi(e){if("function"!=typeof e)throw new TypeError("resolver must be a function");this.state=Ni,this.queue=[],this.outcome=void 0,e!==Mi&&Hi(this,e);}function qi(e,t,i){this.promise=e,"function"==typeof t&&(this.onFulfilled=t,this.callFulfilled=this.otherCallFulfilled),"function"==typeof i&&(this.onRejected=i,this.callRejected=this.otherCallRejected);}function Yi(e,t,i){Pi(function(){var r;try{r=t(i);}catch(t){return Ui.reject(e,t)}r===e?Ui.reject(e,new TypeError("Cannot resolve promise with itself")):Ui.resolve(e,r);});}function Vi(e){var t=e&&e.then;if(e&&("object"==typeof e||"function"==typeof e)&&"function"==typeof t)return function(){t.apply(e,arguments);}}function Hi(e,t){var i=!1;function r(t){i||(i=!0,Ui.reject(e,t));}function n(t){i||(i=!0,Ui.resolve(e,t));}var s=$i(function(){t(n,r);});"error"===s.status&&r(s.value);}function $i(e,t){var i={};try{i.value=e(t),i.status="success";}catch(e){i.status="error",i.value=e;}return i}Wi.prototype.catch=function(e){return this.then(null,e)},Wi.prototype.then=function(e,t){if("function"!=typeof e&&this.state===ji||"function"!=typeof t&&this.state===Di)return this;var i=new this.constructor(Mi);this.state!==Ni?Yi(i,this.state===ji?e:t,this.outcome):this.queue.push(new qi(i,e,t));return i},qi.prototype.callFulfilled=function(e){Ui.resolve(this.promise,e);},qi.prototype.otherCallFulfilled=function(e){Yi(this.promise,this.onFulfilled,e);},qi.prototype.callRejected=function(e){Ui.reject(this.promise,e);},qi.prototype.otherCallRejected=function(e){Yi(this.promise,this.onRejected,e);},Ui.resolve=function(e,t){var i=$i(Vi,t);if("error"===i.status)return Ui.reject(e,i.value);var r=i.value;if(r)Hi(e,r);else{e.state=ji,e.outcome=t;for(var n=-1,s=e.queue.length;++n<s;)e.queue[n].callFulfilled(t);}return e},Ui.reject=function(e,t){e.state=Di,e.outcome=t;for(var i=-1,r=e.queue.length;++i<r;)e.queue[i].callRejected(t);return e},Wi.resolve=function(e){if(e instanceof this)return e;return Ui.resolve(new this(Mi),e)},Wi.reject=function(e){var t=new this(Mi);return Ui.reject(t,e)},Wi.all=function(e){var t=this;if("[object Array]"!==Object.prototype.toString.call(e))return this.reject(new TypeError("must be an array"));var i=e.length,r=!1;if(!i)return this.resolve([]);var n=new Array(i),s=0,o=-1,a=new this(Mi);for(;++o<i;)l(e[o],o);return a;function l(e,o){t.resolve(e).then(function(e){n[o]=e,++s!==i||r||(r=!0,Ui.resolve(a,n));},function(e){r||(r=!0,Ui.reject(a,e));});}},Wi.race=function(e){var t=this;if("[object Array]"!==Object.prototype.toString.call(e))return this.reject(new TypeError("must be an array"));var i=e.length,r=!1;if(!i)return this.resolve([]);var n=-1,s=new this(Mi);for(;++n<i;)o=e[n],t.resolve(o).then(function(e){r||(r=!0,Ui.resolve(s,e));},function(e){r||(r=!0,Ui.reject(s,e));});var o;return s};var Gi={Promise:"undefined"!=typeof Promise?Promise:Zi},Ki=e.createCommonjsModule(function(e,t){function i(e){return e}function r(e,t){for(var i=0;i<e.length;++i)t[i]=255&e.charCodeAt(i);return t}t.newBlob=function(e,i){t.checkSupport("blob");try{return new Blob([e],{type:i})}catch(t){try{var r=new(self.BlobBuilder||self.WebKitBlobBuilder||self.MozBlobBuilder||self.MSBlobBuilder);return r.append(e),r.getBlob(i)}catch(e){throw new Error("Bug : can't construct the Blob.")}}};var n={stringifyByChunk:function(e,t,i){var r=[],n=0,s=e.length;if(s<=i)return String.fromCharCode.apply(null,e);for(;n<s;)"array"===t||"nodebuffer"===t?r.push(String.fromCharCode.apply(null,e.slice(n,Math.min(n+i,s)))):r.push(String.fromCharCode.apply(null,e.subarray(n,Math.min(n+i,s)))),n+=i;return r.join("")},stringifyByChar:function(e){for(var t="",i=0;i<e.length;i++)t+=String.fromCharCode(e[i]);return t},applyCanBeUsed:{uint8array:function(){try{return Nt.uint8array&&1===String.fromCharCode.apply(null,new Uint8Array(1)).length}catch(e){return !1}}(),nodebuffer:function(){try{return Nt.nodebuffer&&1===String.fromCharCode.apply(null,Vt(1)).length}catch(e){return !1}}()}};function s(e){var i=65536,r=t.getTypeOf(e),s=!0;if("uint8array"===r?s=n.applyCanBeUsed.uint8array:"nodebuffer"===r&&(s=n.applyCanBeUsed.nodebuffer),s)for(;i>1;)try{return n.stringifyByChunk(e,r,i)}catch(e){i=Math.floor(i/2);}return n.stringifyByChar(e)}function o(e,t){for(var i=0;i<e.length;i++)t[i]=e[i];return t}t.applyFromCharCode=s;var a={};a.string={string:i,array:function(e){return r(e,new Array(e.length))},arraybuffer:function(e){return a.string.uint8array(e).buffer},uint8array:function(e){return r(e,new Uint8Array(e.length))},nodebuffer:function(e){return r(e,Vt(e.length))}},a.array={string:s,array:i,arraybuffer:function(e){return new Uint8Array(e).buffer},uint8array:function(e){return new Uint8Array(e)},nodebuffer:function(e){return Yt(e)}},a.arraybuffer={string:function(e){return s(new Uint8Array(e))},array:function(e){return o(new Uint8Array(e),new Array(e.byteLength))},arraybuffer:i,uint8array:function(e){return new Uint8Array(e)},nodebuffer:function(e){return Yt(new Uint8Array(e))}},a.uint8array={string:s,array:function(e){return o(e,new Array(e.length))},arraybuffer:function(e){return e.buffer},uint8array:i,nodebuffer:function(e){return Yt(e)}},a.nodebuffer={string:s,array:function(e){return o(e,new Array(e.length))},arraybuffer:function(e){return a.nodebuffer.uint8array(e).buffer},uint8array:function(e){return o(e,new Uint8Array(e.length))},nodebuffer:i},t.transformTo=function(e,i){if(i||(i=""),!e)return i;t.checkSupport(e);var r=t.getTypeOf(i);return a[r][e](i)},t.getTypeOf=function(e){return "string"==typeof e?"string":"[object Array]"===Object.prototype.toString.call(e)?"array":Nt.nodebuffer&&Ht(e)?"nodebuffer":Nt.uint8array&&e instanceof Uint8Array?"uint8array":Nt.arraybuffer&&e instanceof ArrayBuffer?"arraybuffer":void 0},t.checkSupport=function(e){if(!Nt[e.toLowerCase()])throw new Error(e+" is not supported by this platform")},t.MAX_VALUE_16BITS=65535,t.MAX_VALUE_32BITS=-1,t.pretty=function(e){var t,i,r="";for(i=0;i<(e||"").length;i++)r+="\\x"+((t=e.charCodeAt(i))<16?"0":"")+t.toString(16).toUpperCase();return r},t.delay=function(e,t,i){Ti(function(){e.apply(i||null,t||[]);});},t.inherits=function(e,t){var i=function(){};i.prototype=t.prototype,e.prototype=new i;},t.extend=function(){var e,t,i={};for(e=0;e<arguments.length;e++)for(t in arguments[e])arguments[e].hasOwnProperty(t)&&void 0===i[t]&&(i[t]=arguments[e][t]);return i},t.prepareContent=function(e,i,n,s,o){return Gi.Promise.resolve(i).then(function(e){return Nt.blob&&(e instanceof Blob||-1!==["[object File]","[object Blob]"].indexOf(Object.prototype.toString.call(e)))&&"undefined"!=typeof FileReader?new Gi.Promise(function(t,i){var r=new FileReader;r.onload=function(e){t(e.target.result);},r.onerror=function(e){i(e.target.error);},r.readAsArrayBuffer(e);}):e}).then(function(i){var a,l=t.getTypeOf(i);return l?("arraybuffer"===l?i=t.transformTo("uint8array",i):"string"===l&&(o?i=Wt.decode(i):n&&!0!==s&&(i=r(a=i,Nt.uint8array?new Uint8Array(a.length):new Array(a.length)))),i):Gi.Promise.reject(new Error("Can't read the data of '"+e+"'. Is it in a supported JavaScript type (String, Blob, ArrayBuffer, etc) ?"))})};});Ki.newBlob,Ki.applyFromCharCode,Ki.transformTo,Ki.getTypeOf,Ki.checkSupport,Ki.MAX_VALUE_16BITS,Ki.MAX_VALUE_32BITS,Ki.pretty,Ki.delay,Ki.inherits,Ki.extend,Ki.prepareContent;function Ji(e){this.name=e||"default",this.streamInfo={},this.generatedError=null,this.extraStreamInfo={},this.isPaused=!0,this.isFinished=!1,this.isLocked=!1,this._listeners={data:[],end:[],error:[]},this.previous=null;}Ji.prototype={push:function(e){this.emit("data",e);},end:function(){if(this.isFinished)return !1;this.flush();try{this.emit("end"),this.cleanUp(),this.isFinished=!0;}catch(e){this.emit("error",e);}return !0},error:function(e){return !this.isFinished&&(this.isPaused?this.generatedError=e:(this.isFinished=!0,this.emit("error",e),this.previous&&this.previous.error(e),this.cleanUp()),!0)},on:function(e,t){return this._listeners[e].push(t),this},cleanUp:function(){this.streamInfo=this.generatedError=this.extraStreamInfo=null,this._listeners=[];},emit:function(e,t){if(this._listeners[e])for(var i=0;i<this._listeners[e].length;i++)this._listeners[e][i].call(this,t);},pipe:function(e){return e.registerPrevious(this)},registerPrevious:function(e){if(this.isLocked)throw new Error("The stream '"+this+"' has already been used.");this.streamInfo=e.streamInfo,this.mergeStreamInfo(),this.previous=e;var t=this;return e.on("data",function(e){t.processChunk(e);}),e.on("end",function(){t.end();}),e.on("error",function(e){t.error(e);}),this},pause:function(){return !this.isPaused&&!this.isFinished&&(this.isPaused=!0,this.previous&&this.previous.pause(),!0)},resume:function(){if(!this.isPaused||this.isFinished)return !1;this.isPaused=!1;var e=!1;return this.generatedError&&(this.error(this.generatedError),e=!0),this.previous&&this.previous.resume(),!e},flush:function(){},processChunk:function(e){this.push(e);},withStreamInfo:function(e,t){return this.extraStreamInfo[e]=t,this.mergeStreamInfo(),this},mergeStreamInfo:function(){for(var e in this.extraStreamInfo)this.extraStreamInfo.hasOwnProperty(e)&&(this.streamInfo[e]=this.extraStreamInfo[e]);},lock:function(){if(this.isLocked)throw new Error("The stream '"+this+"' has already been used.");this.isLocked=!0,this.previous&&this.previous.lock();},toString:function(){var e="Worker "+this.name;return this.previous?this.previous+" -> "+e:e}};var Xi=Ji,Qi=e.createCommonjsModule(function(e,t){for(var i=new Array(256),r=0;r<256;r++)i[r]=r>=252?6:r>=248?5:r>=240?4:r>=224?3:r>=192?2:1;i[254]=i[254]=1;function n(){Xi.call(this,"utf-8 decode"),this.leftOver=null;}function s(){Xi.call(this,"utf-8 encode");}t.utf8encode=function(e){return Nt.nodebuffer?Yt(e,"utf-8"):function(e){var t,i,r,n,s,o=e.length,a=0;for(n=0;n<o;n++)55296==(64512&(i=e.charCodeAt(n)))&&n+1<o&&56320==(64512&(r=e.charCodeAt(n+1)))&&(i=65536+(i-55296<<10)+(r-56320),n++),a+=i<128?1:i<2048?2:i<65536?3:4;for(t=Nt.uint8array?new Uint8Array(a):new Array(a),s=0,n=0;s<a;n++)55296==(64512&(i=e.charCodeAt(n)))&&n+1<o&&56320==(64512&(r=e.charCodeAt(n+1)))&&(i=65536+(i-55296<<10)+(r-56320),n++),i<128?t[s++]=i:i<2048?(t[s++]=192|i>>>6,t[s++]=128|63&i):i<65536?(t[s++]=224|i>>>12,t[s++]=128|i>>>6&63,t[s++]=128|63&i):(t[s++]=240|i>>>18,t[s++]=128|i>>>12&63,t[s++]=128|i>>>6&63,t[s++]=128|63&i);return t}(e)},t.utf8decode=function(e){return Nt.nodebuffer?Ki.transformTo("nodebuffer",e).toString("utf-8"):function(e){var t,r,n,s,o=e.length,a=new Array(2*o);for(r=0,t=0;t<o;)if((n=e[t++])<128)a[r++]=n;else if((s=i[n])>4)a[r++]=65533,t+=s-1;else{for(n&=2===s?31:3===s?15:7;s>1&&t<o;)n=n<<6|63&e[t++],s--;s>1?a[r++]=65533:n<65536?a[r++]=n:(n-=65536,a[r++]=55296|n>>10&1023,a[r++]=56320|1023&n);}return a.length!==r&&(a.subarray?a=a.subarray(0,r):a.length=r),Ki.applyFromCharCode(a)}(e=Ki.transformTo(Nt.uint8array?"uint8array":"array",e))},Ki.inherits(n,Xi),n.prototype.processChunk=function(e){var r=Ki.transformTo(Nt.uint8array?"uint8array":"array",e.data);if(this.leftOver&&this.leftOver.length){if(Nt.uint8array){var n=r;(r=new Uint8Array(n.length+this.leftOver.length)).set(this.leftOver,0),r.set(n,this.leftOver.length);}else r=this.leftOver.concat(r);this.leftOver=null;}var s=function(e,t){var r;for((t=t||e.length)>e.length&&(t=e.length),r=t-1;r>=0&&128==(192&e[r]);)r--;return r<0?t:0===r?t:r+i[e[r]]>t?r:t}(r),o=r;s!==r.length&&(Nt.uint8array?(o=r.subarray(0,s),this.leftOver=r.subarray(s,r.length)):(o=r.slice(0,s),this.leftOver=r.slice(s,r.length))),this.push({data:t.utf8decode(o),meta:e.meta});},n.prototype.flush=function(){this.leftOver&&this.leftOver.length&&(this.push({data:t.utf8decode(this.leftOver),meta:{}}),this.leftOver=null);},t.Utf8DecodeWorker=n,Ki.inherits(s,Xi),s.prototype.processChunk=function(e){this.push({data:t.utf8encode(e.data),meta:e.meta});},t.Utf8EncodeWorker=s;});Qi.utf8encode,Qi.utf8decode,Qi.Utf8DecodeWorker,Qi.Utf8EncodeWorker;function er(e){Xi.call(this,"ConvertWorker to "+e),this.destType=e;}Ki.inherits(er,Xi),er.prototype.processChunk=function(e){this.push({data:Ki.transformTo(this.destType,e.data),meta:e.meta});};var tr=er,ir=jt.Readable;function rr(e,t,i){ir.call(this,t),this._helper=e;var r=this;e.on("data",function(e,t){r.push(e)||r._helper.pause(),i&&i(t);}).on("error",function(e){r.emit("error",e);}).on("end",function(){r.push(null);});}Ki.inherits(rr,ir),rr.prototype._read=function(){this._helper.resume();};var nr=rr,sr=null;if(Nt.nodestream)try{sr=nr;}catch(e){}function or(e,t){return new Gi.Promise(function(i,r){var n=[],s=e._internalType,o=e._outputType,a=e._mimeType;e.on("data",function(e,i){n.push(e),t&&t(i);}).on("error",function(e){n=[],r(e);}).on("end",function(){try{var e=function(e,t,i){switch(e){case"blob":return Ki.newBlob(Ki.transformTo("arraybuffer",t),i);case"base64":return Wt.encode(t);default:return Ki.transformTo(e,t)}}(o,function(e,t){var i,r=0,n=null,s=0;for(i=0;i<t.length;i++)s+=t[i].length;switch(e){case"string":return t.join("");case"array":return Array.prototype.concat.apply([],t);case"uint8array":for(n=new Uint8Array(s),i=0;i<t.length;i++)n.set(t[i],r),r+=t[i].length;return n;case"nodebuffer":return v.concat(t);default:throw new Error("concat : unsupported type '"+e+"'")}}(s,n),a);i(e);}catch(e){r(e);}n=[];}).resume();})}function ar(e,t,i){var r=t;switch(t){case"blob":case"arraybuffer":r="uint8array";break;case"base64":r="string";}try{this._internalType=r,this._outputType=t,this._mimeType=i,Ki.checkSupport(r),this._worker=e.pipe(new tr(r)),e.lock();}catch(e){this._worker=new Xi("error"),this._worker.error(e);}}ar.prototype={accumulate:function(e){return or(this,e)},on:function(e,t){var i=this;return "data"===e?this._worker.on(e,function(e){t.call(i,e.data,e.meta);}):this._worker.on(e,function(){Ki.delay(t,arguments,i);}),this},resume:function(){return Ki.delay(this._worker.resume,[],this._worker),this},pause:function(){return this._worker.pause(),this},toNodejsStream:function(e){if(Ki.checkSupport("nodestream"),"nodebuffer"!==this._outputType)throw new Error(this._outputType+" is not supported by this method");return new sr(this,{objectMode:"nodebuffer"!==this._outputType},e)}};var lr=ar,hr={base64:!1,binary:!1,dir:!1,createFolders:!0,date:null,compression:null,compressionOptions:null,comment:null,unixPermissions:null,dosPermissions:null};function ur(e){Xi.call(this,"DataWorker");var t=this;this.dataIsReady=!1,this.index=0,this.max=0,this.data=null,this.type="",this._tickScheduled=!1,e.then(function(e){t.dataIsReady=!0,t.data=e,t.max=e&&e.length||0,t.type=Ki.getTypeOf(e),t.isPaused||t._tickAndRepeat();},function(e){t.error(e);});}Ki.inherits(ur,Xi),ur.prototype.cleanUp=function(){Xi.prototype.cleanUp.call(this),this.data=null;},ur.prototype.resume=function(){return !!Xi.prototype.resume.call(this)&&(!this._tickScheduled&&this.dataIsReady&&(this._tickScheduled=!0,Ki.delay(this._tickAndRepeat,[],this)),!0)},ur.prototype._tickAndRepeat=function(){this._tickScheduled=!1,this.isPaused||this.isFinished||(this._tick(),this.isFinished||(Ki.delay(this._tickAndRepeat,[],this),this._tickScheduled=!0));},ur.prototype._tick=function(){if(this.isPaused||this.isFinished)return !1;var e=null,t=Math.min(this.max,this.index+16384);if(this.index>=this.max)return this.end();switch(this.type){case"string":e=this.data.substring(this.index,t);break;case"uint8array":e=this.data.subarray(this.index,t);break;case"array":case"nodebuffer":e=this.data.slice(this.index,t);}return this.index=t,this.push({data:e,meta:{percent:this.max?this.index/this.max*100:0}})};var cr=ur;function fr(e){Xi.call(this,"DataLengthProbe for "+e),this.propName=e,this.withStreamInfo(e,0);}Ki.inherits(fr,Xi),fr.prototype.processChunk=function(e){if(e){var t=this.streamInfo[this.propName]||0;this.streamInfo[this.propName]=t+e.data.length;}Xi.prototype.processChunk.call(this,e);};var dr=fr;var pr=function(){for(var e,t=[],i=0;i<256;i++){e=i;for(var r=0;r<8;r++)e=1&e?3988292384^e>>>1:e>>>1;t[i]=e;}return t}();var gr=function(e,t){return void 0!==e&&e.length?"string"!==Ki.getTypeOf(e)?function(e,t,i,r){var n=pr,s=r+i;e^=-1;for(var o=r;o<s;o++)e=e>>>8^n[255&(e^t[o])];return -1^e}(0|t,e,e.length,0):function(e,t,i,r){var n=pr,s=r+i;e^=-1;for(var o=r;o<s;o++)e=e>>>8^n[255&(e^t.charCodeAt(o))];return -1^e}(0|t,e,e.length,0):0};function mr(){Xi.call(this,"Crc32Probe"),this.withStreamInfo("crc32",0);}Ki.inherits(mr,Xi),mr.prototype.processChunk=function(e){this.streamInfo.crc32=gr(e.data,this.streamInfo.crc32||0),this.push(e);};var _r=mr;function yr(e,t,i,r,n){this.compressedSize=e,this.uncompressedSize=t,this.crc32=i,this.compression=r,this.compressedContent=n;}yr.prototype={getContentWorker:function(){var e=new cr(Gi.Promise.resolve(this.compressedContent)).pipe(this.compression.uncompressWorker()).pipe(new dr("data_length")),t=this;return e.on("end",function(){if(this.streamInfo.data_length!==t.uncompressedSize)throw new Error("Bug : uncompressed data size mismatch")}),e},getCompressedWorker:function(){return new cr(Gi.Promise.resolve(this.compressedContent)).withStreamInfo("compressedSize",this.compressedSize).withStreamInfo("uncompressedSize",this.uncompressedSize).withStreamInfo("crc32",this.crc32).withStreamInfo("compression",this.compression)}},yr.createWorkerFrom=function(e,t,i){return e.pipe(new _r).pipe(new dr("uncompressedSize")).pipe(t.compressWorker(i)).pipe(new dr("compressedSize")).withStreamInfo("compression",t)};var vr=yr,br=function(e,t,i){this.name=e,this.dir=i.dir,this.date=i.date,this.comment=i.comment,this.unixPermissions=i.unixPermissions,this.dosPermissions=i.dosPermissions,this._data=t,this._dataBinary=i.binary,this.options={compression:i.compression,compressionOptions:i.compressionOptions};};br.prototype={internalStream:function(e){var t=null,i="string";try{if(!e)throw new Error("No output type specified.");var r="string"===(i=e.toLowerCase())||"text"===i;"binarystring"!==i&&"text"!==i||(i="string"),t=this._decompressWorker();var n=!this._dataBinary;n&&!r&&(t=t.pipe(new Qi.Utf8EncodeWorker)),!n&&r&&(t=t.pipe(new Qi.Utf8DecodeWorker));}catch(e){(t=new Xi("error")).error(e);}return new lr(t,i,"")},async:function(e,t){return this.internalStream(e).accumulate(t)},nodeStream:function(e,t){return this.internalStream(e||"nodebuffer").toNodejsStream(t)},_compressWorker:function(e,t){if(this._data instanceof vr&&this._data.compression.magic===e.magic)return this._data.getCompressedWorker();var i=this._decompressWorker();return this._dataBinary||(i=i.pipe(new Qi.Utf8EncodeWorker)),vr.createWorkerFrom(i,e,t)},_decompressWorker:function(){return this._data instanceof vr?this._data.getContentWorker():this._data instanceof Xi?this._data:new cr(this._data)}};for(var wr=["asText","asBinary","asNodeBuffer","asUint8Array","asArrayBuffer"],kr=function(){throw new Error("This method has been removed in JSZip 3.0, please check the upgrade guide.")},Sr=0;Sr<wr.length;Sr++)br.prototype[wr[Sr]]=kr;var xr=br,Cr=e.createCommonjsModule(function(e,t){var i="undefined"!=typeof Uint8Array&&"undefined"!=typeof Uint16Array&&"undefined"!=typeof Int32Array;function r(e,t){return Object.prototype.hasOwnProperty.call(e,t)}t.assign=function(e){for(var t=Array.prototype.slice.call(arguments,1);t.length;){var i=t.shift();if(i){if("object"!=typeof i)throw new TypeError(i+"must be non-object");for(var n in i)r(i,n)&&(e[n]=i[n]);}}return e},t.shrinkBuf=function(e,t){return e.length===t?e:e.subarray?e.subarray(0,t):(e.length=t,e)};var n={arraySet:function(e,t,i,r,n){if(t.subarray&&e.subarray)e.set(t.subarray(i,i+r),n);else for(var s=0;s<r;s++)e[n+s]=t[i+s];},flattenChunks:function(e){var t,i,r,n,s,o;for(r=0,t=0,i=e.length;t<i;t++)r+=e[t].length;for(o=new Uint8Array(r),n=0,t=0,i=e.length;t<i;t++)s=e[t],o.set(s,n),n+=s.length;return o}},s={arraySet:function(e,t,i,r,n){for(var s=0;s<r;s++)e[n+s]=t[i+s];},flattenChunks:function(e){return [].concat.apply([],e)}};t.setTyped=function(e){e?(t.Buf8=Uint8Array,t.Buf16=Uint16Array,t.Buf32=Int32Array,t.assign(t,n)):(t.Buf8=Array,t.Buf16=Array,t.Buf32=Array,t.assign(t,s));},t.setTyped(i);}),Er=(Cr.assign,Cr.shrinkBuf,Cr.setTyped,Cr.Buf8,Cr.Buf16,Cr.Buf32,4),Ar=0,Tr=1,zr=2;function Or(e){for(var t=e.length;--t>=0;)e[t]=0;}var Rr=0,Lr=1,Br=2,Ir=29,Fr=256,Pr=Fr+1+Ir,Mr=30,Ur=19,Dr=2*Pr+1,jr=15,Nr=16,Zr=7,Wr=256,qr=16,Yr=17,Vr=18,Hr=[0,0,0,0,0,0,0,0,1,1,1,1,2,2,2,2,3,3,3,3,4,4,4,4,5,5,5,5,0],$r=[0,0,0,0,1,1,2,2,3,3,4,4,5,5,6,6,7,7,8,8,9,9,10,10,11,11,12,12,13,13],Gr=[0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,3,7],Kr=[16,17,18,0,8,7,9,6,10,5,11,4,12,3,13,2,14,1,15],Jr=new Array(2*(Pr+2));Or(Jr);var Xr=new Array(2*Mr);Or(Xr);var Qr=new Array(512);Or(Qr);var en=new Array(256);Or(en);var tn=new Array(Ir);Or(tn);var rn,nn,sn,on=new Array(Mr);function an(e,t,i,r,n){this.static_tree=e,this.extra_bits=t,this.extra_base=i,this.elems=r,this.max_length=n,this.has_stree=e&&e.length;}function ln(e,t){this.dyn_tree=e,this.max_code=0,this.stat_desc=t;}function hn(e){return e<256?Qr[e]:Qr[256+(e>>>7)]}function un(e,t){e.pending_buf[e.pending++]=255&t,e.pending_buf[e.pending++]=t>>>8&255;}function cn(e,t,i){e.bi_valid>Nr-i?(e.bi_buf|=t<<e.bi_valid&65535,un(e,e.bi_buf),e.bi_buf=t>>Nr-e.bi_valid,e.bi_valid+=i-Nr):(e.bi_buf|=t<<e.bi_valid&65535,e.bi_valid+=i);}function fn(e,t,i){cn(e,i[2*t],i[2*t+1]);}function dn(e,t){var i=0;do{i|=1&e,e>>>=1,i<<=1;}while(--t>0);return i>>>1}function pn(e,t,i){var r,n,s=new Array(jr+1),o=0;for(r=1;r<=jr;r++)s[r]=o=o+i[r-1]<<1;for(n=0;n<=t;n++){var a=e[2*n+1];0!==a&&(e[2*n]=dn(s[a]++,a));}}function gn(e){var t;for(t=0;t<Pr;t++)e.dyn_ltree[2*t]=0;for(t=0;t<Mr;t++)e.dyn_dtree[2*t]=0;for(t=0;t<Ur;t++)e.bl_tree[2*t]=0;e.dyn_ltree[2*Wr]=1,e.opt_len=e.static_len=0,e.last_lit=e.matches=0;}function mn(e){e.bi_valid>8?un(e,e.bi_buf):e.bi_valid>0&&(e.pending_buf[e.pending++]=e.bi_buf),e.bi_buf=0,e.bi_valid=0;}function _n(e,t,i,r){var n=2*t,s=2*i;return e[n]<e[s]||e[n]===e[s]&&r[t]<=r[i]}function yn(e,t,i){for(var r=e.heap[i],n=i<<1;n<=e.heap_len&&(n<e.heap_len&&_n(t,e.heap[n+1],e.heap[n],e.depth)&&n++,!_n(t,r,e.heap[n],e.depth));)e.heap[i]=e.heap[n],i=n,n<<=1;e.heap[i]=r;}function vn(e,t,i){var r,n,s,o,a=0;if(0!==e.last_lit)do{r=e.pending_buf[e.d_buf+2*a]<<8|e.pending_buf[e.d_buf+2*a+1],n=e.pending_buf[e.l_buf+a],a++,0===r?fn(e,n,t):(fn(e,(s=en[n])+Fr+1,t),0!==(o=Hr[s])&&cn(e,n-=tn[s],o),fn(e,s=hn(--r),i),0!==(o=$r[s])&&cn(e,r-=on[s],o));}while(a<e.last_lit);fn(e,Wr,t);}function bn(e,t){var i,r,n,s=t.dyn_tree,o=t.stat_desc.static_tree,a=t.stat_desc.has_stree,l=t.stat_desc.elems,h=-1;for(e.heap_len=0,e.heap_max=Dr,i=0;i<l;i++)0!==s[2*i]?(e.heap[++e.heap_len]=h=i,e.depth[i]=0):s[2*i+1]=0;for(;e.heap_len<2;)s[2*(n=e.heap[++e.heap_len]=h<2?++h:0)]=1,e.depth[n]=0,e.opt_len--,a&&(e.static_len-=o[2*n+1]);for(t.max_code=h,i=e.heap_len>>1;i>=1;i--)yn(e,s,i);n=l;do{i=e.heap[1],e.heap[1]=e.heap[e.heap_len--],yn(e,s,1),r=e.heap[1],e.heap[--e.heap_max]=i,e.heap[--e.heap_max]=r,s[2*n]=s[2*i]+s[2*r],e.depth[n]=(e.depth[i]>=e.depth[r]?e.depth[i]:e.depth[r])+1,s[2*i+1]=s[2*r+1]=n,e.heap[1]=n++,yn(e,s,1);}while(e.heap_len>=2);e.heap[--e.heap_max]=e.heap[1],function(e,t){var i,r,n,s,o,a,l=t.dyn_tree,h=t.max_code,u=t.stat_desc.static_tree,c=t.stat_desc.has_stree,f=t.stat_desc.extra_bits,d=t.stat_desc.extra_base,p=t.stat_desc.max_length,g=0;for(s=0;s<=jr;s++)e.bl_count[s]=0;for(l[2*e.heap[e.heap_max]+1]=0,i=e.heap_max+1;i<Dr;i++)(s=l[2*l[2*(r=e.heap[i])+1]+1]+1)>p&&(s=p,g++),l[2*r+1]=s,r>h||(e.bl_count[s]++,o=0,r>=d&&(o=f[r-d]),a=l[2*r],e.opt_len+=a*(s+o),c&&(e.static_len+=a*(u[2*r+1]+o)));if(0!==g){do{for(s=p-1;0===e.bl_count[s];)s--;e.bl_count[s]--,e.bl_count[s+1]+=2,e.bl_count[p]--,g-=2;}while(g>0);for(s=p;0!==s;s--)for(r=e.bl_count[s];0!==r;)(n=e.heap[--i])>h||(l[2*n+1]!==s&&(e.opt_len+=(s-l[2*n+1])*l[2*n],l[2*n+1]=s),r--);}}(e,t),pn(s,h,e.bl_count);}function wn(e,t,i){var r,n,s=-1,o=t[1],a=0,l=7,h=4;for(0===o&&(l=138,h=3),t[2*(i+1)+1]=65535,r=0;r<=i;r++)n=o,o=t[2*(r+1)+1],++a<l&&n===o||(a<h?e.bl_tree[2*n]+=a:0!==n?(n!==s&&e.bl_tree[2*n]++,e.bl_tree[2*qr]++):a<=10?e.bl_tree[2*Yr]++:e.bl_tree[2*Vr]++,a=0,s=n,0===o?(l=138,h=3):n===o?(l=6,h=3):(l=7,h=4));}function kn(e,t,i){var r,n,s=-1,o=t[1],a=0,l=7,h=4;for(0===o&&(l=138,h=3),r=0;r<=i;r++)if(n=o,o=t[2*(r+1)+1],!(++a<l&&n===o)){if(a<h)do{fn(e,n,e.bl_tree);}while(0!=--a);else 0!==n?(n!==s&&(fn(e,n,e.bl_tree),a--),fn(e,qr,e.bl_tree),cn(e,a-3,2)):a<=10?(fn(e,Yr,e.bl_tree),cn(e,a-3,3)):(fn(e,Vr,e.bl_tree),cn(e,a-11,7));a=0,s=n,0===o?(l=138,h=3):n===o?(l=6,h=3):(l=7,h=4);}}Or(on);var Sn=!1;function xn(e,t,i,r){cn(e,(Rr<<1)+(r?1:0),3),function(e,t,i,r){mn(e),r&&(un(e,i),un(e,~i)),Cr.arraySet(e.pending_buf,e.window,t,i,e.pending),e.pending+=i;}(e,t,i,!0);}var Cn={_tr_init:function(e){Sn||(function(){var e,t,i,r,n,s=new Array(jr+1);for(i=0,r=0;r<Ir-1;r++)for(tn[r]=i,e=0;e<1<<Hr[r];e++)en[i++]=r;for(en[i-1]=r,n=0,r=0;r<16;r++)for(on[r]=n,e=0;e<1<<$r[r];e++)Qr[n++]=r;for(n>>=7;r<Mr;r++)for(on[r]=n<<7,e=0;e<1<<$r[r]-7;e++)Qr[256+n++]=r;for(t=0;t<=jr;t++)s[t]=0;for(e=0;e<=143;)Jr[2*e+1]=8,e++,s[8]++;for(;e<=255;)Jr[2*e+1]=9,e++,s[9]++;for(;e<=279;)Jr[2*e+1]=7,e++,s[7]++;for(;e<=287;)Jr[2*e+1]=8,e++,s[8]++;for(pn(Jr,Pr+1,s),e=0;e<Mr;e++)Xr[2*e+1]=5,Xr[2*e]=dn(e,5);rn=new an(Jr,Hr,Fr+1,Pr,jr),nn=new an(Xr,$r,0,Mr,jr),sn=new an(new Array(0),Gr,0,Ur,Zr);}(),Sn=!0),e.l_desc=new ln(e.dyn_ltree,rn),e.d_desc=new ln(e.dyn_dtree,nn),e.bl_desc=new ln(e.bl_tree,sn),e.bi_buf=0,e.bi_valid=0,gn(e);},_tr_stored_block:xn,_tr_flush_block:function(e,t,i,r){var n,s,o=0;e.level>0?(e.strm.data_type===zr&&(e.strm.data_type=function(e){var t,i=4093624447;for(t=0;t<=31;t++,i>>>=1)if(1&i&&0!==e.dyn_ltree[2*t])return Ar;if(0!==e.dyn_ltree[18]||0!==e.dyn_ltree[20]||0!==e.dyn_ltree[26])return Tr;for(t=32;t<Fr;t++)if(0!==e.dyn_ltree[2*t])return Tr;return Ar}(e)),bn(e,e.l_desc),bn(e,e.d_desc),o=function(e){var t;for(wn(e,e.dyn_ltree,e.l_desc.max_code),wn(e,e.dyn_dtree,e.d_desc.max_code),bn(e,e.bl_desc),t=Ur-1;t>=3&&0===e.bl_tree[2*Kr[t]+1];t--);return e.opt_len+=3*(t+1)+5+5+4,t}(e),n=e.opt_len+3+7>>>3,(s=e.static_len+3+7>>>3)<=n&&(n=s)):n=s=i+5,i+4<=n&&-1!==t?xn(e,t,i,r):e.strategy===Er||s===n?(cn(e,(Lr<<1)+(r?1:0),3),vn(e,Jr,Xr)):(cn(e,(Br<<1)+(r?1:0),3),function(e,t,i,r){var n;for(cn(e,t-257,5),cn(e,i-1,5),cn(e,r-4,4),n=0;n<r;n++)cn(e,e.bl_tree[2*Kr[n]+1],3);kn(e,e.dyn_ltree,t-1),kn(e,e.dyn_dtree,i-1);}(e,e.l_desc.max_code+1,e.d_desc.max_code+1,o+1),vn(e,e.dyn_ltree,e.dyn_dtree)),gn(e),r&&mn(e);},_tr_tally:function(e,t,i){return e.pending_buf[e.d_buf+2*e.last_lit]=t>>>8&255,e.pending_buf[e.d_buf+2*e.last_lit+1]=255&t,e.pending_buf[e.l_buf+e.last_lit]=255&i,e.last_lit++,0===t?e.dyn_ltree[2*i]++:(e.matches++,t--,e.dyn_ltree[2*(en[i]+Fr+1)]++,e.dyn_dtree[2*hn(t)]++),e.last_lit===e.lit_bufsize-1},_tr_align:function(e){cn(e,Lr<<1,3),fn(e,Wr,Jr),function(e){16===e.bi_valid?(un(e,e.bi_buf),e.bi_buf=0,e.bi_valid=0):e.bi_valid>=8&&(e.pending_buf[e.pending++]=255&e.bi_buf,e.bi_buf>>=8,e.bi_valid-=8);}(e);}};var En=function(e,t,i,r){for(var n=65535&e|0,s=e>>>16&65535|0,o=0;0!==i;){i-=o=i>2e3?2e3:i;do{s=s+(n=n+t[r++]|0)|0;}while(--o);n%=65521,s%=65521;}return n|s<<16|0};var An=function(){for(var e,t=[],i=0;i<256;i++){e=i;for(var r=0;r<8;r++)e=1&e?3988292384^e>>>1:e>>>1;t[i]=e;}return t}();var Tn,zn=function(e,t,i,r){var n=An,s=r+i;e^=-1;for(var o=r;o<s;o++)e=e>>>8^n[255&(e^t[o])];return -1^e},On={2:"need dictionary",1:"stream end",0:"","-1":"file error","-2":"stream error","-3":"data error","-4":"insufficient memory","-5":"buffer error","-6":"incompatible version"},Rn=0,Ln=1,Bn=3,In=4,Fn=5,Pn=0,Mn=1,Un=-2,Dn=-3,jn=-5,Nn=-1,Zn=1,Wn=2,qn=3,Yn=4,Vn=0,Hn=2,$n=8,Gn=9,Kn=15,Jn=8,Xn=286,Qn=30,es=19,ts=2*Xn+1,is=15,rs=3,ns=258,ss=ns+rs+1,os=32,as=42,ls=69,hs=73,us=91,cs=103,fs=113,ds=666,ps=1,gs=2,ms=3,_s=4,ys=3;function vs(e,t){return e.msg=On[t],t}function bs(e){return (e<<1)-(e>4?9:0)}function ws(e){for(var t=e.length;--t>=0;)e[t]=0;}function ks(e){var t=e.state,i=t.pending;i>e.avail_out&&(i=e.avail_out),0!==i&&(Cr.arraySet(e.output,t.pending_buf,t.pending_out,i,e.next_out),e.next_out+=i,t.pending_out+=i,e.total_out+=i,e.avail_out-=i,t.pending-=i,0===t.pending&&(t.pending_out=0));}function Ss(e,t){Cn._tr_flush_block(e,e.block_start>=0?e.block_start:-1,e.strstart-e.block_start,t),e.block_start=e.strstart,ks(e.strm);}function xs(e,t){e.pending_buf[e.pending++]=t;}function Cs(e,t){e.pending_buf[e.pending++]=t>>>8&255,e.pending_buf[e.pending++]=255&t;}function Es(e,t){var i,r,n=e.max_chain_length,s=e.strstart,o=e.prev_length,a=e.nice_match,l=e.strstart>e.w_size-ss?e.strstart-(e.w_size-ss):0,h=e.window,u=e.w_mask,c=e.prev,f=e.strstart+ns,d=h[s+o-1],p=h[s+o];e.prev_length>=e.good_match&&(n>>=2),a>e.lookahead&&(a=e.lookahead);do{if(h[(i=t)+o]===p&&h[i+o-1]===d&&h[i]===h[s]&&h[++i]===h[s+1]){s+=2,i++;do{}while(h[++s]===h[++i]&&h[++s]===h[++i]&&h[++s]===h[++i]&&h[++s]===h[++i]&&h[++s]===h[++i]&&h[++s]===h[++i]&&h[++s]===h[++i]&&h[++s]===h[++i]&&s<f);if(r=ns-(f-s),s=f-ns,r>o){if(e.match_start=t,o=r,r>=a)break;d=h[s+o-1],p=h[s+o];}}}while((t=c[t&u])>l&&0!=--n);return o<=e.lookahead?o:e.lookahead}function As(e){var t,i,r,n,s,o,a,l,h,u,c=e.w_size;do{if(n=e.window_size-e.lookahead-e.strstart,e.strstart>=c+(c-ss)){Cr.arraySet(e.window,e.window,c,c,0),e.match_start-=c,e.strstart-=c,e.block_start-=c,t=i=e.hash_size;do{r=e.head[--t],e.head[t]=r>=c?r-c:0;}while(--i);t=i=c;do{r=e.prev[--t],e.prev[t]=r>=c?r-c:0;}while(--i);n+=c;}if(0===e.strm.avail_in)break;if(o=e.strm,a=e.window,l=e.strstart+e.lookahead,h=n,u=void 0,(u=o.avail_in)>h&&(u=h),i=0===u?0:(o.avail_in-=u,Cr.arraySet(a,o.input,o.next_in,u,l),1===o.state.wrap?o.adler=En(o.adler,a,u,l):2===o.state.wrap&&(o.adler=zn(o.adler,a,u,l)),o.next_in+=u,o.total_in+=u,u),e.lookahead+=i,e.lookahead+e.insert>=rs)for(s=e.strstart-e.insert,e.ins_h=e.window[s],e.ins_h=(e.ins_h<<e.hash_shift^e.window[s+1])&e.hash_mask;e.insert&&(e.ins_h=(e.ins_h<<e.hash_shift^e.window[s+rs-1])&e.hash_mask,e.prev[s&e.w_mask]=e.head[e.ins_h],e.head[e.ins_h]=s,s++,e.insert--,!(e.lookahead+e.insert<rs)););}while(e.lookahead<ss&&0!==e.strm.avail_in)}function Ts(e,t){for(var i,r;;){if(e.lookahead<ss){if(As(e),e.lookahead<ss&&t===Rn)return ps;if(0===e.lookahead)break}if(i=0,e.lookahead>=rs&&(e.ins_h=(e.ins_h<<e.hash_shift^e.window[e.strstart+rs-1])&e.hash_mask,i=e.prev[e.strstart&e.w_mask]=e.head[e.ins_h],e.head[e.ins_h]=e.strstart),0!==i&&e.strstart-i<=e.w_size-ss&&(e.match_length=Es(e,i)),e.match_length>=rs)if(r=Cn._tr_tally(e,e.strstart-e.match_start,e.match_length-rs),e.lookahead-=e.match_length,e.match_length<=e.max_lazy_match&&e.lookahead>=rs){e.match_length--;do{e.strstart++,e.ins_h=(e.ins_h<<e.hash_shift^e.window[e.strstart+rs-1])&e.hash_mask,i=e.prev[e.strstart&e.w_mask]=e.head[e.ins_h],e.head[e.ins_h]=e.strstart;}while(0!=--e.match_length);e.strstart++;}else e.strstart+=e.match_length,e.match_length=0,e.ins_h=e.window[e.strstart],e.ins_h=(e.ins_h<<e.hash_shift^e.window[e.strstart+1])&e.hash_mask;else r=Cn._tr_tally(e,0,e.window[e.strstart]),e.lookahead--,e.strstart++;if(r&&(Ss(e,!1),0===e.strm.avail_out))return ps}return e.insert=e.strstart<rs-1?e.strstart:rs-1,t===In?(Ss(e,!0),0===e.strm.avail_out?ms:_s):e.last_lit&&(Ss(e,!1),0===e.strm.avail_out)?ps:gs}function zs(e,t){for(var i,r,n;;){if(e.lookahead<ss){if(As(e),e.lookahead<ss&&t===Rn)return ps;if(0===e.lookahead)break}if(i=0,e.lookahead>=rs&&(e.ins_h=(e.ins_h<<e.hash_shift^e.window[e.strstart+rs-1])&e.hash_mask,i=e.prev[e.strstart&e.w_mask]=e.head[e.ins_h],e.head[e.ins_h]=e.strstart),e.prev_length=e.match_length,e.prev_match=e.match_start,e.match_length=rs-1,0!==i&&e.prev_length<e.max_lazy_match&&e.strstart-i<=e.w_size-ss&&(e.match_length=Es(e,i),e.match_length<=5&&(e.strategy===Zn||e.match_length===rs&&e.strstart-e.match_start>4096)&&(e.match_length=rs-1)),e.prev_length>=rs&&e.match_length<=e.prev_length){n=e.strstart+e.lookahead-rs,r=Cn._tr_tally(e,e.strstart-1-e.prev_match,e.prev_length-rs),e.lookahead-=e.prev_length-1,e.prev_length-=2;do{++e.strstart<=n&&(e.ins_h=(e.ins_h<<e.hash_shift^e.window[e.strstart+rs-1])&e.hash_mask,i=e.prev[e.strstart&e.w_mask]=e.head[e.ins_h],e.head[e.ins_h]=e.strstart);}while(0!=--e.prev_length);if(e.match_available=0,e.match_length=rs-1,e.strstart++,r&&(Ss(e,!1),0===e.strm.avail_out))return ps}else if(e.match_available){if((r=Cn._tr_tally(e,0,e.window[e.strstart-1]))&&Ss(e,!1),e.strstart++,e.lookahead--,0===e.strm.avail_out)return ps}else e.match_available=1,e.strstart++,e.lookahead--;}return e.match_available&&(r=Cn._tr_tally(e,0,e.window[e.strstart-1]),e.match_available=0),e.insert=e.strstart<rs-1?e.strstart:rs-1,t===In?(Ss(e,!0),0===e.strm.avail_out?ms:_s):e.last_lit&&(Ss(e,!1),0===e.strm.avail_out)?ps:gs}function Os(e,t,i,r,n){this.good_length=e,this.max_lazy=t,this.nice_length=i,this.max_chain=r,this.func=n;}function Rs(){this.strm=null,this.status=0,this.pending_buf=null,this.pending_buf_size=0,this.pending_out=0,this.pending=0,this.wrap=0,this.gzhead=null,this.gzindex=0,this.method=$n,this.last_flush=-1,this.w_size=0,this.w_bits=0,this.w_mask=0,this.window=null,this.window_size=0,this.prev=null,this.head=null,this.ins_h=0,this.hash_size=0,this.hash_bits=0,this.hash_mask=0,this.hash_shift=0,this.block_start=0,this.match_length=0,this.prev_match=0,this.match_available=0,this.strstart=0,this.match_start=0,this.lookahead=0,this.prev_length=0,this.max_chain_length=0,this.max_lazy_match=0,this.level=0,this.strategy=0,this.good_match=0,this.nice_match=0,this.dyn_ltree=new Cr.Buf16(2*ts),this.dyn_dtree=new Cr.Buf16(2*(2*Qn+1)),this.bl_tree=new Cr.Buf16(2*(2*es+1)),ws(this.dyn_ltree),ws(this.dyn_dtree),ws(this.bl_tree),this.l_desc=null,this.d_desc=null,this.bl_desc=null,this.bl_count=new Cr.Buf16(is+1),this.heap=new Cr.Buf16(2*Xn+1),ws(this.heap),this.heap_len=0,this.heap_max=0,this.depth=new Cr.Buf16(2*Xn+1),ws(this.depth),this.l_buf=0,this.lit_bufsize=0,this.last_lit=0,this.d_buf=0,this.opt_len=0,this.static_len=0,this.matches=0,this.insert=0,this.bi_buf=0,this.bi_valid=0;}function Ls(e){var t;return e&&e.state?(e.total_in=e.total_out=0,e.data_type=Hn,(t=e.state).pending=0,t.pending_out=0,t.wrap<0&&(t.wrap=-t.wrap),t.status=t.wrap?as:fs,e.adler=2===t.wrap?0:1,t.last_flush=Rn,Cn._tr_init(t),Pn):vs(e,Un)}function Bs(e){var t,i=Ls(e);return i===Pn&&((t=e.state).window_size=2*t.w_size,ws(t.head),t.max_lazy_match=Tn[t.level].max_lazy,t.good_match=Tn[t.level].good_length,t.nice_match=Tn[t.level].nice_length,t.max_chain_length=Tn[t.level].max_chain,t.strstart=0,t.block_start=0,t.lookahead=0,t.insert=0,t.match_length=t.prev_length=rs-1,t.match_available=0,t.ins_h=0),i}function Is(e,t,i,r,n,s){if(!e)return Un;var o=1;if(t===Nn&&(t=6),r<0?(o=0,r=-r):r>15&&(o=2,r-=16),n<1||n>Gn||i!==$n||r<8||r>15||t<0||t>9||s<0||s>Yn)return vs(e,Un);8===r&&(r=9);var a=new Rs;return e.state=a,a.strm=e,a.wrap=o,a.gzhead=null,a.w_bits=r,a.w_size=1<<a.w_bits,a.w_mask=a.w_size-1,a.hash_bits=n+7,a.hash_size=1<<a.hash_bits,a.hash_mask=a.hash_size-1,a.hash_shift=~~((a.hash_bits+rs-1)/rs),a.window=new Cr.Buf8(2*a.w_size),a.head=new Cr.Buf16(a.hash_size),a.prev=new Cr.Buf16(a.w_size),a.lit_bufsize=1<<n+6,a.pending_buf_size=4*a.lit_bufsize,a.pending_buf=new Cr.Buf8(a.pending_buf_size),a.d_buf=1*a.lit_bufsize,a.l_buf=3*a.lit_bufsize,a.level=t,a.strategy=s,a.method=i,Bs(e)}Tn=[new Os(0,0,0,0,function(e,t){var i=65535;for(i>e.pending_buf_size-5&&(i=e.pending_buf_size-5);;){if(e.lookahead<=1){if(As(e),0===e.lookahead&&t===Rn)return ps;if(0===e.lookahead)break}e.strstart+=e.lookahead,e.lookahead=0;var r=e.block_start+i;if((0===e.strstart||e.strstart>=r)&&(e.lookahead=e.strstart-r,e.strstart=r,Ss(e,!1),0===e.strm.avail_out))return ps;if(e.strstart-e.block_start>=e.w_size-ss&&(Ss(e,!1),0===e.strm.avail_out))return ps}return e.insert=0,t===In?(Ss(e,!0),0===e.strm.avail_out?ms:_s):(e.strstart>e.block_start&&(Ss(e,!1),e.strm.avail_out),ps)}),new Os(4,4,8,4,Ts),new Os(4,5,16,8,Ts),new Os(4,6,32,32,Ts),new Os(4,4,16,16,zs),new Os(8,16,32,32,zs),new Os(8,16,128,128,zs),new Os(8,32,128,256,zs),new Os(32,128,258,1024,zs),new Os(32,258,258,4096,zs)];var Fs={deflateInit:function(e,t){return Is(e,t,$n,Kn,Jn,Vn)},deflateInit2:Is,deflateReset:Bs,deflateResetKeep:Ls,deflateSetHeader:function(e,t){return e&&e.state?2!==e.state.wrap?Un:(e.state.gzhead=t,Pn):Un},deflate:function(e,t){var i,r,n,s;if(!e||!e.state||t>Fn||t<0)return e?vs(e,Un):Un;if(r=e.state,!e.output||!e.input&&0!==e.avail_in||r.status===ds&&t!==In)return vs(e,0===e.avail_out?jn:Un);if(r.strm=e,i=r.last_flush,r.last_flush=t,r.status===as)if(2===r.wrap)e.adler=0,xs(r,31),xs(r,139),xs(r,8),r.gzhead?(xs(r,(r.gzhead.text?1:0)+(r.gzhead.hcrc?2:0)+(r.gzhead.extra?4:0)+(r.gzhead.name?8:0)+(r.gzhead.comment?16:0)),xs(r,255&r.gzhead.time),xs(r,r.gzhead.time>>8&255),xs(r,r.gzhead.time>>16&255),xs(r,r.gzhead.time>>24&255),xs(r,9===r.level?2:r.strategy>=Wn||r.level<2?4:0),xs(r,255&r.gzhead.os),r.gzhead.extra&&r.gzhead.extra.length&&(xs(r,255&r.gzhead.extra.length),xs(r,r.gzhead.extra.length>>8&255)),r.gzhead.hcrc&&(e.adler=zn(e.adler,r.pending_buf,r.pending,0)),r.gzindex=0,r.status=ls):(xs(r,0),xs(r,0),xs(r,0),xs(r,0),xs(r,0),xs(r,9===r.level?2:r.strategy>=Wn||r.level<2?4:0),xs(r,ys),r.status=fs);else{var o=$n+(r.w_bits-8<<4)<<8;o|=(r.strategy>=Wn||r.level<2?0:r.level<6?1:6===r.level?2:3)<<6,0!==r.strstart&&(o|=os),o+=31-o%31,r.status=fs,Cs(r,o),0!==r.strstart&&(Cs(r,e.adler>>>16),Cs(r,65535&e.adler)),e.adler=1;}if(r.status===ls)if(r.gzhead.extra){for(n=r.pending;r.gzindex<(65535&r.gzhead.extra.length)&&(r.pending!==r.pending_buf_size||(r.gzhead.hcrc&&r.pending>n&&(e.adler=zn(e.adler,r.pending_buf,r.pending-n,n)),ks(e),n=r.pending,r.pending!==r.pending_buf_size));)xs(r,255&r.gzhead.extra[r.gzindex]),r.gzindex++;r.gzhead.hcrc&&r.pending>n&&(e.adler=zn(e.adler,r.pending_buf,r.pending-n,n)),r.gzindex===r.gzhead.extra.length&&(r.gzindex=0,r.status=hs);}else r.status=hs;if(r.status===hs)if(r.gzhead.name){n=r.pending;do{if(r.pending===r.pending_buf_size&&(r.gzhead.hcrc&&r.pending>n&&(e.adler=zn(e.adler,r.pending_buf,r.pending-n,n)),ks(e),n=r.pending,r.pending===r.pending_buf_size)){s=1;break}s=r.gzindex<r.gzhead.name.length?255&r.gzhead.name.charCodeAt(r.gzindex++):0,xs(r,s);}while(0!==s);r.gzhead.hcrc&&r.pending>n&&(e.adler=zn(e.adler,r.pending_buf,r.pending-n,n)),0===s&&(r.gzindex=0,r.status=us);}else r.status=us;if(r.status===us)if(r.gzhead.comment){n=r.pending;do{if(r.pending===r.pending_buf_size&&(r.gzhead.hcrc&&r.pending>n&&(e.adler=zn(e.adler,r.pending_buf,r.pending-n,n)),ks(e),n=r.pending,r.pending===r.pending_buf_size)){s=1;break}s=r.gzindex<r.gzhead.comment.length?255&r.gzhead.comment.charCodeAt(r.gzindex++):0,xs(r,s);}while(0!==s);r.gzhead.hcrc&&r.pending>n&&(e.adler=zn(e.adler,r.pending_buf,r.pending-n,n)),0===s&&(r.status=cs);}else r.status=cs;if(r.status===cs&&(r.gzhead.hcrc?(r.pending+2>r.pending_buf_size&&ks(e),r.pending+2<=r.pending_buf_size&&(xs(r,255&e.adler),xs(r,e.adler>>8&255),e.adler=0,r.status=fs)):r.status=fs),0!==r.pending){if(ks(e),0===e.avail_out)return r.last_flush=-1,Pn}else if(0===e.avail_in&&bs(t)<=bs(i)&&t!==In)return vs(e,jn);if(r.status===ds&&0!==e.avail_in)return vs(e,jn);if(0!==e.avail_in||0!==r.lookahead||t!==Rn&&r.status!==ds){var a=r.strategy===Wn?function(e,t){for(var i;;){if(0===e.lookahead&&(As(e),0===e.lookahead)){if(t===Rn)return ps;break}if(e.match_length=0,i=Cn._tr_tally(e,0,e.window[e.strstart]),e.lookahead--,e.strstart++,i&&(Ss(e,!1),0===e.strm.avail_out))return ps}return e.insert=0,t===In?(Ss(e,!0),0===e.strm.avail_out?ms:_s):e.last_lit&&(Ss(e,!1),0===e.strm.avail_out)?ps:gs}(r,t):r.strategy===qn?function(e,t){for(var i,r,n,s,o=e.window;;){if(e.lookahead<=ns){if(As(e),e.lookahead<=ns&&t===Rn)return ps;if(0===e.lookahead)break}if(e.match_length=0,e.lookahead>=rs&&e.strstart>0&&(r=o[n=e.strstart-1])===o[++n]&&r===o[++n]&&r===o[++n]){s=e.strstart+ns;do{}while(r===o[++n]&&r===o[++n]&&r===o[++n]&&r===o[++n]&&r===o[++n]&&r===o[++n]&&r===o[++n]&&r===o[++n]&&n<s);e.match_length=ns-(s-n),e.match_length>e.lookahead&&(e.match_length=e.lookahead);}if(e.match_length>=rs?(i=Cn._tr_tally(e,1,e.match_length-rs),e.lookahead-=e.match_length,e.strstart+=e.match_length,e.match_length=0):(i=Cn._tr_tally(e,0,e.window[e.strstart]),e.lookahead--,e.strstart++),i&&(Ss(e,!1),0===e.strm.avail_out))return ps}return e.insert=0,t===In?(Ss(e,!0),0===e.strm.avail_out?ms:_s):e.last_lit&&(Ss(e,!1),0===e.strm.avail_out)?ps:gs}(r,t):Tn[r.level].func(r,t);if(a!==ms&&a!==_s||(r.status=ds),a===ps||a===ms)return 0===e.avail_out&&(r.last_flush=-1),Pn;if(a===gs&&(t===Ln?Cn._tr_align(r):t!==Fn&&(Cn._tr_stored_block(r,0,0,!1),t===Bn&&(ws(r.head),0===r.lookahead&&(r.strstart=0,r.block_start=0,r.insert=0))),ks(e),0===e.avail_out))return r.last_flush=-1,Pn}return t!==In?Pn:r.wrap<=0?Mn:(2===r.wrap?(xs(r,255&e.adler),xs(r,e.adler>>8&255),xs(r,e.adler>>16&255),xs(r,e.adler>>24&255),xs(r,255&e.total_in),xs(r,e.total_in>>8&255),xs(r,e.total_in>>16&255),xs(r,e.total_in>>24&255)):(Cs(r,e.adler>>>16),Cs(r,65535&e.adler)),ks(e),r.wrap>0&&(r.wrap=-r.wrap),0!==r.pending?Pn:Mn)},deflateEnd:function(e){var t;return e&&e.state?(t=e.state.status)!==as&&t!==ls&&t!==hs&&t!==us&&t!==cs&&t!==fs&&t!==ds?vs(e,Un):(e.state=null,t===fs?vs(e,Dn):Pn):Un},deflateSetDictionary:function(e,t){var i,r,n,s,o,a,l,h,u=t.length;if(!e||!e.state)return Un;if(2===(s=(i=e.state).wrap)||1===s&&i.status!==as||i.lookahead)return Un;for(1===s&&(e.adler=En(e.adler,t,u,0)),i.wrap=0,u>=i.w_size&&(0===s&&(ws(i.head),i.strstart=0,i.block_start=0,i.insert=0),h=new Cr.Buf8(i.w_size),Cr.arraySet(h,t,u-i.w_size,i.w_size,0),t=h,u=i.w_size),o=e.avail_in,a=e.next_in,l=e.input,e.avail_in=u,e.next_in=0,e.input=t,As(i);i.lookahead>=rs;){r=i.strstart,n=i.lookahead-(rs-1);do{i.ins_h=(i.ins_h<<i.hash_shift^i.window[r+rs-1])&i.hash_mask,i.prev[r&i.w_mask]=i.head[i.ins_h],i.head[i.ins_h]=r,r++;}while(--n);i.strstart=r,i.lookahead=rs-1,As(i);}return i.strstart+=i.lookahead,i.block_start=i.strstart,i.insert=i.lookahead,i.lookahead=0,i.match_length=i.prev_length=rs-1,i.match_available=0,e.next_in=a,e.input=l,e.avail_in=o,i.wrap=s,Pn},deflateInfo:"pako deflate (from Nodeca project)"},Ps=!0,Ms=!0;try{String.fromCharCode.apply(null,[0]);}catch(e){Ps=!1;}try{String.fromCharCode.apply(null,new Uint8Array(1));}catch(e){Ms=!1;}for(var Us=new Cr.Buf8(256),Ds=0;Ds<256;Ds++)Us[Ds]=Ds>=252?6:Ds>=248?5:Ds>=240?4:Ds>=224?3:Ds>=192?2:1;Us[254]=Us[254]=1;function js(e,t){if(t<65537&&(e.subarray&&Ms||!e.subarray&&Ps))return String.fromCharCode.apply(null,Cr.shrinkBuf(e,t));for(var i="",r=0;r<t;r++)i+=String.fromCharCode(e[r]);return i}var Ns={string2buf:function(e){var t,i,r,n,s,o=e.length,a=0;for(n=0;n<o;n++)55296==(64512&(i=e.charCodeAt(n)))&&n+1<o&&56320==(64512&(r=e.charCodeAt(n+1)))&&(i=65536+(i-55296<<10)+(r-56320),n++),a+=i<128?1:i<2048?2:i<65536?3:4;for(t=new Cr.Buf8(a),s=0,n=0;s<a;n++)55296==(64512&(i=e.charCodeAt(n)))&&n+1<o&&56320==(64512&(r=e.charCodeAt(n+1)))&&(i=65536+(i-55296<<10)+(r-56320),n++),i<128?t[s++]=i:i<2048?(t[s++]=192|i>>>6,t[s++]=128|63&i):i<65536?(t[s++]=224|i>>>12,t[s++]=128|i>>>6&63,t[s++]=128|63&i):(t[s++]=240|i>>>18,t[s++]=128|i>>>12&63,t[s++]=128|i>>>6&63,t[s++]=128|63&i);return t},buf2binstring:function(e){return js(e,e.length)},binstring2buf:function(e){for(var t=new Cr.Buf8(e.length),i=0,r=t.length;i<r;i++)t[i]=e.charCodeAt(i);return t},buf2string:function(e,t){var i,r,n,s,o=t||e.length,a=new Array(2*o);for(r=0,i=0;i<o;)if((n=e[i++])<128)a[r++]=n;else if((s=Us[n])>4)a[r++]=65533,i+=s-1;else{for(n&=2===s?31:3===s?15:7;s>1&&i<o;)n=n<<6|63&e[i++],s--;s>1?a[r++]=65533:n<65536?a[r++]=n:(n-=65536,a[r++]=55296|n>>10&1023,a[r++]=56320|1023&n);}return js(a,r)},utf8border:function(e,t){var i;for((t=t||e.length)>e.length&&(t=e.length),i=t-1;i>=0&&128==(192&e[i]);)i--;return i<0?t:0===i?t:i+Us[e[i]]>t?i:t}};var Zs=function(){this.input=null,this.next_in=0,this.avail_in=0,this.total_in=0,this.output=null,this.next_out=0,this.avail_out=0,this.total_out=0,this.msg="",this.state=null,this.data_type=2,this.adler=0;},Ws=Object.prototype.toString,qs=0,Ys=-1,Vs=0,Hs=8;function $s(e){if(!(this instanceof $s))return new $s(e);this.options=Cr.assign({level:Ys,method:Hs,chunkSize:16384,windowBits:15,memLevel:8,strategy:Vs,to:""},e||{});var t=this.options;t.raw&&t.windowBits>0?t.windowBits=-t.windowBits:t.gzip&&t.windowBits>0&&t.windowBits<16&&(t.windowBits+=16),this.err=0,this.msg="",this.ended=!1,this.chunks=[],this.strm=new Zs,this.strm.avail_out=0;var i=Fs.deflateInit2(this.strm,t.level,t.method,t.windowBits,t.memLevel,t.strategy);if(i!==qs)throw new Error(On[i]);if(t.header&&Fs.deflateSetHeader(this.strm,t.header),t.dictionary){var r;if(r="string"==typeof t.dictionary?Ns.string2buf(t.dictionary):"[object ArrayBuffer]"===Ws.call(t.dictionary)?new Uint8Array(t.dictionary):t.dictionary,(i=Fs.deflateSetDictionary(this.strm,r))!==qs)throw new Error(On[i]);this._dict_set=!0;}}function Gs(e,t){var i=new $s(t);if(i.push(e,!0),i.err)throw i.msg||On[i.err];return i.result}$s.prototype.push=function(e,t){var i,r,n=this.strm,s=this.options.chunkSize;if(this.ended)return !1;r=t===~~t?t:!0===t?4:0,"string"==typeof e?n.input=Ns.string2buf(e):"[object ArrayBuffer]"===Ws.call(e)?n.input=new Uint8Array(e):n.input=e,n.next_in=0,n.avail_in=n.input.length;do{if(0===n.avail_out&&(n.output=new Cr.Buf8(s),n.next_out=0,n.avail_out=s),1!==(i=Fs.deflate(n,r))&&i!==qs)return this.onEnd(i),this.ended=!0,!1;0!==n.avail_out&&(0!==n.avail_in||4!==r&&2!==r)||("string"===this.options.to?this.onData(Ns.buf2binstring(Cr.shrinkBuf(n.output,n.next_out))):this.onData(Cr.shrinkBuf(n.output,n.next_out)));}while((n.avail_in>0||0===n.avail_out)&&1!==i);return 4===r?(i=Fs.deflateEnd(this.strm),this.onEnd(i),this.ended=!0,i===qs):2!==r||(this.onEnd(qs),n.avail_out=0,!0)},$s.prototype.onData=function(e){this.chunks.push(e);},$s.prototype.onEnd=function(e){e===qs&&("string"===this.options.to?this.result=this.chunks.join(""):this.result=Cr.flattenChunks(this.chunks)),this.chunks=[],this.err=e,this.msg=this.strm.msg;};var Ks={Deflate:$s,deflate:Gs,deflateRaw:function(e,t){return (t=t||{}).raw=!0,Gs(e,t)},gzip:function(e,t){return (t=t||{}).gzip=!0,Gs(e,t)}},Js=function(e,t){var i,r,n,s,o,a,l,h,u,c,f,d,p,g,m,_,y,v,b,w,k,S,x,C,E;i=e.state,r=e.next_in,C=e.input,n=r+(e.avail_in-5),s=e.next_out,E=e.output,o=s-(t-e.avail_out),a=s+(e.avail_out-257),l=i.dmax,h=i.wsize,u=i.whave,c=i.wnext,f=i.window,d=i.hold,p=i.bits,g=i.lencode,m=i.distcode,_=(1<<i.lenbits)-1,y=(1<<i.distbits)-1;e:do{p<15&&(d+=C[r++]<<p,p+=8,d+=C[r++]<<p,p+=8),v=g[d&_];t:for(;;){if(d>>>=b=v>>>24,p-=b,0===(b=v>>>16&255))E[s++]=65535&v;else{if(!(16&b)){if(0==(64&b)){v=g[(65535&v)+(d&(1<<b)-1)];continue t}if(32&b){i.mode=12;break e}e.msg="invalid literal/length code",i.mode=30;break e}w=65535&v,(b&=15)&&(p<b&&(d+=C[r++]<<p,p+=8),w+=d&(1<<b)-1,d>>>=b,p-=b),p<15&&(d+=C[r++]<<p,p+=8,d+=C[r++]<<p,p+=8),v=m[d&y];i:for(;;){if(d>>>=b=v>>>24,p-=b,!(16&(b=v>>>16&255))){if(0==(64&b)){v=m[(65535&v)+(d&(1<<b)-1)];continue i}e.msg="invalid distance code",i.mode=30;break e}if(k=65535&v,p<(b&=15)&&(d+=C[r++]<<p,(p+=8)<b&&(d+=C[r++]<<p,p+=8)),(k+=d&(1<<b)-1)>l){e.msg="invalid distance too far back",i.mode=30;break e}if(d>>>=b,p-=b,k>(b=s-o)){if((b=k-b)>u&&i.sane){e.msg="invalid distance too far back",i.mode=30;break e}if(S=0,x=f,0===c){if(S+=h-b,b<w){w-=b;do{E[s++]=f[S++];}while(--b);S=s-k,x=E;}}else if(c<b){if(S+=h+c-b,(b-=c)<w){w-=b;do{E[s++]=f[S++];}while(--b);if(S=0,c<w){w-=b=c;do{E[s++]=f[S++];}while(--b);S=s-k,x=E;}}}else if(S+=c-b,b<w){w-=b;do{E[s++]=f[S++];}while(--b);S=s-k,x=E;}for(;w>2;)E[s++]=x[S++],E[s++]=x[S++],E[s++]=x[S++],w-=3;w&&(E[s++]=x[S++],w>1&&(E[s++]=x[S++]));}else{S=s-k;do{E[s++]=E[S++],E[s++]=E[S++],E[s++]=E[S++],w-=3;}while(w>2);w&&(E[s++]=E[S++],w>1&&(E[s++]=E[S++]));}break}}break}}while(r<n&&s<a);r-=w=p>>3,d&=(1<<(p-=w<<3))-1,e.next_in=r,e.next_out=s,e.avail_in=r<n?n-r+5:5-(r-n),e.avail_out=s<a?a-s+257:257-(s-a),i.hold=d,i.bits=p;},Xs=[3,4,5,6,7,8,9,10,11,13,15,17,19,23,27,31,35,43,51,59,67,83,99,115,131,163,195,227,258,0,0],Qs=[16,16,16,16,16,16,16,16,17,17,17,17,18,18,18,18,19,19,19,19,20,20,20,20,21,21,21,21,16,72,78],eo=[1,2,3,4,5,7,9,13,17,25,33,49,65,97,129,193,257,385,513,769,1025,1537,2049,3073,4097,6145,8193,12289,16385,24577,0,0],to=[16,16,16,16,17,17,18,18,19,19,20,20,21,21,22,22,23,23,24,24,25,25,26,26,27,27,28,28,29,29,64,64],io=function(e,t,i,r,n,s,o,a){var l,h,u,c,f,d,p,g,m,_=a.bits,y=0,v=0,b=0,w=0,k=0,S=0,x=0,C=0,E=0,A=0,T=null,z=0,O=new Cr.Buf16(16),R=new Cr.Buf16(16),L=null,B=0;for(y=0;y<=15;y++)O[y]=0;for(v=0;v<r;v++)O[t[i+v]]++;for(k=_,w=15;w>=1&&0===O[w];w--);if(k>w&&(k=w),0===w)return n[s++]=20971520,n[s++]=20971520,a.bits=1,0;for(b=1;b<w&&0===O[b];b++);for(k<b&&(k=b),C=1,y=1;y<=15;y++)if(C<<=1,(C-=O[y])<0)return -1;if(C>0&&(0===e||1!==w))return -1;for(R[1]=0,y=1;y<15;y++)R[y+1]=R[y]+O[y];for(v=0;v<r;v++)0!==t[i+v]&&(o[R[t[i+v]]++]=v);if(0===e?(T=L=o,d=19):1===e?(T=Xs,z-=257,L=Qs,B-=257,d=256):(T=eo,L=to,d=-1),A=0,v=0,y=b,f=s,S=k,x=0,u=-1,c=(E=1<<k)-1,1===e&&E>852||2===e&&E>592)return 1;for(;;){p=y-x,o[v]<d?(g=0,m=o[v]):o[v]>d?(g=L[B+o[v]],m=T[z+o[v]]):(g=96,m=0),l=1<<y-x,b=h=1<<S;do{n[f+(A>>x)+(h-=l)]=p<<24|g<<16|m|0;}while(0!==h);for(l=1<<y-1;A&l;)l>>=1;if(0!==l?(A&=l-1,A+=l):A=0,v++,0==--O[y]){if(y===w)break;y=t[i+o[v]];}if(y>k&&(A&c)!==u){for(0===x&&(x=k),f+=b,C=1<<(S=y-x);S+x<w&&!((C-=O[S+x])<=0);)S++,C<<=1;if(E+=1<<S,1===e&&E>852||2===e&&E>592)return 1;n[u=A&c]=k<<24|S<<16|f-s|0;}}return 0!==A&&(n[f+A]=y-x<<24|64<<16|0),a.bits=k,0},ro=0,no=1,so=2,oo=4,ao=5,lo=6,ho=0,uo=1,co=2,fo=-2,po=-3,go=-4,mo=-5,_o=8,yo=1,vo=2,bo=3,wo=4,ko=5,So=6,xo=7,Co=8,Eo=9,Ao=10,To=11,zo=12,Oo=13,Ro=14,Lo=15,Bo=16,Io=17,Fo=18,Po=19,Mo=20,Uo=21,Do=22,jo=23,No=24,Zo=25,Wo=26,qo=27,Yo=28,Vo=29,Ho=30,$o=31,Go=32,Ko=852,Jo=592,Xo=15;function Qo(e){return (e>>>24&255)+(e>>>8&65280)+((65280&e)<<8)+((255&e)<<24)}function ea(){this.mode=0,this.last=!1,this.wrap=0,this.havedict=!1,this.flags=0,this.dmax=0,this.check=0,this.total=0,this.head=null,this.wbits=0,this.wsize=0,this.whave=0,this.wnext=0,this.window=null,this.hold=0,this.bits=0,this.length=0,this.offset=0,this.extra=0,this.lencode=null,this.distcode=null,this.lenbits=0,this.distbits=0,this.ncode=0,this.nlen=0,this.ndist=0,this.have=0,this.next=null,this.lens=new Cr.Buf16(320),this.work=new Cr.Buf16(288),this.lendyn=null,this.distdyn=null,this.sane=0,this.back=0,this.was=0;}function ta(e){var t;return e&&e.state?(t=e.state,e.total_in=e.total_out=t.total=0,e.msg="",t.wrap&&(e.adler=1&t.wrap),t.mode=yo,t.last=0,t.havedict=0,t.dmax=32768,t.head=null,t.hold=0,t.bits=0,t.lencode=t.lendyn=new Cr.Buf32(Ko),t.distcode=t.distdyn=new Cr.Buf32(Jo),t.sane=1,t.back=-1,ho):fo}function ia(e){var t;return e&&e.state?((t=e.state).wsize=0,t.whave=0,t.wnext=0,ta(e)):fo}function ra(e,t){var i,r;return e&&e.state?(r=e.state,t<0?(i=0,t=-t):(i=1+(t>>4),t<48&&(t&=15)),t&&(t<8||t>15)?fo:(null!==r.window&&r.wbits!==t&&(r.window=null),r.wrap=i,r.wbits=t,ia(e))):fo}function na(e,t){var i,r;return e?(r=new ea,e.state=r,r.window=null,(i=ra(e,t))!==ho&&(e.state=null),i):fo}var sa,oa,aa=!0;function la(e){if(aa){var t;for(sa=new Cr.Buf32(512),oa=new Cr.Buf32(32),t=0;t<144;)e.lens[t++]=8;for(;t<256;)e.lens[t++]=9;for(;t<280;)e.lens[t++]=7;for(;t<288;)e.lens[t++]=8;for(io(no,e.lens,0,288,sa,0,e.work,{bits:9}),t=0;t<32;)e.lens[t++]=5;io(so,e.lens,0,32,oa,0,e.work,{bits:5}),aa=!1;}e.lencode=sa,e.lenbits=9,e.distcode=oa,e.distbits=5;}function ha(e,t,i,r){var n,s=e.state;return null===s.window&&(s.wsize=1<<s.wbits,s.wnext=0,s.whave=0,s.window=new Cr.Buf8(s.wsize)),r>=s.wsize?(Cr.arraySet(s.window,t,i-s.wsize,s.wsize,0),s.wnext=0,s.whave=s.wsize):((n=s.wsize-s.wnext)>r&&(n=r),Cr.arraySet(s.window,t,i-r,n,s.wnext),(r-=n)?(Cr.arraySet(s.window,t,i-r,r,0),s.wnext=r,s.whave=s.wsize):(s.wnext+=n,s.wnext===s.wsize&&(s.wnext=0),s.whave<s.wsize&&(s.whave+=n))),0}var ua={inflateReset:ia,inflateReset2:ra,inflateResetKeep:ta,inflateInit:function(e){return na(e,Xo)},inflateInit2:na,inflate:function(e,t){var i,r,n,s,o,a,l,h,u,c,f,d,p,g,m,_,y,v,b,w,k,S,x,C,E=0,A=new Cr.Buf8(4),T=[16,17,18,0,8,7,9,6,10,5,11,4,12,3,13,2,14,1,15];if(!e||!e.state||!e.output||!e.input&&0!==e.avail_in)return fo;(i=e.state).mode===zo&&(i.mode=Oo),o=e.next_out,n=e.output,l=e.avail_out,s=e.next_in,r=e.input,a=e.avail_in,h=i.hold,u=i.bits,c=a,f=l,S=ho;e:for(;;)switch(i.mode){case yo:if(0===i.wrap){i.mode=Oo;break}for(;u<16;){if(0===a)break e;a--,h+=r[s++]<<u,u+=8;}if(2&i.wrap&&35615===h){i.check=0,A[0]=255&h,A[1]=h>>>8&255,i.check=zn(i.check,A,2,0),h=0,u=0,i.mode=vo;break}if(i.flags=0,i.head&&(i.head.done=!1),!(1&i.wrap)||(((255&h)<<8)+(h>>8))%31){e.msg="incorrect header check",i.mode=Ho;break}if((15&h)!==_o){e.msg="unknown compression method",i.mode=Ho;break}if(u-=4,k=8+(15&(h>>>=4)),0===i.wbits)i.wbits=k;else if(k>i.wbits){e.msg="invalid window size",i.mode=Ho;break}i.dmax=1<<k,e.adler=i.check=1,i.mode=512&h?Ao:zo,h=0,u=0;break;case vo:for(;u<16;){if(0===a)break e;a--,h+=r[s++]<<u,u+=8;}if(i.flags=h,(255&i.flags)!==_o){e.msg="unknown compression method",i.mode=Ho;break}if(57344&i.flags){e.msg="unknown header flags set",i.mode=Ho;break}i.head&&(i.head.text=h>>8&1),512&i.flags&&(A[0]=255&h,A[1]=h>>>8&255,i.check=zn(i.check,A,2,0)),h=0,u=0,i.mode=bo;case bo:for(;u<32;){if(0===a)break e;a--,h+=r[s++]<<u,u+=8;}i.head&&(i.head.time=h),512&i.flags&&(A[0]=255&h,A[1]=h>>>8&255,A[2]=h>>>16&255,A[3]=h>>>24&255,i.check=zn(i.check,A,4,0)),h=0,u=0,i.mode=wo;case wo:for(;u<16;){if(0===a)break e;a--,h+=r[s++]<<u,u+=8;}i.head&&(i.head.xflags=255&h,i.head.os=h>>8),512&i.flags&&(A[0]=255&h,A[1]=h>>>8&255,i.check=zn(i.check,A,2,0)),h=0,u=0,i.mode=ko;case ko:if(1024&i.flags){for(;u<16;){if(0===a)break e;a--,h+=r[s++]<<u,u+=8;}i.length=h,i.head&&(i.head.extra_len=h),512&i.flags&&(A[0]=255&h,A[1]=h>>>8&255,i.check=zn(i.check,A,2,0)),h=0,u=0;}else i.head&&(i.head.extra=null);i.mode=So;case So:if(1024&i.flags&&((d=i.length)>a&&(d=a),d&&(i.head&&(k=i.head.extra_len-i.length,i.head.extra||(i.head.extra=new Array(i.head.extra_len)),Cr.arraySet(i.head.extra,r,s,d,k)),512&i.flags&&(i.check=zn(i.check,r,d,s)),a-=d,s+=d,i.length-=d),i.length))break e;i.length=0,i.mode=xo;case xo:if(2048&i.flags){if(0===a)break e;d=0;do{k=r[s+d++],i.head&&k&&i.length<65536&&(i.head.name+=String.fromCharCode(k));}while(k&&d<a);if(512&i.flags&&(i.check=zn(i.check,r,d,s)),a-=d,s+=d,k)break e}else i.head&&(i.head.name=null);i.length=0,i.mode=Co;case Co:if(4096&i.flags){if(0===a)break e;d=0;do{k=r[s+d++],i.head&&k&&i.length<65536&&(i.head.comment+=String.fromCharCode(k));}while(k&&d<a);if(512&i.flags&&(i.check=zn(i.check,r,d,s)),a-=d,s+=d,k)break e}else i.head&&(i.head.comment=null);i.mode=Eo;case Eo:if(512&i.flags){for(;u<16;){if(0===a)break e;a--,h+=r[s++]<<u,u+=8;}if(h!==(65535&i.check)){e.msg="header crc mismatch",i.mode=Ho;break}h=0,u=0;}i.head&&(i.head.hcrc=i.flags>>9&1,i.head.done=!0),e.adler=i.check=0,i.mode=zo;break;case Ao:for(;u<32;){if(0===a)break e;a--,h+=r[s++]<<u,u+=8;}e.adler=i.check=Qo(h),h=0,u=0,i.mode=To;case To:if(0===i.havedict)return e.next_out=o,e.avail_out=l,e.next_in=s,e.avail_in=a,i.hold=h,i.bits=u,co;e.adler=i.check=1,i.mode=zo;case zo:if(t===ao||t===lo)break e;case Oo:if(i.last){h>>>=7&u,u-=7&u,i.mode=qo;break}for(;u<3;){if(0===a)break e;a--,h+=r[s++]<<u,u+=8;}switch(i.last=1&h,u-=1,3&(h>>>=1)){case 0:i.mode=Ro;break;case 1:if(la(i),i.mode=Mo,t===lo){h>>>=2,u-=2;break e}break;case 2:i.mode=Io;break;case 3:e.msg="invalid block type",i.mode=Ho;}h>>>=2,u-=2;break;case Ro:for(h>>>=7&u,u-=7&u;u<32;){if(0===a)break e;a--,h+=r[s++]<<u,u+=8;}if((65535&h)!=(h>>>16^65535)){e.msg="invalid stored block lengths",i.mode=Ho;break}if(i.length=65535&h,h=0,u=0,i.mode=Lo,t===lo)break e;case Lo:i.mode=Bo;case Bo:if(d=i.length){if(d>a&&(d=a),d>l&&(d=l),0===d)break e;Cr.arraySet(n,r,s,d,o),a-=d,s+=d,l-=d,o+=d,i.length-=d;break}i.mode=zo;break;case Io:for(;u<14;){if(0===a)break e;a--,h+=r[s++]<<u,u+=8;}if(i.nlen=257+(31&h),h>>>=5,u-=5,i.ndist=1+(31&h),h>>>=5,u-=5,i.ncode=4+(15&h),h>>>=4,u-=4,i.nlen>286||i.ndist>30){e.msg="too many length or distance symbols",i.mode=Ho;break}i.have=0,i.mode=Fo;case Fo:for(;i.have<i.ncode;){for(;u<3;){if(0===a)break e;a--,h+=r[s++]<<u,u+=8;}i.lens[T[i.have++]]=7&h,h>>>=3,u-=3;}for(;i.have<19;)i.lens[T[i.have++]]=0;if(i.lencode=i.lendyn,i.lenbits=7,x={bits:i.lenbits},S=io(ro,i.lens,0,19,i.lencode,0,i.work,x),i.lenbits=x.bits,S){e.msg="invalid code lengths set",i.mode=Ho;break}i.have=0,i.mode=Po;case Po:for(;i.have<i.nlen+i.ndist;){for(;_=(E=i.lencode[h&(1<<i.lenbits)-1])>>>16&255,y=65535&E,!((m=E>>>24)<=u);){if(0===a)break e;a--,h+=r[s++]<<u,u+=8;}if(y<16)h>>>=m,u-=m,i.lens[i.have++]=y;else{if(16===y){for(C=m+2;u<C;){if(0===a)break e;a--,h+=r[s++]<<u,u+=8;}if(h>>>=m,u-=m,0===i.have){e.msg="invalid bit length repeat",i.mode=Ho;break}k=i.lens[i.have-1],d=3+(3&h),h>>>=2,u-=2;}else if(17===y){for(C=m+3;u<C;){if(0===a)break e;a--,h+=r[s++]<<u,u+=8;}u-=m,k=0,d=3+(7&(h>>>=m)),h>>>=3,u-=3;}else{for(C=m+7;u<C;){if(0===a)break e;a--,h+=r[s++]<<u,u+=8;}u-=m,k=0,d=11+(127&(h>>>=m)),h>>>=7,u-=7;}if(i.have+d>i.nlen+i.ndist){e.msg="invalid bit length repeat",i.mode=Ho;break}for(;d--;)i.lens[i.have++]=k;}}if(i.mode===Ho)break;if(0===i.lens[256]){e.msg="invalid code -- missing end-of-block",i.mode=Ho;break}if(i.lenbits=9,x={bits:i.lenbits},S=io(no,i.lens,0,i.nlen,i.lencode,0,i.work,x),i.lenbits=x.bits,S){e.msg="invalid literal/lengths set",i.mode=Ho;break}if(i.distbits=6,i.distcode=i.distdyn,x={bits:i.distbits},S=io(so,i.lens,i.nlen,i.ndist,i.distcode,0,i.work,x),i.distbits=x.bits,S){e.msg="invalid distances set",i.mode=Ho;break}if(i.mode=Mo,t===lo)break e;case Mo:i.mode=Uo;case Uo:if(a>=6&&l>=258){e.next_out=o,e.avail_out=l,e.next_in=s,e.avail_in=a,i.hold=h,i.bits=u,Js(e,f),o=e.next_out,n=e.output,l=e.avail_out,s=e.next_in,r=e.input,a=e.avail_in,h=i.hold,u=i.bits,i.mode===zo&&(i.back=-1);break}for(i.back=0;_=(E=i.lencode[h&(1<<i.lenbits)-1])>>>16&255,y=65535&E,!((m=E>>>24)<=u);){if(0===a)break e;a--,h+=r[s++]<<u,u+=8;}if(_&&0==(240&_)){for(v=m,b=_,w=y;_=(E=i.lencode[w+((h&(1<<v+b)-1)>>v)])>>>16&255,y=65535&E,!(v+(m=E>>>24)<=u);){if(0===a)break e;a--,h+=r[s++]<<u,u+=8;}h>>>=v,u-=v,i.back+=v;}if(h>>>=m,u-=m,i.back+=m,i.length=y,0===_){i.mode=Wo;break}if(32&_){i.back=-1,i.mode=zo;break}if(64&_){e.msg="invalid literal/length code",i.mode=Ho;break}i.extra=15&_,i.mode=Do;case Do:if(i.extra){for(C=i.extra;u<C;){if(0===a)break e;a--,h+=r[s++]<<u,u+=8;}i.length+=h&(1<<i.extra)-1,h>>>=i.extra,u-=i.extra,i.back+=i.extra;}i.was=i.length,i.mode=jo;case jo:for(;_=(E=i.distcode[h&(1<<i.distbits)-1])>>>16&255,y=65535&E,!((m=E>>>24)<=u);){if(0===a)break e;a--,h+=r[s++]<<u,u+=8;}if(0==(240&_)){for(v=m,b=_,w=y;_=(E=i.distcode[w+((h&(1<<v+b)-1)>>v)])>>>16&255,y=65535&E,!(v+(m=E>>>24)<=u);){if(0===a)break e;a--,h+=r[s++]<<u,u+=8;}h>>>=v,u-=v,i.back+=v;}if(h>>>=m,u-=m,i.back+=m,64&_){e.msg="invalid distance code",i.mode=Ho;break}i.offset=y,i.extra=15&_,i.mode=No;case No:if(i.extra){for(C=i.extra;u<C;){if(0===a)break e;a--,h+=r[s++]<<u,u+=8;}i.offset+=h&(1<<i.extra)-1,h>>>=i.extra,u-=i.extra,i.back+=i.extra;}if(i.offset>i.dmax){e.msg="invalid distance too far back",i.mode=Ho;break}i.mode=Zo;case Zo:if(0===l)break e;if(d=f-l,i.offset>d){if((d=i.offset-d)>i.whave&&i.sane){e.msg="invalid distance too far back",i.mode=Ho;break}d>i.wnext?(d-=i.wnext,p=i.wsize-d):p=i.wnext-d,d>i.length&&(d=i.length),g=i.window;}else g=n,p=o-i.offset,d=i.length;d>l&&(d=l),l-=d,i.length-=d;do{n[o++]=g[p++];}while(--d);0===i.length&&(i.mode=Uo);break;case Wo:if(0===l)break e;n[o++]=i.length,l--,i.mode=Uo;break;case qo:if(i.wrap){for(;u<32;){if(0===a)break e;a--,h|=r[s++]<<u,u+=8;}if(f-=l,e.total_out+=f,i.total+=f,f&&(e.adler=i.check=i.flags?zn(i.check,n,f,o-f):En(i.check,n,f,o-f)),f=l,(i.flags?h:Qo(h))!==i.check){e.msg="incorrect data check",i.mode=Ho;break}h=0,u=0;}i.mode=Yo;case Yo:if(i.wrap&&i.flags){for(;u<32;){if(0===a)break e;a--,h+=r[s++]<<u,u+=8;}if(h!==(4294967295&i.total)){e.msg="incorrect length check",i.mode=Ho;break}h=0,u=0;}i.mode=Vo;case Vo:S=uo;break e;case Ho:S=po;break e;case $o:return go;case Go:default:return fo}return e.next_out=o,e.avail_out=l,e.next_in=s,e.avail_in=a,i.hold=h,i.bits=u,(i.wsize||f!==e.avail_out&&i.mode<Ho&&(i.mode<qo||t!==oo))&&ha(e,e.output,e.next_out,f-e.avail_out),c-=e.avail_in,f-=e.avail_out,e.total_in+=c,e.total_out+=f,i.total+=f,i.wrap&&f&&(e.adler=i.check=i.flags?zn(i.check,n,f,e.next_out-f):En(i.check,n,f,e.next_out-f)),e.data_type=i.bits+(i.last?64:0)+(i.mode===zo?128:0)+(i.mode===Mo||i.mode===Lo?256:0),(0===c&&0===f||t===oo)&&S===ho&&(S=mo),S},inflateEnd:function(e){if(!e||!e.state)return fo;var t=e.state;return t.window&&(t.window=null),e.state=null,ho},inflateGetHeader:function(e,t){var i;return e&&e.state?0==(2&(i=e.state).wrap)?fo:(i.head=t,t.done=!1,ho):fo},inflateSetDictionary:function(e,t){var i,r=t.length;return e&&e.state?0!==(i=e.state).wrap&&i.mode!==To?fo:i.mode===To&&En(1,t,r,0)!==i.check?po:ha(e,t,r,r)?(i.mode=$o,go):(i.havedict=1,ho):fo},inflateInfo:"pako inflate (from Nodeca project)"},ca={Z_NO_FLUSH:0,Z_PARTIAL_FLUSH:1,Z_SYNC_FLUSH:2,Z_FULL_FLUSH:3,Z_FINISH:4,Z_BLOCK:5,Z_TREES:6,Z_OK:0,Z_STREAM_END:1,Z_NEED_DICT:2,Z_ERRNO:-1,Z_STREAM_ERROR:-2,Z_DATA_ERROR:-3,Z_BUF_ERROR:-5,Z_NO_COMPRESSION:0,Z_BEST_SPEED:1,Z_BEST_COMPRESSION:9,Z_DEFAULT_COMPRESSION:-1,Z_FILTERED:1,Z_HUFFMAN_ONLY:2,Z_RLE:3,Z_FIXED:4,Z_DEFAULT_STRATEGY:0,Z_BINARY:0,Z_TEXT:1,Z_UNKNOWN:2,Z_DEFLATED:8};var fa=function(){this.text=0,this.time=0,this.xflags=0,this.os=0,this.extra=null,this.extra_len=0,this.name="",this.comment="",this.hcrc=0,this.done=!1;},da=Object.prototype.toString;function pa(e){if(!(this instanceof pa))return new pa(e);this.options=Cr.assign({chunkSize:16384,windowBits:0,to:""},e||{});var t=this.options;t.raw&&t.windowBits>=0&&t.windowBits<16&&(t.windowBits=-t.windowBits,0===t.windowBits&&(t.windowBits=-15)),!(t.windowBits>=0&&t.windowBits<16)||e&&e.windowBits||(t.windowBits+=32),t.windowBits>15&&t.windowBits<48&&0==(15&t.windowBits)&&(t.windowBits|=15),this.err=0,this.msg="",this.ended=!1,this.chunks=[],this.strm=new Zs,this.strm.avail_out=0;var i=ua.inflateInit2(this.strm,t.windowBits);if(i!==ca.Z_OK)throw new Error(On[i]);this.header=new fa,ua.inflateGetHeader(this.strm,this.header);}function ga(e,t){var i=new pa(t);if(i.push(e,!0),i.err)throw i.msg||On[i.err];return i.result}pa.prototype.push=function(e,t){var i,r,n,s,o,a,l=this.strm,h=this.options.chunkSize,u=this.options.dictionary,c=!1;if(this.ended)return !1;r=t===~~t?t:!0===t?ca.Z_FINISH:ca.Z_NO_FLUSH,"string"==typeof e?l.input=Ns.binstring2buf(e):"[object ArrayBuffer]"===da.call(e)?l.input=new Uint8Array(e):l.input=e,l.next_in=0,l.avail_in=l.input.length;do{if(0===l.avail_out&&(l.output=new Cr.Buf8(h),l.next_out=0,l.avail_out=h),(i=ua.inflate(l,ca.Z_NO_FLUSH))===ca.Z_NEED_DICT&&u&&(a="string"==typeof u?Ns.string2buf(u):"[object ArrayBuffer]"===da.call(u)?new Uint8Array(u):u,i=ua.inflateSetDictionary(this.strm,a)),i===ca.Z_BUF_ERROR&&!0===c&&(i=ca.Z_OK,c=!1),i!==ca.Z_STREAM_END&&i!==ca.Z_OK)return this.onEnd(i),this.ended=!0,!1;l.next_out&&(0!==l.avail_out&&i!==ca.Z_STREAM_END&&(0!==l.avail_in||r!==ca.Z_FINISH&&r!==ca.Z_SYNC_FLUSH)||("string"===this.options.to?(n=Ns.utf8border(l.output,l.next_out),s=l.next_out-n,o=Ns.buf2string(l.output,n),l.next_out=s,l.avail_out=h-s,s&&Cr.arraySet(l.output,l.output,n,s,0),this.onData(o)):this.onData(Cr.shrinkBuf(l.output,l.next_out)))),0===l.avail_in&&0===l.avail_out&&(c=!0);}while((l.avail_in>0||0===l.avail_out)&&i!==ca.Z_STREAM_END);return i===ca.Z_STREAM_END&&(r=ca.Z_FINISH),r===ca.Z_FINISH?(i=ua.inflateEnd(this.strm),this.onEnd(i),this.ended=!0,i===ca.Z_OK):r!==ca.Z_SYNC_FLUSH||(this.onEnd(ca.Z_OK),l.avail_out=0,!0)},pa.prototype.onData=function(e){this.chunks.push(e);},pa.prototype.onEnd=function(e){e===ca.Z_OK&&("string"===this.options.to?this.result=this.chunks.join(""):this.result=Cr.flattenChunks(this.chunks)),this.chunks=[],this.err=e,this.msg=this.strm.msg;};var ma={Inflate:pa,inflate:ga,inflateRaw:function(e,t){return (t=t||{}).raw=!0,ga(e,t)},ungzip:ga},_a={};(0,Cr.assign)(_a,Ks,ma,ca);var ya=_a,va="undefined"!=typeof Uint8Array&&"undefined"!=typeof Uint16Array&&"undefined"!=typeof Uint32Array?"uint8array":"array";function ba(e,t){Xi.call(this,"FlateWorker/"+e),this._pako=null,this._pakoAction=e,this._pakoOptions=t,this.meta={};}Ki.inherits(ba,Xi),ba.prototype.processChunk=function(e){this.meta=e.meta,null===this._pako&&this._createPako(),this._pako.push(Ki.transformTo(va,e.data),!1);},ba.prototype.flush=function(){Xi.prototype.flush.call(this),null===this._pako&&this._createPako(),this._pako.push([],!0);},ba.prototype.cleanUp=function(){Xi.prototype.cleanUp.call(this),this._pako=null;},ba.prototype._createPako=function(){this._pako=new ya[this._pakoAction]({raw:!0,level:this._pakoOptions.level||-1});var e=this;this._pako.onData=function(t){e.push({data:t,meta:e.meta});};};var wa={STORE:{magic:"\0\0",compressWorker:function(e){return new Xi("STORE compression")},uncompressWorker:function(){return new Xi("STORE decompression")}},DEFLATE:{magic:"\b\0",compressWorker:function(e){return new ba("Deflate",e)},uncompressWorker:function(){return new ba("Inflate",{})}}},ka="PK",Sa="PK",xa="PK",Ca="PK",Ea="PK",Aa="PK\b",Ta=function(e,t){var i,r="";for(i=0;i<t;i++)r+=String.fromCharCode(255&e),e>>>=8;return r},za=function(e,t,i,r,n,s){var o,a,l=e.file,h=e.compression,u=s!==Qi.utf8encode,c=Ki.transformTo("string",s(l.name)),f=Ki.transformTo("string",Qi.utf8encode(l.name)),d=l.comment,p=Ki.transformTo("string",s(d)),g=Ki.transformTo("string",Qi.utf8encode(d)),m=f.length!==l.name.length,_=g.length!==d.length,y="",v="",b="",w=l.dir,k=l.date,S={crc32:0,compressedSize:0,uncompressedSize:0};t&&!i||(S.crc32=e.crc32,S.compressedSize=e.compressedSize,S.uncompressedSize=e.uncompressedSize);var x=0;t&&(x|=8),u||!m&&!_||(x|=2048);var C,E,A,T=0,z=0;w&&(T|=16),"UNIX"===n?(z=798,T|=(C=l.unixPermissions,E=w,A=C,C||(A=E?16893:33204),(65535&A)<<16)):(z=20,T|=63&(l.dosPermissions||0)),o=k.getUTCHours(),o<<=6,o|=k.getUTCMinutes(),o<<=5,o|=k.getUTCSeconds()/2,a=k.getUTCFullYear()-1980,a<<=4,a|=k.getUTCMonth()+1,a<<=5,a|=k.getUTCDate(),m&&(v=Ta(1,1)+Ta(gr(c),4)+f,y+="up"+Ta(v.length,2)+v),_&&(b=Ta(1,1)+Ta(gr(p),4)+g,y+="uc"+Ta(b.length,2)+b);var O="";return O+="\n\0",O+=Ta(x,2),O+=h.magic,O+=Ta(o,2),O+=Ta(a,2),O+=Ta(S.crc32,4),O+=Ta(S.compressedSize,4),O+=Ta(S.uncompressedSize,4),O+=Ta(c.length,2),O+=Ta(y.length,2),{fileRecord:ka+O+c+y,dirRecord:Sa+Ta(z,2)+O+Ta(p.length,2)+"\0\0\0\0"+Ta(T,4)+Ta(r,4)+c+y+p}},Oa=function(e){return Aa+Ta(e.crc32,4)+Ta(e.compressedSize,4)+Ta(e.uncompressedSize,4)};function Ra(e,t,i,r){Xi.call(this,"ZipFileWorker"),this.bytesWritten=0,this.zipComment=t,this.zipPlatform=i,this.encodeFileName=r,this.streamFiles=e,this.accumulate=!1,this.contentBuffer=[],this.dirRecords=[],this.currentSourceOffset=0,this.entriesCount=0,this.currentFile=null,this._sources=[];}Ki.inherits(Ra,Xi),Ra.prototype.push=function(e){var t=e.meta.percent||0,i=this.entriesCount,r=this._sources.length;this.accumulate?this.contentBuffer.push(e):(this.bytesWritten+=e.data.length,Xi.prototype.push.call(this,{data:e.data,meta:{currentFile:this.currentFile,percent:i?(t+100*(i-r-1))/i:100}}));},Ra.prototype.openedSource=function(e){this.currentSourceOffset=this.bytesWritten,this.currentFile=e.file.name;var t=this.streamFiles&&!e.file.dir;if(t){var i=za(e,t,!1,this.currentSourceOffset,this.zipPlatform,this.encodeFileName);this.push({data:i.fileRecord,meta:{percent:0}});}else this.accumulate=!0;},Ra.prototype.closedSource=function(e){this.accumulate=!1;var t=this.streamFiles&&!e.file.dir,i=za(e,t,!0,this.currentSourceOffset,this.zipPlatform,this.encodeFileName);if(this.dirRecords.push(i.dirRecord),t)this.push({data:Oa(e),meta:{percent:100}});else for(this.push({data:i.fileRecord,meta:{percent:0}});this.contentBuffer.length;)this.push(this.contentBuffer.shift());this.currentFile=null;},Ra.prototype.flush=function(){for(var e=this.bytesWritten,t=0;t<this.dirRecords.length;t++)this.push({data:this.dirRecords[t],meta:{percent:100}});var i=this.bytesWritten-e,r=function(e,t,i,r,n){var s=Ki.transformTo("string",n(r));return xa+"\0\0\0\0"+Ta(e,2)+Ta(e,2)+Ta(t,4)+Ta(i,4)+Ta(s.length,2)+s}(this.dirRecords.length,i,e,this.zipComment,this.encodeFileName);this.push({data:r,meta:{percent:100}});},Ra.prototype.prepareNextSource=function(){this.previous=this._sources.shift(),this.openedSource(this.previous.streamInfo),this.isPaused?this.previous.pause():this.previous.resume();},Ra.prototype.registerPrevious=function(e){this._sources.push(e);var t=this;return e.on("data",function(e){t.processChunk(e);}),e.on("end",function(){t.closedSource(t.previous.streamInfo),t._sources.length?t.prepareNextSource():t.end();}),e.on("error",function(e){t.error(e);}),this},Ra.prototype.resume=function(){return !!Xi.prototype.resume.call(this)&&(!this.previous&&this._sources.length?(this.prepareNextSource(),!0):this.previous||this._sources.length||this.generatedError?void 0:(this.end(),!0))},Ra.prototype.error=function(e){var t=this._sources;if(!Xi.prototype.error.call(this,e))return !1;for(var i=0;i<t.length;i++)try{t[i].error(e);}catch(e){}return !0},Ra.prototype.lock=function(){Xi.prototype.lock.call(this);for(var e=this._sources,t=0;t<e.length;t++)e[t].lock();};var La=Ra,Ba=function(e,t,i){var r=new La(t.streamFiles,i,t.platform,t.encodeFileName),n=0;try{e.forEach(function(e,i){n++;var s=function(e,t){var i=e||t,r=wa[i];if(!r)throw new Error(i+" is not a valid compression method !");return r}(i.options.compression,t.compression),o=i.options.compressionOptions||t.compressionOptions||{},a=i.dir,l=i.date;i._compressWorker(s,o).withStreamInfo("file",{name:e,dir:a,date:l,comment:i.comment||"",unixPermissions:i.unixPermissions,dosPermissions:i.dosPermissions}).pipe(r);}),r.entriesCount=n;}catch(e){r.error(e);}return r};function Ia(e,t){Xi.call(this,"Nodejs stream input adapter for "+e),this._upstreamEnded=!1,this._bindStream(t);}Ki.inherits(Ia,Xi),Ia.prototype._bindStream=function(e){var t=this;this._stream=e,e.pause(),e.on("data",function(e){t.push({data:e,meta:{percent:0}});}).on("error",function(e){t.isPaused?this.generatedError=e:t.error(e);}).on("end",function(){t.isPaused?t._upstreamEnded=!0:t.end();});},Ia.prototype.pause=function(){return !!Xi.prototype.pause.call(this)&&(this._stream.pause(),!0)},Ia.prototype.resume=function(){return !!Xi.prototype.resume.call(this)&&(this._upstreamEnded?this.end():this._stream.resume(),!0)};var Fa=Ia,Pa=function(e,t,i){var r,n=Ki.getTypeOf(t),s=Ki.extend(i||{},hr);s.date=s.date||new Date,null!==s.compression&&(s.compression=s.compression.toUpperCase()),"string"==typeof s.unixPermissions&&(s.unixPermissions=parseInt(s.unixPermissions,8)),s.unixPermissions&&16384&s.unixPermissions&&(s.dir=!0),s.dosPermissions&&16&s.dosPermissions&&(s.dir=!0),s.dir&&(e=Ua(e)),s.createFolders&&(r=Ma(e))&&Da.call(this,r,!0);var o="string"===n&&!1===s.binary&&!1===s.base64;i&&void 0!==i.binary||(s.binary=!o),(t instanceof vr&&0===t.uncompressedSize||s.dir||!t||0===t.length)&&(s.base64=!1,s.binary=!0,t="",s.compression="STORE",n="string");var a=null;a=t instanceof vr||t instanceof Xi?t:qt&&$t(t)?new Fa(e,t):Ki.prepareContent(e,t,s.binary,s.optimizedBinaryString,s.base64);var l=new xr(e,a,s);this.files[e]=l;},Ma=function(e){"/"===e.slice(-1)&&(e=e.substring(0,e.length-1));var t=e.lastIndexOf("/");return t>0?e.substring(0,t):""},Ua=function(e){return "/"!==e.slice(-1)&&(e+="/"),e},Da=function(e,t){return t=void 0!==t?t:hr.createFolders,e=Ua(e),this.files[e]||Pa.call(this,e,null,{dir:!0,createFolders:t}),this.files[e]};function ja(e){return "[object RegExp]"===Object.prototype.toString.call(e)}var Na={load:function(){throw new Error("This method has been removed in JSZip 3.0, please check the upgrade guide.")},forEach:function(e){var t,i,r;for(t in this.files)this.files.hasOwnProperty(t)&&(r=this.files[t],(i=t.slice(this.root.length,t.length))&&t.slice(0,this.root.length)===this.root&&e(i,r));},filter:function(e){var t=[];return this.forEach(function(i,r){e(i,r)&&t.push(r);}),t},file:function(e,t,i){if(1===arguments.length){if(ja(e)){var r=e;return this.filter(function(e,t){return !t.dir&&r.test(e)})}var n=this.files[this.root+e];return n&&!n.dir?n:null}return e=this.root+e,Pa.call(this,e,t,i),this},folder:function(e){if(!e)return this;if(ja(e))return this.filter(function(t,i){return i.dir&&e.test(t)});var t=this.root+e,i=Da.call(this,t),r=this.clone();return r.root=i.name,r},remove:function(e){e=this.root+e;var t=this.files[e];if(t||("/"!==e.slice(-1)&&(e+="/"),t=this.files[e]),t&&!t.dir)delete this.files[e];else for(var i=this.filter(function(t,i){return i.name.slice(0,e.length)===e}),r=0;r<i.length;r++)delete this.files[i[r].name];return this},generate:function(e){throw new Error("This method has been removed in JSZip 3.0, please check the upgrade guide.")},generateInternalStream:function(e){var t,i={};try{if((i=Ki.extend(e||{},{streamFiles:!1,compression:"STORE",compressionOptions:null,type:"",platform:"DOS",comment:null,mimeType:"application/zip",encodeFileName:Qi.utf8encode})).type=i.type.toLowerCase(),i.compression=i.compression.toUpperCase(),"binarystring"===i.type&&(i.type="string"),!i.type)throw new Error("No output type specified.");Ki.checkSupport(i.type),"darwin"!==i.platform&&"freebsd"!==i.platform&&"linux"!==i.platform&&"sunos"!==i.platform||(i.platform="UNIX"),"win32"===i.platform&&(i.platform="DOS");var r=i.comment||this.comment||"";t=Ba(this,i,r);}catch(e){(t=new Xi("error")).error(e);}return new lr(t,i.type||"string",i.mimeType)},generateAsync:function(e,t){return this.generateInternalStream(e).accumulate(t)},generateNodeStream:function(e,t){return (e=e||{}).type||(e.type="nodebuffer"),this.generateInternalStream(e).toNodejsStream(t)}};function Za(e){this.data=e,this.length=e.length,this.index=0,this.zero=0;}Za.prototype={checkOffset:function(e){this.checkIndex(this.index+e);},checkIndex:function(e){if(this.length<this.zero+e||e<0)throw new Error("End of data reached (data length = "+this.length+", asked index = "+e+"). Corrupted zip ?")},setIndex:function(e){this.checkIndex(e),this.index=e;},skip:function(e){this.setIndex(this.index+e);},byteAt:function(e){},readInt:function(e){var t,i=0;for(this.checkOffset(e),t=this.index+e-1;t>=this.index;t--)i=(i<<8)+this.byteAt(t);return this.index+=e,i},readString:function(e){return Ki.transformTo("string",this.readData(e))},readData:function(e){},lastIndexOfSignature:function(e){},readAndCheckSignature:function(e){},readDate:function(){var e=this.readInt(4);return new Date(Date.UTC(1980+(e>>25&127),(e>>21&15)-1,e>>16&31,e>>11&31,e>>5&63,(31&e)<<1))}};var Wa=Za;function qa(e){Wa.call(this,e);for(var t=0;t<this.data.length;t++)e[t]=255&e[t];}Ki.inherits(qa,Wa),qa.prototype.byteAt=function(e){return this.data[this.zero+e]},qa.prototype.lastIndexOfSignature=function(e){for(var t=e.charCodeAt(0),i=e.charCodeAt(1),r=e.charCodeAt(2),n=e.charCodeAt(3),s=this.length-4;s>=0;--s)if(this.data[s]===t&&this.data[s+1]===i&&this.data[s+2]===r&&this.data[s+3]===n)return s-this.zero;return -1},qa.prototype.readAndCheckSignature=function(e){var t=e.charCodeAt(0),i=e.charCodeAt(1),r=e.charCodeAt(2),n=e.charCodeAt(3),s=this.readData(4);return t===s[0]&&i===s[1]&&r===s[2]&&n===s[3]},qa.prototype.readData=function(e){if(this.checkOffset(e),0===e)return [];var t=this.data.slice(this.zero+this.index,this.zero+this.index+e);return this.index+=e,t};var Ya=qa;function Va(e){Wa.call(this,e);}Ki.inherits(Va,Wa),Va.prototype.byteAt=function(e){return this.data.charCodeAt(this.zero+e)},Va.prototype.lastIndexOfSignature=function(e){return this.data.lastIndexOf(e)-this.zero},Va.prototype.readAndCheckSignature=function(e){return e===this.readData(4)},Va.prototype.readData=function(e){this.checkOffset(e);var t=this.data.slice(this.zero+this.index,this.zero+this.index+e);return this.index+=e,t};var Ha=Va;function $a(e){Ya.call(this,e);}Ki.inherits($a,Ya),$a.prototype.readData=function(e){if(this.checkOffset(e),0===e)return new Uint8Array(0);var t=this.data.subarray(this.zero+this.index,this.zero+this.index+e);return this.index+=e,t};var Ga=$a;function Ka(e){Ga.call(this,e);}Ki.inherits(Ka,Ga),Ka.prototype.readData=function(e){this.checkOffset(e);var t=this.data.slice(this.zero+this.index,this.zero+this.index+e);return this.index+=e,t};var Ja=Ka,Xa=function(e){var t=Ki.getTypeOf(e);return Ki.checkSupport(t),"string"!==t||Nt.uint8array?"nodebuffer"===t?new Ja(e):Nt.uint8array?new Ga(Ki.transformTo("uint8array",e)):new Ya(Ki.transformTo("array",e)):new Ha(e)};function Qa(e,t){this.options=e,this.loadOptions=t;}Qa.prototype={isEncrypted:function(){return 1==(1&this.bitFlag)},useUTF8:function(){return 2048==(2048&this.bitFlag)},readLocalPart:function(e){var t,i;if(e.skip(22),this.fileNameLength=e.readInt(2),i=e.readInt(2),this.fileName=e.readData(this.fileNameLength),e.skip(i),-1===this.compressedSize||-1===this.uncompressedSize)throw new Error("Bug or corrupted zip : didn't get enough informations from the central directory (compressedSize === -1 || uncompressedSize === -1)");if(null===(t=function(e){for(var t in wa)if(wa.hasOwnProperty(t)&&wa[t].magic===e)return wa[t];return null}(this.compressionMethod)))throw new Error("Corrupted zip : compression "+Ki.pretty(this.compressionMethod)+" unknown (inner file : "+Ki.transformTo("string",this.fileName)+")");this.decompressed=new vr(this.compressedSize,this.uncompressedSize,this.crc32,t,e.readData(this.compressedSize));},readCentralPart:function(e){this.versionMadeBy=e.readInt(2),e.skip(2),this.bitFlag=e.readInt(2),this.compressionMethod=e.readString(2),this.date=e.readDate(),this.crc32=e.readInt(4),this.compressedSize=e.readInt(4),this.uncompressedSize=e.readInt(4);var t=e.readInt(2);if(this.extraFieldsLength=e.readInt(2),this.fileCommentLength=e.readInt(2),this.diskNumberStart=e.readInt(2),this.internalFileAttributes=e.readInt(2),this.externalFileAttributes=e.readInt(4),this.localHeaderOffset=e.readInt(4),this.isEncrypted())throw new Error("Encrypted zip are not supported");e.skip(t),this.readExtraFields(e),this.parseZIP64ExtraField(e),this.fileComment=e.readData(this.fileCommentLength);},processAttributes:function(){this.unixPermissions=null,this.dosPermissions=null;var e=this.versionMadeBy>>8;this.dir=!!(16&this.externalFileAttributes),0===e&&(this.dosPermissions=63&this.externalFileAttributes),3===e&&(this.unixPermissions=this.externalFileAttributes>>16&65535),this.dir||"/"!==this.fileNameStr.slice(-1)||(this.dir=!0);},parseZIP64ExtraField:function(e){if(this.extraFields[1]){var t=Xa(this.extraFields[1].value);this.uncompressedSize===Ki.MAX_VALUE_32BITS&&(this.uncompressedSize=t.readInt(8)),this.compressedSize===Ki.MAX_VALUE_32BITS&&(this.compressedSize=t.readInt(8)),this.localHeaderOffset===Ki.MAX_VALUE_32BITS&&(this.localHeaderOffset=t.readInt(8)),this.diskNumberStart===Ki.MAX_VALUE_32BITS&&(this.diskNumberStart=t.readInt(4));}},readExtraFields:function(e){var t,i,r,n=e.index+this.extraFieldsLength;for(this.extraFields||(this.extraFields={});e.index<n;)t=e.readInt(2),i=e.readInt(2),r=e.readData(i),this.extraFields[t]={id:t,length:i,value:r};},handleUTF8:function(){var e=Nt.uint8array?"uint8array":"array";if(this.useUTF8())this.fileNameStr=Qi.utf8decode(this.fileName),this.fileCommentStr=Qi.utf8decode(this.fileComment);else{var t=this.findExtraFieldUnicodePath();if(null!==t)this.fileNameStr=t;else{var i=Ki.transformTo(e,this.fileName);this.fileNameStr=this.loadOptions.decodeFileName(i);}var r=this.findExtraFieldUnicodeComment();if(null!==r)this.fileCommentStr=r;else{var n=Ki.transformTo(e,this.fileComment);this.fileCommentStr=this.loadOptions.decodeFileName(n);}}},findExtraFieldUnicodePath:function(){var e=this.extraFields[28789];if(e){var t=Xa(e.value);return 1!==t.readInt(1)?null:gr(this.fileName)!==t.readInt(4)?null:Qi.utf8decode(t.readData(e.length-5))}return null},findExtraFieldUnicodeComment:function(){var e=this.extraFields[25461];if(e){var t=Xa(e.value);return 1!==t.readInt(1)?null:gr(this.fileComment)!==t.readInt(4)?null:Qi.utf8decode(t.readData(e.length-5))}return null}};var el=Qa;function tl(e){this.files=[],this.loadOptions=e;}tl.prototype={checkSignature:function(e){if(!this.reader.readAndCheckSignature(e)){this.reader.index-=4;var t=this.reader.readString(4);throw new Error("Corrupted zip or bug: unexpected signature ("+Ki.pretty(t)+", expected "+Ki.pretty(e)+")")}},isSignature:function(e,t){var i=this.reader.index;this.reader.setIndex(e);var r=this.reader.readString(4)===t;return this.reader.setIndex(i),r},readBlockEndOfCentral:function(){this.diskNumber=this.reader.readInt(2),this.diskWithCentralDirStart=this.reader.readInt(2),this.centralDirRecordsOnThisDisk=this.reader.readInt(2),this.centralDirRecords=this.reader.readInt(2),this.centralDirSize=this.reader.readInt(4),this.centralDirOffset=this.reader.readInt(4),this.zipCommentLength=this.reader.readInt(2);var e=this.reader.readData(this.zipCommentLength),t=Nt.uint8array?"uint8array":"array",i=Ki.transformTo(t,e);this.zipComment=this.loadOptions.decodeFileName(i);},readBlockZip64EndOfCentral:function(){this.zip64EndOfCentralSize=this.reader.readInt(8),this.reader.skip(4),this.diskNumber=this.reader.readInt(4),this.diskWithCentralDirStart=this.reader.readInt(4),this.centralDirRecordsOnThisDisk=this.reader.readInt(8),this.centralDirRecords=this.reader.readInt(8),this.centralDirSize=this.reader.readInt(8),this.centralDirOffset=this.reader.readInt(8),this.zip64ExtensibleData={};for(var e,t,i,r=this.zip64EndOfCentralSize-44;0<r;)e=this.reader.readInt(2),t=this.reader.readInt(4),i=this.reader.readData(t),this.zip64ExtensibleData[e]={id:e,length:t,value:i};},readBlockZip64EndOfCentralLocator:function(){if(this.diskWithZip64CentralDirStart=this.reader.readInt(4),this.relativeOffsetEndOfZip64CentralDir=this.reader.readInt(8),this.disksCount=this.reader.readInt(4),this.disksCount>1)throw new Error("Multi-volumes zip are not supported")},readLocalFiles:function(){var e,t;for(e=0;e<this.files.length;e++)t=this.files[e],this.reader.setIndex(t.localHeaderOffset),this.checkSignature(ka),t.readLocalPart(this.reader),t.handleUTF8(),t.processAttributes();},readCentralDir:function(){var e;for(this.reader.setIndex(this.centralDirOffset);this.reader.readAndCheckSignature(Sa);)(e=new el({zip64:this.zip64},this.loadOptions)).readCentralPart(this.reader),this.files.push(e);if(this.centralDirRecords!==this.files.length&&0!==this.centralDirRecords&&0===this.files.length)throw new Error("Corrupted zip or bug: expected "+this.centralDirRecords+" records in central dir, got "+this.files.length)},readEndOfCentral:function(){var e=this.reader.lastIndexOfSignature(xa);if(e<0)throw!this.isSignature(0,ka)?new Error("Can't find end of central directory : is this a zip file ? If it is, see https://stuk.github.io/jszip/documentation/howto/read_zip.html"):new Error("Corrupted zip: can't find end of central directory");this.reader.setIndex(e);var t=e;if(this.checkSignature(xa),this.readBlockEndOfCentral(),this.diskNumber===Ki.MAX_VALUE_16BITS||this.diskWithCentralDirStart===Ki.MAX_VALUE_16BITS||this.centralDirRecordsOnThisDisk===Ki.MAX_VALUE_16BITS||this.centralDirRecords===Ki.MAX_VALUE_16BITS||this.centralDirSize===Ki.MAX_VALUE_32BITS||this.centralDirOffset===Ki.MAX_VALUE_32BITS){if(this.zip64=!0,(e=this.reader.lastIndexOfSignature(Ca))<0)throw new Error("Corrupted zip: can't find the ZIP64 end of central directory locator");if(this.reader.setIndex(e),this.checkSignature(Ca),this.readBlockZip64EndOfCentralLocator(),!this.isSignature(this.relativeOffsetEndOfZip64CentralDir,Ea)&&(this.relativeOffsetEndOfZip64CentralDir=this.reader.lastIndexOfSignature(Ea),this.relativeOffsetEndOfZip64CentralDir<0))throw new Error("Corrupted zip: can't find the ZIP64 end of central directory");this.reader.setIndex(this.relativeOffsetEndOfZip64CentralDir),this.checkSignature(Ea),this.readBlockZip64EndOfCentral();}var i=this.centralDirOffset+this.centralDirSize;this.zip64&&(i+=20,i+=12+this.zip64EndOfCentralSize);var r=t-i;if(r>0)this.isSignature(t,Sa)||(this.reader.zero=r);else if(r<0)throw new Error("Corrupted zip: missing "+Math.abs(r)+" bytes.")},prepareReader:function(e){this.reader=Xa(e);},load:function(e){this.prepareReader(e),this.readEndOfCentral(),this.readCentralDir(),this.readLocalFiles();}};var il=tl;function rl(e){return new Gi.Promise(function(t,i){var r=e.decompressed.getContentWorker().pipe(new _r);r.on("error",function(e){i(e);}).on("end",function(){r.streamInfo.crc32!==e.decompressed.crc32?i(new Error("Corrupted zip : CRC32 mismatch")):t();}).resume();})}function nl(){if(!(this instanceof nl))return new nl;if(arguments.length)throw new Error("The constructor with parameters has been removed in JSZip 3.0, please check the upgrade guide.");this.files={},this.comment=null,this.root="",this.clone=function(){var e=new nl;for(var t in this)"function"!=typeof this[t]&&(e[t]=this[t]);return e};}nl.prototype=Na,nl.prototype.loadAsync=function(e,t){var i=this;return t=Ki.extend(t||{},{base64:!1,checkCRC32:!1,optimizedBinaryString:!1,createFolders:!1,decodeFileName:Qi.utf8decode}),qt&&$t(e)?Gi.Promise.reject(new Error("JSZip can't accept a stream when loading a zip file.")):Ki.prepareContent("the loaded zip file",e,!0,t.optimizedBinaryString,t.base64).then(function(e){var i=new il(t);return i.load(e),i}).then(function(e){var i=[Gi.Promise.resolve(e)],r=e.files;if(t.checkCRC32)for(var n=0;n<r.length;n++)i.push(rl(r[n]));return Gi.Promise.all(i)}).then(function(e){for(var r=e.shift(),n=r.files,s=0;s<n.length;s++){var o=n[s];i.file(o.fileNameStr,o.decompressed,{binary:!0,optimizedBinaryString:!0,date:o.date,dir:o.dir,comment:o.fileCommentStr.length?o.fileCommentStr:null,unixPermissions:o.unixPermissions,dosPermissions:o.dosPermissions,createFolders:t.createFolders});}return r.zipComment.length&&(i.comment=r.zipComment),i})},nl.support=Nt,nl.defaults=hr,nl.version="3.1.5",nl.loadAsync=function(e,t){return (new nl).loadAsync(e,t)},nl.external=Gi;var sl=nl;function ol(e){return null==e}var al={isNothing:ol,isObject:function(e){return "object"==typeof e&&null!==e},toArray:function(e){return Array.isArray(e)?e:ol(e)?[]:[e]},repeat:function(e,t){var i,r="";for(i=0;i<t;i+=1)r+=e;return r},isNegativeZero:function(e){return 0===e&&Number.NEGATIVE_INFINITY===1/e},extend:function(e,t){var i,r,n,s;if(t)for(i=0,r=(s=Object.keys(t)).length;i<r;i+=1)e[n=s[i]]=t[n];return e}};function ll(e,t){Error.call(this),Error.captureStackTrace?Error.captureStackTrace(this,this.constructor):this.stack=(new Error).stack||"",this.name="YAMLException",this.reason=e,this.mark=t,this.message=(this.reason||"(unknown reason)")+(this.mark?" "+this.mark.toString():"");}ll.prototype=Object.create(Error.prototype),ll.prototype.constructor=ll,ll.prototype.toString=function(e){var t=this.name+": ";return t+=this.reason||"(unknown reason)",!e&&this.mark&&(t+=" "+this.mark.toString()),t};var hl=ll;function ul(e,t,i,r,n){this.name=e,this.buffer=t,this.position=i,this.line=r,this.column=n;}ul.prototype.getSnippet=function(e,t){var i,r,n,s,o;if(!this.buffer)return null;for(e=e||4,t=t||75,i="",r=this.position;r>0&&-1==="\0\r\n\u2028\u2029".indexOf(this.buffer.charAt(r-1));)if(r-=1,this.position-r>t/2-1){i=" ... ",r+=5;break}for(n="",s=this.position;s<this.buffer.length&&-1==="\0\r\n\u2028\u2029".indexOf(this.buffer.charAt(s));)if((s+=1)-this.position>t/2-1){n=" ... ",s-=5;break}return o=this.buffer.slice(r,s),al.repeat(" ",e)+i+o+n+"\n"+al.repeat(" ",e+this.position-r+i.length)+"^"},ul.prototype.toString=function(e){var t,i="";return this.name&&(i+='in "'+this.name+'" '),i+="at line "+(this.line+1)+", column "+(this.column+1),e||(t=this.getSnippet())&&(i+=":\n"+t),i};var cl=ul,fl=["kind","resolve","construct","instanceOf","predicate","represent","defaultStyle","styleAliases"],dl=["scalar","sequence","mapping"];var pl=function(e,t){var i,r;if(t=t||{},Object.keys(t).forEach(function(t){if(-1===fl.indexOf(t))throw new hl('Unknown option "'+t+'" is met in definition of "'+e+'" YAML type.')}),this.tag=e,this.kind=t.kind||null,this.resolve=t.resolve||function(){return !0},this.construct=t.construct||function(e){return e},this.instanceOf=t.instanceOf||null,this.predicate=t.predicate||null,this.represent=t.represent||null,this.defaultStyle=t.defaultStyle||null,this.styleAliases=(i=t.styleAliases||null,r={},null!==i&&Object.keys(i).forEach(function(e){i[e].forEach(function(t){r[String(t)]=e;});}),r),-1===dl.indexOf(this.kind))throw new hl('Unknown kind "'+this.kind+'" is specified for "'+e+'" YAML type.')};function gl(e,t,i){var r=[];return e.include.forEach(function(e){i=gl(e,t,i);}),e[t].forEach(function(e){i.forEach(function(t,i){t.tag===e.tag&&r.push(i);}),i.push(e);}),i.filter(function(e,t){return -1===r.indexOf(t)})}function ml(e){this.include=e.include||[],this.implicit=e.implicit||[],this.explicit=e.explicit||[],this.implicit.forEach(function(e){if(e.loadKind&&"scalar"!==e.loadKind)throw new hl("There is a non-scalar type in the implicit list of a schema. Implicit resolving of such types is not supported.")}),this.compiledImplicit=gl(this,"implicit",[]),this.compiledExplicit=gl(this,"explicit",[]),this.compiledTypeMap=function(){var e,t,i={};function r(e){i[e.tag]=e;}for(e=0,t=arguments.length;e<t;e+=1)arguments[e].forEach(r);return i}(this.compiledImplicit,this.compiledExplicit);}ml.DEFAULT=null,ml.create=function(){var e,t;switch(arguments.length){case 1:e=ml.DEFAULT,t=arguments[0];break;case 2:e=arguments[0],t=arguments[1];break;default:throw new hl("Wrong number of arguments for Schema.create function")}if(e=al.toArray(e),t=al.toArray(t),!e.every(function(e){return e instanceof ml}))throw new hl("Specified list of super schemas (or a single Schema object) contains a non-Schema object.");if(!t.every(function(e){return e instanceof pl}))throw new hl("Specified list of YAML types (or a single Type object) contains a non-Type object.");return new ml({include:e,explicit:t})};var _l=ml,yl=new _l({explicit:[new pl("tag:yaml.org,2002:str",{kind:"scalar",construct:function(e){return null!==e?e:""}}),new pl("tag:yaml.org,2002:seq",{kind:"sequence",construct:function(e){return null!==e?e:[]}}),new pl("tag:yaml.org,2002:map",{kind:"mapping",construct:function(e){return null!==e?e:{}}})]});var vl=new pl("tag:yaml.org,2002:null",{kind:"scalar",resolve:function(e){if(null===e)return !0;var t=e.length;return 1===t&&"~"===e||4===t&&("null"===e||"Null"===e||"NULL"===e)},construct:function(){return null},predicate:function(e){return null===e},represent:{canonical:function(){return "~"},lowercase:function(){return "null"},uppercase:function(){return "NULL"},camelcase:function(){return "Null"}},defaultStyle:"lowercase"});var bl=new pl("tag:yaml.org,2002:bool",{kind:"scalar",resolve:function(e){if(null===e)return !1;var t=e.length;return 4===t&&("true"===e||"True"===e||"TRUE"===e)||5===t&&("false"===e||"False"===e||"FALSE"===e)},construct:function(e){return "true"===e||"True"===e||"TRUE"===e},predicate:function(e){return "[object Boolean]"===Object.prototype.toString.call(e)},represent:{lowercase:function(e){return e?"true":"false"},uppercase:function(e){return e?"TRUE":"FALSE"},camelcase:function(e){return e?"True":"False"}},defaultStyle:"lowercase"});function wl(e){return 48<=e&&e<=55}function kl(e){return 48<=e&&e<=57}var Sl=new pl("tag:yaml.org,2002:int",{kind:"scalar",resolve:function(e){if(null===e)return !1;var t,i,r=e.length,n=0,s=!1;if(!r)return !1;if("-"!==(t=e[n])&&"+"!==t||(t=e[++n]),"0"===t){if(n+1===r)return !0;if("b"===(t=e[++n])){for(n++;n<r;n++)if("_"!==(t=e[n])){if("0"!==t&&"1"!==t)return !1;s=!0;}return s}if("x"===t){for(n++;n<r;n++)if("_"!==(t=e[n])){if(!(48<=(i=e.charCodeAt(n))&&i<=57||65<=i&&i<=70||97<=i&&i<=102))return !1;s=!0;}return s}for(;n<r;n++)if("_"!==(t=e[n])){if(!wl(e.charCodeAt(n)))return !1;s=!0;}return s}for(;n<r;n++)if("_"!==(t=e[n])){if(":"===t)break;if(!kl(e.charCodeAt(n)))return !1;s=!0;}return !!s&&(":"!==t||/^(:[0-5]?[0-9])+$/.test(e.slice(n)))},construct:function(e){var t,i,r=e,n=1,s=[];return -1!==r.indexOf("_")&&(r=r.replace(/_/g,"")),"-"!==(t=r[0])&&"+"!==t||("-"===t&&(n=-1),t=(r=r.slice(1))[0]),"0"===r?0:"0"===t?"b"===r[1]?n*parseInt(r.slice(2),2):"x"===r[1]?n*parseInt(r,16):n*parseInt(r,8):-1!==r.indexOf(":")?(r.split(":").forEach(function(e){s.unshift(parseInt(e,10));}),r=0,i=1,s.forEach(function(e){r+=e*i,i*=60;}),n*r):n*parseInt(r,10)},predicate:function(e){return "[object Number]"===Object.prototype.toString.call(e)&&e%1==0&&!al.isNegativeZero(e)},represent:{binary:function(e){return "0b"+e.toString(2)},octal:function(e){return "0"+e.toString(8)},decimal:function(e){return e.toString(10)},hexadecimal:function(e){return "0x"+e.toString(16).toUpperCase()}},defaultStyle:"decimal",styleAliases:{binary:[2,"bin"],octal:[8,"oct"],decimal:[10,"dec"],hexadecimal:[16,"hex"]}}),xl=new RegExp("^(?:[-+]?(?:[0-9][0-9_]*)\\.[0-9_]*(?:[eE][-+][0-9]+)?|\\.[0-9_]+(?:[eE][-+][0-9]+)?|[-+]?[0-9][0-9_]*(?::[0-5]?[0-9])+\\.[0-9_]*|[-+]?\\.(?:inf|Inf|INF)|\\.(?:nan|NaN|NAN))$");var Cl=/^[-+]?[0-9]+e/;var El=new _l({include:[yl],implicit:[vl,bl,Sl,new pl("tag:yaml.org,2002:float",{kind:"scalar",resolve:function(e){return null!==e&&!!xl.test(e)},construct:function(e){var t,i,r,n;return i="-"===(t=e.replace(/_/g,"").toLowerCase())[0]?-1:1,n=[],"+-".indexOf(t[0])>=0&&(t=t.slice(1)),".inf"===t?1===i?Number.POSITIVE_INFINITY:Number.NEGATIVE_INFINITY:".nan"===t?NaN:t.indexOf(":")>=0?(t.split(":").forEach(function(e){n.unshift(parseFloat(e,10));}),t=0,r=1,n.forEach(function(e){t+=e*r,r*=60;}),i*t):i*parseFloat(t,10)},predicate:function(e){return "[object Number]"===Object.prototype.toString.call(e)&&(e%1!=0||al.isNegativeZero(e))},represent:function(e,t){var i;if(isNaN(e))switch(t){case"lowercase":return ".nan";case"uppercase":return ".NAN";case"camelcase":return ".NaN"}else if(Number.POSITIVE_INFINITY===e)switch(t){case"lowercase":return ".inf";case"uppercase":return ".INF";case"camelcase":return ".Inf"}else if(Number.NEGATIVE_INFINITY===e)switch(t){case"lowercase":return "-.inf";case"uppercase":return "-.INF";case"camelcase":return "-.Inf"}else if(al.isNegativeZero(e))return "-0.0";return i=e.toString(10),Cl.test(i)?i.replace("e",".e"):i},defaultStyle:"lowercase"})]}),Al=new _l({include:[El]}),Tl=new RegExp("^([0-9][0-9][0-9][0-9])-([0-9][0-9]?)-([0-9][0-9]?)(?:(?:[Tt]|[ \\t]+)([0-9][0-9]?):([0-9][0-9]):([0-9][0-9])(?:\\.([0-9]*))?(?:[ \\t]*(Z|([-+])([0-9][0-9]?)(?::([0-9][0-9]))?))?)?$");var zl=new pl("tag:yaml.org,2002:timestamp",{kind:"scalar",resolve:function(e){return null!==e&&null!==Tl.exec(e)},construct:function(e){var t,i,r,n,s,o,a,l,h=0,u=null;if(null===(t=Tl.exec(e)))throw new Error("Date resolve error");if(i=+t[1],r=+t[2]-1,n=+t[3],!t[4])return new Date(Date.UTC(i,r,n));if(s=+t[4],o=+t[5],a=+t[6],t[7]){for(h=t[7].slice(0,3);h.length<3;)h+="0";h=+h;}return t[9]&&(u=6e4*(60*+t[10]+ +(t[11]||0)),"-"===t[9]&&(u=-u)),l=new Date(Date.UTC(i,r,n,s,o,a,h)),u&&l.setTime(l.getTime()-u),l},instanceOf:Date,represent:function(e){return e.toISOString()}});var Ol=new pl("tag:yaml.org,2002:merge",{kind:"scalar",resolve:function(e){return "<<"===e||null===e}}),Rl={}.Buffer,Ll="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=\n\r";var Bl=new pl("tag:yaml.org,2002:binary",{kind:"scalar",resolve:function(e){if(null===e)return !1;var t,i,r=0,n=e.length,s=Ll;for(i=0;i<n;i++)if(!((t=s.indexOf(e.charAt(i)))>64)){if(t<0)return !1;r+=6;}return r%8==0},construct:function(e){var t,i,r=e.replace(/[\r\n=]/g,""),n=r.length,s=Ll,o=0,a=[];for(t=0;t<n;t++)t%4==0&&t&&(a.push(o>>16&255),a.push(o>>8&255),a.push(255&o)),o=o<<6|s.indexOf(r.charAt(t));return 0==(i=n%4*6)?(a.push(o>>16&255),a.push(o>>8&255),a.push(255&o)):18===i?(a.push(o>>10&255),a.push(o>>2&255)):12===i&&a.push(o>>4&255),a},predicate:function(e){return Rl&&Rl.isBuffer(e)},represent:function(e){var t,i,r="",n=0,s=e.length,o=Ll;for(t=0;t<s;t++)t%3==0&&t&&(r+=o[n>>18&63],r+=o[n>>12&63],r+=o[n>>6&63],r+=o[63&n]),n=(n<<8)+e[t];return 0==(i=s%3)?(r+=o[n>>18&63],r+=o[n>>12&63],r+=o[n>>6&63],r+=o[63&n]):2===i?(r+=o[n>>10&63],r+=o[n>>4&63],r+=o[n<<2&63],r+=o[64]):1===i&&(r+=o[n>>2&63],r+=o[n<<4&63],r+=o[64],r+=o[64]),r}}),Il=Object.prototype.hasOwnProperty,Fl=Object.prototype.toString;var Pl=new pl("tag:yaml.org,2002:omap",{kind:"sequence",resolve:function(e){if(null===e)return !0;var t,i,r,n,s,o=[],a=e;for(t=0,i=a.length;t<i;t+=1){if(r=a[t],s=!1,"[object Object]"!==Fl.call(r))return !1;for(n in r)if(Il.call(r,n)){if(s)return !1;s=!0;}if(!s)return !1;if(-1!==o.indexOf(n))return !1;o.push(n);}return !0},construct:function(e){return null!==e?e:[]}}),Ml=Object.prototype.toString;var Ul=new pl("tag:yaml.org,2002:pairs",{kind:"sequence",resolve:function(e){if(null===e)return !0;var t,i,r,n,s,o=e;for(s=new Array(o.length),t=0,i=o.length;t<i;t+=1){if(r=o[t],"[object Object]"!==Ml.call(r))return !1;if(1!==(n=Object.keys(r)).length)return !1;s[t]=[n[0],r[n[0]]];}return !0},construct:function(e){if(null===e)return [];var t,i,r,n,s,o=e;for(s=new Array(o.length),t=0,i=o.length;t<i;t+=1)r=o[t],n=Object.keys(r),s[t]=[n[0],r[n[0]]];return s}}),Dl=Object.prototype.hasOwnProperty;var jl=new _l({include:[Al],implicit:[zl,Ol],explicit:[Bl,Pl,Ul,new pl("tag:yaml.org,2002:set",{kind:"mapping",resolve:function(e){if(null===e)return !0;var t,i=e;for(t in i)if(Dl.call(i,t)&&null!==i[t])return !1;return !0},construct:function(e){return null!==e?e:{}}})]});var Nl=new pl("tag:yaml.org,2002:js/undefined",{kind:"scalar",resolve:function(){return !0},construct:function(){},predicate:function(e){return void 0===e},represent:function(){return ""}});var Zl,Wl=new pl("tag:yaml.org,2002:js/regexp",{kind:"scalar",resolve:function(e){if(null===e)return !1;if(0===e.length)return !1;var t=e,i=/\/([gim]*)$/.exec(e),r="";if("/"===t[0]){if(i&&(r=i[1]),r.length>3)return !1;if("/"!==t[t.length-r.length-1])return !1}return !0},construct:function(e){var t=e,i=/\/([gim]*)$/.exec(e),r="";return "/"===t[0]&&(i&&(r=i[1]),t=t.slice(1,t.length-r.length-1)),new RegExp(t,r)},predicate:function(e){return "[object RegExp]"===Object.prototype.toString.call(e)},represent:function(e){var t="/"+e.source+"/";return e.global&&(t+="g"),e.multiline&&(t+="m"),e.ignoreCase&&(t+="i"),t}});try{var ql=e.commonjsRequire;Zl=ql("esprima");}catch(e){"undefined"!=typeof window&&(Zl=window.esprima);}var Yl=new pl("tag:yaml.org,2002:js/function",{kind:"scalar",resolve:function(e){if(null===e)return !1;try{var t="("+e+")",i=Zl.parse(t,{range:!0});return "Program"===i.type&&1===i.body.length&&"ExpressionStatement"===i.body[0].type&&"FunctionExpression"===i.body[0].expression.type}catch(e){return !1}},construct:function(e){var t,i="("+e+")",r=Zl.parse(i,{range:!0}),n=[];if("Program"!==r.type||1!==r.body.length||"ExpressionStatement"!==r.body[0].type||"FunctionExpression"!==r.body[0].expression.type)throw new Error("Failed to resolve function");return r.body[0].expression.params.forEach(function(e){n.push(e.name);}),t=r.body[0].expression.body.range,new Function(n,i.slice(t[0]+1,t[1]-1))},predicate:function(e){return "[object Function]"===Object.prototype.toString.call(e)},represent:function(e){return e.toString()}}),Vl=_l.DEFAULT=new _l({include:[jl],explicit:[Nl,Wl,Yl]}),Hl=Object.prototype.hasOwnProperty,$l=1,Gl=2,Kl=3,Jl=4,Xl=1,Ql=2,eh=3,th=/[\x00-\x08\x0B\x0C\x0E-\x1F\x7F-\x84\x86-\x9F\uFFFE\uFFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF]/,ih=/[\x85\u2028\u2029]/,rh=/[,\[\]\{\}]/,nh=/^(?:!|!!|![a-z\-]+!)$/i,sh=/^(?:!|[^,\[\]\{\}])(?:%[0-9a-f]{2}|[0-9a-z\-#;\/\?:@&=\+\$,_\.!~\*'\(\)\[\]])*$/i;function oh(e){return 10===e||13===e}function ah(e){return 9===e||32===e}function lh(e){return 9===e||32===e||10===e||13===e}function hh(e){return 44===e||91===e||93===e||123===e||125===e}function uh(e){var t;return 48<=e&&e<=57?e-48:97<=(t=32|e)&&t<=102?t-97+10:-1}function ch(e){return 48===e?"\0":97===e?"":98===e?"\b":116===e?"\t":9===e?"\t":110===e?"\n":118===e?"\v":102===e?"\f":114===e?"\r":101===e?"":32===e?" ":34===e?'"':47===e?"/":92===e?"\\":78===e?"":95===e?" ":76===e?"\u2028":80===e?"\u2029":""}function fh(e){return e<=65535?String.fromCharCode(e):String.fromCharCode(55296+(e-65536>>10),56320+(e-65536&1023))}for(var dh=new Array(256),ph=new Array(256),gh=0;gh<256;gh++)dh[gh]=ch(gh)?1:0,ph[gh]=ch(gh);function mh(e,t){this.input=e,this.filename=t.filename||null,this.schema=t.schema||Vl,this.onWarning=t.onWarning||null,this.legacy=t.legacy||!1,this.json=t.json||!1,this.listener=t.listener||null,this.implicitTypes=this.schema.compiledImplicit,this.typeMap=this.schema.compiledTypeMap,this.length=e.length,this.position=0,this.line=0,this.lineStart=0,this.lineIndent=0,this.documents=[];}function _h(e,t){return new hl(t,new cl(e.filename,e.input,e.position,e.line,e.position-e.lineStart))}function yh(e,t){throw _h(e,t)}function vh(e,t){e.onWarning&&e.onWarning.call(null,_h(e,t));}var bh={YAML:function(e,t,i){var r,n,s;null!==e.version&&yh(e,"duplication of %YAML directive"),1!==i.length&&yh(e,"YAML directive accepts exactly one argument"),null===(r=/^([0-9]+)\.([0-9]+)$/.exec(i[0]))&&yh(e,"ill-formed argument of the YAML directive"),n=parseInt(r[1],10),s=parseInt(r[2],10),1!==n&&yh(e,"unacceptable YAML version of the document"),e.version=i[0],e.checkLineBreaks=s<2,1!==s&&2!==s&&vh(e,"unsupported YAML version of the document");},TAG:function(e,t,i){var r,n;2!==i.length&&yh(e,"TAG directive accepts exactly two arguments"),r=i[0],n=i[1],nh.test(r)||yh(e,"ill-formed tag handle (first argument) of the TAG directive"),Hl.call(e.tagMap,r)&&yh(e,'there is a previously declared suffix for "'+r+'" tag handle'),sh.test(n)||yh(e,"ill-formed tag prefix (second argument) of the TAG directive"),e.tagMap[r]=n;}};function wh(e,t,i,r){var n,s,o,a;if(t<i){if(a=e.input.slice(t,i),r)for(n=0,s=a.length;n<s;n+=1)9===(o=a.charCodeAt(n))||32<=o&&o<=1114111||yh(e,"expected valid JSON character");else th.test(a)&&yh(e,"the stream contains non-printable characters");e.result+=a;}}function kh(e,t,i,r){var n,s,o,a;for(al.isObject(i)||yh(e,"cannot merge mappings; the provided source object is unacceptable"),o=0,a=(n=Object.keys(i)).length;o<a;o+=1)s=n[o],Hl.call(t,s)||(t[s]=i[s],r[s]=!0);}function Sh(e,t,i,r,n,s){var o,a;if(n=String(n),null===t&&(t={}),"tag:yaml.org,2002:merge"===r)if(Array.isArray(s))for(o=0,a=s.length;o<a;o+=1)kh(e,t,s[o],i);else kh(e,t,s,i);else e.json||Hl.call(i,n)||!Hl.call(t,n)||yh(e,"duplicated mapping key"),t[n]=s,delete i[n];return t}function xh(e){var t;10===(t=e.input.charCodeAt(e.position))?e.position++:13===t?(e.position++,10===e.input.charCodeAt(e.position)&&e.position++):yh(e,"a line break is expected"),e.line+=1,e.lineStart=e.position;}function Ch(e,t,i){for(var r=0,n=e.input.charCodeAt(e.position);0!==n;){for(;ah(n);)n=e.input.charCodeAt(++e.position);if(t&&35===n)do{n=e.input.charCodeAt(++e.position);}while(10!==n&&13!==n&&0!==n);if(!oh(n))break;for(xh(e),n=e.input.charCodeAt(e.position),r++,e.lineIndent=0;32===n;)e.lineIndent++,n=e.input.charCodeAt(++e.position);}return -1!==i&&0!==r&&e.lineIndent<i&&vh(e,"deficient indentation"),r}function Eh(e){var t,i=e.position;return !(45!==(t=e.input.charCodeAt(i))&&46!==t||t!==e.input.charCodeAt(i+1)||t!==e.input.charCodeAt(i+2)||(i+=3,0!==(t=e.input.charCodeAt(i))&&!lh(t)))}function Ah(e,t){1===t?e.result+=" ":t>1&&(e.result+=al.repeat("\n",t-1));}function Th(e,t){var i,r,n=e.tag,s=e.anchor,o=[],a=!1;for(null!==e.anchor&&(e.anchorMap[e.anchor]=o),r=e.input.charCodeAt(e.position);0!==r&&45===r&&lh(e.input.charCodeAt(e.position+1));)if(a=!0,e.position++,Ch(e,!0,-1)&&e.lineIndent<=t)o.push(null),r=e.input.charCodeAt(e.position);else if(i=e.line,Rh(e,t,Kl,!1,!0),o.push(e.result),Ch(e,!0,-1),r=e.input.charCodeAt(e.position),(e.line===i||e.lineIndent>t)&&0!==r)yh(e,"bad indentation of a sequence entry");else if(e.lineIndent<t)break;return !!a&&(e.tag=n,e.anchor=s,e.kind="sequence",e.result=o,!0)}function zh(e){var t,i,r,n,s=!1,o=!1;if(33!==(n=e.input.charCodeAt(e.position)))return !1;if(null!==e.tag&&yh(e,"duplication of a tag property"),60===(n=e.input.charCodeAt(++e.position))?(s=!0,n=e.input.charCodeAt(++e.position)):33===n?(o=!0,i="!!",n=e.input.charCodeAt(++e.position)):i="!",t=e.position,s){do{n=e.input.charCodeAt(++e.position);}while(0!==n&&62!==n);e.position<e.length?(r=e.input.slice(t,e.position),n=e.input.charCodeAt(++e.position)):yh(e,"unexpected end of the stream within a verbatim tag");}else{for(;0!==n&&!lh(n);)33===n&&(o?yh(e,"tag suffix cannot contain exclamation marks"):(i=e.input.slice(t-1,e.position+1),nh.test(i)||yh(e,"named tag handle cannot contain such characters"),o=!0,t=e.position+1)),n=e.input.charCodeAt(++e.position);r=e.input.slice(t,e.position),rh.test(r)&&yh(e,"tag suffix cannot contain flow indicator characters");}return r&&!sh.test(r)&&yh(e,"tag name cannot contain such characters: "+r),s?e.tag=r:Hl.call(e.tagMap,i)?e.tag=e.tagMap[i]+r:"!"===i?e.tag="!"+r:"!!"===i?e.tag="tag:yaml.org,2002:"+r:yh(e,'undeclared tag handle "'+i+'"'),!0}function Oh(e){var t,i;if(38!==(i=e.input.charCodeAt(e.position)))return !1;for(null!==e.anchor&&yh(e,"duplication of an anchor property"),i=e.input.charCodeAt(++e.position),t=e.position;0!==i&&!lh(i)&&!hh(i);)i=e.input.charCodeAt(++e.position);return e.position===t&&yh(e,"name of an anchor node must contain at least one character"),e.anchor=e.input.slice(t,e.position),!0}function Rh(e,t,i,r,n){var s,o,a,l,h,u,c,f,d=1,p=!1,g=!1;if(null!==e.listener&&e.listener("open",e),e.tag=null,e.anchor=null,e.kind=null,e.result=null,s=o=a=Jl===i||Kl===i,r&&Ch(e,!0,-1)&&(p=!0,e.lineIndent>t?d=1:e.lineIndent===t?d=0:e.lineIndent<t&&(d=-1)),1===d)for(;zh(e)||Oh(e);)Ch(e,!0,-1)?(p=!0,a=s,e.lineIndent>t?d=1:e.lineIndent===t?d=0:e.lineIndent<t&&(d=-1)):a=!1;if(a&&(a=p||n),1!==d&&Jl!==i||(c=$l===i||Gl===i?t:t+1,f=e.position-e.lineStart,1===d?a&&(Th(e,f)||function(e,t,i){var r,n,s,o,a=e.tag,l=e.anchor,h={},u={},c=null,f=null,d=null,p=!1,g=!1;for(null!==e.anchor&&(e.anchorMap[e.anchor]=h),o=e.input.charCodeAt(e.position);0!==o;){if(r=e.input.charCodeAt(e.position+1),s=e.line,63!==o&&58!==o||!lh(r)){if(!Rh(e,i,Gl,!1,!0))break;if(e.line===s){for(o=e.input.charCodeAt(e.position);ah(o);)o=e.input.charCodeAt(++e.position);if(58===o)lh(o=e.input.charCodeAt(++e.position))||yh(e,"a whitespace character is expected after the key-value separator within a block mapping"),p&&(Sh(e,h,u,c,f,null),c=f=d=null),g=!0,p=!1,n=!1,c=e.tag,f=e.result;else{if(!g)return e.tag=a,e.anchor=l,!0;yh(e,"can not read an implicit mapping pair; a colon is missed");}}else{if(!g)return e.tag=a,e.anchor=l,!0;yh(e,"can not read a block mapping entry; a multiline key may not be an implicit key");}}else 63===o?(p&&(Sh(e,h,u,c,f,null),c=f=d=null),g=!0,p=!0,n=!0):p?(p=!1,n=!0):yh(e,"incomplete explicit mapping pair; a key node is missed"),e.position+=1,o=r;if((e.line===s||e.lineIndent>t)&&(Rh(e,t,Jl,!0,n)&&(p?f=e.result:d=e.result),p||(Sh(e,h,u,c,f,d),c=f=d=null),Ch(e,!0,-1),o=e.input.charCodeAt(e.position)),e.lineIndent>t&&0!==o)yh(e,"bad indentation of a mapping entry");else if(e.lineIndent<t)break}return p&&Sh(e,h,u,c,f,null),g&&(e.tag=a,e.anchor=l,e.kind="mapping",e.result=h),g}(e,f,c))||function(e,t){var i,r,n,s,o,a,l,h,u,c,f=!0,d=e.tag,p=e.anchor,g={};if(91===(c=e.input.charCodeAt(e.position)))n=93,a=!1,r=[];else{if(123!==c)return !1;n=125,a=!0,r={};}for(null!==e.anchor&&(e.anchorMap[e.anchor]=r),c=e.input.charCodeAt(++e.position);0!==c;){if(Ch(e,!0,t),(c=e.input.charCodeAt(e.position))===n)return e.position++,e.tag=d,e.anchor=p,e.kind=a?"mapping":"sequence",e.result=r,!0;f||yh(e,"missed comma between flow collection entries"),u=null,s=o=!1,63===c&&lh(e.input.charCodeAt(e.position+1))&&(s=o=!0,e.position++,Ch(e,!0,t)),i=e.line,Rh(e,t,$l,!1,!0),h=e.tag,l=e.result,Ch(e,!0,t),c=e.input.charCodeAt(e.position),!o&&e.line!==i||58!==c||(s=!0,c=e.input.charCodeAt(++e.position),Ch(e,!0,t),Rh(e,t,$l,!1,!0),u=e.result),a?Sh(e,r,g,h,l,u):s?r.push(Sh(e,null,g,h,l,u)):r.push(l),Ch(e,!0,t),44===(c=e.input.charCodeAt(e.position))?(f=!0,c=e.input.charCodeAt(++e.position)):f=!1;}yh(e,"unexpected end of the stream within a flow collection");}(e,c)?g=!0:(o&&function(e,t){var i,r,n,s,o,a=Xl,l=!1,h=t,u=0,c=!1;if(124===(s=e.input.charCodeAt(e.position)))r=!1;else{if(62!==s)return !1;r=!0;}for(e.kind="scalar",e.result="";0!==s;)if(43===(s=e.input.charCodeAt(++e.position))||45===s)Xl===a?a=43===s?eh:Ql:yh(e,"repeat of a chomping mode identifier");else{if(!((n=48<=(o=s)&&o<=57?o-48:-1)>=0))break;0===n?yh(e,"bad explicit indentation width of a block scalar; it cannot be less than one"):l?yh(e,"repeat of an indentation width identifier"):(h=t+n-1,l=!0);}if(ah(s)){do{s=e.input.charCodeAt(++e.position);}while(ah(s));if(35===s)do{s=e.input.charCodeAt(++e.position);}while(!oh(s)&&0!==s)}for(;0!==s;){for(xh(e),e.lineIndent=0,s=e.input.charCodeAt(e.position);(!l||e.lineIndent<h)&&32===s;)e.lineIndent++,s=e.input.charCodeAt(++e.position);if(!l&&e.lineIndent>h&&(h=e.lineIndent),oh(s))u++;else{if(e.lineIndent<h){a===eh?e.result+=al.repeat("\n",u):a===Xl&&l&&(e.result+="\n");break}for(r?ah(s)?(c=!0,e.result+=al.repeat("\n",u+1)):c?(c=!1,e.result+=al.repeat("\n",u+1)):0===u?l&&(e.result+=" "):e.result+=al.repeat("\n",u):e.result+=l?al.repeat("\n",u+1):al.repeat("\n",u),l=!0,u=0,i=e.position;!oh(s)&&0!==s;)s=e.input.charCodeAt(++e.position);wh(e,i,e.position,!1);}}return !0}(e,c)||function(e,t){var i,r,n;if(39!==(i=e.input.charCodeAt(e.position)))return !1;for(e.kind="scalar",e.result="",e.position++,r=n=e.position;0!==(i=e.input.charCodeAt(e.position));)if(39===i){if(wh(e,r,e.position,!0),39!==(i=e.input.charCodeAt(++e.position)))return !0;r=n=e.position,e.position++;}else oh(i)?(wh(e,r,n,!0),Ah(e,Ch(e,!1,t)),r=n=e.position):e.position===e.lineStart&&Eh(e)?yh(e,"unexpected end of the document within a single quoted scalar"):(e.position++,n=e.position);yh(e,"unexpected end of the stream within a single quoted scalar");}(e,c)||function(e,t){var i,r,n,s,o,a,l;if(34!==(a=e.input.charCodeAt(e.position)))return !1;for(e.kind="scalar",e.result="",e.position++,i=r=e.position;0!==(a=e.input.charCodeAt(e.position));){if(34===a)return wh(e,i,e.position,!0),e.position++,!0;if(92===a){if(wh(e,i,e.position,!0),oh(a=e.input.charCodeAt(++e.position)))Ch(e,!1,t);else if(a<256&&dh[a])e.result+=ph[a],e.position++;else if((o=120===(l=a)?2:117===l?4:85===l?8:0)>0){for(n=o,s=0;n>0;n--)(o=uh(a=e.input.charCodeAt(++e.position)))>=0?s=(s<<4)+o:yh(e,"expected hexadecimal character");e.result+=fh(s),e.position++;}else yh(e,"unknown escape sequence");i=r=e.position;}else oh(a)?(wh(e,i,r,!0),Ah(e,Ch(e,!1,t)),i=r=e.position):e.position===e.lineStart&&Eh(e)?yh(e,"unexpected end of the document within a double quoted scalar"):(e.position++,r=e.position);}yh(e,"unexpected end of the stream within a double quoted scalar");}(e,c)?g=!0:!function(e){var t,i,r;if(42!==(r=e.input.charCodeAt(e.position)))return !1;for(r=e.input.charCodeAt(++e.position),t=e.position;0!==r&&!lh(r)&&!hh(r);)r=e.input.charCodeAt(++e.position);return e.position===t&&yh(e,"name of an alias node must contain at least one character"),i=e.input.slice(t,e.position),e.anchorMap.hasOwnProperty(i)||yh(e,'unidentified alias "'+i+'"'),e.result=e.anchorMap[i],Ch(e,!0,-1),!0}(e)?function(e,t,i){var r,n,s,o,a,l,h,u,c=e.kind,f=e.result;if(lh(u=e.input.charCodeAt(e.position))||hh(u)||35===u||38===u||42===u||33===u||124===u||62===u||39===u||34===u||37===u||64===u||96===u)return !1;if((63===u||45===u)&&(lh(r=e.input.charCodeAt(e.position+1))||i&&hh(r)))return !1;for(e.kind="scalar",e.result="",n=s=e.position,o=!1;0!==u;){if(58===u){if(lh(r=e.input.charCodeAt(e.position+1))||i&&hh(r))break}else if(35===u){if(lh(e.input.charCodeAt(e.position-1)))break}else{if(e.position===e.lineStart&&Eh(e)||i&&hh(u))break;if(oh(u)){if(a=e.line,l=e.lineStart,h=e.lineIndent,Ch(e,!1,-1),e.lineIndent>=t){o=!0,u=e.input.charCodeAt(e.position);continue}e.position=s,e.line=a,e.lineStart=l,e.lineIndent=h;break}}o&&(wh(e,n,s,!1),Ah(e,e.line-a),n=s=e.position,o=!1),ah(u)||(s=e.position+1),u=e.input.charCodeAt(++e.position);}return wh(e,n,s,!1),!!e.result||(e.kind=c,e.result=f,!1)}(e,c,$l===i)&&(g=!0,null===e.tag&&(e.tag="?")):(g=!0,null===e.tag&&null===e.anchor||yh(e,"alias node should not have any properties")),null!==e.anchor&&(e.anchorMap[e.anchor]=e.result)):0===d&&(g=a&&Th(e,f))),null!==e.tag&&"!"!==e.tag)if("?"===e.tag){for(l=0,h=e.implicitTypes.length;l<h;l+=1)if((u=e.implicitTypes[l]).resolve(e.result)){e.result=u.construct(e.result),e.tag=u.tag,null!==e.anchor&&(e.anchorMap[e.anchor]=e.result);break}}else Hl.call(e.typeMap,e.tag)?(u=e.typeMap[e.tag],null!==e.result&&u.kind!==e.kind&&yh(e,"unacceptable node kind for !<"+e.tag+'> tag; it should be "'+u.kind+'", not "'+e.kind+'"'),u.resolve(e.result)?(e.result=u.construct(e.result),null!==e.anchor&&(e.anchorMap[e.anchor]=e.result)):yh(e,"cannot resolve a node with !<"+e.tag+"> explicit tag")):yh(e,"unknown tag !<"+e.tag+">");return null!==e.listener&&e.listener("close",e),null!==e.tag||null!==e.anchor||g}function Lh(e){var t,i,r,n,s=e.position,o=!1;for(e.version=null,e.checkLineBreaks=e.legacy,e.tagMap={},e.anchorMap={};0!==(n=e.input.charCodeAt(e.position))&&(Ch(e,!0,-1),n=e.input.charCodeAt(e.position),!(e.lineIndent>0||37!==n));){for(o=!0,n=e.input.charCodeAt(++e.position),t=e.position;0!==n&&!lh(n);)n=e.input.charCodeAt(++e.position);for(r=[],(i=e.input.slice(t,e.position)).length<1&&yh(e,"directive name must not be less than one character in length");0!==n;){for(;ah(n);)n=e.input.charCodeAt(++e.position);if(35===n){do{n=e.input.charCodeAt(++e.position);}while(0!==n&&!oh(n));break}if(oh(n))break;for(t=e.position;0!==n&&!lh(n);)n=e.input.charCodeAt(++e.position);r.push(e.input.slice(t,e.position));}0!==n&&xh(e),Hl.call(bh,i)?bh[i](e,i,r):vh(e,'unknown document directive "'+i+'"');}Ch(e,!0,-1),0===e.lineIndent&&45===e.input.charCodeAt(e.position)&&45===e.input.charCodeAt(e.position+1)&&45===e.input.charCodeAt(e.position+2)?(e.position+=3,Ch(e,!0,-1)):o&&yh(e,"directives end mark is expected"),Rh(e,e.lineIndent-1,Jl,!1,!0),Ch(e,!0,-1),e.checkLineBreaks&&ih.test(e.input.slice(s,e.position))&&vh(e,"non-ASCII line breaks are interpreted as content"),e.documents.push(e.result),e.position===e.lineStart&&Eh(e)?46===e.input.charCodeAt(e.position)&&(e.position+=3,Ch(e,!0,-1)):e.position<e.length-1&&yh(e,"end of the stream or a document separator is expected");}function Bh(e,t){t=t||{},0!==(e=String(e)).length&&(10!==e.charCodeAt(e.length-1)&&13!==e.charCodeAt(e.length-1)&&(e+="\n"),65279===e.charCodeAt(0)&&(e=e.slice(1)));var i=new mh(e,t);for(i.input+="\0";32===i.input.charCodeAt(i.position);)i.lineIndent+=1,i.position+=1;for(;i.position<i.length-1;)Lh(i);return i.documents}function Ih(e,t,i){var r,n,s=Bh(e,i);for(r=0,n=s.length;r<n;r+=1)t(s[r]);}function Fh(e,t){var i=Bh(e,t);if(0!==i.length){if(1===i.length)return i[0];throw new hl("expected a single document in the stream, but found more")}}var Ph={loadAll:Ih,load:Fh,safeLoadAll:function(e,t,i){Ih(e,t,al.extend({schema:jl},i));},safeLoad:function(e,t){return Fh(e,al.extend({schema:jl},t))}};function Mh(e){return function(){throw new Error("Function "+e+" is deprecated and cannot be used.")}}var Uh,Dh={Type:pl,Schema:_l,FAILSAFE_SCHEMA:yl,JSON_SCHEMA:El,CORE_SCHEMA:Al,DEFAULT_SAFE_SCHEMA:jl,DEFAULT_FULL_SCHEMA:Vl,load:Ph.load,loadAll:Ph.loadAll,safeLoad:Ph.safeLoad,safeLoadAll:Ph.safeLoadAll,YAMLException:hl,MINIMAL_SCHEMA:yl,SAFE_SCHEMA:jl,DEFAULT_SCHEMA:Vl,scan:Mh("scan"),parse:Mh("parse"),compose:Mh("compose"),addConstructor:Mh("addConstructor")};class jh{constructor(t,i,r){void 0===r&&(r=null),this.url=t,!i||e.isRelativeURL(i)&&!e.isRelativeURL(this.url)?this.path=e.pathForURL(this.url):this.path=i,this.path_for_parent=i||this.path,this.parent=r,this.container=null,this.parent&&(this.parent.container?this.container=this.parent.container:this.parent.isContainer()&&(this.container=this.parent));}load(){return Wh(this.url)}resourceFor(e){return {url:this.urlFor(e),path:this.pathFor(e),type:this.typeFor(e)}}urlFor(t){return Zh(t)?t:e.isRelativeURL(t)&&this.container?this.parent.urlFor(this.path_for_parent+t):e.addBaseURL(t,this.path)}pathFor(t){return e.pathForURL(t)}typeFor(t){return e.extensionForURL(t)}isContainer(){return !1}}class Nh extends jh{constructor(e,t,i){super(e,t,i),this.zip=null,this.files={},this.root=null,this.path="";}isContainer(){return !0}async load(){if(this.zip=new sl,"string"==typeof this.url){const t=await e.Utils.io(this.url,6e4,"arraybuffer");return await this.zip.loadAsync(t),await this.parseZipFiles(),this.loadRoot()}return this}urlFor(t){return Zh(t)?t:e.isRelativeURL(t)?this.urlForZipFile(e.flattenRelativeURL(t)):super.urlFor(t)}typeFor(t){return e.isRelativeURL(t)?this.typeForZipFile(t):super.typeFor(t)}loadRoot(){return this.findRoot(),Wh(this.urlForZipFile(this.root))}findRoot(){const t=Object.keys(this.files).filter(e=>0===this.files[e].depth).filter(t=>"yaml"===e.extensionForURL(t));if(1===t.length&&(this.root=t[0]),!this.root){let e=`Could not find root scene for bundle '${this.url}': `;throw e+="The zip archive's root level must contain a single scene file with the '.yaml' extension. ",t.length>0?e+=`Found multiple YAML files at the root level: ${t.map(e=>"'"+e+"'").join(", ")}.`:e+="Found NO YAML files at the root level.",Error(e)}}async parseZipFiles(){let t=[],i=[];this.zip.forEach((e,r)=>{r.dir||(t.push(e),i.push(r.async("arraybuffer")));});const r=await Promise.all(i);for(let i=0;i<r.length;i++){let n=t[i],s=n.split("/").length-1;this.files[n]={data:r[i],type:e.extensionForURL(n),depth:s};}}urlForZipFile(t){if(this.files[t])return this.files[t].url||(this.files[t].url=e.createObjectURL(new Blob([this.files[t].data]))),this.files[t].url}typeForZipFile(e){return this.files[e]&&this.files[e].type}}function Zh(e){return !(!e||"global."!==e.slice(0,7))}function Wh(t){return new Promise((i,r)=>{"string"==typeof t?e.Utils.io(t).then(e=>{try{let t=function(e){var t;try{t=Dh.safeLoad(e,{json:!0});}catch(e){throw e}return t}(e);i(t);}catch(e){r(e);}},r):(t=Object.assign({},t),i(t));})}var qh=Uh={async loadScene(t,i){let r=void 0===i?{}:i,n=r.path,s=r.type,o=[];const a=await this.loadSceneRecursive({url:t,path:n,type:s},null,o),l=this.finalize(a),h=l.config,u=l.bundle;if(!h)throw o[0];return o.length>0&&o.forEach(t=>{let i=`Failed to import scene: ${t.url}`;e.log("error",i,t),this.trigger("error",{type:"scene_import",message:i,error:t,url:t.url});}),{config:h,bundle:u}},async loadSceneRecursive(t,i,r){let n=t.url,s=t.path,o=t.type;if(void 0===r&&(r=[]),!n)return {};let a=function(t,i,r,n){return void 0===n&&(n=null),null!=n&&"zip"===n||"string"==typeof t&&!e.isLocalURL(t)&&"zip"===e.extensionForURL(t)?new Nh(t,i,r):new jh(t,i,r)}(n,s,i,o);try{let t=await a.load();if(null==t.import)return this.normalize(t,a),{config:t,bundle:a};Array.isArray(t.import)||(t.import=[t.import]);let i=[];t.import.forEach(t=>{"object"==typeof t&&(t=e.createObjectURL(new Blob([JSON.stringify(t)]))),i.push(a.resourceFor(t));}),delete t.import;const s=i.map(e=>this.loadSceneRecursive(e,a,r)),o=(await Promise.all(s)).map(e=>this.normalize(e.config,e.bundle)).map(e=>e.config);return t=e.mergeObjects(...o,t),this.normalize(t,a),{config:t,bundle:a}}catch(e){return e.url=n,r.push(e),{}}},normalize(e,t){return this.normalizeDataSources(e,t),this.normalizeFonts(e,t),this.normalizeTextures(e,t),this.hoistTextures(e,t),{config:e,bundle:t}},normalizeDataSources(e,t){e.sources=e.sources||{};for(let i in e.sources)this.normalizeDataSource(e.sources[i],t);return e},normalizeDataSource(e,t){if(e.url=t.urlFor(e.url),Array.isArray(e.composite)&&e.composite.forEach(e=>e.url=t.urlFor(e.url)),e.scripts){Array.isArray(e.scripts)&&(e.scripts=e.scripts.reduce((e,t)=>(e[t]=t,e),{}));for(let i in e.scripts)e.scripts[i]=t.urlFor(e.scripts[i]);}return e},normalizeFonts(e,t){e.fonts=e.fonts||{};for(let i in e.fonts)if(Array.isArray(e.fonts[i]))e.fonts[i].forEach(e=>{e.url=e.url&&t.urlFor(e.url);});else{let r=e.fonts[i];r.url=r.url&&t.urlFor(r.url);}return e},normalizeTextures(e,t){if(e.textures=e.textures||{},e.textures)for(let i in e.textures){let r=e.textures[i];r.url&&(r.url=t.urlFor(r.url));}},hoistTextures(t,i){if(t.styles)for(let e in t.styles){let r=t.styles[e],n=r.texture;"string"!=typeof n||t.textures[n]||(r.texture=this.hoistTexture(n,t,i)),r.material&&["emission","ambient","diffuse","specular","normal"].forEach(e=>{let n=null!=r.material[e]&&r.material[e].texture;"string"!=typeof n||t.textures[n]||(r.material[e].texture=this.hoistTexture(n,t,i));});}if(this.hoistStyleShaderUniformTextures(t,i,{include_globals:!1}),t.layers){let r=[t.layers];for(;r.length>0;){let n=r.pop();if("object"==typeof n&&!Array.isArray(n))for(let s in n)if("draw"===s){let e=n[s];for(let r in e){if(e[r].texture){let n=e[r].texture;"string"!=typeof n||t.textures[n]||(e[r].texture=this.hoistTexture(n,t,i));}if(e[r].outline&&e[r].outline.texture){let n=e[r].outline.texture;"string"!=typeof n||t.textures[n]||(e[r].outline.texture=this.hoistTexture(n,t,i));}}}else{if(e.isReserved(s))continue;r.push(n[s]);}}}},hoistStyleShaderUniformTextures(t,i,r){let n=r.include_globals;if(t.styles)for(let r in t.styles){let s=t.styles[r];s.shaders&&s.shaders.uniforms&&e.GLSL.parseUniforms(s.shaders.uniforms).forEach(e=>{let r=e.type,s=e.value,o=e.key,a=e.uniforms;"sampler2D"!==r||"string"!=typeof s||t.textures[s]||!n&&Zh(s)||(a[o]=this.hoistTexture(s,t,i));});}},hoistTexture(e,t,i){let r=Zh(e),n=r?e:i.urlFor(e),s=r?`texture-${n}`:n;return t.textures[s]={url:n},s},applyGlobalProperties(t){if(!t.global||0===Object.keys(t.global).length)return t;const i=function e(t,i,r){void 0===i&&(i=null);void 0===r&&(r={});i=i?i+".":"global.";for(const n in t){const s=i+n,o=t[n];r[s]=o,"object"!=typeof o||Array.isArray(o)||e(o,s,r);}return r}(t.global);return function t(r,n,s){let o;if(null!=n&&"object"==typeof n&&n._global_prop&&n._global_prop[s]?o=n._global_prop[s]:"string"==typeof r&&"global."===r.slice(0,7)&&(o=r),o){null==n._global_prop&&Object.defineProperty(n,"_global_prop",{value:{}}),n._global_prop[s]=o;let t,r=i[o];for(;"string"==typeof r&&"global."===r.slice(0,7);){if((t=t||[o]).indexOf(r)>-1){e.log({level:"warn",once:!0},"Global properties: cyclical reference detected",t),r=null;break}t.push(r),r=i[r];}Object.defineProperty(n,s,{enumerable:!0,get:function(){return r},set:function(e){delete n._global_prop[s],delete n[s],n[s]=e;}});}else if(Array.isArray(r))for(let e=0;e<r.length;e++)t(r[e],r,e);else if("object"==typeof r)for(let e in r)t(r[e],r,e);return r}(t)},finalize(e){let t=e.config,i=e.bundle;return t?(t.global=t.global||{},t.scene=t.scene||{},t.cameras=t.cameras||{},t.lights=t.lights||{},t.styles=t.styles||{},t.layers=t.layers||{},t.camera&&(t.cameras.default=t.camera),0===Object.keys(t.cameras).length&&(t.cameras.default={}),(0===Object.keys(t.lights).length||Object.keys(t.lights).every(e=>!1===t.lights[e].visible))&&(t.lights.default_light={type:"directional"}),{config:t,bundle:i}):{}}};e.subscribeMixin(Uh);class Yh{constructor(){this.tiles={},this.max_proxy_descendant_depth=6,this.max_proxy_ancestor_depth=7,this.children_cache={};}addTile(t){for(this.tiles[t.key]=this.tiles[t.key]||{descendants:0},this.tiles[t.key].tile=t;t.style_z>=0;){if(!(t=e.TileID.parent(t)))return;this.tiles[t.key]||(this.tiles[t.key]={descendants:0}),this.tiles[t.key].descendants++;}}removeTile(t){for(this.tiles[t.key]&&(delete this.tiles[t.key].tile,0===this.tiles[t.key].descendants&&delete this.tiles[t.key]);t.style_z>=0;){if(!(t=e.TileID.parent(t)))return;this.tiles[t.key]&&this.tiles[t.key].descendants>0&&(this.tiles[t.key].descendants--,0!==this.tiles[t.key].descendants||this.tiles[t.key].tile||delete this.tiles[t.key]);}}getAncestor(t){let i=0;for(;i<this.max_proxy_ancestor_depth;){if(!(t=e.TileID.parent(t)))return;if(this.tiles[t.key]&&this.tiles[t.key].tile&&this.tiles[t.key].tile.loaded)return this.tiles[t.key].tile;i++;}}getDescendants(t,i){void 0===i&&(i=0);let r=[];if(i<this.max_proxy_descendant_depth){let n=e.TileID.children(t,this.children_cache);if(!n)return;n.forEach(e=>{this.tiles[e.key]&&(this.tiles[e.key].tile&&this.tiles[e.key].tile.loaded?r.push(this.tiles[e.key].tile):this.tiles[e.key].descendants>0&&r.push(...this.getDescendants(e,i+1)));});}return r}}let Vh={},Hh={};async function $h(t,i,r){void 0===r&&(r=!1),Hh=Vh,Vh={};let n=function(t,i){const r={};let n={};t.forEach(t=>{const s=e.Geo.unitsPerMeter(t.coords.z),o=Math.pow(2,i-t.style_z),a=s*o,l=e.Geo.metersPerPixel(i),h=Object.assign({},t.meshes,t.pending_label_meshes);for(let i in h){const o=h[i];o.forEach(i=>{if(i.labels)for(let o in i.labels){if(t.isProxy()&&!Hh[o])continue;const h=i.labels[o].container.label,u=i.labels[o].container.linked,c=i.labels[o].ranges,f=Object.assign({},i.labels[o].debug,{tile:t,params:h,label_id:o});let d=r[o]={};if(d.discard=Gh.bind(d),d.build_id=t.build_id,Object.assign(d,h),d.layout=Object.assign({},h.layout),d.layout.repeat_scale=.75,d.layout.repeat_distance=d.layout.repeat_distance||0,d.layout.repeat_distance/=a,d.position=[d.position[0]/s+t.min.x,d.position[1]/s+t.min.y],d.unit_scale=l,"point"===d.type)e.LabelPoint.prototype.updateBBoxes.call(d);else if("straight"===d.type)e.LabelLineStraight.prototype.updateBBoxes.call(d,d.position,d.size,d.angle,d.angle,d.offset);else if(h.obbs){const i=h.obbs.map(i=>{let r=i.x,n=i.y,o=i.a,l=i.w,h=i.h;return r=r/s+t.min.x,n=n/s+t.min.y,l/=a,h/=a,new e.OBB(r,n,o,l,h)});d.obbs=i,d.aabbs=i.map(e=>e.getExtent());}n[o]={label:d,linked:u,ranges:c,mesh:i,debug:f};}});}});for(let e in n){const t=n[e];t.linked&&(t.linked=n[t.linked]);}return n=Object.keys(n).map(e=>n[e])}(t,i);e.Collision.startTile("main",{apply_repeat_groups:!0,return_hidden:!0}),e.Collision.addStyle("main","main");const s=await e.Collision.collide(n,"main","main");let o=[];return s.forEach(e=>{let t=0;!0!==e.show||r&&e.label.breach&&!Hh[e.label.id]||(t=1),t&&(Vh[e.label.id]=!0);let i=!0;e.ranges.forEach(r=>{if(!i)return;let n=e.mesh;if(!n.valid)return;let s=n.vertex_layout.offset.a_shape,a=n.vertex_layout.stride;for(let e=0;e<r[1];e++){if(n.vertex_data[r[0]+e*a+s+6]===t)return void(i=!1);n.vertex_data[r[0]+e*a+s+6]=t;}-1===o.indexOf(n)&&o.push(n);});}),o.forEach(e=>e.upload()),t.forEach(e=>e.swapPendingLabels()),{labels:s,containers:n}}function Gh(t,i){if(void 0===i&&(i=null),this.obb)return e.Label.prototype.occluded.call(this,t,i);if(this.obbs)for(let r=0;r<this.obbs.length;r++){let n={aabb:this.aabbs[r],obb:this.obbs[r]};if(e.Label.prototype.occluded.call(n,t,i))return !0}return !1}class Kh{constructor(t){let i=t.scene;this.scene=i,this.tiles={},this.pyramid=new Yh,this.visible_coords={},this.queued_coords=[],this.building_tiles=null,this.renderable_tiles=[],this.collision={tile_keys:null,mesh_set:null,zoom:null,zoom_steps:3},this.main_thread_target=["TileManager",this.scene.id].join("_"),e.WorkerBroker.addTarget(this.main_thread_target,this);}destroy(){this.forEachTile(e=>e.destroy()),this.tiles={},this.pyramid=null,this.visible_coords={},this.queued_coords=[],this.scene=null,e.WorkerBroker.removeTarget(this.main_thread_target);}get view(){return this.scene.view}get style_manager(){return this.scene.style_manager}keepTile(e){this.tiles[e.key]=e,this.pyramid.addTile(e);}hasTile(e){return void 0!==this.tiles[e]}forgetTile(e){if(this.hasTile(e)){let t=this.tiles[e];this.pyramid.removeTile(t);}delete this.tiles[e],this.tileBuildStop(e);}removeTile(t){e.log("trace",`tile unload for ${t}`);var i=this.tiles[t];null!=i&&i.destroy(),this.forgetTile(i.key),this.scene.requestRedraw();}forEachTile(e){for(let t in this.tiles)e(this.tiles[t]);}removeTiles(e){let t=[];for(let i in this.tiles){e(this.tiles[i])&&t.push(i);}for(let e=0;e<t.length;e++){let i=t[e];this.removeTile(i);}}updateTilesForView(){this.visible_coords={};let e=this.view.findVisibleTileCoordinates();for(let t=0;t<e.length;t++){const i=e[t];this.queueCoordinate(i),this.visible_coords[i.key]=i;}this.updateTileStates();}updateTileStates(){return this.forEachTile(e=>{this.updateVisibility(e);}),this.loadQueuedCoordinates(),this.updateProxyTiles(),this.view.pruneTilesForView(),this.updateRenderableTiles(),this.style_manager.updateActiveStyles(this.renderable_tiles),this.style_manager.updateActiveBlendOrders(this.renderable_tiles),this.updateLabels()}updateLabels(){if(this.scene.building&&!this.scene.building.initial)return Promise.resolve({});const t=this.renderable_tiles.filter(e=>e.valid).filter(e=>e.built);return 0===t.length?Promise.resolve({}):(t.sort((e,t)=>e.build_id<t.build_id?-1:e.build_id>t.build_id?1:0),this.collision.zoom===Jh(this.view.zoom,this.collision.zoom_steps)&&this.collision.tile_keys===JSON.stringify(t.map(e=>e.key))&&this.collision.mesh_set===Xh(t)?Promise.resolve({}):(this.collision.task||(this.collision.zoom=Jh(this.view.zoom,this.collision.zoom_steps),this.collision.tile_keys=JSON.stringify(t.map(e=>e.key)),this.collision.mesh_set=Xh(t),this.collision.task={type:"tileManagerUpdateLabels",run:async i=>{const r=await $h(t,this.collision.zoom,this.isLoadingVisibleTiles());this.scene.requestRedraw(),this.collision.task=null,e.Task.finish(i,r),this.updateTileStates();}},e.Task.add(this.collision.task)),this.collision.task.promise))}updateProxyTiles(){if(0===this.view.zoom_direction)return;this.forEachTile(e=>e.setProxyFor(null));let e=!1;this.forEachTile(t=>{if(1===this.view.zoom_direction){if(t.visible&&!t.labeled){const i=this.pyramid.getAncestor(t);i&&(i.setProxyFor(t),e=!0);}}else if(-1===this.view.zoom_direction&&t.visible&&!t.labeled){const i=this.pyramid.getDescendants(t);for(let r=0;r<i.length;r++)i[r].setProxyFor(t),e=!0;}}),e||(this.view.zoom_direction=0);}updateVisibility(t){if(t.visible=!1,t.style_z===this.view.tile_zoom)if(this.visible_coords[t.coords.key])t.visible=!0;else for(let i in this.visible_coords)if(e.TileID.isDescendant(t.coords,this.visible_coords[i])){t.visible=!0;break}}pruneToVisibleTiles(){this.removeTiles(e=>!e.visible);}getRenderableTiles(){return this.renderable_tiles}updateRenderableTiles(){this.renderable_tiles=[];for(let e in this.tiles){let t=this.tiles[e];t.visible&&t.loaded&&this.renderable_tiles.push(t);}return this.renderable_tiles}isLoadingVisibleTiles(){return Object.keys(this.tiles).some(e=>this.tiles[e].visible&&!this.tiles[e].built)}allVisibleTilesLabeled(){return this.renderable_tiles.every(e=>e.labeled)}queueCoordinate(e){this.queued_coords[this.queued_coords.length]=e;}loadQueuedCoordinates(){0!==this.queued_coords.length&&(this.queued_coords.sort((t,i)=>{let r=this.view.center.meters,n=e.Geo.metersPerTile(t.z)/2,s=e.Geo.metersForTile(t);s.x+=n,s.y-=n;let o=e.Geo.metersForTile(i);o.x+=n,o.y-=n;let a=Math.abs(r.x-s.x)+Math.abs(r.y-s.y),l=Math.abs(r.x-o.x)+Math.abs(r.y-o.y);return t.center_dist=a,i.center_dist=l,l>a?-1:l===a?0:1}),this.queued_coords.forEach(e=>this.loadCoordinate(e)),this.queued_coords=[]);}loadCoordinate(t){if(t.z===this.view.center.tile.z)for(let i in this.scene.sources){let r=this.scene.sources[i];if(!r.builds_geometry_tiles||!r.includesTile(t,this.view.tile_zoom))continue;let n=e.TileID.normalizedKey(t,r,this.view.tile_zoom);if(n&&!this.hasTile(n)){e.log("trace",`load tile ${n}, distance from view center: ${t.center_dist}`);let i=new e.Tile({source:r,coords:t,workers:this.scene.workers,style_z:this.view.baseZoom(t.z),view:this.view});this.keepTile(i),this.buildTile(i);}}}buildTile(e,t){this.tileBuildStart(e.key),this.updateVisibility(e),e.build(this.scene.generation,t);}buildTileStylesCompleted(t){let i=t.tile,r=t.progress;if(null==this.tiles[i.key])e.log("trace",`discarded tile ${i.key} in TileManager.buildTileStylesCompleted because previously removed`),e.Tile.abortBuild(i),this.updateTileStates();else if(i.generation!==this.scene.generation)e.log("trace",`discarded tile ${i.key} in TileManager.buildTileStylesCompleted because built with `+`scene config gen ${i.generation}, current ${this.scene.generation}`),e.Tile.abortBuild(i),this.updateTileStates();else{if(this.tiles[i.key]){if(i.id<this.tiles[i.key].id)return e.log("trace",`discarded tile ${i.key} for id ${i.id} in TileManager.buildTileStylesCompleted because built for discarded tile id`),void e.Tile.abortBuild(i);i=this.tiles[i.key].merge(i);}r.done&&(i.built=!0),i.buildMeshes(this.scene.styles,r),this.updateTileStates(),this.scene.requestRedraw();}r.done&&this.tileBuildStop(i.key);}buildTileError(t){e.log("error",`Error building tile ${t.key}:`,t.error),this.forgetTile(t.key),e.Tile.abortBuild(t);}tileBuildStart(t){this.building_tiles=this.building_tiles||{},this.building_tiles[t]=!0,e.log("trace",`tileBuildStart for ${t}: ${Object.keys(this.building_tiles).length}`);}tileBuildStop(t){this.building_tiles&&(e.log("trace",`tileBuildStop for ${t}: ${Object.keys(this.building_tiles).length}`),delete this.building_tiles[t],this.checkBuildQueue());}checkBuildQueue(){this.building_tiles&&0!==Object.keys(this.building_tiles).length||(this.building_tiles=null,this.scene.tileManagerBuildDone());}getDebugProp(e,t){var i=[];for(var r in this.tiles)null==this.tiles[r].debug[e]||"function"==typeof t&&!0!==t(this.tiles[r])||i.push(this.tiles[r].debug[e]);return i}getDebugSum(e,t){var i=0;for(var r in this.tiles)null==this.tiles[r].debug[e]||"function"==typeof t&&!0!==t(this.tiles[r])||(i+=this.tiles[r].debug[e]);return i}getDebugAverage(e,t){return this.getDebugSum(e,t)/Object.keys(this.tiles).length}}function Jh(e,t,i){return void 0===i&&(i=2),(Math.floor(e*t)/t).toFixed(i)}function Xh(e){return JSON.stringify(Object.entries(e).map(e=>{let t=e[1];return Object.entries(t.meshes).map(e=>{return e[1].map(e=>e.created_at)})}))}class Qh{constructor(e,t){t(e),this.value=e,this.setup=t;}set(e){JSON.stringify(this.value)!==JSON.stringify(e)&&(this.setup(e),this.value=e);}}class eu{constructor(e){this.defaults={},this.defaults.culling=!0,this.defaults.culling_face=e.BACK,this.defaults.blending=!1,this.defaults.blending_src=e.ONE_MINUS_SRC_ALPHA,this.defaults.blending_dst=e.ONE_MINUS_SRC_ALPHA,this.defaults.blending_src_alpha=e.ONE,this.defaults.blending_dst_alpha=e.ONE_MINUS_SRC_ALPHA,this.defaults.depth_write=!0,this.defaults.depth_test=!0,e.depthFunc(e.LESS),this.culling=new Qh({cull:this.defaults.culling,face:this.defaults.culling_face},t=>{t.cull?(e.enable(e.CULL_FACE),e.cullFace(t.face)):e.disable(e.CULL_FACE);}),this.blending=new Qh({blend:this.defaults.blending,src:this.defaults.blending_src,dst:this.defaults.blending_dst,src_alpha:this.defaults.blending_src_alpha,dst_alpha:this.defaults.blending_dst_alpha},t=>{t.blend?(e.enable(e.BLEND),t.src_alpha&&t.dst_alpha?e.blendFuncSeparate(t.src,t.dst,t.src_alpha,t.dst_alpha):e.blendFunc(t.src,t.dst)):e.disable(e.BLEND);}),this.depth_write=new Qh({depth_write:this.defaults.depth_write},t=>{e.depthMask(t.depth_write);}),this.depth_test=new Qh({depth_test:this.defaults.depth_test},t=>{t.depth_test?e.enable(e.DEPTH_TEST):e.disable(e.DEPTH_TEST);});}}class tu{constructor(){this.canvas=null,this.gl=null,this.screenshot_canvas=null,this.screenshot_context=null,this.queue_screenshot=null,this.video_capture=null;}setCanvas(e,t){this.canvas=e,this.gl=t;}screenshot(e){let t=(void 0===e?{}:e).background;return null!=this.queue_screenshot?this.queue_screenshot.promise:(this.queue_screenshot={background:t},this.queue_screenshot.promise=new Promise((e,t)=>{this.queue_screenshot.resolve=e,this.queue_screenshot.reject=t;}),this.queue_screenshot.promise)}completeScreenshot(){if(null!=this.queue_screenshot){let t=this.canvas.width,i=this.canvas.height,r=new Uint8Array(t*i*4);this.gl.readPixels(0,0,t,i,this.gl.RGBA,this.gl.UNSIGNED_BYTE,r);let n=this.queue_screenshot.background;n=n&&"transparent"!==n?e.StyleParser.parseColor(n).slice(0,3).map(e=>255*e):null;let s=new Uint8ClampedArray(t*i*4);for(let e=0;e<i;e++)for(let o=0;o<t;o++){let a=4*((i-e-1)*t+o),l=4*(e*t+o),h=r[a+3];s[l+0]=255*r[a+0]/h,s[l+1]=255*r[a+1]/h,s[l+2]=255*r[a+2]/h,s[l+3]=h,n&&(h/=255,s[l+0]=s[l+0]*h+n[0]*(1-h),s[l+1]=s[l+1]*h+n[1]*(1-h),s[l+2]=s[l+2]*h+n[2]*(1-h),s[l+3]=255);}this.screenshot_canvas=this.screenshot_canvas||document.createElement("canvas");let o=this.screenshot_canvas;o.width=t,o.height=i,this.screenshot_context=this.screenshot_context||o.getContext("2d");let a=this.screenshot_context,l=new ImageData(s,t,i);a.putImageData(l,0,0);const h=o.toDataURL("image/png"),u=atob(h.slice(22)),c=new Uint8Array(u.length);for(let e=0;e<u.length;++e)c[e]=u.charCodeAt(e);const f=new Blob([c],{type:"image/png"});this.queue_screenshot.resolve({url:h,blob:f,type:"png"}),this.queue_screenshot=null;}}startVideoCapture(){if("function"!=typeof window.MediaRecorder||!this.canvas||"function"!=typeof this.canvas.captureStream)return e.log("warn","Video capture (Canvas.captureStream and/or MediaRecorder APIs) not supported by browser"),!1;if(this.video_capture)return e.log("warn","Video capture already in progress, call Scene.stopVideoCapture() first"),!1;try{let t=this.video_capture={};t.chunks=[],t.stream=this.canvas.captureStream(),t.options={mimeType:"video/webm"},t.media_recorder=new MediaRecorder(t.stream,t.options),t.media_recorder.ondataavailable=(i=>{if(i.data.size>0&&t.chunks.push(i.data),t.resolve){let i=new Blob(t.chunks,{type:t.options.mimeType}),r=e.createObjectURL(i);if(t.stream){(t.stream.getTracks()||[]).forEach(e=>{e.stop(),t.stream.removeTrack(e);});}t.stream=null,t.media_recorder=null,this.video_capture=null,t.resolve({url:r,blob:i,type:"webm"});}}),t.media_recorder.start();}catch(t){return this.video_capture=null,e.log("error","Scene video capture failed",t),!1}return !0}stopVideoCapture(){return this.video_capture?(this.video_capture.promise=new Promise((e,t)=>{this.video_capture.resolve=e,this.video_capture.reject=t;}),this.video_capture.media_recorder.stop(),this.video_capture.promise):(e.log("warn","No scene video capture in progress, call Scene.startVideoCapture() first"),Promise.resolve({}))}}class iu{constructor(t,i){var r;i=i||{},e.subscribeMixin(this),this.id=iu.id++,this.initialized=!1,this.initializing=null,this.sources={},this.view=new e.View(this,i),this.tile_manager=new Kh({scene:this}),this.num_workers=i.numWorkers||2,!0===i.disableVertexArrayObjects&&(e.VertexArrayObject.disabled=!0),e.Utils.use_high_density_display=void 0===i.highDensityDisplay||i.highDensityDisplay,e.Utils.updateDevicePixelRatio(),this.config=null,this.config_source=t,this.config_bundle=null,this.last_valid_config_source=null,this.styles=null,this.style_manager=new e.StyleManager,this.building=null,this.dirty=!0,i.preUpdate&&this.subscribe({pre_update:i.preUpdate}),i.postUpdate&&this.subscribe({post_update:i.postUpdate}),this.render_loop=!i.disableRenderLoop,this.render_loop_active=!1,this.render_loop_stop=!1,this.render_count=0,this.last_render_count=0,this.render_count_changed=!1,this.frame=0,this.last_main_render=-1,this.last_selection_render=-1,this.media_capture=new tu,this.selection=null,this.selection_feature_count=0,this.fetching_selection_map=null,this.prev_textures=null,this.introspection=!0===i.introspection,this.times={},this.resetTime(),this.container=i.container,this.canvas=null,this.contextOptions=i.webGLContextOptions,this.lights=null,this.background=null,this.createListeners(),this.updating=0,this.generation=iu.generation,this.last_complete_generation=iu.generation,(r=this).debug={profile(t){console.profile(`main thread: ${t}`),e.WorkerBroker.postMessage(r.workers,"self.profile",t);},profileEnd(t){console.profileEnd(`main thread: ${t}`),e.WorkerBroker.postMessage(r.workers,"self.profileEnd",t);},timeRebuild(t,i){void 0===t&&(t=1),void 0===i&&(i={});let n=[],s=()=>{let o=+new Date;r.rebuild(i).then(()=>{if(n.push(+new Date-o),n.length<t)s();else{let i=~~(n.reduce((e,t)=>e+t)/n.length);e.log("info",`Profiled rebuild ${t} times: ${i} avg (${Math.min(...n)} min, ${Math.max(...n)} max)`);}});};s();},geometryCountByStyle(){let e={};return r.tile_manager.getRenderableTiles().forEach(t=>{for(let i in t.meshes)e[i]=e[i]||0,t.meshes[i].forEach(t=>{e[i]+=t.geometry_count;});}),e},geometryCountByBaseStyle(){let e=r.debug.geometryCountByStyle(),t={};for(let i in e){let n=r.styles[i].baseStyle();t[n]=t[n]||0,t[n]+=e[i];}return t},geometryCountTotal(){const e=r.debug.geometryCountByStyle();return Object.keys(e).reduce((t,i)=>e[i]+t,0)},geometrySizeByStyle(){let e={};return r.tile_manager.getRenderableTiles().forEach(t=>{for(let i in t.meshes)e[i]=e[i]||0,t.meshes[i].forEach(t=>{e[i]+=t.buffer_size;});}),e},geometrySizeByBaseStyle(){let e=r.debug.geometrySizeByStyle(),t={};for(let i in e){let n=r.styles[i].baseStyle();t[n]=t[n]||0,t[n]+=e[i];}return t},geometrySizeTotal(){const e=r.debug.geometrySizeByStyle();return Object.keys(e).reduce((t,i)=>e[i]+t,0)},textureSizeTotal:()=>Object.values(e.Texture.textures).map(e=>e.byteSize()).reduce((e,t)=>e+t),layerStats:()=>e.debugSettings.layer_stats?e.debugSumLayerStats(r.tile_manager.getRenderableTiles()):(e.log("warn","Enable the 'layer_stats' debug setting to collect layer stats"),{}),renderableTilesCount:()=>r.tile_manager.getRenderableTiles().length},this.log_level=i.logLevel||"warn",e.log.setLevel(this.log_level),e.log.reset();}static create(e,t){return void 0===t&&(t={}),new iu(e,t)}load(t,i){return void 0===t&&(t=null),void 0===i&&(i={}),this.initializing?this.initializing:(e.log.reset(),this.updating++,this.initialized=!1,this.view_complete=!1,this.times.frame=null,this.times.build=null,(i="string"==typeof i?{base_path:i}:i).blocking=void 0===i.blocking||i.blocking,!1!==this.render_loop&&this.setupRenderLoop(),this.createCanvas(),this.prev_textures=this.config&&Object.keys(this.config.textures),this.initializing=this.loadScene(t,i).then(()=>this.createWorkers()).then(()=>{this.destroyFeatureSelection(),e.WorkerBroker.postMessage(this.workers,"self.clearFunctionStringCache");const t="object"==typeof this.config_source||this.hasSubscribersFor("load"),r=this.updateConfig({serialize_funcs:t,normalize:!1,loading:!0,fade_in:!0});if(!0===i.blocking)return r}).then(()=>{this.freePreviousTextures(),this.updating--,this.initializing=null,this.initialized=!0,this.last_valid_config_source=this.config_source,this.last_valid_options={base_path:i.base_path,file_type:i.file_type},this.requestRedraw();}).catch(t=>{let i,r;if(this.initializing=null,this.updating=0,"YAMLException"===t.name?(i="yaml",r="Error parsing scene YAML"):r="Error initializing scene",this.trigger("error",{type:i,message:r,error:t,url:this.config_source}),r=`Scene.load() failed to load ${JSON.stringify(this.config_source)}: ${t.message}`,this.last_valid_config_source)return e.log("warn",r,t),e.log("info","Scene.load() reverting to last valid configuration"),this.load(this.last_valid_config_source,this.last_valid_base_path);throw e.log("error",r,t),t}),this.initializing)}destroy(){if(this.initialized=!1,this.render_loop_stop=!0,this.destroyListeners(),this.destroyFeatureSelection(),this.canvas&&this.canvas.parentNode&&(this.canvas.parentNode.removeChild(this.canvas),this.canvas=null),this.container=null,this.gl){e.Texture.destroy(this.gl),this.style_manager.destroy(this.gl),this.styles={},e.ShaderProgram.reset();let t=this.gl.getExtension("WEBGL_lose_context");t&&t.loseContext(),this.gl=null;}this.sources={},this.destroyWorkers(),this.tile_manager.destroy(),this.tile_manager=null,e.log.reset();}createCanvas(){if(!this.canvas){this.container=this.container||document.body,this.canvas=document.createElement("canvas"),this.canvas.style.position="absolute",this.canvas.style.top=0,this.canvas.style.left=0,this.container.style.backgroundColor="transparent",this.container.appendChild(this.canvas);try{this.gl=i.getContext(this.canvas,Object.assign({alpha:!0,premultipliedAlpha:!0,stencil:!0,device_pixel_ratio:e.Utils.device_pixel_ratio},this.contextOptions));}catch(e){throw new Error("Couldn't create WebGL context. Your browser may not support WebGL, or it's turned off? Visit http://webglreport.com/ for more info.")}this.resizeMap(this.container.clientWidth,this.container.clientHeight),e.VertexArrayObject.init(this.gl),this.render_states=new eu(this.gl),this.media_capture.setCanvas(this.canvas,this.gl);}}updateExternalScripts(){let e=[...this.external_scripts||[]],t=[];if(this.config.scene.scripts)for(let e in this.config.scene.scripts)-1===t.indexOf(this.config.scene.scripts[e])&&t.push(this.config.scene.scripts[e]);for(let e in this.config.sources){let i=this.config.sources[e];if(i.scripts)for(let e in i.scripts)-1===t.indexOf(i.scripts[e])&&t.push(i.scripts[e]);}return this.external_scripts=t,!(this.external_scripts.length===e.length&&this.external_scripts.every((t,i)=>t===e[i]))}createWorkers(){return this.updateExternalScripts()&&this.destroyWorkers(),this.workers?Promise.resolve():this.makeWorkers()}makeWorkers(){let t=!!this.gl.getExtension("OES_element_index_uint"),i=[];this.workers=[];for(let r=0;r<this.num_workers;r++){let n=new Worker(Tangram.workerURL);this.workers[r]=n,e.WorkerBroker.addWorker(n),e.log("debug",`Scene.makeWorkers: initializing worker ${r}`);let s=r;i.push(e.WorkerBroker.postMessage(n,"self.init",this.id,r,this.num_workers,this.log_level,e.Utils.device_pixel_ratio,t,this.external_scripts).then(t=>(e.log("debug",`Scene.makeWorkers: initialized worker ${t}`),t),t=>(e.log("error",`Scene.makeWorkers: failed to initialize worker ${s}:`,t),Promise.reject(t))));}return this.next_worker=0,Promise.all(i).then(()=>{e.log.setWorkers(this.workers);})}destroyWorkers(){this.selection=null,Array.isArray(this.workers)&&(e.log.setWorkers(null),this.workers.forEach(e=>{e.terminate();}),this.workers=null);}ready(){return !(!this.view.ready()||0===Object.keys(this.sources).length)}updateDevicePixelRatio(){e.Utils.updateDevicePixelRatio()&&e.WorkerBroker.postMessage(this.workers,"self.updateDevicePixelRatio",e.Utils.device_pixel_ratio).then(()=>this.rebuild()).then(()=>this.resizeMap(this.view.size.css.width,this.view.size.css.height));}resizeMap(t,r){0===t&&0===r||(this.dirty=!0,this.view.setViewportSize(t,r),this.gl&&i.resize(this.gl,t,r,e.Utils.device_pixel_ratio));}requestRedraw(){this.dirty=!0;}immediateRedraw(){this.dirty=!0,this.update();}renderLoop(){this.render_loop_active=!0,this.update(),e.Task.setState({user_moving_view:this.view.user_input_active}),e.Task.processAll(),this.render_loop_stop?(this.render_loop_stop=!1,this.render_loop_active=!1):window.requestAnimationFrame(this.renderLoop.bind(this));}setupRenderLoop(){this.render_loop_active||setTimeout(()=>{this.renderLoop();},0);}update(){let t=this.dirty,i=!!this.selection&&this.selection.hasPendingRequests();var r=!(!1===t&&!1===i||!1===this.initialized||this.updating>0||!1===this.ready());return this.trigger("pre_update",r),this.view.update(),!!r&&(this.dirty=!1,this.updateDevicePixelRatio(),this.render({main:t,selection:i}),this.updateViewComplete(),this.media_capture.completeScreenshot(),this.trigger("post_update",r),(!0===this.animated||this.view.isAnimating())&&(this.dirty=!0),this.frame++,e.log("trace","Scene.render()"),!0)}render(t){let i=t.main,r=t.selection;var n=this.gl;if(this.updateBackground(),Object.keys(this.lights).forEach(e=>this.lights[e].update()),this.render_count_changed=!1,i&&(this.render_count=this.renderPass(),this.last_main_render=this.frame,this.render_count!==this.last_render_count&&(this.render_count_changed=!0,this.logFirstFrame(),this.getFeatureSelectionMapSize().then(t=>{this.selection_feature_count=t,e.log("info",`Scene: rendered ${this.render_count} primitives (${t} features in selection map)`);})),this.last_render_count=this.render_count),r){if(this.view.panning||this.view.user_input_active)return void this.selection.clearPendingRequests();!this.selection.locked&&this.last_selection_render<this.last_main_render&&(this.selection.bind(),this.renderPass("selection_program",{allow_blend:!1}),n.bindFramebuffer(n.FRAMEBUFFER,null),n.viewport(0,0,this.canvas.width,this.canvas.height),n.clearColor(...this.background.computed_color),this.last_selection_render=this.frame),this.selection.read();}return !0}renderPass(t,i){void 0===t&&(t="program");let r=(void 0===i?{}:i).allow_blend;r=null==r||r,this.clearFrame();let n,s=0;const o=this.style_manager.getActiveBlendOrders();for(const i of o){const o=i.blend_order,a=i.styles;for(let i=0;i<a.length;i++){let l=this.styles[a[i]];if(null==l)continue;if(l.blend!==n){let t=Object.assign({},e.Style.render_states[l.blend],{blend:r&&l.blend});this.setRenderState(t);}const h=r&&l.blend;if("translucent"===h)this.gl.colorMask(!1,!1,!1,!1),this.renderStyle(l.name,t,o),this.gl.colorMask(!0,!0,!0,!0),this.gl.depthFunc(this.gl.EQUAL),this.gl.enable(this.gl.STENCIL_TEST),this.gl.clearStencil(0),this.gl.clear(this.gl.STENCIL_BUFFER_BIT),this.gl.stencilFunc(this.gl.EQUAL,this.gl.ZERO,255),this.gl.stencilOp(this.gl.KEEP,this.gl.KEEP,this.gl.INCR),s+=this.renderStyle(l.name,t,o),this.gl.disable(this.gl.STENCIL_TEST),this.gl.depthFunc(this.gl.LESS);else if("opaque"!==h&&!0===l.stencil_proxy_tiles){const e=this.tile_manager.getRenderableTiles().filter(e=>e.meshes[l.name]).map(e=>e.proxy_level).reduce((e,t)=>(e.indexOf(t)>-1||e.push(t),e),[]).sort();if(e.length>1){this.gl.enable(this.gl.STENCIL_TEST),this.gl.clearStencil(0),this.gl.clear(this.gl.STENCIL_BUFFER_BIT),this.gl.stencilOp(this.gl.KEEP,this.gl.KEEP,this.gl.REPLACE);for(let i=0;i<e.length;i++)this.gl.stencilFunc(this.gl.GEQUAL,e.length-i,255),s+=this.renderStyle(l.name,t,o,e[i]);this.gl.disable(this.gl.STENCIL_TEST);}else s+=this.renderStyle(l.name,t,o);}else s+=this.renderStyle(l.name,t,o);n=l.blend;}}return s}renderStyle(e,t,i,r){void 0===r&&(r=null);let n,s=this.styles[e],o=!0,a=0,l=this.tile_manager.getRenderableTiles().filter(e=>"number"!=typeof r||e.proxy_level===r).map(t=>{if(t.meshes[e])return [t,t.meshes[e].filter(e=>e.variant.blend_order===i)]}).filter(e=>e),h=Math.max(...l.map(e=>{let t=e[1];return Math.max(...t.map(e=>e.variant.mesh_order))}));for(let i=0;i<h+1;i++)for(let r of l){let l=r[0],h=r[1],u=!0;if(!l.shouldProxyForStyle(e))continue;const c=h.filter(e=>e.variant.mesh_order===i);if(0!==c.length){if(!0===o&&(o=!1,!(n=this.setupStyle(s,t))))return 0;c.forEach(e=>{!0===u&&(u=!1,this.view.setupTile(l,n)),s.render(e)&&this.requestRedraw(),a+=e.geometry_count;});}}return a}setupStyle(e,t){let i;try{if(!(i=e.getProgram(t)))return}catch(t){return void this.trigger("warning",{type:"styles",message:`Error compiling style ${e.name}`,style:e,shader_errors:e.program&&e.program.shader_errors})}i.use(),e.setup(),i.uniform("1f","u_time",this.animated?(+new Date-this.start_time)/1e3:0),this.view.setupProgram(i);for(let e in this.lights)this.lights[e].setupProgram(i);return i}clearFrame(){this.initialized&&(this.render_states.depth_write.set({depth_write:!0}),this.gl.clear(this.gl.COLOR_BUFFER_BIT|this.gl.DEPTH_BUFFER_BIT|this.gl.STENCIL_BUFFER_BIT));}setRenderState(e){let t=void 0===e?{}:e,i=t.depth_test,r=t.depth_write,n=t.cull_face,s=t.blend;if(!this.initialized)return;let o=this.render_states;i=!1!==i&&o.defaults.depth_test,r=!1!==r&&o.defaults.depth_write,n=!1!==n&&o.defaults.culling,s=null!=s?s:o.defaults.blending;let a=this.gl;o.depth_test.set({depth_test:i}),o.depth_write.set({depth_write:r}),o.culling.set({cull:n,face:o.defaults.culling_face}),s?"opaque"===s?o.blending.set({blend:!1}):"overlay"===s||"inlay"===s||"translucent"===s?o.blending.set({blend:!0,src:a.SRC_ALPHA,dst:a.ONE_MINUS_SRC_ALPHA,src_alpha:a.ONE,dst_alpha:a.ONE_MINUS_SRC_ALPHA}):"add"===s?o.blending.set({blend:!0,src:a.ONE,dst:a.ONE}):"multiply"===s&&o.blending.set({blend:!0,src:a.ZERO,dst:a.SRC_COLOR}):o.blending.set({blend:!1});}getFeatureAt(t,i){let r=(void 0===i?{}:i).radius;if(!this.initialized)return e.log("debug","Scene.getFeatureAt() called before scene was initialized"),Promise.resolve();if(0===this.selection_feature_count)return Promise.resolve();this.selection||this.resetFeatureSelection();let n={x:t.x/this.view.size.css.width,y:t.y/this.view.size.css.height};return r=r>0?{x:r/this.view.size.css.width,y:r/this.view.size.css.height}:null,this.selection.getFeatureAt(n,{radius:r}).then(e=>Object.assign(e,{pixel:t})).catch(e=>Promise.resolve({error:e}))}async queryFeatures(t){let i=void 0===t?{}:t,r=i.filter,n=i.unique,s=void 0===n||n,o=i.group_by,a=void 0===o?null:o,l=i.visible,h=void 0===l?null:l,u=i.geometry,c=void 0!==u&&u;if(!this.initialized)return [];r=e.Utils.serializeWithFunctions(r);const f=(s="string"==typeof s?[s]:s)&&(t=>{const i=Array.isArray(s)?e.sliceObject(t.properties,s):t.properties;return c?JSON.stringify({geometry:t.geometry,properties:i}):JSON.stringify(i)}),d=(a=("string"==typeof a||Array.isArray(a))&&a)&&(t=>Array.isArray(a)?JSON.stringify(e.sliceObject(t,a)):t[a]),p=this.tile_manager.getRenderableTiles().map(e=>e.key),g=await e.WorkerBroker.postMessage(this.workers,"self.queryFeatures",{filter:r,visible:h,geometry:c,tile_keys:p}),m=[],_={},y={};return g.forEach(e=>e.forEach(e=>{if(f){const t=f(e);if(_[t])return;_[t]=!0;}if(d){const t=d(e.properties);y[t]=y[t]||[],y[t].push(e);}else m.push(e);})),d?y:m}rebuild(t){let i=void 0===t?{}:t,r=i.initial,n=void 0!==r&&r,s=i.new_generation,o=void 0===s||s,a=i.sources,l=void 0===a?null:a,h=i.serialize_funcs,u=i.profile,c=void 0!==u&&u,f=i.fade_in,d=void 0!==f&&f;return new Promise((t,i)=>{if(this.building){this.building.queued&&this.building.queued.reject&&(e.log("debug","Scene.rebuild: request superceded by a newer call"),this.building.queued.resolve(!1));let r={initial:n,new_generation:o,sources:l,serialize_funcs:h,profile:c,fade_in:d};return this.building.queued={resolve:t,reject:i,options:r},void e.log("trace","Scene.rebuild(): queuing request")}if(this.building={resolve:t,reject:i,initial:n},c&&this.debug.profile("Scene.rebuild"),o){this.generation=++iu.generation;for(let e in this.styles)this.styles[e].setGeneration(this.generation);}this.syncConfigToWorker({serialize_funcs:h}),this.resetWorkerFeatureSelection(l),this.resetTime(),this.tile_manager.pruneToVisibleTiles(),this.tile_manager.forEachTile(e=>{(!l||l.indexOf(e.source.name)>-1)&&this.tile_manager.buildTile(e,{fade_in:d});}),this.tile_manager.updateTilesForView(),this.tile_manager.checkBuildQueue();}).then(()=>{c&&this.debug.profileEnd("Scene.rebuild");})}tileManagerBuildDone(){if(e.TextCanvas.pruneTextCache(),this.building){e.log("info","Scene: build geometry finished"),this.building.resolve&&(this.logFirstBuild(),this.building.resolve(!0));var t=this.building.queued;this.building=null,t?(e.log("debug","Scene: starting queued rebuild() request"),this.rebuild(t.options).then(t.resolve,t.reject)):this.tile_manager.updateLabels();}}loadScene(t,i){void 0===t&&(t=null);let r=void 0===i?{}:i,n=r.base_path,s=r.file_type;return this.config_source=t||this.config_source,"string"==typeof this.config_source?this.base_path=e.pathForURL(n||this.config_source):this.base_path=e.pathForURL(n),this.config_path=this.base_path,qh.loadScene(this.config_source,{path:this.base_path,type:s}).then(e=>{let t=e.config,i=e.bundle;return this.config=t,this.config_bundle=i,this.config})}setDataSource(t,i){if(!t||!i||!i.type||!i.url&&!i.data)return void e.log("error","No name provided or not a valid config:",t,i);let r=null==this.config.sources[t],n=this.config.sources[t]=Object.assign({},i);return n.data&&"object"==typeof n.data&&(n.url=e.createObjectURL(new Blob([JSON.stringify(n.data)])),delete n.data),r?this.updateConfig({rebuild:{sources:[t]}}):this.rebuild({sources:[t]})}createDataSources(t){void 0===t&&(t=!1);const i=[],r=Object.keys(this.sources);let n=0;for(var s in this.config.sources){const r=this.config.sources[s],o=this.sources[s];try{const t=e._extends({},r,{name:s,id:n++});if(this.sources[s]=e.DataSource.create(t,this.sources),!this.sources[s])throw{}}catch(t){delete this.sources[s];const i=`Could not create data source: ${t.message}`;e.log("warn",`Scene: ${i}`,r),this.trigger("warning",{type:"sources",source:r,message:i});}(t||e.DataSource.tileLayoutChanged(this.sources[s],o))&&i.push(s);}r.forEach(e=>{this.config.sources[e]||(delete this.sources[e],i.push(e));}),i.length>0&&this.tile_manager.removeTiles(e=>i.indexOf(e.source.name)>-1);for(let e in this.config.layers){let t=this.config.layers[e];!1!==t.enabled&&t.data&&this.sources[t.data.source]&&(this.sources[t.data.source].builds_geometry_tiles=!0);}}loadTextures(){return e.Texture.createFromObject(this.gl,this.config.textures).then(()=>e.Texture.createDefault(this.gl))}freePreviousTextures(){this.prev_textures&&(this.prev_textures.forEach(t=>{!this.config.textures[t]&&e.Texture.textures[t]&&e.Texture.textures[t].destroy();}),this.prev_textures=null);}updateStyles(){if(!this.initialized&&!this.initializing)throw new Error("Scene.updateStyles() called before scene was initialized");this.styles=this.style_manager.build(this.config.styles),this.style_manager.initStyles(this);for(let e in this.styles)this.styles[e].setGL(this.gl);this.dirty=!0;}get animated(){return !1!==this.config.scene.animated&&this.style_manager.getActiveStyles().some(e=>this.styles[e].animated)}getActiveCamera(){return this.view.getActiveCamera()}setActiveCamera(e){return this.view.setActiveCamera(e)}createLights(){this.lights={};for(let t in this.config.lights){if(!this.config.lights[t]||"object"!=typeof this.config.lights[t])continue;let i=this.config.lights[t];i.name=t.replace("-","_"),i.visible=!1!==i.visible,i.visible&&(this.lights[i.name]=e.Light.create(this.view,i));}e.Light.inject(this.lights);}setBackground(){const t=this.config.scene.background;this.background={},t&&t.color&&(this.background.color=e.StyleParser.createColorPropertyCache(t.color)),this.background.color||(this.background.color=e.StyleParser.createColorPropertyCache([0,0,0,0]));}updateBackground(){const t=this.background.computed_color,i=this.background.computed_color=e.StyleParser.evalCachedColorProperty(this.background.color,{zoom:this.view.tile_zoom});t&&!i.some((e,i)=>t[i]!==e)||(1===i[3]?this.canvas.style.backgroundColor=`rgba(${i.map(e=>Math.floor(255*e)).join(", ")})`:this.canvas.style.backgroundColor="transparent",this.gl.clearColor(...i));}setIntrospection(e){return e!==this.introspection?(this.introspection=null!=e&&e,this.updating++,this.updateConfig({normalize:!1}).then(()=>this.updating--)):Promise.resolve()}updateConfig(t){let i=void 0===t?{}:t,r=i.loading,n=void 0!==r&&r,s=i.rebuild,o=void 0===s||s,a=i.serialize_funcs,l=i.normalize,h=void 0===l||l,u=i.fade_in,c=void 0!==u&&u;this.generation=++iu.generation,this.updating++,this.config=qh.applyGlobalProperties(this.config),h?qh.normalize(this.config,this.config_bundle):(qh.hoistStyleShaderUniformTextures(this.config,this.config_bundle,{include_globals:!0}),qh.normalizeTextures(this.config,this.config_bundle)),this.trigger(n?"load":"update",{config:this.config}),this.style_manager.init(),this.view.reset(),this.createLights(),this.createDataSources(n),this.loadTextures(),this.setBackground(),e.FontManager.loadFonts(this.config.fonts),this.updateStyles();let f=o?this.rebuild(Object.assign({initial:n,new_generation:!1,serialize_funcs:a,fade_in:c},"object"==typeof o&&o)):this.syncConfigToWorker({serialize_funcs:a});return this.updating--,this.view.updateBounds(),this.requestRedraw(),f.then(()=>{this.last_render_count=0,this.requestRedraw();})}syncConfigToWorker(t){let i=(void 0===t?{}:t).serialize_funcs,r=void 0===i||i?e.Utils.serializeWithFunctions(this.config):JSON.stringify(this.config);return e.WorkerBroker.postMessage(this.workers,"self.updateConfig",{config:r,generation:this.generation,introspection:this.introspection},e.debugSettings)}createListeners(){this.listeners={},this.listeners.view={move:()=>this.trigger("move")},this.view.subscribe(this.listeners.view),this.listeners.texture={update:()=>this.dirty=!0,warning:e=>this.trigger("warning",Object.assign({type:"textures"},e))},e.Texture.subscribe(this.listeners.texture),this.listeners.scene_loader={error:e=>this.trigger("error",Object.assign({type:"scene"},e))},qh.subscribe(this.listeners.scene_loader);}destroyListeners(){this.unsubscribeAll(),this.view.unsubscribe(this.listeners.view),e.Texture.unsubscribe(this.listeners.texture),qh.unsubscribe(this.listeners.scene_loader),this.listeners=null;}destroyFeatureSelection(){this.selection&&(this.selection.destroy(),this.selection=null);}resetFeatureSelection(){this.selection=new e.FeatureSelection(this.gl,this.workers,()=>this.building),this.last_render_count=0;}resetWorkerFeatureSelection(t){void 0===t&&(t=null),this.workers&&e.WorkerBroker.postMessage(this.workers,"self.resetFeatureSelection",t);}getFeatureSelectionMapSize(){return null==this.fetching_selection_map&&(this.fetching_selection_map=e.WorkerBroker.postMessage(this.workers,"self.getFeatureSelectionMapSize").then(e=>(this.fetching_selection_map=null,e.reduce((e,t)=>e+t)))),this.fetching_selection_map}resetTime(){this.start_time=+new Date;}updateViewComplete(){!this.render_count_changed&&this.generation===this.last_complete_generation||this.building||this.tile_manager.isLoadingVisibleTiles()||!this.tile_manager.allVisibleTilesLabeled()||(this.tile_manager.updateLabels(),this.last_complete_generation=this.generation,this.trigger("view_complete",{first:!0!==this.view_complete}),this.view_complete=!0);}resetViewComplete(){this.last_complete_generation=null;}screenshot(e){let t=(void 0===e?{}:e).background,i=void 0===t?"white":t;return this.requestRedraw(),this.media_capture.screenshot({background:i})}startVideoCapture(){return this.requestRedraw(),this.media_capture.startVideoCapture()}stopVideoCapture(){return this.media_capture.stopVideoCapture()}logFirstFrame(){0!==this.last_render_count||this.times.first_frame||(this.times.first_frame=+new Date-this.start_time,e.log("debug",`Scene: initial frame time: ${this.times.first_frame}`));}logFirstBuild(){null==this.times.first_build&&(this.times.first_build=+new Date-this.start_time,e.log("debug",`Scene: initial build time: ${this.times.first_build}`));}}var ru;iu.id=0,iu.generation=0;const nu={map:{},scrollWheelZoom:{},doubleClickZoom:{}};return {leafletLayer:function(t){return function(t){if(void 0!==ru)return new ru(t);if(e.Thread.is_main){let i,r=t.leaflet||window.L,n=r.GridLayer?r.GridLayer:r.TileLayer,s=n===r.GridLayer?"1.x":"0.7.x",o={};return n===r.TileLayer&&(o._addTile=function(){},o._removeTile=function(){},o._reset=function(){n.prototype._reset.apply(this,arguments),this.scene&&this.scene.container&&this.scene.canvas&&this.scene.container.appendChild(this.scene.canvas);}),Object.assign(o,{initialize(e){e.showDebug=!!e.showDebug,r.setOptions(this,e),this.updateTangramDebugSettings(),this.createScene(),this.hooks={},this._updating_tangram=!1;},createScene(){this.scene=iu.create(this.options.scene,{numWorkers:this.options.numWorkers,preUpdate:this.options.preUpdate,postUpdate:this.options.postUpdate,continuousZoom:"1.x"===ru.leafletVersion,wrapView:!0!==this.options.noWrap,highDensityDisplay:this.options.highDensityDisplay,logLevel:this.options.logLevel,introspection:this.options.introspection,webGLContextOptions:this.options.webGLContextOptions,disableRenderLoop:this.options.disableRenderLoop});},onAdd(t){this.scene||this.createScene(),n.prototype.onAdd.apply(this,arguments),this.hooks.resize=(()=>{this._updating_tangram=!0,this.updateSize(),this._updating_tangram=!1;}),t.on("resize",this.hooks.resize),this.hooks.move=(()=>{if(!this._updating_tangram){this._updating_tangram=!0,this.scene.view.setPanning(!0);var i=t.getCenter();i.zoom=Math.max(Math.min(t.getZoom(),t.getMaxZoom()||e.Geo.default_view_max_zoom),t.getMinZoom()),this.scene.view.setView(i),this._mapLayerCount>1&&this.scene.immediateRedraw(),this._updating_tangram=!1;}}),t.on("move",this.hooks.move),this.hooks.moveend=(()=>{this.scene.view.setPanning(!1),this.scene.requestRedraw();}),t.on("moveend",this.hooks.moveend),this.hooks.drag=(()=>{this.scene.view.markUserInput();}),t.on("drag",this.hooks.drag),this._map_pane_observer=new MutationObserver(e=>{e.forEach(()=>this.reverseTransform());}),this._map_pane_observer.observe(t.getPanes().mapPane,{attributes:!0}),this.modifyScrollWheelBehavior(t),this.modifyZoomBehavior(t),this.trackMapLayerCounts(t),this.setupSelectionEventHandlers(t),this.setSelectionEvents(this.options.events,{radius:this.options.selectionRadius}),this.scene.container=this.getContainer(),this.updateSize(),this.updateView(),this.resizeOnFirstVisible(),this.scene.subscribe({move:this.onTangramViewUpdate.bind(this)}),this.scene.load(this.options.scene,{base_path:this.options.sceneBasePath,file_type:this.options.sceneFileType,blocking:!1}).then(()=>{this._updating_tangram=!0,this.updateSize(),this.updateView(),this.reverseTransform(),this._updating_tangram=!1,this.fire("init");}).catch(e=>{this.fire("error",e);});},onRemove(e){n.prototype.onRemove.apply(this,arguments),e.off("layeradd layerremove overlayadd overlayremove",this._updateMapLayerCount),e.off("resize",this.hooks.resize),e.off("move",this.hooks.move),e.off("moveend",this.hooks.moveend),e.off("drag",this.hooks.drag),e.off("click",this.hooks.click),e.off("mousemove",this.hooks.mousemove),e.off("mouseout",this.hooks.mouseout),document.removeEventListener("visibilitychange",this.hooks.visibilitychange),this.hooks={},this._map_pane_observer.disconnect(),this.scene&&(this.scene.destroy(),this.scene=null);},createTile(e){var t=e.x+"/"+e.y+"/"+e.z,i=document.createElement("div");if(i.setAttribute("data-tile-key",t),i.style.width="256px",i.style.height="256px",this.options.showDebug){var r=document.createElement("div");r.textContent=t,r.style.position="absolute",r.style.left=0,r.style.top=0,r.style.color="white",r.style.fontSize="16px",r.style.textOutline="1px #000000",r.style.padding="8px",i.appendChild(r),i.style.borderStyle="solid",i.style.borderColor="white",i.style.borderWidth="1px";}return i},modifyScrollWheelBehavior(e){if(this.scene.view.continuous_zoom&&e.scrollWheelZoom&&!1!==this.options.modifyScrollWheel){e.options.zoomSnap=0;const a=e.scrollWheelZoom.enabled();e.scrollWheelZoom.disable(),r.Browser.chrome||r.Browser.safari?e.scrollWheelZoom._onWheelScroll=function(e){var t=r.DomEvent.getWheelDelta(e);this._delta+=t,this._lastMousePos=this._map.mouseEventToContainerPoint(e),this._performZoom(),r.DomEvent.stop(e);}:e.options.wheelDebounceTime=20;const l=(n=function(e){e._moveEnd(!0),e.fire("viewreset");},s=2*e.options.wheelDebounceTime,function(){var e=this,t=arguments;clearTimeout(o),o=setTimeout(function(){o=null,n.apply(e,t);},s);});nu.scrollWheelZoom._performZoom||(nu.scrollWheelZoom._performZoom=e.scrollWheelZoom._performZoom);var t=this;e.scrollWheelZoom._performZoom=function(){if(this._map===t._map){var e=this._map,r=e.getZoom();e._stop();var n=this._delta/(4*this._map.options.wheelPxPerZoomLevel);this._delta=0,r+n>=this._map.getMaxZoom()?n=this._map.getMaxZoom()-r:r+n<=this._map.getMinZoom()&&(n=this._map.getMinZoom()-r),n&&("center"===e.options.scrollWheelZoom?i(t,e.getCenter(),r+n):i(t,this._lastMousePos,r+n),l(e));}else nu.scrollWheelZoom._performZoom.call(this);},a&&e.scrollWheelZoom.enable();}var n,s,o;},modifyZoomBehavior(e){if(this.scene.view.continuous_zoom&&!1!==this.options.modifyZoomBehavior){var t=this;const n=function(t,n,s){e._stop();var o=e._zoom;n=r.latLng(n),s=void 0===s?o:s,s=Math.min(s,e.getMaxZoom());var a=Date.now(),l=75;return e._moveStart(!0),function h(){var u=(Date.now()-a)/l;u<=1?(e._flyToFrame=r.Util.requestAnimFrame(h,e),i(t,n,o+(s-o)*u)):i(t,n,s)._moveEnd(!0);}.call(e),e};if(e.doubleClickZoom){const i=e.doubleClickZoom.enabled();e.doubleClickZoom.disable(),nu.doubleClickZoom._onDoubleClick||(nu.doubleClickZoom._onDoubleClick=e.doubleClickZoom._onDoubleClick),e.doubleClickZoom._onDoubleClick=function(e){if(this._map===t._map){var i=this._map,r=i.getZoom(),s=i.options.zoomDelta,o=e.originalEvent.shiftKey?r-s:r+s;"center"===i.options.doubleClickZoom?n(t,i.getCenter(),o):n(t,i.containerPointToLatLng(e.containerPoint),o);}else nu.doubleClickZoom._onDoubleClick.call(this,e);},i&&e.doubleClickZoom.enable();}e._zoomAnimated&&(nu.map._animateZoom||(nu.map._animateZoom=e._animateZoom),e._animateZoom=function(e,i,s,o){this===t._map?(s&&(this._animatingZoom=!0,this._animateToCenter=e,this._animateToZoom=i,n(t,e,i)),setTimeout(r.Util.bind(this._onZoomTransitionEnd,this),250)):nu.map._animateZoom.call(this,e,i,s,o);});}},updateView(){var t=this._map.getCenter();t.zoom=Math.max(Math.min(this._map.getZoom(),this._map.getMaxZoom()||e.Geo.default_view_max_zoom),this._map.getMinZoom()),this.scene.view.setView(t);},updateSize(){var e=this._map.getSize();this.scene.resizeMap(e.x,e.y);},resizeOnFirstVisible(){let e=!0;this.hooks.visibilitychange=(()=>{e&&(e=!1,this.updateSize());}),document.addEventListener("visibilitychange",this.hooks.visibilitychange);},onTangramViewUpdate(){if(!this._map||this._updating_tangram)return;let e=this._map.getCenter(),t=this.scene.view.center;e.lng===t.lng&&e.lat===t.lat&&this._map.getZoom()===this.scene.view.zoom||(this._updating_tangram=!0,this._map.setView([this.scene.view.center.lat,this.scene.view.center.lng],this.scene.view.zoom,{animate:!1}),this._updating_tangram=!1);},render(){this.scene&&this.scene.update();},reverseTransform(){if(this._map&&this.scene&&this.scene.container){var e=this._map.containerPointToLayerPoint([0,0]);r.DomUtil.setPosition(this.scene.container,e);}},setupSelectionEventHandlers(e){this._selection_events={},this._selection_radius=null,this.hooks.click=(e=>{"function"==typeof this._selection_events.click&&this.scene.getFeatureAt(e.containerPoint,{radius:this._selection_radius}).then(t=>{let i=Object.assign({},t,{leaflet_event:e});this._selection_events.click(i);});}),e.on("click",this.hooks.click),this.hooks.mousemove=(e=>{"function"==typeof this._selection_events.hover&&this.scene.getFeatureAt(e.containerPoint,{radius:this._selection_radius}).then(t=>{let i=Object.assign({},t,{leaflet_event:e});this._selection_events.hover(i);});}),e.on("mousemove",this.hooks.mousemove),this.hooks.mouseout=(e=>{"function"==typeof this._selection_events.hover&&this._selection_events.hover({changed:!0,leaflet_event:e});}),e.on("mouseout",this.hooks.mouseout);},setSelectionEvents(e,t){let i=void 0===t?{}:t,r=i.radius;this._selection_events=Object.assign(this._selection_events,e),this._selection_radius=void 0!==r?r:this._selection_radius;},trackMapLayerCounts(e){this._updateMapLayerCount=(()=>{let t=e.getPanes().mapPane.childNodes;this._mapLayerCount=0;for(let e=0;e<t.length;e++)this._mapLayerCount+=t[e].childNodes.length;}),e.on("layeradd layerremove overlayadd overlayremove",this._updateMapLayerCount),this._updateMapLayerCount();},updateTangramDebugSettings(){e.mergeDebugSettings(this.options.debug||{});}}),i=function(e,t,i){var n=e._map,s=e.scene,o=n.getZoomScale(i),a=n.getSize().divideBy(2),l=t instanceof r.Point?t:n.latLngToContainerPoint(t),h=l.subtract(a).multiplyBy(1-1/o),u=n.containerPointToLatLng(a.add(h));return s&&s.view.markUserInput(),n._move(u,i,{flyTo:!0})},"function"!=typeof(ru=n.extend(o)).remove&&(ru.prototype.remove=function(){this._map&&this._map.removeLayer(this),this.fire("remove");}),ru.layerBaseClass=n,ru.leafletVersion=s,new ru(t)}}(t)},debug:{log:e.log,yaml:Dh,Utils:e.Utils,Geo:e.Geo,Vector:e.Vector,DataSource:e.DataSource,GLSL:e.GLSL,ShaderProgram:e.ShaderProgram,VertexData:e.VertexData,Texture:e.Texture,Material:e.Material,Light:e.Light,Scene:iu,WorkerBroker:e.WorkerBroker,Task:e.Task,StyleManager:e.StyleManager,StyleParser:e.StyleParser,TileID:e.TileID,Collision:e.Collision,FeatureSelection:e.FeatureSelection,TextCanvas:e.TextCanvas,debugSettings:e.debugSettings},version:e.version}});

// Loads each of the chunks produced by the first Rollup pass.

// This allows the rollup ESM build to work within a <script type="module"> tag
// Script modules can't expose exports
try {
	Tangram.debug.ESM = true; // mark build as ES module
	Tangram.debug.SHA = 'b08f8c830cad59c24ca6f79f40b0d1db76b4136e';
	if (true === true && typeof window === 'object') {
	    window.Tangram = Tangram;
	}
} catch(e) {}

export default Tangram;
